/***************************************************************************
                          D2NOESY.cpp  -  description
                             -------------------
    begin                : Mon Aug 21 2000
    copyright            : (C) 2000 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#include <cmath>
#include <stdlib.h>

#include "D2NOESY.h"
#include "noesy_assign.h"

extern bool DEBUG;

void error(char*, char*);
void error(string, string);

D2NOESY::D2NOESY(bool flag, bool haveIC, float hx1, float hx2, float f, string label) : NOESY(flag, haveIC, hx1, hx2, f, label)
{
}
D2NOESY::~D2NOESY()
{
}

const string D2NOESY::Type() const
{
  return "2D";
}


bool D2NOESY::nearH2O(const general_peak& peak) const
{
  return 0;
//    if (waterFlag && (((peak.get_HX2() <= waterShiftUpper) && (peak.get_HX2() >= waterShiftLower)) ||
//        ((peak.get_HX1() <= waterShiftUpper) && (peak.get_HX1() >= waterShiftLower))))
//  //  	if (waterFlag && ((fabs(peak.get_HX1() - waterShift) < tols.HX1) && 
//  //  		(fabs(peak.get_HX2() - waterShift) < tols.HX2)))
//      {
//        return 1; 
//      }
  
//    return 0;  
}

bool D2NOESY::isPossFather(const atom* temp1, pair<float, float> bound) const
{
        // in d20 case 
	
	if (temp1->isH())
	{
		if (waterFlag)
			return 1;
		else if (temp1->isatt_N())
			return 0;
		return 1;
	} else
		return 0;
}

bool D2NOESY::isPossMother(const atom* temp1, pair<float, float> bound) const
{
	if (temp1->isH())
	{
		if (waterFlag)
			return 1;
		else if (temp1->isatt_N())
			return 0;
		return 1;
	}
}

bool D2NOESY::canbeMom(const atom* atom1) const
{
	return 1; 
}

bool D2NOESY::canbeDad(const atom* atom1) const
{
	return 1;
}

void D2NOESY::buildPeak(const vector<string>& subline, const int j)
{
			if (j >= 3)
			{		
				general_peak temp_peak(2);
				if (!peakControlPtr)
				  error("Peak list format not defined : ", "Can not read in peaklist");

				  static int id=0;
				  id++;
				  string label=itoa(id);
				
				  if ((*peakControlPtr)["col.id"] != "")
				  {
				  	id = atoi(subline[atoi((*peakControlPtr)["col.id"].c_str())-1].c_str());
				  }				
				  if ((*peakControlPtr)["col.label"] != "")
				  {
				  	label = subline[atoi((*peakControlPtr)["col.label"].c_str())-1].c_str();
				  }				
					
				  temp_peak.assign_gen(id, label,
					atof(subline[atoi((*peakControlPtr)["col.intensity"].c_str())-1].c_str()),
					atof(subline[atoi((*peakControlPtr)["col.hx1"].c_str())-1].c_str()),
					atof(subline[atoi((*peakControlPtr)["col.hx2"].c_str())-1].c_str()));
									
				  if ((temp_peak.get_HX1() !=0) || (temp_peak.get_HX2() !=0))	
				  {
				      peakList.push_back(temp_peak);
				      //if (DEBUG)
					NOESY_Assign::outputLog() << "Read as-> " << temp_peak << endl;
				  } else
				  {
				    NOESY_Assign::outputLog() << "Skipped ";				    
				    for(vector<string>::const_iterator i = subline.begin(); i!= subline.end(); i++)
				      {
					NOESY_Assign::outputLog() << *i << " "; 
				      }
				    NOESY_Assign::outputLog() << endl;
				  }
			} else
			{
			  error("Bad input format: Cannot Read D3NOESY peaklist from", label());
			  error("  missing required columns", "");
			}
}

// Inter chain

ICD2NOESY::ICD2NOESY(bool flag, bool haveIC, float hx1, float hx2, float f, string label) : D2NOESY(flag, haveIC, hx1, hx2, f, label)
{
  IC = 1;
}

ICD2NOESY::~ICD2NOESY()
{
}

const string ICD2NOESY::Type() const
{
  return "2D IC";
}

// BH root at B,
void D2NOESY::Find_UniqueBH(int i, int j, method mthd)
{
	MatchPeakAtombyAtomIndex MombyIndex;

	for(MatchPeakAtom::iterator iter = MombySimpleMatch.begin(); iter != MombySimpleMatch.end(); iter++)
		if ((*iter).second->get_layer().substr(0,1) == "B")
			MombyIndex.insert(make_pair((*iter).second->get_index(), (*iter)));

	// find all possible (i,j) that root at i
	MatchPeakAtombyAtomIndex::iterator
		res1RootLower = MombyIndex.lower_bound(i);
	MatchPeakAtombyAtomIndex::iterator
		res1RootUpper = MombyIndex.upper_bound(i);

	for(MatchPeakAtombyAtomIndex::iterator iter1 = res1RootLower;
		iter1 != res1RootUpper; iter1++)
	{
		// for each peak, find all possible father
		MatchPeakAtom::iterator fatherLower
			= DadbySimpleMatch.lower_bound((*iter1).second.first);
		MatchPeakAtom::iterator fatherUpper
			= DadbySimpleMatch.upper_bound((*iter1).second.first);

		for(MatchPeakAtom::iterator iterf = fatherLower; iterf != fatherUpper; iterf++)
			if (((*iterf).second->get_index() == j) &&
				((*iterf).second->get_layer() == "N"))
			{
					MakeBuffer((*iterf).first, (*iter1).second.second, (*iterf).second, mthd);
			}
	}
	
	// case 2
	for(MatchPeakAtom::iterator iter = MombySimpleMatch.begin(); iter != MombySimpleMatch.end(); iter++)
		if ((*iter).second->get_layer().substr(0,1) == "N")
			MombyIndex.insert(make_pair((*iter).second->get_index(), (*iter)));

	// find all possible (i,j) that root at j
		res1RootLower = MombyIndex.lower_bound(j);
		res1RootUpper = MombyIndex.upper_bound(j);

	for(MatchPeakAtombyAtomIndex::iterator iter1 = res1RootLower;
		iter1 != res1RootUpper; iter1++)
	{
		// for each peak, find all possible father
		MatchPeakAtom::iterator fatherLower
			= DadbySimpleMatch.lower_bound((*iter1).second.first);
		MatchPeakAtom::iterator fatherUpper
			= DadbySimpleMatch.upper_bound((*iter1).second.first);

		for(MatchPeakAtom::iterator iterf = fatherLower; iterf != fatherUpper; iterf++)
			if (((*iterf).second->get_index() == i) &&
				((*iterf).second->get_layer() == "B"))
			{
					MakeBuffer((*iterf).first, (*iter1).second.second, (*iterf).second, mthd);
			}
	}

}

// root at A
void D2NOESY::Find_UniqueAH(int i, int j, method mthd)
{
	MatchPeakAtombyAtomIndex MombyIndex;

	for(MatchPeakAtom::iterator iter = MombySimpleMatch.begin(); iter != MombySimpleMatch.end(); iter++)
		if ((*iter).second->get_layer().substr(0,1) == "A")
			MombyIndex.insert(make_pair((*iter).second->get_index(), (*iter)));

	// find all possible (i,j) that root at i
	MatchPeakAtombyAtomIndex::iterator
		res1RootLower = MombyIndex.lower_bound(i);
	MatchPeakAtombyAtomIndex::iterator
		res1RootUpper = MombyIndex.upper_bound(i);

	for(MatchPeakAtombyAtomIndex::iterator iter1 = res1RootLower;
		iter1 != res1RootUpper; iter1++)
	{
		// for each peak, find all possible father
		MatchPeakAtom::iterator fatherLower
			= DadbySimpleMatch.lower_bound((*iter1).second.first);
		MatchPeakAtom::iterator fatherUpper
			= DadbySimpleMatch.upper_bound((*iter1).second.first);

		for(MatchPeakAtom::iterator iterf = fatherLower; iterf != fatherUpper; iterf++)
			if (((*iterf).second->get_index() == j) &&
				((*iterf).second->get_layer() == "N"))
			{
					MakeBuffer((*iterf).first, (*iter1).second.second, (*iterf).second, mthd);
			}
	}
	
	// case 2
	for(MatchPeakAtom::iterator iter = MombySimpleMatch.begin(); iter != MombySimpleMatch.end(); iter++)
		if ((*iter).second->get_layer().substr(0,1) == "N")
			MombyIndex.insert(make_pair((*iter).second->get_index(), (*iter)));

	// find all possible (i,j) that root at j
		res1RootLower = MombyIndex.lower_bound(j);
		res1RootUpper = MombyIndex.upper_bound(j);

	for(MatchPeakAtombyAtomIndex::iterator iter1 = res1RootLower;
		iter1 != res1RootUpper; iter1++)
	{
		// for each peak, find all possible father
		MatchPeakAtom::iterator fatherLower
			= DadbySimpleMatch.lower_bound((*iter1).second.first);
		MatchPeakAtom::iterator fatherUpper
			= DadbySimpleMatch.upper_bound((*iter1).second.first);

		for(MatchPeakAtom::iterator iterf = fatherLower; iterf != fatherUpper; iterf++)
			if (((*iterf).second->get_index() == i) &&
				((*iterf).second->get_layer() == "A"))
			{
					MakeBuffer((*iterf).first, (*iter1).second.second, (*iterf).second, mthd);
			}
	}
	
}





