/***************************************************************************
                          Diana.h  -  description
                             -------------------
    begin                : Mon Aug 21 2000
    copyright            : (C) 2000 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#ifndef DIANA_H
#define DIANA_H

#include <map>
#include "atom.h"
#include "hbond.h"
#include "Jval.h"

using namespace std;

typedef pair<const atom* const, const atom* const> atom12;
 
struct atom12less
{
	bool operator()(const atom12& a, const atom12& b) const
	{
		if ((*((*a.first).get_repAtom())) < (*((*b.first).get_repAtom()))) return 1;
		if ((*((*b.first).get_repAtom())) < (*((*a.first).get_repAtom()))) return 0;
		if ((*((*a.second).get_repAtom())) < (*((*b.second).get_repAtom()))) return 1;
		if ((*((*b.second).get_repAtom())) < (*((*a.second).get_repAtom()))) return 0;

		if ((*a.first).get_name() < (*b.first).get_name()) return 1;
		if ((*b.first).get_name() < (*a.first).get_name()) return 0;
		if ((*a.second).get_name() < (*b.second).get_name()) return 1;
		if ((*b.second).get_name() < (*a.second).get_name()) return 0;

		return 0;
	}
}; 

typedef set<atom12, atom12less > atom12set; 

struct DianaConstraint
{
  string     name1; 
  string     name2;  
  float      dist;  // added correction
  float 		 corrections;
  float      ub; 
  float  		 xplorDist; // sum-average
  float      xub; 
  string     cment; 
  string     expr; 
	int				 nn; // number of atoms degenerate
  float      ori_dist; // without correction
  bool       multi; // multiple assignments
  float      range; // the range of ori_dists
};

typedef multimap<atom12, DianaConstraint, atom12less> DianaMap; 
typedef deque<pair<atom12, DianaConstraint> > DianaDeque;

void prtDianaSeq(ostream&, const SEQM&, int);
void prtCyanaSeq(ostream&, const SEQM&, int);
void prtXPLORSeq(ostream&, const SEQM&, int);
void prtCyanaProt(ostream&, Atom_List&); 

// used to generate Diana running file
void prtDianabuffer(ostream&, const SEQM&, int, const DianaMap&, const DianaMap&); // grouped by sequence, atom id
void prtCyanabuffer(ostream&, const SEQM&, int, const DianaMap&, const DianaMap&, bool); // grouped by sequence, atom id
void prtXPLORbuffer(ostream&, const SEQM&, int, const DianaMap&, const DianaMap&, bool); // grouped by sequence, atom id

void prtCyIntraChainAmbi(ostream&, int, const atom12set&, int); // grouped by sequence, atom id

void prtDianabufferDeque(ostream&, const SEQM&, int, const DianaDeque&, const DianaDeque&); // grouped by peak id
void prtCyanabufferDeque(ostream&, const SEQM&, int, const DianaDeque&, const DianaDeque&); // grouped by peak id

void prtDianaHbond(ostream&, float, float, const SEQM&, int, const set<HBOND, less<HBOND> >&);
void prtCyanaHbond(ostream&, float, float, const SEQM&, int, const set<HBOND, less<HBOND> >&);
void prtXPLORHbond(ostream&, float, float, const SEQM&, int, const set<HBOND, less<HBOND> >&);
void prtXplorCACBShift(ostream&, const SEQM&, const Atom_List&);
void prtXplorJvalues(ostream&, const SEQM&, const JvalList&);

#endif 


