/***************************************************************************
                          FindHHPacking.cpp  -  description
                             -------------------
    begin                : Mon Feb 25 2002
    copyright            : (C) 2002 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#include <cmath>
#include <iostream>
#include <fstream>
#include <set>
#include <map>
#include "noesy_assign.h"

map<int, set<int, less<int> >, less<int> > HelixPacking;

typedef multimap<int, pair<int, float>, less<int> > HHcfMax;
typedef set<int, less<int> > HHcKey;
typedef pair<int, int> ij;

typedef map<pair<ij, ij>, float, less<pair<ij, ij> > > HHPair;

void NOESY_Assign::Find_HHPacking(const SS& auto_ss, ostream& out)
{
	set<char, less<char> > HPSet;
	HPSet.insert('I');
	HPSet.insert('V');
	HPSet.insert('L');
	HPSet.insert('F');
	HPSet.insert('M');
	HPSet.insert('A');
	HPSet.insert('G');
	HPSet.insert('W');
	
	// calculate symmetric contact frequency
  matrix cfMax;
	cfMax.clear();
	
	matrix ICcfMax;
	ICcfMax.clear();

	HHcfMax HHcf; // the cf based on helix-helix packing
	HHcf.clear();
	HHcKey HHckey;
	HHckey.clear();
	
	HHcfMax HHcf2; // the cf based on helix-helix packing
	HHcf2.clear();
	HHcKey HHckey2;
	HHckey2.clear();

	HHPair HHpair;
	HHpair.clear();
				
  for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); ++iter)
  {
  	if ( IC() && (*iter)->isIC())	
  	{
    	erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : buildSYMcfMax for " + (*iter)->label() + " ... ");
			(*iter)->Find_SYMcfposs(ICcfMax, _par.sym, out);
		} else
  	{
    	erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : buildSYMcfMax for " + (*iter)->label() + " ... ");
			(*iter)->Find_SYMcfposs(cfMax, _par.sym, out);
		}
  }
	
	// find all possible max. helix packing sets
	for(SS::const_iterator i = auto_ss.begin(); i != auto_ss.end(); ++i)
  {
  	if (i->second.get_type() == 1) //helix i
  	{
  		out << "Helix" << i->first << endl;
  		
  		for(SS::const_iterator j = auto_ss.begin(); j != auto_ss.end(); ++j)
    	{
    		if (((i != j)) && (j->second.get_type() == 1)) //helix j
    		{
    			out << "\t" << j->first << endl;
    			// find all possible helix packing triples
    			for(int ci = (*i->second.get_chain().begin()); ci <= *i->second.get_chain().rbegin(); ++ci)
    			{
    				if (HPSet.find(seq[ci].get_sym_1()) == HPSet.end())
    					continue;
										
    				// chose the highest for each key for each helix
    				int maxS = 0;
    				map<int, float, less<int> > tempCF;
    				tempCF.clear();
    				
    				for(int cj = *j->second.get_chain().begin(); cj <= *j->second.get_chain().rbegin(); ++cj)
    				{
    					if ((cfMax[make_pair(ci,cj)] != 0) && (cfMax[make_pair(ci,cj+1)] != 0)) // ||
    					{
    					
    						if ((HPSet.find(seq[cj].get_sym_1()) != HPSet.end()) ||
    								(HPSet.find(seq[cj+1].get_sym_1()) != HPSet.end())) // ||
    						{
    							float s = cfMax[make_pair(ci,cj)] + cfMax[make_pair(ci,cj+1)];
    							
									if (cj-3 >= (*j->second.get_chain().begin()))
    								s += cfMax[make_pair(ci,cj-3)];
									if (cj+4 <= (*j->second.get_chain().rbegin()))
    								s += cfMax[make_pair(ci,cj+4)];
    							
    							if (s > maxS)
    							{
    								maxS = (int) s;
    								tempCF.insert(make_pair(cj, s));
    							}
    						}
    						
    					}
    				}// end of for cj
    				
    				// for each ci, find the max. cj of this helix
    				if (maxS != 0)
    				{
    					for(map<int, float, less<int> >::iterator ti = tempCF.begin(); ti != tempCF.end(); ++ti)
    					{
    						if (ti->second == maxS)
    						{
    							int maxJ = ti->first;
    							HHcf.insert(make_pair(ci, make_pair(maxJ, ti->second)));
    							HHckey.insert(ci);
     							
    							out << "key:" << ci << seq[ci].get_sym_1()<< " to: " << maxJ << seq[maxJ].get_sym_1()
																							<< " " << maxJ+1 << seq[maxJ+1].get_sym_1() << " s:" << ti->second << endl;
    						}
    					}
    				}
    			}//end of for ci    			
    		}
			}
			
			}
			
			}
    			
    				int intA[4];
    				intA[0] = 0;
    				intA[1] = 1;
    				intA[2] = -3;
    				intA[3] = 4;
    			
    		// for each ci, find the cj which also is a key and added up
    			for(HHcfMax::iterator hi = HHcf.begin(); hi != HHcf.end(); ++hi)
    			{
    				float maxS = 0;
    				out << hi->first << " " << hi->second.first << endl;
    				
    				for(int i=0; i < 4; ++i)
    				{
    					HHcfMax::iterator ml = HHcf.lower_bound(hi->second.first+intA[i]);
    					HHcfMax::iterator mu = HHcf.upper_bound(hi->second.first+intA[i]);
    				
    					for(HHcfMax::iterator mi = ml; mi != mu; ++mi)
    					{
    						out << mi->first << " " << mi->second.first << " " << mi->second.second << endl;
    						if (((mi->second.first == hi->first) || (mi->second.first+1 == hi->first) ||
    									(mi->second.first-3 == hi->first) || (mi->second.first+4 == hi->first))
    								&& (mi->second.second + hi->second.second > maxS))
    						{
    							maxS = mi->second.second + hi->second.second;
    						}
    					}
    				}
    				
    				for(int i=0; i<4; ++i)
    				{
    					HHcfMax::iterator ml = HHcf.lower_bound(hi->second.first+intA[i]);
    					HHcfMax::iterator mu = HHcf.upper_bound(hi->second.first+intA[i]);
    					
     					for(HHcfMax::iterator mi = ml; mi != mu; ++mi)
    					{
    						if (mi->second.second + hi->second.second == maxS)
    						{
    							HHpair.insert(make_pair(
    														make_pair(make_pair(hi->first, hi->second.first), make_pair(mi->first, mi->second.first)), maxS));
    							
    							out << "Loop4 key:" << hi->first << seq[hi->first].get_sym_1()<< " to: " << hi->second.first << seq[hi->second.first].get_sym_1()
																							 << " s:" << hi->second.second << endl;
    							out << "          " << mi->first << seq[mi->first].get_sym_1()<< " to: " << mi->second.first << seq[mi->second.first].get_sym_1()
																							 << " s:" << mi->second.second << " sum" << maxS << endl;
    						}
    					}
     				}
    			}
    	
    		// for each HHpair, keep the one that are symmetric
    		for(HHPair::iterator hpi= HHpair.begin(); hpi != HHpair.end(); ++hpi)
    		{
    			if (HHpair.find(make_pair(hpi->first.second, hpi->first.first)) == HHpair.end())
    			{
    				hpi->second = 0;
    			}
    		}

    		for(HHPair::iterator hpi= HHpair.begin(); hpi != HHpair.end(); ++hpi)
    		{
    			if (hpi->second != 0)
    			{
    				out << "Loop5 key:" << hpi->first.first.first << " to: " << hpi->first.first.second
    														<< hpi->first.second.first << " to: " << hpi->first.second.second
    														<< " S: " << hpi->second << endl;
    			}
    		}
    		
  if (_IC)
	{
	
	// find all possible IC-helix packing triples
	for(SS::const_iterator i = auto_ss.begin(); i != auto_ss.end(); ++i)
  {
  	if (i->second.get_type() == 1) //helix i
  	{
  		out << "Helix" << i->first << endl;
  		for(SS::const_iterator j = auto_ss.begin(); j != auto_ss.end(); ++j)
    	{
    		if ((j->second.get_type() == 1)) //helix j
    		{
    			out << "\t" << j->first << endl;
    			// find all possible helix packing triples
    			for(int ci = (*i->second.get_chain().begin()); ci <= *i->second.get_chain().rbegin(); ++ci)
    			{
    				if (HPSet.find(seq[ci].get_sym_1()) == HPSet.end())
    					continue;
											
    				for(int cj = *j->second.get_chain().begin(); cj <= *j->second.get_chain().rbegin(); ++cj)
    				{
    					if ((ICcfMax[make_pair(ci,cj)] != 0) && (ICcfMax[make_pair(ci,cj+1)] != 0) &&
    							(((cj+4 <= (*j->second.get_chain().rbegin())) && (ICcfMax[make_pair(ci,cj+4)] != 0)) &&
    							 ((cj-3 >= (*j->second.get_chain().begin())) && (ICcfMax[make_pair(ci,cj-3)] !=0))))
    					{
    					
    						if ((HPSet.find(seq[cj].get_sym_1()) != HPSet.end()) ||
    								(HPSet.find(seq[cj+1].get_sym_1()) != HPSet.end()) ||
    								((((cj+4 <= (*j->second.get_chain().rbegin())) &&
    										(ICcfMax[make_pair(ci,cj+4)] != 0) &&
    										(HPSet.find(seq[cj+4].get_sym_1()) != HPSet.end())) ||
    							    ((cj-3 >= (*j->second.get_chain().begin())) &&
    							    	(ICcfMax[make_pair(ci,cj-3)] !=0) &&
    							    	(HPSet.find(seq[cj-3].get_sym_1()) != HPSet.end())))))
    						{
    							float s = ICcfMax[make_pair(ci,cj)] + ICcfMax[make_pair(ci,cj+1)];
    							
									if (cj-3 >= (*j->second.get_chain().begin()))
    								s += ICcfMax[make_pair(ci,cj-3)];
									if (cj+4 <= (*j->second.get_chain().begin()))
    								s += ICcfMax[make_pair(ci,cj+4)];
    								    	    							   						
    							//HHPMap.insert(make_pair(ci, make_pair(make_pair(cj, cj+1), s)));
    							
    							out << "key:" << ci << seq[ci].get_sym_1()<< " to: " << cj << seq[cj].get_sym_1()
    																												<< " " << cj+1 << seq[cj+1].get_sym_1() << " s:" << s << endl;
    						}
    					}
    				}// end of for cj
    			}//end of for ci
    		}
    	}
    }
  }
  }
  // identify and return the potential helix-helix packing patterns
}
