/***************************************************************************
                          FindRingContact.cpp  -  description
                             -------------------
    begin                : Mon Mar 31 2003
    copyright            : (C) 2003 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#include <cmath>
#include <iostream>
#include <fstream>
#include "noesy_assign.h"

extern bool DEBUG;
//H, F, W, Y

extern bool isAr(char);
int NOESY_Assign::runRingContact()
{
	RingContact rc; rc.clear();
  for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); ++iter)
  {
    erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Find_RingPoss for " + (*iter)->label() + " ... ");
		(*iter)->Find_RingPoss(_par.ring, rc);
  }

  // find the complete set (>=2 for ring, and >=2 the other (not sure)
  RingSet ringSet; ringSet.clear();
  for(RingContact::iterator i = rc.begin(); i != rc.end(); ++i)
  {
  	ringSet.insert(i->first);
  }
	for(RingSet::iterator j = ringSet.begin(); j != ringSet.end(); ++j)
	{
		set<atom*, less<atom*> > ar; ar.clear(); // ri
		set<atom*, less<atom*> > ax; ax.clear(); // rj
		set<MomDad, less<MomDad> > ap; ap.clear();
		for(RingContact::iterator ji = rc.lower_bound(*j); ji != rc.upper_bound(*j); ++ji)
		{
			if (ji->second.second.first->get_index() < ji->second.second.second->get_index())
			{
				ap.insert(ji->second.second);
				ar.insert(ji->second.second.first);
				ax.insert(ji->second.second.second);		
			}
			else
			{
				ap.insert(make_pair(ji->second.second.second, ji->second.second.first));
				ar.insert(ji->second.second.second);
				ax.insert(ji->second.second.first);
			}	
			if (DEBUG) NOESY_Assign::outputLog() << "initRing " << j->first << " " << j->second << " " << ji->second.first << endl
						<< ji->second.second.first << endl << ji->second.second.second << endl;
		}

		// for atoms from rx, if have attached, requie the sym
		if (ap.size())
		{
  		if (isAr((*ar.begin())->get_sym_1()) && (ar.size() < 3))
  			rc.erase(*j);
  		if (isAr((*ax.begin())->get_sym_1()) && (ax.size() < 3))
  			rc.erase(*j);
  		if (ap.size() < 4)
  			rc.erase(*j);
		}

	}

	if (DEBUG)
	{
    for(RingContact::iterator i = rc.begin(); i != rc.end(); ++i)
    {
    	NOESY_Assign::outputLog() << "RING " << i->second.first << endl << i->second.second.first << endl << i->second.second.second << endl;
    }
	}

	// find the unique complete ones
	PeakMomDadMap pmdMap; pmdMap.clear();
	for(RingContact::iterator i = rc.begin(); i != rc.end(); ++i)
	{
		pmdMap.insert(i->second);
	}
	for(RingSet::iterator j = ringSet.begin(); j != ringSet.end(); ++j)
	{
		bool good = 1;
		for(RingContact::iterator ji = rc.lower_bound(*j); ji != rc.upper_bound(*j); ++ji)
		{
	   if (pmdMap.count(ji->second.first) !=1)
				good = 0;
		}
		if (!good) 
			rc.erase(*j);
	}
	if (DEBUG)
	{
    for(RingContact::iterator i = rc.begin(); i != rc.end(); ++i)
    {
    	NOESY_Assign::outputLog() << "UNIQUE RING " << i->second.first << endl << i->second.second.first << endl << i->second.second.second << endl;
    }
	}
	int num=0;
  for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); ++iter)
  {
    erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Assign_RingContact for " + (*iter)->label() + " ... ");
		num +=(*iter)->Assign_RingContact(_par.ring, rc);
  }
  return num;
}

