/***************************************************************************
                          FindSecondary.cpp  -  description
                             -------------------
    begin                : Mon Aug 21 2000
    copyright            : (C) 2000 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#include <cmath>
#include <iostream>
#include <fstream>
#include "noesy_assign.h"
#include "faces.h"
#include "EC.h"

#define fabs(x) fabs((double)(x))
#define abs(x) abs((double)(x))
#define fmod(x,y) fmod((double)(x), (double)(y))

extern bool DEBUG;
extern int faces[1000][1000];
extern Faces contactFaces;

void NOESY_Assign::addSS2AA(const SS& ss)
{
  // clear
	for(SEQM::iterator iter = seq.begin(); iter != seq.end(); ++iter)
	{
		iter->second.set_ss(0);
	}

  // addin
	for(SS::const_iterator i = ss.begin(); i != ss.end(); i++)
	{
		const Chain& chain = (*i).second.get_chain();

		for(Chain::const_iterator j = chain.begin();
			j != chain.end(); j++)
		{
			if (seq.count(*j))
				seq.find(*j)->second.set_ss((&(*i).second));
			else
				error("Secondary Structure is confict with seq file", "");
		}
	}
}

////////////////////////////////////////////////
// Find_secondary                             //
//       Combined analysis of CSI, NOE, J, NH //
////////////////////////////////////////////////

typedef multimap<pair<int, int>, pair<int,int>, less<pair<int, int> > > ssRRMap;
typedef multimap<pair<int, int>, pair<pair<int,int>, float>, less<pair<int, int> > > ssRRMapN;
typedef multimap<pair<int, int>, pair<pair<int,int>, pair<float,float> >, less<pair<int, int> > > ssRRMapNInt;

//typedef int FRQ;
typedef map<pair<int,int>, float, less<pair<int, int> > > RRFreq;
typedef map<pair<int,int>, pair<float, float>, less<pair<int, int> > > RRFreqInt;

typedef map<pair<int,int>, pair<float, pair<int, int> >, less<pair<int, int> > > RRFreqRR;
typedef map<int, int, less<int> > CSIMap;

void NOESY_Assign::Find_SSsequential(const CSIMap& indexFilter, const CSIMap& indexCaRaw, const CSIMap& indexCbRaw, const JvalList& Jval,
const NHSlowList& NHList, SS& auto_ssT, const Sec& parSec)
{
  NOESY_Assign::outputSec() << endl << "Combined Analysis of CSI, NOE, Jval and/or NH" << endl;
  NOESY_Assign::outputSec()         << "=============================================" << endl;

  ///////////////////////////////////////////////////////////////////////////
  // find all secoday NOE, including assinged, poss, or deg with assigned //
  ///////////////////////////////////////////////////////////////////////////
  NOEtypeListN SSNOE; SSNOE.clear();
  for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
    {
      if ((*iter)->get_tols().HX2 != 999)  // excluding the 999 dimention
	{
	  if (DEBUG)
	    NOESY_Assign::outputLog() << "SSNOE " << (*iter)->Type() << " " << (*iter)->get_tols().HX2 << endl;
	  (*iter)->Find_SSnoe(SSNOE, parSec);
	}
      //      (*iter)->Find_SSnoe(SSNOE, parSec);
    }


  if (DEBUG)
    NOESY_Assign::outputLog() << "In Secondary" << endl;

  if (DEBUG)
  {
    NOESY_Assign::outputLog() << "SSNOE:" << endl;
    for(NOEtypeListN::iterator i = SSNOE.begin(); i != SSNOE.end(); i++)
      {
	      //NOESY_Assign::outputLog() << (*i).first.first.first << "\t" << (*i).first.first.second
	      //	<< "\t" << (*i).first.second << "\n" << i->second.first << " " << i->second.second << endl;
      }

  }

	// start

  CSIMap ssFilter; ssFilter.clear();
  ssFilter.insert(indexFilter.begin(), indexFilter.end());
  int ssType = 0;

  //////////////////////////////////////////////
  // J is //only used to shrink the terminals   //
  // and remove all violated                  //
  //////////////////////////////////////////////

  // for residue
  JvalList::const_iterator oldJ; oldJ=JList.end();
  for(CSIMap::iterator i = ssFilter.begin(); i != ssFilter.end(); i++)
  {
    JvalList::const_iterator jl = JList.lower_bound(i->first);
    JvalList::const_iterator ju = JList.upper_bound(i->first);
    for(JvalList::const_iterator ji = jl; ji != ju; ji++)
    {
      if (ji->second.get_type() == J3HNHA)
      {
        if (((ji->second.get_lower() > parSec.jH) && (i->second == 1)) // helix but J > 6
            || ((ji->second.get_upper() < parSec.jS) && (i->second == -1))) // strand, but J < 7
        {
          NOESY_Assign::outputSec() << "Residue " << setw(5) << i->first << " is removed from ";
          if (i->second == -1)
          	NOESY_Assign::outputSec() << "extended-element ";
          else
          	NOESY_Assign::outputSec() << "helix ";
          NOESY_Assign::outputSec() << ": J3HnHa inconsistency" << endl;

         	i->second = 0;
          if (DEBUG)
            cout << ji->second << " " << ji->second.get_upper() << " " << ji->second.get_lower() << " " << i->first << " " << i->second << endl;
        }
      }
    }
  }


  if (DEBUG)
    for(CSIMap::iterator i = ssFilter.begin(); i != ssFilter.end(); i++)
    {
      NOESY_Assign::outputSec() << " J: " << i->first << "\t" << i->second << endl;
    }

  //---------------------------------------------------------------
  // combine CSI with NOEs, J and NH, decide the start and end
  //---------------------------------------------------------------

  ssType = 0;

  // shrink first
  // shrink N-terminal
	Atom_Index_List Nbin; Nbin.clear();
	Atom_Index_List Abin; Abin.clear();
	Atom_Index_List Bbin; Bbin.clear();

	for(Atom_List::iterator iter = NOESY_Assign::atoms().begin(); iter != NOESY_Assign::atoms().end(); ++iter)
	{
		if (((*iter).get_name() == "H") && (*iter).isatt_N())// && (*iter).isatt_C())
		{
   			Nbin.insert(make_pair((*iter).get_index(), &(*iter)));
    } else	if (((*iter).get_type() == 'H') && (*iter).isatt_C())
		{
		  if ((*iter).get_layer() == "A")
			  Abin.insert(make_pair((*iter).get_index(), &(*iter)));
    }
  }

  ssType = 0;

if (!NOESY_Assign::MINCON())
  for(CSIMap::iterator i = ssFilter.begin(); i != ssFilter.end(); i++)
  {
    if (i->second == ssType)
      continue;

    // b-strand
    else if ((i->second == -1) && (Nbin.find(i->first+1) != Nbin.end()) && (Abin.find(i->first) != Abin.end()) &&
                            ((SSNOE.find(make_pair(make_pair(i->first, i->first+1), HaHn)) == SSNOE.end())))
     {
      NOESY_Assign::outputSec() << "N-terminal residue " << setw(5) << i->first << " is removed from extended-element : no HaHn(i,i+1) NOE" << endl;
      i->second = 0;
    }
    else if ((i->second == 1) && (

                         ((SSNOE.find(make_pair(make_pair(i->first, i->first+4), HaHn)) == SSNOE.end())) // && // close to water
                        && ((SSNOE.find(make_pair(make_pair(i->first, i->first+2), HnHn)) == SSNOE.end())) //&&
                        ))
    {
      NOESY_Assign::outputSec() << "N-terminal residue " << setw(5) << i->first << " is removed from helix : no HaHn(i,i+4) or HnHn(i,i+2) NOE" << endl;
      i->second = 0;
    }
    ssType = i->second;
  }

  // shrink C-terminal
  ssType = 0;
if (!NOESY_Assign::MINCON())
  for(CSIMap::reverse_iterator i = ssFilter.rbegin(); i != ssFilter.rend(); i++)
  {
    if (i->second == ssType)
      continue;

    // b-strand
    if ((i->second == -1) && (Nbin.find(i->first) != Nbin.end()) && (Abin.find(i->first-1) != Abin.end()) &&
                            ((SSNOE.find(make_pair(make_pair(i->first-1, i->first), HaHn)) == SSNOE.end())))
    {
      NOESY_Assign::outputSec() << "C-terminal residue " << setw(5) << i->first << " is removed from extended-element : no HaHn(i-1, i) NOE" << endl;
      i->second = 0;
    }
    if ((i->second == 1) && (
                         ((SSNOE.find(make_pair(make_pair(i->first-4, i->first), HaHn)) == SSNOE.end()))
                        && ((SSNOE.find(make_pair(make_pair(i->first-2, i->first), HnHn)) == SSNOE.end()))
                       ))
    {
      NOESY_Assign::outputSec() << "C-terminal residue " << setw(5) << i->first << " is removed from helix : no HaHn(i-4,i) or HnHn(i-2,i) NOE" << endl;
      i->second = 0;
    }
    ssType = i->second;
  }

  if (DEBUG)
    for(CSIMap::iterator i = ssFilter.begin(); i != ssFilter.end(); i++)
    {
      NOESY_Assign::outputSec() << "NOE: " << i->first << "\t" << i->second << endl;
    }



  //////////////////////////////////////////////////////////////
  // NH                                                       //
  // strand every elements will have at least two slow NH     //
  // helix will have at least two slow NH and refine the C-Ter//
  //////////////////////////////////////////////////////////////

  if (!NHList.empty())
  {
    int index = ssFilter.begin()->first;
    ssType = 0;
    int count = 0;
    for(CSIMap::iterator i = ssFilter.begin(); i != ssFilter.end(); i++)
    {
      if (i->second != ssType)
      {
        if (((ssType == 1) && (count > 1)) || ((ssType == -1) && (count > 1))) // segments ends
        {
          if (ssType == 1) // helix
          {
            // possible C-teminals, refinement
            int j = i->first-1; // j is the c-terminal

            if (index-i->first > count+5)
                NOESY_Assign::outputSec() << "Warning: very few slow-NH found between " << index << " and " << i->first-1 << endl;
          }

          // nother for b-strand
        } else
        {
          // warning
          if (ssType)
            NOESY_Assign::outputSec() << "Warning: No slow-NH found between " << index << " and " << i->first-1 << endl;
        }
        count = 0;
        ssType = i->second;
        index = i->first;
      }
      if (NHList.find(i->first) != NHList.end())
      {
        count++;
      }
   }
  }

  if (DEBUG)
    for(CSIMap::iterator i = ssFilter.begin(); i != ssFilter.end(); i++)
    {
      NOESY_Assign::outputSec() << "NH: " << i->first << "\t" << i->second << endl;
    }

  //////////////////////////////////
  // NOE                          //
  // refine the N and C-terminals //
  // may extend or shrink         //
  //////////////////////////////////
  ssType = 0;

  // CSI need to be added in
  // extend C-terminal
  for(CSIMap::iterator i = ssFilter.begin(); i != ssFilter.end(); i++)
  {
    if (i->second == ssType)
      continue;

    int ca = 0; int cb = 0;
    if (indexCaRaw.find(i->first) == indexCaRaw.end())
      ca = 0;
    else
      ca = indexCaRaw.find(i->first)->second;
    if (indexCbRaw.find(i->first) == indexCbRaw.end())
      cb = 0;
    else
      cb = indexCbRaw.find(i->first)->second;

    int ca1 = 0; int cb1 = 0;
    if (indexCaRaw.find(i->first+1) == indexCaRaw.end())
      ca1 = 0;
    else
      ca1 = indexCaRaw.find(i->first+1)->second;
    if (indexCbRaw.find(i->first+1) == indexCbRaw.end())
      cb1 = 0;
    else
      cb1 = indexCbRaw.find(i->first+1)->second;

//103     L       NA      -1      1       NA      -1
//104     S       NA      -1      1       NA      -1
//105     Y       NA      -1      1       NA      -1
//106     V       NA      -1      1       NA      -1
//107     H       NA      0       1       NA      0  i
//108     N       NA      0       1       NA      0  i+1
//109     T       NA      -1      1       NA      -1 i+2
//110     P       0       0       0       NA      -1
//111     Y       NA      -1      1       NA      -1
//112     G       0       0       0       NA      -1

    int ca2 = 0; int cb2 = 0;
    if (indexCaRaw.find(i->first+2) == indexCaRaw.end())
      ca2 = 0;
    else
      ca2 = indexCaRaw.find(i->first+2)->second;
    if (indexCbRaw.find(i->first+2) == indexCbRaw.end())
      cb2 = 0;
    else
      cb2 = indexCbRaw.find(i->first+1)->second;

// ca - cab >=0, means possible helix
// for the above example, should keep 00, in order not to merge the two strands

    if ((ssType == -1) &&
	(((ca-cb < 0) && ((ca1-cb1 >=0) || (ca2-cb2>=0))) || ((ca-cb ==0) && (ca1-cb1 > 0))) // only allow -1+0 for extension or if the next one is a breaker - to avoid two strand merge into one strand
//	((ca-cb < 0) || ((ca-cb ==0) && (ca1-cb1 > 0))) // only allow -1+0 for extension or if the next one is a breaker - to avoid two strand merge into one strand
	&& ((NOESY_Assign::MINCON()) || ((SSNOE.find(make_pair(make_pair(i->first-1, i->first), HaHn)) != SSNOE.end()) &&
					 (SSNOE.find(make_pair(make_pair(i->first-1, i->first), HaHn))->second.second.first > 0.5))))
    {
			bool isOK=1;
			bool isIn = 0;
      JvalList::const_iterator jl = JList.lower_bound(i->first);
	    JvalList::const_iterator ju = JList.upper_bound(i->first);
    	for(JvalList::const_iterator ji = jl; ji != ju; ji++)
    	{
      	if (ji->second.get_type() == J3HNHA)
      	{
      		isIn = 1;
        	if (((ji->second.get_upper() < parSec.jS))) // strand, but J < 7
        	{
        		isOK = 0;
        	}
      		break;
      	}
      }

      if (isOK)
      {
	    	NOESY_Assign::outputSec() << "C-terminal Residue " << setw(5) << i->first << " is added temp as extened-element : Ca&b, NOE";
	    	if (isIn) NOESY_Assign::outputSec() << ",J";
	    	NOESY_Assign::outputSec() << " consistency" << endl;
	     	ssType = i->second;
      	i->second = -1;
       	continue;
      }
    }

    if ((NHList.empty()) && (ssType == 1) && ((indexCaRaw.find(i->first) == indexCaRaw.end()) || ((*indexCaRaw.find(i->first)).second != -1))
        && (((SSNOE.find(make_pair(make_pair(i->first-4, i->first), HaHn)) != SSNOE.end()) &&
             ((SSNOE.find(make_pair(make_pair(i->first-4, i->first), HaHn))->second.second.first > 0.5)))
         && ((SSNOE.find(make_pair(make_pair(i->first-2, i->first), HnHn)) != SSNOE.end()) &&
            (SSNOE.find(make_pair(make_pair(i->first-2, i->first), HnHn))->second.second.first > 0.5))))
     {
			bool isOK = 1, isIn = 0;
      JvalList::const_iterator jl = JList.lower_bound(i->first);
	    JvalList::const_iterator ju = JList.upper_bound(i->first);
    	for(JvalList::const_iterator ji = jl; ji != ju; ji++)
    	{
      	if (ji->second.get_type() == J3HNHA)
      	{
      		isIn = 1;
        	if (((ji->second.get_lower() > parSec.jH))) // helix but J > 6
          {
          	isOK = 0;
          }
      	}
      }

      if (isOK)
      {
	    	NOESY_Assign::outputSec() << "C-terminal Residue " << setw(5) << i->first << " is added temp as helix : Ca,NOE";
	    	if (isIn) NOESY_Assign::outputSec() << ",J";
	    	NOESY_Assign::outputSec() << " consistency" << endl;
	     	ssType = i->second;
      	i->second = 1;
				continue;
      }
    }
    ssType = i->second;
  }


  // extend N-terminal
  ssType = 0;
  for(CSIMap::reverse_iterator i = ssFilter.rbegin(); i != ssFilter.rend(); i++)
  {
    if (i->second == ssType)
      continue;

    // b-strand
    int ca = 0; int cb = 0;
    if (indexCaRaw.find(i->first) == indexCaRaw.end())
      ca = 0;
    else
      ca = indexCaRaw.find(i->first)->second;
    if (indexCbRaw.find(i->first) == indexCbRaw.end())
      cb = 0;
    else
      cb = indexCbRaw.find(i->first)->second;

    int ca1 = 0; int cb1 = 0;
    if (indexCaRaw.find(i->first+1) == indexCaRaw.end())
      ca1 = 0;
    else
      ca1 = indexCaRaw.find(i->first+1)->second;
    if (indexCbRaw.find(i->first+1) == indexCbRaw.end())
      cb1 = 0;
    else
      cb1 = indexCbRaw.find(i->first+1)->second;

    int ca2 = 0; int cb2 = 0;
    if (indexCaRaw.find(i->first+2) == indexCaRaw.end())
      ca2 = 0;
    else
      ca2 = indexCaRaw.find(i->first+2)->second;
    if (indexCbRaw.find(i->first+2) == indexCbRaw.end())
      cb2 = 0;
    else
      cb2 = indexCbRaw.find(i->first+1)->second;

    if ((ssType == -1) &&
	(((ca-cb < 0) && ((ca1-cb1 >=0) || (ca2-cb2>=0))) || ((ca-cb ==0) && (ca1-cb1 > 0)))
//	((ca-cb < 0) || ((ca-cb ==0) && (ca1-cb1 > 0)))
	&& ((NOESY_Assign::MINCON()) || ((SSNOE.find(make_pair(make_pair(i->first, i->first+1), HaHn)) != SSNOE.end()) &&
					 (SSNOE.find(make_pair(make_pair(i->first, i->first+1), HaHn))->second.second.first > 0.5))))
    {
			bool isOK=1;
			bool isIn = 0;
      JvalList::const_iterator jl = JList.lower_bound(i->first);
	    JvalList::const_iterator ju = JList.upper_bound(i->first);
    	for(JvalList::const_iterator ji = jl; ji != ju; ji++)
    	{
      	if (ji->second.get_type() == J3HNHA)
      	{
      		isIn = 1;
        	if (((ji->second.get_upper() < parSec.jS))) // strand, but J < 7
        	{
        		isOK = 0;
        	}
      		break;
      	}
      }
      if (isOK)
      {
	    	NOESY_Assign::outputSec() << "N-terminal Residue " << setw(5) << i->first << " is added temp as extened-element : Ca&b, NOE";
	    	if (isIn) NOESY_Assign::outputSec() << ",J";
	    	NOESY_Assign::outputSec() << " consistency" << endl;
		    ssType = i->second;
      	i->second = -1;
       	continue;
      }
    }

    if ((ssType == 1) && ((indexCaRaw.find(i->first) == indexCaRaw.end()) || ((*indexCaRaw.find(i->first)).second != -1))
                        && (((SSNOE.find(make_pair(make_pair(i->first, i->first+4), HaHn)) != SSNOE.end()) &&
                             (SSNOE.find(make_pair(make_pair(i->first, i->first+4), HaHn))->second.second.first > 0.5))
                         && ((SSNOE.find(make_pair(make_pair(i->first, i->first+2), HnHn)) != SSNOE.end()) &&
                            (SSNOE.find(make_pair(make_pair(i->first, i->first+2), HnHn))->second.second.first > 0.5))))
    {
			bool isOK = 1, isIn = 0;
      JvalList::const_iterator jl = JList.lower_bound(i->first);
	    JvalList::const_iterator ju = JList.upper_bound(i->first);
    	for(JvalList::const_iterator ji = jl; ji != ju; ji++)
    	{
      	if (ji->second.get_type() == J3HNHA)
      	{
      		isIn = 1;
        	if (((ji->second.get_lower() > parSec.jH))) // helix but J > 6
          {
          	isOK = 0;
          }
      	}
      }

      if (isOK)
      {
	    	NOESY_Assign::outputSec() << "N-terminal Residue " << setw(5) << i->first << " is added temp as helix : Ca,NOE";
	    	if (isIn) NOESY_Assign::outputSec() << ",J";
	    	NOESY_Assign::outputSec() << " consistency" << endl;
		    ssType = i->second;
      	i->second = 1;
       	continue;
      }
    }
    ssType = i->second;
  }

  ////////////////////////////
  // construct the auto_ssT //
  ////////////////////////////

  // the secondary structure element is added based on the combined analysis of CSI, J, NH and NOE
  Structure auto_stru;
  int type = (*ssFilter.begin()).second; int ID = 1;
  auto_stru.set_type(type);
  auto_stru.set_strand_ID(ID);
  for(CSIMap::iterator i = ssFilter.begin(); i != ssFilter.end(); i++)
  {
    if ((*i).second == type)
	  {
	    auto_stru.addinChain((*i).first);
	  }
    else
	  {
  	  auto_ssT.insert(make_pair(itoa(ID), auto_stru));
  	  auto_stru.erase();
  	  ID++;
  	  auto_stru.set_type((*i).second);
  	  auto_stru.set_strand_ID(ID);
  	  auto_stru.addinChain((*i).first);
  	  type = (*i).second;
	  }
  }
  auto_ssT.insert(make_pair(itoa(ID), auto_stru));
  addSS2AA(auto_ssT);

  //if (DEBUG)
  {
    NOESY_Assign::outputSec() << endl << "Secondary Struture is temporary defined as: " << endl << endl;
    for(SS::iterator i = auto_ssT.begin(); i != auto_ssT.end(); i++)
    {
      NOESY_Assign::outputSec() << "ID=" << (*i).first << "\t" << (*i).second << endl;
    }
    NOESY_Assign::outputSec() << endl;
  }

  // for helices, define the faces
  for(SS::iterator i = auto_ssT.begin(); i != auto_ssT.end(); i++)
  {
    if (i->second.get_type() == 1) // helix
    {
    	for(Chain::const_iterator j = i->second.get_chain().begin(); j != i->second.get_chain().end(); ++j)
     	{
      	if (i->second.get_chain().count(*j+2))
       	{
      		faces[*j][(*j+2)] = -1;
        	faces[(*j+2)][*j] = -1;
        }

	contactFaces.insertHelixLocal(*j, *j, 1);

      	if (i->second.get_chain().count(*j+1))
         		contactFaces.insertHelixLocal(*j, *j+1, 1);

        if (i->second.get_chain().count(*j+2))
         		contactFaces.insertHelixLocal(*j, *j+2, 1);
      	if (i->second.get_chain().count(*j+3))
         		contactFaces.insertHelixLocal(*j, *j+3, 1);
      	if (i->second.get_chain().count(*j+4))
         		contactFaces.insertHelixLocal(*j, *j+4, 1);
      	if (i->second.get_chain().count(*j+5))
         		contactFaces.insertHelixLocal(*j, *j+5, 1);

      	if (i->second.get_chain().count(*j+6))
         		contactFaces.insertHelixLocal(*j, *j+6, 1);

       if (i->second.get_chain().count(*j+7))
         		contactFaces.insertHelixLocal(*j, *j+7, 1);
      	if (i->second.get_chain().count(*j+8))
         		contactFaces.insertHelixLocal(*j, *j+8, 1);
      	if (i->second.get_chain().count(*j+9))
         		contactFaces.insertHelixLocal(*j, *j+9, 1);

				for(int k=10; k < (*(i->second.get_chain().rbegin()) - *(i->second.get_chain().begin()))+1; ++k)
				{
        				if (i->second.get_chain().count(*j+k))
           					contactFaces.insertHelixLocal(*j, *j+k, -1);
				}


				for(int k=0; k < (*(i->second.get_chain().rbegin()) - *(i->second.get_chain().begin()))+1; ++k)
				{
        				if (i->second.get_chain().count(*j+k))
           					contactFaces.insertHelix(*j, *j+k, 1);
				}

      }
    }
  }
}

void NOESY_Assign::Find_SSregister(const CSIMap& indexFilter, const CSIMap& indexCaRaw, const CSIMap& indexCbRaw, const JvalList& Jval,
const NHSlowList& NHList, map<HBOND, ECscore, less<HBOND> >& ECList, SS& auto_ss, deque<HBOND>& anti_registerF,  deque<HBOND>& para_registerF, const Sec& parSec)
{
	SS auto_ssT; auto_ssT = auto_ss;

  CSIMap ssFilter; ssFilter.clear();
  for(SS::iterator i = auto_ssT.begin(); i != auto_ssT.end(); i++)
  {
    	for(Chain::const_iterator j = i->second.get_chain().begin(); j != i->second.get_chain().end(); ++j)
     	{
      	ssFilter[*j] = i->second.get_type();
      }
  }

  /////////////////////////////////////
  // find the possiable secondary NOE //
  /////////////////////////////////////

  NOEtypeListN SSNOE; SSNOE.clear();

  for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
    {
      if ((*iter)->get_tols().HX2 != 999)  // excluding the 999 dimention
	{
	  if (DEBUG)
	    NOESY_Assign::outputLog() << "SSNOE " << (*iter)->label() << " " << (*iter)->get_tols().HX2 << endl;
	  (*iter)->Find_SSnoe(SSNOE, parSec);
	}
    }


  if (DEBUG)
    NOESY_Assign::outputLog() << "In Secondary-sheet" << endl;

  if (DEBUG)
  {
    NOESY_Assign::outputLog() << "SSNOE-sheet" << endl;
    for(NOEtypeListN::iterator i = SSNOE.begin(); i != SSNOE.end(); i++)
      {
	      NOESY_Assign::outputLog() << (*i).first.first.first << "\t" << (*i).first.first.second
					<< "\t" << (*i).first.second << "\n" << i->second.first << " " << i->second.second.first << " " << i->second.second.second << endl;
      }

  }

	/////////////////////////////////
  // SHEET, find the register    //
  /////////////////////////////////

  ssRRMapNInt HaHa_R; HaHa_R.erase(HaHa_R.begin(), HaHa_R.end()); // HaHa
  ssRRMapNInt HnHn_R; HnHn_R.erase(HnHn_R.begin(), HnHn_R.end()); // HnHn
  ssRRMapNInt HaHn_R; HaHn_R.erase(HaHn_R.begin(), HaHn_R.end()); // HaHn
  ssRRMapNInt HnHa_R; HnHa_R.erase(HnHa_R.begin(), HnHa_R.end()); // HnHa
	ssRRMapNInt EC_R; EC_R.erase(EC_R.begin(), EC_R.end()); // EC

  // From the Assigned NOEs find the register
  RRFreq anti_register; anti_register.clear();
  RRFreq para_register; para_register.clear();
  for(SS::iterator i = auto_ssT.begin(); i != auto_ssT.end(); i++)
    for(SS::iterator j = i; j != auto_ssT.end(); j++)
      {
	      if ((j != i) && ((*i).second.get_type() == -1) && ((*j).second.get_type() == -1))
	      {
	        for(Chain::iterator ic = (*i).second.get_chain().begin(); ic != (*i).second.get_chain().end(); ic++)
		      for(Chain::iterator jc = (*j).second.get_chain().begin(); jc != (*j).second.get_chain().end(); jc++)
		      {

						//EC
						if (ECList.find(make_pair((*ic), (*jc))) != ECList.end())
		          EC_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jc)), make_pair(ECList.find(make_pair((*ic), (*jc)))->second.weight, 0))));

						//noes
		        if (SSNOE.find(make_pair(make_pair((*ic), (*jc)), HaHa)) != SSNOE.end())
		          HaHa_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jc)), SSNOE.find(make_pair(make_pair((*ic), (*jc)), HaHa))->second.second)));

			//make_pair(SSNOE.find(make_pair(make_pair((*ic), (*jc)), HaHa))->second.second.first, SSNOE.find(make_pair(make_pair((*ic), (*jc)), HaHa))->second.second.second))));
			if (SSNOE.find(make_pair(make_pair((*ic), (*jc)), HnHn)) != SSNOE.end())
		          HnHn_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jc)), SSNOE.find(make_pair(make_pair((*ic), (*jc)), HnHn))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*ic), (*jc)), HnHn))->second.second, SSNOE.find(make_pair(make_pair((*ic), (*jc)), HnHn))->second.second.second))));
		        if (SSNOE.find(make_pair(make_pair((*ic), (*jc)), HaHn)) != SSNOE.end())
		          HaHn_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jc)), SSNOE.find(make_pair(make_pair((*ic), (*jc)), HaHn))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*ic), (*jc)), HaHn))->second.second, SSNOE.find(make_pair(make_pair((*ic), (*jc)), HaHn))->second.second.second))));
		        if (SSNOE.find(make_pair(make_pair((*ic), (*jc)), HnHa)) != SSNOE.end())
		          HnHa_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jc)), SSNOE.find(make_pair(make_pair((*ic), (*jc)), HnHa))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*ic), (*jc)), HnHa))->second.second, SSNOE.find(make_pair(make_pair((*ic), (*jc)), HaHn))->second.second.second))));

            // symmetry one
						//EC
						if (ECList.find(make_pair((*jc), (*ic))) != ECList.end())
							EC_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jc)), make_pair(ECList.find(make_pair((*jc), (*ic)))->second.weight, 0))));

						//noes
		        if (SSNOE.find(make_pair(make_pair((*jc), (*ic)), HaHa)) != SSNOE.end())
		          HaHa_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jc)), SSNOE.find(make_pair(make_pair((*jc), (*ic)), HaHa))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*jc), (*ic)), HaHa))->second.second, SSNOE.find(make_pair(make_pair((*jc), (*ic)), HaHa))->second.second.second))));
		        if (SSNOE.find(make_pair(make_pair((*jc), (*ic)), HnHn)) != SSNOE.end())
		          HnHn_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jc)), SSNOE.find(make_pair(make_pair((*jc), (*ic)), HnHn))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*jc), (*ic)), HnHn))->second.second, SSNOE.find(make_pair(make_pair((*jc), (*ic)), HnHn))->second.second.second))));
		        if (SSNOE.find(make_pair(make_pair((*jc), (*ic)), HaHn)) != SSNOE.end())
		          HnHa_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jc)), SSNOE.find(make_pair(make_pair((*jc), (*ic)), HaHn))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*jc), (*ic)), HaHn))->second.second, SSNOE.find(make_pair(make_pair((*jc), (*ic)), HaHn))->second.second.second))));
		        if (SSNOE.find(make_pair(make_pair((*jc), (*ic)), HnHa)) != SSNOE.end())
		          HaHn_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jc)), SSNOE.find(make_pair(make_pair((*jc), (*ic)), HnHa))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*jc), (*ic)), HnHa))->second.second, SSNOE.find(make_pair(make_pair((*jc), (*ic)), HnHa))->second.second.second))));
		      }
	      }
	    }


  //extend one-residue at N,C-terminal
	///////////////////////
	// i-1 and i+1
	//////////////////////

	// assigned
  for(SS::iterator i = auto_ssT.begin(); i != auto_ssT.end(); i++)
    for(SS::iterator j = i; j != auto_ssT.end(); j++)
      {
	      if ((j != i) && ((*i).second.get_type() == -1) && ((*j).second.get_type() == -1))
	      {

					//i-1
	              Chain::iterator ic = i->second.get_chain().begin();

		      for(Chain::iterator jc = (*j).second.get_chain().begin(); jc != (*j).second.get_chain().end(); jc++)
		      {
		      	if (abs(*ic-1-*jc)<6)    // avoid close contact that fp FGF 76-81 < 6 should not be used
		      		continue;

			if ((seq.find(*ic-1) != seq.end()) && (seq.find(*ic-2) != seq.end()))
			{
				// a possible bulge, do not include its evidence
				if (seq[*ic-2].get_ss()->get_type() == -1)
				  continue;
			}

			//EC
			if (ECList.find(make_pair((*ic)-1, (*jc))) != ECList.end())
				EC_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic)-1, (*jc)), make_pair(ECList.find(make_pair((*ic)-1, (*jc)))->second.weight, 0))));


		        if (SSNOE.find(make_pair(make_pair((*ic)-1, (*jc)), HaHa)) != SSNOE.end())
		          HaHa_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic)-1, (*jc)), SSNOE.find(make_pair(make_pair((*ic)-1, (*jc)), HaHa))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*ic)-1, (*jc)), HaHa))->second.second, SSNOE.find(make_pair(make_pair((*ic)-1, (*jc)), HaHa))->second.second.second))));
			if (SSNOE.find(make_pair(make_pair((*ic)-1, (*jc)), HnHn)) != SSNOE.end())
		          HnHn_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic)-1, (*jc)), SSNOE.find(make_pair(make_pair((*ic)-1, (*jc)), HnHn))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*ic)-1, (*jc)), HnHn))->second.second, SSNOE.find(make_pair(make_pair((*ic)-1, (*jc)), HnHn))->second.second.second))));
		        if (SSNOE.find(make_pair(make_pair((*ic)-1, (*jc)), HaHn)) != SSNOE.end())
		          HaHn_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic)-1, (*jc)), SSNOE.find(make_pair(make_pair((*ic)-1, (*jc)), HaHn))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*ic)-1, (*jc)), HaHn))->second.second, SSNOE.find(make_pair(make_pair((*ic)-1, (*jc)), HaHn))->second.second.second))));
			if (SSNOE.find(make_pair(make_pair((*ic)-1, (*jc)), HnHa)) != SSNOE.end())
			  HnHa_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic)-1, (*jc)), SSNOE.find(make_pair(make_pair((*ic)-1, (*jc)), HnHa))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*ic)-1, (*jc)), HnHa))->second.second, SSNOE.find(make_pair(make_pair((*ic)-1, (*jc)), HnHa))->second.second.second))));

            // symmetry one
						if (ECList.find(make_pair((*jc), (*ic)-1)) != ECList.end())
							EC_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic)-1, (*jc)), make_pair(ECList.find(make_pair((*jc), (*ic)-1))->second.weight, 0))));


		        if (SSNOE.find(make_pair(make_pair((*jc), (*ic)-1), HaHa)) != SSNOE.end())
		          HaHa_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic)-1, (*jc)), SSNOE.find(make_pair(make_pair((*jc), (*ic)-1), HaHa))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*jc), (*ic)-1), HaHa))->second.second, SSNOE.find(make_pair(make_pair((*jc), (*ic)-1), HaHa))->second.second.second))));
		        if (SSNOE.find(make_pair(make_pair((*jc), (*ic)-1), HnHn)) != SSNOE.end())
		          HnHn_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic)-1, (*jc)), SSNOE.find(make_pair(make_pair((*jc), (*ic)-1), HnHn))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*jc), (*ic)-1), HnHn))->second.second, SSNOE.find(make_pair(make_pair((*jc), (*ic)-1), HnHn))->second.second.second))));
		        if (SSNOE.find(make_pair(make_pair((*jc), (*ic)-1), HaHn)) != SSNOE.end())
		          HnHa_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic)-1, (*jc)), SSNOE.find(make_pair(make_pair((*jc), (*ic)-1), HaHn))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*jc), (*ic)-1), HaHn))->second.second, SSNOE.find(make_pair(make_pair((*jc), (*ic)-1), HaHn))->second.second.second))));
		        if (SSNOE.find(make_pair(make_pair((*jc), (*ic)-1), HnHa)) != SSNOE.end())
		          HaHn_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic)-1, (*jc)), SSNOE.find(make_pair(make_pair((*jc), (*ic)-1), HnHa))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*jc), (*ic)-1), HnHa))->second.second, SSNOE.find(make_pair(make_pair((*jc), (*ic)-1), HnHa))->second.second.second))));
		      }

					//i+1
	        Chain::reverse_iterator iic = i->second.get_chain().rbegin();

		      for(Chain::iterator jc = (*j).second.get_chain().begin(); jc != (*j).second.get_chain().end(); jc++)
		      {
		      	if (abs(*iic+1-*jc) < 6) // was 6 before
		      		continue;

			if ((seq.find(*iic+1) != seq.end()) && (seq.find(*iic+2) != seq.end()))
			{
				// a possible bulge
				if (seq[*iic+2].get_ss()->get_type() == -1)
				  continue;
			}

			//EC
			if (ECList.find(make_pair((*iic)+1, (*jc))) != ECList.end())
				EC_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*iic)+1, (*jc)), make_pair(ECList.find(make_pair((*iic)+1, (*jc)))->second.weight, 0))));


		        if (SSNOE.find(make_pair(make_pair((*iic)+1, (*jc)), HaHa)) != SSNOE.end())
		          HaHa_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*iic)+1, (*jc)),SSNOE.find(make_pair(make_pair((*iic)+1, (*jc)), HaHa))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*iic)+1, (*jc)), HaHa))->second.second, SSNOE.find(make_pair(make_pair((*iic)+1, (*jc)), HaHa))->second.second.second))));
		        if (SSNOE.find(make_pair(make_pair((*iic)+1, (*jc)), HnHn)) != SSNOE.end())
		          HnHn_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*iic)+1, (*jc)),SSNOE.find(make_pair(make_pair((*iic)+1, (*jc)), HnHn))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*iic)+1, (*jc)), HnHn))->second.second, SSNOE.find(make_pair(make_pair((*iic)+1, (*jc)), HnHn))->second.second.second))));
		        if (SSNOE.find(make_pair(make_pair((*iic)+1, (*jc)), HaHn)) != SSNOE.end())
		          HaHn_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*iic)+1, (*jc)),SSNOE.find(make_pair(make_pair((*iic)+1, (*jc)), HaHn))->second.second)));
			// make_pair(SSNOE.find(make_pair(make_pair((*iic)+1, (*jc)), HaHn))->second.second, SSNOE.find(make_pair(make_pair((*iic)+1, (*jc)), HaHn))->second.second.second))));
		        if (SSNOE.find(make_pair(make_pair((*iic)+1, (*jc)), HnHa)) != SSNOE.end())
			  {
		          HnHa_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*iic)+1, (*jc)),SSNOE.find(make_pair(make_pair((*iic)+1, (*jc)), HnHa))->second.second)));
			  //make_pair(SSNOE.find(make_pair(make_pair((*iic)+1, (*jc)), HnHa))->second.second, SSNOE.find(make_pair(make_pair((*iic)+1, (*jc)), HnHa))->second.second.second))));
			  //if (DEBUG) NOESY_Assign::outputSec() << "hnha_r " << *iic+1 << "-" << (*jc) << " " << SSNOE.find(make_pair(make_pair((*iic)+1, (*jc)), HnHa))->second.second << endl;
			  }

            // symmetry one

						if (ECList.find(make_pair((*jc), (*iic)+1)) != ECList.end())
							EC_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*iic)+1, (*jc)), make_pair(ECList.find(make_pair((*jc), (*iic)+1))->second.weight, 0))));


		        if (SSNOE.find(make_pair(make_pair((*jc), (*iic)+1), HaHa)) != SSNOE.end())
		          HaHa_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*iic)+1, (*jc)),SSNOE.find(make_pair(make_pair((*jc), (*iic)+1), HaHa))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*jc), (*iic)+1), HaHa))->second.second, SSNOE.find(make_pair(make_pair((*jc), (*iic)+1), HaHa))->second.second.second))));
		        if (SSNOE.find(make_pair(make_pair((*jc), (*iic)+1), HnHn)) != SSNOE.end())
		          HnHn_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*iic)+1, (*jc)),SSNOE.find(make_pair(make_pair((*jc), (*iic)+1), HnHn))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*jc), (*iic)+1), HnHn))->second.second, SSNOE.find(make_pair(make_pair((*jc), (*iic)+1), HnHn))->second.second.second))));
		        if (SSNOE.find(make_pair(make_pair((*jc), (*iic)+1), HaHn)) != SSNOE.end())
			  {
			    HnHa_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*iic)+1, (*jc)),SSNOE.find(make_pair(make_pair((*jc), (*iic)+1), HnHa))->second.second)));
			    //make_pair(SSNOE.find(make_pair(make_pair((*jc), (*iic)+1), HaHn))->second.second, SSNOE.find(make_pair(make_pair((*jc), (*iic)+1), HaHn))->second.second.second))));
			  //if (DEBUG) NOESY_Assign::outputSec() << "hnha_r " << *iic+1 << "-" << (*jc) << " " << SSNOE.find(make_pair(make_pair((*jc), (*iic)+1), HnHa))->second.second << endl;
			  }
		        if (SSNOE.find(make_pair(make_pair((*jc), (*iic)+1), HnHa)) != SSNOE.end())
		          HaHn_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*iic)+1, (*jc)),SSNOE.find(make_pair(make_pair((*jc), (*iic)+1), HnHa))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*jc), (*iic)+1), HnHa))->second.second, SSNOE.find(make_pair(make_pair((*jc), (*iic)+1), HnHa))->second.second.second))));
		      }

		      //j-1
	        Chain::iterator jc = j->second.get_chain().begin();

		      for(Chain::iterator ic = (*i).second.get_chain().begin(); ic != (*i).second.get_chain().end(); ic++)
		      {
		      	if (abs(*jc-1-*ic)<6) // was 6 before
		      		continue;

			if ((seq.find(*jc-1) != seq.end()) && (seq.find(*jc-2) != seq.end()))
			{
				// a possible bulge
				if (seq[*jc-2].get_ss()->get_type() == -1)
				  continue;
			}


			//EC
			if (ECList.find(make_pair((*ic), (*jc)-1)) != ECList.end())
				EC_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jc)-1), make_pair(ECList.find(make_pair((*ic), (*jc)-1))->second.weight, 0))));

		        if (SSNOE.find(make_pair(make_pair((*ic), (*jc)-1), HaHa)) != SSNOE.end())
		          HaHa_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jc)-1),SSNOE.find(make_pair(make_pair((*ic), (*jc)-1), HaHa))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*ic), (*jc)-1), HaHa))->second.second, SSNOE.find(make_pair(make_pair((*ic), (*jc)-1), HaHa))->second.second.second))));
		        if (SSNOE.find(make_pair(make_pair((*ic), (*jc)-1), HnHn)) != SSNOE.end())
		          HnHn_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jc)-1),SSNOE.find(make_pair(make_pair((*ic), (*jc)-1), HnHn))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*ic), (*jc)-1), HnHn))->second.second, SSNOE.find(make_pair(make_pair((*ic), (*jc)-1), HnHn))->second.second.second))));
		        if (SSNOE.find(make_pair(make_pair((*ic), (*jc)-1), HaHn)) != SSNOE.end())
		          HaHn_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jc)-1),SSNOE.find(make_pair(make_pair((*ic), (*jc)-1), HaHn))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*ic), (*jc)-1), HaHn))->second.second, SSNOE.find(make_pair(make_pair((*ic), (*jc)-1), HaHn))->second.second.second))));
		        if (SSNOE.find(make_pair(make_pair((*ic), (*jc)-1), HnHa)) != SSNOE.end())
		          HnHa_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jc)-1),SSNOE.find(make_pair(make_pair((*ic), (*jc)-1), HnHa))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*ic), (*jc)-1), HnHa))->second.second, SSNOE.find(make_pair(make_pair((*ic), (*jc)-1), HnHa))->second.second.second))));

            // symmetry one
						if (ECList.find(make_pair((*jc)-1, (*ic))) != ECList.end())
							EC_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic)-1, (*jc)-1), make_pair(ECList.find(make_pair((*jc)-1, (*ic)))->second.weight, 0))));


		        if (SSNOE.find(make_pair(make_pair((*jc)-1, (*ic)), HaHa)) != SSNOE.end())
		          HaHa_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jc)-1),SSNOE.find(make_pair(make_pair((*jc)-1, (*ic)), HaHa))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*jc)-1, (*ic)), HaHa))->second.second, SSNOE.find(make_pair(make_pair((*jc)-1, (*ic)), HaHa))->second.second.second))));
		        if (SSNOE.find(make_pair(make_pair((*jc)-1, (*ic)), HnHn)) != SSNOE.end())
		          HnHn_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jc)-1),SSNOE.find(make_pair(make_pair((*jc)-1, (*ic)), HnHn))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*jc)-1, (*ic)), HnHn))->second.second, SSNOE.find(make_pair(make_pair((*jc)-1, (*ic)), HnHn))->second.second.second))));
		        if (SSNOE.find(make_pair(make_pair((*jc)-1, (*ic)), HaHn)) != SSNOE.end())
		          HnHa_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jc)-1),SSNOE.find(make_pair(make_pair((*jc)-1, (*ic)), HaHn))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*jc)-1, (*ic)), HaHn))->second.second, SSNOE.find(make_pair(make_pair((*jc)-1, (*ic)), HaHn))->second.second.second))));
		        if (SSNOE.find(make_pair(make_pair((*jc)-1, (*ic)), HnHa)) != SSNOE.end())
		          HaHn_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jc)-1),SSNOE.find(make_pair(make_pair((*jc)-1, (*ic)), HnHa))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*jc)-1, (*ic)), HnHa))->second.second, SSNOE.find(make_pair(make_pair((*jc)-1, (*ic)), HnHa))->second.second.second))));
		      }

					//j+1
	        Chain::reverse_iterator jjc = j->second.get_chain().rbegin();

		      for(Chain::iterator ic = (*i).second.get_chain().begin(); ic != (*i).second.get_chain().end(); ic++)
		      {
		      	if (abs(*jjc+1-*ic) < 6) // was 6 before
		      		continue;

			if ((seq.find(*jjc+1) != seq.end()) && (seq.find(*jjc+2) != seq.end()))
			{
				// a possible bulge
				if (seq[*jjc+2].get_ss()->get_type() == -1)
				  continue;
			}

			if (ECList.find(make_pair((*ic), (*jjc)+1)) != ECList.end())
				EC_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jjc)+1), make_pair(ECList.find(make_pair((*ic)-1, (*jjc)+1))->second.weight, 0))));

		        if (SSNOE.find(make_pair(make_pair((*ic), (*jjc)+1), HaHa)) != SSNOE.end())
		          HaHa_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jjc)+1),SSNOE.find(make_pair(make_pair((*ic), (*jjc)+1), HaHa))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*ic), (*jjc)+1), HaHa))->second.second, SSNOE.find(make_pair(make_pair((*ic), (*jjc)+1), HaHa))->second.second.second))));
		        if (SSNOE.find(make_pair(make_pair((*ic), (*jjc)+1), HnHn)) != SSNOE.end())
		          HnHn_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jjc)+1),SSNOE.find(make_pair(make_pair((*ic), (*jjc)+1), HnHn))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*ic), (*jjc)+1), HnHn))->second.second, SSNOE.find(make_pair(make_pair((*ic), (*jjc)+1), HnHn))->second.second.second))));
		        if (SSNOE.find(make_pair(make_pair((*ic), (*jjc)+1), HaHn)) != SSNOE.end())
		          HaHn_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jjc)+1),SSNOE.find(make_pair(make_pair((*ic), (*jjc)+1), HaHn))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*ic), (*jjc)+1), HaHn))->second.second, SSNOE.find(make_pair(make_pair((*ic), (*jjc)+1), HaHn))->second.second.second))));
		        if (SSNOE.find(make_pair(make_pair((*ic), (*jjc)+1), HnHa)) != SSNOE.end())
			  {
			        	    HnHa_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jjc)+1),SSNOE.find(make_pair(make_pair((*ic), (*jjc)+1), HnHa))->second.second)));
			    // make_pair(SSNOE.find(make_pair(make_pair((*ic), (*jjc)+1), HnHa))->second.second, SSNOE.find(make_pair(make_pair((*ic), (*jjc)+1), HnHa))->second.second.second))));
			  //if (DEBUG) NOESY_Assign::outputSec() << "hnha_r " << *ic << "-" << (*jjc)+1 << " " << SSNOE.find(make_pair(make_pair((*ic), (*jjc)+1), HnHa))->second.second << endl;
			  }

            // symmetry one
						if (ECList.find(make_pair((*jjc)+1, (*ic))) != ECList.end())
							EC_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jjc)+1), make_pair(ECList.find(make_pair((*jjc)+1, (*ic)))->second.weight, 0))));


		        if (SSNOE.find(make_pair(make_pair((*jjc)+1, (*ic)), HaHa)) != SSNOE.end())
		          HaHa_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jjc)+1),SSNOE.find(make_pair(make_pair((*jjc)+1, (*ic)), HaHa))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*jjc)+1, (*ic)), HaHa))->second.second, SSNOE.find(make_pair(make_pair((*jjc)+1, (*ic)), HaHa))->second.second.second))));
		        if (SSNOE.find(make_pair(make_pair((*jjc)+1, (*ic)), HnHn)) != SSNOE.end())
		          HnHn_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jjc)+1),SSNOE.find(make_pair(make_pair((*jjc)+1, (*ic)), HnHn))->second.second)));
			//make_pair(SSNOE.find(make_pair(make_pair((*jjc)+1, (*ic)), HnHn))->second.second, SSNOE.find(make_pair(make_pair((*jjc)+1, (*ic)), HnHn))->second.second.second))));
		        if (SSNOE.find(make_pair(make_pair((*jjc)+1, (*ic)), HaHn)) != SSNOE.end())
			  {
		          HnHa_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jjc)+1),SSNOE.find(make_pair(make_pair((*jjc)+1, (*ic)), HaHn))->second.second)));
			  //make_pair(SSNOE.find(make_pair(make_pair((*jjc)+1, (*ic)), HaHn))->second.second, SSNOE.find(make_pair(make_pair((*jjc)+1, (*ic)), HaHn))->second.second.second))));
			  //if (DEBUG) NOESY_Assign::outputSec() << "hnha_r " << *ic << "-" << (*jjc)+1 << " " << SSNOE.find(make_pair(make_pair((*jjc)+1, (*ic)), HaHn))->second.second << e
			  }
		        if (SSNOE.find(make_pair(make_pair((*jjc)+1, (*ic)), HnHa)) != SSNOE.end())
			  {
		          HaHn_R.insert(make_pair(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()), make_pair(make_pair((*ic), (*jjc)+1),SSNOE.find(make_pair(make_pair((*jjc)+1, (*ic)), HnHa))->second.second)));
			  //make_pair(SSNOE.find(make_pair(make_pair((*jjc)+1, (*ic)), HnHa))->second.second, SSNOE.find(make_pair(make_pair((*jjc)+1, (*ic)), HnHa))->second.second.second))));
			  //if (DEBUG) NOESY_Assign::outputSec() << "hahn_r " << *ic << "-" << (*jjc)+1 << " " << SSNOE.find(make_pair(make_pair((*jjc)+1, (*ic)), HnHa))->second.second << endl;
			  }
		      }
	      }
      }



  // for each possible strands that aligned, find the consistent pattern, delete the inconsistent and
  // find the buldge also??? --- not supported

  multimap<pair<int,int>, pair<pair<int,int>, float>, less<pair<int, int> > > antiRR_frq; antiRR_frq.clear();
  multimap<pair<int, int>, pair<pair<int,int>, float>, less<pair<int, int> > > paraRR_frq; paraRR_frq.clear();

  NOESY_Assign::outputSec() << "All Poss Registers:" << endl << endl;
  for(SS::iterator i = auto_ssT.begin(); i != auto_ssT.end(); i++)
    for(SS::iterator j = i; j != auto_ssT.end(); j++)
    {
    	if ((j != i) && ((*i).second.get_type() == -1) && ((*j).second.get_type() == -1)) // for b-strands
      {

   	    ssRRMapNInt::iterator HaHa_R_il = HaHa_R.lower_bound(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()));
   	    ssRRMapNInt::iterator HaHa_R_iu = HaHa_R.upper_bound(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()));

   	    ssRRMapNInt::iterator HnHn_R_il = HnHn_R.lower_bound(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()));
   	    ssRRMapNInt::iterator HnHn_R_iu = HnHn_R.upper_bound(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()));

   	    ssRRMapNInt::iterator HaHn_R_il = HaHn_R.lower_bound(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()));
   	    ssRRMapNInt::iterator HaHn_R_iu = HaHn_R.upper_bound(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()));

   	    ssRRMapNInt::iterator HnHa_R_il = HnHa_R.lower_bound(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()));
   	    ssRRMapNInt::iterator HnHa_R_iu = HnHa_R.upper_bound(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()));

				ssRRMapNInt::iterator EC_R_il = EC_R.lower_bound(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()));
   	    ssRRMapNInt::iterator EC_R_iu = EC_R.upper_bound(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()));


    	  ///////////////////////////////////////
    	  // third, use the possible method    //
    		///////////////////////////////////////
    		RRFreqInt RR_frq;

    		//-------------------------------------
    		// case 1. if anti, find the consistent
    		//-------------------------------------
    		RR_frq.erase(RR_frq.begin(), RR_frq.end());
    		bool isfar = 0;
    		bool isAd = 0;

    		for(ssRRMapNInt::iterator HnHn_R_i = HnHn_R_il; HnHn_R_i != HnHn_R_iu; HnHn_R_i++)
    		{
    		  //if (DEBUG) NOESY_Assign::outputSec() << "hn hn" << (*HnHn_R_i).second.first.first << "*" << (*HnHn_R_i).second.first.second << " " << HnHn_R_i->second.first << " " << HnHn_R_i->second.second.first << " " << HnHn_R_i->second.second.second << endl;
    		  RRFreqInt::iterator RR_i;

		  		int factor = 2; // same for para

    		  for(RR_i = RR_frq.begin(); RR_i != RR_frq.end(); RR_i++)
    		  {
    		    if ((((*RR_i).first.first - ((*HnHn_R_i).second.first.first)) == ((*HnHn_R_i).second.first.second - (*RR_i).first.second)) &&
    			          (fmod(abs((*RR_i).first.first - (*HnHn_R_i).second.first.first), 2) == 0))
    			  {
			    if (!NOESY_Assign::MINCON() ||  (HnHn_R_i->second.second.second < 3.8))
			      {
				//if (DEBUG) NOESY_Assign::outputSec() << " to:" << RR_i->first << " " << RR_i->second.first << " " << RR_i->second.second << endl;
				RR_i->second.first += HnHn_R_i->second.second.first/factor; //score
				RR_i->second.second += HnHn_R_i->second.second.second; //Intensity --> distance
				//if (DEBUG) NOESY_Assign::outputSec() << "now " << RR_i->second.first << " " << RR_i->second.second << endl;

				break;
			      }
    			  }
    		  }
    		  if (RR_i == RR_frq.end()) // find the right register
    		  {
		    if (!NOESY_Assign::MINCON() ||  (HnHn_R_i->second.second.second < 3.8))
		      {
			RR_frq.insert(make_pair(make_pair((*HnHn_R_i).second.first.first, (*HnHn_R_i).second.first.second), make_pair(HnHn_R_i->second.second.first/factor, HnHn_R_i->second.second.second))); //6));

			//if (DEBUG) NOESY_Assign::outputSec() << "new " << HnHn_R_i->second.second.first/factor << endl;
		      }

    		   }
    		}


				//EC
				for(ssRRMapNInt::iterator EC_R_i = EC_R_il; EC_R_i != EC_R_iu; EC_R_i++)
    		{
    		  if (DEBUG) NOESY_Assign::outputSec() << "EC " << (*EC_R_i).second.first.first << "*" << (*EC_R_i).second.first.second << " " << EC_R_i->second.second << endl;
    		  RRFreqInt::iterator RR_i;

		  		int factor = 2; // ? same for para

    		  for(RR_i = RR_frq.begin(); RR_i != RR_frq.end(); RR_i++)
    		  {
    		    if ((((*RR_i).first.first - ((*EC_R_i).second.first.first)) == ((*EC_R_i).second.first.second - (*RR_i).first.second))) //&&
    			         // (fmod(abs((*RR_i).first.first - (*EC_R_i).second.first.first), 2) == 0))
    			  {
							if (DEBUG) NOESY_Assign::outputSec() << " to:" << RR_i->first.first << " " << RR_i->first.second << endl;
							RR_i->second.first += EC_R_i->second.second.first/factor; //score
							RR_i->second.second += EC_R_i->second.second.second; //Intensity --> distance
							if (DEBUG) NOESY_Assign::outputSec() << "now " << RR_i->second.second << endl;
							break;
	    			}
    		  }

					if (RR_i == RR_frq.end()) // find the right register
    		  {
		    		//if (!NOESY_Assign::MINCON() ||  (HnHn_R_i->second.second.second < 3.8))
		      	{
								RR_frq.insert(make_pair(make_pair((*EC_R_i).second.first.first, (*EC_R_i).second.first.second), make_pair(EC_R_i->second.second.first/factor, EC_R_i->second.second.second))); //6));

								if (DEBUG) NOESY_Assign::outputSec() << "new " << EC_R_i->second.second.first/factor << endl;
		      }

    		   }
    		}

    		//isAd = 0;
    		for(ssRRMapNInt::iterator HaHa_R_i = HaHa_R_il; HaHa_R_i != HaHa_R_iu; HaHa_R_i++)
    		{
        		  RRFreqInt::iterator RR_i;
    		  for(RR_i = RR_frq.begin(); RR_i != RR_frq.end(); RR_i++)
    		  {
    			  if ((((*RR_i).first.first - ((*HaHa_R_i).second.first.first)) ==
    			      ((*HaHa_R_i).second.first.second - (*RR_i).first.second)) && (fmod(abs((*RR_i).first.first - (*HaHa_R_i).second.first.first), 2) == 1))
    			  {
			    RR_i->second.first += HaHa_R_i->second.second.first;
			    RR_i->second.second += HaHa_R_i->second.second.second;

    			    break;
    			  }
    		  }
    		  if (RR_i == RR_frq.end()) // find the right register
    		  {

    			  pair<int, int> tempRR;
    			  tempRR.first = (*HaHa_R_i).second.first.first;
    			  tempRR.second = (*HaHa_R_i).second.first.second;

    			  if (((*i).second.get_chain().find(tempRR.first+1) != (*i).second.get_chain().end()) &&
    			      ((*j).second.get_chain().find(tempRR.second-1) != (*j).second.get_chain().end()))
    			    RR_frq.insert(make_pair(make_pair(tempRR.first+1, tempRR.second-1), HaHa_R_i->second.second)); //6));
    			  else if (((*i).second.get_chain().find(tempRR.first-1) != (*i).second.get_chain().end()) &&
				   ((*j).second.get_chain().find(tempRR.second+1) != (*j).second.get_chain().end()))
    			    RR_frq.insert(make_pair(make_pair(tempRR.first-1, tempRR.second+1), HaHa_R_i->second.second)); //6));

    			  else
    			    NOESY_Assign::outputSec() << "HaHa: cannot find right NH--0 for " << tempRR.first << "(+1,-1) " << tempRR.second << "(-1,+1)" << endl;
    		   }
    		}


    		for(ssRRMapNInt::iterator HaHn_R_i = HaHn_R_il; HaHn_R_i != HaHn_R_iu; HaHn_R_i++)
    		{
    		  //if (DEBUG) NOESY_Assign::outputSec() << " hahn " << (*HaHn_R_i).second.first.first << "*" << (*HaHn_R_i).second.first.second << " " << HaHn_R_i->second.second<< endl;
    		  RRFreqInt::iterator RR_i;
    		  for(RR_i = RR_frq.begin(); RR_i != RR_frq.end(); RR_i++)
    		  {
    			  if ((((*RR_i).first.first - ((*HaHn_R_i).second.first.first)) == ((*HaHn_R_i).second.first.second - (*RR_i).first.second) - 1) &&
    			      ((fmod(abs((*RR_i).first.first - ((*HaHn_R_i).second.first.first)), 2) == 1) &&
    			     (fmod(abs((*RR_i).first.second - ((*HaHn_R_i).second.first.second)), 2) == 0))) //&&

    			  {

                //if (DEBUG) NOESY_Assign::outputSec() << " to " << RR_i->first.first << " " << RR_i->first.second << endl;

                RR_i->second.first += HaHn_R_i->second.second.first;
		RR_i->second.second += HaHn_R_i->second.second.second;

		//if (DEBUG) NOESY_Assign::outputSec() << "now " << RR_i->second << endl;
    			  }
    		  }
    		}

    		for(ssRRMapNInt::iterator HnHa_R_i = HnHa_R_il; HnHa_R_i != HnHa_R_iu; HnHa_R_i++)
    		{
		  //if (DEBUG) NOESY_Assign::outputSec() << "hnha " << (*HnHa_R_i).second.first.first << "*" << (*HnHa_R_i).second.first.second << " " << HnHa_R_i->second.second<< endl;
    		  RRFreqInt::iterator RR_i;
    		  for(RR_i = RR_frq.begin(); RR_i != RR_frq.end(); RR_i++)
    		  {
    			  if ((((*RR_i).first.first - ((*HnHa_R_i).second.first.first)) == ((*HnHa_R_i).second.first.second - (*RR_i).first.second) - 1) &&
    			      ((fmod(abs((*RR_i).first.first - ((*HnHa_R_i).second.first.first)), 2) == 0) &&
    			       (fmod(abs((*RR_i).first.second - ((*HnHa_R_i).second.first.second)), 2) == 1))) // &&
    			  {
			    //if (DEBUG) NOESY_Assign::outputSec() << " to " << RR_i->first.first << " " << RR_i->first.second << endl;
    			    (*RR_i).second.first += HnHa_R_i->second.second.first;
			    (*RR_i).second.second += HnHa_R_i->second.second.second;
			    //if (DEBUG) NOESY_Assign::outputSec() << " now " << RR_i->second << endl;
    			  }
    		  }
    		}

				for(ssRRMapNInt::iterator EC_R_i = EC_R_il; EC_R_i != EC_R_iu; EC_R_i++)
    		{
    		  if (DEBUG) NOESY_Assign::outputSec() << "EC2 " << (*EC_R_i).second.first.first << "*" << (*EC_R_i).second.first.second << " " << EC_R_i->second.second << endl;
    		  RRFreqInt::iterator RR_i;

		  		int factor = 4; // ? same for para

    		  for(RR_i = RR_frq.begin(); RR_i != RR_frq.end(); RR_i++)
    		  {
						if ((((*RR_i).first.first - ((*EC_R_i).second.first.first)) == ((*EC_R_i).second.first.second - (*RR_i).first.second) - 2) ||
								(((*RR_i).first.first - ((*EC_R_i).second.first.first)) == ((*EC_R_i).second.first.second - (*RR_i).first.second) + 2))// &&
    			  {
							if (DEBUG) NOESY_Assign::outputSec() << " to:" << RR_i->first.first << " " << RR_i->first.second << endl;
							RR_i->second.first += EC_R_i->second.second.first/factor; //score
							RR_i->second.second += EC_R_i->second.second.second; //Intensity --> distance
							if (DEBUG) NOESY_Assign::outputSec() << "now " << RR_i->second.second << endl;
							break;
	    			}
    		  }
    		}


    		NOESY_Assign::outputSec() << "Antiparallel Poss Register for Strands " << (*i).second.get_strand_ID() << "-"
      			          << (*j).second.get_strand_ID() << ": ";

    		float max = 1; float less = 0;
    		pair<int, int> maxRR;
    		for(RRFreqInt::iterator RR_i = RR_frq.begin(); RR_i != RR_frq.end(); RR_i++)
    		{
    		  NOESY_Assign::outputSec() << (*RR_i).first.first << "-" << (*RR_i).first.second << "=" << (*RR_i).second.first << " ";
    		  if ((*RR_i).second.first >= max)
    		  {
    			  less = max;
    			  max = (*RR_i).second.first;
    			  maxRR = (*RR_i).first;
    		   }
    		}

    		NOESY_Assign::outputSec() << endl;

		float cnum = 0; float iMax = 100;
    		if (max > parSec.ap-0.0001)
    		{
    		  for(RRFreqInt::iterator RR_i = RR_frq.begin(); RR_i != RR_frq.end(); RR_i++)
    		  {
    		    if (RR_i->second.first == max)
    		    {
		      cnum++;
		    }
    		  }

		  if (cnum >1) // multiple candidates, pick the one which highest intensities (smallest distance)
		    {
		      float iLess = 0;
		      for(RRFreqInt::iterator RR_i = RR_frq.begin(); RR_i != RR_frq.end(); RR_i++)
			{
			  if (RR_i->second.first == max)
			    {
			      if (RR_i->second.second < iMax)
				{
				  iLess = iMax;
				  iMax = (*RR_i).second.second;
				}
			    }
			}
		    }
		}

    		if (max > parSec.ap-0.0001)
    		{
    		  for(RRFreqInt::iterator RR_i = RR_frq.begin(); RR_i != RR_frq.end(); RR_i++)
    		  {
    		    if (RR_i->second.first == max)
    		    {
		      if ((cnum == 1) || (RR_i->second.second == iMax))
			{
			  NOESY_Assign::outputSec() << endl << "** Poss: Register for Antiparallel Strands " << (*i).second.get_strand_ID() << "-" << (*j).second.get_strand_ID()
						    << " Found : " << RR_i->first.first << " " << RR_i->first.second << " score: " << max << " " << iMax << endl << endl;
			  anti_register.insert(make_pair(make_pair(RR_i->first.first, RR_i->first.second), max));
			}
		    }
    		  }
     		}

    		for(RRFreqInt::iterator ii = RR_frq.begin(); ii != RR_frq.end(); ii++)
		  antiRR_frq.insert(make_pair(make_pair(i->second.get_strand_ID(), j->second.get_strand_ID()), make_pair(ii->first, ii->second.first)));

    		//-------------------------------------
    		// case 2. if para, find the consistent
    		//-------------------------------------
    		RR_frq.erase(RR_frq.begin(), RR_frq.end());
    		isfar = 0;
    		isAd = 0;

    		RRFreq isHaHn; isHaHn.clear();
    		RRFreq isHnHa; isHnHa.clear();
    		RRFreq isHnHn; isHnHn.clear();

    		for(ssRRMapNInt::iterator HaHn_R_i = HaHn_R_il; HaHn_R_i != HaHn_R_iu; HaHn_R_i++)
    		{
    		  RRFreqInt::iterator RR_i;
    		  for(RR_i = RR_frq.begin(); RR_i != RR_frq.end(); RR_i++)
    		  {
    			  if ((((*RR_i).first.first - ((*HaHn_R_i).second.first.first)) == ((*RR_i).first.second - ((*HaHn_R_i).second.first.second))) &&
    			      (fmod(abs((*RR_i).first.first - ((*HaHn_R_i).second.first.first)), 2) == 0))
    			  {
			    isHaHn[RR_i->first]++;
    			    RR_i->second.first += HaHn_R_i->second.second.first;
			    RR_i->second.second += HaHn_R_i->second.second.second;
    			    break;
    			  }
    		  }
    		  if (RR_i == RR_frq.end())
    		  {
    					if ((i->second.get_chain().find(HaHn_R_i->second.first.first) != i->second.get_chain().end()) &&
    							(j->second.get_chain().find(HaHn_R_i->second.first.second) != j->second.get_chain().end()))
    					{
    		      	RR_frq.insert(make_pair(make_pair((*HaHn_R_i).second.first.first, (*HaHn_R_i).second.first.second), HaHn_R_i->second.second)); //6));
    		      	isHaHn[make_pair(HaHn_R_i->second.first.first, HaHn_R_i->second.first.second)]++;
    		      } else
    		      {
    		      	if ((i->second.get_chain().find(HaHn_R_i->second.first.first+2) != i->second.get_chain().end()) &&
    		      			(j->second.get_chain().find(HaHn_R_i->second.first.second+2) != j->second.get_chain().end()))
    		      	{
    		      	isHaHn[make_pair(HaHn_R_i->second.first.first+2, HaHn_R_i->second.first.second+2)]++;
 	   		      		RR_frq.insert(make_pair(make_pair((*HaHn_R_i).second.first.first+2, (*HaHn_R_i).second.first.second+2), HaHn_R_i->second.second)); //6));
    		      	} else
    		      	if ((i->second.get_chain().find(HaHn_R_i->second.first.first-2) != i->second.get_chain().end()) &&
    		      			(j->second.get_chain().find(HaHn_R_i->second.first.second-2) != j->second.get_chain().end()))
    		      	{
    		      	isHaHn[make_pair(HaHn_R_i->second.first.first-2, HaHn_R_i->second.first.second-2)]++;
 	   		      		RR_frq.insert(make_pair(make_pair((*HaHn_R_i).second.first.first-2, (*HaHn_R_i).second.first.second-2), HaHn_R_i->second.second)); //6));
    		      	} else
    			    	NOESY_Assign::outputSec() << "Para HaHn: cannot find right NH--0 for " << HaHn_R_i->second.first.first << "(+2,+2) " << HaHn_R_i->second.first.second << "(-2,-2)" << endl;

    		      }
    		  }
    		}

				//for para - EC is treated like HaHn
				for(ssRRMapNInt::iterator EC_R_i = EC_R_il; EC_R_i != EC_R_iu; EC_R_i++)
    		{
    		  RRFreqInt::iterator RR_i;

					int factor = 2; // ? same for para

    		  for(RR_i = RR_frq.begin(); RR_i != RR_frq.end(); RR_i++)
    		  {
    			  if ((((*RR_i).first.first - ((*EC_R_i).second.first.first)) == ((*RR_i).first.second - ((*EC_R_i).second.first.second))))// &&
    			      //(fmod(abs((*RR_i).first.first - ((*EC_R_i).second.first.first)), 2) == 0))
    			  {
			    			isHaHn[RR_i->first]++;
    			    	RR_i->second.first += EC_R_i->second.second.first/factor;
			    			RR_i->second.second += EC_R_i->second.second.second;
    			    	break;
    			  }
    		  }
    		  if (RR_i == RR_frq.end())
    		  {
    					if ((i->second.get_chain().find(EC_R_i->second.first.first) != i->second.get_chain().end()) &&
    							(j->second.get_chain().find(EC_R_i->second.first.second) != j->second.get_chain().end()))
    					{
    		      	RR_frq.insert(make_pair(make_pair((*EC_R_i).second.first.first, (*EC_R_i).second.first.second), make_pair(EC_R_i->second.second.first/factor, EC_R_i->second.second.second))); //6));
    		      	isHaHn[make_pair(EC_R_i->second.first.first, EC_R_i->second.first.second)]++;
    		      } else
    		      {
    		      	if ((i->second.get_chain().find(EC_R_i->second.first.first+2) != i->second.get_chain().end()) &&
    		      			(j->second.get_chain().find(EC_R_i->second.first.second+2) != j->second.get_chain().end()))
    		      	{
    		      	isHaHn[make_pair(EC_R_i->second.first.first+2, EC_R_i->second.first.second+2)]++;
 	   		      		RR_frq.insert(make_pair(make_pair((*EC_R_i).second.first.first+2, (*EC_R_i).second.first.second+2), make_pair(EC_R_i->second.second.first/factor, EC_R_i->second.second.second))); //6));
    		      	} else
    		      	if ((i->second.get_chain().find(EC_R_i->second.first.first-2) != i->second.get_chain().end()) &&
    		      			(j->second.get_chain().find(EC_R_i->second.first.second-2) != j->second.get_chain().end()))
    		      	{
    		      	isHaHn[make_pair(EC_R_i->second.first.first-2, EC_R_i->second.first.second-2)]++;
 	   		      		RR_frq.insert(make_pair(make_pair((*EC_R_i).second.first.first-2, (*EC_R_i).second.first.second-2), make_pair(EC_R_i->second.second.first/factor, EC_R_i->second.second.second))); //6));
    		      	} else
    			    	NOESY_Assign::outputSec() << "Para EC: cannot find right NH--0 for " << EC_R_i->second.first.first << "(+2,+2) " << EC_R_i->second.first.second << "(-2,-2)" << endl;

    		      }
    		  }
    		}



    		for(ssRRMapNInt::iterator HnHa_R_i = HnHa_R_il; HnHa_R_i != HnHa_R_iu; HnHa_R_i++)
    		{
    		  RRFreqInt::iterator RR_i;
    		  for(RR_i = RR_frq.begin(); RR_i != RR_frq.end(); RR_i++)
    		  {
    			  if ((((*RR_i).first.first - ((*HnHa_R_i).second.first.first)) == ((*RR_i).first.second - ((*HnHa_R_i).second.first.second))) &&
    			      (fmod(abs((*RR_i).first.first - ((*HnHa_R_i).second.first.first)), 2) == 1))
    			  {

    			   	RR_i->second.first += HnHa_R_i->second.second.first;
							RR_i->second.second += HnHa_R_i->second.second.second;
							isHnHa[RR_i->first]++;

    			    break;
    			  }
    		  }

    		  if (RR_i == RR_frq.end())
    		  {

    			  if (((*i).second.get_chain().find((*HnHa_R_i).second.first.first + 1) != (*i).second.get_chain().end()) &&
    			      ((*j).second.get_chain().find((*HnHa_R_i).second.first.second + 1)) != (*j).second.get_chain().end())
    			  {
    		      	isHnHa[make_pair(HnHa_R_i->second.first.first+1, HnHa_R_i->second.first.second+1)]++;
    			    RR_frq.insert(make_pair(make_pair((*HnHa_R_i).second.first.first + 1, (*HnHa_R_i).second.first.second + 1), HnHa_R_i->second.second)); //6));
    			  }
    			  else if (((*i).second.get_chain().find((*HnHa_R_i).second.first.first - 1) != (*i).second.get_chain().end()) &&
    				         ((*j).second.get_chain().find((*HnHa_R_i).second.first.second - 1)) != (*j).second.get_chain().end())
    			  {
    		      	isHnHa[make_pair(HnHa_R_i->second.first.first-1, HnHa_R_i->second.first.second-1)]++;
    			    RR_frq.insert(make_pair(make_pair((*HnHa_R_i).second.first.first - 1, (*HnHa_R_i).second.first.second - 1), HnHa_R_i->second.second)); //6));
    			  }
    		  }
    		}

				for(ssRRMapNInt::iterator HnHn_R_i = HnHn_R_il; HnHn_R_i != HnHn_R_iu; HnHn_R_i++)
			    		{
			    		  RRFreqInt::iterator RR_i;

					  bool in = 0;
			    		  for(RR_i = RR_frq.begin(); RR_i != RR_frq.end(); RR_i++)
			    		  {
			    			  if ((((((*RR_i).first.first - ((*HnHn_R_i).second.first.first)) == -1 + ((*RR_i).first.second - ((*HnHn_R_i).second.first.second))) &&
			    			      (fmod(abs((*RR_i).first.first - ((*HnHn_R_i).second.first.first)), 2) == 1) &&
			    			      (fmod(abs((*RR_i).first.second - ((*HnHn_R_i).second.first.second)), 2) == 0)) ||
			    			     ((((*RR_i).first.first - ((*HnHn_R_i).second.first.first)) == 1 + ((*RR_i).first.second - ((*HnHn_R_i).second.first.second))) &&
			    			      (fmod(abs((*RR_i).first.second - ((*HnHn_R_i).second.first.second)), 2) == 0) &&
			    			      (fmod(abs((*RR_i).first.first - ((*HnHn_R_i).second.first.first)), 2) == 1))) )// &&
			    			  {
						    if (!NOESY_Assign::MINCON() || (HnHn_R_i->second.second.second < 4.5))
						      {
							(*RR_i).second.first += HnHn_R_i->second.second.first/2; // from HnHn only, less confident
							(*RR_i).second.second += HnHn_R_i->second.second.second;

							isHnHn[RR_i->first]++;
							in = 1;
							//break; cann't break - it is ambiguous - support two possible alignments
						      }
			    			  }
			    		  }
					  if (!in && ((!NOESY_Assign::MINCON() || (HnHn_R_i->second.second.second < 4.5))))
					    {
					      //i-1,j
					      if (((*i).second.get_chain().find((*HnHn_R_i).second.first.first - 1)) != (*i).second.get_chain().end())
						{
						  isHnHn[make_pair(HnHn_R_i->second.first.first -1, HnHn_R_i->second.first.second)]++;
						  RR_frq.insert(make_pair(make_pair((*HnHn_R_i).second.first.first-1, (*HnHn_R_i).second.first.second), make_pair(HnHn_R_i->second.second.first/2, HnHn_R_i->second.second.second))); //6));
						}
					      //i+1, j+2
					      else if (((*i).second.get_chain().find((*HnHn_R_i).second.first.first + 1) != (*i).second.get_chain().end()) &&
						       ((*j).second.get_chain().find((*HnHn_R_i).second.first.second + 2)) != (*j).second.get_chain().end())
						{
						  isHnHn[make_pair(HnHn_R_i->second.first.first +1, HnHn_R_i->second.first.second +2)]++;
						  RR_frq.insert(make_pair(make_pair((*HnHn_R_i).second.first.first+1, (*HnHn_R_i).second.first.second+2), make_pair(HnHn_R_i->second.second.first/2, HnHn_R_i->second.second.second))); //6));
						}

					      //i+1,j
					      if (((*i).second.get_chain().find((*HnHn_R_i).second.first.first +1)) != (*i).second.get_chain().end())
						{
						  isHnHn[make_pair(HnHn_R_i->second.first.first +1, HnHn_R_i->second.first.second)]++;
						  RR_frq.insert(make_pair(make_pair((*HnHn_R_i).second.first.first+1, (*HnHn_R_i).second.first.second), make_pair(HnHn_R_i->second.second.first/2, HnHn_R_i->second.second.second))); //6));
						}
					      //i-1, j-2
					      else if (((*i).second.get_chain().find((*HnHn_R_i).second.first.first - 1) != (*i).second.get_chain().end()) &&
						       ((*j).second.get_chain().find((*HnHn_R_i).second.first.second - 2)) != (*j).second.get_chain().end())
						{
						  isHnHn[make_pair(HnHn_R_i->second.first.first -1, HnHn_R_i->second.first.second -2)]++;
						  RR_frq.insert(make_pair(make_pair((*HnHn_R_i).second.first.first-1, (*HnHn_R_i).second.first.second-2), make_pair(HnHn_R_i->second.second.first/2, HnHn_R_i->second.second.second))); //6));
						}
					    }



			}

			for(ssRRMapNInt::iterator EC_R_i = EC_R_il; EC_R_i != EC_R_iu; EC_R_i++)
			{
				if (DEBUG) NOESY_Assign::outputSec() << "EC2 " << (*EC_R_i).second.first.first << "*" << (*EC_R_i).second.first.second << " " << EC_R_i->second.second << endl;
				RRFreqInt::iterator RR_i;

				int factor = 4; // ? same for para

				for(RR_i = RR_frq.begin(); RR_i != RR_frq.end(); RR_i++)
				{
					if ((((*RR_i).first.first - ((*EC_R_i).second.first.first)) == ((*RR_i).first.second - (*EC_R_i).second.first.second) - 2) ||
							(((*RR_i).first.first - ((*EC_R_i).second.first.first)) == ((*RR_i).first.second - (*EC_R_i).second.first.second) + 2))// &&
					{
						if (DEBUG) NOESY_Assign::outputSec() << " to:" << RR_i->first.first << " " << RR_i->first.second << endl;
						RR_i->second.first += EC_R_i->second.second.first/factor; //score
						RR_i->second.second += EC_R_i->second.second.second; //Intensity --> distance
						if (DEBUG) NOESY_Assign::outputSec() << "now " << RR_i->second.first << endl;
						break;
					}
				}
			}


    		NOESY_Assign::outputSec() << "    Parallel Poss Resigter for Strands " << (*i).second.get_strand_ID() << "-"
      			          << (*j).second.get_strand_ID() << ": ";
    		max = 1; less = 0;
    		for(RRFreqInt::iterator RR_i = RR_frq.begin(); RR_i != RR_frq.end(); RR_i++)
    		{
    		  NOESY_Assign::outputSec() << (*RR_i).first.first << "-" << (*RR_i).first.second << "=" << (*RR_i).second.first << " ";
    		  if ((*RR_i).second.first >= max)
    		  {
    			  less = max;
    			  max = (*RR_i).second.first;
    			  maxRR = (*RR_i).first;
    		   }
    		}
    		NOESY_Assign::outputSec() << endl;
    		if (DEBUG) NOESY_Assign::outputSec() << "max " << max << endl;
    		if (max > parSec.p-0.000001)
    		{

		  float cnum = 0;
		  for(RRFreqInt::iterator RR_i = RR_frq.begin(); RR_i != RR_frq.end(); RR_i++)
		    {
		      if (RR_i->second.first == max)
			{
        		  cnum++;
			}
		    }

		  iMax = 100;
		  if (cnum > 1) //multiple candidate, pick the one with the highest intensities
		    {
		      float iLess = 0;
		      for(RRFreqInt::iterator RR_i = RR_frq.begin(); RR_i != RR_frq.end(); RR_i++)
			{
			  if (RR_i->second.first == max)
			    {
			      if ((*RR_i).second.second < iMax)
				{
				  iLess = iMax;
				  iMax = (*RR_i).second.second;
				}
			    }
			}
		    }

		  for(RRFreqInt::iterator RR_i = RR_frq.begin(); RR_i != RR_frq.end(); RR_i++)
		    {
		      if (RR_i->second.first == max)
			{
			  if ((cnum==1) || (RR_i->second.second == iMax)) // (iMax = (*RR_i).second.second))
        		  {
			    NOESY_Assign::outputSec() << endl << "** Poss: Register for Parallel Strands " << (*i).second.get_strand_ID() << "-" << (*j).second.get_strand_ID()
						      << " Found: " << RR_i->first.first << "-" << RR_i->first.second << " score: " << max << " " << iMax << endl << endl;

			    if (NOESY_Assign::MINCON())
			      para_register.insert(make_pair(make_pair(RR_i->first.first, RR_i->first.second), max/2));
			    else
			      para_register.insert(make_pair(make_pair(RR_i->first.first, RR_i->first.second), max));

        		  }
      		 	}
		    }
     		}

     		for(RRFreqInt::iterator ii = RR_frq.begin(); ii != RR_frq.end(); ii++)
		  paraRR_frq.insert(make_pair(make_pair(i->second.get_strand_ID(), j->second.get_strand_ID()), make_pair(ii->first, ii->second.first)));
    	}
    }

  ////////////////////////////////
  // Validate the Registers     //
  ////////////////////////////////

  RRFreqRR antiList; antiList.clear();
  RRFreqRR paraList; paraList.clear();

  RRFreq nhList; nhList.clear(); // number of NH slow exchange data found for this register pairs

  multiset<int,less<int> > RSet; RSet.clear();
	for(RRFreq::iterator iterH = anti_register.begin(); iterH != anti_register.end(); iterH++)
  {
    if (DEBUG)
      NOESY_Assign::outputSec() << "check antiparalle regiter " << iterH->first.first << " " << iterH->first.second << endl;

  	multimap<pair<int,int>, pair<pair<int,int>, float>, less<pair<int, int> > >::iterator aa;
  	for( aa = antiRR_frq.begin(); aa != antiRR_frq.end(); aa++)
  		{
  			if (aa->second.first == iterH->first)
  			break;
  		}

  	if (aa == antiRR_frq.end())
  	 continue;

  	int ii=iterH->first.first, jj=iterH->first.second;

		if (ssFilter[ii] != -1)           // --->
		{
			if (seq.find(ii+1)->second.get_ss()->get_strand_ID() == aa->first.first)
				ii++;
			else
				ii--;
		}
		if (ssFilter[jj] != -1)    // <---
		{
			if (seq.find(jj-1)->second.get_ss()->get_strand_ID() == aa->first.second)
				jj--;
			else
				jj++;
		}

		if (DEBUG)
			NOESY_Assign::outputSec() << "find strand " << ii << " " << jj << endl;   // ok

    const Structure* const strand1 = seq.find(ii)->second.get_ss();
    const Structure* const strand2 = seq.find(jj)->second.get_ss();

    const Chain& chain1 = strand1->get_chain();
    const Chain& chain2 = strand2->get_chain();

	  // find the co-starting point
	  Chain::reverse_iterator j = (Chain::reverse_iterator) chain2.rbegin();
	  Chain::iterator i = chain1.begin();

    int di = iterH->first.first - *i;  // >0
    int dj = *j - iterH->first.second; // >0

    if (DEBUG) NOESY_Assign::outputSec() << "di " << di << " dj " << dj << " " << *j << " " << iterH->first.second << endl;

	  int d;
	  if (di < dj)
    {
      d = di;
      while (j != chain2.rend())
  		{
        if ((*j - iterH->first.second) > d)
  		    j++;
  		  else
  		    break;
  		}
    }
	  else
    {
      d = dj;
      while (i != chain1.end())
  		{
         if ((iterH->first.first - *i) > d)
  		    i++;
  		  else
  		    break;
  		}
    }

    d = abs(d);
    if (DEBUG) NOESY_Assign::outputSec() << "d i j " << d << " " << *i << " " << *j << endl;

	  //starting walking

    bool in = 0;

	  while (i != chain1.end())
    {
      if (j != chain2.rend())
  		{
  		  if (fmod(d, 2) == 1)
 		    {
 		      // not i to j , is i to j+1
 		      if (fabs((*j)-(*iterH).first.second) == d+1)
 			      j++;

 		      i++; if (j != chain2.rend()) j++;
 		      d = 0;
 		    }

 		    if ((i != chain1.end()) && (j != chain2.rend()))
 		    {
 		    // NN
		      in = 1;
        antiList.insert(make_pair(make_pair(*i,*j), make_pair((*iterH).second, (*iterH).first)));
        if (NHList.find(*i) != NHList.end())
          nhList[iterH->first]++;
        if (NHList.find(*j) != NHList.end())
          nhList[iterH->first]++;

        RSet.insert(*i);
        RSet.insert(*j);

  		  if (i != chain1.end()) ++i; // i+1
  		  if (j != chain2.rend()) ++j; // j-1
  		  if (i!= chain1.end()) ++i; // i+2
  		  if (j!= chain2.rend()) j++; // j-2
		    }
  		}
      else
	      break;
    }

	  if (!in) // no aligned NN is found
	    {
       antiList.insert(make_pair(make_pair(iterH->first.first, iterH->first.second), make_pair(iterH->second, iterH->first)));
       nhList[iterH->first]++;
       nhList[iterH->first]++;
       RSet.insert(iterH->first.first);
       RSet.insert(iterH->first.second);
       NOESY_Assign::outputSec() << "did not align, use the registers iterH->first.first iterH->first.second\n";
	    }
  }


	// para_register
	for(RRFreq::iterator iterH = para_register.begin(); iterH != para_register.end(); iterH++)
  {
    if (DEBUG)
      cout << (*iterH).first.first << " " << (*iterH).first.second << " " << (*iterH).second << endl;

  	multimap<pair<int,int>, pair<pair<int,int>, float>, less<pair<int, int> > >::iterator aa;
  	for( aa = paraRR_frq.begin(); aa != paraRR_frq.end(); aa++)
  		{
  			if (aa->second.first == iterH->first)
  			break;
  		}

  	if (aa == paraRR_frq.end())
  	 continue;

  	int ii=iterH->first.first, jj=iterH->first.second;

		if (ssFilter[ii] != -1)
		{
			if (seq.find(ii+1)->second.get_ss()->get_strand_ID() == aa->first.first)
				ii++;
			else
				ii--;
		}
		if (ssFilter[jj] != -1)
		{
			if (seq.find(jj+1)->second.get_ss()->get_strand_ID() == aa->first.second)
				jj++;
			else
				jj--;
		}

    const Structure* strand1 = seq.find(ii)->second.get_ss();
    const Structure* strand2 = seq.find(jj)->second.get_ss();

    if (DEBUG)
      cout << *strand1 << " " << *strand2 << " " << endl;

    Chain::iterator i = strand1->get_chain().begin();
    Chain::iterator j = strand2->get_chain().begin();

	  int di = (*iterH).first.first - *i; // >0
	  int dj = (*iterH).first.second - *j;  // >0

	  int d;
	  if (di < dj)
    {
      d = di;
      while (j != strand2->get_chain().end())
  		{
  		  if (((*iterH).first.second - *j) > d)
  		    j++;
  		  else
  		    break;
  		}
    }
	  else
    {
      d = dj;
      while (i != strand1->get_chain().end())
  		{
  		  if (((*iterH).first.first - *i) > d)
  		    i++;
  		  else
  		    break;
  		}
    }

	  //starting walking

    d = abs(d);

	  while (i != strand1->get_chain().end())
    {
      if (j != strand2->get_chain().end())
  		{
  		  if (fmod(d, 2) == 1)
 		    {
  		  if (i!= strand1->get_chain().end()) ++i; //
  		  if (j!= strand2->get_chain().end()) j++; //  i++; j++;
 		      d = 0;
 		    }

  		  if (i!= strand1->get_chain().end()) ++i; // i+1

 		    if ((i != strand1->get_chain().end()) && (j != strand2->get_chain().end()))
 		    {

  		  // NN
        paraList.insert(make_pair(make_pair(*i,*j), make_pair((*iterH).second, (*iterH).first)));
        if (NHList.find(*i) != NHList.end())
          nhList[iterH->first]++;
        if (NHList.find(*j) != NHList.end())
          nhList[iterH->first]++;

        RSet.insert(*i);
        RSet.insert(*j);

  		  if (j!= strand2->get_chain().end())++j; // j+1
  		  if (i!= strand1->get_chain().end()) ++i; // i+2
  		  if (j!= strand2->get_chain().end()) j++; // j+2

  		  }
  		}
      else
	      break;
    }
  }

  //if (DEBUG)
  {
    // anti
    NOESY_Assign::outputSec() << endl << "Validate the Beta-sheet alignments" << endl;
    NOESY_Assign::outputSec() << "==================================" << endl << endl;

    //if (DEBUG)
    {
      for(RRFreqRR::iterator i = antiList.begin(); i!= antiList.end(); i++)
      {
        NOESY_Assign::outputSec() << "antiList: " << i->first.first << "-" << i->first.second << " " << i->second.first  << " " << i->second.second.first << " " << i->second.second.second << endl;
      }
      // para
      for(RRFreqRR::iterator i = paraList.begin(); i!= paraList.end(); i++)
      {
        NOESY_Assign::outputSec() << "paraList: " << i->first.first << "-" << i->first.second << " " << i->second.first << " " << i->second.second.first << " " << i->second.second.second << endl;
      }

      for(RRFreq::iterator i = nhList.begin(); i != nhList.end(); i++)
      {
        NOESY_Assign::outputSec() << "NH Set: " << i->first.first << " " << i->first.second << " " << i->second << endl;
      }
      NOESY_Assign::outputSec() << endl;
    }
  }

  RRFreqRR wrongRR; wrongRR.clear(); // case (a))
  for(RRFreqRR::iterator i = antiList.begin(); i != antiList.end(); i++)
  {
    if ((RSet.count(i->first.first) > 1) || (RSet.count(i->first.second) > 1))
    {
      NOESY_Assign::outputSec() << "anti-sheet CASE(a): Conflict for hnhn(" << i->first.first  << "," << i->first.second << ")" << endl;
      wrongRR.insert(*i);
    }
  }
  for(RRFreqRR::iterator i = paraList.begin(); i != paraList.end(); i++)
  {
    if ((RSet.count(i->first.first) > 1) || (RSet.count(i->first.second) > 1))
    {
      NOESY_Assign::outputSec() << "para-sheet CASE(a): Conflict for hnhn(" << i->first.first  << "," << i->first.second << ")" << endl;
      wrongRR.insert(*i);
    }
  }

  for(RRFreqRR::iterator i = wrongRR.begin(); i != wrongRR.end(); i++)
  for(RRFreqRR::iterator j = i; j != wrongRR.end(); j++)
  {
      if ((i != j) && ((anti_register.find(i->second.second) != anti_register.end()) || (para_register.find(i->second.second) != para_register.end()))
      						 && ((anti_register.find(j->second.second) != anti_register.end()) || (para_register.find(j->second.second) != para_register.end()))
      			&& ((i->first.first == j->first.first) || (i->first.second == j->first.first) ||
                (i->first.first == j->first.second) || (i->first.second == j->first.second)))
      {
        RRFreqRR::iterator k;
        if (i->second.first < j->second.first)
          k = i;
        else if (i->second.first > j->second.first)
          k = j;
        else
        {
        	if (DEBUG)
        		NOESY_Assign::outputSec() << "TIE" << nhList[i->second.second] << " " << nhList[j->second.second] << endl;
          {
            NOESY_Assign::outputSec() << "*** ERROR CASE(a): Conflict for strands between " << i->first.first << "-" << i->first.second
            		<< " and " << j->first.first << "-" << j->first.second << endl;
					}
          continue;
        }
        //if (!k)
        {
          if ((antiList.find(k->first) != antiList.end()) && (anti_register.find(k->second.second) != anti_register.end()))
          {
            NOESY_Assign::outputSec() << "CASE(a): Remove from anti_register: " << (*k).second.second.first << "-" << k->second.second.second << " " << endl;
            anti_register.erase(anti_register.find(k->second.second));
          }
          if ((paraList.find(k->first) != paraList.end()) && (para_register.find(k->second.second) != para_register.end()))
          {
            NOESY_Assign::outputSec() << "CASE(a): Remove from para_register: " << k->second.second.first << "-" << k->second.second.second << " " << endl;
            para_register.erase(para_register.find(k->second.second));
          }
        }
      }
  }

  // not in from the same strands case(b)
  RRFreq apList; apList.clear();
  apList.insert(anti_register.begin(), anti_register.end());
  apList.insert(para_register.begin(), para_register.end());

  for(RRFreq::iterator i = apList.begin(); i != apList.end(); i++)
  {
    const Structure* strand1 = seq.find((*i).first.first)->second.get_ss();
    const Structure* strand2 = seq.find((*i).first.second)->second.get_ss();

    for(RRFreq::iterator j = i; j != apList.end(); j++)
    {
      const Structure* strand21 = seq.find((*j).first.first)->second.get_ss();
      const Structure* strand22 = seq.find((*j).first.second)->second.get_ss();
      if ((j != i) && (((strand1 == strand21) && (strand2 == strand22)) || ((strand1 == strand22) && (strand2 == strand21))))
      {
        RRFreq::iterator k;
        if (i->second < j->second)
          k = i;
        else if (i->second > j->second)
          k = j;
        else
        {
          if (nhList[i->first] > nhList[j->first])
            k = j;
          else if (nhList[i->first] < nhList[j->first])
            k = i;
          else
          {
            NOESY_Assign::outputSec() << "*** ERROR CASE(b): Conflict for strands between " << i->first.first << "-" << i->first.second << " and " << j->first.first << "-" << j->first.second << endl;
          }
          continue;
        }
        //if (!k)
        {
          if ((anti_register.find(k->first) != anti_register.end()))
          {
            NOESY_Assign::outputSec() << "CASE(b): SS-Remove from anti_register: " << (*k).first.first << "-" << k->first.second << " " << endl;
            anti_register.erase(anti_register.find(k->first));

          }
          if ((para_register.find(k->first) != para_register.end()))
          {
            NOESY_Assign::outputSec() << "CASE(b): SS-Remove from para_register: " << k->first.first << "-" << k->first.second << " " << endl;
            para_register.erase(para_register.find(k->first));
          }
        }
      }
    }
  }


  // case (c)
	set<const Structure*, less<const Structure*> > StrutSet; StrutSet.clear();
	map<const Structure*, pair<pair<int,int>, float>, less<const Structure*> > StrutSet2; StrutSet2.clear();
	map<const Structure*, pair<pair<int,int>, float>, less<const Structure*> > StrutSet3; StrutSet3.clear();

  apList.clear();
  apList.insert(anti_register.begin(), anti_register.end());
  apList.insert(para_register.begin(), para_register.end());

 	map<int, const Structure*> intStructMap; intStructMap.clear();

    for(RRFreq::iterator i = para_register.begin(); i != para_register.end(); i++)
		{
    	multimap<pair<int,int>, pair<pair<int,int>, float>, less<pair<int, int> > >::iterator aa;
    	for( aa = paraRR_frq.begin(); aa != paraRR_frq.end(); aa++)
    		{
    			if (aa->second.first == i->first)
    			break;
    		}

    	if (aa == paraRR_frq.end())
    	 continue;

    	int ii=i->first.first, jj=i->first.second;

  		if (ssFilter[ii] != -1)
  		{
  			if (seq.find(ii+1)->second.get_ss()->get_strand_ID() == aa->first.first)
  				ii++;
  			else
  				ii--;
  		}
  		if (ssFilter[jj] != -1)
  		{
  			if (seq.find(jj+1)->second.get_ss()->get_strand_ID() == aa->first.second)
  				jj++;
  			else
  				jj--;
  		}

  		if (DEBUG)
  			NOESY_Assign::outputSec() << "find strand " << ii << " " << jj << endl;

      const Structure* const strand1 = seq.find(ii)->second.get_ss();
      const Structure* const strand2 = seq.find(jj)->second.get_ss();

      			intStructMap.insert(make_pair(i->first.first, strand1));
			intStructMap.insert(make_pair(i->first.second, strand2));
			StrutSet.insert(strand1);
			StrutSet.insert(strand2);
		}

		    for(RRFreq::iterator i = anti_register.begin(); i != anti_register.end(); i++)
		{
    	multimap<pair<int,int>, pair<pair<int,int>, float>, less<pair<int, int> > >::iterator aa;
    	for( aa = antiRR_frq.begin(); aa != antiRR_frq.end(); aa++)
    		{
    			if (aa->second.first == i->first)
    			break;
    		}

    	if (aa == antiRR_frq.end())
    	 continue;

    	int ii=i->first.first, jj=i->first.second;

  		if (ssFilter[ii] != -1)
  		{
  			if (seq.find(ii+1)->second.get_ss()->get_strand_ID() == aa->first.first)
  				ii++;
  			else
  				ii--;
  		}
  		if (ssFilter[jj] != -1)
  		{
  			if (seq.find(jj+1)->second.get_ss()->get_strand_ID() == aa->first.second)
  				jj++;
  			else
  				jj--;
  		}

  		if (DEBUG)
  			NOESY_Assign::outputSec() << "find strand " << ii << " " << jj << endl;

      const Structure* const strand1 = seq.find(ii)->second.get_ss();
      const Structure* const strand2 = seq.find(jj)->second.get_ss();

			StrutSet.insert(strand1);
			StrutSet.insert(strand2);
      			intStructMap.insert(make_pair(i->first.first, strand1));
			intStructMap.insert(make_pair(i->first.second, strand2));
		}

 for(set<const Structure*, less<const Structure*> >::iterator s = StrutSet.begin(); s != StrutSet.end(); s++)
 {
 		StrutSet2.clear();
 		StrutSet3.clear();
 		if (DEBUG)
 			NOESY_Assign::outputSec() << "primer: " << *(*s) << endl;

 		for(RRFreq::iterator i = apList.begin(); i != apList.end(); i++)
 		{
 			if ((anti_register.find(i->first) != anti_register.end()) || (para_register.find(i->first) != para_register.end()))
 			{
     		const Structure* strand1 = intStructMap.find(i->first.first)->second;
		const Structure* strand2 = intStructMap.find(i->first.second)->second;

      		if (strand1 == (*s))
      			StrutSet2.insert(make_pair(strand2, *i));// a-b, a-d
      		if (strand2 == (*s))
      			StrutSet2.insert(make_pair(strand1, *i));
  		}
 		}

    for(map<const Structure*, pair<pair<int,int>, float>, less<const Structure*> >::iterator s2 = StrutSet2.begin(); s2 != StrutSet2.end(); s2++)
    {
			if (DEBUG)
				NOESY_Assign::outputSec() << "second: " << *((*s2).first) << endl;

    	for(RRFreq::iterator i = apList.begin(); i != apList.end(); i++) // b-d
    	{
   			if ((anti_register.find(i->first) != anti_register.end()) || (para_register.find(i->first) != para_register.end()))
       	{
     		const Structure* strand1 = intStructMap.find(i->first.first)->second;
		const Structure* strand2 = intStructMap.find(i->first.second)->second;

         		if (((strand1 == (*s2).first) && (strand2 != (*s)) && (StrutSet2.find(strand2) != StrutSet2.end())) ||
         					((strand2 == (*s2).first) && (strand1 != (*s)) && (StrutSet2.find(strand1) != StrutSet2.end())))
         		{// loop

              map<float, pair<pair<int,int>, float>, less<float> > loopOrder;

              loopOrder.insert(make_pair((*i).second, *i));
              loopOrder.insert(make_pair(s2->second.second, s2->second));
              if ((strand1 == (*s2).first) && (strand2 != (*s)))
              	loopOrder.insert(make_pair((*(StrutSet2.find(strand2))).second.second, (*(StrutSet2.find(strand2))).second));
							else
								loopOrder.insert(make_pair((*(StrutSet2.find(strand1))).second.second, (*(StrutSet2.find(strand1))).second));

          		if (DEBUG)
          		{
          			for(map<float, pair<pair<int, int>, float>, less<float> >::iterator i = loopOrder.begin(); i!= loopOrder.end(); i++)
          				cout << i->first << " " << i->second.first.first << " " << i->second.first.second << endl;
          		}

              map<float, pair<pair<int,int>,float>, less<float> >::iterator tp;
              tp = loopOrder.begin();

              pair<pair<int,int>,float> k; k = (*tp).second;

              ++tp;

  						if (k.second == tp->second.second) // equal
              {
                   if (nhList[k.first] > nhList[tp->second.first])
                     k = tp->second;
                   else if (nhList[k.first] < nhList[tp->second.first])
                   {}
                   else
                   {
                     NOESY_Assign::outputSec() << "*** ERROR CASE(c): Conflict for strands between " << i->first.first << "-" << i->first.second << " ";
                     NOESY_Assign::outputSec() << s2->second.first.first << "-" << s2->second.first.second << " ";
                     NOESY_Assign::outputSec() << (*StrutSet2.find(strand2)).second.first.first << "-" << (*StrutSet2.find(strand2)).second.first.second << endl;
                     continue;
                   }
              }

              //if (!k)
              {
                if ((anti_register.find(k.first) != anti_register.end()))
                {
                  NOESY_Assign::outputSec() << "CASE(c): SS-Remove from anti_register: " << (k).first.first << "-" << k.first.second << " " << endl;
                  anti_register.erase(anti_register.find(k.first));
                }
                if ((para_register.find(k.first) != para_register.end()))
                {
                  NOESY_Assign::outputSec() << "CASE(c): SS-Remove from para_register: " << k.first.first << "-" << k.first.second << " " << endl;
                  para_register.erase(para_register.find(k.first));
                }
              }

         		} // end loop
         	}	// endl if
    		} // for
   	}  //for
 }

  // (d) each strand should have at least two residues invovled.
  apList.clear();
  apList.insert(anti_register.begin(), anti_register.end());

  for(RRFreq::iterator iterH = apList.begin(); iterH != apList.end(); iterH++)
  {
        set<int, less<int> > strand1count; strand1count.clear();
        set<int, less<int> > strand2count; strand2count.clear();

     		const Structure* strand1 = intStructMap.find(iterH->first.first)->second;
    		const Structure* strand2 = intStructMap.find(iterH->first.second)->second;


				ssRRMapNInt::iterator EC_R_il = EC_R.lower_bound(make_pair(strand1->get_strand_ID(), strand2->get_strand_ID()));
   	    ssRRMapNInt::iterator EC_R_iu = EC_R.upper_bound(make_pair(strand1->get_strand_ID(), strand2->get_strand_ID()));

   	    ssRRMapNInt::iterator HaHa_R_il = HaHa_R.lower_bound(make_pair(strand1->get_strand_ID(), strand2->get_strand_ID()));
   	    ssRRMapNInt::iterator HaHa_R_iu = HaHa_R.upper_bound(make_pair(strand1->get_strand_ID(), strand2->get_strand_ID()));

   	    ssRRMapNInt::iterator HnHn_R_il = HnHn_R.lower_bound(make_pair(strand1->get_strand_ID(), strand2->get_strand_ID()));
   	    ssRRMapNInt::iterator HnHn_R_iu = HnHn_R.upper_bound(make_pair(strand1->get_strand_ID(), strand2->get_strand_ID()));

   	    ssRRMapNInt::iterator HaHn_R_il = HaHn_R.lower_bound(make_pair(strand1->get_strand_ID(), strand2->get_strand_ID()));
   	    ssRRMapNInt::iterator HaHn_R_iu = HaHn_R.upper_bound(make_pair(strand1->get_strand_ID(), strand2->get_strand_ID()));

   	    ssRRMapNInt::iterator HnHa_R_il = HnHa_R.lower_bound(make_pair(strand1->get_strand_ID(), strand2->get_strand_ID()));
   	    ssRRMapNInt::iterator HnHa_R_iu = HnHa_R.upper_bound(make_pair(strand1->get_strand_ID(), strand2->get_strand_ID()));


          pair<int, int> old;  old = make_pair(0,0);
          for(ssRRMapNInt::iterator HnHn_R_i = HnHn_R_il; HnHn_R_i != HnHn_R_iu; HnHn_R_i++)
      		{
      		    if (((iterH->first.first - ((*HnHn_R_i).second.first.first)) == ((*HnHn_R_i).second.first.second - iterH->first.second)) &&
      			          (fmod(abs(iterH->first.first - (*HnHn_R_i).second.first.first), 2) == 0))
      			  {
                if (old != HnHn_R_i->second.first) // second time
                {
        					NOEtypeListN::iterator l = SSNOE.lower_bound(make_pair(
        										make_pair(HnHn_R_i->second.first.second, HnHn_R_i->second.first.first), HnHn));
        					NOEtypeListN::iterator u = SSNOE.upper_bound(make_pair(
        										make_pair(HnHn_R_i->second.first.second, HnHn_R_i->second.first.first), HnHn));
        					for(NOEtypeListN::iterator i=l; i != u; i++)
                  {
                        strand1count.insert(HnHn_R_i->second.first.first);
                        strand2count.insert(HnHn_R_i->second.first.second);
                  }

                  l = SSNOE.lower_bound(make_pair(
        										make_pair(HnHn_R_i->second.first.first, HnHn_R_i->second.first.second), HnHn));
        					u = SSNOE.upper_bound(make_pair(
        										make_pair(HnHn_R_i->second.first.first, HnHn_R_i->second.first.second), HnHn));
        					for(NOEtypeListN::iterator i=l; i != u; i++)
                  {
                        strand1count.insert(HnHn_R_i->second.first.first);
                        strand2count.insert(HnHn_R_i->second.first.second);
                  }
                 }
                old = HnHn_R_i->second.first;
      			  }
      		}

        old = make_pair(0,0);
     		for(ssRRMapNInt::iterator HaHa_R_i = HaHa_R_il; HaHa_R_i != HaHa_R_iu; HaHa_R_i++)
      		{
      			  if (((iterH->first.first - ((*HaHa_R_i).second.first.first)) ==
      			      ((*HaHa_R_i).second.first.second - iterH->first.second)) && (fmod(abs(iterH->first.first - (*HaHa_R_i).second.first.first), 2) == 1))
      			  {
                if (old != HaHa_R_i->second.first) // second time
                {
      					NOEtypeListN::iterator l = SSNOE.lower_bound(make_pair(
      										make_pair(HaHa_R_i->second.first.second, HaHa_R_i->second.first.first), HaHa));
      					NOEtypeListN::iterator u = SSNOE.upper_bound(make_pair(
      										make_pair(HaHa_R_i->second.first.second, HaHa_R_i->second.first.first), HaHa));
        					for(NOEtypeListN::iterator i=l; i != u; i++)
                  {
                        strand1count.insert(HaHa_R_i->second.first.first);
                        strand2count.insert(HaHa_R_i->second.first.second);
                  }

      					l = SSNOE.lower_bound(make_pair(
      										make_pair(HaHa_R_i->second.first.first, HaHa_R_i->second.first.second), HaHa));
      					u = SSNOE.upper_bound(make_pair(
      										make_pair(HaHa_R_i->second.first.first, HaHa_R_i->second.first.second), HaHa));
        					for(NOEtypeListN::iterator i=l; i != u; i++)
                  {
                        strand1count.insert(HaHa_R_i->second.first.first);
                        strand2count.insert(HaHa_R_i->second.first.second);
                  }
                }
                old = HaHa_R_i->second.first;
            }
      		}

      		old = make_pair(0,0);

      		for(ssRRMapNInt::iterator HaHn_R_i = HaHn_R_il; HaHn_R_i != HaHn_R_iu; HaHn_R_i++)
      		{
      			  if (((iterH->first.first - ((*HaHn_R_i).second.first.first)) == ((*HaHn_R_i).second.first.second - iterH->first.second) - 1) &&
      			      ((fmod(abs(iterH->first.first - ((*HaHn_R_i).second.first.first)), 2) == 1) &&
      			     (fmod(abs(iterH->first.second - ((*HaHn_R_i).second.first.second)), 2) == 0)) ) //&& //)

      			  {

                if (old != HaHn_R_i->second.first) // second time
                {
      					NOEtypeListN::iterator l = SSNOE.lower_bound(make_pair(
      										make_pair(HaHn_R_i->second.first.second, HaHn_R_i->second.first.first), HnHa));
      					NOEtypeListN::iterator u = SSNOE.upper_bound(make_pair(
      										make_pair(HaHn_R_i->second.first.second, HaHn_R_i->second.first.first), HnHa));
      					for(NOEtypeListN::iterator i=l; i != u; i++)
                  {
                        strand1count.insert(HaHn_R_i->second.first.first);
                        strand2count.insert(HaHn_R_i->second.first.second);
                  }

      					l = SSNOE.lower_bound(make_pair(
      										make_pair(HaHn_R_i->second.first.first, HaHn_R_i->second.first.second), HaHn));
      					 u = SSNOE.upper_bound(make_pair(
      										make_pair(HaHn_R_i->second.first.first, HaHn_R_i->second.first.second), HaHn));
      					for(NOEtypeListN::iterator i=l; i != u; i++)
                  {
                        strand1count.insert(HaHn_R_i->second.first.first);
                        strand2count.insert(HaHn_R_i->second.first.second);
                  }
                }
                old = HaHn_R_i->second.first;
      			  }
      		}
          old = make_pair(0,0);

      		for(ssRRMapNInt::iterator HnHa_R_i = HnHa_R_il; HnHa_R_i != HnHa_R_iu; HnHa_R_i++)
      		{
      			  if (((iterH->first.first - ((*HnHa_R_i).second.first.first)) == ((*HnHa_R_i).second.first.second - iterH->first.second) - 1) &&
      			      ((fmod(abs(iterH->first.first - ((*HnHa_R_i).second.first.first)), 2) == 0) &&
      			       (fmod(abs(iterH->first.second - ((*HnHa_R_i).second.first.second)), 2) == 1)) )// && //)
      			  {
                 if (old != HnHa_R_i->second.first) // second time
                {
      					NOEtypeListN::iterator l = SSNOE.lower_bound(make_pair(
      										make_pair(HnHa_R_i->second.first.second, HnHa_R_i->second.first.first), HaHn));
      					NOEtypeListN::iterator u = SSNOE.upper_bound(make_pair(
      										make_pair(HnHa_R_i->second.first.second, HnHa_R_i->second.first.first), HaHn));
      					for(NOEtypeListN::iterator i=l; i != u; i++)
                  {
                        strand1count.insert(HnHa_R_i->second.first.first);
                        strand2count.insert(HnHa_R_i->second.first.second);
                  }

      					 l = SSNOE.lower_bound(make_pair(
      										make_pair(HnHa_R_i->second.first.first, HnHa_R_i->second.first.second), HnHa));
      					 u = SSNOE.upper_bound(make_pair(
      										make_pair(HnHa_R_i->second.first.first, HnHa_R_i->second.first.second), HnHa));
      					for(NOEtypeListN::iterator i=l; i != u; i++)
                  {
                        strand1count.insert(HnHa_R_i->second.first.first);
                        strand2count.insert(HnHa_R_i->second.first.second);
                  }
                }
                old = HnHa_R_i->second.first;
      			  }
      		}
          old = make_pair(0,0);

	if (!NOESY_Assign::MINCON())
          if ((strand1count.size() < 2) || (strand2count.size() < 2))
          {
                  NOESY_Assign::outputSec() << "CASE(d) SS-Remove from anti_register: " << iterH->first.first << "-" << iterH->first.second << " " << endl;
                  anti_register.erase(anti_register.find(iterH->first));
          }
   }
    		//
    		// for each register, print all support NOE informations with score
    		//

  apList.clear();
  apList.insert(para_register.begin(), para_register.end());
  for(RRFreq::iterator iterH = apList.begin(); iterH != apList.end(); iterH++)
  {
        set<int, less<int> > strand1count; strand1count.clear();
        set<int, less<int> > strand2count; strand2count.clear();

     		const Structure* strand1 = intStructMap.find(iterH->first.first)->second;
    		const Structure* strand2 = intStructMap.find(iterH->first.second)->second;

   	    ssRRMapNInt::iterator HaHa_R_il = HaHa_R.lower_bound(make_pair(strand1->get_strand_ID(), strand2->get_strand_ID()));
   	    ssRRMapNInt::iterator HaHa_R_iu = HaHa_R.upper_bound(make_pair(strand1->get_strand_ID(), strand2->get_strand_ID()));

   	    ssRRMapNInt::iterator HnHn_R_il = HnHn_R.lower_bound(make_pair(strand1->get_strand_ID(), strand2->get_strand_ID()));
   	    ssRRMapNInt::iterator HnHn_R_iu = HnHn_R.upper_bound(make_pair(strand1->get_strand_ID(), strand2->get_strand_ID()));

   	    ssRRMapNInt::iterator HaHn_R_il = HaHn_R.lower_bound(make_pair(strand1->get_strand_ID(), strand2->get_strand_ID()));
   	    ssRRMapNInt::iterator HaHn_R_iu = HaHn_R.upper_bound(make_pair(strand1->get_strand_ID(), strand2->get_strand_ID()));

   	    ssRRMapNInt::iterator HnHa_R_il = HnHa_R.lower_bound(make_pair(strand1->get_strand_ID(), strand2->get_strand_ID()));
   	    ssRRMapNInt::iterator HnHa_R_iu = HnHa_R.upper_bound(make_pair(strand1->get_strand_ID(), strand2->get_strand_ID()));


          pair<int,int> old = make_pair(0,0);
      		for(ssRRMapNInt::iterator HaHn_R_i = HaHn_R_il; HaHn_R_i != HaHn_R_iu; HaHn_R_i++)
      		{
    			  if (((iterH->first.first - ((*HaHn_R_i).second.first.first)) == (iterH->first.second - ((*HaHn_R_i).second.first.second))) &&
    			      (fmod(abs(iterH->first.first - ((*HaHn_R_i).second.first.first)), 2) == 0))
      			  {

                if (old != HaHn_R_i->second.first) // second time
                {
      					NOEtypeListN::iterator l = SSNOE.lower_bound(make_pair(
      										make_pair(HaHn_R_i->second.first.second, HaHn_R_i->second.first.first), HnHa));
      					NOEtypeListN::iterator u = SSNOE.upper_bound(make_pair(
      										make_pair(HaHn_R_i->second.first.second, HaHn_R_i->second.first.first), HnHa));
      					for(NOEtypeListN::iterator i=l; i != u; i++)
                  {
                        strand1count.insert(HaHn_R_i->second.first.first);
                        strand2count.insert(HaHn_R_i->second.first.second);
                  }

      					 l = SSNOE.lower_bound(make_pair(
      										make_pair(HaHn_R_i->second.first.first, HaHn_R_i->second.first.second), HaHn));
      					 u = SSNOE.upper_bound(make_pair(
      										make_pair(HaHn_R_i->second.first.first, HaHn_R_i->second.first.second), HaHn));
      					for(NOEtypeListN::iterator i=l; i != u; i++)
                  {
                        strand1count.insert(HaHn_R_i->second.first.first);
                        strand2count.insert(HaHn_R_i->second.first.second);
                  }
                }
                old = HaHn_R_i->second.first;
      			  }
      		}

          old = make_pair(0,0);
      		for(ssRRMapNInt::iterator HnHa_R_i = HnHa_R_il; HnHa_R_i != HnHa_R_iu; HnHa_R_i++)
      		{
    			  if (((iterH->first.first - ((*HnHa_R_i).second.first.first)) == (iterH->first.second - ((*HnHa_R_i).second.first.second))) &&
    			      (fmod(abs(iterH->first.first - ((*HnHa_R_i).second.first.first)), 2) == 1))
      			  {

               if (old != HnHa_R_i->second.first) // second time
                {
       					NOEtypeListN::iterator l = SSNOE.lower_bound(make_pair(
      										make_pair(HnHa_R_i->second.first.second, HnHa_R_i->second.first.first), HaHn));
      					NOEtypeListN::iterator u = SSNOE.upper_bound(make_pair(
      										make_pair(HnHa_R_i->second.first.second, HnHa_R_i->second.first.first), HaHn));
      					for(NOEtypeListN::iterator i=l; i != u; i++)
                  {
                        strand1count.insert(HnHa_R_i->second.first.first);
                        strand2count.insert(HnHa_R_i->second.first.second);
                  }

      					 l = SSNOE.lower_bound(make_pair(
      										make_pair(HnHa_R_i->second.first.first, HnHa_R_i->second.first.second), HnHa));
      					 u = SSNOE.upper_bound(make_pair(
      										make_pair(HnHa_R_i->second.first.first, HnHa_R_i->second.first.second), HnHa));
      					for(NOEtypeListN::iterator i=l; i != u; i++)
                  {
                        strand1count.insert(HnHa_R_i->second.first.first);
                        strand2count.insert(HnHa_R_i->second.first.second);
                  }
                }

                old = HnHa_R_i->second.first;
      			  }
      		}


          old = make_pair(0,0);
      		for(ssRRMapNInt::iterator HnHn_R_i = HnHn_R_il; HnHn_R_i != HnHn_R_iu; HnHn_R_i++)
      		{
    			  if (((((iterH->first.first - ((*HnHn_R_i).second.first.first)) == -1 + (iterH->first.second - ((*HnHn_R_i).second.first.second))) &&
    			      (fmod(abs(iterH->first.first - ((*HnHn_R_i).second.first.first)), 2) == 1) &&
    			      (fmod(abs(iterH->first.second - ((*HnHn_R_i).second.first.second)), 2) == 0)) ||
    			     (((iterH->first.first - ((*HnHn_R_i).second.first.first)) == 1 + (iterH->first.second - ((*HnHn_R_i).second.first.second))) &&
    			      (fmod(abs(iterH->first.second - ((*HnHn_R_i).second.first.second)), 2) == 0) &&
    			      (fmod(abs(iterH->first.first - ((*HnHn_R_i).second.first.first)), 2) == 1))) ) //&&
      			  {
      			    //RR_i->second += 2;
      					if (old != HnHn_R_i->second.first) // second time
                {
      					NOEtypeListN::iterator l = SSNOE.lower_bound(make_pair(
      										make_pair(HnHn_R_i->second.first.second, HnHn_R_i->second.first.first), HnHn));
      					NOEtypeListN::iterator u = SSNOE.upper_bound(make_pair(
      										make_pair(HnHn_R_i->second.first.second, HnHn_R_i->second.first.first), HnHn));
      					for(NOEtypeListN::iterator i=l; i != u; i++)
                  {
                        strand1count.insert(HnHn_R_i->second.first.first);
                        strand2count.insert(HnHn_R_i->second.first.second);
                  }
      					 l = SSNOE.lower_bound(make_pair(
      										make_pair(HnHn_R_i->second.first.first, HnHn_R_i->second.first.second), HnHn));
      					 u = SSNOE.upper_bound(make_pair(
      										make_pair(HnHn_R_i->second.first.first, HnHn_R_i->second.first.second), HnHn));
      					for(NOEtypeListN::iterator i=l; i != u; i++)
                  {
                        strand1count.insert(HnHn_R_i->second.first.first);
                        strand2count.insert(HnHn_R_i->second.first.second);
                  }
                }
                old = HnHn_R_i->second.first;
      			  }
      		}
	if (!NOESY_Assign::MINCON())
          if ((strand1count.size() < 2) || (strand2count.size() < 2))
          {
                  NOESY_Assign::outputSec() << "CASE(d) SS-Remove from para_register: " << iterH->first.first << "-" << iterH->first.second << " " << endl;
                  para_register.erase(para_register.find(iterH->first));
          }


   }

  ///////////////////////////////
  // refine the C/N-terminala  //
  // based on inter-strand NOE //
  ///////////////////////////////

  // for the inter-strand sheet part, antiparallel,
  multimap<pair<int, int>, pair<int, int>, less<pair<int, int> > > cEndRR; cEndRR.clear();
  multimap<pair<int, int>, pair<int, int>, less<pair<int, int> > > nEndRR; nEndRR.clear();

  int ssType = 0;

  //c-terminal
  for(RRFreq::iterator iterH = anti_register.begin();
      iterH != anti_register.end(); iterH++)
    {
      ssType = -1;

      // find the co-ending points
      const Structure* strand1 = intStructMap.find(iterH->first.first)->second;   //intStructMap -> map to the correct strand
      const Structure* strand2 = intStructMap.find(iterH->first.second)->second;
      CSIMap::iterator i; CSIMap::reverse_iterator j;
      for(i = ssFilter.find(*(strand1->get_chain().begin())); i != ssFilter.end(); i++)
      {
        if (i->second == ssType)
          continue;
        else
          break;
      }
      for(j = (CSIMap::reverse_iterator)ssFilter.find(*(strand2->get_chain().rbegin())); j != ssFilter.rend(); j++)
      {
        if (j->second == ssType)
          continue;
        else
          break;
      }

      //i--; j--; //extend by one-residue

      if (DEBUG)
        NOESY_Assign::outputSec() << i->first << " " << j->first << endl;

      while (abs(i->first-iterH->first.first) < abs(iterH->first.second-j->first))
      {
        j--;
      }
      while (abs(i->first-iterH->first.first) > abs(iterH->first.second-j->first))
      {
        i--;
      }

      if (DEBUG)
        NOESY_Assign::outputSec() << "refine " << i->first << " " << j->first << endl;
      // find the co-ending points with NOE also
      bool succ= 0;
      do
      {
        if (abs(i->first - j->first) <= 1)
        {
            i--; j--;
        }
        if ((fmod(abs(i->first-iterH->first.first), 2) == 1) &&
            (((SSNOE.find(make_pair(make_pair(i->first, j->first), HaHa)) == SSNOE.end()) &&
              ((SSNOE.find(make_pair(make_pair(j->first, i->first), HaHa)) == SSNOE.end())) &&
							(ECList.find(make_pair(i->first, j->first)) == ECList.end()) &&
							(ECList.find(make_pair(j->first, i->first)) == ECList.end())) &&
              // if both i->first+1 and j->first are out bound, do not count
             ((((i->first+1 > *(strand1->get_chain().rbegin())) && (j->first < *(strand2->get_chain().begin()))) ||
                (abs(i->first+1-j->first) <=2)) ||
              ((SSNOE.find(make_pair(make_pair(i->first+1, j->first), HnHa)) == SSNOE.end()) &&
              (SSNOE.find(make_pair(make_pair(j->first, i->first+1), HaHn)) == SSNOE.end()) &&
							(ECList.find(make_pair(i->first+1, j->first)) == ECList.end()) &&
							(ECList.find(make_pair(j->first, i->first+1)) == ECList.end()))
							)))
        {
          if (DEBUG)
            NOESY_Assign::outputSec() << "case 1" << i->first << " " <<  j->first << " " << i->first+1 << endl;
          i--;
          j--;
        } else
        if ((fmod(abs(i->first-iterH->first.first), 2) == 0) &&
            (((SSNOE.find(make_pair(make_pair(i->first, j->first), HnHn)) == SSNOE.end()) &&
             ((SSNOE.find(make_pair(make_pair(j->first, i->first), HnHn)) == SSNOE.end())) &&
						 (ECList.find(make_pair(i->first, j->first)) == ECList.end()) &&
						 (ECList.find(make_pair(j->first, i->first)) == ECList.end())) &&
             (((i->first+1 > *(strand1->get_chain().rbegin())) && (j->first < *(strand2->get_chain().begin()))) || (abs(i->first+1-j->first) <=2) ||
              ((SSNOE.find(make_pair(make_pair(i->first+1, j->first), HaHn)) == SSNOE.end()) &&
              (SSNOE.find(make_pair(make_pair(j->first, i->first+1), HnHa)) == SSNOE.end()) &&
							(ECList.find(make_pair(i->first+1, j->first)) == ECList.end()) &&
							(ECList.find(make_pair(j->first, i->first+1)) == ECList.end()))
						)))
         {
          if (DEBUG)
            NOESY_Assign::outputSec() << "case 2" << i->first << " " <<  j->first << " " << i->first+1 << endl;
          i--;
          j--;
        } else
        {
          succ = 1;
          break;
        }
      } while ((i != ssFilter.begin()) && (j != ssFilter.rbegin())); // && ((i->second == ssType) || (j->second == ssType)));

      if (succ)
      {
        if (DEBUG)
          NOESY_Assign::outputSec() << "succ" << i->first << " " << j->first << endl;
        cEndRR.insert(make_pair(iterH->first, make_pair(i->first,j->first)));
      }
    }

  for(RRFreq::iterator iterH = para_register.begin();
      iterH != para_register.end(); iterH++)
    {
      ssType = -1;

      // find the co-ending points
     		const Structure* strand1 = intStructMap.find(iterH->first.first)->second;
		const Structure* strand2 = intStructMap.find(iterH->first.second)->second;
      CSIMap::iterator i,j;
      for(i = ssFilter.find(*(strand1->get_chain().begin())); i != ssFilter.end(); i++)
      {
        if (i->second == ssType)
          continue;
        else
          break;
      }
      for(j = ssFilter.find(*(strand2->get_chain().begin())); j != ssFilter.end(); j++)
      {
        if (j->second == ssType)
          continue;
        else
          break;
      }

      while (abs(i->first-iterH->first.first) < abs(iterH->first.second-j->first))
      {
        j--;
      }
      while (abs(i->first-iterH->first.first) > abs(iterH->first.second-j->first))
      {
        i--;
      }

      if (DEBUG)
        NOESY_Assign::outputSec() << "refine para " << i->first << " " << j->first << endl;

      bool succ = 0;
      // find the co-ending points with NOE also
      do
      {
        if ((fmod(abs(i->first-iterH->first.first), 2) == 1) &&
            (((SSNOE.find(make_pair(make_pair(i->first, j->first), HnHa)) == SSNOE.end()) &&
              (SSNOE.find(make_pair(make_pair(j->first, i->first), HaHn)) == SSNOE.end()) &&
							(ECList.find(make_pair(i->first, j->first)) == ECList.end()) &&
							(ECList.find(make_pair(j->first, i->first)) == ECList.end())) &&
             ((((j->first+1 > *(strand2->get_chain().rbegin())) && (i->first > *(strand1->get_chain().rbegin())))  || (abs(j->first+1-i->first) <=2)) ||
              ((SSNOE.find(make_pair(make_pair(i->first, j->first+1), HnHn)) == SSNOE.end()) &&
              (SSNOE.find(make_pair(make_pair(j->first+1, i->first), HnHn)) == SSNOE.end()) &&
							(ECList.find(make_pair(i->first, j->first+1)) == ECList.end()) &&
							(ECList.find(make_pair(j->first+1, i->first)) == ECList.end()))
						)))
        {
        	i--; j--;
        } else
        if ((fmod(abs(i->first-iterH->first.first), 2) == 0) &&
            (((SSNOE.find(make_pair(make_pair(i->first, j->first), HaHn)) == SSNOE.end()) &&
              (SSNOE.find(make_pair(make_pair(j->first, i->first), HnHa)) == SSNOE.end()) &&
							(ECList.find(make_pair(i->first, j->first)) == ECList.end()) &&
							(ECList.find(make_pair(j->first, i->first)) == ECList.end())) &&
             ((((i->first+1 > *(strand1->get_chain().rbegin())) && (j->first > *(strand2->get_chain().rbegin()))) || (abs(i->first+1-j->first) <=2)) ||
              ((SSNOE.find(make_pair(make_pair(i->first+1, j->first), HnHn)) == SSNOE.end()) &&
              (SSNOE.find(make_pair(make_pair(j->first, i->first+1), HnHn)) == SSNOE.end()) &&
							(ECList.find(make_pair(i->first+1, j->first)) == ECList.end()) &&
							(ECList.find(make_pair(j->first, i->first+1)) == ECList.end()))
						)))
        {
        	i--; j--;
        } else
        {
          succ = 1;
          break;
        }
      } while ((i != ssFilter.begin()) && (j != ssFilter.begin())); // && ((i->second == ssType) || (j->second == ssType)));

      if (succ)
      {
        if (DEBUG)
          NOESY_Assign::outputSec() << "succ " << i->first << " " << j->first << endl;
        cEndRR.insert(make_pair(iterH->first, make_pair(i->first,j->first)));
      }
    }

  // n-terminal
  for(RRFreq::iterator iterH = anti_register.begin();
      iterH != anti_register.end(); iterH++)
    {
      ssType = -1;

      // find the co-ending points
     	const Structure* strand1 = intStructMap.find(iterH->first.first)->second;
		  const Structure* strand2 = intStructMap.find(iterH->first.second)->second;
      CSIMap::reverse_iterator i; CSIMap::iterator j;
      for(i= (CSIMap::reverse_iterator)ssFilter.find(*(strand1->get_chain().rbegin())); i!= ssFilter.rend(); i++)
      {
        if (DEBUG) NOESY_Assign::outputSec() << "find the first i " << i->first << endl;
        if (i->second == ssType)
          continue;
        else
          break;
      }
       for(j = ssFilter.find(*(strand2->get_chain().begin())); j != ssFilter.end(); j++)
      {
        if (DEBUG) NOESY_Assign::outputSec() << "find the first j " << j->first << endl;
        if (j->second == ssType)
          continue;
        else
          break;
      }

      if (DEBUG)
	      	NOESY_Assign::outputSec() << i->first << " " << j->first << endl;

      while (abs(i->first-iterH->first.first) < abs(iterH->first.second-j->first))
      {
        j--;
      }
      while (abs(i->first-iterH->first.first) > abs(iterH->first.second-j->first))
      {
        i--;
      }

      if (DEBUG)
        NOESY_Assign::outputSec() << "N-start " << i->first << " " << j->first << endl;

      // find the co-ending points with NOE also
      bool succ = 0;
      do
      {
        if ((fmod(abs(i->first-iterH->first.first), 2) == 1) &&
            (((SSNOE.find(make_pair(make_pair(i->first, j->first), HaHa)) == SSNOE.end()) &&
              (SSNOE.find(make_pair(make_pair(j->first, i->first), HaHa)) == SSNOE.end()) &&
							(ECList.find(make_pair(i->first, j->first)) == ECList.end()) &&
							(ECList.find(make_pair(j->first, i->first)) == ECList.end())) &&
             ((((j->first+1 > *(strand2->get_chain().rbegin())) && (i->first < *(strand1->get_chain().begin()))) || (abs(j->first+1-i->first) <=2)) ||
              ((SSNOE.find(make_pair(make_pair(j->first+1, i->first), HnHa)) == SSNOE.end()) &&
              (SSNOE.find(make_pair(make_pair(i->first, j->first+1), HaHn)) == SSNOE.end()) &&
							(ECList.find(make_pair(i->first+1, j->first)) == ECList.end()) &&
							(ECList.find(make_pair(j->first, i->first+1)) == ECList.end()))
						)))
        {
         if (DEBUG)
            NOESY_Assign::outputSec() << "case N-1 " << i->first << " " <<  j->first << " " << j->first+1 << endl;
         if (DEBUG)
            NOESY_Assign::outputSec() << "rbegin " << *(strand2->get_chain().rbegin()) << " " << endl;
         if (DEBUG)
            NOESY_Assign::outputSec() << "find " << ((SSNOE.find(make_pair(make_pair(j->first+1, i->first), HnHa)) == SSNOE.end()) &&
              (SSNOE.find(make_pair(make_pair(i->first, j->first+1), HaHn)) == SSNOE.end())) << endl;
          i--;
          j--;
        } else
        if ((fmod(abs(i->first-iterH->first.first), 2) == 0) &&
            (((SSNOE.find(make_pair(make_pair(i->first, j->first), HnHn)) == SSNOE.end()) &&
              (SSNOE.find(make_pair(make_pair(j->first, i->first), HnHn)) == SSNOE.end()) &&
							(ECList.find(make_pair(i->first, j->first)) == ECList.end()) &&
							(ECList.find(make_pair(j->first, i->first)) == ECList.end())) &&
             ((((j->first+1 > *(strand2->get_chain().rbegin())+1) && (i->first < *(strand1->get_chain().begin()))) || (abs(j->first+1-i->first) <=2)) ||
              ((SSNOE.find(make_pair(make_pair(j->first+1, i->first), HaHn)) == SSNOE.end()) &&
              (SSNOE.find(make_pair(make_pair(i->first, j->first+1), HnHa)) == SSNOE.end()) &&
							(ECList.find(make_pair(i->first+1, j->first)) == ECList.end()) &&
							(ECList.find(make_pair(j->first, i->first+1)) == ECList.end()))
						)))
         {
         if (DEBUG)
            NOESY_Assign::outputSec() << "case N-2 " << i->first << " " <<  j->first << " " << j->first+1 << endl;
          i--;
          j--;
        } else
        {
          succ = 1;
          break;
        }
      } while ((i != ssFilter.rbegin()) && (j != ssFilter.begin())); // && ((i->second == ssType) || (j->second == ssType)));


      if (succ)
      {
        if (DEBUG)
          NOESY_Assign::outputSec() << i->first << " " << j->first << endl;
        nEndRR.insert(make_pair(iterH->first, make_pair(i->first,j->first)));
      }
    }

  for(RRFreq::iterator iterH = para_register.begin();
      iterH != para_register.end(); iterH++)
    {
      ssType = -1;

      // find the co-ending points
     		const Structure* strand1 = intStructMap.find(iterH->first.first)->second;
		const Structure* strand2 = intStructMap.find(iterH->first.second)->second;
      CSIMap::reverse_iterator i; CSIMap::reverse_iterator j;
       for(i = (CSIMap::reverse_iterator)ssFilter.find(*(strand1->get_chain().rbegin())); i != ssFilter.rend(); i++)
      {
        if (i->second == ssType)
          continue;
        else
          break;
      }
      for(j = (CSIMap::reverse_iterator)ssFilter.find(*(strand2->get_chain().rbegin())); j != ssFilter.rend(); j++)
      {
        if (j->second == ssType)
          continue;
        else
          break;
      }

      while (abs(i->first-iterH->first.first) < abs(iterH->first.second-j->first))
      {
        j--;
      }
      while (abs(i->first-iterH->first.first) > abs(iterH->first.second-j->first))
      {
        i--;
      }

      if (DEBUG)
        NOESY_Assign::outputSec() << "refine paraN " << i->first << " " << j->first << endl;

      bool succ = 0;
      // find the co-ending points with NOE also
      do
      {
        if ((fmod(abs(i->first-iterH->first.first), 2) == 1) &&
            (((SSNOE.find(make_pair(make_pair(i->first, j->first), HnHa)) == SSNOE.end()) &&
              (SSNOE.find(make_pair(make_pair(j->first, i->first), HaHn)) == SSNOE.end())) &&
							(ECList.find(make_pair(i->first, j->first)) == ECList.end()) &&
							(ECList.find(make_pair(j->first, i->first)) == ECList.end())) &&
             ((((j->first-1 < *(strand2->get_chain().begin())) && (i->first < *(strand1->get_chain().begin()))) || (abs(j->first-1-i->first) <=2)) ||
              ((SSNOE.find(make_pair(make_pair(i->first, j->first-1), HnHn)) == SSNOE.end()) &&
              (SSNOE.find(make_pair(make_pair(j->first-1, i->first), HnHn)) == SSNOE.end()) &&
							(ECList.find(make_pair(i->first, j->first-1)) == ECList.end()) &&
							(ECList.find(make_pair(j->first-1, i->first)) == ECList.end()))
						))
        {
        	i--; j--;
        } else
        if ((fmod(abs(i->first-iterH->first.first), 2) == 0) &&
            (((SSNOE.find(make_pair(make_pair(i->first, j->first), HaHn)) == SSNOE.end()) &&
              (SSNOE.find(make_pair(make_pair(j->first, i->first), HnHa)) == SSNOE.end()) &&
							(ECList.find(make_pair(i->first, j->first)) == ECList.end()) &&
							(ECList.find(make_pair(j->first, i->first)) == ECList.end())) &&
             ((((i->first-1 > *(strand1->get_chain().begin())) && (j->first < *(strand2->get_chain().begin()))) || (abs(i->first-1-j->first) <=2)) ||
              ((SSNOE.find(make_pair(make_pair(i->first-1, j->first), HnHn)) == SSNOE.end()) &&
              (SSNOE.find(make_pair(make_pair(j->first, i->first-1), HnHn)) == SSNOE.end()) &&
							(ECList.find(make_pair(i->first-1, j->first)) == ECList.end()) &&
							(ECList.find(make_pair(j->first, i->first-1)) == ECList.end()))
						)))
        {
        	i--; j--;
        } else
        {
          succ = 1;
          break;
        }
      } while ((i != ssFilter.rbegin()) && (j != ssFilter.rbegin())); // && (i->second == ssType) && (j->second == ssType));

      if (succ)
      {
        if (DEBUG)
          NOESY_Assign::outputSec() << "succ" << i->first << " " << j->first << endl;
        nEndRR.insert(make_pair(iterH->first, make_pair(i->first,j->first)));
      }
    }


  // all b-strand residue are included in the segment set
  set<int, less<int> > segment; segment.clear();
  RRFreq HRegister; HRegister.clear();
  HRegister.insert(anti_register.begin(), anti_register.end());
  HRegister.insert(para_register.begin(), para_register.end());

  for(RRFreq::iterator iterH = HRegister.begin();
      iterH != HRegister.end(); iterH++)
  {
    multimap<pair<int, int>, pair<int, int>, less<pair<int, int> > >::iterator ci = cEndRR.find(iterH->first);
    multimap<pair<int, int>, pair<int, int>, less<pair<int, int> > >::iterator ni = nEndRR.find(iterH->first);

    RRFreq::iterator ia = anti_register.find(iterH->first);
    RRFreq::iterator ip = para_register.find(iterH->first);

    if (ia != anti_register.end())
    {
      for(int k = ni->second.first; k <= ci->second.first; k++)
          segment.insert(k);
      for(int k = ci->second.second; k <= ni->second.second; k++)
          segment.insert(k);
    } else
    if (ip != para_register.end())
    {
        for(int k = ni->second.first; k <= ci->second.first; k++)
          segment.insert(k);
        for(int k = ni->second.second; k <= ci->second.second; k++)
          segment.insert(k);
    }
  }

  if (DEBUG)
    for(set<int, less<int> >::iterator i = segment.begin(); i!= segment.end(); i++)
      NOESY_Assign::outputSec() << "segment " << *i << endl;

  for(CSIMap::iterator i = ssFilter.begin(); i != ssFilter.end(); i++)
  {
    if ((i->second == -1) && (segment.find(i->first) == segment.end())) // if not in the segment, unset
    {
	if (ECList.size() || !NOESY_Assign::MINCON())  // check this
	{
      i->second = 0;
      //if (DEBUG)
      NOESY_Assign::outputSec() << "Remove " << i->first << " as NO inter-strand residue found" << endl;
	}
    } else
    if ((i->second != 1) && (segment.find(i->first) != segment.end()))
    {
      JvalList::const_iterator jl = JList.lower_bound(i->first);
      JvalList::const_iterator ju = JList.upper_bound(i->first);
      for(JvalList::const_iterator ji = jl; ji != ju; ji++)
      {
        if (ji->second.get_type() == J3HNHA)
        {
          if (ji->second.get_upper() >= parSec.jS - 0.2) // strand, but J < 7
          {
          	if (i->second != -1)
           	{
      				i->second = -1;
      				NOESY_Assign::outputSec() << "Insert " << i->first << " into b-sheet, supported by inter-strand NOEi & J" << endl;
          	}
          }
        }
      } // end for
    }
  }

  if (DEBUG)
    for(CSIMap::iterator i = ssFilter.begin(); i != ssFilter.end(); i++)
    {
	    NOESY_Assign::outputSec()  << i->first << "\t" << i->second << endl;
    }

  // adjut the regiter to be in the region of the b-sheet or force to add the reigster in the b-sheet
  HRegister.clear();
  HRegister.insert(anti_register.begin(), anti_register.end());
  HRegister.insert(para_register.begin(), para_register.end());

  for(RRFreq::iterator iterH = HRegister.begin();
      iterH != HRegister.end(); iterH++)
  {
    multimap<pair<int, int>, pair<int, int>, less<pair<int, int> > >::iterator ci = cEndRR.find(iterH->first);
    multimap<pair<int, int>, pair<int, int>, less<pair<int, int> > >::iterator ni = nEndRR.find(iterH->first);

    if (DEBUG)
      NOESY_Assign::outputSec() << ni->second.first << "-" << ci->second.first << " " << ci->second.second << "-" << ni->second.second << endl;

    RRFreq::iterator ia = anti_register.find(iterH->first);
    RRFreq::iterator ip = para_register.find(iterH->first);

    if (ia != anti_register.end())
    {
    	int first = ia->first.first;
    	int second = ia->first.second;

      if (DEBUG)
        NOESY_Assign::outputSec() << first << " " << second << " " << ssFilter.find(first)->second << endl;

      // try to find both ssFilter == -1
      bool in = 0;
      if ((ssFilter.find(first))->second != -1) // should be in the terminal
      {
          in = 1;
          if (first == ni->second.first)
          {
              first +=2; second -=2;
          }
          else if (first == ci->second.first)
          {
              first -=2; second +=2;
          }
          if (DEBUG) NOESY_Assign::outputSec() << "2 " << first << " " << second << endl;
      }
      if ((ssFilter.find(second))->second != -1) // should be in the terminal
      {
          in = 1;
          if (second == ni->second.second)
          {
              first +=2; second -=2;
          }
          else if (second == ci->second.second)
          {
              first -=2; second +=2;
          }
          if (DEBUG) NOESY_Assign::outputSec() << "3 " << first << " " << second << endl;
      }

      if (in && (first == ia->first.first) && (second == ia->first.second))      // moved and moved back (bad-bad) case only
      {
          // force to add in as b-sheet
          if (ssFilter.find(second)->second != -1)
          {
      				ssFilter.find(second)->second = -1;
      				NOESY_Assign::outputSec() << "Insert " << second << " into b-sheet, supported by inter-strand NOEi only (short length)" << endl;
          }
          if (ssFilter.find(first)->second != -1)
          {
      				ssFilter.find(first)->second = -1;
      				NOESY_Assign::outputSec() << "Insert " << first << " into b-sheet, supported by inter-strand NOEi only (short length)" << endl;
          }
      }

        float freq = ia->second;

        anti_register.erase(ia);
        anti_register.insert(make_pair(make_pair(first,second), freq));

    } else
    if (ip != para_register.end())
    {
    	int first = ip->first.first;
    	int second = ip->first.second;

      if (DEBUG)
        NOESY_Assign::outputSec() << first << " " << second << " " << ssFilter.find(first)->second << endl;

      // try to find both ssFilter == -1
      bool in = 0;
      if ((ssFilter.find(first))->second != -1) // should be in the terminal
      {
          in = 1;
          if (first == ni->second.first)
          {
              first +=2; second +=2;
          }
          else if (first == ci->second.first)
          {
              first -=2; second -=2;
          }
          if (DEBUG) NOESY_Assign::outputSec() << "2 " << first << " " << second << endl;
      }
      if ((ssFilter.find(second))->second != -1) // should be in the terminal
      {
          in = 1;
          if (second == ni->second.second)
          {
              first +=2; second +=2;
          }
          else if (second == ci->second.second)
          {
              first -=2; second -=2;
          }
          if (DEBUG) NOESY_Assign::outputSec() << "3 " << first << " " << second << endl;
      }

      if (in && (first == ia->first.first) && (second == ia->first.second))      // moved and moved back (bad-bad) case only
      {
          // force to add in as b-sheet
          if (ssFilter.find(second)->second != -1)
          {
      				ssFilter.find(second)->second = -1;
      				NOESY_Assign::outputSec() << "Insert " << second << " into b-sheet, supported by inter-strand NOEi only (short length)" << endl;
          }
          if (ssFilter.find(first)->second != -1)
          {
      				ssFilter.find(first)->second = -1;
      				NOESY_Assign::outputSec() << "Insert " << first << " into b-sheet, supported by inter-strand NOEi only (short length)" << endl;
          }
      }

      float freq = ip->second;

      para_register.erase(ip);
      para_register.insert(make_pair(make_pair(first,second), freq));
    }

  }

	////////////////////////////////
	// save to **F                //
	////////////////////////////////

	for(RRFreq::iterator i = anti_register.begin(); i != anti_register.end(); i++)
  {
      anti_registerF.push_back(i->first);
  }
	for(RRFreq::iterator i = para_register.begin(); i != para_register.end(); i++)
  {
      para_registerF.push_back(i->first);
  }

  {
    // the secondary structure element is added based on the combined analysis of CSI, J, NH and NOE
    auto_ss.clear();
    Structure auto_stru;
    int type = (*ssFilter.begin()).second; int ID = 1;
    auto_stru.set_type(type);
    auto_stru.set_strand_ID(ID);
    for(CSIMap::iterator i = ssFilter.begin(); i != ssFilter.end(); i++)
    {
      if ((*i).second == type)
  	  {
  	    auto_stru.addinChain((*i).first);
  	  }
      else
  	  {
    	  auto_ss.insert(make_pair(itoa(ID), auto_stru));
    	  auto_stru.erase();
    	  ID++;
    	  auto_stru.set_type((*i).second);
    	  auto_stru.set_strand_ID(ID);
    	  auto_stru.addinChain((*i).first);
    	  type = (*i).second;
  	  }
    }
    auto_ss.insert(make_pair(itoa(ID), auto_stru));
    addSS2AA(auto_ss);

    //if (DEBUG)
    {
      NOESY_Assign::outputSec() << endl << "Final Secondary Struture is defined as: " << endl;
      NOESY_Assign::outputSec() <<         "=======================================  " << endl << endl;
      for(SS::iterator i = auto_ss.begin(); i != auto_ss.end(); i++)
      {
        NOESY_Assign::outputSec() << "ID=" << (*i).first << "\t" << (*i).second << endl;
      }
      NOESY_Assign::outputSec() << endl;
    }
  }

  if (!anti_registerF.empty())
	{
	  NOESY_Assign::outputSec() << endl << "AntiParallel Strands Registers Found (Summary): " << endl;
	  NOESY_Assign::outputSec() <<         "==============================================  " << endl;
	  for(deque<HBOND>::iterator i = anti_registerF.begin(); i != anti_registerF.end(); i++)
	    NOESY_Assign::outputSec() << (*i).first << "-" << (*i).second << " ";
	  NOESY_Assign::outputSec() << endl << endl;
	}
	if (!para_registerF.empty())
	{
	  NOESY_Assign::outputSec() << endl << "Parallel Strands Registers Found (Summary): " << endl;
	  NOESY_Assign::outputSec() << "==========================================  " << endl;
	  for(deque<HBOND>::iterator i = para_registerF.begin(); i != para_registerF.end(); i++)
	    NOESY_Assign::outputSec() << (*i).first << "-" << (*i).second << " ";
	  NOESY_Assign::outputSec() << endl;
	}

  // --------------------------------------------------------------------
  // for each register, print all support NOE informations with score
  // --------------------------------------------------------------------

  NOESY_Assign::outputSec() << endl << endl << "Selected Supporting Evidences for All Possible Registers:" << endl;
  NOESY_Assign::outputSec() << "========================================================== " << endl;

  for(SS::iterator i = auto_ssT.begin(); i != auto_ssT.end(); i++)
    for(SS::iterator j = i; j != auto_ssT.end(); j++)
    {
    	if ((j != i) && ((*i).second.get_type() == -1) && ((*j).second.get_type() == -1)) // for b-strands
      {
				ssRRMapNInt::iterator EC_R_il = EC_R.lower_bound(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()));
   	    ssRRMapNInt::iterator EC_R_iu = EC_R.upper_bound(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()));

   	    ssRRMapNInt::iterator HaHa_R_il = HaHa_R.lower_bound(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()));
   	    ssRRMapNInt::iterator HaHa_R_iu = HaHa_R.upper_bound(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()));

   	    ssRRMapNInt::iterator HnHn_R_il = HnHn_R.lower_bound(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()));
   	    ssRRMapNInt::iterator HnHn_R_iu = HnHn_R.upper_bound(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()));

   	    ssRRMapNInt::iterator HaHn_R_il = HaHn_R.lower_bound(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()));
   	    ssRRMapNInt::iterator HaHn_R_iu = HaHn_R.upper_bound(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()));

   	    ssRRMapNInt::iterator HnHa_R_il = HnHa_R.lower_bound(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()));
   	    ssRRMapNInt::iterator HnHa_R_iu = HnHa_R.upper_bound(make_pair((*i).second.get_strand_ID(), (*j).second.get_strand_ID()));


      	NOESY_Assign::outputSec() << endl << "Strands " << (*i).second.get_strand_ID() << "-"
      		<< (*j).second.get_strand_ID() << endl;


    		for(multimap<pair<int, int>, pair<pair<int,int>, float >, less<pair<int, int> > >::iterator RR_i = antiRR_frq.lower_bound(make_pair(i->second.get_strand_ID(), j->second.get_strand_ID()));
    			RR_i != antiRR_frq.upper_bound(make_pair(i->second.get_strand_ID(), j->second.get_strand_ID())); RR_i++)
    		{
		  NOESY_Assign::outputSec() << "  anti_para->" << RR_i->second.first.first << "-" << RR_i->second.first.second << " Total score: " << RR_i->second.second << endl;

          pair<int, int> old;  old = make_pair(0,0);
          for(ssRRMapNInt::iterator HnHn_R_i = HnHn_R_il; HnHn_R_i != HnHn_R_iu; HnHn_R_i++)
      		{
      		    if (((RR_i->second.first.first - ((*HnHn_R_i).second.first.first)) == ((*HnHn_R_i).second.first.second - RR_i->second.first.second)) &&
      			          (fmod(abs(RR_i->second.first.first - (*HnHn_R_i).second.first.first), 2) == 0))
      			  {
      			    //RR_i->second += 2;

                if (old != HnHn_R_i->second.first) // second time
                {
      					NOESY_Assign::outputSec() << "\t" << HnHn_R_i->second.first.first << "Hn-" << HnHn_R_i->second.first.second << "Hn " << endl; //Score: 2" << endl;
      					NOEtypeListN::iterator l = SSNOE.lower_bound(make_pair(
      										make_pair(HnHn_R_i->second.first.second, HnHn_R_i->second.first.first), HnHn));
      					NOEtypeListN::iterator u = SSNOE.upper_bound(make_pair(
      										make_pair(HnHn_R_i->second.first.second, HnHn_R_i->second.first.first), HnHn));
      					for(NOEtypeListN::iterator i=l; i != u; i++)
					  NOESY_Assign::outputSec()	<< "\t\t" << i->second.first << " " << i->second.second.first << " " << i->second.second.second << endl;
      					l = SSNOE.lower_bound(make_pair(
      										make_pair(HnHn_R_i->second.first.first, HnHn_R_i->second.first.second), HnHn));
      					u = SSNOE.upper_bound(make_pair(
      										make_pair(HnHn_R_i->second.first.first, HnHn_R_i->second.first.second), HnHn));
      					for(NOEtypeListN::iterator i=l; i != u; i++)
					  NOESY_Assign::outputSec()	<< "\t\t" << i->second.first << " " << i->second.second.first << " " << i->second.second.second << endl;
                }
                old = HnHn_R_i->second.first;
      			  }
      		}

					old = make_pair(0,0);
	     		for(ssRRMapNInt::iterator EC_R_i = EC_R_il; EC_R_i != EC_R_iu; EC_R_i++)
	      		{
	      			  if (((RR_i->second.first.first - ((*EC_R_i).second.first.first)) ==
	      			      ((*EC_R_i).second.first.second - RR_i->second.first.second))) // && (fmod(abs(RR_i->second.first.first - (*EC_R_i).second.first.first), 2) == 0))
	      			  {
	                if (old != EC_R_i->second.first) // second time
	                {
	     					  NOESY_Assign::outputSec() << "\t" << EC_R_i->second.first.first << "-" << EC_R_i->second.first.second << " EC " << EC_R_i->second.second.first << endl; //Score: 2" << endl;
	                old = EC_R_i->second.first;
	            		}
								}
	      		}

						old = make_pair(0,0);
						for(ssRRMapNInt::iterator EC_R_i = EC_R_il; EC_R_i != EC_R_iu; EC_R_i++)
		      		{
		      			  if (((RR_i->second.first.first - ((*EC_R_i).second.first.first)) ==
		      			      ((*EC_R_i).second.first.second - RR_i->second.first.second -2 )) ||
											((RR_i->second.first.first - ((*EC_R_i).second.first.first)) ==
													((*EC_R_i).second.first.second - RR_i->second.first.second + 2 )))
										//) // && (fmod(abs(RR_i->second.first.first - (*EC_R_i).second.first.first), 2) == 0))
		      			  {
		                if (old != EC_R_i->second.first) // second time
		                {
		     					  NOESY_Assign::outputSec() << "\t" << EC_R_i->second.first.first << "-" << EC_R_i->second.first.second << " EC2 " << EC_R_i->second.second.first << endl; //Score: 2" << endl;
		                old = EC_R_i->second.first;
		            		}
									}
		      		}


        old = make_pair(0,0);
     		for(ssRRMapNInt::iterator HaHa_R_i = HaHa_R_il; HaHa_R_i != HaHa_R_iu; HaHa_R_i++)
      		{
      			  if (((RR_i->second.first.first - ((*HaHa_R_i).second.first.first)) ==
      			      ((*HaHa_R_i).second.first.second - RR_i->second.first.second)) && (fmod(abs(RR_i->second.first.first - (*HaHa_R_i).second.first.first), 2) == 1))
      			  {
                if (old != HaHa_R_i->second.first) // second time
                {
     					  NOESY_Assign::outputSec() << "\t" << HaHa_R_i->second.first.first << "Ha-" << HaHa_R_i->second.first.second << "Ha " << endl; //Score: 2" << endl;
      					NOEtypeListN::iterator l = SSNOE.lower_bound(make_pair(
      										make_pair(HaHa_R_i->second.first.second, HaHa_R_i->second.first.first), HaHa));
      					NOEtypeListN::iterator u = SSNOE.upper_bound(make_pair(
      										make_pair(HaHa_R_i->second.first.second, HaHa_R_i->second.first.first), HaHa));
      					for(NOEtypeListN::iterator i=l; i != u; i++)
					  NOESY_Assign::outputSec()	<< "\t\t" << i->second.first << " " << i->second.second.first << " " << i->second.second.second << endl;

      					l = SSNOE.lower_bound(make_pair(
      										make_pair(HaHa_R_i->second.first.first, HaHa_R_i->second.first.second), HaHa));
      					u = SSNOE.upper_bound(make_pair(
      										make_pair(HaHa_R_i->second.first.first, HaHa_R_i->second.first.second), HaHa));
      					for(NOEtypeListN::iterator i=l; i != u; i++)
					  NOESY_Assign::outputSec()	<< "\t\t" << i->second.first << " " << i->second.second.first << " " << i->second.second.second << endl;
                }
                old = HaHa_R_i->second.first;
            }
      		}

      		old = make_pair(0,0);

      		for(ssRRMapNInt::iterator HaHn_R_i = HaHn_R_il; HaHn_R_i != HaHn_R_iu; HaHn_R_i++)
      		{
      			  if (((RR_i->second.first.first - ((*HaHn_R_i).second.first.first)) == ((*HaHn_R_i).second.first.second - RR_i->second.first.second) - 1) &&
      			      ((fmod(abs(RR_i->second.first.first - ((*HaHn_R_i).second.first.first)), 2) == 1) &&
      			     (fmod(abs(RR_i->second.first.second - ((*HaHn_R_i).second.first.second)), 2) == 0)) ) //&& //)

      			  {

                if (old != HaHn_R_i->second.first) // second time
                {
      					NOESY_Assign::outputSec() << "\t" << HaHn_R_i->second.first.first << "Ha-" << HaHn_R_i->second.first.second << "Hn " << endl; //Score: 2" << endl;
      					NOEtypeListN::iterator l = SSNOE.lower_bound(make_pair(
      										make_pair(HaHn_R_i->second.first.second, HaHn_R_i->second.first.first), HnHa));
      					NOEtypeListN::iterator u = SSNOE.upper_bound(make_pair(
      										make_pair(HaHn_R_i->second.first.second, HaHn_R_i->second.first.first), HnHa));
      					for(NOEtypeListN::iterator i=l; i != u; i++)
					  NOESY_Assign::outputSec()	<< "\t\t" << i->second.first << " " << i->second.second.first <<  " " << i->second.second.second << endl;

      					l = SSNOE.lower_bound(make_pair(
      										make_pair(HaHn_R_i->second.first.first, HaHn_R_i->second.first.second), HaHn));
      					 u = SSNOE.upper_bound(make_pair(
      										make_pair(HaHn_R_i->second.first.first, HaHn_R_i->second.first.second), HaHn));
      					for(NOEtypeListN::iterator i=l; i != u; i++)
					  NOESY_Assign::outputSec()	<< "\t\t" << i->second.first << " " << i->second.second.first << " " << i->second.second.second << endl;
                }
                old = HaHn_R_i->second.first;
      			  }
      		}
          old = make_pair(0,0);

      		for(ssRRMapNInt::iterator HnHa_R_i = HnHa_R_il; HnHa_R_i != HnHa_R_iu; HnHa_R_i++)
      		{
      			  if (((RR_i->second.first.first - ((*HnHa_R_i).second.first.first)) == ((*HnHa_R_i).second.first.second - RR_i->second.first.second) - 1) &&
      			      ((fmod(abs(RR_i->second.first.first - ((*HnHa_R_i).second.first.first)), 2) == 0) &&
      			       (fmod(abs(RR_i->second.first.second - ((*HnHa_R_i).second.first.second)), 2) == 1)) )// && //)
      			  {
                if (old != HnHa_R_i->second.first) // second time
                {
      					NOESY_Assign::outputSec() << "\t" << HnHa_R_i->second.first.first << "Hn-" << HnHa_R_i->second.first.second << "Ha " << endl; //Score: 2" << endl;
      					NOEtypeListN::iterator l = SSNOE.lower_bound(make_pair(
      										make_pair(HnHa_R_i->second.first.second, HnHa_R_i->second.first.first), HaHn));
      					NOEtypeListN::iterator u = SSNOE.upper_bound(make_pair(
      										make_pair(HnHa_R_i->second.first.second, HnHa_R_i->second.first.first), HaHn));
      					for(NOEtypeListN::iterator i=l; i != u; i++)
					  NOESY_Assign::outputSec()	<< "\t\t" << i->second.first << " " << i->second.second.first << " " << i->second.second.second  << endl;

      					 l = SSNOE.lower_bound(make_pair(
      										make_pair(HnHa_R_i->second.first.first, HnHa_R_i->second.first.second), HnHa));
      					 u = SSNOE.upper_bound(make_pair(
      										make_pair(HnHa_R_i->second.first.first, HnHa_R_i->second.first.second), HnHa));
      					for(NOEtypeListN::iterator i=l; i != u; i++)
					  NOESY_Assign::outputSec()	<< "\t\t" << i->second.first << " " << i->second.second.first << " " << i->second.second.second << endl;
                }
                old = HnHa_R_i->second.first;
      			  }
      		}
          old = make_pair(0,0);

    		}

    		//
    		// for each register, print all support NOE informations with score
    		//
    		for(multimap<pair<int, int>, pair<pair<int,int>, float >, less<pair<int, int> > >::iterator RR_i = paraRR_frq.lower_bound(make_pair(i->second.get_strand_ID(), j->second.get_strand_ID()));
    			RR_i != paraRR_frq.upper_bound(make_pair(i->second.get_strand_ID(), j->second.get_strand_ID())); RR_i++)
    		{
    			NOESY_Assign::outputSec() << "  para ->" << RR_i->second.first.first << "-" << RR_i->second.first.second << " Total score: " << RR_i->second.second << endl;

          pair<int,int> old = make_pair(0,0);
      		for(ssRRMapNInt::iterator HaHn_R_i = HaHn_R_il; HaHn_R_i != HaHn_R_iu; HaHn_R_i++)
      		{
    			  if (((RR_i->second.first.first - ((*HaHn_R_i).second.first.first)) == (RR_i->second.first.second - ((*HaHn_R_i).second.first.second))) &&
    			      (fmod(abs(RR_i->second.first.first - ((*HaHn_R_i).second.first.first)), 2) == 0))
      			  {

                if (old != HaHn_R_i->second.first) // second time
                {
      					NOESY_Assign::outputSec() << "\t" << HaHn_R_i->second.first.first << "Ha-" << HaHn_R_i->second.first.second << "Hn " << endl; //Score: 2" << endl;
      					NOEtypeListN::iterator l = SSNOE.lower_bound(make_pair(
      										make_pair(HaHn_R_i->second.first.second, HaHn_R_i->second.first.first), HnHa));
      					NOEtypeListN::iterator u = SSNOE.upper_bound(make_pair(
      										make_pair(HaHn_R_i->second.first.second, HaHn_R_i->second.first.first), HnHa));
      					for(NOEtypeListN::iterator i=l; i != u; i++)
					  NOESY_Assign::outputSec()	<< "\t\t" << i->second.first << " " << i->second.second.first << " " << i->second.second.second << endl;
      					 l = SSNOE.lower_bound(make_pair(
      										make_pair(HaHn_R_i->second.first.first, HaHn_R_i->second.first.second), HaHn));
      					 u = SSNOE.upper_bound(make_pair(
      										make_pair(HaHn_R_i->second.first.first, HaHn_R_i->second.first.second), HaHn));
      					for(NOEtypeListN::iterator i=l; i != u; i++)
					  NOESY_Assign::outputSec()	<< "\t\t" << i->second.first << " " << i->second.second.first << " " << i->second.second.second  << endl;
                }
                old = HaHn_R_i->second.first;
      			  }
      		}

					old = make_pair(0,0);
	     		for(ssRRMapNInt::iterator EC_R_i = EC_R_il; EC_R_i != EC_R_iu; EC_R_i++)
	      		{
	      			  if ((RR_i->second.first.first - ((*EC_R_i).second.first.first)) ==
	      			      (RR_i->second.first.second - ((*EC_R_i).second.first.second))) //&& (fmod(abs(RR_i->second.first.first - (*EC_R_i).second.first.first), 2) == 0))
	      			  {
	                if (old != EC_R_i->second.first) // second time
	                {
	     					  	NOESY_Assign::outputSec() << "\t" << EC_R_i->second.first.first << "-" << EC_R_i->second.first.second << " EC " << EC_R_i->second.second.first << endl; //Score: 2" << endl;
	                	old = EC_R_i->second.first;
									}
	            }
	      		}

						old = make_pair(0,0);
						for(ssRRMapNInt::iterator EC_R_i = EC_R_il; EC_R_i != EC_R_iu; EC_R_i++)
		      		{
		      			  if (((RR_i->second.first.first - ((*EC_R_i).second.first.first)) ==
		      			      (RR_i->second.first.second - ((*EC_R_i).second.first.second -2 ))) ||
											((RR_i->second.first.first - ((*EC_R_i).second.first.first)) ==
													(RR_i->second.first.second - ((*EC_R_i).second.first.second + 2 ))))
										//) // && (fmod(abs(RR_i->second.first.first - (*EC_R_i).second.first.first), 2) == 0))
		      			  {
		                if (old != EC_R_i->second.first) // second time
		                {
		     					  NOESY_Assign::outputSec() << "\t" << EC_R_i->second.first.first << "-" << EC_R_i->second.first.second << " EC2 " << EC_R_i->second.second.first << endl; //Score: 2" << endl;
		                old = EC_R_i->second.first;
		            		}
									}
		      		}


          old = make_pair(0,0);
      		for(ssRRMapNInt::iterator HnHa_R_i = HnHa_R_il; HnHa_R_i != HnHa_R_iu; HnHa_R_i++)
      		{
    			  if (((RR_i->second.first.first - ((*HnHa_R_i).second.first.first)) == (RR_i->second.first.second - ((*HnHa_R_i).second.first.second))) &&
    			      (fmod(abs(RR_i->second.first.first - ((*HnHa_R_i).second.first.first)), 2) == 1))
      			  {

                if (old != HnHa_R_i->second.first) // second time
                {
      					NOESY_Assign::outputSec() << "\t" << HnHa_R_i->second.first.first << "Hn-" << HnHa_R_i->second.first.second << "Ha " << endl; //Score: 2" << endl;
       					NOEtypeListN::iterator l = SSNOE.lower_bound(make_pair(
      										make_pair(HnHa_R_i->second.first.second, HnHa_R_i->second.first.first), HaHn));
      					NOEtypeListN::iterator u = SSNOE.upper_bound(make_pair(
      										make_pair(HnHa_R_i->second.first.second, HnHa_R_i->second.first.first), HaHn));
      					for(NOEtypeListN::iterator i=l; i != u; i++)
					  NOESY_Assign::outputSec()	<< "\t\t" << i->second.first << " " << i->second.second.first << " " << i->second.second.second  << endl;

      					 l = SSNOE.lower_bound(make_pair(
      										make_pair(HnHa_R_i->second.first.first, HnHa_R_i->second.first.second), HnHa));
      					 u = SSNOE.upper_bound(make_pair(
      										make_pair(HnHa_R_i->second.first.first, HnHa_R_i->second.first.second), HnHa));
      					for(NOEtypeListN::iterator i=l; i != u; i++)
					  NOESY_Assign::outputSec()	<< "\t\t" << i->second.first << " " << i->second.second.first << " " << i->second.second.second  << endl;
                }

                old = HnHa_R_i->second.first;
      			  }
      		}


          old = make_pair(0,0);
      		for(ssRRMapNInt::iterator HnHn_R_i = HnHn_R_il; HnHn_R_i != HnHn_R_iu; HnHn_R_i++)
      		{
    			  if (((((RR_i->second.first.first - ((*HnHn_R_i).second.first.first)) == -1 + (RR_i->second.first.second - ((*HnHn_R_i).second.first.second))) &&
    			      (fmod(abs(RR_i->second.first.first - ((*HnHn_R_i).second.first.first)), 2) == 1) &&
    			      (fmod(abs(RR_i->second.first.second - ((*HnHn_R_i).second.first.second)), 2) == 0)) ||
    			     (((RR_i->second.first.first - ((*HnHn_R_i).second.first.first)) == 1 + (RR_i->second.first.second - ((*HnHn_R_i).second.first.second))) &&
    			      (fmod(abs(RR_i->second.first.second - ((*HnHn_R_i).second.first.second)), 2) == 0) &&
    			      (fmod(abs(RR_i->second.first.first - ((*HnHn_R_i).second.first.first)), 2) == 1))) ) //&&
      			  {
      					if (old != HnHn_R_i->second.first) // second time
                {
      					NOESY_Assign::outputSec() << "\t" << HnHn_R_i->second.first.first << "Hn-" << HnHn_R_i->second.first.second << "Hn " << endl; //Score: 2" << endl;
      					NOEtypeListN::iterator l = SSNOE.lower_bound(make_pair(
      										make_pair(HnHn_R_i->second.first.second, HnHn_R_i->second.first.first), HnHn));
      					NOEtypeListN::iterator u = SSNOE.upper_bound(make_pair(
      										make_pair(HnHn_R_i->second.first.second, HnHn_R_i->second.first.first), HnHn));
      					for(NOEtypeListN::iterator i=l; i != u; i++)
					  NOESY_Assign::outputSec()	<< "\t\t" << i->second.first << " " << i->second.second.first << " " << i->second.second.second << endl;
      					 l = SSNOE.lower_bound(make_pair(
      										make_pair(HnHn_R_i->second.first.first, HnHn_R_i->second.first.second), HnHn));
      					 u = SSNOE.upper_bound(make_pair(
      										make_pair(HnHn_R_i->second.first.first, HnHn_R_i->second.first.second), HnHn));
      					for(NOEtypeListN::iterator i=l; i != u; i++)
					  NOESY_Assign::outputSec()	<< "\t\t" << i->second.first << " " << i->second.second.first << " " << i->second.second.second  << endl;
                }
                old = HnHn_R_i->second.first;
      			  }
      		}

				}
		}
	}

  // for b-sheet, define the faces within the strand
  for(SS::iterator i = NOESY_Assign::secondaryStruct().begin(); i != NOESY_Assign::secondaryStruct().end(); i++)
  {
    if (i->second.get_type() == -1)
    {

    	for(Chain::const_iterator j = i->second.get_chain().begin(); j != i->second.get_chain().end(); ++j)
     	{

	contactFaces.insertSheetLocal(*j, *j, 1);
	if (i->second.get_chain().find(*j+1) != i->second.get_chain().end())
       	{
      		faces[*j][(*j+1)] = -1;
        	faces[(*j+1)][*j] = -1;
         }

      	if (i->second.get_chain().find(*j+2) != i->second.get_chain().end())
       	{
      		faces[*j][(*j+1)] = 1;
        	faces[(*j+1)][*j] = 1;
         	contactFaces.insertSheetLocal(*j, *j+2, 1);
        }

        for(int k=3; k < (*(i->second.get_chain().rbegin()) - *(i->second.get_chain().begin()))+1; ++k)
        {
        	if (k == 4)
         		continue;
        	if (i->second.get_chain().find(*j+k) != i->second.get_chain().end())
	         	contactFaces.insertSheetLocal(*j, *j+k, -1);
        }

        int n = (*(i->second.get_chain().rbegin()) - *(i->second.get_chain().begin()))/2 + 1;

        for(int k=0; k<n; ++k)
        {
        	if (i->second.get_chain().find(*j+2*k) != i->second.get_chain().end())
	         	contactFaces.insertSheet(*j, *j+2*k, 1);
        	if (i->second.get_chain().find(*j+1+2*k) != i->second.get_chain().end())
  	       	contactFaces.insertSheet(*j, *j+1+2*k, -1);
        }
      }
    }
  }
}
