/***************************************************************************
                          HSQCC13NOESY.cpp  -  description
                             -------------------
    begin                : Mon Aug 21 2000
    copyright            : (C) 2000 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#include "HSQCC13NOESY.h"

C13NOESY_HSQC::C13NOESY_HSQC(bool flag, bool haveIC, float hx1t, float x1t, float hx2t, float f, string label)
	:D3NOESY(flag, haveIC, hx1t, x1t, hx2t, f, label)
{
}
C13NOESY_HSQC::~C13NOESY_HSQC()
{
}

const string C13NOESY_HSQC::Type() const
{
  return "3DC";  
}

bool C13NOESY_HSQC::canbeMom(const atom* atom1) const
{
  if (atom1->isatt_C()) // && ((atom1->get_shift() < wl) || (atom1->get_shift() > wu)))
		return 1; 
	return 0; 
}

bool C13NOESY_HSQC::canbeDad(const atom* atom1) const
{
	if (waterFlag) return 1;
	if (atom1->isatt_N())
		return 0;
	return 1;
}

bool C13NOESY_HSQC::isPossFather(const atom* temp1, pair<float, float> bound) const
{
	if (temp1->isH())
	{
		return 1;
	} else
		return 0;
}

bool C13NOESY_HSQC::isPossMother(const atom* temp1, pair<float, float> bound) const
{
	deque<atom*> att_C = temp1->att_C();

	for(deque<atom*>::iterator it = att_C.begin();
		it != att_C.end(); it++)
	{
	  if ((*it) && canbeMom(temp1))
		{
			if (((*it)->get_shift() <= bound.second) && ((*it)->get_shift() >= bound.first))
				return 1;
		}
	}
	return 0;
} 	

// inter chain
ICC13NOESY_HSQC::ICC13NOESY_HSQC(bool flag, bool haveIC, float hx1t, float x1t, float hx2t, float f, string label)
	: C13NOESY_HSQC(flag, haveIC, hx1t, x1t, hx2t, f, label)
{
	IC = 1;
}
ICC13NOESY_HSQC::~ICC13NOESY_HSQC()
{
}

const string ICC13NOESY_HSQC::Type() const
{
  return "3DC IC";  
}


// BH root at B,
void C13NOESY_HSQC::Find_UniqueBH(int i, int j, method mthd)
{
	MatchPeakAtombyAtomIndex MombyIndex;

	for(MatchPeakAtom::iterator iter = MombySimpleMatch.begin(); iter != MombySimpleMatch.end(); iter++)
		if ((*iter).second->get_layer().substr(0,1) == "B")
			MombyIndex.insert(make_pair((*iter).second->get_index(), (*iter)));

	// find all possible (i,j) that root at i
	MatchPeakAtombyAtomIndex::iterator
		res1RootLower = MombyIndex.lower_bound(i);
	MatchPeakAtombyAtomIndex::iterator
		res1RootUpper = MombyIndex.upper_bound(i);

	for(MatchPeakAtombyAtomIndex::iterator iter1 = res1RootLower;
		iter1 != res1RootUpper; iter1++)
	{
		// for each peak, find all possible father
		MatchPeakAtom::iterator fatherLower
			= DadbySimpleMatch.lower_bound((*iter1).second.first);
		MatchPeakAtom::iterator fatherUpper
			= DadbySimpleMatch.upper_bound((*iter1).second.first);

		for(MatchPeakAtom::iterator iterf = fatherLower; iterf != fatherUpper; iterf++)
			if (((*iterf).second->get_index() == j) &&
				((*iterf).second->get_layer() == "N"))
			{
					MakeBuffer((*iterf).first, (*iter1).second.second, (*iterf).second, mthd);
			}
	}
}

// root at A
void C13NOESY_HSQC::Find_UniqueAH(int i, int j, method mthd)
{
	MatchPeakAtombyAtomIndex MombyIndex;

	for(MatchPeakAtom::iterator iter = MombySimpleMatch.begin(); iter != MombySimpleMatch.end(); iter++)
		if ((*iter).second->get_layer().substr(0,1) == "A")
			MombyIndex.insert(make_pair((*iter).second->get_index(), (*iter)));

	// find all possible (i,j) that root at i
	MatchPeakAtombyAtomIndex::iterator
		res1RootLower = MombyIndex.lower_bound(i);
	MatchPeakAtombyAtomIndex::iterator
		res1RootUpper = MombyIndex.upper_bound(i);

	for(MatchPeakAtombyAtomIndex::iterator iter1 = res1RootLower;
		iter1 != res1RootUpper; iter1++)
	{
		// for each peak, find all possible father
		MatchPeakAtom::iterator fatherLower
			= DadbySimpleMatch.lower_bound((*iter1).second.first);
		MatchPeakAtom::iterator fatherUpper
			= DadbySimpleMatch.upper_bound((*iter1).second.first);

		for(MatchPeakAtom::iterator iterf = fatherLower; iterf != fatherUpper; iterf++)
			if (((*iterf).second->get_index() == j) &&
				((*iterf).second->get_layer() == "N"))
			{
					MakeBuffer((*iterf).first, (*iter1).second.second, (*iterf).second, mthd);
			}
	}
}

/*******/
/* ALI */
/*******/
ALIC13NOESY_HSQC::ALIC13NOESY_HSQC(bool flag, bool haveIC, float hx1t, float x1t, float hx2t, float f, string label)
	:C13NOESY_HSQC(flag, haveIC, hx1t, x1t, hx2t, f, label)
{
}
ALIC13NOESY_HSQC::~ALIC13NOESY_HSQC()
{
}
bool ALIC13NOESY_HSQC::canbeMom(const atom* atom1) const
{
  if ((atom1->isatt_C()) && !(atom1->isRingAtom())) // && ((atom1->get_shift() < wl) || (atom1->get_shift() > wu)))
    return 1; 
  return 0; 
}
const string ALIC13NOESY_HSQC::Type() const
{
  return "3DC-ALI";  
}

/*******/
/* ARO */
/*******/
AROC13NOESY_HSQC::AROC13NOESY_HSQC(bool flag, bool haveIC, float hx1t, float x1t, float hx2t, float f, string label)
	:C13NOESY_HSQC(flag, haveIC, hx1t, x1t, hx2t, f, label)
{
}
AROC13NOESY_HSQC::~AROC13NOESY_HSQC()
{
}

bool AROC13NOESY_HSQC::canbeMom(const atom* atom1) const
{
  if ((atom1->isatt_C()) && (atom1->isRingAtom()))
    return 1; 
  return 0; 
}
const string AROC13NOESY_HSQC::Type() const
{
  return "3DC-ARO";  
}

