/***************************************************************************
                          HSQCN15NOESY.cpp  -  description
                             -------------------
    begin                : Mon Aug 21 2000
    copyright            : (C) 2000 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#include "HSQCN15NOESY.h"

N15NOESY_HSQC::N15NOESY_HSQC(bool flag, bool haveIC, float hx1t, float x1t, float hx2t, float f, string label)
	: D3NOESY(flag, haveIC, hx1t, x1t, hx2t, f, label)
{
}
N15NOESY_HSQC::~N15NOESY_HSQC()
{
}

const string N15NOESY_HSQC::Type() const
{
  return "3DN"; 
}

//------------------------------------
// nearH2O
//	At the beginning, find the peaks that
// match the H2O regin, and get rid of it
// should be given the chemical shift of H2o by user
// for global referencing problem.
//------------------------------------
bool N15NOESY_HSQC::canbeMom(const atom* atom1) const
{
	if (atom1->isatt_N())
		return 1; 
	return 0;
}

bool N15NOESY_HSQC::canbeDad(const atom* atom1) const
{
	return 1;
}

bool N15NOESY_HSQC::isPossFather(const atom* temp1, pair<float, float> bound) const
{
	if (temp1->isH())
		return 1;
	else
		return 0;
}

bool N15NOESY_HSQC::isPossMother(const atom* temp1, pair<float, float> bound) const
{
		if (temp1->isatt_N() && (((temp1->att_N()->get_shift()) <= bound.second) && ((temp1->att_N()->get_shift()) >= bound.first)))
		return 1;
	return 0;
} 	
// HB root at H,
void N15NOESY_HSQC::Find_UniqueBH(int j, int i, method mthd)
{
	MatchPeakAtombyAtomIndex MombyIndex;

	for(MatchPeakAtom::iterator iter = MombySimpleMatch.begin(); iter != MombySimpleMatch.end(); iter++)
		if ((*iter).second->get_layer() == "N")
			MombyIndex.insert(make_pair((*iter).second->get_index(), (*iter)));

	// find all possible (i,j) that root at i
	MatchPeakAtombyAtomIndex::iterator
		res1RootLower = MombyIndex.lower_bound(i);
	MatchPeakAtombyAtomIndex::iterator
		res1RootUpper = MombyIndex.upper_bound(i);

	for(MatchPeakAtombyAtomIndex::iterator iter1 = res1RootLower;
		iter1 != res1RootUpper; iter1++)
	{
		// for each peak, find all possible father
		MatchPeakAtom::iterator fatherLower
			= DadbySimpleMatch.lower_bound((*iter1).second.first);
		MatchPeakAtom::iterator fatherUpper
			= DadbySimpleMatch.upper_bound((*iter1).second.first);

		for(MatchPeakAtom::iterator iterf = fatherLower; iterf != fatherUpper; iterf++)
			if (((*iterf).second->get_index() == j) &&
				((*iterf).second->get_layer().substr(0,1) == "B"))
			{
					MakeBuffer((*iterf).first, (*iter1).second.second, (*iterf).second, mthd);
			}
	}
}

// root at H
void N15NOESY_HSQC::Find_UniqueAH(int j, int i, method mthd)
{
	MatchPeakAtombyAtomIndex MombyIndex;

	for(MatchPeakAtom::iterator iter = MombySimpleMatch.begin(); iter != MombySimpleMatch.end(); iter++)
		if ((*iter).second->get_layer() == "N")
			MombyIndex.insert(make_pair((*iter).second->get_index(), (*iter)));

	// find all possible (i,j) that root at i
	MatchPeakAtombyAtomIndex::iterator
		res1RootLower = MombyIndex.lower_bound(i);
	MatchPeakAtombyAtomIndex::iterator
		res1RootUpper = MombyIndex.upper_bound(i);

	for(MatchPeakAtombyAtomIndex::iterator iter1 = res1RootLower;
		iter1 != res1RootUpper; iter1++)
	{
		// for each peak, find all possible father
		MatchPeakAtom::iterator fatherLower
			= DadbySimpleMatch.lower_bound((*iter1).second.first);
		MatchPeakAtom::iterator fatherUpper
			= DadbySimpleMatch.upper_bound((*iter1).second.first);

		for(MatchPeakAtom::iterator iterf = fatherLower; iterf != fatherUpper; iterf++)
			if (((*iterf).second->get_index() == j) &&
				((*iterf).second->get_layer().substr(0,1) == "A"))
			{
					MakeBuffer((*iterf).first, (*iter1).second.second, (*iterf).second, mthd);
			}
	}
}


// inter chain
ICN15NOESY_HSQC::ICN15NOESY_HSQC(bool flag, bool haveIC, float hx1t, float x1t, float hx2t, float f, string label)
	: N15NOESY_HSQC(flag, haveIC, hx1t, x1t, hx2t, f, label)
{
	IC = 1;
}
ICN15NOESY_HSQC::~ICN15NOESY_HSQC()
{
}

const string ICN15NOESY_HSQC::Type() const
{
  return "3DN IC"; 
}


