/***************************************************************************
                          Jval.cpp  -  J-coupling const issues
                             -------------------
    begin                : Mon Aug 21 2000
    copyright            : (C) 2000 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/
#include <fstream>
#include <iostream>
#include "Jval.h"

extern string itoa(int);
extern string error(string, string);

ostream& operator<< (ostream& s, const Jval& j)
{
 s << j.id <<  " " << j.label << " index=" << j.index << " value=" << j.value << " error=" << j.error << endl;
 return s;
}


// read in the J value files
istream& operator>> (istream& s, Jval& j)
{
  string AA1, atom1, AA2, atom2;
  int index1=0, index2=0;

  s >> j.id >> j.label >> index1 >> AA1 >> atom1 >> index2 >> AA2 >> atom2 >> j.value >> j.error; // value and error

  ios::iostate state = s.rdstate();
	
	if ((state&ios::failbit) && !(state&ios::eofbit))
	{
		cerr << endl << "*** Bad J-coupling data format found: " << endl
				 <<  j << " ignored" << endl;	
	}

  //AA1 and AA2 is used to check the consistency with the sequence file and the J-type
  //To be finished in the future;
  if ((index1==0) && (index1==0))
    return s;

  if ((atom1=="H") && (atom2=="HA") && (index1==index2))
	  j.type = J3HNHA;
  else if ((atom1=="HA") && (atom2=="HB2") && (index1==index2))
	  j.type =J3HAHB2;
  else if ((atom1=="HA") && (atom2=="HB3") && (index1==index2))
	  j.type = J3HAHB3;
  else if ((atom1=="N") && (atom2=="HB2") && (index1==index2))
	  j.type = J3NHB2;
  else if ((atom1=="N") && (atom2=="HB3") && (index1==index2))
	  j.type = J3NHB3;
  else if ((atom1=="HA") && (atom2=="N") && (index2-index1 == 1))
	  j.type = J3HAN;
  else if ((atom1=="H") && (atom2=="C") && (index1==index2))
	  j.type = J3HNC;
  else if ((atom1=="H") && (atom2=="CB") && (index1==index2))
	  j.type = J3HNCB;
  else if ((atom1=="C") && (atom2=="HB2") && (index1==index2))
	  j.type = J3CHB2;
  else if ((atom1=="C") && (atom2=="HB3") && (index1==index2))
	  j.type = J3CHB3;
  else if ((atom1=="C") && (atom2=="CB") && (index2-index1 == 1))
	  j.type = J3CHB2;
  else if ((atom1=="C") && (atom2=="CG1") && (index1==index2))
	  j.type = J3CHB3;
  else if ((atom1=="C") && (atom2=="C") && (index2-index1 == 1))
	  j.type = J3CHB3;
  else
	{
		error("***Unknown J-values", AA1 + "-" + atom1 + itoa(index1) + "  "
	       + AA2 + "-" + atom2 + itoa(index2) );
	}
  j.index = index1;
  return s;
};

