#include <stdio.h>
#include <iostream>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#include <assert.h>
#include <sys/time.h>
#include <time.h>
#include <list>

#include "K3Dtree.h"

#define SQ(x)			((x) * (x))

#define SWAP(a,b) { register double t=(a);(a)=(b);(b)=t; }

double median_select(double a[], int n)
{
int low, high ;
int median;
int middle, ll, hh;
low = 0 ; high = n-1 ; median = (low + high) / 2;
for (;;) {
if (high <= low) /* One element only */
return a[median] ;
if (high == low + 1) { /* Two elements only */
if (a[low] > a[high])
SWAP(a[low], a[high]) ;
return a[median] ;
}
/* Find median of low, middle and high items; swap into position low */
middle = (low + high) / 2;
if (a[middle] > a[high]) SWAP(a[middle], a[high]) ;
if (a[low] > a[high]) SWAP(a[low], a[high]) ;
if (a[middle] > a[low]) SWAP(a[middle], a[low]) ;
/* Swap low item (now in position middle) into position (low+1) */
SWAP(a[middle], a[low+1]) ;
/* Nibble from each end towards middle, swapping items when stuck */
ll = low + 1;
hh = high;
for (;;) {
do ll++; while (a[low] > a[ll]) ;
do hh--; while (a[hh] > a[low]) ;
if (hh < ll)
break;
SWAP(a[ll], a[hh]) ;
}
/* Swap middle item (in position low) back into correct position */
SWAP(a[low], a[hh]) ;
/* Re-set active partition */
if (hh <= median)
low = ll;
if (hh >= median)
high = hh - 1;
}
}


void K3Dtree::reset_count()
{
  count_all = 0;
}

K3Dtree::K3Dtree(point_list::const_iterator li, point_list::const_iterator ui, int size)
{
    
     total_num=size;
     int depth=0;
     point_node pointList[size]; 
     
     list<point_node>::const_iterator ii;
     int i=0;      
     for(ii=li; ii != ui; ++ii){
       pointList[i]=(*ii); 
       i++;  
      } 
    kd1 =  kdtree(pointList,size,depth);
}

point_list K3Dtree::call_range_search(point_node& point, double range){
    point_list ret; 
    
    double pos[3]; 
    pos[0] = point.x; pos[1] = point.y; pos[2] = point.z; 
    
    point_node res[getTreeSize()];
    
    reset_count();
    
    int set = range_search(kd1, pos, range, res); 
      
     for(int i=0;i<getRetCount();i++){     
       ret.push_back(res[i]); 
      } 
    
    return ret;
}

point_list K3Dtree::call_linear_search(point_list& nodes, point_node& point, double range)
{
	   point_list ret; 
	    
	    double pos[3]; 
	    pos[0] = point.x; pos[1] = point.y; pos[2] = point.z; 

	    for(point_list::iterator node = nodes.begin(); node != nodes.end(); node++)
	    {
	         double dist_sq = SQ(node->x - pos[0])+SQ(node->y - pos[1])+SQ(node->z - pos[2]);
	         if(dist_sq <= SQ(range))
	         {
	        	 ret.push_back(*node);
	         } 
	    }
	    return ret;
}

kdnode* K3Dtree::kdtree (point_node pointList[], int size, int depth)
{     
    
     point_node* p=NULL;
     int axis,i;
     double median;
     int left_count=0 , right_count=0;
     point_node* leftset= NULL;
     point_node* rightset= NULL;
     double mPoint[3];
     kdnode* node = new kdnode[1]; 

    if (pointList==NULL | size==0) 
        return NULL;
    else if(size==1){  // leave 
        point_node p0=pointList[0];
       node->x= p0.x;node->y= p0.y;node->z= p0.z;
       	node->left=NULL; node->right=NULL; 
        node->dir= 4;
        node->location = 0;
        return node;
    }else if(size==2){  // two leaves 
        point_node p1,p2;
        kdnode* k1 = new kdnode[1]; 
        kdnode* k2 = new kdnode[1];
        k1->left=NULL;k1->right=NULL;k2->left=NULL;k2->right=NULL;k1->dir=4;k2->dir=4; 
         axis = depth % 3;
        p1=pointList[0];p2=pointList[1];
        
       if(axis==0){if(p1.x <= p2.x){node->x=p2.x; node->y=p2.y;node->z=p2.z;k1->x=p1.x;k2->x=p2.x;k1->y=p1.y;k2->y=p2.y;k1->z=p1.z;k2->z=p2.z;node->dir=0;node->left=k1; node->right=k2;}else{node->x=p1.x;node->y=p1.y;node->z=p1.z;k1->x=p1.x;k2->x=p2.x;k1->y=p1.y;k2->y=p2.y;k1->z=p1.z;k2->z=p2.z;node->dir=0;node->left=k2; node->right=k1;} }
        if(axis==1){if(p1.y <= p2.y){node->x=p2.x; node->y=p2.y;node->z=p2.z; k1->x=p1.x;k2->x=p2.x;k1->y=p1.y;k2->y=p2.y;k1->z=p1.z;k2->z=p2.z;node->dir=1;node->left=k1; node->right=k2;}else{node->x=p1.x;node->y=p1.y;node->z=p1.z;k1->x=p1.x;k2->x=p2.x;k1->y=p1.y;k2->y=p2.y;k1->z=p1.z;k2->z=p2.z;node->dir=1;node->left=k2; node->right=k1;} }
        if(axis==2){if(p1.z <= p2.z){node->x=p2.x; node->y=p2.y;node->z=p2.z; k1->x=p1.x;k2->x=p2.x;k1->y=p1.y;k2->y=p2.y;k1->z=p1.z;k2->z=p2.z;node->dir=2;node->left=k1; node->right=k2;}else{node->x=p1.x;node->y=p1.y;node->z=p1.z;k1->x=p1.x;k2->x=p2.x;k1->y=p1.y;k2->y=p2.y;k1->z=p1.z;k2->z=p2.z;node->dir=2;node->left=k2; node->right=k1;} }
      return node;       
    }
    else
    {
  
      double oneCoodinates[size];

     p= new point_node[1];
        // Select axis based on depth so that axis cycles through all valid values
        axis = depth % 3;
        
      for(int i=0;i<size;i++){
        
           if(axis==0){ oneCoodinates[i]=  pointList[i].x;}
           if(axis==1){ oneCoodinates[i]=  pointList[i].y;}
           if(axis==2){ oneCoodinates[i]=  pointList[i].z;}
      
      }

         // Sort point list and choose median as pivot element
        //select median from pointList;
            
       median=median_select(oneCoodinates,size);
        
        for(int i=0;i<size;i++ ){       
          if(axis==0){
           if(pointList[i].x == median) 
              {p->x= pointList[i].x; p->y= pointList[i].y;p->z= pointList[i].z;break; }   //axis represent x or y or z direction
           }
          if(axis==1){
           if(pointList[i].y == median) 
              {p->x= pointList[i].x; p->y= pointList[i].y;p->z= pointList[i].z;break; }   //axis represent x or y or z direction
           }
         if(axis==2){
           if(pointList[i].z == median) 
              {p->x= pointList[i].x; p->y= pointList[i].y;p->z= pointList[i].z;break; }   //axis represent x or y or z direction
           }
        }
         
        for(int i=0; i<size;i++) 
        {
           if(axis==0){
            if(pointList[i].x < median) {left_count++;} 
            else {right_count++;} ;
           }
            if(axis==1){
            if(pointList[i].y < median) {left_count++;} 
            else {right_count++;} ;
           } 
           if(axis==2){
            if(pointList[i].z < median) {left_count++;} 
            else {right_count++;} ; 
           }
        }

       if(left_count>0)leftset= new point_node[left_count];
       if(right_count>0)rightset=new point_node[right_count];
    
        left_count=0; right_count=0;
        for(int i=0; i<size;i++) 
        {
          if(axis==0){
            if(pointList[i].x < median) {
                  leftset[left_count].x=pointList[i].x;leftset[left_count].y=pointList[i].y;leftset[left_count].z=pointList[i].z;
                  left_count++;
                  } 
            else {rightset[right_count].x=pointList[i].x;rightset[right_count].y=pointList[i].y;rightset[right_count].z=pointList[i].z;
                  right_count++; 
                  if(pointList[i].x == median){ mPoint[0]=pointList[i].x;mPoint[1]=pointList[i].y;mPoint[2]=pointList[i].z;} 
                 }   
           }


        if(axis==1){
            if(pointList[i].y < median) {
                  leftset[left_count].x=pointList[i].x;leftset[left_count].y=pointList[i].y;leftset[left_count].z=pointList[i].z;
                  left_count++;
                  } 
            else {rightset[right_count].x=pointList[i].x;rightset[right_count].y=pointList[i].y;rightset[right_count].z=pointList[i].z;
                  right_count++; 
                  if(pointList[i].x == median){ mPoint[0]=pointList[i].x;mPoint[1]=pointList[i].y;mPoint[2]=pointList[i].z;} 
                 }   
           }
        
        if(axis==2){
            if(pointList[i].z < median) {
                  leftset[left_count].x=pointList[i].x;leftset[left_count].y=pointList[i].y;leftset[left_count].z=pointList[i].z;
                  left_count++;
                  } 
            else {rightset[right_count].x=pointList[i].x;rightset[right_count].y=pointList[i].y;rightset[right_count].z=pointList[i].z;
                  right_count++; 
                  if(pointList[i].x == median){ mPoint[0]=pointList[i].x;mPoint[1]=pointList[i].y;mPoint[2]=pointList[i].z;} 
                 }   
           }

        }
           
        node->x= p->x;node->y= p->y;node->z= p->z;
      
        node->dir=axis;
   
        node->location = median;
     
        node->left = kdtree(leftset, left_count, depth+1);
               
        node->right = kdtree(rightset, right_count, depth+1);
     
        return node;
    }
}


int K3Dtree::getRetCount(){return count_all;};
int K3Dtree::getTreeSize(){return total_num;};

int K3Dtree::linear_search(point_node nodes[], int size, const double *pos, double range, point_node res1[])
{
      double dist_sq;
      point_node node;
      for(int i=0;i<size;i++){ 
 
         node=nodes[i]; 
         dist_sq = SQ(node.x - pos[0])+SQ(node.y - pos[1])+SQ(node.z - pos[2]);
         if(dist_sq <= SQ(range)){res1[count_all++]=node;} 
      }
      return 0;
}

int K3Dtree::range_search(kdnode *node, const double *pos, double range, point_node res[])
{
	
    double dist_sq, dx;
	int i, ret, added_res = 0;
        
     
	if(!node) return 0;

	dist_sq = 0; 
       
       dist_sq = SQ(node->x - pos[0])+SQ(node->y - pos[1])+SQ(node->z - pos[2]);
       
	if(dist_sq <= SQ(range) && node->dir==4) {
	  
	        res[count_all].x=node->x;res[count_all].y=node->y;res[count_all].z=node->z; count_all++;
                 added_res = 1;
        }
        
	if (node->dir==0) {dx = pos[node->dir] - node->x;}
      	if (node->dir==1) {dx = pos[node->dir] - node->y;}
       	if (node->dir==2) {dx = pos[node->dir] - node->z;}

    ret = range_search(dx <= 0.0 ? node->left : node->right, pos, range,res);
	if(ret >= 0 && fabs(dx) < range) {
                added_res += ret;                                
		ret = range_search(dx <= 0.0 ? node->right : node->left, pos, range,res);
	}
	if(ret == -1) {
		return -1;
	}
	
       added_res += ret;
        return added_res;
        
}

unsigned int get_msec(void)
{
	static struct timeval timeval, first_timeval;
	
	gettimeofday(&timeval, 0);

	if(first_timeval.tv_sec == 0) {
		first_timeval = timeval;
		return 0;
	}
	return (timeval.tv_sec - first_timeval.tv_sec) * 1000 + (timeval.tv_usec - first_timeval.tv_usec) / 1000;
}

