/***************************************************************************
                          NA_struc.cpp  -  related to the structure calculation
                             -------------------
    begin                : Sun Dec 30 2001
    copyright            : (C) 2001 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
***************************************************************************/

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>

#include <cstdio>
#include <fstream>
#include <string>
#include <stdlib.h>

#include <signal.h>
#include <cmath>
#include "noesy_assign.h"

extern bool DEBUG;
extern int LLength;
extern string outputDir;
extern ControlMap control;
extern Command command;
extern set<int, less<int> > CISPeptideBond;
extern set<pair<int,int>, less<pair<int, int> > > SSBond;
extern string controlFileName;
extern PeakListDeque peakListDeque;

// RTT change int to double
double vio[2] = {0.5, 0.1}; // for EC violations
double vio2[2] = {0.9, 0.5}; // for EC violations

bool sameHydro(char, char);

void prtHyperInput(ostream&, const SEQM&, const DianaMap&, bool, bool, bool, int);
void prtXplorCACBShift(ostream&, const SEQM&, const Atom_List&);
void prtXplorJvalues(ostream&, const SEQM&, const JvalList&);
ostream& prtRefinedShiftList(ostream&, Atom_List&);

void NOESY_Assign::runInitFold()
{
        MomDadSet mdSetM2; mdSetM2.clear();
        MomDadPeakAtomMap mdPAMap2; mdPAMap2.clear();
        SUPPORT support; support.clear();

        runBuildProfile(mdSetM2, mdPAMap2, support);
        runSYMUnique(1, mdSetM2, mdPAMap2, support);

        // 6. bootstrapping if have
        if (INITF)
        {
                ////////////////////////////////
                // read in init fold PDB file   //
                ////////////////////////////////

                _pdbList.clear();
                _outputLog << "debug initFold " << initFold.c_str() << endl;
                ifstream pdb(initFold.c_str());
                if (!pdb) error("cannot open input file", initFold.c_str());
                erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Reading initial coord Files " + initFold + "...");
                Pdb pdbStruct(0);

                while (pdb)
                {
                        pdb >> pdbStruct;
                        if (pdbStruct.notEmpty())
                        {
                                erasePrint(cout, " Reading ComparePdbFile: " + initFold);
                                _pdbList.push_back(pdbStruct);
                        }
                        pdbStruct.reset();
                }

                if (DEBUG)
                        for(deque<Pdb>::iterator i = _pdbList.begin(); i != _pdbList.end(); i++)
                                _outputLog << (*i) << endl;
                pdb.close();

                _outputLog << "initF before runAssignPDB" << endl;
                runAssignPDB(mdSetM2, mdPAMap2, support);
        }
}

float NOESY_Assign::runRPF(Pdb& pdb, int type) const
{
        Qscore dpscore; dpscore.reset();

        // final score only
        //Qscore pscore; pscore.reset();eeeeeeeee

        //check mscore
        ASDP::ratio mscore;

        for(deque<NOESY*>::const_iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++) // each peak list
        {
                erasePrint(cout, " quality-estimation for " + (*iter)->label() + " ... ");
                ASDP::ratio temp;
                temp = (*iter)->Validate_Shift(_outputOvw, _outputM);

                if (temp.first/(float)temp.second > 0.5)
                {
                        _outputOvw << "M-score > 0.5, calculation of RPF scores stopped." << endl;
                        continue;
                }

                mscore.first += temp.first;
                mscore.second += temp.second;

                dpscore = (*iter)->Rfactor(pdb, _outputOvw, _outputI, _outputL, dpscore); // calc r factor, + in the Rfactor function

                if (!isS())
                {
                        erasePrint(cout,"prtSimleMatchPeakAtom for " + (*iter)->label() + " ... ");
                        ofstream matchStream((outputDir + "/" + (*iter)->label() + "_match").c_str());
                        if (!matchStream) error("cannot open output file", outputDir + "/" + (*iter)->label() + "_match");
                        (*iter)->prtSimpleMatchPeakAtom(matchStream);
                        matchStream.close();

                        system(("gzip -rf " + outputDir + "/*_match").c_str());
                }
        }

        //_outputOvw << endl << endl << " Final M-score " << mscore.first/(float)mscore.second << " " << mscore.first << "/" << mscore.second << endl;

        Qscore pscore; pscore = dpscore; //return the ave
        float score = (Ffactor(pscore.FN, pscore.FP, 1) - Ffactor(pscore.FNR, pscore.FPR, 1))/(Ffactor(pscore.FNI, pscore.FPI, 1)-Ffactor(pscore.FNR, pscore.FPR, 1));

        cout << "Recall " <<  1-pscore.FN.first/(float)pscore.FN.second << endl;
        cout << "Precision " << 1-pscore.FP.first/(float)pscore.FP.second << endl;
        cout << "F " <<  Ffactor(pscore.FN, pscore.FP, 1) << endl;
        cout << "Score " << score << endl;

        if (type == 0)
                return 1-pscore.FN.first/(float)pscore.FN.second;
        else if (type == 1)
                return 1-pscore.FP.first/(float)pscore.FP.second;
        else if (type == 2)     // F-measure
                return Ffactor(pscore.FN, pscore.FP, 1);
        //_outputOvw << endl << endl <<  " DP-Score: " << score << endl;
        else if (type == 3) // dp
                return score;
}

void NOESY_Assign::runRfactor()
{

        // R factor
        if (isQ() && (Qpdb != ""))
        {
                _pdbList.clear();
                ifstream pdb(Qpdb.c_str());
                if (!pdb) error ("cannot open input file", Qpdb);
                Pdb pdbStruct(0);

                while (pdb)
                {
                        pdb >> pdbStruct;
                        if (pdbStruct.notEmpty())
                        {
                                erasePrint(cout, " Reading ComparePdbFile: " + Qpdb);
                                _pdbList.push_back(pdbStruct);
                        }
                        pdbStruct.reset();
                }

                if (DEBUG)
                {
                        _outputLog << " ComparePdbFile is Read: " << endl;
                        for(deque<Pdb>::iterator i = _pdbList.begin(); i != _pdbList.end(); i++)
                                _outputLog << (*i) << endl;
                }
                pdb.close();
        }

        deque<Qscore> dpscore; dpscore.clear(); //(_pdbList.size()+1);

        for(int i=0; i<= _pdbList.size(); i++)
        {
                Qscore pscore; pscore.reset();
                dpscore.push_back(pscore);
        }

        // final score only

        //check mscore
        ASDP::ratio mscore;

        for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++) // each peak list
        {
                erasePrint(cout, " quality-estimation for " + (*iter)->label() + " ... ");
                ASDP::ratio temp;
                temp = (*iter)->Validate_Shift(_outputOvwRPF, _outputM);

                if (temp.first/(float)temp.second > 0.5)
                {
                        _outputOvwRPF << "Input error: M-score > 0.5" << endl;
                        continue;
                }

                mscore.first += temp.first;
                mscore.second += temp.second;

                (*iter)->Rfactor(_pdbList, _outputOvwRPF, _outputI, _outputL, dpscore); // calc r factor, + in the Rfactor function

                if (!isS())
                {
                        erasePrint(cout,"prtSimleMatchPeakAtom for " + (*iter)->label() + " ... ");
                        ofstream matchStream((outputDir + "/" + (*iter)->label() + "_match").c_str());
                        if (!matchStream) error("cannot open output file", outputDir + "/" + (*iter)->label() + "_match");
                        (*iter)->prtSimpleMatchPeakAtom(matchStream);
                        matchStream.close();

                        system(("gzip -rf " + outputDir + "/*_match").c_str());
                }
        }

        _outputOvwRPF << endl << endl << std::fixed <<  std::setprecision(2) << " Final M-score " << mscore.first/((float)mscore.second+0.00001) << " " << mscore.first << "/" << mscore.second << endl;

        for(int i=0; i<= _pdbList.size(); i++)
        {

                Qscore pscore; pscore = dpscore[i];

                if (i==0)
                        _outputOvwRPF << endl << "Model ave" << endl;
                else
                        _outputOvwRPF << endl << "Model " << i << endl;

                if (!isS())
                {
                        _outputOvwRPF << endl << " Final Recall-score for input query structures: " << 1-pscore.FN.first/(float)pscore.FN.second;
                        _outputOvwRPF << endl << " Final Precision-score for input query structures: " << 1-pscore.FP.first/(float)pscore.FP.second;

                        _outputOvwRPF << endl << endl  << " Final Recall-score for ideal true structures: " << 1-pscore.FNI.first/(float)pscore.FNI.second;
                        //pick the max one between FP, FPI and FPR


                        if (pscore.FP.first/(float)pscore.FP.second < pscore.FPI.first/(float)pscore.FPI.second)
                        {
                                pscore.FPI = pscore.FP;
                        }
                        if (pscore.FPR.first/(float)pscore.FPR.second < pscore.FPI.first/(float)pscore.FPI.second)
                        {
                                pscore.FPI = pscore.FPR;
                        }
                        _outputOvwRPF << endl << " Final Precision-score for ideal true structures: " << 1-pscore.FPI.first/(float)pscore.FPI.second;

                        _outputOvwRPF << endl << endl << " Final Recall-score for freely-rotating-chain model: " << 1-pscore.FNR.first/(float)pscore.FNR.second;
                        _outputOvwRPF << endl << " Final Precision-score for freely-rotating-chain model: " << 1-pscore.FPR.first/(float)pscore.FPR.second;

                        _outputOvwRPF << endl << endl << " Summary:" << endl
                                      << " F-score of input query structures:      " << Ffactor(pscore.FN, pscore.FP, 1); // << " " << Ffactor(pscore.FN, pscore.FP, 2);
                        _outputOvwRPF << endl << " F-score of ideal true structures:       " << Ffactor(pscore.FNI, pscore.FPI, 1); // << " " << Ffactor(ideal, pscore.FPR, 2);
                        _outputOvwRPF << endl << " F-score of freely-rotating-chain model: " << Ffactor(pscore.FNR, pscore.FPR, 1); // << " " << Ffactor(pscore.FNR, pscore.FPR, 2);

                }

                float score = (Ffactor(pscore.FN, pscore.FP, 1) - Ffactor(pscore.FNR, pscore.FPR, 1))/(Ffactor(pscore.FNI, pscore.FPI, 1)-Ffactor(pscore.FNR, pscore.FPR, 1));

                if ((1-pscore.FPR.first/(float)pscore.FPR.second) > ( 1-pscore.FPI.first/(float)pscore.FPI.second))
                {
                        _outputOvwRPF << endl << endl << "Input error: Precision(free-chain model) > Precision(Ideal)" << endl;
                        continue;
                } else
                {
                        if (isS() && (control["EC"] != ""))
                                _outputOvwRPF << endl << endl <<  " DP-Score: " << 1-pscore.FN.first/(float)pscore.FN.second << endl; // use recall instead
                        else
                                _outputOvwRPF << endl << endl <<  " DP-Score: " << score << endl;
                }
        }
}

void NOESY_Assign::runDyana(string current)
{               //--- write the sequence file
        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Writing DYANA sequence file ...");
        string DianaSeq;
        DianaSeq = control["proteinName"] + ".seq";
        ofstream SeqStream((current + DianaSeq).c_str());
        if (!SeqStream) error("cannot open output file", current + DianaSeq);
        prtDianaSeq(SeqStream, seq, LLength);
        SeqStream.close();

        //--- write the upl file
        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Writing DYANA UPL file ...");
        string DianaFile;
        DianaFile = current + control["proteinName"] + ".upl";
        ofstream DianaStream(DianaFile.c_str());
        if (!DianaStream) error("cannot open output file", DianaFile);
        prtDianabuffer(DianaStream, seq, LLength, _DianaBuffer, _ICDianaBuffer);
        DianaStream.close();

        if ((_monster) && ((_ddcycle == 1) && (_dcycle == 0)))
        {
                string DianaFileAmbi;
                DianaFileAmbi = current + control["proteinName"] + ".uplAmbi";
                ofstream DianaStreamAmbi(DianaFileAmbi.c_str());
                if (!DianaStreamAmbi) error("cannot open output file", DianaFileAmbi);

                prtDianabufferDeque(DianaStreamAmbi, seq, LLength, _DianaBufferDeque, _ICDianaBufferDeque);

                DianaStreamAmbi.close();
        }


        //--- write the HBOND file
        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Writing DYANA Hbonds ...");
        ofstream hbondStreamUpl((current+"hbond.upl").c_str()); // upl
        if (!hbondStreamUpl) error("cannot open output file : ", current+"hbond.upl");
        prtDianaHbond(hbondStreamUpl, _par.hbond.upperOH, _par.hbond.upperON, seq, LLength, HBONDList); // DYANA
        ofstream hbondStreamlol((current+"hbond.lol").c_str()); // lol
        if (!hbondStreamUpl) error("cannot open output file : ", current+"hbond.lol");
        prtDianaHbond(hbondStreamlol, _par.hbond.lowerOH, _par.hbond.lowerON, seq, LLength, HBONDList); // DYANA

        if (control["HBOND"] != "") // manual input of hbond
        {
                ifstream from(control["HBOND"].c_str());
                if (!from) error("cann't open input file : ", control["HBOND"]);

                set<HBOND, less<HBOND> > ManuHBONDList; ManuHBONDList.clear();
                string hbond, a1, a2;
                int i1, i2;
                while(from >> hbond >> a1 >> i1 >> a2 >> i2)
                {
                        if ((a1 == "O") && (a2 == "HN"))
                                ManuHBONDList.insert(make_pair(i1,i2));
                        else if ((a1 == "HN") && (a2 == "O"))
                                ManuHBONDList.insert(make_pair(i2,i1));
                        else
                        {
                                error("**Error: Bad format for hbond file : ", control["HBOND"].c_str());
                                error("  " + hbond + " " + a1 + " " + itoa(i1) + " " + a2 + " " + itoa(i2), "");
                        }
                }

                hbondStreamUpl << endl << "#Manual Input\n";
                prtDianaHbond(hbondStreamUpl, _par.hbond.upperOH, _par.hbond.upperON, seq, LLength, ManuHBONDList);
                hbondStreamUpl << "#END Manual Input" << endl;
                hbondStreamlol << endl << "#Manual Input\n";
                prtDianaHbond(hbondStreamlol, _par.hbond.lowerOH, _par.hbond.lowerON, seq, LLength, ManuHBONDList);
                hbondStreamlol << "#END Manual Input" << endl;
        }
        hbondStreamUpl.close();
        hbondStreamlol.close();

        // manual constraints
        if (control["UPL"] != "") // manual input of INPUT
        {
                ifstream from(control["UPL"].c_str());
                if (!from) error("cann't open input file : ", control["UPL"]);
                ofstream to((current+control["proteinName"]+".upl").c_str(), std::ios::out|std::ios::app);
                if (!to) error("cannot open output file : ", current+control["proteinName"]+".upl");

                to << "# Manual Input\n";
                char ch;
                while (from.get(ch)) to.put(ch);
                to << "# END Manual Input" << endl;
        }

        if (control["LOL"] != "") // manual input of LOL
        {
                ifstream from(control["LOL"].c_str());
                if (!from) error("cann't open input file : ", control["LOL"]);
                ofstream to((current+control["proteinName"]+".lol").c_str(), std::ios::out|std::ios::app);
                if (!to) error("cannot open output file : ", current+control["proteinName"]+".lol");

                to << "# Manual Input\n";
                char ch;
                while (from.get(ch)) to.put(ch);
                to << "# END Manual Input" << endl;
        }

        //--- write the aco file
        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Writing DYANA ACO ...");
        if (control["ACO"] != "") // addin manual ones
        {
                ifstream from(control["ACO"].c_str());
                if (!from) error("cann't open input file : ", control["ACO"]);
                ofstream to((current+control["proteinName"]+".aco").c_str(), std::ios::out|std::ios::app);
                if (!to) error("cannot open output file : ", current+control["proteinName"]+".aco");

                to << "#Manual Input\n";
                char ch;
                while (from.get(ch)) to.put(ch);
                to << "#END Manual Input" << endl;
                to.close();
        }

        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Running DYANA ...\n");

        bool suss = 0;
        if (RESTORE)
        {
                // copy the sourcePdb into current directory
                ifstream* from = 0;
                ifstream from1((sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + '/' + control["proteinName"] + ".pdb").c_str());
                ifstream from2((sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + "_final/" + control["proteinName"] + ".pdb").c_str());

                if (from1)
                        from = &from1;
                else if (from2)
                        from = &from2;

                if (!from)
                {
                        {
                                suss = 0;
                                if (!from1)
                                        erasePrint(cout, " Cann't find file " + sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + '/' + control["proteinName"] + ".pdb\n");
                                if (!from2)
                                        erasePrint(cout, " Cann't find file " + sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + "_final/" + control["proteinName"] + ".pdb\n");
                        }
                }
                else
                {
                        erasePrint(cout, " Restoring from " + sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + '/' + control["proteinName"] + ".pdb\n");
                        ofstream to((current+control["proteinName"] + ".pdb").c_str());
                        if (!to) error("cannot open output file : ", current+control["proteinName"] + ".pdb");
                        char ch;
                        while (from->get(ch)) to.put(ch);
                        suss = 1;
                        to.close();
                }

                // copy the sourcePdb.ovw into current directory
                ifstream* ofrom = 0;
                ifstream ofrom1((sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + '/' + control["proteinName"] + ".ovw").c_str());
                ifstream ofrom2((sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + "_final/" + control["proteinName"] + ".ovw").c_str());

                if (ofrom1)
                        ofrom = &ofrom1;
                else if (ofrom2)
                        ofrom = &ofrom2;

                if (!ofrom)
                {
                        erasePrint(cout, " Cann't find file " + sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + '/' + control["proteinName"] + ".ovw\n");
                }
                else
                {
                        erasePrint(cout, " Restoring from " + sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + '/' + control["proteinName"] + ".ovw\n");
                        ofstream to((current+control["proteinName"] + ".ovw").c_str());
                        if (!to) error("cannot open output file : ", current+control["proteinName"] + ".ovw");
                        char ch;
                        while (ofrom->get(ch)) to.put(ch);
                        to.close();
                }
        }

        if ((!RESTORE) || (!suss))
        {
                int dpid;
                if ((dpid = fork()) == -1)
                        error("Cann't fork", "");
                else if (dpid == 0)
                {
                        if (chdir(current.c_str()) == -1)
                                error("**cannot move to directory", current);

                        ofstream shcmd("run-dyana-sh");
                        if (!shcmd) error("cannot open output file", "run-dyana-sh in " + current);

                        shcmd << "#!/bin/sh\n" << command.dyana;
                        if (CISPeptideBond.size())
                        {
                                shcmd << " -cis ";
                                for(set<int, less<int> >::iterator i = CISPeptideBond.begin(); i != CISPeptideBond.end(); ++i)
                                {
                                        if (i != CISPeptideBond.begin())
                                                shcmd << ",";
                                        shcmd << *i;
                                }
                        }
                        if (SSBond.size())
                        {
                                shcmd << " -ss ";
                                for(set<pair<int, int>, less<pair<int, int> > >::iterator i = SSBond.begin(); i != SSBond.end(); ++i)
                                {
                                        if (i != SSBond.begin())
                                                shcmd << ",";
                                        shcmd << i->first << "-" << i->second;
                                }
                        }

                        shcmd << " -st " << seq.begin()->first << " -na " << control["proteinName"] << " -pr dyana" << endl;

                        if (execlp("sh", "sh", "run-dyana-sh", (char*)0) < 0)
                                error("**Can not run DYANA", "Check run-dyana-sh");
                }

                int qstat;
                waitpid(dpid, &qstat, 0);
        }
}
void NOESY_Assign::prtCyanaCons(string current, stringstream& ambiStream)
{               //--- write the sequence file
        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Writing CYANA sequence file ...");
        string cyanaSeq;
        cyanaSeq = control["proteinName"] + ".seq";
        ofstream SeqStream((current + cyanaSeq).c_str());
        if (!SeqStream) error("cannot open output file", current + cyanaSeq);
        prtCyanaSeq(SeqStream, seq, LLength);
        SeqStream.close();

        //--- write the upl file
        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Writing CYANA UPL file ...");
        string DianaFile;

        DianaFile = current + control["proteinName"]+".upl";


        ofstream DianaStream(DianaFile.c_str());
        if (!DianaStream) error("cannot open output file", DianaFile);
        // upl from noesy_assign
        prtCyanabuffer(DianaStream, seq, LLength, _DianaBuffer, _ICDianaBuffer, atoi(control["center"].c_str()));

        DianaStream << ambiStream.str();

        DianaStream.close();


        //--- write the HBOND file
        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Writing CYANA Hbonds ...");
        ofstream hbondStreamUpl((current+"hbond.upl").c_str()); // upl
        if (!hbondStreamUpl) error("cannot open output file : ", current+"hbond.upl");
        prtCyanaHbond(hbondStreamUpl, _par.hbond.upperOH, _par.hbond.upperON, seq, LLength, HBONDList); // DYANA
        ofstream hbondStreamlol((current+"hbond.lol").c_str()); // lol
        if (!hbondStreamUpl) error("cannot open output file : ", current+"hbond.lol");
        prtCyanaHbond(hbondStreamlol, _par.hbond.lowerOH, _par.hbond.lowerON, seq, LLength, HBONDList); // DYANA

        //write the prot file
        if (isEC())
        {
                erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Writing CYANA Prot ...");
                ofstream protStream((current+control["proteinName"]+".prot").c_str()); //
                if (!protStream) error("cannot open output file : ", current+control["proteinName"]+".prot");
                prtCyanaProt(protStream, _atoms); //


                ofstream to((current+control["proteinName"]+".ec").c_str(), std::ios::out|std::ios::app);
                if (!to) error("cannot open output file : ", current+control["proteinName"]+".ec");

                // add in contact map
                /*
                  for(ContactPairSet::iterator i = support.ContactBin.begin(); i != support.ContactBin.end(); i++)
                  {
                  to << i->first << "\t" << i->second << "\t" << 0.125 << "\n";
                  }
                */

                for(map<HBOND, ECscore, less<HBOND> >::iterator i = ECList.begin(); i != ECList.end(); i++)
                {
                        if (i->second.weightAdjust > 0)
                        {
                                if (i->second.weightAdjust > 1) i->second.weightAdjust = 1;
                                to << i->first.first << "\t" << i->first.second << "\t" << i->second.distanceAdjust << "\t" << i->second.weightAdjust << "\t#" << i->second.comment << "\n";
                        }
                        else
                                i->second.weightAdjust = 0;
                }
                to.close();
        }

        /*
          if (control["HBOND"] != "") // manual input of hbond
          {
          ifstream from(control["HBOND"].c_str());
          if (!from) error("cann't open input file : ", control["HBOND"]);

          set<HBOND, less<HBOND> > ManuHBONDList; ManuHBONDList.clear();
          string hbond, a1, a2;
          int i1, i2;
          while(from >> hbond >> a1 >> i1 >> a2 >> i2)
          {
          if ((a1 == "O") && (a2 == "H"))
          ManuHBONDList.insert(make_pair(i1,i2));
          else if ((a1 == "H") && (a2 == "O"))
          ManuHBONDList.insert(make_pair(i2,i1));
          else
          {
          error("**Error: Bad format for hbond file : ", control["HBOND"].c_str());
          error("  " + hbond + " " + a1 + " " + itoa(i1) + " " + a2 + " " + itoa(i2), "");
          }
          }

          hbondStreamUpl << endl << "#Manual Input\n";
          prtCyanaHbond(hbondStreamUpl, _par.hbond.upperOH, _par.hbond.upperON, seq, LLength, ManuHBONDList);
          hbondStreamUpl << "#END Manual Input" << endl;
          hbondStreamlol << endl << "#Manual Input\n";
          prtCyanaHbond(hbondStreamlol, _par.hbond.lowerOH, _par.hbond.lowerON, seq, LLength, ManuHBONDList);
          hbondStreamlol << "#END Manual Input" << endl;
          }
        */

        hbondStreamUpl.close();
        hbondStreamlol.close();

        // manual constraints
        if (control["UPL"] != "") // manual input of INPUT
        {
                ifstream from(control["UPL"].c_str());
                if (!from) error("cann't open input file : ", control["UPL"]);
                ofstream to((current+control["proteinName"]+".upl").c_str(), std::ios::out|std::ios::app);
                if (!to) error("cannot open output file : ", current+control["proteinName"]+".upl");

                to << "# Manual Input\n";
                char ch;
                while (from.get(ch)) to.put(ch);
                to << "# END Manual Input" << endl;
        }

        if (control["LOL"] != "") // manual input of LOL
        {
                ifstream from(control["LOL"].c_str());
                if (!from) error("cann't open input file : ", control["LOL"]);
                ofstream to((current+control["proteinName"]+".lol").c_str(), std::ios::out|std::ios::app);
                if (!to) error("cannot open output file : ", current+control["proteinName"]+".lol");

                to << "# Manual Input\n";
                char ch;
                while (from.get(ch)) to.put(ch);
                to << "# END Manual Input" << endl;
        }

        //--- write the aco file
        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Writing CYANA ACO ...");
        if (control["ACO"] != "") // addin manual ones
        {
                ifstream from(control["ACO"].c_str());
                if (!from) error("cann't open input file : ", control["ACO"]);
                ofstream to((current+control["proteinName"]+".aco").c_str(), std::ios::out|std::ios::app);
                if (!to) error("cannot open output file : ", current+control["proteinName"]+".aco");

                to << "#Manual Input\n";
                char ch;
                while (from.get(ch)) to.put(ch);
                to << "#END Manual Input" << endl;
                to.close();
        }

        //RT --- write the rdc file
        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Writing CYANA RDC ...");
        if (control["RDC"] != "") // addin manual ones
        {
                ifstream from(control["RDC"].c_str());
                if (!from) error("cann't open input file : ", control["RDC"]);
                ofstream to((current+control["proteinName"]+".rdc").c_str(), std::ios::out|std::ios::app);
                if (!to) error("cannot open output file : ", current+control["proteinName"]+".rdc");

                char ch;
                while (from.get(ch)) to.put(ch);
                to.close();
        }

        //RT --- write some special.lib file
        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Writing CYANA Lib ...");
        if (control["lib"] != "") // addin manual ones
        {
                ifstream from(control["lib"].c_str());
                if (!from) error("cann't open input file : ", control["lib"]);
                ofstream to((current+"special.lib").c_str(), std::ios::out|std::ios::app);
                if (!to) error("cannot open output file : ", current+"special.lib");

                char ch;
                while (from.get(ch)) to.put(ch);
                to.close();
        }
}
void NOESY_Assign::runCyana(string current, stringstream& ambiStream)
{
        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Running CYANA ...\n");

        bool suss = 0;
        if (RESTORE)
        {
                // copy the sourcePdb into current directory
                ifstream* from = 0;
                ifstream from1((sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + '/' + control["proteinName"] + ".pdb").c_str());
                ifstream from2((sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + "_final/" + control["proteinName"] + ".pdb").c_str());

                if (from1)
                        from = &from1;
                else if (from2)
                        from = &from2;

                if (!from)
                {
                        {
                                suss = 0;
                                if (!from1)
                                        erasePrint(cout, " Cann't find file " + sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + '/' + control["proteinName"] + ".pdb\n");
                                if (!from2)
                                        erasePrint(cout, " Cann't find file " + sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + "_final/" + control["proteinName"] + ".pdb\n");
                        }
                }
                else
                {
                        erasePrint(cout, " Restoring from " + sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + '/' + control["proteinName"] + ".pdb\n");
                        ofstream to((current+control["proteinName"] + ".pdb").c_str());
                        if (!to) error("cannot open output file : ", current+control["proteinName"] + ".pdb");
                        char ch;
                        while (from->get(ch)) to.put(ch);
                        suss = 1;
                        to.close();
                }

                // copy the sourcePdb.ovw into current directory
                ifstream* ofrom = 0;
                ifstream ofrom1((sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + '/' + control["proteinName"] + ".ovw").c_str());
                ifstream ofrom2((sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + "_final/" + control["proteinName"] + ".ovw").c_str());

                if (ofrom1)
                        ofrom = &ofrom1;
                else if (ofrom2)
                        ofrom = &ofrom2;

                if (!ofrom)
                {
                        erasePrint(cout, " Cann't find file " + sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + '/' + control["proteinName"] + ".ovw\n");
                }
                else
                {
                        erasePrint(cout, " Restoring from " + sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + '/' + control["proteinName"] + ".ovw\n");
                        ofstream to((current+control["proteinName"] + ".ovw").c_str());
                        if (!to) error("cannot open output file : ", current+control["proteinName"] + ".ovw");
                        char ch;
                        while (ofrom->get(ch)) to.put(ch);
                        to.close();
                }
        }

        if ((!RESTORE) || (!suss))
        {
                int dpid;
                if ((dpid = fork()) == -1)
                        error("Cann't fork", "");
                else if (dpid == 0)
                {
                        if (chdir(current.c_str()) == -1)
                                error("**cannot move to directory", current);

                        ofstream shcmd("run-cyana-sh");
                        if (!shcmd) error("cannot open output file", "run-cyana-sh in " + current);

                        shcmd << "#!/bin/sh\n" << command.cyana;
                        if (CISPeptideBond.size())
                        {
                                shcmd << " -cis ";
                                for(set<int, less<int> >::iterator i = CISPeptideBond.begin(); i != CISPeptideBond.end(); ++i)
                                {
                                        if (i != CISPeptideBond.begin())
                                                shcmd << ",";
                                        shcmd << *i;
                                }
                        }
                        if (SSBond.size())
                        {
                                shcmd << " -ss ";
                                for(set<pair<int, int>, less<pair<int, int> > >::iterator i = SSBond.begin(); i != SSBond.end(); ++i)
                                {
                                        if (i != SSBond.begin())
                                                shcmd << ",";
                                        shcmd << i->first << "-" << i->second;
                                }
                        }

                        if (control["lib"] != "")
                        {
                                shcmd << " -lib special.lib ";
                        }

                        if (control["RDC"] != "")
                        {
                                shcmd << " -rdc ";
                        }

                        shcmd << " -st " << seq.begin()->first << " -na " << control["proteinName"] << " -pr cyana" << " -cf " << controlFileName;

                        if (isEC())
                        {
                                if (isAMBI())
                                {
                                        shcmd << " -ambi";
                                } else
                                {
                                        shcmd << " -ec"; //use CANDID
                                        shcmd << " -peaks ";

                                        for(PeakListDeque::iterator pi = peakListDeque.begin(); pi != peakListDeque.end(); pi++)
                                        {
                                                if (pi != peakListDeque.begin()) shcmd << ",";
                                                shcmd << (*pi)["name"];
                                        }
                                }
                        }

                        shcmd << endl;
                        if (execlp("sh", "sh", "run-cyana-sh", (char*)0) < 0)
                                error("**Can not run CYANA", "Check run-cyana-sh at WorkingCycle");
                }

                int qstat;
                waitpid(dpid, &qstat, 0);
        }

}
void NOESY_Assign::runCNS(string current)
{               erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Running CNS ...\n");

        //--- exec cns
        bool suss = 0;
        if (RESTORE)
        {
                // copy the sourcePdb into current directory
                ifstream from((sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + "/cns.pdb").c_str());
                if (!from)
                {
                        suss = 0;
                        erasePrint(cout, " Cann't find file " + sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + "/cns.pdb\n");
                }
                else
                {
                        erasePrint(cout, " Restoring from " + sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + "/cns.pdb\n");
                        ofstream to((current+"cns.pdb").c_str());
                        if (!to) error("cannot open output file : ", current+"cns.pdb");
                        char ch;
                        while (from.get(ch)) to.put(ch);
                        suss = 1;
                        to.close();
                }
        }

        if ((!RESTORE) || (!suss))
        {
                int dpid;
                if ((dpid = fork()) == -1)
                        error("Cann't fork", "");
                else if (dpid == 0)
                {
                        if (chdir(current.c_str()) == -1)
                                error("**cannot move to directory", current);

                        ofstream shcmd("run-cns-sh");
                        if (!shcmd) error("cannot open output file", "run-cns-sh in " + current);

                        shcmd << "#!/bin/sh" << endl;
                        string cmd = //"cd " + current + "\n"+
                                command.cns + " " + control["proteinName"] + " " + control["cnsPath"];
                        shcmd << cmd << endl;

                        if (execlp("sh", "sh", "run-cns-sh", (char*)0) < 0)
                                error("**Can not run CNS", "Check run-cns-sh");
                }

                int qstat;
                waitpid(dpid, &qstat, 0);
        }

}
void NOESY_Assign::prtXplorCons(string current)
{               //--- write the sequence file
        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Writing XPLOR sequence file ...");
        string XPLORSeq;
        XPLORSeq = control["proteinName"] + ".seqXplor";
        ofstream SeqXPLORStream((current + XPLORSeq).c_str());
        if (!SeqXPLORStream) error("cannot open output file", current + XPLORSeq);
        prtXPLORSeq(SeqXPLORStream, seq, LLength);
        SeqXPLORStream.close();

        if (_DianaBuffer.size() || _ICDianaBuffer.size())
        {
                // print XPLOR constraints
                erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Writing XPLOR file ...");
                string XPLORFile;
                XPLORFile = current + control["proteinName"] + "_noe.tbl";
                ofstream XPLORStream(XPLORFile.c_str());
                if (!XPLORStream) error("cannot open output file", XPLORFile);
                // upl from noesy_assign
                prtXPLORbuffer(XPLORStream, seq, LLength, _DianaBuffer, _ICDianaBuffer, atoi(control["center"].c_str()));
                XPLORStream.close();
        }

        // XPLOR hbond

        if (HBONDList.size())
        {
                erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Writing XPLOR Hbonds ...");
                ofstream hbondStreamXpl((current+control["proteinName"]+"_hbond.tbl").c_str()); // upl
                if (!hbondStreamXpl) error("cannot open output file : ", current+control["proteinName"] + "_hbond.tbl");
                prtXPLORHbond(hbondStreamXpl, _par.hbond.upperOH, _par.hbond.upperON, seq, LLength, HBONDList);
        }
        /*
                        if (control["HBOND"] != "") // manual input of hbond
                        {
                        ifstream from(control["HBOND"].c_str());
                        if (!from) error("cann't open input file : ", control["HBOND"]);

                        ofstream to((current+control["proteinName"]+"_hbond.tbl").c_str(), std::ios::out|std::ios::app);
                        if (!to) error("cannot open output file : ", current+control["proteinName"]+"_hbond.tbl");

                        set<HBOND, less<HBOND> > ManuHBONDList; ManuHBONDList.clear();
                        string hbond, a1, a2;
                        int i1, i2;
                        while(from >> hbond >> a1 >> i1 >> a2 >> i2)
                        {
                        if ((a1 == "O") && (a2 == "HN"))
                        ManuHBONDList.insert(make_pair(i1,i2));
                        else if ((a1 == "HN") && (a2 == "O"))
                        ManuHBONDList.insert(make_pair(i2,i1));
                        else
                        {
                        error("**Error: Bad format for hbond file : ", control["HBOND"].c_str());
                        error("  " + hbond + " " + a1 + " " + itoa(i1) + " " + a2 + " " + itoa(i2), "");
                        }
                        }

                        to << endl << "!Manual Input\n";
                        prtXPLORHbond(to, _par.hbond.upperOH, _par.hbond.upperON, seq, LLength, ManuHBONDList);
                        to << "!END Manual Input" << endl;
                        to.close();
                        }
        */

        // manual constraints
        if (control["XPL"] != "") // manual input of INPUT
        {
                ifstream from(control["XPL"].c_str());
                if (!from) error("cann't open input file : ", control["XPL"]);
                ofstream to((current+control["proteinName"]+"_noe.tbl").c_str(), std::ios::out|std::ios::app);
                if (!to) error("cannot open output file : ", current+control["proteinName"]+"_noe.tbl");

                to << "! Manual Input\n";
                char ch;
                while (from.get(ch)) to.put(ch);
                to << "! END Manual Input" << endl;
        }

        //--- write the aco file
        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Writing XPLOR ...");
        if (control["DIHE"] != "") // addin manual ones
        {
                ifstream from(control["DIHE"].c_str());
                if (!from) error("cann't open input file : ", control["DIHE"]);
                ofstream to((current+control["proteinName"]+"_dihe.tbl").c_str(), std::ios::out|std::ios::app);
                if (!to) error("cannot open output file : ", current+control["proteinName"]+"_dihe.tbl");

                to << "!Manual Input\n";
                char ch;
                while (from.get(ch)) to.put(ch);
                to << "!END Manual Input" << endl;
                to.close();
        }

        cyana2xplor(current);

        // print additional XPLOR constraints
        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Writing XPLOR file ...");

        if (_atoms.size())
        {
                string XPLORFile;
                XPLORFile = current + control["proteinName"] + "_shiftsCACB.tbl";
                ofstream XPLORStream(XPLORFile.c_str());
                if (!XPLORStream) error("cannot open output file", XPLORFile);
                prtXplorCACBShift(XPLORStream, seq, _atoms);
        }

        if (JList.size())
        {
                string XPLORFile;
                XPLORFile = current + control["proteinName"] + "_Jhnha.tbl";
                ofstream XPLORStream(XPLORFile.c_str());
                if (!XPLORStream) error("cannot open output file", XPLORFile);
                prtXplorJvalues(XPLORStream, seq, JList);
        }

}



void NOESY_Assign::runXplor(string current)
{       erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Running XPLOR ...\n");

        //--- exec xplor
        bool suss = 0;
        if (RESTORE)
        {
                ifstream* from = 0;
                ifstream from1((sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + '/' + control["proteinName"] + ".pdb").c_str());
                ifstream from2((sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + "_final/" + control["proteinName"] + ".pdb").c_str());

                if (from1)
                        from = &from1;
                else if (from2)
                        from = &from2;

                if (!from)
                {
                        {
                                suss = 0;
                                if (!from1)
                                        erasePrint(cout, " Cann't find file " + sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + '/' + control["proteinName"] + ".pdb\n");
                                if (!from2)
                                        erasePrint(cout, " Cann't find file " + sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + "_final/" + control["proteinName"] + ".pdb\n");
                        }
                }
                else
                {
                        erasePrint(cout, " Restoring from " + sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + '/' + control["proteinName"] + ".pdb\n");
                        ofstream to((current+control["proteinName"] + ".pdb").c_str());
                        if (!to) error("cannot open output file : ", current+control["proteinName"] + ".pdb");
                        char ch;
                        while (from->get(ch)) to.put(ch);
                        suss = 1;
                        to.close();
                }

                // copy the sourcePdb.ovw into current directory
                ifstream* ofrom = 0;
                ifstream ofrom1((sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + '/' +"HydrogenBondData.txt").c_str());
                ifstream ofrom2((sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + "_final/" +"HydrogenBondData.txt").c_str());

                if (ofrom1)
                        ofrom = &ofrom1;
                else if (ofrom2)
                        ofrom = &ofrom2;

                if (!ofrom)
                {
                        erasePrint(cout, " Cann't find file " + sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + '/' +"HydrogenBondData.txt");
                }
                else
                {
                        erasePrint(cout, " Restoring from " + sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + '/' +"HydrogenBondData.txt");
                        ofstream to((current+"HydrogenBondData.txt").c_str());
                        if (!to) error("cannot open output file : ", current+"HydrogenBondData.txt");
                        char ch;
                        while (ofrom->get(ch)) to.put(ch);
                        to.close();
                }
        }

        if ((!RESTORE) || (!suss))
        {
                //rdc
		//RT uncommented this and added a second alignment chance
                if (control["RDC"] != "") // manual input of RDC
                {
                ifstream from(control["RDC"].c_str());
                if (!from) error("cann't open input file : ", control["RDC"]);
                ofstream to((current+control["proteinName"]+"_sani.tbl").c_str());
                if (!to) error("cannot open output file : ", current+control["proteinName"]+"_sani.tbl");

                char ch;
                while (from.get(ch)) to.put(ch);
                }
                if (control["RDCaxis"] != "") // manual input of RDC
                {
                ifstream from(control["RDCaxis"].c_str());
                if (!from) error("cann't open input file : ", control["RDCaxis"]);
                ofstream to((current+"axis.pdb").c_str());
                if (!to) error("cannot open output file : ", current+"axis.pdb");

                char ch;
                while (from.get(ch)) to.put(ch);
                }

	        //rdc second alignment, RTT Jan 2019
                if (control["RDC2"] != "") // manual input of RDC
                {
                        ifstream from(control["RDC2"].c_str());
                        if (!from) error("cann't open input file : ", control["RDC2"]);
                        ofstream to((current+control["proteinName"]+"_sani2.tbl").c_str());
                        if (!to) error("cannot open output file : ", current+control["proteinName"]+"_sani2.tbl");

                        char ch;
                        while (from.get(ch)) to.put(ch);
                }
                if (control["RDCaxis2"] != "") // manual input of RDC
                {
                        ifstream from(control["RDCaxis2"].c_str());
                        if (!from) error("cann't open input file : ", control["RDCaxis2"]);
                        ofstream to((current+"axis2.pdb").c_str());
                        if (!to) error("cannot open output file : ", current+"axis2.pdb");

                        char ch;
                        while (from.get(ch)) to.put(ch);
                }
		//end RTT changes

                int dpid;
                if ((dpid = fork()) == -1)
                        error("Cann't fork", "");
                else if (dpid == 0)
                {
                        if (chdir(current.c_str()) == -1)
                                error("**cannot move to directory", current);

                        ofstream shcmd("run-xplor-sh");
                        if (!shcmd) error("cannot open output file", "run-xplor-sh in " + current);

                        shcmd << "#!/bin/sh" << endl;
                        shcmd << command.xplor;

                        if (CISPeptideBond.size())
                        {
                                shcmd << " -cis ";
                                for(set<int, less<int> >::iterator i = CISPeptideBond.begin(); i != CISPeptideBond.end(); ++i)
                                {
                                        if (i != CISPeptideBond.begin())
                                                shcmd << ",";
                                        shcmd << *i;
                                }
                        }
                        if (SSBond.size())
                        {
                                shcmd << " -ss ";
                                for(set<pair<int, int>, less<pair<int, int> > >::iterator i = SSBond.begin(); i != SSBond.end(); ++i)
                                {
                                        if (i != SSBond.begin())
                                                shcmd << ",";
                                        shcmd << i->first << "-" << i->second;
                                }
                        }

                        shcmd << " -st " << seq.begin()->first << " -na " << control["proteinName"] << " -pr xplor" << " -cf " << controlFileName << endl;

                        if (execlp("sh", "sh", "run-xplor-sh", (char*)0) < 0)
                                error("**Can not run Xplor", "Check run-xplor-sh at WorkingCycle");
                }

                int qstat;
                waitpid(dpid, &qstat, 0);
        }

}

void NOESY_Assign::runHyper(string current)
{

        string fileNameAco = ""; string fileNameDihe = "";
        if ((command.dyana != "") || (command.cyana != ""))
        {
                fileNameAco = current+control["proteinName"]+".aco";
        }

        if ((command.refine != "") || (command.xplor != "") || (command.cns != ""))
        {
                fileNameDihe = current+control["proteinName"]+"_dihe.tbl";
        }

        if (DEBUG) NOESY_Assign::outputLog() << "runHyper " << fileNameAco << " " << fileNameDihe << endl;

        ofstream toAco;
        if (fileNameAco != "")
        {
                toAco.open(fileNameAco.c_str());
                toAco.setf(ios::fixed, ios::floatfield);
                toAco.precision(1);
                if (!toAco) error("cannot open output file : ", fileNameAco);
        }
        ofstream toDihe;
        if (fileNameDihe != "")
        {
                toDihe.open(fileNameDihe.c_str());
                toDihe.setf(ios::fixed, ios::floatfield);
                toDihe.precision(1);
                if (!toDihe) error("cannot open output file : ", fileNameDihe);
        }

        int preType = 0; int nextType = 0;
        for(SEQM::const_iterator iter = seq.begin(); iter != seq.end(); ++iter)
        {
                SEQM::const_iterator niter = iter; niter++;

                if (niter != seq.end())
                {
                        nextType = niter->second.get_ss()->get_type();
                } else
                {
                        nextType = 0;
                }

                if ((*iter).second.get_sym_1() == 'P')
                        continue;

                // helix geometry in proteins, JMB 1988 201:601-19 Barlow and Thornton
                // mean - a-helix -62,-41 and 3-10 -71 and -18
                // use range 10
                // we use phi -81, -52 and psi -51 to -8

                // sheet -119,113 and -139, 135
                // phi -140 to -110 and psi 110 to 135
                // use range 20
                if ((iter->second.get_ss()->get_type() == 1) && (preType ==1) && (nextType == 1)) // && hp && hn)
                {
                        float lb; float ub;

                        //phi range 15
                        //lb = -80.0;ub = -50.0;
                        lb = -90.0;ub = -30.0; // -60
                        if ((command.dyana != "") || (command.cyana != ""))
                        {
                                toAco << iter->first << "\t" << residue_name[(*iter).second.get_sym_1()] << "\tPHI\t" << lb << "\t" << ub << "\t#helix" << endl;
                                NOESY_Assign::outputLog() << iter->first << "\t" << residue_name[(*iter).second.get_sym_1()] << "\tPHI\t" << lb << "\t" << ub << "\t#helix" << endl;
                        }
                        if ((command.refine != "") || (command.xplor != "") || (command.cns != ""))
                        {
                                toDihe << "assign (resid\t" << iter->first-1 << " and name C\t) (resid\t" << iter->first << " and name N\t)\n" <<
                                        "       (resid\t" << iter->first   << " and name CA\t) (resid\t" << iter->first << " and name C\t)" << "\t1.0\t";
                                toDihe << (float) (ub+lb)/2.0 << "\t" << (float) (ub-lb)/2.0 << "\t2" << endl;
                        }
                        //psi range 20
                        //lb = -50.0; ub = -10.0;
                        lb = -75.0; ub = -15.0; // -45
                        if ((command.dyana != "") || (command.cyana != ""))
                        {
                                toAco << iter->first << "\t" << residue_name[(*iter).second.get_sym_1()] << "\tPSI\t" << lb << "\t" << ub << "\t#helix" << endl;
                        }
                        if ((command.refine != "") || (command.xplor != "") || (command.cns != ""))
                        {
                                toDihe << "assign (resid\t" << iter->first << " and name N\t) (resid\t" << iter->first << " and name CA\t)\n" <<
                                        "       (resid\t" << iter->first   << " and name C\t) (resid\t" << iter->first+1 << " and name N\t)" << "\t1.0\t";
                                toDihe << (float) (ub+lb)/2.0 << "\t" << (float) (ub-lb)/2.0 << "\t2" << endl;
                        }
                }
                else if ((iter->second.get_ss()->get_type() == -1) && (preType == -1) && (nextType == -1))
                {
                        float lb; float ub;

                        // b-sheet should allow wider range - more iregularity
                        //phi
                        //lb = -160.0;ub = -90.0;
                        lb = -180.0;ub = -80.0; // (-140,-120) +-40
                        if ((command.dyana != "") || (command.cyana != ""))
                        {
                                toAco << iter->first << "\t" << residue_name[(*iter).second.get_sym_1()] << "\tPHI\t" << lb << "\t" << ub << "\t#sheet" << endl;
                        }
                        if ((command.refine != "") || (command.xplor != "") || (command.cns != ""))
                        {
                                toDihe << "assign (resid\t" << iter->first-1 << " and name C\t) (resid\t" << iter->first << " and name N\t)\n" <<
                                        "       (resid\t" << iter->first   << " and name CA\t) (resid\t" << iter->first << " and name C\t)" << "\t1.0\t";
                                toDihe << (float) (ub+lb)/2.0 << "\t" << (float) (ub-lb)/2.0 << "\t2" << endl;
                        }

                        //psi
                        //lb = 90.0; ub = 155.0;
                        lb = 75.0; ub = 170.0; // (130,115) +- 40
                        if ((command.dyana != "") || (command.cyana != ""))
                        {
                                toAco << iter->first << "\t" << residue_name[(*iter).second.get_sym_1()] << "\tPSI\t" << lb << "\t" << ub << "\t#sheet" << endl;
                        }
                        if ((command.refine != "") || (command.xplor != "") || (command.cns != ""))
                        {
                                toDihe << "assign (resid\t" << iter->first << " and name N\t) (resid\t" << iter->first << " and name CA\t)\n" <<
                                        "       (resid\t" << iter->first   << " and name C\t) (resid\t" << iter->first+1 << " and name N\t)" << "\t1.0\t";
                                toDihe << (float) (ub+lb)/2.0 << "\t" << (float) (ub-lb)/2.0 << "\t2" << endl;
                        }
                }
                preType = iter->second.get_ss()->get_type();
        }
}

void NOESY_Assign::cyana2xplor(string current)
{
        /*  problem with speical residues, also lol is not handled
            if (control["UPL"] != "")
            {

            string pdbstatDis = getenv("PSDIS");

            int dpid;
            if ((dpid = fork()) == -1)
            error("Cann't fork", "");
            else if (dpid == 0)
            {
            if (chdir(current.c_str()) == -1)
            error("**cann't move to diretory", current);

            if (execlp(pdbstatDis.c_str(), pdbstatDis.c_str(), (control["proteinName"]+".pdb").c_str(), control["UPL"], (control["proteinName"]+"_input.tbl").c_str(), (char*)0) < 0)
            error("**Can not run "+pdbstatDis, "");
            }

            int qstat;
            waitpid(dpid, &qstat, 0);

            ifstream from((control["proteinName"]+"_input.tbl").c_str());
            if (!from) error("cann't open input file : ", control["proteinName"]+"_input.tbl in "+ current);

            ofstream to((current+control["proteinName"]+"_noe.tbl").c_str(), std::ios::out|std::ios::app);
            if (!to) error("cannot open output file : ", current+control["proteinName"]+"_noe.tbl");

            to << "# Manual Input\n";
            char ch;
            while (from.get(ch)) to.put(ch);
            to << "# END Manual Input" << endl;
            }
        */

        if (control["ACO"] != "")
        {
                ifstream from(control["ACO"].c_str());
                if (!from) error("cann't open input file : ", control["ACO"]);
                ofstream to((current+control["proteinName"]+"_dihe.tbl").c_str(), std::ios::out|std::ios::app);
                if (!to) error("cannot open output file : ", current+control["proteinName"]+"_dihe.tbl");
                to.setf(ios::fixed, ios::floatfield);
                to.precision(1);
                to << "!Manual Input\n";
                string s;
                while (getline(from, s))
                {
                        istringstream ss(s);
                        vector<string> subline; subline.clear();
                        split(s, subline);

                        if ((s[0] == '#') || (subline.size() <5))
                        {
                        }
                        else
                        {
                                int index; string residue; string type; float lb; float ub;
                                ss >> index >> residue >> type >> lb >> ub;

                                bool in = 1;
                                if (type == "PHI")
                                {
                                        to << "assign (resid\t" << index-1 << " and name C\t) (resid\t" << index << " and name N\t)\n" <<
                                                "       (resid\t" << index   << " and name CA\t) (resid\t" << index << " and name C\t)" << "\t1.0\t";
                                }
                                else if (type == "PSI")
                                {
                                        to << "assign (resid\t" << index << " and name N\t) (resid\t" << index << " and name CA\t)\n" <<
                                                "       (resid\t" << index   << " and name C\t) (resid\t" << index+1 << " and name N\t)" << "\t1.0\t";
                                }
                                else if (type == "CHI1")
                                {
                                        to << "assign (resid\t" << index << " and name N\t) (resid\t" << index << " and name CA\t)\n" <<
                                                "       (resid\t" << index   << " and name CB\t) (resid\t" << index << " and name CG\t)" << "\t1.0\t";

                                } else if (type == "CHI2")
                                {
                                        to << "assign (resid\t" << index << " and name CA\t) (resid\t" << index << " and name CB\t)\n" <<
                                                "       (resid\t" << index   << " and name CG\t) (resid\t" << index << " and name CD\t)" << "\t1.0\t";
                                } else
                                {
                                        in = 0;
                                }

                                if (in == 1)
                                        to << (float) (ub+lb)/2.0 << "\t" << (float) (ub-lb)/2.0 << "\t2" << endl;
                        }
                }
                to << "!END Manual Input" << endl;
                to.close();
        }
}

void NOESY_Assign::runRefine(string current)
{
        if (_DianaBuffer.size() || _ICDianaBuffer.size())
        {
                // print XPLOR constraints
                erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Writing XPLOR distances ...");
                string XPLORFile;
                XPLORFile = current + control["proteinName"] + "_noe.tbl";
                ofstream XPLORStream(XPLORFile.c_str());
                if (!XPLORStream) error("cannot open output file", XPLORFile);
                // upl from noesy_assign
                prtXPLORbuffer(XPLORStream, seq, LLength, _DianaBuffer, _ICDianaBuffer, atoi(control["center"].c_str()));
                XPLORStream.close();
        }

        // XPLOR hbond

        if (HBONDList.size())
        {
                erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Writing XPLOR Hbonds ...");
                ofstream hbondStreamXpl((current+control["proteinName"]+"_hbond.tbl").c_str()); // upl
                if (!hbondStreamXpl) error("cannot open output file : ", current+control["proteinName"] + "_hbond.tbl");
                prtXPLORHbond(hbondStreamXpl, _par.hbond.upperOH, _par.hbond.upperON, seq, LLength, HBONDList);
        }

        if (control["HBOND"] != "") // manual input of hbond
        {
                ifstream from(control["HBOND"].c_str());
                if (!from) error("cann't open input file : ", control["HBOND"]);

                ofstream to((current+control["proteinName"]+"_hbond.tbl").c_str(), std::ios::out|std::ios::app);
                if (!to) error("cannot open output file : ", current+control["proteinName"]+"_hbond.tbl");

                set<HBOND, less<HBOND> > ManuHBONDList; ManuHBONDList.clear();
                string hbond, a1, a2;
                int i1, i2;
                while(from >> hbond >> a1 >> i1 >> a2 >> i2)
                {
                        if ((a1 == "O") && (a2 == "HN"))
                                ManuHBONDList.insert(make_pair(i1,i2));
                        else if ((a1 == "HN") && (a2 == "O"))
                                ManuHBONDList.insert(make_pair(i2,i1));
                        else
                        {
                                error("**Error: Bad format for hbond file : ", control["HBOND"].c_str());
                                error("  " + hbond + " " + a1 + " " + itoa(i1) + " " + a2 + " " + itoa(i2), "");
                        }
                }

                to << endl << "!Manual Input\n";
                prtXPLORHbond(to, _par.hbond.upperOH, _par.hbond.upperON, seq, LLength, ManuHBONDList);
                to << "!END Manual Input" << endl;
                to.close();
        }

        // need to convert the format
        if ((command.xplor != "") || (command.cns != ""))
        {
                if (control["XPL"] != "") // manual input of INPUT
                {
                        ifstream from(control["XPL"].c_str());
                        if (!from) error("cann't open input file : ", control["XPL"]);
                        ofstream to((current+control["proteinName"]+"_noe.tbl").c_str(), std::ios::out|std::ios::app);
                        if (!to) error("cannot open output file : ", current+control["proteinName"]+"_noe.tbl");

                        to << "! Manual Input\n";
                        char ch;
                        while (from.get(ch)) to.put(ch);
                        to << "! END Manual Input" << endl;
                }

                //--- write the aco file
                if (control["DIHE"] != "") // addin manual ones
                {
                        ifstream from(control["DIHE"].c_str());
                        if (!from) error("cann't open input file : ", control["DIHE"]);
                        ofstream to((current+control["proteinName"]+"_dihe.tbl").c_str(), std::ios::out|std::ios::app);
                        if (!to) error("cannot open output file : ", current+control["proteinName"]+"_dihe.tbl");

                        to << "!Manual Input\n";
                        char ch;
                        while (from.get(ch)) to.put(ch);
                        to << "!END Manual Input" << endl;
                        to.close();
                }
        }
        else if ((command.cyana != "")||(command.dyana != ""))
        {
                cyana2xplor(current);
        }
        // end ------

        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Running WaterRefine ...\n");

        //--- exec xplor
        bool suss = 0;
        if (RESTORE)
        {
                ifstream* from = 0;
                ifstream from1((sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + '/' + control["proteinName"] + "_cns.pdb").c_str());
                ifstream from2((sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + "_final/" + control["proteinName"] + "_cns.pdb").c_str());

                if (from1)
                        from = &from1;
                else if (from2)
                        from = &from2;

                if (!from)
                {
                        {
                                suss = 0;
                                if (!from1)
                                        erasePrint(cout, " Cann't find file " + sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + '/' + control["proteinName"] + "_cns.pdb\n");
                                if (!from2)
                                        erasePrint(cout, " Cann't find file " + sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + "_final/" + control["proteinName"] + "_cns.pdb\n");
                        }
                }
                else
                {
                        erasePrint(cout, " Restoring from " + sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + '/' + control["proteinName"] + "_cns.pdb\n");
                        ofstream to((current+control["proteinName"] + "_cns.pdb").c_str());
                        if (!to) error("cannot open output file : ", current+control["proteinName"] + "_cns.pdb");
                        char ch;
                        while (from->get(ch)) to.put(ch);
                        suss = 1;
                        to.close();
                }

                // copy the sourcePdb.ovw into current directory
                ifstream* ofrom = 0;
                ifstream ofrom1((sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + '/' +"HydrogenBondData.txt").c_str());
                ifstream ofrom2((sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + "_final/" +"HydrogenBondData.txt").c_str());

                if (ofrom1)
                        ofrom = &ofrom1;
                else if (ofrom2)
                        ofrom = &ofrom2;

                if (!ofrom)
                {
                        erasePrint(cout, " Cann't find file " + sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + '/' +"HydrogenBondData.txt");
                }
                else
                {
                        erasePrint(cout, " Restoring from " + sourcePdbPath + "CYCLE" + itoa(_ddcycle) + '-' + itoa(_dcycle) + '/' +"HydrogenBondData.txt");
                        ofstream to((current+"HydrogenBondData.txt").c_str());
                        if (!to) error("cannot open output file : ", current+"HydrogenBondData.txt");
                        char ch;
                        while (ofrom->get(ch)) to.put(ch);
                        to.close();
                }
        }

        if ((!RESTORE) || (!suss))
        {
                //rdc
                /*
                  if (control["RDC"] != "") // manual input of RDC
                  {
                  ifstream from(control["RDC"].c_str());
                  if (!from) error("cann't open input file : ", control["RDC"]);
                  ofstream to((current+control["proteinName"]+"_sani.tbl").c_str());
                  if (!to) error("cannot open output file : ", current+control["proteinName"]+"_sani.tbl");

                  char ch;
                  while (from.get(ch)) to.put(ch);
                  }
                  if (control["RDCaxis"] != "") // manual input of RDC
                  {
                  ifstream from(control["RDCaxis"].c_str());
                  if (!from) error("cann't open input file : ", control["RDCaxis"]);
                  ofstream to((current+"axis.pdb").c_str());
                  if (!to) error("cannot open output file : ", current+"axis.pdb");

                  char ch;
                  while (from.get(ch)) to.put(ch);
                  }
                */

                if (command.refine != "")  // if have defined the refine command
                {
                        int dpid;
                        if ((dpid = fork()) == -1)
                                error("Cann't fork", "");
                        else if (dpid == 0)
                        {
                                if (chdir(current.c_str()) == -1)
                                        error("**cannot move to directory", current);

                                ofstream shcmd("run-refine-sh");
                                if (!shcmd) error("cannot open output file", "run-refine-sh in " + current);

                                shcmd << "#!/bin/sh" << endl;
                                shcmd << command.refine;

                                if (CISPeptideBond.size())
                                {
                                        shcmd << " -cis ";
                                        for(set<int, less<int> >::iterator i = CISPeptideBond.begin(); i != CISPeptideBond.end(); ++i)
                                        {
                                                if (i != CISPeptideBond.begin())
                                                        shcmd << ",";
                                                shcmd << *i;
                                        }
                                }
                                if (SSBond.size())
                                {
                                        shcmd << " -ss ";
                                        for(set<pair<int, int>, less<pair<int, int> > >::iterator i = SSBond.begin(); i != SSBond.end(); ++i)
                                        {
                                                if (i != SSBond.begin())
                                                        shcmd << ",";
                                                shcmd << i->first << "-" << i->second;
                                        }
                                }

                                shcmd << " -st " << seq.begin()->first << " -na " << control["proteinName"] << " -pr cns "<< " -cf " << controlFileName << endl;

                                if (execlp("sh", "sh", "run-refine-sh", (char*)0) < 0)
                                        error("**Can not run Xplor", "Check run-refine-sh at WorkingCycle");
                        }

                        int qstat;
                        waitpid(dpid, &qstat, 0);
                }
        }
}
void NOESY_Assign::runVioPdb()
{
        int ovwP = _outputOvw.tellp();
        int newAssign = 1;
        int unAssign = 0;
        float psold = 0, pscore = 0;
        int ddcycleold;
        int dcycleold;
        bool final= 0;

        //add to the EC list
      if (isEC())
        {
          findECfromSS(_secondaryStruct, _antiParaRegister, _paraRegister, ECList);
        }

        /*
          MomDadSet mdSetM2; mdSetM2.clear();
          MomDadPeakAtomMap mdPAMap2; mdPAMap2.clear();
          SUPPORT support; support.clear();

          runBuildProfile(mdSetM2, mdPAMap2, support); // rebuild
        */

        do   // iterative make assignments
        {
                if (final) // finish the whole calculation
                {
                        break;
                }



                // for minimum constraint analysis
                runMulti();

                ////////////////////////////
                // Write Ovw Report       //
                ////////////////////////////
                _outputOvw.seekp(ovwP);
                _outputOvw << "Summary" << endl;
                _outputOvw << "=======" << endl;
                for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
                {
                        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+ itoa(_dcycle)+ " : prtOvw for " + (*iter)->label() + " ... ");
                        _outputOvw << endl << endl;
                        _outputOvw << "PeakList " << (*iter)->label() << ":" << endl << endl;
                        (*iter)->prtProcessStat(_outputOvw);
                        (*iter)->prtAssignStat(_outputOvw);
                        (*iter)->prtPeakStat(_outputOvw);
                        _outputOvw << endl << endl;
                }

                /////////////////////////////////
                // Write Cycle--_dcycle Report   //
                /////////////////////////////////
                string current = (outputDir + "/WorkingCycle" + itoa(_ddcycle)+'-'+itoa(_dcycle)+'/');
                mkdir(current.c_str(), 0750);

                /*
                  string assignFile = "NAssign";

                  ofstream rassignStream((current+assignFile+".gen").c_str());
                  if (!rassignStream) error("cannot open output file", current+assignFile+".gen");
                  prtAssign(rassignStream);
                  rassignStream.close();
                */

                for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
                {
                        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : prtSource for " + (*iter)->label() + " ... ");
                        ofstream rsrcStream((current+(*iter)->label()+"_assSparky").c_str());
                        if (!rsrcStream) error("cannot open output file", current+(*iter)->label()+"_assSparky");
                        (*iter)->prtSource(rsrcStream);
                        rsrcStream.close();
                }

                for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
                {
                        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : prtAssignList for " + (*iter)->label() + " ... ");
                        ofstream rsrcStream((current+(*iter)->label()+"_assOrder").c_str());
                        if (!rsrcStream) error("cannot open output file", current+(*iter)->label()+"_assOrder");
                        (*iter)->prtAssignList(rsrcStream);
                        rsrcStream.close();
                }


                for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
                {
                        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : prtZeroAssign for " + (*iter)->label() + " ... ");
                        ofstream rsrcStream((current+(*iter)->label()+"_zeroAss").c_str());
                        if (!rsrcStream) error("cannot open output file", current+(*iter)->label()+"_assOrder");
                        (*iter)->prtZeroAssignList(rsrcStream);
                        rsrcStream.close();
                }

                {
                        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : prtRefined_Shift ...");
                        ofstream rsrcStream((current+"RefinedShift.bmrb").c_str());
                        if (!rsrcStream) error("cannot open output file", current+"RefinedShift.bmrb");
                        prtRefinedShiftList(rsrcStream, _atoms);
                        rsrcStream.close();
                }

                acfMax.clear();
                for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
                {
                        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : buildAcfMax for " + (*iter)->label() + " ... ");
                        (*iter)->buildAcfMax(acfMax);
                }

                //--- comparison of the simlated data


                /////////////////////////////////////
                //           Calc Structures              //
                ////////////////////////////////////

                // Calibrate
                _DianaBuffer.clear();
                _ICDianaBuffer.clear();

                _DianaBufferDeque.clear();
                _ICDianaBufferDeque.clear();

                int id = 0;

                stringstream ambiStream; ambiStream.clear();

                for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
                {
                        erasePrint(cout, " Cycle " + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Calibrating for " + (*iter)->label() + " ... ");
                        float kd = (*iter)->Calibrate(); // kd is the average distance of bb atoms

                        if (DEBUG) NOESY_Assign::outputLog() << itoa(_ddcycle) << " kd0 " << kd << (*iter)->label() << endl;

                        int recali = 0; // only allow recali 20 times?
                        while ((abs(floor(kd*100+0.5)/100.0 - NOESY_Assign::KD()) > 0.05) && (recali < 20))
                        {
                                recali++;
                                (*iter)->setKCON(pow(NOESY_Assign::KD()/kd, 6)); // set average
                                kd = (*iter)->Calibrate(); // with new KCON
                                if (DEBUG) NOESY_Assign::outputLog() << "kd1 " << kd << (*iter)->label() << endl;
                        }
                        if (abs(floor(kd*100+0.5)/100.0 - NOESY_Assign::KD()) > 0.05)
                        {
                                cout << "Warning - calibrated average distance " << kd << "  is different from the input -k " << NOESY_Assign::KD() << endl;
                        }

                        if (NOESY_Assign::isAMBI())
                                (*iter)->CalibrateAmbi(ambiStream, LLength, _par.sym); // not used anymore, use pdb information, not for AmbiCASP
                }


                //print AMBICASP with calibrated distance
                for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
                {
                        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : prtAmbCASPList for " + (*iter)->label() + " ... ");
                        ofstream rsrcStream((outputDir + "/" +(*iter)->label()+"_AmbiCSA").c_str());
                        ofstream rsrcStreamRaw((outputDir + "/" +(*iter)->label()+"_AmbiRaw").c_str());

                        rsrcStream.setf(ios::fixed, ios::floatfield);
                        rsrcStream.precision(1);

                        rsrcStreamRaw.setf(ios::fixed, ios::floatfield);
                        rsrcStreamRaw.precision(1);

                        if (!rsrcStream) error("cannot open output file", current+(*iter)->label()+"_AmbiCSA");
                        if (!rsrcStreamRaw) error("cannot open output file", current+(*iter)->label()+"_AmbiRaw");
                        (*iter)->prtAmbiCASP(rsrcStreamRaw, rsrcStream, _DianaBuffer);
                        rsrcStream.close();
                        rsrcStreamRaw.close();
                }

                //--- ovw, text for gnuplot

                /////////////
                // HYPER   //
                /////////////
                //--- write the hyper input file
                string method = "";

                runHyper(current);

                /*
//////////////////////
//--- exec diana    //
//////////////////////
if (command.dyana !="")
{
runDyana(current);
}

                */


                         prtCyanaCons(current, ambiStream); // include EClist
                         prtXplorCons(current);

                         //////////////////////
                         //--- exec cyana    //
                         //////////////////////
                         if (command.cyana !="")
                         {
                                 runCyana(current, ambiStream);
                                 method = "cyana";
                         }

                         /*
                ///////////
                //  CNS  //
                ///////////

                else if (command.cns !="")
                {
                runCNS(current);
                method = "cns";
                }
                         */

                         ///////////////////////////
                         //   XPLOR               //
                         ///////////////////////////
                         else if (command.xplor !="")
                         {
                                 runXplor(current);
                                 method = "xplor";
                         }


                         /////////////////////////////
                         // read in PDB, ovw files      //
                         /////////////////////////////

                         // need to comment out to use RmaxQ
                         //for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
                         //  {
                         //    (*iter)->resetRmax();
                         //  }

                         //string coorFile = current + control["proteinName"] + "_" + method + ".pdb"; // the non-refinment one first
                         string coorFile = current + control["proteinName"] + ".pdb"; // the non-refinment one first
                         readPdb(coorFile);


                         //////////////////////
                         // Validating       //
                         //////////////////////

                         // check if have unconsistent and strongly violated assignment
                         // if have, unassign and go to rerun the structure calculation
                         usvv = 0;
                         for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
                         {
                                 //if (!isEC())
                                 {
                                         erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Doing strongValidate for " + (*iter)->label() + " ...");
                                         usvv += (*iter)->strongValidate(_par.vio, _outputUA, atoi(control["center"].c_str()));
                                 }
/*
  if (isEC())
  {
  erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Doing validateEC for " + (*iter)->label() + " ...");
  usvv += validateEC(current+control["proteinName"]+".ovw", 1, ECList, _atoms, 5.0+vio[_ddcycle], 5.0+vio2[_ddcycle]);
  }
*/
                         }

                         erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Doing validateEC  ...");
                         if (isAMBI())
                                 usvv += adjustEC(current+control["proteinName"]+".ovw", 1, ECList, _atoms, vio[_dcycle], vio2[_dcycle]);
                         bool isRefine = 0;
                         cout << _ddcycle << " " << atoi(control["nCycles"].c_str()) -1 << endl;
                         if (((usvv == 0) || (_dcycle == 2)) && (_ddcycle >=atoi(control["nCycles"].c_str()) -1) && !isEC() && (command.refine != ""))// run refinement for the last 2 cycles
                         {
                                 //cout << "in refinement\n";
                                 isRefine = 1;
                                 runRefine(current);
                                 coorFile = current + control["proteinName"] + "_cns.pdb";
                                 readPdb(coorFile);
                                 //validation agaist the cns structure
                                 for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
                                 {
                                         erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Doing strongValidate for " + (*iter)->label() + " ...");
                                         usvv += (*iter)->strongValidate(_par.vio, _outputUA, atoi(control["center"].c_str()));
                                 }
                         }

                         if (DEBUG)
                         {
                                 // printing the match file w/ reasoning
                                 for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
                                 {
                                         erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : prtSimleMatchPeakAtom for " + (*iter)->label() + " ... ");
                                         ofstream matchStream((current + (*iter)->label() + "_match").c_str());
                                         if (!matchStream) error("cannot open output file", current + (*iter)->label() + "_match");
                                         (*iter)->prtSimpleMatchPeakAtom(matchStream);
                                         matchStream.close();
                                 }

                                 system(("gzip -r " + current + "*_match").c_str());
                         }


                         //--------------------------------------------------------------//
                         //                                                              //
                         // Start to Run NOESY_Assign, Cycle: _dcycle+1                   //
                         //--------------------------------------------------------------//

                         //////////////////////////////////////////////
                         //--- exec mv copy all resuts to xx cycle              //
                         //////////////////////////////////////////////
                         if (rename(current.c_str(),(outputDir + "/CYCLE" + itoa(_ddcycle)+'-'+itoa(_dcycle)).c_str()) < 0)
                         {
                                 error("*** Can't mv " + current + " to " + outputDir + "/CYCLE" + itoa(_ddcycle)+'-'+itoa(_dcycle), " : Please check");
                         }

                         //////////////////////////////////////////////
                         //--- exec viewer make it background running//
                         //////////////////////////////////////////////
                         if (command.viewer != "")
                         {
                                 signal(SIGCHLD, SIG_IGN);
                                 int rpid;
                                 if ((rpid = fork()) == -1)
                                         error("Cann't fork", "");
                                 else if (rpid == 0)
                                 { // background running
                                         // second child , have no control terminal, not a process group leader
                                         if (chdir((outputDir + "/CYCLE" + itoa(_ddcycle)+'-'+itoa(_dcycle)).c_str()) < 0)
                                                 error("*** Can't chang directory to ", " : " + outputDir + "/CYCLE" + itoa(_ddcycle)+'-'+itoa(_dcycle));
                                         if (execlp("/bin/sh", "sh", "-c", (command.viewer + " " + control["proteinName"]).c_str(), (char*)0) < 0)
                                                 error("**Cannot not run View", "Check the viewerCommand");
                                 }
                         }

                         _dcycle++;

                         //////////////////////
                         // Validating       //
                         //////////////////////

                         // check if have unconsistent and strongly violated assignment
                         // if have, unassign and go to rerun the structure calculation

                         if (_ddcycle != 1) // not the first cycle
                         {
                                 string dirName = outputDir + "/CYCLE" + itoa(_ddcycle)+'-'+itoa(_dcycle-1)+"/";

                                 {
                                         if ((command.refine !="") && isRefine)
                                                 findHbond(usvv, dirName+"HydrogenBondData.txt", 0, _secondaryStruct, _antiParaRegister, _paraRegister, NHList, acfMax, HBONDList);
                                         else if (command.xplor !="")
                                                 findHbond(usvv, dirName+"HydrogenBondData.txt", 0, _secondaryStruct, _antiParaRegister, _paraRegister, NHList, acfMax, HBONDList);
                                         else if ((command.dyana !="") || (command.cyana != ""))
                                                 findHbond(usvv, dirName+control["proteinName"]+".ovw", 1, _secondaryStruct, _antiParaRegister, _paraRegister, NHList, acfMax, HBONDList);
                                 }
                         }

                         if (((usvv == 0) && (!isAMBI())) || ((_dcycle == 3) && (!isAMBI())) || (isEC() && !isAMBI()) || (isAMBI() && (_dcycle == 3))) // no violation or unassigned ? cycles
                         {
                                 ////////////////////////////////////////////////////////////////////
                                 // if no unconsistent and strongly violated assignment,           //
                                 // make more assignments                                          //
                                 ////////////////////////////////////////////////////////////////////


                                 // move the last cycle to final
                                 if (rename((outputDir + "/CYCLE" + itoa(_ddcycle)+'-'+itoa(_dcycle-1)).c_str(),(outputDir + "/CYCLE" + itoa(_ddcycle)+'-'+itoa(_dcycle-1)+"_final").c_str() ) < 0)
                                 {
                                         error("*** Can't mv " +outputDir + "/CYCLE" + itoa(_ddcycle)+'-'+itoa(_dcycle-1) + " to " + outputDir + "/CYCLE" +'-'+itoa(_dcycle)+itoa(_ddcycle-1)+"_final", " : Please check");
                                 }

                                 _ddcycle++; int olddcycle = _dcycle;
                                 _dcycle = 0;
                                 if ((_ddcycle > atoi(control["nCycles"].c_str())) || ((newAssign == 0) && (unAssign == 0)) || (pscore < psold))
                                 {
                                         final = 1;
                                         if (command.refine == "")
                                         {
                                                 // no refine - print the constraints for manual refinement
                                                 string cu = outputDir + "/CYCLE" + itoa(_ddcycle-1)+'-'+itoa(olddcycle-1)+"_final/";
                                                 runRefine(cu);
                                         }
                                         continue;
                                 }

                                 // a new cycle
                                 newAssign = 0;
                                 unAssign = 0;

                                 ddcycleold = _ddcycle;
                                 dcycleold = _dcycle;
                                 psold = pscore;

                                 runRefine_Shift();

                                 MomDadSet mdSetM2;
                                 mdSetM2.clear();
                                 MomDadPeakAtomMap mdPAMap2;
                                 mdPAMap2.clear();
                                 SUPPORT support;
                                 support.clear();

                                 runBuildProfile(mdSetM2, mdPAMap2, support); // rebuild, as new more peaks have assignments
                                 //newAssign += runAssignPDB(mdSetM2, mdPAMap2, support); // add pdb profile and assigned by pdb
                                 if (isEC())
                                 {
                                         string filename =
                                                 outputDir + "/CYCLE" + itoa(_ddcycle-1) + '-' + itoa(olddcycle-1) + "_final/" + control["proteinName"]+".ovw";
                                         //if (_ddcycle <= 3) filename = "";
                                         //string filename = "";

                                         validateEC(filename, 1, ECList, _atoms, 0.1, 0.5);
                                         newAssign += findEC(ECList, seq, _atoms);
                                         //newAssign += runAssignPDB(mdSetM2, mdPAMap2, support); // add pdb profile and assigned by pdb
                                 }

                                 {
                                         newAssign += runAssignPDB(mdSetM2, mdPAMap2, support); // add pdb profile and assigned by pdb
                                 }

                                 if ((command.hyper != "") && (_ddcycle > 2)) // at cycle5-0, start to update stereospecific assignments
                                 {
                                         ifstream* stereoFile; stereoFile = 0;

                                         ifstream stereoFile1((outputDir + "/CYCLE" + itoa(_ddcycle-1)+'-'+itoa(olddcycle-1) + "/StereoAssignedInfoFile").c_str());
                                         ifstream stereoFile2((outputDir + "/CYCLE" + itoa(_ddcycle-1)+'-'+itoa(olddcycle-1) + "_final/StereoAssignedInfoFile").c_str());

                                         if (!stereoFile1)
                                                 stereoFile = &stereoFile1;
                                         else
                                                 stereoFile = &stereoFile2;

                                         if (!stereoFile) error("cannot open input file", outputDir + "/CYCLE" + itoa(_ddcycle-1)+'-'+itoa(olddcycle-1) + "/StereoAssignedInfoFile");
                                         erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Reading StereoAssignment Files ...");

                                         string line;
                                         while (getlineNA(*stereoFile, line))
                                         {
                                                 vector<string> subline; subline.clear();
                                                 split(line, subline, " \t\n\v");

                                                 // update the resonance assignments
                                                 stereoAssign(subline);
                                         }
                                 }
                         }
        } while (1);

        if (newAssign == 0)
                _outputOvw << "No more assignment can be made. The program is finished." << endl;
        else
                _outputOvw << control["nCycles"] << " assignment cycles have been calculated." << endl;

        if (pscore < psold)
                _outputOvw << "Best Performance Cycle : "       <<              ddcycleold << "-" << dcycleold << " with score: " << psold << endl;

}

void NOESY_Assign::stereoAssign(const vector<string>& subline)
{
        if (subline.size() == 4)
        {
                // if the interchain, ignore
                if (atoi(subline[1].c_str()) > seq.rbegin()->first)
                        return;

                set<atom*, less<atom*> > temp; temp.clear();
                for(Atom_List::iterator iter = _atoms.begin(); iter != _atoms.end(); ++iter)
                {
                        if ((iter->get_index() == atoi(subline[1].c_str())) && (iter->isH()) &&
                            (iter->get_sym_1() == symbol3_1[subline[0]]))
                        {
                                if (iter->get_layer() == layer_map[iter->get_sym_1()][subline[2]])
                                        temp.insert(&(*iter));
                        }
                }

                if (temp.size() != 2)
                        error("Cannot make StereoAssignments for", subline[0]+subline[1]+subline[2]+" to " + subline[3]);
                if (subline[2] != subline[3])
                {
                        _outputError << endl << "Swapping\n" << *(temp.begin()) << "\n" << *(temp.rbegin()) << endl;

                        string t = (*temp.begin())->get_name();
                        (*temp.begin())->set_name((*temp.rbegin())->get_name());
                        (*temp.rbegin())->set_name(t);
                }

                if ((*temp.begin())->get_ambi() != 1)
                {
                        (*temp.begin())->set_ambi(1);
                        (*temp.rbegin())->set_ambi(1);
                        _outputError << "StereoAssignment in Cycle" << NOESY_Assign::ddcycle() << "-" << NOESY_Assign::dcycle() << ":\n" << *(temp.begin()) << "\n" << *(temp.rbegin()) << endl << endl;
                }
        }
}

int NOESY_Assign::runAssignPDB(MomDadSet& mdSetM2, MomDadPeakAtomMap& mdPAMap2, SUPPORT& support)
{
        int total = 0;

        // removde the one not supported by PDB
        map<NOESY*, mdsMap> mdsMapMap; mdsMapMap.clear();
        map<NOESY*, peakmdMap> peakmdMapMap; peakmdMapMap.clear();

        SUPPORT pdbSupport; pdbSupport.clear();
        runContactFaces();

        for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
                if (((*iter)->Type().find("IC") >= (*iter)->Type().length()))
                {
                        mdsMap mdsd; mdsd.clear();
                        peakmdMap peakmd; peakmd.clear();

                        // add pdb profile for BEST method
                        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : AddPDBProfile for " + (*iter)->label() + " ... ");
                        (*iter)->AddPDBProfile(mdPAMap2, pdbSupport, mdsd, peakmd);
                        mdsMapMap.insert(make_pair(*iter, mdsd));
                        peakmdMapMap.insert(make_pair(*iter, peakmd));
                }
        if (DEBUG)
        {
                for(MomDadSet::iterator i = pdbSupport.mdSetM.begin(); i != pdbSupport.mdSetM.end(); ++i)
                {
                        NOESY_Assign::outputLog() << "Intra PDB group " << sameHydro(i->first->get_sym_1(), i->second->get_sym_1()) << " " << i->first->get_index() << " " << i->second->get_index() << endl;

                        for(MomDadPeakAtomMap::iterator j = pdbSupport.mdPAMapM.lower_bound(*i); j != pdbSupport.mdPAMapM.upper_bound(*i); ++j)
                                NOESY_Assign::outputLog() << j->second.first << endl << j->second.second.first << endl << j->second.second.second << endl;
                }
        }

        // SYM method
        MomDadSet mds; mds.clear();
        SUPPORT usupport; // = support;
        ContactPairSet cs; cs.clear();

        // used by SYM method
        MomDadSet mdSetA12; mdSetA12.clear();
        MomDadPeakAtomMap mdPAMapA; mdPAMapA.clear();
        map<MomDad, MomDad, less<MomDad> > mdAM2; mdAM2.clear();

        Find_USYM(_par.sym, mdPAMap2, support, mdPAMapA, mdAM2, mdSetA12);

        for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
                if (((*iter)->Type().find("IC") >= (*iter)->Type().length()))
                {
                        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Assign_SYM for " + (*iter)->label() + " ... ");
                        total += (*iter)->Assign_SYM(-1, _par.sym, mdPAMapA, mdAM2,
                                                     mdSetA12, mdSetM2, mdPAMap2, support);
                }

        /////////////////////////////////////
        //                                 //
        // createIntra chain ContactSYMBin //
        /////////////////////////////////////

        if ((NOESY_Assign::ddcycle() >=3) || isEC() || isINITF())
        {
                for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
                        if (((*iter)->Type().find("IC") >= (*iter)->Type().length()))
                        {
                                erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : AssignGood_Pdb for " + (*iter)->label() + " ... ");
                                total += (*iter)->AssignGood_Pdb(_par.pdb, mdPAMap2, support, mdsMapMap.find(*iter)->second, peakmdMapMap.find(*iter)->second);
                        }

                if (!isEC() && !isINITF())
                {
                        for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
                        {
                                erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Doing Validation for " + (*iter)->label() + " ...");
                                if ((*iter)->Type().find("IC") >= (*iter)->Type().length())
                                {
                                        total += (*iter)->weakValidate(_par.vio, _outputUA, mdsMapMap.find(*iter)->second, peakmdMapMap.find(*iter)->second);
                                }
                        }

                        for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
                                if (((*iter)->Type().find("IC") >= (*iter)->Type().length()) && ((*iter)->Type().substr(0,2) != "4D")) //&&  (!NOESY_Assign::MINCON()))
                                {
                                        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : AssignFar_Pdb for " + (*iter)->label() + " ... ");

                                        total += (*iter)->AssignFar_Pdb(_par.pdb, mdPAMap2, pdbSupport, mdsMapMap.find(*iter)->second, peakmdMapMap.find(*iter)->second);
                                }
                }
        }

        if ((NOESY_Assign::ddcycle() >= 3) && (!NOESY_Assign::MINCON()))
        {
                for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
                {
                        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : findISO for " + (*iter)->label() + " ... ");
                        total += (*iter)->findISO(_par.vio, _outputUA);

                }
        }

        support.insert(pdbSupport);
        return total;
}

// only used for iterative read from structure calulcations
void NOESY_Assign::readPdb(string coorFile)
{
        _pdbList.clear();

        ifstream pdb(coorFile.c_str());
        if (!pdb)
        {
                error("Structural Calculation Stopped: PDB file not generated (See log files in the WorkingCycle for debug)", coorFile.c_str());
        }
        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Reading Pdb Files ...");
        Pdb pdbStruct(0);

        while (pdb)
        {
                pdb >> pdbStruct;
                if (pdbStruct.notEmpty())
                {
                        erasePrint(cout, " Reading ComparePdbFile: " + Qpdb);
                        _pdbList.push_back(pdbStruct);
                }
                pdbStruct.reset();
        }

        if (DEBUG)
        {
                int count = 0;
                for(deque<Pdb>::iterator i = _pdbList.begin(); i != _pdbList.end(); i++)
                {
                        count++;
                        _outputLog << "model" << count << endl;
                        _outputLog << (*i) << endl;
                }
        }
        pdb.close();
}
