/***************************************************************************
                          Report.cpp  -  description
                             -------------------
    begin                : Mon Aug 21 2000
    copyright            : (C) 2000 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/
#include <cmath>
#include <stdlib.h>
#include <iomanip>
#include <sstream>
#include "noesy_assign.h"

extern bool DEBUG;
extern string outputDir;
extern ControlMap control;

void NOESY_Assign::initReportOvw(string controlFileName, const time_t& time_start, int argc, char** argv)
{
		// ovw file
	if (control["proteinName"] == "")
	  error("no proteinName defined", controlFileName);

	if (!isQ()) 
	  {
	    _outputOvw.open((outputDir + "/" + control["proteinName"] + "_AS.ovw").c_str());
	    if (!_outputOvw) error("cannot write to file", outputDir + "/" + control["proteinName"] + "_AS.ovw");
	    _outputOvw << "# created by " << versionHeader() << endl;
	    _outputOvw << "# " << ctime(&time_start) << endl << endl;
	  }

	_outputOvwRPF.open((outputDir + "/" + control["proteinName"] + "_DP.ovw").c_str());
	if (!_outputOvwRPF) error("cannot write to file", outputDir + "/" + control["proteinName"] + "_DP.ovw");
	_outputOvwRPF << "# created by " << versionHeader() << endl;
	_outputOvwRPF << "# " << ctime(&time_start) << endl << endl;

		
	// Error Log
	_outputError.open((outputDir + "/" + control["proteinName"] + ".note").c_str());
	if (!_outputError) error("cannot write to file", outputDir + "/" + control["proteinName"] + ".note");
	_outputError <<"# created by " << versionHeader() << endl << endl;

	if (!isS()) // full report
	  {
	// secondary structure
	_outputSec.open((outputDir + "/" + control["proteinName"] + "_AS.sec").c_str());
	if (!_outputSec) error("cannot write to file", outputDir + "/" + control["proteinName"] + "_AS.sec");
	_outputSec <<"# created by " << versionHeader() << endl << endl;
	
	// examination file
	_outputExm.open((outputDir + "/" + control["proteinName"] + ".exm").c_str());
	if (!_outputExm) error("cannot write to file", outputDir + "/" + control["proteinName"] + ".exm");
	_outputExm <<"# created by " << versionHeader() << endl << endl;

	// unassigned peak list file
	_outputUA.open((outputDir + "/" + control["proteinName"] + "_AS.unassign").c_str());
	if (!_outputUA) error("cannot write to file", outputDir + "/" + control["proteinName"] + "_AS.unassign");
	_outputUA <<"# created by " << versionHeader() << endl << endl;
		
	// log file for debug
	//if (DEBUG)
	{
	_outputLog.open((outputDir + "/" + control["proteinName"] + ".log").c_str());
	if (!_outputLog) error("cannot write to file", outputDir + "/" + control["proteinName"] + ".log");
	_outputLog <<"# created by " << versionHeader() << endl << endl;
	}

	// Rfactor file
	if (!isS())
	{
	  _outputI.open((outputDir + "/" + control["proteinName"] + "_DP.QR").c_str());
	  if (!_outputI) error("cannot write to file", outputDir + "/" + control["proteinName"] + "_DP.QR");
	  _outputI <<"# created by " << versionHeader() << endl << endl;
	  _outputL.open((outputDir + "/" + control["proteinName"] + "_DP.QP").c_str());
	  if (!_outputL) error("cannot write to file", outputDir + "/" + control["proteinName"] + "_DP.QP");
	  _outputL <<"# created by " << versionHeader() << endl << endl;
	  }

	// valiation file
	//if (isV()||isQ())
	if (!isS())
	{
	_outputM.open((outputDir + "/" + control["proteinName"] + "_DP.QM").c_str());
	if (!_outputM) error("cannot write to file", outputDir + "/" + control["proteinName"] + "_DP.QM");
	_outputM <<"# created by " << versionHeader() << endl;
	}
	  }

	_outputOvwRPF << "# Command: ";
	for (int i = 0; i < argc; ++i)
	  {
	    if (argv[i][0]=='-')
	      _outputOvwRPF << endl << "#\t\t";
	    _outputOvwRPF << argv[i] << " ";
	  }
	_outputOvwRPF << endl << endl;

	_outputOvw << "# Command: ";
	for (int i = 0; i < argc; ++i)
	  {
	    if (argv[i][0]=='-')
	      _outputOvw << endl << "#\t\t";
	    _outputOvw << argv[i] << " ";
	  }
	_outputOvw << endl << endl;
	
	_outputOvw << "NALIB Path is " << getenv("ASLIB") << endl;
	_outputOvw << "Using average b.b. distance " <<  _KD << " for calibration." << endl;
	
	/////////////////////////////////////////////////////////
	// Initial some Global Statistic Report                //
	/////////////////////////////////////////////////////////
	
	// Constraint Stat by Cycles
	reportConstraintStat << "Constraint Stat.:" << endl;
	reportConstraintStat << "Cycle Total_Constraints Intraresidue[i=j] Interresidue[(i-j)=1] Interresidue[1<(i<j)<5]  Interresidue[(i-j)>=5]" << endl;
	reportConstraintStat << "                                          --------------------- -----------------------  ---------------------" << endl;
	reportConstraintStat << "                                          TOT   BB    BS    SS  TOT   BB    BS    SS     TOT   BB    BS    SS " << endl;
	
}

void NOESY_Assign::prtAssign(ostream& outStream)
{
  for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
    {
      (*iter)->prtConstraintList(outStream); 
    }
  
}

void NOESY_Assign::prt_cfpdb(deque<Pdb>& pdbList, ostream& out)
{
  matrix cfMax; cfMax.erase(cfMax.begin(), cfMax.end());

  for(Atom_List::iterator miter = _atoms.begin(); miter != _atoms.end(); ++miter)
  {
    if ((*miter).isH() && ((*miter).isatt_C() || (*miter).isatt_N()))
	  {
	    Atom_List::iterator diter; diter = miter;
	    while(diter != _atoms.end())
	    {
	      diter++;
	      if (
		      (*diter).isH() && ((*diter).isatt_C() || (*diter).isatt_N()) && (fabs((*miter).get_shift() - (*diter).get_shift()) > 0.3999999))
		    {
		      // get min distances from all structure, all possible combination
		      SampleStatistic distStat; distStat.reset();
		      for(deque<Pdb>::iterator iterp = pdbList.begin(); iterp != pdbList.end(); iterp++)
		      {
		        float dist = (*iterp).get_dist(NOESY_Assign::outputError(), miter->get_index(), miter->get_sym_1(), miter->get_name(), miter->get_ambi(),
		      																diter->get_index(), diter->get_sym_1(), diter->get_name(), diter->get_ambi());
		        distStat += dist;
		      }

		      if ((distStat.min() > 0.0000001) && (distStat.mean() < 5.0)) // && (distStat.max() < 7.0))
		      {
  			    int l, s;
	  		    if (miter->get_index() > diter->get_index())
			      {
				      l = miter->get_index();
				      s = diter->get_index();
			      }
			    else
			      {
				      l = diter->get_index();
				      s = miter->get_index();
			      }
			
			      matrix::iterator i = cfMax.find(pair<int,int>(s, l));
			      if (i != cfMax.end())
			        (*i).second++;
			      else
			        cfMax.insert(pair<pair<int, int>, int>(pair<int, int>(s,l), 1));
			    }
			   }
		   }
		}
  }

	out.setf(ios::right, ios::adjustfield);
	out << "   ";
	for(SEQM::const_iterator iter = seq.begin(); iter != seq.end(); ++iter)
    out << setw(3) << (*iter).first << " ";
  out << endl;
	for(SEQM::const_iterator iter = seq.begin(); iter != seq.end(); ++iter)
	  {
	    out << setw(3) << (*iter).first << " ";
	    for(SEQM::const_iterator tt = seq.begin(); tt != iter; tt++)
	      out << setw(4) << "0 ";
	    for(SEQM::const_iterator jter = iter; jter != seq.end(); ++jter)
	    {
	      out << setw(3) << cfMax[pair<int, int>((*iter).first, (*jter).first)] << " ";
	    }
	    out << endl;
	  }
}

void NOESY_Assign::prtNoisePeak()
{  	
  // reprint out NOISE peaks identified
  string reportFile = outputDir + "/NAReport";
	for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
	{
	  ofstream noiseStream((outputDir+"/"+(*iter)->label()+"_noise").c_str());
	  if (!noiseStream) error("cannot open output file : ", outputDir +"/"+(*iter)->label()+"_noise");
	  (*iter)->prtNoise(noiseStream);
	  noiseStream.close();
  }
}

