/***************************************************************************
                          SmplStat.cpp  -  description
                             -------------------
    begin                : Mon Aug 21 2000
    copyright            : (C) 2000 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#include <cmath>
#include <vector>
#include "SmplStat.h"

extern bool DEBUG;

SampleStatistic::SampleStatistic(double t)
{
  reset();
  this->operator+=(t);
}

SampleStatistic:: SampleStatistic(){ reset();}
SampleStatistic::~SampleStatistic() {}
int SampleStatistic::  count() const {return(n);}
double SampleStatistic:: min() const {return(minValue);}
double SampleStatistic:: max() const {return(maxValue);}
double SampleStatistic:: mean() const {return(x);} // mean 
double SampleStatistic:: stdDev() const {return nstdDev;} //stdDev

const SmplData& SampleStatistic::samples() const
{
	return mdataSet;
}

const SmplVect& SampleStatistic::oriSamples() const
{
	return mdata;
}

double SampleStatistic::median() const  // midrange
{
	return (min()+max())/2.0;
	
//	vector<double> temp; temp.clear();
//	for(SmplData::iterator i = mdata.begin(); i != mdata.end(); ++i)
//	{
//		temp.push_back(*i);
//	}
//	//return ((temp[0]+temp[n-1])/2.0);
//	return ((temp[(int)ceil((n+1)/4)-1]+temp[n-(int)floor((n+1)/4)-1])/2);
//	//return (temp[(int)ceil((n+1)/2)]);
//	//return (temp[(int)ceil((n+1)/2)]+temp[(int)floor((n+1)/2)])/2.0;
}


double SampleStatistic::median2() const  // median
{

	vector<double> temp; temp.clear();
	for(SmplData::iterator i = mdataSet.begin(); i != mdataSet.end(); ++i)
	{
		temp.push_back(*i);
	}
	return ((temp[(int)ceil((n+1)/4)-1]+temp[n-(int)floor((n+1)/4)-1])/2);
}



void SampleStatistic::reset()
{
  n = 0; 
  x = x2 = nstdDev = 0; 
  maxValue = -1000000;
  minValue = 1000000;
  mdata.clear();
  mdataSet.clear();
}

void SampleStatistic::operator+=(double sample)
{
  if (!n)
  {
    minValue = sample;
    maxValue = sample;
  }
  if (sample < minValue) minValue = sample; 
  if (sample > maxValue) maxValue = sample; 

  mdata.push_back(sample); 
	mdataSet.insert(sample);
	
  float diff = sample - x;
  x += diff/(double)(n+1);
  x2 += diff * (sample - x);
  if (n) nstdDev = sqrt(x2/(n));
  else nstdDev = sqrt(x2);
  n++; 
}
