/***************************************************************************
                          atom.cpp  -  description
                             -------------------
    begin                : Mon Aug 21 2000
    copyright            : (C) 2000 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#include <iostream>
#include <cstdio>
#include <string>
#include <cmath>
#include "atom.h"
#include "AA.h"
#include "ss.h"
#include "noesy_assign.h"

#define abs(x) abs((double)(x))

extern bool DEBUG;
void error(char*, char*);
void error(string, string);

extern map<char, map<string, string, less<string> >, less<char> > layer_map;

bool operator!=(const atom& a, const atom&b)
{
  if ((a < b) || (b < a))
    return 1;
  else
    return 0;
}


bool operator==(const atom& a, const atom& b)
{
  if ((a.get_type() == b.get_type()) &&
      (a.get_index() == b.get_index()) &&
      (a.get_name() == b.get_name()) &&
      (a.get_shift() == b.get_shift()))
    return 1;
  else
    return 0;
}

bool operator<(const atom& a, const atom& b)
{
	if (a.get_index() < b.get_index()) return 1; 
	else if (a.get_index() == b.get_index()) 
	{	
		if (a.get_type() != b.get_type())
		{
			if (a.get_type() == 'N') return 1; 
			if (b.get_type() == 'N') return 0; 
			if (a.get_type() == 'C') return 1; 
			if (b.get_type() == 'C') return 0; 
			else 
			{
				return 1; 
			}
		}

		if (a.get_layer() == b.get_layer())  return 0;

		if (a.get_layer() == "N") return 1; 
		if (b.get_layer() == "N") return 0;
		if (a.get_layer() == "A") return 1; 
		if (b.get_layer() == "A") return 0;
		if (a.get_layer() == "B") return 1; 
		if (b.get_layer() == "B") return 0;
		if (a.get_layer() == "G") return 1; 
		if (b.get_layer() == "G") return 0;
		if (a.get_layer() == "G1") return 1; 
		if (b.get_layer() == "G1") return 0;
		if (a.get_layer() == "G2") return 1; 
		if (b.get_layer() == "G2") return 0;
		if (a.get_layer() == "D") return 1; 
		if (b.get_layer() == "D") return 0;
		if (a.get_layer() == "D1") return 1; 
		if (b.get_layer() == "D1") return 0;
		if (a.get_layer() == "D2") return 1; 
		if (b.get_layer() == "D2") return 0;
		if (a.get_layer() == "E") return 1; 
		if (b.get_layer() == "E") return 0;
		if (a.get_layer() == "E1") return 1; 
		if (b.get_layer() == "E1") return 0;
		if (a.get_layer() == "E2") return 1; 
		if (b.get_layer() == "E2") return 0;
		if (a.get_layer() == "Z") return 1; 
		if (b.get_layer() == "Z") return 0;
		if (a.get_layer() == "Z1") return 1; 
		if (b.get_layer() == "Z1") return 0;
		if (a.get_layer() == "Z2") return 1; 
		if (b.get_layer() == "Z2") return 0;
		if (a.get_layer() == "H") return 1; 
		if (b.get_layer() == "H") return 0;
		if (a.get_layer() == "H1") return 1; 
		if (b.get_layer() == "H1") return 0;
		if (a.get_layer() == "H2") return 1; 
		if (b.get_layer() == "H2") return 0;
		return 0;  
	} else return 0; 
} 

atom::atom() 
{
	index = -1;
	name  = (char)0;
	symbol_1 = (char)0;
	type  = 0;

	shift = 0;
	HCCHshift = -1000000; 

	ambi  = 1; 
	atom_ID = 0; 

	attached_N = 0;
	attached_C.erase(attached_C.begin(), attached_C.end());
	
	layer = "NA";
	_repAtom = 0;
}

atom::atom(int atom_index)
{
	index = atom_index; 
	name  = (char)0;
	symbol_1 = (char)0;
	type  = 0;

	shift = 0; HCCHshift = -1000000; 

	ambi  = 1; 
	atom_ID = 0;

	attached_N = 0;
	attached_C.erase(attached_C.begin(), attached_C.end());
	
	layer = "NA";
	_repAtom = 0;
}

atom::atom(int atom_ID_ID, int atom_index, string atom_name, char atom_type, float atom_shift, int atom_ambi)
{
	atom_ID = atom_ID_ID;
	index = atom_index; 
	name = atom_name; 
	type = atom_type;

	shift = atom_shift; //HCCHshift = atom_shift; 
	
	ambi  = atom_ambi; 

	attached_N = 0;
	attached_C.erase(attached_C.begin(), attached_C.end());
	
	layer = get_layer();
}

void atom::set(int atom_ID_ID, int atom_index, string atom_name, char atom_type, float atom_shift, int atom_ambi) 
{
	atom_ID = atom_ID_ID; 
	index = atom_index;
	name = atom_name;
	type = atom_type;

	shift = atom_shift; //HCCHshift = atom_shift;

	ambi = atom_ambi;
	
	layer = get_layer();
}
  
int atom::get_ID() const
{
	return atom_ID;
}

char atom::get_sym_1() const
{
	return symbol_1;
}

float atom::get_shift() const
{
	if (HCCHshift != -1000000)
	  return HCCHshift;
	else
		return shift;
}

float atom::get_difforig() const
{
	if (HCCHshift != -1000000)
		return (get_i_shift() - HCCHshift); // updated diff
	else
		return -1000000; // not updated
}

float atom::get_XdiffAb(float peak, float c) const
{
	if (peak == NOTEXIST)
	return 0;
	
	//if (DEBUG) NOESY_Assign::outputLog() << peak << endl;
	//peak = floor(peak*100+0.500001)/100.0;
	//if (DEBUG) NOESY_Assign::outputLog() << peak << endl;
	
  if (isH() && isatt_C())
    {
      float min; min = 100; 
      for(deque<atom*>::const_iterator ia = attached_C.begin(); ia != attached_C.end(); ia++)
    	{
    		//float shift = floor((*ia)->get_shift()*100+0.500001)/100.0;
    	  	float shift = (*ia)->get_shift(); 

    	  int l = (int) floor(-1*(peak - shift)/c);
    	  int u = (int) ceil(-1*(peak - shift)/c);

    	  float fl =  floor((peak+l*c - shift)*100+0.5001)/100.0;
    	  float fu =  floor((peak+u*c - shift)*100+0.5001)/100.0;

    	  float dis;
    	  if (fabs(fl) < fabs(fu))
    	    {
         		dis = fl;
    	    }
    	  else
    	    {
         		dis = fu;
    	    }

    	  if (fabs(dis) < fabs(min))
    	    {
    	      min  = dis;
    	    }
    	  
    	  if ((peak > 29.6) && (peak < 29.8))
    	  {
    		  if (DEBUG) 
    			  NOESY_Assign::outputLog() << "peak shift" << peak << " " << shift  << " " << l << " " << u << " " << fl << " " << fu << " " << dis << " " << min << endl; 
    	  }
    	}
			return min;
    }
  else if (isH() && isatt_N())
    {
    	//float shift = floor(attached_N->get_shift()*100+0.500001)/100.0;
   	  	float shift = attached_N->get_shift(); 
   	
      int l = (int) floor(-1*(peak-shift)/c);
      int u = (int) ceil(-1*(peak-shift)/c);
      
      float fl =  floor((peak+l*c - shift)*100+0.5001)/100.0;
      float fu =  floor((peak+u*c - shift)*100+0.5001)/100.0;
      
      if (fabs(fl) < fabs(fu))
    	{
     		return fl;
    	}
      else
    	{
     		return fu;
    	}

   }
  else
    {
    	//float shift = floor(get_shift()*100+0.500001)/100.0;
   	  	float shift = get_shift(); 
    	
      int l = (int) floor(-1*(peak-shift)/c);
      int u = (int) ceil(-1*(peak-shift)/c);
      
      float fl =  floor((peak+l*c - shift)*100+0.5001)/100.0;
      float fu =  floor((peak+u*c - shift)*100+0.5001)/100.0;
      
      if (fabs(fl) < fabs(fu))
    	{
     		return fl;
    	}
      else
    	{
     		return fu;
    	}
    }
  return 0;
}

float atom::get_HdiffAb(float peak, float c) const
{
	if (peak == NOTEXIST)
	return 0;
	
	//peak = floor(peak*1000+0.500001)/1000.0;
	//float shift = floor(get_shift()*1000+0.500001)/1000.0;
	float shift = get_shift(); 
	
	if (peak == 999)
	  {
	    return 0; //speical for ILV data
	  }

  int l = (int) floor(-1*(peak-shift)/c);
  int u = (int) ceil(-1*(peak-shift)/c);

  float fl =  floor((peak+l*c - shift)*1000.0+0.5001)/1000.0;
  float fu =  floor((peak+u*c - shift)*1000.0+0.5001)/1000.0;

  if (fabs(fl) < fabs(fu))
    return fl;
  else
    return fu;
}

float atom::get_Xdiff(float peak, float c) const
{
  return fabs(get_XdiffAb(peak, c));
}
float atom::get_Hdiff(float peak, float c) const
{
  return fabs(get_HdiffAb(peak, c));
}

float atom::get_i_shift() const
{
	return shift;
}

string atom::get_name() const
{
	return name;
}  
int atom::get_index() const
{
	return index;
} 

void atom::set_repAtom(atom* rep)
{_repAtom = rep;}

atom* atom::get_repAtom() const
{
	if (!_repAtom)
	{
		error("Error", "atom " + symbol_1 + index + name + " no repAtom found.");
	}
	return _repAtom;
}

string atom::get_layer() const
{

	if (layer == "NA")
	{
    	// backbone, Alpha, belta, gamma, ... (0, 1, 2, 3, 4, 5..)
    	map<char, map<string, string, less<string> >, less<char> >::iterator i;
    	
    	i = layer_map.find(symbol_1);
    	
    	if (i != layer_map.end())
    	{
    		map<string, string, less<string> >::iterator j;
    		
    		j = i->second.find(name);
    		
    		if (j != i->second.end())
    			layer = j->second;
    		else
		  {
		    layer = 'X'; 
		    //error("Error", "atom " + symbol_1 + index + name + " not defined.");
		  }
    	} 	
	}
	
	return layer;
}

char atom::get_type() const
{
	return type;
}

int atom::get_ambi() const
{
	return ambi;
}

void atom::set_att_N(atom* atom1)
{ 
	attached_N = atom1;
} 

void atom::set_att_C(atom* atom1)
{
	if (atom1) attached_C.push_back(atom1);
}

// isatt_N return the N's index if there is a N at the same layer 
// as the atom 
bool atom::isatt_N() const
{
	if (attached_N)
		return 1; 
	return 0;
}
bool atom::isatt_C() const
{
	if (attached_C.size() != 0)
		return 1;  
	return 0;
}

bool atom::isH() const
{
	if (type == 'H') return 1;
	return 0;
}
bool atom::isN() const
{
	if (type == 'N') return 1;
	return 0;
}
bool atom::isC() const
{
	if (type == 'C') return 1;
	return 0;
}

// for calibration
bool atom::isUnique() const // uniquely assigned
{
	if ((ambi == 1) && ((symbol_1 != 'A') || (name != "HB")) &&
		((symbol_1 != 'I') || ((name != "HG2") && (name != "HD1"))) &&
		((symbol_1 != 'L') || ((name != "HD1") && (name != "HD2"))) &&
		((symbol_1 != 'T') || (name != "HG2")) &&
	    ((symbol_1 != 'K') || (name != "HZ")) && 
		((symbol_1 != 'V') || ((name != "HG1") && (name != "HG2"))) &&
		((symbol_1 != 'M') || (name != "HE")))
		return 1; 
	return 0;   
}

// check how may two bond-connected methelne
// stereospecific assignment did not count
// ambi code is not used- may have errors
int atom::ambiDegree() const
{
	if (((name == "H") ||
				((symbol_1 == 'A') && (name == "HB")) || // methyl group
				((symbol_1 == 'I') && ((name == "HG2") || (name == "HD1"))) ||
				((symbol_1 == 'T') && (name == "HG2")) ||
				((symbol_1 == 'M') && (name == "HE")) ||
			
				((symbol_1 == 'F') && (name == "HZ")) ||
				((symbol_1 == 'I') && (name == "HB")) ||	
			
			 ((symbol_1 != 'G') && (name == "HA")) ||
			 ((symbol_1 == 'V') && (name == "HB")) ||
			 ((symbol_1 == 'T') && (name == "HB")) ||
			 ((symbol_1 == 'L') && (name == "HG")) ||
 			 ((symbol_1 == 'R') && (name == "HE")) ||
 			 ((symbol_1 == 'K') && (name == "HZ")) ||
			 ((symbol_1 == 'W') && (get_layer() != "B")) ||
			 ((symbol_1 == 'H') && (get_layer() != "B"))))
		return 1;
	else if (name.find_first_not_of("Q"))
		return name.find_first_not_of("Q");
	else
		return 2;
}

int atom::get_deg() const
{
	// if methyl group
	if (((symbol_1 == 'A') && (name == "HB")) || // methyl group
				((symbol_1 == 'I') && ((name == "HG2") || (name == "HD1"))) ||
				((symbol_1 == 'T') && (name == "HG2")) ||
				((symbol_1 == 'M') && (name == "HE")) ||
				((symbol_1 == 'L') && (name.substr(0,2) == "HD")) ||
				((symbol_1 == 'V') && (name.substr(0,2) == "HG")))
			return 3;
	if (((symbol_1 == 'L') && (name.substr(0,2) == "QD")) ||
			((symbol_1 == 'V') && (name.substr(0,2) == "QG")))
			return 6;
			
	// if methelne group
	if (name.substr(0,1) == "Q")
		return 2;
	return 1;
}

// for calibration only - spebare methele group 
int atom::get_degCount() const
{
	// if methyl group
	if (((symbol_1 == 'A') && (name == "HB")) || // methyl group
				((symbol_1 == 'I') && ((name == "HG2") || (name == "HD1"))) ||
				((symbol_1 == 'T') && (name == "HG2")) ||
				((symbol_1 == 'M') && (name == "HE")) ||
				((symbol_1 == 'L') && (name.substr(0,2) == "HD")) ||
				((symbol_1 == 'V') && (name.substr(0,2) == "HG")))
			return 3;
	if (((symbol_1 == 'L') && (name.substr(0,2) == "QD")) ||
			((symbol_1 == 'V') && (name.substr(0,2) == "QG")))
			return 3;
			
	// if methelne group
	if ((isRingAtom()) && (name.substr(0,1) == "Q")) 
		return 2;
	return 1;
}

bool atom::isRingAtom() const
{
	if ((get_sym_1() == 'H') || (get_sym_1() == 'F') || (get_sym_1() == 'W')|| (get_sym_1() == 'Y'))
	{
		if ((get_name() != "H") && (get_layer() != "A") && (get_layer() != "B"))
			return 1;
	}
	return 0;
}

bool atom::isMethyleneAtom() const
{
	if (isRingAtom()) return 0; 
	else if ((get_name() == "H")) return 0; 
	else if (get_layer() == "A")
	{
		if (get_sym_1() == 'G') return 1; 
		else return 0; 
	} else if (get_layer() == "B")
	{
		if ((get_sym_1() == 'A') || (get_sym_1() == 'I') || (get_sym_1() == 'V')) return 0;
		else return 1; 
	} else if (get_layer().substr(0,1) == "G")
	{
		if ((get_sym_1() == 'C') || (get_sym_1() == 'S') || (get_sym_1() == 'T')) return 0;
		else return 1; 
	} else if (get_layer().substr(0,1) == "D")
	{	
		if ((get_sym_1() == 'D') || (get_sym_1() == 'I') || (get_sym_1() == 'T')) return 0;
		else return 1; 
	} else if (get_layer().substr(0,1) == "E")
	{	
		if ((get_sym_1() == 'R') || (get_sym_1() == 'E') || (get_sym_1() == 'M')) return 0;
		else return 1; 
	} else if (get_layer().substr(0,1) == "H")
		return 1; 
	else
		return 0; 
}
// beta-sheet hnhn and helix hnhn(i,i+2) are around 4 and not usefull for structure calculations, may overconstrained 
bool atom::isSecLongHNHN(const atom* at) const
{
  if ((get_name() != "H") || (at->get_name() != "H"))
    {
      return 0; 
    }

  const Structure* const chain1 = seq.find(get_index())->second.get_ss();
  const Structure* const chain2 = seq.find(at->get_index())->second.get_ss(); 

  if (chain1->get_chain() == chain2->get_chain())
    {
      if (chain1->get_type() == 1) // helix
	{
	  if (abs(get_index()-at->get_index()) == 2)
	    {
	      return 1; 
	    }
	} 
      else if (chain1->get_type() == -1) // sheet
	{
	  if (abs(get_index() -at->get_index()) == 1)
	    {
	      return 1; 
	    }
	}
    }

  return 0; 
}

// fixed distance 
bool atom::isMeaningless(const atom* at) const
{
	// has to be intra
	if (get_index() != at->get_index())
		return 0;

	// all to HN is meaningful
	if ((get_name() == "H") || (at->get_name() == "H"))
		return 0;

	// methlyene group
	if (get_layer() == at->get_layer())
			return 1;

	// methlyene methyle group
	if (((get_sym_1() == 'V') || (get_sym_1() == 'L')) &&
			(get_layer().substr(0,1) == at->get_layer().substr(0,1)))
		return 1;

	// ring
	if (get_sym_1() == 'P')
		return 1;

	if ((isRingAtom()) && at->isRingAtom())
		return 1;

	// qsudo-atom, QB-QG
	if (length3(at))
	{
		return 1;
		
	}
	return 0;
}

bool atom::MethylFiveBonds(const atom* atom2) const
{
  if ((atom2) && (index == atom2->get_index()))
    {
      if ((get_deg() >=3) || (atom2->get_deg() >=3)) // methyl group, or L-QD, V-QG
	{
	  
	  //if (DEBUG) NOESY_Assign::outputLog() << "methyle " << this << endl << atom2 << endl;	    
	  
	  if (get_layer() == "N")
	    {
	      if (atom2->get_layer().substr(0,1) == "G")
		{
		  return 1; 
		} 
	      else
		{
		  return 0; 
		}
	    }
	  if (get_layer() == "A")
	    {
	      if (atom2->get_layer().substr(0,1) == "D")
		{
		  return 1; 
		}
	      else
		{
		  return 0; 
		}
	    }
	  if (get_layer() == "B")
	    {
	      if (atom2->get_layer().substr(0,1) == "E")
		{
		  return 1; 
		}
	      else
		{
		  return 0; 
		}
	    }
	  if (atom2->get_layer() == "N")
	    {
	      if (get_layer().substr(0,1) == "G")
		return 1; 
	      else
		return 0; 
	    }
	  if (atom2->get_layer() == "A")
	    {
	      if (get_layer().substr(0,1) == "D")
		return 1; 
	      else
		return 0; 
	    }
	  if (atom2->get_layer() == "B")
	    {
	      if (get_layer().substr(0,1) == "E")
		return 1; 
	      else
		return 0; 
	    }
	}
    }
  return 0; 
}
bool atom::length3(const atom* atom2) const // if connected by 3 bonds
{
	if ((atom2) && (index == atom2->get_index()))
	{
		if (get_layer() == "N")
		{
			if ((atom2->get_layer().substr(0,1) == "A"))
			{
				return 1;
			} else
			{
				return 0;
			}
		} else if (get_layer().substr(0,1) == "A")
		{
			if ((atom2->get_layer().substr(0,1) == "N") ||
					(atom2->get_layer().substr(0,1) == "B"))
			{
					return 1;
			} 
		} else if (get_layer().substr(0,1) == "B")
		{
			if (//(atom2->get_layer().substr(0,1) == "N") ||
					(atom2->get_layer().substr(0,1) == "A") ||
					//(atom2->get_layer().substr(0,1) == "B") ||
					(atom2->get_layer().substr(0,1) == "G")) //||
					//(atom2->get_layer().substr(0,1) == "D"))
			{
				return 1;
			} else
			{
				return 0;
			}
		} else if (get_layer().substr(0,1) == "G")
		{
			if (//(atom2->get_layer().substr(0,1) == "A") ||
					(atom2->get_layer().substr(0,1) == "B") ||
					//(atom2->get_layer().substr(0,1) == "G") ||
					(atom2->get_layer().substr(0,1) == "D"))// ||
					//(atom2->get_layer().substr(0,1) == "E"))
			{
				if ((get_sym_1() == 'I') && (atom2->get_name() == "HD1") && (get_name() == "HG2"))
					return 0;
				return 1;
			} else
			{
				return 0;
			}
		} else if (get_layer().substr(0,1) == "D")
		{
			if (//(atom2->get_layer().substr(0,1) == "B") ||
					(atom2->get_layer().substr(0,1) == "G") ||
					//(atom2->get_layer().substr(0,1) == "D") ||
					(atom2->get_layer().substr(0,1) == "E"))// ||
					//(atom2->get_layer().substr(0,1) == "Z"))
			{
				if ((get_sym_1() == 'I') && (atom2->get_name() == "HG2") && (get_name() == "HD1"))
					return 0;

				// if ring, up to three bond connected
				if (((get_sym_1() == 'F') || (get_sym_1() == 'Y') || (get_sym_1() == 'H')) &&
						(((atom2->get_name() == "HE2") && (get_name() == "HD1")) ||
						 ((atom2->get_name() == "HD2") && (get_name() == "HD1")) ||
						 ((atom2->get_name() == "HD1") && (get_name() == "HD2")) ||
						 ((atom2->get_name() == "HE1") && (get_name() == "HD2")) ||
						  (atom2->get_name() == "HZ")))
					return 0;
				if ((get_sym_1() == 'W') &&
						((atom2->get_name() == "HE3") ||
						 (atom2->get_layer().substr(0,1) == "H")||
						 (atom2->get_layer().substr(0,1) == "Z")))
					return 0;

				return 1;
			} 
		} else if (get_layer().substr(0,1) == "E")
		{
			if (//(atom2->get_layer().substr(0,1) == "G") ||
					(atom2->get_layer().substr(0,1) == "D") ||
					//(atom2->get_layer().substr(0,1) == "E") ||
					(atom2->get_layer().substr(0,1) == "Z"))
					//(atom2->get_layer().substr(0,1) == "H"))
			{
				// if ring, up to three-bond connected
				if (((get_sym_1() == 'F') || (get_sym_1() == 'Y')) &&
						(((atom2->get_name() == "HD1") && (get_name() == "HE2")) ||
						 ((atom2->get_name() == "HE2") && (get_name() == "HE1")) ||
						 ((atom2->get_name() == "HE1") && (get_name() == "HE2")) ||
						 ((atom2->get_name() == "HD2") && (get_name() == "HE1"))))
						  //(atom2->get_name() == "HZ")))
					return 0;
				if (((get_sym_1() == 'H')) &&
						(((atom2->get_name() == "HD1") && (get_name() == "HE2")) ||
						 ((atom2->get_name() == "HD2") && (get_name() == "HE1"))))
						  //(atom2->get_name() == "HZ")))
					return 0;
				if ((get_sym_1() == 'W') && (get_name() == "HE3") &&
						((atom2->get_name() == "HD1") ||
						 (atom2->get_name() == "HE1") ||
						 (atom2->get_name() == "HZ2") ||
						 (atom2->get_layer().substr(0,1) == "H")))
					return 0;
				if ((get_sym_1() == 'W') && (get_name() == "HE1") &&
						(
						 (atom2->get_name() == "HE3") ||
						 (atom2->get_name() == "HZ3") ||
						 (atom2->get_layer().substr(0,1) == "H")))
					return 0;
				return 1;
			} else
			{
				return 0;
			}
		} else if (get_layer().substr(0,1) == "Z")
		{
			if (//(atom2->get_layer().substr(0,1) == "D") ||
					(atom2->get_layer().substr(0,1) == "E") ||
					//(atom2->get_layer().substr(0,1) == "Z") ||
					(atom2->get_layer().substr(0,1) == "H"))
			{
				// if ring,
				if ((get_sym_1() == 'W') && (get_name() == "HZ2") &&
						(//(atom2->get_name() == "HD1") ||
						 (atom2->get_name() == "HE3")))
						 //(atom2->get_name() == "HZ3")))
					return 0;
				if ((get_sym_1() == 'W') && (get_name() == "HZ3") &&
						(//(atom2->get_name() == "HD1") ||
						 //(atom2->get_name() == "HZ2") ||
						 (atom2->get_name() == "HE1")))
					return 0;
				return 1;
			} else
			{
				return 0;
			}
		} else if (get_layer().substr(0,1) == "H")
		{
			if (atom2->get_layer().substr(0,1) == "Z")
				return 1;
			else
				return 0;
		}
	}
	return 0;
}
bool atom::less4(const atom* atom2) const // if connected by less 4 bonds
{
	if ((atom2) && (index == atom2->get_index()))
	{
		if (get_layer() == "N")
		{
			if ((atom2->get_layer().substr(0,1) == "A") ||
					(atom2->get_layer().substr(0,1) == "N") ||
					(atom2->get_layer().substr(0,1) == "B"))
			{
				return 1;
			} else
			{
				return 0;
			}
		} else if (get_layer().substr(0,1) == "A")
		{
			if ((atom2->get_layer().substr(0,1) == "N") ||
					(atom2->get_layer().substr(0,1) == "A") ||
					(atom2->get_layer().substr(0,1) == "B") ||
					(atom2->get_layer().substr(0,1) == "G"))
			{
					return 1;
			} else
			{
				if ((get_sym_1() == 'P') && (atom2->get_layer().substr(0,1) == "D"))
					return 1;
				else
					return 0;
			}	
		} else if (get_layer().substr(0,1) == "B")
		{
			if ((atom2->get_layer().substr(0,1) == "N") ||
					(atom2->get_layer().substr(0,1) == "A") ||
					(atom2->get_layer().substr(0,1) == "B") ||
					(atom2->get_layer().substr(0,1) == "G") ||
					(atom2->get_layer().substr(0,1) == "D"))
			{
				return 1;
			} else
			{
				return 0;
			}
		} else if (get_layer().substr(0,1) == "G")
		{
			if ((atom2->get_layer().substr(0,1) == "A") ||
					(atom2->get_layer().substr(0,1) == "B") ||
					(atom2->get_layer().substr(0,1) == "G") ||
					(atom2->get_layer().substr(0,1) == "D") ||
					(atom2->get_layer().substr(0,1) == "E"))
			{
				//if ((get_sym_1() == 'C') && (atom2->get_name() == "HA"))
				//	return 0;
				if ((get_sym_1() == 'I') && (atom2->get_name() == "HD1") && (get_name() == "HG2"))
					return 0;
				return 1;
			} else
			{
				return 0;
			}
		} else if (get_layer().substr(0,1) == "D")
		{
			if ((atom2->get_layer().substr(0,1) == "B") ||
					(atom2->get_layer().substr(0,1) == "G") ||
					(atom2->get_layer().substr(0,1) == "D") ||
					(atom2->get_layer().substr(0,1) == "E") ||
					(atom2->get_layer().substr(0,1) == "Z"))
			{	
				if ((get_sym_1() == 'I') && (atom2->get_name() == "HG2") && (get_name() == "HD1"))
					return 0;
				
				// if ring, up to three bond connected
				if (((get_sym_1() == 'F') || (get_sym_1() == 'Y') || (get_sym_1() == 'H')) &&
						(((atom2->get_name() == "HE2") && (get_name() == "HD1")) ||
						 ((atom2->get_name() == "HD2") && (get_name() == "HD1")) ||
						 ((atom2->get_name() == "HD1") && (get_name() == "HD2")) ||
						 ((atom2->get_name() == "HE1") && (get_name() == "HD2")) ||
						  (atom2->get_name() == "HZ")))
					return 0;
				if ((get_sym_1() == 'W') &&
						((atom2->get_name() == "HE3") ||
						 (atom2->get_layer().substr(0,1) == "H")||
						 (atom2->get_layer().substr(0,1) == "Z")))
					return 0;
					
				return 1;
			} else
			{
				if ((get_sym_1() == 'P') && (atom2->get_layer() == "A"))
					return 1;
				return 0;
			}
		} else if (get_layer().substr(0,1) == "E")
		{
			if ((atom2->get_layer().substr(0,1) == "G") ||
					(atom2->get_layer().substr(0,1) == "D") ||
					(atom2->get_layer().substr(0,1) == "E") ||
					(atom2->get_layer().substr(0,1) == "Z") ||
					(atom2->get_layer().substr(0,1) == "H"))
			{	
				// if ring, up to three-bond connected
				if (((get_sym_1() == 'F') || (get_sym_1() == 'Y')) &&
						(((atom2->get_name() == "HD1") && (get_name() == "HE2")) ||
						 ((atom2->get_name() == "HE2") && (get_name() == "HE1")) ||
						 ((atom2->get_name() == "HE1") && (get_name() == "HE2")) ||
						 ((atom2->get_name() == "HD2") && (get_name() == "HE1"))))
						  //(atom2->get_name() == "HZ")))
					return 0;
				if (((get_sym_1() == 'H')) &&
						(((atom2->get_name() == "HD1") && (get_name() == "HE2")) ||
						 ((atom2->get_name() == "HD2") && (get_name() == "HE1"))))
						  //(atom2->get_name() == "HZ")))
					return 0;
				if ((get_sym_1() == 'W') && (get_name() == "HE3") &&
						((atom2->get_name() == "HD1") ||
						 (atom2->get_name() == "HE1") ||
						 (atom2->get_name() == "HZ2") ||
						 (atom2->get_layer().substr(0,1) == "H")))
					return 0;
				if ((get_sym_1() == 'W') && (get_name() == "HE1") &&
						(
						 (atom2->get_name() == "HE3") ||
						 (atom2->get_name() == "HZ3") ||
						 (atom2->get_layer().substr(0,1) == "H")))
					return 0;					
				return 1;
			} else
			{
				return 0;
			}
		} else if (get_layer().substr(0,1) == "Z")
		{
			if ((atom2->get_layer().substr(0,1) == "D") ||
					(atom2->get_layer().substr(0,1) == "E") ||
					(atom2->get_layer().substr(0,1) == "Z") ||
					(atom2->get_layer().substr(0,1) == "H"))
			{	
				// if ring,
				if ((get_sym_1() == 'F') && (atom2->get_layer().substr(0,1) == "D"))
					return 0;			
				if ((get_sym_1() == 'W') && (get_name() == "HZ2") &&
						((atom2->get_name() == "HD1") ||
						 (atom2->get_name() == "HE3") ||
						 (atom2->get_name() == "HZ3")))
					return 0;
				if ((get_sym_1() == 'W') && (get_name() == "HZ3") &&
						((atom2->get_name() == "HD1") ||
						 (atom2->get_name() == "HZ2") ||
						 (atom2->get_name() == "HE1")))
					return 0;					
				return 1;
			} else
			{
				return 0;
			}
		} else if (get_layer().substr(0,1) == "H")
		{
			if ((atom2->get_layer().substr(0,1) == "E") ||
					(atom2->get_layer().substr(0,1) == "H") ||
					(atom2->get_layer().substr(0,1) == "Z"))
			{	
				// if ring,
				if ((get_sym_1() == 'W') && (get_name() == "HH2") &&
						(
						 (atom2->get_layer().substr(0,1) == "E")))
					return 0;					
				return 1;
			} else
			{
				return 0;
			}
		}			
	}
	return 0;
}


bool atom::isDeC(const atom* atom2) const // this two atoms are degenrated or have close shift
{
	if ((atom2) && (index == atom2->get_index()) && (get_layer().substr(0,1) == atom2->get_layer().substr(0,1))) 
		return 1;
		
	if ((atom2) && (fabs(atom2->get_shift()-get_shift()) < 0.04))
		return 1;
		
	return 0;
}

bool atom::isDe(const atom* atom2) const // this two atoms are degenrated or have ambiguity
{
	// methyl region usually badly peakk picked
	// often seem hb -> hg1, hb -> hg2, but not the resverse
	// we group them togther to have build more evidence for these groups
	if ((atom2) && (index == atom2->get_index()) && ((get_layer() == atom2->get_layer())  ||
		
	  // carbon
		((symbol_1 == 'V') && ((name == "CG1") || (name == "CG2")) &&
			((atom2->get_name() == "CG1") || (atom2->get_name() == "CG2"))) ||
		((symbol_1 == 'L') && ((name == "CD1") || (name == "CD2")) &&
			((atom2->get_name() == "CD1") || (atom2->get_name() == "CD2"))) ||
	  // proton
		((symbol_1 == 'V') && ((name == "HG1") || (name == "HG2")) &&
			((atom2->get_name() == "HG1") || (atom2->get_name() == "HG2"))) ||
		((symbol_1 == 'L') && ((name == "HD1") || (name == "HD2")) &&
			((atom2->get_name() == "HD1") || (atom2->get_name() == "HD2")))))
		return 1; 
	return 0;
}

const AA& atom::residue() const
{
	return seq.find(index)->second;
}

atom* const atom::att_N() const
{
	return attached_N;
}

const deque<atom*>& atom::att_C() const
{
	return attached_C; 
} 

atom* const atom::att_C(float peak, float c) const
{
	peak = floor(peak*100+0.500001)/100.0;

	float min = 100;  
	atom* atommin; atommin = 0; 
	for(deque<atom*>::const_iterator ia = attached_C.begin(); ia != attached_C.end(); ia++)
	{
		float shift = floor((*ia)->get_shift()*100+0.500001)/100.0;
		
	  int l = (int) floor(-1*(peak-shift)/c);
	  int u = (int) ceil(-1*(peak-shift)/c);
	  
	  float fl =  floor(fabs(peak+l*c - shift)*100+0.500001)/100.0;
	  float fu =  floor(fabs(peak+u*c - shift)*100+0.500001)/100.0;

	  float dis;
	  if (fl < fu)
	    dis = fl;
	  else 
	    dis = fu;

	  if (dis < min) 
	    {
	      min  = dis; 
	      atommin = *ia;
	    }
	}
	return atommin; 
} 

atom* const atom::att_X(float peak, float c) const
{
	if (peak == NOTEXIST)
		return 0;
		
	if (isatt_N())
		return att_N();  
	else if (isatt_C())
		return att_C(peak, c);
	else
		return 0;
}

void  atom::set_shift(float ishift)
{
	shift = ishift;
}

void  atom::set_ambi(int iambi)
{
	ambi = iambi;
}
void  atom::set_name(const string& new_name)
{
	name = new_name; 
} 
void atom::set_atom_ID(int index)
{
	atom_ID = index; 
}
void atom::set_HCCHshift(float value)
{
	HCCHshift = value; 
}

//--- if both, if for Val, Leu pseudo 
void atom::get_pseudoboth(pseudo& temp) const
{
	temp.reset();
	temp.pname = name;
	
	if (((symbol_1 == 'V') && ((name == "HG1") || (name == "HG2"))) 
		|| ((symbol_1 == 'L') && ((name == "HD1") || (name == "HD2")))) 
	{
		if (symbol_1 == 'V') 
		{
			if (name == "HG1") 
				temp.pname = "QG1";
			else temp.pname = "QG2"; 
		}
		if (symbol_1 == 'L')
		{
			if (name == "HD1") 
				temp.pname = "QD1";
			else temp.pname = "QD2";  
 		}

		temp.div = 3; 
		temp.cerct = 1.0; 
		temp.cment = "m;bothMe"; 
	} else 
	{
		temp.div = 1;
		temp.cment = "both";
	}
}


//--- only the case that need add pseudo for degenerate or only one NOE case
void atom::get_pseudoRaw(const atom* atom2, pseudo& temp, bool haveIntra) const
{
	// for HN no correction
	if (name == "H")
	{
		temp.pname = "HN";
	}
	// special case for Gly
	if ((ambi != 1) && ((symbol_1 == 'G') && ((name == "QA") ||
		(name == "HA3") || (name == "HA2"))))
	{
		//--- correction and pname;
		if ((abs(atom2->get_index() - index) < 2) && haveIntra) // intra & sequencetial
		{
			temp.cerct = 0.6;
			temp.pname = "QA";
			temp.cment = "i;";

		} else
		{
			temp.cerct = 1.0;
			temp.pname = "QA";
			temp.cment = "m;";
		}
		//--- if degenerate or only one
		if (name == "QA")
		{
			temp.cment += "da";
			temp.div = 2;
		} else if ((name == "HA3") || (name == "HA2"))
		{
			temp.cment += "one";
			temp.div = 1;
		}
	}
	// Betas
	else if ((ambi != 1) &&
		((symbol_1 != 'A') && ((name == "HB2") || (name == "HB3") || (name == "QB"))))
	{
		if (((abs(atom2->get_index() - index) < 1) && haveIntra) &&
			(atom2->get_name() == "HN")) // intra HN
		{
			temp.cerct = 0.6;
			temp.pname = "QB";
			temp.cment = "i;";
		} else // non intra HN
		{
			temp.cerct = 1.0;
			temp.pname = "QB";
			temp.cment = "m;";
		}

		if (name == "QB")
		{
			temp.cment += "db";
			temp.div = 2;
		} else if ((name == "HB2") || (name == "HB3"))
		{
			temp.cment += "one";
			temp.div = 1;
		}
	}

	// Gammas
	else if ((ambi != 1) &&
		(((symbol_1 != 'I') && (symbol_1 != 'T') && (symbol_1 != 'V')) &&
		((name == "HG2") || (name == "HG3") || (name == "QG"))))
	{
		temp.cerct = 1.0;
		temp.pname = "QG";
		temp.cment = "m;";

		if (name == "QG")
		{
			temp.cment += "dg";
			temp.div = 2;
		} else if ((name == "HG2") || (name == "HG3"))
		{
			temp.cment += "one";
			temp.div = 1;
		}
	}

	// special gammas of ILE
	else if ((ambi != 1) &&
		((symbol_1 == 'I') && ((name == "HG12") || (name == "HG13")
		|| (name == "QG1"))))
	{
		temp.cerct = 1.0;
		temp.pname = "QG1";
		temp.cment = "m;";

		if (name == "QG1")
		{
			temp.cment += "dg";
			temp.div = 2;
		} else if ((name == "HG12") || (name == "HG13"))
		{
			temp.cment += "one";
			temp.div = 1;
		}
	}

	// Deltas not for Tyr, Phe, His
	else if ((ambi != 1) &&
		(((symbol_1 != 'Y') && (symbol_1 != 'F') && (symbol_1 != 'H') &&
		(symbol_1 != 'I') && (symbol_1 != 'L'))
		&& ((name == "HD2") || (name == "HD3") || (name == "QD"))))
	{
		temp.cerct = 1.0;
		temp.pname = "QD";
		temp.cment = "m;";

		if (name == "QD")
		{
			temp.cment += "dd";
			temp.div = 2;
		} else if ((name == "HD2") || (name == "HD3"))
		{
			temp.cment += "one";
			temp.div = 1;
		}
	}
	// Deltas for Asn
	else if ((ambi != 1) &&
		((symbol_1 == 'N') && ((name == "HD21") || (name == "HD22")
		|| (name == "QD2"))))
	{
		temp.cerct = 1.0;
		temp.pname = "QD2";
		temp.cment = "m;";

		if (name == "QD2")
		{
			temp.cment += "dd";
			temp.div = 2;
		} else if ((name == "HD21") || (name == "HD22"))
		{
			temp.cment += "one";
			temp.div = 1;
		}
	}

	// Epsilons not for Tyr, Phe, His, Trp
	else if ((ambi != 1) &&
		(((symbol_1 != 'Y') && (symbol_1 != 'F') &&
		(symbol_1 != 'H') && (symbol_1 != 'W') && (symbol_1 != 'M')) &&
		((name == "HE2") || (name == "HE3") || (name == "QE"))))
	{
		temp.cerct = 1.0;
		temp.pname = "QE";
		temp.cment = "m;";

		if (name == "QE")
		{
			temp.cment += "de";
			temp.div = 2;
		} else if ((name == "HE2") || (name == "HE3"))
		{
			temp.cment += "one";
			temp.div = 1;
		}
	}

	// Epsilons for Gln
	else if ((ambi != 1) &&
		((symbol_1 == 'Q') && ((name == "HE21") || (name == "HE22")
		|| (name == "QE2"))))
	{
		temp.cerct = 1.0;
		temp.pname = "QE2";
		temp.cment = "m;";

		if (name == "QE2")
		{
			temp.cment += "de";
			temp.div = 2;
		} else if ((name == "HE21") || (name == "HE22"))
		{
			temp.cment += "one";
			temp.div = 1;
		}
	}

	// Aromatics, Special case of Phe and Tyr
	else if ((ambi != 1) &&
		(((symbol_1 == 'Y') || (symbol_1 == 'F')) &&
		((name == "QR") || (name == "HD1") || (name == "HD2") || (name == "QD") ||
		(name == "HE2") || (name == "HE1") || (name == "QE"))))
	{
		if (name == "QR")
		{
			temp.pname = "QR";
			temp.cerct = 2.4;
			temp.cment = "q;dall";
			temp.div = 4;
		} else
		{
			if ((abs(atom2->get_index() - index) < 1) && haveIntra && // intra, ha
					(atom2->get_name() == "HA"))
			{
				temp.cerct = 1.0;
				temp.cment = "m;";
			} else // not intra-ha
			{
				temp.cerct = 2.0;
				temp.cment = "r;";
			}

			if (name == "QD")
			{
				//temp.pname = "CG";
				temp.pname = "QD";
				temp.cment += "dd";
				temp.div = 2;
			} else if (name == "QE")
			{
				//temp.pname = "CZ";
				temp.pname = "QE";
				temp.cment += "de";
				temp.div = 2;
			} else if ((name == "HD1") || (name == "HD2"))
			{
				//temp.pname = "CG";
				temp.pname = "QD";
				temp.cment += "one";
				temp.div = 1;
			} else if ((name == "HE1") || (name == "HE2"))
			{
				//temp.pname = "CZ";
				temp.pname = "QE";
				temp.cment += "one";
				temp.div = 1;
			}
		}
	}
	// Mehtyl for Thr, Ala, Ile, Met
	else if (((symbol_1 == 'T') || (symbol_1 == 'I')) && (name == "HG2"))
	{
		temp.pname = "QG2";
		temp.div = 3;
		if ((abs(atom2->get_index() - index) < 1) && haveIntra &&
			(atom2->get_name() == "HA")) // intra-ha
		{
			temp.cerct = 0.6;
			temp.cment = "i;Me";
		} else // nointr-ha
		{
			temp.cerct = 1.0;
			temp.cment = "m;Me";
		}
	}
	else if ((symbol_1 == 'A') && (name == "HB"))
	{
		temp.pname = "QB";
		temp.div = 3;
		if ((abs(atom2->get_index() - index) < 1) && haveIntra &&
			(atom2->get_name() == "HA")) // meaningless, not used
		{
			temp.cerct = 0.2;
			temp.cment = " ;Me";
		} else if ((abs(atom2->get_index() - index) < 1) && haveIntra &&
			(atom2->get_name() == "H")) // intra-ha
		{
			temp.cerct = 0.6;
			temp.cment = "i;Me";
		} else
		{
			temp.cerct = 1.0;
			temp.cment = "m;Me";
		}
	}
	//--- may need more work
	else if ((symbol_1 == 'M') && (name == "HE"))
	{
		temp.pname = "QE";
		temp.div = 3;
		temp.cerct = 1.0;
		temp.cment = "m;Me";
	}
	else if ((ambi == 1) && (symbol_1 == 'V') && (name == "HG1"))
	{
		temp.pname = "QG1";
		temp.div = 3;
		temp.cerct = 1.0;
		temp.cment = "m;Me";
	}
	else if ((ambi == 1) && (symbol_1 == 'V') && (name == "HG2"))
	{
		temp.pname = "QG2";
		temp.div = 3;
		temp.cerct = 1.0;
		temp.cment = "m;Me";
	}
	else if ((ambi == 1) && (symbol_1 == 'L') && (name == "HD1"))
	{
		temp.pname = "QD1";
		temp.div = 3;
		temp.cerct = 1.0;
		temp.cment = "m;Me";
	}
	else if ((ambi == 1) && (symbol_1 == 'L') && (name == "HD2"))
	{
		temp.pname = "QD2";
		temp.div = 3;
		temp.cerct = 1.0;
		temp.cment = "m;Me";
	}
	// degenerate methyls or only one for Val and Leu
	else if ((ambi != 1) && (((symbol_1 == 'V') && ((name == "QG") || (name == "HG1")
		|| (name == "HG2"))) || ((symbol_1 == 'L') && ((name == "QD")
		|| (name == "HD1") || (name == "HD2")))))
	{
		{
			if (symbol_1 == 'V') temp.pname = "QQG";
			if (symbol_1 == 'L') temp.pname = "QQD";
 		}

		if ((name == "QG") || (name == "QD"))
			temp.div = 6;
		else temp.div = 3;

		if ((abs(atom2->get_index() - index) < 1) && haveIntra &&
			(atom2->get_name() == "HA"))
		{
			if (symbol_1 == 'V')
			{
				temp.cerct = 0.6;
				temp.cment = "i;dMe";
			}
			if (symbol_1 == 'L')
			{
				temp.cerct = 1.7;
				temp.cment = "s;dMe";
			}
		} else if ((abs(atom2->get_index() - index) < 1) && haveIntra &&
			(atom2->get_name() == "H"))
		{
			if (symbol_1 == 'V')
			{
				temp.cerct = 1.7;
				temp.cment = "s;dMe";
			}
			if (symbol_1 == 'L')
			{
				temp.cerct = 1.7; //2.2;
				temp.cment = "s;dMe"; //"r;dMe";
			}
		} else
		{
			temp.cerct = 2.4;
			temp.cment = "q;dMe";
		}
	}

	//for EC constraint only 
	else if ((ambi != 1) && (((symbol_1 == 'V') && ((name == "QCG") || (name == "CG1")
		|| (name == "CG2"))) || ((symbol_1 == 'L') && ((name == "QCD")
		|| (name == "CD1") || (name == "CD2")))))
	{
		{
			if (symbol_1 == 'V') temp.pname = "QCG";
			if (symbol_1 == 'L') temp.pname = "QCD";
 		}
	}

	else if ((symbol_1 == 'I') && (name == "HD1"))
	  {
		temp.pname = "QD1";
		temp.div = 3;
		temp.cerct = 1.0;
		temp.cment = "m;Me";
	  }
	else if ((symbol_1 == 'K') && (name == "HZ"))
	  {
	    temp.cerct = 1.0;
	    temp.pname = "QZ";
	    temp.cment = "m;";

	    temp.div = 3;
	}
	if (temp.pname == "")
		temp.pname = name;
}

// have intra
void atom::get_pseudo(const atom* atom2, pseudo& temp) const
{	
	temp.reset();
	get_pseudoRaw(atom2, temp, 1); // have intra
	
	if (DEBUG) NOESY_Assign::outputLog() << get_name() << " " << get_ambi() << " " << get_layer() << " " << symbol_1 << " " << temp.pname << endl; 
}

// set ambi = 2 for methyle groups 
void atom::set_layer(string l)
{
	layer = l; 
}
void atom::set_symbol_1(char s)
{
	symbol_1 = s; 
}

void atom::get_pseudo2(const atom* atom2, pseudo& temp) const
{
	if (isMethyleneAtom() && (get_ambi() == 1))
	{
		atom atom3; 
		atom3.set(1234567, get_index(), get_name(), get_type(), get_shift(), 2); 
		atom3.set_layer(get_layer()); atom3.set_symbol_1(get_sym_1()); 
		
		if (DEBUG) NOESY_Assign::outputLog() << "aaa " << atom3 << "bbb"<< endl; 
		atom3.get_pseudo(atom2, temp);
	} else
		get_pseudo(atom2, temp); 
}
// no intra
void atom::get_pseudoIC(const atom* atom2, pseudo& temp) const
{
	temp.reset();
	get_pseudoRaw(atom2, temp, 0); // no intra
}


