/***************************************************************************
                          atomlist.cpp  -  description
                             -------------------
    begin                : Mon Aug 21 2000
    copyright            : (C) 2000 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#include <algorithm>
#include <fstream>
#include <cmath>
#include "atom.h"

extern void error(char*, char*);
extern void error(string, string);

/******************************************************
make_att_list: 

input: Atom_List
operation: 
	for each atom in the Atom_List, if it is a H, 
find the N or C that it attached to, and modify the 
corresponding member in the atom object in Atom_List.

Atom_List is ordered by atom_index_list

make_att_list,
make_att,
*****************************************************/
void make_att(int index, Atom_Index_List& atoms_index)
{
	// find all the atoms that belong to that index
	Atom_Index_List::iterator iter_lower = atoms_index.lower_bound(index);
	Atom_Index_List::iterator iter_upper = atoms_index.upper_bound(index); 

	for(Atom_Index_List::iterator iter_p = iter_lower; iter_p != iter_upper; ++iter_p)
	{
		if ((*iter_p).second->isH())
		for(Atom_Index_List::iterator iter = iter_lower; iter != iter_upper; ++iter)
		{
				
			int len = (*iter).second->get_layer().length(); 
			int plen = (*iter_p).second->get_layer().length();
			
			if (plen < len) len = plen;
				   
			if ((*iter_p).second->get_layer().substr(0, len) == 
			    (*iter).second->get_layer().substr(0, len))
			{
				if ((*iter).second->get_type() == 'N')
				{
					(*iter_p).second->set_att_N((*iter).second); 
					break; 
				}
				else if ((*iter).second->get_type() == 'C')
				{
					(*iter_p).second->set_att_C((*iter).second); 
				}
			}
		}
	}
}	

int makeMerge(ostream& outputLog, Atom_List& atoms, char atype, float cutoff)
{
	Atom_Shift_List atoms_shift;
	set<float, less<float> > shiftValue;
	
  for(Atom_List::iterator iter = atoms.begin(); iter != atoms.end(); ++iter)
    if (atype == 'H')
    {
      if ((*iter).isH())
      {
        atoms_shift.insert(make_pair((*iter).get_shift(), &(*iter)));
        shiftValue.insert((*iter).get_shift());
      }
	  } else
	  {
      if (!(*iter).isH())
      {
        atoms_shift.insert(make_pair((*iter).get_shift(), &(*iter)));
        shiftValue.insert((*iter).get_shift());
      }
	  }

  int sum = 0;
	// from the index, find the possible degenerated one
	Atom_List toDelete; toDelete.clear();
	
  for(set<float, less<float> >::iterator v = shiftValue.begin(); v != shiftValue.end(); v++)
  {
      for(Atom_Shift_List::iterator p = atoms_shift.lower_bound((*v) - cutoff);
            p != atoms_shift.upper_bound((*v) + cutoff); p++)
      {
        for(Atom_Shift_List::iterator q = atoms_shift.lower_bound((*v) - cutoff);
              q != atoms_shift.upper_bound((*v) + cutoff); q++)
        {
          if ((p != q) && ((*p).second != (*q).second) &&
              (fabs((*q).second->get_shift() - (*p).second->get_shift()) < cutoff) &&
              ((*q).second->get_index() == (*p).second->get_index()))
          {
            string pLayer = "";
            string qLayer = "";

            // if in the same layer
            if (isdigit(((*p).second->get_name()[(*p).second->get_name().length()-1])))
              pLayer = (*p).second->get_name().substr(1, (*p).second->get_name().length()-2);
            if (isdigit(((*q).second->get_name()[(*q).second->get_name().length()-1])))
              qLayer = (*q).second->get_name().substr(1, (*q).second->get_name().length()-2);

            if ((pLayer == qLayer) && (pLayer != "") && (p->second->get_sym_1() != 'V')
                  && ((p->second->get_sym_1() != 'L') || (pLayer == "B")) // degenrate  // exclude V and L methyl
                  && ((p->second->get_sym_1() != 'W') || (pLayer == "B")))
            {
              //degenerate
              if ((((*p).second->get_name()[0] != 'Q') && ((*q).second->get_name()[0] != 'Q')) ||
                  (((*p).second->get_name()[0] == 'Q') && ((*q).second->get_name()[0] == 'Q')))
              {
                sum++;
                outputLog << "***Chemcial Shifts are degenerate and merged" << endl;
                outputLog << "   " << (*p).second << endl;
                outputLog << "   " << (*q).second << endl;

                if (atype == 'H')
                  (*p).second->set_name("Q" + pLayer);
                else if (atype == 'N')
		  p->second->set_name("QN" + pLayer); 
		else if (atype == 'C')
                  (*p).second->set_name("QC" + pLayer);

                
                  (*p).second->set_ambi(2);

                  // if amro, set to 3
                  if (((((*p).second->get_sym_1() == 'F') || ((*p).second->get_sym_1() == 'Y')) &&
		                  (((atype != 'H') && ((pLayer[1] == 'D') || (pLayer[1] == 'E'))) || ((atype == 'H') && ((pLayer[0] == 'D') || (pLayer[0] == 'E'))))) ||
		                  (((*p).second->get_sym_1() == 'W') &&
		                  (((atype != 'H') && (pLayer[1] == 'Z')) || ((atype == 'H') && (pLayer[0] == 'Z')))))
                    (*p).second->set_ambi(3);
                

                (*p).second->set_shift(((*p).second->get_shift() + (*q).second->get_shift())/2.0);

                toDelete.push_back(*((*q).second));

                 (*p).second->resetLayer();
                (*q).second = (*p).second;
                outputLog << "to-" << (*p).second << endl << endl;
              } else
              {
                outputLog << "*** Error: Chemcial Shifts are degenerate but merged failed" << endl;
                outputLog << "   " << (*p).second << endl;
                outputLog << "   " << (*q).second << endl << endl;
              }
            }
          }
        }
      }
  }

  atoms_shift.clear();

  for(Atom_List::iterator i = toDelete.begin(); i != toDelete.end(); i++)
  {
    atoms.remove(*i);
    outputLog << "Deleted: " << (*i) << endl;
  }

  outputLog << endl;
  return sum;
}

typedef multimap<int, pair<float, atom*>, less<int> > ResShiftAtom;

string itoa(int);

int makeMergeVL(ostream& outputLog, Atom_List& atoms, float cutoff, float Ccutoff)
{

  ResShiftAtom resShift;      resShift.clear();
  ResShiftAtom CresShift;     CresShift.clear();
  set<int, less<int> > resIndex;       resIndex.clear();
      
  for(Atom_List::iterator iter = atoms.begin(); iter != atoms.end(); ++iter)
    {
      if (((iter->get_sym_1() == 'V') && (iter->get_name().substr(0,2) == "HG")) ||
             ((iter->get_sym_1() == 'L') && (iter->get_name().substr(0,2) == "HD")))
      {
        resShift.insert(make_pair(iter->get_index(), make_pair(iter->get_shift(), &(*iter))));
        resIndex.insert(iter->get_index());
      }
      else if (((iter->get_sym_1() == 'V') && (iter->get_name().substr(0,2) == "CG")) ||
             ((iter->get_sym_1() == 'L') && (iter->get_name().substr(0,2) == "CD")))
      {
        CresShift.insert(make_pair(iter->get_index(), make_pair(iter->get_shift(), &(*iter))));
      }
	  }

  int sum = 0;
	// from the index, find the possible degenerated one
	Atom_List toDelete; toDelete.clear();
  
  for(set<int, less<int> >::iterator v = resIndex.begin(); v != resIndex.end(); v++)
   {
      
              if (resShift.count(*v) > 2)
              {
                  error("Error", "H-Methyl groups for residue " + itoa(*v) + " is greater to two ! ");
              }
              if (CresShift.count(*v) > 2)
              {
                  error("Error", "C-Methyl groups for residue " + itoa(*v) + " is greater to two ! ");
              }

              if (resShift.count(*v) < 2)
                continue;
                                          
              ResShiftAtom::iterator vl = resShift.lower_bound(*v);
              ResShiftAtom::iterator vu = vl; vu++;
                
              if ((fabs(vl->second.first - vu->second.first) < cutoff))
              {
                  
                ResShiftAtom::iterator cvl = CresShift.lower_bound(*v);
                ResShiftAtom::iterator cvu = cvl;

                if (cvl != CresShift.end())
                  cvu++;

                if ((cvl == CresShift.end()) || (fabs(cvl->second.first - cvu->second.first) < Ccutoff)) // not assigned or degenerate
                {
                    
                  string pLayer = "";
                  
                  // if in the same layer
                  if (isdigit(((*vl).second.second->get_name()[(*vl).second.second->get_name().length()-1])))
                        pLayer = (*vl).second.second->get_name().substr(1, (*vl).second.second->get_name().length()-2);

                    // degenerate
                    sum++;
                    outputLog << "***Chemcial Shifts are degenerate and merged" << endl;
                    outputLog << "   " << (*vl).second.second << endl;
                    outputLog << "   " << (*vu).second.second << endl;

                    if (cvl != CresShift.end())
                    {
                      outputLog << "   " << (*cvl).second.second << endl;
                      outputLog << "   " << (*cvu).second.second << endl;
                    }
                    
                    vl->second.second->set_name("Q" + pLayer);
                    vl->second.second->set_ambi(2);
                    vl->second.second->set_shift((vl->second.second->get_shift()+vu->second.second->get_shift())/2.0);
                    toDelete.push_back(*(vu->second.second));
                    (*vl).second.second->resetLayer();
                    (*vu).second.second = (*vl).second.second;
                    outputLog << "to-" << vl->second.second << endl;
 
                    if (cvl != CresShift.end())
                    {
                      cvl->second.second->set_name("QC" + pLayer);
                      cvl->second.second->set_ambi(2);
                      cvl->second.second->set_shift((cvl->second.second->get_shift()+cvu->second.second->get_shift())/2.0);
                      toDelete.push_back(*(cvu->second.second));
                      (*cvl).second.second->resetLayer();
                      (*cvu).second.second = (*cvl).second.second;
                      outputLog << "to-" << cvl->second.second << endl << endl;
                    }                                       
                   }
              }
   }


  for(Atom_List::iterator i = toDelete.begin(); i != toDelete.end(); i++)
  {
    atoms.remove(*i);
    outputLog << "Deleted: " << (*i) << endl;
  }

  outputLog << endl;
  return sum;
}

// not reliable 
void checkHisStat(ostream& outputLog, const Atom_List& atoms, map<int, string, less<int> >& hisStat) // modify the atoms
{
  /////////////////////////////////////////////
	// convert the Atom_List to Atom_Index_List
	/////////////////////////////////////////////
  multimap<int, string, less<int> > atoms_index;  
  for(Atom_List::const_iterator iter = atoms.begin(); iter != atoms.end(); ++iter)
    {
      atoms_index.insert(make_pair((*iter).get_index(), iter->get_name()));
    }
		
	//////////////////////////////////////////////////////	
	// for each atoms in the residue, find the _repAtom	
	/////////////////////////////////////////////////////
	for(SEQM::iterator iter = seq.begin(); iter != seq.end(); ++iter)
	{
	  if (iter->second.get_sym_1() != 'H') continue; 

  	// find all the atoms that belong to that index
	  multimap<int, string, less<int> >::iterator iter_lower = atoms_index.lower_bound(iter->first);
	  multimap<int, string, less<int> >::iterator iter_upper = atoms_index.upper_bound(iter->first);

	bool he2 = 0; bool hd1=0; 
  	for(multimap<int, string, less<int> >::iterator iter_p = iter_lower; iter_p != iter_upper; ++iter_p)
  	{
	  if (iter_p->second == "HE2")
	    {
	      he2 = 1; 
	    }
	  if (iter_p->second == "HD1")
	    {
	      hd1 = 1; 
	    }
	}

	if (he2 && hd1)
	  {
	    hisStat.insert(make_pair(iter->first, "+")); // should be the default 
	  } else if (he2)
	  {
	    hisStat.insert(make_pair(iter->first, "T")); 
	  } else if (!he2 && !hd1)
	  {
	    hisStat.insert(make_pair(iter->first, "M")); 
	  } else
	  {
	    hisStat.insert(make_pair(iter->first, "")); 
	  }
	}
}
void make_att_list(ostream& outputLog, Atom_List& atoms, float Hm, float Xm) // modify the atoms
{
  // first, converge if degenerate or close
 	// create index
	
  while (makeMerge(outputLog, atoms, 'H', Hm))
  {};
  while (makeMerge(outputLog, atoms, 'X', Xm))
  {};

  while (makeMergeVL(outputLog, atoms, Hm, Xm))
  {};
 
  /////////////////////////////////////////////
	// convert the Atom_List to Atom_Index_List
	/////////////////////////////////////////////
	Atom_Index_List atoms_index;  
	for(Atom_List::iterator iter = atoms.begin(); iter != atoms.end(); ++iter)
	{
		atoms_index.insert(make_pair((*iter).get_index(), &(*iter)));
	}

	/////////////////////////////////
	// for each residue in the SEQ,
	/////////////////////////////////
	for(SEQM::iterator iter = seq.begin(); iter != seq.end(); ++iter)
		make_att((*iter).first, atoms_index); 	
		
	//////////////////////////////////////////////////////	
	// for each atoms in the residue, find the _repAtom	
	/////////////////////////////////////////////////////
	for(SEQM::iterator iter = seq.begin(); iter != seq.end(); ++iter)
	{
  	// find all the atoms that belong to that index
  	Atom_Index_List::iterator iter_lower = atoms_index.lower_bound(iter->first);
  	Atom_Index_List::iterator iter_upper = atoms_index.upper_bound(iter->first);

  	deque<atom*> repAtomList;
  	
  	for(Atom_Index_List::iterator iter_p = iter_lower; iter_p != iter_upper; ++iter_p)
  	{
  		{
  			deque<atom*>::iterator i;
  			for(i = repAtomList.begin(); i!= repAtomList.end(); ++i)
  			{
  				if ((iter_p->second->isDe(*i)) && (iter_p->second->get_type() == (*i)->get_type()))
  					break;
  			}
  			if (i == repAtomList.end())
  			{
  				iter_p->second->set_repAtom(iter_p->second);
  				repAtomList.push_back(iter_p->second);
  			}
  			else
  				iter_p->second->set_repAtom(*i);
  		}
  	}
	}
}

ostream& prtRefinedShiftList(ostream& s, Atom_List& atoms) 
{
  s << "#Atom   Residue" << endl;
  s << "#shift  Seq     Residue Atom    Atom    Shift/  Error/  Ambiguity" << endl;
  s << "#assign code    Label   Name    Type    ppm     ppm     Code" << endl;
  s << "#---------------------------------------------------------------" << endl;
  s << "#" << endl;
	for(Atom_List::iterator i = atoms.begin(); i != atoms.end(); ++i)
	{
		s << i->get_ID() << "\t" << i->get_index() << "\t" << residue_name[i->get_sym_1()] << "\t" << i->get_name() << "\t"
			<< i->get_type() << "\t" << i->get_shift() << "\t" << "." << "\t" << i->get_ambi();

		if (i->get_shift() != i->get_i_shift())
		{
			s << "\t#orig: " << i->get_i_shift() << " diff: ";
			(i->isH()) ? s.precision(2) : s.precision(1);
			s << fabs(i->get_i_shift()-i->get_shift());
			s.precision(6);
		}	else if (i->get_difforig() > -100)
		{
			s << "\t#same";
		}
	
		s << endl;
	}
	return s;
}

