/***************************************************************************
                          calibrate.cpp  -  description
                             -------------------
    begin                : Tue Oct 1 2002
    copyright            : (C) 2002 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/
#include <iostream>
#include <iomanip>

#include <sstream>
#include <cmath>
#include "experiment.h"
#include "noesy_assign.h"
#include "Diana.h"

SampleStatistic getDistStat(deque<Pdb>& , pair<const atom*, const atom*>, bool);

#define pow(x,y) pow((double) x, (double)y)
extern bool DEBUG;

string itoa(int);

ostream& erasePrint(ostream&, const string);

void error(char*, char*);
void error(string, string);


void NOESY::get_pseudoboth(const atom* org1, pseudo& pdo1) const
{
  org1->get_pseudoboth(pdo1);
}

void NOESY::get_pseudo(const atom* org1, const atom* org2, pseudo& pdo1) const
{
  if (isIC())
  {
    org1->get_pseudoIC(org2, pdo1);
  }
  else
  {
		if (DEBUG) NOESY_Assign::outputLog() << "in pseudo" << endl;
    org1->get_pseudo(org2, pdo1);
  }
}
void NOESY::get_pseudo2(const atom* org1, const atom* org2, pseudo& pdo1) const
{
  if (isIC())
  {
    org1->get_pseudoIC(org2, pdo1);
  }
  else
  {
	if (DEBUG) NOESY_Assign::outputLog() << "in pseudo2" << endl;
    org1->get_pseudo2(org2, pdo1);
  }
}


float NOESY::get_KC() const
{	
  //if (DEBUG) NOESY_Assign::outputLog() << NOESY_Assign::ddcycle() << endl; 
  if (KCON == 0)
    {
    if (NOESY_Assign::ddcycle() <= 1) // no structure information 
	{
	  //if (KCON == 0)
	    {
		//--- the average intensity,
		//--- average intensity of assignable peaks or average intensity of assigned peak
		//--- not include peak that near diangal
		
		float sum1 = 0; float c1 = 0;
		for(PeakAssignReason::const_iterator pi = AssignList.begin(); pi != AssignList.end(); pi++)
		{			
			// only the backbone
			if (((pi->second.first.first->get_layer() == "N") ||
			     (pi->second.first.first->get_layer() == "A") ||
			     ((pi->second.first.first->get_layer() == "B") && (pi->second.first.first->get_sym_1() != 'A'))) &&
			   (((pi->second.first.second->get_layer() == "N") ||
			     (pi->second.first.second->get_layer() == "A") ||
			     ((pi->second.first.second->get_layer() == "B") && (pi->second.first.second->get_sym_1() != 'A')))))
			{
				// excluding close to diangla peaks
				if (fabs(pi->first->get_HX1() - pi->first->get_HX2()) > 0.05)
				{
					sum1 += fabs((*pi).first->get_Intensity()); // /multi);
					c1++;
//					if (DEBUG)
//						NOESY_Assign::outputLog() << "GENCON " << pi->second.first.first << endl << pi->second.first.second << endl;
				}
			}
		}

		if (c1 < 20) // use all peaks
		  {
		    sum1 = 0; c1 = 0;
		    for(PeakList::const_iterator iter = peakList.begin(); iter != peakList.end(); ++iter)
		      {
			if ((MombySimpleMatch.find((&*iter)) != MombySimpleMatch.end()) &&
			    (DadbySimpleMatch.find((&*iter)) != DadbySimpleMatch.end()) &&
			    (fabs((*iter).get_HX1() - (*iter).get_HX2()) > 0.05))
			  {
			    sum1 += fabs((*iter).get_Intensity());
			    c1++;
			  }
		      }
		  }

		if (DEBUG) NOESY_Assign::outputExm() << endl << "cycle " << NOESY_Assign::ddcycle() << " AVA I for calibration = " << (sum1/c1) << " c1=" << c1 << endl ;
		if (c1 > 20)
		  {
		    KCON = (sum1/c1)/NOESY_Assign::KCalibration() ;
		    if (DEBUG) NOESY_Assign::outputExm() << ">20 " << NOESY_Assign::ddcycle() << " KCON = " << KCON << endl;
		  }
		else
		  {
		    KCON = (sum1/c1)/NOESY_Assign::KCalibration() ;
		    if (DEBUG) NOESY_Assign::outputExm() << "all " << NOESY_Assign::ddcycle() << " KCON = " << KCON << endl;
		  }
	    }
	} 
	else // structure information is avaiable 
	{
	  if (DEBUG) NOESY_Assign::outputExm() << NOESY_Assign::ddcycle() << label() << endl; 
	  SampleStatistic kvalues; kvalues.reset(); 
	  for(PeakAssignReason::const_iterator pi = AssignList.begin(); pi != AssignList.end(); pi++)
	    {
	      int multi = AssignList.count(pi->first);
	      if (multi > 1) continue; //removie peaks with multiple assignments 

	      // excluding close to diangla peaks
	      if (fabs(pi->first->get_HX1() - pi->first->get_HX2()) <= 0.05) continue;
		
	      pair<atom*, atom*> ab; 
	      if (*(pi->second.first.first) < *(pi->second.first.second))
		ab = pi->second.first; 
	      else
		ab = make_pair(pi->second.first.second, pi->second.first.first); 
	      
	      SampleStatistic distStat = getDistStat(NOESY_Assign::pdbList(), ab, 0);
	      kvalues += log(fabs((pi->first->get_Intensity()))/pow(distStat.median(), -6)); // summation distance  

	      if (DEBUG) NOESY_Assign::outputExm() << pi->first->get_Intensity() << "\t" << pi->first->get_ID() <<  "\t" << distStat.median() << endl;   
	    }

	  KCON = exp(kvalues.mean()); 
	  if (DEBUG) NOESY_Assign::outputExm() << "cycle " << NOESY_Assign::ddcycle() <<" KCON = " << KCON << endl;
	  if (DEBUG)
	    {
	      for(SmplVect::const_iterator ki = kvalues.oriSamples().begin(); ki != kvalues.oriSamples().end(); ki++)
		{
		  NOESY_Assign::outputExm() << *ki << endl;
		}
	      NOESY_Assign::outputExm() << endl;
 
	      for(PeakAssignReason::const_iterator pi = AssignList.begin(); pi != AssignList.end(); pi++)
		{
		  int multi = AssignList.count(pi->first);
		  if (multi > 1) continue; //removie peaks with multiple assignments 

		  // excluding close to diangla peaks
		  if (fabs(pi->first->get_HX1() - pi->first->get_HX2()) <= 0.05) continue;
		
		  pair<atom*, atom*> ab; 
		  if (*(pi->second.first.first) < *(pi->second.first.second))
		    ab = pi->second.first; 
		  else
		    ab = make_pair(pi->second.first.second, pi->second.first.first); 
		  
		  SampleStatistic distStat = getDistStat(NOESY_Assign::pdbList(), ab, 0);
		  
		  if (DEBUG) NOESY_Assign::outputExm() << pi->first->get_Intensity() << "\t" << pi->first->get_ID() <<  "\t" << distStat.median() << "\t" << pow(KCON/fabs(pi->first->get_Intensity()), 1/6) << endl;   
		}
	      
	    }
	}
    }
  return KCON;
}

typedef pair<bool, SampleStatistic> mthI;
typedef multimap<atom12, mthI, atom12less > atom12mthI;
 	
float NOESY::Calibrate() const
{
	if (DEBUG) NOESY_Assign::outputLog() << "In Calibration " << endl;
	
	SampleStatistic kd; kd.reset(); 
	float kcon = get_KC();	
	if (DEBUG) NOESY_Assign::outputLog() << "kcon =" << kcon << endl; 

	//--- calibrate
	atom12mthI cstrbuffer;  cstrbuffer.erase(cstrbuffer.begin(), cstrbuffer.end());

	//--- check if the peak has more than one non-degenerate assignemnts, not for test
	for(PeakList::const_iterator iter = peakList.begin(); iter != peakList.end(); ++iter)
	{	
	  bool multi = 0;
		PeakAssignReason::const_iterator pl = AssignList.lower_bound(&(*iter));
		PeakAssignReason::const_iterator pu = AssignList.upper_bound(&(*iter));

		float intensity = fabs((*iter).get_Intensity());
		if ((!intensity) && (AssignList.count(&(*iter))) !=0)
		  {
		    intensity = 0.0000001;
		    cerr << " *** Calibration error: Peak Intensity equal 0 and is set to 0.0000001: "  << (*iter) << endl;
		    NOESY_Assign::outputError() << " *** Calibration error: Peak Intensity equal 0 and is set to 0.0000001: "  << (*iter) << endl;
		  }
    
		//--- see if multiple assignment
		atom12set temp; temp.erase(temp.begin(), temp.end());
		if (AssignList.count(&(*iter)) > 1)
		{
			for(PeakAssignReason::const_iterator pi = pl; pi != pu; pi++)
			{
				// allow ambi
				if (((*pi).second.second.count(AMBI) != 0) && ((*pi).second.second.count(PDB) == 0) &&
						((*pi).second.second.count(SYMPdb) == 0))
						break;
				
					if ((*pi).second.first.first->isUnique() &&
						(*pi).second.first.second->isUnique())
					{
						temp.insert((*pi).second.first);
					} else
					{ 	atom12set::iterator ai;
						for(ai = temp.begin(); ai != temp.end(); ai++)
						{
							if ((*pi).second.first.first->isDe((*ai).first) &&
								(*pi).second.first.second->isDe((*ai).second) &&
								(((*pi).second.first.first != (*ai).first) ||
								((*pi).second.first.second != (*ai).second)))
									break;
						}
						if (ai == temp.end()) // NOT DEGENERATE
							temp.insert((*pi).second.first);
					}
				
			}
			
			if (temp.size())
			{
			  multi = 1;
			}
		}

		// for each assignments 
		for(PeakAssignReason::const_iterator pi = pl; pi != pu; pi++)
		{
			// if they are equal, then they are gemical cross peaks and ignored
			if (pi->second.first.first == pi->second.first.second)
				continue;
				
			// if have DEL_VIO and PDBM, set dist = distance in the structure to avoid reassigned the one violated
			PeakAssignReason::const_iterator rl = BufferReason.lower_bound(&(*iter));
			PeakAssignReason::const_iterator ru = BufferReason.upper_bound(&(*iter));
			PeakAssignReason::const_iterator ri;
			bool vio = 0;
			for(ri = rl; ri != ru; ri++)
			  {
			    if (((*ri).second.first.first == pi->second.first.first) &&
				((*ri).second.first.second == pi->second.first.second))
			      {
				if ((ri->second.second.count(DEL_VIO) != 0) || (ri->second.second.count(ADJ_CALI) != 0)) // violated in at least one model 
				  vio = 1;
				break;
			      }
			  }	

			if (vio && (pi->second.second.count(PDB) != 0)) // SET TO 5.0 IF VIOLATED AND REASSIGNED
			{
				if (DEBUG) NOESY_Assign::outputLog() << "VIO and PDB" << endl; 
			       pair<atom*, atom*> ab; 
			       if (*(pi->second.first.first) < *(pi->second.first.second)) // use the distance in the structure 
				 ab = pi->second.first; 
			       else
				 ab = make_pair(pi->second.first.second, pi->second.first.first); 
			       
			       SampleStatistic distStat = getDistStat(NOESY_Assign::pdbList(), ab, 0);
			       intensity = (float) kcon * pow(distStat.median(), -6); // set to the distance in the structures 
				if (DEBUG) NOESY_Assign::outputLog() << "reset intensity" << pi->first << endl;  
			}

			// cns only
 			if (((*pi).second.second.count(AMBI) != 0) && ((*pi).second.second.count(PDB) == 0) &&
 					((*pi).second.second.count(SYMPdb) == 0))
 			break;

			// EC only expect
			//if (NOESY_Assign::isEC() && ((pi->second.second.count(EXPECT) == 0) && (pi->second.second.count(PDB) == 0) && (pi->second.second.count(SYMPdb) == 0)))
			 // break; 

			if (multi)
			  {
			    if (NOESY_Assign::ddcycle() == 1)
				intensity = kcon/(float)pow(4,6);  
			    else
			     {
			       pair<atom*, atom*> ab; 
			       if (*(pi->second.first.first) < *(pi->second.first.second))
				 ab = pi->second.first; 
			       else
				 ab = make_pair(pi->second.first.second, pi->second.first.first); 
			       
			       SampleStatistic distStat = getDistStat(NOESY_Assign::pdbList(), ab, 0);
			       intensity = (float) kcon * pow(distStat.median(), -6); // set to the distance in the structures
				if (DEBUG) NOESY_Assign::outputLog() << "reset intensity" << pi->first << endl;  
			     }
			  }

			//if (DEBUG) NOESY_Assign::outputLog() << "pass multi" << endl; 

			if (*((*pi).second.first.first) < *((*pi).second.first.second))
			{
				atom12mthI::iterator atom12iter =
					cstrbuffer.find((*pi).second.first);
				
				if (atom12iter == cstrbuffer.end())
					cstrbuffer.insert(make_pair((*pi).second.first, mthI(multi, SampleStatistic(intensity))));
				else
				{
					(*atom12iter).second.first = min((*atom12iter).second.first, multi);
					(*atom12iter).second.second +=intensity;
				}
			}
			else if ((isIC()) || (*((*pi).second.first.second) < *((*pi).second.first.first)) || (*((*pi).second.first.second) == *((*pi).second.first.first)))
			{
				atom12mthI::iterator atom12iter =
					cstrbuffer.find(atom12((*pi).second.first.second, (*pi).second.first.first));
				
				if (atom12iter == cstrbuffer.end())
					cstrbuffer.insert(make_pair(atom12((*pi).second.first.second, (*pi).second.first.first), mthI(multi, SampleStatistic(intensity))));
				else
				{
					(*atom12iter).second.first = min((*atom12iter).second.first, multi);
					(*atom12iter).second.second +=intensity;
				}
			}
		}
	}

	//--- check the pseudo case nonsteroassigned diastereotpoic groups 
	if (DEBUG) NOESY_Assign::outputLog() << "before cstrbuffer" << endl; 
	for(atom12mthI::iterator ai = cstrbuffer.begin();  ai != cstrbuffer.end(); ai++)
	{
		vector<pair<pseudo, pseudo> > ptempPairVector; ptempPairVector.clear();

		vector<float> intmin; intmin.clear();
		vector<float> intmax; intmax.clear();
		vector<float> ori_distMax; ori_distMax.clear(); // for hyper
		vector<float> ori_dist; ori_dist.clear(); // for hyper
		vector<float> xplor_dist; xplor_dist.clear();
		
		pseudo ptemp1; pseudo ptemp2;
		pseudo ptemp1both; pseudo ptemp2both;

		atom12mthI::iterator an; an = ai; an++; // the first one not degenerate
		atom12mthI::iterator ait; ait = ai; // the last one that degenerate
		
		int dnum = 1;
		while (an != cstrbuffer.end())
		{	
			if ((ai->first.first->isDe(an->first.first)) && (ai->first.second->isDe(an->first.second)))
			{
				ait = an;
				++an;
				++dnum;
			} else
			{
				break;
			}
		}
		
	    if (DEBUG) NOESY_Assign::outputLog() << ai->first.first << endl << ai->first.second << endl << dnum << endl; 

	    float dist = 0; // diana dist
		float xplorDist = 0; // xplordist
		
		
		  if (dnum > 1)  // more than one peak 
		{
			  multiset<float, less<float> > dset; 		
			  set<const atom*> a1; set<const atom*> a2; 
			  for(atom12mthI::iterator i = ai; i != an; ++i)
			    {
			      dset.insert(pow(kcon/(*i).second.second.max(), 1/6.0));
			      a1.insert(i->first.first); a2.insert(i->first.second); 
			      
			      if (DEBUG) NOESY_Assign::outputLog() << i->first.first << endl << i->first.second << " " << pow(kcon/(*i).second.second.max(), 1/6.0) << endl; 
			    }
			  
			  if (DEBUG) NOESY_Assign::outputLog() << endl; 
			  
			  // use one constraint to avoid spin diffusion 
			  if (a1.size() == 2)
				  get_pseudo2((*ai).first.first, (*ai).first.second, ptemp1); // avoid spin diffusion 
			  else
				  get_pseudo(ai->first.first, ai->first.second, ptemp1);
			  
			  if (a2.size() == 2)
				  get_pseudo2((*ai).first.second, (*ai).first.first, ptemp2); // avoid spin diffusion 
			  else
				  get_pseudo(ai->first.second, ai->first.first, ptemp2); 
			  
			  ptempPairVector.push_back(make_pair(ptemp1, ptemp2));
		 

		  multiset<float>::iterator di = dset.begin(); 
		  float d1 = (*di); //A 
		  di++;
		  int count = 1; 
		  while(di != dset.end()) // count number of different distances 
		    {
		      if (*di > 1.12*d1)
			{
			  break; 
			}
		      count++; 
		      di++;
		    }

		  if (dnum == 4) //table 1 from Fletcher et al 
		  {
			  if (count == 1) // only one A
			  {
				  dist = 1.12*1.12*d1;
				  xplorDist = d1;			
			  }
			  else if (count < 4)
			  {
				  dist = 1.12*d1;
				  xplorDist = d1/1.12;			
			  }
			  else
			  {
				  dist = d1;
				  xplorDist = d1/1.26; 
			  }
		  } else
		  {
			  if (count == 1)
			  {
				  dist = 1.12*d1;
				  xplorDist = d1;
			  } else
			  {
				  dist = d1; 
				  xplorDist = d1/1.12; 
			  }
		  }
		  dist *= pow(ai->first.first->get_degCount() * ai->first.second->get_degCount(), 1/6); // corrected for methyle group 
		  
		} else //dnum ==1 
		  {
		    get_pseudo((*ai).first.first, (*ai).first.second, ptemp1);
		    get_pseudo((*ai).first.second, (*ai).first.first, ptemp2);
		    ptempPairVector.push_back(make_pair(ptemp1, ptemp2));
		    
		    float intless = fabs((*ai).second.second.max())/(float)(ptemp1.div * ptemp2.div); // correctad for center average 
		    
		      dist = pow((double)kcon/intless, (double)1/6.0); //center average 
		      xplorDist = pow((double)kcon/ai->second.second.max(), (double)1/6.0); //summation 

		      if (DEBUG) NOESY_Assign::outputLog() << kcon << " " << ai->second.second.max() << " " <<  pow((double)kcon/ai->second.second.max(), (double)1/6.0) << endl; 
		    
		  }

		    if (!IC &&  
			(((((*ai).first.first->get_layer() <= "B") && ((*ai).first.first->get_sym_1() != 'A')) || ((*ai).first.first->get_layer() == "N")) &&
			 ((((*ai).first.second->get_layer() <= "B") && ((*ai).first.second->get_sym_1() != 'A')) || ((*ai).first.second->get_layer() == "N"))))
		      {
			// backbone 
			kd += xplorDist; 
		      }

		  /* don't bin them */
		if (xplorDist < 2.29) xplorDist = 2.3; // strong -> 2.7
   		else if (xplorDist < 2.89) xplorDist = 2.9; //medium -> 3.4 
		else if (xplorDist < 3.59) xplorDist = 3.6; // weak 4.6
		else if (xplorDist < 3.99) xplorDist = 4.0; 
		else if (xplorDist < 4.99) 
		  xplorDist = 5.0; // very weak  -> 6.0 
	        else
		  xplorDist = max(get_Rmax()-1.0, 5.0); 		
		
		// center-average
		if (dist < 2.29) dist = 2.3; // strong
   		else if (dist < 2.89) dist = 2.9; //medium
		else if (dist < 3.59) dist = 3.6; // weak
		else if (dist < 3.99) dist = 4.0; 
		else if (dist < 4.99) 
		  dist = 5.0; // very weak 		
		else 
		  dist = max(get_Rmax()-1.0, 5.0); 
  
		/*
  		if (dist < 2.49) dist = 2.5; // strong
   		else
		  dist = ceil(dist); // 3.0, 4.0, 5.0
		if (dist > 4.01)
		  dist = 4.0;
		*/
	  
		// sum-average
		  {
		    if (!IC &&  
			(((((*ai).first.first->get_layer() <= "B") && ((*ai).first.first->get_sym_1() != 'A')) || ((*ai).first.first->get_layer() == "N")) &&
			 ((((*ai).first.second->get_layer() <= "B") && ((*ai).first.second->get_sym_1() != 'A')) || ((*ai).first.second->get_layer() == "N"))))
		      {
			// backbone 
		      }
		    else
		      {
			// all interchain or side-chain do not calibrate
			// set to weak - 4.0 + 1.0(ub)
			// if very weak - use 5.0+1.0 
			
			// including aromatic weak peaks
			//xplorDist += 0.5; 
			
			//if (xplorDist < 4.01)
			  //xplorDist = 4.0;
		      	xplorDist = get_Rmax()-1.0; 

			  //if (dist < 4.01)
			//  dist = 4.0;
			xplorDist = get_Rmax()-1.0; 
			
		      }
		  }

	float ub = 0;
	float xub = 0;

	
   	if (dist > 3.000001)  //???? dist is adjust here, ub is for dist only - not for xplorDist
	  ub = 1.0;
	else if (dist > 2.500001)
	  ub = 0.5;
	else
	  ub = 0.4;

   	if (xplorDist > 3.000001)  //???? for xplorDist
	  xub = 1.0;
	else if (xplorDist > 2.500001)
	  xub = 0.5;
	else
	  xub = 0.4;

	if (DEBUG) NOESY_Assign::outputLog() << "dist " << dist << " ub " << ub << endl; 
	if (DEBUG) NOESY_Assign::outputLog() << "xdist " << xplorDist << " xub " << xub << endl; 
	// xplorDist - target distance for xplor
	// center-average distance, corrected will be addin in also
	dist = dist+ub;
           
     if ((ptempPairVector[0].first.pname == "") || (ptempPairVector[0].second.pname == ""))
		{
		  NOESY_Assign::outputError() << "***ERROR: Cannot find the qseudo name for atom:"
			  << (*ai).first.first << endl
				<< (*ai).first.second << endl
				<< (*ai).first.first->get_layer()
				<< (*ai).first.first->get_name() << "*"
				<< (*ai).first.second->get_layer()
				<< (*ai).first.second->get_name() << "*"
				<< "*" << ptempPairVector[0].first.pname << "*" << ptempPairVector[0].second.pname
				<< "*" << dist << "X" << xplorDist << "*" << label() << endl << flush;
    }
    		
		if (DEBUG)
		{
			if ((ptempPairVector[0].first.pname == "") || (ptempPairVector[0].second.pname == ""))
			{
				NOESY_Assign::outputError() << "ERROR: Cannot find the pseudo name for atom";
			}
			
			int j =0;
			NOESY_Assign::outputLog() << "dnum = " << dnum << endl;
			NOESY_Assign::outputLog() << "size = " << ptempPairVector.size() << endl;
			if (ptempPairVector.size() > 1)
			  {
			    NOESY_Assign::outputLog() << "size>1" << endl;
			  }

		for(atom12mthI::iterator i = ai; i != an; ++i)
  		{
  			if (j < ptempPairVector.size())
  			{
  			NOESY_Assign::outputLog() << (*i).first.first << endl
  				<< (*i).first.second << endl
  				<< (*i).first.first->get_layer()
  				<< (*i).first.first->get_name() << "*"
  				<< (*i).first.second->get_layer()
  				<< (*i).first.second->get_name() << "*"
  				<< "*" << ptempPairVector[j].first.pname << "*" << ptempPairVector[j].second.pname
						  << "*" << dist << "X" << xplorDist << "*" << ub << "*" << xub << "*" << label() << endl << flush;
  				++j;
				if (ptempPairVector.size() == 1) // only one constraints 
				  {
				    j = 0; 
				  }
  			}
  			else
  				 break;
  		}
		}

		int j=0;		
		for(atom12mthI::iterator i = ai; i != an; ++i)
		{
			if (j < ptempPairVector.size())
			{
  			pseudo ptemp1; ptemp1 = ptempPairVector[j].first;
  			pseudo ptemp2; ptemp2 = ptempPairVector[j].second;
  			
  			if ((ptemp1.pname != "") && (ptemp2.pname != ""))	
  			{
    			string cment;
    			cment = ptemp1.cment + "," + ptemp2.cment;
					
    			DianaConstraint dC;
    			dC.name1 = ptemp1.pname; dC.name2 = ptemp2.pname;
    			
    			if (DEBUG)
    				NOESY_Assign::outputLog() << j << " " << dC.name1 << " " << dC.name2 << " corrections: " << ptemp1.cerct << " " << ptemp2.cerct << endl;

    			dC.dist = dist + ptemp1.cerct + ptemp2.cerct;
			dC.corrections = ptemp1.cerct + ptemp2.cerct;
    			dC.nn = ptemp1.div * ptemp2.div;
    			dC.cment = cment; dC.expr = label();
    			//dC.ori_dist = ori_distAvg[j]; dC.range = range[j];
			dC.ub = ub; 
			dC.xub = xub; 
       
    			dC.xplorDist = xplorDist;
    			
    			if ((*i).second.first == 1)  dC.multi = 1;
    			else dC.multi = 0;

			// check if exists 
			DianaMap::iterator conL = NOESY_Assign::DianaBuffer().lower_bound((*i).first); 
			DianaMap::iterator conU = NOESY_Assign::DianaBuffer().upper_bound((*i).first); 
			DianaMap::iterator con; 
			for(con = conL; con != conU; con++)
			  {
			    if ((con->second.expr == label()) && (con->second.name1 == dC.name1) && (con->second.name2 == dC.name2))// old one 
			      {
				con->second.xplorDist = xplorDist;
				con->second.xub = xub;  
				break; 
			      }
			  }
			
			if (con == conU) // new 
			  {
			    if (isIC())
			      NOESY_Assign::ICDianaBuffer().insert(make_pair((*i).first, dC));
			    else
			      NOESY_Assign::DianaBuffer().insert(make_pair((*i).first, dC));
			  }
  			}
			++j;
			if (ptempPairVector.size() == 1) // only one constraints
			  {
			    j = 0; 
			  }
			} else
				break;
		}
				
		ai = ait;	
		//ai = an; 
	}	
	if (kd.count() > 0)
	  return kd.mean();
	else
	  return NOESY_Assign::KD(); 
}


// not supported for XPLOR yet
void NOESY::CalibrateAmbi(stringstream& ambiStream, int LLength, const Sym& parSym) 
{
  //cout << "In CalibrateAmbi " << endl;

	//int id=0;

	for(PeakList::const_iterator iter = peakList.begin(); iter != peakList.end(); ++iter)
	{
	  //cout << (*iter) << parSym.htol << " " << parSym.xtol << endl; 
		if (AssignList.find(&(*iter)) == AssignList.end())
		{
			//--- find the best match that never tested
			MatchPeakAtom::const_iterator iml = MombySimpleMatch.lower_bound(&(*iter));
			MatchPeakAtom::const_iterator imu = MombySimpleMatch.upper_bound(&(*iter));
			MatchPeakAtom::const_iterator idl = DadbySimpleMatch.lower_bound(&(*iter));
			MatchPeakAtom::const_iterator idu = DadbySimpleMatch.upper_bound(&(*iter));

			//--- find the possible assignments that could for tested
			atom12set temp; temp.erase(temp.begin(), temp.end());
			for(MatchPeakAtom::const_iterator im = iml; im != imu; im++)
			for(MatchPeakAtom::const_iterator id = idl; id != idu; id++)
			  {
			    //cout << "check " << im->second << endl << id->second << endl; 
			    if ((im->second->get_Hdiff(iter->get_HX1(), iter->SWSF_HX1()) < parSym.htol + 0.000001) && 
				(im->second->get_Xdiff(iter->get_X1(), iter->SWSF_X1()) < parSym.xtol + 0.000001) &&
				(id->second->get_Hdiff(iter->get_HX2(), iter->SWSF_HX1()) < parSym.htol + 0.000001) && 
				(id->second->get_Xdiff(iter->get_X2(), iter->SWSF_X2()) < parSym.xtol + 0.000001))
			      { 
				temp.insert(make_pair((*im).second, (*id).second));
				//cout << "inserted" << endl; 
			      }
			  }

			// order by distance, pick the top ones 
			if (NOESY_Assign::pdbList().size() != 0)
			  {
			    multimap<float, atom12, less<float> > tempDist; 
			    for(atom12set::iterator ta=temp.begin(); ta != temp.end(); ta++)
			      {
				pair<const atom*, const atom*> ab;
				if (ta->first < ta->second)
				  ab = *ta;
				else
				  ab = make_pair(ta->second, ta->first);
				
                                SampleStatistic tm; tm.reset(); tm = getDistStat(NOESY_Assign::pdbList(), ab, isIC());
                                if (tm.samples().size())
                                  cout << " Dis: " << tm.min() << "-" << tm.max() << " " << tm.mean() << "(mean)" << tm.median() << "(mid) " ;
				tempDist.insert(make_pair(tm.median(), *ta)); 
			      }

			    float summation = 0; 
			    for(multimap<float, atom12, less<float> >::iterator ti = tempDist.begin(); ti!= tempDist.end(); ti++)
			      {
				if (summation > pow((double)5.0, (double)-6.0))
				  {
				    temp.erase(ti->second); 
				  }
				summation += pow((double)ti->first, (double)-6.0); 
			      }
			  }

			/*
			PeakAssignReason::const_iterator bl = BufferReason.lower_bound(&(*iter));
			PeakAssignReason::const_iterator bu = BufferReason.upper_bound(&(*iter));
			
			for(PeakAssignReason::const_iterator bi = bl; bi != bu; bi++)
			{
			  if (//((*bi).second.second.count(DEL_ASS) != 0) ||
			      //((*bi).second.second.count(DEL_LOC) != 0) ||
			      //((*bi).second.second.count(DEL_LSS) != 0) ||
			      ///((*bi).second.second.count(DEL_INTRA) != 0) ||
			      ((*bi).second.second.count(DEL_INT) != 0))
			      //((*bi).second.second.count(DEL_IC)  != 0) ||
			      //((*bi).second.second.count(DEL_SAM) != 0) ||
			      //((*bi).second.second.count(DEL_DIA) != 0) ||
			      //((*bi).second.second.count(DEL_PDB) != 0))
			    {
			      temp.erase((*bi).second.first);
			      //cout << "ERASE:" << (*bi).second.first.first << endl
			      //	<< (*bi).second.first.second << endl;
			    } 
			}
			*/


			if ((temp.size() != 0)) // && ((num == 0) || (temp.size() <= num)))
			{
			  //cout << "prtCyIntraChainAmbi" << endl; 
			  prtCyIntraChainAmbi(ambiStream, LLength, temp, iter->get_ID()); 
			}
		}
	}	
}

