/***************************************************************************
                          combinedSym -  symmetry method
                             -------------------
    copyright            : (C) 2007 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#include <iostream>
#include <algorithm>
#include <sstream>
#include <cmath>
#include <iomanip>
#include "experiment.h"
#include "noesy_assign.h"
#include "par.h"

#define swap(a,b) ({ typeof(a) temp = (a); (a) = (b); (b) = temp; })
#define abs(x) abs((double)(x))

extern bool DEBUG;
extern int faces[1000][1000];

SampleStatistic getDistStat(deque<Pdb>& , pair<const atom*, const atom*>, bool);
float sumAverage(const SampleStatistic&);

typedef map<int, AA, less<int> > SEQM;
typedef map<pair<int, int>, float, less<pair<int, int> > > matrix;

string itoa(int);

typedef pair<int, int> respair;
typedef multiset<respair, less<respair> > resSet;

typedef multimap<const general_peak* const, const general_peak* const, less<const general_peak* const> > PeakPairMap;
typedef multimap<MomDad, MomDad, less<MomDad> > MomDadMap;

typedef const general_peak* PeakPtr;
typedef multimap<PeakPtr, MomDad, less<PeakPtr> > PeakPtrMomDadMap;
typedef set<PeakPtr, less<PeakPtr> > PeakPtrMomDadSet;

bool sameHydro(char a, char b)
{
	// -1 -> +/- charge
	// 0 -> no prefer
	// 1 -> hydrophobic

	map<char, int, less<char> > AAcharge;
	AAcharge['P']=0;
	AAcharge['G']=1;
	AAcharge['A']=1;
	AAcharge['R']=-1;
	AAcharge['N']=0;
	AAcharge['D']=0;
	AAcharge['C']=1;
	AAcharge['Q']=0;
	AAcharge['E']=0;
	AAcharge['H']=0;
	AAcharge['I']=1;
	AAcharge['L']=1;
	AAcharge['K']=-1;
	AAcharge['M']=1;
	AAcharge['F']=1;
	AAcharge['S']=0;
	AAcharge['T']=0;
	AAcharge['W']=1; //?
	AAcharge['Y']=0; //?
	AAcharge['V']=1;

	if ((AAcharge[a] * AAcharge[b] != -1))
		return 1;

	return 0;
}

void NOESY_Assign::BuildM2Map(MomDadSet& mdSetM2, MomDadPeakAtomMap& mdPAMapM2)
{
	////////////////////////////////
	// grouped by two-bond connection
	// and filter
	////////////////////////////////
	for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
	{
	  if ((*iter)->get_tols().HX2 == 999)
	    continue;

		const PeakList& peakList = (*iter)->get_peakList();
		const PeakMomDadMap& BList = (*iter)->get_BList();

	for(PeakList::const_iterator iter = peakList.begin(); iter != peakList.end(); iter++)
	{
    	PeakMomDadMap::const_iterator i1l = BList.lower_bound(&(*iter));
    	PeakMomDadMap::const_iterator i1u = BList.upper_bound(&(*iter));

    		for(PeakMomDadMap::const_iterator i1 = i1l; i1 != i1u; i1++)
		{

   		atom* a = i1->second.first->get_repAtom();
 			atom* b = i1->second.second->get_repAtom();

 			if (a < b)
 				mdSetM2.insert(make_pair(a,b));
 			else
 				mdSetM2.insert(make_pair(b,a));

 			if (a < b)
 				mdPAMapM2.insert(make_pair(make_pair(a,b), *i1)); // assignments
 			else
 				mdPAMapM2.insert(make_pair(make_pair(b,a), *i1));
     		}
	}

	}
}

// used by filter function
AtomShift NOESY_Assign::BuildXatomShift(const MomDad& md, const MomDadPeakAtomMap& mdPAMap)
{
		AtomShift XatomShift;

		for(MomDadPeakAtomMap::const_iterator i = mdPAMap.lower_bound(md); i != mdPAMap.upper_bound(md); ++i)
		{
		  if (DEBUG)
		  {
		  	NOESY_Assign::outputLog() << "BuildXatomShift " << i->second.first << endl
		  		<< i->second.second.first << endl
		  		<< i->second.second.second << endl;
		  }

			//hx1,hx2
			float h1diff = i->second.second.first->get_HdiffAb(i->second.first->get_HX1(), i->second.first->SWSF_HX1());
			float h2diff = i->second.second.second->get_HdiffAb(i->second.first->get_HX2(), i->second.first->SWSF_HX2());

			if ((XatomShift.find(i->second.second.first) == XatomShift.end()))
			{
				vector<float> vf;
				vf.push_back(h1diff);
				XatomShift.insert(make_pair(i->second.second.first, vf));
			} else
			{
				XatomShift.find(i->second.second.first)->second.push_back(h1diff);
			}
			if ((XatomShift.find(i->second.second.second) == XatomShift.end()))
			{
				vector<float> vf;
				vf.push_back(h2diff);
				XatomShift.insert(make_pair(i->second.second.second, vf));
			} else
			{
				XatomShift.find(i->second.second.second)->second.push_back(h2diff);
			}

			// find all c/n-atoms and all difference from c/n-atoms
			atom* a1 = i->second.second.first->att_X(i->second.first->get_X1(), i->second.first->SWSF_X1());
			float a1diff = i->second.second.first->get_XdiffAb(i->second.first->get_X1(), i->second.first->SWSF_X1());

			atom* a2 = i->second.second.second->att_X(i->second.first->get_X2(), i->second.first->SWSF_X2());
			float a2diff = i->second.second.second->get_XdiffAb(i->second.first->get_X2(), i->second.first->SWSF_X2());

			if ((a1) && (XatomShift.find(a1) == XatomShift.end()))
			{
				vector<float> vf;
				vf.push_back(a1diff);
				XatomShift.insert(make_pair(a1, vf));
			} else if (a1)
			{
				XatomShift.find(a1)->second.push_back(a1diff);
			}
			if ((a2) && (XatomShift.find(a2) == XatomShift.end()))
			{
				vector<float> vf;
				vf.push_back(a2diff);
				XatomShift.insert(make_pair(a2, vf));
			} else if (a2)
			{
				XatomShift.find(a2)->second.push_back(a2diff);
			}
		} // end for

		for(map<atom*, vector<float>, less<atom*> >::iterator i = XatomShift.begin(); i != XatomShift.end(); ++i)
		{
			sort(i->second.begin(), i->second.end());
		}
		return XatomShift;
}

////////////////////////////
// removed the assignment with large tolerance for support analysis and symmetry analysis
// some effect:
// if one assignment matches to multiple peaks, keep the better matched one
// need to refine the procedure
// problem: -0.027,-0.023, 0.023, 0.039, median = 0.023 and reject -0.027, -0.023
////////////////////////////
void NOESY_Assign::filter(const MomDadSet& mdSet, MomDadPeakAtomMap& mdPAMap)
{
 	MomDadPeakAtomMap tmdPAMap; tmdPAMap.clear(); // the erasing list

	for(MomDadSet::iterator mdsi = mdSet.begin(); mdsi != mdSet.end(); ++mdsi)
	{
		if (mdsi->first == mdsi->second)
			continue;

		if (DEBUG)
		{
			NOESY_Assign::outputLog() << "Filter " << mdsi->first << endl
				<< mdsi->second << " " << mdsi->first->ambiDegree() * mdsi->second->ambiDegree() << endl;
		}

		AtomShift XatomShift; // for hx1, hx2, x1, x2
		XatomShift.clear();

		XatomShift = BuildXatomShift(*mdsi, mdPAMap);

 		// 1. remove the outliners for each hx
   		tmdPAMap.clear();
 		for(MomDadPeakAtomMap::iterator i = mdPAMap.lower_bound(*mdsi); i != mdPAMap.upper_bound(*mdsi); ++i)
 		{
			//hx1,hx2
			float h1diff = i->second.second.first->get_HdiffAb(i->second.first->get_HX1(), i->second.first->SWSF_HX1());
			float h2diff = i->second.second.second->get_HdiffAb(i->second.first->get_HX2(), i->second.first->SWSF_HX2());
 			float median; median = 1000;

 			if ((XatomShift.find(i->second.second.first)->second.size() > 2))
 			{
			  median = XatomShift.find(i->second.second.first)->second[XatomShift.find(i->second.second.first)->second.size()/2];
 			} else
 			{
 				for(vector<float>::iterator j = XatomShift.find(i->second.second.first)->second.begin(); j != XatomShift.find(i->second.second.first)->second.end(); ++j)
 				{
 					if (fabs(median) > fabs(*j))
 					{
 						median = *j;
 					}
 				}
 			}
 			if (fabs(h1diff-median) > _par.sym.hsym + 0.001)
 			{
 				tmdPAMap.insert(*i);
	 			if (DEBUG)
				  NOESY_Assign::outputLog() << "ERASE ourliner HX1:" << i->second.second.first << endl << i->second.first << " " << h1diff << " " << median << " " << fabs(h1diff - median) << endl;
 			}

 			median = 1000;
 			if ((XatomShift.find(i->second.second.second)->second.size() > 2))
 			{
 				median = XatomShift.find(i->second.second.second)->second[XatomShift.find(i->second.second.second)->second.size()/2];
 			} else
 			{
 				for(vector<float>::iterator j = XatomShift.find(i->second.second.second)->second.begin(); j != XatomShift.find(i->second.second.second)->second.end(); ++j)
 				{
 					if (fabs(median) > fabs(*j))
 					{
 						median = *j;
 					}
 				}
 			}
  			if (fabs(h2diff - median) > _par.sym.hsym + 0.001)
 			{
 				tmdPAMap.insert(*i);
	 			if (DEBUG)
 					NOESY_Assign::outputLog() << "ERASE ourliner HX2:" << i->second.second.second << endl << i->second.first << " " << fabs(h2diff - median) << endl;
 			}

 			// find all c/n-atoms and all difference from c/n-atoms
 			atom* a1 = i->second.second.first->att_X(i->second.first->get_X1(), i->second.first->SWSF_X1());
 			float a1diff = i->second.second.first->get_XdiffAb(i->second.first->get_X1(), i->second.first->SWSF_X1());

 			atom* a2 = i->second.second.second->att_X(i->second.first->get_X2(), i->second.first->SWSF_X2());
 			float a2diff = i->second.second.second->get_XdiffAb(i->second.first->get_X2(), i->second.first->SWSF_X2());

 			median = 1000;
 			if (a1)
 			{
 				if (XatomShift.find(a1)->second.size() > 2)
 				{
 					median = XatomShift.find(a1)->second[XatomShift.find(a1)->second.size()/2];
 				}else
   			{
   				for(vector<float>::iterator j = XatomShift.find(a1)->second.begin(); j != XatomShift.find(a1)->second.end(); ++j)
   				{
   					if (fabs(median) > fabs(*j))
   					{
   						median = *j;
   					}
   				}
   			}
 				if (fabs(a1diff - median) > _par.sym.xsym + 0.01)
 				{
	 				tmdPAMap.insert(*i);
					if (DEBUG)
   					NOESY_Assign::outputLog() << "ERASE outliner X1:" << a1 << endl << i->second.first << " " << fabs(a1diff - median) << endl;
 				}
 			}

 			median = 1000;
 			if (a2)
 			{
 				if (XatomShift.find(a2)->second.size() > 2)
 				{
 					median = XatomShift.find(a2)->second[XatomShift.find(a2)->second.size()/2];
 				} else
   			{
   				for(vector<float>::iterator j = XatomShift.find(a2)->second.begin(); j != XatomShift.find(a2)->second.end(); ++j)
   				{
   					if (fabs(median) > fabs(*j))
   					{
   						median = *j;
   					}
   				}
   			}
				if (fabs(a2diff - median) > _par.sym.xsym + 0.01)
 				{
	 				tmdPAMap.insert(*i);
					if (DEBUG)
   					NOESY_Assign::outputLog() << "ERASE outliner X2:" << a2 << endl << i->second.first << " " << fabs(a2diff - median) << endl;
 				}
 			}
 		} // end for

 		for(MomDadPeakAtomMap::iterator i = tmdPAMap.begin(); i != tmdPAMap.end(); ++i)
   	{
   		for(MomDadPeakAtomMap::iterator j = mdPAMap.lower_bound(i->first); j != mdPAMap.upper_bound(i->first); ++j)
   		{
   			if (j->second == i->second)
   			{
   				mdPAMap.erase(j);
   				break;
   			}
  		}
   	}

   	tmdPAMap.clear();
		XatomShift.clear();
		XatomShift = BuildXatomShift(*mdsi, mdPAMap);

 		// 2. (-0.03, 0.03)
		// some effect |hx1-hx2| < 0.03 remove the one that are not symmetric
 		for (map<atom*, vector<float>, less<atom*> >::iterator si = XatomShift.begin(); si != XatomShift.end(); ++si)
 		{

  		int max = si->second.size()-1;
  		int min = 0;

  		while (((si->first->get_type() == 'H') && (si->second[max] - si->second[min] > _par.sym.hsym + 0.001)) ||
  					 ((si->first->get_type() != 'H') && (si->second[max] - si->second[min] > _par.sym.xsym + 0.01)))
  		{

  			float median = 0;
  			if (si->second.size() > 2)
  				median = si->second[(max-min+1)/2];

  			float d1 = si->second[max] - median;
  			float d2 = median - si->second[min];

  			if (d1 > d2)
  			{
  				// remove the peaks with d1
  				for(MomDadPeakAtomMap::iterator i = mdPAMap.lower_bound(*mdsi); i != mdPAMap.upper_bound(*mdsi); ++i)
  				{
    				if ((si->first->get_type() == 'H'))
    				{
   						//hx1,hx2
   						float hdiff = -10000;
   						if (si->first == i->second.second.first)
   							hdiff = i->second.second.first->get_HdiffAb(i->second.first->get_HX1(), i->second.first->SWSF_HX1());
   						else if (si->first == i->second.second.second)
  						  hdiff = i->second.second.second->get_HdiffAb(i->second.first->get_HX2(), i->second.first->SWSF_HX2());

   						if (fabs(hdiff -(d1 + median)) < 0.0001)
   						{
   							tmdPAMap.insert(*i);
   							--max;
							if (DEBUG)
     	 						NOESY_Assign::outputLog() << "ERASE hx-sym:" << si->first << endl << i->second.first << endl;
   						}
   					} else
   					{
   						// find all c/n-atoms and all difference from c/n-atoms
   						float xdiff = -10000;
   						atom* a1 = i->second.second.first->att_X(i->second.first->get_X1(), i->second.first->SWSF_X1());
   						atom* a2 = i->second.second.second->att_X(i->second.first->get_X2(), i->second.first->SWSF_X2());

   						if ( a1 && (si->first == a1))
   							xdiff = i->second.second.first->get_XdiffAb(i->second.first->get_X1(), i->second.first->SWSF_X1());
   						else if (a2 && (si->first == a2))
   							xdiff = i->second.second.second->get_XdiffAb(i->second.first->get_X2(), i->second.first->SWSF_X2());
   						if (fabs(xdiff - (d1 + median)) < 0.0001)
   						{
   							tmdPAMap.insert(*i);
   							--max;
							if (DEBUG)
     	 						NOESY_Assign::outputLog() << "ERASE x-sym:" << si->first << endl << i->second.first << endl;
   						}
   					}
 					}
  			} else
  			{
  				// remove the peaks with d2
  				for(MomDadPeakAtomMap::iterator i = mdPAMap.lower_bound(*mdsi); i != mdPAMap.upper_bound(*mdsi); ++i)
  				{
    				if ((si->first->get_type() == 'H'))
    				{
   						//hx1,hx2
   						float hdiff = -10000;
   						if (si->first == i->second.second.first)
   							hdiff = i->second.second.first->get_HdiffAb(i->second.first->get_HX1(), i->second.first->SWSF_HX1());
   						else if (si->first == i->second.second.second)
  						  hdiff = i->second.second.second->get_HdiffAb(i->second.first->get_HX2(), i->second.first->SWSF_HX2());

   						if (fabs(hdiff - (median - d2)) < 0.0001)
   						{
   							tmdPAMap.insert(*i);
   							++min;
							if (DEBUG)
     	 						NOESY_Assign::outputLog() << min << " ERASE d2 hx-sym:" << si->first << endl << i->second.first << endl;
   						}
   					} else
   					{
   						// find all c/n-atoms and all difference from c/n-atoms
   						float xdiff = -10000;
   						atom* a1 = i->second.second.first->att_X(i->second.first->get_X1(), i->second.first->SWSF_X1());
   						atom* a2 = i->second.second.second->att_X(i->second.first->get_X2(), i->second.first->SWSF_X2());

   						if ( a1 && (si->first == a1))
   							xdiff = i->second.second.first->get_XdiffAb(i->second.first->get_X1(), i->second.first->SWSF_X1());
   						else if (a2 && (si->first == a2))
   							xdiff = i->second.second.second->get_XdiffAb(i->second.first->get_X2(), i->second.first->SWSF_X2());
   						if (fabs(xdiff - (median - d2)) < 0.0001)
   						{
   							tmdPAMap.insert(*i);
   							++min;
							if (DEBUG)
     	 						NOESY_Assign::outputLog() << min << "ERASE d2 x-sym:" << si->first << endl << i->second.first << endl;
   						}
   					}
 					}
  			}
  		} // end while
 		} // end for

 		tmdPAMap.clear();
	}
}

void NOESY_Assign::BuildSYMContact(const MomDadSet& mdSetM2, const MomDadPeakAtomMap& mdPAMapM2, SUPPORT& support)
{
//	MomDadPeakAtomMap mdPAMapM; mdPAMapM.clear(); // supports (need to reduce noise support)
//																								// include 1) assigned peak list-assignments
//																								//         2) unassigned peaks with possible assignments
//																								//        x3) do not need to have at least one-pair of sym (n15-noesy)
//																								//         4) backbone or both hydrophoic or both hydrophilic

	////////////////////////////////////////////////////
	// group two- three- four-bond connected contacts //
	// group by contact bin														//
	// for filtering                                  //
	////////////////////////////////////////////////////

	//MomDadSet mdSet12assigned; mdSet12assigned.clear();
	for(MomDadSet::iterator mdsi = mdSetM2.begin(); mdsi != mdSetM2.end(); ++mdsi)
	{
 		// build ASset
 		MomDadSet ASset; ASset.clear();
 		multimap<atom*, pair<atom*, pair<float, float> >, less<atom*> > CHset1; CHset1.clear(); // attached dimension

 		for(MomDadPeakAtomMap::const_iterator mdmi = mdPAMapM2.lower_bound(*mdsi); mdmi != mdPAMapM2.upper_bound(*mdsi); ++mdmi)
 		{
			for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
			{
				//const PeakList& peakList = (*iter)->get_peakList();
				const PeakMomDadMap& BList = (*iter)->get_BList();

				if (BList.find(mdmi->second.first) == BList.end())
					continue;

				const PeakAssignReason& AssignList = (*iter)->get_AssignList();

				if (AssignList.find(mdmi->second.first) != AssignList.end()) // 1) assigned peak list
				{
					for(PeakAssignReason::const_iterator il = AssignList.lower_bound(mdmi->second.first); il != AssignList.upper_bound(mdmi->second.first); ++il)
					{
						if (il->second.first == mdmi->second.second)
						{
					 		ASset.insert(mdmi->second.second);
              if (DEBUG)
                NOESY_Assign::outputLog() << "BuildSYMContact assigned peak" << mdmi->second.first << endl;

   						atom* a1 = mdmi->second.second.first->att_X(mdmi->second.first->get_X1(), mdmi->second.first->SWSF_X1());
      						CHset1.insert(make_pair(mdmi->second.second.first, make_pair(a1, make_pair(mdmi->second.first->get_X1(), mdmi->second.first->SWSF_X1()))));
      						break;
						}
					}
				} else // 2) unassigned peaks, only the one with good matches included
				{
 					ASset.insert(mdmi->second.second);
          if (DEBUG)
            NOESY_Assign::outputLog() << "BuildSYMContact unassigned peak" << mdmi->second.first << endl;

 					atom* a1 = mdmi->second.second.first->att_X(mdmi->second.first->get_X1(), mdmi->second.first->SWSF_X1());
    					CHset1.insert(make_pair(mdmi->second.second.first, make_pair(a1, make_pair(mdmi->second.first->get_X1(), mdmi->second.first->SWSF_X1()))));
				}

				break;
			} // end NOEPeakList;
 		}

 		if (ASset.size() == 0)
 			continue;

  	int i = mdsi->first->get_index();
  	int j = mdsi->second->get_index();
  	pair<int, int> ij;
  	if (i<j) ij = make_pair(i,j);
  	else	ij = make_pair(j,i);

 		//ContactBin <-- at least of pair of cSym from the same carbon pairs
 		multimap<atom*, pair<atom*, pair<float, float> >, less<atom*> >::iterator ch11l; ch11l = CHset1.lower_bound(ASset.begin()->first);
 		multimap<atom*, pair<atom*, pair<float, float> >, less<atom*> >::iterator ch11u; ch11u = CHset1.upper_bound(ASset.begin()->first);
 		multimap<atom*, pair<atom*, pair<float, float> >, less<atom*> >::iterator ch12l; ch12l = CHset1.lower_bound(ASset.begin()->second);
 		multimap<atom*, pair<atom*, pair<float, float> >, less<atom*> >::iterator ch12u; ch12u = CHset1.upper_bound(ASset.begin()->second);

 		MomDadSet::iterator csi;
 		for(csi = ASset.begin(); csi != ASset.end(); ++csi)
 		{
 			if (csi == ASset.begin())
 				continue;

 			multimap<atom*, pair<atom*, pair<float, float> >, less<MomDad> >::iterator ch11i;
 			multimap<atom*, pair<atom*, pair<float, float> >, less<MomDad> >::iterator ch12i;
 			for(ch11i = ch11l; ch11i != ch11u; ++ch11i)
 			{
 				for(ch12i = ch12l; ch12i != ch12u; ++ch12i)
 				{
 					if ((csi->first->att_X(ch12i->second.second.first, ch12i->second.second.second) == ch12i->second.first) &&
 							(csi->second->att_X(ch11i->second.second.first, ch11i->second.second.second) == ch11i->second.first))
 					{
 						break;
 					}
 				}
 				if (ch12i != ch12u)
 					break;
 			}

 			if (ch11i != ch11u)
				break;
 		}

 		if (csi != ASset.end())
 		{
 			// 4) backbone to backbone or both hydrophobic or both hydrophlic
			{
	 			//mdSet12assigned.insert(*mdsi);
  			support.ContactBin.insert(ij);
        if (DEBUG)
          NOESY_Assign::outputLog() << "support IN CONTACT BINS" << endl;
  		}
 		}

 		// Evidence
 		// ASset.size() >=2, excluding cases that possible assignments with ambi = 1, and only observe 1
		if (((mdsi->first->ambiDegree() * mdsi->second->ambiDegree() <= ASset.size()) && (ASset.size() >=2)))
 		{
  		support.MomDadBin.insert(make_pair(ij, *mdsi));
  		if (DEBUG)
  			NOESY_Assign::outputLog() << "support IN MomDadBin " << ij.first << " " << ij.second << " " << ASset.size() << endl
  																<< mdsi->first << endl << mdsi->second << endl;
  	}
  	else
  	{
  		if (DEBUG)
  			NOESY_Assign::outputLog() << "support NOT IN MomDadBin " << ij.first << " " << ij.second << " " << ASset.size() << endl
  																<< mdsi->first << endl << mdsi->second << endl;
  	}
	}
}

void NOESY_Assign::BuildmdPAMapM(const Sym& parSym, const MomDadPeakAtomMap& mdPAMapM2, SUPPORT& support)
{
	for(ContactPairSet::const_iterator ci = support.ContactBin.begin(); ci != support.ContactBin.end(); ++ci)
	{
		for(ContactMomDadMap::const_iterator mi = support.MomDadBin.lower_bound(*ci); mi != support.MomDadBin.upper_bound(*ci); ++mi)
		{
			//MomDadSet mdSetMtemp; mdSetMtemp.clear();
			MomDadPeakAtomMap mdPAMapMtemp; mdPAMapMtemp.clear();
			for(MomDadPeakAtomMap::const_iterator i = mdPAMapM2.lower_bound(mi->second); i != mdPAMapM2.upper_bound(mi->second); ++i)
			{
				for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
				{
					const PeakMomDadMap& BList = (*iter)->get_BList();

					if (BList.find(i->second.first) == BList.end())
						continue;

					const PeakAssignReason& AssignList = (*iter)->get_AssignList();

				if (AssignList.find(i->second.first) != AssignList.end()) // 1) assigned peak list
				{
					for(PeakAssignReason::const_iterator il = AssignList.lower_bound(i->second.first); il != AssignList.upper_bound(i->second.first); ++il)
					{
						if (il->second.first == i->second.second)
						{
      				mdPAMapMtemp.insert(make_pair(support.MomDadBin.find(*ci)->second, i->second));
      				break;
						}
					}
				} else // 2) unassigned peaks
				{
				  if ((i->second.second.first->get_Hdiff(i->second.first->get_HX1(), i->second.first->SWSF_HX1()) < parSym.htol + 0.000001) &&
				      (i->second.second.second->get_Hdiff(i->second.first->get_HX2(), i->second.first->SWSF_HX2()) < parSym.htol + 0.000001) &&
				      (i->second.second.first->get_Xdiff(i->second.first->get_X1(), i->second.first->SWSF_X1()) < parSym.xtol + 0.000001) &&
				      (i->second.second.second->get_Xdiff(i->second.first->get_X2(), i->second.first->SWSF_X2()) < parSym.xtol + 0.000001))
				    {
				      mdPAMapMtemp.insert(make_pair(support.MomDadBin.find(*ci)->second, i->second));
				    }
				} // end if

				} // end for NOEPeakList
			} // end for

			for(MomDadPeakAtomMap::iterator mdi = mdPAMapMtemp.begin(); mdi != mdPAMapMtemp.end(); ++mdi)
			  {
			    support.mdPAMapM.insert(*mdi);
			    support.mdSetM.insert(mdi->first);
			  }
		}
	}
}

bool smallRes(char c) // <= three atoms from alpha to side chains
{
	if ((c == 'A') || (c == 'G') || (c == 'D') || (c == 'C') || (c == 'S') || (c == 'T') )
		return 1;
	else return 0;
}

bool tail(const atom* const a)
{
	if ((a->get_sym_1() == 'M') && (a->get_name() == "HE"))
		return 1;
	if ((a->get_sym_1() == 'K') && (a->get_layer() == "E"))
		return 1;
	if ((a->get_sym_1() == 'R') && (a->get_layer() == "E"))
		return 1;
	if ((a->get_sym_1() == 'R') && (a->get_layer()[0] == 'H'))
		return 1;
	if ((a->get_sym_1() == 'Q') && (a->get_layer()[0] == 'E'))
		return 1;
	if ((a->get_sym_1() == 'W') && (a->get_layer()[0] == 'Z'))
		return 1;
	if ((a->get_sym_1() == 'W') && (a->get_layer()[0] == 'H'))
		return 1;
	if ((a->get_sym_1() == 'F') && (a->get_layer()[0] == 'Z'))
		return 1;
	return 0;
}

void NOESY_Assign::Find_4SYM(const Sym& parSym, MomDadSet& mdSet4, MomDadSet& mdSetO, const MomDadSet& mdSetM2, const MomDadPeakAtomMap& mdPAMapM2,SUPPORT& support)
{
  if (DEBUG) NOESY_Assign::outputLog() << "\n-> In Find_4SYM" << endl;

  NOESY_Assign::outputExm() << endl << "# Possible Symmetric Peaks " << endl;

	// find repAtoms that have at lest 2 assignments and symmetric
	MomDadSet mdSet12; mdSet12.clear();
	for(MomDadSet::iterator mdsi = mdSetM2.begin(); mdsi != mdSetM2.end(); ++mdsi)
	{
 		MomDadSet ASset; ASset.clear();
 		for(MomDadPeakAtomMap::const_iterator mdmi = mdPAMapM2.lower_bound(*mdsi); mdmi != mdPAMapM2.upper_bound(*mdsi); ++mdmi)
 		{
 			ASset.insert(mdmi->second.second);
 		}

		set<atom*, less<atom*> > Xatoms; //
		Xatoms.clear();

		for(MomDadPeakAtomMap::const_iterator i = mdPAMapM2.lower_bound(*mdsi); i != mdPAMapM2.upper_bound(*mdsi); ++i)
		{
			atom* a1 = i->second.second.first->att_X(i->second.first->get_X1(), i->second.first->SWSF_X1());

			if (a1)
			{
				Xatoms.insert(a1);
			}
		} // end for

 		if ((ASset.size() >= 2) && (Xatoms.size() >= 2)) // two x1 (symmetric)
 		{
 			mdSet12.insert(*mdsi);
 		}
	}

	///////////////////////////////////////////////
	//  complete-4SYM
	//////////////////////////////////////////////

	// find repAtoms that have at least 6/8 assignments,
	//MomDadSet mdSet4;
	mdSet4.clear();
	PeakPtrMomDadMap pmdMap; pmdMap.clear();
	PeakPtrMomDadSet pmdSet; pmdSet.clear();
	for(MomDadSet::iterator mdsi = mdSet12.begin(); mdsi != mdSet12.end(); ++mdsi)
	{
		if (mdsi->first->ambiDegree() * mdsi->second->ambiDegree() == 4)
		{
  		MomDadSet ASset; ASset.clear();
  		for(MomDadPeakAtomMap::const_iterator mdmi = mdPAMapM2.lower_bound(*mdsi); mdmi != mdPAMapM2.upper_bound(*mdsi); ++mdmi)
  		{
  			ASset.insert(mdmi->second.second);
  		}

  		if (ASset.size() >= 6)
  		{
  			mdSet4.insert(*mdsi);

  			for(MomDadPeakAtomMap::const_iterator mdpi = mdPAMapM2.lower_bound(*mdsi); mdpi != mdPAMapM2.upper_bound(*mdsi); ++mdpi)
  			{
      		pmdMap.insert(make_pair(mdpi->second.first, *mdsi));
      		pmdSet.insert(mdpi->second.first);
      	}
  		}
		}
	}

	if (DEBUG)
	{
  	// find the unique 4-pairs
  	for(MomDadSet::iterator mdsi = mdSet4.begin(); mdsi != mdSet4.end(); ++mdsi)
  	{
  		for(MomDadPeakAtomMap::const_iterator mdmi = mdPAMapM2.lower_bound(*mdsi); mdmi != mdPAMapM2.upper_bound(*mdsi); ++mdmi)
  		{
  			if (DEBUG)
  				NOESY_Assign::outputLog() << "4SYM: 4-pairs " << mdmi->second.first << endl << mdmi->second.second.first << endl << mdmi->second.second.second << endl;
  		}
   		if (DEBUG)
  			NOESY_Assign::outputLog() << endl;
  	}
	}

	// find the unique one
	//MomDadSet tmdSet4; tmdSet4 = mdSet4;
	MomDadSet tmdSet4(mdSet4);
	for(MomDadSet::iterator mdsi = tmdSet4.begin(); mdsi != tmdSet4.end(); ++mdsi)
	{

 		MomDadSet ASset; ASset.clear();
 		for(MomDadPeakAtomMap::const_iterator mdmi = mdPAMapM2.lower_bound(*mdsi); mdmi != mdPAMapM2.upper_bound(*mdsi); ++mdmi)
 		{
 			ASset.insert(mdmi->second.second);
 		}

 		if ((ASset.size() ==8) && (mdsi->first->ambiDegree() * mdsi->second->ambiDegree() == 4))
 		{
  		for(MomDadPeakAtomMap::const_iterator mdpi = mdPAMapM2.lower_bound(*mdsi); mdpi != mdPAMapM2.upper_bound(*mdsi); ++mdpi)
  		{
  			if (pmdMap.count(mdpi->second.first) > 1)
  			{
  				if (mdSet4.find(*mdsi) != mdSet4.end())
  					mdSet4.erase(mdSet4.find(*mdsi));
  				break;
  			}

    			// match tolerance
    			if ((mdpi->second.second.first->get_Hdiff(mdpi->second.first->get_HX1(), mdpi->second.first->SWSF_HX1()) < parSym.htol + 0.000001) &&
			    (mdpi->second.second.second->get_Hdiff(mdpi->second.first->get_HX2(), mdpi->second.first->SWSF_HX2()) < parSym.htol + 0.000001) &&
			    (mdpi->second.second.first->get_Xdiff(mdpi->second.first->get_X1(), mdpi->second.first->SWSF_X1()) < parSym.xtol + 0.000001) &&
			    (mdpi->second.second.second->get_Xdiff(mdpi->second.first->get_X2(), mdpi->second.first->SWSF_X2()) < parSym.xtol + 0.000001))
			  {
			  }
			else
			  {
			    if (DEBUG) NOESY_Assign::outputLog() << "not well-matched" << endl;
			    if (mdSet4.find(*mdsi) != mdSet4.end())
			      mdSet4.erase(mdSet4.find(*mdsi));
			  }
     }
    }else
    {
    		mdSet4.erase(mdSet4.find(*mdsi));
    }
	}

	if (DEBUG)
	{
		//NOESY_Assign::outputLog() << "Unique 4-pairs" << endl;

  	for(MomDadSet::iterator mdsi = mdSet4.begin(); mdsi != mdSet4.end(); ++mdsi)
  	{
  		for(MomDadPeakAtomMap::const_iterator mdmi = mdPAMapM2.lower_bound(*mdsi); mdmi != mdPAMapM2.upper_bound(*mdsi); ++mdmi)
  		{
  			if (DEBUG)
  				NOESY_Assign::outputLog() << "4SYM: Unique 4-pairs "  << mdmi->second.first << endl << mdmi->second.second.first << endl << mdmi->second.second.second << endl;
  		}
   		if (DEBUG)
  			NOESY_Assign::outputLog() << endl;
  	}
	}

	///////////////////////////////////////////////
	//  comple 2-SYM (not have sub-1-SYM group)
	//////////////////////////////////////////////

	//MomDadSet mdSetO;
	mdSetO.clear();
	PeakPtrMomDadMap pmdMapO; pmdMapO.clear();
	PeakPtrMomDadSet pmdSetO; pmdSetO.clear();

	for(MomDadSet::iterator mdsi = mdSet12.begin(); mdsi != mdSet12.end(); ++mdsi)
	{
		// some times HB* overlapped with HA ambi=4 and ambi=2 are overlapped

  		MomDadSet ASset; ASset.clear();
  		for(MomDadPeakAtomMap::const_iterator mdmi = mdPAMapM2.lower_bound(*mdsi); mdmi != mdPAMapM2.upper_bound(*mdsi); ++mdmi)
  		{
  			ASset.insert(mdmi->second.second);
  		}

  		if ((ASset.size() >= 2))
  		{
  			if ((mdsi->first->ambiDegree() * mdsi->second->ambiDegree() != 4) || (ASset.size() >= 4)) // if expected 4-pairs, only see one pairs, ignore them
  				mdSetO.insert(*mdsi);

  			for(MomDadPeakAtomMap::const_iterator mdpi = mdPAMapM2.lower_bound(*mdsi); mdpi != mdPAMapM2.upper_bound(*mdsi); ++mdpi)
  			{
      		pmdMapO.insert(make_pair(mdpi->second.first, *mdsi));
      		pmdSetO.insert(mdpi->second.first);
      	}
  		}

	}

	if (DEBUG)
	{
		//NOESY_Assign::outputLog() << "Finding Other-pairs" << endl;

  	// find Other-pairs
  	for(MomDadSet::iterator mdsi = mdSetO.begin(); mdsi != mdSetO.end(); ++mdsi)
  	{
  		for(MomDadPeakAtomMap::const_iterator mdmi = mdPAMapM2.lower_bound(*mdsi); mdmi != mdPAMapM2.upper_bound(*mdsi); ++mdmi)
  		{
  			if (DEBUG)
  				NOESY_Assign::outputLog() << "4SYM: Other-pairs " << mdmi->second.first << endl << mdmi->second.second.first << endl << mdmi->second.second.second << endl;
  		}
  		if (DEBUG)
  			NOESY_Assign::outputLog() << endl;
  	}
	}

	// find the unique one
	//MomDadSet tmdSetO; tmdSetO = mdSetO;
	MomDadSet tmdSetO(mdSetO);
	for(MomDadSet::iterator mdsi = tmdSetO.begin(); mdsi != tmdSetO.end(); ++mdsi)
	{

 		MomDadSet ASset; ASset.clear();
 		for(MomDadPeakAtomMap::const_iterator mdmi = mdPAMapM2.lower_bound(*mdsi); mdmi != mdPAMapM2.upper_bound(*mdsi); ++mdmi)
 		{
 			ASset.insert(mdmi->second.second);
 		}

 		if ((ASset.size() == 4) && (mdsi->first->ambiDegree() * mdsi->second->ambiDegree() == 2))
 		{
  		for(MomDadPeakAtomMap::const_iterator mdpi = mdPAMapM2.lower_bound(*mdsi); mdpi != mdPAMapM2.upper_bound(*mdsi); ++mdpi)
  		{
  			if ((pmdMapO.count(mdpi->second.first) > 1) || (pmdMap.count(mdpi->second.first) != 0)) // || // not a subset of 4-sym
  			{
  				if (mdSetO.find(*mdsi) != mdSetO.end())
  					mdSetO.erase(mdSetO.find(*mdsi));
  				break;
  			}

    			// match tolerance
    			if ((mdpi->second.second.first->get_Hdiff(mdpi->second.first->get_HX1(), mdpi->second.first->SWSF_HX1()) < parSym.htol + 0.000001) &&
			    (mdpi->second.second.second->get_Hdiff(mdpi->second.first->get_HX2(), mdpi->second.first->SWSF_HX2()) < parSym.htol + 0.000001) &&
			    (mdpi->second.second.first->get_Xdiff(mdpi->second.first->get_X1(), mdpi->second.first->SWSF_X1()) < parSym.xtol + 0.000001) &&
			    (mdpi->second.second.second->get_Xdiff(mdpi->second.first->get_X2(), mdpi->second.first->SWSF_X2()) < parSym.xtol + 0.000001))
			  {
			  }
			else
			  {
			    if (DEBUG) NOESY_Assign::outputLog() << "not well-matched" << endl;
			    if (mdSetO.find(*mdsi) != mdSetO.end())
			      mdSetO.erase(mdSetO.find(*mdsi));
			  }

		}
    } else
    {
    	mdSetO.erase(mdSetO.find(*mdsi));
    }
	}

	if (DEBUG)
	{

  	// find the unique 2-pairs
  	for(MomDadSet::iterator mdsi = mdSetO.begin(); mdsi != mdSetO.end(); ++mdsi)
  	{
  		for(MomDadPeakAtomMap::const_iterator mdmi = mdPAMapM2.lower_bound(*mdsi); mdmi != mdPAMapM2.upper_bound(*mdsi); ++mdmi)
  		{
  			if (DEBUG)
  				NOESY_Assign::outputLog() << "4SYM: Unique 2-pairs " << mdmi->second.first << endl << mdmi->second.second.first << endl << mdmi->second.second.second << endl;
  		}
  		if (DEBUG)
  			NOESY_Assign::outputLog() << endl;
  	}
	}

}




// check if the contact bin has a lot of peaks supporting
// low means support requirement is low
// for cycle1, = 1
// for hsym - low = 0;
// for iterative analysis low = -1;
bool isSupported(int low, MomDad mdRep, const MomDadSet ASset,
const MomDadPeakAtomMap& mdPAMapM2, const SUPPORT& support, const ContactPairSet& AssignedContactBin)
{
 	pair<int, int> mij; // ij index
 	MomDad mdRepSwap = make_pair(mdRep.second, mdRep.first);

 	MomDad* mdpoint; // ij pair

 	if (mdRep.first->get_index() < mdRep.second->get_index())
 	{
 		mij = make_pair(mdRep.first->get_index(), mdRep.second->get_index());
 		mdpoint = &mdRep;
 	}
 	else
 	{
 		mij = make_pair(mdRep.second->get_index(), mdRep.first->get_index());
 		mdpoint = &mdRepSwap;
 	}

 	bool bb = 0;
	if (((mdRep.first->get_layer() == "N") || (mdRep.first->get_layer() == "A")) && ((mdRep.second->get_layer() == "N") || (mdRep.second->get_layer() == "A")))
    	{
      		if (DEBUG)
        		NOESY_Assign::outputLog() << "support backbone " << mij.first << " " << mij.second << endl;
    		bb = 1;
    	}

 	bool flag = 0; // supported or not

	if (mij.first == mij.second) // intra
    	{
     		if (DEBUG)
     			NOESY_Assign::outputLog() << "support INTRA CONTACT " << endl;
     		flag = 1;
    	}
 	else
 	{
 		// itself should have encough support - at least two atoms from each residue.
 		// except for bb, 4D noesy, and interchain

     		set<atom*, less<atom*> > atomSetmi; atomSetmi.clear(); // total number of h-atoms from residue i
    		set<atom*, less<atom*> > atomSetmj; atomSetmj.clear(); // total number of h-atoms from residue j
    		set<atom*, less<atom*> > atomSetmxi; atomSetmxi.clear(); // total number of x-atoms from residue i
    		set<atom*, less<atom*> > atomSetmxj; atomSetmxj.clear(); // total number of x-atoms from residue j

		// count number of support
     		bool haveAttXi = 0, haveAttXj = 0;
		for(MomDadPeakAtomMap::const_iterator mdmi = support.mdPAMapM.lower_bound(support.MomDadBin.find(mij)->second);
			mdmi != support.mdPAMapM.upper_bound(support.MomDadBin.find(mij)->second); ++mdmi)
    		{
 			if (DEBUG)
 				NOESY_Assign::outputLog() << "support itself peak: " << mdmi->second.first << endl;

			atom* a = mdmi->second.second.first;
			atom* b = mdmi->second.second.second;

			atom* xa = mdmi->second.second.first->att_X(mdmi->second.first->get_X1(), mdmi->second.first->SWSF_X1());
			atom* xb = mdmi->second.second.second->att_X(mdmi->second.first->get_X2(), mdmi->second.first->SWSF_X2());

			if (a->get_index() == mij.first)
			{
				atomSetmi.insert(a);
				atomSetmj.insert(b);

				if (a->isatt_N() || a->isatt_C())
					haveAttXi = 1;
				if (b->isatt_N() || b->isatt_C())
					haveAttXj = 1;

				if (xa)
					atomSetmxi.insert(xa);
				if (xb)
					atomSetmxj.insert(xb);
			}
			else
			{
				atomSetmj.insert(a);
				atomSetmi.insert(b);

				if (a->isatt_N() || a->isatt_C())
					haveAttXj = 1;
				if (b->isatt_N() || b->isatt_C())
					haveAttXi = 1;

				if (xa)
					atomSetmxj.insert(xa);
				if (xb)
					atomSetmxi.insert(xb);
			}
		}

		// test 1 - for cycle 1
		if (((((atomSetmi.size() < 2) && (atomSetmxi.size() < 2)) && (!tail(mdpoint->first)) && (low != -1) && (!smallRes(seq.find(mij.first)->second.get_sym_1())))
			|| (((atomSetmj.size() < 2) && (atomSetmxj.size() < 2)) && (!tail(mdpoint->second)) && (low != -1) && (!smallRes(seq.find(mij.second)->second.get_sym_1())))))
		{
 			if (DEBUG)
 				NOESY_Assign::outputLog() << "self<2 for both ij " << atomSetmi.size() << " " << atomSetmj.size() << endl;
			return 0;
 		}

		if (low != -1)  // test 2 for cycle 1
			if (((atomSetmxi.size() < 1) && haveAttXi) || ((atomSetmxj.size() < 1) && haveAttXj))
			{
 				if (DEBUG)
 					NOESY_Assign::outputLog() << "attached < 1 " << atomSetmxi.size() << " " << atomSetmxj.size() << endl;
				return 0;
 			}

    		// find additional support from adjacent residue pair
 		for(int i = -4; i <= 4; ++i)
 			{
 				for(int j = -4; j <= 4; ++j)
 				{
 					pair<int,int> ij;
 					int ti = i+mdRep.first->get_index();
 					int tj = j+mdRep.second->get_index();
 					if (ti<tj) ij = make_pair(ti,tj);
 					else ij = make_pair(tj,ti);

					// i && j -> in case of missing assignment of residue k
					// a1->k, a2->k becomes a1->c, a2->d
 					if (((abs(i)+abs(j)>=0) && ((abs(i)*abs(j) < 2))) && (support.ContactBin.find(ij) != support.ContactBin.end()))
 					{
 						if (!low && (abs(i)+abs(j) == 0)) // for hSYM only low=0
 							continue;

 						if (!low && (abs(i)+abs(j) >= 4)) // for hSYM
 							continue;

 						if ((low < 0) && (abs(i)+abs(j) >= 1)) // for iterative analysis, looking for itself support
 							continue;

 						if ((i*j==0) && (abs(i+j) == 1)) // no sequencial i,i+1 to j support
 							continue;

 						if (DEBUG)
 							NOESY_Assign::outputLog() << "support contact " << low << " " << ij.first << " " << ij.second << " " << flag << endl;

						if ((low > -1) && !sameHydro(seq.find(ij.first)->second.get_sym_1(), seq.find(ij.second)->second.get_sym_1()))
						{
			 				if (DEBUG)
 								NOESY_Assign::outputLog() << "not in the same hydro " << endl;
							continue;
 						}

         		if ((low && //(ASset.size() != 1) &&
         			((abs(ij.first - ij.second) <= 2) || (abs(mij.first - mij.second) ==2)) &&
         			((ij.first != mij.first) || (ij.second != mij.second)))) // i,i+1, i,i+2 can not be used as support, excpet for it self
         		{
         			if (DEBUG)
         				NOESY_Assign::outputLog() << "can not support: i,i+1, i,i+2" << endl;
         			continue;
 						}
         		// 72-79 cannot be supported by 76-79
         		// mij.first - mij.second >=5, can not be supported by ij.first-ij.second <=4
         		if (
         			((abs(ij.first - ij.second) <= 4) && (abs(mij.first - mij.second) >4)) &&
         			((ij.first == mij.first) || (ij.second == mij.second))) // i,i+1, i,i+2 can not be used as support, excpet for it self
         		{
         			if (DEBUG)
         				NOESY_Assign::outputLog() << "can not support: subregion" << endl;
         			continue;
         		}

						// for helix, no +1/-1 allowed
						if ((abs(i)+abs(j) == 2))
						{
							// no 1,1, -1,1, -1,-1 for helixcal residues
							if ((seq[mij.first].get_ss()->get_type() == 1) && (seq[mij.second].get_ss()->get_type() == 1) && i && j &&
									(seq[ij.first].get_ss()->get_type() == 1) && (seq[ij.second].get_ss()->get_type() == 1))
							{
			 					if (DEBUG)
 									NOESY_Assign::outputLog() << "helical faces can not support 1+1" << endl;
								continue;
							}
							// no 0-2
							if ((!i||!j) && (((seq[mdRep.first->get_index()].get_ss()->get_type() == 1) && (seq[ti].get_ss()->get_type() == 1) && i) ||
							((seq[mdRep.second->get_index()].get_ss()->get_type() == 1) && (seq[tj].get_ss()->get_type() == 1) && j)))
							{
			 					if (DEBUG)
 									NOESY_Assign::outputLog() << "helical faces can not support 0-2" << endl;
								continue;
							}
						}
						if ((abs(i)+abs(j) > 2))
						{
							// only helical residue can use 0-3,-4
							if ((((seq[mdRep.first->get_index()].get_ss()->get_type() != 1) || (seq[ti].get_ss()->get_type() != 1)) && i) ||
								  (((seq[mdRep.second->get_index()].get_ss()->get_type() != 1) || (seq[tj].get_ss()->get_type() != 1)) && j))
							{
								if (DEBUG)
									NOESY_Assign::outputLog() << i << " " << j << " " << (seq[mdRep.first->get_index()].get_ss()->get_type() != 1) <<
											" " << (seq[mdRep.second->get_index()].get_ss()->get_type() != 1) << endl;
			 					if (DEBUG)
 									NOESY_Assign::outputLog() << "non-helical faces can not support 3,4" << endl;
								continue;
							}
						}


			// ------------------------
           		// all supporting rules start here
         		if (AssignedContactBin.find(ij)!= AssignedContactBin.end())
         		{
    				if (DEBUG)
    					NOESY_Assign::outputLog() << "Assigned CONTACT " << ti << " " << tj << endl;
    				flag = 1;
    				break;
         		}

 			// should have other support or high self-support (for both cycle 1 and iterative analysis)
        		set<atom*, less<atom*> > atomSeti; atomSeti.clear(); // total number of atoms from residue i
        		set<atom*, less<atom*> > atomSetj; atomSetj.clear(); // total number of atoms from residue j
        		multimap<MomDad, MomDad, less<MomDad> > symPair; symPair.clear(); // total sym pair
        		MomDadSet ASset2; ASset2.clear(); // all momdad, index for symPair
        		MomDadSet ASset3; ASset3.clear(); // total number of peaks
        		MomDadSet ijMomDadSet; ijMomDadSet.clear(); // number of subM2 groups

         		for(ContactMomDadMap::const_iterator mi = support.MomDadBin.lower_bound(ij); mi != support.MomDadBin.upper_bound(ij); ++mi)
        		{
				for(MomDadPeakAtomMap::const_iterator mdmi = support.mdPAMapM.lower_bound(mi->second); mdmi != support.mdPAMapM.upper_bound(mi->second); ++mdmi)
          			{
          			if (mdmi->second.second.first < mdmi->second.second.second)
          			{
          				symPair.insert(make_pair(mdmi->second.second, mdmi->second.second));
          				ASset2.insert(mdmi->second.second);
          			} else
          			{
          				ASset2.insert(make_pair(mdmi->second.second.second, mdmi->second.second.first));
          				symPair.insert(make_pair(make_pair(mdmi->second.second.second, mdmi->second.second.first), mdmi->second.second));
          			}

           			//total number of atoms_rep from residue i and j
				atom* a = mdmi->second.second.first->get_repAtom();
				atom* b = mdmi->second.second.second->get_repAtom();

				pair<atom*, atom*> ab;

				if (a>b) ab = make_pair(a,b);
				else 		 ab = make_pair(b,a);

				ijMomDadSet.insert(ab); // number of atoms_rep, subM2

          			// total number of atoms
				a = mdmi->second.second.first;
				b = mdmi->second.second.second;

				if (a->get_index() == ij.first)
					atomSeti.insert(a); //#of atoms
				else
					atomSetj.insert(a); //# of atoms

				if (b->get_index() == ij.first)
					atomSeti.insert(b);
				else
					atomSetj.insert(b);

          			if (DEBUG)
				{
          				NOESY_Assign::outputLog() << "support used: " << mdmi->second.first << endl;
					NOESY_Assign::outputLog() << "ijMomDadSets: " << ab.first << endl << ab.second << endl;
				}
           			ASset3.insert(mdmi->second.second);// # of peaks
          			} // end for mdPAMapM
        		}

   			int ts = 0; //number of sym peaks
   			for(MomDadSet::iterator as2i = ASset2.begin(); as2i != ASset2.end(); ++as2i)
   			{
   				MomDadSet tt; tt.clear();
   				for(multimap<MomDad, MomDad, less<MomDad> >::iterator si = symPair.lower_bound(*as2i); si != symPair.upper_bound(*as2i); ++si)
   				{
   					tt.insert(si->second);
   				}
   				if (tt.size()==2)
   					++ts;
   			}

          		if (DEBUG)
          			NOESY_Assign::outputLog() << "no of sym = " << ts << endl;
          		if (DEBUG)
          			NOESY_Assign::outputLog() << "atomSeti = " << atomSeti.size() << " atomSetj = " << atomSetj.size() << " ijMomDadSet = " << ijMomDadSet.size() << endl;


   			// For cycle 1: have at least one pair of symmetry for this contact bin
   			if ((ts == 0) && (low > 0) )
   			{
          			if (DEBUG)
          				NOESY_Assign::outputLog() << "no of sym == 0" << endl;
   				continue;
   			}

            		// low = -1, for TOP ones, one sym and two subM2 group
			if ((ts) && (low == -1)
				&& (ijMomDadSet.size() >=2)) //repreentative group have two
			{
 				if (DEBUG)
 					NOESY_Assign::outputLog() << "case symPair CONTACT-iterative repAtom >=2 " << ti << " " << tj << endl;
 				flag = 1;
 				break;
			}
            		// if have >=3 atoms, ok, at least three peaks, and two subM2 group
            		if ((low == -1) && (ijMomDadSet.size() >= 2) // in case no sym - for BEST method
            			//&& (atomSeti.size()+atomSetj.size() >= 3)
              			&& (ASset3.size() >=2))
            		{
     				if (DEBUG)
     				NOESY_Assign::outputLog() << "case BEST support atomi+j >=2 " << ti << " " << tj << endl;
     				flag = 1;
     				break;
            		}

            		// for small residues, allow two symmtry pairs.
          		if ((ts >=2) && (smallRes(seq.find(ij.first)->second.get_sym_1()) && smallRes(seq.find(ij.second)->second.get_sym_1())))
     			{
 				if (DEBUG)
 					NOESY_Assign::outputLog() << "case symPair CONTACT-small " << ti << " " << tj << endl;
 				flag = 1;
 				break;
     			}

          		// test 3 - have at least 2 pair of repAtom sym atoms (two subM2 group)
   			if (ts >=2)
   			{
          			if ((atomSeti.size() < 2) )// && (((*atomSeti.begin())->get_layer() == "N") || ((*atomSeti.begin())->get_layer() == "A")))
    				{
            				if (DEBUG)
            					NOESY_Assign::outputLog() << "no of atom from " << ij.first << " < 2" << endl;
    				}
          			else if ((atomSetj.size() < 2) )// && (((*atomSetj.begin())->get_layer() == "N") || ((*atomSetj.begin())->get_layer() == "A")))
     				{
            				if (DEBUG)
            					NOESY_Assign::outputLog() << "no of atom from " << ij.second << " < 2" << endl;
     				} else if (((ts <3) && ((abs(i)+abs(j) == 0)) && !tail(mdpoint->first) && !tail(mdpoint->second)) && (ijMomDadSet.size() < 3))
     				{ // three pairs of symmetry for intra or atleast three pair of atoms
            				if (DEBUG)
            					NOESY_Assign::outputLog() << "no of sym-self(0) " << " < 3 and " << "no of ijMomDadSet < 3 " << endl;
				}
 				else
 				{
 					if (DEBUG)
 						NOESY_Assign::outputLog() << "case symPair CONTACT " << ti << " " << tj << endl;
 					flag = 1;
 					break;
 				}
   			}

		// case Gly - other cases only for residues with few assignments or short side-chains
           	// contact is backbone and one is Gly
           	if ((low > 0) && ((mdRep.first->get_layer() == "N") || (mdRep.first->get_layer() == "A")) &&
           		((mdRep.second->get_layer() == "N") || (mdRep.second->get_layer() == "A")))
           	{
           		if ((mdRep.first->get_sym_1() == 'G') || (mdRep.second->get_sym_1() == 'G'))
           		{
      				if (DEBUG)
      					NOESY_Assign::outputLog() << "case Gly CONTACT BB" << ti << " " << tj << endl;
      				flag = 1;
      				break;
      			}
           	}

           	// case 4D or IC - for 4D noesy only which have lower contact profile
           	// 4D -> 1) many peaks are missing and 2) less ambiguity, resolved by the fourth dimension
          	if ((ijMomDadSet.size() >= 2) && (low > 0))
          	{

     			if (DEBUG)
     				NOESY_Assign::outputLog() << "in >=3 4D " << atomSeti.size() << " " << atomSetj.size() << endl;

          		// at least two atoms from each bb residue,
          		if ((atomSeti.size() < 2))// && (((*atomSeti.begin())->get_layer() == "N") || ((*atomSeti.begin())->get_layer() == "A")))
          			{}
          		else if ((atomSetj.size() < 2))// && (((*atomSetj.begin())->get_layer() == "N") || ((*atomSetj.begin())->get_layer() == "A")))
          			{}
     			else
     			{
     				if (DEBUG)
     					NOESY_Assign::outputLog() << "case 4D or IC CONTACT " << ti << " " << tj << endl;
     				flag = 1;
     				break;
     			}
          	}

        	if (flag)
        	break;
        		//} // end for MomDadBin
 				} // end if supports

 			if (flag)
 				break;
 			} // end for loop j

 		if (flag)
 			break;
 		} // end for loop i
	} // end if

	if (flag)
		return 1;
	else
		return 0;
}

bool isSupported(pair<int,int> mij, const ContactPairSet&  ContactBinUniqueSYM)
{
 		// +/- 2 residue
 		bool flag = 0;
 		{
			//int i = 0; int j = 0;
 			for(int i = -4; i <= 4; ++i)
 			{
 				for(int j = -4; j <= 4; ++j)
 				{
					if (!i && !j)
						continue;

 					pair<int,int> ij;
 					int ti = i+ mij.first; //mdRep.first->get_index();
 					int tj = j+ mij.second; //mdRep.second->get_index();

					if ((ti < 0) || (tj < 0))
					  continue;

 					if (ti<tj) ij = make_pair(ti,tj);
 					else ij = make_pair(tj,ti);


 					if (DEBUG)
					  {
					    NOESY_Assign::outputLog() << " sym " << i << " " << j << " " << ij.first << " " << ij.second << " " << flag << " "  << mij.first << " " << mij.second << endl;
					  NOESY_Assign::outputLog() << faces[ij.first][mij.first] << endl;
					  NOESY_Assign::outputLog() << faces[ij.second][mij.second] << endl;
					  }


          	if ((abs(i)+abs(j) >= 2) && (!i || !j) && ((faces[ij.first][mij.first] == -1) || (faces[ij.second][mij.second] == -1)))
						{
			 				if (DEBUG)
 								NOESY_Assign::outputLog() << "faces can not support" << endl;
							continue;
						}

 					if (DEBUG)
					  NOESY_Assign::outputLog() << "here1" << endl;

					// i && j -> in case of missing assignment of residue k
					// a1->k, a2->k becomes a1->c, a2->d
 					if (((abs(i)+abs(j)>=0) && (abs(i)*abs(j) < 2)) && (ContactBinUniqueSYM.find(ij) != ContactBinUniqueSYM.end()))
 					{
 					        if (DEBUG)
       							NOESY_Assign::outputLog() << "CONTACT UniqueSyM" << ti << " " << tj << endl;
       						flag = 1;
       						//break;
            				}
 					if (DEBUG)
					  NOESY_Assign::outputLog() << "here2" << endl;
            			 	//if (flag)
 					//	break;
 				} // end for

 				//if (flag)
 				//	break;
 			} // end for
		} // end if
	if (flag)
		return 1;
	else
		return 0;
}

void NOESY_Assign::Find_USYM(const Sym& parSym, const MomDadPeakAtomMap& mdPAMapM2, SUPPORT& support,
MomDadPeakAtomMap& mdPAMapA, map<MomDad, MomDad, less<MomDad> >& mdAM2, MomDadSet& mdSetA12)
{
	// build the mapping
	MomDadSet mdSetA; mdSetA.clear(); // group in atom level
	PeakPtrMomDadMap pmdMapA; pmdMapA.clear();
	PeakPtrMomDadMap pmdMap; pmdMap.clear();

	// add the one that symmetry by itself
	mdSetA.clear(); // group in atom level
	pmdMapA.clear();
	pmdMap.clear();
	pmdMap.clear();
	MomDadPeakAtomMap tmdPAMapA; tmdPAMapA.clear();
	set<PeakPtr> symPeak; symPeak.clear();
	for(MomDadPeakAtomMap::const_iterator mi = mdPAMapM2.begin(); mi != mdPAMapM2.end(); ++mi)
	{
		atom* a = mi->second.second.first;
		atom* b = mi->second.second.second;

		if (DEBUG)
		  NOESY_Assign::outputLog() << "mdPAMapM2 USYM : " << mi->second.first << endl << a << endl << b << endl;

		pair<atom*, atom*> ab;

		if (a>b) ab = make_pair(a,b);
		else 		 ab = make_pair(b,a);

		mdSetA.insert(ab);
 		tmdPAMapA.insert(make_pair(ab, mi->second));
 		mdAM2.insert(make_pair(ab, mi->first));
 		pmdMap.insert(mi->second);
	}

	MomDadSet tmdSetA12; tmdSetA12.clear();
	MomDadPeakAtomMap ttmdPAMapA; ttmdPAMapA.clear();
	for(MomDadSet::iterator mdsi = mdSetA.begin(); mdsi != mdSetA.end(); ++mdsi)
	{
	  if (DEBUG) NOESY_Assign::outputLog() << "USYM" << mdsi->first << endl << mdsi->second << endl;
 		MomDadSet ASset; ASset.clear();
    //map<atom*, set<float> > atomshift; atomshift.clear();
    SampleStatistic diffmh1; diffmh1.reset(); // for mom
    SampleStatistic diffdh1; diffdh1.reset(); // for dad
    SampleStatistic diffmx1; diffmx1.reset();
    SampleStatistic diffdx1; diffdx1.reset();

    // another possible
    SampleStatistic diffmh2; diffmh2.reset();
    SampleStatistic diffdh2; diffdh2.reset();

    vector<MomDadPeakAtomMap::iterator> peak1; peak1.clear();
    vector<MomDadPeakAtomMap::iterator> peak2; peak2.clear();
 		for(MomDadPeakAtomMap::iterator mdmi = tmdPAMapA.lower_bound(*mdsi); mdmi != tmdPAMapA.upper_bound(*mdsi); ++mdmi)
 		{
		  if (DEBUG)
		    NOESY_Assign::outputLog() << mdmi->second.first << endl;

    	ASset.insert(mdmi->second.second);

			float diffhx1 = mdmi->second.second.first->get_HdiffAb(mdmi->second.first->get_HX1(), mdmi->second.first->SWSF_HX1());
			float diffhx2 = mdmi->second.second.second->get_HdiffAb(mdmi->second.first->get_HX2(), mdmi->second.first->SWSF_HX2());

			if (mdmi->second.second.first == mdsi->first)
			{
				diffmh1 += diffhx1;
				diffdh1 += diffhx2;
				peak1.push_back(mdmi);
			} else
			{
				diffmh2 += diffhx1;
				diffdh2 += diffhx2;
				peak2.push_back(mdmi);
			}
    }

		if ((ASset.size() < 2)) // not symmetric
		  continue;

    if (DEBUG)
      NOESY_Assign::outputLog() << "find-diff ";

    // must be well symmetric
		// 0.04 (diffhx1 = -0.02 .. 0.02) not (diffhx1=0 ..0.04) for hx1, hx2
		// diffhx1 means for atom - hx1, the match quality
    for(int i = 0; i < peak1.size(); ++i) // possible symmetry peak pairs
      for(int j = 0; j < peak2.size(); ++j)
		{
  			if (DEBUG)
    			NOESY_Assign::outputLog() << peak1[i]->second.first << endl << peak2[j]->second.first << endl
       				<< peak1[i]->second.second.first << endl << peak1[i]->second.second.second << endl;

				float diffmh1minus = fabs(diffmh1.oriSamples()[i] - diffdh2.oriSamples()[j]);
				float diffmh1sum   = fabs(diffmh1.oriSamples()[i] + diffdh2.oriSamples()[j]);
				float diffdh1minus = fabs(diffdh1.oriSamples()[i] - diffmh2.oriSamples()[j]);
				float diffdh1sum   = fabs(diffdh1.oriSamples()[i] + diffmh2.oriSamples()[j]);

				// exlude ones that not well matched
				if ((diffmh1minus+diffdh1minus > 0.069)) // && ((diffmh1sum > 0.01) || (diffdh1sum > 0.01)))
				{
					if (DEBUG)
						NOESY_Assign::outputLog() << "h1 " << diffmh1.oriSamples()[i] << " " << diffdh2.oriSamples()[j] << " " << diffmh1minus << " " << diffmh1sum << endl;
					if (DEBUG)
						NOESY_Assign::outputLog() << "h2 " << diffdh1.oriSamples()[i] << " " << diffmh2.oriSamples()[j] << " " << diffdh1minus << " " << diffdh1sum << endl;
					continue;
				}

				if (peak1[i]->second.first == peak2[j]->second.first)
				  {
				    if (DEBUG)
				      NOESY_Assign::outputLog() << "same peak" << endl;
				    continue;
				  }


				if ((((peak1[i]->second.first->get_D() !=4) || (fabs(peak1[i]->second.first->get_X1() - peak1[i]->second.first->get_X2()) < 0.1))
				     && (fabs(peak1[i]->second.first->get_HX1() - peak1[i]->second.first->get_HX2()) < 0.01)) ||
				    (((peak2[j]->second.first->get_D() != 4) || (fabs(peak2[j]->second.first->get_X1() - peak2[j]->second.first->get_X2()) < 0.1))
				     && (fabs(peak2[j]->second.first->get_HX1() - peak2[j]->second.first->get_HX2()) < 0.01)))
				  {
				    if (DEBUG)
				      NOESY_Assign::outputLog() << "diangle peaks" << endl;
				    continue;
				  }


			tmdSetA12.insert(*mdsi);
			ttmdPAMapA.insert(*(peak1[i]));
			ttmdPAMapA.insert(*(peak2[j]));
   			pmdMapA.insert(make_pair(peak1[i]->second.first, *mdsi)); // symmetric peak at atom level
  			pmdMapA.insert(make_pair(peak2[j]->second.first, *mdsi)); // symmetric peak at atom level
  			symPeak.insert(peak1[i]->second.first);
  			symPeak.insert(peak2[j]->second.first);
     		if (DEBUG)
       		NOESY_Assign::outputLog() << "ok" << endl;
		}
	}

	tmdPAMapA.clear(); tmdPAMapA = ttmdPAMapA; ttmdPAMapA.clear();

	// if iterative, for each pmdMapA, find the shortest one
	if (NOESY_Assign::pdbList().size() && (NOESY_Assign::ddcycle() > 1))
	{
		//PeakPtrMomDadMap tpmdMapA; tpmdMapA.clear(); tpmdMapA = pmdMapA;
  		for(set<PeakPtr>::const_iterator i = symPeak.begin(); i!= symPeak.end(); ++i)
  		{
		  if ((*i)->get_D() ==2)
		    continue;

    			if (DEBUG)
			  NOESY_Assign::outputLog() << "CleanSYMbyDist " << *i << endl; // peak
  			{
			  //multimap<double, PeakPtrMomDadMap::iterator> mds; mds.clear();
			  multimap<double, MomDadSet::iterator> mds; mds.clear();
				float dsum = 0;

				MomDadSet tmd; tmd.clear();

      				for(PeakPtrMomDadMap::iterator j = pmdMapA.lower_bound(*i); j != pmdMapA.upper_bound(*i); ++j)
       				{
				  tmd.insert(j->second);
        			}

      				//for(PeakPtrMomDadMap::iterator j = pmdMapA.lower_bound(*i); j != pmdMapA.upper_bound(*i); ++j)
				for(MomDadSet::iterator mi = tmd.begin(); mi!= tmd.end(); mi++)
       				{
				  //double d = getDistStat(NOESY_Assign::pdbList(), make_pair(j->second.second, j->second.first), 0).median();
				  double d = getDistStat(NOESY_Assign::pdbList(), make_pair(mi->first, mi->second), 0).median();

        				mds.insert(make_pair(d, mi));

						dsum += pow(d, -4.0);
        			}


        			//for(multimap<double, PeakPtrMomDadMap::iterator>::iterator mi=mds.begin(); mi != mds.end(); ++mi)
				for(multimap<double, MomDadSet::iterator>::iterator mi = mds.begin(); mi != mds.end(); mi++)
        			{
				  if (DEBUG)
				    NOESY_Assign::outputLog() << *i << " " << mi->first << " " << mi->second->first << endl << mi->second->second << endl; // peak, dist, atom1, atom2


					double dctrtop = floor(pow(mi->first, -4.0)*100/dsum+0.51)/100.0;

          				if (((dctrtop < 0.4)))
					  {
					    PeakPtrMomDadMap::iterator pi;
					    for(pi = pmdMapA.lower_bound(*i); pi != pmdMapA.upper_bound(*i); ++pi)
					      {
						if (pi->second == *(mi->second))
						  break;
					      }
					    if (pi != pmdMapA.upper_bound(*i))
					      pmdMapA.erase(pi);

					    MomDadPeakAtomMap::iterator mdmi;
					    for(mdmi = tmdPAMapA.lower_bound(*(mi->second)); mdmi != tmdPAMapA.upper_bound(*(mi->second)); ++mdmi)
					      {
						if (mdmi->second.first == *i)
						  break;
					      }

					    if (mdmi != tmdPAMapA.upper_bound(*(mi->second)))
					      {
						tmdPAMapA.erase(mdmi);
						if (DEBUG)
						  NOESY_Assign::outputLog() << "is removed" << endl;
					      }
					    if (DEBUG)
					      NOESY_Assign::outputLog() << dctrtop << " removed" << endl;
					  } // end if bb
        			} // //
  			} // end for
  		}

   	// reset tmdSetA12, and remove non-symmetric ones
	mdSetA.clear();
  	//mdSetA = tmdSetA12; tmdSetA12.clear();
	mdSetA.insert(tmdSetA12.begin(), tmdSetA12.end()); tmdSetA12.clear();
  	for(MomDadSet::iterator mdsi = mdSetA.begin(); mdsi != mdSetA.end(); ++mdsi)
  	{
   		MomDadSet ASset; ASset.clear();
   		for(MomDadPeakAtomMap::iterator mdmi = tmdPAMapA.lower_bound(*mdsi); mdmi != tmdPAMapA.upper_bound(*mdsi); ++mdmi)
   		{
   			ASset.insert(mdmi->second.second);
			if (DEBUG)
			  NOESY_Assign::outputLog() << "check " << mdmi->second.first << endl << mdmi->second.second.first << endl << mdmi->second.second.second << endl;
   		}

   		if ((ASset.size() >= 2)) // two x1 (symmetric)
   		{
   			tmdSetA12.insert(*mdsi);

			if (DEBUG)
			  NOESY_Assign::outputLog() << ">=2 ok" << endl;
   		}

		if (DEBUG) NOESY_Assign::outputLog() << endl;
  	}
	} // endif

	// find the unique one
	//MomDadSet tmdSetA; tmdSetA.clear();
	//tmdSetA = tmdSetA12;
	MomDadSet tmdSetA(tmdSetA12);
	for(MomDadSet::iterator mdsi = tmdSetA.begin(); mdsi != tmdSetA.end(); ++mdsi)
	{
		if (DEBUG)
			NOESY_Assign::outputLog() << "Find-Unique " << mdsi->first << endl << mdsi->second << endl;

		bool goodSym=1;
		bool notOK=0;
		//bool old = 0;

 		for(MomDadPeakAtomMap::iterator mdpi = tmdPAMapA.lower_bound(*mdsi); mdpi != tmdPAMapA.upper_bound(*mdsi); ++mdpi)
 		{
			// unique
		  MomDadSet md; md.clear();;
		  for(PeakPtrMomDadMap::iterator pi = pmdMapA.lower_bound(mdpi->second.first); pi != pmdMapA.upper_bound(mdpi->second.first); pi++)
		    {
		      md.insert(pi->second);
		      if (DEBUG)
			NOESY_Assign::outputLog() << pi->first << endl << pi->second.first << endl << pi->second.second << endl;
		    }
		  if (md.size() > 1)
		    {
 				notOK=1;
				break;
		    }

			// goodSymmetry
			float diffhx1 = mdpi->second.second.first->get_HdiffAb(mdpi->second.first->get_HX1(), mdpi->second.first->SWSF_HX1());
			float diffhx2 = mdpi->second.second.second->get_HdiffAb(mdpi->second.first->get_HX2(), mdpi->second.first->SWSF_HX2());

			if ((diffhx1 > 0.039) || (diffhx2 > 0.039))
			{
				goodSym = 0;
				break;
			}

	  		//if ((*iter)->get_tols().HX2 == 999)
			//if ((mdpi->second.first->get_tols().HX2 == 999))
			if ((mdpi->second.first->get_HX2() == 999))
			  {
			    goodSym = 0;
			    break;
			  }


    			// match tolerance
    			if ((mdpi->second.second.first->get_Hdiff(mdpi->second.first->get_HX1(), mdpi->second.first->SWSF_HX1()) < parSym.htol + 0.000001) &&
			    (mdpi->second.second.second->get_Hdiff(mdpi->second.first->get_HX2(), mdpi->second.first->SWSF_HX2()) < parSym.htol + 0.000001) &&
			    (mdpi->second.second.first->get_Xdiff(mdpi->second.first->get_X1(), mdpi->second.first->SWSF_X1()) < parSym.xtol + 0.000001) &&
			    (mdpi->second.second.second->get_Xdiff(mdpi->second.first->get_X2(), mdpi->second.first->SWSF_X2()) < parSym.xtol + 0.000001))
			  {
			  }
			else
			  {
			    notOK = 1;
			    if (DEBUG) NOESY_Assign::outputLog() << "not well-matched" << endl;
			  }

			if (DEBUG)
				NOESY_Assign::outputLog() << mdpi->second.first << " !goodSym=" << !goodSym << " notOK=" << notOK << endl;
    }

    if (!goodSym || notOK)
    {
    	if (DEBUG)
     		NOESY_Assign::outputLog() << "remove ";

 			if (tmdSetA12.find(*mdsi) != tmdSetA12.end())
 				tmdSetA12.erase(tmdSetA12.find(*mdsi));
 		}
    if (DEBUG)
     	NOESY_Assign::outputLog() << endl;
	} // end of find unique

	if (DEBUG)
	{
  	// find the new unique 2-pairs
  	for(MomDadSet::iterator mdsi = tmdSetA12.begin(); mdsi != tmdSetA12.end(); ++mdsi)
  	{
 			for(MomDadPeakAtomMap::iterator mdmi = tmdPAMapA.lower_bound(*mdsi); mdmi != tmdPAMapA.upper_bound(*mdsi); ++mdmi)
  		{
  			if (DEBUG)
  				NOESY_Assign::outputLog() << "Unique alone A-pairs" << mdmi->second.first << endl << mdmi->second.second.first << endl << mdmi->second.second.second << endl;
  		}
  		if (DEBUG)
  			NOESY_Assign::outputLog() << endl;
  	}
	}

	mdSetA12.insert(tmdSetA12.begin(), tmdSetA12.end());
	mdPAMapA.insert(tmdPAMapA.begin(), tmdPAMapA.end());

	// build support
	//ContactPairSet ContactBinUniqueSYM; ContactBinUniqueSYM.clear();
  	for(MomDadSet::iterator mdsi = mdSetA12.begin(); mdsi != mdSetA12.end(); ++mdsi)
  	{
  		pair<int, int> mij;
 			if (mdsi->first->get_index() < mdsi->second->get_index())
 				mij = make_pair(mdsi->first->get_index(), mdsi->second->get_index());
 			else
 				mij = make_pair(mdsi->second->get_index(), mdsi->first->get_index());
			support.ContactBinUniqueSYM.insert(mij);
 	}
}
