/***************************************************************************
                          experiment.cpp  -  description
                             -------------------
    begin                : Mon Aug 21 2000
    copyright            : (C) 2000 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#include <iostream>
#include <iomanip>

#include <sstream>
#include <cmath>
#include "experiment.h"
#include "noesy_assign.h"

#define pow(x,y) pow((double)x, (double)y)
#define fmod(x,y) fmod((double)x, (double)y)
#define abs(x) abs((double)(x))

extern bool DEBUG;
SampleStatistic getDistStat(deque<Pdb>& , pair<const atom*, const atom*> , bool );
float sumAverage(const SampleStatistic&);

string itoa(int);

ostream& erasePrint(ostream&, const string);

void error(char*, char*);
void error(string, string);

const PeakList& NOESY::get_peakList() const
{ return peakList; }

const PeakMomDadMap& NOESY::get_BList() const
{ return BList; }

const PeakAssignReason& NOESY::get_AssignList() const
{
  return AssignList;
}
const PeakAssignReason& NOESY::get_BufferReason() const
{
  return BufferReason;
}
bool NOESY::isHaveIC() const
{return (haveIC || IC);}
bool NOESY::isIC() const
{return IC;}

float diff(float a, float b)
{
	double l = floor(a/b);
	double u = ceil(a/b);
	float rl = fabs(a - l*b);
	float ru = fabs(a - u*b);
	return (rl < ru) ? rl : ru;
}

tol NOESY::get_tols() const
{
  return tols;
}
void NOESY::setKCON(float n)
{
  if (DEBUG) NOESY_Assign::outputLog() << "in setKCON " << KCON << " " << n << endl;
  KCON *= n;
  if (DEBUG) NOESY_Assign::outputLog() << KCON << endl;
}

bool NOESY::nearH2o(const atom* atom1) const
{
  if ((atom1->get_shift() >= wl) &&(atom1->get_shift() <= wu))
    return 1;
  return 0;
}

bool NOESY::closeBy(const general_peak& peak1, const general_peak& peak2) const
{

	if ((diff(fabs(peak1.get_HX1()-peak2.get_HX1()), SWSF_HX1) < 2*tols.HX1+0.01) &&
			(diff(fabs(peak1.get_HX2()-peak2.get_HX2()), SWSF_HX2) < 2*tols.HX2+0.01) &&
			(diff(fabs(peak1.get_X1()-peak2.get_X1()), SWSF_X1) < 2*tols.X1+0.1) &&
			(diff(fabs(peak1.get_X2()-peak2.get_X2()), SWSF_X2) < 2*tols.X2+0.1))
		return 1;
	else
		return 0;
}

// for assigned max
void NOESY::buildAcfMax(matrix& cfMax) const
{
//  matrix cfMax;

	for(PeakList::const_iterator i = peakList.begin(); i != peakList.end(); i++)
	  {
	    PeakAssignReason::const_iterator il = AssignList.lower_bound(&(*i));
      PeakAssignReason::const_iterator iu = AssignList.upper_bound(&(*i));

      int l,s;
      for(PeakAssignReason::const_iterator i = il; i != iu; i++)
      {
        if (i->second.first.first->get_index() > i->second.first.second->get_index())
        {
          l = i->second.first.first->get_index();
          s = i->second.first.second->get_index();
        } else
        {
          l = i->second.first.second->get_index();
          s = i->second.first.first->get_index();
        }

		    matrix::iterator mi = cfMax.find(pair<int,int>(s, l));
			    if (mi != cfMax.end())
			      (*mi).second++;
			    else
			      cfMax.insert(pair<pair<int, int>, int>
					   (pair<int, int>(s,l), 1));
			}
	  }
}

void NOESY::initReport()
{
	/////////////////////////////////////////////////////////
	// Initial some Statistic Report                       //
	/////////////////////////////////////////////////////////
	ostringstream reportStream;
	// Peak -- Assignment by Cycles
	reportStream << endl << "                                   #Peak-Assignment Stat#" << endl << endl;
	reportStream <<         "Cycle Total_Assignable_Peak Unambiguous_Assigned Ambiguous_Assigned Total_Assigned Peak_Not_Assigned" << endl;
	reportStream << "      --------------------- -------------------- ------------------ -------------- -----------------" << endl;
	reportStream << "      #         %           #         %          #        %         #      %       #       %        " << endl;
	reportStream << NOESY_Assign::EOS();
	string temp(reportStream.str());
	reportPeakStat += temp.substr(0, temp.find(NOESY_Assign::EOS()));

	// Assignment Only by Cycles
	ostringstream reportStreamAss;
	reportStreamAss << endl << "                                     #Assignment Stat#" << endl << endl;
	reportStreamAss << "Cycle Total_Assignment Intraresidue[i=j] Interresidue[(i-j)=1] Interresidue[1<(i<j)<5]  Interresidue[(i-j)>=5]" << endl;
	reportStreamAss << "                                         --------------------- -----------------------  ----------------------" << endl;
	reportStreamAss << "                                         TOT   BB    BS    SS  TOT   BB    BS    SS     TOT   BB    BS    SS " << endl;
  reportStreamAss << NOESY_Assign::EOS();
	string temp2(reportStreamAss.str());
	reportAssignStat += temp2.substr(0, temp2.find(NOESY_Assign::EOS()));

	// NOESY_Assign process Stat by Cycles
	ostringstream reportStreamProcess;
	reportStreamProcess << endl << "                                #NOESY_Assign Process#" << endl << endl;
	reportStreamProcess << "Cycle Method Peak_Assigned Assignment_Made Assignment_Removed Peak_Not_Assigned Noise_Peak";
	if (NOESY_Assign::COMPARE())
	  reportStreamProcess << " Correct Consistent Inconsistent" << endl;
	else
	  reportStreamProcess << endl;
	reportStreamProcess << "             ------------- ---------------    " << endl;
	reportStreamProcess << "             new   cum     new    cum         " << endl;
  reportStreamProcess << NOESY_Assign::EOS();
	string temp3(reportStreamProcess.str());
	reportProcessStat += temp3.substr(0, temp3.find(NOESY_Assign::EOS()));
}

NOESY::NOESY()
{
	Rmax = 0;
	RmaxQ = 0;
	iperc = 0.125;
  KCON = 0;
  haveIC = 0;
  IC = 0;
  initReport();
  peakControlPtr = 0;
  dianaBufferPtr = 0;
  KCON = 0; // default is 0 - to use the default calibration - not 0 - use the new kcon
  wl = wu = 0;
}
NOESY::NOESY(bool flag, bool ihaveIC, float HX1t, float HX2t, float f, string sl)
{
	Rmax = 0;
	RmaxQ = 0;
	iperc = f;
	waterFlag = flag;
	slabel = sl;
	KCON = 0;
	tols.set_tol(HX1t, HX2t);
  haveIC = ihaveIC;
  IC = 0;
	initReport();
  peakControlPtr = 0;
  dianaBufferPtr = 0;
  KCON = 0;
  wl = wu = 0;
}

NOESY::NOESY(bool flag, bool ihaveIC, float HX1t, float X1t, float HX2t, float f, string sl)
{
	Rmax = 0;
	RmaxQ = 0;
	iperc = f;
	waterFlag = flag;
	slabel = sl;
  haveIC = ihaveIC;
  IC = 0;
	tols.set_tol(HX1t, X1t, HX2t);
	KCON = 0;
  peakControlPtr = 0;
  dianaBufferPtr = 0;
  KCON = 0;
	initReport();
  wl = wu = 0;
}
NOESY::NOESY(bool flag, bool ihaveIC, float HX1t, float X1t, float HX2t, float X2t, float f, string sl)
{
	Rmax = 0;
	RmaxQ = 0;
	iperc = f;
	waterFlag = flag;
	slabel = sl;
  haveIC = ihaveIC;
  IC = 0;
	tols.set_tol(HX1t, X1t, HX2t, X2t);
	KCON = 0;
  peakControlPtr = 0;
  dianaBufferPtr = 0;
  KCON = 0;
	initReport();
  wl = wu = 0;
}

NOESY::~NOESY()
{
}

 void NOESY::setSWSF(tol bl, float HX1, float HX2)
{
  SWSF_HX1 = HX1;
  SWSF_HX2 = HX2;
  SWSF_X1 = 100000;
  SWSF_X2 = 100000;
  sign = bl;

  for(PeakList::iterator pi = peakList.begin(); pi != peakList.end(); pi++)
  {
  	pi->setSWSF(HX1, HX2);
  }
}

 void NOESY::setSWSF(tol bl, float HX1, float X1, float HX2)
{
  SWSF_HX1 = HX1;
  SWSF_HX2 = HX2;
  SWSF_X1  = X1;
  SWSF_X2 = 100000;
  sign = bl;

  for(PeakList::iterator pi = peakList.begin(); pi != peakList.end(); pi++)
  {
  	pi->setSWSF(HX1, X1, HX2);
  }
}

 void NOESY::setSWSF(tol bl, float HX1, float X1, float HX2, float X2)
{
  SWSF_HX1 = HX1;
  SWSF_HX2 = HX2;
  SWSF_X1  = X1;
  SWSF_X2  = X2;
  sign = bl;

  for(PeakList::iterator pi = peakList.begin(); pi != peakList.end(); pi++)
  {
  	pi->setSWSF(HX1, X1, HX2, X2);
  }
}

void NOESY::setWaterLine(float lb, float ub)
{
  wl = lb; wu = ub;
}
float NOESY::getWL()
{
  return wl;
}
float NOESY::getWU()
{
  return wu;
}

void NOESY::setPeakControlPtr(ControlMap* p)
{
  peakControlPtr = p;
}

void NOESY::setDianaBufferPtr(DianaMap* d)
{
  dianaBufferPtr = d;
}


const string& NOESY::label() const
{
	return slabel;
}

 void NOESY::Corr_Spectrum(float HX1, float HX2)
{
	for(PeakList::iterator iter_p = peakList.begin(); iter_p != peakList.end(); iter_p++)
	{
		(*iter_p).adjast_assign(HX1, HX2);
	}
}

 void NOESY::Corr_Spectrum(float HX, float X, float H)
{
	for(PeakList::iterator iter_p = peakList.begin(); iter_p != peakList.end(); iter_p++)
	{
		(*iter_p).adjast_assign(HX, H, X);
	}
}
 void NOESY::Corr_Spectrum(float HX, float X, float HX2, float X2)
{
	for(PeakList::iterator iter_p = peakList.begin(); iter_p != peakList.end(); iter_p++)
	{
		(*iter_p).adjast_assign(HX, HX2, X, X2);
	}
}

const pair<float, float> NOESY::get_bound_HX2(const general_peak& peak) const // get the bound of HX2
{

	float hx2 = floor(peak.get_HX2()*1000.0+0.500001)/1000.0;
	return make_pair(floor((hx2 - tols.HX2)*1000.0+0.500001)/1000.0,
		floor((hx2 + tols.HX2)*1000.0+0.500001)/1000.0);
}

const pair<float, float> NOESY::get_bound_HX1(const general_peak& peak) const // get the bound of HX1
{

	float hx1 = floor(peak.get_HX1()*1000.0+0.500001)/1000.0;
	return pair<float, float>(floor((hx1 - tols.HX1)*1000.0+0.500001)/1000.0,
		floor((hx1 + tols.HX1)*1000.0+0.500001)/1000.0);
}

const pair<float, float> NOESY::get_bound_X1(const general_peak& peak) const // get the bound of HX2
{

	float x1 = floor(peak.get_X1()*100.0+0.500001)/100.0;
	return make_pair(floor((x1 - tols.X1)*100.0+0.500001)/100.0,
		floor((x1 + tols.X1)*100.0+0.500001)/100.0);

}


const pair<float, float> NOESY::get_bound_X2(const general_peak& peak) const // get the bound of HX1
{

	float x2 = floor(peak.get_X2()*100.0+0.500001)/100.0;
	return pair<float, float>(floor((x2 - tols.X2)*100.0+0.500001)/100.0,
		floor((x2 + tols.X2)*100.0+0.500001)/100.0);
}


void NOESY::MakeBuffer(const general_peak* ipeak, atom* matom, atom* iatom, method mth)
{
	PeakAssignReason::iterator AssignLower = BufferReason.lower_bound(ipeak);
	PeakAssignReason::iterator AssignUpper = BufferReason.upper_bound(ipeak);

	PeakAssignReason::iterator fiter;
	for(fiter = AssignLower; fiter != AssignUpper; fiter++)
	{
		if (((*fiter).second.first.first == matom) &&
			((*fiter).second.first.second == iatom))
			break;
	}

	// no such assignments
	if (fiter == AssignUpper)
	{
		setMethod temp;
		temp.insert(mth);
		BufferReason.insert(make_pair(ipeak, make_pair(make_pair(matom, iatom), temp)));
	}
	else // have some assignments already
	{
		(*fiter).second.second.insert(mth);
	}

	if (DEBUG) NOESY_Assign::outputLog() << "Make buffer: " << ipeak << endl << matom << endl << iatom << " " << mth << endl;
}

//-------------------------------
// MakeAssign -- assign to Mom --- for local referencing ;
// 	Assuming is unique assignment, or confirmed by new mothed
//      before assign H
//-------------------------------
void NOESY::MakeMomAssign(const general_peak* ipeak,  atom* matom)
{
	MatchAtomPeak::iterator ml = MotherAtom.lower_bound(matom);
	MatchAtomPeak::iterator mu = MotherAtom.upper_bound(matom);
	MatchAtomPeak::iterator mi;
	for(mi = ml; mi != mu; mi++)
	{
		if ((*mi).second == ipeak) break;
	}
	if (mi == mu)
	{
		MotherAtom.insert(make_pair(matom, ipeak));
		atom* const atommin = matom->att_X(ipeak->get_X1(), SWSF_X1);
		AttMomAtom.insert(make_pair(atommin, ipeak));
	}
}

// for 4D NOESY
void NOESY::MakeDadAssign(const general_peak* ipeak, atom* datom)
{
	MatchAtomPeak::iterator dl = FatherAtom.lower_bound(datom);
	MatchAtomPeak::iterator du = FatherAtom.upper_bound(datom);
	MatchAtomPeak::iterator di;
	for(di = dl; di != du; di++)
	{
		if ((*di).second == ipeak) break;
	}
	if (di == du)
	{
		FatherAtom.insert(make_pair(datom, ipeak));
		atom* const atommin = datom->att_X(ipeak->get_X2(), SWSF_X2);
		AttDadAtom.insert(make_pair(atommin, ipeak));
	}
}

void NOESY::MakeParUnAssign(bool isMom, const general_peak* ipeak, atom* matom,
	MatchAtomPeak& ParAtom, MatchAtomPeak& AttParAtom)
{
  MatchAtomPeak::iterator ml = ParAtom.lower_bound(matom);
  MatchAtomPeak::iterator mu = ParAtom.upper_bound(matom);
  MatchAtomPeak::iterator mi;
  for(mi = ml; mi != mu; mi++)
    {
      if ((*mi).second == ipeak) break;
    }
  if (mi != mu)
    {
      ParAtom.erase(mi);
    }

  atom* atommin;

  if (isMom)
   	atommin = matom->att_X(ipeak->get_X1(), SWSF_X1);
  else
		atommin = matom->att_X(ipeak->get_X2(), SWSF_X2);

  MatchAtomPeak::iterator al = AttParAtom.lower_bound(atommin);
  MatchAtomPeak::iterator au = AttParAtom.upper_bound(atommin);
  MatchAtomPeak::iterator ai;
  for(ai = al; ai != au; ai++)
    {
      if ((*ai).second == ipeak) break;
    }
  if (ai != au)
    {
      AttParAtom.erase(ai);
    }
}

void NOESY::UnAssign(const general_peak* ipeak, atom* matom, atom* iatom, method mth, int& IA, int& IAI)
{
	PeakAssignReason::iterator AssignLower = AssignList.lower_bound(ipeak);
	PeakAssignReason::iterator AssignUpper = AssignList.upper_bound(ipeak);

	PeakAssignReason::iterator fiter;
	for(fiter = AssignLower; fiter != AssignUpper; fiter++)
	{
		if (((*fiter).second.first.first == matom) && ((*fiter).second.first.second == iatom))
			break;
	}

	// no such assignments
	if (fiter == AssignUpper)
	{
		NOESY_Assign::outputLog() << "Try to unassignment null assignment" << endl;
		NOESY_Assign::outputLog() << ipeak << endl << matom << endl << iatom << "\t" << mth << endl;
		error("NoSuchAssignment", "");
	}
	else
	{
		if (DEBUG) NOESY_Assign::outputLog() << "Unassign: ";
		MakeBuffer(ipeak, matom, iatom, mth);

    	    if (NOESY_Assign::COMPARE())
    		  {
    		      IA++;
    		      //disAtomPairStat::iterator diter;
            	pair<const atom*, const atom*> ab;
    		      if (matom < iatom)
            		ab = make_pair(matom, iatom);
    		      else
            		ab = make_pair(iatom, matom);

			SampleStatistic distStat = getDistStat(NOESY_Assign::powerPdbList(), ab,isIC());

			if (distStat.count() && sumAverage(distStat) > 5.0)
    			      {
    			        IAI++;
    			        NOESY_Assign::outputError() << "***SINCONSISTENT, unassigned*** " << ipeak << " d: " << sumAverage(distStat) << mth << endl;
    			        NOESY_Assign::outputError() << "***SINCONSISTENT, unassigned*** " << matom << endl;
    			        NOESY_Assign::outputError() << "***SINCONSISTENT, unassigned*** " << iatom << endl;
    			      }
    			      else if (distStat.count() && sumAverage(distStat) < 5.0)
    			      {
    			        NOESY_Assign::outputError() << "***SCONSISTENT, unassigned*** " << ipeak << " d: " << sumAverage(distStat) << " " << mth << endl;
    			        NOESY_Assign::outputError() << "***SCONSISTENT, unassigned*** " << matom << endl;
    			        NOESY_Assign::outputError() << "***SCONSISTENT, unassigned*** " << iatom << endl;
    			      }
    		  } // end compare

		{
			MatchAtomPeak::iterator fl = FatherAtom.lower_bound(iatom);
			MatchAtomPeak::iterator fu = FatherAtom.upper_bound(iatom);
			MatchAtomPeak::iterator fi;
			for(fi = fl; fi != fu; fi++)
			{
				if ((*fi).second == ipeak) break;
			}
			if (fi != fu)
				FatherAtom.erase(fi);

			MatchAtomPeak::iterator ml = MotherAtom.lower_bound(matom);
			MatchAtomPeak::iterator mu = MotherAtom.upper_bound(matom);
			MatchAtomPeak::iterator mi;
			for(mi = ml; mi != mu; mi++)
			{
				if ((*mi).second == ipeak) break;
			}
			if (mi != mu)
			{
				MotherAtom.erase(mi);
			}

			if (ipeak->get_D() >= 3)
			  {
			    atom* atommin = matom->att_X(ipeak->get_X1(), SWSF_X1);
			    MatchAtomPeak::iterator al = AttMomAtom.lower_bound(atommin);
			    MatchAtomPeak::iterator au = AttMomAtom.upper_bound(atommin);
			    MatchAtomPeak::iterator ai;
			    for(ai = al; ai != au; ai++)
			      {
				if ((*ai).second == ipeak) break;
			      }
			    if (ai != au)
			      {
				AttMomAtom.erase(ai);
			      }
			  }

			if (ipeak->get_D() >= 4)
			  {
			    atom* datommin = iatom->att_X(ipeak->get_X2(), SWSF_X2);
			    MatchAtomPeak::iterator dl = AttDadAtom.lower_bound(datommin);
			    MatchAtomPeak::iterator du = AttDadAtom.upper_bound(datommin);
			    MatchAtomPeak::iterator di;
			    for(di = dl; di != du; di++)
			      {
				if ((*di).second == ipeak) break;
			      }
			    if (di != du)
			      {
				AttDadAtom.erase(di);
			      }
			  }
		}
		AssignList.erase(fiter);
	}
}

void NOESY::MakeAssign(const general_peak* ipeak, atom* matom, atom* iatom, method mth)
{
	PeakAssignReason::iterator AssignLower = AssignList.lower_bound(ipeak);
	PeakAssignReason::iterator AssignUpper = AssignList.upper_bound(ipeak);

	PeakAssignReason::iterator fiter;
	for(fiter = AssignLower; fiter != AssignUpper; fiter++)
	{
		if (((*fiter).second.first.first == matom) &&
			((*fiter).second.first.second == iatom))
			break;
	}

	bool d2o = 0;


	if (fiter != AssignUpper)
	{
	  (*fiter).second.second.insert(mth);
	  if (DEBUG) NOESY_Assign::outputLog() << "new method: ";
	  MakeBuffer(ipeak, matom, iatom, mth);
	}
	else if (fiter == AssignUpper) //new assignment
	{
	  bool expected = 0;
	  PeakAssignReason::iterator rl = BufferReason.lower_bound(ipeak);
	  PeakAssignReason::iterator ru = BufferReason.upper_bound(ipeak);
	  for(PeakAssignReason::iterator ri = rl; ri != ru; ri++)
    	  {
      		if (((*ri).second.first.first == matom) && ((*ri).second.first.second == iatom))
      		{

	  		if (((*ri).second.second.count(DEL_ASS) != 0) || ((*ri).second.second.count(DEL_SAM) != 0))
					return;
			if ((ri->second.second.count(INTRA) != 0) || (ri->second.second.count(ESEQ) != 0) || (ri->second.second.count(ESS) != 0))
				expected = 1;
			break;
		}
          }
	  setMethod temp;
	  temp.insert(mth);
	  if (expected)
		  temp.insert(EXPECT);
	  AssignList.insert(make_pair(ipeak, make_pair(make_pair(matom, iatom), temp)));

	  MakeBuffer(ipeak, matom, iatom, mth);

	  {
	  	pair<int, int> ij;
	  	if (matom->get_index() < iatom->get_index())
			  ij = make_pair(matom->get_index(), iatom->get_index());
	  	else
			  ij = make_pair(iatom->get_index(), matom->get_index());

          	NOESY_Assign::asContactSet().insert(ij);
	  }
	  if (DEBUG) NOESY_Assign::outputLog() << "new assignment: ";

	} //end if new assignment

	if (mth != MULTI)
	{
		//--- remove the other possible assignments that as same as this one
		if (DEBUG)
		{
			NOESY_Assign::outputLog() << "closeby peaks : " << endl;
		}
		for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
		{
			if (((*iter).get_ID() != ipeak->get_ID()) && closeBy(*ipeak, (*iter)))
			{
				if (DEBUG)
					NOESY_Assign::outputLog() << *iter << endl;

				MatchPeakAtom::iterator ml = MombySimpleMatch.lower_bound(&(*iter));
				MatchPeakAtom::iterator mu = MombySimpleMatch.upper_bound(&(*iter));
				MatchPeakAtom::iterator dl = DadbySimpleMatch.lower_bound(&(*iter));
				MatchPeakAtom::iterator du = DadbySimpleMatch.upper_bound(&(*iter));

				for(MatchPeakAtom::iterator mi = ml; mi != mu; mi++)
				for(MatchPeakAtom::iterator di = dl; di != du; di++)
				{
					if (((*mi).second == matom) && ((*di).second == iatom))
					{
						MakeBuffer(&(*iter), matom, iatom, DEL_SAM);
						break;
					}
				}
			}
		}
	}
	  //if ((mth == EXPECT) || loc)
	  if (!d2o)
	  {
		//--- local referencing
		//--- check if in the list already
		  {
		    MatchAtomPeak::iterator ml = MotherAtom.lower_bound(matom);
		    MatchAtomPeak::iterator mu = MotherAtom.upper_bound(matom);
		    MatchAtomPeak::iterator mi;
		    for(mi = ml; mi != mu; mi++)
		      {
							if ((*mi).second == ipeak) break;
		      }
		    if (mi == mu)
		      {
        			MotherAtom.insert(make_pair(matom, ipeak));
							if (ipeak->get_D() >= 3)
							{
        				atom* atommin = matom->att_X(ipeak->get_X1(), SWSF_X1);
        				AttMomAtom.insert(make_pair(atommin, ipeak));
            	}
		      }
		  }

		  {
		    MatchAtomPeak::iterator dl = FatherAtom.lower_bound(iatom);
		    MatchAtomPeak::iterator du = FatherAtom.upper_bound(iatom);
		    MatchAtomPeak::iterator di;
		    for(di = dl; di != du; di++)
		      {
							if ((*di).second == ipeak) break;
		      }
		    if (di == du)
		      {
        			FatherAtom.insert(make_pair(iatom, ipeak));
           		if (ipeak->get_D() >=4)
             	{
        				atom* atommin = iatom->att_X(ipeak->get_X2(), SWSF_X2);
        				AttDadAtom.insert(make_pair(atommin, ipeak));
           		}
		      }
		  }
	}
}

void NOESY::MakeAssignMap(MomDadPeakMap& assignMap, method mth, int& wN, int& iN)
{
  if (DEBUG) NOESY_Assign::outputLog() << "in MakeAssignMap" << endl;
  //--- for each assignment, find the best matched observed peak, make assignments

  // list of assingments grouped by carbons
  set<pair<MomDad, MomDad>, less<pair<MomDad, MomDad> > > assignSet; assignSet.clear(); // problem
	multimap<pair<MomDad, MomDad>, const general_peak* const, less<pair<MomDad, MomDad> > > asmap; asmap.clear();

	// peak1=a,b,c and peak2 = a then remove a from peak1.
  multimap<const general_peak*, pair<MomDad, MomDad>, less<const general_peak*> > peakmap; peakmap.clear();
  PeakSet peakset; peakset.clear();

  for(MomDadPeakMap::iterator pi = assignMap.begin(); pi != assignMap.end(); pi++)
  {
    MomDad md = make_pair(pi->first.first->att_X(pi->second->get_X1(), SWSF_X1),
    pi->first.second->att_X(pi->second->get_X2(), SWSF_X2)); // how about X-not assigned
    peakmap.insert(make_pair(pi->second, make_pair(md, pi->first)));
    peakset.insert(pi->second); // all peaks to be assigned
  }

  for(MomDadPeakMap::iterator pi = assignMap.begin(); pi != assignMap.end(); pi++)
    {
      MomDad md = make_pair(pi->first.first->att_X(pi->second->get_X1(), SWSF_X1),
      pi->first.second->att_X(pi->second->get_X2(), SWSF_X2));
      assignSet.insert(make_pair(md, pi->first));
      asmap.insert(make_pair(make_pair(md, pi->first), pi->second));
    }

  // one MomDad -> to more than one peaks
  for(set<pair<MomDad, MomDad>, less<pair<MomDad, MomDad> > >::iterator si = assignSet.begin(); si != assignSet.end(); si++)
    {
    	if (DEBUG)
     		NOESY_Assign::outputLog() << "+++> Assign: " << si->second.first << endl << si->second.second << endl;

      multimap<pair<MomDad, MomDad>, const general_peak* const, less<pair<MomDad, MomDad> > >::iterator pl = asmap.lower_bound(*si);
      multimap<pair<MomDad, MomDad>, const general_peak* const, less<pair<MomDad, MomDad> > >::iterator pu = asmap.upper_bound(*si);

      const general_peak* pmin = 0; float dmin = 100;
      const MomDad* mdmin = 0;

      multimap<pair<MomDad, MomDad>, const general_peak* const, less<pair<MomDad, MomDad> > >::iterator pi;

      // find the closest one, the distance is weighted by 2 for the H-X pairs
      for(pi = pl; pi != pu; pi++)
    	{
    	  float dist = 0;

    	    dist += pow((*pi).first.second.first->get_Hdiff((*pi).second->get_HX1(), SWSF_HX1),1.2) +
    	            pow((*pi).first.second.second->get_Hdiff((*pi).second->get_HX2(), SWSF_HX2), 1.2)+
		    pow((*pi).first.second.first->get_Xdiff((*pi).second->get_X1(), SWSF_X1)/10.0, 1.25) +
    	            pow((*pi).first.second.second->get_Xdiff((*pi).second->get_X2(), SWSF_X2)/10.0, 1.25);

    	  if (DEBUG)
    	     NOESY_Assign::outputLog() << "---> " << (*pi).second << "\t" << dist << endl;
	  if (DEBUG)
        	 NOESY_Assign::outputLog() << pi->first.second.first << endl << pi->first.second.second << endl;

    	  if (dist + 0.00000001 < dmin)
    	    {
    	      dmin = dist; pmin = (*pi).second;
           	mdmin = &(pi->first.second);
    	    }
    	}

      if ( DEBUG )
      {
         NOESY_Assign::outputLog() << "minP is " << pmin << "\t" << dmin << endl;
         NOESY_Assign::outputLog() << mdmin->first->get_Hdiff(pmin->get_HX1(), SWSF_HX1) << "\t" <<
  	    	mdmin->second->get_Hdiff(pmin->get_HX2(), SWSF_HX2) << "\t";

         if (pmin->get_D() >= 3)
	     			NOESY_Assign::outputLog() << mdmin->first->get_Xdiff(pmin->get_X1(), SWSF_X1) << "\t";
         if (pmin->get_D() > 3)
	      		NOESY_Assign::outputLog() << mdmin->second->get_Xdiff(pmin->get_X2(), SWSF_X2) << "\t";
         NOESY_Assign::outputLog() << endl;
      }

      //--- check correctness, only for simulated data
      PeakAssignReason::iterator AssignLower = AssignList.lower_bound(pmin);
      PeakAssignReason::iterator AssignUpper = AssignList.upper_bound(pmin);

      PeakAssignReason::iterator fiter;
      for(fiter = AssignLower; fiter != AssignUpper; fiter++)
    	{
    	  if (((*fiter).second.first.first == mdmin->first) &&
    	      ((*fiter).second.first.second == mdmin->second))
    	    break;
    	}

      // new assignments
      if ((fiter == AssignUpper) && NOESY_Assign::COMPARE())
    	{
       	pair<const atom*, const atom*> ab;
 	      if (mdmin->first < mdmin->second)
       		ab = make_pair(mdmin->first, mdmin->second);
 	      else
       		ab = make_pair(mdmin->second, mdmin->first);

 				SampleStatistic distStat = getDistStat(NOESY_Assign::powerPdbList(), ab, isIC());

				if (distStat.samples().size())
   	    {
   	      wN++;
          if (sumAverage(distStat)> 5.0)
       		{
       		  iN++;
       		  NOESY_Assign::outputError() << "***INCONSISTENT*** " << pmin << " sumAvg " << sumAverage(distStat) << " " << mth << endl;
       		  NOESY_Assign::outputError() << "***INCONSISTENT*** " << mdmin->first << endl;
       		  NOESY_Assign::outputError() << "***INCONSISTENT*** " << mdmin->second << endl;
       		}
         	if (DEBUG)
       		  NOESY_Assign::outputLog() << " min " << sumAverage(distStat) << " " << mth << endl;
   	    }
    	}

    	//--- make assignment
      MakeAssign(pmin, mdmin->first, mdmin->second, mth);

			//----------------------------------------------------------------
			// find degenerate one and assign it also, if within tolerance
      if (BufferReason.find(pmin) != BufferReason.end())
      {
      	PeakAssignReason::iterator jl = BufferReason.lower_bound(pmin);
      	PeakAssignReason::iterator ju = BufferReason.upper_bound(pmin);

       	PeakAssignReason::iterator j;
        for(j = jl; j != ju; ++j)
        {
        	// if degenerate
         	if ((j->second.second.find(DEL_DEG) != j->second.second.end())
          		&& (j->second.second.find(DEL_LOC) == j->second.second.end())
          		&& (mdmin->first->get_index() == j->second.first.first->get_index())
            	&& (mdmin->second->get_index() == j->second.first.second->get_index()))
         		break;
        }

        if ((j != ju) &&
        		(!mdmin->first->isDe(j->second.first.first) &&
          	 (j->second.first.first->get_Hdiff(pmin->get_HX1(), SWSF_HX1) < 0.02 + 0.000001)) &&
        		(!mdmin->second->isDe(j->second.first.second) &&
          	 (j->second.first.second->get_Hdiff(pmin->get_HX2(), SWSF_HX2) < 0.02 + 0.000001)))
       	{
      		MakeAssign(pmin, j->second.first.first, j->second.first.second, mth);
       	}
      }
    }
}


// by default, the best match based on match tolerance , now changed to fnd the
// the Strong matches.
void NOESY::Find_bestMatch(const general_peak* peak, const MomDadSet& temp, PeakMomDadMap& BestList)
{
  //--- find the best matches
  for(MomDadSet::iterator mdsi = temp.begin(); mdsi != temp.end(); mdsi++)
    {
	{
	  BestList.insert(make_pair(peak, (*mdsi)));
	}
    }
}


// using now
void NOESY::Find_BbestMatch()
{
	if (DEBUG) NOESY_Assign::outputLog() << "\n-> Find_BbestMatch()" << endl;

	BList.clear();
	for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
	{
		if (BList.find(&(*iter)) == BList.end())
		{
			//--- find the best match that never tested
			MatchPeakAtom::iterator iml = MombySimpleMatch.lower_bound(&(*iter));
			MatchPeakAtom::iterator imu = MombySimpleMatch.upper_bound(&(*iter));
			MatchPeakAtom::iterator idl = DadbySimpleMatch.lower_bound(&(*iter));
			MatchPeakAtom::iterator idu = DadbySimpleMatch.upper_bound(&(*iter));

			//--- find the possible assignments that could for tested
			MomDadSet temp; temp.erase(temp.begin(), temp.end());
			for(MatchPeakAtom::iterator im = iml; im != imu; im++)
			for(MatchPeakAtom::iterator id = idl; id != idu; id++)
			  temp.insert(MomDad((*im).second, (*id).second));

			PeakAssignReason::iterator bl = BufferReason.lower_bound(&(*iter));
			PeakAssignReason::iterator bu = BufferReason.upper_bound(&(*iter));
			for(PeakAssignReason::iterator bi = bl; bi != bu; bi++)
			{
				if (((*bi).second.second.count(DEL_ASS) != 0) ||
				    ((bi->second.second.count(DEL_D2O) != 0) && (abs(bi->second.first.first->get_index()-bi->second.first.second->get_index()) >=3)) || //D2O data - only short range
				    (((*bi).second.second.count(DEL_LOC) != 0) && (bi->second.second.count(DEL_D2O) == 0)) ||
				    ((*bi).second.second.count(DEL_LSS) != 0) ||
				    ((*bi).second.second.count(DEL_INTRA) != 0) ||
				    ((*bi).second.second.count(DEL_IC)  != 0) ||
				    ((*bi).second.second.count(DEL_DEG) != 0) ||
				    ((*bi).second.second.count(DEL_DIA) != 0) ||
				    ((*bi).second.second.count(DEL_INT) != 0)) // ||
				{
					temp.erase((*bi).second.first);
				}
			}
			Find_bestMatch(&(*iter), temp, BList);
		}
	}
}

void NOESY::removeIC(const PeakList& ICpeakList)
{
  if (DEBUG) NOESY_Assign::outputLog() << "\n->In removeIC " << endl;
  for(PeakList::iterator i = peakList.begin(); i != peakList.end(); i++)
    {
      for(PeakList::const_iterator j = ICpeakList.begin(); j != ICpeakList.end(); j++)
	{
	  if ((fabs((*i).get_HX1() - (*j).get_HX1()) < 0.05999999) &&
	      (fabs((*i).get_HX2() - (*j).get_HX2()) < 0.05999999) &&
	      (((*j).get_D() != 3) || (fabs((*i).get_X1() - (*j).get_X1()) < 0.5999999)) &&
	      (((*j).get_D() != 4) || (fabs((*i).get_X2() - (*j).get_X2()) < 0.5999999)))
	    {
	      if (DEBUG)NOESY_Assign::outputLog() << "IC removed from the original peak list" << label() << (*i) << endl;
	      MatchPeakAtom::iterator iterMl = MombySimpleMatch.lower_bound(&(*i));
	      MatchPeakAtom::iterator iterMu = MombySimpleMatch.upper_bound(&(*i));

	      MatchPeakAtom::iterator iterFl = DadbySimpleMatch.lower_bound(&(*i));
	      MatchPeakAtom::iterator iterFu = DadbySimpleMatch.upper_bound(&(*i));

	      for(MatchPeakAtom::iterator iterM = iterMl; iterM != iterMu; iterM++)
		for(MatchPeakAtom::iterator iterF = iterFl; iterF != iterFu; iterF++)
		  {
		      MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, DEL_IC);
		  }
	    }
	}
    }
}

//----------------------------------
//SimpleMatch: By using loose tolerance, find all possible assignments,
//and assumes including all right answers.
//Global veriables:
//output: MombySimpleMatch, DadbySimpleMatch
//input : atoms_shifts, peakList
//-----------------------------------
//typedef multimap<MomDad, general_peak*, less<MomDad> > MomDadPeak;
typedef multiset<int, less<int> > countset;
typedef set<int, less<int> > countid;

bool NOESY::nearDIA(const general_peak& peak) const
{
  	if ((fabs(peak.get_HX1() - peak.get_HX2()) < 0.03999999) &&
	    ((peak.get_D() != 4) ||
	     (fabs(peak.get_X1() - peak.get_X2()) < 0.3999999)))
  		return 1;
	return 0;
}

void NOESY::SimpleMatch()
{
  if (DEBUG) NOESY_Assign::outputLog() << "In SimpleMatch " << endl;
  //--- find the well resolved peaks
  wellResolvedPeak.clear();

  for(PeakList::iterator p1 = peakList.begin(); p1 != peakList.end(); p1++)
    {
      PeakList::iterator p2;
      for(p2 = peakList.begin(); p2 != peakList.end(); p2++)
    	{
    	  if ((p1 != p2) &&
    	      (fabs((*p1).get_HX1() - (*p2).get_HX1()) < 0.03999999) &&
    	      (fabs((*p1).get_HX2() - (*p2).get_HX2()) < 0.03999999) &&
    	      (fabs((*p1).get_X1() - (*p2).get_X1()) < 0.399999) &&
    	      (fabs((*p1).get_X1() - (*p2).get_X1()) < 0.399999) &&
    	      (fabs((*p1).get_X2() - (*p2).get_X2()) < 0.399999) &&
    	      (fabs((*p1).get_X2() - (*p2).get_X2()) < 0.399999))
    	    {
    	      break;
    	    }
    	}
      if (p2 == peakList.end())
    	{
    	  wellResolvedPeak.insert(&(*p1));
    	}
    }

  Atom_Shift_List atoms_shifts; atoms_shifts.clear();
  float min_H, min_X, max_H, max_X;
  min_H = min_X = 1000;
  max_H = max_X = -1000;
  for(Atom_List::iterator iter = NOESY_Assign::atoms().begin(); iter != NOESY_Assign::atoms().end(); ++iter)
    {
      //if (DEBUG)
      //	NOESY_Assign::outputLog() << "in simple match atoms " << *iter << endl;

      atoms_shifts.insert(make_pair((*iter).get_shift(), &(*iter)));
      if ((*iter).isH() && ((*iter).get_shift() < min_H))
				min_H = (*iter).get_shift();
      if ((*iter).isH() && ((*iter).get_shift() > max_H))
				max_H = (*iter).get_shift();
      if (!(*iter).isH() && ((*iter).get_shift() < min_X))
				min_X = (*iter).get_shift();
      if (!(*iter).isH() && ((*iter).get_shift() > max_X))
				max_X = (*iter).get_shift();
    }

  for(PeakList::iterator iter=peakList.begin(); iter != peakList.end(); ++iter)
    {
      if (DEBUG)
	NOESY_Assign::outputLog() << (*iter) << endl;

      if (!nearH2O(*iter))
	{
	  MombySimpleMatch.erase(&(*iter));
	  DadbySimpleMatch.erase(&(*iter));

	  // find all the possible atoms and modify the simple_match_father
	{

	// find all possible bounds
  deque<pair<pair<float, float>, int> > bound_HX1; bound_HX1.clear();
  deque<pair<pair<float, float>, int> > bound_HX2; bound_HX2.clear();
  deque<pair<pair<float, float>, int> > bound_X1;  bound_X1.clear();
  deque<pair<pair<float, float>, int> > bound_X2;  bound_X2.clear();

	for(int N = 0; N <= 0.01 + max(fabs((*iter).get_HX1()-max_H), fabs((*iter).get_HX1()-min_H))/SWSF_HX1; N++)
	{

	   int s[2]={-1,1};
	   for(int i=0; i<2; i++)
     {
    		general_peak tp = (*iter);
    		tp.adjast_assign(s[i]*N*SWSF_HX1,0,0,0);

		bound_HX1.push_back(make_pair(get_bound_HX1(tp), N));

    		if (N==0) break;
	    }
	}
	// HX2
	for(int N = 0; N <= 0.01 + max(fabs((*iter).get_HX2()-max_H), fabs((*iter).get_HX2()-min_H))/SWSF_HX2; N++)
	{
	   int s[2]={-1,1};
	   for(int i=0; i<2; i++)
     {
    		general_peak tp = (*iter);
    		tp.adjast_assign(0,s[i]*N*SWSF_HX2,0,0);

		bound_HX2.push_back(make_pair(get_bound_HX2(tp), N));

    		if (N==0) break;
	    }
	}

	// X1
	for(int N2 = 0; N2 < 0.1 + max(fabs((*iter).get_X1()-max_X), fabs((*iter).get_X1()-min_X))/SWSF_X1; N2++)	// Dad
	{
	  if (DEBUG)
	    NOESY_Assign::outputLog() <<"in X1 1 " << N2 << endl;

	  int s[2]={-1,1};
	  for(int i=0; i<2; i++)
	    {
	      general_peak tp = (*iter);
	      tp.adjast_assign(0,0,s[i]*N2*SWSF_X1,0);

	      bound_X1.push_back(make_pair(get_bound_X1(tp), N2));

	      if (N2==0) break;
	    }
	}

	// X2
	if (DEBUG)
	  NOESY_Assign::outputLog() << max(fabs((*iter).get_X2()-max_X), fabs((*iter).get_X2()-min_X))/SWSF_X2 << endl;

	for(int N2 = 0; N2 < 0.1+max(fabs((*iter).get_X2()-max_X), fabs((*iter).get_X2()-min_X))/SWSF_X2; N2++)	// Dad
	{
	  if (DEBUG)
	    NOESY_Assign::outputLog() <<"in X2 1 " << N2 << endl;

	  int s[2]={-1,1};
	  for(int i=0; i<2; i++)
	    {
	      general_peak tp = (*iter);
	      tp.adjast_assign(0,0,0,s[i]*N2*SWSF_X2);

	      bound_X2.push_back(make_pair(get_bound_X2(tp), N2));

	      if (N2==0) break;
	    }
	}

	// filter HX1 and HX2 using X1 and X2
	map<atom*, pair<int, int>, less<atom*> > MomAtom;
	map<atom*, pair<int, int>, less<atom*> > DadAtom;
	MomAtom.clear();
	DadAtom.clear();

	for(deque<pair<pair<float, float>, int> >::iterator ai= bound_HX1.begin(); ai != bound_HX1.end(); ++ai)
	{
  	Atom_Shift_List::iterator as_iter_begin = atoms_shifts.lower_bound(ai->first.first);
  	Atom_Shift_List::iterator as_iter_end = atoms_shifts.upper_bound(ai->first.second);

    for(Atom_Shift_List::iterator aiter = as_iter_begin; aiter != as_iter_end; ++aiter)
    {
    	for(deque<pair<pair<float, float>, int> >::iterator qi = bound_X1.begin(); qi != bound_X1.end(); ++qi)
    	{

    		if (isPossMother(aiter->second, qi->first))
    		{
    			MomAtom.insert(make_pair(aiter->second, make_pair(ai->second, qi->second)));
					MombySimpleMatch.insert(make_pair((&*iter), aiter->second));
    		}
    	}

    }
	}

	for(deque<pair<pair<float, float>, int> >::iterator ai= bound_HX2.begin(); ai != bound_HX2.end(); ++ai)
	{
  	Atom_Shift_List::iterator as_iter_begin = atoms_shifts.lower_bound(ai->first.first);
  	Atom_Shift_List::iterator as_iter_end = atoms_shifts.upper_bound(ai->first.second);

    for(Atom_Shift_List::iterator aiter = as_iter_begin; aiter != as_iter_end; ++aiter)
    {
      //if (DEBUG)
      //NOESY_Assign::outputLog() << "foundmatch " << aiter->second << endl;

    	for(deque<pair<pair<float, float>, int> >::iterator qi = bound_X2.begin(); qi != bound_X2.end(); ++qi)
    	{
	    //if (DEBUG)
	    //NOESY_Assign::outputLog() << "bound " << qi->first.first << " " << qi->first.second << endl;

    		if (isPossFather(aiter->second, qi->first))
    		{
    			DadAtom.insert(make_pair(aiter->second, make_pair(ai->second, qi->second)));
					DadbySimpleMatch.insert(make_pair((&*iter), aiter->second));
					//if (DEBUG)
					//NOESY_Assign::outputLog() <<"add to DadbySimpleMatch " << aiter->second << endl;

    		}
    	}
    }
	}

	// intensity filter
	MatchPeakAtom::iterator ml = MombySimpleMatch.lower_bound(&(*iter));
	MatchPeakAtom::iterator mu = MombySimpleMatch.upper_bound(&(*iter));
  MatchPeakAtom::iterator dl = DadbySimpleMatch.lower_bound(&(*iter));
  MatchPeakAtom::iterator du = DadbySimpleMatch.upper_bound(&(*iter));

  for(MatchPeakAtom::iterator iterM = ml; iterM != mu; iterM++)
  {
    for(MatchPeakAtom::iterator iterF = dl; iterF != du; iterF++)
		{

			bool signB = 0;

  		int t = 1; // unfolded

  		if (sign.HX1)
  		{
  			signB = 1;

  			t *= (int) pow(-1,fmod(MomAtom.find(iterM->second)->second.first, 2));
  		}

  		if (sign.X1)
  		{
  			signB = 1;
  			t *= (int) pow(-1,fmod(MomAtom.find(iterM->second)->second.second, 2));

  		}

  		if (sign.HX2)
  		{
  			signB = 1;
  			t *= (int) pow(-1,fmod(DadAtom.find(iterF->second)->second.first, 2));
  		}

  		if (sign.X2)
  		{
  			signB = 1;
  			t *= (int) pow(-1,fmod(DadAtom.find(iterF->second)->second.second, 2));

 		}

  		// rule-out
  		if ((signB) && (((iter->get_Intensity() > 0) && (t == -1)) || //(fmod(t, 2) == 0)) ||
  				((iter->get_Intensity() < 0) && (t == 1)))) //(fmod(t, 2) == 1))))
  		{
  			MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, DEL_INT);
			}
		}
  }

		}
	}

  //--- find the noiseList
  if ((MombySimpleMatch.find(&(*iter)) == MombySimpleMatch.end()) || (DadbySimpleMatch.find(&(*iter)) == DadbySimpleMatch.end()))
  {
	  noiseList.push_back(&(*iter));
  }
 }

 // for d20
  for(MatchPeakAtom::iterator iterM = MombySimpleMatch.begin(); iterM != MombySimpleMatch.end(); iterM++)
    {
      MatchPeakAtom::iterator iterFl = DadbySimpleMatch.lower_bound((*iterM).first);
      MatchPeakAtom::iterator iterFu = DadbySimpleMatch.upper_bound((*iterM).first);
      for(MatchPeakAtom::iterator iterF = iterFl; iterF != iterFu; iterF++)
    	{
    	  if (!canbeDad((*iterF).second) || !canbeMom((*iterM).second))
    	    MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, DEL_D2O);
    	}
    }

}
