/***************************************************************************
                          experiment.h  -  description
                             -------------------
    begin                : Mon Aug 21 2000
    copyright            : (C) 2000 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#ifndef EXPER_H
#define EXPER_H

#include <map>
#include "SmplStat.h"
#include "peaklist.h"
#include "atom.h"
#include "tol.h"
#include "method.h"
#include "pdbreader.h"
#include "Diana.h"
#include "hbond.h"
#include "ss.h"
#include "AA.h"
#include "par.h"
#include "qscore.h"

using namespace std;
using namespace ASDP;

typedef map<int, int, less<int> > CSIMap;
typedef map<string, string, less<string> > ControlMap;

//--- peak ---//
enum NOEtype {HnHn, HaHn, HnHa, HaHa, HaHb, HbHa, EC};
typedef map<pair<pair<int, int>, NOEtype>, pair<const general_peak*, pair<float, float> >, less<pair<pair<int, int>, NOEtype> > > NOEtypeListN;
typedef multimap<pair<pair<int, int>, NOEtype>, const general_peak*, less<pair<pair<int, int>, NOEtype> > > NOEtypeList;


typedef deque<general_peak> PeakList;
typedef deque<const general_peak*> PeakPtrList;
typedef set<const general_peak*, less<const general_peak*> > PeakSet;

typedef multimap<const MomDad, const general_peak* const, less<const MomDad> > MomDadPeakMap;
typedef map<pair<int, int>, float, less<pair<int, int> > > matrix;


// for each dimension index by peak
//MombySimpleMatch, DadbySimpleMatch
// Contents of atom can be modified
typedef multimap<const general_peak* const, atom* const, less<const general_peak* const> > MatchPeakAtom;

//FatherAtom MotherAtom
typedef multimap<const atom* const, const general_peak* const, less<const atom* const> > MatchAtomPeak;

// for each dimension index by atom_index for secondary structure
typedef multimap<int, pair<const general_peak* const, atom* const>, less<int> >MatchPeakAtombyAtomIndex;

//--- assign buffer ---//

// Contents of MomDad can be modified
typedef pair<const general_peak* const, MomDad> PeakMomDad;

typedef pair<MomDad, setMethod> AssignMethod;
typedef multimap<const general_peak* const, AssignMethod, less<const general_peak* const> > PeakAssignReason;

typedef multimap<const general_peak* const, MomDad, less<const general_peak* const> > PeakMomDadMap;
typedef map<atom*, vector<float>, less<atom*> > AtomShift;
typedef multimap<MomDad, PeakMomDad, less<MomDad> > MomDadPeakAtomMap;

typedef set<pair<int, int>, less<pair<int, int> > > ContactPairSet;
typedef multimap<pair<int, int>, MomDad, less<pair<int,int> > > ContactMomDadMap;

typedef map<string, float, less<string> > LMap;

typedef multimap<pair<int, int>, PeakMomDad, less<pair<int, int> > > RingContact;
typedef set<pair<int,int>, less<pair<int, int> > > RingSet;

typedef map<MomDad, SampleStatistic> mdsMap;
typedef multimap<const general_peak*, MomDad> peakmdMap;


class SUPPORT {
public:
  	MomDadSet mdSetM;
  	MomDadPeakAtomMap mdPAMapM;
  	ContactPairSet ContactBin;
  	ContactMomDadMap MomDadBin;

	ContactPairSet ContactBinUniqueSYM;

	SUPPORT();
	~SUPPORT();

	void clear();
  	void insert(const SUPPORT&);
};

class NOESY {
protected:
	tol tols;
	tol sign;

	float iperc;
	bool waterFlag; // means if sampe is in d20
  bool haveIC;
  bool IC;

  ControlMap* peakControlPtr;
	DianaMap* dianaBufferPtr;

	//--- label ---//
	string slabel;

	PeakList peakList;
	PeakPtrList noiseList;
	PeakSet wellResolvedPeak;

	//--- Match ---//
	// by loose tol, so that the possible list include answer
	MatchPeakAtom MombySimpleMatch;
	MatchPeakAtom DadbySimpleMatch;

	// for each dimension index by atom, local referencing
	MatchAtomPeak FatherAtom;
	MatchAtomPeak MotherAtom;
	MatchAtomPeak AttMomAtom;
	MatchAtomPeak AttDadAtom;

	//--- buffer to store the reasonling result, a buffer to make assignemnt
	PeakAssignReason BufferReason;

	//--- assignment ---//
	//assingment list to store the reasonling result, a buffer to make assignemnt

	// U set
	PeakAssignReason AssignList;

	// B set
	PeakMomDadMap BList;

	mutable float KCON;
	mutable float Rmax;
	mutable float RmaxQ;

	float SWSF_X1;
	float SWSF_X2;
	float SWSF_HX1;
	float SWSF_HX2;


	float wl; // water suppression line
	float wu;

  mutable string reportAssignStat;
  mutable string reportProcessStat;
	mutable string reportPeakStat;

public:

	NOESY();
	NOESY(bool, bool, float, float, float, string);
	NOESY(bool, bool, float, float, float, float, string);
	NOESY(bool, bool, float, float, float, float, float, string);
	virtual ~NOESY();

	void setKCON(float);

	void setSWSF(tol, float, float);
	void setSWSF(tol, float, float, float);
	void setSWSF(tol, float, float, float, float);

	void setWaterLine(float, float);
	float getWL();
	float getWU();

	const string& label() const;
	const PeakList& get_peakList() const; // { return peakList; } const;
	const PeakMomDadMap& get_BList() const;
	const PeakAssignReason& get_AssignList() const;
	const PeakAssignReason& get_BufferReason() const;

	void removeIC(const PeakList&);
  bool isHaveIC() const; //{return haveIC;} const;
	bool isIC() const; //{return IC;} const;

	//--- I/O ---//
	void initReport();
  virtual void buildPeak(const vector<string>&, int) = 0;

	void setPeakControlPtr(ControlMap*);
	void setDianaBufferPtr(DianaMap*);
	void setDisFromPdbPtr(disAtomPairStat*);

	ostream& prtAmbiCASP(ostream&, ostream&, const DianaMap&) const;
	ostream& prtSimpleMatchPeakAtom(ostream&) const;
	ostream& prtAssignList(ostream&) const;
	ostream& prtZeroAssignList(ostream&) const;
	virtual ostream& prtConstraintList(ostream&) const;
	ostream& prtSource(ostream&) const; // to sparky
	virtual ostream& prtOvw(ostream&) const;
	ostream& prtNoise(ostream&) const;
	ostream& prtAssignStat(ostream&) const;
	ostream& prtPeakStat(ostream&) const;
	ostream& prtProcessStat(ostream&) const;

	//--- simple match ---//

	//--- just match by loose tol ---//
	void SimpleMatch();

	virtual bool nearH2O(const general_peak&) const = 0;
	bool nearH2o(const atom*) const;

	bool nearDIA(const general_peak&)const;
	bool isD20() const { return !waterFlag;};
	virtual bool isPossMother(const atom*, pair<float, float>) const = 0;
	virtual bool isPossFather(const atom*, pair<float, float>) const = 0;

  virtual bool canbeMom(const atom*) const = 0;
  virtual bool canbeDad(const atom*) const = 0;
	virtual const string Type() const = 0;
  float get_KC() const;
	float get_Rmax() const;
	float get_RmaxQ() const;
	void resetRmax() const;
	void resetRmaxQ() const;

	tol get_tols() const;

	void setRmaxQ(const float r) const;
	void setRmax(const float r) const;

	bool closeBy(const general_peak&, const general_peak&) const;

	/////////////////////////////////////
	// Assignment
	/////////////////////////////////////

	//--- method Assign_Multi, have noise filter ---//
	//void Assign_Unique(float, float, float, float);
	void Assign_Multi(const Ambi&, int);

	//--- method check Vio, and find the next B ---//
	void Empty_B() { BList.clear(); };
	//void Find_Vio_B();

	//--- method CF ---//
	void createContactBin();
	void Assign_CF(const Cf&);

	// Strong matched intra or seq
	void Assign_Expect(bool, const Expect&);
	void Assign_Expect_1_M(bool, const Expect&, int&, int&);
	void Assign_Expect_N_1(const Expect&, int&, int&);
	void Assign_Expect_N_M(const Expect&, int&, int&);
	void FindCisXP();

	//--- Local referencing for HX-X
	void Assign_MLoc(const Locref&);
	void Assign_DLoc(const Locref&);
	void Assign_Loc(bool, MatchAtomPeak&, MatchAtomPeak&, const Locref&);
	const MatchAtomPeak&  get_MotherAtom() const;
	const MatchAtomPeak&  get_FatherAtom() const;
	const MatchAtomPeak&  get_AttMomAtom() const;
	const MatchAtomPeak&  get_AttDadAtom() const;

	float get_HdiffAb(bool, const atom* const, const general_peak* const) const;
	float get_XdiffAb(bool, const atom* const, const general_peak* const) const;

	//--- sym
	AtomShift BuildXatomShift(const MomDad&, const MomDadPeakAtomMap&);
	void filter(const Sym&, const Locref&, const MomDadSet&, MomDadPeakAtomMap&);

	void AddPDBProfile(const MomDadPeakAtomMap&, SUPPORT&, mdsMap&, peakmdMap&);
  	int AssignGood_Pdb(const PdbP& , const MomDadPeakAtomMap&, const SUPPORT& , const mdsMap&, const peakmdMap&);
  	int AssignFar_Pdb(const PdbP& , const MomDadPeakAtomMap&, const SUPPORT& , const mdsMap&, const peakmdMap&);
	int findISO(const Vio&, ostream&);

	int Assign_4SYM(const Sym&, const MomDadSet&, const MomDadSet&, const MomDadSet&, const MomDadPeakAtomMap&, SUPPORT&);
	//int Assign_highSYM(const Sym&, const MomDadSet&, const MomDadSet&, const MomDadPeakAtomMap&, SUPPORT&);
	int Assign_SYM(int, const Sym&, const MomDadPeakAtomMap& mdPAMapA, map<MomDad, MomDad, less<MomDad> >& mdAM2,
											const MomDadSet&, const MomDadSet&, const MomDadPeakAtomMap&, SUPPORT&);
	int Assign_Unique(const Ambi&, const MomDadPeakAtomMap&, SUPPORT&);
	int Assign_UniqueLOC(const Ambi&, const MomDadPeakAtomMap& , const SUPPORT&, const peakmdMap&);
	void Build_UniqueLOC(const Ambi&, const MomDadPeakAtomMap&, SUPPORT& , MomDadSet&); //peakmdMap& );
	void Find_UniqueLOC(const Ambi&, const MomDadPeakAtomMap&, SUPPORT& , peakmdMap& );
	int findDEL_DEGLOC(const Locref&, ostream&); // find DEL_DEG
	void resetDEL_DEGLOC();
	void Assign_CfSymPp(ostream&);
  	void insertSideChainContactFaces();

	//--- EXPECT  ---//
	void Find_Expect_IntraSeq();
	//virtual void Find_Secondary();
	void Find_Helix();
	virtual void Find_Sheet();
	int del_DELLSS(ostream&);
	int del_DELTOP(ostream&);

	virtual void Find_UniqueHH(int, int, method);
	virtual void Find_UniqueAH(int, int, method)=0;
	virtual void Find_UniqueBH(int, int, method)=0;
	virtual void Find_UniqueAA(int, int, method);
	virtual void Find_UniqueAB(int, int, method);

	void Find_BbestMatch();
	void Find_bestMatch(const general_peak*, const MomDadSet&, PeakMomDadMap&);
	void Find_AllMatch(const general_peak*, MomDadSet&);

	// used for secondary structure calculation
	void Find_possSSnoe(NOEtypeList&, const Sec&);
	void Find_AssignSSnoe(NOEtypeList&);
  	void Find_SSnoe(NOEtypeListN&, const Sec&);
  	bool Find_possHaHnNoe(int dist, int indexNH);
  	bool Find_possHnHnNoe(int dist, int indexNH);

	void Find_RingPoss(const Ring&, RingContact&);
	int Assign_RingContact(const Ring&, RingContact&);

	//void weakValidate();
	int strongValidate(const Vio&, ostream&, bool);
	//void FindGood_Pdb(const mdsMap&, const peakmdMap&);
	int weakValidate(const Vio&, ostream&, const mdsMap&, const peakmdMap& peakmd);

	const pair<float, float> get_bound_HX1(const general_peak&) const; // get the bound of H
	const pair<float, float> get_bound_HX2(const general_peak&) const;// get the bound of HN

	const pair<float, float> get_bound_X1(const general_peak&) const; // get the bound of H
	const pair<float, float> get_bound_X2(const general_peak&) const;// get the bound of HN

	void MakeBuffer(const general_peak*, atom*, atom*, method);
	void MakeMomAssign(const general_peak*, atom*);
	void MakeDadAssign(const general_peak*, atom*);

	void MakeParUnAssign(bool, const general_peak*, atom*,MatchAtomPeak&, MatchAtomPeak&);

	void MakeAssign(const general_peak*, atom*, atom*, method);
  	void MakeAssignMap(MomDadPeakMap&, method, int&, int&);
	void UnAssign(const general_peak*, atom*, atom*, method, int&, int&);

	void Corr_Spectrum(float, float);
	void Corr_Spectrum(float, float, float);
	void Corr_Spectrum(float, float, float, float);

	// calibration;
	float Calibrate() const;
	void CalibrateAmbi(stringstream&, int, const Sym&);

	virtual void get_pseudoboth(const atom*, pseudo&) const;
	virtual void get_pseudo(const atom*, const atom*, pseudo&) const;
	virtual void get_pseudo2(const atom*, const atom*, pseudo&) const;

  friend istream& operator>> (istream&, NOESY&);
  friend ostream& operator<< (ostream&, const NOESY&);
  void prt_cfposs(ostream& out) const;
  void buildAcfMax(matrix&) const;
	void Find_SYMcfposs(matrix&, const Sym&, ostream&) const;

  //R-factor
	ASDP::ratio Validate_Shift(ostream&, ostream&) const;
	void Rfactor(deque<Pdb>&, ostream&, ostream&, ostream&, deque<Qscore>&);
	Qscore Rfactor(Pdb&, ostream&, ostream&, ostream&, Qscore&) const;
	void IFactor(deque<Pdb>&, ostream&, ostream&, deque<ASDP::ratio>&, deque<ASDP::ratio>&, bool);
	void LFactor(deque<Pdb>&, ostream&, ostream&, deque<ASDP::ratio>&, deque<ASDP::ratio>&, deque<ASDP::ratio>&, deque<ASDP::ratio>&, bool);
	ASDP::ratio IFactor(Pdb&, ostream&, ostream&, bool) const;
	pair<ASDP::ratio, ASDP::ratio> LFactor(Pdb&, ostream&, ostream&, bool) const;

  virtual char attType() const {return ' ';};
};

bool isSupported(pair<int,int>, const ContactPairSet&);
bool isSupported(int, MomDad, const MomDadSet, const MomDadPeakAtomMap&, const SUPPORT&, const ContactPairSet&);


#endif
