/***************************************************************************
                          experiment_Assign.cpp  -  description
                             -------------------
    begin                : Mon Aug 21 2000
    copyright            : (C) 2000 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#include <iostream>
#include <sstream>
#include <cmath>
#include <iomanip>
#include "experiment.h"
#include "noesy_assign.h"
#include "par.h"
#include "faces.h"

//#define swap(a,b) ({ typeof(a) temp = (a); (a) = (b); (b) = temp; })
#define fabs(x) fabs((double)x)
#define fmod(x,y) fmod((double)x, (double)y)
#define abs(x) abs((double)(x))

extern bool DEBUG;
extern Faces contactFaces;

typedef map<int, AA, less<int> > SEQM;
typedef map<pair<int, int>, float, less<pair<int, int> > > matrix;


string itoa(int);

typedef pair<int, int> respair;
typedef multiset<respair, less<respair> > resSet;

extern set<int, less<int> > CISPeptideBond;
extern int faces[1000][1000];


//----------------------------------
// Assign_Multi using contact profile
//
// Find the peaks with <= multi candidates, with noise filter
//----------------------------------
int NOESY::Assign_Unique(const Ambi& parUnique, const MomDadPeakAtomMap& mdPAMapM2, SUPPORT& support)
{
  if (DEBUG) NOESY_Assign::outputLog() << "\n->Assign_Unique for " << label() << endl;
  NOESY_Assign::outputExm() << endl << "# Possible uniquely-matched Peaks from " << label() << endl;

  ostringstream outStream;

  //--- find the unique peak,
  int old = AssignList.size();
  int unOld = 0;
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
    {
      if (AssignList.find(&(*iter)) == AssignList.end())
			unOld++;
    }

  int wN = 0; int iN = 0;

  MomDadPeakMap assignMap; assignMap.erase(assignMap.begin(), assignMap.end());
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
  {
      MatchPeakAtom::iterator iml = MombySimpleMatch.lower_bound(&(*iter));
			MatchPeakAtom::iterator imu = MombySimpleMatch.upper_bound(&(*iter));
			MatchPeakAtom::iterator idl = DadbySimpleMatch.lower_bound(&(*iter));
			MatchPeakAtom::iterator idu = DadbySimpleMatch.upper_bound(&(*iter));

			//---------------------------------
			// count # of possible assignments
			//---------------------------------
			int countM = 0, countF=0;
			// one < 0.03/0.03, the second one is 0.03 away


			float mdiff = 100, mdiff2 = 100;
			float ddiff = 100, ddiff2 = 100;

			MatchPeakAtom::iterator mm=iml;
			MatchPeakAtom::iterator md=idl;

			PeakAssignReason::iterator rl = BufferReason.lower_bound(&(*iter));
	    PeakAssignReason::iterator ru = BufferReason.upper_bound(&(*iter));

			MatchPeakAtom::iterator i0 = iml;
			MatchPeakAtom::iterator d0 = idl;
			for(MatchPeakAtom::iterator i=iml; i!=imu; i++)
			for(MatchPeakAtom::iterator d=idl; d!=idu; d++)
			{
				bool flag = 0;
	      for(PeakAssignReason::iterator ri = rl; ri != ru; ri++)
    		{
    		  if (((*ri).second.first.first == (*i).second) &&
    		      ((*ri).second.first.second == (*d).second) &&
							(((*ri).second.second.count(DEL_INT) != 0) ||
							 (ri->second.second.count(DEL_PDB) != 0) ||
							 (ri->second.second.count(DEL_VIO) != 0) ||
							 (ri->second.second.count(DEL_LSS) != 0) ||
							 (ri->second.second.count(DEL_D2O) != 0) ||
							 //(ri->second.second.count(DEL_SF) != 0) ||
							 (ri->second.second.count(DEL_TOP) != 0) ||
							  (ri->second.second.count(DEL_DEG) != 0)))
    		  {
    		  	flag = 1;
    		    break;
    		  }
    		} // end for ri

    		if (flag)
    			continue;

    		// calculate the differences
 				float diff = i->second->get_Hdiff(iter->get_HX1(), SWSF_HX1) + 0.1 * i->second->get_Xdiff(iter->get_X1(), SWSF_X1);

			  if (diff <= mdiff)
			  {
			  	mm = i;
			  	mdiff2 = mdiff;
			  	mdiff = diff;
			  }

			  if (diff <= 0.06000001)
			  {
  			  if ((i0 == i))
  				{
  			   	countM++;
  			  }
  			  else if ((i0 != i) && (i0->second->get_index() != i->second->get_index())) // within same contactbin, don't count
  			  {
  			  	countM++;
  			  }
  			}

				diff = d->second->get_Hdiff(iter->get_HX2(), SWSF_HX2) + 0.1 * d->second->get_Xdiff(iter->get_X2(), SWSF_X2);
			  if (diff <= ddiff)
			  {
			  	md = d;
			  	ddiff2 = ddiff;
			  	ddiff = diff;
			  }

			  if (diff <= 0.06000001)
			  {
  			  if (d0 == d)
  				{
  			    countF++;
  			  }
  			  else if ((d0 != d) && (d0->second->get_index() != d->second->get_index()))
  			  {
  			    countF++;
  			  }
  			} // end if

  			i0 = i;
  			d0 = d;
			} // end match

		if (iter->get_D() == 4)
			continue;

	  if ((AssignList.find(&(*iter)) == AssignList.end()) &&
	  		(mdiff2 - mdiff > 0.03) && (ddiff2 - ddiff > 0.03) &&
	  		(countM*countF == 1))
	  {
  		MakeBuffer(&(*iter), mm->second, md->second, U);
      NOESY_Assign::outputExm() << endl << (*iter) << endl << (*mm).second << " diff: " << (*mm).second->get_Hdiff((*iter).get_HX1(), SWSF_HX1);
      if ((*iter).get_D() > 2)
         NOESY_Assign::outputExm() << "-" << (*mm).second->get_Xdiff((*iter).get_X1(), SWSF_X1);
      NOESY_Assign::outputExm() << endl << (*md).second << " diff: " << (*md).second->get_Hdiff((*iter).get_HX2(), SWSF_HX2);
      if ((*iter).get_D() > 3)
         NOESY_Assign::outputExm() << "-" << (*md).second->get_Xdiff((*iter).get_X2(), SWSF_X2);

       bool bb = 0;
      //MomDad mdpair;
   		pair<int, int> mij;
   		if (mm->second->get_index() < md->second->get_index())
   		{
   			mij = make_pair(mm->second->get_index(), md->second->get_index());
   			//mdpair = make_pair(mm->second, md->second);
   		}
   		else
   		{
   			mij = make_pair(md->second->get_index(), mm->second->get_index());
   			//mdpair = make_pair(md->second, mm->second);
   		}

			MomDad mdpair = make_pair(mm->second, md->second);
	 		MomDadSet ASset; ASset.insert(mdpair);

			ContactPairSet AssignedContactBin; AssignedContactBin.clear();
			if (!bb && (isSupported(1, mdpair, ASset, mdPAMapM2, support, AssignedContactBin))
			    || (NOESY_Assign::MINCON() && !NOESY_Assign::isEC()))
			  {
			    if (((*mm).second->get_Xdiff((*iter).get_X1(), SWSF_X1) < parUnique.xtol + 0.000001) &&
				((*md).second->get_Xdiff((*iter).get_X2(), SWSF_X2) < parUnique.xtol + 0.000001) &&
				((*mm).second->get_Hdiff((*iter).get_HX1(), SWSF_HX1) < parUnique.htol + 0.000001) &&
				((*md).second->get_Hdiff((*iter).get_HX2(), SWSF_HX2) < parUnique.htol + 0.000001))
			      {
				NOESY_Assign::outputExm() << " Assigned" << endl;
				assignMap.insert(make_pair(MomDad((*mm).second, (*md).second), (*mm).first));
			      }
			    else
			      NOESY_Assign::outputExm() << " NotWellMatched" << endl;
			  }
			else
			  {
			    NOESY_Assign::outputExm() << " NotSupported/Suspicious" << endl;
			    if (bb)
			      NOESY_Assign::outputExm() << "-backbone" << endl;
			  }
	  }
 }

  MakeAssignMap(assignMap, UNIQUE, wN, iN);

	if (AssignList.size() - old == 0)
    return 0;

  int un = 0;
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
    {
      if (AssignList.find(&(*iter)) == AssignList.end())
	un++;
    }
  outStream.setf(ios::left, ios::adjustfield);
  outStream.width(6);
  outStream << (itoa(NOESY_Assign::ddcycle())+"-"+itoa(NOESY_Assign::dcycle())).c_str();
  outStream.width(7);

	outStream << "U";
  outStream.width(6);
  outStream << unOld - un;
  outStream.width(8);
  outStream << peakList.size() - un;
  //outStream.width(16);
  //outStream << "-";
  outStream.width(7);
  outStream << AssignList.size() - old;
  outStream.width(9);
  outStream << AssignList.size();
  outStream.width(19);
  outStream << "-";
  outStream.width(18);
  outStream << un;
  outStream.width(11);
  outStream << noiseList.size();

  if (NOESY_Assign::COMPARE())
  {
  outStream.width(8);
    outStream << "-";
  outStream.width(11);
  if (NOESY_Assign::COMPARE())
    outStream << AssignList.size() - old - iN;
  else
    outStream << "-";
  outStream.width(11);
  if (NOESY_Assign::COMPARE())
    outStream << iN;
  else
    outStream << "-";
  }

  outStream << "\n" << NOESY_Assign::EOS();
  string temp(outStream.str());
  reportProcessStat += temp.substr(0,temp.find(NOESY_Assign::EOS()));

  return AssignList.size() - old;
}

// old version

//----------------------------------
// Assign_Multi
//
// Find the peaks with <= multi candidates, with noise filter
// iperc -- cutoff for the lowest intensity
//----------------------------------
void NOESY::Assign_Multi(const Ambi& parAmbi, int multi)
{
  if (DEBUG) NOESY_Assign::outputLog() << "\n->Assign_Multi with <=" << multi << " candidates" << endl;
  NOESY_Assign::outputExm() << endl << "# Peaks with <=" << multi << " candidates from " << label() << endl;

  ostringstream outStream;

  //--- calc the 0.5% intensity of the highest intensity
  float intensity10 = 0;

  // find the possible d = 2A assignments
  PeakSet p2; p2.erase(p2.begin(), p2.end());
  // count the average
  if (p2.size() == 0)
    {
      for(PeakSet::iterator ip = wellResolvedPeak.begin();
    	  ip != wellResolvedPeak.end(); ip++)
       	{
       	  if ((fabs((*ip)->get_Intensity()) > intensity10) &&
       	      (((*ip)->get_D() != 3) || (fabs((*ip)->get_HX1() - (*ip)->get_HX2()) > 0.1)) &&
       	      (((*ip)->get_D() != 4) || ((fabs((*ip)->get_HX1() - (*ip)->get_HX2()) > 0.1) ||
       	      													 (fabs((*ip)->get_X1() - (*ip)->get_X2()) > 1.0))) &&
       	      ((MombySimpleMatch.find((*ip)) != MombySimpleMatch.end()) ||
       	       (DadbySimpleMatch.find((*ip)) != DadbySimpleMatch.end())))
       	    intensity10 = fabs((*ip)->get_Intensity());
       	}
    } else
    {
     	SampleStatistic peakI; peakI.reset();
     	for(PeakSet::iterator ip = p2.begin(); ip != p2.end(); ip++)
     	  {
     	    if (DEBUG) NOESY_Assign::outputLog() << (*ip) << endl;
     	    peakI += fabs((*ip)->get_Intensity());
     	  }
     	intensity10 = peakI.max();
    }

  NOESY_Assign::outputExm() << endl << "Highest intensity is " << intensity10 << endl;
  intensity10 *= iperc;
  NOESY_Assign::outputExm() << "Noise intensity is " << intensity10 << " ( " << iperc*100 << "% )" << endl;

  //--- find the unique peak, check the intensity
  int old = AssignList.size();
  int unOld = 0;
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
    {
      if (AssignList.find(&(*iter)) == AssignList.end())
			unOld++;
    }

  int wN = 0; int iN = 0;

  MomDadPeakMap assignMap; assignMap.erase(assignMap.begin(), assignMap.end());
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
  {
      MatchPeakAtom::iterator iml = MombySimpleMatch.lower_bound(&(*iter));
			MatchPeakAtom::iterator imu = MombySimpleMatch.upper_bound(&(*iter));
			MatchPeakAtom::iterator idl = DadbySimpleMatch.lower_bound(&(*iter));
			MatchPeakAtom::iterator idu = DadbySimpleMatch.upper_bound(&(*iter));

			int countM = 0, countF=0;

			MatchPeakAtom::iterator i0 = iml;
			for(MatchPeakAtom::iterator i=iml; i!=imu; i++)
			{
			  if (i0 == iml)
			    countM++;
			  else if ((i0 != iml) && (i0->second->get_index() != i->second->get_index())) // within same contactbin, don't count
			    countM++;
			}
			MatchPeakAtom::iterator d0 = idl;
			for(MatchPeakAtom::iterator d=idl; d!=idu; d++)
			{
			  if (d0 == idl)
			    countF++;
			  else if ((d0 != idl) && (d0->second->get_index() != d->second->get_index()))
			    countF++;
			}

			int countD = 0;
			int countDINT = 0;
		 	for(MatchPeakAtom::iterator mi = iml; mi != imu; mi++)
  	  for(MatchPeakAtom::iterator di = idl; di != idu; di++)
  	  {
	      PeakAssignReason::iterator rl = BufferReason.lower_bound(&(*iter));
	      PeakAssignReason::iterator ru = BufferReason.upper_bound(&(*iter));

	      PeakAssignReason::iterator ri;
	      for(ri = rl; ri != ru; ri++)
    		{

    		  if (((*ri).second.first.first == (*mi).second) &&
    		      ((*ri).second.first.second == (*di).second) &&
    		  		((((*ri).second.second.count(DEL_PDB) != 0) &&
    		       ((*ri).second.second.count(PDB) != 0))||
    		       ((*ri).second.second.count(DEL_LSS) != 0) ||
    		       ((*ri).second.second.count(DEL_TOP) != 0) ||
    		       ((*ri).second.second.count(DEL_INTRA) != 0) ||
							 ((*ri).second.second.count(DEL_INT) != 0) ||
    		       ((*ri).second.second.count(DEL_DIA) != 0) ||
    		       ((*ri).second.second.count(DEL_IC) != 0))) // ||
    		       //((*ri).second.second.count(DEL_SAM) != 0)))
    		  {
    		    countD++;
    		    break;
    		  }
    		}
    	}

    	for(MatchPeakAtom::iterator mi = iml; mi != imu; mi++)
  	  for(MatchPeakAtom::iterator di = idl; di != idu; di++)
  	  {
	      PeakAssignReason::iterator rl = BufferReason.lower_bound(&(*iter));
	      PeakAssignReason::iterator ru = BufferReason.upper_bound(&(*iter));

	      PeakAssignReason::iterator ri;
	      for(ri = rl; ri != ru; ri++)
    		{

    		  if (((*ri).second.first.first == (*mi).second) &&
    		      ((*ri).second.first.second == (*di).second) &&
							((*ri).second.second.count(DEL_INT) != 0))
    		  {
    		    countDINT++;
    		    break;
    		  }
    		}
    	}

    //if ((MombySimpleMatch.count(&(*iter)) == 1) && (DadbySimpleMatch.count(&(*iter)) == 1))
	  if ((AssignList.find(&(*iter)) == AssignList.end()) &&
	  		(((multi == 1) && (countM * countF - countDINT == 1)) || ((multi > 1)&&(countM * countF - countD <= multi))))
	  {
   	  for(MatchPeakAtom::iterator mi = iml; mi != imu; mi++)
  	  for(MatchPeakAtom::iterator di = idl; di != idu; di++)
  	  {
	      PeakAssignReason::iterator rl = BufferReason.lower_bound(&(*iter));
	      PeakAssignReason::iterator ru = BufferReason.upper_bound(&(*iter));
	      PeakAssignReason::iterator ri;
	      for(ri = rl; ri != ru; ri++)
    		{
    		  if (((*ri).second.first.first == (*mi).second) &&
    		      ((*ri).second.first.second == (*di).second) &&
    		      ((((*ri).second.second.count(DEL_PDB) != 0) &&
    		       ((*ri).second.second.count(PDB) != 0)) ||
    		       ((*ri).second.second.count(MULTI) !=0) ||
    		       ((*ri).second.second.count(DEL_LSS) != 0) ||
    		       ((*ri).second.second.count(DEL_TOP) != 0) ||
    		       ((*ri).second.second.count(DEL_INTRA) != 0) ||
    					 ((*ri).second.second.count(DEL_INT) != 0) ||
    		       ((*ri).second.second.count(DEL_DIA) != 0) ||
    		       ((*ri).second.second.count(DEL_IC) != 0)))// ||
    		       //((*ri).second.second.count(DEL_SAM) != 0)))
    		    break;
    		}

	      if (ri == ru)
    		{
  					if (multi == 1)
  						MakeBuffer(&(*iter), mi->second, di->second, U);
  					else
  						MakeBuffer(&(*iter), mi->second, di->second, MULTI);
           NOESY_Assign::outputExm() << endl << (*iter) << endl << (*mi).second << " diff: " << (*mi).second->get_Hdiff((*iter).get_HX1(), SWSF_HX1);
           if ((*iter).get_D() > 2)
             NOESY_Assign::outputExm() << "-" << (*mi).second->get_Xdiff((*iter).get_X1(), SWSF_X1);
           NOESY_Assign::outputExm() << endl << (*di).second << " diff: " << (*di).second->get_Hdiff((*iter).get_HX2(), SWSF_HX2);
           if ((*iter).get_D() > 3)
             NOESY_Assign::outputExm() << "-" << (*di).second->get_Xdiff((*iter).get_X2(), SWSF_X2);

           if ((fabs((*iter).get_Intensity()) > intensity10) &&
     	        (((*iter).get_D() == 2) ||
     	        (((*iter).get_D() == 3) &&
     	        ((*mi).second->get_Xdiff((*iter).get_X1(), SWSF_X1) < parAmbi.xtol + 0.000001)) ||
     	        (((*iter).get_D() == 4) &&
     	        ((*mi).second->get_Xdiff((*iter).get_X1(), SWSF_X1) < parAmbi.xtol + 0.000001) &&
     	        ((*di).second->get_Xdiff((*iter).get_X2(), SWSF_X2) < parAmbi.xtol + 0.000001))) &&
     	        ((*mi).second != (*di).second) &&
     	        ((*mi).second->get_Hdiff((*iter).get_HX1(), SWSF_HX1) < parAmbi.htol + 0.000001) &&
     	        ((*di).second->get_Hdiff((*iter).get_HX2(), SWSF_HX2) < parAmbi.htol + 0.000001))
     	    {
     	      NOESY_Assign::outputExm() << " Assigned" << endl;
     	      assignMap.insert(make_pair(MomDad((*mi).second, (*di).second), (*mi).first));
     	    }
           else
           {
     	      if (fabs((*iter).get_Intensity()) <= intensity10)
     	        NOESY_Assign::outputExm() << " WeakPeak" << endl;
     	      else
     	        NOESY_Assign::outputExm() << " NotWellMatched" << endl;
           }
        }
      }
    }
 }

 	(multi == 1) ? MakeAssignMap(assignMap, dUNIQUE, wN, iN) : MakeAssignMap(assignMap, AMBI, wN, iN);
//  MakeAssignMap(assignMap, reason, wN, iN);

	if (AssignList.size() - old == 0)
    return;

  int un = 0;
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
    {
      if (AssignList.find(&(*iter)) == AssignList.end())
	un++;
    }
  outStream.setf(ios::left, ios::adjustfield);
  outStream.width(6);
  outStream << (itoa(NOESY_Assign::ddcycle())+"-"+itoa(NOESY_Assign::dcycle())).c_str();
  outStream.width(7);

	outStream << ((multi == 1) ? "dU" : "M");
  outStream.width(6);
  outStream << unOld - un;
  outStream.width(8);
  outStream << peakList.size() - un;
  //outStream.width(16);
  //outStream << "-";
  outStream.width(7);
  outStream << AssignList.size() - old;
  outStream.width(9);
  outStream << AssignList.size();
  outStream.width(19);
  outStream << "-";
  outStream.width(18);
  outStream << un;
  outStream.width(11);
  outStream << noiseList.size();

  if (NOESY_Assign::COMPARE())
  {
  outStream.width(8);
  //if (NOESY_Assign::COMPARE())
    //    outStream << AssignList.size() - old -wN;
    //else
    outStream << "-";
  outStream.width(11);
  if (NOESY_Assign::COMPARE())
    outStream << AssignList.size() - old - iN;
  //outStream << wN - iN;
  else
    outStream << "-";
  outStream.width(11);
  if (NOESY_Assign::COMPARE())
    outStream << iN;
  else
    outStream << "-";
  }

  outStream << "\n" << NOESY_Assign::EOS();
  string temp(outStream.str());
  reportProcessStat += temp.substr(0,temp.find(NOESY_Assign::EOS()));
}

void NOESY::Assign_Expect_N_M(const Expect& parExpect, int& wN, int& iN)
{
  if (DEBUG) NOESY_Assign::outputLog() << "In Assign_N_M" << endl;
  NOESY_Assign::outputExm() << endl << "#In Assign_Expect_1_M " << label() << endl;

  //--- (n, m) mapping from expected peak to observed peak
  PeakAssignReason GoodReason;
  MomDadPeakMap momDadPeak;
  MomDadSet momDadSet;

  GoodReason.clear();
  momDadPeak.clear();
  momDadSet.clear();

  //-- find all peak that have good reason to be assigned
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
    {
      //if (AssignList.find(&(*iter)) == AssignList.end())
    	{
    	  PeakAssignReason::iterator rl = BufferReason.lower_bound(&(*iter));
    	  PeakAssignReason::iterator ru = BufferReason.upper_bound(&(*iter));

    	  for(PeakAssignReason::iterator ri = rl; ri != ru; ri++)
    	    {
    	      if (((*ri).second.second.count(DEL_ASS) == 0) && // ?
    		  			((*ri).second.second.count(DEL_IC) == 0) &&
    		  			((*ri).second.second.count(DEL_INTRA) == 0) &&
        				((*ri).second.second.count(DEL_INT) == 0) &&
         				((*ri).second.second.count(DEL_DEG) == 0) &&
    		 				((*ri).second.second.count(DEL_DIA) == 0) && // diangoal
    		  			((*ri).second.second.count(DEL_SAM) == 0) && // redudent assignments
    		  			((*ri).second.second.count(DEL_TOP) == 0) && // redudent assignments
    		  			((*ri).second.second.count(DEL_LSS) == 0))   // excluded from the secondary structure
        		{
        		  if (((*ri).second.second.count(INTRA) != 0) ||
        		      ((*ri).second.second.count(ESEQ) != 0) ||
        		      ((*ri).second.second.count(ESS) != 0))
        		    {
        		      //--- build the map between observed peak and expected peak
        		      GoodReason.insert(*ri);
        		      //--- build the map between expected peak and observed peak
        		      momDadPeak.insert(make_pair((*ri).second.first, (*ri).first));
        		      momDadSet.insert((*ri).second.first);
        		    }
        		}
    	    }
    	}
    }

  MomDadPeakMap assignMap; assignMap.erase(assignMap.begin(), assignMap.end());
  for(MomDadSet::iterator mds = momDadSet.begin(); mds != momDadSet.end(); mds++)
    {
      MomDadPeakMap::iterator mdl = momDadPeak.lower_bound(*mds);
      MomDadPeakMap::iterator mdu = momDadPeak.upper_bound(*mds);
      MomDadPeakMap::iterator mdi;
      for(mdi = mdl; mdi != mdu; mdi++)
    	{
    	  if (GoodReason.count((*mdi).second) != 1)
    	    break;
    	}
    	//if (DEBUG) cerr << "in for" << endl;
      bool pr = 0;
      if (mdi == mdu) // 1-> M mapping
      {
      	pr = 1;
      }
    	{
    	  //--- find the S-type matches, put in assignMap
    	  bool flag = 0;
    	  for(mdi = mdl; mdi != mdu; mdi++)
    	  {
	    // hx1-x1 should have strong match for 3d-noesy (hx2 have low resolution) and hx2-x2 for 4d
	    if (((mdi->second->get_D() < 3) || ((mds->first->get_Hdiff((*mdi).second->get_HX1(), SWSF_HX1) < parExpect.htol + 0.000001) &&
		((*mds).first->get_Xdiff((*mdi).second->get_X1(), SWSF_X1) < parExpect.xtol + 0.000001))) &&
		((mdi->second->get_D() != 4) || ((mds->second->get_Hdiff((*mdi).second->get_HX2(), SWSF_HX2) < parExpect.htol + 0.000001) &&
						 ((*mds).second->get_Xdiff((*mdi).second->get_X2(), SWSF_X2) < parExpect.xtol + 0.000001))))
              {
          		  	flag = 1;
          		  	assignMap.insert((*mdi));
				//if (DEBUG) NOESY_Assign::outputLog() << "in " << endl;
              }
          }
    	     	//else
      	  if (!flag)
      	    {
      	    	//if (DEBUG) cerr << "in else " << pr << endl;
      	      //--- report to user for the expected peaks that no observed peak mapped within S-type matches
      	      if (pr)
      	      {
      	        NOESY_Assign::outputExm() << endl << "NO Assignment for: " << endl << (*mds).first << endl << (*mds).second << endl;
      	        for(mdi = mdl; mdi != mdu; mdi++)
      		      {
      		        NOESY_Assign::outputExm()  << "Poss" << (*mdi).second;
                  NOESY_Assign::outputExm() << " diff: " << (*mds).first->get_Hdiff((*mdi).second->get_HX1(), SWSF_HX1);
                  if ((*mdi).second->get_D() > 2)
                    NOESY_Assign::outputExm() << "-" << (*mds).first->get_Xdiff((*mdi).second->get_X1(), SWSF_X1);

                  NOESY_Assign::outputExm() << " " << (*mds).second->get_Hdiff((*mdi).second->get_HX2(), SWSF_HX2);
                  if ((*mdi).second->get_D() > 3)
                    NOESY_Assign::outputExm() << "-" << (*mds).second->get_Xdiff((*mdi).second->get_X2(), SWSF_X2);

      						{
      							NOESY_Assign::outputExm() << " " << pow((double)get_KC()/fabs(mdi->second->get_Intensity()), (double)1.0/6.0);
      						}
                  NOESY_Assign::outputExm() << endl;
      		      }
      		      NOESY_Assign::outputExm() << endl;
      		    } // end if pr
      	    } // end if

    } // end if
  }
  MakeAssignMap(assignMap, EXPECT, wN, iN);
}

void NOESY::Assign_Expect(bool pr, const Expect& parExpect)
{
  ostringstream outStream;

  int old = AssignList.size();
  int unOld = 0;
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
    {
      if (AssignList.find(&(*iter)) == AssignList.end())
			unOld++;
    }

  int wN = 0; int iN = 0;

  Assign_Expect_N_M(parExpect, wN, iN);

  if (AssignList.size() - old == 0)
    return;

  int un = 0;
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
    {
      if (AssignList.find(&(*iter)) == AssignList.end())
	un++;
    }

  outStream.setf(ios::left, ios::adjustfield);
  outStream.width(6);
  outStream << (itoa(NOESY_Assign::ddcycle())+"-"+itoa(NOESY_Assign::dcycle())).c_str();
  outStream.width(7);
  outStream << "E";
  outStream.width(6);
  outStream << unOld - un;
  outStream.width(8);
  outStream << peakList.size() - un;
  //outStream.width(16);
  //outStream << "-";
  outStream.width(7);
  outStream << AssignList.size() - old;
  outStream.width(9);
  outStream << AssignList.size();
  outStream.width(19);
  outStream << "-";
  outStream.width(18);
  outStream << un;
  outStream.width(11);
  outStream << noiseList.size();

  if (DEBUG)
    NOESY_Assign::outputLog() << "IN ASSIGN_EXPECT wN:" << wN << " iN: " << iN << " " << AssignList.size() - old - wN << endl;

  if (NOESY_Assign::COMPARE())
  {
  outStream.width(8);
  outStream << "-";
  outStream.width(11);
  if (NOESY_Assign::COMPARE())
    outStream << AssignList.size() - old - iN;
    //outStream << wN - iN;
  else
    outStream << "-";
  outStream.width(11);
  if (NOESY_Assign::COMPARE())
    outStream << iN;
  else
    outStream << "-";
  }
  outStream << "\n" << NOESY_Assign::EOS();
  string temp(outStream.str());
  reportProcessStat += temp.substr(0,temp.find(NOESY_Assign::EOS()));
}


///////////////////////////////////////////////////////
// decide cis/trans of all XP from aa/ad assignments
///////////////////////////////////////////////////////
void NOESY::FindCisXP()
{
  if (DEBUG) NOESY_Assign::outputLog() << "In FindCis" << endl;

  ostringstream outStream;

  if (DEBUG) NOESY_Assign::outputLog() << "In FindCis" << endl;
  int NV = 0, IAI = 0, IA = 0, UA = 0; // CA =  0;

  // find all possible aa/ad assignments
	multimap<int, PeakMomDad, less<int> > XPPeaks;
	set<int, less<int> > XPPeakSet;

	for(PeakAssignReason::iterator i = AssignList.begin(); i != AssignList.end(); ++i)
	{
		// i, i-1
	  if (((i->second.first.second->get_index() - i->second.first.first->get_index()) == -1))
		{
			if (i->second.first.first->get_sym_1() == 'P') // cis-trans Pro
			{
				if (i->second.first.second->get_layer() == "A")
				{
					if ((i->second.first.first->get_layer() == "A") || (i->second.first.first->get_layer() == "D"))
						XPPeaks.insert(make_pair(i->second.first.first->get_index(), make_pair(i->first, i->second.first)));
						XPPeakSet.insert(i->second.first.first->get_index());
				}
			}
		} else // i, i+1
		if (((i->second.first.second->get_index() - i->second.first.first->get_index()) == 1))
		{
			if (i->second.first.second->get_sym_1() == 'P') // cis-trand proline
			{
				if (i->second.first.first->get_layer() == "A")
				{
					if ((i->second.first.second->get_layer() == "A") || (i->second.first.second->get_layer() == "D"))
						XPPeaks.insert(make_pair(i->second.first.second->get_index(), make_pair(i->first, i->second.first)));
						XPPeakSet.insert(i->second.first.second->get_index());
				}
			}
		}
	}

	for(set<int,less<int> >::iterator xpi = XPPeakSet.begin(); xpi != XPPeakSet.end(); ++xpi)
	{
		// select the one with highest intensity
		PeakMomDad* mi; mi = 0;
		float intensity = 0;
		for (multimap<int, PeakMomDad, less<int> >::iterator pi = XPPeaks.lower_bound(*xpi); pi != XPPeaks.upper_bound(*xpi); ++pi)
		{
			if (fabs(pi->second.first->get_Intensity()) > intensity)
			{
				intensity = fabs(pi->second.first->get_Intensity());
				mi = &(pi->second);
			}
		}

	}

}

void NOESY::insertSideChainContactFaces()
{
	set<pair<int, int> > ctSet; ctSet.clear();
	for(PeakAssignReason::iterator iter = AssignList.begin(); iter != AssignList.end(); ++iter)
	{
		// not used if labelled as DEL_SF

		// not sequentail
		if (abs(iter->second.first.first->get_index()-iter->second.first.second->get_index()) < 2)
			continue;

		bool ok = 1;
		for(PeakAssignReason::iterator j = BufferReason.lower_bound(iter->first); j != BufferReason.upper_bound(iter->first); ++j)
		{
			if ((j->second.first.first == iter->second.first.first) && (j->second.first.second == iter->second.first.second))
			{
				if (j->second.second.count(DEL_SF) != 0)
					ok = 0;
				break;
			}
		}
		if (!ok)
			continue;

		bool sidechain1 = ((iter->second.first.first->get_sym_1() != 'P') && (iter->second.first.first->get_layer() != "A") && (iter->second.first.first->get_layer() != "N") && (iter->second.first.first->get_layer() != "B"));
	  bool sidechain2 = ((iter->second.first.second->get_sym_1() != 'P') && (iter->second.first.second->get_layer() != "A") && (iter->second.first.second->get_layer() != "N") && (iter->second.first.second->get_layer() != "B"));

		bool bb1 = (((iter->second.first.first->get_layer() == "A") && (iter->second.first.first->get_sym_1() != 'G')) || (iter->second.first.first->get_layer() == "N"));
	  bool bb2 = (((iter->second.first.second->get_layer() == "A") && (iter->second.first.second->get_sym_1() != 'G')) || (iter->second.first.second->get_layer() == "N"));

		// if both side-chain, counts
		if (sidechain1 && sidechain2)
		{
			if (DEBUG)
				NOESY_Assign::outputLog() << "in SF >B" << iter->first << endl << iter->second.first.first << endl << iter->second.first.second << endl;
 			ctSet.insert(make_pair(iter->second.first.first->get_index(), iter->second.first.second->get_index()));
 			ctSet.insert(make_pair(iter->second.first.second->get_index(), iter->second.first.first->get_index()));
		}
		else
		{
			// only-helix allow b-contacts
			if ((((seq[iter->second.first.first->get_index()].get_ss()->get_type() == 1) && !bb1) || sidechain1) &&
		 		  (((seq[iter->second.first.second->get_index()].get_ss()->get_type() == 1) && !bb2) || sidechain2))
			{
				if (DEBUG)
					NOESY_Assign::outputLog() << "in SF " << iter->first << endl << iter->second.first.first << endl << iter->second.first.second << endl;
				ctSet.insert(make_pair(iter->second.first.first->get_index(), iter->second.first.second->get_index()));
				ctSet.insert(make_pair(iter->second.first.second->get_index(), iter->second.first.first->get_index()));
			}
		}
	}

	for(PeakAssignReason::iterator iter = AssignList.begin(); iter != AssignList.end(); ++iter)
	{
		if ((ctSet.find(make_pair(iter->second.first.first->get_index(), iter->second.first.second->get_index())) != ctSet.end()))
			contactFaces.insertCTatoms(iter->second.first.first->get_index(), iter->second.first.first->get_repAtom()->get_name(), iter->second.first.second->get_index(), iter->second.first.second->get_repAtom()->get_name());
	}

	if (DEBUG)
			NOESY_Assign::outputLog() << "done" << endl;
}

//----------------------------------
// Assign_CF from BList
//
// Find the assignemnt by contact frequency
//----------------------------------
void NOESY::createContactBin()
{
	for(PeakAssignReason::iterator iter = AssignList.begin(); iter != AssignList.end(); iter++)
	{
		if (((*iter).second.second.count(TEST) == 0) &&
		    ((NOESY_Assign::pdbList().size() == 0) &&
		     ((*iter).second.second.count(CORE) == 0)))
		{
			int n1 = (*iter).second.first.first->get_index();
			int n2 = (*iter).second.first.second->get_index();
			if (n1 > n2)
			{	swap(n1, n2);
			}

			// for helix, from alpha, for coil or sheet, from beta
			if ((((((*iter).second.first.first->residue().get_ss()->get_type() != -1) ||
			        (*iter).second.first.first->get_layer() != "A")) &&
			     ((*iter).second.first.first->get_layer() != "N")) &&
			    (((((*iter).second.first.second->residue().get_ss()->get_type() != -1) ||
			        (*iter).second.first.second->get_layer() != "A")) &&
			     ((*iter).second.first.second->get_layer() != "N")))
			  NOESY_Assign::ContactSet().insert(respair(n1, n2));
		}
	}
}

void NOESY::Assign_CF(const Cf& parCf)
{
  if (DEBUG) NOESY_Assign::outputLog() << "\n-> In Assign_CF" << endl;
  ostringstream outStream;

	int old = AssignList.size();
	int unOld = 0;
	for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
	{
		if (AssignList.find(&(*iter)) == AssignList.end())
			unOld++;
	}

	int wN = 0; int iN = 0;

	//--- setp 1, find the new one that fits in the contact bin
	MomDadPeakMap assignMap; assignMap.erase(assignMap.begin(), assignMap.end());
	for(PeakList::iterator i = peakList.begin(); i != peakList.end(); i++)
	  {

	    if (AssignList.find(&(*i)) == AssignList.end())
	      {
		MatchPeakAtom::iterator pl = MombySimpleMatch.lower_bound(&(*i));
		MatchPeakAtom::iterator pu = MombySimpleMatch.upper_bound(&(*i));

		MatchPeakAtom::iterator dl = DadbySimpleMatch.lower_bound(&(*i));
		MatchPeakAtom::iterator du = DadbySimpleMatch.upper_bound(&(*i));

		PeakAssignReason::iterator rl = BufferReason.lower_bound(&(*i));
		PeakAssignReason::iterator ru = BufferReason.upper_bound(&(*i));

		MomDadSet mds; mds.erase(mds.begin(), mds.end());
		for(PeakAssignReason::iterator ri = rl; ri != ru; ri++)
		  {
		    if (((*ri).second.second.count(DEL_ASS) != 0) || // ?
			((*ri).second.second.count(DEL_IC) != 0) ||
			((*ri).second.second.count(DEL_DIA) != 0) || // diangoal
			((*ri).second.second.count(DEL_INTRA) != 0) || // diangoal
     	((*ri).second.second.count(DEL_DEG) == 0) &&
			((*ri).second.second.count(DEL_INT) != 0) || // diangoal
			((*ri).second.second.count(DEL_TOP) != 0) || // diangoal
			//((*ri).second.second.count(DEL_SAM) != 0) || // redudent assignments
			((*ri).second.second.count(DEL_LSS) != 0))   // excluded from the secondary structure
		      {
			mds.insert((*ri).second.first);
		      }
		  }

		bool ifhave = 0;
		MatchPeakAtom::iterator pi;
		MatchPeakAtom::iterator di;

		//PeakMomDadMap::iterator mi;

		for(pi = pl; pi != pu; pi++)
		  {
		    for(di = dl; di != du; di++)
		      {
			if (mds.find(MomDad((*pi).second, (*di).second)) == mds.end())
			{
			    ifhave = 1;
			    if (!isIC())
			    {
			      int n1 = (*pi).second->get_index();
			      int n2 = (*di).second->get_index();
			      if (n1 > n2) 	swap(n1, n2);

			      // exclude:
			      //           <=1
			      //           seq or intra
			      if ((abs(n1-n2) < 2) ||
				        (abs(seq[n1].get_ss()->get_strand_ID() - seq[n2].get_ss()->get_strand_ID()) == 0))
			        break;
			    }
			}
		      }
		    if (di != du)
		      break;
		  }
		int count = 0;

		if ((pi == pu) && (di == du) && (ifhave))
		  {
		    PeakMomDadMap::iterator ml = BList.lower_bound(&(*i));
		    PeakMomDadMap::iterator mu = BList.upper_bound(&(*i));
		    PeakMomDadMap::iterator mi;
		    PeakMomDadMap::iterator di;
		    //MomDad temp;
		    for(di = ml; di != mu; di++)
		    {
				  int n1 = (*di).second.first->get_index();
				  int n2 = (*di).second.second->get_index();
				  if (n1 > n2) 	swap(n1, n2);
				  if (((int)NOESY_Assign::ContactSet().count(respair(n1, n2)) >= parCf.freq))
				  {
				    if (DEBUG)
				    {
				      NOESY_Assign::outputLog() << endl << (*i) << endl << (*di).second.first << " diff: " << (*di).second.first->get_Hdiff((*i).get_HX1(), SWSF_HX1);
              if ((*i).get_D() > 2)
                NOESY_Assign::outputLog() << "-" << (*di).second.first->get_Xdiff((*i).get_X1(), SWSF_X1);
              NOESY_Assign::outputLog() << endl << (*di).second.second << " diff: " << (*di).second.second->get_Hdiff((*i).get_HX2(), SWSF_HX2);
              if ((*i).get_D() > 3)
                NOESY_Assign::outputLog() << "-" << (*di).second.second->get_Xdiff((*i).get_X2(), SWSF_X2);
              NOESY_Assign::outputLog() << endl;
            }
					  count++;
					  mi = di;
				  }
			  }

		    if ((count == 1) &&
			 ((*mi).second.first->get_Hdiff((*i).get_HX1(), SWSF_HX1) < parCf.htol + 0.000001) &&
			 ((*mi).second.second->get_Hdiff((*i).get_HX2(), SWSF_HX2) < parCf.htol + 0.000001) &&
			 (((*i).get_D() == 2) ||
			  (((*i).get_D() == 3) &&
			   ((*mi).second.first->get_Xdiff((*i).get_X1(), SWSF_X1) < parCf.xtol + 0.000001))||
			  (((*i).get_D() == 4) &&
			   ((*mi).second.first->get_Xdiff((*i).get_X1(), SWSF_X1) < parCf.xtol + 0.000001) &&
			   ((*mi).second.second->get_Xdiff((*i).get_X2(), SWSF_X2) < parCf.xtol + 0.000001))) &&
			 ((*mi).second.first->get_sym_1() != 'P') &&
			 ((*mi).second.second->get_sym_1() != 'P') &&
		((((*mi).second.first->residue().get_ss()->get_type() != -1) &&
		  ((*mi).second.second->residue().get_ss()->get_type() != -1)) ||
			 ((((*mi).second.first->get_sym_1() != 'L') &&
			   ((*mi).second.first->get_sym_1() != 'F') &&
			   ((*mi).second.first->get_sym_1() != 'I') &&
			   ((*mi).second.first->get_sym_1() != 'Y') &&
			   ((*mi).second.first->get_sym_1() != 'W') &&
			   ((*mi).second.first->get_sym_1() != 'V')) ||
			  ((*mi).second.first->get_layer() != "B")) &&
			 ((((*mi).second.second->get_sym_1() != 'L') &&
			   ((*mi).second.second->get_sym_1() != 'F') &&
			   ((*mi).second.second->get_sym_1() != 'I') &&
			   ((*mi).second.second->get_sym_1() != 'Y') &&
			   ((*mi).second.second->get_sym_1() != 'W') &&
			   ((*mi).second.second->get_sym_1() != 'V')) ||
			  ((*mi).second.second->get_layer() != "B")) &&	 // the HB of Val or Leu, I, F Y W are always barried by the bulky side chains
			 (((*mi).second.first->get_sym_1() != 'I') ||
			  ((*mi).second.first->get_layer() != "G1")) &&
			 (((*mi).second.second->get_sym_1() != 'I') ||
			  ((*mi).second.second->get_layer() != "G1")) &&
			 ((*mi).second.first->get_layer() != "A") &&
			 ((*mi).second.second->get_layer() != "A")) &&
			 ((*mi).second.first->get_layer() != "N") &&
			 ((*mi).second.second->get_layer() != "N") &&
			 ((isIC()) || (abs(seq[(*mi).second.first->get_index()].get_ss()->get_strand_ID() -
			      seq[(*mi).second.second->get_index()].get_ss()->get_strand_ID()) >= 1)))
		      	assignMap.insert(pair<const MomDad, general_peak*>((*mi).second, &(*i)));
		      else if (count > 1)
		      	if (DEBUG) NOESY_Assign::outputLog() << "Count > 1, not assign" << endl;
		  }
	      }
	  }

	MakeAssignMap(assignMap, CF, wN,iN);

		if (AssignList.size() - old == 0)
    return;

	int un = 0;
	for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
	  {
	    if (AssignList.find(&(*iter)) == AssignList.end())
			un++;
	  }

  outStream.setf(ios::left, ios::adjustfield);
  outStream.width(6);
  outStream << (itoa(NOESY_Assign::ddcycle())+"-"+itoa(NOESY_Assign::dcycle())).c_str();
  outStream.width(7);
  outStream << "CF";
  outStream.width(6);
  outStream << unOld - un;
  outStream.width(8);
  outStream << peakList.size() - un;
  outStream.width(7);
  outStream << AssignList.size() - old;
  outStream.width(9);
  outStream << AssignList.size();
  outStream.width(19);
  outStream << "-";
  outStream.width(18);
  outStream << un;
  outStream.width(11);
  outStream << noiseList.size();

  if (NOESY_Assign::COMPARE())
  {
     outStream.width(8);
       outStream << "-";
     outStream.width(11);
     if (NOESY_Assign::COMPARE())
       outStream << AssignList.size() - old - iN;
       //outStream << wN - iN;
     else
       outStream << "-";
     outStream.width(11);
     if (NOESY_Assign::COMPARE())
       outStream << iN;
     else
       outStream << "-";
  }
  outStream << "\n" << NOESY_Assign::EOS();
  string temp(outStream.str());
  reportProcessStat += temp.substr(0,temp.find(NOESY_Assign::EOS()));
}

void NOESY::Find_Expect_IntraSeq()
{
	for(MatchPeakAtom::iterator iterM = MombySimpleMatch.begin(); iterM != MombySimpleMatch.end(); iterM++)
	{
		MatchPeakAtom::iterator iterFl = DadbySimpleMatch.lower_bound((*iterM).first);
		MatchPeakAtom::iterator iterFu = DadbySimpleMatch.upper_bound((*iterM).first);
		for(MatchPeakAtom::iterator iterF = iterFl; iterF != iterFu; iterF++)
		{
			// i, i-1
		  if ((((*iterF).second->get_index() - (*iterM).second->get_index()) == -1))
 			{
				if (iterM->second->get_sym_1() == 'P') // cis-trans Pro
				{
  				if ((*iterF).second->get_layer() == "A")
  				{
  					if (((*iterM).second->get_layer() == "A") || ((*iterM).second->get_layer() == "D"))
  						MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, ESEQ);
  				}
				}

				if ((*iterM).second->get_layer() == "N") // n15
				{
					if (((*iterF).second->get_layer().substr(0,1) == "A") || ((*iterF).second->get_layer().substr(0,1) == "B") ||
							((*iterF).second->get_layer() == "N"))
						MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, ESEQ);
					else if ((iterF->second->get_layer().substr(0,1) == "G") && (iterF->second->get_deg() >=3))
					  MakeBuffer(iterF->first, iterM->second, iterF->second, ESEQ);
				}
			} else // i, i+1
			if (((*iterF).second->get_index() - ((*iterM).second->get_index()) == 1))
			{
				if (iterF->second->get_sym_1() == 'P') // cis-trand proline
				{
  				if ((*iterM).second->get_layer() == "A")
  				{
  					if (((*iterF).second->get_layer() == "A") || ((*iterF).second->get_layer() == "D"))
  						MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, ESEQ);
  				}
				}

				if (((*iterF).second->get_layer() == "N") && ((*iterM).second->get_layer() == "N")) //n15
					MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, ESEQ);
				else if (((*iterM).second->get_layer().substr(0,1) == "A") || ((*iterM).second->get_layer().substr(0,1) == "B"))
				{
					if ((*iterF).second->get_layer() == "N")
						MakeBuffer((*iterF).first, (*iterM).second,(*iterF).second, ESEQ);
				}
				else if ((iterM->second->get_layer().substr(0,1) == "G") && (iterM->second->get_deg() >=3)) // methyl G
				  {
				    if (iterF->second->get_layer() == "N")
				      MakeBuffer(iterF->first, iterM->second, iterF->second, ESEQ);
				  }


			} else // i, i
			if ((*iterF).second->get_index() - ((*iterM).second->get_index()) == 0)
			{
				if (iterM->second->less4(iterF->second))
					MakeBuffer(iterF->first, iterM->second, iterF->second, INTRA);
				if (iterM->second->MethylFiveBonds(iterF->second))
				  {
				    MakeBuffer(iterF->first, iterM->second, iterF->second, INTRA);
				    if (DEBUG) NOESY_Assign::outputLog() << "methyle five bonds\n";
				  }
				if (iterM->second->get_sym_1() == 'L')
				{
					if (((iterM->second->get_layer() == "N") && ((iterF->second->get_layer() == "G") || (iterF->second->get_layer().substr(0,1) == "D"))) ||
						((iterF->second->get_layer() == "N") && ((iterM->second->get_layer() == "G") || (iterM->second->get_layer().substr(0,1) == "D"))))
					{
				    		MakeBuffer(iterF->first, iterM->second, iterF->second, INTRA);
				    		if (DEBUG) NOESY_Assign::outputLog() << "LEU intra";
					}
				}
				if (iterM->second->get_sym_1() == 'I')
				{
					if (((iterM->second->get_layer() == "N") && ((iterF->second->get_layer().substr(0,1) == "D"))) ||
						((iterF->second->get_layer() == "N") && ((iterM->second->get_layer().substr(0,1) == "D"))))
					{
				    		MakeBuffer(iterF->first, iterM->second, iterF->second, INTRA);
				    		if (DEBUG) NOESY_Assign::outputLog() << "ILE intra";
					}
				}
			}
		}
	}
}

//--------------------------------------------------
// Find_Secondary
// 	for B-sheet, for each inter strand,
//		ii-2 ii-1 ii(i-1) | i i+1 i+2
//              jj+2 jj+1 jj(j+1) | j j-1 j-2
//      for left side, ii-1, ii and jj+1 jj
//      for right side, i, i+1 and j j-1
//      for each pattern inter,
//		HNN(i, j), HAN(i+1, j), HAA(i+1, j-1), HNA(i+2, j-1)
//      for each pattern intra,
//              HNN(i, i+1), HNA(i, i+1), HAN(i, i+1), HAA(i, i+1)
//      may addin more detail stuff and more thoughts?????
//      how about budge???
//--------------------------------------------------
void NOESY::Find_Helix()
{
	if (DEBUG)
		NOESY_Assign::outputLog() << "In Find_Helix()" << endl;

  // for the seq part
  for(MatchPeakAtom::iterator iterM = MombySimpleMatch.begin(); iterM != MombySimpleMatch.end(); iterM++)
    {
      MatchPeakAtom::iterator iterFl = DadbySimpleMatch.lower_bound((*iterM).first);
      MatchPeakAtom::iterator iterFu = DadbySimpleMatch.upper_bound((*iterM).first);
      for(MatchPeakAtom::iterator iterF = iterFl; iterF != iterFu; iterF++)
	{

	  // helix, in the same strand caaaaaaac
	  // a, a and in same segment  aaaaaaaaaaaa
	  if (((seq[(*iterF).second->get_index()].get_ss()->get_type() == 1) &&
	       (seq[(*iterM).second->get_index()].get_ss()->get_type() == 1) &&
	       (seq[(*iterF).second->get_index()].get_ss()->get_strand_ID() ==
		seq[(*iterM).second->get_index()].get_ss()->get_strand_ID()))) // ||

	    {
  	    	// spin diffusion
  	     if (abs(iterF->second->get_index()-iterM->second->get_index()) > 1)
  	     {
  				if ((iterF->second->get_index() < iterM->second->get_index()))
  				{
  					if (iterM->second->get_layer() == "A")
  					{
  						MakeBuffer(iterF->first, iterM->second, iterF->second, DEL_SF);
  					} else
  					if (iterF->second->get_layer() == "N")
  					{
  						MakeBuffer(iterF->first, iterM->second, iterF->second, DEL_SF);
  					}
  				}
  				else if ((iterF->second->get_index() > iterM->second->get_index()))
  				{
  					if (iterM->second->get_layer() == "N")
  					{
  						MakeBuffer(iterF->first, iterM->second, iterF->second, DEL_SF);
  					} else
  					if (iterF->second->get_layer() == "A")
  					{
  						MakeBuffer(iterF->first, iterM->second, iterF->second, DEL_SF);
  					}
  				}
  			 }

				// i, i-1
				if (((*iterF).second->get_index() - ((*iterM).second->get_index()) == -1) &&
					(seq[(*iterF).second->get_index()].get_ss()->get_type() == 1) &&
					(seq[(*iterM).second->get_index()].get_ss()->get_type() == 1))
				{
					if ((((*iterF).second->get_layer().substr(0,1) == "A") ||
						((*iterF).second->get_layer() == "N") ||
						((*iterF).second->get_layer().substr(0,1) == "B")) &&
						((*iterM).second->get_layer() == "N"))
						MakeBuffer((*iterF).first,
							(*iterM).second,(*iterF).second, ESS);
      		} else // i, i-2
     		  if (((*iterF).second->get_index() - ((*iterM).second->get_index()) == -2) &&
     		      (seq[(*iterF).second->get_index()].get_ss()->get_type() == 1) &&
     		      (seq[(*iterM).second->get_index()].get_ss()->get_type() == 1))

     		    {
     		      if (((*iterF).second->get_layer() == "N") && ((*iterM).second->get_layer() == "N"))
     						MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, ESS);
		      else
			      MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, DEL_SF);

					faces[iterF->second->get_index()][iterM->second->get_index()] = -1;
					faces[iterM->second->get_index()][iterF->second->get_index()] = -1;
				} else // i, i-3
				if (((*iterF).second->get_index() - ((*iterM).second->get_index()) == -3) &&
					(seq[(*iterF).second->get_index()].get_ss()->get_type() == 1) &&
					(seq[(*iterM).second->get_index()].get_ss()->get_type() == 1))
				{
		      if (((*iterF).second->get_layer().substr(0,1) == "A") &&
			  ((*iterM).second->get_layer().substr(0,1) == "B"))
						MakeBuffer((*iterF).first,(*iterM).second, (*iterF).second, ESS);
		      else if (((*iterF).second->get_layer().substr(0,1) == "A") &&
						(((*iterM).second->get_layer() == "N")))
						MakeBuffer((*iterF).first,
							(*iterM).second, (*iterF).second, ESS);
				} else // i, i-4
				if (((*iterF).second->get_index() - ((*iterM).second->get_index()) == -4) &&
					(seq[(*iterF).second->get_index()].get_ss()->get_type() == 1) &&
					(seq[(*iterM).second->get_index()].get_ss()->get_type() == 1))
				{
		      	if (((*iterF).second->get_layer().substr(0,1) == "A") && (((*iterM).second->get_layer() == "N")))
							MakeBuffer((*iterF).first,(*iterM).second, (*iterF).second, ESS);
				} else // i, i+1
       if (((*iterF).second->get_index() - ((*iterM).second->get_index()) == 1) &&
		  (seq[(*iterF).second->get_index()].get_ss()->get_type() == 1) &&
		  (seq[(*iterM).second->get_index()].get_ss()->get_type() == 1))
		{
					if (((*iterF).second->get_layer() == "N") &&
						((*iterM).second->get_layer() == "N"))
						MakeBuffer((*iterF).first,(*iterM).second,(*iterF).second, ESS);
					else if ((((*iterM).second->get_layer().substr(0,1) == "A") ||
		       ((*iterM).second->get_layer().substr(0,1) == "B")) &&
		      ((*iterF).second->get_layer() == "N"))
		    MakeBuffer((*iterF).first,(*iterM).second,(*iterF).second, ESS);
		} else // i, i+2
		  if (((*iterF).second->get_index() - ((*iterM).second->get_index()) == 2) &&
		      (seq[(*iterF).second->get_index()].get_ss()->get_type() == 1) &&
		      (seq[(*iterM).second->get_index()].get_ss()->get_type() == 1))

		    {
		      if (((*iterF).second->get_layer() == "N") && ((*iterM).second->get_layer() == "N"))
						MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, ESS);
		      else
			      	MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, DEL_SF);

					faces[iterF->second->get_index()][iterM->second->get_index()] = -1;
					faces[iterM->second->get_index()][iterF->second->get_index()] = -1;

		    } else // i, i+3
		  if (((*iterF).second->get_index() - ((*iterM).second->get_index()) == 3) &&
		      (seq[(*iterF).second->get_index()].get_ss()->get_type() == 1) &&
		      (seq[(*iterM).second->get_index()].get_ss()->get_type() == 1))
		    {
		      if (((*iterM).second->get_layer().substr(0,1) == "A") &&
			  (((*iterF).second->get_layer() == "N") ||
			   ((*iterF).second->get_layer().substr(0,1) == "B")))
			MakeBuffer((*iterF).first,
				   (*iterM).second, (*iterF).second, ESS);
		    } else // i, i+4
		  if (((*iterF).second->get_index() - ((*iterM).second->get_index()) == 4) &&
		      (seq[(*iterF).second->get_index()].get_ss()->get_type() == 1) &&
		      (seq[(*iterM).second->get_index()].get_ss()->get_type() == 1))
		    {
		      if (((*iterM).second->get_layer().substr(0,1) == "A") &&
			  			(((*iterF).second->get_layer() == "N")))
						MakeBuffer((*iterF).first,(*iterM).second, (*iterF).second, ESS);
		    } else // i, j, j>=5
		      if (abs((*iterF).second->get_index() - (*iterM).second->get_index()) > 8)
					{
			  		MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, DEL_LSS);
					}
		      else
		      if (abs((*iterF).second->get_index() - (*iterM).second->get_index()) > 4)
					{
			  		MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, DEL_SF);
					faces[iterF->second->get_index()][iterM->second->get_index()] = -1;
					faces[iterM->second->get_index()][iterF->second->get_index()] = -1;
					}
	    }
	}
    }
}
void NOESY::Find_Sheet()
{
	if (DEBUG)
		NOESY_Assign::outputLog() << "In Find_Sheet()" << endl;

	int sheetFaces[1000][1000];
	for(int i=0; i<1000; ++i)
	for(int j=0; j<1000; ++j)
	{
		sheetFaces[i][j] = 0;
	}

  // for the seq part
  for(MatchPeakAtom::iterator iterM = MombySimpleMatch.begin(); iterM != MombySimpleMatch.end(); iterM++)
    {
      MatchPeakAtom::iterator iterFl = DadbySimpleMatch.lower_bound((*iterM).first);
      MatchPeakAtom::iterator iterFu = DadbySimpleMatch.upper_bound((*iterM).first);
      for(MatchPeakAtom::iterator iterF = iterFl; iterF != iterFu; iterF++)
	{
	  // sheet in the same segment bbbbbbbb
	  if (((seq[(*iterF).second->get_index()].get_ss()->get_type() == -1) &&
	       (seq[(*iterM).second->get_index()].get_ss()->get_type() == -1) &&
	       (seq[(*iterF).second->get_index()].get_ss()->get_strand_ID() ==
		seq[(*iterM).second->get_index()].get_ss()->get_strand_ID()))) // ||

	    {
				// i, i-1
				if (((*iterF).second->get_index() - ((*iterM).second->get_index()) == -1) &&
					(seq[(*iterF).second->get_index()].get_ss()->get_type() == -1) &&
					(seq[(*iterM).second->get_index()].get_ss()->get_type() == -1))
				{
					if ((((*iterF).second->get_layer().substr(0,1) == "A") || ((*iterF).second->get_layer() == "N") ||
						((*iterF).second->get_layer().substr(0,1) == "B")) && ((*iterM).second->get_layer() == "N"))
						MakeBuffer((*iterF).first,(*iterM).second,(*iterF).second, ESS);
				} else // i, i+1
	      if (((*iterF).second->get_index() - ((*iterM).second->get_index()) == 1) &&
		  				(seq[(*iterF).second->get_index()].get_ss()->get_type() == -1) &&
		 					 (seq[(*iterM).second->get_index()].get_ss()->get_type() == -1))
    		{
    			if (((*iterF).second->get_layer() == "N") &&
    					((*iterM).second->get_layer() == "N"))
    				MakeBuffer((*iterF).first, (*iterM).second,(*iterF).second, ESS);
    		  if ((((*iterM).second->get_layer().substr(0,1) == "A") || ((*iterM).second->get_layer().substr(0,1) == "B")) &&
    		      ((*iterF).second->get_layer() == "N"))
    		    MakeBuffer((*iterF).first, (*iterM).second,(*iterF).second, ESS);
    		} else // i, j, j>=4
  		  if (abs((*iterF).second->get_index() - (*iterM).second->get_index()) >= 3)
  		  {
  		     MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, DEL_LSS);
  		  }

		  // side-chain to side-chain only
        	  if (abs((*iterF).second->get_index() - (*iterM).second->get_index()) ==1)
  		  {
					sheetFaces[iterF->second->get_index()][iterM->second->get_index()] = -1;
					sheetFaces[iterM->second->get_index()][iterF->second->get_index()] = -1;
  		  }
	    }


	}
    }

  // inter strand for b-sheet
	// extending the faces, adding more DEL_LSS
  	for(int i=0; i<1000; ++i)
   	for(int j=0; j<1000; ++j)
    {
    	if (sheetFaces[i][j]) // local
     	{
      	if (DEBUG)
      		NOESY_Assign::outputLog() << "before1 sheetFaces[" << i << "][" << j << "] = " << sheetFaces[i][j] << endl;

        if (sheetFaces[i][j] == 1)
        	contactFaces.insertSheetLocal(i,j,sheetFaces[i][j]);
      }
    }


  set<pair<const Chain* const, const Chain* const>, less<pair<const Chain* const, const Chain* const> > >intraStrand;

  // for the inter-strand sheet part, antiparallel,
  for(deque<HBOND>::iterator iterH = NOESY_Assign::antiParaRegister().begin();
      iterH != NOESY_Assign::antiParaRegister().end(); iterH++)
    {
      const Structure* const strand1 = seq[(*iterH).first].get_ss();
      const Structure* const strand2 = seq[(*iterH).second].get_ss();

      const Chain& chain1 = strand1->get_chain();
      const Chain& chain2 = strand2->get_chain();

      if (DEBUG) NOESY_Assign::outputLog() << "chain1 ";
      for(Chain::iterator i=chain1.begin(); i != chain1.end(); i++)
	{
	        	if (DEBUG)
	  NOESY_Assign::outputLog() << *i << endl;
	}
      if (DEBUG) NOESY_Assign::outputLog() << "chain2 ";
      for(Chain::iterator j=chain2.begin(); j != chain2.end(); j++)
	{
	        	if (DEBUG)
	  NOESY_Assign::outputLog() << *j << endl;
	}


      if (intraStrand.find(make_pair(&chain1, &chain2)) == intraStrand.end())
	{
	  intraStrand.insert(make_pair(&chain1, &chain2));

	  //chain1 and chain2 forms sheets

	  // find the co-starting point
	  Chain::reverse_iterator j = (Chain::reverse_iterator) chain2.rbegin();
	  Chain::iterator i = chain1.begin();

	  int di = abs((*i)-(*iterH).first);
	  int dj = abs((*j)-(*iterH).second);

	  int d;
	  if (di < dj)
	    {
	      d = di;
	      while (j != chain2.rend())
		{
		  if (fabs((*j)-(*iterH).second) >= d+1)
		    j++;
		  else
		    break;
		}
	    }
	  else
	    {
	      d = dj;
	      while (i != chain1.end())
		{
		  if (fabs((*i)-(*iterH).first) > d)
		    i++;
		  else
		    break;
		}
	    }

	  //starting walking
	  while (i != chain1.end())
	    {
	      if (j != chain2.rend())
		{
		  if (fmod(d, 2) == 1)
		    {
		      // not i to j , is i to j+1    83-97
		      if (fabs((*j)-(*iterH).second) == d+1)
			    {
			      Find_UniqueAH(*i, *j, ESS);
			      sheetFaces[*i][*j] = -1;
			      sheetFaces[*j][*i] = -1;
			      contactFaces.insertSheet(*i, *j, -1);
    //   	if (DEBUG)
	  // NOESY_Assign::outputLog() << "1 set face[" << *i << "][" << *j << "] = " << sheetFaces[*i][*j] << endl;

			      j++;
			    }

		      // Ci, Cj                     83-96
		      Find_UniqueAA(*i, *j, ESS);
					sheetFaces[*i][*j] = 1;
					sheetFaces[*j][*i] = 1;
			    contactFaces.insertSheet(*i, *j, 1);
    //   	if (DEBUG)
	  // NOESY_Assign::outputLog() << "2 set face[" << *i << "][" << *j << "] = " << sheetFaces[*i][*j] << endl;

		      // 83-95
					sheetFaces[*i][*j-1] = -1;
		      sheetFaces[*j-1][*i] = -1;
			    contactFaces.insertSheet(*i, *j-1, -1);
    //   	if (DEBUG)
	  // NOESY_Assign::outputLog() << "3 set face[" << *i << "][" << *j-1 << "] = " << sheetFaces[*i][*j-1] << endl;

		      i++;
		      // Cj, Ni+1                   84-96
		      Find_UniqueAH(*j, *i, ESS);
					sheetFaces[*i][*j] = -1;
		      sheetFaces[*j][*i] = -1;
			    contactFaces.insertSheet(*i, *j, -1);

    //   	if (DEBUG)
	  // NOESY_Assign::outputLog() << "4 set face[" << *i << "][" << *j << "] = " << sheetFaces[*i][*j] << endl;
		      j++;
		      d = 0;
		    }

		  // now i = i+1, j=j-1             84-95
		  // Ni, Nj
			Find_UniqueHH(*i, *j, ESS);
			sheetFaces[*i][*j] = 1;
			sheetFaces[*j][*i] = 1;
			contactFaces.insertSheet(*i, *j, 1);

    //   	if (DEBUG)
	  // NOESY_Assign::outputLog() << "5 set face[" << *i << "][" << *j << "] = " << sheetFaces[*i][*j] << endl;
		  // i, j
		  ++i; // i+1                        85-95
		  if (i != chain1.end())
		  {
		    // i+1, j,  Ci+1, Nj
		    Find_UniqueAH(*i, *j, ESS);
				sheetFaces[*i][*j] = -1;
		    sheetFaces[*j][*i] = -1;
			  contactFaces.insertSheet(*i, *j, -1);
    //   	if (DEBUG)
	  // NOESY_Assign::outputLog() << "6 set face[" << *i << "][" << *j << "] = " << sheetFaces[*i][*j] << endl;

			}
		  ++j; // j-1
		  if ((j != chain2.rend()) && (i != chain1.end()))
		  {
		    // i+1, j-1                     85-94
		    Find_UniqueAA(*j, *i, ESS);
					sheetFaces[*i][*j] = 1;
		      sheetFaces[*j][*i] = 1;
			    contactFaces.insertSheet(*i, *j, 1);
    //   	if (DEBUG)
	  // NOESY_Assign::outputLog() << "7 set face[" << *i << "][" << *j << "] = " << sheetFaces[*i][*j] << endl;   //73-83

			  // 84-94
					sheetFaces[*i-1][*j] = -1;
		      sheetFaces[*j][*i-1] = -1;
			    contactFaces.insertSheet(*i-1, *j, -1);
    //   	if (DEBUG)
	  // NOESY_Assign::outputLog() << "8 set face[" << *i-1 << "][" << *j << "] = " << sheetFaces[*i-1][*j] << endl; //72-83

			}

		  if (i!= chain1.end()) ++i; // i+2     86-94                                                         //74
		  if (i!= chain1.end() && (j != chain2.rend())) // Ni+2, Cj-1
		  {
		    Find_UniqueAH(*j, *i, ESS);   //   86-94
					sheetFaces[*i][*j] = -1;
		      sheetFaces[*j][*i] = -1;
			    contactFaces.insertSheet(*i, *j, -1);
    //   	if (DEBUG)
	  // NOESY_Assign::outputLog() << "9 set face[" << *i << "][" << *j << "] = " << sheetFaces[*i][*j] << endl;     //74-83 but it was 75-83
		  }
		  else
		    break;

		  if (j!= chain2.rend()) j++;  //86-93, next cycle                                                    //82

    //   	if (DEBUG)
	  // NOESY_Assign::outputLog() << "j++ " << *j << endl;


		  if (i!= chain1.end() && (j != chain2.rend()))
		  {
		  		// 85-93
					sheetFaces[*i-1][*j] = -1;
		      sheetFaces[*j][*i-1] = -1;
			    contactFaces.insertSheet(*i-1, *j, -1);
    //   	if (DEBUG)
	  // NOESY_Assign::outputLog() << "10 set face[" << *i-1 << "][" << *j << "] = " << sheetFaces[*i-1][*j] << endl;   //73-82, but it was 74-81
		  }

		  // now i+2, j-2
		}
	      else
		break;
	    }
	}
    }

    if (DEBUG)
    	NOESY_Assign::outputLog() << "step 1" << endl;

	// extending the faces, adding more DEL_LSS
  	for(int i=0; i<1000; ++i)
   	for(int j=0; j<1000; ++j)
    {
    	if (sheetFaces[i][j]) // local
     	{
      	//if (DEBUG)
      	//	NOESY_Assign::outputLog() << "before sheetFaces[" << i << "][" << j << "] = " << sheetFaces[i][j] << endl;

        if (sheetFaces[i][j] == 1)
        	contactFaces.insertSheetLocal(i,j,sheetFaces[i][j]);
      }
    }


  // for the inter-strand sheet part, parallel, paraRegister (i,j) --- HaHn of I and J
  // need to be tested
  // HAHN(i,j), HNHN(i+1,j), HNHA(i+1,j+1), HNHN(i+1,j+2)
  for(deque<HBOND>::iterator iterH = NOESY_Assign::paraRegister().begin();
      iterH != NOESY_Assign::paraRegister().end(); iterH++)
    {
      const Structure* strand1 = seq[(*iterH).first].get_ss();
      const Structure* strand2 = seq[(*iterH).second].get_ss();

      const Chain& chain1 = strand1->get_chain();
      const Chain& chain2 = strand2->get_chain();

      if (intraStrand.find(make_pair(&chain1, &chain2)) == intraStrand.end())
	{
	  intraStrand.insert(make_pair(&chain1, &chain2));

	  //chain1 and chain2 forms sheets

	  // find the co-starting point
	  Chain::iterator j = chain2.begin();
	  Chain::iterator i = chain1.begin();

	  int di = abs((*i)-(*iterH).first);
	  int dj = abs((*j)-(*iterH).second);

	  int d;
	  if (di < dj)
	    {
	      d = di;
	      while (j != chain2.end())
		{
		  if (fabs((*j)-(*iterH).second) > d)
		    j++;
		  else
		    break;
		}
	    }
	  else
	    {
	      d = dj;
	      while (i != chain1.end())
		{
		  if (fabs((*i)-(*iterH).first) > d)
		    i++;
		  else
		    break;
		}
	    }

	  //starting walking
	  while (i != chain1.end())
	    {
	      if (j != chain2.end())
		{
		  if (fmod(d, 2) == 1)
		    {
		      // i--- I+1, j --- J+1   48-13
		      Find_UniqueAH(*j, *i, ESS);
					sheetFaces[*i][*j] = 1;
		      sheetFaces[*j][*i] = 1;
			    contactFaces.insertSheet(*i, *j, 1);

		      j++; // i - j+1          48-14
		      Find_UniqueHH(*j, *i, ESS);
					sheetFaces[*i][*j] = -1;
		      sheetFaces[*j][*i] = -1;
			    contactFaces.insertSheet(*i, *j, -1);

		      i++; // 49
		      d = 0;

			    // 49-13
					sheetFaces[*i][*j-1] = -1;
		      sheetFaces[*j-1][*i] = -1;
			    contactFaces.insertSheet(*i, *j-1, -1);
		    }

		  // now i = I, j = J          49-14
		  Find_UniqueAH(*i, *j, ESS);
					sheetFaces[*i][*j] = 1;
		      sheetFaces[*j][*i] = 1;
			    contactFaces.insertSheet(*i, *j, 1);

		  i++; // I+1                 50-14
		  if (i != chain1.end())
		  {
		    Find_UniqueHH(*i, *j, ESS);
					sheetFaces[*i][*j] = -1;
		      sheetFaces[*j][*i] = -1;
			    contactFaces.insertSheet(*i, *j, -1);
			}

		  ++j; // j+1                 50-15
		  if ((j != chain2.end()) && (i != chain1.end()))
		  {
		    	Find_UniqueAH(*j, *i, ESS);
					sheetFaces[*i][*j] = 1;
		      sheetFaces[*j][*i] = 1;
			    contactFaces.insertSheet(*i, *j, 1);

					// 49-15
					sheetFaces[*i-1][*j] = -1;
		      sheetFaces[*j][*i-1] = -1;
			    contactFaces.insertSheet(*i-1, *j, -1);
			}
		  if (j != chain2.end()) ++j; // i+2       50, 16
		  if (i!= chain1.end() && (j != chain2.end()))
		  {
		    	Find_UniqueHH(*i, *j, ESS);
					sheetFaces[*i][*j] = -1;
		      sheetFaces[*j][*i] = -1;
			    contactFaces.insertSheet(*i, *j, -1);
		  }
		  else
		    break;

		  if (i!= chain1.end()) i++; //  51 16, next loop

		  // 51 15
		  if ((i!= chain1.end()) && (j != chain2.end()))
		  {
					sheetFaces[*i][*j-1] = -1;
		      sheetFaces[*j-1][*i] = -1;
			    contactFaces.insertSheet(*i, *j-1, -1);
			}
		  // now i+2, j+2
		}
	      else
		break;
	    }
	}
    }

	// extending the faces, adding more DEL_LSS
  	for(int i=0; i<1000; ++i)
   	for(int j=0; j<1000; ++j)
    {
    	if (sheetFaces[i][j]) // local
     	{
      	//if (DEBUG)
      	//	NOESY_Assign::outputLog() << "sheetFaces[" << i << "][" << j << "] = " << sheetFaces[i][j] << endl;

        if (sheetFaces[i][j] == 1)
        	contactFaces.insertSheetLocal(i,j,sheetFaces[i][j]);
      }
    }

  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); ++iter)
  {
    //--- possible mother
    MatchPeakAtom::iterator momlower = MombySimpleMatch.lower_bound(&(*iter));
    MatchPeakAtom::iterator momupper = MombySimpleMatch.upper_bound(&(*iter));

    //--- possible father
    MatchPeakAtom::iterator fatherLower = DadbySimpleMatch.lower_bound(&(*iter));
    MatchPeakAtom::iterator fatherUpper = DadbySimpleMatch.upper_bound(&(*iter));

    //--- all possible assignemnts and distances for one peak
    for(MatchPeakAtom::iterator iterm = momlower; iterm != momupper; iterm++)
		{
	  	for(MatchPeakAtom::iterator iterf = fatherLower; iterf != fatherUpper; iterf++)
	    {
				// not bb
	    	if ((sheetFaces[iterm->second->get_index()][iterf->second->get_index()] == -1)
	     			&& (iterm->second->get_sym_1() != 'P') && (iterf->second->get_sym_1() != 'P')
	     			&& (((iterm->second->get_layer() != "N") &&
	     		(iterm->second->get_layer() != "A") && (iterm->second->get_layer() != "B")) && ((iterf->second->get_layer() != "N") && (iterf->second->get_layer() != "A") && (iterf->second->get_layer() != "B"))))
				  MakeBuffer((*iterf).first, (*iterm).second, (*iterf).second, DEL_LSS);
				else if ((sheetFaces[iterm->second->get_index()][iterf->second->get_index()] == -1) && (((iterm->second->get_layer() != "N") &&
	     		(iterm->second->get_layer() != "A")) || ((iterf->second->get_layer() != "N") && (iterf->second->get_layer() != "A"))))
				  MakeBuffer((*iterf).first, (*iterm).second, (*iterf).second, DEL_SF);
			}
		}
	}

	contactFaces.extendSS();
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); ++iter)
  {
    //--- possible mother
    MatchPeakAtom::iterator momlower = MombySimpleMatch.lower_bound(&(*iter));
    MatchPeakAtom::iterator momupper = MombySimpleMatch.upper_bound(&(*iter));

    //--- possible father
    MatchPeakAtom::iterator fatherLower = DadbySimpleMatch.lower_bound(&(*iter));
    MatchPeakAtom::iterator fatherUpper = DadbySimpleMatch.upper_bound(&(*iter));

    //--- all possible assignemnts and distances for one peak
    for(MatchPeakAtom::iterator iterm = momlower; iterm != momupper; iterm++)
		{
	  	for(MatchPeakAtom::iterator iterf = fatherLower; iterf != fatherUpper; iterf++)
	    {
				// not bb
				if ((contactFaces.get_sheetFace(iterm->second->get_index(), iterf->second->get_index()) == -1)
						&& (iterm->second->get_sym_1() != 'P') && (iterf->second->get_sym_1() != 'P')
						&& (((iterm->second->get_layer() != "N") &&
	     		(iterm->second->get_layer() != "A") && (iterm->second->get_layer() != "B")) && ((iterf->second->get_layer() != "N") && (iterf->second->get_layer() != "A") && (iterf->second->get_layer() != "B"))))
				  MakeBuffer((*iterf).first, (*iterm).second, (*iterf).second, DEL_LSS);
			}
		}
	}
}

int NOESY::del_DELLSS(ostream& uaStream)
{
  ostringstream outStream;
	if (DEBUG)
		NOESY_Assign::outputLog() << "In del_DELLSS()" << endl;

  int NV = 0, IAI = 0, IA = 0, UA = 0; // CA =  0;

  PeakList unPeak; unPeak.clear();
  for(PeakAssignReason::iterator iter = AssignList.begin(); iter != AssignList.end(); iter++)
  {
    	bool del = 0;
      PeakAssignReason::iterator rl = BufferReason.lower_bound(iter->first);
      PeakAssignReason::iterator ru = BufferReason.upper_bound(iter->first);
      PeakAssignReason::iterator ri;
      for(ri = rl; ri != ru; ri++)
  		{
  		  if (((*ri).second.first.first == iter->second.first.first) &&
  		      ((*ri).second.first.second == iter->second.first.second))
        {
    			if ((ri->second.second.count(DEL_LSS) != 0))
  						del = 1;
  				break;
  			}
  		}

		  if (del)
  		{
    		if (DEBUG)
      	{
    			if (!UA)
    			{
    				uaStream << endl;
    				uaStream << "==================" << endl;
    				uaStream << (itoa(NOESY_Assign::ddcycle())+"-"+itoa(NOESY_Assign::dcycle())).c_str() << " " << label() << endl;
    			}

          uaStream << (*iter).first << endl << (*iter).second.first.first << endl
    		      << (*iter).second.first.second;
       		for(setMethod::iterator im = (*iter).second.second.begin(); im != (*iter).second.second.end(); im++)
       			uaStream << (*im) << " ";
    			uaStream << endl;
  				uaStream << "Conflict with Secondary Structure " << endl;
    		  uaStream << endl;
      	}
	      // delete the assignment
	      unPeak.push_back(*((*iter).first));
	      // unassingn constraints and assignments
	      UnAssign((*iter).first, (*iter).second.first.first,
		       (*iter).second.first.second, DEL_LSS, IA, IAI);
	      UA++;
	    } // end !pdb
  }

  // if no violation, return
  if (!UA)
    return UA;

  for(PeakList::iterator pi = unPeak.begin(); pi != unPeak.end(); pi++)
  {
    if (AssignList.find(&(*pi)) == AssignList.end())
  	{
  	  NV++;
  	}
  }

  int un = 0;
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
  {
    if (AssignList.find(&(*iter)) == AssignList.end())
		un++;
  }

  outStream.setf(ios::left, ios::adjustfield);
  outStream.width(6);
  outStream << (itoa(NOESY_Assign::ddcycle())+"-"+itoa(NOESY_Assign::dcycle())).c_str();
  outStream.width(7);
  outStream << "DLSS";
  outStream.width(6);
  outStream << "-";
  outStream.width(8);
  outStream << peakList.size() - un; //???
  outStream.width(7);
  outStream << "-";
  outStream.width(9);
  outStream << AssignList.size();
  outStream.width(19);
  outStream << UA;
  outStream.width(18);
  outStream << un;
  outStream.width(11);
  outStream << noiseList.size();

  if (NOESY_Assign::COMPARE())
  {
    outStream.width(8);
      outStream << "-";
    outStream.width(11);
    if (NOESY_Assign::COMPARE())
      outStream << -1 * (IA - IAI);
    else
      outStream << "-";
    outStream.width(11);
    if (NOESY_Assign::COMPARE())
      outStream << -1 * IAI;
    else
      outStream << "-";
  }
  outStream << "\n" << NOESY_Assign::EOS();
  string temp(outStream.str());
  reportProcessStat += temp.substr(0,temp.find(NOESY_Assign::EOS()));

  return UA;
}

//label it only, don't unassign
//if labelled, not used for sym/best assignment method
int NOESY::del_DELTOP(ostream& uaStream)
{
  ostringstream outStream;

  //if (DEBUG) NOESY_Assign::outputLog() << "Structure Validating" << endl;
  int NV = 0, IAI = 0, IA = 0, UA = 0; // CA =  0;

  for(PeakAssignReason::iterator i = BufferReason.begin(); i != BufferReason.end(); ++i)
  {
  	if (i->second.second.find(DEL_TOP) != i->second.second.end())
  	{
  			i->second.second.erase(i->second.second.find(DEL_TOP));
    }
  }

// in SF, only helix to allow HB contacts

  for(MatchPeakAtom::iterator iterM = MombySimpleMatch.begin(); iterM != MombySimpleMatch.end(); iterM++)
  {
      MatchPeakAtom::iterator iterFl = DadbySimpleMatch.lower_bound((*iterM).first);
      MatchPeakAtom::iterator iterFu = DadbySimpleMatch.upper_bound((*iterM).first);

      for(MatchPeakAtom::iterator iterF = iterFl; iterF != iterFu; iterF++)
			{
				bool bbM = ((iterM->second->get_layer() == "A") || (iterM->second->get_layer() == "N"));
				bool bbF = ((iterF->second->get_layer() == "A") || (iterF->second->get_layer() == "N"));

				// spin diffusion
				// for helix 107 - 10, 11, 7, 14, no ha107 to 14side-chain
          if (((seq[iterF->second->get_index()].get_ss()->get_type() == 1) &&  // helix-something packing
							 (seq[iterM->second->get_index()].get_ss()->get_type() == 1) &&
							 (seq[(*iterF).second->get_index()].get_ss()->get_strand_ID() !=
									seq[(*iterM).second->get_index()].get_ss()->get_strand_ID()) &&
							 ((iterM->second->get_layer() == "A") || (iterF->second->get_layer() == "A")||
							  (iterM->second->get_layer() == "N") || (iterF->second->get_layer() == "N"))))
					{
							//-3, 4
							if ((contactFaces.get_ctFace(iterM->second->get_index(), iterF->second->get_index()-7) == 1) &&
									(seq[(*iterF).second->get_index()-7].get_ss()->get_strand_ID() ==
										seq[(*iterF).second->get_index()].get_ss()->get_strand_ID()) &&
									((iterF->second->get_layer() == "A")) && // || bbM) &&
									(contactFaces.get_ctFace(iterM->second->get_index(), iterF->second->get_index()) == 1))
							{
								MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, DEL_TOP);
							}
							else if ((contactFaces.get_ctFace(iterM->second->get_index()-7, iterF->second->get_index()) == 1) &&
									(seq[(*iterM).second->get_index()-7].get_ss()->get_strand_ID() ==
										seq[(*iterM).second->get_index()].get_ss()->get_strand_ID()) &&
									((iterM->second->get_layer() == "A")) && // || bbF) &&
									(contactFaces.get_ctFace(iterM->second->get_index(), iterF->second->get_index()) == 1))
							{
								MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, DEL_TOP);
							}
							else if ((contactFaces.get_ctFace(iterM->second->get_index(), iterF->second->get_index()+7) == 1) &&
									(seq[(*iterF).second->get_index()+7].get_ss()->get_strand_ID() ==
										seq[(*iterF).second->get_index()].get_ss()->get_strand_ID()) &&
									((iterF->second->get_layer() == "N")) && // || bbM) &&
									(contactFaces.get_ctFace(iterM->second->get_index(), iterF->second->get_index()) == 1))
							{
								MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, DEL_TOP);
							}
							else if ((contactFaces.get_ctFace(iterM->second->get_index()+7, iterF->second->get_index()) == 1) &&
									(seq[(*iterM).second->get_index()+7].get_ss()->get_strand_ID() ==
										seq[(*iterM).second->get_index()].get_ss()->get_strand_ID()) &&
									((iterM->second->get_layer() == "N")) && // || bbF) &&
									(contactFaces.get_ctFace(iterM->second->get_index(), iterF->second->get_index()) == 1))
							{
								MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, DEL_TOP);
							}

							//k, K+5
							if ((contactFaces.get_ctFace(iterM->second->get_index(), iterF->second->get_index()-5) == 1) &&
									(seq[(*iterF).second->get_index()-5].get_ss()->get_strand_ID() ==
										seq[(*iterF).second->get_index()].get_ss()->get_strand_ID()) &&
									((iterF->second->get_layer() == "A")) && // || bbM) &&
									(contactFaces.get_ctFace(iterM->second->get_index(), iterF->second->get_index()) == 1))
							{
								MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, DEL_TOP);
							}
							else if ((contactFaces.get_ctFace(iterM->second->get_index()-5, iterF->second->get_index()) == 1) &&
									(seq[(*iterM).second->get_index()-5].get_ss()->get_strand_ID() ==
										seq[(*iterM).second->get_index()].get_ss()->get_strand_ID()) &&
									((iterM->second->get_layer() == "A")) && // || bbF) &&
									(contactFaces.get_ctFace(iterM->second->get_index(), iterF->second->get_index()) == 1))
							{
								MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, DEL_TOP);
							}
							else if ((contactFaces.get_ctFace(iterM->second->get_index(), iterF->second->get_index()+5) == 1) &&
									(seq[(*iterF).second->get_index()+5].get_ss()->get_strand_ID() ==
										seq[(*iterF).second->get_index()].get_ss()->get_strand_ID()) &&
									((iterF->second->get_layer() == "N")) && // || bbM) &&
									(contactFaces.get_ctFace(iterM->second->get_index(), iterF->second->get_index()) == 1))
							{
								MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, DEL_TOP);
							}
							else if ((contactFaces.get_ctFace(iterM->second->get_index()+5, iterF->second->get_index()) == 1) &&
									(seq[(*iterM).second->get_index()+5].get_ss()->get_strand_ID() ==
										seq[(*iterM).second->get_index()].get_ss()->get_strand_ID()) &&
									((iterM->second->get_layer() == "N")) && // || bbF) &&
									(contactFaces.get_ctFace(iterM->second->get_index(), iterF->second->get_index()) == 1))
							{
								MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, DEL_TOP);
							}

							//k+1, K+3
							if ((contactFaces.get_ctFace(iterM->second->get_index(), iterF->second->get_index()-2) == 1) &&
									(seq[(*iterF).second->get_index()-2].get_ss()->get_strand_ID() ==
										seq[(*iterF).second->get_index()].get_ss()->get_strand_ID()) &&
									((iterF->second->get_layer() == "A")) && // || bbM) &&
									(contactFaces.get_ctFace(iterM->second->get_index(), iterF->second->get_index()) == 1))
							{
								MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, DEL_TOP);
							}
							else if ((contactFaces.get_ctFace(iterM->second->get_index()-2, iterF->second->get_index()) == 1) &&
									(seq[(*iterM).second->get_index()-2].get_ss()->get_strand_ID() ==
										seq[(*iterM).second->get_index()].get_ss()->get_strand_ID()) &&
									((iterM->second->get_layer() == "A")) && // || bbF) &&
									(contactFaces.get_ctFace(iterM->second->get_index(), iterF->second->get_index()) == 1))
							{
								MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, DEL_TOP);
							}
							else if ((contactFaces.get_ctFace(iterM->second->get_index(), iterF->second->get_index()+2) == 1) &&
									(seq[(*iterF).second->get_index()+2].get_ss()->get_strand_ID() ==
										seq[(*iterF).second->get_index()].get_ss()->get_strand_ID()) &&
									((iterF->second->get_layer() == "N")) && // || bbM) &&
									(contactFaces.get_ctFace(iterM->second->get_index(), iterF->second->get_index()) == 1))
							{
								MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, DEL_TOP);
							}
							else if ((contactFaces.get_ctFace(iterM->second->get_index()+2, iterF->second->get_index()) == 1) &&
									(seq[(*iterM).second->get_index()+2].get_ss()->get_strand_ID() ==
										seq[(*iterM).second->get_index()].get_ss()->get_strand_ID()) &&
									((iterM->second->get_layer() == "N")) && // || bbF) &&
									(contactFaces.get_ctFace(iterM->second->get_index(), iterF->second->get_index()) == 1))
							{
								MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, DEL_TOP);
							}
					}

 				if ((contactFaces.get_ctHelixLocalFace(iterM->second->get_index(), iterF->second->get_index()) == -1))
      	{
          {
	 					if (fabs(iterM->second->get_index()-iterF->second->get_index()) > 5)
						  {
 						  MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, DEL_TOP);
						  if (DEBUG)
						    NOESY_Assign::outputLog() << "DEL_TOP HelixLocalFace -1" << iterF->first << endl << iterM->second << endl << iterF->second << endl;
						  }
         	}
       	}
        // helix to helix face
				else if	((contactFaces.get_ctHelixPlaneFace(iterM->second->get_index(), iterF->second->get_index()) == -1) && // only both helical
				                 (seq[iterM->second->get_index()].get_ss()->get_type() == 1) &&
				                 (seq[iterF->second->get_index()].get_ss()->get_type() == 1))
 				{
 					if (fabs(iterM->second->get_index()-iterF->second->get_index()) > 5)
 					  MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, DEL_TOP);
 				}
     		else if ((contactFaces.get_ctSheetLocalFace(iterM->second->get_index(), iterF->second->get_index()) == -1))
       	{
 					  MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, DEL_TOP);

					  if (DEBUG)
					    NOESY_Assign::outputLog() << "DEL_TOP get_ctSheetLocalFace -1" << iterF->first << endl << iterM->second << endl << iterF->second << endl;

        }
        else if (((contactFaces.get_ctSheetPlaneFace(iterM->second->get_index(), iterF->second->get_index()) == -1) ||
					  (contactFaces.get_ctHelixPlaneFace(iterM->second->get_index(), iterF->second->get_index()) == -1))
					  && (((iterM->second->get_layer() == "N") || (iterM->second->get_sym_1() == 'P') || (iterM->second->get_layer() == "A")) ||
					  		((iterF->second->get_layer() == "N") || (iterF->second->get_sym_1() == 'P') || (iterF->second->get_layer() == "A"))))
				{
					MakeBuffer(iterF->first, iterM->second, iterF->second, DEL_SF);
				}
        // other related to non-helix, only side-chain - to side-chain
				else if (((contactFaces.get_ctSheetPlaneFace(iterM->second->get_index(), iterF->second->get_index()) == -1) ||
					  (contactFaces.get_ctHelixPlaneFace(iterM->second->get_index(), iterF->second->get_index()) == -1))
	    	          && (iterM->second->get_layer() != "N") && (iterF->second->get_layer() != "N")
	               	&& (iterM->second->get_sym_1() != 'P') && (iterF->second->get_sym_1() != 'P')
	    	          && ((iterM->second->get_layer() != "B") || (seq[iterM->second->get_index()].get_ss()->get_type() == 1))
	               	&& ((iterF->second->get_layer() != "B") || (seq[iterF->second->get_index()].get_ss()->get_type() == 1))
	                && (iterM->second->get_layer() != "A") && (iterF->second->get_layer() != "A"))
				{
          bool del = 1;
          PeakAssignReason::iterator rl = BufferReason.lower_bound(iterM->first);
          PeakAssignReason::iterator ru = BufferReason.upper_bound(iterM->first);
          PeakAssignReason::iterator ri;
          for(ri = rl; ri != ru; ri++)
      		{
      		  if (((*ri).second.first.first == iterM->second) &&
      		      ((*ri).second.first.second == iterF->second))
            {
        			if (ri->second.second.count(DEL_SF) != 0)
      						del = 0;
      				break;
      			}
      		}
          if (del)
	    MakeBuffer((*iterF).first, (*iterM).second, (*iterF).second, DEL_TOP);
	  if (DEBUG)
	    NOESY_Assign::outputLog() << "DEL_TOP PlaneFace other" << iterF->first << endl << iterM->second << endl << iterF->second << endl
				      << contactFaces.get_ctSheetPlaneFace(iterM->second->get_index(), iterF->second->get_index()) << " "
				      << contactFaces.get_ctHelixPlaneFace(iterM->second->get_index(), iterF->second->get_index());
    		}
			}
	}

  return UA;
}
