/***************************************************************************
                          experiment_Locref.cpp  -  local referenecing
                             -------------------
    begin                : Mon Jul 29 2002
    copyright            : (C) 2002 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/
//#include <iostream>
//#include <iomanip>

//#include <strstream>
#include <cmath>
#include "experiment.h"
#include "noesy_assign.h"

#define pow(x,y) pow((double)x, (double)y)
extern bool DEBUG;

typedef set<pair<atom*, general_peak*>, less<pair<atom*, general_peak*> > > AtomPeakPS;

const MatchAtomPeak& NOESY::get_MotherAtom() const
{
	return MotherAtom;
}
const MatchAtomPeak& NOESY::get_FatherAtom() const
{
	return FatherAtom;
}
const MatchAtomPeak& NOESY::get_AttMomAtom() const
{
	return AttMomAtom;
}
const MatchAtomPeak& NOESY::get_AttDadAtom() const
{
	return AttDadAtom;
}

void NOESY::Assign_MLoc(const Locref& parLocref)
{
  if (DEBUG) NOESY_Assign::outputLog() << "\n-> In Assign_MLoc" << endl;
 if (DEBUG) NOESY_Assign::outputLog() << MotherAtom.size() << " " << AttMomAtom.size() << endl;
	Assign_Loc(1, MotherAtom, AttMomAtom, parLocref);	
}

void NOESY::Assign_DLoc(const Locref& parLocref)
{
  if (DEBUG) NOESY_Assign::outputLog() << "\n-> In Assign_DLoc" << endl;
  if (DEBUG) NOESY_Assign::outputLog() << FatherAtom.size() << " " << AttDadAtom.size() << endl;
	Assign_Loc(0, FatherAtom, AttDadAtom, parLocref);
}


float NOESY::get_HdiffAb(bool isMom, const atom* const atom, const general_peak* const peak) const
{
	if (isMom)
		return atom->get_HdiffAb(peak->get_HX1(), SWSF_HX1);
	else
		return atom->get_HdiffAb(peak->get_HX2(), SWSF_HX2);
}

float NOESY::get_XdiffAb(bool isMom, const atom* const atom, const general_peak* const peak) const
{
	if (isMom)
		return atom->get_XdiffAb(peak->get_X1(), SWSF_X1);
	else
		return atom->get_XdiffAb(peak->get_X2(), SWSF_X2);
}

void NOESY::Assign_Loc(bool isMom, MatchAtomPeak& ParAtom, MatchAtomPeak& AttParAtom, const Locref& parLocref)
{
 
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
    {
	{
	  if (DEBUG) NOESY_Assign::outputLog() << "-->" << (*iter) << endl;
	
		MatchPeakAtom::iterator il;
		MatchPeakAtom::iterator iu;
		if (isMom)
		{
	  	il = MombySimpleMatch.lower_bound(&(*iter));
	  	iu = MombySimpleMatch.upper_bound(&(*iter));
		} else
		{
	  	il = DadbySimpleMatch.lower_bound(&(*iter));
	  	iu = DadbySimpleMatch.upper_bound(&(*iter));
		}
		
 		MatchPeakAtom::iterator dl;
 		MatchPeakAtom::iterator du;
 		if (isMom)
 		{
 	  	dl = DadbySimpleMatch.lower_bound(&(*iter));
 	  	du = DadbySimpleMatch.upper_bound(&(*iter));
 		} else
 		{
 	  	dl = MombySimpleMatch.lower_bound(&(*iter));
 	  	du = MombySimpleMatch.upper_bound(&(*iter));
 		}

	  //--- for each possible assignment, check if the peak's shift is on the root region
	  // keep the smaller one for hx1 roots
	  
		float dmin = 100; 
		multimap<float, atom*, less<float> > roots;  

	  for(MatchPeakAtom::iterator i = il; i != iu; i++)
	    {
	      if (DEBUG) NOESY_Assign::outputLog() << "***>" << (*i).second << endl;
	      SampleStatistic sampleA; sampleA.reset();
	
	      MatchAtomPeak::iterator al = ParAtom.lower_bound((*i).second);
	      MatchAtomPeak::iterator au = ParAtom.upper_bound((*i).second);
	
	      if (DEBUG)
	      {
	      	if (isMom)
	      		NOESY_Assign::outputLog() << "For HX1" << endl;
	      	else
	      		NOESY_Assign::outputLog() << "For HX2" << endl;
	      }
	
	      for(MatchAtomPeak::iterator ai = al; ai != au; ai++)
    		{
    		  sampleA += get_HdiffAb(isMom, ai->first, ai->second);
    		  		  	
    		  if (DEBUG)
    		  {
		    NOESY_Assign::outputLog() << (*ai).second << " " << get_HdiffAb(isMom, ai->first, ai->second) << endl;    			
		  }
    		}
    		
	      if (DEBUG) NOESY_Assign::outputLog() << "HX mean " << sampleA.mean() << endl;

	      SampleStatistic sampleM; sampleM.reset();
	      MatchAtomPeak::iterator ml;
	      MatchAtomPeak::iterator mu;
	      
	      if (isMom)
		{
		  ml = AttParAtom.lower_bound((*i).second->att_X((*iter).get_X1(), SWSF_X1));
		  mu = AttParAtom.upper_bound((*i).second->att_X((*iter).get_X1(), SWSF_X1));
		} else
		  {
		    ml = AttParAtom.lower_bound((*i).second->att_X((*iter).get_X2(), SWSF_X2));
		    mu = AttParAtom.upper_bound((*i).second->att_X((*iter).get_X2(), SWSF_X2));
		  }
	      
	      if (DEBUG)
	      {
	      	if (isMom)
	      		NOESY_Assign::outputLog() << "For X1" << endl;
	      	else
	      		NOESY_Assign::outputLog() << "For X2" << endl;
	      }
	
	      for(MatchAtomPeak::iterator mi = ml; mi != mu; mi++)
    		{
		  sampleM += get_XdiffAb(isMom, mi->first, mi->second);
		  if (DEBUG) NOESY_Assign::outputLog() << mi->second << " " << get_XdiffAb(isMom, mi->first, mi->second) << endl;
    		}
    			
	      if (DEBUG) NOESY_Assign::outputLog() << "X mean " << sampleM.mean() << endl;


	      // find the best match 
	      float dist = fabs(sampleA.mean() - get_HdiffAb(isMom, i->second, &(*iter))) + fabs(sampleM.mean() - get_XdiffAb(isMom, i->second, &(*iter)))/10.0; 
	      roots.insert(make_pair(dist, i->second)); 

	      if (dist + 0.000001 < dmin)
		{
		  dmin = dist; 
		}

	      //--- if not both in confindent region, Label DEL_LOC
 
	      if ((sampleA.count()) && (sampleA.stdDev() < parLocref.htol2 + 0.000001))
    		{
		  // 0.04 - 0.004 = 0.036
		  if (fabs(get_HdiffAb(isMom, i->second, &(*iter)) - sampleA.mean()) > parLocref.htol + 0.005)
    		    {
		      MakeParUnAssign(isMom, &*iter, i->second, ParAtom, AttParAtom);
		      
    		      if (DEBUG) NOESY_Assign::outputLog() << "CASE HX" << "mean: " << sampleA.mean() << "std: " << sampleA.stdDev() <<
    				   "diff:  " << get_HdiffAb(isMom, i->second, &(*iter)) //<< (*i).second->get_HdiffAb((*iter).get_HX1(), SWSF_HX1)
							   << endl;
		      
    		      for(MatchPeakAtom::iterator di = dl; di != du; di++)
			{
			  if (isMom)
			    MakeBuffer(&(*iter), (*i).second, (*di).second, DEL_LOC);
			  else
			    MakeBuffer(&*iter, di->second, i->second, DEL_LOC);
			}
    		    }
    		  else
    		    {
    		      if (DEBUG) NOESY_Assign::outputLog() << "HX mean: " << sampleA.mean() <<
    				   "diff:  " << get_HdiffAb(isMom, i->second, &(*iter))//<< (*i).second->get_HdiffAb((*iter).get_HX1(), SWSF_HX1)
    				      << endl;
    		    }
				}

	      if ((sampleM.count()) && (sampleM.stdDev() < parLocref.xtol2 + 0.000001))
    		{
    			// 0.4 - 0.04 = 0.36
    			if (fabs(get_XdiffAb(isMom, i->second, &(*iter)) - sampleM.mean()) > parLocref.xtol + 0.05)
    		    {
    		    	MakeParUnAssign(isMom, &(*iter), i->second, ParAtom, AttParAtom);
    		    	
    		      if (DEBUG) NOESY_Assign::outputLog() << "CASE X" << " mean: " << sampleM.mean() << " std: " << sampleM.stdDev() <<
    				   " diff:  " << get_XdiffAb(isMom, i->second, &(*iter)) //(*i).second->get_XdiffAb((*iter).get_X1(), SWSF_X1)
    				      << endl;
    		      for(MatchPeakAtom::iterator di = dl; di != du; di++)
        			{
        				if (isMom)
        			  	MakeBuffer(&(*iter), (*i).second, (*di).second, DEL_LOC);
        			  else
        			  	MakeBuffer(&*iter, di->second, i->second, DEL_LOC);
        			}
    		    }
    		  else
    		    {
    		      if (DEBUG) NOESY_Assign::outputLog() << "X mean: " << sampleM.mean() <<
    				   "diff:  " << get_XdiffAb(isMom, i->second, &(*iter)) //(*i).second->get_XdiffAb((*iter).get_X1(), SWSF_X1)
    				      << endl;
    		    }
    		} // if 
	    } // for 

	  // remove roots 
	  for(multimap<float, atom*, less<float> >::iterator i = roots.begin(); i != roots.end(); i++)
	    {
	      if (fabs(i->first - dmin) > 0.021)
		{
		  for(MatchPeakAtom::iterator di = dl; di != du; di++)
		    {
		      if (isMom)
			MakeBuffer((&*iter), i->second, di->second, DEL_LOC); 
		      else
			MakeBuffer(&*iter, di->second, i->second, DEL_LOC); 
		    }
		  if (DEBUG) NOESY_Assign::outputLog() << "CASE root" << " dmin " << dmin << "this one: " << i->first << endl; 
		}
	    }

	}
    }

}

void NOESY::resetDEL_DEGLOC()
{
  for(PeakAssignReason::iterator i = BufferReason.begin(); i != BufferReason.end(); ++i)
  {
	if (i->second.second.count(DEL_LOC))
  	{
  		i->second.second.erase(i->second.second.find(DEL_LOC));
        }
  	if (i->second.second.count(DEL_DEG))
  	{
  		i->second.second.erase(i->second.second.find(DEL_DEG));
       }
  }
}

	  
int NOESY::findDEL_DEGLOC(const Locref& parLocref, ostream& uaStream)
{
ostringstream outStream;

//if (DEBUG) NOESY_Assign::outputLog() << "Structure Validating" << endl;
int NV = 0, IAI = 0, IA = 0, UA = 0; // CA =  0;

PeakList unPeak; unPeak.erase(unPeak.begin(), unPeak.end());

for(PeakList::iterator iter=peakList.begin(); iter != peakList.end(); ++iter)
{
	multimap<MomDad, MomDad, less<MomDad> > atpair; atpair.clear();
	set<MomDad, less<MomDad> > atset; atset.clear();
	set<MomDad> delset; delset.clear();
	
	MatchPeakAtom::iterator ml = MombySimpleMatch.lower_bound(&*iter);
	MatchPeakAtom::iterator mu = MombySimpleMatch.upper_bound(&*iter);

	MatchPeakAtom::iterator dl = DadbySimpleMatch.lower_bound(&*iter);
	MatchPeakAtom::iterator du = DadbySimpleMatch.upper_bound(&*iter);

	
	for(MatchPeakAtom::iterator mi = ml; mi != mu; ++mi)
	for(MatchPeakAtom::iterator di = dl; di != du; ++di)
	{
	    {					
	      atom* a = mi->second->get_repAtom();
	      atom* b = di->second->get_repAtom();

	      atset.insert(make_pair(a,b));
	      atpair.insert(make_pair(make_pair(a,b), make_pair(mi->second, di->second)));
	    } 
	}
	

	//DEL_SAM
set<MomDad> delsetSAM; delsetSAM.clear();
	for(PeakAssignReason::iterator r = BufferReason.lower_bound(&*iter); r != BufferReason.upper_bound(&*iter); ++r)
	{
		if (r->second.second.find(DEL_SAM) != r->second.second.end())
		{
			delsetSAM.insert(r->second.first);
	delset.insert(r->second.first);
		}
	}

	//DEL_DEG
for(set<MomDad, less<MomDad> >::iterator as = atset.begin(); as != atset.end(); ++as)
{
if (atpair.count(*as) > 1)
{
// if not stereo-specificly assigned
set<atom*> MomSet; MomSet.clear();
set<atom*> DadSet; DadSet.clear();
for(multimap<MomDad, MomDad, less<MomDad> >::iterator ap = atpair.lower_bound(*as); ap != atpair.upper_bound(*as); ++ap)
			{
				MomSet.insert(ap->second.first);
				DadSet.insert(ap->second.second);
			}

			bool Mstereo = 0, Dstereo = 0;
			if ((MomSet.size() > 1))
			{
				for(set<atom*>::iterator i = MomSet.begin(); i != MomSet.end(); ++i)
				{
					if ((*i)->get_ambi() == 1)
					{
						Mstereo = 1;
						break;
					}
				}
			}
			if ((DadSet.size() > 1))
			{
				for(set<atom*>::iterator i = DadSet.begin(); i != DadSet.end(); ++i)
				{
					if ((*i)->get_ambi() == 1)
					{
						Dstereo = 1;
						break;
					}
				}
			}

			if ((((MomSet.size() > 1) && (Mstereo)) || (MomSet.size() == 1))
					 && (((DadSet.size() > 1) && (Dstereo)) || (DadSet.size() == 1)))
				continue;

			// find the best one
			float min=10000;
			MomDad* mdmin =0;
			for(multimap<MomDad, MomDad, less<MomDad> >::iterator ap = atpair.lower_bound(*as); ap != atpair.upper_bound(*as); ++ap)
			  {
			    float dist;
			    dist = pow(ap->second.first->get_Hdiff(iter->get_HX1(), SWSF_HX1),1.2) +
			      pow(ap->second.second->get_Hdiff(iter->get_HX2(), SWSF_HX2), 1.2)+
			      pow(ap->second.first->get_Xdiff(iter->get_X1(), SWSF_X1)/10.0, 1.25) +
			      pow(ap->second.second->get_Xdiff(iter->get_X2(), SWSF_X2)/10.0, 1.25);
			    if (dist < min)
			      {
				min = dist;
				mdmin = &(ap->second);
			      }
			  }

			// keep the best one
			if (mdmin)
			  {
			    //if (DEBUG)
			    //	NOESY_Assign::outputLog() << "min : " << min << " " << mdmin->first << " " << mdmin->second << endl;

			    for(multimap<MomDad, MomDad, less<MomDad> >::iterator ap = atpair.lower_bound(*as); ap != atpair.upper_bound(*as); ++ap)
			      {
				if (ap->second != *mdmin) 
				  {
				    MakeBuffer(&*iter, ap->second.first, ap->second.second, DEL_DEG);
				  }
			      }
			  }
}
}

		for(set<MomDad>::iterator di = delset.begin(); di != delset.end(); ++di)
		{       
             	// if have some assignment, unassign the one with large shifts
              if (AssignList.find(&*iter) != AssignList.end())
              {
              	PeakAssignReason::iterator jl = AssignList.lower_bound(&*iter);
              	PeakAssignReason::iterator ju = AssignList.upper_bound(&*iter);

               	PeakAssignReason::iterator j;
                for(j = jl; j != ju; ++j)
                {
                	if ((j->second.first.first == di->first) && (j->second.first.second == di->second))
                 		break;
                }

                if (j == ju)
                	continue;

                bool sam = 0;
                if (delsetSAM.find(*di) != delsetSAM.end())
                {
                	sam = 1;
                }

                if (!sam) // report in unassign file
                { 
              			if (!UA)
              			{
              				uaStream << endl;
              				uaStream << "==================" << endl;
              				uaStream << (itoa(NOESY_Assign::ddcycle())+"-"+itoa(NOESY_Assign::dcycle())).c_str() << " " << label() << endl;
              			}

                 	// not supported by pdb, or assigned by more than one reasons
                  if ((j->second.second.size() != 1) || (j->second.second.find(PDB) == j->second.second.end()))
                  {
                    uaStream << (*iter) << endl << di->first << endl
              		      << di->second;
                 		for(setMethod::iterator im = j->second.second.begin(); im != j->second.second.end(); im++)
                 			uaStream << (*im) << " ";
              			uaStream << endl;
            				uaStream << "Not supported by local shift reference " << endl;
              		  uaStream << endl;
  								}
								}
								
        	      // delete the assignment
        	      unPeak.push_back(*iter);
        	      // unassingn constraints and assignments

               	if (sam)
		{
			UnAssign(j->first, j->second.first.first, j->second.first.second, DEL_SAM, IA, IAI);   
			
		}             
		else
			UnAssign(j->first, j->second.first.first, j->second.first.second, DEL_LOC, IA, IAI);
        	UA++;
              }
    } // end for
 	}

  // if no violation, return
  if (!UA)
    return UA;

  for(PeakList::iterator pi = unPeak.begin(); pi != unPeak.end(); pi++)
  {
    if (AssignList.find(&(*pi)) == AssignList.end())
  	{
  	  NV++;
  	}
  }

  int un = 0;
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
  {
    if (AssignList.find(&(*iter)) == AssignList.end())
		un++;
  }

  outStream.setf(ios::left, ios::adjustfield);
  outStream.width(6);
  outStream << (itoa(NOESY_Assign::ddcycle())+"-"+itoa(NOESY_Assign::dcycle())).c_str();
  outStream.width(7);
  outStream << "DLOC";
  outStream.width(6);
  outStream << "-";
  outStream.width(8);
  outStream << peakList.size() - un; //???
  outStream.width(7);
  outStream << "-";
  outStream.width(9);
  outStream << AssignList.size();
  outStream.width(19);
  outStream << UA;
  outStream.width(18);
  outStream << un;
  outStream.width(11);
  outStream << noiseList.size();

  if (NOESY_Assign::COMPARE())
  {
    outStream.width(8);
      outStream << "-";
    outStream.width(11);
    if (NOESY_Assign::COMPARE())
      outStream << -1 * (IA - IAI);
    else
      outStream << "-";
    outStream.width(11);
    if (NOESY_Assign::COMPARE())
      outStream << -1 * IAI;
    else
      outStream << "-";
  }
  outStream << "\n" << NOESY_Assign::EOS();
  string temp(outStream.str());
  reportProcessStat += temp.substr(0,temp.find(NOESY_Assign::EOS()));

  return UA;
}
