/**************************************************************************
                          experiment_Pdb.cpp  -  All metheds related with Pdb
                             -------------------
    begin                : Mon Aug 21 2000
    copyright            : (C) 2000 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#include <iostream>
#include <sstream>

#include <cmath>
#include "experiment.h"
#include "noesy_assign.h"
#include "faces.h"

#define pow(x,y) pow((double)x, (double)y)
#define abs(x) abs((double)(x))

extern Command command;

string itoa(int);
extern bool DEBUG;
extern Faces contactFaces;

void find_looseContraint(const atom* , const atom* , const DianaMap&, const DianaConstraint*& );
SampleStatistic getDistStat(deque<Pdb>& , pair<const atom*, const atom*> , bool );
SampleStatistic getDistSumAveStat(deque<Pdb>& , pair<const atom*, const atom*> , pair<const string, const string>, bool );
float sumAverage(const SampleStatistic&);

bool sameHydro(char, char);
extern int faces[1000][1000];
bool tail(const atom* const);

void NOESY::resetRmax() const
{
	Rmax = 0;
}
void NOESY::resetRmaxQ() const
{
	RmaxQ = 0;
}

void NOESY::setRmaxQ(const float r) const
{
  RmaxQ = r; 
}
void NOESY::setRmax(const float r) const
{
  Rmax = r; 
}

float NOESY::get_Rmax() const
{
	return Rmax; 
	//return 5.0;
}
float NOESY::get_RmaxQ() const
{
	return RmaxQ;
}

int get_deg(const atom* a, const string& name) 
{
	// if methyl group
	if (((a->get_sym_1() == 'A') && (name == "QB")) || // methyl group
				((a->get_sym_1() == 'I') && ((name == "QG2") || (name == "QD1"))) ||
				((a->get_sym_1() == 'T') && (name == "QG2")) ||
				((a->get_sym_1() == 'M') && (name == "QE")) ||
				((a->get_sym_1() == 'L') && (name.substr(0,2) == "QD")) ||
				((a->get_sym_1() == 'V') && (name.substr(0,2) == "QG")))
			return 3;
	if (((a->get_sym_1() == 'L') && (name.substr(0,2) == "QQ")) ||
			((a->get_sym_1() == 'V') && (name.substr(0,2) == "QQ")))
			return 6;

	// if methelne group
	if (name.substr(0,1) == "Q")
		return 2;

	if (name.substr(0,1) == "C")
		return 2;
		
	return 1;
}

int NOESY::strongValidate(const Vio& parVio, ostream& uaStream, bool center)
{
  ostringstream outStream;

  if (DEBUG) NOESY_Assign::outputLog() << "Structure Validating" << endl;
  int NV = 0, IAI = 0, IA = 0, UA = 0; // CA =  0;

  PeakList unPeak; unPeak.clear();
  
  int Nmeaningless = 0; 
  
	for(PeakList::iterator piter = peakList.begin(); piter != peakList.end(); piter++)
	{
 		PeakAssignReason delAssign; delAssign.clear();
 		
		for(PeakAssignReason::iterator iter = AssignList.lower_bound(&*piter); iter != AssignList.upper_bound(&*piter); iter++)
  		{
  
	if (DEBUG) NOESY_Assign::outputLog() << "in VIO" << endl;
      	if (DEBUG) NOESY_Assign::outputLog() << (*iter).first << endl;
	if (DEBUG) NOESY_Assign::outputLog() << (*iter).second.first.first << endl;
	if (DEBUG) NOESY_Assign::outputLog() << (*iter).second.first.second << endl << flush;
	if (DEBUG) NOESY_Assign::outputLog() << "after VIO" << endl;
					      
		  // find the constraint distance
        atom* a1 = iter->second.first.first;
        atom* a2 = iter->second.first.second;

	if (a1 == a2) // diagonal
      		continue;
       
        pair<atom*, atom*> a12;
        if (*a1 < *a2)
        	a12 = make_pair(a1, a2);
        else
        	a12 = make_pair(a2, a1);

	const DianaConstraint* temp = 0;
     	find_looseContraint(a12.first, a12.second, *dianaBufferPtr, temp);
		       					  
      // --- find the distance between the atoms
      	if (temp)
    	{
    	  if (DEBUG) NOESY_Assign::outputLog() << temp->name1 << "\t" << temp->name2 << " " << temp->dist << " " << temp->corrections << endl;

	  float dF = 0;
	  float nn = get_deg(a12.first, temp->name1)*get_deg(a12.second, temp->name2);

	  float dConEff = 0; 
	  if (center)
	    dConEff = temp->dist - temp->corrections + dF; // /temp->nn;
	  else
	    dConEff = temp->xplorDist + temp->xub + dF; 
																
    SampleStatistic diff; diff.reset();
    SampleStatistic distStat; distStat.reset();

    int ambi1 = 0;
    int ambi2 = 0;

        
    ambi1 = a12.first->get_ambi();
    ambi2 = a12.second->get_ambi();
    
    distStat = getDistSumAveStat(NOESY_Assign::pdbList(), a12, make_pair(temp->name1, temp->name2), isIC()); // SumAverage
    for(SmplVect::const_iterator i = distStat.oriSamples().begin(); i != distStat.oriSamples().end(); ++i)
      {
						//float ieff = pow(pow(*i, -6.0)/nn, -1/6.0);
						
		        if (DEBUG)
 		        	NOESY_Assign::outputLog() << *i << " " << ambi1 << "  " << ambi2 << endl;

	diff += *i - dConEff;
     }
      
    // find the first one that violated
    int indexZero=0;
    int indexStrong=0;
    for(SmplData::iterator i = diff.samples().begin(); i != diff.samples().end(); ++i)
      {
	if (*i < 0.101)
	  {
	    ++indexZero;
	    ++indexStrong;
	  }
	else if (*i < 0.501)
	  {
	    ++indexStrong;
	  }
	else
	  break;
      }
    
    if (DEBUG)
      {
	NOESY_Assign::outputLog() << "dConEff=" << dConEff << " diffDist: min " << diff.min() << " mean " << diff.mean() << " max " <<
	  diff.max() << " #<=0.1 " << indexZero << " #<=0.5 " << indexStrong << " " << diff.count() << endl;
      }

    SmplData::iterator d2 =  diff.samples().begin();
    if (diff.samples().size()> 1)
      ++d2;
    float diff2 = *d2;

    if (((!indexZero) && (diff.mean() > 0.201))) 
      {
	if (!UA)
	  {
	    uaStream << endl;
	    uaStream << "==================" << endl;
	    uaStream << (itoa(NOESY_Assign::ddcycle())+"-"+itoa(NOESY_Assign::dcycle())).c_str() << " " << label() << endl;
	  }
	
	uaStream << (*iter).first << endl << (*iter).second.first.first << endl
		 << (*iter).second.first.second;
	
	for(setMethod::iterator im = (*iter).second.second.begin();
	    im != (*iter).second.second.end(); im++)
	  uaStream << (*im) << " ";
    			uaStream << endl;
			uaStream << "#vio<=0.1 " << indexZero << " sumHDIST: min " << distStat.min() << " mean " << distStat.mean() << " max " <<
			  distStat.max(); 
			if (center)
			  uaStream << " constraint= " << temp->dist << "-" << temp->corrections << "+" << dF << " effect=" << dConEff << endl;
			else
			  uaStream << " constraint= " << temp->xplorDist << "+" << temp->xub+dF << " effect=" << dConEff << endl;
			
			if ((!indexZero))
			  uaStream << "All violated" << endl;
			uaStream << endl;
			
			// delete the assignment
			unPeak.push_back(*((*iter).first)); 	
			// unassingn constraints and assignments
			if (DEBUG) NOESY_Assign::outputLog() << "before UnAssign" << endl;
			delAssign.insert(*iter);		       
			if (DEBUG) NOESY_Assign::outputLog() << "after UnAssign" << endl;
			
			if ((iter->second.first.first->isMeaningless(iter->second.first.second)) || (iter->second.first.first->isSecLongHNHN(iter->second.first.second)))
			  {
			    Nmeaningless++;
			  }
			
		
	      UA++;
      } else if (diff.max() > 0.01) // && (NOESY_Assign::ddcycle() >=2)) // have some violation in some models -> may be too tight 
      {
	if (DEBUG) NOESY_Assign::outputLog() << "ADJ_CALI " << iter->first << endl << iter->second.first.first << endl 
					       << iter->second.first.second << endl; 
	MakeBuffer(iter->first, iter->second.first.first, iter->second.first.second, ADJ_CALI); 

      }
    
	
	} // if temp
		else
		{
		  if (*((*iter).second.first.first) != *((*iter).second.first.second))
		    {
		      NOESY_Assign::outputError() << "***ERROR Cannot find the psudo atom from ContraintBuffer" << endl;
		      NOESY_Assign::outputError() << (*iter).first << endl << (*iter).second.first.first << endl
						  << (*iter).second.first.second << endl;
		    }
		}
		}
	 
		if (delAssign.size()) 
		  {	
		    for(PeakAssignReason::iterator iter = delAssign.begin(); iter != delAssign.end(); ++iter) 
		      {
	 		UnAssign((*iter).first, (*iter).second.first.first, (*iter).second.first.second, DEL_VIO, IA, IAI);
		      }
		  }
		
	}

  // if no violation, return
  if (!UA)
    return UA;

  for(PeakList::iterator pi = unPeak.begin(); pi != unPeak.end(); pi++)
    {
      if (AssignList.find(&(*pi)) == AssignList.end())
    	{
    	  NV++;
    	}
    }

  int un = 0;
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
    {
      if (AssignList.find(&(*iter)) == AssignList.end())
	un++;
    }

  outStream.setf(ios::left, ios::adjustfield);
  outStream.width(6);
  
  if (UA-Nmeaningless == 0)
  {
  	outStream << (itoa(NOESY_Assign::ddcycle())+"-"+itoa(NOESY_Assign::dcycle()-1)).c_str();
  } else
  	outStream << (itoa(NOESY_Assign::ddcycle())+"-"+itoa(NOESY_Assign::dcycle())).c_str();
	
  outStream.width(7);
  outStream << "VIO";
  outStream.width(6);
  outStream << "-";
  outStream.width(8);
  outStream << peakList.size() - un; //???
  outStream.width(7);
  outStream << "-";
  outStream.width(9);
  outStream << AssignList.size();
  outStream.width(19);
  outStream << UA;
  outStream.width(18);
  outStream << un;
  outStream.width(11);
  outStream << noiseList.size();

  if (NOESY_Assign::COMPARE())
  {
  outStream.width(8);
    outStream << "-";
  outStream.width(11);
  if (NOESY_Assign::COMPARE())
    outStream << -1 * (IA - IAI);
  else
    outStream << "-";
  outStream.width(11);
  if (NOESY_Assign::COMPARE())
    outStream << -1 * IAI;
  else
    outStream << "-";
  }
  outStream << "\n" << NOESY_Assign::EOS();
  string temp(outStream.str());
  reportProcessStat += temp.substr(0,temp.find(NOESY_Assign::EOS()));
  
  return UA-Nmeaningless;
}


int NOESY::weakValidate(const Vio& parVio, ostream& uaStream, const mdsMap& mdsd, const peakmdMap& peakmd)
{
  ostringstream outStream;

  int NV = 0, IAI = 0, IA = 0, UA = 0; // CA =  0;
  bool UA2 = 0;
 
  PeakList unPeak; unPeak.clear();
  set<int> contacts; contacts.clear();
  for(ContactPairSet::iterator i = NOESY_Assign::asContactSet().begin(); i != NOESY_Assign::asContactSet().end(); ++i)
  {
	  if (abs(i->first-i->second) >=5)
	 {
	  	contacts.insert(i->first);
	  	contacts.insert(i->second);
	  }
  }
  for(PeakList::iterator piter = peakList.begin(); piter != peakList.end(); piter++)
	{
		// only unassign if there is another better choice
		PeakAssignReason delAssign; delAssign.clear();
		set<MomDad> delIso; delIso.clear();

	//assigned list 
  	for(PeakAssignReason::iterator aiter = AssignList.lower_bound(&*piter); aiter != AssignList.upper_bound(&*piter); aiter++)
  	{
    
		//assigned by method pdb means it was just assigned at this cycle 
		//assigned by other methods, but not by pdb or EXPECT means it is not the best choice 		
   		if (((aiter->second.second.count(PDB) == 0)) && (aiter->second.second.count(SYM) == 0) && (aiter->second.second.count(SYM2) == 0) && (aiter->second.second.count(SYM4) == 0) && (aiter->second.second.count(EXPECT) == 0))      	
		{	
			delAssign.insert(*aiter);
		} else // assigned by SYM, not by PDB or EXPECT   
		  if (aiter->second.second.count(OLD) != 0)
		    {
		      //aiter->second.second.erase(aiter->second.second.find(OLD));
		      if ((aiter->second.second.count(PDB) == 0) && (aiter->second.second.count(EXPECT) == 0))
		  {
		  
		    //if (haveOne)
		      {
			delAssign.insert(*aiter); 
		      }
		  }
		  		     
		    }
	 // isolated one
		else if (	//(aiter->second.second.count(BEST)==0) && 
		(abs(aiter->second.first.first->get_index()-aiter->second.first.second->get_index()) >= 5))
	{
		bool isolated = 1;
		bool haveContact1 = 0;
		bool haveContact2 = 0;
		for(int k=-5; k<=5; k++)
		for(int m=-5; m<=5; m++)
		{
			if (!k&&!m) continue;
			//if (abs(k)+abs(m) > 5) continue;
			pair<int, int> kmij;
			int kmi = k+aiter->second.first.first->get_index();
			int kmj = m+aiter->second.first.second->get_index();

			if (kmi<kmj) kmij = make_pair(kmi,kmj);
			else kmij = make_pair(kmj,kmi);
			if (NOESY_Assign::asContactSet().find(kmij) != NOESY_Assign::asContactSet().end())
			{
				isolated = 0;
//				if (DEBUG)
//                                       NOESY_Assign::outputLog() << "supported by " << kmij.first << " " << kmij.second << endl;	
				break;
			}	
			if ((contacts.find(kmi) != contacts.end()) && (k))
			{
				haveContact1 = 1;
//				if (DEBUG)
//                                       NOESY_Assign::outputLog() << "supported by " << kmi  << endl;	
			}
			if ((contacts.find(kmj) != contacts.end()) && m)
			{
				haveContact2 = 1;
//				if (DEBUG)
//                                       NOESY_Assign::outputLog() << "supported by "  << kmj << endl;	
			}
		}	
		if ((isolated) && (!haveContact1 || !haveContact2))
		{
			if (DEBUG)
			  NOESY_Assign::outputLog() << "isolated" << endl;
			
			// make a note, not unassigned untill cycle 3 by find_iso method 
			MakeBuffer(aiter->first, aiter->second.first.first, aiter->second.first.second, DEL_ISO);			
		}
	}	
    }
	
  for(PeakAssignReason::iterator i = BufferReason.begin(); i != BufferReason.end(); ++i)
  {
   if (i->second.second.find(OLD) != i->second.second.end())
  	{
  		i->second.second.erase(i->second.second.find(OLD));
  	}
  }

	  set<MomDad> delsetDEG; delsetDEG.clear();
 		for(PeakAssignReason::iterator r = BufferReason.lower_bound(&*piter); r != BufferReason.upper_bound(&*piter); ++r)
 		{
 			if (r->second.second.find(DEL_DEG) != r->second.second.end())
 			{
 				delsetDEG.insert(r->second.first);
 			}
 		}

    if (delAssign.size()) // && (delAssign.size() < AssignList.count(&*piter))) // not unique
    {
		  for(PeakAssignReason::iterator iter = delAssign.begin(); iter != delAssign.end(); ++iter) 
  		{
				bool print=0;
				
    		if (iter->second.second.size())
      	{
       		print = 1;
       	}

		PeakAssignReason::iterator rl = BufferReason.lower_bound(iter->first);
		PeakAssignReason::iterator ru = BufferReason.upper_bound(iter->first);
		PeakAssignReason::iterator ri;
 
		
		for(ri = rl; ri != ru; ri++)
		  {
		    if (((*ri).second.first.first == iter->second.first.first) &&
			((*ri).second.first.second == iter->second.first.second))
		      {
         		if ((ri->second.second.count(BEST) != 0))
			  print = 1;
			break;
		      }
		  }

        // if label as DEL_DEG, do not report
				bool deg = 0;
        if (delsetDEG.find(iter->second.first) != delsetDEG.end())
        {
        	deg = 1;
        }
        
    		if ((!deg) && print)
      	{
    			if (!UA2)
    			{
    				uaStream << endl;
    				uaStream << "==================" << endl;
    				uaStream << (itoa(NOESY_Assign::ddcycle())+"-"+itoa(NOESY_Assign::dcycle())).c_str() << " " << label() << endl;
            UA2 = 1;
    			}

          uaStream << (*iter).first << endl << (*iter).second.first.first << endl
    		      << (*iter).second.first.second;
       		for(setMethod::iterator im = (*iter).second.second.begin(); im != (*iter).second.second.end(); im++)
		  {
		    if (*im != OLD)
       			uaStream << (*im) << " ";
		  }
    			uaStream << endl;

			    if (delIso.find(iter->second.first) != delIso.end())
				    uaStream << "Isolated" << endl;
			    else	
				    uaStream << "Not best supported by PDB " << endl;
    		  uaStream << endl;
         //UA2 = 1;
       }
	      // delete the assignment
	      unPeak.push_back(*((*iter).first));
	      // unassingn constraints and assignments

	      if (deg)
	      	UnAssign((*iter).first, (*iter).second.first.first,(*iter).second.first.second, DEL_DEG, IA, IAI);
	      else if (delIso.find(iter->second.first) != delIso.end())
		UnAssign((*iter).first, (*iter).second.first.first, iter->second.first.second, DEL_ISO, IA, IAI);
	      else 
	      	UnAssign((*iter).first, (*iter).second.first.first,(*iter).second.first.second, DEL_PDB, IA, IAI);
	       
	      UA++;
	    } // end for iter
	  } // end if
  }

  for(PeakAssignReason::iterator i = AssignList.begin(); i != AssignList.end(); ++i)
  {
   if (i->second.second.find(OLD) != i->second.second.end())
  	{
  		i->second.second.erase(i->second.second.find(OLD));
  	}
  }

  // if no violation, return
  if (!UA)
    return UA;

  for(PeakList::iterator pi = unPeak.begin(); pi != unPeak.end(); pi++)
  {
    if (AssignList.find(&(*pi)) == AssignList.end())
  	{
  	  NV++;
  	}
  }

  int un = 0;
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
  {
    if (AssignList.find(&(*iter)) == AssignList.end())
		un++;
  }

  outStream.setf(ios::left, ios::adjustfield);
  outStream.width(6);
  outStream << (itoa(NOESY_Assign::ddcycle())+"-"+itoa(NOESY_Assign::dcycle())).c_str();
  outStream.width(7);
  outStream << "DPDB";
  outStream.width(6);
  outStream << "-";
  outStream.width(8);
  outStream << peakList.size() - un; //???
  outStream.width(7);
  outStream << "-";
  outStream.width(9);
  outStream << AssignList.size();
  outStream.width(19);
  outStream << UA;
  outStream.width(18);
  outStream << un;
  outStream.width(11);
  outStream << noiseList.size();

  if (NOESY_Assign::COMPARE())
  {
    outStream.width(8);
      outStream << "-";
    outStream.width(11);
    if (NOESY_Assign::COMPARE())
      outStream << -1 * (IA - IAI);
    else
      outStream << "-";
    outStream.width(11);
    if (NOESY_Assign::COMPARE())
      outStream << -1 * IAI;
    else
      outStream << "-";
  }
  outStream << "\n" << NOESY_Assign::EOS();
  string temp(outStream.str());
  reportProcessStat += temp.substr(0,temp.find(NOESY_Assign::EOS()));

  return UA;
}

// after cycle 3, removide isolated contacts 
int NOESY::findISO(const Vio& parVio, ostream& uaStream)
{
  ostringstream outStream;

  //if (DEBUG) NOESY_Assign::outputLog() << "Structure Validating" << endl;
  int NV = 0, IAI = 0, IA = 0, UA = 0; // CA =  0;

  PeakList unPeak; unPeak.erase(unPeak.begin(), unPeak.end());
  set<int> contacts; contacts.clear();
  for(ContactPairSet::iterator i = NOESY_Assign::asContactSet().begin(); i != NOESY_Assign::asContactSet().end(); ++i)
  {
	  if (abs(i->first-i->second) >5)
	 {
	  	contacts.insert(i->first);
	  	contacts.insert(i->second);
		}
  }
  
  	for(PeakAssignReason::iterator aiter = AssignList.begin(); aiter != AssignList.end(); aiter++)
  	{
   		PeakAssignReason::iterator br; //br = 0;
     	bool in = 0;
      for(br = BufferReason.lower_bound(aiter->first); br != BufferReason.upper_bound(aiter->first); ++br)
      {
      	if (br->second.first == aiter->second.first)
       	{
        	in = 1;
       		break;
        }
      }
      
			if (in && (br->second.second.count(DEL_ISO)==1))
    	{
    		if (DEBUG)
    			NOESY_Assign::outputLog() << "isolated from last cycle " << aiter->first << endl;
			
		//check if it is still isolated now 
    		bool isolated = 1;
    		bool haveContact1 = 0;
    		bool haveContact2 = 0;
    		for(int k=-5; k<=5; k++)
    		for(int m=-5; m<=5; m++)
    		{
    			if (!k&&!m) continue;
    			//if (abs(k)+abs(m) > 5) continue;
    			pair<int, int> kmij;
    			int kmi = k+aiter->second.first.first->get_index();
    			int kmj = m+aiter->second.first.second->get_index();
       		if (kmi<kmj) kmij = make_pair(kmi,kmj);
    			else kmij = make_pair(kmj,kmi);
    			if (NOESY_Assign::asContactSet().find(kmij) != NOESY_Assign::asContactSet().end())
    			{
    				isolated = 0;
    //				if (DEBUG)
    //                                       NOESY_Assign::outputLog() << "supported by " << kmij.first << " " << kmij.second << endl;
    				break;
    			}
    			if ((contacts.find(kmi) != contacts.end()) && (k))
    			{
    				haveContact1 = 1;
    //				if (DEBUG)
    //                                       NOESY_Assign::outputLog() << "supported by " << kmi  << endl;
    			}
    			if ((contacts.find(kmj) != contacts.end()) && m)
    			{
    				haveContact2 = 1;
    //				if (DEBUG)
    //                                       NOESY_Assign::outputLog() << "supported by "  << kmj << endl;
    			}
    		} // end for
    		if ((isolated) && (!haveContact1 || !haveContact2))
    		{
    			if (DEBUG)
    			  NOESY_Assign::outputLog() << "isolated" << endl;

	bool print=0;

    	if (aiter->second.second.size())
      	{
       		print = 1;
       	}

    	if (print)
      	{
    			if (!UA)
    			{
    				uaStream << endl;
    				uaStream << "==================" << endl;
    				uaStream << (itoa(NOESY_Assign::ddcycle())+"-"+itoa(NOESY_Assign::dcycle())).c_str() << " " << label() << endl;
    			}

          uaStream << (*aiter).first << endl << (*aiter).second.first.first << endl
    		      << (*aiter).second.first.second;
       		for(setMethod::iterator im = (*aiter).second.second.begin(); im != (*aiter).second.second.end(); im++)
       			uaStream << (*im) << " ";
    			uaStream << endl;

					uaStream << "Isolated by PDB " << endl;
    		  uaStream << endl;
      	}
	      // delete the assignment
	      unPeak.push_back(*((*aiter).first));
	      // unassingn constraints and assignments
	      UnAssign((*aiter).first, (*aiter).second.first.first, aiter->second.first.second, DEL_ISO, IA, IAI);
	      UA++;
	    } // end for iter
	  } // end if
  }

  // if no violation, return
  if (!UA)
    return UA;

  for(PeakList::iterator pi = unPeak.begin(); pi != unPeak.end(); pi++)
  {
    if (AssignList.find(&(*pi)) == AssignList.end())
  	{
  	  NV++;
  	}
  }

  int un = 0;
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
  {
    if (AssignList.find(&(*iter)) == AssignList.end())
		un++;
  }

  outStream.setf(ios::left, ios::adjustfield);
  outStream.width(6);
  outStream << (itoa(NOESY_Assign::ddcycle())+"-"+itoa(NOESY_Assign::dcycle())).c_str();
  outStream.width(7);
  outStream << "DISO";
  outStream.width(6);
  outStream << "-";
  outStream.width(8);
  outStream << peakList.size() - un; //???
  outStream.width(7);
  outStream << "-";
  outStream.width(9);
  outStream << AssignList.size();
  outStream.width(19);
  outStream << UA;
  outStream.width(18);
  outStream << un;
  outStream.width(11);
  outStream << noiseList.size();

  if (NOESY_Assign::COMPARE())
  {
    outStream.width(8);
    outStream << "-";
    outStream.width(11);
    if (NOESY_Assign::COMPARE())
      outStream << -1 * (IA - IAI);
    else
      outStream << "-";
    outStream.width(11);
    if (NOESY_Assign::COMPARE())
      outStream << -1 * IAI;
    else
      outStream << "-";
  }
  outStream << "\n" << NOESY_Assign::EOS();
  string temp(outStream.str());
  reportProcessStat += temp.substr(0,temp.find(NOESY_Assign::EOS()));

  return UA;
}

typedef map<const MomDad, int, less<const MomDad> > atommap;
typedef pair<const SampleStatistic, const MomDad> StatAtomAtom;


typedef pair<int, int> respair;
typedef multimap<const respair, const general_peak* const, less<const respair> > resPeak;

typedef deque<float> floatdeque;

typedef multimap<float, const StatAtomAtom, less<float> > disOrderSet;
typedef pair<int, int> indexindex;

typedef map<const indexindex, const MomDadSet, less<const indexindex> > testAssignBin;

typedef set<indexindex, less<indexindex> > indexindexSet;


void NOESY::AddPDBProfile(const MomDadPeakAtomMap& mdPAMapM2, SUPPORT& support, mdsMap& mdsd, peakmdMap& peakmd)
{
  if (DEBUG) NOESY_Assign::outputLog() << "\n-> In AddPDBProfile" << endl;
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
  {
		mdsMap mdsdsub; mdsdsub.clear();

    if (DEBUG) NOESY_Assign::outputLog() << (*iter) << endl;

    //--- possible mother
    MatchPeakAtom::iterator momlower = MombySimpleMatch.lower_bound(&(*iter));
    MatchPeakAtom::iterator momupper = MombySimpleMatch.upper_bound(&(*iter));

    //--- possible father
    MatchPeakAtom::iterator fatherLower = DadbySimpleMatch.lower_bound(&(*iter));
    MatchPeakAtom::iterator fatherUpper = DadbySimpleMatch.upper_bound(&(*iter));

    //--- all possible assignemnts and distances for one peak
    for(MatchPeakAtom::iterator iterm = momlower; iterm != momupper; iterm++)
		{
	  	for(MatchPeakAtom::iterator iterf = fatherLower; iterf != fatherUpper; iterf++)
	    {
	    	bool del = 0;
	      PeakAssignReason::iterator rl = BufferReason.lower_bound(&(*iter));
	      PeakAssignReason::iterator ru = BufferReason.upper_bound(&(*iter));
	      PeakAssignReason::iterator ri;
	      for(ri = rl; ri != ru; ri++)
    		{
    		  if (((*ri).second.first.first == (*iterm).second) &&
    		      ((*ri).second.first.second == (*iterf).second))
          {
            	if (
									 (ri->second.second.count(DEL_DEG) != 0) ||
									 (ri->second.second.count(DEL_INT) != 0) ||
									 (ri->second.second.count(DEL_LSS) != 0) ||
									 ((ri->second.second.count(DEL_LOC) != 0)  
									  	  && (ri->second.second.count(DEL_D2O) == 0) 
										  && (ri->second.second.count(DEL_SAM) == 0)))
									 del = 1;
             	break;
					}
    		}

	      if (!del)
				{
    				pair<const atom*, const atom*> ab;
      		  if (iterm->second < iterf->second)
    					ab = make_pair(iterm->second, iterf->second);
    				else
    					ab = make_pair(iterf->second, iterm->second);

         		// r-6 summation average
    				SampleStatistic disStat = getDistStat(NOESY_Assign::pdbList(), ab, isIC());
        		if (disStat.oriSamples().size() == NOESY_Assign::pdbList().size())
          	{
			
			if ((ri != BufferReason.end()) && (ri->second.second.count(DEL_D2O) != 0) && (disStat.min() > 5.0)) //||
				continue;
			
					    mdsdsub.insert(make_pair(make_pair(iterm->second, iterf->second), disStat));
							mdsd.insert(make_pair(make_pair(iterm->second, iterf->second), disStat)); // keep the mom-dad order
						}
        }
	    } // end for
		} // end for

		if (!mdsdsub.size()) // no candidate
			continue;

		// for each structure, find the top one, with the contributions
		for(int i = 0; i < NOESY_Assign::pdbList().size(); ++i)
		{
			float dsum = 0; // r-6 summation
			multimap<float, MomDad> dmd; dmd.clear();
			for(map<MomDad, SampleStatistic>::iterator j = mdsdsub.begin(); j != mdsdsub.end(); ++j)
			{
				//mdsd.insert(*j);
			  dmd.insert(make_pair(j->second.oriSamples()[i], j->first));
			  	if (j->second.oriSamples()[i] < 2.5) // was 3.0
					dsum += pow(2.5, -4.0);
				else dsum += pow(j->second.oriSamples()[i], -4.0);
			}

			// select the top 10 and recalculate 
			if (dmd.size() > 6)
			{
				int c=0; dsum = 0;
				multimap<float, MomDad>::iterator di; 
				for(di= dmd.begin(); di != dmd.end(); ++di)
				{
					++c;
			  	if (di->first < 2.5)  // was 3.0
					dsum += pow(2.5, -4.0);
				else	dsum += pow(di->first, -4.0);	
					if (c > 6)
						break;
				}
				if (di != dmd.end())
					dmd.erase(di, dmd.end());
			}
			
  		if (DEBUG)
  		{
  			NOESY_Assign::outputLog() << "For structure " << i << " dsum = " << dsum << endl;
  		}

	float dctrsum = 0; // total contributionn to the r-6 summation
	multimap<float, MomDad>::iterator di; di = dmd.begin();
	int c = 0, cid = 0;

	//remember the largest di that have 'yes'	
        PeakAssignReason::iterator al = AssignList.lower_bound(&(*iter));
        PeakAssignReason::iterator au = AssignList.upper_bound(&(*iter));
        PeakAssignReason::iterator ai;

	int dmax = -1, dmaxid = 0;

	float dctrsumT = 0;
	for(multimap<float, MomDad>::iterator dmi = dmd.begin(); dmi != dmd.end(); ++dmi)
	{
			  	if (dmi->first < 2.5) // was 3.0
					dctrsumT += floor(pow(2.5, -4.0)*100/dsum + 0.51)/100.0;
				else
					dctrsumT += floor(pow(dmi->first, -4.0)*100/dsum + 0.51)/100.0;
					if (dctrsumT >= 0.75)
					break;
	}			
	dmax = -1;
	
	float dctrpre = 0;
	set<pair<int, int> > inpeakmdset; 
	inpeakmdset.clear(); 

	do
	{
        
	PeakAssignReason::iterator rl = BufferReason.lower_bound(&(*iter));
        PeakAssignReason::iterator ru = BufferReason.upper_bound(&(*iter));
        PeakAssignReason::iterator ri;
        bool vio = 0;
        for(ri = rl; ri != ru; ri++)
    		{
    		  if (((*ri).second.first.first == di->second.first) &&
    		      ((*ri).second.first.second == di->second.second))
           {
            	if ( //((ri->second.second.count(DEL_PDB) != 0) && (ri->second.second.count(PDB) != 0))
	       			    //((ri->second.second.count(DEL_VIO) != 0) && (ri->second.second.count(SYM) != 0)))
				    ((ri->second.second.count(DEL_LOC) != 0) && (ri->second.second.count(DEL_SAM) != 0)) ||
				    ((ri->second.second.count(DEL_LOC) != 0) && (ri->second.second.count(DEL_D2O) != 0)) ||
				    ((ri->second.second.count(DEL_D2O) != 0) && (di->first > 5.0))) //||
	       			
             		vio = 1;
             	break;
           }
    		}

        bool old = 0;
	float dctrtop = 0;
	if (di->first < 2.5) // was 3.0
		dctrtop = floor(pow(2.5, -4.0)*100/dsum + 0.51)/100.0;
	else dctrtop = floor(pow(di->first, -4.0)*100/dsum + 0.51)/100.0;
        dctrsum += dctrtop;

  	if (DEBUG)
  	{
  		NOESY_Assign::outputLog() << di->second.first << endl << di->second.second << " " << di->first << " " << dctrtop << " !vio " << !vio  << " old " << dmax << " pre=" << dctrpre << endl;
  	}

      	if (!vio)
     	{
		if ((di->first < 2.51) || ((c <=2) && (dctrtop > 0.10))) // maximum top three 
	  		{
	        		if (DEBUG)
  	        			NOESY_Assign::outputLog() << "Yes " << endl;
				peakmd.insert(make_pair(&*iter, di->second));
	  		}
	    		++c;
        }
        ++di;        
				++cid;
				dctrpre = dctrtop;
 
      } while ((dctrsum < 0.75) && (di != dmd.end())); 
		}
	}

	// rule in
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
  {
		// get the frq
   	map<MomDad, int> mds; mds.clear();
		for(multimap<const general_peak*, MomDad>::const_iterator i = peakmd.lower_bound(&*iter);
					i != peakmd.upper_bound(&*iter); ++i)
		{
			++mds[i->second];
		}
		set<MomDad> goodmds; goodmds.clear();
		for(map<MomDad, int>::iterator imds = mds.begin(); imds != mds.end(); ++imds)
		{
			// the one that selected by most of the structures are assignments -
			// means this part of the structure is converging
			map<MomDad, SampleStatistic>::const_iterator j = mdsd.find(imds->first);

			if (j == mdsd.end())
				continue;
			// not backbone	
			if (((j->first.first->get_layer() == "N") || (j->first.first->get_layer() == "A")) &&
           				((j->first.second->get_layer() == "N") || (j->first.second->get_layer() == "A")))
    	{
    		continue;
    	}
				
  		int frq = imds->second;

 			if (DEBUG)
 			{
 				NOESY_Assign::outputLog() << "FRQ: " << frq << *iter << endl
 						<< j->first.first << endl << j->first.second << endl;
 			}

	// put into the support data 
			if (((frq/(float)NOESY_Assign::pdbList().size() > 0.69))) // && (mds.size() == 1))) 
     	{
						// add in the profile
						int pi = imds->first.first->get_index();
						int pj = imds->first.second->get_index();
						pair<int, int> ij;
						if (pi<pj) ij = make_pair(pi,pj);
						else ij = make_pair(pj,pi);

						atom* a = imds->first.first->get_repAtom();
			 			atom* b = imds->first.second->get_repAtom();
						pair<atom*, atom*> atomab;
						if (a < b)
							atomab = make_pair(a,b);
						else
							atomab = make_pair(b,a);

						// adding support
						MomDadPeakAtomMap::iterator pmdi;
						MomDadPeakAtomMap::iterator pmdl;
						MomDadPeakAtomMap::iterator pmdu;
						if (support.MomDadBin.find(ij) == support.MomDadBin.end())
						{
							pmdl = support.mdPAMapM.end();
							pmdu = pmdl;
						}
						else
						{
							pmdl = support.mdPAMapM.lower_bound(support.MomDadBin.find(ij)->second);
							pmdu = support.mdPAMapM.upper_bound(support.MomDadBin.find(ij)->second);
						}

						for(pmdi = pmdl; pmdi != pmdu; ++pmdi)
						{
							if ((pmdi->second.first == &*iter) && (pmdi->second.second == imds->first))
								break;
						}
						if (pmdi == pmdu) // no such peak-momdad entry in the list
						{
							if (pmdl == pmdu) // such momdad entry
							{
								support.ContactBin.insert(ij);
								support.MomDadBin.insert(make_pair(ij, atomab));
								support.mdSetM.insert(atomab);
							}
							support.mdPAMapM.insert(make_pair(support.MomDadBin.find(ij)->second, make_pair(&*iter, imds->first)));
						}
			    	if (DEBUG)
  						NOESY_Assign::outputLog() << "AddPDBProfile Support IN MomDadBin " << endl;
      }
		}
	}	
}

int NOESY::AssignGood_Pdb(const PdbP& parPdb, const MomDadPeakAtomMap& mdPAMapM2, const SUPPORT& support, const mdsMap& mdsd, const peakmdMap& peakmd)
{
  ostringstream outStream;
  if (DEBUG) NOESY_Assign::outputLog() << "\n-> In AssignGood_Pdb" << endl;

  int old = AssignList.size();
  int unOld = 0;
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
  {
    if (AssignList.find(&(*iter)) == AssignList.end())
			unOld++;
  }

  //reset PDBU, PDBM
  for(PeakAssignReason::iterator i = AssignList.begin(); i != AssignList.end(); ++i)
  {
   if (i->second.second.find(PDB) != i->second.second.end())
  	{
  		i->second.second.erase(i->second.second.find(PDB));
  	}
  }
  for(PeakAssignReason::iterator i = BufferReason.begin(); i != BufferReason.end(); ++i)
  {
   if (i->second.second.find(DEL_SAM) != i->second.second.end())
  	{
  		i->second.second.erase(i->second.second.find(DEL_SAM));
  	}
   if (i->second.second.find(DEL_ISO) != i->second.second.end())
  	{
  		i->second.second.erase(i->second.second.find(DEL_ISO));
  	}
  }

  int wN = 0; int iN = 0;
  //int NV = 0, CA = 0, IAI = 0, IA = 0, UA = 0;
  PeakList unPeak; unPeak.erase(unPeak.begin(), unPeak.end());

  //--- 2. find these < cutoff, good and unique one and assign.

  MomDadPeakMap assignMap; assignMap.clear();
  MomDadPeakMap assignMapM; assignMapM.clear();

	// rule in
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
  {
		// get the frq
    map<MomDad, int> mds; mds.clear();
		for(multimap<const general_peak*, MomDad>::const_iterator i = peakmd.lower_bound(&*iter);
					i != peakmd.upper_bound(&*iter); ++i)
		{
			++mds[i->second]; 
		}

		// calculate dsum using median distance 
		double dsum = 0; 
		for(map<MomDad, int>::iterator imds = mds.begin(); imds != mds.end(); ++imds)
		{
			// the one that selected by most of the structures are assignments -
		  map<MomDad, SampleStatistic>::const_iterator j = mdsd.find(imds->first); 
		 
		  if ((j != mdsd.end())) 
		    dsum += pow(j->second.median(), -4.0); 
		}


		for(map<MomDad, int>::iterator imds = mds.begin(); imds != mds.end(); ++imds)
		{
			// the one that selected by most of the structures are assignments -
			// means this part of the structure is converging
			map<MomDad, SampleStatistic>::const_iterator j = mdsd.find(imds->first);

			if (j == mdsd.end())
				continue;
				
			int frq = imds->second;

 			if (DEBUG)
 			{
 				NOESY_Assign::outputLog() << "FRQ: " << frq << *iter << endl
 						<< j->first.first << endl << j->first.second << endl;
 			}
		
		PeakAssignReason::iterator al = AssignList.lower_bound(&(*iter));
		PeakAssignReason::iterator au = AssignList.upper_bound(&(*iter));
		
		PeakAssignReason::iterator ai;
		bool old = 0;
		for(ai = al; ai != au; ai++)
		  {
		    if (((*ai).second.first.first == j->first.first) &&
			((*ai).second.first.second == j->first.second))
		      {
 			  old = 1;
			break;
		      }
		  }
		
		double dctr = pow(j->second.median(), -4.0); 
		
		if (DEBUG)
		  NOESY_Assign::outputLog() << "dctr/dsum= " << dctr/dsum << " j->second.max=" << j->second.max() << " get_Rmax() " << get_Rmax() << endl;
			
      	
    	if ((old) && ((frq/(float)NOESY_Assign::pdbList().size() > 0.69)) && (j->second.median() < get_Rmax())) // assigned before
	  // no violation, but may be > 5.0 (due to ambigurity) - remove 
     	{
      		if (DEBUG) NOESY_Assign::outputLog() << "old " << endl;
		assignMap.insert(make_pair(j->first, (&(*iter))));
      	}
	else if ((dctr/dsum > 0.45) && (j->second.median() < get_Rmax()) && (frq/(float)NOESY_Assign::pdbList().size() > 0.69)) 
  	{
	    if (((iter->get_D() < 3) || ((j->first.first->get_Hdiff(iter->get_HX1(), SWSF_HX1) < 0.03 + 0.000001) &&
		(j->first.first->get_Xdiff(iter->get_X1(), SWSF_X1) < 0.3 + 0.000001))) &&
		((iter->get_D() != 4) || ((j->first.second->get_Hdiff(iter->get_HX2(), SWSF_HX2) < 0.03 + 0.000001) &&
		 (j->first.second->get_Xdiff(iter->get_X2(), SWSF_X2) < 0.3 + 0.000001))))
	      {
		if (DEBUG) NOESY_Assign::outputLog() << " top mean < " << j->second.median() << " " << get_Rmax() << endl;
      		assignMap.insert(make_pair(j->first, (&(*iter))));
	      }
  	}
		  }
	}

  old = AssignList.size();
  unOld = 0;
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
  {
    if (AssignList.find(&(*iter)) == AssignList.end())
			unOld++;
  }

  //MakeAssignMap(assignMap, PDB, wN, iN);
  PeakSet aList; aList.clear();
  PeakSet nList; nList.clear(); 
	if (DEBUG)
	  {
	    //PeakSet aList; aList.clear(); 
	    for(MomDadPeakMap::iterator ami = assignMap.begin(); ami != assignMap.end(); ami++)
	      {
		aList.insert(ami->second);
		if (AssignList.find(ami->second) == AssignList.end())
		  {
		    nList.insert(ami->second); 
		  }
	      }
	  }

	MakeAssignMap(assignMap, PDB, wN, iN); 

	if (DEBUG)
	  {
	    int count = 0; 
	    for(PeakSet::iterator ali = aList.begin(); ali != aList.end(); ali++)
	      {
		if (AssignList.count(*ali) > 1)
		  {
		    if (nList.find(*ali) != nList.end())
		      NOESY_Assign::outputLog() << "new "; 
		    NOESY_Assign::outputLog() << "multi assignments " << AssignList.count(*ali) << " for " << *ali << endl; 
		    count++; 
		  }
	      }
	    NOESY_Assign::outputLog() << "total " << count << endl; 
	  }


	
  if (AssignList.size() - old == 0)
    return 0;

  int un = 0;
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
  {
    if (AssignList.find(&(*iter)) == AssignList.end())
		un++;
  }

  outStream.setf(ios::left, ios::adjustfield);
  outStream.width(6);
  outStream << (itoa(NOESY_Assign::ddcycle())+"-"+itoa(NOESY_Assign::dcycle())).c_str();
  outStream.width(7);
  outStream << "PDB";
  outStream.width(6);
  outStream << unOld - un;
  outStream.width(8);
  outStream << peakList.size() - un;
  outStream.width(7);
  outStream << AssignList.size() - old;
  outStream.width(9);
  outStream << AssignList.size();
  outStream.width(19);
  outStream << "-";
  outStream.width(18);
  outStream << un;
  outStream.width(11);
  outStream << noiseList.size();

  if (NOESY_Assign::COMPARE())
  {
     outStream.width(8);
     outStream << "-";
     outStream.width(11);
     if (NOESY_Assign::COMPARE())
       outStream << AssignList.size() - old - iN;
      else
       outStream << "-";
     outStream.width(11);
     if (NOESY_Assign::COMPARE())
       outStream << iN;
     else
       outStream << "-";
  }
  outStream << "\n" << NOESY_Assign::EOS();
  string temp(outStream.str());
  reportProcessStat += temp.substr(0,temp.find(NOESY_Assign::EOS()));

  return AssignList.size() - old;
}


int NOESY::AssignFar_Pdb(const PdbP& parPdb, const MomDadPeakAtomMap& mdPAMapM2, const SUPPORT& support, const mdsMap& mdsd, const peakmdMap& peakmd)
{
  ostringstream outStream;
  if (DEBUG) NOESY_Assign::outputLog() << "\n-> In AssignFar_Pdb" << endl;

  for(PeakAssignReason::iterator i = AssignList.begin(); i != AssignList.end(); ++i)
  {
   if ((i->second.second.find(BEST) != i->second.second.end()) && (i->second.second.find(PDB) == i->second.second.end()))
  	{
  		i->second.second.erase(i->second.second.find(BEST)); // last cycle
  	}
  }
  
  int old = AssignList.size();
  int unOld = 0;
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
  {
    if (AssignList.find(&(*iter)) == AssignList.end())
			unOld++;
  }

  int wN = 0; int iN = 0;
  PeakList unPeak; unPeak.erase(unPeak.begin(), unPeak.end());

  MomDadPeakMap assignMap; assignMap.clear();
  MomDadPeakMap assignMapM; assignMapM.clear();

  //ContactPairSet AssignedContactBin; // AssignedContactBin.clear();
  if (DEBUG)
  for(ContactPairSet::iterator i = NOESY_Assign::asContactSet().begin(); i != NOESY_Assign::asContactSet().end(); ++i)
  {
  		NOESY_Assign::outputLog() << "asContact: " << i->first << " " << i->second << endl;
  }
 
	// rule in
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
  {
  	if (AssignList.find(&*iter) != AssignList.end())
   		continue;
     
		// get the frq
   	map<MomDad, int> mds; mds.clear();
	for(multimap<const general_peak*, MomDad>::const_iterator i = peakmd.lower_bound(&*iter);
					i != peakmd.upper_bound(&*iter); ++i)
	{
		++mds[i->second];
	}

      PeakAssignReason::iterator rl = BufferReason.lower_bound(&(*iter));
      PeakAssignReason::iterator ru = BufferReason.upper_bound(&(*iter));
      PeakAssignReason::iterator ri;
     
      
		// for mds, if only one of them is < R-max(), rule in
		set<MomDad> goodmds; goodmds.clear();
		for(map<MomDad, int>::iterator imds = mds.begin(); imds != mds.end(); ++imds)
		{
			// the one that selected by most of the structures are assignments -
			// means this part of the structure is converging
			map<MomDad, SampleStatistic>::const_iterator j = mdsd.find(imds->first);

			if (j == mdsd.end())
				continue;

  		int frq = imds->second;

 			if (DEBUG)
 			{
 				NOESY_Assign::outputLog() << "FRQ: " << frq << "-" << mds.size() << *iter << endl
 						<< j->first.first << endl << j->first.second << endl;
 			}

      bool old = 0;
      for(ri = rl; ri != ru; ri++)
  		{
  		  if (((*ri).second.first.first == j->first.first) &&
  		      ((*ri).second.first.second == j->first.second))
         {
         	if ((ri->second.second.count(BEST) != 0) || (ri->second.second.count(UNIQUE) != 0) || (ri->second.second.count(PDB) != 0) || ((ri->second.second.count(DEL_TOP) != 0) || 
												    (ri->second.second.count(DEL_SF) != 0) 
												    || ri->second.second.count(DEL_D2O) != 0) || (ri->second.second.count(SYM) != 0) || (ri->second.second.count(DEL_SAM) != 0))
           		old = 1;
           	break;
         }
  		}

    	if ((old) || (faces[j->first.first->get_index()][j->first.second->get_index()] == -1)
     			|| (contactFaces.get_ctFace(j->first.first->get_index(), j->first.second->get_index()) == -1))
     	{
     		if (DEBUG)
      		NOESY_Assign::outputLog() << "old or in d20 or face not support" << endl;
     		continue;
      }

     //MomDad mdpair;
  		pair<int, int> mij;
  		if (j->first.first->get_index() < j->first.second->get_index())
  		{
  			mij = make_pair(j->first.first->get_index(), j->first.second->get_index());
  		}
  		else
  		{
  			mij = make_pair(j->first.second->get_index(), j->first.first->get_index());
  		}

					vector<float> t; t.clear();
					for(SmplData::iterator k = j->second.samples().begin(); k != j->second.samples().end(); ++k)
					{
						t.push_back(*k);
					}

					int id = (int) ceil(j->second.samples().size()*0.2);

					if (j->second.samples().size() == 1)
						id = 0;

  		 bool minAll = 0;

		 if (DEBUG)
		   	NOESY_Assign::outputLog() << (mds.size() == 1) << " " << (frq == NOESY_Assign::pdbList().size())
		      	<< " " << (sameHydro(j->first.second->get_sym_1(), j->first.first->get_sym_1())) << endl;
     // make assignments
        // for support, long range only
       if ( 

     				// close to distance
  					((
      			 ((mds.size() == 1) && (frq == NOESY_Assign::pdbList().size()))))// consistent and the only choice
       			&& (sameHydro(j->first.second->get_sym_1(), j->first.first->get_sym_1())))
   			{

         	if (DEBUG)
          		NOESY_Assign::outputLog() << "top = 10" << endl;
				  bool newCT = 1;
					if ((!newCT) && !minAll)
					{
							if (DEBUG)
								NOESY_Assign::outputLog() << "not new contact" << endl;
							continue;
					}

					MomDadSet ASset; ASset.insert(j->first);

    			ContactPairSet AssignedContactBin; AssignedContactBin.clear();
          
	  if (NOESY_Assign::MINCON() || isSupported(-1, j->first, ASset, mdPAMapM2, support, AssignedContactBin))
  				{
           	if ((j->first.first->get_Hdiff((*iter).get_HX1(), SWSF_HX1) < parPdb.htol + 0.000001) &&
           		  (j->first.first->get_Xdiff((*iter).get_X1(), SWSF_X1) < parPdb.xtol + 0.000001) &&
           		  (j->first.second->get_Hdiff((*iter).get_HX2(), SWSF_HX2) < parPdb.htol + 0.000001) &&
               		(j->first.second->get_Xdiff((*iter).get_X2(), SWSF_X2) < parPdb.xtol + 0.000001))
        		{
        		  if (DEBUG) NOESY_Assign::outputLog() << "supported, Unique, sumarge < 5.0 or = 10 "  << endl;
        		    assignMap.insert(make_pair(j->first, (&(*iter))));
        		}
        	} // end isSupported
     	} // end if size == 1
		}
	}

  old = AssignList.size();
  unOld = 0;
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
  {
    if (AssignList.find(&(*iter)) == AssignList.end())
			unOld++;
  }

	MakeAssignMap(assignMap, BEST, wN, iN);

  if (AssignList.size() - old == 0)
    return 0;

  int un = 0;
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
  {
    if (AssignList.find(&(*iter)) == AssignList.end())
		un++;
  }

  outStream.setf(ios::left, ios::adjustfield);
  outStream.width(6);
  outStream << (itoa(NOESY_Assign::ddcycle())+"-"+itoa(NOESY_Assign::dcycle())).c_str();
  outStream.width(7);
  outStream << "BEST";
  outStream.width(6);
  outStream << unOld - un;
  outStream.width(8);
  outStream << peakList.size() - un;
  outStream.width(7);
  outStream << AssignList.size() - old;
  outStream.width(9);
  outStream << AssignList.size();
  outStream.width(19);
  outStream << "-";
  outStream.width(18);
  outStream << un;
  outStream.width(11);
  outStream << noiseList.size();

  if (NOESY_Assign::COMPARE())
  {
     outStream.width(8);
       outStream << "-";
     outStream.width(11);
     if (NOESY_Assign::COMPARE())
       outStream << AssignList.size() - old - iN;
      else
       outStream << "-";
     outStream.width(11);
     if (NOESY_Assign::COMPARE())
       outStream << iN;
     else
       outStream << "-";
  }
  outStream << "\n" << NOESY_Assign::EOS();
  string temp(outStream.str());
  reportProcessStat += temp.substr(0,temp.find(NOESY_Assign::EOS()));

  return AssignList.size() - old;
}






















