/***************************************************************************
                          experiment_Qfactor.cpp  -  I/M/L scores
                             -------------------
    begin                : Mon Jul 29 2002
    copyright            : (C) 2002 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/


#include <cmath>
#include "experiment.h"
#include "noesy_assign.h"

#define pow(x,y) pow((double)x, (double)y)
#define abs(x) abs((double)(x))

extern bool DEBUG;


/////////////////////////////////////////
// Validate_Shift with peakList        //
// -- M score                          //
// -- Refinement of inputs             //
/////////////////////////////////////////
typedef multimap<const atom* const, const general_peak* const, less<const atom* const> > AtomPeak;
typedef deque<float> floatlist;
typedef set<const general_peak*, less<const general_peak*> > PeakSet;

ASDP::ratio NOESY::Validate_Shift(ostream& summaryOutput, ostream& outputVal) const
{
	////////////////////////////////////
	// generate expected assignments
	////////////////////////////////////

  AtomPeak MomPeak; AtomPeak DadPeak; AtomPeak CPeak; AtomPeak CPeak2;
  // cases for local referencing (not true)
  //--- bin
  Atom_Index_List Abin;
  Atom_Index_List Bbin;
  Atom_Index_List Gbin; Atom_Index_List G1bin; Atom_Index_List G2bin;
  Atom_Index_List Dbin; Atom_Index_List D1bin; Atom_Index_List D2bin;
  Atom_Index_List Ebin; Atom_Index_List E1bin; Atom_Index_List E2bin;
  Atom_Index_List Zbin;
  Atom_Index_List Hbin;

  char symbol;
  for(Atom_List::iterator iter = NOESY_Assign::atoms().begin(); iter != NOESY_Assign::atoms().end(); ++iter)
  	{
      if (((*iter).get_type() == 'H') && ((*iter).isatt_C() || (Type() =="2D")))  // && ((*iter).att_C().size() != 0))
	{
	  if ((*iter).get_layer() == "A")
	    Abin.insert(make_pair((*iter).get_index(), &(*iter)));
	  else if ((*iter).get_layer() == "B")
	    Bbin.insert(make_pair((*iter).get_index(), &(*iter)));
	  else if ((*iter).get_layer() == "G")
	    {
	      symbol = (*iter).get_sym_1();
	      if ((symbol != 'C') && (symbol != 'S'))
		Gbin.insert(make_pair((*iter).get_index(), &(*iter)));
	    }
	  else if ((*iter).get_layer() == "G1")
	    {
	      symbol = (*iter).get_sym_1();
	      if ((symbol == 'I') || (symbol == 'V'))
		G1bin.insert(make_pair((*iter).get_index(), &(*iter)));
	    }
	  else if ((*iter).get_layer() == "G2")
	    {
	      symbol = (*iter).get_sym_1();
	      if ((symbol == 'I') || (symbol == 'V') || (symbol == 'T'))
		G2bin.insert(make_pair((*iter).get_index(), &(*iter)));
	    }
	  else if ((*iter).get_layer() == "D")
	    {
	      symbol = (*iter).get_sym_1();
	      if ((symbol != 'N') && (symbol != 'D') && (symbol != 'H') && (symbol != 'W'))
		Dbin.insert(make_pair((*iter).get_index(), &(*iter)));
	    }
	  else if ((*iter).get_layer() == "D1")
	    {
	      symbol = (*iter).get_sym_1();
	      if ((symbol == 'I') || (symbol == 'L') || (symbol == 'W')|| (symbol == 'F') || (symbol == 'Y'))
		D1bin.insert(make_pair((*iter).get_index(), &(*iter)));
	    }
	  else if ((*iter).get_layer() == "D2")
	    {
	      symbol = (*iter).get_sym_1();
	      if ((symbol == 'L') || (symbol == 'H')|| (symbol == 'F') || (symbol == 'Y') || (symbol == 'N'))
		D2bin.insert(make_pair((*iter).get_index(), &(*iter)));
	    }
	  else if ((*iter).get_layer() == "E")
	    {
	      symbol = (*iter).get_sym_1();
	      if ((symbol != 'R') && (symbol != 'Q') && (symbol != 'E') &&
		  (symbol != 'H') && (symbol != 'W'))
		Ebin.insert(make_pair((*iter).get_index(), &(*iter)));
	    }
	  else if ((*iter).get_layer() == "E2")
	    {
	      symbol = (*iter).get_sym_1();
	      if ((symbol == 'F') || (symbol == 'Y')||(symbol == 'H'))
		E2bin.insert(make_pair((*iter).get_index(), &(*iter)));
	    }
	  else if ((*iter).get_layer() == "E1")
	    {
	      symbol = (*iter).get_sym_1();
	      if ((symbol == 'H')|| (symbol == 'F') || (symbol == 'Y'))
		E1bin.insert(make_pair((*iter).get_index(), &(*iter)));
	    }
	  else if ((*iter).get_layer() == "E3")
	    {
	      symbol = (*iter).get_sym_1();
	      if ((symbol == 'W'))
		Ebin.insert(make_pair((*iter).get_index(), &(*iter)));
	    }
	  else if ((*iter).get_layer() == "Z")
	    {
	      symbol = (*iter).get_sym_1();
	      if ((symbol == 'F'))
		Zbin.insert(make_pair((*iter).get_index(), &(*iter)));
	    }
	  else if ((*iter).get_layer() == "Z2")
	    {
	      symbol = (*iter).get_sym_1();
	      if ((symbol == 'W'))
		Zbin.insert(make_pair((*iter).get_index(), &(*iter)));
	    }
	  else if ((*iter).get_layer() == "Z3")
	    {
	      symbol = (*iter).get_sym_1();
	      if ((symbol == 'W'))
		Zbin.insert(make_pair((*iter).get_index(), &(*iter)));
	    }
	  else if ((*iter).get_layer() == "H2")
	    {
	      symbol = (*iter).get_sym_1();
	      if ((symbol == 'W'))
		Hbin.insert(make_pair((*iter).get_index(), &(*iter)));
	    }
	}
    }

  Atom_Index_List N_bin;
  Atom_Index_List N_Dbin;
  Atom_Index_List N_Ebin;
  Atom_Index_List N_Zbin;
  Atom_Index_List N_Hbin;

  if (waterFlag)
  {
    for(Atom_List::iterator iter = NOESY_Assign::atoms().begin(); iter != NOESY_Assign::atoms().end(); ++iter)
      {
        if (((*iter).get_type() == 'H') && ((*iter).isatt_N() || (Type() =="2D"))) // && ((*iter).att_N() != 0))
       	{
       	  if ((*iter).get_layer() == "N")
       	    N_bin.insert(make_pair((*iter).get_index(), &(*iter)));
       	  else if ((*iter).get_layer() == "D1")
       	    {
       	      symbol = (*iter).get_sym_1();
       	      if ((symbol == 'H'))
       		N_Dbin.insert(make_pair((*iter).get_index(), &(*iter)));
       	    }
       	  else if ((*iter).get_layer() == "D2")
       	    {
       	      symbol = (*iter).get_sym_1();
       	      if ((symbol == 'N'))
       		N_Dbin.insert(make_pair((*iter).get_index(), &(*iter)));
       	    }
       	  else if ((*iter).get_layer() == "E")
       	    {
       	      symbol = (*iter).get_sym_1();
       	      if ((symbol == 'R'))
       		N_Ebin.insert(make_pair((*iter).get_index(), &(*iter)));
       	    }
       	  else if ((*iter).get_layer() == "E1")
       	    {
       	      symbol = (*iter).get_sym_1();
       	      if ((symbol == 'W'))
       		N_Ebin.insert(make_pair((*iter).get_index(), &(*iter)));
       	    }
       	  else if ((*iter).get_layer() == "E2")
       	    {
       	      symbol = (*iter).get_sym_1();
       	      if ((symbol == 'Q') || (symbol == 'H'))
       		N_Ebin.insert(make_pair((*iter).get_index(), &(*iter)));
       	    }
       	  else if ((*iter).get_layer() == "Z")
       	    {
       	      symbol = (*iter).get_sym_1();
       	      if ((symbol == 'L'))
       		N_Zbin.insert(make_pair((*iter).get_index(), &(*iter)));
       	    }
       	  else if ((*iter).get_layer() == "H1")
       	    {
       	      symbol = (*iter).get_sym_1();
       	      if ((symbol == 'R'))
       		N_Hbin.insert(make_pair((*iter).get_index(), &(*iter)));
       	    }
       	  else if ((*iter).get_layer() == "H2")
       	    {
       	      symbol = (*iter).get_sym_1();
       	      if ((symbol == 'R'))
       		N_Hbin.insert(make_pair((*iter).get_index(), &(*iter)));
       	    }
       	}
      }
	}

  //--- starting the simulation
  MomDadSet SIMUCOSYassignList;
  //int count = 0;
  for(SEQM::const_iterator iter = seq.begin(); iter != seq.end(); ++iter)
    {
      Atom_Index_List::iterator Al = Abin.lower_bound((*iter).first);
      Atom_Index_List::iterator Au = Abin.upper_bound((*iter).first);

      Atom_Index_List::iterator Bl = Bbin.lower_bound((*iter).first);
      Atom_Index_List::iterator Bu = Bbin.upper_bound((*iter).first);

      Atom_Index_List::iterator Gl = Gbin.lower_bound((*iter).first);
      Atom_Index_List::iterator Gu = Gbin.upper_bound((*iter).first);

      Atom_Index_List::iterator G1l = G1bin.lower_bound((*iter).first);
      Atom_Index_List::iterator G1u = G1bin.upper_bound((*iter).first);
      Atom_Index_List::iterator G2l = G2bin.lower_bound((*iter).first);
      Atom_Index_List::iterator G2u = G2bin.upper_bound((*iter).first);

      Atom_Index_List::iterator Dl = Dbin.lower_bound((*iter).first);
      Atom_Index_List::iterator Du = Dbin.upper_bound((*iter).first);

      Atom_Index_List::iterator D1l = D1bin.lower_bound((*iter).first);
      Atom_Index_List::iterator D1u = D1bin.upper_bound((*iter).first);
      Atom_Index_List::iterator D2l = D2bin.lower_bound((*iter).first);
      Atom_Index_List::iterator D2u = D2bin.upper_bound((*iter).first);

      Atom_Index_List::iterator El = Ebin.lower_bound((*iter).first);
      Atom_Index_List::iterator Eu = Ebin.upper_bound((*iter).first);
      Atom_Index_List::iterator E1l = E1bin.lower_bound((*iter).first);
      Atom_Index_List::iterator E1u = E1bin.upper_bound((*iter).first);
      Atom_Index_List::iterator E2l = E2bin.lower_bound((*iter).first);
      Atom_Index_List::iterator E2u = E2bin.upper_bound((*iter).first);

      Atom_Index_List::iterator Zl = Zbin.lower_bound((*iter).first);
      Atom_Index_List::iterator Zu = Zbin.upper_bound((*iter).first);

      Atom_Index_List::iterator Hl = Hbin.lower_bound((*iter).first);
      Atom_Index_List::iterator Hu = Hbin.upper_bound((*iter).first);

      Atom_Index_List::iterator Nl = N_bin.lower_bound((*iter).first);
      Atom_Index_List::iterator Nu = N_bin.upper_bound((*iter).first);

      Atom_Index_List::iterator NDl = N_Dbin.lower_bound((*iter).first);
      Atom_Index_List::iterator NDu = N_Dbin.upper_bound((*iter).first);

      Atom_Index_List::iterator NEl = N_Ebin.lower_bound((*iter).first);
      Atom_Index_List::iterator NEu = N_Ebin.upper_bound((*iter).first);

      Atom_Index_List::iterator NZl = N_Zbin.lower_bound((*iter).first);
      Atom_Index_List::iterator NZu = N_Zbin.upper_bound((*iter).first);

      Atom_Index_List::iterator NHl = N_Hbin.lower_bound((*iter).first);
      Atom_Index_List::iterator NHu = N_Hbin.upper_bound((*iter).first);

if ((Type().substr(0,3) =="3DC") || (Type()=="4DCC") || (Type() =="2D"))
{
      //--- 1. attached to the same C
      // a
      for(Atom_Index_List::iterator Ai1 = Al; Ai1 != Au; Ai1++)
	for(Atom_Index_List::iterator Ai2 = Al; Ai2 != Au; Ai2++)
	  {
	    if (Ai1 != Ai2)
	      SIMUCOSYassignList.insert(MomDad((*Ai1).second, (*Ai2).second));
	  }
      // b
      for(Atom_Index_List::iterator Bi1 = Bl; Bi1 != Bu; Bi1++)
	for(Atom_Index_List::iterator Bi2 = Bl; Bi2 != Bu; Bi2++)
	  {
	    if (Bi1 != Bi2)
	      SIMUCOSYassignList.insert(MomDad((*Bi1).second, (*Bi2).second));
	  }
      // g
      for(Atom_Index_List::iterator Gi1 = Gl; Gi1 != Gu; Gi1++)
	for(Atom_Index_List::iterator Gi2 = Gl; Gi2 != Gu; Gi2++)
	  {
	    if (Gi1 != Gi2)
	      SIMUCOSYassignList.insert(MomDad((*Gi1).second, (*Gi2).second));
	  }

      for(Atom_Index_List::iterator G1i1 = G1l; G1i1 != G1u; G1i1++)
	for(Atom_Index_List::iterator G1i2 = G1l; G1i2 != G1u; G1i2++)
	  {
	    if (G1i1 != G1i2)
	      SIMUCOSYassignList.insert(MomDad((*G1i1).second, (*G1i2).second));
	  }
      for(Atom_Index_List::iterator G2i1 = G2l; G2i1 != G2u; G2i1++)
	for(Atom_Index_List::iterator G2i2 = G2l; G2i2 != G2u; G2i2++)
	  {
	    if (G2i1 != G2i2)
	      SIMUCOSYassignList.insert(MomDad((*G2i1).second, (*G2i2).second));
	  }

      for(Atom_Index_List::iterator G1i1 = G1l; G1i1 != G1u; G1i1++)  // VAL HG1 and HG2
	for(Atom_Index_List::iterator G2i2 = G2l; G2i2 != G2u; G2i2++)
	  {
	    if (G1i1 != G2i2)
	      SIMUCOSYassignList.insert(MomDad((*G1i1).second, (*G2i2).second));
	  }

      // d
      for(Atom_Index_List::iterator Di1 = Dl; Di1 != Du; Di1++)
	for(Atom_Index_List::iterator Di2 = Dl; Di2 != Du; Di2++)
	  {
	    if (Di1 != Di2)
	      SIMUCOSYassignList.insert(MomDad((*Di1).second, (*Di2).second));
	  }
      for(Atom_Index_List::iterator D1i1 = D1l; D1i1 != D1u; D1i1++)
	for(Atom_Index_List::iterator D1i2 = D1l; D1i2 != D1u; D1i2++)
	  {
	    if (D1i1 != D1i2)
	      SIMUCOSYassignList.insert(MomDad((*D1i1).second, (*D1i2).second));
	  }
      for(Atom_Index_List::iterator D2i1 = D2l; D2i1 != D2u; D2i1++)
	for(Atom_Index_List::iterator D2i2 = D2l; D2i2 != D2u; D2i2++)
	  {
	    if (D2i1 != D2i2)
	      SIMUCOSYassignList.insert(MomDad((*D2i1).second, (*D2i2).second));
	  }
      for(Atom_Index_List::iterator D1i1 = D1l; D1i1 != D1u; D1i1++)  // LEU HD1 and HD2
	for(Atom_Index_List::iterator D2i2 = D2l; D2i2 != D2u; D2i2++)
	  {
	    if (D1i1 != D2i2)
	      SIMUCOSYassignList.insert(MomDad((*D1i1).second, (*D2i2).second));
	  }
      // e
      for(Atom_Index_List::iterator Ei1 = El; Ei1 != Eu; Ei1++)
	for(Atom_Index_List::iterator Ei2 = El; Ei2 != Eu; Ei2++)
	  {
	    if (Ei1 != Ei2)
	      SIMUCOSYassignList.insert(MomDad((*Ei1).second, (*Ei2).second));
	  }
      for(Atom_Index_List::iterator E1i1 = E1l; E1i1 != E1u; E1i1++)
	for(Atom_Index_List::iterator E1i2 = E1l; E1i2 != E1u; E1i2++)
	  {
	    if (E1i1 != E1i2)
	      SIMUCOSYassignList.insert(MomDad((*E1i1).second, (*E1i2).second));
	  }
      for(Atom_Index_List::iterator E2i1 = E2l; E2i1 != E2u; E2i1++)
	for(Atom_Index_List::iterator E2i2 = E2l; E2i2 != E2u; E2i2++)
	  {
	    if (E2i1 != E2i2)
	      SIMUCOSYassignList.insert(MomDad((*E2i1).second, (*E2i2).second));
	  }
      for(Atom_Index_List::iterator E1i1 = E1l; E1i1 != E1u; E1i1++)
	for(Atom_Index_List::iterator E2i2 = E2l; E2i2 != E2u; E2i2++)
	  {
	    if (E1i1 != E2i2)
	      SIMUCOSYassignList.insert(MomDad((*E1i1).second, (*E2i2).second));
	  }


      //--- 2. attached to the adjacent C
      if (waterFlag && (Type() != "4DCC"))
	{
	  //--- AN
	  for(Atom_Index_List::iterator Ai1 = Al; Ai1 != Au; Ai1++)
	    for(Atom_Index_List::iterator Ni2 = Nl; Ni2 != Nu; Ni2++)
	      {
		SIMUCOSYassignList.insert(MomDad((*Ai1).second, (*Ni2).second));
	      }

	  //--- G/D to N
	  for(Atom_Index_List::iterator G1i1 = G1l; G1i1 != G1u; G1i1++)
	    {
	      if (G1i1->second->get_sym_1() == 'V')
		for(Atom_Index_List::iterator Ni2 = Nl; Ni2 != Nu; Ni2++)
		{
		  SIMUCOSYassignList.insert(MomDad((*G1i1).second, (*Ni2).second));
		}
	    }

	  for(Atom_Index_List::iterator G2i1 = G2l; G2i1 != G2u; G2i1++)
	  {
	    if (G2i1->second->get_sym_1() == 'V')
	      for(Atom_Index_List::iterator Ni2 = Nl; Ni2 != Nu; Ni2++)
		{
		  SIMUCOSYassignList.insert(MomDad((*G2i1).second, (*Ni2).second));
		}
	  }

	  for(Atom_Index_List::iterator D1i1 = D1l; D1i1 != D1u; D1i1++)
	  {
	    if (D1i1->second->get_sym_1() == 'L')
	      for(Atom_Index_List::iterator Ni2 = Nl; Ni2 != Nu; Ni2++)
		{
		  SIMUCOSYassignList.insert(MomDad((*D1i1).second, (*Ni2).second));
		}
	  }

	  for(Atom_Index_List::iterator D2i1 = D2l; D2i1 != D2u; D2i1++)
	  {
	    if (D2i1->second->get_sym_1() == 'L')
	      for(Atom_Index_List::iterator Ni2 = Nl; Ni2 != Nu; Ni2++)
		{
		  SIMUCOSYassignList.insert(MomDad((*D2i1).second, (*Ni2).second));
		}
	  }
	}


      //--- AB
      for(Atom_Index_List::iterator Ai1 = Al; Ai1 != Au; Ai1++)
	for(Atom_Index_List::iterator Bi2 = Bl; Bi2 != Bu; Bi2++)
	  {
	    SIMUCOSYassignList.insert(MomDad((*Ai1).second, (*Bi2).second));
	    SIMUCOSYassignList.insert(MomDad((*Bi2).second, (*Ai1).second));
	  }

      //--- BG
      for(Atom_Index_List::iterator Gi1 = Gl; Gi1 != Gu; Gi1++)
	for(Atom_Index_List::iterator Bi2 = Bl; Bi2 != Bu; Bi2++)
	  {
	    SIMUCOSYassignList.insert(MomDad((*Gi1).second, (*Bi2).second));
	    SIMUCOSYassignList.insert(MomDad((*Bi2).second, (*Gi1).second));
	  }

      //--- BG1
      for(Atom_Index_List::iterator G1i1 = G1l; G1i1 != G1u; G1i1++)
	for(Atom_Index_List::iterator Bi2 = Bl; Bi2 != Bu; Bi2++)
	  {
	    SIMUCOSYassignList.insert(MomDad((*G1i1).second, (*Bi2).second));
	    SIMUCOSYassignList.insert(MomDad((*Bi2).second, (*G1i1).second));
	  }

      //--- BG2
      for(Atom_Index_List::iterator G2i1 = G2l; G2i1 != G2u; G2i1++)
	for(Atom_Index_List::iterator Bi2 = Bl; Bi2 != Bu; Bi2++)
	  {
	    SIMUCOSYassignList.insert(MomDad((*G2i1).second, (*Bi2).second));
	    SIMUCOSYassignList.insert(MomDad((*Bi2).second, (*G2i1).second));
	  }

      //--- BD
      if ((iter->second.get_sym_1() == 'F') || (iter->second.get_sym_1() == 'Y'))
	{
	  for(Atom_Index_List::iterator Bi1 = Bl; Bi1 != Bu; Bi1++)
	    for(Atom_Index_List::iterator Di2 = Dl; Di2 != Du; Di2++)
	      {
		SIMUCOSYassignList.insert(MomDad((*Bi1).second, (*Di2).second));
		SIMUCOSYassignList.insert(MomDad((*Di2).second, (*Bi1).second));
	      }

	  for(Atom_Index_List::iterator Bi1 = Bl; Bi1 != Bu; Bi1++)
	    for(Atom_Index_List::iterator D1i2 = D1l; D1i2 != D1u; D1i2++)
	      {
		SIMUCOSYassignList.insert(MomDad((*Bi1).second, (*D1i2).second));
		SIMUCOSYassignList.insert(MomDad((*D1i2).second, (*Bi1).second));
	      }
	  //--- BD2
	  for(Atom_Index_List::iterator Bi1 = Bl; Bi1 != Bu; Bi1++)
	    for(Atom_Index_List::iterator D2i2 = D2l; D2i2 != D2u; D2i2++)
	      {
		SIMUCOSYassignList.insert(MomDad((*Bi1).second, (*D2i2).second));
		SIMUCOSYassignList.insert(MomDad((*D2i2).second, (*Bi1).second));
	      }
	}

      //--- GD
      for(Atom_Index_List::iterator Gi1 = Gl; Gi1 != Gu; Gi1++)
	for(Atom_Index_List::iterator Di2 = Dl; Di2 != Du; Di2++)
	  {
	    SIMUCOSYassignList.insert(MomDad((*Gi1).second, (*Di2).second));
	    SIMUCOSYassignList.insert(MomDad((*Di2).second, (*Gi1).second));
	  }

      //--- GD1
      for(Atom_Index_List::iterator Gi1 = Gl; Gi1 != Gu; Gi1++)
	for(Atom_Index_List::iterator D1i2 = D1l; D1i2 != D1u; D1i2++)
	  {
	    SIMUCOSYassignList.insert(MomDad((*Gi1).second, (*D1i2).second));
	    SIMUCOSYassignList.insert(MomDad((*D1i2).second, (*Gi1).second));
	  }

      //--- GD2
      for(Atom_Index_List::iterator Gi1 = Gl; Gi1 != Gu; Gi1++)
	for(Atom_Index_List::iterator D2i2 = D2l; D2i2 != D2u; D2i2++)
	  {
	    SIMUCOSYassignList.insert(MomDad((*Gi1).second, (*D2i2).second));
	    SIMUCOSYassignList.insert(MomDad((*D2i2).second, (*Gi1).second));
	  }

      //--- G1D1
      for(Atom_Index_List::iterator G1i1 = G1l; G1i1 != G1u; G1i1++)
	for(Atom_Index_List::iterator D1i2 = D1l; D1i2 != D1u; D1i2++)
	  {
	    SIMUCOSYassignList.insert(MomDad((*G1i1).second, (*D1i2).second));
	    SIMUCOSYassignList.insert(MomDad((*D1i2).second, (*G1i1).second));
	  }

      //--- G2D1
      if ((*iter).second.get_sym_1() != 'I')
	{
	  for(Atom_Index_List::iterator G2i1 = G2l; G2i1 != G2u; G2i1++)
	    for(Atom_Index_List::iterator D1i2 = D1l; D1i2 != D1u; D1i2++)
	      {
		SIMUCOSYassignList.insert(MomDad((*G2i1).second, (*D1i2).second));
		SIMUCOSYassignList.insert(MomDad((*D1i2).second, (*G2i1).second));
	      }
	}

      //--- G1D2
      for(Atom_Index_List::iterator G1i1 = G1l; G1i1 != G1u; G1i1++)
	for(Atom_Index_List::iterator D2i2 = D2l; D2i2 != D2u; D2i2++)
	  {
	    SIMUCOSYassignList.insert(MomDad((*G1i1).second, (*D2i2).second));
	    SIMUCOSYassignList.insert(MomDad((*D2i2).second, (*G1i1).second));
	  }

      //--- G2D2
      for(Atom_Index_List::iterator G2i1 = G2l; G2i1 != G2u; G2i1++)
	for(Atom_Index_List::iterator D2i2 = D2l; D2i2 != D2u; D2i2++)
	  {
	    SIMUCOSYassignList.insert(MomDad((*G2i1).second, (*D2i2).second));
	    SIMUCOSYassignList.insert(MomDad((*D2i2).second, (*G2i1).second));
	  }

      //--- ED
      for(Atom_Index_List::iterator Ei1 = El; Ei1 != Eu; Ei1++)
	for(Atom_Index_List::iterator Di2 = Dl; Di2 != Du; Di2++)
	  {
	    SIMUCOSYassignList.insert(MomDad((*Ei1).second, (*Di2).second));
	    SIMUCOSYassignList.insert(MomDad((*Di2).second, (*Ei1).second));
	  }

      //--- ED1
      if ((*iter).second.get_sym_1() != 'W')
	{
	  for(Atom_Index_List::iterator Ei1 = El; Ei1 != Eu; Ei1++)
	    for(Atom_Index_List::iterator D1i2 = D1l; D1i2 != D1u; D1i2++)
	      {
		SIMUCOSYassignList.insert(MomDad((*Ei1).second, (*D1i2).second));
		SIMUCOSYassignList.insert(MomDad((*D1i2).second, (*Ei1).second));
	      }
	}

      //--- ED2
      if ((*iter).second.get_sym_1() != 'H')
	{
	  for(Atom_Index_List::iterator Ei1 = El; Ei1 != Eu; Ei1++)
	    for(Atom_Index_List::iterator D2i2 = D2l; D2i2 != D2u; D2i2++)
	      {
		SIMUCOSYassignList.insert(MomDad((*Ei1).second, (*D2i2).second));
		SIMUCOSYassignList.insert(MomDad((*D2i2).second, (*Ei1).second));
	      }
	}

      //--- E1D1
      if (((*iter).second.get_sym_1() == 'F') || (iter->second.get_sym_1() == 'Y'))
	{
	  for(Atom_Index_List::iterator E1i1 = E1l; E1i1 != E1u; E1i1++)
	    for(Atom_Index_List::iterator D1i2 = D1l; D1i2 != D1u; D1i2++)
	      {
		SIMUCOSYassignList.insert(MomDad((*E1i1).second, (*D1i2).second));
		SIMUCOSYassignList.insert(MomDad((*D1i2).second, (*E1i1).second));
	      }
	}

      //--- E2D2
      if (((*iter).second.get_sym_1() == 'F') || (iter->second.get_sym_1() == 'Y'))
	{
	  for(Atom_Index_List::iterator E2i1 = E2l; E2i1 != E2u; E2i1++)
	    for(Atom_Index_List::iterator D2i2 = D2l; D2i2 != D2u; D2i2++)
	      {
		SIMUCOSYassignList.insert(MomDad((*E2i1).second, (*D2i2).second));
		SIMUCOSYassignList.insert(MomDad((*D2i2).second, (*E2i1).second));
	      }
	}

      if (waterFlag && (Type() != "4DCC"))
	{
      //--- END
      for(Atom_Index_List::iterator Ei1 = El; Ei1 != Eu; Ei1++)
	for(Atom_Index_List::iterator Di2 = NDl; Di2 != NDu; Di2++)
	  {
	    SIMUCOSYassignList.insert(MomDad((*Ei1).second, (*Di2).second));
	  }

      //--- NED
      for(Atom_Index_List::iterator Ei1 = NEl; Ei1 != NEu; Ei1++)
	for(Atom_Index_List::iterator Di2 = Dl; Di2 != Du; Di2++)
	  {
	    SIMUCOSYassignList.insert(MomDad((*Di2).second, (*Ei1).second));
	  }
	}
      //--- EZ
      for(Atom_Index_List::iterator Ei1 = El; Ei1 != Eu; Ei1++)
	for(Atom_Index_List::iterator Zi2 = Zl; Zi2 != Zu; Zi2++)
	  {
	    if (((*iter).second.get_sym_1() != 'W') ||
		(((*Ei1).second->get_name() != "HE3") &&
		 ((*Zi2).second->get_name() != "HZ2")))
	      {
		SIMUCOSYassignList.insert(MomDad((*Ei1).second, (*Zi2).second));
		SIMUCOSYassignList.insert(MomDad((*Zi2).second, (*Ei1).second));
	      }
	  }

      if (waterFlag && (Type() != "4DCC"))
    	{
          //--- ENZ
          for(Atom_Index_List::iterator Ei1 = El; Ei1 != Eu; Ei1++)
    	for(Atom_Index_List::iterator Zi2 = NZl; Zi2 != NZu; Zi2++)
    	  {
    	      {
    		SIMUCOSYassignList.insert(MomDad((*Ei1).second, (*Zi2).second));
    	      }
    	  }
    	}
      //--- ZH
      for(Atom_Index_List::iterator Hi1 = Hl; Hi1 != Hu; Hi1++)
      	for(Atom_Index_List::iterator Zi2 = Zl; Zi2 != Zu; Zi2++)
      	  {
      	      SIMUCOSYassignList.insert(MomDad((*Hi1).second, (*Zi2).second));
      	      SIMUCOSYassignList.insert(MomDad((*Zi2).second, (*Hi1).second));
      	  }
 }

if ((Type()=="3DN")|| (Type() =="2D"))
{
      //--- 1. attached to the same N

      // d
      for(Atom_Index_List::iterator Di1 = NDl; Di1 != NDu; Di1++)
	for(Atom_Index_List::iterator Di2 = NDl; Di2 != NDu; Di2++)
	  {
	    if (Di1 != Di2)
	      SIMUCOSYassignList.insert(MomDad((*Di1).second, (*Di2).second));
	  }
      // e
      for(Atom_Index_List::iterator Ei1 = NEl; Ei1 != NEu; Ei1++)
	for(Atom_Index_List::iterator Ei2 = NEl; Ei2 != NEu; Ei2++)
	  {
	    if (Ei1 != Ei2)
	      SIMUCOSYassignList.insert(MomDad((*Ei1).second, (*Ei2).second));
	  }

      // z
      for(Atom_Index_List::iterator Zi1 = NZl; Zi1 != NZu; Zi1++)
	for(Atom_Index_List::iterator Zi2 = NZl; Zi2 != NZu; Zi2++)
	  {
	    if (Zi1 != Zi2)
	      SIMUCOSYassignList.insert(MomDad((*Zi1).second, (*Zi2).second));
	  }
      // h
      for(Atom_Index_List::iterator Hi1 = NHl; Hi1 != NHu; Hi1++)
	for(Atom_Index_List::iterator Hi2 = NHl; Hi2 != NHu; Hi2++)
	  {
	    if (Hi1 != Hi2)
	      SIMUCOSYassignList.insert(MomDad((*Hi1).second, (*Hi2).second));
	  }
      //--- 2. attached to the adjacent N

      //--- AN
      for(Atom_Index_List::iterator Ai1 = Al; Ai1 != Au; Ai1++)
	for(Atom_Index_List::iterator Ni2 = Nl; Ni2 != Nu; Ni2++)
	  {
	    SIMUCOSYassignList.insert(MomDad((*Ni2).second, (*Ai1).second));
	  }

      //--- G/D to N
      for(Atom_Index_List::iterator G1i1 = G1l; G1i1 != G1u; G1i1++)
	{
	  if (G1i1->second->get_sym_1() == 'V')
	    for(Atom_Index_List::iterator Ni2 = Nl; Ni2 != Nu; Ni2++)
	      {
		SIMUCOSYassignList.insert(MomDad((*Ni2).second, (*G1i1).second));
	      }
	}

      for(Atom_Index_List::iterator G2i1 = G2l; G2i1 != G2u; G2i1++)
	{
	  if (G2i1->second->get_sym_1() == 'V')
	    for(Atom_Index_List::iterator Ni2 = Nl; Ni2 != Nu; Ni2++)
	      {
		SIMUCOSYassignList.insert(MomDad((*Ni2).second, (*G2i1).second));
	      }
	}

      for(Atom_Index_List::iterator D1i1 = D1l; D1i1 != D1u; D1i1++)
	{
	  if (D1i1->second->get_sym_1() == 'L')
	    for(Atom_Index_List::iterator Ni2 = Nl; Ni2 != Nu; Ni2++)
	      {
		SIMUCOSYassignList.insert(MomDad((*Ni2).second, (*D1i1).second));
	      }
	}

      for(Atom_Index_List::iterator D2i1 = D2l; D2i1 != D2u; D2i1++)
	{
	  if (D2i1->second->get_sym_1() == 'L')
	    for(Atom_Index_List::iterator Ni2 = Nl; Ni2 != Nu; Ni2++)
	      {
		SIMUCOSYassignList.insert(MomDad((*Ni2).second, (*D2i1).second));
	      }
	}

      //--- END
      for(Atom_Index_List::iterator Ei1 = El; Ei1 != Eu; Ei1++)
	for(Atom_Index_List::iterator Di2 = NDl; Di2 != NDu; Di2++)
	  {
	    SIMUCOSYassignList.insert(MomDad((*Di2).second, (*Ei1).second));
	  }


      //--- NED
      for(Atom_Index_List::iterator Ei1 = NEl; Ei1 != NEu; Ei1++)
	for(Atom_Index_List::iterator Di2 = Dl; Di2 != Du; Di2++)
	  {
	    SIMUCOSYassignList.insert(MomDad((*Ei1).second, (*Di2).second));
	  }


      //--- ENZ
      for(Atom_Index_List::iterator Ei1 = El; Ei1 != Eu; Ei1++)
	for(Atom_Index_List::iterator Zi2 = NZl; Zi2 != NZu; Zi2++)
	  {
	      {
		SIMUCOSYassignList.insert(MomDad((*Zi2).second, (*Ei1).second));
	      }
	  }
}

}

// refine the list by removing simulated peaks that have no C/N assigments or close to diangle

  //MomDadSet tempSet; tempSet = SIMUCOSYassignList;
  MomDadSet tempSet(SIMUCOSYassignList);
  for(MomDadSet::iterator iter =  tempSet.begin(); iter != tempSet.end(); iter++)
    {
      if ((fabs((*iter).first->get_shift() - (*iter).second->get_shift()) < 0.3999999) || //diangle
	  ((!canbeMom(iter->first) || !canbeDad(iter->second)) ))
        {
	  MomDadSet::iterator i = SIMUCOSYassignList.find((*iter));
	  SIMUCOSYassignList.erase((*i));
        }
    }


  /////////////////////////////////////
  // find the best match first
  ////////////////////////////////////

  //int time = 0;
  AtomSet atomset; atomset.clear();
  MomDadSet assmomdad; assmomdad.clear();
  //PeakSet asspeakset; asspeakset.erase(asspeakset.begin(), asspeakset.end());

  PeakMomDadMap COSYpeakmomdad; COSYpeakmomdad.erase(COSYpeakmomdad.begin(), COSYpeakmomdad.end());
  PeakSet peakset;  peakset.erase(peakset.begin(), peakset.end());

  // find the best match and assignment it
  for(MomDadSet::iterator iter =  SIMUCOSYassignList.begin(); iter != SIMUCOSYassignList.end(); iter++)
    {
      float distmin = 100;
      PeakList::const_iterator ti;
      for(PeakList::const_iterator di = peakList.begin(); di != peakList.end(); di++)
	{
	  if (((*iter).first->get_Hdiff((*di).get_HX1(), SWSF_HX1) <= tols.HX1) &&
	      ((Type() == "2D") || ((*iter).first->get_Xdiff((*di).get_X1(), SWSF_X1) <= tols.X1)) &&
	      ((*iter).second->get_Hdiff((*di).get_HX2(), SWSF_HX2) <= tols.HX2) &&
	      ((Type() != "4DCC") ||
	      	((*iter).second->get_Xdiff((*di).get_X2(), SWSF_X2) <=tols.X2)))
	    {
	      float dist = (*iter).first->get_Hdiff((*di).get_HX1(), SWSF_HX1) +
			(*iter).second->get_Hdiff((*di).get_HX2(), SWSF_HX2);

	      if (Type() != "2D")
		{
		  dist += (*iter).first->get_Xdiff((*di).get_X1(), SWSF_X1)/10.0;
		}

	      if (Type() == "4DCC")
		{
		  dist += (*iter).second->get_Xdiff((*di).get_X2(), SWSF_X2)/10.0;
		}

	      dist = floor(dist * 100.0 + 0.5)/100.0;
	      if (dist < distmin)
		{
		  distmin = dist; ti = di;
		}
	    }
	}

      if (distmin != 100) // connect the peak with the simulated peak-lists
	{
	  COSYpeakmomdad.insert(make_pair(&(*ti), (*iter)));
	  peakset.insert((&*ti));
	  assmomdad.insert(*iter);

	  outputVal << "matched" << iter->first << endl << iter->second << endl;
	}
    }

  //////////////////////////////////////////////
  //--- report the one that Have no matches
  /////////////////////////////////////////////

  if (NOESY_Assign::NOESY_Assign::isV()||NOESY_Assign::NOESY_Assign::isQ())
  outputVal << "# Checking " << label() << ": Simulated Peak NOT Matched in the Peak List:" << endl << endl;

  int total = 0;
  for(MomDadSet::iterator iter =  SIMUCOSYassignList.begin(); iter != SIMUCOSYassignList.end(); iter++)
    {
      if (assmomdad.find(*iter) == assmomdad.end())
    	{
    	  total++;
    	  if (NOESY_Assign::NOESY_Assign::isV()||NOESY_Assign::NOESY_Assign::isQ())
    	  	outputVal << (*iter).first << endl << (*iter).second << endl << endl;
    	}
    }

  //////////////////////////////////////////////
  //--- find the unique one with good qutity
  //////////////////////////////////////////////
  if (DEBUG)NOESY_Assign::outputLog() << "# Find the unique one with good qunitity and local refine:" << endl;
  assmomdad.erase(assmomdad.begin(), assmomdad.end());

  int NOPoss = 0;
  for(PeakList::const_iterator iter = peakList.begin(); iter != peakList.end();iter++)
    {
      if (!nearH2O(*iter) && ((Type() == "4DCC") || (!nearDIA(*iter))))
      {
      	if ((MombySimpleMatch.find((&*iter)) == MombySimpleMatch.end()) ||
      	    (DadbySimpleMatch.find((&*iter)) == DadbySimpleMatch.end()))
      	  NOPoss++;
      }
      if ((COSYpeakmomdad.count((&*iter)) == 1))
     	{
     	  PeakMomDadMap::iterator pi = COSYpeakmomdad.find((&*iter));
     	  if (DEBUG)
     	  {
     	  NOESY_Assign::outputLog() << endl << (*iter) << endl;
     	  NOESY_Assign::outputLog() << "HX1:" << (*pi).second.first << endl;
     	  NOESY_Assign::outputLog() << "HX2:" << (*pi).second.second << endl;
     	  }

     	  MomPeak.insert(make_pair((*pi).second.first, &(*iter)));
     	  DadPeak.insert(make_pair((*pi).second.second, &(*iter)));

	  if ((Type().substr(0,3) == "3DC"))
	    {
	      CPeak.insert(make_pair((*pi).second.first->att_C((*iter).get_X1(), SWSF_X1), &(*iter)));
	    } else if ((Type() == "3DN"))
	    {
	      CPeak.insert(make_pair((*pi).second.first->att_N(), &(*iter)));
	    } else if (Type() == "4DCC")
	    {
	      CPeak.insert(make_pair((*pi).second.first->att_C((*iter).get_X1(), SWSF_X1), &(*iter)));
	      CPeak2.insert(make_pair((*pi).second.second->att_C((*iter).get_X2(), SWSF_X2), &(*iter)));
	    }

     	  atomset.insert((*pi).second.first);
     	  atomset.insert((*pi).second.second);

	  if (Type().substr(0,3) =="3DC")
	    {
	      atomset.insert((*pi).second.first->att_C((*iter).get_X1(), SWSF_X1));
	    } else if (Type()=="3DN")
	    {
	      atomset.insert((*pi).second.first->att_N());
	    } else if (Type()=="4DCC")
	    {
	      atomset.insert((*pi).second.first->att_C((*iter).get_X1(), SWSF_X1));
	      atomset.insert((*pi).second.first->att_C((*iter).get_X2(), SWSF_X2));
	    }

     	  assmomdad.insert((*pi).second);
     	}
    }

    //////////////////////////////
    //--- local referencing
    //////////////////////////////

    if (NOESY_Assign::isV()||NOESY_Assign::isQ()) outputVal << endl << "# Local referencing" << endl;
		// HX1-X1
    float diffHX = 0, diffX = 0, diffHXab = 0, diffXab = 0;
    int totalHX = 0, totalX = 0;

    // HX2-X2
    float diffHX2 = 0, diffX2 = 0, diffHXab2 = 0, diffXab2 = 0;
    int totalHX2 = 0, totalX2 = 0;

    for(AtomSet::iterator iter = atomset.begin(); iter != atomset.end(); iter++)
    {
      //--- count the avg, std and remove the outliner
      float X = 0, sumX = 0; int XN = 0;
      float Xab = 0, sumXab = 0;
      float X2 = 0, sumX2 = 0; int XN2 = 0;
      float X2ab = 0, sumXab2 = 0;
      floatlist valuelist; valuelist.erase(valuelist.begin(), valuelist.end());
      floatlist valuelist2; valuelist2.erase(valuelist2.begin(), valuelist2.end());

      if (((*iter)->get_type() == 'H'))
    	{
    	  if (NOESY_Assign::isV()||NOESY_Assign::isQ()) outputVal << endl << "-->" << (*iter) << endl;
    	  AtomPeak::iterator Ml = MomPeak.lower_bound(*iter);
    	  AtomPeak::iterator Mu = MomPeak.upper_bound(*iter);
    	  AtomPeak::iterator Dl = DadPeak.lower_bound(*iter);
    	  AtomPeak::iterator Du = DadPeak.upper_bound(*iter);

    	  if (Ml != Mu)
    	    {
    	      if (NOESY_Assign::isV()||NOESY_Assign::isQ()) outputVal << "as HX1:" << endl ;
    	      for(AtomPeak::iterator Mi = Ml; Mi != Mu; Mi++)
    	        if (NOESY_Assign::isV()||NOESY_Assign::isQ()) outputVal << "     " << (*Mi).second  << endl;
    	      for(AtomPeak::iterator Mi = Ml; Mi != Mu; Mi++)
    		    {
              sumX += (*iter)->get_HdiffAb(Mi->second->get_HX1(), SWSF_HX1);   // sum of differences
              sumXab += (*iter)->get_Hdiff(Mi->second->get_HX1(), SWSF_HX1);   // sum of differences
    		      valuelist.push_back((*Mi).second->get_HX1());
    		    }
    	    }

    	  if (Dl != Du)
    	    {
    	      if (NOESY_Assign::isV()||NOESY_Assign::isQ()) outputVal << "as HX2:" << endl ;
    	      for(AtomPeak::iterator Di = Dl; Di != Du; Di++)
    	        if (NOESY_Assign::isV()||NOESY_Assign::isQ()) outputVal << "     " << (*Di).second  << endl;
    	      for(AtomPeak::iterator Di = Dl; Di != Du; Di++)
    		    {
              sumX2 += (*iter)->get_HdiffAb(Di->second->get_HX2(), SWSF_HX2);   // sum of differences
              sumXab2 += (*iter)->get_Hdiff(Di->second->get_HX2(), SWSF_HX2);   // sum of differences
     		      valuelist2.push_back((*Di).second->get_HX2());
    		    }
    	    }
    	}

      else if ((*iter)->get_type() == 'C')
    	{
    	  if (NOESY_Assign::isV()||NOESY_Assign::isQ()) outputVal << endl << "-->" << (*iter) << endl;
    	  AtomPeak::iterator Cl = CPeak.lower_bound(*iter);
    	  AtomPeak::iterator Cu = CPeak.upper_bound(*iter);

    	  //--- select one as the local referencing
    	  if (Cl != Cu)
    	    {
    	      if (NOESY_Assign::isV()||NOESY_Assign::isQ()) outputVal << "as C: " << endl;
    	      for(AtomPeak::iterator Ci = Cl; Ci != Cu; Ci++)
    	       if (NOESY_Assign::isV()||NOESY_Assign::isQ()) outputVal << "     " << (*Ci).second << endl;
    	      for(AtomPeak::iterator Ci = Cl; Ci != Cu; Ci++)
    		    {
              sumX += (*iter)->get_XdiffAb(Ci->second->get_X1(), SWSF_X1);   // sum of differences
              sumXab += (*iter)->get_Xdiff(Ci->second->get_X1(), SWSF_X1);   // sum of differences
    		      valuelist.push_back((*Ci).second->get_X1());
    		    }
    	    }

    	  if (Type() == "4DCC")
    	  {
       	  AtomPeak::iterator Cl = CPeak2.lower_bound(*iter);
       	  AtomPeak::iterator Cu = CPeak2.upper_bound(*iter);

       	  //--- select one as the local referencing
       	  if (Cl != Cu)
       	    {
       	      if (NOESY_Assign::isV()||NOESY_Assign::isQ()) outputVal << "as C2: " << endl;
       	      for(AtomPeak::iterator Ci = Cl; Ci != Cu; Ci++)
       	       if (NOESY_Assign::isV()||NOESY_Assign::isQ()) outputVal << "     " << (*Ci).second << endl;
       	               //  << "         " << (*(COSYpeakmomdad.find((*Ci).second))).second.second << endl;
       	      for(AtomPeak::iterator Ci = Cl; Ci != Cu; Ci++)
       		    {
                sumX2 += (*iter)->get_XdiffAb(Ci->second->get_X2(), SWSF_X2);   // sum of differences
                sumXab2 += (*iter)->get_Xdiff(Ci->second->get_X2(), SWSF_X2);   // sum of differences
       		      valuelist2.push_back((*Ci).second->get_X2());
       		    }
       	    }
    	  }
    	}

      else if ((*iter)->get_type() == 'N')
     	{
     	  if (NOESY_Assign::isV()||NOESY_Assign::isQ()) outputVal << endl << "-->" << (*iter) << endl;
     	  AtomPeak::iterator Cl = CPeak.lower_bound(*iter);
     	  AtomPeak::iterator Cu = CPeak.upper_bound(*iter);

     	  //--- select one as the local referencing
     	  if (Cl != Cu)
     	    {
       		if (NOESY_Assign::isV()||NOESY_Assign::isQ()) outputVal << "as N: " << endl;
     	    for(AtomPeak::iterator Ci = Cl; Ci != Cu; Ci++)
     	    	if (NOESY_Assign::isV()||NOESY_Assign::isQ()) outputVal << "     " << (*Ci).second << endl; // << (*(COSYpeakmomdad.find((*Ci).second))).second.second << endl;
     	      for(AtomPeak::iterator Ci = Cl; Ci != Cu; Ci++)
     		{
          sumX += (*iter)->get_XdiffAb(Ci->second->get_X1(), SWSF_X1);   // sum of differences
          sumXab += (*iter)->get_Xdiff(Ci->second->get_X1(), SWSF_X1);   // sum of differences
     		  valuelist.push_back((*Ci).second->get_X1());
     		}
     	    }
     	}

      XN = valuelist.size();
      if (XN > 0)
    	{
    	  X   = sumX / (float)(XN);
        Xab = sumXab / (float)(XN);
    	  if ((*iter)->get_type() == 'H')
    	    {
            diffHX += X;
            diffHXab += Xab;
    	      if (NOESY_Assign::isV()||NOESY_Assign::isQ()) outputVal << "From peak list, the HX1 diff is " << X << endl; //(*iter)->get_HdiffAb(X, SWSF_HX1) << endl;

    	      totalHX++;
    	    }
    	  if ((*iter)->get_type() != 'H')
    	    {
            diffX += X;
            diffXab += Xab;
    	      if (NOESY_Assign::isV()||NOESY_Assign::isQ()) outputVal << "From peak list, the C diff is " << X << endl; //(*iter)->get_XdiffAb(X, SWSF_X1) << endl;
    	      totalX++;
    	    }
    	}
      XN2 = valuelist2.size();
      if (XN2 > 0)
    	{
    	  X2   = sumX2 / (float)(XN2);
        X2ab = sumXab2/ (float)(XN2);
    	  if ((*iter)->get_type() == 'H')
    	    {
            diffHX2 += X2;
            diffHXab2 += X2ab;
    	      if (NOESY_Assign::isV()||NOESY_Assign::isQ()) outputVal << "From peak list, the HX2 diff is " << X2 << endl; //<< (*iter)->get_HdiffAb(X2, SWSF_HX2) << endl;
    	      totalHX2++;
    	    }
    	  if ((*iter)->get_type() != 'H')
    	    {
            diffX2 += X;
            diffXab2 += Xab;
    	      if (NOESY_Assign::isV()||NOESY_Assign::isQ()) outputVal << "From peak list, the C2 diff is " << X2 << endl; //(*iter)->get_XdiffAb(X2, SWSF_X2) << endl;
    	      totalX2++;
    	    }
    	}
    }

  summaryOutput.precision(2);
  summaryOutput << endl << "# Summary for ";
  summaryOutput << label() << endl << "Total simulated peaks: " << SIMUCOSYassignList.size() << " ";
  summaryOutput << "Number of peaks NOT matched in the Peak List: " << total << endl << "M score = "
  << total/((float)SIMUCOSYassignList.size()+0.0001) << endl;

  summaryOutput << endl << "Average shift in HX: " << diffHX/(float)totalHX << " Average ab-shift in HX: " << diffHXab/(float)totalHX << "(Assignment used: " << totalHX << ")" << endl;

if (Type() != "2D")
  summaryOutput << "Average shift in X: " << diffX/(float)totalX << " Average ab-shift in X: " << diffXab/(float)totalX << "(Assignment used: " << totalX << ")" << endl;
  summaryOutput << "Average shift in HX2: " << diffHX2/(float)totalHX2 << " Average ab-shift in HX2: " << diffHXab2/(float)totalHX2 << "(Assignment used: " << totalHX2 << ")" << endl;

  if (Type() == "4DCC")
  {
  summaryOutput << "Average shift in X2: " << diffX2/(float)totalX2 << " Average ab-shift in X2: " << diffXab2/(float)totalX2 << "(Assignment used: " << totalX2 << ")" << endl;
  }

  summaryOutput.precision(6);

  return make_pair(total, SIMUCOSYassignList.size());
}

////////////////////////////////////////
// Rfactor                            //
//   call I and L factor functions    //
////////////////////////////////////////

//summaryOutput - overall summary for Pscore
//others report for each cycle
void NOESY::Rfactor(deque<Pdb>& pdbList,ostream& summaryOutput, ostream& outputI, ostream& outputL, deque<Qscore>& dq)
{
	summaryOutput << endl << "#For peaklist " << label() << ":" << endl;
	summaryOutput << " Rmax: " << get_RmaxQ() << endl;
	deque<ASDP::ratio> Iscore(pdbList.size()+1);  deque<ASDP::ratio> Iwscore(pdbList.size()+1);
	deque<ASDP::ratio> Lall(pdbList.size()+1); deque<ASDP::ratio> Lallw(pdbList.size()+1);
	deque<ASDP::ratio> Lideal(pdbList.size()+1); deque<ASDP::ratio> Lidealw(pdbList.size()+1);
	int TP = 0, FP = 0;

	bool noLong = 0;

  //summaryOutput <<"before IFactor" << endl;
	//tms timeLockS; clock_t startT = times(&timeLockS);
	IFactor(pdbList, summaryOutput, outputI, Iscore, Iwscore, noLong);

//summaryOutput << "after IFactor" << endl;

  LFactor(pdbList, summaryOutput, outputL, Lall, Lallw, Lideal, Lidealw, noLong);

  //summaryOutput << "after LFactor" << endl;
	//tms timeLockE; clock_t endT = times(&timeLockE); // endT = clock();

	//runSummary(summaryOutput, startT, endT, timeLockS, timeLockE);

	for(int i=0; i<= pdbList.size(); i++)
	  {
	    float Iscoreratio = Iscore[i].first/(float)Iscore[i].second;
	    float Iwscoreratio = Iwscore[i].first/(float)Iwscore[i].second;

	    float Lallratio = Lall[i].first/(float)Lall[i].second;
	    float Lallwratio = Lallw[i].first/(float)Lallw[i].second;

	    float Lidealratio = Lideal[i].first/(float)Lideal[i].second;
	    float Lidealwratio = Lidealw[i].first/(float)Lidealw[i].second;

	    if (!NOESY_Assign::isS())
	      {
		summaryOutput << endl;
		if (i==0)
		  summaryOutput << "Model ave" << endl;
		else
		  summaryOutput << "Model " << i << endl;

		summaryOutput << " Recall: " << 1-Iscoreratio << endl; //" " << 1-Iwscoreratio << "(w)" << endl;
		summaryOutput << " Precision: " << endl;
		summaryOutput << "       all: " << 1-Lallratio << " " << " allw: " << 1-Lallwratio << endl;
		summaryOutput << "        <3: " << 1-Lidealratio << " " << " <3w: " << 1-Lidealwratio << endl;

		ASDP::ratio Iideal = make_pair(0,1);
		summaryOutput << endl << " F-measure:(Recall,Precision)  " << Ffactor(Iscore[i], Lall[i], 1) << endl;
		summaryOutput << " F-measure:(Recall,PrecisionW) " << Ffactor(Iscore[i], Lallw[i], 1) << endl << endl;
		summaryOutput << " F-measure:(0,PrecisionIdeal)  " << Ffactor(Iideal, Lideal[i], 1) << endl;
		summaryOutput << " F-measure:(0,PrecisionIdealW) " << Ffactor(Iideal, Lidealw[i], 1) << endl;

		if (DEBUG)
		  {
		    summaryOutput << " F-measure:(RecallW,Precision) " << Ffactor(Iwscore[i], Lall[i], 1) << endl;
		    summaryOutput << " F-measure:(RecallW,PrecisionW)" << Ffactor(Iwscore[i], Lallw[i], 1) << endl;
		  }
	      }
	  }

	    // random structure
	    summaryOutput << endl << "For freely-rotating-chain model:" << endl;
      deque<Pdb> pdbListR; pdbListR.clear();

	    deque<ASDP::ratio> IscoreR(pdbListR.size()+1);
	    deque<ASDP::ratio> IwscoreR(pdbListR.size()+1);
	    deque<ASDP::ratio> LallR(pdbListR.size()+1);
	    deque<ASDP::ratio> LallwR(pdbListR.size()+1);
	    deque<ASDP::ratio> LlngR(pdbListR.size()+1);
	    deque<ASDP::ratio> LlngwR(pdbListR.size()+1);

	    //int TPR = 0, FPR = 0;


	    ostringstream dummuy1;
	    ostringstream dummuy2;

	    //startT = times(&timeLockS);

      //summaryOutput <<"before IFactor" << endl;

	    IFactor(pdbListR, summaryOutput, dummuy2, IscoreR, IwscoreR, noLong);

      //summaryOutput <<"after IFactor" << endl;

	    LFactor(pdbListR, summaryOutput, dummuy2, LallR, LallwR, LlngR, LlngwR, noLong);

      //summaryOutput <<"after LFactor" << endl;

	    //endT = times(&timeLockE); // endT = clock();
	    //runSummary(summaryOutput, startT, endT, timeLockS, timeLockE);

	for(int i=0; i<= pdbListR.size(); i++)
	  {

	    float IscoreRratio = IscoreR[i].first/(float)IscoreR[i].second;
	    float IwscoreRratio = IwscoreR[i].first/(float)IwscoreR[i].second;

	    float LallRratio = LallR[i].first/(float)LallR[i].second;
	    float LallwRratio = LallwR[i].first/(float)LallwR[i].second;

	    if (!NOESY_Assign::isS())
	      {
		summaryOutput << endl << " Recall: " << 1-IscoreRratio << endl;
		summaryOutput << " Precision: " << endl;
		summaryOutput << "       all: " << 1-LallRratio << " " << " allw: " << 1-LallwRratio << endl;

		summaryOutput << endl << " F-measure:(RecallFree,PrecisionFree)  " << Ffactor(IscoreR[i], LallR[i], 1) << endl;
		summaryOutput << " F-measure:(ReallFree,PrecisionFreeW) " << Ffactor(IscoreR[i], LallwR[i], 1) << endl;

		if (DEBUG)
		  {
		    summaryOutput << " F-measure:(FNfreew,FPfreeall) " << Ffactor(IwscoreR[i], LallR[i], 1) << endl;
		    summaryOutput << " F-measure:(FNfreew,FPfreeallw)" << Ffactor(IwscoreR[i], LallwR[i], 1) << endl;
		  }
	      }
	  }
	    // final report
	for(int i=0; i<= pdbList.size(); i++)
	  {
	    Qscore q; q.reset();
	q.FN = Iscore[i];
	q.FP = Lallw[i];
	q.FNR = IscoreR[0];
	q.FPR = LallwR[0];

	q.FNI = make_pair(0,1);
	q.FPI = Lidealw[i];
/*
	//max of precison of Lidealw, Lallw and LallwR
	float LallwRratio = LallwR[0].first/(float)LallwR[0].second;
	float Lidealwratio = Lidealw[i].first/(float)Lidealw[i].second;
	float Lallwratio = Lallw[i].first/(float)Lallw[i].second;

	//summaryOutput << "allwR" << LallwRratio << "ideal " << Lidealwratio << "allw " << Lallwratio << endl;
	ratio minL;

	if ((Lidealwratio < LallwRratio) && (Lidealwratio < Lallwratio))
	{
	   minL = Lidealw[i];
	} else if ((LallwRratio < Lidealwratio) && (LallwRratio < Lallwratio))
	{
	   minL = LallwR[0];
	} else
	{
	   minL = Lallw[i];
	}

	//summaryOutput << "minL " << minL.first/(float)minL.second;

	q.FPI = minL;

	//q.FPI = max(Lidealw[i], max(Lallw[i], LallwR[0]));
*/
	dq[i] += q;
	  }

}

Qscore NOESY::Rfactor(Pdb& pdb,ostream& summaryOutput, ostream& outputI, ostream& outputL, Qscore& oldScore) const
{
	summaryOutput << endl << "#For peaklist " << label() << ":" << endl;
	summaryOutput << " Rmax: " << get_RmaxQ() << endl;
	//deque<ratio> Iscore; Iscore.clear(); deque<ratio> Iwscore; Iwscore.clear();
	//deque<ratio> Lall; Lall.clear(); deque<ratio> Lallw; Lallw.clear();
	//deque<ratio> Lideal; Lideal.clear(); deque<ratio> Lidealw; Lidealw.clear();
	int TP = 0, FP = 0;

	bool noLong = 0;

	//tms timeLockS; clock_t startT = times(&timeLockS);
	ASDP::ratio Iscore = IFactor(pdb, summaryOutput, outputI, noLong);
	pair<ASDP::ratio, ASDP::ratio> Lscore = LFactor(pdb, summaryOutput, outputL, noLong);

	//tms timeLockE; clock_t endT = times(&timeLockE); // endT = clock();

	//runSummary(summaryOutput, startT, endT, timeLockS, timeLockE);
	/*
	float Iscoreratio = Iscore.first/(float)Iscore.second;
	//float Iwscoreratio = Iwscore[i].first/(float)Iwscore[i].second;

	//float Lallratio = Lall[i].first/(float)Lall[i].second;
	float Lallwratio = Lscore.first.first/(float)Lscore.first.second;

	//float Lidealratio = Lscore.second.first/(float)Lscore.second.second;
	float Lidealwratio = Lscore.second.first/(float)Lscore.second.second;

	    if (!NOESY_Assign::isS())
	      {
		summaryOutput << endl;
		if (i==0)
		  summaryOutput << "Model ave" << endl;
		else
		  summaryOutput << "Model " << i << endl;

		summaryOutput << " Recall: " << 1-Iscoreratio << endl; //" " << 1-Iwscoreratio << "(w)" << endl;
		summaryOutput << " Precision: " << endl;
		summaryOutput << "       all: " << 1-Lallratio << " " << " allw: " << 1-Lallwratio << endl;
		summaryOutput << "        <3: " << 1-Lidealratio << " " << " <3w: " << 1-Lidealwratio << endl;

		ASDP::ratio Iideal = make_pair(0,1);
		//summaryOutput << endl << " F-measure:(Recall,Precision)  " << Ffactor(Iscore[i], Lall[i], 1) << endl;
		summaryOutput << " F-measure:(Recall,PrecisionW) " << Ffactor(Iscore, Lscore.first, 1) << endl << endl;
		//summaryOutput << " F-measure:(0,PrecisionIdeal)  " << Ffactor(Iideal, Lideal[i], 1) << endl;
		summaryOutput << " F-measure:(0,PrecisionIdealW) " << Ffactor(Iideal, Lscore.second, 1) << endl;

		if (DEBUG)
		  {
		    //summaryOutput << " F-measure:(RecallW,Precision) " << Ffactor(Iwscore[i], Lall[i], 1) << endl;
		    //summaryOutput << " F-measure:(RecallW,PrecisionW)" << Ffactor(Iwscore[i], Lallw[i], 1) << endl;
		  }
	      }
	  }
	*/

	    // random structure
	    summaryOutput << endl << "For freely-rotating-chain model:" << endl;
	    //deque<ratio> IscoreR; IscoreR.clear();
	    //deque<ratio> IwscoreR; IwscoreR.clear();
	    //deque<ratio> LallR; LallR.clear();
	    //deque<ratio> LallwR; LallwR.clear();
	    //deque<ratio> LlngR; LlngR.clear();
	    //deque<ratio> LlngwR; LlngwR.clear();

	    //int TPR = 0, FPR = 0;

	    Pdb pdbR; pdbR.reset();
	    //deque<Pdb> pdbListR; pdbListR.clear();

	    ostringstream dummuy1;
	    ostringstream dummuy2;

	    //startT = times(&timeLockS);
	    ASDP::ratio IscoreR = IFactor(pdbR, summaryOutput, dummuy2, noLong);
	    pair<ASDP::ratio, ASDP::ratio> LscoreR = LFactor(pdbR, summaryOutput, dummuy2, noLong);

	    //endT = times(&timeLockE); // endT = clock();
	    //runSummary(summaryOutput, startT, endT, timeLockS, timeLockE);
	    /*
	for(int i=0; i<= pdbListR.size(); i++)
	  {

	    float IscoreRratio = IscoreR[i].first/(float)IscoreR[i].second;
	    float IwscoreRratio = IwscoreR[i].first/(float)IwscoreR[i].second;

	    float LallRratio = LallR[i].first/(float)LallR[i].second;
	    float LallwRratio = LallwR[i].first/(float)LallwR[i].second;

	    if (!NOESY_Assign::isS())
	      {
		summaryOutput << endl << " Recall: " << 1-IscoreRratio << endl;
		summaryOutput << " Precision: " << endl;
		summaryOutput << "       all: " << 1-LallRratio << " " << " allw: " << 1-LallwRratio << endl;

		summaryOutput << endl << " F-measure:(RecallFree,PrecisionFree)  " << Ffactor(IscoreR[i], LallR[i], 1) << endl;
		summaryOutput << " F-measure:(ReallFree,PrecisionFreeW) " << Ffactor(IscoreR[i], LallwR[i], 1) << endl;

		if (DEBUG)
		  {
		    summaryOutput << " F-measure:(FNfreew,FPfreeall) " << Ffactor(IwscoreR[i], LallR[i], 1) << endl;
		    summaryOutput << " F-measure:(FNfreew,FPfreeallw)" << Ffactor(IwscoreR[i], LallwR[i], 1) << endl;
		  }
	      }
	  }
	    */

// final report
Qscore q; q.reset();
q.FN = Iscore;
q.FP = Lscore.first;
q.FNR = IscoreR;
q.FPR = LscoreR.first;

q.FNI = make_pair(0,1);
 q.FPI = Lscore.second; //ideal


 cout << q.FN << " " << q.FP << " " << q.FNR << " " << q.FPR << " " << q.FNI << " " << q.FPI << endl;

 Qscore newScore = oldScore;
 newScore +=q;

return newScore;

}

////////////////////////////////////////////////////////////////////
// return 1-Nci,i/Ntotal, original I factor for each structure		//
//   sumT - total number of peaks                                 //
//   tempSet - momdad set in the peak list                        //
////////////////////////////////////////////////////////////////////

int getBondNum(char a)
{
	if (a == 'A')
		return 1;
	else if (a == 'B')
		return 2;
	else if (a == 'G')
		return 3;
	else if (a == 'D')
		return 4;
	else if (a == 'E')
		return 5;
	else if (a == 'Z')
		return 6;
	else if (a == 'H')
		return 7;
	else
		return 8;
}


// sumAverage
SampleStatistic getDistSumAveStat(deque<Pdb>& pdbList, pair<const atom*, const atom*> ab, pair<const string, const string> abname, bool IC)
{
 	SampleStatistic distStat; distStat.reset();

	if (pdbList.size())
	{
   	for(deque<Pdb>::iterator iterp = pdbList.begin(); iterp != pdbList.end(); iterp++)
    {
  		float dist = NOMATCH;
  	  if (IC)
  	  	dist = (*iterp).get_distIC(NOESY_Assign::outputError(), ab.first->get_index(), ab.first->get_sym_1(), abname.first, ab.first->get_ambi(),
  	      												ab.second->get_index(), ab.second->get_sym_1(), abname.second, ab.second->get_ambi());
  		else
  		 	dist = (*iterp).get_dist(NOESY_Assign::outputError(), ab.first->get_index(), ab.first->get_sym_1(), abname.first, ab.first->get_ambi(),
  	      												ab.second->get_index(), ab.second->get_sym_1(), abname.second, ab.second->get_ambi());
  		if (dist != NOMATCH)
    	{
       	distStat += dist;
      }
    }
	}

	return distStat;
}


// summation
SampleStatistic getDistStat(deque<Pdb>& pdbList, pair<const atom*, const atom*> ab, bool IC)
{

  //cout << *ab.first << *ab.second << endl;
 	SampleStatistic distStat; distStat.reset();

	if (pdbList.size())
	{
   	for(deque<Pdb>::iterator iterp = pdbList.begin(); iterp != pdbList.end(); iterp++)
    {
      //cout << "in getDistStat" << endl;

  		float dist = NOMATCH;
  	  if (IC)
  	  	dist = (*iterp).get_distIC(NOESY_Assign::outputError(), ab.first->get_index(), ab.first->get_sym_1(), ab.first->get_name(), ab.first->get_ambi(),
  	      												ab.second->get_index(), ab.second->get_sym_1(), ab.second->get_name(), ab.second->get_ambi());
  		else
  		 	dist = (*iterp).get_dist(NOESY_Assign::outputError(), ab.first->get_index(), ab.first->get_sym_1(), ab.first->get_name(), ab.first->get_ambi(),
  	      												ab.second->get_index(), ab.second->get_sym_1(), ab.second->get_name(), ab.second->get_ambi());
  		//if (dist != NOMATCH)
      	distStat += dist;

    }
	} else // random structure
	{
		float radii = 1.5;
		int f = 3;
		int n = 0;
		if (ab.first->get_index() != ab.second->get_index())
		{
			if ((ab.first->get_layer() == "N") && (ab.second->get_layer() == "N"))
			{
				n = abs(ab.first->get_index()-ab.second->get_index())*f+2;
			} else if (ab.first->get_layer() == "N")
			{
				// +2 to CB or HA
				n = abs(ab.first->get_index()-ab.second->get_index())*f+2+getBondNum(ab.second->get_layer()[0]);
			} else if (ab.second->get_layer() == "N")
			{
				n = abs(ab.first->get_index()-ab.second->get_index())*f+getBondNum(ab.first->get_layer()[0]);
			} else
			{
				n = abs(ab.first->get_index()-ab.second->get_index())*f+getBondNum(ab.first->get_layer()[0])+getBondNum(ab.second->get_layer()[0]);
			}

		} else
		{
		  if (ab.first == ab.second)
		    n = 0;
		  else if (ab.first->get_layer() == "N")
			{
				n = 2+getBondNum(ab.second->get_layer()[0]);
			} else if (ab.second->get_layer() == "N")
		   {
				n = 2+getBondNum(ab.first->get_layer()[0]);
			} else
			{
				n = abs(getBondNum(ab.first->get_layer()[0])-getBondNum(ab.second->get_layer()[0])) + 2;
			}
		}

		//if (n < 2) n = 2;
		float a=1/3.0;
		float correction = (1+a)/(1-a) - (2*a*(1-pow(a,n))/(n*pow((1-a),2)));
		distStat += pow(n*radii*radii*correction, 1/2.0);

	}


  return distStat;
}

// min = 1.8 ang
float sumAverage(const SampleStatistic& sp)
{
	if (sp.samples().size() == 0)
		return NOMATCH;

	float corDist = 0;
 	for(SmplData::iterator i = sp.samples().begin(); i!= sp.samples().end(); ++i)
 	{
 		corDist += pow(*i, -6.0);
 	}

  corDist = pow(corDist/sp.samples().size(), -1/6.0);
  if (corDist < 1.8)
  	corDist = 1.8;

 	return floor(corDist*100+0.05)/100.0;
}

float summation(const SampleStatistic& sp)
{
	if (sp.samples().size() == 0)
		return NOMATCH;

	float corDist = 0;
 	for(SmplData::iterator i = sp.samples().begin(); i!= sp.samples().end(); ++i)
 	{
 		corDist += pow(*i, -6.0);
 	}

  corDist = pow(corDist, -1/6.0);
  if (corDist < 1.8)
  	corDist = 1.8;

 	return floor(corDist*100+0.05)/100.0;
}

void NOESY::IFactor(deque<Pdb>& pdbList, ostream& summaryOutput, ostream& outputI, deque<ASDP::ratio>& Iscore, deque<ASDP::ratio>& Iwscore, bool noLong)
{
  //summaryOutput << "in IFactor " << label() << endl;

  deque<int> sum1(pdbList.size()+1); //sum1.clear();
  int sumT = 0;
  deque<float> sum1w(pdbList.size()+1); //sum1w.clear();
  float  sumTw=0;

  //summaryOutput << "before for loop " << label() << " size " << pdbList.size()<< endl;

  for(int i=0; i <= pdbList.size(); i++)
    {
      sum1[i]=0; sum1w[i]=0;
      //summaryOutput << "i " << i << " " << sum1[i] << " " << sum1w[i] << endl;
    }

  outputI << "# Inconsistent peaks for " << label() << endl;

  //summaryOutput << "# Inconsistent peaks for " << label() << endl;


  erasePrint(cout, " Doing Recall analysis for " + label() + " ...");

  for(PeakList::const_iterator iter = peakList.begin(); iter != peakList.end(); iter++)
  {
    int countTotal = 0; // total of possible assignments
    int countDINT  = 0; // total of DEL_INT

      MatchPeakAtom::const_iterator il = MombySimpleMatch.lower_bound((&*iter));
      MatchPeakAtom::const_iterator iu = MombySimpleMatch.upper_bound((&*iter));

      MatchPeakAtom::const_iterator dl = DadbySimpleMatch.lower_bound((&*iter));
      MatchPeakAtom::const_iterator du = DadbySimpleMatch.upper_bound((&*iter));

      deque<int> count1(pdbList.size()+1);
      for(int i=0; i <= pdbList.size(); i++)
	{
	  count1[i]=0;
	}

      for(MatchPeakAtom::const_iterator i = il; i != iu; i++)
      {
      	for(MatchPeakAtom::const_iterator d = dl; d != du; d++)
      	{
	  countTotal++;

      	  bool isDINT = 0;
	  PeakAssignReason::const_iterator rl = BufferReason.lower_bound(&(*iter));
	  PeakAssignReason::const_iterator ru = BufferReason.upper_bound(&(*iter));

	  PeakAssignReason::const_iterator ri;
	  for(ri = rl; ri != ru; ri++)
	    {
	      if (((*ri).second.first.first == (*i).second) &&
		  ((*ri).second.first.second == (*d).second) &&
		  ((*ri).second.second.count(DEL_INT) != 0))
		{
		  isDINT = 1;
		  countDINT++;
		  break;
		}
	    }

	  if ((isDINT == 0)) // && (dset.find(d->second->get_index()) != dset.end()))    // within loose-range
	    {
	      pair<atom*, atom*> ab;
	      if (*(i->second) < *(d->second))
		ab = make_pair(i->second, d->second);
	      else
		ab = make_pair(d->second, i->second);

	      if (!noLong || (abs(ab.first->get_index()-ab.second->get_index()) <= 4)) // not count for noLong model
		{
		  SampleStatistic distStat = getDistStat(pdbList, ab, 0);

		  for(int i=0; i<= pdbList.size(); i++)
		    {
		      float dist = 100;

		      if (distStat.count() ==0)
			dist = 100000;
		      else
		      {
		      	if (i==0)
				dist = distStat.median();
		      	else
				dist = (distStat.oriSamples())[i-1];
		      }

		      if (dist < 1.0) dist = 1.0;

		      if (dist < get_RmaxQ())
			{
			  count1[i]++;
			  if (DEBUG)
			    outputI << "ok " << (*iter) << " " << dist << " " << get_RmaxQ() << endl;
			}
		    }
		} // noLong
	    } // isDINT
      	} // for
      }

      if (DEBUG)
	outputI << "after ok" << endl;

      if (countTotal - countDINT > 0) // for folded peak
    	{
    	    sumT++; // total number of peaks
	    sumTw += fabs((*iter).get_Intensity());

	    for(int i=0; i<= pdbList.size(); i++)
	      {
		if (count1[i] ==0) // inconsstent
		  {
		  	if (i ==0)
		    outputI << "->" << " " << (*iter) << endl;

		    sum1[i]++; // total number of peaks that inconsistent
		    sum1w[i] += fabs((*iter).get_Intensity());
		  }
	      }
    	}
  }

  summaryOutput.precision(3);

  for(int i=0; i<= pdbList.size(); i++)
    {
      Iscore[i] = make_pair(sum1[i], sumT);
      Iwscore[i] = make_pair(sum1w[i], sumTw);

      if (!NOESY_Assign::isS())
	{
	  if (i==0)
	    summaryOutput << "Model ave";
	  else
	    summaryOutput << "Model " << setw(3) << i;
	  summaryOutput << " Inconsistent score: " << Iscore[i].first/(float)Iscore[i].second << "(" << sum1[i] << "/" << sumT << ") " << endl; // inconsistent
	  //cout << " Inconsistent score: " << Iscore[i].first/(float)Iscore[i].second << "(" << sum1[i] << "/" << sumT << ") " << endl; // inconsistent
	}
    }

}

ASDP::ratio NOESY::IFactor(Pdb& pdb, ostream& summaryOutput, ostream& outputI, bool noLong) const
{

  //cout << "in IFactor " << pdb << endl;

  deque<Pdb> pdbList; pdbList.clear();
  if (pdb.notEmpty())
    {
      pdbList.push_back(pdb);
    }

  deque<ASDP::ratio> Iscore;
  deque<ASDP::ratio> Iwscore;

  deque<int> sum1(pdbList.size()+1);
  int sumT = 0;

  for(int i=0; i <= pdbList.size(); i++)
    {
      sum1[i]=0;
    }

  outputI << "# Inconsistent peaks for " << label() << endl;
  erasePrint(cout, " Doing Recall analysis for " + label() + " ...");

  for(PeakList::const_iterator iter = peakList.begin(); iter != peakList.end(); iter++)
  {
    int countTotal = 0; // total of possible assignments
    int countDINT  = 0; // total of DEL_INT

      MatchPeakAtom::const_iterator il = MombySimpleMatch.lower_bound((&*iter));
      MatchPeakAtom::const_iterator iu = MombySimpleMatch.upper_bound((&*iter));

      MatchPeakAtom::const_iterator dl = DadbySimpleMatch.lower_bound((&*iter));
      MatchPeakAtom::const_iterator du = DadbySimpleMatch.upper_bound((&*iter));

      deque<int> count1(pdbList.size()+1);
      for(int i=0; i <= pdbList.size(); i++)
	{
	  count1[i]=0;
	}

      for(MatchPeakAtom::const_iterator i = il; i != iu; i++)
      {
      	for(MatchPeakAtom::const_iterator d = dl; d != du; d++)
      	{
	  countTotal++;

      	  bool isDINT = 0;
	  PeakAssignReason::const_iterator rl = BufferReason.lower_bound(&(*iter));
	  PeakAssignReason::const_iterator ru = BufferReason.upper_bound(&(*iter));

	  PeakAssignReason::const_iterator ri;
	  for(ri = rl; ri != ru; ri++)
	    {
	      if (((*ri).second.first.first == (*i).second) &&
		  ((*ri).second.first.second == (*d).second) &&
		  ((*ri).second.second.count(DEL_INT) != 0))
		{
		  isDINT = 1;
		  countDINT++;
		  break;
		}
	    }

	  if ((isDINT == 0)) // && (dset.find(d->second->get_index()) != dset.end()))    // within loose-range
	    {
	      pair<atom*, atom*> ab;
	      if (*(i->second) < *(d->second))
		ab = make_pair(i->second, d->second);
	      else
		ab = make_pair(d->second, i->second);

	      if (!noLong || (abs(ab.first->get_index()-ab.second->get_index()) <= 4)) // not count for noLong model
		{
		  SampleStatistic distStat = getDistStat(pdbList, ab, 0);

		  for(int i=0; i<= pdbList.size(); i++)
		    {
		      float dist = 100;

		      if (distStat.count() ==0)
			dist = 100000;
		      else
		      {
		      	if (i==0)
				dist = distStat.median();
		      	else
				dist = (distStat.oriSamples())[i-1];
		      }

		      if (dist < 1.0) dist = 1.0;

		      if (dist < get_RmaxQ())
			{
			  count1[i]++;
			  if (DEBUG)
			    outputI << "ok " << (*iter) << " " << dist << " " << get_RmaxQ() << endl;
			}
		    }
		} // noLong
	    } // isDINT
      	} // for
      }

      if (DEBUG)
	outputI << "after ok" << endl;

      if (countTotal - countDINT > 0) // for folded peak
    	{
    	    sumT++; // total number of peaks

	    for(int i=0; i<= pdbList.size(); i++)
	      {
		if (count1[i] ==0) // inconsstent
		  {
		  	if (i ==0)
		    outputI << "->" << " " << (*iter) << endl;

		    sum1[i]++; // total number of peaks that inconsistent
		  }
	      }
    	}
  }

  summaryOutput.precision(3);

  for(int i=0; i<= pdbList.size(); i++)
    {
      Iscore[i] = make_pair(sum1[i], sumT);

      if (!NOESY_Assign::isS())
	{
	  if (i==0)
	    summaryOutput << "Model ave";
	  else
	    summaryOutput << "Model " << setw(3) << i;
	  summaryOutput << " Inconsistent score: " << Iscore[i].first/(float)Iscore[i].second << "(" << sum1[i] << "/" << sumT << ") " << endl; // inconsistent
	  cout << " Inconsistent score: " << i << " " << Iscore[i].first/(float)Iscore[i].second << "(" << sum1[i] << "/" << sumT << ") " << endl; // inconsistent
	}

      //cout << "Iscore " << Iscore[i] << endl;
    }

  return Iscore[0];
}

////////////////////////////////////////////////////////////////////
// return 1-Nlf,k/Nlp,k, original L factor for each structure	  //
//   tempSet - momdad in the peak list                            //
//   if simulated one is not found in tempSet, count              //
////////////////////////////////////////////////////////////////////


void get_range(deque<Pdb>& pdbList, const atom* a, Atom_Index_List& atommap, deque<atom*>& al)
{
  if (pdbList.size())
  {
    set<int, less<int> > pa;

   	for(deque<Pdb>::iterator iterp = pdbList.begin(); iterp != pdbList.end(); iterp++)
    {
      iterp->get_range(NOESY_Assign::outputError(), a->get_index(), a->get_sym_1(), a->get_name(), a->get_ambi(), 5.100001, pa);
    }

    for(set<int, less<int> >::iterator pai = pa.begin(); pai != pa.end(); pai++)
    {
        Atom_Index_List::iterator ial = atommap.lower_bound(*pai);
        Atom_Index_List::iterator iau = atommap.upper_bound(*pai);

        for(Atom_Index_List::iterator ia = ial; ia != iau; ia++)
        {
           al.push_back(ia->second);
        }

    }
  } else // random structures - return all intra and sequential atoms
  {
    for(int ind = -2; ind < 3; ind++)
      for(Atom_Index_List::iterator i = atommap.lower_bound(a->get_index()-ind); i != atommap.upper_bound(a->get_index()-ind); i++)
      {
          if (i != atommap.end())
            al.push_back(i->second);
      }
  }
}

bool isNAB(string layer)
{
  if ((layer == "N") || (layer == "A") || (layer == "B"))
    return 1;
  return 0;
}

// precision
void NOESY::LFactor(deque<Pdb>& pdbList,
ostream& summaryOutput, ostream& outputL,
deque<ASDP::ratio>& LallScore, deque<ASDP::ratio>& LallScorew, deque<ASDP::ratio>& LlongScore, deque<ASDP::ratio>& LlongScorew, bool noLong)
{
  erasePrint(cout, " Doing Precision analysis for " + label() + " ...");

  // interactions that shown in the peak list
  cMomDadSet tempSet; tempSet.clear();
  for(PeakList::const_iterator iter = peakList.begin(); iter != peakList.end(); iter++)
  {
      MatchPeakAtom::const_iterator il = MombySimpleMatch.lower_bound((&*iter));
      MatchPeakAtom::const_iterator iu = MombySimpleMatch.upper_bound((&*iter));

      MatchPeakAtom::const_iterator dl = DadbySimpleMatch.lower_bound((&*iter));
      MatchPeakAtom::const_iterator du = DadbySimpleMatch.upper_bound((&*iter));

      for(MatchPeakAtom::const_iterator i = il; i != iu; i++)
      	for(MatchPeakAtom::const_iterator d = dl; d != du; d++)
      	{
	           tempSet.insert(MomDad((*i).second, (*d).second)); // ordered by momdad
	      }
  }

  // initilize the atomllistmap
  Atom_Index_List atommap; atommap.clear();
  for(Atom_List::iterator iatom = NOESY_Assign::atoms().begin(); iatom != NOESY_Assign::atoms().end(); iatom++)
  {
    atommap.insert(make_pair(iatom->get_index(), &(*iatom)));
  }


  deque<int> tAllR(pdbList.size()+1);  // = 0, tLngR = 0; // total consistent interactions

	// weighted
	deque<float> tAllw(pdbList.size()+1);  // = 0, tLngw=0; // total interactions
	deque<float> tAllRw(pdbList.size()+1); // = 0, tLngRw = 0;

	// consistent with structure, but not mapped
	deque<float> all(pdbList.size()+1);  // = 0, lng = 0;
	deque<float> allw(pdbList.size()+1); // = 0, lngw = 0;

  deque<disMomDadSetStat> HHList; HHList.clear();

  for(int i=0; i <= pdbList.size(); i++)
    {
      tAllR[i] = 0;
      tAllw[i] =0;
      tAllRw[i]=0;
      all[i]=0;
      allw[i]=0;

      disMomDadSetStat HH; HH.clear();
      HHList.push_back(HH);
    }

  for(Atom_List::const_iterator miter = NOESY_Assign::atoms().begin(); miter != NOESY_Assign::atoms().end(); ++miter)
    {
      if ((*miter).isH()) // && ((*miter).isatt_C() || (*miter).isatt_N()))
    	{

          deque<atom*> drange; drange.clear();
          get_range(pdbList, &*miter, atommap, drange);

	  if (DEBUG)
	    NOESY_Assign::outputError() << "LFactor miter -->" << *miter << endl;
        //////  end range search
        for(deque<atom*>::const_iterator diter = drange.begin(); diter != drange.end(); diter++)
    	    {
    	      //diter++;

	      if (*(*diter) <  *miter) continue;
	      if (*(*diter) == *miter) continue;

	    if (DEBUG)
              NOESY_Assign::outputError() << *diter << " " << endl;

						if ((*diter)->isH() &&
							 	((!noLong) || (abs((*diter)->get_index()-miter->get_index()) <=4)) &&  // noLong or all  noLong=0 always
						    ((canbeMom(&*miter) && canbeDad(*diter)) || (canbeMom(*diter) && canbeDad(&*miter))) // check if canbe mom or dad
           				&& (fabs((*miter).get_shift() - (*diter)->get_shift()) > 0.3999999)) // not close to DIA
        		{
 								cMomDad ab = make_pair(&*miter, *diter);
	            	//float dist = sumAverage(getDistStat(pdbList, ab, isIC()));
	    SampleStatistic distStat = getDistStat(pdbList, ab, isIC());


	    for(int i=0; i <= pdbList.size(); i++)
	      {
		float dist;

		if (distStat.count() == 0)
		  dist = 1000000;
		else
		{
			if (i ==0)
				dist = distStat.median(); // the default
			else
		  		dist = (distStat.oriSamples())[i-1];
		}

		if (dist < 1.0)
    {
      //NOESY_Assign::outputError() << dist << " dist<1.0" << endl;
		  dist = 1.0;
    }
    //NOESY_Assign::outputError() << dist << " final dist" << endl;


		if (dist < get_RmaxQ())
		  {
		    if (HHList[i].find(make_pair(miter->get_repAtom(),(*diter)->get_repAtom())) != HHList[i].end()) // find an entry
		      {
			disMomDadSetStat::iterator found = HHList[i].find(make_pair(miter->get_repAtom(),(*diter)->get_repAtom()));
			found->second.first.insert(ab);
			if (dist < found->second.second)
			  found->second.second = dist;
		      }
		    else // new entry
		      {
			cMomDadSet temp; temp.clear(); temp.insert(ab);
			HHList[i].insert(make_pair(make_pair(miter->get_repAtom(),(*diter)->get_repAtom()),
						   make_pair(temp, dist)));
		      }
		  } // end if dist
	      }
	      }// end if can be mom and data
    	    } // end while
    	} // end if is H
    } // end for

  if (!NOESY_Assign::isS())
    outputL << endl << "# Missing interactons for " << label() << endl;

  deque<int> tideal(pdbList.size()+1);  deque<int> ideal(pdbList.size()+1);
  deque<float> tidealw(pdbList.size()+1);  deque<float> idealw(pdbList.size()+1);
  for(int i=0; i <= pdbList.size(); i++)
    {
      tideal[i] =0;
      ideal[i]=0;
      tidealw[i]=0;
      idealw[i]=0;
    }

  for(int i = 0; i <= pdbList.size(); i++)
    for(disMomDadSetStat::const_iterator mdi = HHList[i].begin(); mdi != HHList[i].end(); mdi++)
    {

      float weight = isNAB(mdi->first.first->get_layer())*isNAB(mdi->first.second->get_layer())+1;

    	{
    	  //total++;
	  int diff = abs(mdi->first.first->get_index() - mdi->first.second->get_index());

        //if (mdi->second.second < get_RmaxQ()) // expected < Rmax
        {
	   cMomDadSet::const_iterator ii;

	   // one side

if (canbeMom(mdi->first.first) && canbeDad(mdi->first.second) && !nearH2o(mdi->first.first))// check if canbe mom or dad
  {

    //outputL << endl << mdi->first.first << getWL() << getWU() << endl;

    // for ideal-structure
    if ((mdi->first.first->isMeaningless(mdi->first.second)) || (mdi->first.first->length3(mdi->first.second))

	       // methyl
	       || (mdi->first.first->MethylFiveBonds(mdi->first.second))

	       // Leu
	       || ((mdi->first.first->get_sym_1() == 'L') && (mdi->first.first->get_layer() == "N")
		&& ((mdi->first.second->get_layer() == "G") || (mdi->first.second->get_layer() == "D")))
	       || ((mdi->first.first->get_sym_1() == 'L') && (mdi->first.second->get_layer() == "N")
		&& ((mdi->first.first->get_layer() == "G") || (mdi->first.first->get_layer() == "D")))

	       || ((mdi->first.first->get_sym_1() == 'L') && (mdi->first.second->get_layer() == "D")
		   && ((mdi->first.first->get_layer() == "D")))
	       || ((mdi->first.first->get_sym_1() == 'V') && (mdi->first.second->get_layer() == "G")
		   && ((mdi->first.first->get_layer() == "G")))

	       // ILE
	       || ((mdi->first.first->get_sym_1() == 'I') && (mdi->first.first->get_layer() == "N")
		&& ((mdi->first.second->get_layer() == "D")))
	       || ((mdi->first.first->get_sym_1() == 'I') && (mdi->first.second->get_layer() == "N")
		   && ((mdi->first.first->get_layer() == "D"))))

      	   {
      		   tideal[i]++;
      		   tidealw[i] += pow(mdi->second.second*0.1, -6.0)*weight;

      	   }
	   				// for query structure
           ++tAllR[i];
           tAllRw[i] += pow(mdi->second.second*0.1, -6.0)*weight;

 	   for(ii = (*mdi).second.first.begin(); ii != (*mdi).second.first.end(); ii++)
	     {
	       if ((tempSet.find((*ii)) != tempSet.end())) // || (tempSet.find(make_pair((*ii).second, (*ii).first)) != tempSet.end()))
		 break;
	     }

	   if (ii == (*mdi).second.first.end()) // not in the peak list
	     {
	       if (i ==0) // only report ave fps
		{

		  if (!NOESY_Assign::isS())
		    {
		      for(cMomDadSet::iterator ij = (*mdi).second.first.begin(); ij != (*mdi).second.first.end(); ij++)
			{
			  outputL << endl << (*ij).first << endl << (*ij).second << endl << "::";
			}
		      outputL << "sumAvg: " << mdi->second.second << endl;
		    }
		}

		  // not in the peak list for ideal structure
		  if ((mdi->first.first->isMeaningless(mdi->first.second)) || (mdi->first.first->length3(mdi->first.second)))
		    {
		      ideal[i]++;
		      idealw[i] += pow(mdi->second.second*0.1, -6.0)*weight;
		    }

	       // no maped for query structure
		  all[i]++;
		  allw[i] += pow(mdi->second.second*0.1, -6.0)*weight;
             }
  }

if (canbeMom(mdi->first.second) && canbeDad(mdi->first.first) && !nearH2o(mdi->first.second))// check if canbe mom or dad
  {

    if ((mdi->first.first->isMeaningless(mdi->first.second)) || (mdi->first.first->length3(mdi->first.second))
	       // methyl
	       || (mdi->first.first->MethylFiveBonds(mdi->first.second))

	       // Leu
	       || ((mdi->first.first->get_sym_1() == 'L') && (mdi->first.first->get_layer() == "N")
		&& ((mdi->first.second->get_layer() == "G") || (mdi->first.second->get_layer() == "D")))
	       || ((mdi->first.first->get_sym_1() == 'L') && (mdi->first.second->get_layer() == "N")
		&& ((mdi->first.first->get_layer() == "G") || (mdi->first.first->get_layer() == "D")))

	       || ((mdi->first.first->get_sym_1() == 'L') && (mdi->first.second->get_layer() == "D")
		   && ((mdi->first.first->get_layer() == "D")))
	       || ((mdi->first.first->get_sym_1() == 'V') && (mdi->first.second->get_layer() == "G")
		   && ((mdi->first.first->get_layer() == "G")))

	       // ILE
	       || ((mdi->first.first->get_sym_1() == 'I') && (mdi->first.first->get_layer() == "N")
		&& ((mdi->first.second->get_layer() == "D")))
	       || ((mdi->first.first->get_sym_1() == 'I') && (mdi->first.second->get_layer() == "N")
		   && ((mdi->first.first->get_layer() == "D"))))
      	   {
      		   tideal[i]++;
      		   tidealw[i] += pow(mdi->second.second*0.1, -6.0)*weight;

      	   }   				// for query structure
           ++tAllR[i];
           tAllRw[i] += pow(mdi->second.second*0.1, -6.0)*weight;

	   // symmetric side
	   for(ii = (*mdi).second.first.begin(); ii != (*mdi).second.first.end(); ii++)
	     {
	       if (tempSet.find(make_pair((*ii).second, (*ii).first)) != tempSet.end())
		 break;
	     }

	   if (ii == (*mdi).second.first.end()) // not in the peak list
	     {
	       if (i ==0) // only report ave fps
		{

		  if (!NOESY_Assign::isS())
		    {
		      for(cMomDadSet::iterator ij = (*mdi).second.first.begin(); ij != (*mdi).second.first.end(); ij++)
			{
			  outputL << endl << (*ij).second << endl << (*ij).first << endl << "::";
			}
		      outputL << "sumAvg: " << mdi->second.second << endl;
		    }
		}

	       // not in the peak list for ideal structure
	       if ((mdi->first.first->isMeaningless(mdi->first.second)) || (mdi->first.first->length3(mdi->first.second)))
		 {
		   ideal[i]++;
		   idealw[i] += pow(mdi->second.second*0.1, -6.0)*weight;
		 }

	       // no maped for query structure
	       all[i]++;
	       allw[i] += pow(mdi->second.second*0.1, -6.0)*weight;

         //outputL << "debug i " << i << " " << mdi->second.second << " weight" << weight << endl;

         // if (!NOESY_Assign::isS())
         // {
         //     for(cMomDadSet::iterator ij = (*mdi).second.first.begin(); ij != (*mdi).second.first.end(); ij++)
         //     {
         //       outputL << (*ij).second << endl << (*ij).first << endl << "::";
         //     }
         //     outputL << "sumAvg: " << mdi->second.second << endl;
         //  }

             }
  }
	}
    	}
    }

  for(int i =0; i<= pdbList.size(); i++)
    {
      if (!NOESY_Assign::isS())
	{
	  if (i==0)
	    summaryOutput << "Model ave";
	  else
	    summaryOutput << "Model " << setw(3) << i;

	  summaryOutput << setw(3) << "  Missing interactions: " << all[i]/((float)tAllR[i]+0.00001) << " (" << all[i] << "/" << tAllR[i] << ")" << endl
			<< "              weighted: " << allw[i]/(tAllRw[i]+0.00001) << " (" << allw[i] << "/" << tAllRw[i] << ")" << endl
			<< "                    <3: " << ideal[i]/((float)tideal[i]+0.00001) << " (" << ideal[i] << "/" << tideal[i] << ")" << endl
							  << "                   <3w: " << idealw[i]/((float)tidealw[i]+0.00001) << " (" << idealw[i] << "/" << tidealw[i] << ")" << endl;
	}

      LallScore[i] = make_pair(all[i], tAllR[i]+0.00001);
      LallScorew[i] = make_pair(allw[i], tAllRw[i]+0.00001);

      LlongScore[i] = make_pair(ideal[i], tideal[i]+0.00001);
      LlongScorew[i] = make_pair(idealw[i], tidealw[i]+0.00001);
    }
}


pair<ASDP::ratio, ASDP::ratio> NOESY::LFactor(Pdb& pdb,
				  ostream& summaryOutput, ostream& outputL, bool noLong) const
{

  deque<Pdb> pdbList; pdbList.clear();
  if (pdb.notEmpty())
    {
      pdbList.push_back(pdb);
    }

  deque<ASDP::ratio> LallScore;
  deque<ASDP::ratio> LallScorew;
  deque<ASDP::ratio> LlongScore;
  deque<ASDP::ratio> LlongScorew;

  erasePrint(cout, " Doing Precision analysis for " + label() + " ...");

  // interactions that shown in the peak list
  cMomDadSet tempSet; tempSet.clear();
  for(PeakList::const_iterator iter = peakList.begin(); iter != peakList.end(); iter++)
  {
      MatchPeakAtom::const_iterator il = MombySimpleMatch.lower_bound((&*iter));
      MatchPeakAtom::const_iterator iu = MombySimpleMatch.upper_bound((&*iter));

      MatchPeakAtom::const_iterator dl = DadbySimpleMatch.lower_bound((&*iter));
      MatchPeakAtom::const_iterator du = DadbySimpleMatch.upper_bound((&*iter));

      for(MatchPeakAtom::const_iterator i = il; i != iu; i++)
      	for(MatchPeakAtom::const_iterator d = dl; d != du; d++)
      	{
	  tempSet.insert(MomDad((*i).second, (*d).second)); // ordered by momdad
	}
  }

  // initilize the atomllistmap
  Atom_Index_List atommap; atommap.clear();
  for(Atom_List::iterator iatom = NOESY_Assign::atoms().begin(); iatom != NOESY_Assign::atoms().end(); iatom++)
  {
    atommap.insert(make_pair(iatom->get_index(), &(*iatom)));
  }


  deque<int> tAllR(pdbList.size()+1);  // = 0, tLngR = 0; // total consistent interactions

	// weighted
	deque<float> tAllw(pdbList.size()+1);  // = 0, tLngw=0; // total interactions
	deque<float> tAllRw(pdbList.size()+1); // = 0, tLngRw = 0;

	// consistent with structure, but not mapped
	deque<float> all(pdbList.size()+1);// = 0, lng = 0;
	deque<float> allw(pdbList.size()+1);  // = 0, lngw = 0;

  deque<disMomDadSetStat> HHList; HHList.clear();

  for(int i=0; i <= pdbList.size(); i++)
    {
      tAllR[i] = 0;
      tAllw[i] =0;
      tAllRw[i]=0;
      all[i]=0;
      allw[i]=0;

      disMomDadSetStat HH; HH.clear();
      HHList.push_back(HH);
    }
  //cout << "i m here0\n";

  for(Atom_List::const_iterator miter = NOESY_Assign::atoms().begin(); miter != NOESY_Assign::atoms().end(); ++miter)
    {
      if ((*miter).isH()) // && ((*miter).isatt_C() || (*miter).isatt_N()))
    	{

          deque<atom*> drange; drange.clear();
          get_range(pdbList, &*miter, atommap, drange);
	  //cout << "i m here0.0\n";

	  if (DEBUG)
	    NOESY_Assign::outputError() << "-->" << *miter << endl;
        //////  end range search
        for(deque<atom*>::const_iterator diter = drange.begin(); diter != drange.end(); diter++)
    	    {
    	      //diter++;

	      if (*(*diter) <  *miter) continue;
	      if (*(*diter) == *miter) continue;

	    if (DEBUG)
              NOESY_Assign::outputError() << *diter << " " << endl;

						if ((*diter)->isH() &&
							 	((!noLong) || (abs((*diter)->get_index()-miter->get_index()) <=4)) &&  // noLong or all  noLong=0 always
						    ((canbeMom(&*miter) && canbeDad(*diter)) || (canbeMom(*diter) && canbeDad(&*miter))) // check if canbe mom or dad
           				&& (fabs((*miter).get_shift() - (*diter)->get_shift()) > 0.3999999)) // not close to DIA
        		{
 								cMomDad ab = make_pair(&*miter, *diter);
	            	//float dist = sumAverage(getDistStat(pdbList, ab, isIC()));
	    SampleStatistic distStat = getDistStat(pdbList, ab, isIC());


	    for(int i=0; i <= pdbList.size(); i++)
	      {
		float dist;

		if (distStat.count() == 0)
		  dist = 1000000;
		else
		{
			if (i ==0)
				dist = distStat.median(); // the default
			else
		  		dist = (distStat.oriSamples())[i-1];
		}

		if (dist < 1.0)
		  dist = 1.0;

		if (dist < get_RmaxQ())
		  {
		    if (HHList[i].find(make_pair(miter->get_repAtom(),(*diter)->get_repAtom())) != HHList[i].end()) // find an entry
		      {
			disMomDadSetStat::iterator found = HHList[i].find(make_pair(miter->get_repAtom(),(*diter)->get_repAtom()));
			found->second.first.insert(ab);
			if (dist < found->second.second)
			  found->second.second = dist;
		      }
		    else // new entry
		      {
			cMomDadSet temp; temp.clear(); temp.insert(ab);
			HHList[i].insert(make_pair(make_pair(miter->get_repAtom(),(*diter)->get_repAtom()),
						   make_pair(temp, dist)));
		      }
		  } // end if dist
	      }
	      }// end if can be mom and data
    	    } // end while
    	} // end if is H
    } // end for

  if (!NOESY_Assign::isS())
    outputL << endl << "# Missing interactons for " << label() << endl;

  //cout << "i m here1\n";

  deque<int> tideal(pdbList.size()+1);  deque<int> ideal(pdbList.size()+1);
  deque<float> tidealw(pdbList.size()+1); deque<float> idealw(pdbList.size()+1);
  for(int i=0; i <= pdbList.size(); i++)
    {
      tideal[i] =0;
      ideal[i]=0;
      tidealw[i]=0;
      idealw[i]=0;
    }

  for(int i = 0; i <= pdbList.size(); i++)
    for(disMomDadSetStat::const_iterator mdi = HHList[i].begin(); mdi != HHList[i].end(); mdi++)
    {

      float weight = isNAB(mdi->first.first->get_layer())*isNAB(mdi->first.second->get_layer())+1;

    	{
    	  //total++;
	  int diff = abs(mdi->first.first->get_index() - mdi->first.second->get_index());

        //if (mdi->second.second < get_RmaxQ()) // expected < Rmax
        {
	   cMomDadSet::const_iterator ii;

	   // one side

if (canbeMom(mdi->first.first) && canbeDad(mdi->first.second) && !nearH2o(mdi->first.first))// check if canbe mom or dad
  {

    //outputL << endl << mdi->first.first << getWL() << getWU() << endl;

    // for ideal-structure
    if ((mdi->first.first->isMeaningless(mdi->first.second)) || (mdi->first.first->length3(mdi->first.second))

	       // methyl
	       || (mdi->first.first->MethylFiveBonds(mdi->first.second))

	       // Leu
	       || ((mdi->first.first->get_sym_1() == 'L') && (mdi->first.first->get_layer() == "N")
		&& ((mdi->first.second->get_layer() == "G") || (mdi->first.second->get_layer() == "D")))
	       || ((mdi->first.first->get_sym_1() == 'L') && (mdi->first.second->get_layer() == "N")
		&& ((mdi->first.first->get_layer() == "G") || (mdi->first.first->get_layer() == "D")))

	       || ((mdi->first.first->get_sym_1() == 'L') && (mdi->first.second->get_layer() == "D")
		   && ((mdi->first.first->get_layer() == "D")))
	       || ((mdi->first.first->get_sym_1() == 'V') && (mdi->first.second->get_layer() == "G")
		   && ((mdi->first.first->get_layer() == "G")))

	       // ILE
	       || ((mdi->first.first->get_sym_1() == 'I') && (mdi->first.first->get_layer() == "N")
		&& ((mdi->first.second->get_layer() == "D")))
	       || ((mdi->first.first->get_sym_1() == 'I') && (mdi->first.second->get_layer() == "N")
		   && ((mdi->first.first->get_layer() == "D"))))

      	   {
      		   tideal[i]++;
      		   tidealw[i] += pow(mdi->second.second*0.1, -6.0)*weight;

      	   }
	   				// for query structure
           ++tAllR[i];
           tAllRw[i] += pow(mdi->second.second*0.1, -6.0)*weight;

 	   for(ii = (*mdi).second.first.begin(); ii != (*mdi).second.first.end(); ii++)
	     {
	       if ((tempSet.find((*ii)) != tempSet.end())) // || (tempSet.find(make_pair((*ii).second, (*ii).first)) != tempSet.end()))
		 break;
	     }

	   if (ii == (*mdi).second.first.end()) // not in the peak list
	     {
	       if (i ==0) // only report ave fps
		{

		  if (!NOESY_Assign::isS())
		    {
		      for(cMomDadSet::iterator ij = (*mdi).second.first.begin(); ij != (*mdi).second.first.end(); ij++)
			{
			  outputL << endl << (*ij).first << endl << (*ij).second << endl << "::";
			}
		      outputL << "sumAvg: " << mdi->second.second << endl;
		    }
		}

		  // not in the peak list for ideal structure
		  if ((mdi->first.first->isMeaningless(mdi->first.second)) || (mdi->first.first->length3(mdi->first.second)))
		    {
		      ideal[i]++;
		      idealw[i] += pow(mdi->second.second*0.1, -6.0)*weight;
		    }

	       // no maped for query structure
		  all[i]++;
		  allw[i] += pow(mdi->second.second*0.1, -6.0)*weight;
             }
  }

if (canbeMom(mdi->first.second) && canbeDad(mdi->first.first) && !nearH2o(mdi->first.second))// check if canbe mom or dad
  {

    if ((mdi->first.first->isMeaningless(mdi->first.second)) || (mdi->first.first->length3(mdi->first.second))
	       // methyl
	       || (mdi->first.first->MethylFiveBonds(mdi->first.second))

	       // Leu
	       || ((mdi->first.first->get_sym_1() == 'L') && (mdi->first.first->get_layer() == "N")
		&& ((mdi->first.second->get_layer() == "G") || (mdi->first.second->get_layer() == "D")))
	       || ((mdi->first.first->get_sym_1() == 'L') && (mdi->first.second->get_layer() == "N")
		&& ((mdi->first.first->get_layer() == "G") || (mdi->first.first->get_layer() == "D")))

	       || ((mdi->first.first->get_sym_1() == 'L') && (mdi->first.second->get_layer() == "D")
		   && ((mdi->first.first->get_layer() == "D")))
	       || ((mdi->first.first->get_sym_1() == 'V') && (mdi->first.second->get_layer() == "G")
		   && ((mdi->first.first->get_layer() == "G")))

	       // ILE
	       || ((mdi->first.first->get_sym_1() == 'I') && (mdi->first.first->get_layer() == "N")
		&& ((mdi->first.second->get_layer() == "D")))
	       || ((mdi->first.first->get_sym_1() == 'I') && (mdi->first.second->get_layer() == "N")
		   && ((mdi->first.first->get_layer() == "D"))))
      	   {
      		   tideal[i]++;
      		   tidealw[i] += pow(mdi->second.second*0.1, -6.0)*weight;

      	   }   				// for query structure
           ++tAllR[i];
           tAllRw[i] += pow(mdi->second.second*0.1, -6.0)*weight;

	   // symmetric side
	   for(ii = (*mdi).second.first.begin(); ii != (*mdi).second.first.end(); ii++)
	     {
	       if (tempSet.find(make_pair((*ii).second, (*ii).first)) != tempSet.end())
		 break;
	     }

	   if (ii == (*mdi).second.first.end()) // not in the peak list
	     {
	       if (i ==0) // only report ave fps
		{

		  if (!NOESY_Assign::isS())
		    {
		      for(cMomDadSet::iterator ij = (*mdi).second.first.begin(); ij != (*mdi).second.first.end(); ij++)
			{
			  outputL << endl << (*ij).second << endl << (*ij).first << endl << "::";
			}
		      outputL << "sumAvg: " << mdi->second.second << endl;
		    }
		}

	       // not in the peak list for ideal structure
	       if ((mdi->first.first->isMeaningless(mdi->first.second)) || (mdi->first.first->length3(mdi->first.second)))
		 {
		   ideal[i]++;
		   idealw[i] += pow(mdi->second.second*0.1, -6.0)*weight;
		 }

	       // no maped for query structure
	       all[i]++;
	       allw[i] += pow(mdi->second.second*0.1, -6.0)*weight;
             }
  }
	}
    	}
    }
  //cout << "i m here2\n";

  for(int i =0; i<= pdbList.size(); i++)
    {
      if (!NOESY_Assign::isS())
	{
	  if (i==0)
	    summaryOutput << "Model ave";
	  else
	    summaryOutput << "Model " << setw(3) << i;

	  summaryOutput << setw(3) << "  Missing interactions: " << (all[i]/((float)tAllR[i]+0.00001)) << " (" << all[i] << "/" << tAllR[i] << ")" << endl
			<< "              weighted: " << (allw[i]/(tAllRw[i]+0.00001)) << " (" << allw[i] << "/" << tAllRw[i] << ")" << endl
			<< "                    <3: " << (ideal[i]/((float)tideal[i]+0.00001)) << " (" << ideal[i] << "/" << tideal[i] << ")" << endl
			<< "                   <3w: " << (idealw[i]/((float)tidealw[i]+0.00001)) << " (" << idealw[i] << "/" << tidealw[i] << ")" << endl;
	  cout << setw(3) << i << "  Missing interactions: " << (all[i]/((float)tAllR[i])+0.00001) << " (" << all[i] << "/" << tAllR[i] << ")" << endl
	       << "              weighted: " << (allw[i]/(tAllRw[i]+0.00001)) << " (" << allw[i] << "/" << tAllRw[i] << ")" << endl
	       << "                    <3: " << (ideal[i]/((float)tideal[i]+0.00001)) << " (" << ideal[i] << "/" << tideal[i] << ")" << endl
	       << "                   <3w: " << (idealw[i]/((float)tidealw[i]+0.00001)) << " (" << idealw[i] << "/" << tidealw[i] << ")" << endl;
	}

      LallScore[i] = make_pair(all[i], tAllR[i]+0.00001);
      LallScorew[i] = make_pair(allw[i], tAllRw[i]+0.00001);

      LlongScore[i] = make_pair(ideal[i], tideal[i]+0.00001);
      LlongScorew[i] = make_pair(idealw[i], tidealw[i]+0.00001);
    }

  return pair<ASDP::ratio, ASDP::ratio>(LallScorew[0], LlongScorew[0]);
}
