/***************************************************************************
                          experiment_Ring.cpp  -  description
                             -------------------
    begin                : Mon Mar 31 2003
    copyright            : (C) 2003 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/
#include <iostream>
#include <sstream>
#include <cmath>
#include <iomanip>
#include "experiment.h"
#include "noesy_assign.h"
#include "par.h"

extern bool DEBUG;
SampleStatistic getDistStat(deque<Pdb>& , pair<const atom*, const atom*> , bool );
float sumAverage(const SampleStatistic&);

bool isAr(char a)
{
	if ((a == 'F') || (a== 'H') || (a=='Y'))
		return 1;
	return 0;
}

void NOESY::Find_RingPoss(const Ring& parRing, RingContact& rc)
{
	for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); ++iter)
	{
		if (AssignList.find(&*iter) != AssignList.end()) // 1) not assigned peak list
			continue;
			
    PeakMomDadMap::iterator i1l = BList.lower_bound(&(*iter));
    PeakMomDadMap::iterator i1u = BList.upper_bound(&(*iter));

    for(PeakMomDadMap::iterator i1 = i1l; i1 != i1u; i1++)
		{					
				atom* a = i1->second.first;
				atom* b = i1->second.second;

				pair<atom*, atom*> ab;

				if (a<b) ab = make_pair(a,b);
				else 		 ab = make_pair(b,a);
					
    		// filtered by distance
       	bool dok = 0;
				SampleStatistic distStat = getDistStat(NOESY_Assign::pdbList(), ab, isIC());
				if (distStat.count())
				{
					if (sumAverage(distStat) < parRing.meanCutoff)
						dok = 1;
				}
				      
    		if (dok)
				{
          if ((i1->second.first->get_Hdiff(i1->first->get_HX1(), SWSF_HX1) < parRing.htol + 0.000001) &&
          		(i1->second.second->get_Hdiff(i1->first->get_HX2(), SWSF_HX2) < parRing.htol + 0.000001) &&
          		(i1->second.first->get_Xdiff(i1->first->get_X1(), SWSF_X1) < parRing.xtol + 0.000001) &&
          		(i1->second.second->get_Xdiff(i1->first->get_X2(), SWSF_X2) < parRing.xtol + 0.000001))
  				{
  					if ((i1->second.first->isRingAtom() || i1->second.second->isRingAtom()) &&
  							(!isAr(i1->second.first->get_sym_1()) || i1->second.first->isRingAtom()) &&
  							(!isAr(i1->second.second->get_sym_1()) || i1->second.second->isRingAtom()))
  					{
    						if (i1->second.first->get_index() < i1->second.second->get_index())
    							rc.insert(make_pair(make_pair(i1->second.first->get_index(), i1->second.second->get_index()), *i1));
    						else
    							rc.insert(make_pair(make_pair(i1->second.second->get_index(), i1->second.first->get_index()), *i1));
        		}
  				}
				}
     }
	}
}

int NOESY::Assign_RingContact(const Ring& parRing, RingContact& rc)
{
  ostringstream outStream;

  int old = AssignList.size();
  int unOld = 0;
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
    {
      if (AssignList.find(&(*iter)) == AssignList.end())
			unOld++;
    }

  int wN = 0; int iN = 0;

  MomDadPeakMap assignMap; assignMap.clear();

	// assigned the unique complete ones
	PeakMomDadMap pmdMap; pmdMap.clear();
	RingSet ringSet; ringSet.clear();
	for(RingContact::iterator i = rc.begin(); i != rc.end(); ++i)
	{
		ringSet.insert(i->first);
	}
	for(RingSet::iterator j = ringSet.begin(); j != ringSet.end(); ++j)
	{
			for(RingContact::iterator ji = rc.lower_bound(*j); ji != rc.upper_bound(*j); ++ji)
			{
				if (BList.find(ji->second.first) != BList.end())
				{
  				if (DEBUG)
  					NOESY_Assign::outputLog() << "FindRingContact " << ji->second.first << endl << ji->second.second.first << endl
  																		<< ji->second.second.second << endl;
  				assignMap.insert(make_pair(ji->second.second, ji->second.first));
				}
			}
	}
		
	MakeAssignMap(assignMap, RING, wN, iN);
	
	if (AssignList.size() - old == 0)
    return 0;

  int un = 0;
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
    {
      if (AssignList.find(&(*iter)) == AssignList.end())
			un++;
    }

  outStream.setf(ios::left, ios::adjustfield);
  outStream.width(6);
  outStream << (itoa(NOESY_Assign::ddcycle())+"-"+itoa(NOESY_Assign::dcycle())).c_str();
  outStream.width(7);
  outStream << "RING";
  outStream.width(6);
  outStream << unOld - un;
  outStream.width(8);
  outStream << peakList.size() - un;
  outStream.width(7);
  outStream << AssignList.size() - old;
  outStream.width(9);
  outStream << AssignList.size();
  outStream.width(19);
  outStream << "-";
  outStream.width(18);
  outStream << un;
  outStream.width(11);
  outStream << noiseList.size();

  if (DEBUG)
    NOESY_Assign::outputLog() << "IN ASSIGN_RING wN:" << wN << " iN: " << iN << " " << AssignList.size() - old - wN << endl;

  if (NOESY_Assign::COMPARE())
  {
  outStream.width(8);
    outStream << "-";
  outStream.width(11);
  if (NOESY_Assign::COMPARE())
    outStream << AssignList.size() - old - iN;
   else
    outStream << "-";
  outStream.width(11);
  if (NOESY_Assign::COMPARE())
    outStream << iN;
  else
    outStream << "-";
  }
  outStream << "\n" << NOESY_Assign::EOS();
  string temp(outStream.str());
  reportProcessStat += temp.substr(0,temp.find(NOESY_Assign::EOS()));

	return AssignList.size() - old;
}

