/***************************************************************************
                          experiment_SS.cpp  -  description
                             -------------------
    begin                : Mon Aug 21 2000
    copyright            : (C) 2000 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#include <iostream>
#include <cmath>
#include "experiment.h"
#include "noesy_assign.h"

#define abs(x) abs((double)(x))

extern bool DEBUG;

////////////////////////////////////////////////////////
// find the contact frequence for each residue-pairs	//
//    with 0.03 and 0.3 tolerances										//
//    and weighted by finding symmetric peaks 				//
////////////////////////////////////////////////////////
void NOESY::Find_SYMcfposs(matrix& cfMax, const Sym& parSym, ostream& out) const
{

	// build long-range symmtry contact maps.
		
	// put all possible assignments between 0.03 and 0.3 tolerances into contact bin
	// shall include all potential contacts with good data sets.
	// if have symmtry, all ready add two by doing so
	
	// if only pick peaks have only long-range contacts, will exclude the one contact that degenerate with close range contacts
	// if only have symmetry peaks, will exclude the real contact that do not have symmetry peaks
	for(PeakList::const_iterator i = peakList.begin(); i != peakList.end(); i++)
	{
  		
	  MatchPeakAtom::const_iterator pl = MombySimpleMatch.lower_bound(&(*i));
		MatchPeakAtom::const_iterator pu = MombySimpleMatch.upper_bound(&(*i));
	
		MatchPeakAtom::const_iterator dl = DadbySimpleMatch.lower_bound(&(*i));
		MatchPeakAtom::const_iterator du = DadbySimpleMatch.upper_bound(&(*i));

		for(MatchPeakAtom::const_iterator pi = pl; pi != pu; pi++)
		{
		      {
						for(MatchPeakAtom::const_iterator di = dl; di != du; di++)
			  		{
							{			
			    			int l, s;
			    			if (pi->second->get_index() > di->second->get_index())
			      		{
									l = pi->second->get_index();
									s = di->second->get_index();
			      		}
			    			else
			      		{
									l = di->second->get_index();
									s = pi->second->get_index();
			      		}
			    			matrix::iterator i = cfMax.find(pair<int,int>(s, l));
			    			
			    			float fr=1;
			    			if (i != cfMax.end())
			      				(*i).second += fr ;
			    			else
			      				cfMax.insert(make_pair(make_pair(s,l), fr));
			    			
			    			matrix::iterator j = cfMax.find(pair<int,int>(l, s));
			    			if (j != cfMax.end())
			      				(*j).second += fr;
			    			else
			      				cfMax.insert(make_pair(make_pair(l,s), fr));
			      	}
			  		}
		      }
		}
	}

}

void NOESY::Find_UniqueHH(int i, int j, method mthd)
{
	MatchPeakAtombyAtomIndex MombyIndex;

	for(MatchPeakAtom::iterator iter = MombySimpleMatch.begin(); iter != MombySimpleMatch.end(); iter++)
		if ((*iter).second->get_layer() == "N")
			MombyIndex.insert(make_pair((*iter).second->get_index(), (*iter)));

	// find all possible (i,j) that root at i
	MatchPeakAtombyAtomIndex::iterator
		res1RootLower = MombyIndex.lower_bound(i);
	MatchPeakAtombyAtomIndex::iterator
		res1RootUpper = MombyIndex.upper_bound(i);

	for(MatchPeakAtombyAtomIndex::iterator iter1 = res1RootLower;
		iter1 != res1RootUpper; iter1++)
	{
		// for each peak, find all possible father
		MatchPeakAtom::iterator fatherLower
			= DadbySimpleMatch.lower_bound((*iter1).second.first);
		MatchPeakAtom::iterator fatherUpper
			= DadbySimpleMatch.upper_bound((*iter1).second.first);

		for(MatchPeakAtom::iterator iterf = fatherLower; iterf != fatherUpper; iterf++)
			if (((*iterf).second->get_index() == j) &&
				((*iterf).second->get_layer() == "N"))
					MakeBuffer((*iterf).first, (*iter1).second.second, (*iterf).second, mthd);
	}

	// find all possible (i,j) that root at j
	MatchPeakAtombyAtomIndex::iterator
		res2RootLower = MombyIndex.lower_bound(j);
	MatchPeakAtombyAtomIndex::iterator
		res2RootUpper = MombyIndex.upper_bound(j);

	for(MatchPeakAtombyAtomIndex::iterator iter2 = res2RootLower;
		iter2 != res2RootUpper; iter2++)
	{
		// for each peak, find all possible father
		MatchPeakAtom::iterator fatherLower
			= DadbySimpleMatch.lower_bound((*iter2).second.first);
		MatchPeakAtom::iterator fatherUpper
			= DadbySimpleMatch.upper_bound((*iter2).second.first);

		for(MatchPeakAtom::iterator iterf = fatherLower; iterf != fatherUpper; iterf++)
			if (((*iterf).second->get_index() == i) &&
				((*iterf).second->get_layer() == "N"))
			{
					MakeBuffer((*iterf).first, (*iter2).second.second, (*iterf).second, mthd);
			}
	}
}



void NOESY::Find_UniqueAA(int i, int j, method mthd)
{
	MatchPeakAtombyAtomIndex MombyIndex;

	for(MatchPeakAtom::iterator iter = MombySimpleMatch.begin(); iter != MombySimpleMatch.end(); iter++)
		if ((*iter).second->get_layer().substr(0,1) == "A")
			MombyIndex.insert(make_pair((*iter).second->get_index(), (*iter)));

	// find all possible (i,j) that root at i
	MatchPeakAtombyAtomIndex::iterator
		res1RootLower = MombyIndex.lower_bound(i);
	MatchPeakAtombyAtomIndex::iterator
		res1RootUpper = MombyIndex.upper_bound(i);

	for(MatchPeakAtombyAtomIndex::iterator iter1 = res1RootLower;
		iter1 != res1RootUpper; iter1++)
	{
		// for each peak, find all possible father
		MatchPeakAtom::iterator fatherLower
			= DadbySimpleMatch.lower_bound((*iter1).second.first);
		MatchPeakAtom::iterator fatherUpper
			= DadbySimpleMatch.upper_bound((*iter1).second.first);

		for(MatchPeakAtom::iterator iterf = fatherLower; iterf != fatherUpper; iterf++)
			if (((*iterf).second->get_index() == j) &&
				((*iterf).second->get_layer().substr(0,1) == "A"))
			{
					MakeBuffer((*iterf).first, (*iter1).second.second, (*iterf).second, mthd);
			}
	}

	// find all possible (i,j) that root at j
	MatchPeakAtombyAtomIndex::iterator
		res2RootLower = MombyIndex.lower_bound(j);
	MatchPeakAtombyAtomIndex::iterator
		res2RootUpper = MombyIndex.upper_bound(j);

	for(MatchPeakAtombyAtomIndex::iterator iter2 = res2RootLower;
		iter2 != res2RootUpper; iter2++)
	{
		// for each peak, find all possible father
		MatchPeakAtom::iterator fatherLower
			= DadbySimpleMatch.lower_bound((*iter2).second.first);
		MatchPeakAtom::iterator fatherUpper
			= DadbySimpleMatch.upper_bound((*iter2).second.first);

		for(MatchPeakAtom::iterator iterf = fatherLower; iterf != fatherUpper; iterf++)
			if (((*iterf).second->get_index() == i) &&
				((*iterf).second->get_layer() == "A"))
			{
					MakeBuffer((*iterf).first, (*iter2).second.second, (*iterf).second, mthd);
			}
	}
}

void NOESY::Find_UniqueAB(int i, int j, method mthd)
{
	MatchPeakAtombyAtomIndex MombyIndex;

	// root at A
	for(MatchPeakAtom::iterator iter = MombySimpleMatch.begin(); iter != MombySimpleMatch.end(); iter++)
		if ((*iter).second->get_layer().substr(0,1) == "A")
			MombyIndex.insert(make_pair((*iter).second->get_index(), (*iter)));

	// find all possible (i,j) that root at i
	MatchPeakAtombyAtomIndex::iterator
		res1RootLower = MombyIndex.lower_bound(i);
	MatchPeakAtombyAtomIndex::iterator
		res1RootUpper = MombyIndex.upper_bound(i);

	for(MatchPeakAtombyAtomIndex::iterator iter1 = res1RootLower;
		iter1 != res1RootUpper; iter1++)
	{
		// for each peak, find all possible father
		MatchPeakAtom::iterator fatherLower
			= DadbySimpleMatch.lower_bound((*iter1).second.first);
		MatchPeakAtom::iterator fatherUpper
			= DadbySimpleMatch.upper_bound((*iter1).second.first);

		for(MatchPeakAtom::iterator iterf = fatherLower; iterf != fatherUpper; iterf++)
			if (((*iterf).second->get_index() == j) &&
				((*iterf).second->get_layer().substr(0,1) == "B"))
			{
					MakeBuffer((*iterf).first, (*iter1).second.second, (*iterf).second, mthd);
			}
	}

	// root at B
	MombyIndex.erase(MombyIndex.begin(), MombyIndex.end());

	for(MatchPeakAtom::iterator iter = MombySimpleMatch.begin(); iter != MombySimpleMatch.end(); iter++)
		if ((*iter).second->get_layer().substr(0,1) == "B")
			MombyIndex.insert(make_pair((*iter).second->get_index(), (*iter)));

	// find all possible (i,j) that root at j
	res1RootLower = MombyIndex.lower_bound(j);
	res1RootUpper = MombyIndex.upper_bound(j);

	for(MatchPeakAtombyAtomIndex::iterator iter1 = res1RootLower;
		iter1 != res1RootUpper; iter1++)
	{
		// for each peak, find all possible father
		MatchPeakAtom::iterator fatherLower
			= DadbySimpleMatch.lower_bound((*iter1).second.first);
		MatchPeakAtom::iterator fatherUpper
			= DadbySimpleMatch.upper_bound((*iter1).second.first);

		for(MatchPeakAtom::iterator iterf = fatherLower; iterf != fatherUpper; iterf++)
			if (((*iterf).second->get_index() == i) &&
				((*iterf).second->get_layer().substr(0,1) == "A"))
			{
					MakeBuffer((*iterf).first, (*iter1).second.second, (*iterf).second, mthd);
			}
	}
}

// used to find helix h-bonding
bool NOESY::Find_possHaHnNoe(int dist, int indexNH)
{
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
  {
 	  PeakAssignReason::iterator AssignLower = AssignList.lower_bound(&(*iter));
	  PeakAssignReason::iterator AssignUpper = AssignList.upper_bound(&(*iter));
	  for(PeakAssignReason::iterator Ai = AssignLower; Ai != AssignUpper; Ai++)
		{
		  if ((Ai->second.first.first->get_index() == indexNH) && (Ai->second.first.first->get_layer() == "N"))
		  {
    		if (((Ai->second.first.first->get_index() - dist) == Ai->second.first.second->get_index()) && (Ai->second.first.second->get_layer() == "A"))
    		{
	    		return 1;
	  		}
	  	}
	  	else if ((Ai->second.first.second->get_index() == indexNH) && (Ai->second.first.second->get_layer() == "N"))
	  	{
    		if (((Ai->second.first.second->get_index() - dist) == Ai->second.first.first->get_index()) && (Ai->second.first.first->get_layer() == "A"))
    		{
	    		return 1;
	  		}
	  	}
		}
	}

	return 0;
}

bool NOESY::Find_possHnHnNoe(int dist, int indexNH)
{
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
  {
 	  PeakAssignReason::iterator AssignLower = AssignList.lower_bound(&(*iter));
	  PeakAssignReason::iterator AssignUpper = AssignList.upper_bound(&(*iter));
	  for(PeakAssignReason::iterator Ai = AssignLower; Ai != AssignUpper; Ai++)
		{
		  if ((Ai->second.first.first->get_index() == indexNH) && (Ai->second.first.first->get_layer() == "N"))
		  {
    		if (((Ai->second.first.first->get_index() - dist) == Ai->second.first.second->get_index()) && (Ai->second.first.second->get_layer() == "N"))
    		{
	    		return 1;
	  		}
	  	}
	  	else if ((Ai->second.first.second->get_index() == indexNH) && (Ai->second.first.second->get_layer() == "N"))
	  	{
    		if (((Ai->second.first.second->get_index() - dist) == Ai->second.first.first->get_index()) && (Ai->second.first.first->get_layer() == "N"))
    		{
	    		return 1;
	  		}
	  	}
		}
	}

	return 0;
}

void NOESY::Find_possSSnoe(NOEtypeList& NOEtypeL, const Sec& parSec)
{
  for(PeakMomDadMap::iterator iter = BList.begin(); iter != BList.end(); iter++)
    {
      if ((AssignList.find((*iter).first) == AssignList.end()) &&
	  ((*iter).second.first->get_Hdiff((*iter).first->get_HX1(), SWSF_HX1) < parSec.htol + 0.000001) &&
	  ((*iter).second.first->get_Xdiff((*iter).first->get_X1(), SWSF_X1) < parSec.xtol + 0.000001) &&
	  ((*iter).second.second->get_Hdiff((*iter).first->get_HX2(), SWSF_HX2) < parSec.htol + 0.000001) &&
	  (((*iter).first->get_D() == 3) || (*iter).second.second->get_Xdiff((*iter).first->get_X2(), SWSF_X2) < parSec.xtol + 0.000001))
	{
	
	  // distance = 1
	  if (((*iter).second.first->get_index() + 1) == (*iter).second.second->get_index())
	    {
	
	    // helix 2.6-2.8
		if (((*iter).second.first->get_layer() == "N") && ((*iter).second.second->get_layer() == "N") &&
		    (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn))
		     == NOEtypeL.end()) && (pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0) < 4.5))
		  {
		    NOEtypeL.insert(make_pair(make_pair(make_pair((*iter).second.first->get_index(),
		    		(*iter).second.second->get_index()), HnHn), iter->first));
		    NOEtypeL.insert(make_pair(make_pair(make_pair((*iter).second.second->get_index(),
		    		(*iter).second.first->get_index()), HnHn), iter->first));
		  }
		
		  // b-strand 2.2
		else if (((*iter).second.first->get_layer() == "A") && ((*iter).second.second->get_layer() == "N") && 
			 (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn))
			  == NOEtypeL.end()) && (pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0) < 4.5))
		  {
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn), iter->first));
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HnHa), iter->first));
		  }
	      }
	    else if (((*iter).second.first->get_index() - 1) == (*iter).second.second->get_index())
	      {
		if (((*iter).second.first->get_layer() == "N") && ((*iter).second.second->get_layer() == "N") && 
		    (NOEtypeL.find(make_pair
				   (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn))
		     == NOEtypeL.end()) && (pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0) < 4.5))
		  {
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HnHn), iter->first));
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn), iter->first));
		  }
		else if (((*iter).second.first->get_layer() == "N") && ((*iter).second.second->get_layer() == "A") &&
			 (NOEtypeL.find(make_pair
					(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa))
			  == NOEtypeL.end()) && (pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0) < 4.5))
		  {
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HaHn), iter->first));
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa), iter->first));
		  }
	      }
	
	
	   // distance = 2
	    else if (((*iter).second.first->get_index() + 2) == (*iter).second.second->get_index())
	      {
		if (((*iter).second.first->get_layer() == "N") && ((*iter).second.second->get_layer() == "N") &&
		    (NOEtypeL.find(make_pair
				   (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn))
		     == NOEtypeL.end()))
		  {
		    NOEtypeL.insert(make_pair(make_pair
				  (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn), iter->first));
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HnHn), iter->first));
		  }
		else if (((*iter).second.first->get_layer() == "A") && ((*iter).second.second->get_layer() == "N") &&
		         (NOEtypeL.find(make_pair
				        (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn))
		     == NOEtypeL.end()))
      {		
		  NOEtypeL.insert(make_pair(make_pair
				  (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn), iter->first));
		  NOEtypeL.insert(make_pair(make_pair
				  (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HnHa), iter->first));
	   }
	      }
	 else if (((*iter).second.first->get_index() - 2) == (*iter).second.second->get_index())
	      {
		if (((*iter).second.first->get_layer() == "N") && ((*iter).second.second->get_layer() == "N") &&
		    (NOEtypeL.find(make_pair
				   (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn))
		     == NOEtypeL.end()))
		  {
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HnHn), iter->first));
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn), iter->first));
		  }
		else if (((*iter).second.first->get_layer() == "N") && ((*iter).second.second->get_layer() == "A") &&
			 (NOEtypeL.find(make_pair
					(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa))
			  == NOEtypeL.end()))
		  {
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HaHn), iter->first));
			    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa), iter->first));
		  }
	      }
	      // distance = 3
	    else if (((*iter).second.first->get_index() + 3) == (*iter).second.second->get_index())
	      {
	
	      // helix 3.3-3.4
		if (((*iter).second.first->get_layer() == "A") && ((*iter).second.second->get_layer() == "N") &&
		    (NOEtypeL.find(make_pair
				   (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn))
		     == NOEtypeL.end()) && (pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0) < 4.5))
		  {
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn), iter->first));
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HnHa), iter->first));
		  }
		else if (((*iter).second.first->get_layer() == "A") && ((*iter).second.second->get_layer() == "B") &&
			 (NOEtypeL.find(make_pair
					(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHb))
		     == NOEtypeL.end()))
		  {
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHb), iter->first));	
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HbHa), iter->first));	
		  }
	      }
	    else if (((*iter).second.first->get_index() - 3) == (*iter).second.second->get_index())
	      {
		if (((*iter).second.first->get_layer() == "N") && ((*iter).second.second->get_layer() == "A") &&
		    (NOEtypeL.find(make_pair
				   (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa))
		     == NOEtypeL.end()) && (pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0) < 4.5))
		  {
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HaHn), iter->first));
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa), iter->first));
		  }
		else if (((*iter).second.first->get_layer() == "B") && ((*iter).second.second->get_layer() == "A") &&
		    (NOEtypeL.find(make_pair
				   (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HbHa))
		     == NOEtypeL.end()))
		  {
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HaHb), iter->first));
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HbHa), iter->first));
		  }
	      }
	
	      // distance = 4
	    else if (((*iter).second.first->get_index() + 4) == (*iter).second.second->get_index())
	      {
		if (((*iter).second.first->get_layer() == "A") && ((*iter).second.second->get_layer() == "N") &&
		    (NOEtypeL.find(make_pair
				   (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn))
		     == NOEtypeL.end()))
		  {
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn), iter->first));
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HnHa), iter->first));
		  }
	      }
	    else if (((*iter).second.first->get_index() - 4) == (*iter).second.second->get_index())
	      {
		if (((*iter).second.first->get_layer() == "N") && ((*iter).second.second->get_layer() == "A") &&
		    (NOEtypeL.find(make_pair
				   (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa))
		     == NOEtypeL.end()))
		  {
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HaHn), iter->first));
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa), iter->first));
		  }
	      }
	
	      // long range - allow at most two for each contact bin - save symmetry information
	    if (abs((*iter).second.first->get_index() - (*iter).second.second->get_index()) > 3)
	      {
	      //hnha sheet 3.0-3.2
		if ((((*iter).second.first->get_layer() == "N") && ((*iter).second.second->get_layer() == "A")) &&
		    (NOEtypeL.find(make_pair
				   (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa))
		     == NOEtypeL.end())) // new && (pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0) < 4.5)))
		  {
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa), iter->first));
		  }
		
		  // hahn sheet 3.0-3.2
		else if ((((*iter).second.first->get_layer() == "A") && ((*iter).second.second->get_layer() == "N")) &&
			 (NOEtypeL.find(make_pair
					(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn))
			  == NOEtypeL.end())) // new && (pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0) < 4.5)))
		  {
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn), iter->first));
		  }
		
		  // HnHn for sheets 3.3-4.0
		else if (((*iter).second.first->get_layer() == "N") && ((*iter).second.second->get_layer() == "N") &&
			 (NOEtypeL.find(make_pair
					(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn))
			  == NOEtypeL.end())) // new && (pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0) < 4.0)) // 4.5
		  {
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn), iter->first));
		  }
		
		  // HaHa < 2.3 A for antiparallel strands only
		else if (((*iter).second.first->get_layer() == "A") && ((*iter).second.second->get_layer() == "A") &&
			 (NOEtypeL.find(make_pair
					(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHa))
			  == NOEtypeL.end()))// && (pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0) < 4.5)) // from 3.0 to 4.0

		  {
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHa), iter->first));
		  }
	      }
	  }
    }
}

// at the beginning, using for determing secondary secodary
// at the end, using to draw the connectivity map
void NOESY::Find_AssignSSnoe(NOEtypeList& NOEtypeL)
{
  for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
    {	
	{
	  PeakAssignReason::iterator AssignLower = AssignList.lower_bound(&(*iter));
	  PeakAssignReason::iterator AssignUpper = AssignList.upper_bound(&(*iter));
	  for(PeakAssignReason::iterator Ai = AssignLower; Ai != AssignUpper; Ai++)
	    {
	
	    // distance = 1
	      if (((*Ai).second.first.first->get_index() + 1) == (*Ai).second.first.second->get_index())
		{
		
		  // helix 2.6-2.8
		  if (((*Ai).second.first.first->get_layer() == "N") && ((*Ai).second.first.second->get_layer() == "N") &&
		      (NOEtypeL.find(make_pair
				     (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HnHn))
		       == NOEtypeL.end()))
		    {
		      NOEtypeL.insert(make_pair(make_pair(make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HnHn), &*iter));
		      NOEtypeL.insert(make_pair(make_pair(make_pair((*Ai).second.first.second->get_index(), (*Ai).second.first.first->get_index()), HnHn), &*iter));
		    }
		
		    // b-strand 2.2
		  else if (((*Ai).second.first.first->get_layer() == "A") && ((*Ai).second.first.second->get_layer() == "N") &&
			   (NOEtypeL.find(make_pair
					  (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HaHn))
			    == NOEtypeL.end()))
		    {
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HaHn), &*iter));
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.second->get_index(), (*Ai).second.first.first->get_index()), HnHa), &*iter));
		    }
		}
	      else if (((*Ai).second.first.first->get_index() - 1) == (*Ai).second.first.second->get_index())
		{
		  if (((*Ai).second.first.first->get_layer() == "N") && ((*Ai).second.first.second->get_layer() == "N") &&
		      (NOEtypeL.find(make_pair
				     (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HnHn))
		       == NOEtypeL.end()))
		    {
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.second->get_index(), (*Ai).second.first.first->get_index()), HnHn), &*iter));
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HnHn), &*iter));
		    }
		  else if (((*Ai).second.first.first->get_layer() == "N") && ((*Ai).second.first.second->get_layer() == "A") &&
			   (NOEtypeL.find(make_pair
					  (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HnHa))
			    == NOEtypeL.end()))
		    {
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.second->get_index(), (*Ai).second.first.first->get_index()), HaHn), &*iter));
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HnHa), &*iter));
		    }
		}
		
		  // distance = 2
	      else if (((*Ai).second.first.first->get_index() + 2) == (*Ai).second.first.second->get_index())
		{
		  if (((*Ai).second.first.first->get_layer() == "N") && ((*Ai).second.first.second->get_layer() == "N") &&
		      (NOEtypeL.find(make_pair
				     (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HnHn))
		       == NOEtypeL.end()))
		    {
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HnHn), &*iter));
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.second->get_index(), (*Ai).second.first.first->get_index()), HnHn), &*iter));
				}
		  else if (((*Ai).second.first.first->get_layer() == "A") && ((*Ai).second.first.second->get_layer() == "N") &&
		      (NOEtypeL.find(make_pair
				     (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HaHn))
		       == NOEtypeL.end()))
				{
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HaHn), &*iter));
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.second->get_index(), (*Ai).second.first.first->get_index()), HnHa), &*iter));
	      }
	  }
	      else if (((*Ai).second.first.first->get_index() - 2) == (*Ai).second.first.second->get_index())
		{
		  if (((*Ai).second.first.first->get_layer() == "N") && ((*Ai).second.first.second->get_layer() == "N") &&
		      (NOEtypeL.find(make_pair
				     (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HnHn))
		       == NOEtypeL.end()))
		    {
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.second->get_index(), (*Ai).second.first.first->get_index()), HnHn), &*iter));
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HnHn), &*iter));
		    }
		  else if (((*Ai).second.first.first->get_layer() == "N") && ((*Ai).second.first.second->get_layer() == "A") &&
			   (NOEtypeL.find(make_pair
					  (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HnHa))
			    == NOEtypeL.end()))
		    {
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.second->get_index(), (*Ai).second.first.first->get_index()), HaHn), &*iter));
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HnHa), &*iter));
		    }
		}
		
		// distance = 3
	      else if (((*Ai).second.first.first->get_index() + 3) == (*Ai).second.first.second->get_index())
		{
		  // helix 3.3-3.4
		  if (((*Ai).second.first.first->get_layer() == "A") && ((*Ai).second.first.second->get_layer() == "N") &&
		      (NOEtypeL.find(make_pair
				     (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HaHn))
		       == NOEtypeL.end()))
		    {
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HaHn), &*iter));
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.second->get_index(), (*Ai).second.first.first->get_index()), HnHa), &*iter));
		    }
		  else if (((*Ai).second.first.first->get_layer() == "A") && ((*Ai).second.first.second->get_layer() == "B") &&
			   (NOEtypeL.find(make_pair
					  (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HaHb))
			    == NOEtypeL.end()))
		    {
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HaHb), &*iter));	
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.second->get_index(), (*Ai).second.first.first->get_index()), HbHa), &*iter));	
		    }
		}
	      else if (((*Ai).second.first.first->get_index() - 3) == (*Ai).second.first.second->get_index())
		{
		  if (((*Ai).second.first.first->get_layer() == "N") && ((*Ai).second.first.second->get_layer() == "A") &&
		      (NOEtypeL.find(make_pair
				     (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HnHa))
		       == NOEtypeL.end()))
		    {
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.second->get_index(), (*Ai).second.first.first->get_index()), HaHn), &*iter));
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HnHa), &*iter));
		    }
		  else if (((*Ai).second.first.first->get_layer() == "B") && ((*Ai).second.first.second->get_layer() == "A") &&
			   (NOEtypeL.find(make_pair
					  (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HbHa))
			    == NOEtypeL.end()))
		    {
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.second->get_index(), (*Ai).second.first.first->get_index()), HaHb), &*iter));
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HbHa), &*iter));
		    }
		}
		
		  // distance = 4
	      else if (((*Ai).second.first.first->get_index() + 4) == (*Ai).second.first.second->get_index())
		{
		  if (((*Ai).second.first.first->get_layer() == "A") && ((*Ai).second.first.second->get_layer() == "N") &&
		      (NOEtypeL.find(make_pair
				     (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HaHn))
		       == NOEtypeL.end()))
		    {
		      NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HaHn), &*iter));
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.second->get_index(), (*Ai).second.first.first->get_index()), HnHa), &*iter));
		    }
		}
	      else if (((*Ai).second.first.first->get_index() - 4) == (*Ai).second.first.second->get_index())
		{
		  if (((*Ai).second.first.first->get_layer() == "N") && ((*Ai).second.first.second->get_layer() == "A") &&
		      (NOEtypeL.find(make_pair
				     (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HnHa))
		       == NOEtypeL.end()))
		    {
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.second->get_index(), (*Ai).second.first.first->get_index()), HaHn), &*iter));
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HnHa), &*iter));
		    }
		}
		
		// long range
	 if (abs((*Ai).second.first.first->get_index() - (*Ai).second.first.second->get_index()) > 3)
		{
		  if ((((*Ai).second.first.first->get_layer() == "A") && ((*Ai).second.first.second->get_layer() == "N")) &&
		      (NOEtypeL.find(make_pair
				     (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HaHn))
		       == NOEtypeL.end()))
		    {
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HaHn), &*iter));
		    }
		  else if ((((*Ai).second.first.first->get_layer() == "N") && ((*Ai).second.first.second->get_layer() == "A")) &&
			   (NOEtypeL.find(make_pair
					  (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HnHa))
			    == NOEtypeL.end()))
		    {
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HnHa), &*iter));
		    }
		  else if (((*Ai).second.first.first->get_layer() == "N") && ((*Ai).second.first.second->get_layer() == "N") &&
			   (NOEtypeL.find(make_pair
					  (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HnHn))
			    == NOEtypeL.end()))
		    {
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HnHn), &*iter));
		    }
		  else if (((*Ai).second.first.first->get_layer() == "A") && ((*Ai).second.first.second->get_layer() == "A") &&
			   (NOEtypeL.find(make_pair
					  (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HaHa))
			    == NOEtypeL.end()))
		    {
		      NOEtypeL.insert(make_pair(make_pair
				      (make_pair((*Ai).second.first.first->get_index(), (*Ai).second.first.second->get_index()), HaHa), &*iter));
		    }
		}
	    }
	}
    }
}

void NOESY::Find_SSnoe(NOEtypeListN& NOEtypeL, const Sec& parSec)
{
  for(PeakMomDadMap::iterator iter = BList.begin(); iter != BList.end(); iter++)
    {
      
      bool assignedPair=0;
      bool assignedPeak=0;
      bool possPair=0;
      int assignedCount= 0;
      
      // can be used if only assigned once
      if ((AssignList.find((*iter).first) != AssignList.end()))
      {
          assignedPeak=1;  

        assignedCount = AssignList.count((*iter).first);
        PeakAssignReason::const_iterator rl = AssignList.lower_bound((*iter).first);
  			PeakAssignReason::const_iterator ru = AssignList.upper_bound((*iter).first);
  			for(PeakAssignReason::const_iterator ri = rl; ri != ru; ri++)
  			{
  			  if (((*ri).second.first.first == (*iter).second.first) &&
  			      ((*ri).second.first.second == (*iter).second.second))
          {
             assignedPair=1;
    			   break;
  				}
  			}
      }
      
      if (
	  ((*iter).second.first->get_Hdiff((*iter).first->get_HX1(), SWSF_HX1) < parSec.htol + 0.000001) &&
	  ((*iter).second.first->get_Xdiff((*iter).first->get_X1(), SWSF_X1) < parSec.xtol + 0.000001) &&
	  ((*iter).second.second->get_Hdiff((*iter).first->get_HX2(), SWSF_HX2) < parSec.htol + 0.000001) &&
	  (((*iter).first->get_D() == 3) || (*iter).second.second->get_Xdiff((*iter).first->get_X2(), SWSF_X2) < parSec.xtol + 0.000001))
          possPair=1;

    float score = 0;
	  if (possPair || assignedPair)
    {
        if (possPair && assignedPeak && !assignedPair && (assignedCount == 1))
	  {
	    if (NOESY_Assign::isEC())
	      score = 0.375; 
	    else
	      score = 0.25;
	  }
        else if (possPair && !assignedPeak)
          score = 1.0;
        else if (assignedPair)
          score = 1.25; 
    }
          
      if (DEBUG)
          NOESY_Assign::outputLog() << "in Find_SSnoe: " << iter->first << "\n" << iter->second.first << "\n" << iter->second.second << " " << score << endl
                                    << iter->second.first->get_index() << " " << iter->second.second->get_index() << endl << endl;

	if (score > 0)
	{
    // distance = 1
	  if (((*iter).second.first->get_index() + 1) == (*iter).second.second->get_index())
	    {
	    // helix 2.6-2.8
		if (((*iter).second.first->get_layer() == "N") && ((*iter).second.second->get_layer() == "N") &&
		    ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn)) == NOEtypeL.end()) ||
         ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn)) != NOEtypeL.end()) &&       
		    (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn))->second.second.first < score)))
		     && (pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0) < 4.5))
		  {
        if (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn)) != NOEtypeL.end())
        {
          NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.first->get_index(), iter->second.second->get_index()), HnHn)));
          NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.second->get_index(), iter->second.first->get_index()), HnHn)));
        }
        
		    NOEtypeL.insert(make_pair(make_pair(make_pair((*iter).second.first->get_index(),
								  (*iter).second.second->get_index()), HnHn), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		    NOEtypeL.insert(make_pair(make_pair(make_pair((*iter).second.second->get_index(),
								  (*iter).second.first->get_index()), HnHn), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		  }

		  // b-strand 2.2
		else if (((*iter).second.first->get_layer() == "A") && ((*iter).second.second->get_layer() == "N") &&
		    ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn)) == NOEtypeL.end()) ||
         ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn)) != NOEtypeL.end()) &&
		    (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn))->second.second.first < score)))
        && (pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0) < 4.5))
		  {
        if (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn)) != NOEtypeL.end())
        {
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.first->get_index(), iter->second.second->get_index()), HaHn)));
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.second->get_index(), iter->second.first->get_index()), HnHa)));
        }
        
        NOEtypeL.insert(make_pair(make_pair
				  (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HnHa), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		  }
	      }
	    else if (((*iter).second.first->get_index() - 1) == (*iter).second.second->get_index())
	      {
		if (((*iter).second.first->get_layer() == "N") && ((*iter).second.second->get_layer() == "N") &&
		    ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn)) == NOEtypeL.end()) ||
         ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn)) != NOEtypeL.end()) &&
		    (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn))->second.second.first < score)))
        && (pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0) < 4.5))
		  {
        if (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn)) != NOEtypeL.end())
        {
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.first->get_index(), iter->second.second->get_index()), HnHn)));
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.second->get_index(), iter->second.first->get_index()), HnHn)));
        }
        NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HnHn), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		  }
		else if (((*iter).second.first->get_layer() == "N") && ((*iter).second.second->get_layer() == "A") &&
		    ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa)) == NOEtypeL.end()) ||
         ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa)) != NOEtypeL.end()) &&
		    (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa))->second.second.first < score)))
       && (pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0) < 4.5))
		  {
        if (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa)) != NOEtypeL.end())
        {
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.first->get_index(), iter->second.second->get_index()), HnHa)));
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.second->get_index(), iter->second.first->get_index()), HaHn)));
        }
        
        NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HaHn), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
      }
	      }


	   // distance = 2
	    else if (((*iter).second.first->get_index() + 2) == (*iter).second.second->get_index())
	      {
		if (((*iter).second.first->get_layer() == "N") && ((*iter).second.second->get_layer() == "N") &&
		    ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn)) == NOEtypeL.end()) ||
         ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn)) != NOEtypeL.end()) &&
		    (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn))->second.second.first < score))))
		  {
        if (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn)) != NOEtypeL.end())
        {
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.first->get_index(), iter->second.second->get_index()), HnHn)));
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.second->get_index(), iter->second.first->get_index()), HnHn)));
        }

        NOEtypeL.insert(make_pair(make_pair
				  (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HnHn), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		  }
		else if (((*iter).second.first->get_layer() == "A") && ((*iter).second.second->get_layer() == "N") &&
		    ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn)) == NOEtypeL.end()) ||
         ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn)) != NOEtypeL.end()) &&
		    (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn))->second.second.first < score))))
      {
        if (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn)) != NOEtypeL.end())
        {
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.first->get_index(), iter->second.second->get_index()), HaHn)));
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.second->get_index(), iter->second.first->get_index()), HnHa)));
        }
      NOEtypeL.insert(make_pair(make_pair
				  (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		  NOEtypeL.insert(make_pair(make_pair
				  (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HnHa), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
	   }
	      }
	 else if (((*iter).second.first->get_index() - 2) == (*iter).second.second->get_index())
	      {
		if (((*iter).second.first->get_layer() == "N") && ((*iter).second.second->get_layer() == "N") &&
		    ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn)) == NOEtypeL.end()) ||
         ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn)) != NOEtypeL.end()) &&
		    (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn))->second.second.first < score))))
		  {
        if (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn)) != NOEtypeL.end())
        {
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.first->get_index(), iter->second.second->get_index()), HnHn)));
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.second->get_index(), iter->second.first->get_index()), HnHn)));
        }

        NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HnHn), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		  }
		else if (((*iter).second.first->get_layer() == "N") && ((*iter).second.second->get_layer() == "A") &&
		    ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa)) == NOEtypeL.end()) ||
         ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa)) != NOEtypeL.end()) &&
		    (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa))->second.second.first < score))))
		  {
        if (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa)) != NOEtypeL.end())
        {
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.first->get_index(), iter->second.second->get_index()), HnHa)));
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.second->get_index(), iter->second.first->get_index()), HaHn)));
        }
        
        NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HaHn), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
			    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		  }
	      }
	      // distance = 3
	    else if (((*iter).second.first->get_index() + 3) == (*iter).second.second->get_index())
	      {

	      // helix 3.3-3.4
		if (((*iter).second.first->get_layer() == "A") && ((*iter).second.second->get_layer() == "N") &&
		    ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn)) == NOEtypeL.end()) ||
         ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn)) != NOEtypeL.end()) &&
		    (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn))->second.second.first < score)))
        && (pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0) < 4.5))
		  {
        if (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn)) != NOEtypeL.end())
        {
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.first->get_index(), iter->second.second->get_index()), HaHn)));
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.second->get_index(), iter->second.first->get_index()), HnHa)));
        }
        NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HnHa), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		  }
		else if (((*iter).second.first->get_layer() == "A") && ((*iter).second.second->get_layer() == "B") &&
		    ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHb)) == NOEtypeL.end()) ||
         ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHb)) != NOEtypeL.end()) &&
		    (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHb))->second.second.first < score))))
		  {
        if (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHb)) != NOEtypeL.end())
        {
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.first->get_index(), iter->second.second->get_index()), HaHb)));
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.second->get_index(), iter->second.first->get_index()), HbHa)));
        }
        NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHb), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HbHa), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		  }
	      }
	    else if (((*iter).second.first->get_index() - 3) == (*iter).second.second->get_index())
	      {
		if (((*iter).second.first->get_layer() == "N") && ((*iter).second.second->get_layer() == "A") &&
		    ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa)) == NOEtypeL.end()) ||
         ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa)) != NOEtypeL.end()) &&
		    (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa))->second.second.first < score)))
        && (pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0) < 4.5))
		  {
        if (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa)) != NOEtypeL.end())
        {
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.first->get_index(), iter->second.second->get_index()), HnHa)));
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.second->get_index(), iter->second.first->get_index()), HaHn)));
        }
        NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HaHn), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		  }
		else if (((*iter).second.first->get_layer() == "B") && ((*iter).second.second->get_layer() == "A") &&
		    ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HbHa)) == NOEtypeL.end()) ||
         ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HbHa)) != NOEtypeL.end()) &&
		    (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HbHa))->second.second.first < score))))
		  {
        if (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HbHa)) != NOEtypeL.end())
        {
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.first->get_index(), iter->second.second->get_index()), HbHa)));
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.second->get_index(), iter->second.first->get_index()), HaHb)));
        }
        NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HaHb), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HbHa), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		  }
	      }

	      // distance = 4
	    else if (((*iter).second.first->get_index() + 4) == (*iter).second.second->get_index())
	      {
		if (((*iter).second.first->get_layer() == "A") && ((*iter).second.second->get_layer() == "N") &&
        ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn)) == NOEtypeL.end()) ||
         ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn)) != NOEtypeL.end()) &&
		    (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn))->second.second.first < score))))
		  {
        if (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn)) != NOEtypeL.end())
        {
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.first->get_index(), iter->second.second->get_index()), HaHn)));
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.second->get_index(), iter->second.first->get_index()), HnHa)));
        }
        NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HnHa), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		  }
	      }
	    else if (((*iter).second.first->get_index() - 4) == (*iter).second.second->get_index())
	      {
		if (((*iter).second.first->get_layer() == "N") && ((*iter).second.second->get_layer() == "A") &&
		    ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa)) == NOEtypeL.end()) ||
         ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa)) != NOEtypeL.end()) &&
		    (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa))->second.second.first < score))))
		  {
        if (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa)) != NOEtypeL.end())
        {
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.first->get_index(), iter->second.second->get_index()), HnHa)));
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.second->get_index(), iter->second.first->get_index()), HaHn)));
        }
        NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.second->get_index(), (*iter).second.first->get_index()), HaHn), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		  }
	      }

	      // one-way, not symmmetric
	    if (abs((*iter).second.first->get_index() - (*iter).second.second->get_index()) > 3)
	      {
	      //hnha sheet 3.0-3.2
		if ((((*iter).second.first->get_layer() == "N") && ((*iter).second.second->get_layer() == "A")) &&
		    ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa)) == NOEtypeL.end()) ||
         ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa)) != NOEtypeL.end()) &&
		    (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa))->second.second.first < score))))
		  {
       if (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa)) != NOEtypeL.end())
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.first->get_index(), iter->second.second->get_index()), HnHa)));

        NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHa), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));

		  }

		  // hahn sheet 3.0-3.2
		else if ((((*iter).second.first->get_layer() == "A") && ((*iter).second.second->get_layer() == "N")) &&
		    ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn)) == NOEtypeL.end()) ||
         ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn)) != NOEtypeL.end()) &&
		    (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn))->second.second.first < score))))
		  {
        if (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn)) != NOEtypeL.end())
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.first->get_index(), iter->second.second->get_index()), HaHn)));
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHn), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		  }

		  // HnHn for sheets 3.3-4.0
		else if (((*iter).second.first->get_layer() == "N") && ((*iter).second.second->get_layer() == "N") &&
		    ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn)) == NOEtypeL.end()) ||
         ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn)) != NOEtypeL.end()) &&
		    (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn))->second.second.first < score))))
		  {
        if (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn)) != NOEtypeL.end())
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.first->get_index(), iter->second.second->get_index()), HnHn)));
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HnHn), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		  }

		  // HaHa < 2.3 A for antiparallel strands only
		else if (((*iter).second.first->get_layer() == "A") && ((*iter).second.second->get_layer() == "A") &&
 	    ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHa)) == NOEtypeL.end()) ||
         ((NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHa)) != NOEtypeL.end()) &&
		    (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHa))->second.second.first < score))))

		  {
        if (NOEtypeL.find(make_pair(make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHa)) != NOEtypeL.end())
        NOEtypeL.erase(NOEtypeL.find(make_pair(make_pair(iter->second.first->get_index(), iter->second.second->get_index()), HaHa)));
		    NOEtypeL.insert(make_pair(make_pair
				    (make_pair((*iter).second.first->get_index(), (*iter).second.second->get_index()), HaHa), make_pair(iter->first, make_pair(score, pow((double)get_KC()/(*iter).first->get_Intensity(), (double)1.0/6.0)))));
		  }
	      }
	  }
    }
}
