/***************************************************************************
                          experiment_Sym.cpp  -  description
                             -------------------
    begin                : Mon Oct 28 2002
    copyright            : (C) 2002 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#include <iostream>
#include <sstream>
#include <cmath>
#include <iomanip>
#include "experiment.h"
#include "noesy_assign.h"
#include "par.h"

#define swap(a,b) ({ typeof(a) temp = (a); (a) = (b); (b) = temp; })

extern bool DEBUG;
extern int faces[1000][1000];

SampleStatistic getDistStat(deque<Pdb>& , pair<const atom*, const atom*>, bool);
float sumAverage(const SampleStatistic&);

typedef map<int, AA, less<int> > SEQM;
typedef map<pair<int, int>, float, less<pair<int, int> > > matrix;

string itoa(int);

typedef pair<int, int> respair;
typedef multiset<respair, less<respair> > resSet;

typedef multimap<const general_peak* const, const general_peak* const, less<const general_peak* const> > PeakPairMap;
typedef multimap<MomDad, MomDad, less<MomDad> > MomDadMap;

typedef const general_peak* PeakPtr;
typedef multimap<PeakPtr, MomDad, less<PeakPtr> > PeakPtrMomDadMap;
typedef set<PeakPtr, less<PeakPtr> > PeakPtrMomDadSet;

// complete-sysm 8 peaks or 4 peaks
int NOESY::Assign_4SYM(const Sym& parSym, const MomDadSet& mdSet4, const MomDadSet& mdSetO, const MomDadSet& mdSetM2, const MomDadPeakAtomMap& mdPAMapM2,SUPPORT& support)
{
  if (DEBUG) NOESY_Assign::outputLog() << "\n-> In Assign_4SYM" << endl;

  ostringstream outStream;

	int old = AssignList.size();
	int unOld = 0;
	for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
	{
		if (AssignList.find(&(*iter)) == AssignList.end())
			unOld++;
	}

	int wN = 0; int iN = 0;

	MomDadPeakMap assignMap; assignMap.clear();
 	for(MomDadSet::iterator mdsi = mdSet4.begin(); mdsi != mdSet4.end(); ++mdsi)
 	{
 		MomDadSet ASset; ASset.clear();
   		bool ok = 1;
		bool in = 0; 
		for(MomDadPeakAtomMap::const_iterator mdmi = mdPAMapM2.lower_bound(*mdsi); mdmi != mdPAMapM2.upper_bound(*mdsi); ++mdmi)
		{
			// check if this in this peak list 
			if (BList.find(mdmi->second.first) == BList.end())
				continue;
		
			in = 1; 
			if (AssignList.find(mdmi->second.first) == AssignList.end()) // 1) assigned peak list
 				ASset.insert(mdmi->second.second);

			// match tolerance
			if ((mdmi->second.second.first->get_Hdiff(mdmi->second.first->get_HX1(), mdmi->second.first->SWSF_HX1()) < parSym.htol + 0.000001) &&
				(mdmi->second.second.second->get_Hdiff(mdmi->second.first->get_HX2(), mdmi->second.first->SWSF_HX2()) < parSym.htol + 0.000001) &&
				(mdmi->second.second.first->get_Xdiff(mdmi->second.first->get_X1(), mdmi->second.first->SWSF_X1()) < parSym.xtol + 0.000001) &&
				(mdmi->second.second.second->get_Xdiff(mdmi->second.first->get_X2(), mdmi->second.first->SWSF_X2()) < parSym.xtol + 0.000001))
 			{
				if (AssignList.find(mdmi->second.first) != AssignList.end())
					ok = 0;
				
 	  		} else
 	  		ok = 0;
 	  	} // end for

		ok *= in; 
		
 	if (ok)
	{
  		for(MomDadPeakAtomMap::const_iterator mdmi = mdPAMapM2.lower_bound(*mdsi); mdmi != mdPAMapM2.upper_bound(*mdsi); ++mdmi)
 		{
			if (BList.find(mdmi->second.first) == BList.end())
				continue;
  			
			assignMap.insert(make_pair(mdmi->second.second, mdmi->second.first));
 		}
    	}
 	}

	MakeAssignMap(assignMap, SYM4, wN, iN);

	///////////////////////////////////////////////
	//  comple 2-SYM (not have sub-1-SYM group)
	//////////////////////////////////////////////
	assignMap.clear();
	
 	for(MomDadSet::iterator mdsi = mdSetO.begin(); mdsi != mdSetO.end(); ++mdsi)
 	{
		if (DEBUG)
			NOESY_Assign::outputLog() << mdsi->first << endl << mdsi->second << endl << endl; 
		
   		bool ok = 1; bool in = 0; 
		for(MomDadPeakAtomMap::const_iterator mdmi = mdPAMapM2.lower_bound(*mdsi); mdmi != mdPAMapM2.upper_bound(*mdsi); ++mdmi)
		{
			if (DEBUG) NOESY_Assign::outputLog() << mdmi->second.first << endl; 
			// check if this in this peak list 
			if (BList.find(mdmi->second.first) == BList.end())
				continue;
		
			in = 1; 
			
			if (DEBUG) NOESY_Assign::outputLog() << "in BList" << endl;
			
			if (AssignList.find(mdmi->second.first) != AssignList.end()) // 1) assigned
 				ok = 0;

			// match tolerance
			if ((mdmi->second.second.first->get_Hdiff(mdmi->second.first->get_HX1(), mdmi->second.first->SWSF_HX1()) < parSym.htol + 0.000001) &&
				(mdmi->second.second.second->get_Hdiff(mdmi->second.first->get_HX2(), mdmi->second.first->SWSF_HX2()) < parSym.htol + 0.000001) &&
				(mdmi->second.second.first->get_Xdiff(mdmi->second.first->get_X1(), mdmi->second.first->SWSF_X1()) < parSym.xtol + 0.000001) &&
				(mdmi->second.second.second->get_Xdiff(mdmi->second.first->get_X2(), mdmi->second.first->SWSF_X2()) < parSym.xtol + 0.000001))
 			{
				if (AssignList.find(mdmi->second.first) != AssignList.end())
					ok = 0;
 	  	} else
 	  		ok = 0;
 	  	} // end for
		
		ok *= in; 

 	  if (ok)
		{
    	// assignment all in this case
   		for(MomDadPeakAtomMap::const_iterator mdmi = mdPAMapM2.lower_bound(*mdsi); mdmi != mdPAMapM2.upper_bound(*mdsi); ++mdmi)
   		{
			if (BList.find(mdmi->second.first) == BList.end())
				continue;
  	  	
			assignMap.insert(make_pair(mdmi->second.second, mdmi->second.first));
   		}
    }
 	}

	MakeAssignMap(assignMap, SYM2, wN, iN);

	if (AssignList.size() - old == 0)
    return 0;

	int un = 0;
	for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
	{
		if (AssignList.find(&(*iter)) == AssignList.end())
			un++;
	}

  outStream.setf(ios::left, ios::adjustfield);
  outStream.width(6);
  outStream << (itoa(NOESY_Assign::ddcycle())+"-"+itoa(NOESY_Assign::dcycle())).c_str();
  outStream.width(7);
  outStream << "SYM4";
  outStream.width(6);
  outStream << unOld - un; // new peak assigned 
  outStream.width(8);
  outStream << peakList.size() - un;
  outStream.width(7);
  outStream << AssignList.size() - old; // new assignments 
  outStream.width(9);
  outStream << AssignList.size();
  outStream.width(19);
  outStream << "-";
  outStream.width(18);
  outStream << un;
  outStream.width(11);
  outStream << noiseList.size();

  if (NOESY_Assign::COMPARE())
  {
  outStream.width(8);
    outStream << "-";
  outStream.width(11);
  if (NOESY_Assign::COMPARE())
    outStream << AssignList.size() - old - iN;
  else
    outStream << "-";
  outStream.width(11);
  if (NOESY_Assign::COMPARE())
    outStream << iN;
  else
    outStream << "-";
  }
  outStream << "\n" << NOESY_Assign::EOS();
  string temp(outStream.str());
  reportProcessStat += temp.substr(0,temp.find(NOESY_Assign::EOS()));
  return AssignList.size() - old;
}

int NOESY::Assign_SYM(int low, const Sym& parSym, const MomDadPeakAtomMap& mdPAMapA, map<MomDad, MomDad, less<MomDad> >& mdAM2,
const MomDadSet& mdSetA12, const MomDadSet& mdSetM2, const MomDadPeakAtomMap& mdPAMapM2, SUPPORT& support)
{
  if (DEBUG) NOESY_Assign::outputLog() << "\n-> In Assign_SYM" << endl;

  ostringstream outStream;

	int old = AssignList.size();
	int unOld = 0;
	for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
	{
		if (AssignList.find(&(*iter)) == AssignList.end())
			unOld++;
	}

	int wN = 0; int iN = 0;

	// remove the one that have previously assigned but not unique sym anymore 
	for(PeakAssignReason::iterator i = AssignList.begin(); i != AssignList.end(); ++i)
	  {
	    if (i->second.second.find(SYM) != i->second.second.end())
	      {
			
		if ((mdSetA12.find(i->second.first) == mdSetA12.end()) && 
		    (mdSetA12.find(make_pair(i->second.first.second, i->second.first.first)) == mdSetA12.end()))
		  {
		    i->second.second.erase(i->second.second.find(SYM));
		  } else
		    {
		      MakeAssign(i->first, i->second.first.first, i->second.first.second, OLD); // label it as OLD
		    }
	      }
	  }

	
	//MomDadSet tmdSetA12; tmdSetA12 = mdSetA12;
	//MomDadSet ttSetA; ttSetA = tmdSetA12;
	MomDadSet tmdSetA12(mdSetA12); 
	MomDadSet ttSetA(tmdSetA12); 
	
	// avoid repeated assignments
	for(MomDadSet::iterator mdsi = ttSetA.begin(); mdsi != ttSetA.end(); ++mdsi)
	{
		if (DEBUG)
			NOESY_Assign::outputLog() << "varify " << mdsi->first << endl << mdsi->second << endl;

		//bool goodSym=1;
		//bool notOK=0;
		bool old = 0;

  		for(MomDadPeakAtomMap::const_iterator mdpi = mdPAMapM2.lower_bound(mdAM2[*mdsi]); mdpi != mdPAMapM2.upper_bound(mdAM2[*mdsi]); ++mdpi)
 		{
			if (BList.find(mdpi->second.first) == BList.end())
				continue;
		
			for(PeakAssignReason::iterator i = BufferReason.lower_bound(mdpi->second.first); i != BufferReason.upper_bound(mdpi->second.first); ++i)
			{
				if ((i->second.first.first == mdpi->second.second.first) && (i->second.first.second == mdpi->second.second.second))
				{
					if ((i->second.second.count(SYM)) || (i->second.second.count(PDB)) || (i->second.second.count(DEL_TOP)) 
					    || (i->second.second.count(DEL_SF)) || (i->second.second.count(BEST)) || (i->second.second.count(DEL_SAM)))
						old = 1;
						
					if (DEBUG && old)
					{
		    				NOESY_Assign::outputLog() << " B: " ;
    						for(setMethod::iterator si = (*i).second.second.begin(); si != (*i).second.second.end(); si++)
						{
    							NOESY_Assign::outputLog() << (*si) << " ";
						}
					}

					break;
				}
			}
		}
    		if (old)
    		{
    			if (DEBUG)
     				NOESY_Assign::outputLog() << "old remove ";

 			if (tmdSetA12.find(*mdsi) != tmdSetA12.end())  // studied before 
 				tmdSetA12.erase(tmdSetA12.find(*mdsi));
 		}
    		if (DEBUG)
     			NOESY_Assign::outputLog() << endl;
	}
	

// 1. supported by two bonds (assign all?) and complete, and also supported by +/- 2 residues
MomDadPeakMap assignMap; assignMap.clear();
ContactPairSet AssignedContactBin; AssignedContactBin.clear();
int oldsize = 0;
do
{
	oldsize = AssignedContactBin.size();

  for(MomDadSet::const_iterator mdsi = tmdSetA12.begin(); mdsi != tmdSetA12.end(); ++mdsi)
	{
		if (DEBUG)
			NOESY_Assign::outputLog() << "find SUPPORT " << mdsi->first << endl << mdsi->second << endl;

 		MomDadSet ASset; ASset.clear();
		MomDadSet UASset; UASset.clear();
 		for(MomDadPeakAtomMap::const_iterator mdmi = mdPAMapM2.lower_bound(mdAM2[*mdsi]); mdmi != mdPAMapM2.upper_bound(mdAM2[*mdsi]); ++mdmi)
 		{
			if (BList.find(mdmi->second.first) == BList.end())
				continue;
			
			if (DEBUG)
				NOESY_Assign::outputLog() << "++" << mdmi->second.first << endl;

 			ASset.insert(mdmi->second.second);

			PeakAssignReason::iterator rl = BufferReason.lower_bound(mdmi->second.first);
	  		PeakAssignReason::iterator ru = BufferReason.upper_bound(mdmi->second.first);

	 	 	for(PeakAssignReason::iterator ri = rl; ri != ru; ri++)
	 		{
				if (ri->second.second.count(U) != 0) // Unique
				{
					UASset.insert(mdmi->second.second);
				}
			}
 		}


   	pair<int, int> mij;
 		if (mdsi->first->get_index() < mdsi->second->get_index())
 			mij = make_pair(mdsi->first->get_index(), mdsi->second->get_index());
 		else
 			mij = make_pair(mdsi->second->get_index(), mdsi->first->get_index());

 		bool bb = 0;
		if (((mdsi->first->get_layer() == "N") || (mdsi->first->get_layer() == "A")) &&
           				((mdsi->second->get_layer() == "N") || (mdsi->second->get_layer() == "A")))
    {
    	bb = 1;
    }

		/////////////////////
		// make assignment
		////////////////////
 		if ( 
   			(((low > 0) && NOESY_Assign::MINCON()) || isSupported(low, *mdsi, ASset, mdPAMapM2, support, AssignedContactBin)) ||
		 		// one unique sym pair and compelte
		    (((UASset.size()) || bb) && (low > 0) && isSupported(mij, support.ContactBinUniqueSYM)))
 		{

		  if (DEBUG)
		    NOESY_Assign::outputLog() << "Good" << endl;
		      {
			//if (low > 0)
			{
			  for(MomDadPeakAtomMap::const_iterator mdmi = mdPAMapA.lower_bound(*mdsi); mdmi != mdPAMapA.upper_bound(*mdsi); ++mdmi)
			    {
			      	if (BList.find(mdmi->second.first) == BList.end())
				continue;

			      // match tolerance
			      if ((mdmi->second.second.first->get_Hdiff(mdmi->second.first->get_HX1(), mdmi->second.first->SWSF_HX1()) < parSym.htol + 0.000001) &&
				  (mdmi->second.second.second->get_Hdiff(mdmi->second.first->get_HX2(), mdmi->second.first->SWSF_HX2()) < parSym.htol + 0.000001) &&
				  (mdmi->second.second.first->get_Xdiff(mdmi->second.first->get_X1(), mdmi->second.first->SWSF_X1()) < parSym.xtol + 0.000001) &&
				  (mdmi->second.second.second->get_Xdiff(mdmi->second.first->get_X2(), mdmi->second.first->SWSF_X2()) < parSym.xtol + 0.000001))
				{
				  if (DEBUG)
				    NOESY_Assign::outputLog() << "Assign SYM " << mdmi->second.second.first << endl << mdmi->second.second.second << endl << mdmi->second.first << endl;
				  assignMap.insert(make_pair(mdmi->second.second, mdmi->second.first));
				  AssignedContactBin.insert(mij);
				}
			    } // end for
			}
		      } 
		  } // end if
	}
}	while (AssignedContactBin.size() > oldsize);

	MakeAssignMap(assignMap, SYM, wN, iN);

	if (AssignList.size() - old == 0)
    return 0;

	int un = 0;
	for(PeakList::iterator iter = peakList.begin(); iter != peakList.end(); iter++)
	{
		if (AssignList.find(&(*iter)) == AssignList.end())
			un++;
	}

  outStream.setf(ios::left, ios::adjustfield);
  outStream.width(6);
  outStream << (itoa(NOESY_Assign::ddcycle())+"-"+itoa(NOESY_Assign::dcycle())).c_str();
  outStream.width(7);
  outStream << "SYM";
  outStream.width(6);
  outStream << unOld - un;
  outStream.width(8);
  outStream << peakList.size() - un;
  outStream.width(7);
  outStream << AssignList.size() - old;
  outStream.width(9);
  outStream << AssignList.size();
  outStream.width(19);
  outStream << "-";
  outStream.width(18);
  outStream << un;
  outStream.width(11);
  outStream << noiseList.size();

  if (NOESY_Assign::COMPARE())
  {
  outStream.width(8);
    outStream << "-";
  outStream.width(11);
  if (NOESY_Assign::COMPARE())
    outStream << AssignList.size() - old - iN;
   else
    outStream << "-";
  outStream.width(11);
  if (NOESY_Assign::COMPARE())
    outStream << iN;
  else
    outStream << "-";
  }
  outStream << "\n" << NOESY_Assign::EOS();
  string temp(outStream.str());
  reportProcessStat += temp.substr(0,temp.find(NOESY_Assign::EOS()));

  return AssignList.size() - old;
}

