/***************************************************************************
                          faces.cpp  -  description
                             -------------------
    begin                : Tue May 13 2003
    copyright            : (C) 2003 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#include <set>
#include <cmath>

#include "faces.h"
#include "ss.h"
#include "AA.h"
#include "SmplStat.h"

using namespace std;

#define abs(x) abs((double)(x))

extern bool DEBUG;

Faces::Faces(){
	resetSS();
	resetCT();
}

Faces::~Faces(){
}

void Faces::resetSS()
{
	helixFaces.clear();
	sheetFaces.clear();
	helixLocalFaces.clear();
	sheetLocalFaces.clear();
}

void Faces::resetCT()
{
	ctAtoms.clear();
	ctFaces.clear();
	ctHelixFaces.clear();
	ctSheetFaces.clear();
	ctHelixLocalFaces.clear();
	ctSheetLocalFaces.clear();
}

void Faces::insertHelix(int i, int j, int v)
{
	if (helixFaces.find(make_pair(i,j)) == helixFaces.end())
	{
		helixFaces.insert(make_pair(make_pair(i,j), v));
		helixFaces.insert(make_pair(make_pair(j,i), v));
	}	
	else if (helixFaces[make_pair(i,j)] != v)
		error("conflict helix topology ", itoa(j) + " " + itoa(i));
}

void Faces::insertHelixLocal(int i, int j, int v)
{
	if (helixLocalFaces.find(make_pair(i,j)) == helixLocalFaces.end())
	{
		helixLocalFaces.insert(make_pair(make_pair(i,j), v));
		helixLocalFaces.insert(make_pair(make_pair(j,i), v));
	}
	else if (helixLocalFaces[make_pair(i,j)] != v)
		error("conflict helix local topology ", itoa(j) + " " + itoa(i));
}

void Faces::insertSheetLocal(int i, int j, int v)
{
	if (sheetLocalFaces.find(make_pair(i,j)) == sheetLocalFaces.end())
	{
		sheetLocalFaces.insert(make_pair(make_pair(i,j), v));
		sheetLocalFaces.insert(make_pair(make_pair(j,i), v));
	}
	else if (sheetLocalFaces[make_pair(i,j)] != v)
		error("conflict sheet local topology ", itoa(j) + " " + itoa(i));
}

void Faces::insertSheet(int i, int j, int v)
{
	if (sheetFaces.find(make_pair(i,j)) == sheetFaces.end())
	{
		sheetFaces.insert(make_pair(make_pair(i,j), v));
		sheetFaces.insert(make_pair(make_pair(j,i), v));
	}	
	else if (sheetFaces[make_pair(i,j)] != v)
		error("conflict sheet topology ", itoa(j) + " " + itoa(i));
}

void Faces::insertCTatoms(int i, string ia, int j, string ja)
{
		ctAtoms.insert(make_pair(make_pair(i,j), make_pair(ia, ja)));
		ctAtoms.insert(make_pair(make_pair(j,i), make_pair(ja, ia)));		
}

void Faces::insertCT()
{
	for(set<pair<pair<int, int>, pair<string, string> > >:: iterator ci = ctAtoms.begin() ; ci != ctAtoms.end(); ++ci)
	{
		ctFaces.insert(make_pair(ci->first, 1));
	}
}

int Faces::get_helixFace(int i, int j)
{
	if (helixFaces.find(make_pair(i,j)) != helixFaces.end())
		return helixFaces[make_pair(i,j)];
	return 0;
}

int Faces::get_sheetFace(int i, int j)
{
	if (sheetFaces.find(make_pair(i,j)) != sheetFaces.end())
	{
		return sheetFaces[make_pair(i,j)];
	}
	return 0;
}
	
int Faces::get_ctFace(int i, int j)
{
	if (ctFaces.find(make_pair(i,j)) != ctFaces.end())
		return ctFaces.find(make_pair(i,j))->second;
	return 0;
}

int Faces::get_ctHelixLocalFace(int i, int j)
{
	if (ctHelixLocalFaces.find(make_pair(i,j)) != ctHelixLocalFaces.end())
		return ctHelixLocalFaces[make_pair(i,j)];
	return 0;
}

int Faces::get_ctSheetLocalFace(int i, int j)
{
	if (ctSheetLocalFaces.find(make_pair(i,j)) != ctSheetLocalFaces.end())
		return ctSheetLocalFaces[make_pair(i,j)];
	return 0;
}

int Faces::get_ctHelixPlaneFace(int i, int j)
{
	if (ctHelixFaces.find(make_pair(i,j)) != ctHelixFaces.end())
		return ctHelixFaces[make_pair(i,j)];
	return 0;
}

int Faces::get_ctSheetPlaneFace(int i, int j)
{
	if (ctSheetFaces.find(make_pair(i,j)) != ctSheetFaces.end())
	{
		return ctSheetFaces[make_pair(i,j)];
	}
	return 0;
}

// only initialized once
void Faces::extendSS()
{	
	set<int> indexSet; indexSet.clear();
	for(map<pair<int,int>, int>::iterator i = sheetFaces.begin(); i != sheetFaces.end(); ++i)
	{
		indexSet.insert(i->first.first);
		indexSet.insert(i->first.second);
	}
	int newc = 0;
	do
	{
		newc = 0;
		for(set<int>::iterator i=indexSet.begin(); i!=indexSet.end(); ++i) // row
		for(set<int>::iterator j=indexSet.begin(); j!=indexSet.end(); ++j) // column
  	{       
  		set<int> productVec; productVec.clear();

  		for(set<int>::iterator k=indexSet.begin(); k!=indexSet.end(); ++k)
  		{
			if (sheetFaces.find(make_pair(*i,*k)) == sheetFaces.end())
				continue;
			if (sheetFaces.find(make_pair(*k,*j)) == sheetFaces.end())
				continue;

			if (DEBUG)
			{
			  cerr<< "extendSS " << *i << " " << *k << " " << *j << " " << sheetFaces[make_pair(*i,*k)] << " " << sheetFaces[make_pair(*k, *j)] << endl;
			}
  			productVec.insert(sheetFaces[make_pair(*i, *k)]*sheetFaces[make_pair(*k,*j)]); // 1x1=1 and -1*-1 = 1;
  		}

		if (!productVec.size())
			continue;
		
		int max = *(productVec.rbegin());
		int min = *(productVec.begin());
	
    	if (max*min != 1) // different faces 
     	{
  			error("Error1: inconsist sheet secondary structure:", itoa(*i) + " " + itoa(*j));
      }
       
     	if (sheetFaces.find(make_pair(*i, *j)) == sheetFaces.end()) // new
     	{
     	sheetFaces.insert(make_pair(make_pair(*i,*j), min)); 
       	++newc;
    	}
     	else
      {
      	if (sheetFaces[make_pair(*i,*j)] != min)
	{
		if (DEBUG)
		{
			cerr << sheetFaces[make_pair(*i,*j)] << " " << min << endl;
		}
	  			error("Error2: inconsist sheet secondary structure:", itoa(*i) + " " + itoa(*j));
	}
      }
  	}
	} while (newc);
}

// iterative generated
void Faces::combineSSCT()
{
	set<int> indexSet; indexSet.clear();
		
	set<int> indexCTSet; indexCTSet.clear();
	for(map<pair<int,int>, int>::iterator i = ctFaces.begin(); i != ctFaces.end(); ++i)
	{
		indexCTSet.insert(i->first.first);
		indexCTSet.insert(i->first.second);
	}
	set<int> indexSheetSet; indexSheetSet.clear();
	for(map<pair<int,int>, int>::iterator i = sheetFaces.begin(); i != sheetFaces.end(); ++i)
	{
		indexSheetSet.insert(i->first.first);
		indexSheetSet.insert(i->first.second);
	}
	set<int> indexHelixSet; indexHelixSet.clear();
	for(map<pair<int,int>, int>::iterator i = helixFaces.begin(); i != helixFaces.end(); ++i)
	{
		indexHelixSet.insert(i->first.first);
		indexHelixSet.insert(i->first.second);
	}

	// check the consistency of CTset, if inconsistent remove
	// sequencial
	set<pair<int, int> > inconSet; inconSet.clear();
	
		for(set<int>::iterator i=indexCTSet.begin(); i!=indexCTSet.end(); ++i) // row
		for(set<int>::iterator j=indexSheetSet.begin(); j!=indexSheetSet.end(); ++j) // column
		{
				// 133-91
  			if (ctFaces.find(make_pair(*i,*j)) == ctFaces.end())
  				continue;

      	//i-j, i-j+1, and j is in b-sheet, j+1 may or may not
       
       	// 133-92
      	if ((ctFaces.find(make_pair(*i, *j+1)) != ctFaces.end())) // maybe inconsist
				{
					if (indexSheetSet.find(*j+1) != indexSheetSet.end())
					{
						inconSet.insert(make_pair(*i, *j));
						inconSet.insert(make_pair(*i, *j+1));
						inconSet.insert(make_pair(*j, *i));
						inconSet.insert(make_pair(*j+1, *i));
						
						if (DEBUG)
						{
			  			cerr<< "**Inconsistent-contact: " << *i << " " << *j << " and " << *i << " " << *j+1 << endl;
			     	}
						continue;
					}
					else
					{
						// j-1 have contact with j+1,
						if ((indexSheetSet.find(*j-1) != indexSheetSet.end()) && (ctFaces.find(make_pair(*j-1,*j+1)) != ctFaces.end()))
						{

						inconSet.insert(make_pair(*i, *j));
						inconSet.insert(make_pair(*i, *j+1));
						inconSet.insert(make_pair(*j, *i));
						inconSet.insert(make_pair(*j+1, *i));

							if (DEBUG)
							{
			  				cerr<< "**Inconsistent-contact: " << *i << " " << *j << " and " << *i << " " << *j+1 << endl;
			     		}
							continue;
						}
					}
				}

				// 133, 90
      	if ((ctFaces.find(make_pair(*i, *j-1)) != ctFaces.end())) // maybe inconsist
				{
					if (indexSheetSet.find(*j-1) != indexSheetSet.end())
					{
						if (DEBUG)
						{
			  			cerr<< "**Inconsistent-contacts: " << *i << " " << *j << " and " << *i << " " << *j-1 << endl;
			     	}
						inconSet.insert(make_pair(*i, *j));
						inconSet.insert(make_pair(*i, *j-1));
						inconSet.insert(make_pair(*j, *i));
						inconSet.insert(make_pair(*j-1, *i));
						continue;
					}
					else
					{
						// j-1 have contact with j+1,- 92
						if ((indexSheetSet.find(*j+1) != indexSheetSet.end()) && (ctFaces.find(make_pair(*j-1,*j+1)) != ctFaces.end()))
						{
						if (DEBUG)
						{
			  			cerr<< "**Inconsistent-contacts sheet toplogy: " << *i << " " << *j << " and " << *i << " " << *j-1 << endl;
			     	}
						inconSet.insert(make_pair(*i, *j));
						inconSet.insert(make_pair(*i, *j-1));
						inconSet.insert(make_pair(*j, *i));
						inconSet.insert(make_pair(*j-1, *i));
							continue;
						}
					}
				} 
		}

		//37-35, 37-90, 35-90=-1
		for(set<int>::iterator i=indexCTSet.begin(); i!=indexCTSet.end(); ++i) // row
		{
			set<int> jset; jset.clear();
			for(set<int>::iterator j=indexSheetSet.begin(); j!=indexSheetSet.end(); ++j) // column
			{
  				if (ctSheetFaces.find(make_pair(*i,*j)) == ctSheetFaces.end())
  					continue;
      
      				jset.insert(*j);
      			}

      			set<int> inset; inset.clear();
      for(set<int>::iterator ji = jset.begin(); ji != jset.end(); ++ji)
      {
      	for(set<int>::iterator jj = jset.begin(); jj != jset.end(); ++jj)
       	{
        	if (ctSheetFaces.find(make_pair(*ji, *jj)) == ctSheetFaces.end())
         		continue;
          if (ctSheetFaces[make_pair(*ji, *jj)] == -1)
          {
          	inset.insert(*ji);
           	inset.insert(*jj);
          }
        }
      }

      for(set<int>::iterator ji = inset.begin(); ji != inset.end(); ++ji)
      {
          	inconSet.insert(make_pair(*i, *ji));
            inconSet.insert(make_pair(*ji, *i));
						if (DEBUG)
						{
			  			cerr<< "**Inconsistent-contact-2: " << *i << " " << *ji << endl;
			     	}

		continue;

	}

	}

	// each incon i -j , i can not inteact withe other residues form j's sheet
	for(set<pair<int, int> >::iterator k = inconSet.begin(); k != inconSet.end(); ++k)
	{
		ctFaces.erase(ctFaces.lower_bound(*k), ctFaces.upper_bound(*k));

		if (DEBUG)
		{
			cerr<< "**Inconsistent-contact removed-final: " << k->first << " " << k->second << endl;
		}
	}
		
	// have to have two b-strand contact
	multimap<int, int> okHelix; okHelix.clear();
	set<pair<int,int> >hpkey; hpkey.clear();
	set<int> key; key.clear();
	multimap<pair<int, int>, int > hpset; hpset.clear();
	for(set<int>::iterator j=indexHelixSet.begin(); j!=indexHelixSet.end(); ++j)
	{
		for(set<int>::iterator i=indexSheetSet.begin(); i!=indexSheetSet.end(); ++i)
		{
			if (ctFaces.find(make_pair(*j,*i)) != ctFaces.end())
			{
				int h1 = seq[*i].get_ss()->get_strand_ID();
				int h2 = seq[*j].get_ss()->get_strand_ID();
				hpset.insert(make_pair(make_pair(h2,h1), *i));
				hpkey.insert(make_pair(h2,h1));
				key.insert(h2);
			}
		}
	}
	//
	
	for(set<pair<int, int> >::iterator i = hpkey.begin(); i != hpkey.end(); ++i)
	{
		if (hpset.count(*i) < 2)
		   continue;
			//okHelix.erase(okHelix.find(make_pair(i->first, i->second)));
  		else 
		{
			bool conflict = 0;
		for(multimap<pair<int, int>, int>::iterator k=hpset.lower_bound(*i); k!=hpset.upper_bound(*i); ++k)
		{
			for(multimap<pair<int, int>, int>::iterator m=hpset.lower_bound(*i); m != hpset.upper_bound(*i); ++m)
  			{
				if (k == m) continue;
				if (ctSheetFaces.find(make_pair(k->second, m->second)) == ctSheetFaces.end())
					continue;

				// if = -1, delete the whole helix 
  				if (ctSheetFaces.find(make_pair(k->second, m->second))->second == -1)
				{
					conflict = 1;
					break;
				}	
			}

			if (conflict)
				break;
		}	
		if (!conflict)
			okHelix.insert(make_pair(i->first, i->second));

		}
	}
	
//	// two sheet must be adjacent


  	for(set<int>::iterator i = key.begin(); i != key.end(); ++i)
  	{
				if (DEBUG)
				{
					if (okHelix.count(*i) > 1)
					{
						cerr << "helix-ok" << *i;
						for(multimap<int,int>::iterator j = okHelix.lower_bound(*i); j != okHelix.upper_bound(*i); ++j)
						{
							cerr << " " << j->second;
						}
						cerr << endl;
					}
				}
  	}

	///////////////////////////////////////////////////////
	// extend to ct - sheetFaces

	indexSet.clear();
	indexSet.insert(indexSheetSet.begin(), indexSheetSet.end());
	indexSet.insert(indexCTSet.begin(), indexCTSet.end());

	ctSheetFaces.clear();
	ctSheetFaces.insert(sheetFaces.begin(), sheetFaces.end());
	
	int newc = 0;
  cerr << endl;
	do
	{
		newc = 0;
		for(set<int>::iterator i=indexCTSet.begin(); i!=indexCTSet.end(); ++i) // row
		for(set<int>::iterator j=indexSet.begin(); j!=indexSet.end(); ++j) // column
  	{
		if (abs(*i-*j) <=1)
			continue;
      if (inconSet.find(make_pair(*i, *j)) != inconSet.end())
      	continue;

        if (indexHelixSet.find(*i) != indexHelixSet.end())
	      	if (okHelix.count(seq[*i].get_ss()->get_strand_ID()) < 2) // this helix is in one face
  	     		continue;
        if (indexHelixSet.find(*j) != indexHelixSet.end())
	      	if (okHelix.count(seq[*j].get_ss()->get_strand_ID()) < 2) // this helix is in one face
  	     		continue;
              
  		set<int> productVec; productVec.clear();

  		for(set<int>::iterator k=indexSet.begin(); k!=indexSet.end(); ++k)
  		{
    		if (abs(*i-*k) <=2)
      		continue;
  			if (ctFaces.count(make_pair(*i,*k)) < 2)
  				continue;
      
     		if (ctSheetFaces.find(make_pair(*k,*j)) == ctSheetFaces.end())
  				continue;

    		if (ctSheetFaces[make_pair(*k,*j)] == 0) // don't know
		  {
		    if (DEBUG) cerr << *k << "-" << *j << "==0 continue" << endl; 
		  continue;
		  }

				if ((indexSheetSet.find(*k) == indexSheetSet.end())) 
					continue;
      
			  if (DEBUG)
			  {
			    cerr << "Sheet " << *i << "->" << *k << "->" << *j << " ctFaces: " << *i << "-" << *k << " " << ctFaces.find(make_pair(*i,*k))->second;
          cerr << " ctSheetFaces: " << *k << "-" << *j << " " << ctSheetFaces[make_pair(*k, *j)]
                << "\tInferred ctSheetFaces between " << *i << "-" << *j << " = " << ctFaces.find(make_pair(*i, *k))->second*ctSheetFaces[make_pair(*k,*j)] << endl;
			  }
  			productVec.insert(ctFaces.find(make_pair(*i, *k))->second*ctSheetFaces[make_pair(*k,*j)]); // 1x1=1 and -1*-1 = 1;
  		}

		if (!productVec.size())
			continue;

		int max = *(productVec.rbegin());
		int min = *(productVec.begin());
		
			// 58-14-9 (1, 1)
			// 58-15-9 (1,-1)
			// 58 has contact to both 14 and 15 (in the same strand)
    	if (max*min != 1) // different faces
     	{
      	//if (DEBUG)
       		cerr << " *** inconsistent sheet toplogy contacts: " << *i << "-" << *j << " ignored" << endl;
      	continue;
//  			error("Error: inconsist sheet topology:", itoa(*i) + " " + itoa(*j));
      }

     	if (ctSheetFaces.find(make_pair(*i, *j)) == ctSheetFaces.end()) // new
     	{
      	ctSheetFaces.insert(make_pair(make_pair(*i,*j), min));
      	ctSheetFaces.insert(make_pair(make_pair(*j,*i), min));
        	++newc;
    	}
     	else // face is represented by min 
      {
	if (ctSheetFaces[make_pair(*i, *j)] == 0)
	  {
	    if (DEBUG) cerr << *i << " " << *j << "==0" << endl; 
	  continue; 
	  }

      	if (ctSheetFaces[make_pair(*i,*j)] != min)
      	{
      		//if (DEBUG)
      		{
	       		cerr << " *** Sheet topology contacts (ctSheetFaces) between " << *i << "-" << *j << "  was = " << ctSheetFaces[make_pair(*i,*j)] << endl;
        		for(set<int>::iterator k=indexSet.begin(); k!=indexSet.end(); ++k)
        		{
          		if (abs(*i-*k) <=2)
            		continue;
        			if (ctFaces.count(make_pair(*i,*k)) < 2)
        				continue;

           		if (ctSheetFaces.find(make_pair(*k,*j)) == ctSheetFaces.end())
        				continue;
      				if ((indexSheetSet.find(*k) == indexSheetSet.end())) // && (indexSheetSet.find(*i) == indexSheetSet.end()) &&
      					continue;

      			  //if (DEBUG)
      			  {
      			    cerr << " Sheet " << *i << "->" << *k << "->" << *j << " ctFaces: " << *i << "-" << *k << " " << ctFaces.find(make_pair(*i,*k))->second;
                cerr << "  ctSheetFaces: " << *k << "-" << *j << " " << ctSheetFaces[make_pair(*k, *j)]
                      << "\tInferred ctSheetFaces between " << *i << "-" << *j << " = " << ctFaces.find(make_pair(*i, *k))->second*ctSheetFaces[make_pair(*k,*j)] << endl;
      			  }
           } // end for
      		}
       		cerr << " *** inconsistent sheet toplogy contacts: " << *i << "-" << *j << " set to 0" << endl;
		
		ctSheetFaces[make_pair(*i,*j)] = 0;
		ctSheetFaces[make_pair(*j,*i)] = 0;
        	++newc;
      	}
      }
  	}
	} while (newc);

	////////////////////////////////////////////////
	// extend to ctSheet - helixPlaneFaces
	map<pair<int, int>, int> ctCTSheetFaces; ctCTSheetFaces.clear();
	ctCTSheetFaces.insert(ctSheetFaces.begin(), ctSheetFaces.end());

	for(set<int>::iterator i=indexSet.begin(); i!=indexSet.end(); ++i) // row
	for(set<int>::iterator j=indexSet.begin(); j!=indexSet.end(); ++j)
	{
		if (ctFaces.count(make_pair(*i,*j)) > 1)
			for(map<pair<int, int>, int>::iterator k = ctFaces.lower_bound(make_pair(*i,*j));
						k != ctFaces.upper_bound(make_pair(*i,*j)); ++k)
						ctCTSheetFaces.insert(*k);
	}

	// need to decide which contact face first
		
	// build the helixPlane faces

	//do
	//{
		//newc = 0;
		for(set<int>::iterator i=indexSet.begin(); i!=indexSet.end(); ++i) // row
		for(set<int>::iterator j=indexHelixSet.begin(); j!=indexHelixSet.end(); ++j) // column
  	{
			set<int> productVec; productVec.clear();

  		for(set<int>::iterator k=indexSet.begin(); k!=indexSet.end(); ++k)
  		{
  			if (ctCTSheetFaces.find(make_pair(*i,*k)) == ctCTSheetFaces.end()) // i from another segment, k is in Helix1
  				continue;

  			if (helixFaces.find(make_pair(*k,*j)) == helixFaces.end()) // k and j are in same Helix1
  				continue;

        if (indexHelixSet.find(*i) != indexHelixSet.end())
	      	if (okHelix.count(seq[*i].get_ss()->get_strand_ID()) < 2) // this helix is in one face
  	     		continue;

       if (okHelix.count(seq[*j].get_ss()->get_strand_ID()) < 2) // this helix is in one face
       		continue;
         
			if (DEBUG)
			{
			  cerr<< "Helix " << *i << " " << *k << " " << *j << " " << ctCTSheetFaces[make_pair(*i,*k)] << " " << helixFaces[make_pair(*k, *j)] << endl;
			}

  			productVec.insert(ctCTSheetFaces[make_pair(*i, *k)]*helixFaces[make_pair(*k,*j)]); // 1x1=1 and -1*-1 = 1;
  		}

		if (!productVec.size())
			continue;

		int max = *(productVec.rbegin());
		int min = *(productVec.begin());

     	{
      	ctHelixFaces.insert(make_pair(make_pair(*i,*j), min));
      	ctHelixFaces.insert(make_pair(make_pair(*j,*i), min));
    	}
  	}

	// remove non-helical relatied relations
	vector<map<pair<int,int>, int>::iterator> deli;
	for(map<pair<int,int>, int>::iterator i = ctHelixFaces.begin(); i != ctHelixFaces.end(); ++i)
	{
		if ((indexHelixSet.find(i->first.first) == indexHelixSet.end()) &&
				(indexHelixSet.find(i->first.second) == indexHelixSet.end()))
				deli.push_back(i);
	}
	for(vector<map<pair<int,int>, int>::iterator>::iterator j = deli.begin(); j != deli.end(); ++j)
	{
		ctHelixFaces.erase(*j);
	}


		// assuming the helixFaces is good
	set<int> indexSHSet; indexSHSet.clear();
	indexSHSet.insert(indexSheetSet.begin(), indexSheetSet.end());
	indexSHSet.insert(indexHelixSet.begin(), indexHelixSet.end());
	
	for(set<int>::iterator i=indexSet.begin(); i!=indexSet.end(); ++i) // row
		for(set<int>::iterator j=indexSHSet.begin(); j!=indexSHSet.end(); ++j) // column
  	{
		set<int> productVec; productVec.clear();
		
  		for(set<int>::iterator k=indexHelixSet.begin(); k!=indexHelixSet.end(); ++k)
  		{
  			if (ctHelixFaces.find(make_pair(*i,*k)) == ctHelixFaces.end()) // i from another segment, k is in Helix1
  				continue;

  			if (ctHelixFaces.find(make_pair(*k,*j)) == ctHelixFaces.end()) // k and j have interactions
  				continue;
			
      	if (okHelix.count(seq[*k].get_ss()->get_strand_ID()) < 2) // this helix is in one face
       		continue;
          
			if (DEBUG)
			{
			  cerr<< "HelixFaces " << *i << " " << *k << " " << *j << " " << ctHelixFaces[make_pair(*i,*k)] << " " << ctHelixFaces[make_pair(*k, *j)] << endl;
			}

  			productVec.insert(ctHelixFaces[make_pair(*i, *k)]*ctHelixFaces[make_pair(*k,*j)]); // 1x1=1 and -1*-1 = 1;
  		}

		if (!productVec.size())
			continue;

		int max = *(productVec.rbegin());
		int min = *(productVec.begin());

		if (min > -1)
			continue;

     	{
      	ctHelixFaces.insert(make_pair(make_pair(*i,*j), min));
      	ctHelixFaces.insert(make_pair(make_pair(*j,*i), min));
    	}
  	}

	///////////////////////////////////////////////////
	// extend to ct- helixLocalFaces		
	//set<int> indexHelixSet;

    // i, k,k+1,k+4,k+5
		for(set<int>::iterator i=indexCTSet.begin(); i!=indexCTSet.end(); ++i) // row
		for(set<int>::iterator j=indexHelixSet.begin(); j!=indexHelixSet.end(); ++j) // column
  	{
  			//set<int> productVec; productVec.clear();
				int min = 0;
				if (helixLocalFaces.find(make_pair(*i, *j)) != helixLocalFaces.end()) // both from same helix segment
				{
							continue;
				}

    	for(set<int>::iterator k=indexCTSet.begin(); k!=indexCTSet.end(); ++k)
  		{
  			if (ctFaces.count(make_pair(*i,*k)) < 2) // i from another segment, k in helix1
  				continue;

  			if (helixLocalFaces.find(make_pair(*k,*j)) == helixLocalFaces.end()) // k and j are in helix1
  				continue;

  			if ((ctFaces.count(make_pair(*i,*k+1)) < 2) || (helixLocalFaces.find(make_pair(*k+1,*j)) == helixLocalFaces.end())) // i from another segment, k in helix1
					continue;

  			if ((ctFaces.count(make_pair(*i,*k+4)) < 2) || (helixLocalFaces.find(make_pair(*k+4,*j)) == helixLocalFaces.end())) // i from another segment, k in helix1
					continue;
					if ((*j != *k) && (*j != *k+1) && (*j != *k+3) && (*j != *k+4) && (*j != *k+5) && (*j != *k-3))
				{
      		if (DEBUG)
    			{
    			  cerr<< "HelixLocal2 " << *i << " " << *k << "(+1) " << *j << " -1";
    			}
       		min = -1;
					break;
				}
  		}

			if (min == -1)
			{
  			if (DEBUG)
  				cerr<< "min " << min << endl;

        	ctHelixLocalFaces.insert(make_pair(make_pair(*i,*j), min));
  				ctHelixLocalFaces.insert(make_pair(make_pair(*j,*i), min));

        // change the ctFaces.
        if (ctFaces.find(make_pair(*i,*j)) != ctFaces.end())
        {
      		ctFaces.erase(ctFaces.lower_bound(make_pair(*i,*j)), ctFaces.upper_bound(make_pair(*i, *j)));
      		ctFaces.erase(ctFaces.lower_bound(make_pair(*j,*i)), ctFaces.upper_bound(make_pair(*j, *i)));
        }
      }
  	}

   	// i2, i, k,k+1,K+4,k+5
		for(set<int>::iterator i2=indexCTSet.begin(); i2!=indexCTSet.end(); ++i2) // row 102
		for(set<int>::iterator j=indexHelixSet.begin(); j!=indexHelixSet.end(); ++j) // column  47
  	{
  			//set<int> productVec; productVec.clear();
				int min = 0;
				if (helixLocalFaces.find(make_pair(*i2, *j)) != helixLocalFaces.end()) // both from same helix segment
				{
							continue;
				}

			for(set<int>::iterator i=indexCTSet.begin(); i != indexCTSet.end(); ++i)
			{
				if (abs(*i-*i2) < 6) // i2 102, i-95
					continue;

				if (abs(*i-*i2) == 7)
					continue;
					
     		if (helixLocalFaces.find(make_pair(*i,*i2)) == helixLocalFaces.end()) // k and j are in helix1
     				continue;

       	for(set<int>::iterator k=indexCTSet.begin(); k!=indexCTSet.end(); ++k)
     		{
  			if (ctFaces.count(make_pair(*i,*k)) < 2) // i from another segment, k in helix1
  				continue;

  			if (helixLocalFaces.find(make_pair(*k,*j)) == helixLocalFaces.end()) // k and j are in helix1
  				continue;

  			if ((ctFaces.count(make_pair(*i,*k+1)) < 2) || (helixLocalFaces.find(make_pair(*k+1,*j)) == helixLocalFaces.end())) // i from another segment, k in helix1
					continue;

  			if ((ctFaces.count(make_pair(*i,*k+4)) < 2) || (helixLocalFaces.find(make_pair(*k+4,*j)) == helixLocalFaces.end())) // i from another segment, k in helix1
					continue;
       		if (((*j == *k) || (*j == *k+1))) // && (ctFaces.find(make_pair(*i, *j)) != ctFaces.end()))
   				{
         		if (DEBUG)
       			{
       			  cerr<< "HelixLocal3 " << *i2 << " " << *i << " " << *k << "(+1) " << *j << " -1";
       			}
          		min = -1;
   					break;
   				}
     		}
			}

			if (min == -1)
			{
  			if (DEBUG)
  				cerr<< "min " << min << endl;

        	ctHelixLocalFaces.insert(make_pair(make_pair(*i2,*j), min));
  				ctHelixLocalFaces.insert(make_pair(make_pair(*j,*i2), min));

        // change the ctFaces.
        if (ctFaces.find(make_pair(*i2,*j)) != ctFaces.end())
        {
      		ctFaces.erase(ctFaces.lower_bound(make_pair(*i2,*j)), ctFaces.upper_bound(make_pair(*i2, *j)));
      		ctFaces.erase(ctFaces.lower_bound(make_pair(*j,*i2)), ctFaces.upper_bound(make_pair(*j, *i2)));
        }
      }
  	}

    // 95-50-51-58
		for(set<int>::iterator i=indexCTSet.begin(); i!=indexCTSet.end(); ++i) // row
		for(set<int>::iterator j=indexHelixSet.begin(); j!=indexHelixSet.end(); ++j) // column
  	{
				int min = 0;
				if (helixLocalFaces.find(make_pair(*i, *j)) != helixLocalFaces.end()) // both from same helix segment
				{
							continue;
				}

    	for(set<int>::iterator k=indexCTSet.begin(); k!=indexCTSet.end(); ++k)
  		{
  			if (ctFaces.count(make_pair(*i,*k)) < 2) // i from another segment, k in helix1
  				continue;

  			if (helixLocalFaces.find(make_pair(*k,*j)) == helixLocalFaces.end()) // k and j are in helix1
  				continue;

  			if ((ctFaces.count(make_pair(*i,*k+1)) < 2) || (helixLocalFaces.find(make_pair(*k+1,*j)) == helixLocalFaces.end())) // i from another segment, k in helix1
					continue;

  			if ((ctFaces.count(make_pair(*i,*k-3)) < 2) || (helixLocalFaces.find(make_pair(*k-3,*j)) == helixLocalFaces.end())) // i from another segment, k in helix1
					continue;
				if ((*j != *k) && (*j != *k+1) && (*j != *k+4) && (*j != *k-2) && (*j != *k-3) && (*j != *k-4))
				{
      		if (DEBUG)
    			{
    			  cerr<< "HelixLocal2-2 " << *i << " " << *k << "(+1) " << *j << " -1";
    			}
       		min = -1;
					break;
				}
  		}

			if (min == -1)
			{
  			if (DEBUG)
  				cerr<< "min " << min << endl;

        	ctHelixLocalFaces.insert(make_pair(make_pair(*i,*j), min));
  				ctHelixLocalFaces.insert(make_pair(make_pair(*j,*i), min));

        // change the ctFaces.
        if (ctFaces.find(make_pair(*i,*j)) != ctFaces.end())
        {
      		ctFaces.erase(ctFaces.lower_bound(make_pair(*i,*j)), ctFaces.upper_bound(make_pair(*i, *j)));
      		ctFaces.erase(ctFaces.lower_bound(make_pair(*j,*i)), ctFaces.upper_bound(make_pair(*j, *i)));
        }
      }
  	}

   	// 102-95-5051-47
		for(set<int>::iterator i2=indexCTSet.begin(); i2!=indexCTSet.end(); ++i2) // row 102
		for(set<int>::iterator j=indexHelixSet.begin(); j!=indexHelixSet.end(); ++j) // column  47
  	{
				int min = 0;
				if (helixLocalFaces.find(make_pair(*i2, *j)) != helixLocalFaces.end()) // both from same helix segment
				{
							continue;
				}

			for(set<int>::iterator i=indexCTSet.begin(); i != indexCTSet.end(); ++i)
			{
				if (abs(*i-*i2) < 6) // i2 102, i-95
					continue;
				if (abs(*i-*i2) == 7)
					continue;

     		if (helixLocalFaces.find(make_pair(*i,*i2)) == helixLocalFaces.end()) // k and j are in helix1
     				continue;

       	for(set<int>::iterator k=indexCTSet.begin(); k!=indexCTSet.end(); ++k)
     		{
  			if (ctFaces.count(make_pair(*i,*k)) < 2) // i from another segment, k in helix1
  				continue;

  			if (helixLocalFaces.find(make_pair(*k,*j)) == helixLocalFaces.end()) // k and j are in helix1
  				continue;

  			if ((ctFaces.count(make_pair(*i,*k+1)) < 2) || (helixLocalFaces.find(make_pair(*k+1,*j)) == helixLocalFaces.end())) // i from another segment, k in helix1
					continue;

  			if ((ctFaces.count(make_pair(*i,*k-3)) < 2) || (helixLocalFaces.find(make_pair(*k-3,*j)) == helixLocalFaces.end())) // i from another segment, k in helix1
					continue;
       		if (((*j == *k) || (*j == *k+1))) 
   				{
         		if (DEBUG)
       			{
       			  cerr<< "HelixLocal3-2 " << *i2 << " " << *i << " " << *k << "(+1) " << *j << " -1";
       			}
          		min = -1;
    					break;
   				}
     		}
			}

			if (min == -1)
			{
  			if (DEBUG)
  				cerr<< "min " << min << endl;

        	ctHelixLocalFaces.insert(make_pair(make_pair(*i2,*j), min));
  				ctHelixLocalFaces.insert(make_pair(make_pair(*j,*i2), min));

        // change the ctFaces.
        if (ctFaces.find(make_pair(*i2,*j)) != ctFaces.end())
        {
      		ctFaces.erase(ctFaces.lower_bound(make_pair(*i2,*j)), ctFaces.upper_bound(make_pair(*i2, *j)));
      		ctFaces.erase(ctFaces.lower_bound(make_pair(*j,*i2)), ctFaces.upper_bound(make_pair(*j, *i2)));
        }
      }
  	}

    // 7-10-11-14 have interact with 112 and 107. only 107 is real
  	for(set<int>::iterator j=indexHelixSet.begin(); j!=indexHelixSet.end(); ++j) // row
		{
			// find the conflict set
			set<int> conSet; conSet.clear();
			for(set<int>::iterator i=indexCTSet.begin(); i!=indexCTSet.end(); ++i) // column
    	{
  				if (helixLocalFaces.find(make_pair(*i, *j)) != helixLocalFaces.end()) // both from same helix segment
  				{
  							continue;
  				}

    			if (ctFaces.find(make_pair(*i,*j)) == ctFaces.end()) // i from another segment, k in helix1
    				continue;

    			if ((ctFaces.find(make_pair(*i,*j+1)) == ctFaces.end()) || (helixLocalFaces.find(make_pair(*j+1,*j)) == helixLocalFaces.end())) // i from another segment, k in helix1
  					continue;

    			if ((ctFaces.find(make_pair(*i,*j+4)) == ctFaces.end()) || (helixLocalFaces.find(make_pair(*j+4,*j)) == helixLocalFaces.end())) // i from another segment, k in helix1
  					continue;

    			if ((ctFaces.find(make_pair(*i,*j-3)) == ctFaces.end()) || (helixLocalFaces.find(make_pair(*j-3,*j)) == helixLocalFaces.end())) // i from another segment, k in helix1
  					continue;

      		conSet.insert(*i);
    	} // end for i

     	// j = 7 and conset=107,112
      //int best = 0; besti = -1000;dd
			if (conSet.size() > 1)
			{
  			if (DEBUG)
  				cerr<< "conflict-0,1,4,-3 " << *j << " ";

				multimap<int, pair<int, int> > score; score.clear();
        for(set<int>::iterator coni = conSet.begin(); coni != conSet.end(); ++coni)
        {
        	if (DEBUG)
         		cerr << "with " << *coni << " ";
          score.insert(make_pair(ctFaces.count(make_pair(*j, *coni)), make_pair(*j, *coni)));
          score.insert(make_pair(ctFaces.count(make_pair(*j+1, *coni)), make_pair(*j+1, *coni)));
          score.insert(make_pair(ctFaces.count(make_pair(*j+4, *coni)), make_pair(*j+4, *coni)));
          score.insert(make_pair(ctFaces.count(make_pair(*j-3, *coni)), make_pair(*j-3, *coni)));
        }

         // the lowest two
         int c=0;
         for(map<int, pair<int, int> >::iterator si = score.begin(); si != score.end(); ++si)
         {
						if (DEBUG)
							cerr << " " << si->first << " " <<si->second.first <<  " " << si->second.second << " " << endl;
		
          	if ((si->first <= 1)) 
           	{
            	++c;
             
		        	int min = -1;

        			if (DEBUG)
         				cerr << " rm-" << si->first << " " << si->second.first << " " << si->second.second << " ";

             	ctHelixLocalFaces.insert(make_pair(make_pair(si->second.first,si->second.second), min));
    					ctHelixLocalFaces.insert(make_pair(make_pair(si->second.second,si->second.first), min));

            	// change the ctFaces.
          		if (ctFaces.find(si->second) != ctFaces.end())
          		{
          			ctFaces.erase(ctFaces.lower_bound(make_pair(si->second.first,si->second.second)),
             									ctFaces.upper_bound(make_pair(si->second.first, si->second.second)));
          			ctFaces.erase(ctFaces.lower_bound(make_pair(si->second.second,si->second.first)),
             									ctFaces.upper_bound(make_pair(si->second.second, si->second.first)));
            	} // end if 
           	} // end if
          } // end si
        //} // end for
        if (DEBUG) cerr << endl;
      } // end if 
  	}

    // 7-10-11-14 have interact with 112 and 107. only 107 is real
  	for(set<int>::iterator j=indexHelixSet.begin(); j!=indexHelixSet.end(); ++j) // row
		{
			// find the conflict set
			set<int> conSet; conSet.clear();
			for(set<int>::iterator i=indexCTSet.begin(); i!=indexCTSet.end(); ++i) // column
    	{
  				if (helixLocalFaces.find(make_pair(*i, *j)) != helixLocalFaces.end()) // both from same helix segment
  				{
  							continue;
  				}

    			if (ctFaces.find(make_pair(*i,*j)) == ctFaces.end()) // i from another segment, k in helix1
    				continue;

    			if ((ctFaces.find(make_pair(*i,*j+1)) == ctFaces.end()) || (helixLocalFaces.find(make_pair(*j+1,*j)) == helixLocalFaces.end())) // i from another segment, k in helix1
  					continue;

    			if ((ctFaces.find(make_pair(*i,*j+4)) == ctFaces.end()) || (helixLocalFaces.find(make_pair(*j+4,*j)) == helixLocalFaces.end())) // i from another segment, k in helix1
  					continue;

    			if ((ctFaces.find(make_pair(*i,*j+3)) == ctFaces.end()) || (helixLocalFaces.find(make_pair(*j+3,*j)) == helixLocalFaces.end())) // i from another segment, k in helix1
  					continue;

      		conSet.insert(*i);
    	} // end for i

     	// j = 7 and conset=107,112
			if (conSet.size() > 1)
			{
  			if (DEBUG)
  				cerr<< "conflict-0,1,4,+3 " << *j << " ";

				multimap<int, pair<int, int> > score; score.clear();
        for(set<int>::iterator coni = conSet.begin(); coni != conSet.end(); ++coni)
        {
        	if (DEBUG)
         		cerr << "with " << *coni << " ";
          score.insert(make_pair(ctFaces.count(make_pair(*j, *coni)), make_pair(*j, *coni)));
          score.insert(make_pair(ctFaces.count(make_pair(*j+1, *coni)), make_pair(*j+1, *coni)));
          score.insert(make_pair(ctFaces.count(make_pair(*j+4, *coni)), make_pair(*j+4, *coni)));
          score.insert(make_pair(ctFaces.count(make_pair(*j+3, *coni)), make_pair(*j+3, *coni)));
        }

         // the lowest two
         int c=0;
         for(map<int, pair<int, int> >::iterator si = score.begin(); si != score.end(); ++si)
         {
						if (DEBUG)
							cerr << " " << si->first << " " <<si->second.first <<  " " << si->second.second << " " << endl;

          	if ((si->first <= 1)) 
           	{
            	++c;

		        	int min = -1;

        			if (DEBUG)
         				cerr << " rm-" << si->first << " " << si->second.first << " " << si->second.second << " ";

             	ctHelixLocalFaces.insert(make_pair(make_pair(si->second.first,si->second.second), min));
    					ctHelixLocalFaces.insert(make_pair(make_pair(si->second.second,si->second.first), min));

            	// change the ctFaces.
          		if (ctFaces.find(si->second) != ctFaces.end())
          		{
          			ctFaces.erase(ctFaces.lower_bound(make_pair(si->second.first,si->second.second)),
             									ctFaces.upper_bound(make_pair(si->second.first, si->second.second)));
          			ctFaces.erase(ctFaces.lower_bound(make_pair(si->second.second,si->second.first)),
             									ctFaces.upper_bound(make_pair(si->second.second, si->second.first)));
            	} // end if
           	} // end if
          } // end si
        //} // end for
        if (DEBUG) cerr << endl;
      } // end if
  	}

    // 7-10-11-14 have interact with 112 and 107. only 107 is real
  	for(set<int>::iterator j=indexHelixSet.begin(); j!=indexHelixSet.end(); ++j) // row
		{
			// find the conflict set
			set<int> conSet; conSet.clear();
			for(set<int>::iterator i=indexCTSet.begin(); i!=indexCTSet.end(); ++i) // column
    	{
  				if (helixLocalFaces.find(make_pair(*i, *j)) != helixLocalFaces.end()) // both from same helix segment
  				{
  							continue;
  				}

    			if (ctFaces.find(make_pair(*i,*j)) == ctFaces.end()) // i from another segment, k in helix1
    				continue;

    			if ((ctFaces.find(make_pair(*i,*j+1)) == ctFaces.end()) || (helixLocalFaces.find(make_pair(*j+1,*j)) == helixLocalFaces.end())) // i from another segment, k in helix1
  					continue;

    			if ((ctFaces.find(make_pair(*i,*j+4)) == ctFaces.end()) || (helixLocalFaces.find(make_pair(*j+4,*j)) == helixLocalFaces.end())) // i from another segment, k in helix1
  					continue;

    			if ((ctFaces.find(make_pair(*i,*j+5)) == ctFaces.end()) || (helixLocalFaces.find(make_pair(*j+5,*j)) == helixLocalFaces.end())) // i from another segment, k in helix1
  					continue;

      		conSet.insert(*i);
    	} // end for

     	// j = 7 and conset=107,112
			if (conSet.size() > 1)
			{
  			if (DEBUG)
  				cerr<< "conflict-0,1,4,5" << *j << " ";

				multimap<int, pair<int, int> > score; score.clear();
        for(set<int>::iterator coni = conSet.begin(); coni != conSet.end(); ++coni)
        {
        	if (DEBUG)
         		cerr << "with " << *coni << " ";
          score.insert(make_pair(ctFaces.count(make_pair(*j, *coni)), make_pair(*j, *coni)));
          score.insert(make_pair(ctFaces.count(make_pair(*j+1, *coni)), make_pair(*j+1, *coni)));
          score.insert(make_pair(ctFaces.count(make_pair(*j+4, *coni)), make_pair(*j+4, *coni)));
          score.insert(make_pair(ctFaces.count(make_pair(*j+5, *coni)), make_pair(*j+5, *coni)));
				}
				int c=0;
        for(map<int, pair<int, int> >::iterator si = score.begin(); si != score.end(); ++si)
        {
						if (DEBUG)
						cerr << " " << si->first << " " <<si->second.first <<  " " << si->second.second << " " << endl;
          	if ((si->first <=1)) 
           	{
            	++c;
		        	int min = -1;

        			if (DEBUG)
         				cerr << " rm-" << si->first << " " << si->second.first << " " << si->second.second << " ";

             	ctHelixLocalFaces.insert(make_pair(make_pair(si->second.first,si->second.second), min));
    					ctHelixLocalFaces.insert(make_pair(make_pair(si->second.second,si->second.first), min));

            	// change the ctFaces.
          		if (ctFaces.find(si->second) != ctFaces.end())
          		{
          			ctFaces.erase(ctFaces.lower_bound(make_pair(si->second.first,si->second.second)),
             									ctFaces.upper_bound(make_pair(si->second.first, si->second.second)));
          			ctFaces.erase(ctFaces.lower_bound(make_pair(si->second.second,si->second.first)),
             									ctFaces.upper_bound(make_pair(si->second.second, si->second.first)));
            	}
           	}
          }
        //} // end for
        if (DEBUG) cerr << endl;
      }
  	}

	// a1-a2-b2-b1, x-a1=1, x=b=-1
	// find the a1,b1 group and a2,b2 group
	map<pair<int, int>, set<int> > compatablePair; compatablePair.clear();

    	for(set<int>::iterator i=indexHelixSet.begin(); i!=indexHelixSet.end(); ++i)
    	for(set<int>::iterator k=indexHelixSet.begin(); k!=indexHelixSet.end(); ++k)
  		{
  			if (ctFaces.count(make_pair(*i,*k)) < 2) // i and k have contact
  				continue;

  			if (helixLocalFaces.find(make_pair(*i,*k)) != helixLocalFaces.end()) // k and i are in helix1
  				continue;

			//i and k are from two different helices and have contact
			//I+/2 -> helix for k and K+/2 -> helix of i = -1;
			pair<int, int> ab;
			set<int> abset; abset.clear();
			if (*i < *k) ab = make_pair(*i, *k);
			else
				ab = make_pair(*k, *i);
			abset.insert(*k); abset.insert(*i);

			map<pair<int,int>, set<int> >::iterator in = compatablePair.begin();
			bool old = 0;
			while(in != compatablePair.end())
			{
				if ((helixLocalFaces.find(make_pair(in->first.first, ab.first)) != helixLocalFaces.end()) &&
				    (helixLocalFaces.find(make_pair(in->first.second, ab.second)) != helixLocalFaces.end()))
				{

				  if (DEBUG)
				    cerr << "pair " << ab.first << " " << ab.second << endl;
					in->second.insert(ab.first);
					in->second.insert(ab.second);
					old = 1;
					break;
				}
				++in;
			}

			if (!old)
			{
				compatablePair.insert(make_pair(ab, abset));
				if (DEBUG)
					cerr << " add new " << ab.first << " " << ab.second << endl;
			}
		}


		if (DEBUG)
		{
			for(map<pair<int, int>, set<int> >::iterator in = compatablePair.begin(); in != compatablePair.end(); ++in)
			{
				cerr << "compatablePair " << in->first.first << " " << in->first.second << " ";
			        for(set<int>::iterator ins = in->second.begin(); ins != in->second.end(); ++ins)
					cerr	<< *ins << " ";
				cerr << endl;
			}
		}

		// helix to helix
		for(set<int>::iterator i=indexHelixSet.begin(); i!=indexHelixSet.end(); ++i) // row
		for(set<int>::iterator j=indexHelixSet.begin(); j!=indexHelixSet.end(); ++j) // column
  	{
				if (helixLocalFaces.find(make_pair(*i, *j)) != helixLocalFaces.end()) // both from same helix segment
				{
							continue;
				}

			pair<int, int> ab;
			if (*i < *j) ab = make_pair(*i, *j);
			else
				ab = make_pair(*j, *i);
			map<pair<int,int>, set<int> >::iterator in;
			int min = 0;

			if (DEBUG)
				cerr << "checking " << ab.first << " " << ab.second << endl;
			for(in = compatablePair.begin(); in != compatablePair.end(); ++in)
			{
				if ((helixLocalFaces.find(make_pair(in->first.first, ab.first)) != helixLocalFaces.end()) && // find the com set
				    (helixLocalFaces.find(make_pair(in->first.second, ab.second)) != helixLocalFaces.end()))
				{
					if (DEBUG)
						cerr << "in compatablePair" << endl;

					if (DEBUG)
						for(set<int>::iterator i = in->second.begin(); i != in->second.end(); ++i)
						{
							cerr << *i << " ";
						}
					
					if (DEBUG)
						cerr << ab.second+2 << ":" << (in->second.find(ab.second+2) != in->second.end());
					if (DEBUG)
						cerr << ab.second+1 << ":" << (in->second.find(ab.second+1) != in->second.end());
					if (((in->second.find(ab.first+2) != in->second.end()) && (in->second.find(ab.first+1) != in->second.end())) ||
					    ((in->second.find(ab.first-2) != in->second.end()) && (in->second.find(ab.first-1) != in->second.end())) ||
					    ((in->second.find(ab.second+2) != in->second.end()) && (in->second.find(ab.second+1) != in->second.end())) ||
					    ((in->second.find(ab.second-2) != in->second.end()) && (in->second.find(ab.second-1) != in->second.end())))
						min = -1;
					break;
				}
			}

			if (min)
			{
     		if (DEBUG)
  			{
  			  cerr<< "HelixLocalCom2 " << *i << " "  << *j << " " << -1 << endl ;
  			}

      	ctHelixLocalFaces.insert(make_pair(make_pair(*i,*j), min));
				ctHelixLocalFaces.insert(make_pair(make_pair(*j,*i), min));
			}
  	}
   
		for(set<int>::iterator i=indexCTSet.begin(); i!=indexCTSet.end(); ++i) // row
		for(set<int>::iterator j=indexHelixSet.begin(); j!=indexHelixSet.end(); ++j) // column
  	{
  		set<int> productVec; productVec.clear();

				if (helixLocalFaces.find(make_pair(*i, *j)) != helixLocalFaces.end()) // both from same helix segment
				{
							continue;
				}

				if (abs(*i-*j) < 5)
					continue;
				
    	for(set<int>::iterator k=indexCTSet.begin(); k!=indexCTSet.end(); ++k)
  		{
  			if (ctFaces.find(make_pair(*i,*k)) == ctFaces.end()) // i from another segment, k in helix1
  				continue;
  			if (helixLocalFaces.find(make_pair(*k,*j)) == helixLocalFaces.end()) // k and j are in helix1
  				continue;
		        		
     		if (DEBUG)
  			{
  			  cerr<< "HelixLocal " << *i << " " << *k << " " << *j << " " << ctFaces.find(make_pair(*i,*k))->second << " " << helixLocalFaces[make_pair(*k, *j)] ;
  			}
  			productVec.insert(ctFaces.find(make_pair(*i, *k))->second*helixLocalFaces[make_pair(*k,*j)]); // 1x1=1 and 1*-1 = -1;
  		}

  		if (!productVec.size())
  			continue;

     	// may have possible different faces
      // example: 33-112 and 33-113 then 33-112-111 = 1 and 33-113-111 = -1
      // pick the min
  		int max = *(productVec.rbegin());
  		int min = *(productVec.begin());

	if (DEBUG)
		cerr<< "min " << min << endl;
	
     	if (ctHelixLocalFaces.find(make_pair(*i, *j)) == ctHelixLocalFaces.end()) // new
     	{
      	ctHelixLocalFaces.insert(make_pair(make_pair(*i,*j), min));
				ctHelixLocalFaces.insert(make_pair(make_pair(*j,*i), min));
    	}
     	else
      {
      	if (ctHelixLocalFaces[make_pair(*i,*j)] > min) // choose -1
      	{
	      	ctHelixLocalFaces.insert(make_pair(make_pair(*i,*j), min));
					ctHelixLocalFaces.insert(make_pair(make_pair(*j,*i), min));
        }
      }
  	}
			

	///////////////////////////////////////////////////
	// extend to ct- sheetLocalFaces
	//set<int> indexHelixSet;
	set<pair<int, int> > ssSet; ssSet.clear();
	for(map<pair<int, int>, int>::iterator i = sheetLocalFaces.begin();  i != sheetLocalFaces.end(); ++i)
	{
		ssSet.insert(make_pair(seq[i->first.first].get_ss()->get_strand_ID(), seq[i->first.second].get_ss()->get_strand_ID()));
	}
	
		for(set<int>::iterator i=indexCTSet.begin(); i!=indexCTSet.end(); ++i) // row
		for(set<int>::iterator j=indexSheetSet.begin(); j!=indexSheetSet.end(); ++j) // column
  	{
  		set<int> productVec; productVec.clear();

				// avoid 61-59-62 (1,-1) cases
				if ((indexSheetSet.find(*i) != indexSheetSet.end())
							&& (indexSheetSet.find(*j) != indexSheetSet.end())) // both are in b-sheet
				{
					if ((abs(*i-*j) <=2)) // i and j are in same b-strand
						continue;
				}

    	for(set<int>::iterator k=indexSet.begin(); k!=indexSet.end(); ++k)
  		{
  			if (ctFaces.find(make_pair(*i,*k)) == ctFaces.end()) // i from another segment, k in sheet1
  				continue;
      
				// not aligned 52-44 (42-40 aligned) -> donot use 
				if ((ssSet.find(make_pair(seq[*i].get_ss()->get_strand_ID(), seq[*k].get_ss()->get_strand_ID())) != ssSet.end()) &&
					(sheetLocalFaces.find(make_pair(*i, *k)) == sheetLocalFaces.end()))
					continue;
				
     		if (sheetLocalFaces.find(make_pair(*k,*j)) == sheetLocalFaces.end()) // k and j are in sheet1
  				continue;

//				// avoid 61-40-43 (1,-1) cases
				if (sheetLocalFaces.find(make_pair(*i, *k+2)) != sheetLocalFaces.end()) // both from b-sheet and = 1
				{
						if ((sheetLocalFaces[make_pair(*i,*k+2)] == 1))				// is the center, adjacents, not the faces to faces
							continue;
				}
				
				if (sheetLocalFaces.find(make_pair(*i, *k-2)) != sheetLocalFaces.end()) // both from b-sheet and = 1
				{
						if ((sheetLocalFaces[make_pair(*i,*k-2)] == 1))					// is the center
							continue;
				}
				// no-b - b
				bool noBB = 0;

				if (ssSet.find(make_pair(seq[*i].get_ss()->get_strand_ID(), seq[*k].get_ss()->get_strand_ID())) == ssSet.end())
					noBB = 1;
					
				bool center1 = 1, center2=1;
				if ((indexSheetSet.find(*i) == indexSheetSet.end()) || noBB)
				{
  				if (ctFaces.find(make_pair(*i, *k+2)) != ctFaces.end()) // both from b-sheet and = 1
  				{
        			{
        				if (abs(*k+2-*j) <=1)				// is the center, adjacents, not the faces to faces
  								continue;
              } 
  				} else
      			center1 = 0;
         
  				if (ctFaces.find(make_pair(*i, *k-2)) != ctFaces.end()) // both from b-sheet and = 1
  				{
        			{
        				if (abs(*k-2-*j) <=1)				// is the center, adjacents, not the faces to faces
  								continue;
              } 
  				} else
      			center2 = 0;
         
      		if (!center1 && !center2)
        	{
         		if (abs(*k-*j) <=4)
           		continue;
         	}
				}
      
  			if (DEBUG)
  			{
  			  cerr<< "sheetLocal " << *i << " " << *k << " " << *j << " " << ctFaces.find(make_pair(*i,*k))->second << " " << sheetLocalFaces[make_pair(*k, *j)] << endl;
  			}
  			productVec.insert(ctFaces.find(make_pair(*i, *k))->second*sheetLocalFaces[make_pair(*k,*j)]); // 1x1=1 and -1*-1 = 1;
  		}

  		if (!productVec.size())
  			continue;

     	// may have possible different faces
      // example: 33-112 and 33-113 then 33-112-111 = 1 and 33-113-111 = -1
      // pick the min
  		int max = *(productVec.rbegin());
  		int min = *(productVec.begin());

     	if (ctSheetLocalFaces.find(make_pair(*i, *j)) == ctSheetLocalFaces.end()) // new
     	{
      	ctSheetLocalFaces.insert(make_pair(make_pair(*i,*j), min));
				ctSheetLocalFaces.insert(make_pair(make_pair(*j,*i), min));
    	}
     	else
      {
      	if (ctSheetLocalFaces[make_pair(*i,*j)] > min) // choose -1
      	{
	      	ctSheetLocalFaces.insert(make_pair(make_pair(*i,*j), min));
					ctSheetLocalFaces.insert(make_pair(make_pair(*j,*i), min));
        }
      }
  	}

	if (DEBUG)
	{
		for(map<pair<int, int>, int>::iterator i = ctSheetLocalFaces.begin(); i != ctSheetLocalFaces.end(); ++i)
		{
			cerr << "sheetLocalFaces " << i->first.first << " " << i->first.second << " " << i->second << endl;
		}
	}
	
	// merge
	indexSet.clear();
	indexSet.insert(indexHelixSet.begin(), indexHelixSet.end());
	indexSet.insert(indexSheetSet.begin(), indexSheetSet.end());
	indexSet.insert(indexCTSet.begin(), indexCTSet.end());
	for(set<int>::iterator i=indexSet.begin(); i!=indexSet.end(); ++i)
	{
		for(set<int>::iterator j=indexSet.begin(); j!=indexSet.end();++j)
		{
			pair<int,int> ij=make_pair(*i,*j);

			set<int> value; value.clear(); // -1 is the smallest one
			if (ctHelixLocalFaces.find(ij) != ctHelixLocalFaces.end())
			{
				value.insert(ctHelixLocalFaces[ij]);
			}
			if (ctSheetLocalFaces.find(ij) != ctSheetLocalFaces.end())
			{
				value.insert(ctSheetLocalFaces[ij]);
			}
			if (ctSheetFaces.find(ij) != ctSheetFaces.end())
			{
				value.insert(ctSheetFaces[ij]);
			}
			
			if (ctFaces.find(ij) != ctFaces.end()) // have contact
			{
  			if ((value.size()) && (ctFaces.find(ij)->second != *(value.begin())))
     		{        
      			if ((ctHelixLocalFaces.find(ij) != ctHelixLocalFaces.end()) && (ctFaces.find(ij)->second != ctHelixLocalFaces[ij]))
         		{
            	if (DEBUG)
             		cerr << *i << " " << *j << " ctFaces " << ctFaces.find(ij)->second << " ctHelixLocalFaces " << ctHelixLocalFaces[ij] << endl;
           		ctFaces.erase(ctFaces.lower_bound(ij), ctFaces.upper_bound(ij));
           		ctFaces.insert(make_pair(ij, -1));
            }
      			if ((ctSheetLocalFaces.find(ij) != ctSheetLocalFaces.end()) && (ctFaces.find(ij)->second != ctSheetLocalFaces[ij]))
         		{
            	if (DEBUG)
             		cerr << *i << " " << *j << " ctFaces " << ctFaces.find(ij)->second << " ctSheetLocalFaces " << ctSheetLocalFaces[ij] << endl;
           		ctFaces.erase(ctFaces.lower_bound(ij), ctFaces.upper_bound(ij));
         			ctFaces.insert(make_pair(ij, -1));
      			}
      			if ((ctSheetFaces.find(ij) != ctSheetFaces.end()) && (ctFaces.find(ij)->second != ctSheetFaces[ij]))
         		{
            	if (DEBUG)
             		cerr << *i << " " << *j << " ctFaces " << ctFaces.find(ij)->second << " ctSheetFaces " << ctSheetFaces[ij] << endl;
           		ctFaces.erase(ctFaces.lower_bound(ij), ctFaces.upper_bound(ij));
         			ctFaces.insert(make_pair(ij, -1));
      			}
				}
			}
			else
			{
				if (value.size() && (*(value.begin()) == -1))
					ctFaces.insert(make_pair(ij, *(value.begin())));
				else if ((ctHelixFaces.find(ij) != ctHelixFaces.end()) && (ctHelixFaces[ij] == -1))
  				ctFaces.insert(make_pair(ij, ctHelixFaces[ij]));
      
			}
		}
	}
}
