/***************************************************************************
 *   Copyright (C) 2006 by Yuanpeng Janet Huang                            *
 *   yphuang@cabm.rutgers.edu                                              *
 ***************************************************************************/
#include <iostream>
#include <cstdio>
#include <iomanip>
#include <string>
#include <cctype>
#include "Diana.h"
#include "AA.h"
#include "hbond.h"


string versionHeader();

extern map<char, string, less<char> > residue_name;
extern map<char, map<string, string, less<string> >, less<char> > layer_map;
extern map<string, char, less<string> > symbol3_1;

extern set<int, less<int> > CISPeptideBond;
extern set<pair<int,int>, less<pair<int, int> > > SSBond;
extern map<int, string, less<int> > specialName; // check hist, for strucctural calculations 

extern bool hasRDC;
extern set<int, less<int> > rdcORI; 

void find_looseContraint(const atom* , const atom* , const DianaMap&, const DianaConstraint*& );
//void get_pseudo(const atom*, const atom*, pseudo&);

//print resName, support all special names.

string getResName4SeqOnly(SEQM::const_iterator si, set<int, less<int> >& ssRes)
{
  
    	//if (si->first == 53)
    	if (CISPeptideBond.find(si->first) != CISPeptideBond.end())
	  return  "c" + residue_name[(*si).second.get_sym_1()];
    	else if (ssRes.find(si->first) != ssRes.end())
	  return residue_name[(*si).second.get_sym_1()] + "S";
	else if (specialName.find(si->first) != specialName.end())
	  {
	    return  specialName[si->first]; 
	  }
	else if (si->second.get_sym_1() == ' ')
	{
	   return si->second.get_special(); 
	}
	else 
	  return residue_name[(*si).second.get_sym_1()]; // default
}


string getResName(SEQM::const_iterator si, set<int, less<int> >& ssRes)
{
  
    	//if (si->first == 53)
//
//    	if (CISPeptideBond.find(si->first) != CISPeptideBond.end())
//	  return  "c" + residue_name[(*si).second.get_sym_1()];
//    	else if (ssRes.find(si->first) != ssRes.end())
//	  return residue_name[(*si).second.get_sym_1()] + "S";
//	else if (specialName.find(si->first) != specialName.end())
//	  {
//	    return  specialName[si->first]; 
//	  }

	 if (si->second.get_sym_1() == ' ')
	{
	   return si->second.get_special(); 
	}
	else 
	  return residue_name[(*si).second.get_sym_1()]; // default
}

void prtCyanaSeq(ostream& s, const SEQM& seq, int LLength)
{
  s <<"# created by " << versionHeader() << endl << endl;

  set<int, less<int> > ssRes; ssRes.clear();
  for (set<pair<int,int>, less<pair<int, int> > >::iterator i=SSBond.begin(); i!= SSBond.end(); ++i)
  {
  	ssRes.insert(i->first);
  	ssRes.insert(i->second);
  }

  int count = 1;
  for(SEQM::const_iterator si = seq.begin(); si != seq.end(); ++si)
    {
      s << getResName4SeqOnly(si, ssRes) << " "; 


      if (si == seq.begin())
					s << (*si).first << " ";
      if (count >= 5)
    	{
    	  s << endl;
    	  count = 1;
    	}
      else
				count++;
    }
      
  if (LLength)
    {
      // Add in 500 LL atoms
      for(int i = (*seq.rbegin()).first + 1; i < LLength + (*seq.rbegin()).first + (*seq.begin()).first ; i++)
    	{
    	  if (i == (*seq.rbegin()).first + 1)
    	    {
    	      s << "PL" << " ";
    	    }
    	  else if (i == (*seq.begin()).first + LLength-1 + (*seq.rbegin()).first)
    	    {
    	      s << "LP" << " ";
    	    }
    	  else
    	    {
    	      s << "LL5" << " ";
    	    }
    	  if (count >= 5)
    	    {
    	      s << endl;
    	      count = 1;
    	    }
    	  else
    	    count++;
    	}

      for(SEQM::const_iterator si = seq.begin(); si != seq.end(); si++)
    	{
	    	if (CISPeptideBond.find(si->first) != CISPeptideBond.end())
  	  		s << "c" << residue_name[(*si).second.get_sym_1()] << " ";    	
    		else if (ssRes.find(si->first) != ssRes.end())
    			s << residue_name[(*si).second.get_sym_1()] << "S ";
      	else s << residue_name[(*si).second.get_sym_1()] << " ";
    	
    	  if (count >= 5)
    	    {
    	      s << endl;
    	      count = 1;
    	    }
    	  else
    	    count++;
    	}
    }

    s << endl;

    int lastResi = seq.rbegin()->first; 
    if (hasRDC)
      {
	//s << "PL LL5 LL5 LL5 LL5 LL5 ORI" << endl; 
	s << "PL "; lastResi++;  
	for(set<int, less<int> >::iterator i = rdcORI.begin(); i != rdcORI.end(); ++i)
	  {
	    for(int k=0; k<(*i)-lastResi-1; k++)
	      {
		s << "LL5 "; 
	      } 
	    s << "ORI\n"; 
	    lastResi = *i; 
	  }
	      s << endl; 
	    }

    for(set<pair<int, int>, less<pair<int, int> > >::iterator i = SSBond.begin(); i != SSBond.end(); ++i)
      {
	s << "link SG " << i->first << " SG " << i->second << endl;
      }
  
  s << flush << endl;
  return;
}

void prtCyanaProtEntry(ostream& s, int a, float b, string d, string e, int f)
{
	s << a << "\t" << b << "\t" << d << "\t" << e << "\t" << f << endl; 
}
void prtCyanaProt(ostream& s, Atom_List& atoms)
{
  s <<"# created by " << versionHeader() << endl << endl;

  for(Atom_List::iterator p = atoms.begin(); p != atoms.end(); ++p)
    {
      
	if ((p->get_name() == "QG") && (p->get_sym_1() == 'V'))
	{
		prtCyanaProtEntry(s, p->get_ID(), p->get_shift(), "0.020", "QG1", p->get_index());
		prtCyanaProtEntry(s, p->get_ID(), p->get_shift(), "0.020", "QG2", p->get_index());
	}
	else if ((p->get_name() == "QCG") && (p->get_sym_1() == 'V'))
	{
		prtCyanaProtEntry(s, p->get_ID(), p->get_shift(), "0.200", "CG1", p->get_index());
		prtCyanaProtEntry(s, p->get_ID(), p->get_shift(), "0.200", "CG2", p->get_index());
	}
	else if ((p->get_name() == "QD") && (p->get_sym_1() == 'L'))
	{
		prtCyanaProtEntry(s, p->get_ID(), p->get_shift(), "0.020", "QD1", p->get_index());
		prtCyanaProtEntry(s, p->get_ID(), p->get_shift(), "0.020", "QD2", p->get_index());
	} 
	else if ((p->get_name() == "QCD") && (p->get_sym_1() == 'L'))
	{
		prtCyanaProtEntry(s, p->get_ID(), p->get_shift(), "0.200", "CD1", p->get_index());
		prtCyanaProtEntry(s, p->get_ID(), p->get_shift(), "0.200", "CD2", p->get_index());
	}
	else
	{
		string c = "0.200"; 
		if (p->isH())
		{
			c = "0.020"; 
		}
		prtCyanaProtEntry(s, p->get_ID(), p->get_shift(), c, p->get_name(), p->get_index());
	}
     }
  s << flush << endl;
  return;
}

// print the intra chain distances orderby by atom id, if have more than one, merge
void prtCyIntraChain(ostream& s, int Correction, const DianaMap& dianaMap, bool center)
{
	atom12set tempset; tempset.clear();
  set<int, less<int> > ssRes; ssRes.clear();
  for (set<pair<int,int>, less<pair<int, int> > >::iterator i=SSBond.begin(); i!= SSBond.end(); ++i)
  {
  	ssRes.insert(i->first);
  	ssRes.insert(i->second);
  }

	for(DianaMap::const_iterator iter = dianaMap.begin(); iter != dianaMap.end(); iter++)
	{
		tempset.insert((*iter).first);
	}
	
	int oindex = 99999; 
	string preN1 = ""; string preN2 = ""; 
	int preI1 = 0; int preI2 = 0; 

	for(atom12set::const_iterator iter = tempset.begin(); iter != tempset.end(); iter++)
	{
		const DianaConstraint* temp;
		find_looseContraint(iter->first, iter->second, dianaMap, temp);
		
		string n1 = temp->name1;
		string n2 = temp->name2;

     		if (n1  == "HN")
		{
			n1 = "H"; 
		}
		if (n2  == "HN")
		{
			n2 = "H"; 
		} 
     
		if ((n1 == preN1) && (n2 == preN2) && (preI1 == iter->first->get_index()) && (preI2 == iter->second->get_index())) 
		  continue;
		preN1 = n1; preN2 = n2; preI1 = iter->first->get_index(); preI2 = iter->second->get_index(); 

		if (iter->first->get_index() != oindex)
		{
		  s << endl << "#" << iter->first->get_sym_1() << iter->first->get_index() + Correction << endl;
		  oindex = iter->first->get_index();
		}
		
		if ((iter->first->isMeaningless(iter->second)) || (iter->first->isSecLongHNHN(iter->second)))
		  {
		    s << "#";
		  }
		

		s << setw(5) << iter->first->get_index() + Correction << " "
			<< setw(3) << getResName(seq.find(iter->first->get_index()), ssRes) << " "; 
		
		s.setf(ios::left, ios::adjustfield);
		s << setw(5) << n1.c_str(); // n1
		
		s.setf(ios::right, ios::adjustfield); s.width(4);
		s << iter->second->get_index() + Correction << " ";
		s.width(3);
		s << getResName(seq.find(iter->second->get_index()), ssRes) << " "; 
	
		s.setf(ios::left, ios::adjustfield); s.width(5);
		s << n2.c_str(); // n2
		s.setf(ios::right, ios::adjustfield);
		s.setf(ios::fixed, ios::floatfield);
		s.width(7); s.precision(2);
		
		s << temp->xplorDist  + temp->xub; 

		s.width(8);
		s << "#"; 
		if (center) 
		  s << (*temp).cment;
		s << endl; 
	}	
}
// print the intra chain distances orderby by atom id, if have more than one, merge
void prtCyIntraChainAmbi(ostream& s, int Correction, const atom12set& tempset, int number)
{
  //atom12set tempset; tempset.clear();

  set<int, less<int> > ssRes; ssRes.clear();
  for (set<pair<int,int>, less<pair<int, int> > >::iterator i=SSBond.begin(); i!= SSBond.end(); ++i)
  {
  	ssRes.insert(i->first);
  	ssRes.insert(i->second);
  }
  /*
	for(DianaMap::const_iterator iter = dianaMap.begin(); iter != dianaMap.end(); iter++)
	{
		tempset.insert((*iter).first);
	}
  */	
	string preN1 = ""; string preN2 = ""; 
	int preI1 = 0; int preI2 = 0; 

	//if (DEBUG)
	cout << "peak " << endl;   

	int index = 0; 
	for(atom12set::const_iterator iter = tempset.begin(); iter != tempset.end(); iter++)
	{
	  //const DianaConstraint* temp;
	  //find_looseContraint(iter->first, iter->second, dianaMap, temp);
	  //if (DEBUG)
	  //cout << iter->first << endl << iter->second << endl;  
	  
	  index++; 
	  pseudo temp1; pseudo temp2; 

	  iter->first->get_pseudo(iter->second, temp1);
	  iter->second->get_pseudo(iter->first, temp2);

	  
	  string n1 = temp1.pname;
	  string n2 = temp2.pname;

	  if (n1  == "HN")
	    {
	      n1 = "H"; 
	    }
	  if (n2  == "HN")
	    {
	      n2 = "H"; 
	    } 
     
	  //if ((n1 == preN1) && (n2 == preN2) && (preI1 == iter->first->get_index()) && (preI2 == iter->second->get_index())) 
	  //	  continue;
	  preN1 = n1; preN2 = n2; preI1 = iter->first->get_index(); preI2 = iter->second->get_index(); 

	  /*if (iter->first->get_index() != oindex)
	    {
	      s << endl << "#" << iter->first->get_sym_1() << iter->first->get_index() + Correction << endl;
	      oindex = iter->first->get_index();
	    }
		
		if ((iter->first->isMeaningless(iter->second)) || (iter->first->isSecLongHNHN(iter->second)))
		  {
		    s << "#";
		  }
	  */

	  s << setw(5) << iter->first->get_index() + Correction << " "
	    << setw(3) << getResName(seq.find(iter->first->get_index()), ssRes) << " "; 
	  
	  s.setf(ios::left, ios::adjustfield);
	  s << setw(5) << n1.c_str(); // n1
	  
	  s.setf(ios::right, ios::adjustfield); s.width(4);
	  s << iter->second->get_index() + Correction << " ";
	  s.width(3);
	  s << getResName(seq.find(iter->second->get_index()), ssRes) << " "; 
	  
	  s.setf(ios::left, ios::adjustfield); s.width(5);
	  s << n2.c_str(); // n2
	  s.setf(ios::right, ios::adjustfield);
	  s.setf(ios::fixed, ios::floatfield);
	  s.width(7); s.precision(2);
	  
	  if (index == 1)
	    s << "5.00    0.125  " << "#" << number << endl; 
	  else 
	    s << "0.00" << endl; 
	  //s << temp->xplorDist  + temp->xub; 

	  s.width(8);
	  //s << "#"; 
	  //if (center) 
	  //  s << (*temp).cment;
	  //s << endl; 
	}
	s << endl; 
}

// print Inter Chain distances
void prtCyInterChain(ostream& s, int Correction1, int Correction2, const DianaMap& dianaMap, bool center)
{
	atom12set tempset; tempset.erase(tempset.begin(), tempset.end());
  set<int, less<int> > ssRes; ssRes.clear();
  for (set<pair<int,int>, less<pair<int, int> > >::iterator i=SSBond.begin(); i!= SSBond.end(); ++i)
  {
  	ssRes.insert(i->first);
  	ssRes.insert(i->second);
  }

	for(DianaMap::const_iterator iter = dianaMap.begin(); iter != dianaMap.end(); iter++)
	{
		tempset.insert((*iter).first);
	}

	int oindex = 99999;
	for(atom12set::const_iterator iter = tempset.begin(); iter != tempset.end(); iter++)
	{
		const DianaConstraint* temp;
		find_looseContraint(iter->first, iter->second, dianaMap, temp);

		//--- output
		if (iter->first->get_index() != oindex)
		{
				s << endl << "#" << iter->first->get_sym_1() << iter->first->get_index() + Correction1 << endl;
				oindex = iter->first->get_index();
		}

    if (iter->first->isMeaningless(iter->second))
    {
     	s << "#";
    }

		s << setw(5) << iter->first->get_index() + Correction1 << " "
			<< setw(3) << residue_name[iter->first->get_sym_1()];
    if (ssRes.find(iter->first->get_index()) != ssRes.end())
    	s << "S ";
    else s << "  ";
		
		s.setf(ios::left, ios::adjustfield);
		s << setw(5) << (*temp).name1.c_str();
		
		s.setf(ios::right, ios::adjustfield); s.width(4);
		s << iter->second->get_index() + Correction2 << " ";
		s.width(3);
		s << residue_name[iter->second->get_sym_1()];
    if (ssRes.find(iter->second->get_index()) != ssRes.end())
    	s << "S ";
    else s << "  ";
	
		s.setf(ios::left, ios::adjustfield); s.width(5);
		s << (*temp).name2.c_str();
		s.setf(ios::right, ios::adjustfield);
		s.setf(ios::fixed, ios::floatfield);
		s.width(7); s.precision(2);

		s << (*temp).dist;
		s.width(8);
		s << "#" << (*temp).cment << "  " << (*temp).expr.c_str() << endl;
	}	
}

void prtCyanabuffer(ostream& s, const SEQM& seq, int LLength, const DianaMap& dianaMap, const DianaMap& ICdianaMap, bool center)
{
	s <<"# created by " << versionHeader() << endl << endl;
	prtCyIntraChain(s, 0, dianaMap, center);
	if (LLength) // inter-Chain
	  {
	    prtCyInterChain(s, 0, LLength + (*seq.rbegin()).first, ICdianaMap, center);
	    prtCyInterChain(s, LLength + (*seq.rbegin()).first, 0, ICdianaMap, center);
	    prtCyIntraChain(s, LLength + (*seq.rbegin()).first, dianaMap, center);
	  }
	s << flush;
}

///////////////////////////////////////
// hbond is defined from O to HN     //
///////////////////////////////////////
void prtCyanaHbond(ostream& s, float OH, float ON, const SEQM& seq, int LLength, const set<HBOND, less<HBOND> >& HBONDList)
{
  set<int, less<int> > ssRes; ssRes.clear();
  for (set<pair<int,int>, less<pair<int, int> > >::iterator i=SSBond.begin(); i!= SSBond.end(); ++i)
  {
  	ssRes.insert(i->first);
  	ssRes.insert(i->second);
  }

	s <<"# created by " << versionHeader() << endl << endl;
  for(set<HBOND, less<HBOND> >::const_iterator i = HBONDList.begin(); i != HBONDList.end(); i++)
  {

  if (seq.find(i->second)->second.get_sym_1() != 'P')
	{
	  s << setw(3) << i->first << " " << setw(3) << getResName(seq.find(i->first), ssRes) << " "; //residue_name[seq.find(i->first)->second.get_sym_1()];
		
		s.setf(ios::left, ios::adjustfield);
		s << setw(5) << "O";
		
		s.setf(ios::right, ios::adjustfield); s.width(4);
		s << i->second << " ";
		s.width(3);
		s << getResName(seq.find(i->second), ssRes) << " ";
		s.setf(ios::left, ios::adjustfield); s.width(5);
		s << "N";
		s.setf(ios::right, ios::adjustfield);
		s.setf(ios::fixed, ios::floatfield);
		s.width(7); s.precision(2);
		s << ON;
		s << " # hbond" << endl;
		s << setw(3) << i->first << " " << setw(3) << getResName(seq.find(i->first), ssRes) << " "; //residue_name[seq.find(i->first)->second.get_sym_1()] << "  ";
		
		s.setf(ios::left, ios::adjustfield);
		s << setw(5) << "O";
		
		s.setf(ios::right, ios::adjustfield); s.width(4);
		s << i->second << " ";
		s.width(3);
		s << getResName(seq.find(i->second), ssRes) << " "; //residue_name[seq.find(i->second)->second.get_sym_1()];
  
		s.setf(ios::left, ios::adjustfield); s.width(5);
		s << "H";
		s.setf(ios::right, ios::adjustfield);
		s.setf(ios::fixed, ios::floatfield);
		s.width(7); s.precision(2);
		s << OH;
		s << " # hbond" << endl;

    if (LLength) // dimer
    {
  	  s << setw(3) << i->first+LLength+(*seq.rbegin()).first << " " << setw(3) << residue_name[seq.find(i->first)->second.get_sym_1()] << "  ";
  		
  		s.setf(ios::left, ios::adjustfield);
  		s << setw(5) << "O";
  		
  		s.setf(ios::right, ios::adjustfield); s.width(4);
  		s << i->second+LLength+(*seq.rbegin()).first << " ";
  		s.width(3);
  		s << residue_name[seq.find(i->second)->second.get_sym_1()];
  	
     if (ssRes.find(i->first) != ssRes.end())
      s << setw(1) << "S ";
    else s << setw(1) << "  ";

 		s.setf(ios::left, ios::adjustfield); s.width(5);
  		s << "N";
  		s.setf(ios::right, ios::adjustfield);
  		s.setf(ios::fixed, ios::floatfield);
  		s.width(7); s.precision(2);
  		s << ON;
  		s << " # hbond -- chain2" << endl;
  		
  	  s << setw(3) << i->first+LLength+(*seq.rbegin()).first << " " << setw(3) << residue_name[seq.find(i->first)->second.get_sym_1()] << "  ";
  		
  		s.setf(ios::left, ios::adjustfield);
  		s << setw(5) << "O";
  		
  		s.setf(ios::right, ios::adjustfield); s.width(4);
  		s << i->second+LLength+(*seq.rbegin()).first << " ";
  		s.width(3);
  		s << residue_name[seq.find(i->second)->second.get_sym_1()];

    if (ssRes.find(i->second) != ssRes.end())
      s << setw(1) << "S ";
    else s << setw(1) << "  ";
  	
  		s.setf(ios::left, ios::adjustfield); s.width(5);
  		s << "H";
  		s.setf(ios::right, ios::adjustfield);
  		s.setf(ios::fixed, ios::floatfield);
  		s.width(7); s.precision(2);
  		s << OH;
  		s << " # hbond -- chain2" << endl;
    }
  }
  }
  s << flush;
}
