/***************************************************************************
                          io_Diana.cpp  -  prepare files for DYANA calculation
                             -------------------
    begin                : Mon Aug 21 2000
    copyright            : (C) 2000 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#include <iostream>
#include <cstdio>
#include <iomanip>
#include <string>
#include <cctype>
#include "Diana.h"
#include "AA.h"
#include "hbond.h"

#include "noesy_assign.h"

string versionHeader();

extern bool DEBUG; 
extern map<char, string, less<char> > residue_name;
extern map<char, map<string, string, less<string> >, less<char> > layer_map;
extern map<string, char, less<string> > symbol3_1;

extern set<int, less<int> > CISPeptideBond;
extern set<pair<int,int>, less<pair<int, int> > > SSBond;

void prtDianaStat(ostream& s)
{
}

void prtDianaSeq(ostream& s, const SEQM& seq, int LLength)
{
  s <<"# created by " << versionHeader() << endl << endl;

  set<int, less<int> > ssRes; ssRes.clear();
  for (set<pair<int,int>, less<pair<int, int> > >::iterator i=SSBond.begin(); i!= SSBond.end(); ++i)
  {
  	ssRes.insert(i->first);
  	ssRes.insert(i->second);
  }

  int count = 1;
  for(SEQM::const_iterator si = seq.begin(); si != seq.end(); ++si)
    {
    	if (CISPeptideBond.find(si->first) != CISPeptideBond.end())
    		s << "c" << residue_name[(*si).second.get_sym_1()] << " ";
    	else if (ssRes.find(si->first) != ssRes.end())
    		s << residue_name[(*si).second.get_sym_1()] << "S ";
    	else if (si->second.get_sym_1() == 'K')
    		s << residue_name[(*si).second.get_sym_1()] << "+ ";
    	else if (si->second.get_sym_1() == 'R')
    		s << residue_name[(*si).second.get_sym_1()] << "+ ";
    	else if (si->second.get_sym_1() == 'D')
    		s << residue_name[(*si).second.get_sym_1()] << "- ";
    	else if (si->second.get_sym_1() == 'E')
    		s << residue_name[(*si).second.get_sym_1()] << "- ";
      else if (si->second.get_sym_1() == 'H')
        s << residue_name[(*si).second.get_sym_1()] << "+ ";    
      else s << residue_name[(*si).second.get_sym_1()] << " ";

      if (si == seq.begin())
					s << (*si).first << " ";
      if (count >= 5)
    	{
    	  s << endl;
    	  count = 1;
    	}
      else
				count++;
    }
      
  if (LLength)
    {
      // Add in 500 LL atoms
      for(int i = (*seq.rbegin()).first + 1; i < LLength + (*seq.rbegin()).first + (*seq.begin()).first ; i++)
    	{
    	  if (i == (*seq.rbegin()).first + 1)
    	    {
    	      s << "PL" << " ";
    	    }
    	  else if (i == (*seq.begin()).first + LLength-1 + (*seq.rbegin()).first)
    	    {
    	      s << "LP" << " ";
    	    }
    	  else
    	    {
    	      s << "LL5" << " ";
    	    }
    	  if (count >= 5)
    	    {
    	      s << endl;
    	      count = 1;
    	    }
    	  else
    	    count++;
    	}

      for(SEQM::const_iterator si = seq.begin(); si != seq.end(); si++)
    	{
	    	if (CISPeptideBond.find(si->first) != CISPeptideBond.end())
  	  		s << "c" << residue_name[(*si).second.get_sym_1()] << " ";    	
    		else if (ssRes.find(si->first) != ssRes.end())
    			s << residue_name[(*si).second.get_sym_1()] << "S ";
    	else if (si->second.get_sym_1() == 'K')
    		s << residue_name[(*si).second.get_sym_1()] << "+ ";
    	else if (si->second.get_sym_1() == 'R')
    		s << residue_name[(*si).second.get_sym_1()] << "+ ";
    	else if (si->second.get_sym_1() == 'D')
    		s << residue_name[(*si).second.get_sym_1()] << "- ";
    	else if (si->second.get_sym_1() == 'E')
    		s << residue_name[(*si).second.get_sym_1()] << "- ";
      else if (si->second.get_sym_1() == 'H')
        s << residue_name[(*si).second.get_sym_1()] << "+ ";
      	else s << residue_name[(*si).second.get_sym_1()] << " ";
    	
    	  if (count >= 5)
    	    {
    	      s << endl;
    	      count = 1;
    	    }
    	  else
    	    count++;
    	}
    }

    s << endl;
    				for(set<pair<int, int>, less<pair<int, int> > >::iterator i = SSBond.begin(); i != SSBond.end(); ++i)
    				{
    					s << "link SG " << i->first << " SG " << i->second << endl;
    				}
  
  s << flush << endl;
  return;
}

// atom1-atom2 have assignments from more than one peak list, choose the tighter one 
// should name it as find_tightContraint
// CtR69A - the tight one is too tight -  maybe revert back to the loose ones 
void find_looseContraint(const atom* atom1, const atom* atom2, const DianaMap& dianaMap, const DianaConstraint*& temp)
{
		if (dianaMap.find(make_pair(atom1, atom2)) == dianaMap.end())
		{
			NOESY_Assign::outputLog() << "no map " << atom1 << endl << atom2 << endl;	
			return;
		}
		DianaMap::const_iterator dl = dianaMap.lower_bound(make_pair(atom1, atom2));
		DianaMap::const_iterator du = dianaMap.upper_bound(make_pair(atom1, atom2));

		if (DEBUG)
		  NOESY_Assign::outputLog() << "find_loose" << " " << dl->first.first << endl << dl->first.second << endl << endl;

		float dist = 100; temp = 0;
		float dist2= 100; const DianaConstraint* temp2 = 0;

		float distmin = 100; float distmax = 0; 
		for(DianaMap::const_iterator di = dl; di != du; di++)
		{
			if ((di->second.multi))
				continue;
			if (di->second.xplorDist < dist) // choose the tighter one - changed 
			{
				dist = (*di).second.xplorDist;
				temp = &(di->second);
			}

			if (di->second.xplorDist < distmin)
			  {
			    distmin = di->second.xplorDist;
			  }
			if (di->second.xplorDist > distmax)
			  {
			    distmax = di->second.xplorDist; 
			  }

			if (DEBUG) 
			  NOESY_Assign::outputLog() << di->first.first << endl << di->first.second << di->second.dist << " " << di->second.xplorDist << " " << di->second.expr << endl; 
		}

		if (DEBUG)
		  {
		    if (distmax - distmin > 0.5)
		      NOESY_Assign::outputLog() << "diff dist" << endl; 
		  }
		// if all multi, choose the first one.
		if (dist == 100) //??
		{
			dist = dl->second.xplorDist;
			temp = &(dl->second);
		}

		if (DEBUG)
		  NOESY_Assign::outputLog() << "xplorDist =" << temp->xplorDist << endl;
		// choose the looser one without psedo corrections, not by multi
		for(DianaMap::const_iterator di = dl; di != du; di++)
		{

		  if (DEBUG) NOESY_Assign::outputLog() << di->first.first << endl << di->first.second << endl << di->second.name1 << " " << di->second.name2 << " " << di->second.expr << endl; 

			if (((di->second.name1[0] != 'Q') && (temp->name1[0] == 'Q')) || // choose the looser one
			     ((di->second.name2[0] != 'Q') && (temp->name2[0] == 'Q')))
			{
				if ((di->second.multi))
					continue;
				//if (di->second.xplorDist > dist2)
				if (di->second.xplorDist < dist2) // tighers
			  {
				  dist2 = (*di).second.xplorDist;
				  temp2 = &(di->second);
			  }
			}
		}

		if (dist2 != 100)
		{
			temp = temp2;
			if (DEBUG) NOESY_Assign::outputLog() << "xplordist2-noQ " << temp2->xplorDist << endl; 
		  return;
		}

		if (DEBUG) NOESY_Assign::outputLog() << "return " << temp->xplorDist << endl; 
		return;
}

// print the intra chain distances orderby by atom id, if have more than one, merge
void prtIntraChain(ostream& s, int Correction, const DianaMap& dianaMap)
{
	atom12set tempset; tempset.clear();
  set<int, less<int> > ssRes; ssRes.clear();
  for (set<pair<int,int>, less<pair<int, int> > >::iterator i=SSBond.begin(); i!= SSBond.end(); ++i)
  {
  	ssRes.insert(i->first);
  	ssRes.insert(i->second);
  }

	for(DianaMap::const_iterator iter = dianaMap.begin(); iter != dianaMap.end(); iter++)
	{
		tempset.insert((*iter).first);
	}
	
	int oindex = 99999;
	for(atom12set::const_iterator iter = tempset.begin(); iter != tempset.end(); iter++)
	{
		const DianaConstraint* temp;
		find_looseContraint(iter->first, iter->second, dianaMap, temp);
		
		string n1 = temp->name1;
		string n2 = temp->name2;

  	if ((iter->first->get_sym_1() == 'G') && (n1 == "HA2"))
   		n1 = "HA1";
  	if ((iter->first->get_sym_1() == 'G') && (n1 == "HA3"))
   		n1 = "HA2";

  	if ((iter->second->get_sym_1() == 'G') && (n2 == "HA2"))
   		n2 = "HA1";
   	if ((iter->second->get_sym_1() == 'G') && (n2 == "HA3"))
   		n2 = "HA2";
     
		if (iter->first->get_index() != oindex)
		{
				s << endl << "#" << iter->first->get_sym_1() << iter->first->get_index() + Correction << endl;
				oindex = iter->first->get_index();
		}
		
		if ((iter->first->isMeaningless(iter->second)) || (iter->first->isSecLongHNHN(iter->second)))
		  {
		    s << "#";
		  }

		s << setw(5) << iter->first->get_index() + Correction << " "
			<< setw(3) << residue_name[iter->first->get_sym_1()];
		s.width(1);
    if (ssRes.find(iter->first->get_index()) != ssRes.end())
    	s << "S ";
    else s << "  ";
			
		s.setf(ios::left, ios::adjustfield);
		s << setw(5) << n1.c_str(); // n1
		
		s.setf(ios::right, ios::adjustfield); s.width(4);
		s << iter->second->get_index() + Correction << " ";
		s.width(3);
		s << residue_name[iter->second->get_sym_1()];
		s.width(1);
    if (ssRes.find(iter->second->get_index()) != ssRes.end())
    	s << "S ";
    else s << "  ";
	
		s.setf(ios::left, ios::adjustfield); s.width(5);
		s << n2.c_str(); // n2
		s.setf(ios::right, ios::adjustfield);
		s.setf(ios::fixed, ios::floatfield);
		s.width(7); s.precision(2);
				
		s << (*temp).dist;
		s.width(8);
		s << "#" << (*temp).cment << "  " << (*temp).expr.c_str() << endl;
	}	
}

// print Inter Chain distances
void prtInterChain(ostream& s, int Correction1, int Correction2, const DianaMap& dianaMap)
{
	atom12set tempset; tempset.erase(tempset.begin(), tempset.end());
  set<int, less<int> > ssRes; ssRes.clear();
  for (set<pair<int,int>, less<pair<int, int> > >::iterator i=SSBond.begin(); i!= SSBond.end(); ++i)
  {
  	ssRes.insert(i->first);
  	ssRes.insert(i->second);
  }

	for(DianaMap::const_iterator iter = dianaMap.begin(); iter != dianaMap.end(); iter++)
	{
		tempset.insert((*iter).first);
	}

	int oindex = 99999;
	for(atom12set::const_iterator iter = tempset.begin(); iter != tempset.end(); iter++)
	{
		const DianaConstraint* temp;
		find_looseContraint(iter->first, iter->second, dianaMap, temp);

		//--- output
		if (iter->first->get_index() != oindex)
		{
				s << endl << "#" << iter->first->get_sym_1() << iter->first->get_index() + Correction1 << endl;
				oindex = iter->first->get_index();
		}

    if (iter->first->isMeaningless(iter->second))
    {
     	s << "#";
    }

		s << setw(5) << iter->first->get_index() + Correction1 << " "
			<< setw(3) << residue_name[iter->first->get_sym_1()];
    if (ssRes.find(iter->first->get_index()) != ssRes.end())
    	s << "S ";
    else s << "  ";
		
		s.setf(ios::left, ios::adjustfield);
		s << setw(5) << (*temp).name1.c_str();
		
		s.setf(ios::right, ios::adjustfield); s.width(4);
		s << iter->second->get_index() + Correction2 << " ";
		s.width(3);
		s << residue_name[iter->second->get_sym_1()];
    if (ssRes.find(iter->second->get_index()) != ssRes.end())
    	s << "S ";
    else s << "  ";
	
		s.setf(ios::left, ios::adjustfield); s.width(5);
		s << (*temp).name2.c_str();
		s.setf(ios::right, ios::adjustfield);
		s.setf(ios::fixed, ios::floatfield);
		s.width(7); s.precision(2);

		s << (*temp).dist;
		s.width(8);
		s << "#" << (*temp).cment << "  " << (*temp).expr.c_str() << endl;
	}	
}

void prtDianabuffer(ostream& s, const SEQM& seq, int LLength, const DianaMap& dianaMap, const DianaMap& ICdianaMap)
{
	s <<"# created by " << versionHeader() << endl << endl;
  prtIntraChain(s, 0, dianaMap);
	if (LLength) // inter-Chain
	{
    prtInterChain(s, 0, LLength + (*seq.rbegin()).first, ICdianaMap);
    prtInterChain(s, LLength + (*seq.rbegin()).first, 0, ICdianaMap);
    prtIntraChain(s, LLength + (*seq.rbegin()).first, dianaMap);
  }
	s << flush;
}

//////////////////////////////////////////////////////////////////////////
// print the intra chain distances ordered by peak ID, allow reducency  //
// used to print ambigous constraints in dyana format										//
//																																			//	
//	correction --- used for multimoner correction												//
//////////////////////////////////////////////////////////////////////////

void prtIntraChainDeque(ostream& s, int Correction, const DianaDeque& dianaDeque)
{

	for(DianaDeque::const_iterator temp = dianaDeque.begin(); temp != dianaDeque.end(); ++temp)
	{
		//--- output
		s << setw(3) << (*temp).first.first->get_index() + Correction << " "
			<< setw(3) << residue_name[(*temp).first.first->get_sym_1()]
			<< " " ;
				
		s.setf(ios::left, ios::adjustfield);
		s << setw(5) << (*temp).second.name1.c_str();
		
		s.setf(ios::right, ios::adjustfield); s.width(4);
		s << (*temp).first.second->get_index() + Correction << " ";
		s.width(3);
		s << residue_name[(*temp).first.second->get_sym_1()].c_str()
		  << "  ";
	
		s.setf(ios::left, ios::adjustfield); s.width(5);
		s << (*temp).second.name2.c_str();
		s.setf(ios::right, ios::adjustfield);
		s.setf(ios::fixed, ios::floatfield);
		s.width(7); s.precision(2);
		s << (*temp).second.dist;
		s.width(8);
		s << "#" << (*temp).second.cment << "  " << (*temp).second.expr.c_str() << endl;
	}	
}

// print Inter Chain distances
void prtInterChainDeque(ostream& s, int Correction1, int Correction2, const DianaDeque& dianaDeque)
{
	for(DianaDeque::const_iterator temp = dianaDeque.begin(); temp != dianaDeque.end(); ++temp)
	{
		s << setw(3) << (*temp).first.first->get_index() + Correction1 << " "
			<< setw(3) << residue_name[(*temp).first.first->get_sym_1()]
			<< " ";
				
		s.setf(ios::left, ios::adjustfield);
		s << setw(5) << (*temp).second.name1.c_str();
		
		s.setf(ios::right, ios::adjustfield); s.width(4);
		s << (*temp).first.second->get_index() + Correction2 << " ";
		s.width(3);
		s << residue_name[(*temp).first.second->get_sym_1()].c_str()
		  << "  ";
	
		s.setf(ios::left, ios::adjustfield); s.width(5);
		s << (*temp).second.name2.c_str();
		s.setf(ios::right, ios::adjustfield);
		s.setf(ios::fixed, ios::floatfield);
		s.width(7); s.precision(2);
		s << (*temp).second.dist;
		s.width(8);
		s << "#" << (*temp).second.cment << "  " << (*temp).second.expr.c_str() << endl;
	}	
}

void prtDianabufferDeque(ostream& s, const SEQM& seq, int LLength, const DianaDeque& dianaDeque, const DianaDeque& ICdianaDeque) // grouped by peak id
{
	s << endl << "#Ambiguous constraints " << endl << endl;

  prtIntraChainDeque(s, 0, dianaDeque);
	if (LLength) // inter-Chain
	{
    prtInterChainDeque(s, 0, LLength + (*seq.rbegin()).first, ICdianaDeque);
    prtInterChainDeque(s, LLength + (*seq.rbegin()).first, 0, ICdianaDeque);
    prtIntraChainDeque(s, LLength + (*seq.rbegin()).first, dianaDeque);
  }
	s << flush;
}

///////////////////////////////////////
// hbond is defined from O to HN     //
///////////////////////////////////////
void prtDianaHbond(ostream& s, float OH, float ON, const SEQM& seq, int LLength, const set<HBOND, less<HBOND> >& HBONDList)
{
  set<int, less<int> > ssRes; ssRes.clear();
  for (set<pair<int,int>, less<pair<int, int> > >::iterator i=SSBond.begin(); i!= SSBond.end(); ++i)
  {
  	ssRes.insert(i->first);
  	ssRes.insert(i->second);
  }

	s <<"# created by " << versionHeader() << endl << endl;
  for(set<HBOND, less<HBOND> >::const_iterator i = HBONDList.begin(); i != HBONDList.end(); i++)
  {

  if (seq.find(i->second)->second.get_sym_1() != 'P')
	{
	  s << setw(3) << i->first << " " << setw(3) << residue_name[seq.find(i->first)->second.get_sym_1()];
		
	  
    if (ssRes.find(i->first) != ssRes.end())
      s << setw(1) << "S ";
    else s << setw(1) << "  ";

		s.setf(ios::left, ios::adjustfield);
		s << setw(5) << "O";
		
		s.setf(ios::right, ios::adjustfield); s.width(4);
		s << i->second << " ";
		s.width(3);
		s << residue_name[seq.find(i->second)->second.get_sym_1()] << "  ";
	
		s.setf(ios::left, ios::adjustfield); s.width(5);
		s << "N";
		s.setf(ios::right, ios::adjustfield);
		s.setf(ios::fixed, ios::floatfield);
		s.width(7); s.precision(2);
		s << ON;
		s << " # hbond" << endl;
	  s << setw(3) << i->first << " " << setw(3) << residue_name[seq.find(i->first)->second.get_sym_1()] << "  ";
		
		s.setf(ios::left, ios::adjustfield);
		s << setw(5) << "O";
		
		s.setf(ios::right, ios::adjustfield); s.width(4);
		s << i->second << " ";
		s.width(3);
		s << residue_name[seq.find(i->second)->second.get_sym_1()];
	
    if (ssRes.find(i->second) != ssRes.end())
      s << setw(1) << "S ";
    else s << setw(1) << "  ";

		s.setf(ios::left, ios::adjustfield); s.width(5);
		s << "HN";
		s.setf(ios::right, ios::adjustfield);
		s.setf(ios::fixed, ios::floatfield);
		s.width(7); s.precision(2);
		s << OH;
		s << " # hbond" << endl;

    if (LLength) // dimer
    {
  	  s << setw(3) << i->first+LLength+(*seq.rbegin()).first << " " << setw(3) << residue_name[seq.find(i->first)->second.get_sym_1()] << "  ";
  		
  		s.setf(ios::left, ios::adjustfield);
  		s << setw(5) << "O";
  		
  		s.setf(ios::right, ios::adjustfield); s.width(4);
  		s << i->second+LLength+(*seq.rbegin()).first << " ";
  		s.width(3);
  		s << residue_name[seq.find(i->second)->second.get_sym_1()];
  	
     if (ssRes.find(i->first) != ssRes.end())
      s << setw(1) << "S ";
    else s << setw(1) << "  ";

 		s.setf(ios::left, ios::adjustfield); s.width(5);
  		s << "N";
  		s.setf(ios::right, ios::adjustfield);
  		s.setf(ios::fixed, ios::floatfield);
  		s.width(7); s.precision(2);
  		s << ON;
  		s << " # hbond -- chain2" << endl;
  		
  	  s << setw(3) << i->first+LLength+(*seq.rbegin()).first << " " << setw(3) << residue_name[seq.find(i->first)->second.get_sym_1()] << "  ";
  		
  		s.setf(ios::left, ios::adjustfield);
  		s << setw(5) << "O";
  		
  		s.setf(ios::right, ios::adjustfield); s.width(4);
  		s << i->second+LLength+(*seq.rbegin()).first << " ";
  		s.width(3);
  		s << residue_name[seq.find(i->second)->second.get_sym_1()];

    if (ssRes.find(i->second) != ssRes.end())
      s << setw(1) << "S ";
    else s << setw(1) << "  ";
  	
  		s.setf(ios::left, ios::adjustfield); s.width(5);
  		s << "HN";
  		s.setf(ios::right, ios::adjustfield);
  		s.setf(ios::fixed, ios::floatfield);
  		s.width(7); s.precision(2);
  		s << OH;
  		s << " # hbond -- chain2" << endl;
    }
  }
  }
  s << flush;
}
