/***************************************************************************
                          io_XPLOR.cpp  -  prepare files for XPLOR calculation
                             -------------------
    begin                : Tue Oct 1 2002
    copyright            : (C) 2002 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#include <iostream>
#include <cstdio>
#include <iomanip>
#include <string>
#include <cctype>
#include "Diana.h"
#include "AA.h"
#include "hbond.h"
#include "atom.h"
#include "Jval.h"

string versionHeader();

extern map<char, string, less<char> > residue_name;
extern map<char, map<string, string, less<string> >, less<char> > layer_map;
extern map<string, char, less<string> > symbol3_1;
void find_looseContraint(const atom* , const atom* , const DianaMap&, const DianaConstraint*& );

void prtXplorCACBShift(ostream& s, const SEQM& seq, const Atom_List& atoms)
{
	map<int, float> CAshift; CAshift.clear();
	map<int, float> CBshift; CBshift.clear();
	
  for(Atom_List::const_iterator iter = atoms.begin(); iter != atoms.end(); ++iter)
  	if (iter->get_name() == "CA")
   	{
     	CAshift.insert(make_pair(iter->get_index(), iter->get_shift()));
    }
    else if (iter->get_name() == "CB")
    {
     	CBshift.insert(make_pair(iter->get_index(), iter->get_shift()));    	
    }
 
  s << "! created by " << versionHeader() << endl << endl;
  for(SEQM::const_iterator si = seq.begin(); si != seq.end(); si++)
  {
  	if ((seq.find(si->first-1) != seq.end()) && (seq.find(si->first+1) != seq.end())) 
   	{
			if ((CAshift.find(si->first) != CAshift.end()) && (CBshift.find(si->first) != CBshift.end()))
      {
  			s << "! " << residue_name[(*si).second.get_sym_1()] << " " << si->first << endl;
	  		s << "\tassign (resid " << setw(3) << si->first-1 << " and name C ) (resid " << setw(3) << si->first
		  		<< " and name N ) (resid " << setw(3) << si->first << " and name CA )" << endl
			  	<< "\t       (resid " << setw(3) << si->first << " and name C ) (resid " << setw(3) << si->first+1
				  << " and name N )\t";
				s << CAshift[si->first];
        s << "\t" << CBshift[si->first] << endl;
      }
			else if ((CAshift.find(si->first) != CAshift.end()) && ((*si).second.get_sym_1() == 'G'))
      {
			  s << "! " << residue_name[(*si).second.get_sym_1()] << " " << si->first << endl;
			  s << "\tassign (resid " << setw(3) << si->first-1 << " and name C ) (resid " << setw(3) << si->first
				  << " and name N ) (resid " << setw(3) << si->first << " and name CA )" << endl
				  << "\t       (resid " << setw(3) << si->first << " and name C ) (resid " << setw(3) << si->first+1
				  << " and name N )\t";
        s << CAshift[si->first];
        s << "\t $noexpectation" << endl;
      }
		}
  }
}

void prtXplorJvalues(ostream& s, const SEQM& seq, const JvalList& JList)
{
  s << "! created by " << versionHeader() << endl << endl;
  for(SEQM::const_iterator si = seq.begin(); si != seq.end(); si++)
  {
  	if (seq.find(si->first-1) != seq.end())
   	{
    	JvalList::const_iterator jl = JList.lower_bound(si->first);
    	JvalList::const_iterator ju = JList.upper_bound(si->first);
     
    	for(JvalList::const_iterator ji = jl; ji != ju; ji++)
        if (ji->second.get_type() == J3HNHA)
        {
    			s << "! " << residue_name[(*si).second.get_sym_1()] << " " << si->first << endl;
    			s << "\tassign (resid " << setw(3) << si->first-1 << " and name C ) (resid " << setw(3) << si->first
    				<< " and name N )" << endl
    				<< "\t       (resid " << setw(3) << si->first << " and name CA)"
    				<< " (resid " << setw(3) << si->first << " and name C )\t"
    				<< ji->second.get_value() << " " << ji->second.get_error() << endl;
       		break;
  			}
		}
  }	
}

void prtXPLORSeq(ostream& s, const SEQM& seq, int LLength)
{
  s <<"! created by " << versionHeader() << endl << endl;

  int count = 1;
  for(SEQM::const_iterator si = seq.begin(); si != seq.end(); si++)
    {
      s << residue_name[(*si).second.get_sym_1()] << " ";
      if (count >= 5)
    	{
    	  s << endl;
    	  count = 1;
    	}
      else
				count++;
    }

  s << flush << endl;
  return;
}


void prtXPLORIntraChain(ostream& s, int Correction, const DianaMap& dianaMap, bool center)
{
	atom12set tempset; tempset.clear();
	  
	for(DianaMap::const_iterator iter = dianaMap.begin(); iter != dianaMap.end(); iter++)
	{
		tempset.insert((*iter).first);
	}
	
	int oindex = 99999;

	int old1=0, old2=0;
	string oldn1="", oldn2="";
		
	for(atom12set::const_iterator iter = tempset.begin(); iter != tempset.end(); iter++)
	{
		const DianaConstraint* temp;
		find_looseContraint(iter->first, iter->second, dianaMap, temp);
		
		//--- output
		if (iter->first->get_index() != oindex)
		{
			s << "!" << iter->first->get_sym_1() << iter->first->get_index() << endl;
			oindex = iter->first->get_index();
		}
		
		string n1 = temp->name1;
		string n2 = temp->name2;

  // if is a repeat, skip
		if ((oldn1==n1) && (oldn2==n2) && (iter->first->get_index() == old1) && (iter->second->get_index() == old2))
			continue;

		while (n1[0] == 'Q')
			n1 = n1.substr(1, n1.length()) + "#";
		while (n2[0] == 'Q')
			n2 = n2.substr(1, n1.length()) + "#";

		// phe and tyr
		if (n1 == "CG")
		{
			n1 = "D#";
		}
		if (n1 == "CZ")
		{
			n1 = "E#";
		}
		if (n2 == "CG")
		{
			n2 = "D#";
		}
		if (n2 == "CZ")
		{
			n2 = "E#";
		}
		
		if (n1[n1.length()-1] == '#')
			n1 = "H" + n1;
		if (n2[n2.length()-1] == '#')
			n2 = "H" + n2;
		
 		if ((n1[n1.length()-1] == '3') && (!iter->first->isRingAtom()))
 			n1[n1.length()-1] = '1';

 		if ((n2[n2.length()-1] == '3') && (!iter->second->isRingAtom()))
 			n2[n2.length()-1] = '1';
		
		if ((iter->first->get_index() != iter->second->get_index()) || (n1 != n2))
		{
			s.clear();
			
  		s.setf(ios::left, ios::adjustfield);

		if ((iter->first->isMeaningless(iter->second)) || (iter->first->isSecLongHNHN(iter->second)))
		{
		  s << "!";
		}
      
  		s << "assign (resid " << setw(4) << iter->first->get_index()   << " and name " << setw(5) << n1.c_str() << " )(resid "
  													 << setw(4) << iter->second->get_index() << " and name " << setw(5) << n2.c_str() << " ) ";
  		
  		s.setf(ios::right, ios::adjustfield);
  		s.setf(ios::fixed, ios::floatfield);
  		s.width(7); s.precision(1);

			float lb = 0, ub = temp->xub;
			float dist = temp->xplorDist; 

			s << temp->xplorDist << " " << temp->xplorDist - 1.8 << " "; //???
 
			s << ub << endl; 
			old1 = iter->first->get_index();
			old2 = iter->second->get_index();
			oldn1 = temp->name1;
			oldn2 = temp->name2;
		}		
	}	
}


void prtXPLORbuffer(ostream& s, const SEQM& seq, int LLength, const DianaMap& dianaMap, const DianaMap& ICdianaMap, bool center)
{
	s <<"! created by " << versionHeader() << endl << endl;
  prtXPLORIntraChain(s, 0, dianaMap, center);
	s << flush;
}



///////////////////////////////////////
// hbond is defined from O to HN     //
///////////////////////////////////////
void prtXPLORHbond(ostream& s, float OH, float ON, const SEQM& seq, int LLength, const set<HBOND, less<HBOND> >& HBONDList)
{
	s <<"! created by " << versionHeader() << endl << endl;
  for(set<HBOND, less<HBOND> >::const_iterator i = HBONDList.begin(); i != HBONDList.end(); i++)
  {

  if (seq.find(i->second)->second.get_sym_1() != 'P')
	{	
		s.setf(ios::left, ios::adjustfield);
		s << "assign (resid " << setw(4) << i->first   << " and name " << setw(5) << "O" << " )(resid "
													 << setw(4) << i->second << " and name " << setw(5) << "N" << " ) ";
		
		s.setf(ios::right, ios::adjustfield);
		s.setf(ios::fixed, ios::floatfield);
		s.width(7); s.precision(2);
		s << ON-0.2 << "  0.2  0.2" << endl;
		
		s.setf(ios::left, ios::adjustfield);
		s << "assign (resid " << setw(4) << i->first   << " and name " << setw(5) << "O"  << " )(resid "
													 << setw(4) << i->second << " and name " << setw(5) << "HN" << " ) ";
		
		s.setf(ios::right, ios::adjustfield);
		s.setf(ios::fixed, ios::floatfield);
		s.width(7); s.precision(1);
		s << OH-0.3 << "  0.3  0.3" << endl;
		
  }
  }
  s << flush;
}
