/***************************************************************************
                          io_atom.cpp  -  description
                             -------------------
    begin                : Mon Aug 21 2000
    copyright            : (C) 2000 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#include <iostream>
#include <iomanip>
#include <cmath>
#include <stdlib.h>

#include "atom.h"
#include "AA.h"
#include "ss.h"

extern ostream& operator<<(ostream&, const AA&);

extern bool DEBUG;

// output the atom
ostream& operator<<(ostream& s, const atom& temp_atom)
{
  s.setf(ios::right, ios::adjustfield);
  s << "atom: " << setw(5) << temp_atom.atom_ID << " ";
	s << setw(3) << temp_atom.index;
  s.setf(ios::left, ios::adjustfield);
	s << temp_atom.residue() << "-";
	s << setw(4) << temp_atom.name.c_str() << " ";
	s << setw(3) << temp_atom.get_layer().c_str() << " ";
	s << setw(5) << temp_atom.get_shift() << " ";
			
	s << "ambi=" << temp_atom.ambi;
	s << " ";
	if (temp_atom.attached_N) s << "{ " << temp_atom.attached_N->get_name() << ":" << temp_atom.attached_N->get_shift() << " }";
	else if (temp_atom.attached_C.size() !=0 )
	{
		s << "{ ";
		for(deque<atom*>::const_iterator it = temp_atom.attached_C.begin();
			it != temp_atom.attached_C.end(); it++)
			if (*it) s << (*it)->get_name() << ":" << (*it)->get_shift() << " ";
		s << "}";
	}
	else
	{
	  if (temp_atom.type == 'H')
	    s << "{attached_atom is not_assigned}";
	}
	return s;
}

ostream& operator<<(ostream& s, const atom * temp_atom)
{
  s << (*temp_atom) ;
	return s;
}

// read atom 	
extern map<string, char, less<string> > symbol3_1;

istream& operator>>(istream& s, atom& temp_atom)
{
  string symbol_3="";
  string errorR="";

 		s >> temp_atom.atom_ID >> temp_atom.index >> symbol_3 >> temp_atom.name
			    >> temp_atom.type >> temp_atom.shift >>  errorR >> temp_atom.ambi;
				

  ios::iostate state = s.rdstate();
	
	   		
 	for (string::iterator p = symbol_3.begin(); p != symbol_3.end(); p++)
 	{
 		*p = toupper(*p);
 	}
    		
 	if (symbol_3.length() >= 3) // if input is > 3 letters
	  {
	    temp_atom.symbol_1 = symbol3_1[symbol_3.substr(0,3)];
	    if (symbol_3.length() > 3)
	      {
		specialName.insert(make_pair(temp_atom.index, symbol_3)); // save the speical name
	      }
	  }
 	else if (symbol_3.length() == 1)// if input is 1 letter
 	  temp_atom.symbol_1 = symbol_3[0];

	if (errorR != ".")
	  {
	    temp_atom.errorA = atof(errorR.c_str());
	  }
	
	return s;
}

