/***************************************************************************
                          io_experiment.cpp  -  description
                             -------------------
    begin                : Mon Aug 21 2000
    copyright            : (C) 2000 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#include <iostream>
#include <sstream>
#include <iomanip>
#include <string>
#include <iterator>
#include <cmath>
#include <stdlib.h>

#include "experiment.h"
#include "noesy_assign.h"
#include "Diana.h"

#define abs(x) abs((double)(x))

ostream& operator<<(ostream&, const method);

string itoa(int);
void split(string, vector<string>&, string);

extern bool DEBUG;

istream& getlineNA(istream&, string&, char);
SampleStatistic getDistStat(deque<Pdb>& , pair<const atom*, const atom*>, bool);
float sumAverage(const SampleStatistic&);
void find_looseContraint(const atom* , const atom* , const DianaMap&, const DianaConstraint*& );

bool isBB(string layer)
{
  if ((layer == "N") || (layer == "A"))
    return 1;
  return 0;
}

istream& operator>> (istream& s, NOESY& NOE)
{
  string line;
  while (getlineNA(s, line, '\n'))
  {
      vector<string> subline;
      split(line, subline, " \t\n\v");

			if (subline.size())
			  NOE.buildPeak(subline, subline.size());
  }
  return s;
}

ostream& operator<<(ostream& s, const NOESY& NOE)
{
	copy(NOE.peakList.begin(), NOE.peakList.end(), ostream_iterator<general_peak>(s, "\n"));
	return s;
}


ostream& NOESY::prtProcessStat(ostream& s) const
{
  s << reportProcessStat << endl;
  return s;
}
ostream& NOESY::prtPeakStat(ostream& s) const
{
  ostringstream outStream;
  int UNambi = 0; int Ambi = 0; 
  for(PeakList::const_iterator i = peakList.begin(); i != peakList.end(); i++)
    {
      PeakAssignReason::const_iterator il = AssignList.lower_bound(&(*i));
      PeakAssignReason::const_iterator iu = AssignList.upper_bound(&(*i));

      int count= 0;
      for(PeakAssignReason::const_iterator iter = il; iter != iu; iter++)
	{
	  count++;
	}
      if (count == 1)
	UNambi++;
      else if (count > 1)
	Ambi++;
    }
  outStream.setf(ios::left, ios::adjustfield);
  outStream.setf(ios::fixed);
  outStream.precision(0);
  outStream.width(6);
  outStream << (itoa(NOESY_Assign::ddcycle())+"-"+itoa(NOESY_Assign::dcycle())).c_str();
  outStream.width(10);
  outStream << peakList.size()-noiseList.size();
  outStream.width(12);
  outStream << "100"; 
  outStream.width(10);
  outStream << UNambi;
  outStream.width(11);
  outStream << UNambi*100/(float)(peakList.size()-noiseList.size());
  outStream.width(9);
  outStream << Ambi;
  outStream.width(10);
  outStream << Ambi*100/(float)(peakList.size()-noiseList.size());
  outStream.width(7);
  outStream << Ambi+UNambi;
  outStream.width(8);
  outStream << (Ambi+UNambi)*100/(float)(peakList.size()-noiseList.size());
  outStream.width(8);
  outStream << peakList.size()-noiseList.size()-Ambi-UNambi;
  outStream.width(9);
  outStream << (peakList.size()-noiseList.size()-Ambi-UNambi)*100/(float)(peakList.size()-noiseList.size());
  outStream << "\n" << NOESY_Assign::EOS();

  string temp(outStream.str());
  reportPeakStat += temp.substr(0,temp.find(NOESY_Assign::EOS()));
  s << reportPeakStat << endl;
  return s;
}

ostream& NOESY::prtAssignStat(ostream& s) const
{
  ostringstream outStream;

  int Intra = 0;
  int bbSeq = 0, bsSeq = 0, ssSeq = 0;
  int bbMid = 0, bsMid = 0, ssMid = 0;
  int bbLong = 0, bsLong = 0, ssLong = 0;

  int bbSeqSS = 0, bsSeqSS = 0, ssSeqSS = 0;
  int bbMidSS = 0, bsMidSS = 0, ssMidSS = 0;
  int bbLongSS = 0, bsLongSS = 0, ssLongSS = 0;

  int UNambi = 0; int Ambi = 0; 
  for(PeakList::const_iterator i = peakList.begin(); i != peakList.end(); i++)
    {
      PeakAssignReason::const_iterator il = AssignList.lower_bound(&(*i));
      PeakAssignReason::const_iterator iu = AssignList.upper_bound(&(*i));

      int count= 0;
      for(PeakAssignReason::const_iterator iter = il; iter != iu; iter++)
	{
	  count++;
	  int dis = abs((*iter).second.first.first->get_index() - 
			(*iter).second.first.second->get_index()); 
	  if (dis == 0) 
	    {
	      Intra++;
	    } 
	  else if (dis == 1)
	    {
	      if (isBB((*iter).second.first.first->get_layer()) && 
		  isBB((*iter).second.first.second->get_layer()))
		{
		  bbSeq++;
		  if ((*iter).second.second.count(EXPECT) != 0)
		    bbSeqSS++;
		}
	      else if (isBB((*iter).second.first.first->get_layer()) ||
		       isBB((*iter).second.first.second->get_layer()))
		{
		  bsSeq++;
		  if ((*iter).second.second.count(EXPECT) != 0)
		    bsSeqSS++;
		}
	      else
		{
		  ssSeq++;
		    if ((*iter).second.second.count(EXPECT) != 0)
		      ssSeqSS++;
		}
	    }
	  else if (dis <5)
	    {
	      if (isBB((*iter).second.first.first->get_layer()) && 
		  isBB((*iter).second.first.second->get_layer()))
		{
		  bbMid++;
		  if ((*iter).second.second.count(EXPECT) != 0)
		    bbMidSS++;
		}
	      else if (isBB((*iter).second.first.first->get_layer()) ||
		       isBB((*iter).second.first.second->get_layer()))
		{
		  bsMid++;
		  if ((*iter).second.second.count(EXPECT) != 0)
		    bsMidSS++;
		}
	      else
		{
		  ssMid++;
		    if ((*iter).second.second.count(EXPECT) != 0)
		      ssMidSS++;
		}	   
	    }
	  else
	    {
	      if (isBB((*iter).second.first.first->get_layer()) && 
		  isBB((*iter).second.first.second->get_layer()))
		{
		  bbLong++;
		  if ((*iter).second.second.count(EXPECT) != 0)
		    bbLongSS++;
		}
	      else if (isBB((*iter).second.first.first->get_layer()) ||
		       isBB((*iter).second.first.second->get_layer()))
		{
		  bsLong++;
		  if ((*iter).second.second.count(EXPECT) != 0)
		    bsLongSS++;
		}
	      else
		{
		  ssLong++;
		    if ((*iter).second.second.count(EXPECT) != 0)
		      ssLongSS++;
		}
	    }
	}
      if (count == 1)
	UNambi++;
      else if (count > 1)
	Ambi++;
    }
  outStream.setf(ios::left, ios::adjustfield);
  outStream.width(6);
  outStream <<(itoa(NOESY_Assign::ddcycle())+"-"+itoa(NOESY_Assign::dcycle())).c_str();
  outStream.width(17);
  outStream <<Intra + (bbSeq + bsSeq + ssSeq)+(bbMid + bsMid + ssMid)+(bbLong + bsLong + ssLong);
  outStream.width(18);
  outStream <<Intra;
  outStream.width(6);
  outStream <<(bbSeq + bsSeq + ssSeq);
  outStream.width(6);
  outStream <<bbSeq;
  outStream.width(6);
  outStream <<(bsSeq);
  outStream.width(4);
  outStream <<(ssSeq);
  outStream.width(6);
  outStream <<(bbMid + bsMid + ssMid);
  outStream.width(6);
  outStream <<bbMid;
  outStream.width(6);
  outStream <<(bsMid);
  outStream.width(7);
  outStream <<(ssMid);
  outStream.width(6);
  outStream <<(bbLong + bsLong + ssLong);
  outStream.width(6);
  outStream <<bbLong;
  outStream.width(6);
  outStream <<(bsLong);
  outStream.width(6);
  outStream <<(ssLong);
  outStream << "\n" << NOESY_Assign::EOS();

  string temp(outStream.str());
  reportAssignStat += temp.substr(0,temp.find(NOESY_Assign::EOS()));
  s << reportAssignStat << endl;
  return s;
}

// reformat
// add distance, reasoning information for each pair
// prtMatchPeakAtom --- old format
ostream& NOESY::prtSimpleMatchPeakAtom(ostream& s) const
{
	for(PeakList::const_iterator iter = peakList.begin(); iter != peakList.end(); iter++)
	{
		pair<MatchPeakAtom::const_iterator, MatchPeakAtom::const_iterator> iterPairMom = MombySimpleMatch.equal_range(&(*iter));
		pair<MatchPeakAtom::const_iterator, MatchPeakAtom::const_iterator> iterPairDad = DadbySimpleMatch.equal_range(&(*iter));
		
 		s << "#" << endl << "#" << endl << (*iter) << " poss "
 			<< MombySimpleMatch.count(&(*iter)) << " * " << DadbySimpleMatch.count(&(*iter)) << endl; 		
 		for(MatchPeakAtom::const_iterator iterMom = iterPairMom.first; iterMom != iterPairMom.second; iterMom++)
 		{
 		  s << endl << "HX1: " << (*iterMom).second << " diff: " << (*iterMom).second->get_Hdiff((*iter).get_HX1(), SWSF_HX1); // mom
      if ((*iter).get_D() > 2)
      {
        s << "-" << (*iterMom).second->get_Xdiff((*iter).get_X1(), SWSF_X1) << endl;
      }
 			else
 				s << endl;
 				
 			for(MatchPeakAtom::const_iterator iterDad = iterPairDad.first; iterDad != iterPairDad.second; iterDad++)
 			{
 				s << "   " << (*iterDad).second << " diff: " << (*iterDad).second->get_Hdiff((*iter).get_HX2(), SWSF_HX2); // dad
        if ((*iter).get_D() > 3)
          s << "-" << (*iterDad).second->get_Xdiff((*iter).get_X2(), SWSF_X2);
 				
 				// distance
     		pair<const atom*, const atom*> ab;
       if (iterDad->second < iterMom->second)
       	ab = make_pair(iterDad->second, iterMom->second);
       else
       	ab = make_pair(iterMom->second, iterDad->second);
        
				SampleStatistic tm; tm.reset(); tm = getDistStat(NOESY_Assign::pdbList(), ab, isIC());
 				if (tm.samples().size())
				  s << " Dis: " << tm.min() << "-" << tm.max() << " " << tm.mean() << "(mean)" << tm.median() << "(mid) " ;
         
	      if (NOESY_Assign::COMPARE())
	      {
	      		SampleStatistic tm; tm.reset(); tm = getDistStat(NOESY_Assign::powerPdbList(), ab, isIC());
 						if (tm.samples().size())
     					s << " Mod: " << tm.min() << "," << tm.mean() << "," << tm.max();
	      }
 				// reason
  			//--- list the BufferReason

	      		//bool FB = 0;
  			PeakAssignReason::const_iterator rl = BufferReason.lower_bound(&(*iter));
  			PeakAssignReason::const_iterator ru = BufferReason.upper_bound(&(*iter));
  			for(PeakAssignReason::const_iterator ri = rl; ri != ru; ri++)
  			{
  			  if (((*ri).second.first.first == (*iterMom).second) &&
  			      ((*ri).second.first.second == (*iterDad).second))
          {
    				s << " B: " ;
    				for(setMethod::iterator i = (*ri).second.second.begin();
    					i != (*ri).second.second.end(); i++)
				{
    					s << (*i) << " ";
				}
    				break;
  				}
  			}
  			//--- list the AssignedReason
  			rl = AssignList.lower_bound(&(*iter));
  			ru = AssignList.upper_bound(&(*iter));
  			for(PeakAssignReason::const_iterator ri = rl; ri != ru; ri++)
  			{
  			  if (((*ri).second.first.first == (*iterMom).second) &&
  			      ((*ri).second.first.second == (*iterDad).second))
          {
    				s << endl << "     Assigned by: " ;
    				for(setMethod::iterator i = (*ri).second.second.begin();
    					i != (*ri).second.second.end(); i++)
    				s << (*i) << " ";
    				break;
  				}
  			}
     		s << endl;
 			}
 		}
 		s << endl;
	}
	return s;
}

ostream& NOESY::prtAmbiCASP(ostream& sraw, ostream& scsa, const DianaMap& dianaMap) const
{
  int peakCount = 0;
  atom12set tempset; tempset.clear();
  /*for(DianaMap::const_iterator iter = dianaMap.begin(); iter != dianaMap.end(); iter++)
    {
      tempset.insert((*iter).first);
    }
  */
  
	for(PeakList::const_iterator iter = peakList.begin(); iter != peakList.end(); iter++)
	{
	  peakCount++;

	  //calibate the intensity
	  float distance = 0;
	  //float confi = 0;
	  
	  float intensity = fabs(iter->get_Intensity());
	  if (!intensity) intensity = 0.0000001; 
	  distance = pow(get_KC()/(float)intensity,1/6.0);
	  distance = floor(distance*10+0.5)/10.0; 

	  float disbin = 0; 
	  if (distance < 2.29) disbin = 2.3;
	  else if (distance < 2.89) disbin = 2.9;
	  else if (distance < 3.59) disbin = 3.6;
	  else if (distance < 3.99) disbin = 4.0;
	  else if (distance < 4.99)
	    disbin = 5.0;
	  else
	    disbin = max(get_Rmax()+0.0, 5.0); 
	  

	  //--- list the AssignedReason
	  PeakAssignReason::const_iterator rl = AssignList.lower_bound(&(*iter));
	  PeakAssignReason::const_iterator ru = AssignList.upper_bound(&(*iter));
	  
	  if (ru != rl) //has assignment 
	    {
	      //s << "in assigned" << endl; 
	      for(PeakAssignReason::const_iterator iterA = rl; iterA != ru; iterA++)
		{
		  //if (abs(iterA->second.first.first->get_index() - iterA->second.first.second->get_index()) <2) // print all non-intra noe assignments
		  //  {
		  //    s << "#short range removed\n";
		  //    s << "#";		      
		  //  }

		  if (iterA->second.first.first == iterA->second.first.second) // diagnoal peak, skip 
		    continue; 

		  bool rm = 0; 
		  if ((iterA->second.first.first->isMeaningless(iterA->second.first.second)) || (iterA->second.first.first->isSecLongHNHN(iterA->second.first.second)))
		    {
		      rm = 1; 
		    }
		  
		  const DianaConstraint* temp;
		  pair<atom*, atom*>ab;
		  
		  if (iterA->second.first.first < iterA->second.first.second)
		    ab = iterA->second.first;
		  else
		    ab = make_pair(iterA->second.first.second, iterA->second.first.first); 

		  if (dianaMap.find(ab) != dianaMap.end())
		    {
		      find_looseContraint(ab.first, ab.second, dianaMap, temp); 
		      disbin = temp->xplorDist + temp->xub;

		  
		      //s << "after looseContraint" << endl;
		      if (!rm)
			{
			  //scsa << "#"; 
			
			  scsa << iterA->second.first.first->get_index() << "\t" << iterA->second.first.second->get_index() << "\t" 
			       << peakCount << "\t" << disbin << "\t" //<< confi << "\t"
			       << iterA->second.first.first->get_name() << "\t" << iterA->second.first.second->get_name() << endl; 
			  
			  scsa << "#"; 
			  for(setMethod::iterator im = (*iterA).second.second.begin();
			      im != (*iterA).second.second.end(); im++)
			    scsa << (*im) << " " ;
			  scsa << endl;
			  
			  sraw << iterA->second.first.first->get_index() << "\t" << iterA->second.first.second->get_index() << "\t" 
			       << peakCount << "\t" << distance << "\t" //<< confi << "\t"
			       << iterA->second.first.first->get_name() << "\t" << iterA->second.first.second->get_name() << endl; 
			  
			  sraw << "#"; 
			  for(setMethod::iterator im = (*iterA).second.second.begin();
			      im != (*iterA).second.second.end(); im++)
			    sraw << (*im) << " " ;
			  sraw << endl;
			}
		    } // only print the one with calibrated distance 
		}
	    } // end rul != rl 

	  // print ambi assignments
	  {
	    
	    pair<MatchPeakAtom::const_iterator, MatchPeakAtom::const_iterator> iterPairMom = MombySimpleMatch.equal_range(&(*iter));
	    pair<MatchPeakAtom::const_iterator, MatchPeakAtom::const_iterator> iterPairDad = DadbySimpleMatch.equal_range(&(*iter));
	    
	    for(MatchPeakAtom::const_iterator iterMom = iterPairMom.first; iterMom != iterPairMom.second; iterMom++)
	      {
		for(MatchPeakAtom::const_iterator iterDad = iterPairDad.first; iterDad != iterPairDad.second; iterDad++)
		  {

		    float disbin2 = 0;
		    disbin2 = disbin;
		    
		    if (ru != rl) {
		      scsa << "#assigned "; // if assigned, comment all #
		      sraw << "#assigned "; 
		    }
		    
		    if ((((iterMom->second->get_layer() <= "B") && (iterMom->second->get_sym_1() != 'A')) || (iterMom->second->get_layer() == "N"))
			&& (((iterDad->second->get_layer() <= "B") && (iterDad->second->get_sym_1() != 'A')) || (iterDad->second->get_layer() == "N")))
		      {
			//backbone
			//scsa << "bb" << iterMom->second << " " << iterDad->second << endl;
		      } else
		      {
			disbin2 = max(get_Rmax()+0.0, 5.0); // - 1.0;
		      }
 
		    scsa << iterMom->second->get_index() << "\t" << iterDad->second->get_index() << "\t"
			 << peakCount << "\t" << disbin2 << "\t" // << confi << "\t"
			 << iterMom->second->get_name() << "\t" << iterDad->second->get_name() << endl;
		    
		    sraw << iterMom->second->get_index() << "\t" << iterDad->second->get_index() << "\t"
			 << peakCount << "\t" << distance << "\t" // << confi << "\t"
			 << iterMom->second->get_name() << "\t" << iterDad->second->get_name() << endl; 		  
		
		  }
	      }
	  }
	  scsa << endl;
	  sraw << endl; 
	}
	return scsa;
}




ostream& NOESY::prtAssignList(ostream& s) const
{
	s << "  Rmax = "  << get_Rmax() << endl;

	PeakAssignReason Intra; 
	PeakAssignReason Seq; 
	PeakAssignReason Mid; 
	PeakAssignReason Lon; 

	for(PeakAssignReason::const_iterator iter = AssignList.begin(); iter != AssignList.end(); iter++)
	{
		int dis = abs((*iter).second.first.first->get_index() - 
			(*iter).second.first.second->get_index()); 
		if (dis == 0) 
			Intra.insert(*iter); 
		else if (dis == 1)
			Seq.insert(*iter); 
		else if (dis <5)
			Mid.insert(*iter); 
		else Lon.insert(*iter); 
	}
	s << "# number of intra-residue assignments: " << Intra.size() << endl << endl;

	for(PeakList::const_iterator iiter = peakList.begin(); iiter != peakList.end(); iiter++)
	{
		if (Intra.find(&(*iiter)) != Intra.end())
		{
		  PeakAssignReason::const_iterator iterL = Intra.lower_bound(&(*iiter));
		  PeakAssignReason::const_iterator iterU = Intra.upper_bound(&(*iiter));			
		  for(PeakAssignReason::const_iterator iter = iterL; iter != iterU; iter++)
		    {
		      s << endl << (*iter).first << endl;
		      s << (*iter).second.first.first << endl
    			<< (*iter).second.first.second <<  " ";
		      for(setMethod::iterator im = (*iter).second.second.begin();
			  im != (*iter).second.second.end(); im++)
    			s << (*im) << " " ;
		      s << endl;
		    }
		}
	}
	s << endl << "# number of interresidue[(i-j)=1] assignments: " << Seq.size() << endl  << endl;
	
	for(PeakList::const_iterator iiter = peakList.begin(); iiter != peakList.end(); iiter++)
	{
		if (Seq.find(&(*iiter)) != Seq.end())
		{
			PeakAssignReason::const_iterator iterL = Seq.lower_bound(&(*iiter));
			PeakAssignReason::const_iterator iterU = Seq.upper_bound(&(*iiter));			
  	  for(PeakAssignReason::const_iterator iter = iterL; iter != iterU; iter++)
  	  {
     		s << endl << (*iter).first << endl;
     		s << (*iter).second.first.first << endl
     			<< (*iter).second.first.second <<  " ";
     		for(setMethod::iterator im = (*iter).second.second.begin();
     			im != (*iter).second.second.end(); im++)
     			s << (*im) << " ";
     		s << endl;
     	}
   	}
	}
	s << endl << "# number of interresidue[1<(i-j)<5] assignments: " << Mid.size() << endl  << endl;

	for(PeakList::const_iterator iiter = peakList.begin(); iiter != peakList.end(); iiter++)
	{
		if (Mid.find(&(*iiter)) != Mid.end())
		{
			PeakAssignReason::const_iterator iterL = Mid.lower_bound(&(*iiter));
			PeakAssignReason::const_iterator iterU = Mid.upper_bound(&(*iiter));			
  	  for(PeakAssignReason::const_iterator iter = iterL; iter != iterU; iter++)
  	  {
    		s << endl << (*iter).first << endl;
    		s << (*iter).second.first.first << endl
    			<< (*iter).second.first.second <<  " ";
    		for(setMethod::iterator im = (*iter).second.second.begin();
    			im != (*iter).second.second.end(); im++)
    			s << (*im) << " ";
    		s << endl;
    	}
  	}
	}
	
	s << endl << "# number of interresidue[(i-j)>=5] assignments: " << Lon.size() << endl  << endl;

  for(PeakList::const_iterator iiter = peakList.begin(); iiter != peakList.end(); iiter++)
	{
		if (Lon.find(&(*iiter)) != Lon.end())
		{
			PeakAssignReason::const_iterator iterL = Lon.lower_bound(&(*iiter));
			PeakAssignReason::const_iterator iterU = Lon.upper_bound(&(*iiter));			
  	  for(PeakAssignReason::const_iterator iter = iterL; iter != iterU; iter++)
  	  {
    		s << endl << (*iter).first << endl;
    		s << (*iter).second.first.first << endl
    			<< (*iter).second.first.second <<  " ";
    		for(setMethod::iterator im = (*iter).second.second.begin();
    			im != (*iter).second.second.end(); im++)
    			s << (*im) << " ";
    		s << endl;
    	}
  	}
  }	
	return s; 
}	

ostream& NOESY::prtZeroAssignList(ostream& s) const
{
	for(PeakList::const_iterator iiter = peakList.begin(); iiter != peakList.end(); iiter++)
	{			
	  if (AssignList.find(&(*iiter)) == AssignList.end())
	    {
	      s << (*iiter) << endl;
	    }
	}
	return s; 
}	

ostream& NOESY::prtConstraintList(ostream& s) const
{
	int num= 1; 
	s << "KC: " << get_KC() << "\t" << label() << endl; 
	for(PeakList::const_iterator iter = peakList.begin(); iter != peakList.end(); iter++)
	{
		PeakAssignReason::const_iterator AssignLower = AssignList.lower_bound(&(*iter));
		PeakAssignReason::const_iterator AssignUpper = AssignList.upper_bound(&(*iter));
		
		if (AssignLower != AssignUpper)
		{
			for(PeakAssignReason::const_iterator piter = AssignLower; piter != AssignUpper; piter++)
			{
				s << num++ << "\t";
				s << (*piter).second.first.first->get_name() << "\t";
				s << (*piter).second.first.first->get_sym_1() << "\t";
				s << (*piter).second.first.first->get_index() << "\t";
				s << (*piter).second.first.first->get_ambi() << "\t"; 
						
				s << (*piter).second.first.second->get_name() << "\t";
				s << (*piter).second.first.second->get_sym_1() << "\t";
				s << (*piter).second.first.second->get_index() << "\t";
				s << (*piter).second.first.second->get_ambi() << "\t"; 

				s << (*iter).get_Intensity() << "\t"; 

				s << (*iter).get_ID() << "\t"; 

				if (((*piter).second.second.count(TEST) > 0) ||
					((NOESY_Assign::pdbList().size() == 0) &&
					(((*piter).second.second.count(CORE) > 0) ||
					((*piter).second.second.count(CF) > 0))))
					s << 1 << "\t"; 
				else 	
					s << 0 << "\t"; 
				s << label() << endl;
			} // end if 
		}
	}
	return s; 
}

ostream& NOESY::prtSource(ostream& ass) const
{
	for(PeakList::const_iterator iiter = peakList.begin(); iiter != peakList.end(); iiter++)
	{			
	 	map<int, string, less<int> > labels; labels.clear();
	 	map<int, float, less<int> > shifts;  shifts.clear();
	
		// shifts
		shifts.insert(make_pair(atoi((*peakControlPtr)["col.hx2"].c_str()),(*iiter).get_HX2()));
		if ((*iiter).get_D() == 4) shifts.insert(make_pair(atoi((*peakControlPtr)["col.x2"].c_str()),(*iiter).get_X2()));
		if ((*iiter).get_D() > 2) shifts.insert(make_pair(atoi((*peakControlPtr)["col.x1"].c_str()),(*iiter).get_X1()));
		shifts.insert(make_pair(atoi((*peakControlPtr)["col.hx1"].c_str()),(*iiter).get_HX1()));
		
	  // labels
		if (AssignList.find(&(*iiter)) != AssignList.end())
		{
			PeakAssignReason::const_iterator iter_L = AssignList.lower_bound(&(*iiter));
			PeakAssignReason::const_iterator iter_U = AssignList.upper_bound(&(*iiter));
			
			// hx2
    	for(PeakAssignReason::const_iterator iter = iter_L; iter != iter_U; iter++)
      {
      	string t = iter->second.first.second->get_sym_1() +
      			itoa(iter->second.first.second->get_index()) +
      			iter->second.first.second->get_name();
      			
      	map<int, string, less<int> >::iterator i =
      	  labels.find(atoi((*peakControlPtr)["col.hx2"].c_str()));
      	if (i == labels.end())
	  labels.insert(make_pair(atoi((*peakControlPtr)["col.hx2"].c_str()), t));
      	else
	  i->second += "/" + t;
      }

      // x2        				
      if ((*iiter).get_D() == 4)
      {
      	for(PeakAssignReason::const_iterator iter = iter_L; iter != iter_U; iter++)
      	{
      		string t = iter->second.first.second->get_sym_1() +
      			itoa(iter->second.first.second->get_index()) +
      			iter->second.first.second->att_X(iter->first->get_X2(), SWSF_X2)->get_name();
      			      		
      		map<int, string, less<int> >::iterator i =
      	  	labels.find(atoi((*peakControlPtr)["col.x2"].c_str()));
      		if (i == labels.end())
      			labels.insert(make_pair(atoi((*peakControlPtr)["col.x2"].c_str()), t));
      		else
      			i->second += "/" + t;
  			}	
			}		
			
			// x1	
      if ((*iiter).get_D() > 2)
      {
      	for(PeakAssignReason::const_iterator iter = iter_L; iter != iter_U; iter++)
      	{
      		string t = iter->second.first.first->get_sym_1() +
      			itoa(iter->second.first.first->get_index()) +
      			iter->second.first.first->att_X(iter->first->get_X1(), SWSF_X1)->get_name();
      			      		
      		map<int, string, less<int> >::iterator i =
      	  	labels.find(atoi((*peakControlPtr)["col.x1"].c_str()));
      		if (i == labels.end())
      			labels.insert(make_pair(atoi((*peakControlPtr)["col.x1"].c_str()), t));
      		else
      			i->second += "/" + t;
  			}	  			
      }
    	
      // hx1
			for(PeakAssignReason::const_iterator iter = iter_L; iter != iter_U; iter++)
    	{    	    	
	      	
      	string t = iter->second.first.first->get_sym_1() +
      			itoa(iter->second.first.first->get_index()) +
      			iter->second.first.first->get_name();
      			
      	map<int, string, less<int> >::iterator i =
      	  labels.find(atoi((*peakControlPtr)["col.hx1"].c_str()));
      	if (i == labels.end())
      		labels.insert(make_pair(atoi((*peakControlPtr)["col.hx1"].c_str()), t));
      	else
      		i->second += "/" + t;
			}		
 	} else
	{
	  if ((*iiter).get_D() == 2) ass << setw(30) << setiosflags(ios::left)<< "?-?";
	  else if ((*iiter).get_D() == 3) ass << setw(30) << setiosflags(ios::left)<< "?-?-?";
	  else if ((*iiter).get_D() == 4) ass << setw(30) << setiosflags(ios::left)<< "?-?-?-?";
	}

	if (labels.size())
	{
		string ls = labels.begin()->second;
		for(map<int, string, less<int> >::iterator l = labels.begin(); l != labels.end(); l++)
		  {
		    if (l != labels.begin())
		      ls += "-" + l->second;
		  }
		ass << setw(30) << setiosflags(ios::left) << ls.c_str();
	}
	for(map<int, float, less<int> >::iterator s = shifts.begin(); s != shifts.end(); s++)
	  {
	    ass << "  " << setw(10) << setiosflags(ios::left) << s->second;
	  }
	
	ass << endl;
 }
 return ass;
}

ostream& NOESY::prtOvw(ostream& outStream) const
{

  return outStream;
}

ostream& NOESY::prtNoise(ostream& outStream) const
{
	for(PeakPtrList::const_iterator iter = noiseList.begin(); iter != noiseList.end(); iter++)
	{
		outStream << (*iter) << endl; 
	}
	return outStream;
}

