/***************************************************************************
                          io_hyper.cpp  -  description
                             -------------------
    begin                : Mon Aug 21 2000
    copyright            : (C) 2000 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#include <iostream>
#include <cmath>
#include <iomanip>
#include "Diana.h"
#include "Jval.h"

string versionHeader();

typedef multimap<int, atom12, less<int> > HNAB_N; // index of the first one 
extern bool DEBUG;
extern set<pair<int,int>, less<pair<int, int> > > SSBond;

void prtIntraHyper(ostream& s, const SEQM& seq, const DianaMap& DianaBuffer, bool haveNOE, bool haveJ, int correction)
{
  set<int, less<int> > ssRes; ssRes.clear();
  for (set<pair<int,int>, less<pair<int, int> > >::iterator i=SSBond.begin(); i!= SSBond.end(); ++i)
  {
  	ssRes.insert(i->first);
  	ssRes.insert(i->second);
  }
  // atom12 atom1 < atom2; order by index, N B A
  // only the d1 to d9 is included
  HNAB_N tempset; tempset.erase(tempset.begin(), tempset.end());
  DianaMap::const_iterator ip = DianaBuffer.begin();

  for(DianaMap::const_iterator iter = DianaBuffer.begin(); iter != DianaBuffer.end(); iter++)
  {
      if (((((*iter).first.first->get_index() == (*iter).first.second->get_index()) &&
	   (((*iter).first.second->get_layer() == "B")|| // N-B, A-B
	    ((*iter).first.second->get_layer() == "A"))) || // N-A
	  (((*iter).first.first->get_index() - (*iter).first.second->get_index() == -1) &&
	   (((*iter).first.first->get_layer() == "A") ||
	    ((*iter).first.first->get_layer() == "B") ||
	    ((*iter).first.first->get_layer() == "N")) &&
	   ((*iter).first.second->get_layer() == "N"))) &&
	  ((ip == DianaBuffer.begin()) ||
	   (((*ip).first.first != (*iter).first.first) ||
	    ((*ip).first.second != (*iter).first.second))))
	    {
	      tempset.insert(pair<int, atom12>((*iter).first.first->get_index(), (*iter).first));
	      ip = iter;
	    }
  }

  float distance_error = 0.3;

  int preType = 0; int nextType = 0;  
  for(SEQM::const_iterator iter = seq.begin(); iter != seq.end(); ++iter)
    {
      SEQM::const_iterator niter = iter; niter++;

      if (niter != seq.end())
	{
	  nextType = niter->second.get_ss()->get_type();

	} else
	{
	  nextType = 0; 
	}

      if ((*iter).second.get_sym_1() == 'P')
				continue;


      s << " Residue_" << residue_name[(*iter).second.get_sym_1()];

    if (ssRes.find(iter->first) != ssRes.end())
      s << setw(1) << "S";
    
      s << "_"
        << (*iter).first + correction <<"[999,999,999]" << endl;
      s << "  {" << endl;

      if ((iter->second.get_ss()->get_type() == 1) && (preType ==1) && (nextType == 1)) // && hp && hn) 
	{

// helix geometry in proteins, JMB 1988 201:601-19 Barlow and Thornton 
// mean - a-helix -62,-41 and 3-10 -71 and -18 
// use range 10
// we use phi -81, -52 and psi -51 to -8 
 
// sheet -119,113 and -139, 135
// phi -140 to -110 and psi 110 to 135 
// use range 20 

	  s << "\tallow 'helix' { phi -80.0 -50.0" << endl; // alfa and 310 
	  s << "\t                psi -50.0 -10.0" << endl; // alfa and 310	  
	  s << "\t              }" << endl;
	}
      else if ((iter->second.get_ss()->get_type() == -1) && (preType == -1) && (nextType == -1))
	{
	  s << "\tallow 'sheet' { phi -160.0 -90.0" << endl; //
	  s << "\t                psi 90.0 155.0" << endl; // both ...
	  s << "\t              }" << endl;
	}

      preType = iter->second.get_ss()->get_type(); 

	if (haveNOE)
	{
      HNAB_N::iterator hl = tempset.lower_bound((*iter).first);
      HNAB_N::iterator hu = tempset.upper_bound((*iter).first);
      for(HNAB_N::iterator hi = hl; hi != hu; hi++)
	{
	  if (DEBUG) s << endl;

	  DianaMap::const_iterator dl = DianaBuffer.lower_bound((*hi).second);
	  DianaMap::const_iterator du = DianaBuffer.upper_bound((*hi).second);

	  bool flag = 0;
	  bool nflag = 0;
	  float dist = 0, dmax = 0, c = 0, dmin=100;
	  for(DianaMap::const_iterator di = dl; di != du; di++)
	    {
	      if (DEBUG)
    		{
    		  s << "-- " << (*di).first.first << "\n";
    		  s << "-- " << (*di).first.second << " d:" << (*di).second.ori_dist << " r:" << (*di).second.range
    		    << "\t" << (*di).second.expr << "\t" << (*di).second.multi << "\n";
    		}

	      if ((*di).second.multi == 1)
					flag = 1;
	      else
					nflag = 1;
    		{
    		  if ((*di).second.ori_dist+(*di).second.range > dmax)
    		    dmax = (*di).second.ori_dist+(*di).second.range;
    		  if ((*di).second.ori_dist-(*di).second.range < dmin)
    		    dmin = (*di).second.ori_dist-(*di).second.range;
    		  c++;
    		}
	    }

	  dist = (dmin+dmax)/2.0;
	  if ((nflag) && ((c>1) || (fabs(dmin-dmax) > 0.001)))
	    {
	      distance_error = (dmax-dmin)/2.0 + 0.3;
	      if (flag)
    		{
    		  if  ((((*hi).second.second->get_index() == (*iter).first)) && (distance_error < 0.5))
    		    distance_error = 0.5;
    		  else if ((((*hi).second.second->get_index() != (*iter).first)) && (distance_error < 0.7))
    		    distance_error = 0.7;
    		}
	    }
	  else
	    {
	      if (flag)
					continue;
	      if  ((*hi).second.second->get_index() == (*iter).first)
					distance_error = 0.3; //max(0.3, distance_error);
	      else
					distance_error = 0.5; //max(0.5, distance_error);
	    }
	

	  if ((*hi).second.second->get_index() == (*iter).first)
	    {
	      if ( 
		  ((*hi).second.second->get_name() == "HA") ||
		  ((*hi).second.second->get_name() == "HA2"))
		s << "\t" << "dhn-ha  " << "\t" << dist << "\t" << dist + distance_error << "\t" << dist - distance_error << endl;
	      else if ((*hi).second.second->get_name() == "HA3")
		s << "\t" << "d1p  " << "\t" << dist << "\t" << dist + distance_error << "\t" << dist - distance_error << endl;
	      else if (
		       ((*hi).second.second->get_name() == "HB2"))
		{
		  if ((*hi).second.first->get_layer() == "N")
		    s << "\t" << "dhn-hb2 " << "\t" << dist << "\t" << dist + distance_error << "\t" << dist - distance_error << endl;
		  else
		    s << "\t" << "dha-hb2 " << "\t" << dist << "\t" << dist + distance_error << "\t" << dist - distance_error << endl;
		}
	      else if (((*hi).second.second->get_name() == "HB3")) 
		{
		  if ((*hi).second.first->get_layer() == "N")
		    s << "\t" << "dhn-hb3 " << "\t" << dist << "\t" << dist + distance_error << "\t" << dist - distance_error << endl;
		  else
		    s << "\t" << "dha-hb3 " << "\t" << dist << "\t" << dist + distance_error<< "\t" << dist - distance_error << endl;
		}
	    }
	  else
	    {
	      if (
		  ((*hi).second.first->get_name() == "HA") ||
		  ((*hi).second.first->get_name() == "HA2"))
		s << "\t" << "dha-hn  " << "\t" << dist << "\t" << dist + distance_error << "\t" << dist - distance_error << endl;
	      else if ((*hi).second.first->get_name() == "HA3")
		s << "\t" << "d7p  " << "\t" << dist << "\t" << dist + distance_error << "\t" << dist - distance_error << endl;
	      else if (
	      	       ((*hi).second.first->get_name() == "HB2"))
		{
		  s << "\t" << "dhb2-hn " << "\t" << dist << "\t" << dist + distance_error << "\t" << dist - distance_error << endl;
		}
	      else if (((*hi).second.first->get_name() == "HB3"))
		{
		  s << "\t" << "dhb3-hn " << "\t" << dist << "\t" << dist + distance_error << "\t" << dist - distance_error << endl;
		}
	      else if ((*hi).second.first->get_layer() == "N")
		s << "\t" << "dhn-hn  " << "\t" << dist << "\t" << dist + distance_error << "\t" << dist - distance_error << endl;
	    }
	}
	} // haveNOE
	
	if (haveJ)
	{
      JvalList::iterator jl = JList.lower_bound((*iter).first);
      JvalList::iterator ju = JList.upper_bound((*iter).first);
      for(JvalList::iterator ji = jl; ji != ju; ji++)
    	{
    	  string type;
    	  switch ((*ji).second.get_type())
    	    {
    	    case J3HNHA: type  = "jhn-ha  "; break;
    	    case J3HAHB2:type  = "jha-hb2 "; break;
    	    case J3HAHB3:type  = "jha-hb3 "; break;
    	    case J3NHB2: type  = "jn-hb2  "; break;
    	    case J3NHB3: type  = "jn-hb3  "; break;
    	    case J3HAN:  type  = "jha-n   "; break;
    	    case J3HNC:  type  = "jhn-c   "; break;
    	    case J3HNCB:  type = "jhn-cb  "; break;
    	    case J3CHB2:  type = "jc-hb2  "; break;
    	    case J3CHB3:  type = "jc-hb3  "; break;
    	    case J3CCB:  type  = "jc-cb   "; break;
    	    case J3CCG1:  type = "jc-cg1  "; break;
    	    case J3CC:  type   = "jc-c    "; break;
    	    default: type      = "";
    	    }
    	  if (type != "")
    	    {
    	      s << "\t" << type << "\t" << ((*ji).second.get_upper() + (*ji).second.get_lower())/2.0 << "\t" << (*ji).second.get_upper()
    		<< "\t" << (*ji).second.get_lower() << endl;
    	    }
    	}

      RDJvalList::iterator rl = RDJList.lower_bound((*iter).first);
      RDJvalList::iterator ru = RDJList.upper_bound((*iter).first);
      for(RDJvalList::iterator ri = rl; ri != ru; ri++)
    	{
    	  string type;
    	  switch ((*ri).second.get_type())
    	    {
    	    case RDHNHB: type = "rdhn-hbr"; break;
    	    case RDHAHB: type = "rdha-hbr"; break;
    	    case RJHAHB: type = "rjha-hbr"; break;
    	    case RJHNHB: type = "rjhn-hbr"; break;
    	    default: type = "";
    	    }
    	  if (type != "")
    	    {
    	      s << "\t" << type << "\t" << ((*ri).second.get_upper() + (*ri).second.get_lower())/2.0 << "\t" << (*ri).second.get_upper()
    		<< "\t" << (*ri).second.get_lower() << endl;
    	    }
    	}
	
	} // haveJ
      s << "  }" << endl;
    }
}

void prtHyperInput(ostream& s, const SEQM& seq, const DianaMap& DianaBuffer, bool haveNOE, bool haveJ, bool haveRotamer, int LLength)
{
s.precision(3); 

// Header for HYPER input file
  s << "--" << endl; 
  s << "-- Input file for hyper created by " << versionHeader() << endl;
  s << "--" << endl; 
	if (haveRotamer)
	{
		s << "rotamer { chi }" << endl;
	}
  s << "djlist 'Input for HYPER from NOESY_Assign'" << endl;
  s << "{" << endl;
  prtIntraHyper(s, seq, DianaBuffer, haveNOE, haveJ, 0);
  if (LLength) prtIntraHyper(s, seq, DianaBuffer, haveNOE, haveJ, LLength+(*seq.rbegin()).first);
  s << "}" << endl << flush;
}




