/***************************************************************************
                          main.cpp  -  the main control file to do NOESY_Assign

                             -------------------
    begin                : Tue Aug 22 2000
    copyright            : (C) 2000 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/
#include <unistd.h>
#include <cstdlib>
#include <getopt.h>

#include "utility.h"
#include "noesy_assign.h"
#include "faces.h"
#include <time.h>
#include <sys/param.h>

void CSI_method(bool, ostream&, const SEQM&, const Atom_List&, CSIMap&, CSIMap&, CSIMap&, bool, bool);
void prtXeasyShift(ostream&, const Atom_List&);

static int iso_flag;

bool DEBUG = 0;
int LLength = 0;

string outputDir = "";
string controlFileName = "";
ControlMap control;
Command command;

CSIMap indexFilter; // CSI and secondary structure
CSIMap indexCaRaw;
CSIMap indexCbRaw;

map<char, string, less<char> > residue_name;
map<char, map<string, string, less<string> >, less<char> > layer_map;
map<string, char, less<string> > symbol3_1;
map<int, string, less<int> > specialName; // not used anymore

//seq list
SEQM seq;
//resonance assignment list
//peak list
PeakListDeque peakListDeque;
// analyze NOESY peak-list
NOESY_Assign na;

// used in secondary structure determination
// and hyper dihedral angle constraint generation
JvalList JList;
RDJvalList RDJList;

bool hasRDC = 0;

set<int, less<int> > CISPeptideBond;
set<pair<int,int>, less<pair<int, int> > > SSBond;
set<int, less<int> > rdcORI;

//int faces[1000][1000]; // 1-> same faces, 0-> na. -1->different faces
  int faces[1000][1000];
Faces contactFaces;

void init();

// Java/JNI interface will be added here
// na methods listed here will be put into GUI interface
// more na methods that should be interacted with users will be added
int main(int argc, char* argv[])
{

	//count the running time
	time_t time_start = time(NULL);
	tms timeLockS; clock_t startT = times(&timeLockS);
	clock_t start = clock();

	cout << endl << versionHeader() << endl;
	cout << " Rutgers, The State University of New Jersey" << endl << endl;

	if (argc == 1)
	  {
	    cout << getHelp() << endl;
	    return EXIT_SUCCESS;
	  }

	for(int i = 0; i<1000; ++i)
	for(int j=0; j<1000; ++j)
		faces[i][j] = 0;

	CISPeptideBond.clear();

  ////////////////////////////////////////////////////////////////////////////////////////////////
  // setenv, if NALIB has been set in the environment, then use the one set in the environment  //
  ////////////////////////////////////////////////////////////////////////////////////////////////
  if (!getenv("ASLIB"))
		error("Error", "Please define enviormental variable ASLIB");

	init(); // initation --- more work, should be remove and move to lib directiory

	////////////////////////////////////////////////
	// parsing the arguments --- more work        //
	////////////////////////////////////////////////
	//int _LLength = 100; // can be a input variable

	static struct option long_options[] =
	  {
	    /* These options set a flag. */
	    //  {"verbose", no_argument,       &verbose_flag, 1},
	    //{"brief",   no_argument,       &verbose_flag, 0},
	    /* These options don't set a flag.
	       We distinguish them by their indices. */
	    //{"add",     no_argument,       0, 'a'},
	    //{"append",  no_argument,       0, 'b'},
	    //{"delete",  required_argument, 0, 'd'},
	    //{"create",  required_argument, 0, 'c'},
	    {"iso",    no_argument, &iso_flag, 'i'},
	    {0, 0, 0, 0}
	  };

	int option_index = 0;

  int co;
  while ((co = getopt_long(argc, argv, "c:C:o:O:f:F:k:K:mM:r:q:QsSRtThHnNjJdDvVsSiIaA", long_options, &option_index)) != -1)
    switch (co)
    {
    case 0:
      //if (long_options[option_index].flag != 0)
      //break;
     break;


    	case 'h':
    	case 'H':
    		cout << getHelp() << endl;
    		return EXIT_SUCCESS;
    		break;
      case 'c':		// control file
      case 'C':
      	controlFileName = optarg;
    	  break;
      case 'o':		// output file
      case 'O':
      	outputDir = optarg;
    	  break;
      case 'k':		// calibration factor
      case 'K':
        na.setKD(atof(optarg));
        break ;
      case 'm':
				na.setMINCON();
				break;
      case 'M':        // if have Manual structure for comparision
			  na.setSIM();
			  na.setCOMPARE();
			  na.setManualFileName(optarg); // struture name
			  break;
		  case 'r':
		    na.setRESTORE();
		    na.setSourcePdbPath(optarg); // the previous calculation path
		    break;
		  case 'f':
		  case 'F':
		  	na.setINITF();
		  	na.setInitFold(optarg); // initial fold
		  	break;
		  case 'R':
				na.setHaveRotamer();
				break;
		  case 'q':
      case 'Q':
        	na.setIsQ();
        	na.setQpdb(optarg); // the structure name
		  	na.setInitFold(optarg); // initial fold
        break;
      case 'a':
      case 'A':
      	na.setIsAMBI();
	break;
      case  's':
      case  'S':
        na.setIsS();
	break;
      case 'd':
    case 'D':  // debug
        DEBUG = 1;
        break ;
      case 'v':
      case 'V':
        na.setIsV();
        break;
      case 'n':
	na.setNOCSI();
      case 'N':
        na.setHaveNOE();
        break;
      case 'j':
      case 'J':
      	na.setHaveJ();
      	break;
      case 'i':
      case 'I':
	iso_flag = 1;
      	break;
      case 't':
      case 'T':
	break;
      default:
        cout << " \n *** Warning: UNIMPLEMENTED option: " << (char)co << endl;
    }

  if (controlFileName == "")
		error("no controlFile name given", "Please use -c control_file");

  createOutputDir(outputDir);

  // readin and analysis controlfile
  readControlFile(controlFileName, control, peakListDeque, command);

  //////////////////////////////////////
  // start NOESY_Assign               //
  //////////////////////////////////////

  na.initReportOvw(controlFileName, time_start, argc, argv);

  // readin input file that listed in the control-file
	na.readInputFile(controlFileName);
       if (control["RDC"] != "")
          {
            hasRDC = 1;
          }

	indexFilter.clear();
  indexCaRaw.clear();
  indexCbRaw.clear();

	//--------------------------------------------------------------//
	//                                                              //
	// Start to Run NOESY_Assign, Cycle 1                           //
	//--------------------------------------------------------------//

	// 1. do simple matching
	na.runSimpleMatch();


	if (na.isQ())
	{
	  na.runRfactor();
	  tms timeLockE; clock_t endT = times(&timeLockE);
	  runSummary(NOESY_Assign::outputOvwRPF(), startT, endT, timeLockS, timeLockE);
	  clock_t end = clock();
		return EXIT_SUCCESS;
	}

	na.prtNoisePeak();

	//2. analysis secondary structure
	na.runSRExpect();
	na.runRefine_Shift();

	na.runSRExpect(); // may add some new assignments
	na.runRefine_Shift();

	cout << "mincon" << NOESY_Assign::MINCON() << endl;

	CSI_method(!NOESY_Assign::MINCON(), NOESY_Assign::outputSec(), seq, NOESY_Assign::atoms(), indexFilter, indexCaRaw, indexCbRaw, na.isCSI(), iso_flag);
	if(na.IC())
	{
		na.runRemoveIC();
	}
	SS auto_ssT; auto_ssT.clear();
	na.runFindSecondarySegment(); // helix
	na.runHelixExpect();

	//3. assigned unique and sym
	na.runInitFold();  // also -f using input pdb files

	//4. find strands
	na.runFindSecondaryRegister();
	na.runSheetExpect();

	if (na.isINITF())
	{
		na.runContactFaces();
		na.runRefine_Shift();
	}

 	na.runFindHHPacking();

	//7. find h-bonds
	na.runFindCisXP();
	na.runFindHbond();

	////////////////////////////////////
	// 8. iterative vio + pdb         //
	// more control will be added     //
	// such as DYANA, HYPER, view     //
	////////////////////////////////////
	na.runVioPdb(); // getECfromSS
	na.runRfactor();

// get CPU time info
	tms timeLockE; clock_t endT = times(&timeLockE);
	runSummary(NOESY_Assign::outputOvw(), startT, endT, timeLockS, timeLockE);
	cout << endl << "Done" << endl;

  return EXIT_SUCCESS;
}
