/***************************************************************************
                          nc4noesy_hsqc.cpp  -  description
                             -------------------
    begin                : Mon Mar 3 2003
    copyright            : (C) 2003 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#include "nc4noesy_hsqc.h"

NC3NOESY_HSQC::NC3NOESY_HSQC(){
}
NC3NOESY_HSQC::~NC3NOESY_HSQC(){
}

NC3NOESY_HSQC::NC3NOESY_HSQC(bool flag, bool haveIC, float hx1t, float x1t, float hx2t, float f, string label)
	:D3NOESY(flag, haveIC, hx1t, x1t, hx2t, f, label)
{
}

const string NC3NOESY_HSQC::Type() const
{
  return "3DCN";
}

bool NC3NOESY_HSQC::canbeMom(const atom* atom1) const
{
  if (atom1->isatt_N() || (atom1->isatt_C())) // && ((atom1->get_shift() < wl) || (atom1->get_shift() > wu))))
    return 1;
  return 0;
}

bool NC3NOESY_HSQC::canbeDad(const atom* atom1) const
{
	return 1; 
}

bool NC3NOESY_HSQC::isPossFather(const atom* temp1, pair<float, float> bound) const
{
	if (temp1->isH())
	{
		return 1;
	} else
		return 0;
}

bool NC3NOESY_HSQC::isPossMother(const atom* temp1, pair<float, float> bound) const
{
	if (temp1->isatt_N() && (((temp1->att_N()->get_shift()) <= bound.second) && ((temp1->att_N()->get_shift()) >= bound.first)))
		return 1;
		
	deque<atom*> att_C = temp1->att_C();

	for(deque<atom*>::iterator it = att_C.begin();
		it != att_C.end(); it++)
	{
	  if ((*it) && canbeMom(temp1))
		{
			if (((*it)->get_shift() <= bound.second) && ((*it)->get_shift() >= bound.first))
				return 1;
		}
	}
	return 0;
}

// BH root at B,
void NC3NOESY_HSQC::Find_UniqueBH(int i, int j, method mthd)
{
	MatchPeakAtombyAtomIndex MombyIndex;

	for(MatchPeakAtom::iterator iter = MombySimpleMatch.begin(); iter != MombySimpleMatch.end(); iter++)
		if ((*iter).second->get_layer().substr(0,1) == "B")
			MombyIndex.insert(make_pair((*iter).second->get_index(), (*iter)));

	// find all possible (i,j) that root at i
	MatchPeakAtombyAtomIndex::iterator
		res1RootLower = MombyIndex.lower_bound(i);
	MatchPeakAtombyAtomIndex::iterator
		res1RootUpper = MombyIndex.upper_bound(i);

	for(MatchPeakAtombyAtomIndex::iterator iter1 = res1RootLower;
		iter1 != res1RootUpper; iter1++)
	{
		// for each peak, find all possible father
		MatchPeakAtom::iterator fatherLower
			= DadbySimpleMatch.lower_bound((*iter1).second.first);
		MatchPeakAtom::iterator fatherUpper
			= DadbySimpleMatch.upper_bound((*iter1).second.first);

		for(MatchPeakAtom::iterator iterf = fatherLower; iterf != fatherUpper; iterf++)
			if (((*iterf).second->get_index() == j) &&
				((*iterf).second->get_layer() == "N"))
			{
					MakeBuffer((*iterf).first, (*iter1).second.second, (*iterf).second, mthd);
			}
	}

	//MatchPeakAtombyAtomIndex MombyIndex;
	MombyIndex.clear();
	
	for(MatchPeakAtom::iterator iter = MombySimpleMatch.begin(); iter != MombySimpleMatch.end(); iter++)
		if ((*iter).second->get_layer() == "N")
			MombyIndex.insert(make_pair((*iter).second->get_index(), (*iter)));

	// find all possible (i,j) that root at i
		res1RootLower = MombyIndex.lower_bound(i);
		res1RootUpper = MombyIndex.upper_bound(i);

	for(MatchPeakAtombyAtomIndex::iterator iter1 = res1RootLower;
		iter1 != res1RootUpper; iter1++)
	{
		// for each peak, find all possible father
		MatchPeakAtom::iterator fatherLower
			= DadbySimpleMatch.lower_bound((*iter1).second.first);
		MatchPeakAtom::iterator fatherUpper
			= DadbySimpleMatch.upper_bound((*iter1).second.first);

		for(MatchPeakAtom::iterator iterf = fatherLower; iterf != fatherUpper; iterf++)
			if (((*iterf).second->get_index() == j) &&
				((*iterf).second->get_layer().substr(0,1) == "B"))
			{
					MakeBuffer((*iterf).first, (*iter1).second.second, (*iterf).second, mthd);
			}
	}

}

// root at A
void NC3NOESY_HSQC::Find_UniqueAH(int i, int j, method mthd)
{
	MatchPeakAtombyAtomIndex MombyIndex;

	for(MatchPeakAtom::iterator iter = MombySimpleMatch.begin(); iter != MombySimpleMatch.end(); iter++)
		if ((*iter).second->get_layer().substr(0,1) == "A")
			MombyIndex.insert(make_pair((*iter).second->get_index(), (*iter)));

	// find all possible (i,j) that root at i
	MatchPeakAtombyAtomIndex::iterator
		res1RootLower = MombyIndex.lower_bound(i);
	MatchPeakAtombyAtomIndex::iterator
		res1RootUpper = MombyIndex.upper_bound(i);

	for(MatchPeakAtombyAtomIndex::iterator iter1 = res1RootLower;
		iter1 != res1RootUpper; iter1++)
	{
		// for each peak, find all possible father
		MatchPeakAtom::iterator fatherLower
			= DadbySimpleMatch.lower_bound((*iter1).second.first);
		MatchPeakAtom::iterator fatherUpper
			= DadbySimpleMatch.upper_bound((*iter1).second.first);

		for(MatchPeakAtom::iterator iterf = fatherLower; iterf != fatherUpper; iterf++)
			if (((*iterf).second->get_index() == j) &&
				((*iterf).second->get_layer() == "N"))
			{
					MakeBuffer((*iterf).first, (*iter1).second.second, (*iterf).second, mthd);
			}
	}
	//atchPeakAtombyAtomIndex MombyIndex;
	MombyIndex.clear();
	
	for(MatchPeakAtom::iterator iter = MombySimpleMatch.begin(); iter != MombySimpleMatch.end(); iter++)
		if ((*iter).second->get_layer() == "N")
			MombyIndex.insert(make_pair((*iter).second->get_index(), (*iter)));

	// find all possible (i,j) that root at i
		res1RootLower = MombyIndex.lower_bound(i);
		res1RootUpper = MombyIndex.upper_bound(i);

	for(MatchPeakAtombyAtomIndex::iterator iter1 = res1RootLower;
		iter1 != res1RootUpper; iter1++)
	{
		// for each peak, find all possible father
		MatchPeakAtom::iterator fatherLower
			= DadbySimpleMatch.lower_bound((*iter1).second.first);
		MatchPeakAtom::iterator fatherUpper
			= DadbySimpleMatch.upper_bound((*iter1).second.first);

		for(MatchPeakAtom::iterator iterf = fatherLower; iterf != fatherUpper; iterf++)
			if (((*iterf).second->get_index() == j) &&
				((*iterf).second->get_layer().substr(0,1) == "A"))
			{
					MakeBuffer((*iterf).first, (*iter1).second.second, (*iterf).second, mthd);
			}
	}
}

// 4D noesy 
NC4NOESY_HSQC::NC4NOESY_HSQC(){
}
NC4NOESY_HSQC::~NC4NOESY_HSQC(){
}

NC4NOESY_HSQC::NC4NOESY_HSQC(bool flag, bool haveIC, float hx1t, float x1t, float hx2t, float x2t, float f, string label)
	:D4NOESY(flag, haveIC, hx1t, x1t, hx2t, x2t, f, label)
{
}

const string NC4NOESY_HSQC::Type() const
{
  return "4DNC";
}

bool NC4NOESY_HSQC::canbeMom(const atom* atom1) const
{
	if (atom1->isatt_N())
		return 1;
	return 0;
}

bool NC4NOESY_HSQC::canbeDad(const atom* atom1) const
{
	if (atom1->isatt_C())
		return 1;
	return 0;
}

bool NC4NOESY_HSQC::isPossFather(const atom* temp1, pair<float, float> bound) const
{
	deque<atom*> att_C = temp1->att_C();

	for(deque<atom*>::iterator it = att_C.begin();
		it != att_C.end(); it++)
	{
		if (*it)
				if (((*it)->get_shift() <= bound.second) && ((*it)->get_shift() >= bound.first))
				return 1;
	}
	return 0;
}

bool NC4NOESY_HSQC::isPossMother(const atom* temp1, pair<float, float> bound) const
{
	if (temp1->isatt_N() && (((temp1->att_N()->get_shift()) <= bound.second) && ((temp1->att_N()->get_shift()) >= bound.first)))
		return 1;
	return 0;
}


// maynot needed 
CN4NOESY_HSQC::CN4NOESY_HSQC(){
}
CN4NOESY_HSQC::~CN4NOESY_HSQC(){
}

CN4NOESY_HSQC::CN4NOESY_HSQC(bool flag, bool haveIC, float hx1t, float x1t, float hx2t, float x2t, float f, string label)
	:D4NOESY(flag, haveIC, hx1t, x1t, hx2t, x2t, f, label)
{
}

const string CN4NOESY_HSQC::Type() const
{
  return "4DCN";
}

bool CN4NOESY_HSQC::canbeMom(const atom* atom1) const
{
	if (atom1->isatt_C())
		return 1;
	return 0;
}

bool CN4NOESY_HSQC::canbeDad(const atom* atom1) const
{
	if (atom1->isatt_N())
		return 1;
	return 0;
}

bool CN4NOESY_HSQC::isPossMother(const atom* temp1, pair<float, float> bound) const
{
	deque<atom*> att_C = temp1->att_C();

	for(deque<atom*>::iterator it = att_C.begin();
		it != att_C.end(); it++)
	{
		if (*it)
				if (((*it)->get_shift() <= bound.second) && ((*it)->get_shift() >= bound.first))
				return 1;
	}
	return 0;
}

bool CN4NOESY_HSQC::isPossFather(const atom* temp1, pair<float, float> bound) const
{
	if (temp1->isatt_N() && (((temp1->att_N()->get_shift()) <= bound.second) && ((temp1->att_N()->get_shift()) >= bound.first)))
		return 1;
	return 0;
}
