/***************************************************************************
                          noesy_assign.cpp  -  mainly related to init_fold analysis
                             -------------------
    begin                : Fri Dec 14 2001
    copyright            : (C) 2001 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/
#include <fstream>
#include <string>
#include <cmath>

#include "noesy_assign.h"
#include "template.h"
#include "faces.h"
#include "hbond.h"

extern bool DEBUG;
extern string outputDir;
extern ControlMap control;
extern Command command;
extern PeakListDeque peakListDeque;
extern set<int, less<int> > CISPeptideBond;
extern set<pair<int,int>, less<pair<int, int> > > SSBond;
extern set<int, less<int> > rdcORI;

extern CSIMap indexFilter;
extern CSIMap indexCaRaw;
extern CSIMap indexCbRaw;

extern int LLength;

bool sameHydro(char, char);
extern Faces contactFaces;

istream& operator>> (istream&, NHSlowList&);
istream& operator>> (istream&, set<HBOND, less<HBOND> >&);
istream& operator>> (istream&, map<HBOND, ECscore, less<HBOND> >&);

void prtHyperInput(ostream&, const SEQM&, const DianaMap&, bool, bool, bool, int);
void CSI_method(bool, ostream&, const SEQM&, const Atom_List&, CSIMap&, CSIMap&, CSIMap&);
void checkHisStat(ostream&, const Atom_List&, map<int, string, less<int> >&);

bool NOESY_Assign::_monster = 0;
bool NOESY_Assign::_IC = 0;
int NOESY_Assign::_dcycle = 0;
int NOESY_Assign::_ddcycle = 1;
bool NOESY_Assign::_EC = 0;
bool NOESY_Assign::_AMBI = 0;
bool NOESY_Assign::_V = 0;
bool NOESY_Assign::_S = 0;
bool NOESY_Assign::_Q = 0;
bool NOESY_Assign::_csi = 1;
float	NOESY_Assign::_KCalibration = 0.001835 ; // old is 0.003804
float NOESY_Assign::_KD = 3.4;

const char * const NOESY_Assign::_EOS = "End of Stream";

Atom_List NOESY_Assign::_atoms;
deque<Pdb> NOESY_Assign::_pdbList; // for bootscrapping, q-factor, initfold
deque<Pdb> NOESY_Assign::_powerPdbList; // for compare only

DianaMap NOESY_Assign::_DianaBuffer;
DianaMap NOESY_Assign::_ICDianaBuffer;

DianaDeque NOESY_Assign::_DianaBufferDeque;
DianaDeque NOESY_Assign::_ICDianaBufferDeque;

ofstream NOESY_Assign::_outputExm;
ofstream NOESY_Assign::_outputLog;
ofstream NOESY_Assign::_outputOvw;
ofstream NOESY_Assign::_outputOvwRPF;
ofstream NOESY_Assign::_outputROvw;
ofstream NOESY_Assign::_outputSec;
ofstream NOESY_Assign::_outputError;

ofstream NOESY_Assign::_outputI;
ofstream NOESY_Assign::_outputL;
ofstream NOESY_Assign::_outputM;
ofstream NOESY_Assign::_outputUA;


 bool NOESY_Assign::_haveNOE = 0;
 bool NOESY_Assign::_haveJ = 0;
 bool NOESY_Assign::_haveRotamer = 0;

 bool NOESY_Assign::_COMPARE = 0;
 bool NOESY_Assign::_MINCON = 0;

 ContactPairSet NOESY_Assign::_asContactSet;

 Par NOESY_Assign::_par;

	resSet NOESY_Assign::_ContactSet;
	deque<HBOND> NOESY_Assign::_antiParaRegister;  // antiparalley_register
	deque<HBOND> NOESY_Assign::_paraRegister; // para_register

	SS NOESY_Assign::_secondaryStruct;


NOESY_Assign::NOESY_Assign(){
	SIM = 0;
 	RESTORE = 0;

	INITF = 0;
	initFold = "";

  manualFileName="";
  sourcePdbPath="";
  current="";
  Qpdb="";

  _asContactSet.clear();

  usvv = 0;
}

NOESY_Assign::~NOESY_Assign(){
}

void NOESY_Assign::setSourcePdbPath(const char* t)
{
	sourcePdbPath = t;
	if (sourcePdbPath != "")
	{
  	if (*sourcePdbPath.rbegin() != '/')
    	sourcePdbPath += "/";
	}
}

float NOESY_Assign::RmaxQ()
{
	float rmax;
	for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
	  {
	    float tp = (*iter)->get_RmaxQ();
	    if (tp > rmax)
	      rmax = tp;
	  }
	return rmax;
}

float NOESY_Assign::Rmax()
{
  float rmax;
  for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
    {
      float tp = (*iter)->get_Rmax();
      if (tp > rmax)
	rmax = tp;
    }
  return rmax;
}

//--------------------------------------------------------------------//
//                                                                    //
// Reading Input File                                                 //
//--------------------------------------------------------------------//
void NOESY_Assign::readInputFile(string controlFileName)
{
	////////////////////////////////////////////////////////
	// check the control file and also read input files  ///
	////////////////////////////////////////////////////////

	_outputOvw << "\nINPUT" << endl;
	_outputOvw << "=====" << endl << endl;

	// proteinName
	_outputOvw << "proteinName: " << control["proteinName"] << endl;

	// Number of Cycles
	if (control["nCycles"] == "")
	  error("no nCycles defined", controlFileName);
	_outputOvw << "nCycles: " << control["nCycles"] << endl;

	// residues have cis-peptide
	if (control["cis"] != "")
	{
		string temp = control["cis"];
		while (temp.find_first_of(",") <= temp.size())
		{
			int i = atoi((temp.substr(0, temp.find_first_of(","))).c_str());
	    CISPeptideBond.insert(i);
	    temp = temp.substr(temp.find_first_of(",")+1, temp.size());
		}
		if (temp !="")
		{
			int i = atoi(temp.c_str());
	    CISPeptideBond.insert(i);
		}
		_outputOvw << "cis: ";
		for(set<int, less<int> >::iterator i = CISPeptideBond.begin(); i != CISPeptideBond.end(); ++i)
		{
		 		_outputOvw << *i << ",";
		}
		_outputOvw << endl;
	}

	// residues have disulfide-bonds
	if (control["ssbond"] != "")
	{
		string temp = control["ssbond"];
		while (temp.find_first_of(",") <= temp.size())
		{
			string ijpair = temp.substr(0, temp.find_first_of(","));

			if (ijpair.find("-") <= ijpair.size())
			{
				int i = atoi((ijpair.substr(0, ijpair.find("-"))).c_str());
				int j = atoi((ijpair.substr(ijpair.find("-")+1, ijpair.size())).c_str());
	    	SSBond.insert(make_pair(i,j));
	    }
	    temp = temp.substr(temp.find_first_of(",")+1, temp.size());
		}
		if (temp !="")
		{
			int i = atoi((temp.substr(0, temp.find("-"))).c_str());
			int j = atoi((temp.substr(temp.find("-")+1, temp.size())).c_str());
	    SSBond.insert(make_pair(i,j));
		}

		_outputOvw << "ssbond: ";
		for(set<pair<int,int>, less<pair<int,int> > >::iterator i = SSBond.begin(); i != SSBond.end(); ++i)
		{
		 		_outputOvw << i->first << "-" << i->second << ",";
		}
		_outputOvw << endl;
	}

	// rdcOri for cyana
	if (control["rdcORI"] != "")
	{
		string temp = control["rdcORI"];
		while (temp.find_first_of(",") <= temp.size())
		{
		  int i = atoi((temp.substr(0, temp.find_first_of(","))).c_str());
		  rdcORI.insert(i);
		  temp = temp.substr(temp.find_first_of(",")+1, temp.size());
		}
		if (temp !="")
		  {
		    int i = atoi(temp.c_str());
		    rdcORI.insert(i);
		  }
		_outputOvw << "rdcORI: ";
		for(set<int, less<int> >::iterator i = rdcORI.begin(); i != rdcORI.end(); ++i)
		  {
		    _outputOvw << *i << " ";
		  }
		_outputOvw << endl;
	}

	// read in sequence file
	if (control["seqFile"] == "")
	  error("no seqFile defined", controlFileName);
	else
	{
	  ifstream seqStream(control["seqFile"].c_str());
	  if (!seqStream) error("cannot open sequence file", control["seqFile"]);
	  _outputLog << "\n-> Sequence Read As:" << endl;
	  seqStream >> seq;
	  seqStream.close();
	  _outputOvw << "seqFile: " << control["seqFile"] << endl;
	}

	//read in chemicalshift file
	if (control["chemicalShiftFile"] == "")
	  error("no chemicalShiftFile defined", controlFileName);
	_outputOvw << "chemicalShiftFile: " << control["chemicalShiftFile"] << endl;
	ifstream shiftStream(control["chemicalShiftFile"].c_str());
	if (!shiftStream) error("cannot open chemicalShift file", control["chemicalShiftFile"]);
	_outputLog << "\n-> Atoms Read As:" << endl;
	shiftStream >> _atoms;
	shiftStream.close();

	float Hm = 0.01, Xm = 0.1;
	make_att_list(_outputLog, _atoms, Hm, Xm); // if degeneate or close, merge
	//if (DEBUG)
	{
	  _outputLog << "\n-> Final Atoms:" << endl;
	  _outputLog << _atoms;
	}

	if (!isQ())
	  {

	//read in JList file
	if (control["JListFile"] != "")
	{
  	_outputOvw << "JListFile: " << control["JListFile"] << endl;
  	ifstream JListStream(control["JListFile"].c_str());
  	if (!JListStream) error("cannot open J-val file", control["JListFile"]);

  	_outputLog << "\n-> JList Read As:" << endl;
  	JListStream >> JList;

	}

	//read in NHSlowList
	if (control["NHSlowList"] != "")
	{
  	_outputOvw << "NHSlowList: " << control["NHSlowList"] << endl;
  	ifstream ListStream(control["NHSlowList"].c_str());
  	if (!ListStream) error("cannot open NH-val file", control["NHSlowList"]);
  	ListStream >> NHList;
  	ListStream.close();
    //if (DEBUG)
    {
  	  _outputLog << "\n-> NHSlowList Read As:" << endl;

	    for(NHSlowList::iterator i = NHList.begin(); i != NHList.end(); i++)
	      _outputLog << (*i) << " ";
    }
	}

	//read in EC
	if (control["EC"] != "")
	{
  	_outputOvw << "EC: " << control["EC"] << endl;
  	ifstream ECStream(control["EC"].c_str());
  	if (!ECStream) error("cannot open EC file", control["EC"]);
  	ECStream >> ECList;
  	ECStream.close();
	_EC = 1;
    //if (DEBUG)
    {
  	  _outputLog << "\n-> EC Read As:" << endl;

	  for(map<HBOND, ECscore, less<HBOND> >::iterator i = ECList.begin(); i != ECList.end(); i++)
	    _outputLog << (*i).first.first << "\t" << (*i).first.second << "\t" << (*i).second.weightAdjust << "\n";
    }
	}
	  }

	//read in ParTable
	if (control["par"] != "")
	{
  	_outputOvw << "par table: " << control["par"] << endl;
  }
  else
  {
  	string NALIBPath = getenv("ASLIB");
  	control["par"] = NALIBPath + "/par.tbl";
  	_outputOvw << "par table: " << "Default" << endl;
  }

  	ifstream ListStream(control["par"].c_str());
  	if (!ListStream) error("cannot open par file", control["par"]);
  	ListStream >> _par;
  	ListStream.close();
    //if (DEBUG)
    {
  	  _outputLog << "\n-> par Table is Read As:" << endl;

  	  _outputLog << _par << endl;
    }


  ////////////////////////////////////
  // check and read in peakList   ///
  ////////////////////////////////////

    tol tt; tt.reset();

  _outputOvw << "PeakList: " << endl;
  for(PeakListDeque::iterator pi = peakListDeque.begin(); pi != peakListDeque.end(); pi++)
  {
    // check the attribution first
  	if ((*pi)["name"] == "")
	    error("no PeakList File defined", (*pi)["name"]);
	  else
	    _outputOvw << "File: " << (*pi)["name"] << endl;
  	if ((*pi)["dimension"] == "")
	    error("no dimension defined", (*pi)["name"]);
    else
      _outputOvw << "  Dimension: " << (*pi)["dimension"];
  	if ((*pi)["IC"] == "")
	    error("no IC defined", (*pi)["name"]);
    else
  	if ((*pi)["iperc"] == "")
	    (*pi)["iperc"] = "0.0125";
    _outputOvw << "  iperc: " << (*pi)["iperc"];

  	if ((*pi)["waterFlag"] == "")
	  error("no waterFlag defined", (*pi)["name"]);
    else
      _outputOvw << "  waterFlag: " << (*pi)["waterFlag"];


	//Rmax, RmaxQ
	if ((*pi)["Rmax"] == "")
	  {
	    (*pi)["Rmax"] = "5.0";
	  }

	if ((*pi)["RmaxQ"] == "")
	  {
	    (*pi)["RmaxQ"] = "5.0";
	  }



	float wl = 0; float wu = 0;
  	if ((*pi)["water"] != "")
	    {
	      string temp = (*pi)["water"];
	      if (temp.find_first_of("-") <= temp.size())
		{
		  wl = atof((temp.substr(0, temp.find_first_of("-"))).c_str());
		  temp = temp.substr(temp.find_first_of("-")+1, temp.size());
		}
	      if (temp !="")
		{
		  wu = atof(temp.c_str());
		}
	      _outputOvw << " water suppresion range: " << wl << "-" << wu << endl;
	    }

  	if ((*pi)["col.intensity"] == "")
	    error("no col.intensity defined", (*pi)["name"]);
    else
      _outputOvw << "  col.intensity: " << (*pi)["col.intensity"];
  	if ((*pi)["col.label"] != "")
      _outputOvw << "  col.label: " << (*pi)["col.label"];
  	if ((*pi)["col.id"] != "")
      _outputOvw << "  col.id: " << (*pi)["col.id"];

    _outputOvw << endl;

    // hx1
  	if ((*pi)["col.hx1"] == "")
	    error("no col.hx1 defined", (*pi)["name"]);
    else
      _outputOvw << "  col.hx1: " << (*pi)["col.hx1"];
    if ((*pi)["hx1.tol"] == "")
	    error("no hx1.tol defined ", (*pi)["name"]);
    else
      _outputOvw << "  hx1.tol: " << (*pi)["hx1.tol"];
    if ((*pi)["hx1.sw"] == "")
	    error("no hx1.sw defined ", (*pi)["name"]);
    else
      _outputOvw << "  hx1.sw: " << (*pi)["hx1.sw"];
    if ((*pi)["hx1.shift"] == "")
	    error("no hx1.shift defined ", (*pi)["name"]);
    else
      _outputOvw << "  hx1.shift: " << (*pi)["hx1.shift"];
    if ((*pi)["hx1.type"] == "")
	    error("no hx1.type defined ", (*pi)["name"]);
    else
      _outputOvw << "  hx1.type: " << (*pi)["hx1.type"] << endl;
  	if ((*pi)["hx1.sign"] == "")
	    error("no hx1 sign defined", (*pi)["hx1.sign"]);
    else
      _outputOvw << "  hx1.sign: " << (*pi)["hx1.sign"] << endl;
		tt.HX1 = atoi((*pi)["hx1.sign"].c_str());


    // hx2
  	if ((*pi)["col.hx2"] == "")
	    error("no col.hx2 defined", (*pi)["name"]);
    else
      _outputOvw << "  col.hx2: " << (*pi)["col.hx2"];
    if ((*pi)["hx2.tol"] == "")
	    error("no hx2.tol defined ", (*pi)["name"]);
    else
      _outputOvw << "  hx2.tol: " << (*pi)["hx2.tol"];
    if ((*pi)["hx2.sw"] == "")
	    error("no hx2.sw defined ", (*pi)["name"]);
    else
      _outputOvw << "  hx2.sw: " << (*pi)["hx2.sw"];
    if ((*pi)["hx2.shift"] == "")
	    error("no hx2.shift defined ", (*pi)["name"]);
    else
      _outputOvw << "  hx2.shift: " << (*pi)["hx2.shift"];
    if ((*pi)["hx2.type"] == "")
	    error("no hx2.type defined ", (*pi)["name"]);
    else
      _outputOvw << "  hx2.type: " << (*pi)["hx2.type"] << endl;
  	if ((*pi)["hx2.sign"] == "")
	    error("no hx2 sign defined", (*pi)["hx2.sign"]);
    else
      _outputOvw << "  hx2.sign: " << (*pi)["hx2.sign"] << endl;
		tt.HX2 = atoi((*pi)["hx2.sign"].c_str());

    // x1
    if (atoi((*pi)["dimension"].c_str()) >= 3)
    {
    	if ((*pi)["col.x1"] == "")
  	    error("no col.x1 defined", (*pi)["name"]);
      else
        _outputOvw << "  col.x1: " << (*pi)["col.x1"];
      if ((*pi)["x1.tol"] == "")
  	    error("no x1.tol defined ", (*pi)["name"]);
      else
        _outputOvw << "   x1.tol: " << (*pi)["x1.tol"];
      if ((*pi)["x1.sw"] == "")
  	    error("no x1.sw defined ", (*pi)["name"]);
      else
        _outputOvw << "    x1.sw: " << (*pi)["x1.sw"];
      if ((*pi)["x1.shift"] == "")
  	    error("no x1.shift defined ", (*pi)["name"]);
      else
        _outputOvw << "   x1.shift: " << (*pi)["x1.shift"];
      if ((*pi)["x1.type"] == "")
  	    error("no x1.type defined ", (*pi)["name"]);
      else
        _outputOvw << "   x1.type: " << (*pi)["x1.type"] << endl;
  	if ((*pi)["x1.sign"] == "")
	    error("no x1 sign defined", (*pi)["x1.sign"]);
    else
      _outputOvw << "  x1.sign: " << (*pi)["x1.sign"] << endl;
		tt.X1= atoi((*pi)["x1.sign"].c_str());

    }
    // x2
    if (atoi((*pi)["dimension"].c_str()) == 4)
    {
    	if ((*pi)["col.x2"] == "")
  	    error("no col.x2 defined", (*pi)["name"]);
      else
        _outputOvw << "  col.x2: " << (*pi)["col.x2"];
      if ((*pi)["x2.tol"] == "")
  	    error("no x2.tol defined" , (*pi)["name"]);
      else
        _outputOvw << "   x2.tol: " << (*pi)["x2.tol"];
      if ((*pi)["x2.sw"] == "")
  	    error("no x2.sw defined " , (*pi)["name"]);
      else
        _outputOvw << "    x2.sw: " << (*pi)["x2.sw"];
      if ((*pi)["x2.shift"] == "")
  	    error("no x2.shift defined " , (*pi)["name"]);
      else
        _outputOvw << "   x2.shift: " << (*pi)["x2.shift"];
      if ((*pi)["x2.type"] == "")
  	    error("no x2.type defined" , (*pi)["name"]);
      else
        _outputOvw << "   x2.type: " << (*pi)["x2.type"] << endl;
  	if ((*pi)["x2.sign"] == "")
	    error("no x2 sign defined", (*pi)["x2.sign"]);
    else
      _outputOvw << "  x2.sign: " << (*pi)["x2.sign"] << endl;
		tt.X2 = atoi((*pi)["x2.sign"].c_str());
    }

    // read in
    string fileLabel = (*pi)["name"];
    fileLabel.erase(0, fileLabel.rfind("/")+1);

    switch (atoi(((*pi)["dimension"] + (*pi)["IC"]).c_str()))
    {
      case 20: // 2d noesy, not IC
      {

		    D2NOESYDeque.push_back(D2NOESY(atoi((*pi)["waterFlag"].c_str()), atoi((*pi)["haveIC"].c_str()), atof((*pi)["hx1.tol"].c_str()),
		                                   atof((*pi)["hx2.tol"].c_str()), atof((*pi)["iperc"].c_str()), fileLabel));

		    ifstream peakFile((*pi)["name"].c_str());
		    if (!peakFile) error("cannot open input file", (*pi)["name"]);
		    D2NOESYDeque.back().setPeakControlPtr(&(*pi));
		    _outputLog << "\n-> " << (*pi)["name"].c_str() << " is read:" << endl;
		    peakFile >> D2NOESYDeque.back(); // how to read, need to worry
		    peakFile.close();

		    D2NOESYDeque.back().setDianaBufferPtr(&_DianaBuffer);
		    D2NOESYDeque.back().setSWSF(tt, atof((*pi)["hx1.sw"].c_str()), atof((*pi)["hx2.sw"].c_str()));
		    D2NOESYDeque.back().setRmax(atof((*pi)["Rmax"].c_str()));
		    D2NOESYDeque.back().setRmaxQ(atof((*pi)["RmaxQ"].c_str()));
		    D2NOESYDeque.back().Corr_Spectrum(atof((*pi)["hx1.shift"].c_str()), atof((*pi)["hx2.shift"].c_str()));

		    NOEPeakList.push_back(&D2NOESYDeque.back());

		    _outputOvw << "  read as 2Dnoesy " << endl;
        break;
      }
      case 21: // 2d noesy, and IC
		  {
		    ICD2NOESYDeque.push_back(ICD2NOESY(atoi((*pi)["waterFlag"].c_str()), 0, atof((*pi)["hx1.tol"].c_str()),
						       atof((*pi)["hx2.tol"].c_str()), atof((*pi)["iperc"].c_str()),
		                                       fileLabel));

		    ifstream peakFile((*pi)["name"].c_str());
		    if (!peakFile) error("cannot open input file", (*pi)["name"]);
		    ICD2NOESYDeque.back().setPeakControlPtr(&(*pi));
		    _outputLog << "\n-> " << (*pi)["name"].c_str() << " is read:" << endl;
		    peakFile >> ICD2NOESYDeque.back(); // how to read, need to worry
		    peakFile.close();

		    ICD2NOESYDeque.back().setDianaBufferPtr(&_ICDianaBuffer);

		    ICD2NOESYDeque.back().setSWSF(tt, atof((*pi)["hx1.sw"].c_str()), atof((*pi)["hx2.sw"].c_str()));
		    ICD2NOESYDeque.back().setRmax(atof((*pi)["Rmax"].c_str()));
		    ICD2NOESYDeque.back().setRmaxQ(atof((*pi)["RmaxQ"].c_str()));
		    ICD2NOESYDeque.back().Corr_Spectrum(atof((*pi)["hx1.shift"].c_str()), atof((*pi)["hx2.shift"].c_str()));

		    NOEPeakList.push_back(&ICD2NOESYDeque.back());

		    _outputOvw << "  read as 2DICnoesy " << endl;

		    _IC = 1;
        break;
      }
      case 30: // 3dnoesy, not IC
      {
        if ((*pi)["x1.type"] == "N15")
        {
  		    ND3NOESYDeque.push_back(N15NOESY_HSQC(atoi((*pi)["waterFlag"].c_str()), atoi((*pi)["haveIC"].c_str()), atof((*pi)["hx1.tol"].c_str()), atof((*pi)["x1.tol"].c_str()),
  		                                     atof((*pi)["hx2.tol"].c_str()), atof((*pi)["iperc"].c_str()), fileLabel));

  		    ifstream peakFile((*pi)["name"].c_str());
  		    if (!peakFile) error("cannot open input file", (*pi)["name"]);
		      ND3NOESYDeque.back().setPeakControlPtr(&(*pi));
			    _outputLog << "\n-> " << (*pi)["name"].c_str() << " is read:" << endl;
  		    peakFile >> ND3NOESYDeque.back(); // how to read, need to worry
  		    peakFile.close();

		    ND3NOESYDeque.back().setDianaBufferPtr(&_DianaBuffer);
  		    ND3NOESYDeque.back().setSWSF(tt, atof((*pi)["hx1.sw"].c_str()), atof((*pi)["x1.sw"].c_str()), atof((*pi)["hx2.sw"].c_str()));
		    ND3NOESYDeque.back().setRmax(atof((*pi)["Rmax"].c_str()));
		    ND3NOESYDeque.back().setRmaxQ(atof((*pi)["RmaxQ"].c_str()));
  		    ND3NOESYDeque.back().Corr_Spectrum(atof((*pi)["hx1.shift"].c_str()), atof((*pi)["x1.shift"].c_str()), atof((*pi)["hx2.shift"].c_str()));

  		    NOEPeakList.push_back(&ND3NOESYDeque.back());

		      _outputOvw << "  read as 3DN15noesy " << endl;
        }
	else if ((*pi)["x1.type"] == "CN")
	  {
  		    NC3NOESYDeque.push_back(NC3NOESY_HSQC(atoi((*pi)["waterFlag"].c_str()), atoi((*pi)["haveIC"].c_str()), atof((*pi)["hx1.tol"].c_str()), atof((*pi)["x1.tol"].c_str()),atof((*pi)["hx2.tol"].c_str()), atof((*pi)["iperc"].c_str()), fileLabel));

  		    ifstream peakFile((*pi)["name"].c_str());
  		    if (!peakFile) error("cannot open input file", (*pi)["name"]);
		      NC3NOESYDeque.back().setPeakControlPtr(&(*pi));
	 		    _outputLog << "\n-> " << (*pi)["name"].c_str() << " is read:" << endl;
 			    peakFile >> NC3NOESYDeque.back();
  		    peakFile.close();

		    NC3NOESYDeque.back().setDianaBufferPtr(&_DianaBuffer);
  		    NC3NOESYDeque.back().setSWSF(tt, atof((*pi)["hx1.sw"].c_str()), atof((*pi)["x1.sw"].c_str()), atof((*pi)["hx2.sw"].c_str()));
		    NC3NOESYDeque.back().setRmax(atof((*pi)["Rmax"].c_str()));
		    NC3NOESYDeque.back().setRmaxQ(atof((*pi)["RmaxQ"].c_str()));
  		    NC3NOESYDeque.back().Corr_Spectrum(atof((*pi)["hx1.shift"].c_str()), atof((*pi)["x1.shift"].c_str()), atof((*pi)["hx2.shift"].c_str()));

		    if ((wl != 0)&& (wu !=0))
		      {
			NC3NOESYDeque.back().setWaterLine(wl, wu);
		      }

  		    NOEPeakList.push_back(&NC3NOESYDeque.back());

		      _outputOvw << "  read as 3DCNnoesy " << endl;
	  }
         else if ((*pi)["x1.type"].substr(0,3) == "C13")
        {
	  if ((*pi)["x1.type"] == "C13-ali")
	  {
  		    ALICD3NOESYDeque.push_back(ALIC13NOESY_HSQC(atoi((*pi)["waterFlag"].c_str()), atoi((*pi)["haveIC"].c_str()), atof((*pi)["hx1.tol"].c_str()), atof((*pi)["x1.tol"].c_str()),atof((*pi)["hx2.tol"].c_str()), atof((*pi)["iperc"].c_str()), fileLabel));

  		    ifstream peakFile((*pi)["name"].c_str());
  		    if (!peakFile) error("cannot open input file", (*pi)["name"]);
		      ALICD3NOESYDeque.back().setPeakControlPtr(&(*pi));
	 		    _outputLog << "\n-> " << (*pi)["name"].c_str() << " is read:" << endl;
 			    peakFile >> ALICD3NOESYDeque.back();
  		    peakFile.close();

		      ALICD3NOESYDeque.back().setDianaBufferPtr(&_DianaBuffer);
  		    ALICD3NOESYDeque.back().setSWSF(tt, atof((*pi)["hx1.sw"].c_str()), atof((*pi)["x1.sw"].c_str()), atof((*pi)["hx2.sw"].c_str()));
		    ALICD3NOESYDeque.back().setRmax(atof((*pi)["Rmax"].c_str()));
		    ALICD3NOESYDeque.back().setRmaxQ(atof((*pi)["RmaxQ"].c_str()));
  		    ALICD3NOESYDeque.back().Corr_Spectrum(atof((*pi)["hx1.shift"].c_str()), atof((*pi)["x1.shift"].c_str()), atof((*pi)["hx2.shift"].c_str()));

		    if ((wl != 0)&& (wu !=0))
		      {
			ALICD3NOESYDeque.back().setWaterLine(wl, wu);
		      }

  		    NOEPeakList.push_back(&ALICD3NOESYDeque.back());

		      _outputOvw << "  read as ALI-3DC13noesy " << endl;
	  }
	else if ((*pi)["x1.type"] == "C13-aro")
	  {
  		    AROCD3NOESYDeque.push_back(AROC13NOESY_HSQC(atoi((*pi)["waterFlag"].c_str()), atoi((*pi)["haveIC"].c_str()), atof((*pi)["hx1.tol"].c_str()), atof((*pi)["x1.tol"].c_str()),atof((*pi)["hx2.tol"].c_str()), atof((*pi)["iperc"].c_str()), fileLabel));

  		    ifstream peakFile((*pi)["name"].c_str());
  		    if (!peakFile) error("cannot open input file", (*pi)["name"]);
		      AROCD3NOESYDeque.back().setPeakControlPtr(&(*pi));
	 		    _outputLog << "\n-> " << (*pi)["name"].c_str() << " is read:" << endl;
 			    peakFile >> AROCD3NOESYDeque.back();
  		    peakFile.close();

		      AROCD3NOESYDeque.back().setDianaBufferPtr(&_DianaBuffer);
  		    AROCD3NOESYDeque.back().setSWSF(tt, atof((*pi)["hx1.sw"].c_str()), atof((*pi)["x1.sw"].c_str()), atof((*pi)["hx2.sw"].c_str()));
		    AROCD3NOESYDeque.back().setRmax(atof((*pi)["Rmax"].c_str()));
		    AROCD3NOESYDeque.back().setRmaxQ(atof((*pi)["RmaxQ"].c_str()));
  		    AROCD3NOESYDeque.back().Corr_Spectrum(atof((*pi)["hx1.shift"].c_str()), atof((*pi)["x1.shift"].c_str()), atof((*pi)["hx2.shift"].c_str()));

  		    NOEPeakList.push_back(&AROCD3NOESYDeque.back());

		      _outputOvw << "  read as 3DC13noesy " << endl;

	  }
	else
	  {
	  CD3NOESYDeque.push_back(C13NOESY_HSQC(atoi((*pi)["waterFlag"].c_str()), atoi((*pi)["haveIC"].c_str()), atof((*pi)["hx1.tol"].c_str()), atof((*pi)["x1.tol"].c_str()),atof((*pi)["hx2.tol"].c_str()), atof((*pi)["iperc"].c_str()), fileLabel));

  		    ifstream peakFile((*pi)["name"].c_str());
  		    if (!peakFile) error("cannot open input file", (*pi)["name"]);
		      CD3NOESYDeque.back().setPeakControlPtr(&(*pi));
	 		    _outputLog << "\n-> " << (*pi)["name"].c_str() << " is read:" << endl;
 			    peakFile >> CD3NOESYDeque.back();
  		    peakFile.close();

		      CD3NOESYDeque.back().setDianaBufferPtr(&_DianaBuffer);
  		    CD3NOESYDeque.back().setSWSF(tt, atof((*pi)["hx1.sw"].c_str()), atof((*pi)["x1.sw"].c_str()), atof((*pi)["hx2.sw"].c_str()));
		    CD3NOESYDeque.back().setRmax(atof((*pi)["Rmax"].c_str()));
		    CD3NOESYDeque.back().setRmaxQ(atof((*pi)["RmaxQ"].c_str()));
  		    CD3NOESYDeque.back().Corr_Spectrum(atof((*pi)["hx1.shift"].c_str()), atof((*pi)["x1.shift"].c_str()), atof((*pi)["hx2.shift"].c_str()));

		    if ((wl != 0)&& (wu !=0))
		      {
			CD3NOESYDeque.back().setWaterLine(wl, wu);
		      }

  		    NOEPeakList.push_back(&CD3NOESYDeque.back());

		      _outputOvw << "  read as 3DC13noesy " << endl;
	  }
	}
        		    else
		      error("Unknown x1.type", (*pi)["name"]);
        break;
      }
      case 31: // 3dnoesy, and IC
      {
        if ((*pi)["x1.type"] == "N15")
        {
  		    ICN15NOESYDeque.push_back(ICN15NOESY_HSQC(atoi((*pi)["waterFlag"].c_str()), 0, atof((*pi)["hx1.tol"].c_str()), atof((*pi)["x1.tol"].c_str()),
  		                                     atof((*pi)["hx2.tol"].c_str()), atof((*pi)["iperc"].c_str()), fileLabel));

  		    ifstream peakFile((*pi)["name"].c_str());
  		    if (!peakFile) error("cannot open input file", (*pi)["name"]);
			    _outputLog << "\n-> " << (*pi)["name"].c_str() << " is read:" << endl;
		      ICN15NOESYDeque.back().setPeakControlPtr(&(*pi));
  		    peakFile >> ICN15NOESYDeque.back(); // how to read, need to worry
  		    peakFile.close();

		      ICN15NOESYDeque.back().setDianaBufferPtr(&_ICDianaBuffer);
  		    ICN15NOESYDeque.back().setSWSF(tt, atof((*pi)["hx1.sw"].c_str()), atof((*pi)["x1.sw"].c_str()), atof((*pi)["hx2.sw"].c_str()));
		    ICN15NOESYDeque.back().setRmax(atof((*pi)["Rmax"].c_str()));
		    ICN15NOESYDeque.back().setRmaxQ(atof((*pi)["RmaxQ"].c_str()));
  		    ICN15NOESYDeque.back().Corr_Spectrum(atof((*pi)["hx1.shift"].c_str()), atof((*pi)["x1.shift"].c_str()), atof((*pi)["hx2.shift"].c_str()));

    		  NOEPeakList.push_back(&ICN15NOESYDeque.back());

		      _outputOvw << "  read as 3DICN15noesy " << endl;
          _IC = 1;
        }
        else if ((*pi)["x1.type"] == "C13")
        {
  		    ICC13NOESYDeque.push_back(ICC13NOESY_HSQC(atoi((*pi)["waterFlag"].c_str()), 0, atof((*pi)["hx1.tol"].c_str()), atof((*pi)["x1.tol"].c_str()),
  		                                     atof((*pi)["hx2.tol"].c_str()), atof((*pi)["iperc"].c_str()), fileLabel));

  		    ifstream peakFile((*pi)["name"].c_str());
  		    if (!peakFile) error("cannot open input file", (*pi)["name"]);
			    _outputLog << "\n-> " << (*pi)["name"].c_str() << " is read:" << endl;
		      ICC13NOESYDeque.back().setPeakControlPtr(&(*pi));
  		    peakFile >> ICC13NOESYDeque.back(); // how to read, need to worry
  		    peakFile.close();

  		    NOEPeakList.push_back(&ICC13NOESYDeque.back());

		      ICC13NOESYDeque.back().setDianaBufferPtr(&_ICDianaBuffer);
  		    ICC13NOESYDeque.back().setSWSF(tt, atof((*pi)["hx1.sw"].c_str()), atof((*pi)["x1.sw"].c_str()), atof((*pi)["hx2.sw"].c_str()));
		    ICC13NOESYDeque.back().setRmax(atof((*pi)["Rmax"].c_str()));
		    ICC13NOESYDeque.back().setRmaxQ(atof((*pi)["RmaxQ"].c_str()));
  		    ICC13NOESYDeque.back().Corr_Spectrum(atof((*pi)["hx1.shift"].c_str()), atof((*pi)["x1.shift"].c_str()), atof((*pi)["hx2.shift"].c_str()));

		      _outputOvw << "  read as 3DICC13noesy " << endl;
  		    _IC = 1;
        }
		    else
		      error("Unknown X1.TYPE", (*pi)["NAME"]);
         break;
      }
      case 40: // 4dnooesy, not IC
		  {
		  	// cc
        if (((*pi)["x1.type"] == "C13") && ((*pi)["x2.type"] == "C13"))
        {
  		    CC4NOESYDeque.push_back(CC4NOESY_HSQC(atoi((*pi)["waterFlag"].c_str()), atoi((*pi)["haveIC"].c_str()), atof((*pi)["hx1.tol"].c_str()), atof((*pi)["x1.tol"].c_str()),
  		                                     atof((*pi)["hx2.tol"].c_str()), atof((*pi)["x2.tol"].c_str()), atof((*pi)["iperc"].c_str()),fileLabel));

  		    ifstream peakFile((*pi)["name"].c_str());
  		    if (!peakFile) error("cannot open input file", (*pi)["name"]);
  		    CC4NOESYDeque.back().setPeakControlPtr(&(*pi));
  		    _outputLog << "\n-> " << (*pi)["name"].c_str() << " is read:" << endl;
  		    peakFile >> CC4NOESYDeque.back(); // how to read, need to worry
  		    peakFile.close();

  		    CC4NOESYDeque.back().setDianaBufferPtr(&_DianaBuffer);
   		    CC4NOESYDeque.back().setSWSF(tt, atof((*pi)["hx1.sw"].c_str()), atof((*pi)["x1.sw"].c_str()),
  		                                                                atof((*pi)["hx2.sw"].c_str()), atof((*pi)["x2.sw"].c_str()));
		    CC4NOESYDeque.back().setRmax(atof((*pi)["Rmax"].c_str()));
		    CC4NOESYDeque.back().setRmaxQ(atof((*pi)["RmaxQ"].c_str()));
  		    CC4NOESYDeque.back().Corr_Spectrum(atof((*pi)["hx1.shift"].c_str()), atof((*pi)["x1.shift"].c_str()),
  		                                         atof((*pi)["hx2.shift"].c_str()), atof((*pi)["x2.shift"].c_str()));

    		  NOEPeakList.push_back(&CC4NOESYDeque.back());

  		    _outputOvw << "  read as 4DCC13noesy " << endl;
        }
        // NN
        else if (((*pi)["x1.type"] == "N15") && ((*pi)["x2.type"] == "N15"))
        {
  		    NN4NOESYDeque.push_back(NN4NOESY_HSQC(atoi((*pi)["waterFlag"].c_str()), atoi((*pi)["haveIC"].c_str()), atof((*pi)["hx1.tol"].c_str()), atof((*pi)["x1.tol"].c_str()),
  		                                     atof((*pi)["hx2.tol"].c_str()), atof((*pi)["x2.tol"].c_str()), atof((*pi)["iperc"].c_str()),fileLabel));

  		    ifstream peakFile((*pi)["name"].c_str());
  		    if (!peakFile) error("cannot open input file", (*pi)["name"]);
  		    NN4NOESYDeque.back().setPeakControlPtr(&(*pi));
  		    _outputLog << "\n-> " << (*pi)["name"].c_str() << " is read:" << endl;
  		    peakFile >> NN4NOESYDeque.back(); // how to read, need to worry
  		    peakFile.close();

  		    NN4NOESYDeque.back().setDianaBufferPtr(&_DianaBuffer);
  		    NN4NOESYDeque.back().setSWSF(tt, atof((*pi)["hx1.sw"].c_str()), atof((*pi)["x1.sw"].c_str()),
  		                                                                atof((*pi)["hx2.sw"].c_str()), atof((*pi)["x2.sw"].c_str()));
		    NN4NOESYDeque.back().setRmax(atof((*pi)["Rmax"].c_str()));
		    NN4NOESYDeque.back().setRmaxQ(atof((*pi)["RmaxQ"].c_str()));
  		    NN4NOESYDeque.back().Corr_Spectrum(atof((*pi)["hx1.shift"].c_str()), atof((*pi)["x1.shift"].c_str()),
  		                                         atof((*pi)["hx2.shift"].c_str()), atof((*pi)["x2.shift"].c_str()));

    		  NOEPeakList.push_back(&NN4NOESYDeque.back());

  		    _outputOvw << "  read as 4DNN15noesy " << endl;
        }
        // NC
        else if (((*pi)["x1.type"] == "N15") && ((*pi)["x2.type"] == "C13"))
        {
  		    NC4NOESYDeque.push_back(NC4NOESY_HSQC(atoi((*pi)["waterFlag"].c_str()), atoi((*pi)["haveIC"].c_str()), atof((*pi)["hx1.tol"].c_str()), atof((*pi)["x1.tol"].c_str()),
  		                                     atof((*pi)["hx2.tol"].c_str()), atof((*pi)["x2.tol"].c_str()), atof((*pi)["iperc"].c_str()),fileLabel));

  		    ifstream peakFile((*pi)["name"].c_str());
  		    if (!peakFile) error("cannot open input file", (*pi)["name"]);
  		    NC4NOESYDeque.back().setPeakControlPtr(&(*pi));
  		    _outputLog << "\n-> " << (*pi)["name"].c_str() << " is read:" << endl;
  		    peakFile >> NC4NOESYDeque.back(); // how to read, need to worry
  		    peakFile.close();

  		    NC4NOESYDeque.back().setDianaBufferPtr(&_DianaBuffer);
  		    NC4NOESYDeque.back().setSWSF(tt, atof((*pi)["hx1.sw"].c_str()), atof((*pi)["x1.sw"].c_str()),
  		                                                                atof((*pi)["hx2.sw"].c_str()), atof((*pi)["x2.sw"].c_str()));
		    NC4NOESYDeque.back().setRmax(atof((*pi)["Rmax"].c_str()));
		    NC4NOESYDeque.back().setRmaxQ(atof((*pi)["RmaxQ"].c_str()));
  		    NC4NOESYDeque.back().Corr_Spectrum(atof((*pi)["hx1.shift"].c_str()), atof((*pi)["x1.shift"].c_str()),
  		                                         atof((*pi)["hx2.shift"].c_str()), atof((*pi)["x2.shift"].c_str()));

    		  NOEPeakList.push_back(&NC4NOESYDeque.back());

  		    _outputOvw << "  read as 4DN15C13noesy " << endl;
        }
        // CN
        else if (((*pi)["x1.type"] == "C13") && ((*pi)["x2.type"] == "N15"))
        {
  		    CN4NOESYDeque.push_back(CN4NOESY_HSQC(atoi((*pi)["waterFlag"].c_str()), atoi((*pi)["haveIC"].c_str()), atof((*pi)["hx1.tol"].c_str()), atof((*pi)["x1.tol"].c_str()),
  		                                     atof((*pi)["hx2.tol"].c_str()), atof((*pi)["x2.tol"].c_str()), atof((*pi)["iperc"].c_str()),fileLabel));

  		    ifstream peakFile((*pi)["name"].c_str());
  		    if (!peakFile) error("cannot open input file", (*pi)["name"]);
  		    CN4NOESYDeque.back().setPeakControlPtr(&(*pi));
  		    _outputLog << "\n-> " << (*pi)["name"].c_str() << " is read:" << endl;
  		    peakFile >> CN4NOESYDeque.back(); // how to read, need to worry
  		    peakFile.close();

  		    CN4NOESYDeque.back().setDianaBufferPtr(&_DianaBuffer);
  		    CN4NOESYDeque.back().setSWSF(tt, atof((*pi)["hx1.sw"].c_str()), atof((*pi)["x1.sw"].c_str()),
  		                                                                atof((*pi)["hx2.sw"].c_str()), atof((*pi)["x2.sw"].c_str()));
		    CN4NOESYDeque.back().setRmax(atof((*pi)["Rmax"].c_str()));
		    CN4NOESYDeque.back().setRmaxQ(atof((*pi)["RmaxQ"].c_str()));
  		    CN4NOESYDeque.back().Corr_Spectrum(atof((*pi)["hx1.shift"].c_str()), atof((*pi)["x1.shift"].c_str()),
  		                                         atof((*pi)["hx2.shift"].c_str()), atof((*pi)["x2.shift"].c_str()));

    		  NOEPeakList.push_back(&CN4NOESYDeque.back());

  		    _outputOvw << "  read as 4DC13N15noesy " << endl;
        }
		    else
		      error("Unknown x1.type and x2.type", (*pi)["name"]);
        break;
      }
      case 41: //4dnoesy, and IC
      {
        _outputOvw << "  4DICnoesy: " << (*pi)["NAME"] << " NOT SUPPORTED BY NOESY_Assign" << endl;
        break;
      }
      default:
        ;
    }
  }

	if(IC())
	{
		LLength = _LLength;
	}

  ///////////////////////////////////////////////////////////////////////
	//if Manual Structure avaiable, compare with the NOESY_Assign       ///
	//read a bundle structures or average                               ///
	///////////////////////////////////////////////////////////////////////
	if (_COMPARE)
	{
		ifstream pdb(manualFileName.c_str());
		if (!pdb) error ("cannot open input file", manualFileName);
		Pdb pdbStruct(0);

		while (pdb)
		  {
		    pdb >> pdbStruct;
		    if (pdbStruct.notEmpty())
		      {
			erasePrint(cout, " Reading ComparePdbFile: " + Qpdb);
			_pdbList.push_back(pdbStruct);
		      }
		    pdbStruct.reset();
		  }

		if (DEBUG)
		{
		  _outputLog << " ComparePdbFile is Read: " << endl;
		  for(deque<Pdb>::iterator i = _powerPdbList.begin(); i != _powerPdbList.end(); i++)
		    _outputLog << (*i) << endl;
		}
		pdb.close();
	}

	////////////////////////////////////////////////
	// copy all input files into source directory //
	////////////////////////////////////////////////
	if (!isS())
	{
	mkdir((outputDir+"/source").c_str(), 0750);
	string fileList;

	// control files
	fileList += controlFileName + " ";

	// general input files
	if (control["JListFile"] != "")
		fileList += control["JListFile"] + " ";
	if (control["NHSlowList"] != "")
		fileList += control["NHSlowList"] + " ";
	if (control["par"] != "")
		fileList += control["par"] + " ";
	if (control["chemicalShiftFile"] != "")
		fileList += control["chemicalShiftFile"] + " ";
	if (control["seqFile"] != "")
		fileList += control["seqFile"] + " ";
	if (control["RDC"] != "")
		fileList += control["RDC"] + " ";
	if (control["EC"] != "")
	  fileList += control["EC"] + " ";

	// peak lists
	for(PeakListDeque::iterator pi = peakListDeque.begin(); pi != peakListDeque.end(); pi++)
  {
    // check the attribution first
  	if ((*pi)["name"] != "")
			fileList += (*pi)["name"] + " ";
	}

	// intput structures
	if (INITF)
		fileList += initFold + " ";

	if (isQ())
	  fileList += Qpdb + " ";
	if (_COMPARE)
	  fileList += manualFileName + " ";

	// manual contraints
	if (control["UPL"] != "")
		fileList += control["UPL"] + " ";
	if (control["LOL"] !="")
		fileList += control["LOL"] + " ";
	if (control["HBOND"] != "")
		fileList += control["HBOND"] + " ";
 	if (control["ACO"] != "")
		fileList += control["ACO"] + " ";
 	if (control["XPL"] != "")
		fileList += control["XPL"] + " ";
 	if (control["DIHE"] != "")
		fileList += control["DIHE"] + " ";

		system(("cp -Pp "+ fileList + outputDir+"/source").c_str());
 		system(("gzip -r " + outputDir+"/source").c_str());
	}
}

void NOESY_Assign::runSimpleMatch()
{
	//////////////////////////////////////
	// SimpleMatch                      //
	//////////////////////////////////////

	ASDP::ratio mscore;
	for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
	{
	  erasePrint(cout, " Doing SimpleMatch for " + (*iter)->label() + " ... ");
	  (*iter)->SimpleMatch();

	  if ((isV()) && ((*iter)->Type().find("IC") >= (*iter)->Type().length()))
	  {
	    erasePrint(cout, " Valiade inputs for " + (*iter)->label() + "...");
	    ASDP::ratio temp;
	    temp = (*iter)->Validate_Shift(_outputOvw, _outputM);
	    mscore.first += temp.first;
	    mscore.second += temp.second;
	  }

	  (*iter)->findDEL_DEGLOC(_par.locref, _outputUA);
	}

	if (isV())
	_outputOvw << "Overall M-score: " << mscore.first/(float)mscore.second << " " << mscore.first << "/" << mscore.second << endl;

}


void NOESY_Assign::runSRExpect() // intra or seq
{
  ///////////////////////////////////////////////////////
	// Identify Expected Intra or Seq Peak               //
	///////////////////////////////////////////////////////
	for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
	  {
	    erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Find_IntraSeq for " + (*iter)->label() + " ... ");
      (*iter)->Find_Expect_IntraSeq();
	    erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Assign_Expect for " + (*iter)->label() + " ... ");
      (*iter)->Assign_Expect(0, _par.expect);
   }
}

bool NOESY_Assign::runRefine_Shift()
{
  if (DEBUG) NOESY_Assign::outputLog() << "in runRefine_Shift() " << endl;
	//reset DEL_LOC
	for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
	{
	  (*iter)->resetDEL_DEGLOC();
	}

	bool news = 0;
	for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
	  {
	    if ((*iter)->Type().substr(0,2) == "3D")
	      {
	    	erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Assign_Loc for " + (*iter)->label() + " ... ");
		(*iter)->Assign_MLoc(_par.locref);
	      }

	  }
	if (DEBUG) NOESY_Assign::outputLog() << "starting refinement " << endl;

	for(Atom_List::iterator i = NOESY_Assign::atoms().begin(); i != NOESY_Assign::atoms().end(); ++i)
	{

	  if (DEBUG) NOESY_Assign::outputLog() << "working on " << *i << endl;

	  SampleStatistic sampleA; sampleA.reset();
	  for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
	    {
	      // Hmom
	      bool isMom = 0;
	      if ((*iter)->get_MotherAtom().find(&*i) != (*iter)->get_MotherAtom().end())
  		{
		  isMom = 1;
		  MatchAtomPeak::const_iterator al = (*iter)->get_MotherAtom().lower_bound(&(*i));
		  MatchAtomPeak::const_iterator au = (*iter)->get_MotherAtom().upper_bound(&(*i));

		  if (DEBUG)
		    {
		      NOESY_Assign::outputLog() << "For HX1" << endl;
		    }

        for(MatchAtomPeak::const_iterator ai = al; ai != au; ai++)
     	{
     		  sampleA += (*iter)->get_HdiffAb(isMom, ai->first, ai->second);

     		  if (DEBUG)
     		  {
     		  	NOESY_Assign::outputLog() << (*ai).second << " " << (*iter)->get_HdiffAb(isMom, ai->first, ai->second) << endl;
     			}
     		}
	}

	if (DEBUG) NOESY_Assign::outputLog() << sampleA.min() << " " << sampleA.max() << sampleA.median() << endl;

// Hdad
  		if ((*iter)->get_FatherAtom().find(&*i) != (*iter)->get_FatherAtom().end())
  		{
				 isMom = 0;
         MatchAtomPeak::const_iterator al = (*iter)->get_FatherAtom().lower_bound(&(*i));
         MatchAtomPeak::const_iterator au = (*iter)->get_FatherAtom().upper_bound(&(*i));

         if (DEBUG)
         {
          	NOESY_Assign::outputLog() << "For HX2" << endl;
         }

        for(MatchAtomPeak::const_iterator ai = al; ai != au; ai++)
     	{
     		  sampleA += (*iter)->get_HdiffAb(isMom, ai->first, ai->second);

     		  if (DEBUG)
     		  {
     		  	NOESY_Assign::outputLog() << (*ai).second << " " << (*iter)->get_HdiffAb(isMom, ai->first, ai->second) << endl;
     			}
     		}
	}

	if (DEBUG) NOESY_Assign::outputLog() << sampleA.median() << endl;

			// Xmom
  		if ((*iter)->get_AttMomAtom().find(&*i) != (*iter)->get_AttMomAtom().end())
  		{
    		 isMom = 1;
         MatchAtomPeak::const_iterator al = (*iter)->get_AttMomAtom().lower_bound(&(*i));
         MatchAtomPeak::const_iterator au = (*iter)->get_AttMomAtom().upper_bound(&(*i));

         if (DEBUG)
         {
         		NOESY_Assign::outputLog() << "For X1" << endl;
         }

        for(MatchAtomPeak::const_iterator ai = al; ai != au; ai++)
     	{
     		  sampleA += (*iter)->get_XdiffAb(isMom, ai->first, ai->second);

     		  if (DEBUG)
     		  {
     		  	NOESY_Assign::outputLog() << (*ai).second << " " << (*iter)->get_XdiffAb(isMom, ai->first, ai->second) << endl;
     			}
     		}
      	}
			// Xdad
  		if ((*iter)->get_AttDadAtom().find(&*i) != (*iter)->get_AttDadAtom().end())
  		{
    		 isMom = 0;
         MatchAtomPeak::const_iterator al = (*iter)->get_AttDadAtom().lower_bound(&(*i));
         MatchAtomPeak::const_iterator au = (*iter)->get_AttDadAtom().upper_bound(&(*i));

         if (DEBUG)
         {
         		NOESY_Assign::outputLog() << "For X2" << endl;
         }

        for(MatchAtomPeak::const_iterator ai = al; ai != au; ai++)
     	{
     		  sampleA += (*iter)->get_XdiffAb(isMom, ai->first, ai->second);

     		  if (DEBUG)
     		  {
     		  	NOESY_Assign::outputLog() << (*ai).second << " " << (*iter)->get_XdiffAb(isMom, ai->first, ai->second) << endl;
     			}
     	}
      }
		}

    		if (DEBUG) NOESY_Assign::outputLog() << "Mean " << sampleA.mean() << endl;


		if (i->isH())
		{
		  if ((sampleA.count()) && (sampleA.stdDev() < _par.locref.htol + 0.000001))
		    {
		      if (i->get_difforig() < -100)
			sampleA += 0;
		      else
			sampleA += i->get_difforig();

		      if (DEBUG) NOESY_Assign::outputLog() << *i << endl << "old H-shift: " << i->get_shift()
      					<< " diff: " << i->get_shift()+sampleA.mean()
							   << " new: " << sampleA.mean();

        float diff = 0;

	if (sampleA.mean() > 0)
        {
        	diff = floor(sampleA.mean()*1000+0.00001)/1000.0;
		if (DEBUG) NOESY_Assign::outputLog() << "case 1 " << diff << endl;
        }
        if (sampleA.mean() < 0)
        {
        	diff = ceil(sampleA.mean()*1000-0.00001)/1000.0;
		if (DEBUG) NOESY_Assign::outputLog() << "case 2 " << diff << endl;
	}

	// minchange 0.005
        if (fabs(sampleA.mean()) < 0.005)
	{
		if ((fabs(sampleA.max()) > 0.041) || (fabs(sampleA.min()) > 0.041))
		{
			if ((sampleA.mean() > 0.001))
				diff = 0.005;
			else if ((sampleA.mean() < -0.001))
				diff = -0.005;
			else
				diff = 0;
		} else
		  diff = 0; // set to 0 for small differences

		if (DEBUG) NOESY_Assign::outputLog() << "case 2.2" << diff << endl;
	}

	if (diff)
	  {
	    i->set_HCCHshift(i->get_shift()+diff);
	    news = 1;
	  } else
	    i->set_HCCHshift(i->get_shift());
	if (DEBUG) NOESY_Assign::outputLog() << " new H-shift: " << i->get_shift() << endl;
		    }
		}
		else  // X-shift
		{
		  if ((sampleA.count()) && (sampleA.stdDev() < _par.locref.xtol + 0.000001))
		    {

		      if (i->get_difforig() < -100)
			sampleA += 0;
		      else
			sampleA += i->get_difforig();

		      if (DEBUG) NOESY_Assign::outputLog() << *i << endl << "old X-shift: " << i->get_shift()
							   << " diff: " << i->get_shift()+sampleA.mean()
							   << " new: " << sampleA.mean();

        float diff = 0;


        if (sampleA.mean() > 0) // close to 0
        {
        	diff = floor(sampleA.mean()*100+0.00001)/100.0;
        }
        if (sampleA.mean() < 0)
        {
        	diff = ceil(sampleA.mean()*100-0.00001)/100.0;
        }


	// minchange is 0.05
        if (fabs(sampleA.mean()) < 0.05)
	{
		if ((fabs(sampleA.max()) > 0.41) || (fabs(sampleA.min()) > 0.41))
		{
			if (sampleA.mean() > 0.01)
				diff = 0.05;
			else if (sampleA.mean() < -0.01)
				diff = -0.05;
			else
				diff = 0;
		} else
			diff = 0;
	}

	if (diff)
	  {
	    i->set_HCCHshift(i->get_shift()+diff);
	    news = 1;
	  } else
	    i->set_HCCHshift(i->get_shift());

	if (DEBUG) NOESY_Assign::outputLog() << "new X-shift: " << i->get_shift() << " origdiff: " << i->get_difforig() << endl;
     }
		}
  }

	for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
	{
	  (*iter)->findDEL_DEGLOC(_par.locref, _outputUA);
		erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Find_BbestMatch for " + (*iter)->label() + " ... ");
		(*iter)->Find_BbestMatch(); // need to be modified
  }
  return news;
}

void NOESY_Assign::runFindCisXP()
{
  ////////////////////////////////
	// Identify CISXP             //
	////////////////////////////////
	for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
	  {
	    erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Find_CisXP for " + (*iter)->label() + " ... ");
      (*iter)->FindCisXP();
   }
}

void NOESY_Assign::runRemoveIC()
{
	////////////////////////////////////////////////////////////////////////////////////
	// Remove the interchain peaks from the general peak list                         //
	// after doing the expect assigments, the rest peak if also in the IC, remove it. //
	////////////////////////////////////////////////////////////////////////////////////
	if (_IC)
  {
	  erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Removing IC from the original peak list ...");
    for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
      if (((*iter)->isHaveIC()) && ((*iter)->Type().find("IC") >= (*iter)->Type().length()))
    	  for(deque<NOESY*>::iterator iter2 = NOEPeakList.begin(); iter2 != NOEPeakList.end(); iter2++)
	      {
	        if (((*iter2)->Type().find("IC") < (*iter2)->Type().length()) && ((*iter2)->Type().find((*iter)->Type()) < (*iter2)->Type().length()))
           (*iter)->removeIC((*iter2)->get_peakList());
        }
   }
}

void NOESY_Assign::runBuildProfile(MomDadSet& mdSetM2, MomDadPeakAtomMap& mdPAMap2, SUPPORT& support)
{
	mdSetM2.clear(); // reset
	mdPAMap2.clear();
	support.clear();

        erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : BuildM2Map for ... ");
	BuildM2Map(mdSetM2, mdPAMap2); // groupd by repAtom mdSetM2 ( list of repAtoms), mdPAMap2 (link with peaks)

	erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : BuildSYMContact for ... ");
	BuildSYMContact(mdSetM2, mdPAMap2, support); // group by contact bin, add to support.ContactBin, MomDadBin

 	BuildmdPAMapM(_par.sym, mdPAMap2, support);

  if (DEBUG)
  {
 		for(MomDadSet::iterator i = support.mdSetM.begin(); i != support.mdSetM.end(); ++i)
 		{
 			NOESY_Assign::outputLog() << "support.mdPAMapM group " << sameHydro(i->first->get_sym_1(), i->second->get_sym_1()) << " " << i->first->get_index() << " " << i->second->get_index() << endl;
 			for(MomDadPeakAtomMap::iterator j = support.mdPAMapM.lower_bound(*i); j != support.mdPAMapM.upper_bound(*i); ++j)
 				NOESY_Assign::outputLog() << j->second.first << endl << j->second.second.first << endl << j->second.second.second << endl;
 		}
  }

  }


int NOESY_Assign::runSYMUnique(int weak, MomDadSet& mdSetM2, MomDadPeakAtomMap& mdPAMap2, SUPPORT& support)
{
	int count = 0;
	MomDadSet mdSet4; mdSet4.clear();
	MomDadSet mdSet0; mdSet0.clear();

	Find_4SYM(_par.sym, mdSet4, mdSet0, mdSetM2, mdPAMap2, support);

  	for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
	  if (((*iter)->Type().find("IC") >= (*iter)->Type().length()))
 	  {
	  	erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Assign_4SYM for " + (*iter)->label() + " ... ");
			count += (*iter)->Assign_4SYM(_par.sym, mdSet4, mdSet0, mdSetM2, mdPAMap2, support);
	  }

	if (weak > -1)
  	for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
	  if (((*iter)->Type().find("IC") >= (*iter)->Type().length()))
 	  {
 	 	  erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Assign_Unique for " + (*iter)->label() + " ... ");
		count += (*iter)->Assign_Unique(_par.unique, mdPAMap2, support);
	  }

	MomDadSet mdSetA12; mdSetA12.clear();
	MomDadPeakAtomMap mdPAMapA; mdPAMapA.clear();
	map<MomDad, MomDad, less<MomDad> > mdAM2; mdAM2.clear();

	Find_USYM(_par.sym, mdPAMap2, support, mdPAMapA, mdAM2, mdSetA12);
	  //
	for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
	  if (((*iter)->Type().find("IC") >= (*iter)->Type().length()))
 	  {
	  	erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Assign_SYM for " + (*iter)->label() + " ... ");
		count += (*iter)->Assign_SYM(weak, _par.sym, mdPAMapA, mdAM2,
					mdSetA12,mdSetM2, mdPAMap2, support);
	}

	  return count;
}

void NOESY_Assign::runFindSecondarySegment()
{
	///////////////////////////////////////
	//Determine secondary structure      //
	///////////////////////////////////////
	erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Find_secondarySegment ... ");
	Find_SSsequential(indexFilter, indexCaRaw, indexCbRaw, JList, NHList, _secondaryStruct, _par.sec);
}

void NOESY_Assign::runFindSecondaryRegister()
{
	///////////////////////////////////////
	//Determine secondary structure      //
	///////////////////////////////////////
	erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Find_secondary ... ");
	Find_SSregister(indexFilter, indexCaRaw, indexCbRaw, JList, NHList, ECList, _secondaryStruct,
									_antiParaRegister, _paraRegister, _par.sec);
	_outputOvw << "\nSecondary Structure" << endl;
	_outputOvw <<   "===================" << endl << endl;
	_outputOvw << _secondaryStruct << endl;
	if (_antiParaRegister.size())
	{
	  _outputOvw << "Antiparallel Strands Registers: " << endl;
	  _outputOvw << "=============================== " << endl << endl;
	  _outputOvw << _antiParaRegister << endl;
	}
	if (_paraRegister.size())
	{
	  _outputOvw << "Parallel Strands Registers: " << endl;
	  _outputOvw << "=========================== " << endl << endl;
	  _outputOvw << _paraRegister << endl;
	}
}

void NOESY_Assign::runFindHHPacking()
{
	///////////////////////////////////////
	//Determine Helix-Helix packing      //
	///////////////////////////////////////
	erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Find_HHPacking ... ");
}

void NOESY_Assign::runHelixExpect() // mainly mid and long range expect
{
	//////////////////////////////////
	// Make Expect assignments      //
	//////////////////////////////////
  for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
	  {
	    erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Find_Helix for " + (*iter)->label() + " ... ");
	    (*iter)->Find_Helix();
	    erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Assign_Helix for " + (*iter)->label() + " ... ");
	    (*iter)->Assign_Expect(1, _par.expect);
			(*iter)->del_DELLSS(_outputUA);
	  }
}

void NOESY_Assign::runSheetExpect() // mainly mid and long range expect
{
	//////////////////////////////////
	// Make Expect assignments      //
	//////////////////////////////////
  for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
	  {
	    erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Find_Sheet for " + (*iter)->label() + " ... ");
	    (*iter)->Find_Sheet();
	    erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Assign_Sheet for " + (*iter)->label() + " ... ");
	    (*iter)->Assign_Expect(1, _par.expect);
			(*iter)->del_DELLSS(_outputUA);
	  }
}

int NOESY_Assign::runContactFaces()
{
	int count = 0;
	// build contactFaces
	contactFaces.resetCT();

  for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
	{
	   erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : createSideChainContactBin for " + (*iter)->label() + " ... ");
		 (*iter)->insertSideChainContactFaces();
	}

	contactFaces.insertCT();
	contactFaces.combineSSCT();

	if (DEBUG)
	{
		for(int i = 0; i < 1000; ++i)
		for(int j = 0; j < 1000; ++j)
		{
			if (contactFaces.get_ctFace(i,j))
			{
				NOESY_Assign::outputLog() << "contactFace " << i << " " << j << " " << contactFaces.get_ctFace(i,j) << endl;
			}
		}
	}

	// reset DEL_TOP
  for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
	  {
	    erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : DELTOP for " + (*iter)->label() + " ... ");
			count += (*iter)->del_DELTOP(_outputUA);
	  }
	return count;
}

void NOESY_Assign::runContact()
{
	//////////////////////////////////
	//                              //
	// createIntra chain ContactBin //
	//////////////////////////////////
	_ContactSet.clear();
  for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
	  if (((*iter)->Type().find("IC") >= (*iter)->Type().length()) && ((*iter)->Type().substr(0,2) != "4D"))
	  {
	    erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : createContactBin for " + (*iter)->label() + " ... ");
	    (*iter)->createContactBin();
	  }

  if (DEBUG)
  {
    _outputLog << "In ContactSet: " << endl;
    for(resSet::iterator iter = _ContactSet.begin(); iter != _ContactSet.end();iter++)
    {
      _outputLog << (*iter).first << " " << (*iter).second << endl;
    }
  }

  // check for 4D and 2D NOESY, not sure
  for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
	  if (((*iter)->Type().find("IC") >= (*iter)->Type().length()) && ((*iter)->Type().substr(0,2) != "4D"))
 	  {
	    (*iter)->Assign_CF(_par.cf);
	  }

  ///////////////////////////////////
	// Create Inter chain ContactBin //
  ///////////////////////////////////

  _ContactSet.clear();
  for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
	  if ((*iter)->Type().find("IC") < (*iter)->Type().length())
    {
      (*iter)->createContactBin();
    }

	if (DEBUG)
	{
	  _outputLog << "In Inter Chain ContactSet: " << endl;

	  for(resSet::iterator iter = _ContactSet.begin(); iter != _ContactSet.end();iter++)
	  {
		  _outputLog << (*iter).first << " " << (*iter).second << endl;
	  }
	}

	// not sure for 2D and 4D noesy
	for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
	  if ((*iter)->Type().find("IC") < (*iter)->Type().length())
    {
	    erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : createContactBin for " + (*iter)->label() + " ... ");
      (*iter)->Assign_CF(_par.cf);
    }

	_ContactSet.clear();
}


void NOESY_Assign::runFindHbond() // init, not structure
{
	string temp = "";
	findHbond(0,temp, 1, _secondaryStruct, _antiParaRegister, _paraRegister, NHList, acfMax, HBONDList);
}

void NOESY_Assign::runMulti()
{
		if ((!usvv) && (command.cns !="") && (control["multiCycle"] != "") && (_ddcycle >= atoi(control["multiCycle"].c_str())))
		{
			if (control["multiCutoff"] == "")
				control["multiCutoff"] = "2";

      for(deque<NOESY*>::iterator iter = NOEPeakList.begin(); iter != NOEPeakList.end(); iter++)
    	{
    	  erasePrint(cout, " Cycle" + itoa(_ddcycle)+'-'+itoa(_dcycle) + " : Assign_Multi for " + (*iter)->label() + " ... ");
    	  (*iter)->Assign_Multi(_par.ambi, atoi(control["multiCutoff"].c_str()));
    	}
		}
}
