/***************************************************************************
                          noesy_assign.h  -  description
                             -------------------
    begin                : Fri Dec 14 2001
    copyright            : (C) 2001 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/


#ifndef NOESY_ASSIGN_H
#define NOESY_ASSIGN_H

/**
  *@author Yuanpeng Janet Huang
  */
#include <sys/time.h>
#include <sys/times.h>
#include <sstream>

#include "Jval.h"

#include "par.h"
#include "utility.h"
#include "Diana.h"
#include "D2NOESY.h"
#include "HSQCN15NOESY.h"
#include "HSQCC13NOESY.h"
#include "CC4NOESY.h"
#include "nc4noesy_hsqc.h"
#include "nn4noesy_hsqc.h"
#include "EC.h"

using namespace std;

const int _LLength = 100;

typedef pair<int, int> respair;
typedef multiset<respair, less<respair> > resSet;
typedef map<pair<int, int>, float, less<pair<int, int> > > matrix;

typedef map<NOESY*, MomDadSet, less<NOESY*> > MDSETMAP;
typedef	map<NOESY*, MomDadPeakAtomMap, less<NOESY*> > MDPAMAP;

class NOESY_Assign {
private:
	static int _dcycle;
	static int _ddcycle;

	bool SIM;
	static bool _MINCON;
	static bool _COMPARE;
	bool RESTORE;

	static bool _haveNOE;
	static bool _haveJ;
	static bool _haveRotamer;

	static bool _V;
	static bool _EC;
	static bool _AMBI;

  	static bool _Q;
	static bool _S;

	static bool _IC;
	static	float _KCalibration; // old is 0.003804
	static float _KD;

	static bool _monster;
	static bool _csi;

	static const char * const _EOS;// = "End of Stream";

	int INITF;
	string initFold;
  	string manualFileName;
  	string sourcePdbPath;

  	string Qpdb;
	int usvv;

	static	Atom_List _atoms;
	static	deque<Pdb> _pdbList;
	static	deque<Pdb> _powerPdbList;

	static resSet _ContactSet; // only for CF function
	static ContactPairSet _asContactSet; // for all assignment
	static deque<HBOND> _antiParaRegister;  // antiparalley_register
	static deque<HBOND> _paraRegister; // para_register
  	set<HBOND, less<HBOND> > HBONDList;

	NHSlowList NHList;
	map<HBOND, ECscore, less<HBOND> > ECList;

  	matrix acfMax; // assigned residue matrix

	static	SS _secondaryStruct;
	static Par _par;

	deque<NOESY*> NOEPeakList;

	deque<D2NOESY> D2NOESYDeque;
	deque<C13NOESY_HSQC> CD3NOESYDeque;
	deque<ALIC13NOESY_HSQC> ALICD3NOESYDeque;
	deque<AROC13NOESY_HSQC> AROCD3NOESYDeque;
	deque<N15NOESY_HSQC> ND3NOESYDeque;
	deque<NC3NOESY_HSQC> NC3NOESYDeque;

	deque<CC4NOESY_HSQC> CC4NOESYDeque;
	deque<NN4NOESY_HSQC> NN4NOESYDeque;
	deque<NC4NOESY_HSQC> NC4NOESYDeque;
	deque<CN4NOESY_HSQC> CN4NOESYDeque;

	deque<ICD2NOESY> ICD2NOESYDeque;
	deque<ICC13NOESY_HSQC> ICC13NOESYDeque;
	deque<ICN15NOESY_HSQC> ICN15NOESYDeque;

	static	DianaMap _DianaBuffer;
	static	DianaMap _ICDianaBuffer;

	static DianaDeque _DianaBufferDeque;
	static DianaDeque _ICDianaBufferDeque;

  	string current;

	//--- report
	ostringstream reportConstraintStat;
	static ofstream _outputExm;
	static ofstream _outputLog;
	static ofstream _outputOvw;
	static ofstream _outputOvwRPF;
	static ofstream _outputROvw;
	static ofstream _outputSec;
	static ofstream _outputError;
	static ofstream _outputUA;

	static ofstream _outputM;
	static ofstream _outputI;
	static ofstream _outputL;

public:

	NOESY_Assign();
	~NOESY_Assign();

	// configuration

       	void setKCalibration(float n) {_KCalibration = n;};
	void setKD(float n) {_KD = n;};
	void setSIM(){SIM = 1;};
	void setMINCON(){_MINCON = 1;};
	void setCOMPARE(){_COMPARE = 1;};
	void setManualFileName(const char* t){manualFileName=t;};
	void setRESTORE(){RESTORE=1;};
	void setSourcePdbPath(const char*);

	void setINITF(){INITF=1;};
	void setNOCSI() {_csi = 0;};
	void setInitFold(const char* t){initFold = t;};
	void setHaveRotamer(){_haveRotamer = 1;};
	void setIsQ(){_Q = 1;};
	void setIsS(){_S = 1;};
	void setIsEC(){_EC = 1;};
	void setIsAMBI(){_AMBI = 1;};
	void setQpdb(const char* t){Qpdb = t;};
	void setIsV(){_V = 1;};
	void setHaveNOE(){_haveNOE = 1;};
	void setHaveJ(){_haveJ = 1;};
	void setHaveMonster(){_monster = 1;};

  	void initReportOvw(string, const time_t&, int, char**);
  	void readInputFile(string);
	void runSimpleMatch();
  	void runCSIMethod();
	void runRfactor();
	void runSRExpect();
	void runSheetExpect();
	void runHelixExpect();
	int runContactFaces();
	void runRemoveIC();
	bool runRefine_Shift();

	float runRPF(Pdb&, int) const;

	void BuildSYMContact(const MomDadSet&, const MomDadPeakAtomMap&, SUPPORT&);
	void BuildmdPAMapM(const Sym&, const MomDadPeakAtomMap&, SUPPORT&);
	void BuildM2Map(MomDadSet&, MomDadPeakAtomMap&);
	AtomShift BuildXatomShift(const MomDad&, const MomDadPeakAtomMap&);
	void filter(const MomDadSet&, MomDadPeakAtomMap&);


	void runBuildProfile(MomDadSet&, MomDadPeakAtomMap&, SUPPORT&);
	void Find_4SYM(const Sym&, MomDadSet&, MomDadSet&, const MomDadSet&, const MomDadPeakAtomMap&, SUPPORT&);
	void Find_USYM(const Sym&, const MomDadPeakAtomMap&, SUPPORT&, MomDadPeakAtomMap&, map<MomDad, MomDad, less<MomDad> >&, MomDadSet&);

	int runSYMUnique(int, MomDadSet&, MomDadPeakAtomMap&, SUPPORT&);

	int  runRingContact();
	int  runAssignPDB(MomDadSet&, MomDadPeakAtomMap&, SUPPORT&);
	int  validateEC(const string, bool, map<HBOND, ECscore, less<HBOND> >&, Atom_List&, float, float);
	int  adjustEC(const string, bool, map<HBOND, ECscore, less<HBOND> >&, Atom_List&, float, float);
	int  findEC(map<HBOND, ECscore, less<HBOND> >&, const SEQM&, Atom_List&);
	int  findECfromSS(SS&, deque<HBOND>&, deque<HBOND>&, map<HBOND, ECscore, less<HBOND> >&); 


  	void runInitFold();
	void runVioPdb();
	void runDyana(string);
	void runCyana(string, stringstream&);
	void runCNS(string);
	void runXplor(string);

	void prtCyanaCons(string, stringstream&);
	void prtXplorCons(string);

	void runRefine(string);
	void runHyper(string);
	void readPdb(string);
	void cyana2xplor(string);


	void runFindSecondaryRegister();
	void runFindSecondarySegment();

	void runFindHHPacking();
	void runContact();
	void runFindHbond();
	void runFindCisXP();
	void runMulti();

	void stereoAssign(const vector<string>&);

	static  bool isV(){return _V;};
	static bool isQ(){return _Q;};
	static bool isS(){return _S;};
	static bool isEC() {return _EC;};
	static bool isAMBI() {return _AMBI;};
	bool isINITF(){return INITF;};
	bool isCSI() {return _csi;};

	void prtNoisePeak();
  	void prtAssign(ostream&);
	void prt_cfpdb(deque<Pdb>&, ostream&);

	void addSS2AA(const SS& ss);
	void Find_SSsequential(const CSIMap&, const CSIMap&, const CSIMap&, const JvalList&, const NHSlowList&, SS&, const Sec&);
	void Find_SSregister(const CSIMap&, const CSIMap&, const CSIMap&, const JvalList&, const NHSlowList&, map<HBOND, ECscore, less<HBOND> >& ECList, SS&, deque<HBOND>&, deque<HBOND>&, const Sec&);
	void Find_HHPacking(const SS&, ostream&);
	void findHbond(int, const string, bool, SS&, deque<HBOND>&, deque<HBOND>&, const NHSlowList&, matrix&, set<HBOND, less<HBOND> >&);

	void simulateHHFromPdb(const deque<Pdb>&, disMomDadSetStat&, bool) const;
	void calcDistance(const deque<Pdb>&, disAtomPairStat&, disAtomPairStat&) const;

  	static int dcycle(){return _dcycle;};
	static int ddcycle(){return _ddcycle;};
	static ofstream& outputExm(){return _outputExm;};
	static ofstream& outputLog(){return _outputLog;};
	static ofstream& outputSec(){return _outputSec;};
	static ofstream& outputOvw(){return _outputOvw;};
	static ofstream& outputOvwRPF(){return _outputOvwRPF;};
	static ofstream& outputROvw(){return _outputROvw;};
	static ofstream& outputError(){return _outputError;};

	static float KCalibration(){return _KCalibration;}; // old is 0.003804
	static float KD() {return _KD;};

	static const string EOS(){return _EOS;};

	static	Atom_List& atoms(){return _atoms;};
	static	deque<Pdb>& pdbList(){return _pdbList;};
	static	deque<Pdb>& powerPdbList(){return _powerPdbList;};

	static	DianaMap& DianaBuffer(){return _DianaBuffer;};
	static	DianaMap& ICDianaBuffer(){return _ICDianaBuffer;};
	static	DianaDeque& DianaBufferDeque(){return _DianaBufferDeque;};
	static	DianaDeque& ICDianaBufferDeque(){return _ICDianaBufferDeque;};

	static const bool IC(){return _IC;};

	static bool haveNOE(){return _haveNOE;};
	static bool haveJ(){return _haveJ;};
	static bool haveRotamer(){return _haveRotamer;};
	static bool COMPARE(){return _COMPARE;};
	static bool MINCON(){return _MINCON;};

	static deque<HBOND>& antiParaRegister(){return _antiParaRegister;};  // antiparalley_register
	static deque<HBOND>& paraRegister(){return _paraRegister;}; // para_register

	static resSet& ContactSet(){return _ContactSet;};
	static ContactPairSet& asContactSet(){return _asContactSet;};
	static SS& secondaryStruct(){return _secondaryStruct;};
	static Par& par(){return _par;};
	float Rmax();
	float RmaxQ();
};

#endif
