/***************************************************************************
                          pdbreader.cpp  -  description
                             -------------------
    begin                : Mon Aug 21 2000
    copyright            : (C) 2000 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#include <cmath>
#include <fstream>
#include <stdlib.h>

#include "SmplStat.h"
#include "pdbreader.h"
#include "AA.h"
#include "noesy_assign.h"

extern bool DEBUG;
extern int LLength;

void error(char*, char*);
void error(string, string);

float summation(const SampleStatistic&);
float sumAverage(const SampleStatistic&);

//const double PI= 3.14159265358979323846;    /* Pi definition */
//const double radperdeg = 0.01745329251994;   /* Radians per degree */
//---------------------------------------------------------//
//--- Diana format
// #ATOM     71  HG23THR     5       7.168   3.868  -6.821
// #ATOM      1  N   SER     2       1.325   0.000   0.000
// #012345678901234567890123456789012345678901234567890123
// #0        1         2         3         4         5
// #123412123451123451231112341123123456781234567812345678
// #1   1 1    11    1  111   11  1       1       1
// #rec * ser  *name res*chain****
//extern ResS_phi order_phi;
//extern ResS_psi order_psi;

//////////////////////////////////////////////////
//    PDBAtom methods                           //
//////////////////////////////////////////////////

// from rosetta - it uses xplor format
pdbAtom::pdbAtom(int serialP, string nameP, string resNameP, char chainIDP, int resSeqP, float xP, float yP, float zP)
{
  serial = serialP;
  name = nameP;
  resName = resNameP;
  chainID = chainIDP;
  resSeq = resSeqP;
  x = xP;
  y = yP;
  z = zP;

  // name maybe need to convert to standard
  name = name.substr(name.find_first_not_of(" \t\n\v"),
		     name.find_last_not_of(" \t\n\v") - name.find_first_not_of(" \t\n\v")+1);
  if ((name[0] != 'H') && (name[1] == 'H'))
    name = name.substr(1, name.length()-1) + name[0];

  if (name == "H")
    name = "HN";


  if ((resName == "GLY") && (name == "HA1"))
    {
      name = "HA2";
    }
  else if ((resName == "GLY") && (name == "HA2"))
    {
      name = "HA3";
    }

  else if ((resName != "ALA") && (name == "HB1"))
    {
      name = "HB2";
    }
  else if ((resName != "ALA") && (name == "HB2"))
    {
      name = "HB3";

    }
  else if ((resName != "THR") && (name == "HG1"))
    {
      name = "HG2";
    }
  else if ((resName != "THR") && (name == "HG2"))
    {
      name = "HG3";
    }

  else if ((resName == "ILE") && (name == "HG11"))
    {
      name = "HG12";
    }
  else if ((resName == "ILE") && (name == "HG12"))
    {
      name = "HG13";
    }

  else if (((resName != "PHE") && (resName != "TYR") && (resName != "TRP") && (resName != "HIS")) && (name == "HD1"))
    {
      name = "HD2";
    }
  else if (((resName != "PHE") && (resName != "TYR") && (resName != "TRP") && (resName != "HIS")) && (name == "HD2"))
    {
      name = "HD3";
    }

  else if (((resName != "ASN")) && (name == "HD22"))
    {
      name = "HD21";
    }
  else if (((resName != "ASN")) && (name == "HD21"))
    {
      name = "HD22";
    }
  else if (((resName != "PHE") && (resName != "TYR") && (resName != "TRP") && (resName != "HIS") && (resName != "MET")) && (name == "HE1"))
    {
      name = "HE2";
    }
  else if (((resName != "PHE") && (resName != "TYR") && (resName != "TRP") && (resName != "HIS") && (resName != "MET")) && (name == "HE2"))
    {
      name = "HE3";
    }
  else if (((resName != "GLN")) && (name == "HE21"))
    {
      name = "HE22";
    }
  else if (((resName != "GLN")) && (name == "HE22"))
    {
      name = "HE21";
    }
}


pdbAtom::pdbAtom(string line, int type, int length)
{
	if (type == 0) //pdb
	{
		name    = line.substr(12, 5);
		resName = line.substr(17, 3);

		string resSeqS  = line.substr(22, 4);
		string chainIDS = line.substr(21, 1);
		string serialS  = line.substr(6, 5);
		string xS 	= line.substr(30, 8);
		string yS  	= line.substr(38, 8);
		string zS	= line.substr(46, 8);

		// name maybe need to convert to standard
		name = name.substr(name.find_first_not_of(" \t\n\v"),
				   name.find_last_not_of(" \t\n\v") - name.find_first_not_of(" \t\n\v")+1);
		if ((name[0] != 'H') && (name[1] == 'H'))
		  name = name.substr(1, name.length()-1) + name[0];

		if (name == "H")
			name = "HN";

		x = atof(xS.c_str()); y = atof(yS.c_str()); z = atof(zS.c_str());
		serial = atoi(serialS.c_str()); chainID = chainIDS[0];
		resSeq = atoi(resSeqS.c_str());

		if (chainID == ' ')
		  chainID = '1';
	}
	else if (type == 1) //dyana
	{
		name    = line.substr(12, 5);
		resName = line.substr(17, 3);

		string resSeqS  = line.substr(22, 4);
		string chainIDS = line.substr(21, 1);
		string serialS  = line.substr(6, 5);
		string xS 	= line.substr(30, 8);
		string yS  	= line.substr(38, 8);
		string zS	= line.substr(46, 8);

		// name maybe need to convert to standard
		name = name.substr(name.find_first_not_of(" \t\n\v"),
				   name.find_last_not_of(" \t\n\v") - name.find_first_not_of(" \t\n\v")+1);
		x = atof(xS.c_str()); y = atof(yS.c_str()); z = atof(zS.c_str());
		serial = atoi(serialS.c_str()); chainID = chainIDS[0];
		resSeq = atoi(resSeqS.c_str());
	}
}

float calcDist(const pdbAtom& atom1, const pdbAtom& atom2)
{
	return floor(sqrt(pow((double)(atom1.x-atom2.x),(double)2)+
		pow((double)(atom1.y - atom2.y),(double)2)+
		pow((double)(atom1.z-atom2.z),(double)2)) * 100 + 0.05)/100.0;
}

const string& pdbAtom::get_name() const
{
	return name;
}

int pdbAtom::get_resSeq() const
{
	return resSeq;
}
char pdbAtom::get_chainID() const
{
	return chainID;
}


//////////////////////////////////////////////////
//    PDB methods                               //
//////////////////////////////////////////////////

Pdb::Pdb()
{
	chainIDSet.clear(); nameAtomList.clear(); disPdbAtomPairStatList.clear(); data_points.clear(); pdbmap.clear();
}

Pdb::Pdb(int ttype)
{
	type = ttype;chainIDSet.clear(); nameAtomList.clear(); disPdbAtomPairStatList.clear(); data_points.clear(); pdbmap.clear();
}

void Pdb::reset()
{
   nameAtomList.clear();
   chainIDSet.clear();
   disPdbAtomPairStatList.clear();
   data_points.clear();
   pdbmap.clear();
   resAtom.clear();
}

//not used in the program
void Pdb::insertPdbAtom(const pdbAtom& tempAtom)
{
  nameAtomList.insert(make_pair(make_pair(make_pair(tempAtom.get_resSeq(), tempAtom.get_name()),tempAtom.get_chainID()), tempAtom));
  chainIDSet.insert(tempAtom.get_chainID());
  //resAtomList.insert(make_pair(make_pair(tempAtom.get_resSeq(), tempAtom.get_chainID()), tempAtom));
  resAtom.insert(make_pair(make_pair(tempAtom.get_resSeq(), tempAtom.get_chainID()), tempAtom));
  cout << "insert" << tempAtom.get_resSeq() << " " << tempAtom.get_chainID() << " " << tempAtom << endl;
}


void Pdb::fromNametoList(int seq, const string& name, char chainID, nameAtom& temp) const
{
	pair<nameAtom::const_iterator, nameAtom::const_iterator>
		atomPair = nameAtomList.equal_range(make_pair(make_pair(seq, name), chainID));

//  if ((seq == 94) && (name == "HG"))
//  {
//      cout << "fromNametoList 94 HG" << endl;
//      cout << nameAtomList.size() << endl;
//      if (atomPair.first == atomPair.second)
//        cout << "empty" << endl;
//      //cout << std::distance(atomPair.first, nameAtomList.begin()) << " " << std::distance(atomPair.second, nameAtomList.begin()) << endl;
//  }

	for(nameAtom::const_iterator iter = atomPair.first; iter != atomPair.second; iter++)
		temp.insert((*iter));
	return;
}

void Pdb::get_Atom(int seq, char chainID, list<pdbAtom>& temp) const
{
  //cout << "in get_Atom" << " " << seq <<  " " << chainID << endl;
  pair<resAtomList::const_iterator, resAtomList::const_iterator>
    resPair = resAtom.equal_range(make_pair(seq, chainID));
  for(resAtomList::const_iterator iter= resPair.first; iter != resPair.second; iter++)
    {
      //cout << iter->first.first << " " << iter->first.second << iter->second << endl;
      temp.push_back((*iter).second);
    }
  return;
}

void Pdb::get_Atom(int seq, char res, const string& name, int ambi, char chainID,
	nameAtom& tempNameAtomList) const
{

	//if (DEBUG)
	//{
	//	cout << "in get_Atom " << seq << " " << res << " " << name << " " << ambi << " " << chainID << endl;
	//}
	if (name == "H")
	{
		fromNametoList(seq, "HN", chainID, tempNameAtomList);
		return;
	}

	// alway unique, not matter how ambi=?
		if ((res == 'A') && ((name == "HB") ||(name == "QB")))
		{
			fromNametoList(seq, "HB1", chainID, tempNameAtomList);
			fromNametoList(seq, "HB2", chainID, tempNameAtomList);
			fromNametoList(seq, "HB3", chainID, tempNameAtomList);
			return;
		} else
		if (((res == 'I') || (res == 'T'))
			&& ((name == "HG2") || (name == "QG2")))
		{
			fromNametoList(seq, "HG21", chainID, tempNameAtomList);
			fromNametoList(seq, "HG22", chainID, tempNameAtomList);
			fromNametoList(seq, "HG23", chainID, tempNameAtomList);
			return;
		} else
		if (((res == 'I'))
			&& ((name == "HD1") || (name == "QD1")))
		{
			fromNametoList(seq, "HD11", chainID, tempNameAtomList);
			fromNametoList(seq, "HD12", chainID, tempNameAtomList);
			fromNametoList(seq, "HD13", chainID, tempNameAtomList);
			return;
		} else
		if ((res == 'K') && ((name == "HZ") || (name == "QZ")))
		{
			fromNametoList(seq, "HZ1", chainID,  tempNameAtomList);
			fromNametoList(seq, "HZ2", chainID, tempNameAtomList);
			fromNametoList(seq, "HZ3", chainID, tempNameAtomList);
			return;
		} else
		if (((res == 'M') )
			&& ((name == "HE") || (name == "QE")))
		{
			fromNametoList(seq, "HE1", chainID, tempNameAtomList);
			fromNametoList(seq, "HE2", chainID, tempNameAtomList);
			fromNametoList(seq, "HE3", chainID, tempNameAtomList);
			return;
		}
		if (((res == 'R') )
			&& ((name == "QH1")))
		{
			fromNametoList(seq, "HH11", chainID, tempNameAtomList);
			fromNametoList(seq, "HH12", chainID, tempNameAtomList);
			return;
		}
		if (((res == 'R') )
			&& ((name == "QH2")))
		{
			fromNametoList(seq, "HH21", chainID, tempNameAtomList);
			fromNametoList(seq, "HH22", chainID, tempNameAtomList);
			return;
		}
		if (((res == 'R') )
			&& ((name == "QH")))
		{
			fromNametoList(seq, "HH21", chainID, tempNameAtomList);
			fromNametoList(seq, "HH22", chainID, tempNameAtomList);
			fromNametoList(seq, "HH11", chainID, tempNameAtomList);
			fromNametoList(seq, "HH12", chainID, tempNameAtomList);
			return;
		}

	// gly - if ambi=1, ha3->ha2, ha2-> ha1, else ha3-> ha1
	// bmrb - 2/3 diana - ha1/2, xplor 2/1, so also degenerarte
	if (res == 'G')
	{
  	if ((ambi <= 1) && (res == 'G') && (name == "HA2")) // for atoms that are sure there and uniquely mapped (used for constraint vilolation check)
  						     // or the stop case for get_dist
  	{
  		fromNametoList(seq, "HA1", chainID, tempNameAtomList);

  		if (!tempNameAtomList.size())
  			fromNametoList(seq, "HA2", chainID, tempNameAtomList);

  		return;
  	} else
  	if ((ambi <= 1) && (res == 'G') && (name == "HA3")) // for atoms that are sure there and uniquely mapped (used for constraint vilolation check)
  						     // or the stop case for get_dist
  	{
  		nameAtom tt; tt.clear();
  		fromNametoList(seq, "HA1", chainID, tt);
  		if (tt.size()) //1-2 dyana format
  			fromNametoList(seq, "HA2", chainID, tempNameAtomList);
  		else //2-3
  			fromNametoList(seq, "HA3", chainID, tempNameAtomList);
  		return;
  	}
  	else
  	if (((res == 'G'))
  		&& ((name == "QA") || (name == "HA2") || (name == "HA3")))
  	{
  		fromNametoList(seq, "HA3", chainID, tempNameAtomList);
  		fromNametoList(seq, "HA2", chainID, tempNameAtomList);
  		fromNametoList(seq, "HA1", chainID, tempNameAtomList);
  		return;
  	}
	}

	if (ambi <= 1) // stero-specific assignments
	{
		if (((res == 'V'))
			&& ((name == "HG1") || (name == "QG1")))
		{
			fromNametoList(seq, "HG11", chainID, tempNameAtomList);
			fromNametoList(seq, "HG12", chainID, tempNameAtomList);
			fromNametoList(seq, "HG13", chainID, tempNameAtomList);
			return;
		} else
		if (((res == 'V'))
			&& ((name == "HG2") || (name == "QG2")))
		{
			fromNametoList(seq, "HG21", chainID, tempNameAtomList);
			fromNametoList(seq, "HG22", chainID, tempNameAtomList);
			fromNametoList(seq, "HG23", chainID, tempNameAtomList);
			return;
		} else
		if (((res == 'L'))
			&& ((name == "HD1")|| (name == "QD1")))
		{
			fromNametoList(seq, "HD11", chainID, tempNameAtomList);
			fromNametoList(seq, "HD12", chainID, tempNameAtomList);
			fromNametoList(seq, "HD13", chainID, tempNameAtomList);
			return;
		} else
		if (((res == 'L'))
			&& ((name == "HD2") || (name == "QD2")))
		{
			fromNametoList(seq, "HD21", chainID, tempNameAtomList);
			fromNametoList(seq, "HD22", chainID, tempNameAtomList);
			fromNametoList(seq, "HD23", chainID, tempNameAtomList);
			return;
		} else
		if (((res == 'C') || (res == 'D') || (res == 'E') || (res == 'F') || (res == 'H')
			|| (res == 'K') || (res == 'L') || (res == 'M')|| (res == 'N')|| (res == 'P')
			|| (res == 'Q') || (res == 'R')|| (res == 'S')|| (res == 'W')|| (res == 'Y'))
			&& ((name == "QB") )) //#some restraints with QB and ambi=1
		{
		  fromNametoList(seq, "HB1", chainID, tempNameAtomList);
		  fromNametoList(seq, "HB2", chainID, tempNameAtomList);
		  fromNametoList(seq, "HB3", chainID, tempNameAtomList);
		  return;
		} else
		if (((res == 'E') || (res == 'K') || (res == 'M')|| (res == 'P') || (res == 'Q')|| (res == 'R'))
			&& ((name == "QG") ))
		{
		  fromNametoList(seq, "HG1", chainID, tempNameAtomList);
		  fromNametoList(seq, "HG2", chainID, tempNameAtomList);
		  fromNametoList(seq, "HG3", chainID, tempNameAtomList);
		  return;
		} else
		if (((res == 'K') || (res == 'P') || (res == 'R'))
			&& ((name == "QD")))
		{
		  fromNametoList(seq, "HD1", chainID, tempNameAtomList);
		  fromNametoList(seq, "HD2", chainID, tempNameAtomList);
		  fromNametoList(seq, "HD3", chainID, tempNameAtomList);
		  return;
		} else
		if (((res == 'K'))
			&& ((name == "QE")))
		{
		  fromNametoList(seq, "HE1", chainID,  tempNameAtomList);
		  fromNametoList(seq, "HE2", chainID, tempNameAtomList);
		  fromNametoList(seq, "HE3", chainID, tempNameAtomList);
		  return;
		} else
		if (((res == 'Q'))
			&& ((name == "QE2") ))
		{
			fromNametoList(seq, "HE21", chainID, tempNameAtomList);
			fromNametoList(seq, "HE22", chainID, tempNameAtomList);
			return;
		} else
		if (((res == 'W'))
			&& ((name == "QZ") ))
		{
		  fromNametoList(seq, "HZ1", chainID, tempNameAtomList);
		  fromNametoList(seq, "HZ2", chainID, tempNameAtomList);
		  fromNametoList(seq, "HZ3", chainID, tempNameAtomList);
		  return;
		}
	}
	else if (ambi > 1) // non-stereospcific
	{
		if (((res == 'C') || (res == 'D') || (res == 'E') || (res == 'F') || (res == 'H')
			|| (res == 'K') || (res == 'L') || (res == 'M')|| (res == 'N')|| (res == 'P')
			|| (res == 'Q') || (res == 'R')|| (res == 'S')|| (res == 'W')|| (res == 'Y'))
			&& ((name == "QB") ||(name == "HB2") || (name == "HB3")))
		{
		  fromNametoList(seq, "HB1", chainID, tempNameAtomList);
		  fromNametoList(seq, "HB2", chainID, tempNameAtomList);
		  fromNametoList(seq, "HB3", chainID, tempNameAtomList);
		  return;
		} else
		if (((res == 'E') || (res == 'K') || (res == 'M')|| (res == 'P') || (res == 'Q')|| (res == 'R'))
			&& ((name == "QG") ||(name == "HG2") || (name == "HG3")))
		{
		  fromNametoList(seq, "HG1", chainID, tempNameAtomList);
		  fromNametoList(seq, "HG2", chainID, tempNameAtomList);
		  fromNametoList(seq, "HG3", chainID, tempNameAtomList);
		  return;
		} else
		if (((res == 'I'))
			&& ((name == "QG1") ||(name == "HG12") || (name == "HG13")))
		{
		  fromNametoList(seq, "HG11", chainID, tempNameAtomList);
		  fromNametoList(seq, "HG12", chainID, tempNameAtomList);
		  fromNametoList(seq, "HG13", chainID, tempNameAtomList);
		  return;
		} else
		if (((res == 'V'))
			&& ((name == "QG") || (name == "QQG") || (name == "QG1") || (name == "QG2") || (name == "HG1") || (name == "HG2")))
		{
			fromNametoList(seq, "HG11", chainID, tempNameAtomList);
			fromNametoList(seq, "HG12", chainID, tempNameAtomList);
			fromNametoList(seq, "HG13", chainID, tempNameAtomList);
			fromNametoList(seq, "HG21", chainID, tempNameAtomList);
			fromNametoList(seq, "HG22", chainID, tempNameAtomList);
			fromNametoList(seq, "HG23", chainID, tempNameAtomList);
			return;
		} else
		if (((res == 'V'))
			&& (name == "QCG"))
		{
			fromNametoList(seq, "CG1", chainID, tempNameAtomList);
			fromNametoList(seq, "CG2", chainID, tempNameAtomList);
			return;
		} else
		if (((res == 'K') || (res == 'P') || (res == 'R'))
			&& ((name == "QD") ||(name == "HD2") || (name == "HD3")))
		{
		  fromNametoList(seq, "HD1", chainID, tempNameAtomList);
		  fromNametoList(seq, "HD2", chainID, tempNameAtomList);
		  fromNametoList(seq, "HD3", chainID, tempNameAtomList);
		  return;
		} else
		if (((res == 'F') ||  (res == 'Y'))
			&& ((name == "QD") || (name == "CG") || (name == "HD1") || (name == "HD2")))
			//&& ((name == "QD") || (name == "HD1") || (name == "HD2")))
		{
			fromNametoList(seq, "HD1", chainID, tempNameAtomList);
			fromNametoList(seq, "HD2", chainID, tempNameAtomList);
			return;
		} else
		if (((res == 'N'))
			&& ((name == "QD2") || (name == "HD22") || (name == "HD21")))
		{
			fromNametoList(seq, "HD21", chainID, tempNameAtomList);
			fromNametoList(seq, "HD22", chainID, tempNameAtomList);
			return;
		} else
		if (((res == 'L'))
			&& ((name == "QD") || (name == "QQD") || (name == "QD1") || (name == "QD2") || (name == "HD1") || (name == "HD2")))
		{
			fromNametoList(seq, "HD11", chainID, tempNameAtomList);
			fromNametoList(seq, "HD12", chainID, tempNameAtomList);
			fromNametoList(seq, "HD13", chainID, tempNameAtomList);
			fromNametoList(seq, "HD21", chainID, tempNameAtomList);
			fromNametoList(seq, "HD22", chainID, tempNameAtomList);
			fromNametoList(seq, "HD23", chainID, tempNameAtomList);
			return;
		} else
		if (((res == 'L'))
			&& (name == "QCD"))
		{
			fromNametoList(seq, "CD1", chainID, tempNameAtomList);
			fromNametoList(seq, "CD2", chainID, tempNameAtomList);
			return;
		} else
		if (((res == 'K'))
			&& ((name == "QE") ||(name == "HE2") || (name == "HE3")))
		{
		  fromNametoList(seq, "HE1", chainID,  tempNameAtomList);
		  fromNametoList(seq, "HE2", chainID, tempNameAtomList);
		  fromNametoList(seq, "HE3", chainID, tempNameAtomList);
		  return;
		} else
		if (((res == 'F') || (res == 'Y'))
			&& ((name == "QE") || (name == "CZ") || (name == "HE1") || (name == "HE2")))
			//&& ((name == "QE") || (name == "HE1") || (name == "HE2")))
		{
		  fromNametoList(seq, "HE1", chainID, tempNameAtomList);
		  fromNametoList(seq, "HE2", chainID, tempNameAtomList);
		  return;
		} else
		if (((res == 'Q'))
			&& ((name == "QE2") || (name == "HE22") || (name == "HE21")))
		{
			fromNametoList(seq, "HE21", chainID, tempNameAtomList);
			fromNametoList(seq, "HE22", chainID, tempNameAtomList);
			return;
		} else
		if (((res == 'W'))
			&& ((name == "QZ") ||(name == "HZ2") || (name == "HZ3")))
		{
		  fromNametoList(seq, "HZ1", chainID, tempNameAtomList);
		  fromNametoList(seq, "HZ2", chainID, tempNameAtomList);
		  fromNametoList(seq, "HZ3", chainID, tempNameAtomList);
		  return;
		}
		if (((res == 'W'))
		    && ((name == "HD1")))
		{
		  fromNametoList(seq, "HD1", chainID, tempNameAtomList);
		  fromNametoList(seq, "HD", chainID, tempNameAtomList);
		  return;
		}
		if (((res == 'W'))
		    && ((name == "HH2")))
		{
		  fromNametoList(seq, "HH2", chainID, tempNameAtomList);
		  fromNametoList(seq, "HH", chainID, tempNameAtomList);
		  return;
		}
		if (((res == 'W'))
		    && ((name == "HH2")))
		{
		  fromNametoList(seq, "HH2", chainID, tempNameAtomList);
		  fromNametoList(seq, "HH", chainID, tempNameAtomList);
		  return;
		}
	}
	fromNametoList(seq, name, chainID, tempNameAtomList);
	return;
}

// for rule-in PDB and back calculation
// if not degenerate, dmin
// if degenerate, sum-average

// from atom in chemical shift to atoms in structure,
// not uniquely mapped, so ambiguirity issue need to
// consider

void find_newName(char sym1, const nameAtom& atom1Pair, string name1new[])
{
      	if ((sym1 == 'L') && (atom1Pair.size() == 6))
       	{
         	name1new[0] = "HD1";
          name1new[1] = "HD2";
        }
        else if ((sym1 == 'V') && (atom1Pair.size() == 6))
       	{
         	name1new[0] = "HG1";
          name1new[1] = "HG2";
        }
        else if (atom1Pair.size() == 2)
        {
        	int i = 0;
          for(nameAtom::const_iterator iteratom1 = atom1Pair.begin(); iteratom1 != atom1Pair.end(); iteratom1++)
      		{
        		name1new[i++] = iteratom1->first.first.second;
        	}
        }
}

// recursive algorithm
float Pdb::get_dist(ostream& outputError, int index1, char sym1, const string& name1, int ambi1, char chain1,
										int index2, char sym2, const string& name2, int ambi2, char chain2)
{

  //cout << index1 << " " << sym1 << " " << name1 << " " << ambi1 << " " << chain1 << "::" << index2 << " " << sym2 << " " << name2 << " " << ambi2 << " " << chain2 << endl;

	SeqNameChainAmbi a = make_pair(make_pair(index1, name1), make_pair(chain1, ambi1));
	SeqNameChainAmbi b = make_pair(make_pair(index2, name2), make_pair(chain2, ambi2));

	disPdbAtomPairStat::iterator i = disPdbAtomPairStatList.find(make_pair(a,b));
	if (i!= disPdbAtomPairStatList.end())
		return i->second;

		nameAtom atom1Pair;
		nameAtom atom2Pair;

		atom1Pair.clear();
		atom2Pair.clear();

		get_Atom(index1, sym1, name1, ambi1, chain1, atom1Pair);
		get_Atom(index2, sym2, name2, ambi2, chain2, atom2Pair);


		if ((atom1Pair.size() == 0) && (ambi1 == 1) && (sym1 == 'N') && (name1 == "QD2"))
		{
			get_Atom(index1, sym1, name1, 2, chain1, atom1Pair);
		}

		if ((atom1Pair.size() == 0) && (ambi1 == 1) && (sym1 == 'L') && (name1 == "QQD"))
		{
			get_Atom(index1, sym1, name1, 2, chain1, atom1Pair);
		}

		if ((atom1Pair.size() == 0) && (ambi1 == 1) && (sym1 == 'V') && (name1 == "QQG"))
		{
			get_Atom(index1, sym1, name1, 2, chain1, atom1Pair);
		}

		if ((atom2Pair.size() == 0) && (ambi2 == 1) && (sym2 == 'N') && (name2 == "QD2"))
		{
			get_Atom(index2, sym2, name2, 2, chain2, atom2Pair);
		}

		if ((atom2Pair.size() == 0) && (ambi2 == 1) && (sym2 == 'L') && (name2 == "QQD"))
		{
			get_Atom(index2, sym2, name2, 2, chain2, atom2Pair);
		}

		if ((atom2Pair.size() == 0) && (ambi2 == 1) && (sym2 == 'V') && (name2 == "QQG"))
		{
			get_Atom(index2, sym2, name2, 2, chain2, atom2Pair);
		}




    set<string, less<string> > atom1Set; atom1Set.clear();
    set<string, less<string> > atom2Set; atom2Set.clear();
    for(nameAtom::iterator i = atom1Pair.begin(); i != atom1Pair.end(); ++i)
    	atom1Set.insert(i->first.first.second);
    for(nameAtom::iterator i = atom2Pair.begin(); i != atom2Pair.end(); ++i)
    	atom2Set.insert(i->first.first.second);

		if ((atom1Pair.size() == 0) || (atom2Pair.size() == 0))
		{

			if (atom1Pair.size() == 0)
			{
				outputError << "IN get_dist *** IN PDB FILE: NO MATCH atom1 " << index1
				 << sym1 << "-"
				 << name1 << "-"
				 << "chain=" << chain1 << " "
				 << "ambi=" << ambi1 << " ";
				cout << "*** IN PDB FILE: NO MATCH atom1 " << index1
				<< sym1 << "-"
				<< name1 << "-"
				<< "chain=" << chain1 << " "
				<< "ambi=" << ambi1 << " ";
			}
			if (atom2Pair.size() == 0)
			{
				outputError << "IN get_dist *** IN PDB FILE: NO MATCH atom2 " << index2
				 << sym2  << "-"
				 << name2 << "-"
				 << "chain=" << chain2 << " "
				 << "ambi=" << ambi2 << " ";

			  cout << "*** IN PDB FILE: NO MATCH atom2 " << index2
				<< sym2  << "-"
				<< name2 << "-"
				<< "chain=" << chain2 << " "
				<< "ambi=" << ambi2 << " ";
			}
			outputError << endl << flush;
			cout << endl << flush;
		}

		//--- ambigurity case, find the min distance
		// if not degenerate, dmin
		// if degenerate, sum-average

		// unique, return sum-average

		float dist = NOMATCH;
		if (((atom2Set.size() == 1) || (atom2Set.size() == 3)) &&
				((atom1Set.size() == 1) || (atom1Set.size() == 3))) // 1x1, 1x3, 3x3
		{
			SampleStatistic dsum; dsum.reset();
  		for(nameAtom::iterator iteratom1 = atom2Pair.begin(); iteratom1 != atom2Pair.end(); iteratom1++)
  		{
    		for(nameAtom::iterator iteratom2 = atom1Pair.begin(); iteratom2 != atom1Pair.end(); iteratom2++)
    		{
    			float dist = calcDist((*iteratom1).second, (*iteratom2).second); // endcase
       		dsum += dist;
    		}
  		}
      dist = summation(dsum);
		}
		else if (((atom1Set.size() == 2) || (atom1Set.size() == 6)) && ((name1[0] != 'Q') && (name1[0] != 'C'))) // 2x1, 2x2, 2x3, 2x6
		{
  			float dmin = NOMATCH;
     		string name1new[2];

       	find_newName(sym1, atom1Pair, name1new);

				for(int i=0; i<2; ++i)
				{
  	      float dist = get_dist(outputError, index1, sym1, name1new[i], 0, chain1, index2, sym2, name2, ambi2, chain2);
    			if (dist < dmin) dmin = dist;
    		}
       dist = dmin;
		}
		else if (((atom2Set.size() == 2) || (atom2Set.size() == 6)) && ((name2[0] != 'Q') && (name2[0] != 'C'))) // 1x2, 3x2, 6x2
		{
  			float dmin = NOMATCH;
     		string name2new[2];

       	find_newName(sym2, atom2Pair, name2new);

				for(int i=0; i<2; ++i)
				{
  	      float dist = get_dist(outputError, index1, sym1, name1, ambi1, chain1, index2, sym2, name2new[i], 0, chain2);
    			if (dist < dmin) dmin = dist;
    		}
       dist = dmin;
		}
		else //if ((name2[0] == 'Q') && (name1[0] == 'Q')) // both degenrate, sum-average
		{
			SampleStatistic dsum; dsum.reset();
  		for(nameAtom::iterator iteratom1 = atom2Pair.begin(); iteratom1 != atom2Pair.end(); iteratom1++)
  		{
    		for(nameAtom::iterator iteratom2 = atom1Pair.begin(); iteratom2 != atom1Pair.end(); iteratom2++)
    		{
    			float dist = calcDist((*iteratom1).second, (*iteratom2).second); // endcase
    			dsum += dist;
    		}
  		}
      dist = summation(dsum);
		}

		disPdbAtomPairStatList.insert(make_pair(make_pair(a,b), dist));

    //cout << dist << endl;
		return dist;
}


// will be extended to support more than two chains
// intra chain
float Pdb::get_dist(ostream& outputError, int index1, char sym1, const string& name1, int ambi1,
										int index2, char sym2, const string& name2, int ambi2)
{
	float dist = -1;
	for(set<char, less<char> >::const_iterator i=chainIDSet.begin(); i != chainIDSet.end(); i++)
	{
		float dist2 = get_dist(outputError, index1, sym1, name1, ambi1, (*i), index2, sym2, name2, ambi2, (*i));
		if (dist2 > dist) dist = dist2;
	}
	//cout << dist << endl;
	return dist;
}


// used for EC contact for all C-C atom distances
float Pdb::get_dist(ostream& outputError, int index1, int index2)
{
	float dist = NOMATCH;
	//cout << "in get_dist\n";
	for(set<char, less<char> >::const_iterator i=chainIDSet.begin(); i != chainIDSet.end(); i++)
	{
	  //float dist2 = get_dist(outputError, index1, (*i), index2, (*i));
	  list<pdbAtom> resi; list<pdbAtom> resj;
	  resi.clear(); resj.clear();

	  get_Atom(index1, *i, resi);
	  get_Atom(index2, *i, resj);

	  //outputError << "in get_dist2\n";

	  for(list<pdbAtom>::iterator ri = resi.begin(); ri != resi.end(); ri++)
	    for(list<pdbAtom>::iterator rj = resj.begin(); rj != resj.end(); rj++)
	      {
		if ((ri->get_name()[0] == 'C') && (rj->get_name()[0] == 'C'))
		  {
		    float dist2 =  calcDist(*ri, *rj);
		    //if (dist2 < 5)
		    //  NOESY_Assign::outputError() << "distance pair" << *ri << endl << *rj << " " << dist2 << " " << dist << endl;
		    if (dist2 < dist) dist = dist2;
		  }
	      }

	}
	//outputError << dist << endl;
	return dist;
}


// will be extended to support more than two chains
// inter chain
float Pdb::get_distIC(ostream& outputError, int index1, char sym1, const string& name1, int ambi1,
											int index2, char sym2, const string& name2, int ambi2)
{
	float dist = -1;

	for(set<char, less<char> >::const_iterator i=chainIDSet.begin(); i != chainIDSet.end(); i++)
	for(set<char, less<char> >::const_iterator j=chainIDSet.begin(); j != chainIDSet.end(); j++)
	{
		if (j==i) continue;

		float dist2 = get_dist(outputError, index1, sym1, name1, ambi1, (*i), index2, sym2, name2, ambi2, (*j));
		if (dist2 > dist) dist = dist2;
	}
	return dist;
}

void Pdb::build_kdtree()
{
  for(nameAtom::iterator i = nameAtomList.begin(); i != nameAtomList.end(); i++)
    {
        data_points.push_back(point_node(i->second.get_x(), i->second.get_y(), i->second.get_z()));
        pdbmap.insert(make_pair(make_pair(make_pair(i->second.get_x(), i->second.get_y()), i->second.get_z()), (i->first).first.first));
    }

  if (data_points.size())
    d = new K3Dtree(data_points.begin(), data_points.end(), data_points.size());
}
// give the atom name, find all pdb atoms matched to and find all atoms near by and return its name.
void Pdb::get_range(ostream& outputError, int index1, char sym1, const string& name1,
                                  int ambi1, float range, set<int, less<int> >& range_atoms)
{

  if (DEBUG)
  {
  }
	for(set<char, less<char> >::const_iterator i=chainIDSet.begin(); i != chainIDSet.end(); i++)
	{
    // 1. map the atom to pdbatom
	  SeqNameChainAmbi a = make_pair(make_pair(index1, name1), make_pair(*i, ambi1));

		nameAtom atom1Pair; atom1Pair.clear();

		get_Atom(index1, sym1, name1, ambi1, *i, atom1Pair);

		if ((atom1Pair.size() == 0))
		{

			if (atom1Pair.size() == 0)
			{
				outputError << "*** IN PDB FILE: NO MATCH atom1 " << index1
				 << sym1 << "-"
				 << name1 << "-"
				 << "chain=" << *i << " "
				 << "ambi=" << ambi1 << " ";
			}
			outputError << endl << flush;
		}

		//cout << "stop here 2\n";

    // 2. do range search
    point_list res1; // point within the range
    for(nameAtom::iterator iteratom1 = atom1Pair.begin(); iteratom1 != atom1Pair.end(); iteratom1++)
    {
          // find all pdbatoms within the range and return
          point_node center(iteratom1->second.get_x(), iteratom1->second.get_y(), iteratom1->second.get_z());
	  //cout << "stop here 2.1\n";

         res1 = d->call_range_search(center, range);
	 //cout << "stop here 2.2\n";

          for(point_list::const_iterator ires1 = res1.begin(); ires1 != res1.end(); ires1++)
          {
            map<pair<pair<double, double>, double>, int, less<pair<pair<double, double>, double> > >::iterator ip =
                      pdbmap.find(make_pair(make_pair(ires1->x, ires1->y), ires1->z));
            if (ip != pdbmap.end())
            {
              range_atoms.insert(ip->second);
	      //cout << "stop here 2\n";

            }
          }
    }
  }
}

ostream& operator<< (ostream& s, const pdbAtom& temp)
{
	s << "ATOM--" << temp.serial << "-\t-" << temp.name << "-\t-";
	s << temp.resName << "--" << temp.chainID << "--";
	s << temp.resSeq << "-\t-";
	s << temp.x << "-";
	s << temp.y << "-";
	s << temp.z << "-";
	return s;
}

ostream& operator<<(ostream& s, const Pdb& pdb)
{
  for(set<char, less<char> >::iterator i = pdb.chainIDSet.begin(); i != pdb.chainIDSet.end(); i++)
	{
	  s << "chainID :-" << (*i) << "-" << endl;
	}

	for(nameAtom::const_iterator i2 = pdb.nameAtomList.begin(); i2 != pdb.nameAtomList.end(); i2++)
	{
		s << (*i2).first.first.first << "-\t-" << (*i2).first.first.second
		  << "-\t-" << (*i2).first.second
		  << "-\t-" << ((*i2).second) << endl;
	}
	return s;
}

istream& operator>>(istream& s, Pdb& pdb)
{
  //--- should be secondary structure
  //--- atoms
  string line;

  pdb.chainIDSet.erase(pdb.chainIDSet.begin(), pdb.chainIDSet.end());

  while (getline(s, line))
    {
      if (line.length() < 3)
	continue;

      if ((line.substr(0,4) == "ATOM"))
    	{
	  if ((line.substr(17,2) == "PL") || (line.substr(17,3) == "LL5") || (line.substr(17,3) == "ORI"))
	    continue;

    	  pdbAtom tempAtom(line, pdb.type, seq.rbegin()->first + LLength);

	  if (tempAtom.get_name()[0]=='Q')
	    continue;

    	  pdb.nameAtomList.insert(make_pair(make_pair(make_pair(tempAtom.get_resSeq(), tempAtom.get_name()),tempAtom.get_chainID()), tempAtom));
    	  pdb.chainIDSet.insert(tempAtom.get_chainID());
	  pdb.resAtom.insert(make_pair(make_pair(tempAtom.get_resSeq(), tempAtom.get_chainID()), tempAtom));
    	}
      else if (line.substr(0,3) == "END")
    		break;
    }

  pdb.build_kdtree();
  return s;
}
