/***************************************************************************
                          pdbreader.h  -  readin Pdb coordinates
                             -------------------
    begin                : Mon Aug 21 2000
    copyright            : (C) 2000 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#ifndef PDBREADER_H
#define PDBREADER_H

#define NOMATCH 10000

#include <string>
#include <map>
#include <deque>

#include "K3Dtree.h"

using namespace std;
string itoa(int);

// each atom entry in the pdb line is corresponding to a pdbAtom class
class pdbAtom
{
private:
	int serial; 
	string name; 
	string resName; 
	char chainID;
	int resSeq; 
	float x; 
	float y;
	float z; 
public:
	pdbAtom(){};
	pdbAtom(string, int, int);
	pdbAtom(int, string, string, char, int, float, float, float); 

	~pdbAtom() {};
	
	void set_x(float xx) { x = xx;};
	void set_y(float yy) { y = yy;};
	void set_z(float zz) { z = zz;};
	
	float get_x() const {return x;};
	float get_y() const {return y;};
	float get_z() const {return z;};
	const string& get_name() const;
	int    get_resSeq() const;
	char get_chainID()const;
	
	friend float calcDist(const pdbAtom&, const pdbAtom&);
	friend ostream& operator<< (ostream&, const pdbAtom&);
};

typedef pair<int, string> SeqName;
typedef pair<SeqName, char> SeqNameChain;
typedef deque<SeqName> SeqNameDeque;
typedef multimap<SeqNameChain, pdbAtom> nameAtom;

typedef pair<SeqName, pair<char, int> > SeqNameChainAmbi;
typedef pair<SeqNameChainAmbi, SeqNameChainAmbi> SeqNameChainAmbiPair;

typedef map<SeqNameChainAmbiPair, float> disPdbAtomPairStat;

typedef pair<int, char> ResChain;
typedef multimap<ResChain, pdbAtom> resAtomList;  

class Pdb
{
protected:
	int type; 

	set<char, less<char> > chainIDSet; // multimers

	nameAtom nameAtomList;
	resAtomList resAtom; 
	
 
	point_list data_points; // used for range search
	K3Dtree* d; 
	
  map<pair<pair<double, double>, double>, int, less<pair<pair<double, double>, double> > > pdbmap;
  
	disPdbAtomPairStat  disPdbAtomPairStatList; // for internal use
	
public: 
	Pdb();
	Pdb(int ttype);
	~Pdb() {};

	void insertPdbAtom(const pdbAtom&);

	// given a name and get the pdbAtom  
	bool notEmpty() const { return nameAtomList.size();};
	void reset();
	void get_Atom(int, char, const string&, int, char, nameAtom&) const; 
	void get_Atom(int, char, list<pdbAtom>&) const; 
	
	const set<char, less<char> >& get_chainIDSet() { return chainIDSet; };

	void fromNametoList(int, const string&, char, nameAtom&) const;

	const nameAtom& get_nameAtomList() { return nameAtomList;};
  
	float get_dist(ostream&, int, int); // used for EC contacts
 	float get_dist(ostream&, int, char, const string&, int, char, int, char, const string&, int, char); // chain specific
 	float get_dist(ostream&, int, char, const string&, int, int, char, const string&, int) ; // intra chain
 	float get_distIC(ostream&, int, char, const string&, int, int, char, const string&, int) ; // inter chain

	// give the atom name, find all pdb atoms matched to and find all atoms near by and return its name.
	void build_kdtree();
	void get_range(ostream&, int, char, const string&, int, float, set<int, less<int> >&);

	friend ostream& operator<< (ostream&, const Pdb&);
	friend istream& operator>> (istream&, Pdb&); 
}   ;


#endif
