/***************************************************************************
                          template.h  -  description
                             -------------------
    begin                : Mon Aug 21 2000
    copyright            : (C) 2000 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#ifndef TEMPLATE_H
#define TEMPLATE_H

#include <string>
#include <map>
#include <iostream>
#include <vector>
#include <iterator>
#include <list>
#include <deque>
#include <map>
#include <algorithm>
#include <utility>
#include "AA.h"
#include "peaklist.h"
#include "atom.h"
#include "ss.h"
#include "pdbreader.h"
#include "noesy_assign.h"

using namespace std;

ostream& operator<<(ostream&, const general_peak&);
ostream& operator<<(ostream&, const general_peak*);

istream& operator>>(istream&, AA&);
ostream& operator<<(ostream&, const AA&);

istream& operator>>(istream&, atom&);
ostream& operator<<(ostream&, const atom&);
ostream& operator<<(ostream&, const atom*);

ostream& operator<<(ostream&, const pair<const string, Structure>&);
istream& operator>>(istream&, Structure&);

ostream& operator<<(ostream&, const Pdb&);
istream& operator>>(istream&, Pdb&);

ostream& operator<<(ostream&, const HBOND&);

istream& getlineNA(istream&, string&, char);

namespace std {
istream& operator>>(istream& s, pair<int,int>& temp)
{
	s >> temp.first >> temp.second;
	return s;
}

ostream& operator<<(ostream& s, const pair<int, int>& temp)
{
	s << temp.first << " " << temp.second;
	return s;
}
}

/* input and output for the pair structure 
 */ 
template<class Key, class T> istream& operator>>(istream&s, pair<Key, T>& atom_pair)
{
	s >> atom_pair.first >> atom_pair.second;
	return s; 
}

template<class Key, class T> ostream& operator<<(ostream&s, const pair<const Key, T>& atom_pair) 
{
	s << atom_pair.first << "\t" << atom_pair.second;
	return s;
}

// for vector
template<class T2> ostream& operator<<(ostream& s, const vector<T2>& alist)
{
	copy(alist.begin(), alist.end(), ostream_iterator<T2>(s, "\n"));
	return s;
}

//////////////////////////////
// for list and deque such as atomList, Pdb
/////////////////////////////
template<class T2> istream& operator>>(istream& s, list<T2>& alist)
{

  string inputLine;
  while (getlineNA(s, inputLine))
  {
    if (inputLine != "")
    {
      T2 temp;
      istringstream ist(inputLine);

      while (ist >> temp)
      {
        alist.push_back(temp);
        NOESY_Assign::outputLog() << "Read as-> " << temp << endl;
	}
			
      ios::iostate state = ist.rdstate();
    	
    	if ((state&ios::failbit) && !(state&ios::eofbit))
    	{
    		cerr << "*** Skipped line: " << inputLine << endl;	
    	}
    }
  }
	return s;
}


template<class T2> ostream& operator<<(ostream& s, const list<T2>& alist)
{
	copy(alist.begin(), alist.end(), ostream_iterator<T2>(s, "\n"));
	return s;
}

template<class T2> istream& operator>>(istream& s, deque<T2>& alist)
{

  string inputLine;
  while (getlineNA(s, inputLine))
  {
    if (inputLine != "")
    {
      T2 temp;
      istringstream ist(inputLine);

      while (ist >> temp)
      {
        alist.push_back(temp);
        NOESY_Assign::outputLog() << "Read as-> " << temp << endl;
	}
      ios::iostate state = ist.rdstate();
    	
    	if ((state&ios::failbit) && !(state&ios::eofbit))
    	{
    		cerr << "*** Line: " << inputLine << endl;	
    	}
    }
  }
	return s;
}


template<class T2> ostream& operator<<(ostream& s, const deque<T2>& alist)
{
	copy(alist.begin(), alist.end(), ostream_iterator<T2>(s, "\n"));
	return s;
}

////////////////////////////
// for map --- seq, Jlist
////////////////////////////

template<class Key, class T>istream& operator>>(istream& s, map<Key, T, less<Key> >& alist)
{
  string inputLine;
  while (getlineNA(s, inputLine))
  {
    if (inputLine != "")
    {
      pair<Key, T> temp;
      istringstream ist(inputLine);

      while (ist >> temp)
      {
        alist.insert(make_pair(temp.first, temp.second));
        NOESY_Assign::outputLog() << "Read as-> " << temp.first << " " << temp.second << endl;
	  }
      ios::iostate state = ist.rdstate();
    	
    	if ((state&ios::failbit) && !(state&ios::eofbit))
    	{
    		cerr << "*** Line: " << inputLine << endl;	
    	}
    }
  }
	return s;
}

template<class Key, class T>istream& operator>>(istream& s, multimap<Key, T, less<Key> >& alist)
{
  string inputLine;
  while (getlineNA(s, inputLine))
  {
    if (inputLine != "")
    {
      
      istringstream ist(inputLine);

			T temp;

      while (ist >> temp)
      {
         alist.insert(make_pair(temp.get_index(), temp));
         
         NOESY_Assign::outputLog() << "Read as-> " << temp.get_index() << " " << temp << endl;
      }

      ios::iostate state = ist.rdstate();
    	
    	if ((state&ios::failbit) && !(state&ios::eofbit))
    	{
    		cerr << "*** Line: " << inputLine << endl;	
    	}
    }
  }
	return s;	
}

/* output of the map structure 
 */ 
template<class Key, class T>ostream& operator<<(ostream& s, const map<Key, T, less<Key> >& alist) 
{
	copy(alist.begin(), alist.end(), ostream_iterator<const pair<const Key, T> >(s, "\n"));
	return s; 
}

template<class Key, class T>ostream& operator<<(ostream& s, const multimap<Key, T, less<Key> >& alist) 
{
	copy(alist.begin(), alist.end(), ostream_iterator<const pair<const Key, T> >(s, "\n"));
	return s; 
}

#endif
