/***************************************************************************
                          utility.cpp  -  description
                             -------------------
    begin                : Mon Dec 10 2001
    copyright            : (C) 2001 by Yuanpeng Janet Huang
    email                : yphuang@cabm.rutgers.edu
 ***************************************************************************/

#include <stdlib.h>
#include "utility.h"
#include <time.h>
#include <sys/param.h>


void split(string line, vector<string>& subline, const string delim)
{
    if (line != "")
    {
			while (line.find_first_not_of(delim) < line.length())
			  {
			    // remove the leading whitespaces
			    line = line.substr(line.find_first_not_of(delim),line.length() - line.find_first_not_of(delim));

			    // get the first words
			    if (line.find_first_of(delim) < line.length())
			      {
				      subline.push_back(line.substr(0, line.find_first_of(delim)));
				      line = line.substr(line.find_first_of(delim),line.length() - line.find_first_of(delim));
			      }
			    else
			      {
				      subline.push_back(line);
				      line = "";
			      }
			  }
    }
}

string itoa(int a)
{
	if (a == 0)
		return "0";

	string temp = "";
	int last;
	while (a)
	{
		last = a%10;
		temp += last + 48;
		a = a/10;
	}

	return string(temp.rbegin(), temp.rend());
}

void error(const char* s, const char* s2)
{
	cerr << endl << s << " : " << s2 << '\n';
	exit(1);
}

void error(const string s, const string s2)
{
	cerr << endl << s << " : " << s2 << '\n';
	exit(1);
}

ostream& erasePrint(ostream& s, const string line)
{
  static int length = 0;
  //  for(int i=0; i<length; i++)
  //{
  //  s << "\b";
  //}
  //for(int i=0; i<length; i++)
  //{
  //  s << " " << flush;
  //}
  //for(int i=0; i<length; i++)
  //{
  //  s << "\b";
  //}
  s << line << flush;
  length = line.length();
  return s;
}

istream& getlineNA(istream& s, string& inputLine, char delim)
{
  if (getline(s, inputLine, delim) && (inputLine.find("#") < inputLine.length()))
  {
	  inputLine = inputLine.substr(0, inputLine.find("#"));
  }

  //remove the leading white space and ending white space
  if (inputLine.find_first_not_of(" \t\n\v") < inputLine.length())
    inputLine = inputLine.substr(inputLine.find_first_not_of(" \t\n\v"),
				        inputLine.length() - inputLine.find_first_not_of(" \t\n\v"));
  if (inputLine.find_last_not_of(" \t\n\v") < inputLine.length())
    inputLine = inputLine.substr(0, inputLine.find_last_not_of(" \t\n\v")+1);
  return s;
}

string versionHeader()
{
  return " ASDP Version 2.3 Copyright(C) 2020 Janet Huang and Gaetano T. Montelione";
}

string getHelp()
{
	string helpString;
	helpString += "     Options:\n";
	helpString += "\t -c control_file      Required\n";
	helpString += "\t -o output_dir        Required\n";
	helpString += "\t -d                   For debug\n";
	helpString += "\t -h                   Help\n";
	helpString += "\t -m                   Exclude PCT-filter of Cycle1 for symmetry analysis\n";
	helpString += "\t -n                   Exclude CSI-based secondary structure analysis\n";
	helpString += "\t -f structure_file    Initial fold for bootstrapping\n";
	helpString += "\t -i                   Deuterium Correction (Deuterated Proteins Only). Select this option\n";
	helpString += "\t                          if the input CA/CB chemical shifts need to be adjusted for deuterium isotope effect\n";
	helpString += "\t -j                   Include J-coupling constant data for angle constraint analysis (HYPER)\n";
	helpString += "\t -k float_number      Calibration distance (default 3.4)\n";
	helpString += "\t -N                   Include NOE assignments in HYPER caluclation (under development)\n";
	helpString += "\t -q structure_file    RPF-Calculate the F and DP scores of the input structure_file (IUPAC naming)\n";
	helpString += "\t -r path              Restore from a prior output_dir\n";
	helpString += "\t -R                   Include rotamer constraints in HYPER calculation (under development)\n";
	helpString += "\t -v                   Calculate the M score and average chemical shifts\n";
	//helpString += "\t -e                   Run EC constraint calculation\n";
	helpString += "\t -s                   Only print out ovw file\n";
	helpString += "\t -t                   Only do one run\n";

	return helpString;
}

void readControlFile(const string& controlFileName, ControlMap& control, PeakListDeque& peakListDeque, Command& command)
{
	erasePrint(cout, " Reading Control_File ...");

	///////////////////////////////
	//read in the control file   //
	///////////////////////////////

	ifstream controlFileStream(controlFileName.c_str());
	if (!controlFileStream) error("cannot open controlFile", controlFileName.c_str());
	string inputLine;
	int lnum=0;
	while(getlineNA(controlFileStream, inputLine))
	{
	  lnum++;
	  if ((inputLine[0] == '[') && (inputLine[inputLine.length()-1] == ']')) // a line with []
	  {
	    inputLine = inputLine.substr(1,inputLine.length()-2);
	    if (inputLine == "General")
	    {
        while ((controlFileStream.peek() != EOF) && (controlFileStream.peek() != '['))
        {
          string attrLine="";
          getlineNA(controlFileStream, attrLine);
					lnum++;
          if (attrLine != "")
          {
            if (attrLine.find("=") < attrLine.length())
              control[attrLine.substr(0, attrLine.find('='))] = attrLine.substr(attrLine.find('=')+1, attrLine.length()-attrLine.find('=')-1);
            else
              error("bad format --- missing = at Line " + itoa(lnum) + " " + attrLine + " in ", controlFileName);
          }
        }
	    }
	    else if ((controlFileStream.peek() != EOF) && (inputLine == "hyperCommand") && (controlFileStream.peek() != '['))
	    {
	      getlineNA(controlFileStream, command.hyper);
				lnum++;
	    }
	    else if ((controlFileStream.peek() != EOF) && (inputLine == "viewerCommand") && (controlFileStream.peek() != '['))
	    {
	      getlineNA(controlFileStream, command.viewer);
        lnum++;
	    }
	    else if ((controlFileStream.peek() != EOF) && (inputLine == "dyanaCommand") && (controlFileStream.peek() != '['))
	    {
	      getlineNA(controlFileStream, command.dyana);
				lnum++;
	    }
	    else if ((controlFileStream.peek() != EOF) && (inputLine == "cyana-2.1Command") && (controlFileStream.peek() != '['))
	    {
	      getlineNA(controlFileStream, command.cyana);
				lnum++;
	    }
	    else if ((controlFileStream.peek() != EOF) && (inputLine == "cnsCommand") && (controlFileStream.peek() != '['))
	    {
	      getlineNA(controlFileStream, command.cns);
				lnum++;
	    }
	    else if ((controlFileStream.peek() != EOF) && (inputLine == "xplorCommand") && (controlFileStream.peek() != '['))
	    {
	      getlineNA(controlFileStream, command.xplor);
				lnum++;
	    }
	    else if ((controlFileStream.peek() != EOF) && (inputLine == "refineCommand") && (controlFileStream.peek() != '['))
	    {
	      getlineNA(controlFileStream, command.refine);
				lnum++;
	    }
	    else if ((controlFileStream.peek() != EOF) && (inputLine == "noneCommand") && (controlFileStream.peek() != '['))
	    {
	   	string temp;
	      getlineNA(controlFileStream, temp); //skip
				lnum++;
	    }
	    else
	    {   // peak list
        ControlMap peakList;
        peakList["name"] = inputLine;
        while ((controlFileStream.peek() != EOF) && (controlFileStream.peek() != '['))
        {
          string attrLine="";
          getlineNA(controlFileStream, attrLine);
					lnum++;
          if (attrLine != "")
          {
            if (attrLine.find("=") < attrLine.length())
              peakList[attrLine.substr(0, attrLine.find('='))] = attrLine.substr(attrLine.find('=')+1, attrLine.length()-attrLine.find('=')-1);
            else
              error("bad format - at line" + itoa(lnum) + " " + attrLine + " in " , controlFileName);
          }
        }
        peakListDeque.push_back(peakList);
	    }
	  } else
	  {
	    error("bad format - at Line " + itoa(lnum) + " " + inputLine + " in ", controlFileName);
	  }
	}
	controlFileStream.close();

	// set CNS input path
	if (command.cns != "")
	{
		if (control["cnsPath"] == "")
		{
			error("no cnsPath defined", controlFileName);
		}
		else
		{
			if (*control["cnsPath"].begin() != '/')
			{
				char* currentWD = getcwd(NULL,0);

				if (currentWD != NULL)
					control["cnsPath"] = string(currentWD) + "/" + control["cnsPath"];
				else
					error("System error", "Cannot get PWD (getcwd)");
				free(currentWD);
			}
		}
	}
}

void runSummary(ostream& outputOvw, const clock_t& startT, const clock_t& endT, const tms& timeLockS, const tms& timeLockE)
{
  long hz; hz = sysconf(_SC_CLK_TCK);
  outputOvw << endl << "Statistics: " << endl;
  outputOvw << "=========== " << endl << endl;
  outputOvw << "NOESY_Assign:" << endl;
  outputOvw << "      User CPU Time: " << setw(10) << setprecision(3) << (double) (timeLockE.tms_utime - timeLockS.tms_utime)/(hz) << " sec." << endl;
  outputOvw << "      Syst CPU Time: " << setw(10) << setprecision(3) << (double)(timeLockE.tms_stime - timeLockS.tms_stime)/(hz) << " sec."  << endl;
  outputOvw << "Other Processes:" << endl;
  outputOvw << "      User CPU Time: " << setw(10) << setprecision(3) << (double)(timeLockE.tms_cutime - timeLockS.tms_cutime)/(hz) << " sec."  << endl;
  outputOvw << "      Syst CPU Time: " << setw(10) << setprecision(3) << (double)(timeLockE.tms_cstime - timeLockS.tms_cstime)/(hz) << " sec."   << endl;
  outputOvw << endl << "Total Elapsed Time: " << setw(10) << setprecision(3) << (double)(endT - startT)/(hz) << " sec. " << endl;
}

void createOutputDir(string outputDir)
{
	////////////////////////////////////////////////////////////
	// prepare the output files                             ////
	////////////////////////////////////////////////////////////

	if (outputDir == "")
	  error("no output directory defined", "Please use -o output_dir");
	if (ifstream(outputDir.c_str()))
	  {
	  } else
	  {
	    mkdir(outputDir.c_str(), 0750);
	  }
}
