#include "pdbstat.h"
extern void write_aco(FILE *rtto, int option);
extern void do_average_selected(char tipo[MAXWORD]);
extern void do_average_coords(char tipo[MAXWORD]);
extern void DumpHeader(FILE *rtto);

int       flagall;
static char WriteUPL, WriteLOL ;

/* ========================================================================= */
void writing(char argumento[MAXWORD])
/*
 * Driver to write a variety of things. between them coordinates, 
 * constraints, etc.
 *
 * (Roberto Tejero, RTT, May 1993, CABM-Rutgers University).
 *
 */
{
     
     static FILE     *rtto;
     static int      i;
     static char     exclude[7] ;
     static char What[10], Format[10], Who[50], FileName[100];
     static struct   models *model ;
     static div_t    Cont ;

     WriteUPL = WriteLOL = FALSE ;
     
     /*
      * Look for special exclusions in writing a file like exclude all RMSD's >
      * greater than a cutoff or all ENERGIES greater than a cutoff.
      */
     strcpy(What,"");
     strcpy(Format,"");
     strcpy(Who,"");
     strcpy(FileName,"");
     
     sscanf(argumento,"%*s %s %s", exclude, palabra);
     if (strncasecmp(exclude,"exclu",2)==0) {
	  if ((strncasecmp(palabra,"rmsd",2)==0) || (strncasecmp(palabra,"all",1)==0)) {
	       printf("> WRITER: Rmsd Cutoff ?_ : ");
	       GetFromKeyBoard(exclude, MAXWORD, stdin);
	       RMSD_MODEL_CUTOFF = atof(exclude) ;
	  }
	  if ((strncasecmp(palabra,"energy",2)==0) || (strncasecmp(palabra,"all",1)==0)) {
	       printf("> WRITER: Energy Cutoff ?_ : ");
	       GetFromKeyBoard(exclude, MAXWORD, stdin) ;
	       ENERGY_CUTOFF = atof(exclude);
	  }
     } else {
	  RMSD_MODEL_CUTOFF = 9999.0 ;
	  ENERGY_CUTOFF = 9999.0 ;
     }
     
     /* 
      * Analysis of arguments in Command
      */
     sscanf(argumento,"%*s %s %*s %*s %*s", What);
     if ( strncasecmp(What,"coor",3)==0 ) {
	  sscanf(argumento,"%*s %*s %s", Format);
	  sscanf(argumento,"%*s %*s %*s %s", Who);
	  if (strncasecmp(Who,"all",3)!=0 && strncasecmp(Who,"bac",3)!=0 &&
	      strncasecmp(Who,"hea",3)!=0 && strncasecmp(Who,"sel",3)!=0) {
	       strcpy(Who,"all");
	       sscanf(argumento,"%*s %*s %*s %s", FileName);
	  } else {
	       sscanf(argumento,"%*s %*s %*s %*s %s", FileName);
	  }
     } else if ( strncasecmp(What,"aco",3)==0 ) {
	  sscanf(argumento,"%*s %*s %s %s", Format, FileName);
     } else if ( strncasecmp(What,"cons",3)==0 ) {
	  sscanf(argumento,"%*s %*s %s %s", Format, FileName);
     } else if ( strncasecmp(What,"upl",3)==0 ) {
	  sscanf(argumento,"%*s %*s %s %s", Format, FileName);
     } else if ( strncasecmp(What,"lol",3)==0 ) {
	  sscanf(argumento,"%*s %*s %s %s", Format, FileName);
     }
     
     flagall=0;
     if (strcmp(FileName,"")==0) {
	  printf("\n> WRITER: Output file name ?:_  ");
	  GetFromKeyBoard(palabra, MAXWORD, stdin);
     } else {
	  strcpy(palabra,FileName);
     }
     strcat(history[count_history]," ");
     strcat(history[count_history],palabra);
     if (strncmp(palabra,"exit",4)==0) return;
     if ((rtto=fopen(palabra,"w")) == NULL) {  /* some error */
	  fprintf(stderr,"\n writing(): **Trying to open file** ->  ") ; 
	  perror(palabra);
	  fprintf(stderr, " writing(): Please make sure you have _WRITE_ permission\n");
	  return ;
     }
     
 otravez:
     if (strcmp(What,"")==0) {
	  printf("> WRITER: Coords, constraints, aco or sequence file? :_   ");
	  GetFromKeyBoard(palabra, MAXWORD, stdin);
     } else {
	  strcpy(palabra, What);
     }
     
     if ((strcmp(palabra,"c")==0) || (strcmp(palabra,"co") ==0)) {
	  printf("> WRITER: ** Ambiguous, it can be coord or constr. Type again ** \n");
	  goto otravez;
     }
     
     strcat(history[count_history]," ");
     strcat(history[count_history],palabra);
     if (strncmp(palabra,"exit",4)==0) return;
     
     /*
      * Residues file for HYPER.
      * Given a set of dihedrals calculated this piece just write down a special
      * file I needed for HYPER. No other usefulness. It is not documented as
      * it should have no utility for anyone but the author.
      */
     if (strncmp(palabra,"Residues",2)==0) {
	  model = MODELS+key_model;
	  fprintf(stdout," > Writing Residue file for %d residues\n",
		  model->number_of_residues);
	  for (i=1; i <= model->number_of_residues; i++) {
	       if (model->DIH[i].phi > 1111.0) model->DIH[i].phi = 999.0;
	       if (model->DIH[i].psi > 1111.0) model->DIH[i].psi = 999.0;
	       if (model->DIH[i].chi1 > 1111.0) model->DIH[i].chi1 = 999.0;
	       fprintf(rtto," Residue_%3s_%d[%d,%d,%d]\n",
		       model->DIH[i].resname, model->DIH[i].resnumber, (int) model->DIH[i].phi,
		       (int) model->DIH[i].psi, (int) model->DIH[i].chi1);
	  }
	  fclose(rtto);
	  return;
     } 
     
     /* 
      * CONSTRAINTS, UPL, LOL etc 
      */
     if ( strncmp(palabra,"cons",3)==0 || strncmp(palabra,"upl",3)==0 || strncmp(palabra,"lol",3)==0 )  {
	     WriteUPL = WriteLOL = FALSE ;
	  if (strncmp(palabra,"cons",3)==0) {
		  WriteUPL = TRUE ;
		  WriteLOL = FALSE ;
	  }
	  if (strncmp(palabra,"upl",3)==0) { 
		  WriteUPL = TRUE ;
		  WriteLOL = FALSE ;
	  }
	  if (strncmp(palabra,"lol",3)==0) {
		  WriteUPL = FALSE ;
		  WriteLOL = TRUE ;
	  }
	  if (kount_dist <= 0) {
	       printf("> WRITER_cnstr: ** Warning: NO constraint file read\n");
	       return ;
	  }
	  strcat(history[count_history]," constraints");
     coge:
	  if ( strcmp(Format,"")==0) {
	       printf("> WRITER_constr: Output format [congen|discover|ecepp|disman|dyana|cyana|diana|xplor|cns]? : ");
	       GetFromKeyBoard(palabra, MAXWORD, stdin);
	  } else {
	       strcpy(palabra,Format);
	  }
	  strcat(history[count_history]," ");
	  strcat(history[count_history],palabra);
	  if (strncmp(palabra,"exit",4)==0) return;
	  
	  if ((strcmp(palabra,"di")==0) || (strcmp(palabra,"d")==0) || (strcmp(palabra,"dis")==0)) {
	       printf("\n> WRITER_constr: ** Ambiguous, it can be disman or discover, type again ** \n");
	       goto coge;
	  } else if (strncmp(palabra,"congen",2) == 0)  {
	       strcat(history[count_history]," congen");
	       option=1;
	  } else if (strncmp(palabra,"disman",4) == 0) {
	       strcat(history[count_history]," disman");
	       option=2;
	  } else if (WriteUPL && (strncmp(palabra,"dyana",4) == 0)) { 
	       strcat(history[count_history]," dyana");
	       option=3;
	  } else if (WriteLOL && (strncmp(palabra,"dyana",4) == 0)) { 
	       strcat(history[count_history]," dyana");
	       option=31;
	  } else if (WriteUPL && (strncmp(palabra,"diana",4) == 0)) { 
	       strcat(history[count_history]," diana");
	       option=300;
	  } else if (WriteLOL && (strncmp(palabra,"diana",4) == 0)) { 
	       strcat(history[count_history]," diana");
	       option=301;
	  } else if (strncmp(palabra,"impact",1) == 0) {
	       strcat(history[count_history], " impact");
	       option=4;
	  } else if (strncmp(palabra,"discover",4) == 0) {
	       strcat(history[count_history]," discover");
	       option=5;
	  } else if (strncmp(palabra,"cns",2) == 0) {
	       strcat(history[count_history]," cns");
	       option=6;
	  } else if (strncmp(palabra,"xplor",1) == 0) {
	       strcat(history[count_history]," xplor");
	       option=6;
	  } else if (strncmp(palabra,"charmm",3) == 0) {
	       strcat(history[count_history]," charmm");
	       option=7;
	  } else if (strncmp(palabra,"ecepp",3) == 0) {
	       strcat(history[count_history]," ecepp");
	       option=20;
	  } else {
		  fprintf(stdout,"\n> WRITER_constr: ** format `%s' unknown or not supported **\n",
				  palabra);
	  }
	  escribe = 1;
	  readwrite(rtto,WriteUPL,WriteLOL);
	  fclose(rtto);
	  escribe=0;
     }
     
     
     /* 
      * ACO CONSTRAINTS 
      */
     if (strncasecmp(palabra,"aco",3) == 0)  {
	  if (acocount <= 0) {
	       printf("> WRITER_cnstr: ** Warning: NO ACO constraint file read\n");
	       return ;
	  }
	  strcat(history[count_history]," aco ");
     cogeaco:
	  if ( strcmp(Format,"")==0) {
	       printf("> WRITER_constr: Output format (congen | discover | impact | disman | diana | xplor | cns)? : ");
	       GetFromKeyBoard(palabra, MAXWORD, stdin);
	  } else {
	       strcpy(palabra,Format);
	  }
	  strcat(history[count_history]," ");
	  strcat(history[count_history],palabra);
	  if (strncmp(palabra,"exit",4)==0) return;
	  
	  if ((strcmp(palabra,"di") == 0) || (strcmp(palabra,"d") == 0) || (strcmp(palabra,"dis") == 0)) {
	       printf("\n> WRITER_constr: *** Ambiguous, it can be disman or discover, type again *** \n");
	       goto cogeaco;
	  }
	  if (strncasecmp(palabra,"congen",3) == 0)  {
	       strcat(history[count_history]," congen");
	       option=1;
	  }
	  if (strncasecmp(palabra,"disman",4) == 0) {
	       strcat(history[count_history]," disman");
	       option=2;
	  }
	  if (strncasecmp(palabra,"cyana",3) == 0 ) {
	       strcat(history[count_history]," cyana");
	       option=3;
	  }
	  if (strncasecmp(palabra,"dyana",3) == 0 ) {
	       strcat(history[count_history]," dyana");
	       option=3;
	  }
	  if (strncasecmp(palabra,"diana",3) == 0 ) {
	       strcat(history[count_history]," diana");
	       option=3;
	  }
	  if (strncasecmp(palabra,"impact",3) == 0) {
	       strcat(history[count_history], " impact");
	       option=4;
	  }
	  if (strncasecmp(palabra,"discover",4) == 0) {
	       strcat(history[count_history]," discover");
	       option=5;
	  }
	  if (strncasecmp(palabra,"cns",3) == 0) {
	       strcat(history[count_history]," cns");
	       option=6;
	  }
	  if (strncasecmp(palabra,"xplor",3) == 0) {
	       strcat(history[count_history]," xplor");
	       option=6;
	  }
	  if (strncasecmp(palabra,"charmm",3) == 0) {
	       strcat(history[count_history]," charmm");
	       option=7;
	  }
	  escribe = 1;
	  write_aco(rtto,option);
	  fclose(rtto);
	  escribe=0;
     }
     
     /* 
      * SEQUENCE 
      */
     if (strncmp(palabra,"sequence",2)==0) {
	  strcat(history[count_history]," sequence ");       
	  printf("> WRITE_seq: One, Three letter or full name (one/thr/full)? : ");
	  GetFromKeyBoard(palabra, MAXWORD, stdin) ;
	  strcat(history[count_history],palabra);
	  if (strncasecmp(palabra,"three",2)==0) {
	       printf("> WRITE_seq: Format (congen/disman)? : ");
	       GetFromKeyBoard(palabra, MAXWORD, stdin);
	       strcat(history[count_history],palabra);       
	       write_sequence(rtto,palabra,"three");
	       fclose(rtto);
	  } else if (strncasecmp(palabra,"one",2)==0) {
	       write_sequence(rtto,"disman","one");	 
	       fclose(rtto);
	  } else if (strncasecmp(palabra,"full",2)==0) {
	       write_sequence(rtto,"disman","full");
	       fclose(rtto);
	  } else{
	       write_sequence(rtto,"disman","three");
	       fclose(rtto);	 
	  }
     }
     
     /* 
      *  COORDINATES 
      */
     if (strncmp(palabra,"coordinates",3) == 0) {
	  strcat(history[count_history]," coordinates");
	  if ( strcmp(Who,"") == 0 ) {
	       printf("> WRITER: ... backbone, heavy, full set? (back/heavy/all/select): ");
	       GetFromKeyBoard(palabra, MAXWORD, stdin);
	  } else {
	       strcpy(palabra,Who);
	  }
	  
	  if (strncasecmp(palabra,"backbone",3)==0) {
	       BACKBONE = TRUE ;
	       HEAVY = FALSE ;
	       SELECTED = FALSE ;
	  } else if (strncasecmp(palabra,"heavy",3)==0) {
	       HEAVY = TRUE ;	   
	       BACKBONE = FALSE ;
	       SELECTED = FALSE ;
	  } else if (strncasecmp(palabra,"selected",3)==0) {
	       HEAVY = FALSE ;	   
	       BACKBONE = FALSE ;
	       SELECTED = TRUE ;
	  } else {
	       BACKBONE = FALSE ;
	       HEAVY = FALSE ;
	       SELECTED = FALSE ;
	  }
	  
	  if (total_models > 1) {
	       if (strncasecmp(Who,"sele",2)==0 || strncasecmp(Who,"all",2)==0) {
		    strcpy(palabra,"all");
	       } else if (strncasecmp(Who,"heavy",3)==0 || strncasecmp(Who,"bac",3)==0) {
		    strcpy(palabra,"all");
	       } else {
		    printf("> WRITER: What model ?_ : ");
		    GetFromKeyBoard(palabra, MAXWORD, stdin);
		    sscanf(palabra,"%*s %s",exclude);
	       }
	       if (strncasecmp(palabra,"average",2)==0)  {
		    if (strncasecmp(exclude,"selec",1)==0)
			 do_average_selected("selected");
		    else
			 do_average_coords("backbone");
		    key_model = AverageModel;
		    goto coor_writer;
	       } else  if (strncasecmp(palabra,"all",2)==0) {
		    printf("> WRITER:  --  All models to be written \n");
		    flagall = 1;
		    goto coor_writer;
	       } else  if (strncasecmp(palabra,"selected",2)==0) {
		    flagall=11;
		    goto coor_writer;
	       }
	       if ((atoi(palabra) > total_models) && (atoi(palabra) != AverageModel))
		    printf("> WRITER: ** Model should be between 1 and %d \n", key_model);
	       key_model = atoi(palabra);
	       goto coor_writer;
	  }
	  
     coor_writer:
	  
	  if (DEBUG >= 2) {
	       printf("\n RMSD_MODEL_CUTOFF   = %f", RMSD_MODEL_CUTOFF);
	       printf("\n ENERGY_MODEL_CUTOFF = %f\n\n", ENERGY_CUTOFF);
	  }
	  
	  /* Printing only files with RMSD lower or equal than a certain value  */
	  for (i = 1 ; i <= total_models; i++) {
	       model = MODELS + i ;
	       model->written = FALSE ;
	       /* 
		* Only selected models 
		*/
	       if (flagall==11 && !model->Selected) model->written = TRUE;
	       /*
		* Only selected RMSD values
		*/
	       if (flagall==11 && model->rmsd>RMSD_MODEL_CUTOFF) model->written = TRUE ;
	       /*
		* Printing only models with an ENERGY lower or equal than 
		* certain value 
		*/
	       if (flagall==11 && ENERGY_CUTOFF != 9999.0 && 
		   model->energy>ENERGY_CUTOFF) model->written = TRUE ;
	  }
	  
	  if (flagall == 11) flagall=1;
	  
	  if (strcmp(Format,"")==0) {
	       printf("> COORD_writer: Format  (pdb/congen/RasMol) ? : ");
	       GetFromKeyBoard(palabra, MAXWORD, stdin);
	  } else {
	       strcpy(palabra,Format);
	  }
	  
	  if (flagall == 1) strcat(history[count_history]," all");
	  
	  strcat(history[count_history]," ");
	  strcat(history[count_history],palabra);
	  if (strncmp(palabra,"exit",4)==0) return;
	  
	  /* pdb format */
	  if (strncmp(palabra,"pdb",1)==0)  {
	       strcat(history[count_history]," pdb");
	       temporalcount = 0 ;   /* Initialize total atom counter */
	       if (flagall == 1) {
		    fprintf(stdout,"  In Progess  | ");

		    /* Place to dump general header */
		    DumpHeader(rtto);

		    /* Loop through models */
		    for (i = 1 ; i <= total_models ; i++) {
			 key_model = i;  model = MODELS+i ;
			 temporalcount = 0 ;   /* Initialize total atom counter, new PDB behaviour */
			 Cont = div(i,4);
			 if (Cont.rem == 1) fprintf(stdout,"\b\b/ ");
			 if (Cont.rem == 2) fprintf(stdout,"\b\b- ");
			 if (Cont.rem == 3) fprintf(stdout,"\b\b\\ ");
			 if (Cont.rem == 0) fprintf(stdout,"\b\b| ");
			 fflush(stdout);
			 if (!model->written) write_coords_pdb(rtto, 7);
		    }
		    fprintf(stdout,"\n");
	       } else
		    write_coords_pdb(rtto, 0);
	       
	       /*
		* Writing the end of the file
		* Now this MASTER is copied with NO meaning. please FIX this 
		*/
	       fprintf(rtto, "MASTER\n");
	       fprintf(rtto, "END\n");
	       fclose(rtto);
	  }  else if (strncmp(palabra,"disman",1)==0) {
	       /*
		* disman format
		*/
	       printf("> COORD_writer: *NOT YET* developed .. ");
	  } else if (strncmp(palabra,"congen",1)==0)  {
	       
	       /*
		* pdb congen style
		*/
	       strcat(history[count_history]," pdb for congen");
	       if (flagall == 1) {
		    fprintf(stdout,"  In Progess  | ");
		    for (i = 1 ; i <= total_models ; i++) {
			 Cont = div(i,4);
			 if (Cont.rem == 1) fprintf(stdout,"\b\b/ ");
			 if (Cont.rem == 2) fprintf(stdout,"\b\b- ");
			 if (Cont.rem == 3) fprintf(stdout,"\b\b\\ ");
			 if (Cont.rem == 0) fprintf(stdout,"\b\b| ");
			 key_model = i;
			 check_label_pdb(2);
			 write_coords_pdb(rtto,0);
		    } 
	       } else {
		    check_label_pdb(2);
		    write_coords_pdb(rtto,0);
	       }
	       fclose(rtto);
	  } else if ((strncmp(palabra,"rasmol",3)==0) || (strncmp(palabra,"RasMol",3) == 0)) {
	       /*
		* Special file for multiple readings using RasMol
		* molecular graphics package
		*/
	       strcat(history[count_history]," RasMol");
	       /*
		* Initialize the counter for atoms
		*/
	       temporalcount = 0 ;
	       if (flagall == 1) {
		    fprintf(stdout,"  In Progress  | ");
		    for (i = 1 ; i <= total_models ; i++) {
			 Cont = div(i,4);
			 if (Cont.rem == 1) fprintf(stdout,"\b\b/ ");
			 if (Cont.rem == 2) fprintf(stdout,"\b\b- ");
			 if (Cont.rem == 3) fprintf(stdout,"\b\b\\ ");
			 if (Cont.rem == 0) fprintf(stdout,"\b\b| ");
			 key_model = i;
			 model = MODELS+i ;
			 if (model->written == FALSE)
			      write_coords_pdb(rtto,6);
		    }
	       } else
		    write_coords_pdb(rtto,6);
	       fprintf(rtto,"ENDMDL\n");
	       fclose(rtto);
	  } else if (strncmp(palabra,"xplor",1)==0)  {
	       
	       /*
		* pdb xplor style
		*/
	       
	       /***************************************************************
       strcat(history[count_history]," pdb for xplor");
       if (flagall == 1)
       for (i = 1 ; i <= total_models ; i++) {
       key_model = i;
       check_label_pdb(3);
       write_coords_pdb_xplor(rtto);
       } else {
       check_label_pdb(3);
       write_coords_pdb_xplor(rtto);
       }
       fclose(rtto);
	       ******************************************************************/
	       printf("> COORD_writer: *NOT YET* developed .. ");
	  } else {
	       
	       /*
		* If everything fails
		*/
	       strcat(history[count_history],palabra);
	       strcat(history[count_history]," is * wrong typing buddy * ");
	       printf("> COORD_writer: * ---- Wrong typing BUDDY !! ----- *\n");
	       printf("> COORD_writer: * Option NOT recognized, type again please *\n");
	       goto coor_writer;
	  }
     }
     return;
}

