/* $Id: */

/*
 * Module to hold the routines responsible for removing meaningless constraints.
 * It relays on Distances.Lib where a schematic of sich distances is held. This
 * allows for some flexiblity as the user can edit and modify Distances.Lib to 
 * include new rules.
 *
 * Roberto Tejero, CABM, March 2003
 */

#include "pdbstat.h"
#define min(A,B) ((A) < (B) ? (A) : (B))

/* ========================================================================= */
char SameChain(struct noe *Noe)
{
	  /* Intra assignment */
	  if (strcmp(Noe->chain_id_1,Noe->chain_id_2) == 0)
		      return TRUE ;
	    else
		      return FALSE;
}

/* ========================================================================= */
char IsIntra(struct noe *Noe)
{
	  /* Intra assignment */
	  if (Noe->residue_number1 == Noe->residue_number2)
		      return TRUE ;
	    else
		      return FALSE;
}

/* ========================================================================= */
char IsSequential(struct noe *Noe)
{
	  /* Sequential assignment */
	  if ((int)abs(Noe->residue_number1 - Noe->residue_number2) == 1)
		      return TRUE ;
	    else
		      return FALSE;
}

/* ========================================================================= */
char IsLongRange(struct noe *Noe)
{
	  /* Long Range assignment */
	  if (abs(Noe->residue_number1 - Noe->residue_number2) > 1)
		      return TRUE ;
	    else
		      return FALSE;
}

/* ========================================================================= */
int GetLen(char Name[5]) {
  /*
   * Little routine to get the length of the string. In order to use * as 
   * a wildcard the calculation of the length stops when it hits a * in the 
   * string. It is  used by EliminateNoRestriction
   */
  static int i=0;
  static int Len = 0;

   Len = (int) strlen(Name) ;
   for (i=0; i <= (int)strlen(Name); i++)
	    if (Name[i] == '*' || Name[i] == '#') {
	          Len = i;
	          break ;
	    }
  return (int)Len;
}

/* ========================================================================= */
void ShowKnownDistances(struct KnownDist *kd) {
	static short No=0; 
	while ( kd->next != NULL ) {
		No++ ;
		fprintf(stdout,"  %4d  %-5.5s %-5.5s %-3.3s %lf %lf   %-13.13s %-13.13s\n",
		        No, kd->Atom1Name, kd->Atom2Name, kd->ResidueName,
		        kd->Dmin, kd->Dmax, kd->Comment1, kd->Comment2);
		kd++ ;
	}
	return ;
}
     
/* ========================================================================= */
void GetKnownDistances(FILE *InputFile)
{
  /*
   * Reading Library of known distances.
   * In the file being read (Distances.Lib) information about known
   * and fixed distances is stored. The procedure is designed so we
   * add more records to that file as we learn more known distances
   * and it can be used without recompiling the program.
   */
  static int   RecordN=0 ;
  static char  InputLine[MAXWORD] ;
  static char  TempChar[10] ;
  struct KnownDist *Curdis ;

  while (fgets(InputLine,MAXWORD,InputFile) != NULL) {
    if ((InputLine[0] != '#') && (strlen(InputLine) > 1)) {
      RecordN++ ;
      DistKnown = (struct KnownDist *)
	realloc(DistKnown,sizeof(struct KnownDist)*(RecordN+1));
 
      Curdis = DistKnown + RecordN ;
      Curdis->Record = RecordN ;
      sscanf(InputLine,"%s %s %s %lf %lf %s %s %s\n",
	     Curdis->Atom1Name, Curdis->Atom2Name, Curdis->ResidueName,
	     &Curdis->Dmin, &Curdis->Dmax, TempChar,
	     Curdis->Comment1, Curdis->Comment2);
      if (strncasecmp(TempChar,"Intra",1)==0) {
	Curdis->Intra = TRUE ;
	Curdis->Sequential = FALSE ;
      } else if (strncasecmp(TempChar,"Seque",1)==0) {
	Curdis->Intra = FALSE ;
	Curdis->Sequential = TRUE ;
      } else
	fprintf(stderr,"\n ** Error in Distances.Lib, Intra or Seque missing \n\n");
     Curdis->next = Curdis ; 
    }
  }
  MaxDistKnown = RecordN ;
  return ;
}

/* ========================================================================= */
int EliminateDuplicatedNoes(struct noe *Noe) {
	/*
	 * Finds duplicated (same noe restraint with different distance 
	 * and removes them (well, avoids to write them without actually
	 * delete them
	 */

 static struct noe *NoeTmp ; 
 static int NoeCntDuplicated=0 ;

 NoeCntDuplicated = 0 ;
 while ( Noe->next != NULL ) {
	 if ( !Noe->ToPrint ) {
		 Noe++; 
		 continue ;
	 }
	 NoeTmp = (Noe+1) ;
 	while ( NoeTmp->next != NULL ) {
		if ( !NoeTmp->ToPrint ) {
			NoeTmp++ ;
			continue ;
		}
	        if (Noe->residue_number1 == NoeTmp->residue_number1 && Noe->residue_number2 == NoeTmp->residue_number2 &&
	            strcmp(Noe->atom_type1, NoeTmp->atom_type1)==0  && strcmp(Noe->atom_type2, NoeTmp->atom_type2)==0    &&
	            strcmp(Noe->segid_1, NoeTmp->segid_1)==0        && strcmp(Noe->segid_2, NoeTmp->segid_2)==0) {

			/* Take minimum distance and comment duplicate record */
			Noe->upper  = min(Noe->upper, NoeTmp->upper); 
			Noe->lower  = min(Noe->lower, NoeTmp->lower); 
			Noe->volume = min(Noe->volume,NoeTmp->volume); 
			Noe->equil  = min(Noe->equil, NoeTmp->equil); 
			Noe->rglow  = min(Noe->rglow, NoeTmp->rglow); 
			Noe->rgupp  = min(Noe->rgupp, NoeTmp->rgupp); 

			/* Delete duplicated */
			NoeTmp->ToPrint = FALSE ;
			sprintf(NoeTmp->comment," Duplicated constraint with %4d", Noe->Number); 

			/* Issue message */
			if ( DEBUG >= 1 )
		        	fprintf(stdout,"%5d-> %3s %2s %3d %-4s %3s %2s %3d %-4s [%5.2f %5.2f] -- %s \n",
				    NoeTmp->Number, 
				    NoeTmp->residue_type1, NoeTmp->chain_id_1, NoeTmp->residue_number1, NoeTmp->atom_type1,
	                            NoeTmp->residue_type2, NoeTmp->chain_id_2, NoeTmp->residue_number2, NoeTmp->atom_type2,
				    NoeTmp->lower, NoeTmp->upper, NoeTmp->comment );
			NoeCntDuplicated++ ;
		}
		NoeTmp++ ;
	}
	Noe++ ;
 }
 /* Dump final results */
 if ( DEBUG > 1 ) 
 	fprintf(stdout," > EliminateDuplicatedNoes():  %4d duplicated  restraints \n", NoeCntDuplicated);
 return ( NoeCntDuplicated );
}



/* ========================================================================= */
int EliminateNoRestriction(struct noe *Noe) {
     
 struct KnownDist *Known;
 static int  Len1, Len2;
 static int NoeCntMeaningless=0 ; 


 NoeCntMeaningless = 0;
 while ( Noe->next != NULL ) {
      if ( SameChain(Noe) && IsIntra(Noe) && Noe->ToPrint ) {
	   Known = &DistKnown[1];
	   while ( Known->next != NULL )  {
		if ( Known->Intra ) {
		     Len1 = GetLen(Known->Atom1Name);
		     Len2 = GetLen(Known->Atom2Name);
		     if (((strncmp(Noe->Atom1Diana,Known->Atom1Name,Len1)==0) && 
			  (strncmp(Noe->Atom2Diana,Known->Atom2Name,Len2)==0)) ||
			 ((strncmp(Noe->Atom2Diana,Known->Atom1Name,Len1)==0) && 
			  (strncmp(Noe->Atom1Diana,Known->Atom2Name,Len2)==0))) {
			  if ( (strcmp(Noe->residue_type1,Known->ResidueName)==0) ||
			       (strcmp(Known->ResidueName,"***")==0) ) {
			       if (Noe->upper > Known->Dmax) {
				    Noe->ToPrint = FALSE;
				    NoeCntMeaningless++ ;
				    if ( DEBUG >= 1 )
				    	fprintf(stdout,"%5d-> %3s %2s %3d %-4s %3s %2s %3d %-4s [%5.2f %5.2f] -- %s I %.2f[%.2f]\n",
						    Noe->Number, 
						    Noe->residue_type1, Noe->chain_id_1, Noe->residue_number1, Noe->atom_type1,
	                                            Noe->residue_type2, Noe->chain_id_2, Noe->residue_number2, Noe->atom_type2,
						    Noe->lower, Noe->upper, Known->Comment1, Noe->upper, Known->Dmax);
			       } else if (Noe->upper < Known->Dmin) {
			            Noe->ToPrint = FALSE ;
				    NoeCntMeaningless++ ;
				    if ( DEBUG >= 1 )
				    	fprintf(stdout,"%5d-> %3s %2s %3d %-4s %3s %2s %3d %-4s [%5.2f %5.2f] -- %s \n",
						    Noe->Number, 
						    Noe->residue_type1, Noe->chain_id_1, Noe->residue_number1, Noe->atom_type1,
	                                            Noe->residue_type2, Noe->chain_id_2, Noe->residue_number2, Noe->atom_type2,
						    Noe->lower, Noe->upper, Known->Comment2);
			       }
			       break ;
			  }
		     }
		}
		Known++;
	   }
      }
      if ( SameChain(Noe) && IsSequential(Noe) && Noe->ToPrint ) {
	      Known = &DistKnown[1];
	      while ( Known->next != NULL ) {
		Len1 = GetLen(Known->Atom1Name);
		Len2 = GetLen(Known->Atom2Name);
		if ( Known->Sequential ) {
		     if ( strncmp(Noe->Atom1Diana,Known->Atom1Name,Len1)==0 && 
			  strncmp(Noe->Atom2Diana,Known->Atom2Name,Len2)==0 ) {
			  if ( (strcmp(Noe->residue_type2,Known->ResidueName)==0) ||
			       (strcmp(Known->ResidueName,"***")==0) ) {
			       if (Noe->upper > Known->Dmax) {
				    Noe->ToPrint = FALSE;
				    NoeCntMeaningless++ ;
				    if ( DEBUG >= 1 )
				   	 fprintf(stdout,"%5d-> %3s %2s %3d %-4s %3s %2s %3d %-4s [%5.2f %5.2f] -- %s I %.2f[%.2f] \n",
						    Noe->Number, 
						    Noe->residue_type1, Noe->chain_id_1, Noe->residue_number1, Noe->atom_type1,
	                                            Noe->residue_type2, Noe->chain_id_2, Noe->residue_number2, Noe->atom_type2,
						    Noe->lower, Noe->upper, Known->Comment1, Noe->upper, Known->Dmax);
			       } else if (Noe->upper < Known->Dmin) { 
				    Noe->ToPrint = FALSE;
				    NoeCntMeaningless++ ;
				    if ( DEBUG >= 1 )
				    	fprintf(stdout,"%5d-> %3s %2s %3d %-4s %3s %2s %3d %-4s [%5.2f %5.2f] -- %s \n",
						    Noe->Number, 
						    Noe->residue_type1, Noe->chain_id_1, Noe->residue_number1, Noe->atom_type1,
	                                            Noe->residue_type2, Noe->chain_id_2, Noe->residue_number2, Noe->atom_type2,
						    Noe->lower, Noe->upper, Known->Comment2);
			       }
			       break ;
			  }
		     }
		}
		Known++ ;
	   }
      }
      Noe++ ;
 }
 /* Dump final results */
 if ( DEBUG > 1) 
 	fprintf(stdout," > EliminateNoRestriction() :  %4d meaningless restraints \n", NoeCntMeaningless);
 return (NoeCntMeaningless);
}

/* ========================================================================= */
void EliminateNoeSelected(struct noe *Noe, char Type[MAXWORD]) {
     
 static char Intra, Sequential, LongRange ;

 Intra = Sequential = LongRange = FALSE ;

 if (DEBUG > 0) 
	 fprintf(stdout," EliminateNoeSelected(): Eliminating, %s\n",Type);

 if ( strncasecmp(Type,"intra",2)==0 ) {
	 Intra = TRUE ;
	 LongRange = FALSE ;
	 Sequential = FALSE ;
 } else if ( strncasecmp(Type,"sequential",2)==0 ) {
	 Sequential = TRUE ;
	 Intra = FALSE ;
	 LongRange = FALSE ;
 } else if ( strncasecmp(Type,"LongRange",2)==0 ) {
	 LongRange = TRUE ;
	 Intra = FALSE ;
	 Sequential = FALSE ;
 }
     
  while ( Noe->next != NULL ) {
      if ( SameChain(Noe) && IsIntra(Noe) && Intra ) {
	      Noe->ToPrint = FALSE ;
 	      if (DEBUG > 0) 
	 		fprintf(stdout," EliminateNoeSelected(): Eliminating intra\n");

      } 
      if ( SameChain(Noe) && IsSequential(Noe) && Sequential ) {
	      Noe->ToPrint = FALSE ;
 	      if (DEBUG > 0) 
	 		fprintf(stdout," EliminateNoeSelected(): Eliminating sequential\n");

      }
      if ( SameChain(Noe) && IsLongRange(Noe) && LongRange ) {
	      Noe->ToPrint = FALSE ;
 	      if (DEBUG > 0) 
	 		fprintf(stdout," EliminateNoeSelected(): Eliminating long range\n");

      }
      Noe++ ;
 }
 return ;
}
