/*$Id: ProcessAnyAtom.c,v 1.6 2003/10/06 12:13:32 roberto Exp roberto $*/

#include "pdbstat.h"

extern void FromDianaToIUPAC(char Result[6], char Target[6], char ResNam[5]) ;

/* ========================================================================= */
/*
 * Map any atom to an index inside model coords.
 * An attempt to have a map from a residue number and name to an index inside
 * the model, the index points to the atom it is called, iat.
 */
void iatNoe(struct noe *N) {
	static struct models *md ;
	static struct atom_pdb *at ;
	static int  iat, Len1, Len2, cnt1, cnt2;
	static char np1[6], np2[6];
	static char tmprD[MAXLINE], tmpr1[MAXLINE], tmpr2[MAXLINE];

	md = MODELS + 1;

	if ( md->number_of_atoms <= 0 ) {
		return ;
	}
	/* Fix names to our convention inside PDBStat basically adding a white
	 * space at the beginning and adapting Q atoms to our convention */
     	FromDianaToIUPAC(np1,N->atom_type1,seqres[N->residue_number1]);
     	FromDianaToIUPAC(np2,N->atom_type2,seqres[N->residue_number2]);

	np1[5] = np2[5] = '\0' ;

	Len1 = Len2 = iat = 0 ;
	cnt1 = cnt2 = 0 ;
	while ( np1[Len1] != '\0' && np1[Len1] != '*' && np1[Len1] != '#' ) {
		Len1++ ;
	}
	while ( np2[Len2] != '\0' && np2[Len2] != '*' && np2[Len2] != '#' ) {
		Len2++ ;
	}
	for (iat = 1; iat <= md->number_of_atoms; iat++) {
		at = (md->PDB) + iat ;
		if ( np1[4] == '*' || np1[4] == '#' )
			np1[0] = at->pdb_iupac[0] ;
		if ( np1[3] == '*' || np1[3] == '#' )
			np1[0] = at->pdb_iupac[0] ;
		if ( np1[2] == '*' || np1[2] == '#' )
			np1[0] = at->pdb_iupac[0] ;
		
		if ( np2[4] == '*' || np2[4] == '#' )
			np2[0] = at->pdb_iupac[0] ;
		if ( np2[3] == '*' || np2[3] == '#' )
			np2[0] = at->pdb_iupac[0] ;
		if ( np2[2] == '*' || np2[2] == '#' )
			np2[0] = at->pdb_iupac[0] ;

		if ((at->pdb_num_res == N->residue_number1) && (strncasecmp(at->pdb_iupac,np1,Len1)==0)) { 
		 	N->iat1[++cnt1]=iat ;
		} 
		if ((at->pdb_num_res == N->residue_number2) && (strncasecmp(at->pdb_iupac,np2,Len2)==0)) { 
		 	N->iat2[++cnt2]=iat ;
		} 
	}
	N->iat1[0] = cnt1 ;
	N->iat2[0] = cnt2 ;

	/* Offer some debug printing for the time being */
	if ( DEBUG > 0 ) {
		sprintf(tmpr1,"    iatNoe(): atom/s [%-4s %s %3d], iat1 -> ", 
				N->atom_type1, seqres[N->residue_number1], N->residue_number1);
		for ( cnt1 = 1; cnt1 <= N->iat1[0]; cnt1++) {
			sprintf(tmprD," %4d", N->iat1[cnt1]);
		        strcat(tmpr1,tmprD);
		}

		sprintf(tmpr2," atom/s [%-4s %s %3d], iat2 -> ", 
				N->atom_type2, seqres[N->residue_number2], N->residue_number2);
		for ( cnt2 = 1; cnt2 <= N->iat2[0]; cnt2++) {
			sprintf(tmprD," %4d", N->iat2[cnt2]);
			strcat(tmpr2,tmprD);
		}
		
		fprintf(stdout," %-80s %-80s\n", tmpr1, tmpr2);
	}
	return ;
}


/* ========================================================================= */
/* Prepare an atom for further processing in distances */		
void ProcessAnyAtom(char Uno, char ResTarget[5], char AtTarget[6], int Cnt, struct models *model) {
     static int mcn ;
     
     if ( ResAt == NULL )
	  ResAt = (struct ResAtom *)malloc(sizeof(struct ResAtom));
     
     for (mcn = 0; mcn < 6; mcn++) 
	     ResAt->Label[mcn]='\0';
     
     /* Select residue */
     if (strcmp(ResTarget,"*")==0) {
	  ResAt->Number = model->PDB[Cnt].pdb_num_res; 
     } else if (strcmp(ResTarget,"#")==0)  {
	  if ( Uno ) 
	       ResAt->NumberOne = model->PDB[Cnt].pdb_num_res ;
	  ResAt->Number = ResAt->NumberOne;
     } else if (strcmp(ResTarget,"i1")==0) {
	  ResAt->Number = ResAt->NumberOne + 1;
     } else {
	  if ( isalpha(ResTarget[0]) ){
	       ResTarget[0] = ' ';
	  }
	  ResAt->Number = atoi(ResTarget);
     }
     
     /* Select atom name */
     if (AtTarget[0]=='*') {                 
	  strcpy(ResAt->Label,model->PDB[Cnt].pdb_iupac);
	  ResAt->Len = 4 ;
     } else {
	  if ( !isdigit(AtTarget[0]) ) {
	       ResAt->Label[0] = ' ';
	       strcat(ResAt->Label,AtTarget);
	  } else
	       for (mcn = 0; mcn <= strlen(AtTarget); mcn++) ResAt->Label[mcn]=AtTarget[mcn];
	  ResAt->Len = 0 ;
	  for (mcn = 0 ; mcn < 4 ; mcn++) {
	       if ( ResAt->Label[mcn] == '*') {
		    ResAt->Len = mcn ;
	       } else if ( ResAt->Label[mcn]=='\0')
		    ResAt->Label[mcn] = ' ';
	  }
	  ResAt->Label[4]='\0' ;
	  if (ResAt->Len == 0) ResAt->Len = 4;
     }
     if (((ResAt->Label[0]==' ') && ((ResAt->Label[1]=='H')||(ResAt->Label[1]=='h'))) && 
	 ((ResAt->Label[2]=='*') || (ResAt->Label[3]=='*')))
	  ResAt->Label[0] = model->PDB[Cnt].pdb_iupac[0];
     
     if ( DEBUG > 0 )
	  fprintf(stdout, " OrigAt: `%s' %6d  -- ResAt: `%s' %6d %6d,  Index %6d \n", 
		  model->PDB[Cnt].pdb_iupac, model->PDB[Cnt].pdb_num_res,
		  ResAt->Label, ResAt->Number, ResAt->Len, Cnt ); 
     
     return ;
}
