/*$Id: align_sequences.c,v 3.32 2003/10/06 12:13:32 roberto Exp roberto $*/

#include "pdbstat.h"
extern void classify_by_sequence(void) ;
void ToOne(void) ;

static short  i,j,k ;
static char   **alinea, **alinea_tmp ;
static struct models *Model ;
static int    origen_align ;

/*
 * align_sequences
 * Very crude and simple routine to align sequences. It tries to get
 * maximum identity between two sequences not taking into account insertions
 * nor deletions.
 *
 * It was designed for a little play but it is being of great interest and 
 * use in some applications. -- Valencia, March 2000
 */
void align_sequences(char tt[MAXWORD]) 
{
     static int    TmpLen, Inicio ;
     static int    coincidencias, temporder, maximo;
     static int    desplaza, desplazamax;
     
     Model = MODELS ;
     
     coincidencias = 0;
     temporder = 0;
     maximo = 0 ;
     desplaza = 0 ;  
     desplazamax = 0 ;
     
     /*
      * Classification of the models in function of sequence lentgh.
      * Model with longer sequence is going to be the first.
      */
     classify_by_sequence();
     
     origen_align = (int)((MODELS+1)->number_of_residues/2)+1 ;
     TmpLen = (2 * origen_align) + (MODELS+1)->number_of_residues ;
     
     /* Dinamyc allocation of memory for alignments */
     alinea      =  (char **)malloc(sizeof(char) * 900) ;
     alinea_tmp  =  (char **)malloc(sizeof(char) * 900) ;
     for (i = 0 ; i <= total_models ; i++) {
	  alinea[i]     =  (char *)malloc(sizeof(char) * 1200) ;
	  alinea_tmp[i] =  (char *)malloc(sizeof(char) * 1200) ;
     }
     
     /* Initialization of sequences */
     for (i = 0 ; i <= total_models ; i++) {
	  for (k = 0 ; k <= TmpLen-1 ; k++)  
	       alinea[i][k]=' ';
     }
     
     /*
      * Change residue names to ONE letter and print original sequences
      * on screen
      */
     fprintf(stdout,"\n\n ----  Original Sequences ---- \n");
     ToOne();
     
     fprintf(stdout,"\n ----  After alignment ---- \n");
     
     /*
      * Align procedure starts.
      * The procedure is really simple, only looking for identity in the sequence
      * and even this is done really simpleminded with no smart algorithm at all.
      * We try to align the sequences in different lentghs and measure the 
      * number of identities, that's it.
      */
     for (i = 1 ; i <= total_models ; i++) {
	  Model = MODELS + i;
	  temporder    = 0 ;
	  desplazamax  = 0 ;
	  
	  /* Start displacement of the sequence to see how many matches we get.  */
	  /* This only gives the sequence identity between both sequences */
	  for (desplaza = 0 ;  
	       desplaza < (TmpLen-(MODELS+i)->number_of_residues)-1 ;  
	       desplaza++) {
	       coincidencias = 0 ;
	       
	       for (k = 0 ; k <= Model->number_of_residues ; k++)
		    alinea_tmp[i][desplaza+k]=alinea[i][k+origen_align+1];
	       
	       for (maximo = 0 ; maximo <= (desplaza - 1) ;  maximo++)
		    alinea_tmp[i][maximo]=' ';
	       
	       alinea_tmp[i][Model->number_of_residues+desplaza+1]='\0' ;
	       
	       for (j = 1 ; j <= (int)strlen(alinea[1]) ; j++) { 
		    if ( ( alinea[1][j] == alinea_tmp[i][j] ) &&
			 ( alinea[1][j] != ' ') )
			 coincidencias++ ;
		    if (coincidencias > temporder) {
			 temporder = coincidencias ;
			 desplazamax = desplaza ;
		    }
	       }
	  }
	  
	  for (k = 0 ; k <= (int)strlen(alinea[i]) ; k++) 
	       alinea_tmp[i][desplazamax+k] = alinea[i][k+origen_align] ;
	  
	  for (maximo = 0 ; maximo <= desplazamax ; maximo++) 
	       alinea_tmp[i][maximo] = ' ';
	  
	  alinea_tmp[i][origen_align+desplazamax+Model->number_of_residues+1] = '\0';
	  
	  fprintf(stdout," Sequential Identity: %d/%d\n", 
		  temporder-1,Model->number_of_residues); 
     }
     
     if (strncmp(tt,"debug",3)==0)
	  fprintf(stdout,"1234|6789|1234|6789|1234|6789|1234|6789|1234|6789|\n");
     
     /* Locate start of the sequence with first character different of blank */
     Inicio = 99999 ;
     for (i = 1 ; i <= total_models ; i++) 
	  for (j=0;j<=(int)strlen(alinea_tmp[i]); j++) {
	       if ( alinea_tmp[i][j] != ' ' ) {
		    if (j < Inicio) {
			 Inicio = j;
			 break;
		    }
	       }
	  }
     
     /* Convert non coincident places to dashes to get a readable output */
     for (j=Inicio; j<=(int)strlen(alinea_tmp[1]); j++)
	  for (i=2; i<=total_models; i++) {
	       if (alinea_tmp[i][j] == '\0') break ;
	       if (alinea_tmp[i][j] != alinea_tmp[1][j]) {
		    alinea_tmp[i][j]='-';
	       }
	  }
     
     /* Write sequence data to screen */
     for (i = 1 ; i <= total_models ; i++) {
	  fprintf(stdout," %3d - ", (MODELS+i)->modelno);
	  for (j=Inicio; j<=(int)strlen(alinea_tmp[i]); j++)
	       fprintf(stdout,"%c", alinea_tmp[i][j]);
	  fprintf(stdout,"\n");
     }
     
     /* Free allocated memory */
     free(alinea); 
     free(alinea_tmp); 
     
     return ;
}


/*=======================================================================*/

/*
 * ToOne
 * Convert residue names in three leter code to one letter not destructively, 
 * i.e. if the chain has non standard aminos just give a warning message
 * but do not stop the calculation.
 * Some of the aminos here are from special libraries like the one used 
 * by ECEPPAK (ECEPP). If the amino is special or unknown it gets 'U' assigned
 * as the one letter code.
 */
void ToOne(void) {
     for (i = 1 ; i <= total_models ; i++) {
	  Model = MODELS + i ;
	  fprintf(stdout," %3d - ", Model->modelno);
	  for (j = 1 ; j <= Model->number_of_residues ; j++) {
	       if (strncmp(Model->DIH[j].resname,"ALA",3)==0)  {
		    alinea[i][origen_align+j]='A';
	       } else if (strncmp(Model->DIH[j].resname,"ARG",3)==0)  {
		    alinea[i][origen_align+j]='R';
	       } else if (strncmp(Model->DIH[j].resname,"ASN",3)==0)  {
		    alinea[i][origen_align+j]='N';
	       } else if (strncmp(Model->DIH[j].resname,"ASP",3)==0)  {
		    alinea[i][origen_align+j]='D';
	       } else if (strncmp(Model->DIH[j].resname,"ASX",3)==0)  {
		    alinea[i][origen_align+j]='D';
	       } else if (strncmp(Model->DIH[j].resname,"CYS",3)==0)  {
		    alinea[i][origen_align+j]='C';
	       } else if (strncmp(Model->DIH[j].resname,"GLN",3)==0)  {
		    alinea[i][origen_align+j]='Q';
	       } else if (strncmp(Model->DIH[j].resname,"GLU",3)==0)  {
		    alinea[i][origen_align+j]='E';
	       } else if (strncmp(Model->DIH[j].resname,"GLX",3)==0)  {
		    alinea[i][origen_align+j]='E';
	       } else if (strncmp(Model->DIH[j].resname,"GLY",3)==0)  {
		    alinea[i][origen_align+j]='G';
	       } else if (strncmp(Model->DIH[j].resname,"HIS",3)==0)  {
		    alinea[i][origen_align+j]='H';
	       } else if (strncmp(Model->DIH[j].resname,"HIE",3)==0)  {   /* ECEPP */
		    alinea[i][origen_align+j]='H';
	       } else if (strncmp(Model->DIH[j].resname,"ILE",3)==0)  {
		    alinea[i][origen_align+j]='I';
	       } else if (strncmp(Model->DIH[j].resname,"LEU",3)==0)  {
		    alinea[i][origen_align+j]='L';
	       } else if (strncmp(Model->DIH[j].resname,"LYS",3)==0)  {
		    alinea[i][origen_align+j]='K';
	       } else if (strncmp(Model->DIH[j].resname,"MET",3)==0)  {
		    alinea[i][origen_align+j]='M';
	       } else if (strncmp(Model->DIH[j].resname,"PHE",3)==0) {
		    alinea[i][origen_align+j]='F';
	       } else if (strncmp(Model->DIH[j].resname,"PRO",3)==0)  {
		    alinea[i][origen_align+j]='P';
	       } else if (strncmp(Model->DIH[j].resname,"PRU",3)==0)  {  /*ECEPP.PRO UP*/
		    alinea[i][origen_align+j]='P';
	       } else if (strncmp(Model->DIH[j].resname,"SER",3)==0)  {
		    alinea[i][origen_align+j]='S';
	       } else if (strncmp(Model->DIH[j].resname,"THR",3)==0)  {
		    alinea[i][origen_align+j]='T';
	       } else if (strncmp(Model->DIH[j].resname,"TRP",3)==0)  {
		    alinea[i][origen_align+j]='W';
	       } else if (strncmp(Model->DIH[j].resname,"TYR",3)==0)  {
		    alinea[i][origen_align+j]='Y';
	       } else if (strncmp(Model->DIH[j].resname,"VAL",3)==0)  {
		    alinea[i][origen_align+j]='V';
	       } else if (strncmp(Model->DIH[j].resname,"NCC",3)==0)  { /* ECEPP */
		    alinea[i][origen_align+j]='U';
	       } else if (strncmp(Model->DIH[j].resname,"ACE",3)==0)  { /* ECEPP */
		    alinea[i][origen_align+j]='U';
	       } else {
		    /*
		     * We store unknown residues as 'U'. Look into sequence to find
		     * them. The storage is made to have align working properly even if
		     * a residue in the sequence is special or unknown.
		     */
		    alinea[i][origen_align+j]='U';
		    fprintf(stderr,"\n * Residue _UNKNOWN_ in sequence: %s\n",
			    Model->DIH[j].resname);
	       }
	  }
	  alinea[i][Model->number_of_residues+origen_align+1]='\0' ;
	  for (k = origen_align+1; k<=(int)strlen(alinea[i]); k++)
	       fprintf(stdout,"%c", alinea[i][k]);
	  fprintf(stdout,"\n");
	  strcpy(alinea_tmp[i],alinea[i]); 
     }
     return ;
}
