/*$Id: analyze_constraints.c,v 3.32 2003/10/06 12:13:32 roberto Exp roberto $*/

#include "pdbstat.h"
extern void do_codes_for_constraints(void);
extern FILE *safeopen(char *file, char *mode, int longit) ;

/* ========================================================================= */
void write_analysis(void)
/*
  --- Mar, 2002. HBOND calc corrected
  ---
  --- Jan, 1997. Code revisited and checked. Several bugs fixed
  ---
  ---  o  The way H-BOND and DISULFIDE constraints were calculated was wrong.
  ---     The if for selection was N *OR* O instead of N *AND* O. Look down 
  ---     for a more clear explanation.
  ---
  --- This routine has been written down to make the analysis of a given
  --- set of constraints. It should classify the constraints and make a
  --- little statistical analysis. The important point here is that also 
  --- produces an output called KYCNS.OUT needed if you want to use the
  --- program 'violaciones' (violaciones.f). This program after reading
  --- KYCNS.OUT is able to produce an elaborated output with data sorted
  --- in a table about mean violation, violation/residue, etc.
  ---
  --- (Roberto Tejero. RTT, May 1993, CABM-Rutgers University).
  ---
*/
{
     static unsigned short  **key_NOE;
     static int  *numberNOE;
     static unsigned short  *consres_intra;
     static float  *consres_inter, *flag;
     static int    new_kount_dist, ResCt;
     static int    IntraChain, InterChain ;
     static int    sequential, sequential_bckbck, sequential_bcksdc, sequential_sdcsdc;
     static int    medium_r, medium_bckbck, medium_bcksdc, medium_sdcsdc;
     static int    long_r, intra_residue, hydrogen_bond, hbond_long, disulfide_cons;
     static float  check_total;
     static char   t[MAXWORD];
     static unsigned short    i, k ;
     static FILE   *fout, *fkycns, *fcons;
     static struct noe *Noe ;
     static struct models *Model0 ;
     
     Model0 = MODELS ;
     
     /*
      * Checking that we have the needed info to run this module
      */
     if (!sequence_true) {
	  fprintf(stdout,"> ANALYZE_cns: ** First you need to read a sequence file \n");
	  fprintf(stdout,"> ANALYZE_cns: ** for this molecule \n");
	  return;
     }
     
     /*  Open file KYCNS.OUT */
     fkycns = safeopen("KYCNS.OUT","w",(int)strlen("KYCNS.OUT"));
     if (DEBUG > 0) 
	     fprintf(stdout," > ANALYZE_cns: KYCNS.OUT file opened\n");
     fprintf(fkycns,"  RES  NCNS            KYCNS\n");
     
     /* USEFUL.DATA */
     strcpy(output_name,name_of_the_file_constr);
     strcat(output_name,"_USEFUL_DATA\0");
     if (DEBUG > 0) 
	     fprintf(stdout," > ANALYZE_cns: Calling safeopen\n");
     fout = safeopen(output_name,"w",(int)strlen(output_name));
     
     /* Allocate space for data */
     if (Model0->number_of_residues <= 0) {
	  fprintf(stderr," ** analyze_constraints(): Trying to allocate with _ZERO_ residues \n");	  
	  fprintf(stderr," ** analyze_constraints(): Try to read a sequence file first       \n");
	  return ;
     }
     
     if ( Model0->starting_res > 1 )
	  ResCt = Model0->number_of_residues + Model0->starting_res - 1 ;
     else
	  ResCt = Model0->number_of_residues ;
     
     key_NOE = (unsigned short **) malloc(sizeof(unsigned short *) * (ResCt+2));
     for (i = 0; i <= ResCt+1; i++)
	  key_NOE[i] = (unsigned short *) malloc(sizeof(unsigned short)*(kount_dist+1));
     
     flag = (float *) malloc(sizeof(float) * (kount_dist+1));
     consres_intra = (unsigned short *) malloc(sizeof(unsigned short) * (ResCt+1));
     consres_inter = (float *) malloc(sizeof(float) * (ResCt+1));
     numberNOE = (int *) malloc(sizeof(int) * (ResCt+1));
     
     /* Initialize values */
     for (i = 0; i <= ResCt; i++) {
	  *(numberNOE+i)=0;
	  *(consres_intra+i)=0;
	  *(consres_inter+i)=0.0;
     }
     
     sequential = 0;
     medium_r  = 0;
     intra_residue  = 0;
     sequential_bckbck  = 0;
     sequential_bcksdc  = 0;
     sequential_sdcsdc  = 0;
     medium_bckbck   = 0;
     medium_bcksdc   = 0;
     medium_sdcsdc   = 0;
     hydrogen_bond  = 0;
     hbond_long     = 0;
     disulfide_cons = 0;
     check_total    = 0;
     long_r        = 0;
     InterChain     = 0;
     IntraChain     = 0;
     
     /* Let's start */
     Noe = NOE ;
     
     for (i = 1 ; i <= kount_dist  ; i++)  {
	  *(numberNOE+(Noe+i)->residue_number1) += 1 ;
	  
	  if ((Noe+i)->residue_number1 != (Noe+i)->residue_number2) 
	       *(numberNOE+(Noe+i)->residue_number2) += 1 ;
	  
	  key_NOE[(Noe+i)->residue_number1][numberNOE[(Noe+i)->residue_number1]] = i;
	  
	  if ((Noe+i)->residue_number1 != (Noe+i)->residue_number2) 
	       key_NOE[(Noe+i)->residue_number2][numberNOE[(Noe+i)->residue_number2]]= i;
	  
	  if ((Noe+i)->residue_number1 != (Noe+i)->residue_number2) 
	       *(flag+i)=0.5;     
	  
	  if ((Noe+i)->residue_number1 == (Noe+i)->residue_number2) 
	       *(flag+i)=1.0;
	  
	  /*
	   * Inter chain constraints in cases of x-mers (dimers, trimers etc.
	   */
	  if ( strcmp((Noe+i)->chain_id_1,(Noe+i)->chain_id_2) != 0 )
	       InterChain++ ;
	  else 
	       IntraChain++ ;

	  /*
	   * Hydrogen bond constraints. The code was broken, sometime in the past the code
	   * was discarded, so I decided to include it again
	   * Aneerban pointed out that some files have hbonds written the other way around, i.e.
	   * giving first HN or N and then the O (fixed on Sep 2002), thanks Aneerban.
	   * RTT, CABM -- Rutgers University 2002.
	   */
	  if ((strcasecmp((Noe+i)->atom_type1,"O") == 0) && (strcasecmp((Noe+i)->atom_type2,"HN") == 0))  {
	       hydrogen_bond++;
	       if (fabs((Noe+i)->residue_number1-(Noe+i)->residue_number2) >= 5.0) hbond_long++ ;
	       goto fin;
	  }
	  
	  if ((strcasecmp((Noe+i)->atom_type1,"O") == 0) && (strcasecmp((Noe+i)->atom_type2,"N") == 0))  {
	       hydrogen_bond++;
	       if (fabs((Noe+i)->residue_number1-(Noe+i)->residue_number2) > 5) hbond_long++ ;
	       goto fin;
	  }
	  /* Some constraint files are written the other way around with N data on first record */
	  if ((strcasecmp((Noe+i)->atom_type1,"HN") == 0) && (strcasecmp((Noe+i)->atom_type2,"O") == 0))  {
	       hydrogen_bond++;
	       if (fabs((Noe+i)->residue_number1-(Noe+i)->residue_number2) > 5) hbond_long++ ;
	       goto fin;
	  }
	  
	  if ((strcasecmp((Noe+i)->atom_type1,"N") == 0) && (strcasecmp((Noe+i)->atom_type2,"O") == 0))  {
	       hydrogen_bond++;
	       if (fabs((Noe+i)->residue_number1-(Noe+i)->residue_number2) > 5) hbond_long++ ;
	       goto fin;
	  }
	  
	  /*
	   * DISULFIDE constraints (a jump to the end of the loop is done)
	   * No further classification is done
	   *
	   * Here before was (*OR*) || instead of (*AND*) && in the if 
	   * That was a mistake for selecting DISULFIDE constraints
	   * it should be: 
	   *               residue 1 SG *AND* residue 2 SG.
	   */
	  if ((strcasecmp((Noe+i)->atom_type1,"SG") == 0) && (strcasecmp((Noe+i)->atom_type2,"SG") == 0))  {
	       disulfide_cons++;
	       goto fin;
	  }
	  
	  /*
	   * Intra residue (same residue number) count as ONE constraint 
	   * for residue.
	   */
	  if ((Noe+i)->residue_number1 == (Noe+i)->residue_number2)  {
	       intra_residue++;
	       *(consres_intra+(Noe+i)->residue_number1) += 1;
	  }
	  
	  /*
	   * Inter residue between 1 and 5 residues apart count as HALF constraint
	   * for each residue involved.
	   */
	  if ((fabs((Noe+i)->residue_number1-(Noe+i)->residue_number2) >= 1.0) &&
	      (fabs((Noe+i)->residue_number1-(Noe+i)->residue_number2) < 5.0)) {
	       *(consres_inter+(Noe+i)->residue_number1) += 0.5 ;
	       *(consres_inter+(Noe+i)->residue_number2) += 0.5 ;
	  }
	  
	  /*
	   * Inter residue greater than five residues apart count as HALF constraint
	   * for each residue involved.
	   */
	  if (fabs((Noe+i)->residue_number1-(Noe+i)->residue_number2) >= 5.0)  {
	       long_r++;
	       *(consres_inter+(Noe+i)->residue_number1) += 0.5 ;
	       *(consres_inter+(Noe+i)->residue_number2) += 0.5 ;
	  }
	  
	  /*
	   * Classification in 
	   *         backbone-backbone
	   *         backbone-sidechain
	   *         sidechain-sidechain
	   * Inter residue. ONE residue apart
	   */
	  if (fabs((Noe+i)->residue_number1 - (Noe+i)->residue_number2) == 1.0)  {
	       sequential++;
	       if ((strcasecmp((Noe+i)->atom_type1,"N") == 0) || (strcasecmp((Noe+i)->atom_type1,"CA") == 0) ||
		   (strcasecmp((Noe+i)->atom_type1,"C") == 0) || (strcasecmp((Noe+i)->atom_type1,"O") == 0) ||
		   (strcasecmp((Noe+i)->atom_type1,"HN") == 0) || (strcasecmp((Noe+i)->atom_type1,"H") == 0) ||
		   (strcasecmp((Noe+i)->atom_type1,"HA") == 0))
		    if ((strcasecmp((Noe+i)->atom_type2,"N") == 0) || (strcasecmp((Noe+i)->atom_type2,"CA") == 0) ||
			(strcasecmp((Noe+i)->atom_type2,"C") == 0) || (strcasecmp((Noe+i)->atom_type2,"O") == 0) ||
			(strcasecmp((Noe+i)->atom_type2,"HN") == 0) || (strcasecmp((Noe+i)->atom_type2,"H") == 0) ||
			(strcasecmp((Noe+i)->atom_type2,"HA") == 0))
			 sequential_bckbck = sequential_bckbck+1;
		    else
			 sequential_bcksdc = sequential_bcksdc+1;
	       else 
		    sequential_sdcsdc = sequential_sdcsdc+1;
	  }
	  
	  /*
	   * Classification in 
	   *         backbone-backbone
	   *         backbone-sidechain
	   *         sidechain-sidechain
	   *
	   * Inter residue. Between 1 and 5 residues apart.
	   */
	  if ((fabs((Noe+i)->residue_number1 - (Noe+i)->residue_number2) > 1.0) &&
	      (fabs((Noe+i)->residue_number1 - (Noe+i)->residue_number2) < 5.0))  {
	       medium_r++;
	       if ((strcasecmp((Noe+i)->atom_type1,"N") == 0) || (strcasecmp((Noe+i)->atom_type1,"CA") == 0) ||
		   (strcasecmp((Noe+i)->atom_type1,"C") == 0) || (strcasecmp((Noe+i)->atom_type1,"O") == 0) ||
		   (strcasecmp((Noe+i)->atom_type1,"HN") == 0) || (strcasecmp((Noe+i)->atom_type1,"H") == 0) ||
		   (strcasecmp((Noe+i)->atom_type1,"HA") == 0))
		    if ((strcasecmp((Noe+i)->atom_type2,"N") == 0) || (strcasecmp((Noe+i)->atom_type2,"CA") == 0) ||
			(strcasecmp((Noe+i)->atom_type2,"O") == 0) || (strcasecmp((Noe+i)->atom_type2,"HN") == 0) ||
			(strcasecmp((Noe+i)->atom_type2,"H") == 0) || (strcasecmp((Noe+i)->atom_type2,"HA") == 0))
			 medium_bckbck=medium_bckbck+1;
		    else
			 medium_bcksdc=medium_bcksdc+1;
	       else 
		    medium_sdcsdc=medium_sdcsdc+1;
	  }
	  
     fin:  ;
	  
     }  /* main loop for analyzing constraints. for (i=1;i<kount... */
     
     new_kount_dist = intra_residue + sequential + medium_r + long_r + disulfide_cons ;
     
     printf("\n -------------- SUMMARY OF CONSTRAINTS --------------- \n");
     printf(" TOTAL NUMBER OF NOE CONSTRAINTS           : %5d \n",new_kount_dist);
     printf("      INTRA-RESIDUE CONSTRAINTS  (I=J)     : %5d \n",intra_residue);
     printf("      SEQUENTIAL    CONSTRAINTS  (I-J)=1   : %5d \n",sequential);
     printf("          BACKBONE-BACKBONE                : %9d \n",sequential_bckbck);
     printf("          BACKBONE-SIDE CHAIN              : %9d \n",sequential_bcksdc);
     printf("          SIDE CHAIN-SIDE CHAIN            : %9d \n",sequential_sdcsdc);
     printf("      MEDIUM RANGE  CONSTRAINTS 1<(I-J)<5  : %5d \n",medium_r);
     printf("          BACKBONE-BACKBONE                : %9d \n",medium_bckbck);
     printf("          BACKBONE-SIDE CHAIN              : %9d \n",medium_bcksdc);
     printf("          SIDE CHAIN-SIDE CHAIN            : %9d \n",medium_sdcsdc);
     printf("      LONG  RANGE   CONSTRAINTS  (I-J)>=5  : %5d \n",long_r);
     printf(" TOTAL HYDROGEN BOND RESTRAINTS            : %5d \n",hydrogen_bond);
     printf("      LONG RANGE H-BOND RESTR.   (I-J)>=5  : %5d \n",hbond_long);
     printf(" DISULFIDE CONSTRAINTS                     : %5d \n",disulfide_cons);
     printf(" INTRA-CHAIN CONSTRAINTS                   : %5d\n",IntraChain );
     printf(" INTER-CHAIN CONSTRAINTS                   : %5d\n",InterChain );
     printf("\n\n");
     
     fflush(stderr);
     fflush(stdout);
     
     /* Write info in KYCNS.OUT file */
     for (k = 1 ; k <= ResCt ; k++) {
	  fprintf(fkycns,"%5d  %3d  ",k,numberNOE[k]);
	  for (i = 1; i <= numberNOE[k] ;i++) 
	       fprintf(fkycns,"%5d%4.1f", key_NOE[k][i], flag[key_NOE[k][i]]);
	  fprintf(fkycns,"\n");
     }
     
     fflush(fkycns);
     fclose(fkycns);
     
     /*
      * USEFUL.DATA
      */
     fprintf(fout,"\n -------------- SUMMARY OF CONSTRAINTS --------------- \n");
     fprintf(fout," TOTAL NUMBER OF NOE CONSTRAINTS           : %5d\n",new_kount_dist);
     fprintf(fout,"      INTRA-RESIDUE CONSTRAINTS  (I=J)     : %5d\n",intra_residue);
     fprintf(fout,"      SEQUENTIAL    CONSTRAINTS  (I-J)=1   : %5d\n",sequential);
     fprintf(fout,"          BACKBONE-BACKBONE                : %9d\n",sequential_bckbck);
     fprintf(fout,"          BACKBONE-SIDE CHAIN              : %9d\n",sequential_bcksdc);
     fprintf(fout,"          SIDE CHAIN-SIDE CHAIN            : %9d\n",sequential_sdcsdc);
     fprintf(fout,"      MEDIUM RANGE  CONSTRAINTS 1<(I-J)<5  : %5d\n",medium_r);
     fprintf(fout,"          BACKBONE-BACKBONE                : %9d\n",medium_bckbck);
     fprintf(fout,"          BACKBONE-SIDE CHAIN              : %9d\n",medium_bcksdc);
     fprintf(fout,"          SIDE CHAIN-SIDE CHAIN            : %9d\n",medium_sdcsdc);
     fprintf(fout,"      LONG  RANGE   CONSTRAINTS  (I-J)>=5  : %5d\n",long_r);
     fprintf(fout," TOTAL HYDROGEN BOND RESTRAINTS            : %5d\n",hydrogen_bond);
     fprintf(fout,"      LONG RANGE H-BOND RESTR.   (I-J)>=5  : %5d \n",hbond_long);
     fprintf(fout," DISULFIDE CONSTRAINTS                     : %5d\n",disulfide_cons);
     fprintf(fout," INTRA-CHAIN CONSTRAINTS                   : %5d\n",IntraChain );
     fprintf(fout," INTER-CHAIN CONSTRAINTS                   : %5d\n",InterChain );
     
     fprintf(fout,"\n RES   #   INTRA  INTER \n");
     
     for (i = 1 ; i <= ResCt ; i++)   {
	  fprintf(fout," %3s  %2d %6d %6.1f \n", seqres[i], i, consres_intra[i], consres_inter[i]);
	  check_total= (float) consres_intra[i] + consres_inter[i] + check_total;
     }
     fprintf(fout,"\n TOTAL NUMBER OF CONSTRAINTS (CHECKING): %6.1f \n", check_total);
     fflush(fout);
     fclose(fout);
     
     if (DEBUG > 0) printf(" > ANALYZE_cns: calling do_codes_for_cosntraints\n");
     
     do_codes_for_constraints();
     
     /*
      * Write a file for further use in analyzing the violations after congen runs
      * is a list (two columns) with the constraints in format 
      */
     if (DEBUG > 0) printf(" > ANALYZE_cns: CONS.OUT file\n");
     printf("> ANALYZE_cns: Do you want a CONS.OUT file [default no] ?_ ");
     GetFromKeyBoard(t,MAXWORD,stdin); 
     if ((t[0] == '\n') ||   (t[0] == ' '))
	  strcpy(t,"no");
     if (strncasecmp(t,"yes",1)==0)    {
	  fcons=safeopen("CONS.OUT","w",(int)strlen("CONS.OUT"));
	  for (i = 1 ; i <= kount_dist ; i++)
	       fprintf(fcons,"%5.2f  %5.2f\n", (Noe+i)->lower, (Noe+i)->upper);
	  fflush(fcons);
	  fclose(fcons);
     }
     fflush(stderr);
     fflush(stdout);
     
     /* Free memory */
     for (i = 0; i <= ResCt+1; i++)
	  free(key_NOE[i]);
     free(key_NOE);
     free(numberNOE);
     free(consres_intra);
     free(consres_inter);
     free(flag);
     return;
}

