/*$Id: any_angle.c,v 3.32 2003/10/06 12:13:32 roberto Exp roberto $*/
#include "pdbstat.h"
#define min(A,B) ((A) < (B) ? (A) : (B))
#define max(A,B) ((A) > (B) ? (A) : (B))
extern FILE *efopen(char *, char *);
extern void ProcessAnyAtom(char Uno, char ResTarget[5], char AtTarget[6], int Cnt, struct models *model);

/* ========================================================================= */
void any_angle(char mod[5], 
	       char res1[5], char at1[6],
	       char res2[5], char at2[6],
	       char res3[5], char at3[6],
	       FILE *angles, char Detailed)
		  
{
  /*
   * ---
   * --- Evaluates any angle in a molecule
   * ---
   * --- (Roberto Tejero, RTT, Feb 1997, Valencia University).
   * ---
   */
  
  static int number1=0, number2=0, number3=0 ;
  static int tmp_model, ini, fin ;
  static int i,j,k ;
  static int NumberOfAngles = 0;
  static float SumOfAngles = 0.0, Lowest=9999.99, Highest=-9999.99;
  static struct models *model ;
  static char Uno = FALSE ;

  tmp_model = key_model ;

  if (DEBUG > 0) 
    printf(" > ANY_ANG: Locating ANG %3d %-s %3d %-s %3d %-s \n",
	   atoi(res1), at1, atoi(res2), at2, atoi(res3), at3);

  if (strcmp(mod,"*")==0) {
    ini = 1;
    fin = total_models;
  } else {
    ini = atoi(mod) ;
    fin = atoi(mod);
  }

  if (ini > AverageModel) {
    fprintf(stdout,"> ANY_ANGLE: no such model in memory \n");
    return ;
  }

  for (key_model = ini; key_model <= fin; key_model++) {
    model = MODELS+key_model ;
    for (i = 1; i <= model->number_of_atoms; ++i) {
	    Uno = TRUE ;
            ProcessAnyAtom(Uno, res1, at1, i, model);
      
      if ((model->PDB[i].pdb_num_res == ResAt->Number ) &&
	  (strncasecmp(model->PDB[i].pdb_iupac,ResAt->Label,ResAt->Len)==0)) {      
	number1 = i ;
	if (DEBUG > 0)
	  printf(" > ANY_DIHE: Atom 1 (%3d %3s) located\n", atoi(res1), at1);
      } else {
	      continue ;
      }
      
      for (j = 1; j <= model->number_of_atoms; ++j) {
	    Uno = FALSE ;
            ProcessAnyAtom(Uno, res2, at2, j, model);
	
	if ((model->PDB[j].pdb_num_res == ResAt->Number) &&
	    (strncasecmp(model->PDB[j].pdb_iupac,ResAt->Label,ResAt->Len)==0)) {
	  number2 = j ; 
	  if (DEBUG > 0)
	    printf(" > ANY_DIHE: Atom 2 (%3d %3s) located\n", atoi(res2), at2);
	} else {
	  continue ;
	}
	
	for (k = 1; k <= model->number_of_atoms; ++k) {
	    Uno = FALSE ;
            ProcessAnyAtom(Uno, res3, at3, k, model);
	  
	  if ((model->PDB[k].pdb_num_res == ResAt->Number) &&
	      (strncasecmp(model->PDB[k].pdb_iupac,ResAt->Label,ResAt->Len)==0)) {
	    number3 = k ; 
	    if (DEBUG > 0)
	      printf(" > ANY_DIHE: Atom 3 (%3d %3s) located\n", atoi(res3), at3);
	    
	    if ((number1 != number2) && (number1 != number3) &&
		(number2 != number3)) {
	      
	      if (Detailed)
		fprintf(angles,"  MDL %2d AT1(%3d %-4s) AT2(%3d %-4s) AT3(%3d %-4s) ANGLE:%8.2f\n",
			key_model,
			model->PDB[number1].pdb_num_res, model->PDB[number1].pdb_iupac,
			model->PDB[number2].pdb_num_res, model->PDB[number2].pdb_iupac,
			model->PDB[number3].pdb_num_res, model->PDB[number3].pdb_iupac,
			angulo(number1,number2,number3)/radperdeg);      
	      
	      if (Detailed)
		printf("  MDL %2d AT1(%3d %-4s) AT2(%3d %-4s) AT3(%3d %-4s) ANGLE:%8.2f\n",
		       key_model,
		       model->PDB[number1].pdb_num_res, model->PDB[number1].pdb_iupac,
		       model->PDB[number2].pdb_num_res, model->PDB[number2].pdb_iupac,
		       model->PDB[number3].pdb_num_res, model->PDB[number3].pdb_iupac,
		       angulo(number1,number2,number3)/radperdeg);      
	      
	      NumberOfAngles++;
	      SumOfAngles=SumOfAngles+angulo(number1,number2,number3)/radperdeg;
	      Lowest=min(Lowest,angulo(number1,number2,number3)/radperdeg);
	      Highest=max(Highest,angulo(number1,number2,number3)/radperdeg);
	    }
	  }
	}
      }
    }
  }
  
  printf("\n  Number of angles: %d (SUMM: %.3f, AVGE: %.3f) [Lwst: %.3f, Hghst: %.3f] \n",
	 NumberOfAngles, SumOfAngles, SumOfAngles/NumberOfAngles, Lowest, Highest);

  if ( !Detailed ) {
    if ((strcmp(at1,"h")==0) && (strcmp(at2,"n")==0) && (strcmp(at3,"ca")==0))
      fprintf(angles,"             h*n*ct     %10.1f\n", SumOfAngles/NumberOfAngles);
    
    if ((strcmp(at1,"n")==0) && (strcmp(at2,"ca")==0) && (strcmp(at3,"ha")==0))
      fprintf(angles,"             n*ct*hc    %10.1f\n", SumOfAngles/NumberOfAngles);

    if ((strcmp(at1,"n")==0) && (strcmp(at2,"ca")==0) && (strcmp(at3,"cb")==0))
      fprintf(angles,"             n*ct*ct    %10.1f\n", SumOfAngles/NumberOfAngles);

    if ((strcmp(at1,"n")==0) && (strcmp(at2,"ca")==0) && (strcmp(at3,"c")==0))
      fprintf(angles,"             n*ct*c     %10.1f\n", SumOfAngles/NumberOfAngles);

    if ((strcmp(at1,"ca")==0) && (strcmp(at2,"cb")==0) && (strcmp(at3,"hb")==0))
      fprintf(angles,"             ct*ct*hc   %10.1f\n", SumOfAngles/NumberOfAngles);

    if ((strcmp(at1,"ca")==0) && (strcmp(at2,"c")==0) && (strcmp(at3,"n")==0))
      fprintf(angles,"             ct*c*n     %10.1f\n", SumOfAngles/NumberOfAngles);
 
    if ((strcmp(at1,"c")==0) && (strcmp(at2,"n")==0) && (strcmp(at3,"h")==0))
      fprintf(angles,"             c*n*h      %10.1f\n", SumOfAngles/NumberOfAngles);
  } else
    fprintf(angles,"\n  Number of angles: %d (SUMM: %.3f, AVGE: %.3f) [Lwst: %.3f, Hghst: %.3f] \n",
	    NumberOfAngles, SumOfAngles, SumOfAngles/NumberOfAngles, Lowest, Highest);
  
  key_model = tmp_model ;
  return ;
}
