/*$Id: any_dihedral.c,v 3.32 2003/10/06 12:13:32 roberto Exp roberto $*/
#include "pdbstat.h"

extern void ProcessAnyAtom(char Uno, char ResTarget[5], char AtTarget[6], int Cnt, struct models *model) ;


/* ========================================================================= */
void any_dihedral(char mod[5], 
		  char res1[5], char at1[6],
		  char res2[5], char at2[6],
		  char res3[5], char at3[6],
		  char res4[5], char at4[6],
		  FILE *dihedrals)
		  
{
  /*
   * ---
   * --- Evaluates any dihedral --hopefully!-- in a molecule
   * ---
   * --- (Roberto Tejero, RTT, Feb 1997, Valencia University).
   * ---
   */
  
     static int    tmp_model, ini, fin ;
     static int    icn, jcn, kcn, lcn ;
     static struct models *model ;
     static char   Uno = FALSE ;
     extern struct ResAtom *ResAt ;
     
     tmp_model = key_model ;
     
     if (DEBUG > 0) 
	  printf(" > ANY_DIHE: searching for DIHE %3d %-s %3d %-s %3d %-s %3d %-s\n",
		 atoi(res1), at1, atoi(res2), at2, atoi(res3), at3, atoi(res4), at4);
     
     if (strcmp(mod,"*")==0) {
	  ini = 1;
	  fin = total_models;
     }  else {
	  ini = atoi(mod) ;
	  fin = atoi(mod);
     }
     
     if (ini > AverageModel) {
	  fprintf(stdout,"> ANY_DIHE: no such model in memory \n");
	  return ;
     }
     
     for (key_model = ini; key_model <= fin; key_model++) {
	  model = MODELS+key_model ;
	  
	  if (DEBUG> 0) fprintf(stdout," any_dihedral(): Working on %6d atoms\n", model->number_of_atoms);
	  
	  icn = 0;
	  /* Search for atom 1 */
	  while ( ++icn <= model->number_of_atoms ) { 
	       Uno = TRUE ; 
	       ProcessAnyAtom(Uno, res1, at1, icn, model);
	       
	       if ((model->PDB[icn].pdb_num_res == ResAt->Number) &&
		   (strncasecmp(model->PDB[icn].pdb_iupac,ResAt->Label,ResAt->Len)==0)) {      
		    if (DEBUG > 0) printf(" > ANY_DIHE: Atom 1 (%3d %3s) located, Index (%6d)\n", 
					  atoi(res1), at1, icn);
	       } else {
		    continue ;
	       }
	       
	       /* Search for atom 2 */
	       for (jcn = 1; jcn <= model->number_of_atoms; jcn++) {
		    Uno = FALSE ;	
		    ProcessAnyAtom(Uno, res2, at2, jcn, model);
		    
		    if ((model->PDB[jcn].pdb_num_res == ResAt->Number) &&
			(strncasecmp(model->PDB[jcn].pdb_iupac,ResAt->Label,ResAt->Len)==0)) {
			 if (DEBUG > 0) printf(" > ANY_DIHE: Atom 2 (%3d %3s) located, Index (%6d)\n", 
					       atoi(res2), at2, jcn);
		    } else {
			 continue ;
		    }
		    
		    /* Search for atom 3 */
		    for (kcn = 1; kcn <= model->number_of_atoms; kcn++) {
			 
			 Uno = FALSE ;	
			 ProcessAnyAtom(Uno, res3, at3, jcn, model);
			 
			 if ((model->PDB[kcn].pdb_num_res == ResAt->Number) &&
			     (strncasecmp(model->PDB[kcn].pdb_iupac,ResAt->Label,ResAt->Len)==0)) {
			      if (DEBUG > 0) printf(" > ANY_DIHE: Atom 3 (%3d %3s) located, Index (%6d)\n", 
						    atoi(res3), at3, kcn);
			 } else {
			      continue ;
			 }
			 
			 /* Search for atom 4 */
			 for (lcn = 1; lcn <= model->number_of_atoms; lcn++) {
			      Uno = FALSE ; 
			      ProcessAnyAtom(Uno, res4, at4, jcn, model);
			      
			      if ((model->PDB[lcn].pdb_num_res == ResAt->Number ) &&
				  (strncasecmp(model->PDB[lcn].pdb_iupac,ResAt->Label,ResAt->Len)==0)) {
				   
				   if (DEBUG > 0) printf(" > ANY_DIHE: Atom 4 (%3d %3s) located, indexes (%6d,%6d,%6d,%6d)\n", 
							 atoi(res4), at4, icn, jcn, kcn, lcn);
				   /*
				    * The chain_id stuff is for a quick fix for dimers/x-mers, when evaluating
				    * omega, phi or psi, please fix it 
				    */
				   if ( (icn != jcn) && (icn != kcn) && (icn != lcn) && (jcn != kcn) &&
				        (jcn != lcn) && (kcn != lcn) && 
				        (model->PDB[icn].pdb_chain_id[0] == model->PDB[jcn].pdb_chain_id[0]) &&   
				        (model->PDB[icn].pdb_chain_id[0] == model->PDB[kcn].pdb_chain_id[0]) &&
				        (model->PDB[icn].pdb_chain_id[0] == model->PDB[lcn].pdb_chain_id[0]) ) {					
					fprintf(dihedrals," MDL %2d AT1(%3d %2s %-4s) AT2(%3d %2s %-4s) AT3(%3d %2s %-4s) AT4(%3d %2s %-4s) DIH:%8.2f\n",
						key_model,
						model->PDB[icn].pdb_num_res, model->PDB[icn].pdb_chain_id, model->PDB[icn].pdb_iupac,
						model->PDB[jcn].pdb_num_res, model->PDB[jcn].pdb_chain_id, model->PDB[jcn].pdb_iupac,
						model->PDB[kcn].pdb_num_res, model->PDB[kcn].pdb_chain_id, model->PDB[kcn].pdb_iupac,
						model->PDB[lcn].pdb_num_res, model->PDB[lcn].pdb_chain_id, model->PDB[lcn].pdb_iupac,
						dihang(icn,jcn,kcn,lcn)/radperdeg); 
					
					printf(" MDL %2d AT1(%3d %2s %-4s) AT2(%3d %2s %-4s) AT3(%3d %2s %-4s) AT4(%3d %2s %-4s) DIH:%8.2f\n",
					       key_model,
					       model->PDB[icn].pdb_num_res, model->PDB[icn].pdb_chain_id, model->PDB[icn].pdb_iupac,
					       model->PDB[jcn].pdb_num_res, model->PDB[jcn].pdb_chain_id, model->PDB[jcn].pdb_iupac,
					       model->PDB[kcn].pdb_num_res, model->PDB[kcn].pdb_chain_id, model->PDB[kcn].pdb_iupac,
					       model->PDB[lcn].pdb_num_res, model->PDB[lcn].pdb_chain_id, model->PDB[lcn].pdb_iupac,
					       dihang(icn,jcn,kcn,lcn)/radperdeg); 
				   }
			      }
			 } /* for lcn, 4th atom */
		    } /* for kcn, 3rd atom */
	       } /* for jcn, 2nd atom */
	  } /* while icn, 1st atom */
     } /* for (key_model  */
     key_model = tmp_model ;
     return ;
}


/* ========================================================================= */
float DihedralResModel(int modeln, int residue, char Angle[4], char ResName[5]) {
  static int i, at1, at2, at3, at4;
  static struct models *model ;
  static int tmp_model ;
  static float Dih;
  
  tmp_model = key_model ;
  model = MODELS+modeln ;
  key_model = modeln ;

  at1 = at2 = at3 = at4 = 0;
  
  if ( DEBUG > 5 )
    fprintf(stdout,"  DihedralResModel(): model %2d, residue %3d, Angle `%s' \n",
		    modeln, residue, Angle);

  if (strncmp(Angle,"PHI",3)==0) {
    for (i = 1; i <= model->number_of_atoms; ++i) {
      if (model->PDB[i].pdb_num_res > residue) break ;
      if (model->PDB[i].pdb_num_res == residue-1) 
	if (strncasecmp(model->PDB[i].pdb_iupac," c  ",3)==0) at1 = i;
      if (model->PDB[i].pdb_num_res == residue) {
	if (strncasecmp(model->PDB[i].pdb_iupac," n  ",3)==0) at2 = i;
	if (strncasecmp(model->PDB[i].pdb_iupac," ca ",3)==0) at3 = i;
	if (strncasecmp(model->PDB[i].pdb_iupac," c  ",3)==0) at4 = i;
      }
    }
  } else if (strncmp(Angle,"PSI",3)==0) {
    for (i = 1; i <= model->number_of_atoms; ++i) {
      if (model->PDB[i].pdb_num_res > residue+1) break ;
      if (model->PDB[i].pdb_num_res == residue) {
	if (strncasecmp(model->PDB[i].pdb_iupac," n  ",3)==0) at1 = i;
	if (strncasecmp(model->PDB[i].pdb_iupac," ca ",3)==0) at2 = i;
	if (strncasecmp(model->PDB[i].pdb_iupac," c  ",3)==0) at3 = i;
      }
      if (model->PDB[i].pdb_num_res == residue+1) {
	if (strncasecmp(model->PDB[i].pdb_iupac," n  ",3)==0) at4 = i;
      }
    }
  } else if (strncmp(Angle,"OMEG2",4)==0) {
    for (i = 1; i <= model->number_of_atoms; ++i) {
      if (model->PDB[i].pdb_num_res > residue+1) break ;
      if (model->PDB[i].pdb_num_res == residue) {
	if (strncasecmp(model->PDB[i].pdb_iupac," o  ",3)==0) at1 = i;
	if (strncasecmp(model->PDB[i].pdb_iupac," c  ",3)==0) at2 = i;
      }
      if (model->PDB[i].pdb_num_res == residue+1) {
	if (strncasecmp(model->PDB[i].pdb_iupac," n  ",3)==0) at3 = i;
	if (strncasecmp(model->PDB[i].pdb_iupac," ca ",3)==0) at4 = i;
      }
    }
  } else if (strncmp(Angle,"OMEGA",4)==0) {
    for (i = 1; i <= model->number_of_atoms; ++i) {
      if (model->PDB[i].pdb_num_res > residue+1) break ;
      if (model->PDB[i].pdb_num_res == residue) {
	if (strncasecmp(model->PDB[i].pdb_iupac," ca ",3)==0) at1 = i;
	if (strncasecmp(model->PDB[i].pdb_iupac," c  ",3)==0) at2 = i;
      }
      if (model->PDB[i].pdb_num_res == residue+1) {
	if (strncasecmp(model->PDB[i].pdb_iupac," n  ",3)==0) at3 = i;
	if (strncasecmp(model->PDB[i].pdb_iupac," ca ",3)==0) at4 = i;
      }
    }
  } else if (strncmp(Angle,"CHI1",4)==0) {
    for (i = 1; i <= model->number_of_atoms; ++i) {
      if (model->PDB[i].pdb_num_res > residue) break ;
      if (model->PDB[i].pdb_num_res == residue) 
	if (strncasecmp(model->PDB[i].pdb_iupac," n  ",3)==0) at1 = i;
      if (model->PDB[i].pdb_num_res == residue) {
	if (strncasecmp(model->PDB[i].pdb_iupac," ca ",3)==0) at2 = i;
	if (strncasecmp(model->PDB[i].pdb_iupac," cb ",3)==0) at3 = i;
	if (strncasecmp(ResName,"THR",3)==0) {
	  if (strncasecmp(model->PDB[i].pdb_iupac," og1 ",4)==0) at4 = i;
	} else if (strncasecmp(ResName,"CYS",3)==0) { 
	  if (strncasecmp(model->PDB[i].pdb_iupac," sg ",4)==0) at4 = i;
	} else if (strncasecmp(ResName,"SER",3)==0) { 
	  if (strncasecmp(model->PDB[i].pdb_iupac," og ",4)==0) at4 = i;
	} else if (strncasecmp(ResName,"VAL",3)==0) { 
	  if (strncasecmp(model->PDB[i].pdb_iupac," cg1 ",4)==0) at4 = i;
	} else if (strncasecmp(ResName,"ILE",3)==0) { 
	  if (strncasecmp(model->PDB[i].pdb_iupac," cg1 ",4)==0) at4 = i;
	} else {
	  if (strncasecmp(model->PDB[i].pdb_iupac," cg ",3)==0) at4 = i;
	}
      }
    }
  } else if (strncmp(Angle,"CHI2",4)==0) {
    for (i = 1; i <= model->number_of_atoms; ++i) {
      if (model->PDB[i].pdb_num_res > residue) break ;
      if (model->PDB[i].pdb_num_res == residue) 
	if (strncasecmp(model->PDB[i].pdb_iupac," ca ",4)==0) at1 = i;
      if (model->PDB[i].pdb_num_res == residue) {
	if (strncasecmp(model->PDB[i].pdb_iupac," cb ",4)==0) at2 = i;
	if (strncasecmp(ResName,"THR",3)==0) {
	  if (strncasecmp(model->PDB[i].pdb_iupac," og1 ",4)==0) at3 = i;
	} else if (strncasecmp(ResName,"CYS",3)==0) { 
	  if (strncasecmp(model->PDB[i].pdb_iupac," sg ",4)==0) at3 = i;
	} else if (strncasecmp(ResName,"SER",3)==0) { 
	  if (strncasecmp(model->PDB[i].pdb_iupac," og ",4)==0) at3 = i;
	} else if (strncasecmp(ResName,"VAL",3)==0) { 
	  if (strncasecmp(model->PDB[i].pdb_iupac," cg1 ",4)==0) at3 = i;
	} else if (strncasecmp(ResName,"ILE",3)==0) { 
	  if (strncasecmp(model->PDB[i].pdb_iupac," cg1 ",4)==0) at3 = i;
	} else {
	  if (strncasecmp(model->PDB[i].pdb_iupac," cg ",4)==0) at3 = i;
	}
	if (strncasecmp(ResName,"ARG",3)==0) {
	  if (strncasecmp(model->PDB[i].pdb_iupac," cd  ",4)==0) at4 = i;
	} else if (strncasecmp(ResName,"ASN",3)==0) {
	  if (strncasecmp(model->PDB[i].pdb_iupac," od1 ",4)==0) at4 = i;
	} else if (strncasecmp(ResName,"ASP",3)==0) {
	  if (strncasecmp(model->PDB[i].pdb_iupac," od1 ",4)==0) at4 = i;
	} else if (strncasecmp(ResName,"CYS",3)==0) {
	  if (strncasecmp(model->PDB[i].pdb_iupac," hg  ",4)==0) at4 = i;
	} else if (strncasecmp(ResName,"GLN",3)==0) {
	  if (strncasecmp(model->PDB[i].pdb_iupac," cd  ",4)==0) at4 = i;
	} else if (strncasecmp(ResName,"GLU",3)==0) {
	  if (strncasecmp(model->PDB[i].pdb_iupac," cd  ",4)==0) at4 = i;
	} else if (strncasecmp(ResName,"HIS",3)==0) { 
	  if (strncasecmp(model->PDB[i].pdb_iupac," nd1 ",4)==0) at4 = i;
	} else if (strncasecmp(ResName,"ILE",3)==0) { 
	  if (strncasecmp(model->PDB[i].pdb_iupac," cd1 ",4)==0) at4 = i;
	} else if (strncasecmp(ResName,"LEU",3)==0) { 
	  if (strncasecmp(model->PDB[i].pdb_iupac," cd1 ",4)==0) at4 = i;
	} else if (strncasecmp(ResName,"LYS",3)==0) { 
	  if (strncasecmp(model->PDB[i].pdb_iupac," cd  ",4)==0) at4 = i;
	} else if (strncasecmp(ResName,"MET",3)==0) { 
	  if (strncasecmp(model->PDB[i].pdb_iupac," sd  ",4)==0) at4 = i;
	} else if (strncasecmp(ResName,"PHE",3)==0) { 
	  if (strncasecmp(model->PDB[i].pdb_iupac," cd1 ",4)==0) at4 = i;
        } else if (strncasecmp(ResName,"SER",3)==0) {
	  if (strncasecmp(model->PDB[i].pdb_iupac," hg  ",4)==0) at4 = i;
	} else if (strncasecmp(ResName,"THR",3)==0) { 
	  if (strncasecmp(model->PDB[i].pdb_iupac," hg1 ",4)==0) at4 = i;
	} else if (strncasecmp(ResName,"TRP",3)==0) { 
	  if (strncasecmp(model->PDB[i].pdb_iupac," cd1 ",4)==0) at4 = i;
	} else if (strncasecmp(ResName,"TYR",3)==0) { 
	  if (strncasecmp(model->PDB[i].pdb_iupac," cd1 ",4)==0) at4 = i;
	} else if (strncasecmp(ResName,"VAL",3)==0) { 
	  if (strncasecmp(model->PDB[i].pdb_iupac,"1hg1 ",4)==0) at4 = i;
	} else {
	  if (strncasecmp(model->PDB[i].pdb_iupac," cd ",3)==0) at4 = i;
	}
      }
    }
  }

  if ( at1 != 0 && at2 != 0 && at3 != 0 && at4 != 0 )
     Dih = dihang(at1,at2,at3,at4)/radperdeg; 

  key_model = tmp_model ;
  return Dih ;
}

