/*$Id: any_distance.c,v 3.32 2003/10/06 12:13:32 roberto Exp roberto $ */

#include "pdbstat.h"
#define min(A,B) ((A) < (B) ? (A) : (B))
#define max(A,B) ((A) > (B) ? (A) : (B))

extern void ProcessAnyAtom(char Uno, char ResTarget[5], char AtTarget[6], int Cnt, struct models *model);
void FromDianaToIUPAC(char Result[6], char Target[6], char ResNam[5]) ;


/* ========================================================================= */
void any_distance(char mod[5], char res1[5], char at1[6], 
		  char res2[5], char at2[6], FILE *distances,
		  float cutoff, char Detailed, char Reverse)
{
  /*
   * ---
   * --- Evaluates any distance in a molecule
   * ---
   * --- (Roberto Tejero, RTT, Feb 1997, Valencia University).
   * ---
   */
  
  static int i, j, Inicio;
  static int number1=0, number2=0 ;
  static int tmp_model, ini, fin ;
  int      NumberOfDistances = 0 ;
  float    SumOfDistances = 0.0, SumOfDistances_6 = 0.0;
  float    Lowest=9999.99, Highest=-9999.99 ;
  double   sixth = (double) (-6.0) ;
  
  static struct models *model ;
  char Uno = FALSE ;

  tmp_model = key_model ;
  
  if (strcmp(mod,"*")==0) {
    ini = 1;
    fin = total_models;
  } else {
    ini = atoi(mod) ;
    fin = atoi(mod);
  }
  
  if (ini > AverageModel) {
    fprintf(stdout,"> ANY_DIST: no such model in memory \n");
    return ;
  }

  for (key_model=ini; key_model <= fin; key_model++) {
    model = MODELS+key_model ;
    for (i = 1; i < model->number_of_atoms; ++i) {
	    Uno = TRUE ;
	    ProcessAnyAtom(Uno, res1, at1, i, model);
      /* location of atom */
      if ((model->PDB[i].pdb_num_res == ResAt->Number) &&
	  (strncasecmp(model->PDB[i].pdb_iupac,ResAt->Label,ResAt->Len)==0)) {      
	number1 = i ;
      } else 
	continue ;

      if ( Reverse )
	Inicio = 1 ;
      else
	Inicio = i ;

      for (j = Inicio; j < model->number_of_atoms; ++j) {

	 Uno = FALSE ;
	 ProcessAnyAtom(Uno, res2, at2, j, model);
	
	if ((model->PDB[j].pdb_num_res == ResAt->Number) &&
	    (strncasecmp(model->PDB[j].pdb_iupac,ResAt->Label,ResAt->Len)==0)) {
	  
	  number2 = j ;
	  
	  if ((number1 != number2) && (distance(number1,number2) <= cutoff)) {
	    
	    NumberOfDistances++ ;
	    SumOfDistances+=distance(number1,number2) ;
	    SumOfDistances_6+=(float) pow((double)distance(number1,number2),sixth) ;

	    Lowest=min(Lowest,distance(number1,number2));
	    Highest=max(Highest,distance(number1,number2));

	    if (Detailed)
	      printf("   MDL %2d  AT1 (%3d %-s) AT2 (%3d %-s) distance:%7.3f\n",
		     key_model,
		     model->PDB[number1].pdb_num_res, model->PDB[number1].pdb_iupac,
		     model->PDB[number2].pdb_num_res, model->PDB[number2].pdb_iupac,
		     distance(number1,number2));      
	    
	    if (Detailed)
	      fprintf(distances,
		      "   MDL %2d  AT1 (%3d %-s) AT2 (%3d %-s) distance:%7.3f\n",
		      key_model,
		      model->PDB[number1].pdb_num_res, model->PDB[number1].pdb_iupac,
		      model->PDB[number2].pdb_num_res, model->PDB[number2].pdb_iupac,
		      distance(number1,number2));      
	  }
	}
      }
    }
  }

  if (Detailed) {
    printf("\n  Number of distances is %7d (SUMM:%12.2f, AVGE: %7.3f), [Lwst:%7.3f, Hghst:%7.3f]",
	   NumberOfDistances, SumOfDistances, SumOfDistances/NumberOfDistances, Lowest, Highest);
  } else {
    printf("\n Num. of dist. <= %.1f for ALL hydr. is %7d (SUMM:%12.2f, AVGE: %7.3f, AVGE_6TH: %.6f)",
	   cutoff, NumberOfDistances, SumOfDistances, 
	   SumOfDistances/NumberOfDistances, SumOfDistances_6/NumberOfDistances);
  }
  
  if ( !Detailed ) {
    if ((strcmp(at1,"n")==0) && (strcmp(at2,"ca")==0))
      fprintf(distances,"             ln*ca      %10.3f\n", SumOfDistances/NumberOfDistances);
    
   if ((strcmp(at1,"c")==0) && (strcmp(at2,"n")==0))
      fprintf(distances,"             ln*c       %10.3f\n", SumOfDistances/NumberOfDistances);

   if ((strcmp(at1,"n")==0) && (strcmp(at2,"h")==0))
      fprintf(distances,"             ln*hn      %10.3f\n", SumOfDistances/NumberOfDistances);

   if ((strcmp(at1,"ca")==0) && (strcmp(at2,"cb")==0))
      fprintf(distances,"             lca*cb     %10.3f\n", SumOfDistances/NumberOfDistances);

   if ((strcmp(at1,"ca")==0) && (strcmp(at2,"c")==0))
      fprintf(distances,"             lca*c      %10.3f\n", SumOfDistances/NumberOfDistances);

   if ((strcmp(at1,"ca")==0) && (strcmp(at2,"ha")==0))
      fprintf(distances,"             lc*h       %10.3f\n", SumOfDistances/NumberOfDistances);
  } else
    fprintf(distances,"\n  Number of distances is %d (SUMM:%12.2f, AVGE: %7.3f), [Lwst:%7.3f, Hghst:%7.3f]\n",
	    NumberOfDistances, SumOfDistances, SumOfDistances/NumberOfDistances, Lowest, Highest);

  key_model = tmp_model ;

  return ;
}

/* ========================================================================= */
void any_distance_backbone(float cutoff, FILE *distances)
{
  /*
   * ---
   * --- Evaluates any distance in a molecule
   * ---
   * --- (Roberto Tejero, RTT, Feb 1997, Valencia University).
   * ---
   */

  int  number1, number2;
  unsigned short i, j;
  int      NumberOfDistances = 0 ;
  float    SumOfDistances = 0.0,  SumOfDistances_6 = 0.0 ;
  static struct models *model ;
  float    Lowest=9999.99, Highest=-9999.99;
  double   sixth = (double) (-6.0);

  for (key_model=1; key_model <= total_models; key_model++) {
    model = MODELS+key_model ;
    for (i = 1; i < model->number_of_atoms; ++i) {
      if ((strncasecmp(model->PDB[i].pdb_iupac," H  ",4)==0) ||
	  (strncasecmp(model->PDB[i].pdb_iupac," HA ",4)==0) ||
	  (strncasecmp(model->PDB[i].pdb_iupac,"1HA ",4)==0) ||
	  (strncasecmp(model->PDB[i].pdb_iupac,"2HA ",4)==0) ||
	  (strncasecmp(model->PDB[i].pdb_iupac,"3HA ",4)==0))
	number1 = i ;
      else 
	goto no_distance ;
      
      for (j = i; j < model->number_of_atoms; ++j) {
	if ((strncasecmp(model->PDB[j].pdb_iupac," H  ",4)==0) ||
	    (strncasecmp(model->PDB[j].pdb_iupac," HA ",4)==0) ||
	    (strncasecmp(model->PDB[j].pdb_iupac,"1HA ",4)==0) ||
	    (strncasecmp(model->PDB[j].pdb_iupac,"2HA ",4)==0) ||
	    (strncasecmp(model->PDB[j].pdb_iupac,"3HA ",4)==0)) {
	  number2 = j ;
	  
	  if ((number1 != number2) && 
	      (distance(number1,number2) <= cutoff)) {
	    
	    if (cutoff < 9999.999) {
	      NumberOfDistances++ ;
	      SumOfDistances+=distance(number1,number2) ;
	      SumOfDistances_6+=(float) pow((double)distance(number1,number2),sixth) ;

	      Lowest=min(Lowest,distance(number1,number2));
	      Highest=max(Highest,distance(number1,number2));
	    }

	    if (cutoff > 9999.00) {
	      printf("   MDL %2d  AT1 (%3d %-s) AT2 (%3d %-s) distance:%7.3f\n",
		     key_model,
		     model->PDB[number1].pdb_num_res, model->PDB[number1].pdb_iupac,
		     model->PDB[number2].pdb_num_res, model->PDB[number2].pdb_iupac,
		     distance(number1,number2));      
	    }
 
	    fprintf(distances,
		    "   MDL %2d  AT1 (%3d %-s) AT2 (%3d %-s) distance:%7.3f\n",
		    key_model,
		    model->PDB[number1].pdb_num_res, model->PDB[number1].pdb_iupac,
		    model->PDB[number2].pdb_num_res, model->PDB[number2].pdb_iupac,
		    distance(number1,number2));      
	  }
	}
      }
    no_distance:
      number1 = 0 ;
      number2 = 0 ;
    }
  }
  
  if (cutoff < 9999.999) {
    printf("\n Num. of dist. <= %.1f for  BACKBONE is %5d (SUMM:%9.2f, AVGE: %5.3f, AVGE_6TH: %.6f)",
	   cutoff,
	   NumberOfDistances, SumOfDistances, 
	   SumOfDistances/NumberOfDistances,
	   SumOfDistances_6/NumberOfDistances);
    
    fprintf(distances,"\n Num. of distances is %5d (SUMM:%.2f, AVGE: %.3f), [Lwst:%.3f, Hghst:%.3f]\n",
	    NumberOfDistances, SumOfDistances, 
	    SumOfDistances/NumberOfDistances,
	    Lowest, Highest);
  }

  key_model = 1;
  return ;
}




/* ========================================================================= */
float DistNoe(int mod, int residue1, char Id1[2], char at1[6], int residue2, char Id2[2], char at2[6], 
	      FILE *DumpedNoes, char Average[10])
{
     /*
      */
     static unsigned short i, j;
     static char     atom1[6], atom2[6], FoundAt1[6], FoundAt2[6];
     static int      number1=0, number2=0, tmp_model;
     static int      len1, len2, NumDist=0 ;
     static float    DistTemp=0.0 , DistSixth = 0.0, DistCent ;
     static float    Center1x, Center1y, Center1z;
     static float    Center2x, Center2y, Center2z;
     static short    NumCoords1, NumCoords2;
     static struct models *model ;
     
     if (strcasecmp(Average,"")==0) strcpy(Average,"ave");
     
     tmp_model = key_model ;
     key_model = mod ;
     model = MODELS+key_model ;
     
     NumDist = 0;
     DistTemp = 0.0;
     DistSixth = 0.0;
     len1 = len2 = 5; /* Define length of atom name string */
     
     if (DEBUG >= 10)
	  fprintf(DumpedNoes,"SeeNoes(): Dealing with %3d `%s' %3d `%s'",
		  residue1, at1, residue2, at2);
     
     /*
      * Adjusting the names of atoms to the special format inside PDBSTAT 
      * basically is to add a blank space at the beginning when no number 
      * is present.
      */
     FromDianaToIUPAC(atom1,at1,seqres[residue1]);
     FromDianaToIUPAC(atom2,at2,seqres[residue2]);
     
     if (DEBUG >= 10)
	  fprintf(DumpedNoes,"  Intermediate %3d `%s' %3d `%s'",
		  residue1, atom1, residue2, atom2);
     
     /* Dealing with pseudoatoms represented as HB*, HG*, etc */
     for (i = 0; i <= (int)strlen(atom1); i++)
	  if ((atom1[i] == '*' || atom1[i] == '#')) {
	       len1 = i;
	       break;
	  }
     for (i = 0; i <= (int)strlen(atom2); i++)
	  if ((atom2[i] == '*' || atom2[i]=='#')) {
	       len2 = i;
	       break;
	  }
     
     strcpy(FoundAt1,atom1);
     strcpy(FoundAt2,atom2);
     
     if (DEBUG >= 10)
	  fprintf(DumpedNoes,"  Converted in %3d `%s' %3d `%s'\n",
		  residue1, FoundAt1, residue2, FoundAt2);
     
     /* This for center avergaing */
     Center1x = Center1y = Center1z = 0.0;
     Center2x = Center2y = Center2z = 0.0;
     NumCoords1 = NumCoords2 = 0;
     
     /* Selection of target distances */
     for (i = 1; i <= model->number_of_atoms; ++i) {
	  if ( strcmp(model->PDB[i].pdb_chain_id,Id1) !=0 )
	       continue ;
	  if ( atom1[0] == ' ' ) {
	       if ( atom1[1] == 'H' || atom1[1] == 'h' ) {
		    if ( atom1[2] == '*' || atom1[3] == '*' || atom1[2] == '#' || atom1[3] == '#')
			 FoundAt1[0] = model->PDB[i].pdb_iupac[0];
		    if ( atom1[1] == model->PDB[i].pdb_iupac[1] && atom1[2] == model->PDB[i].pdb_iupac[2] ) {
			 if ( atom1[3] == model->PDB[i].pdb_iupac[3] ) {
			      if (atom1[4] == '*' || atom1[4] == '#') { 
				   strncpy(FoundAt1,model->PDB[i].pdb_iupac,4);
			      }
			 }
		    }
	       }
	  }
	  if (model->PDB[i].pdb_num_res > residue1) 
	       break ;
	  else if (model->PDB[i].pdb_num_res != residue1) 
	       continue ;
	  if (strncasecmp(model->PDB[i].pdb_iupac,FoundAt1,len1)==0) {      
	       number1 = i ;
	       /* for center averaging  */
	       NumCoords1++ ;
	       Center1x += model->PDB[i].pdb_xcrd;
	       Center1y += model->PDB[i].pdb_ycrd;
	       Center1z += model->PDB[i].pdb_zcrd;
	  } else if ( strncmp(atom1," H  ",4)==0 && strncmp(model->PDB[i].pdb_iupac," H  ",4)==0 ) {
	       number1 = i;
	       /* for center averaging  */
	       NumCoords1++ ;
	       Center1x += model->PDB[i].pdb_xcrd;
	       Center1y += model->PDB[i].pdb_ycrd;
	       Center1z += model->PDB[i].pdb_zcrd;
	  } else if ( strncmp(atom1," H  ",4)==0 && strncmp(model->PDB[i].pdb_iupac," HN ",4)==0 ) {
	       number1 = i;
	       /* for center averaging  */
	       NumCoords1++ ;
	       Center1x += model->PDB[i].pdb_xcrd;
	       Center1y += model->PDB[i].pdb_ycrd;
	       Center1z += model->PDB[i].pdb_zcrd;
	  } else	
	       continue ;
	  
	  NumCoords2 = 0;
	  Center2x = Center2y = Center2z = 0.0;
	  for (j = 1; j <= model->number_of_atoms; ++j) {
	       if ( strcmp(model->PDB[j].pdb_chain_id,Id2) !=0 )
		    continue ;
	       if ( atom2[0]==' ') {
		    if ( atom2[1]=='H' || atom2[1]=='h' ) {
			 if ( atom2[2]=='*'  || atom2[3]=='*' || atom2[2] == '#' || atom2[3] == '#')
			      FoundAt2[0]=model->PDB[j].pdb_iupac[0];
			 if (atom2[1] == model->PDB[j].pdb_iupac[1] && atom2[2] == model->PDB[j].pdb_iupac[2] ){
			      if ( atom2[3] == model->PDB[j].pdb_iupac[3] ) {
				   if ( atom2[4]=='*' || atom2[4]=='#') { 
					strncpy(FoundAt2,model->PDB[j].pdb_iupac,4);
				   }
			      }
			 }
		    }
	       }
	       
	       if (model->PDB[j].pdb_num_res > residue2) 
		    break ;
	       else if (model->PDB[j].pdb_num_res != residue2) 
		    continue ;
	       if  ( (strncasecmp(model->PDB[j].pdb_iupac,FoundAt2,len2)==0) ||
		     (strncmp(atom2," H  ",4)==0 && strncmp(model->PDB[j].pdb_iupac," H  ",4)==0) ||
		     (strncmp(atom2," H  ",4)==0 && strncmp(model->PDB[j].pdb_iupac," HN ",4)==0)) {
		    number2 = j ;
		    if (DEBUG > 10) 
			 fprintf(DumpedNoes,"DistNoe(): Located  -- %3d `%s'(`%s') %3d `%s'(`%s') \n",
				 model->PDB[i].pdb_num_res, model->PDB[i].pdb_iupac,atom1,
				 model->PDB[j].pdb_num_res, model->PDB[j].pdb_iupac,atom2);
		    if ( number1 != number2 ) {
			 DistTemp += distance(number1,number2);
			 DistSixth += (float)pow(distance(number1,number2),-6.0);
			 NumDist++ ;
			 NumCoords2++ ;
			 /* for center averaging  */
			 Center2x += model->PDB[j].pdb_xcrd;
			 Center2y += model->PDB[j].pdb_ycrd;
			 Center2z += model->PDB[j].pdb_zcrd;
		    } else {
			 if ( DEBUG > 0 )
			      fprintf(DumpedNoes," ** SeeNoes(): Same atom, distance zero for NOE  %3d `%s' %3d `%s' %s %s\n",
				      residue1, at1, residue2, at2, atom1, atom2);
		    }
	       } else 
		    continue ;
	  }
     }
     
     /* Check for divison by zero in case of center averaging  */
     if ( NumCoords1 != 0 && NumCoords2 != 0 ) {
	  Center1x /= NumCoords1 ;
	  Center1y /= NumCoords1 ;
	  Center1z /= NumCoords1 ;
	  Center2x /= NumCoords2 ;
	  Center2y /= NumCoords2 ;
	  Center2z /= NumCoords2 ;
     } 
     
     /* Center averaged distance */
     DistCent = sqrt((float)pow((Center2x-Center1x),2)+
		     (float)pow((Center2y-Center1y),2)+
		     (float)pow((Center2z-Center1z),2));
     
     if (DEBUG > 2) {
	  fprintf(DumpedNoes," DistNoe(): %2d total, sumDis: %f, aveDis: %f Coords1 %d Coords2 %d \n",
		  NumDist, DistSixth, pow(DistSixth/NumDist,(-1.0/6.0)), NumCoords1, NumCoords2);
     }
     
     key_model = tmp_model ;
     
     if (NumDist == 0) 
	  return 99.99;
     else if (strncasecmp(Average,"cen",3)==0) 
	  /* return DistTemp/NumDist ; */
	  return DistCent ;
     else if (strncasecmp(Average,"ave",3)==0)
	  return (float)pow((DistSixth/NumDist),(-1.0/6.0)) ; 
     else if (strncasecmp(Average,"sum",3)==0)
	  return (float)pow(DistSixth,(-1.0/6.0)) ; 
     else {
	  fprintf(stdout,"\n *** Something wrong in DistNoe \n");
	  return 11.11;
     }
}



void FromDianaToIUPAC(char Result[6], char Target[6], char ResNam[5]) {
     /* 
      * Convert from DYANA pseudoatom to our name convention
      * close to IUPAC
      */

     static short itm ;

     /* Convert to upper case where possible */
     for ( itm = 0; itm < 6; itm++ ) {
	  if ( isdigit(Target[itm]) ) 
	       break ;
	  if ( isalpha(Target[itm]) )
	       Target[itm] = toupper(Target[itm]) ;
     }
     
     if ( DEBUG >= 10 ) 
	  fprintf(stdout,"  FromDianaToIPAC(): dealing with atom '%s' ", Target);
     if ( Target[0] == 'Q' ) {  /* Any pseudoatom starting with Q */
	  if ( Target[1] == 'Q' ) {   /* QQG, QQD */
	       Result[0] = ' ' ;
	       Result[1] = 'H' ;
	       Result[2] = Target[2] ;
	       Result[3] = '*' ;
	       Result[4] = ' ' ;
	       Result[5] = '\0';
	  } else if ( Target[2] == '\0') {     /* QB, QG, QD, ... */
	       Result[0] = ' ' ;
	       Result[1] = 'H' ;
	       Result[2] = Target[1] ;
	       Result[3] = '*' ;
	       Result[4] = ' ' ;
	       Result[5] = '\0';
	  } else if (Target[3] == '\0') {     /* QG1, QH1, ...  */
	       Result[0] = ' ' ;
	       Result[1] = 'H' ;
	       Result[2] = Target[1] ;
	       Result[3] = Target[2] ;
	       Result[4] = '*' ;
	       Result[5] = '\0';
	  } 
     } else if ( Target[0] == 'H' && Target[1] == 'N' ) {
	  Result[0] = ' ' ;
	  Result[1] = 'H'; 
	  Result[2] = ' '; 
	  Result[3] = ' '; 
	  Result[4] = ' '; 
	  Result[5] = '\0'; 
     } else if ( (Target[0] == 'H') && 
		 Target[2] != '*' && Target[3] != '*' &&
		 Target[2] != '#' && Target[3] != '#' ) { 
	  switch (strlen(Target)) {   /* Different from Q, any H atom */
	  case 4:
	       Result[0] = Target[3];
	       Result[1] = Target[0];
	       Result[2] = Target[1];
	       Result[3] = Target[2];
	       Result[4] = ' ';
	       break;
	  case 3:
	       Result[0] = Target[2];
	       Result[1] = Target[0];
	       Result[2] = Target[1];
	       Result[3] = ' ';
	       Result[4] = ' ';
	       break;
	  case 2:
	       Result[0] = ' ';
	       Result[1] = Target[0];
	       Result[2] = Target[1];
	       Result[3] = ' ';
	       Result[4] = ' ';
	       break;
	  case 1:
	       Result[0] = ' ';
	       Result[1] = Target[0];
	       Result[2] = ' ';
	       Result[3] = ' ';
	       Result[4] = ' ';
	       break;
	  default:
	       fprintf(stderr,"\n Ummm .. something wrong in any_distance called ");
	       fprintf(stderr,"\n from SeeNoes or SeeResidualViol with atom 1  \n");
	       exit (-1);
	       break ;
	  }
	  /* Special cases for TYR, PHE, HIS, TRP */
	  if ( Target[1] == 'D' || Target[1] == 'E' || Target[1] == 'H' || Target[1] == 'Z' )
	       if (strcmp(ResNam,"PHE") == 0 || strcmp(ResNam,"TYR") == 0 || 
		   strcmp(ResNam,"TRP") == 0 || strcmp(ResNam,"HIS") == 0) {
		    Result[0] = ' ';
		    Result[1] = Target[0];
		    Result[2] = Target[1];
		    if ( Target[2] == '\0')
			 Result[3] = ' ';
		    else
			 Result[3] = Target[2];
		    Result[4] = ' ';
	       }
	  if (strcmp(ResNam,"THR")==0 && strcmp(Target,"HG1")==0) {
		  Result[0] = ' ';
		  Result[1] = Target[0] ;
		  Result[2] = Target[1] ;
		  Result[3] = Target[2] ;
		  Result[4] = ' ';
	  }
     } else {    /* Any heavy atom */
	  Result[0] = ' ';
	  Result[1] = Target[0];
	  
	  if ( Target[1] == '\0' )
	       Result[2] = ' ';
	  else
	       Result[2] = Target[1];
	  
	  if ( Target[2] == '\0' )
	       Result[3] = ' ';
	  else
	       Result[3] = Target[2];
	  
	  if ( Target[3] == '\0' )
	       Result[4] = ' ';
	  else
	       Result[4] = Target[3];
     }
     Result[5] = '\0';  /* make sure we close the string */
     if ( DEBUG >= 10 ) 
	  fprintf(stdout,"  -- Converted to '%s' \n", Result );
     return  ;
}
