/*$Id: atom_properties.c,v 3.32 2003/10/06 12:13:32 roberto Exp roberto $*/

#include "pdbstat.h"
extern FILE *efopen(char *file, char *mode);

void atom_properties(int Model) 
{
  /*
   * Sets the atom properties for atoms in model
   */
  static short int i = 0, k = 0 ;
  static struct models *model = NULL ;

  /*  for (k = 1 ; k <= total_models ; ++k) { */
  model = MODELS + Model ;
  model->PrincipalAxisFrame[0] = model->PrincipalAxisFrame[1] = model->PrincipalAxisFrame[2] = 0.0 ;
  for (i = 1 ; i <= model->number_of_atoms ; ++i) {
    model->PrincipalAxisFrame[0] = model->PrincipalAxisFrame[0] + fabs(model->PDB[i].coord[0])/model->number_of_atoms ;
    model->PrincipalAxisFrame[1] = model->PrincipalAxisFrame[1] + fabs(model->PDB[i].coord[1])/model->number_of_atoms ;
    model->PrincipalAxisFrame[2] = model->PrincipalAxisFrame[2] + fabs(model->PDB[i].coord[2])/model->number_of_atoms ;
    /*
     * Chain prop. it can be backbone or sidechain
     * as backbone we consider N,CA,C (not O).
     * At this moment we discard the hydrogens
     */
    if ((strncmp(model->PDB[i].pdb_iupac," N  ",4)==0)||
	(strncmp(model->PDB[i].pdb_iupac," NT ",4)==0)||
	(strncmp(model->PDB[i].pdb_iupac," NP ",4)==0)||
	(strncmp(model->PDB[i].pdb_iupac," C  ",4)==0)||
	(strncmp(model->PDB[i].pdb_iupac," CA ",4)==0))
      (void) strcpy(model->PDB[i].chain,"backbone") ;
    else if (model->PDB[i].pdb_iupac[1]!='H')
      (void) strcpy(model->PDB[i].chain,"sidechain") ;
    
    /*
     * Type prop. It can be heavy, hydrogen, pseudoatom or lone pair
     * Right now we consider any protein composed only of N,C,S,O
     */
    if ((model->PDB[i].pdb_iupac[1]=='H') &&
	((model->PDB[i].pdb_iupac[2]==' ') ||
	 (model->PDB[i].pdb_iupac[2]=='A') ||
	 (model->PDB[i].pdb_iupac[2]=='B') ||
	 (model->PDB[i].pdb_iupac[2]=='C') ||     /* BOC */
	 (model->PDB[i].pdb_iupac[2]=='G') ||
	 (model->PDB[i].pdb_iupac[2]=='D') ||
	 (model->PDB[i].pdb_iupac[2]=='E') ||
	 (model->PDB[i].pdb_iupac[2]=='H') ||
	 (model->PDB[i].pdb_iupac[2]=='O') ||     /* TIMG */
	 (model->PDB[i].pdb_iupac[2]=='X') ||     /* HXT */
	 (model->PDB[i].pdb_iupac[2]=='Z') ||
	 (model->PDB[i].pdb_iupac[2]=='T'))) {
      (void) strcpy(model->PDB[i].type,"hydrogen") ;
      model->PDB[i].AtomType[0] = model->PDB[i].pdb_iupac[1] ;
      model->PDB[i].AtomType[1] = '\0';
    } else if ((strncmp(model->PDB[i].pdb_iupac," N",2)==0) ||
	       (strncmp(model->PDB[i].pdb_iupac," C",2)==0) ||
	       (strncmp(model->PDB[i].pdb_iupac," C1",3)==0) ||   /* BOC */
	       (strncmp(model->PDB[i].pdb_iupac," C2",3)==0) ||   /* BOC */
	       (strncmp(model->PDB[i].pdb_iupac," C3",3)==0) ||   /* BOC */
	       (strncmp(model->PDB[i].pdb_iupac," C4",3)==0) ||   /* BOC */
	       (strncmp(model->PDB[i].pdb_iupac," S",2)==0)  ||
	       (strncmp(model->PDB[i].pdb_iupac," O1",3)==0) ||   /* BOC */
	       (strncmp(model->PDB[i].pdb_iupac," O2",3)==0) ||   /* BOC */
	       (strncmp(model->PDB[i].pdb_iupac," O",2)==0)) {
      (void) strcpy(model->PDB[i].type,"heavy") ;
      model->PDB[i].AtomType[0] = model->PDB[i].pdb_iupac[1] ;
      model->PDB[i].AtomType[1] = '\0';
    } else if (strncmp(model->PDB[i].pdb_iupac," Q",2)==0)
      (void) strcpy(model->PDB[i].type,"pseudoatom") ;
    else if (strncmp(model->PDB[i].pdb_iupac," LP",3)==0)
      (void) strcpy(model->PDB[i].type,"lone_pair") ;
    else {
      printf("> ATOM_properties: *WARNING* model %2d chain %s res %s %d atom %s %d *Unknown*.\n",
	     k, model->PDB[i].pdb_chain_id,
	     model->PDB[i].pdb_res, model->PDB[i].pdb_num_res,
	     model->PDB[i].pdb_iupac, model->PDB[i].pdb_num);
      (void) strcpy(model->PDB[i].type,"not_class") ;
    }
  }
  /* } */

  FirstResidue = (MODELS+1)->PDB[1].pdb_num_res ; 
  LastResidue = (MODELS+1)->PDB[(MODELS+1)->number_of_atoms].pdb_num_res ; 
  
  if (DEBUG > 0 )
    printf(" > ATOM_prop: Going back to MAIN ... \n");
  return ; 
}


/* =========================================================================*/
void model_properties(int modelo) 
{
     /*
      * Prints out model properties and some information
      */
     static char t[MAXWORD], PhiPsiC[31], PhiC[10], PsiC[10] ;
     static short i;
     static struct models *model ;
     
     model = MODELS + modelo ;
     fpi = efopen("DELETE.ME","w") ;
     key_model = modelo;
     phi_psi_chi(fpi,0);
     fclose(fpi);
     
     fprintf(stdout,"> MODEL_prop: MODEL number: %d, Family: %s \n", model->modelno, model->family) ;
     fprintf(stdout,"> MODEL_prop: MODEL Serial Number: %d \n", model->SerialNumber) ;
     fprintf(stdout,"> MODEL_prop: MODEL rmsd: %.2f \n", (float) model->rmsd) ;
     fprintf(stdout,"> MODEL_prop: Number of residues: %4d, Number of atoms: %6d \n",
	     model->number_of_residues, model->number_of_atoms);
     
     fprintf(stdout,"> MODEL_prop: MODEL CHAIN RESNAME SERNUM RESNUM #atoms   PHI       PSI     OMEGA\n");
     for (i = 1 ; i <= model->number_of_residues ; i++) {
	  strcpy(PhiPsiC,"");
	  
	  if (model->DIH[i].phi > 1000) 
		  	sprintf(PhiC,"   ---   ");
	  else                          
		  	sprintf(PhiC,"%9.2f", model->DIH[i].phi);
	  strcat(PhiPsiC,PhiC);
	  
	  if (model->DIH[i].psi > 1000) 
		  	sprintf(PsiC,"   ---   ");
	  else                          
		  	sprintf(PsiC,"%9.2f", model->DIH[i].psi);
	  strcat(PhiPsiC,PsiC);
	  
	  if (model->DIH[i].omega > 1000) 
		  	sprintf(PsiC,"   ---   ");
	  else                          
		  	sprintf(PsiC,"%9.2f", model->DIH[i].omega);
	  strcat(PhiPsiC,PsiC);
	  
	  fprintf(stdout,"> MODEL_prop:  %2d     %2s    %-4s %5d %6d %7d %s\n",
		  modelo, model->DIH[i].chain_id, model->DIH[i].resname, model->DIH[i].orden, 
		  model->DIH[i].resnumber, model->DIH[i].number_of_atoms, PhiPsiC);

	  if (fmod(i,23) == 0.0) {
	       fprintf(stdout,"> MODEL_prop: Type return to continue, 'q' (or 'e') + return to quit \n");
	       GetFromKeyBoard(t,MAXWORD,stdin) ;
	       if ((strncmp(t,"exit",1)==0) || (strncmp(t,"quit",1)==0))
		    return ;
	  }
     }
     return ;
}


/* =========================================================================*/
void resetResidueNumbers(char Model[10], short NewResOrig)
{
  static int i, NewStart ;
  static short ModelBegin, ModelEnd, k;
  static struct models  *model;


  if ((strncasecmp(Model,"all",3)==0) || (strcasecmp(Model,"*")==0)) {
    ModelBegin = 1;
    ModelEnd = total_models;
  } else 
    ModelBegin = ModelEnd = (short)atoi(Model) ;

  for (k = ModelBegin ; k <= ModelEnd ; ++k) {
    NewStart = NewResOrig ;
    model = MODELS+k ;
    model->starting_res = NewResOrig ;
    model->PDB[1].pdb_num_res = NewStart ;
    for (i = 2 ; i <= model->number_of_atoms ; ++i) 
      if (model->PDB[i-1].pdb_num_res_orig != model->PDB[i].pdb_num_res_orig) {
        model->PDB[i].pdb_num_res = ++NewStart ;
      } else
        model->PDB[i].pdb_num_res = NewStart;
  }

  /* Update sequence data in sequence */
  i = 1;
  model = MODELS + 1;
  NewStart = NewResOrig ;
  strcpy(seqres[NewStart],model->PDB[1].pdb_res);
  for (k=2; k<model->number_of_atoms; k++) {
	  if (model->PDB[k].pdb_num_res != model->PDB[k-1].pdb_num_res)
	  	strcpy(seqres[++NewStart],model->PDB[k].pdb_res); 	
  }
  
  /* Update names and values in DIH */
  for (k = ModelBegin ; k <= ModelEnd ; ++k) {
    NewStart = NewResOrig ;
    model = MODELS + k;
    model->DIH[1].resnumber = NewStart;
    for (i = 2 ; i <= model->number_of_residues ; ++i) 
      model->DIH[i].resnumber = ++NewStart;
  }

  FirstResidue = (MODELS+1)->PDB[1].pdb_num_res ; 
  LastResidue = (MODELS+1)->PDB[(MODELS+1)->number_of_atoms].pdb_num_res ; 
  return ;
}


/* =========================================================================*/
void restoreResidueNumbers(char Model[10])
{
  static short ModelBegin, ModelEnd, k;
  static unsigned int i;
  static struct models *model ;

  if (strncasecmp(Model,"all",3)==0) {
    ModelBegin=1;
    ModelEnd=total_models;
  } else 
    ModelBegin=ModelEnd=(short)atoi(Model) ;

  for (k = ModelBegin ; k <= ModelEnd ; ++k) {
    model = MODELS+k ;
    for (i = 1 ; i <= model->number_of_atoms ; ++i) 
      model->PDB[i].pdb_num_res = model->PDB[i].pdb_num_res_orig ;
    
    for (i = 1 ; i <= model->number_of_residues ; ++i) 
      model->DIH[i].resnumber = model->DIH[i].resnumber_orig ;
  }
  FirstResidue = (MODELS+1)->PDB[1].pdb_num_res ; 
  LastResidue = (MODELS+1)->PDB[(MODELS+1)->number_of_atoms].pdb_num_res ; 
  return ;
}
