/*$Id: calc_order_parameters.c,v 3.32 2003/10/06 12:13:32 roberto Exp roberto $*/

#include "pdbstat.h"
FILE *safeopen(char *file, char *mode, int longit);

 /*
  * Calculating order parameters for PHI and PSI . Order parameter for
  * Dihedrals PHI and PSI are calculated as described in:
  * Hyberts, SG, Golderbg, MS, Havel, TF, Wagner, G.,1992, "The solution
  * structure of eglin c based on measurements of many NOEs and coupling 
  * constants and its comparison with X-ray structures". 
  * Protein Sci., 1:736-751.
  */
void calc_order_parameters(float cutoff)
{     
     static float  order_phi[MAXRESIDUES], sinphi, cosphi ,
	  order_psi[MAXRESIDUES], sinpsi, cospsi ,
	  order_chi[MAXRESIDUES], sinchi, coschi ;
     static short i,k ;
     static struct models *Modeli, *One;
     static int TotalRanges, TmpFrom, TmpTo ;
     
     One = MODELS + 1 ;
     
     fpi = safeopen("Order_phi_psi","w",(int)strlen("Order_phi_psi"));
     
     for (i = 1; i < MAXRESIDUES ; i++) {
	  order_phi[i] = 0.0 ;
	  order_psi[i] = 0.0 ;
	  order_chi[i] = 0.0 ;
     }
     
     fprintf(stdout,"> ORDER_param: ... Model: %3d", i);
     
     for (i = 1; i <= total_models ; i++) {
	  fprintf(stdout,"\b\b\b%3d", i);
	  key_model = i;
	  phi_psi_chi(fpi,0);
     }
     
     fprintf(stdout,"\n");
     
     for (k = 2; k < One->number_of_residues ; k++) {
	  sinphi = 0.0 ;
	  sinpsi = 0.0 ;
	  sinchi = 0.0 ;
	  cosphi = 0.0 ;
	  cospsi = 0.0 ;
	  coschi = 0.0 ;
	  
	  for (i = 1 ; i <= total_models ; i++) {
	       Modeli = MODELS + i;
	       sinphi = sinphi + sin(Modeli->DIH[k].phi*radperdeg) ;
	       cosphi = cosphi + cos(Modeli->DIH[k].phi*radperdeg) ;
	       sinpsi = sinpsi + sin(Modeli->DIH[k].psi*radperdeg) ;
	       cospsi = cospsi + cos(Modeli->DIH[k].psi*radperdeg) ;
	       if (Modeli->DIH[k].chi1 != 11111.0) {
		    sinchi = sinchi + sin(Modeli->DIH[k].chi1*radperdeg) ;
		    coschi = coschi + cos(Modeli->DIH[k].chi1*radperdeg) ;
	       } else
		    sinchi = coschi = 11111.0 ;  
	  }
	  
	  order_phi[k] = sqrt(pow(sinphi,2) + pow(cosphi,2)) / total_models ;
	  order_psi[k] = sqrt(pow(sinpsi,2) + pow(cospsi,2)) / total_models ;
	  if (sinchi != 11111.0) 
	       order_chi[k] = sqrt(pow(sinchi,2) + pow(coschi,2)) / total_models ;
	  else
	       order_chi[k] = 11111.0 ;
     }
  
     fprintf(fpi,"\n    CHAIN                                     .GT.  SUM.GT.");
     fprintf(fpi,"\n RES  ID DIH    S(phi)    S(psi)    S(chi1)    %.2f     1.6 ", cutoff); 
     fprintf(fpi,"\n ----------------------------------------------------------- \n");
     
     printf("\n\n    CHAIN                                      .GT.  SUM.GT.");
     printf("\n RES  ID DIH    S(phi)    S(psi)    S(chi1)     %.2f     1.6 ", cutoff); 
     printf("\n ----------------------------------------------------------- \n");
     
     /*
      * Picking up ranges of residues with order parameter greater than
      * cutoff
      */
     RANGES.number=0 ;
     for (i = 2 ; i < One->number_of_residues ; i++) {
	  fprintf(fpi," %3s",One->DIH[i].resname);
	  fprintf(fpi," %2s",One->DIH[i].chain_id);
	  fprintf(fpi," %3d",One->DIH[i].resnumber);
	  fprintf(fpi," %9.3f",order_phi[i]);
	  fprintf(fpi," %9.3f",order_psi[i]);
	  if (order_chi[i] != 11111.0) 
	       fprintf(fpi," %9.3f",order_chi[i]);
	  else
	       fprintf(fpi,"          ");
	  
	  printf(" %3s",One->DIH[i].resname);
	  printf(" %2s",One->DIH[i].chain_id);
	  printf(" %3d",One->DIH[i].resnumber);
	  printf(" %9.3f",order_phi[i]);
	  printf(" %9.3f",order_psi[i]);
	  if (order_chi[i] != 11111.0) 
	       printf(" %9.3f",order_chi[i]);
	  else
	       printf("          ");
	  
	  if ((order_phi[i] > cutoff) && (order_psi[i] > cutoff)) {
	      if ( One->DIH[i-1].chain_id[0] != One->DIH[i].chain_id[0] ) {
		    RANGES.number++ ;
		    strcpy(RANGES.chain_id[RANGES.number],One->DIH[i].chain_id);
		    RANGES.from[RANGES.number] = One->DIH[i].resnumber ;
	       } else  if ((order_phi[i-1] < cutoff) || (order_psi[i-1] < cutoff)) {
		    RANGES.number++ ;
		    strcpy(RANGES.chain_id[RANGES.number],One->DIH[i].chain_id);
		    RANGES.from[RANGES.number] = One->DIH[i].resnumber ;
	       }
	       
	       if ((order_phi[i+1] < cutoff) || (order_psi[i+1] < cutoff)) {
		    strcpy(RANGES.chain_id[RANGES.number],One->DIH[i].chain_id);
		    RANGES.to[RANGES.number] =  One->DIH[i].resnumber ;
	       }
	       
	       if ((order_phi[i+1] > cutoff) || (order_psi[i+1] > cutoff)) {
		    strcpy(RANGES.chain_id[RANGES.number],One->DIH[i].chain_id);
		    RANGES.to[RANGES.number] =  One->DIH[i].resnumber ;
	       }
	       
	       printf("  %6d   ",One->DIH[i].resnumber);
	       fprintf(fpi,"  %6d   ",One->DIH[i].resnumber);
	  } else {
	       printf("           ");
	       fprintf(fpi,"           ");
	  }
	  
	  /*
	   * Residues with combined order parameter greater that 1.6
	   * To use in ProCheck  (or for superposition)
	   */
	  if (order_phi[i] >= 0.8 && order_psi[i] >= 0.8 ) {
	       fprintf(fpi," %5d ",One->DIH[i].resnumber);
	       printf(" %5d ",One->DIH[i].resnumber);
	  } else {
	       fprintf(fpi,"     ");
	       printf("    ");
	  }
	  fprintf(fpi,"\n");
	  printf("\n");
     }
     fprintf(fpi,"\n");
     printf("\n");
     fclose(fpi);
     
     /*
      * Place to refine the ranges, i.e., if a range includes only _one_ residue, delete it
      */
     TotalRanges = RANGES.number ;
     for (i = 1; i <= RANGES.number; i++) {
	  if (RANGES.from[i] == RANGES.to[i] ) 
		  strcpy(RANGES.quality[i],"bad");
	  else
		  strcpy(RANGES.quality[i],"good");
     }

     RANGES.number = 0;
     for (i = 1; i <= TotalRanges; i++) {
	     TmpFrom=RANGES.from[i] ;
	     TmpTo=RANGES.to[i];
	     if (strcmp(RANGES.quality[i],"bad") == 0) {
		     continue;
	     } else {
		     RANGES.number++ ;
		     RANGES.from[RANGES.number]=TmpFrom;
		     RANGES.to[RANGES.number]=TmpTo;
		     strcpy(RANGES.quality[i],"good");
	     }
     }

     /*
      * Summary of calculations with number of ranges and values
      */
     printf("\n Ranges: %d", RANGES.number);
     
     for (i = 1; i <= total_models; i++) {
	  (MODELS+i)->ranges.number = RANGES.number ;
	  for (k = 1; k <= RANGES.number; k++) {
	       strcpy((MODELS+i)->ranges.chain_id[k], RANGES.chain_id[k]);
	       (MODELS+i)->ranges.from[k] = RANGES.from[k] ;
	       (MODELS+i)->ranges.to[k] = RANGES.to[k] ;
	  }
     }
     
     for (i = 1 ; i <= RANGES.number ; i++)
	  printf("\n from: %2s %4d   to   %2s %4d", 
		 RANGES.chain_id[i], RANGES.from[i], RANGES.chain_id[i], RANGES.to[i]);
     
     return ;
}
