/*$Id: center_mass.c,v 3.32 2003/10/06 12:13:32 roberto Exp roberto $*/
#include "pdbstat.h"
extern int   is_allowed(char Id, int resnumber, int modelnumber) ;

/* ========================================================================= */
void CenterMass(int choice)
/*
  ---
  --- Calculate the center of masses of one molecule
  --- (Roberto Tejero, RTT, June 1993, CABM-Rutgers University).
  ---
*/
{
  int           count;
  float         x_cm, y_cm, z_cm;
  float         masstot, xtot, ytot, ztot;
  float         radius_of_gyration;
  double        rgi, rgitot ;
  struct models *model ;
 
  x_cm = 0.0; y_cm = 0.0; z_cm = 0.0;
  xtot = 0.0; ytot = 0.0; ztot = 0.0;
  masstot = 0.0; rgitot = 0.0;

  model = MODELS+key_model ;
  MAX = model->number_of_atoms + 1;

  if ((choice > 0) && (choice < 100)) {
    printf("> INFO_mol: Working PDB model %d in file, model no. %d \n",
	   key_model, model->modelno);
    printf("> INFO_mol: Molecule `%s' model #%d, TOTAL RESIDUES: %d\n",
	   model->family, model->modelno, model->number_of_residues);
  }
  
  /*
   * Check if we have a molecule read anyway
   */
  if (MAX <= 0) {
    printf("> INFO_mol: ** No molecule read ** \n");
    return;
  }

  /*
   * Only for average if equal 999. In this case we are trying to move
   * the structure to the center of masses using as obtained from the 
   * backbone atoms N,CA,C
   */
  if (choice == 999) {
    for(count = 1 ; count < MAX ; count++) {
      if (is_allowed('*',model->PDB[count].pdb_num_res,key_model)==0){
	if ((strncmp(model->PDB[count].pdb_iupac," N  ",4)==0)||
	    (strncmp(model->PDB[count].pdb_iupac," NT ",4)==0)||
	    (strncmp(model->PDB[count].pdb_iupac," NP ",4)==0)||
	    (strncmp(model->PDB[count].pdb_iupac," CA ",4)==0)||
	    (strncmp(model->PDB[count].pdb_iupac," C  ",4)==0)) {
	  xtot = xtot + model->PDB[count].pdb_masa * model->PDB[count].pdb_xcrd ;
	  ytot = ytot + model->PDB[count].pdb_masa * model->PDB[count].pdb_ycrd ;
	  ztot = ztot + model->PDB[count].pdb_masa * model->PDB[count].pdb_zcrd ;
	  masstot = masstot + model->PDB[count].pdb_masa ;
	} else { 
	  ;	
	}
      } else {
	;
      }
    }
    /*
     * If choice is different of 999
     */
  } else if (choice == 995) {   /* ONLY selected set */
    for(count = 1 ; count < MAX ; count++)
      if (model->PDB[count].Selected) {
	xtot = xtot + model->PDB[count].pdb_masa * model->PDB[count].pdb_xcrd ;
	ytot = ytot + model->PDB[count].pdb_masa * model->PDB[count].pdb_ycrd ;
	ztot = ztot + model->PDB[count].pdb_masa * model->PDB[count].pdb_zcrd ;
	masstot = masstot + model->PDB[count].pdb_masa ;
      }
  } else {
    for(count = 1 ; count < MAX ; count++) {
      xtot = xtot + model->PDB[count].pdb_masa * model->PDB[count].pdb_xcrd ;
      ytot = ytot + model->PDB[count].pdb_masa * model->PDB[count].pdb_ycrd ;
      ztot = ztot + model->PDB[count].pdb_masa * model->PDB[count].pdb_zcrd ;
      masstot = masstot + model->PDB[count].pdb_masa ;
    }
  }
  
  x_cm = xtot / masstot ;
  y_cm = ytot / masstot ;
  z_cm = ztot / masstot ;
  
  model->x_centermass = x_cm ;
  model->y_centermass = y_cm ;
  model->z_centermass = z_cm ;
  
  for(count = 1 ; count < MAX ; count++) {
    rgi = pow((double)(model->PDB[count].pdb_xcrd - x_cm),2) +
      pow((double)(model->PDB[count].pdb_ycrd - y_cm),2) +
      pow((double)(model->PDB[count].pdb_zcrd - z_cm),2);
    rgitot = rgitot + (double)model->PDB[count].pdb_masa * rgi ; 
  }
  model->RadiusOfGyration = radius_of_gyration = (float)sqrt(rgitot/(double)masstot);
  
  /*
   * Translation of the coordinates to the center of masses
   * NOT when asking for info, choice = 1
   */  
  if (choice != 1)
    for(count = 1 ; count < MAX ; count++) {
      model->PDB[count].pdb_xcrd -= x_cm ;
      model->PDB[count].pdb_ycrd -= y_cm ;
      model->PDB[count].pdb_zcrd -= z_cm ;
      
      model->PDB[count].coord[0] = model->PDB[count].pdb_xcrd ;
      model->PDB[count].coord[1] = model->PDB[count].pdb_ycrd ;
      model->PDB[count].coord[2] = model->PDB[count].pdb_zcrd ;
    }
  
  /*
   * Print only in case choice != 0 
   */
  if ((choice > 0) && (choice < 100)) {    
    printf("> INFO_mol: Radius of Gyration <S2> :  %9.4f  angstroms \n", radius_of_gyration);
    printf("> INFO_mol: Center of Masses: x_cm(%.3f), y_cm(%.3f), z_cm(%.3f) \n", x_cm, y_cm, z_cm);
  }
  return ;
}

