/*$Id: contacts.c,v 3.32 2003/10/06 12:13:32 roberto Exp roberto $*/

#include "pdbstat.h"

/* ========================================================================= */
void close_cntcts(float threshold_close, char atom_type[5], FILE *closecontact)
/*
 * This module searches for close contacts in the molecule. It considers close
 * contact any distance under the threshold you give to the program, so be
 * careful because it can produce a VERY LONG output for high threshold values
 *
 * (Roberto Tejero, RTT, June 1993, CABM-Rutgers University).
 *
 */
{
     static float         dd;
/*  FILE          *closecontact; */
     static int    i, k, cnstr ;
     static int    caracter ;
     static struct models *Mod ;
     static struct noe    *Noe ;
     
     Mod = MODELS+key_model ;
     cnstr = 0;
     caracter = 0;
     
     /*
      * Translating atom type (decoding)
      */
     k = (int) strlen(atom_type);
     for (i = 0 ; i <= k ; i++)
	  atom_type[i] = toupper(atom_type[i]);
     
     printf("> CLOSE_cntcts: Contacts for %.2f angs threshold and %s atom type\n",
	    threshold_close, atom_type);
     
     if (DEBUG > 2) {
	  printf(" > CLOSE_cntcts: Sample of close contacts for `%s' model #%d\n",
		 name_of_the_file, Mod->modelno);
	  printf(" > CLOSE_cntcts:        ATOM 1                ATOM 2          DIST \n");
	  printf(" > CLOSE_cntcts: ------------------      -----------------  --------\n");
     }
     
     fprintf(closecontact,"\n Close contacts for molecule `%s' model #%d, threshold: %.2f\n", 
	     name_of_the_file, Mod->modelno, threshold_close);
     fprintf(closecontact,"         ATOM 1                ATOM 2         DIST \n");
     fprintf(closecontact,"  ------------------      -----------------  -------- \n");
     
     MAX = Mod->number_of_atoms + 1;
     
     if (MAX > MAXATOMS) {
	  fprintf(stderr,"> CLOSE_cntcts: ** Error more atoms than MAXATOMS \n");
	  return;
     }
     
     if (MAX <= 0) {
	  fprintf(stderr,"> CLOSE_cntcts: ** Oh! only  few few atoms (actually zero) \n");
	  fprintf(stderr,"> CLOSE_cntcts: ** May be you have *NOT* model in memory **\n");
	  return;
     }
     
     /*
      * The atom selection begins here to prepare NOE calculation
      */
     for (i = 1 ; i < MAX ; i++)
	  for (k = i+1 ; k < MAX ; k++) {
	       if ((Mod->PDB[i].pdb_iupac[1] != 'Q') && (Mod->PDB[k].pdb_iupac[1] != 'Q')) {
		    if ((atom_type[0] == 'H') && (atom_type[1] == 'E')) {
			 if ((Mod->PDB[i].pdb_iupac[1] != 'H') && (Mod->PDB[k].pdb_iupac[1] != 'H')) 
			      goto continua ;
			 else 
			      continue;
		    }
		    if ((atom_type[0] == 'H') && (atom_type[1] != 'E')) {
			 if ((Mod->PDB[i].pdb_iupac[1] == 'H') && (Mod->PDB[k].pdb_iupac[1] == 'H')) 
			      goto continua ;
			 else 
			      continue ;
		    }
	       } else 
		    continue ;
	       
	  continua:
	       if (((strncmp(Mod->PDB[i].pdb_iupac," H ",3)==0) ||
		    (strncmp(Mod->PDB[i].pdb_iupac,"1H ",3)==0) ||
		    (strncmp(Mod->PDB[i].pdb_iupac,"2H ",3)==0) ||
		    (strncmp(Mod->PDB[i].pdb_iupac,"3H ",3)==0) ||
		    (strncmp(Mod->PDB[i].pdb_iupac," H1",3)==0) ||
		    (strncmp(Mod->PDB[i].pdb_iupac," H2",3)==0) ||
		    (strncmp(Mod->PDB[i].pdb_iupac," H3",3)==0) ||
		    (strncmp(Mod->PDB[i].pdb_iupac," HN",3)==0)) &&
		   (Mod->PDB[i].pdb_num_res == Mod->PDB[k].pdb_num_res))
		    if (strncmp(Mod->PDB[k].pdb_iupac," N ",3)==0) continue ;
	       
	       if ((strncmp(Mod->PDB[i].pdb_iupac," N ",3)==0) &&
		   (Mod->PDB[i].pdb_num_res == Mod->PDB[k].pdb_num_res))
		    if ((strncmp(Mod->PDB[k].pdb_iupac," CA",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," H ",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"1H ",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"2H ",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"3H ",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," H1",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," H2",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," H3",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," HN",3)==0)) 
			 continue;
	       
	       if ((strncmp(Mod->PDB[i].pdb_iupac," N ",3)==0) &&
		   (Mod->PDB[i].pdb_num_res == Mod->PDB[k].pdb_num_res - 1))
		    if ((strncmp(Mod->PDB[k].pdb_iupac," C ",3)==0)) 
			 continue;
	       
	       if ((strncmp(Mod->PDB[i].pdb_iupac," ND",3)==0) &&
		   (Mod->PDB[i].pdb_num_res == Mod->PDB[k].pdb_num_res))
		    if ((strncmp(Mod->PDB[k].pdb_iupac," CE",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," HD",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"1HD",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"2HD",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"3HD",3)==0)) 
			 continue ;
	       
	       if ((strncmp(Mod->PDB[i].pdb_iupac," NE",3)==0) &&
		   (Mod->PDB[i].pdb_num_res == Mod->PDB[k].pdb_num_res))
		    if ((strncmp(Mod->PDB[k].pdb_iupac," CZ",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," HE",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," CE",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"1HE",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"2HE",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"3HE",3)==0)) 
			 continue ;
	       
	       if ((strncmp(Mod->PDB[i].pdb_iupac," NZ",3)==0) &&
		   (Mod->PDB[i].pdb_num_res == Mod->PDB[k].pdb_num_res))
		    if ((strncmp(Mod->PDB[k].pdb_iupac," CZ",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," HZ",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"1HZ",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"2HZ",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"3HZ",3)==0)) 
			 continue ;
	       
	       if ((strncmp(Mod->PDB[i].pdb_iupac," NH",3)==0) &&
		   (Mod->PDB[i].pdb_num_res == Mod->PDB[k].pdb_num_res))
		    if ((strncmp(Mod->PDB[k].pdb_iupac," HH",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"1HH",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"2HH",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"3HH",3)==0)) 
			 continue ;
	       
	       if ((strncmp(Mod->PDB[i].pdb_iupac," CA",3)==0) &&
		   (Mod->PDB[i].pdb_num_res == Mod->PDB[k].pdb_num_res))
		    if ((strncmp(Mod->PDB[k].pdb_iupac," CB",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," C ",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"1HA",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"2HA",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," HA",3)==0)) 
			 continue;
	       
	       if ((strncmp(Mod->PDB[i].pdb_iupac," CB",3)==0) &&
		   (Mod->PDB[i].pdb_num_res == Mod->PDB[k].pdb_num_res))
		    if ((strncmp(Mod->PDB[k].pdb_iupac," CG",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," SG",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," OG",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," HB",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"1HB",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"2HB",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"3HB",3)==0)) 
			 continue;
	       
	       if ((strncmp(Mod->PDB[i].pdb_iupac," CG",3)==0) &&
		   (Mod->PDB[i].pdb_num_res == Mod->PDB[k].pdb_num_res))
		    if ((strncmp(Mod->PDB[k].pdb_iupac," CD",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," SD",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," ND",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," OD",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"1HG",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"2HG",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"3HG",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," HG",3)==0)) 
			 continue;
	       
	       if ((strncmp(Mod->PDB[i].pdb_iupac," CD",3)==0) &&
		   (Mod->PDB[i].pdb_num_res == Mod->PDB[k].pdb_num_res))
		    if ((strncmp(Mod->PDB[k].pdb_iupac," CE",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," NE",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," OE",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," SE",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"1HD",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"2HD",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"3HD",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," HD",3)==0)) 
			 continue;
	       
	       if ((strncmp(Mod->PDB[i].pdb_iupac," CE",3)==0) &&
		   (Mod->PDB[i].pdb_num_res == Mod->PDB[k].pdb_num_res))
		    if ((strncmp(Mod->PDB[k].pdb_iupac," CZ",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," NE",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," NZ",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," OZ",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," SZ",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"1HE",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"2HE",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"3HE",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," HE",3)==0)) 
			 continue;
	       
	       if ((strncmp(Mod->PDB[i].pdb_iupac," CZ",3)==0) &&
		   (Mod->PDB[i].pdb_num_res == Mod->PDB[k].pdb_num_res))
		    if ((strncmp(Mod->PDB[k].pdb_iupac," CH",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," NH",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," OH",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," SH",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"1HZ",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"2HZ",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"3HZ",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," HZ",3)==0)) 
			 continue;
	       
	       if ((strncmp(Mod->PDB[i].pdb_iupac," CH",3)==0) &&
		   (Mod->PDB[i].pdb_num_res == Mod->PDB[k].pdb_num_res))
		    if ((strncmp(Mod->PDB[k].pdb_iupac," HH",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"1HH",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"2HH",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"3HH",3)==0)) 
			 continue;
	       
	       if ((strncmp(Mod->PDB[i].pdb_iupac," OD2",3)==0) &&
		   (Mod->PDB[i].pdb_num_res == Mod->PDB[k].pdb_num_res))
		    if ((strncmp(Mod->PDB[k].pdb_iupac,"2HD",3)==0)) 
			 continue;
	       
	       if ((strncmp(Mod->PDB[i].pdb_iupac," OE2",3)==0) &&
		   (Mod->PDB[i].pdb_num_res == Mod->PDB[k].pdb_num_res))
		    if ((strncmp(Mod->PDB[k].pdb_iupac,"2HE",3)==0)) 
			 continue;
	       
	       if ((strncmp(Mod->PDB[i].pdb_iupac," OG",3)==0) &&
		   (Mod->PDB[i].pdb_num_res == Mod->PDB[k].pdb_num_res))
		    if ((strncmp(Mod->PDB[k].pdb_iupac," HG",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac,"1HG",3)==0)) 
			 continue;
	       
	       if ((strncmp(Mod->PDB[i].pdb_iupac," OH",3)==0) &&
		   (Mod->PDB[i].pdb_num_res == Mod->PDB[k].pdb_num_res))
		    if ((strncmp(Mod->PDB[k].pdb_iupac," HH",3)==0)) 
			 continue;
	       
	       if ((strncmp(Mod->PDB[i].pdb_iupac," C ",3)==0) &&
		   (Mod->PDB[i].pdb_num_res == Mod->PDB[k].pdb_num_res-1))
		    if ((strncmp(Mod->PDB[k].pdb_iupac," N ",3)==0)) 
			 continue;
	       
	       if ((strncmp(Mod->PDB[i].pdb_iupac," C ",3)==0) &&
		   (Mod->PDB[i].pdb_num_res == Mod->PDB[k].pdb_num_res))
		    if ((strncmp(Mod->PDB[k].pdb_iupac," O ",3)==0) ||
			(strncmp(Mod->PDB[k].pdb_iupac," OXT",4)==0)) 
			 continue;
	       
	       if ((strncmp(Mod->PDB[i].pdb_iupac," SD",3)==0) &&
		   (Mod->PDB[i].pdb_num_res == Mod->PDB[k].pdb_num_res))
		    if ((strncmp(Mod->PDB[k].pdb_iupac," CE",3)==0)) 
			 continue;
	       
	       if ((strncmp(Mod->PDB[i].pdb_iupac," SG",3)==0) &&
		   (Mod->PDB[i].pdb_num_res == Mod->PDB[k].pdb_num_res))
		    if ((strncmp(Mod->PDB[k].pdb_iupac," HG",3)==0)) 
			 continue;
	       
	       dd=(float)sqrt(pow((Mod->PDB[i].pdb_xcrd - Mod->PDB[k].pdb_xcrd),2)+
			      pow((Mod->PDB[i].pdb_ycrd - Mod->PDB[k].pdb_ycrd),2)+
			      pow((Mod->PDB[i].pdb_zcrd - Mod->PDB[k].pdb_zcrd),2));
	       
	       if (dd <= threshold_close) {
		    if ((Mod->PDB[i].pdb_num > MAXATOMS) || (Mod->PDB[k].pdb_num > MAXATOMS)) {
			 fprintf(stderr,"> CLOSE_cntcts: ** Too many atoms, I'm sorry there is an error \n");
			 fprintf(stderr,"> CLOSE_cntcts: ** Look into close_cntcts.c, atoms .gt. MAXATOMS \n");
			 return ;
		    }
		    
		    if (DEBUG > 2)    /* print every 500th */
			 if ( !(cnstr % 500) )
			      printf(" %19d %-3s %2s %3d %-5s  %5d %-3s %2s %3d %-5s %6.2f\n",
				     Mod->PDB[i].pdb_num, 
				     Mod->PDB[i].pdb_res, Mod->PDB[i].pdb_chain_id, Mod->PDB[i].pdb_num_res,
				     Mod->PDB[i].pdb_iupac, Mod->PDB[k].pdb_num, Mod->PDB[k].pdb_res,
				     Mod->PDB[k].pdb_chain_id, Mod->PDB[k].pdb_num_res, Mod->PDB[k].pdb_iupac, dd);
		    
		    
		    fprintf(closecontact, "%5d %-3s %2s %3d %-5s  %5d %-3s %2s %3d %-5s %6.2f\n",
			    Mod->PDB[i].pdb_num, Mod->PDB[i].pdb_res, Mod->PDB[i].pdb_chain_id, Mod->PDB[i].pdb_num_res,
			    Mod->PDB[i].pdb_iupac, Mod->PDB[k].pdb_num, Mod->PDB[k].pdb_res, Mod->PDB[k].pdb_chain_id,
			    Mod->PDB[k].pdb_num_res, Mod->PDB[k].pdb_iupac, dd);
		    
		    cnstr++;
		    
		    NOE = (struct noe *)realloc(NOE,sizeof(struct noe) * (cnstr + 1)) ;
		    
		    Noe = NOE+cnstr ;
		    
		    /*
		     * Fill out the NOE arrays with data
		     */
		    Noe->Number = cnstr ;
		    strcpy(Noe->chain_id_1,Mod->PDB[i].pdb_chain_id);
		    Noe->residue_number1 = Mod->PDB[i].pdb_num_res;
		    strcpy(Noe->chain_id_2,Mod->PDB[k].pdb_chain_id);
		    Noe->residue_number2 = Mod->PDB[k].pdb_num_res;
		    strcpy(Noe->residue_type1,Mod->PDB[i].pdb_res);
		    strcpy(Noe->residue_type2,Mod->PDB[k].pdb_res);
		    Noe->atom_number1 = Mod->PDB[i].pdb_num ;
		    Noe->atom_number2 = Mod->PDB[k].pdb_num ;
		    Noe->lower = 0.0 ;
		    Noe->upper = dd ;
		    Noe->volume = 0.0 ;
		    strcpy(Noe->comment," -- NOE from close contact"); 
		    
		    /* Convert PDB iupac name to NOE convention name */
		    for (caracter = 1; caracter <= 3 ; caracter++) {
			 Noe->atom_type1[caracter-1] = Mod->PDB[i].pdb_iupac[caracter] ;
			 Noe->atom_type2[caracter-1] = Mod->PDB[k].pdb_iupac[caracter] ;
		    }
		    
		    for (caracter = 0; caracter <= 4; caracter++) 
			 if ((Noe->atom_type1[caracter] == '\0') || (Noe->atom_type1[caracter] == ' ')) {
			      Noe->atom_type1[caracter] = Mod->PDB[i].pdb_iupac[0] ;
			      break ;
			 }
		    
		    for (caracter = 0; caracter <= 4; caracter++) 
			 if ((Noe->atom_type2[caracter] == '\0') || (Noe->atom_type2[caracter] == ' ')) {
			      Noe->atom_type2[caracter] = Mod->PDB[k].pdb_iupac[0] ;
			      break ;
			 }
		    
		    Noe->atom_type1[4]='\0';
		    Noe->atom_type2[4]='\0';
		    Noe->code[0]='\0';
		    Noe->Atom1Diana[0]='\0';
		    Noe->Atom1Xplor[0]='\0';
		    Noe->Atom1Congen[0]='\0';
		    Noe->Atom1Disman[0]='\0';
		    Noe->Atom1Pdb[0]='\0';
		    Noe->Atom2Diana[0]='\0';
		    Noe->Atom2Xplor[0]='\0';
		    Noe->Atom2Congen[0]='\0';
		    Noe->Atom2Disman[0]='\0';
		    Noe->Atom2Pdb[0]='\0';
		    Noe->ToPrint = TRUE ;
		    Noe->next = Noe ;
		    
		    if (DEBUG > 4)
			 printf(" Debug. CONSTR: %5d %-3s %3d %-5s %-3s %3d %-5s %6.2f\n",
				Noe->Number, Noe->residue_type1, Noe->residue_number1, Noe->atom_type1,
				Noe->residue_type2, Noe->residue_number2, Noe->atom_type2, dd);
		    
	       }
	  }
     
     if (Mod->number_of_residues <= 0) {
	  printf("> CLOSE_cntcts: ** To make PS file we need read sequence first\n");
	  return ;
     }
     printf("> CLOSE_cntcts: %d Close contacts written to file `%s' \n", cnstr, output_name);
     kount_dist = cnstr ;
     fflush(closecontact);
     return ;
}
