/*$Id: draw_maps.c,v 3.32 2003/10/06 12:13:32 roberto Exp roberto $*/

#include <unistd.h>
#include "pdbstat.h"

/* ========================================================================= */
void Draw_maps(char atom_file[MAXWORD], int residues)

/*
 * Jan, 1997. 
 * The code was revised and cleaned. 
 *  o  A bug was found in the classification of constraints as bb, bs 
 *     and ss. The symbols disagreed with the classes bb, bs and ss.
 *  o  Added filled symbols for INTRAresidue constraints.
 *
 * All known bugs have been fixed. Now we can scale the symbols and select
 * between different choices for symbols. square, circle, triangle, equis,
 * diamond. If one wants to chage the current selection should change the
 * code in this module and recompile. 
 *
 * Jul, 1993.
 * On Jul, 1, 1993 I reorganized a lot this module just trying to make it
 * portable and independent. The code now is quite different from the ori
 * ginal one. The response of the routine has been checked and is satis
 * factory. There are still a few things to do but the main part of the 
 * work has been done (I hope!). There are a few nasty dependencies, so if
 * you like to use and/or modify it you must become familiar with these
 * `peculiarities'. The variable plot[i][k] was designed to make more
 * space-efficient (low size output) but now it's not really used. It's 
 * no so important as we need to save space only when making contact map
 * from coordinates (where we suppose a lot of contacts lower than 5 A)
 *
 * Roberto Tejero, RTT.
 *
 * This routine was taken from Guy's Lab software. In George's words it was
 * developed sometime in 1989. It is used to make a PS file. It has been
 * slightly modified (some lines have been throwed away) to be implemented
 * here.
 *
 * As it only draws a point for multiple ocurrences of the same contact the
 * output has been reduced taking only the first ocurrence of the contact,
 * this little improvement saves lots of space (the size of the PS file is
 * greatly reduced), but at the same time we lost some information about
 * contacts when we make different grayscale pictures. This part need to be
 * improved and/or checked fully
 *
 * Some notes taken from: PostScript Language Reference Manual from Adobe
 * Systems Inc.
 *
 *     a) The default reference system of coordinates gives a unit along
 *        each axe (X or Y) of 1/72 inch, that means for USA paper a top
 *        of coordinates of 8.5x11 (612 for X and 792 for Y)
 *
 *     b) As far as I know the origen of coordinates is at left bottom of
 *        a vertical sheet (0,0). So right top should be 576,792
 *              
 *            0,792 _____ 612,792
 *                 |     |
 *                 |     |
 *                 |     |
 *                 |_____|
 *               0,0    612,0
 *
 * (Roberto Tejero, RTT, June 1993, CABM-Rutgers University).
 *
 */
{
     static FILE           *psptr;
     static char           suffix[3];
     static char           psstr[70];
     static char           bckbck[70], bcksdc[70], sdcsdc[70];
     static float          bb, bs, ss ;
     static float          tamano, escala ;
     static int            tic_quant ;
     static float          tic_pos ;
     static float          unit_size ;
     static float          tic_posit ;
     static float          number_posit;
     static float          font2 ;
     static float          lower_tic, upper_tic;
     static int     cnstr , vacio ;
     static int            each_quant ;
     static float          black_white ;
     static unsigned short     **plotted;
     static char           TitleLine[5][MAXWORD], t[MAXWORD];
     static short          line = 0 ;
     static int            Comienzo = 0 ;
     static int     i=0,j=0,k=0;
     static struct models *Key ;

     static div_t  Origen ;
     
     if (DEBUG >= 1) fprintf(stdout," > Draw_maps(): entering\n");
     
     Key = MODELS + key_model ;
     
     /*
      * Make size of file name workable to open a file with ext .ps
      */
     for (j = (int) strlen(atom_file) ; j >= 0 ; j--)
	  if (atom_file[j] == '/') break;
     
     for (k = j + 1 ; k <= (int) strlen(atom_file) ; k++) {
	  atom_file[i] = atom_file[k];
	  i++;
     }
     
     atom_file[i] = '\0';
     
     /*
      * Add .ps at the end of the file name
      */
     suffix[0] = '.';
     suffix[1] = 'p';
     suffix[2] = 's';
     
     strcpy(psstr, atom_file);
     strncat(psstr, suffix, 3);
     
     /*
      * Opening postscript file
      */
     printf("> DRAW_cntct: Making PS file for `%s'\n",psstr);
     if ((psptr = fopen(psstr, "w")) == NULL) {
	  printf("\n> DRAW_cntct: *** Error opening post-script file ");
	  return;
     }
     
     /* 
      * This to take into account that starting residue can be different
      * from 1
      */
     Comienzo = Key->starting_res ;
     if ( Comienzo == 1 ) {
	  Comienzo = 0;
     } else if (Comienzo > 1 ) {
	  residues = residues + Comienzo - 1  ;
     }
     if (DEBUG > 2) {
	  printf(" > draw_maps(): Residues starting at: %d\n", Comienzo);
     }
     
     /* Allocate memory for plotted */
     if (Key->number_of_residues <= 0) {
	  fprintf(stderr," ** Draw_maps():  Trying to allocate _ZERO_ residues \n");
	  fprintf(stderr," ** Draw_maps():  Try reading a sequence file first \n");
	  return ;
     }
     
     plotted = (unsigned short **) malloc(sizeof(unsigned short *)*(Key->number_of_residues+Comienzo+3));
     for (i = 0; i <= Key->number_of_residues+Comienzo+2; i++)
	  plotted[i] = (unsigned short *) malloc(sizeof(unsigned short)*(Key->number_of_residues+Comienzo+3));
     
     
     /* Some initialization */
     each_quant = 5;
     tic_posit = 0.0;
     number_posit = 0.0;
     black_white = 0.0 ;
     line = 0 ;
     black_white = black_white + 1.0; /*  0.0 for black, 1.0 for white */
     /*  others gray   */
     vacio = 1 ;    /* should be zero if you want the figure filled up */
     font2 = 70.0 ; /* font size for numbers in frame */
     
     /*
      * These values are for different gray scale. If you
      * want only black squares turn all values to 0.0
      *
      * Look down in the code as this can be little confusing.
      * For intraresidue constraints we define vacio = 0.0 and
      * bb = bs = ss = 0.0
      *
      * Normal values
      *
      *  bb = 0.20 ;
      *  bs = 0.50 ;
      *  ss = 0.80 ;
      *  
      */
     bb = 0.20 ;
     bs = 0.50 ;
     ss = 0.80 ;
     
     /*
      * This is for the square size. If you want squares with
      * no separation in between (old style , EGF anf TGF papers)
      * make tamano = 1.0;
      */
     tamano = 0.70 ;
     escala = 1.00 ;    /* spanish for scale. It measures the scale we */
     /* should apply to the points out of the frame */
     
     /*
      * Get title for the plot
      */
     printf("> DRAW_maps: Please enter title for plot (up to 5 lines) \n"); 
     printf("> DRAW_maps: To finish just type an asterisk '*' by itself \n");   
     while ( line < 4 ) {
	  GetFromKeyBoard(t, MAXWORD, stdin);
	  if ( strcmp(t,"*") ==0 ) break ;
	  line++ ;
	  (void) strcpy(TitleLine[line],t) ;
     }
     
     /*
      * Defaults for symbols
      */
     strcpy(bckbck," backbone - backbone (filled diamond if intraresidue) ");
     strcpy(bcksdc," backbone - side chain (filled if intraresidue)");
     strcpy(sdcsdc," side chain - side chain (filled if intraresidue)");
     
     if ( (!sequence_true) && (Key->number_of_residues <= 0)) {
	  printf("> DRAW_cntct: ** No info about sequence to make contact map!! \n");
	  return ;
     }
     
     i = 0;
     
     /*
      * Selecting number of residues for plotting purposes
      */
     if (residues > 80) 
	  each_quant = 10;
     else 
	  each_quant = 5 ;
     
     /*
      * Initializing data
      */
     for (j = 1 ; j <= Key->number_of_residues+Comienzo ; j++)
	  for (k = 1 ; k <= Key->number_of_residues+Comienzo ; k++)
	       plotted[j][k] =  99;
     
     /*
      * Header for post-script file and frame of the picture
      */
     fprintf(psptr, "%%! Header file \n");
     fprintf(psptr, "%%%% Creator: PDBSTAT -- Roberto Tejero \n");
     fprintf(psptr, "%%%% For user: %s\n", getlogin());
     fprintf(psptr, "%%%% Title: %s\n", atom_file);
     fprintf(psptr, "gsave\n");
     fprintf(psptr, "newpath\n");
     fprintf(psptr, "/Helvetica-Bold findfont %d scalefont setfont \n", 12);
     
     /*
      * Writing title for plot
      */
     for (j=1 ; j<=line ;j++) 
	  fprintf(psptr, "%.3f %.3f moveto (%s) show \n", 86.0, 724.0-j*12 , TitleLine[j]);
     
     fprintf(psptr, "/Courier findfont %d scalefont setfont\n", 20);
     fprintf(psptr, "%.2f %.2f moveto (Residue number) show\n", 252.00, 40.00);
     fprintf(psptr, "/Courier findfont %d scalefont setfont\n", 20);
     fprintf(psptr, "%.2f %.2f moveto 90.00 rotate (Residue number) show -90.00 rotate\n", 45.00, 252.00);
     
     /* 
      * Library of shapes
      */
     
     /*  --- This is for a circle --- */
     fprintf(psptr, "/circle\n");
     fprintf(psptr, "  { %.3f %.3f %.3f %.3f %.3f arc\n", 0.0, 0.0, tamano * 0.5, 0.0, 360.0);
     fprintf(psptr, "  closepath  } def\n");
     
     /*  --- This is for a square --- */
     fprintf(psptr, "/square\n");
     fprintf(psptr, "  { %.3f %.3f rlineto\n", -tamano, 0.00);
     fprintf(psptr, "  %.3f %.3f rlineto\n", 0.00, -tamano);
     fprintf(psptr, "  %.3f %.3f rlineto\n", tamano, 0.0);
     fprintf(psptr, "  closepath  } def\n");
     
     /*  --- This is for a triangle --- */
     fprintf(psptr, "/triangle\n");
     fprintf(psptr, "  { %.3f %.3f rmoveto\n", -tamano * 0.5, 0.00);
     fprintf(psptr, "  %.3f %.3f rlineto\n", -tamano * 0.5, -tamano);
     fprintf(psptr, "  %.3f %.3f rlineto\n", tamano, 0.00);
     fprintf(psptr, "  closepath  } def\n");
     
     /*  --- This is for a equis --- */
     fprintf(psptr, "/equis\n");
     fprintf(psptr, " { %.3f %.3f rlineto\n", -tamano, -tamano);
     fprintf(psptr, "  %.3f %.3f rmoveto\n", tamano, 0.00);
     fprintf(psptr, "  %.3f %.3f rlineto  } def\n", -tamano, tamano);
     
     /*  --- This is for a diamond --- */
     fprintf(psptr, "/diamond\n");
     fprintf(psptr, " { %.3f %.3f rmoveto\n", -tamano * 0.5, 0.0);
     fprintf(psptr, "  %.3f %.3f rlineto\n", -tamano * 0.5, -tamano * 0.5);
     fprintf(psptr, "  %.3f %.3f rlineto\n", tamano * 0.5, -tamano * 0.5);
     fprintf(psptr, "  %.3f %.3f rlineto\n", tamano * 0.5, tamano * 0.5);
     fprintf(psptr, "  closepath  } def\n");
     
     /*
      * Explanation of symbols we are going to use.
      * This part goes into the file explaining the different symbols
      */
     escala = 10.0 ;
     if ((bb != 0.0) && (bs != 0.0) &&  (ss != 0.0)) {
	  
	  /* ---  backbone - backbone --- is a equis */
	  fprintf(psptr, "%.3f %.3f moveto \n", 94.0, 647.0);
	  fprintf(psptr, "%.9f %.9f scale\n equis\n %.9f %.9f scale\n", 
		  escala, escala, 1.0/escala, 1.0/escala);
	  if (vacio == 1) {
	       fprintf(psptr, " %.9f %.9f scale\n", 0.2, 0.2);
	       fprintf(psptr, " stroke\n");
	       fprintf(psptr, " %.9f %.9f scale\n", 1.0/0.20, 1.0/0.20);
	  } else {
	       fprintf(psptr, "%.2f setgray\n", bb);
	       fprintf(psptr, " fill\n");
	  }
	  fprintf(psptr, "%.2f setgray\n", 0.0);
	  fprintf(psptr, "%.3f %.3f moveto\n", 100.0, 640.0);
	  fprintf(psptr, "/Courier-Bold findfont\n");
	  fprintf(psptr, "%d scalefont\n", 12);
	  fprintf(psptr, "setfont\n");
	  fprintf(psptr, "(%s) show\n", bckbck);
	  
	  /* ---  backbone - side chain --- is a circle*/
	  fprintf(psptr, "%.3f %.3f moveto \n", 94.0, 632.0);
	  fprintf(psptr, "%.3f %.3f moveto \n", 
		  94.0, 632.0 - tamano * escala * 0.5);
	  fprintf(psptr, "%.3f %.3f translate \n", 
		  94.0 - tamano * escala * 0.5, 
		  632.0 - tamano * escala * 0.5);
	  fprintf(psptr, "%.9f %.9f scale\n circle\n %.9f %.9f scale\n",
		  escala, escala, 1.0/escala, 1.0/escala);
	  if (vacio == 1) {
	       fprintf(psptr, " %.9f %.9f scale\n", 0.2, 0.2);
	       fprintf(psptr, "stroke\n");
	       fprintf(psptr, " %.9f %.9f scale\n", 1.0/0.20, 1.0/0.20);
	  } else {
	       fprintf(psptr, "%.2f setgray\n", bs);
	       fprintf(psptr, "fill\n");
	  }
	  fprintf(psptr, "%.3f %.3f translate\n", 
		  -94.0+tamano*5.0,-632.0+tamano*5.0);
	  fprintf(psptr, "%.2f setgray\n", 0.0);
	  fprintf(psptr, "%.3f %.3f moveto\n", 100.0, 625.0);
	  fprintf(psptr, "/Courier-Bold findfont\n");
	  fprintf(psptr, "%d scalefont\n", 12);
	  fprintf(psptr, "setfont\n");
	  fprintf(psptr, "(%s) show\n", bcksdc);
	  
	  /* ---  side chain - side chain --- is a square*/
	  fprintf(psptr, "%.3f %.3f moveto\n", 94.0, 617.0);
	  fprintf(psptr, "%.9f %.9f scale\n square\n %.9f %.9f scale\n", 
		  escala, escala, 1.0/escala, 1.0/escala);    
	  if (vacio == 1) {
	       fprintf(psptr, " %.9f %.9f scale\n", 0.2, 0.2);
	       fprintf(psptr, "stroke\n");
	       fprintf(psptr, " %.9f %.9f scale\n", 1.0/0.20, 1.0/0.20);
	  } else {
	       fprintf(psptr, "%.2f setgray\n", ss);
	       fprintf(psptr, "fill\n");
	  }
	  fprintf(psptr, "%.2f setgray\n", 0.0);
	  fprintf(psptr, "%.3f %.3f moveto\n", 100.0, 610.0);
	  fprintf(psptr, "/Courier-Bold findfont\n");
	  fprintf(psptr, "%d scalefont\n", 12);
	  fprintf(psptr, "setfont\n");
	  fprintf(psptr, "(%s) show\n", sdcsdc);
     }
     
     /*
      * Drawing the frame for the figure. The standard
      * dimensions are 486 x 486 as we have translated
      * the origin to 90,90 (that makes 576 x 576 for 
      * the piece of paper) 576 / 72 = 8 inches
      */
     fprintf(psptr, "newpath\n");
     fprintf(psptr, "%.3f %.3f translate\n", 90.0, 90.0);
     fprintf(psptr, "%.3f %.3f moveto\n", 0.0, 0.0);
     fprintf(psptr, "%.3f %.3f rlineto\n", 0.0, 486.0);
     fprintf(psptr, "%.3f %.3f rlineto\n", 486.0, 0.0);
     fprintf(psptr, "%.3f %.3f rlineto\n", 0.0, -486.0);
     fprintf(psptr, "closepath\n");
     
     /*
      * Changing scale to make thin lines. As we are making
      * the scale 5 times bigger, the dimension is 5 times
      * also 2430 = 5 * 486
      */
     fprintf(psptr, "%.9f %.9f scale\n", 0.2, 0.2);
     unit_size = 2430.0 / (residues-Comienzo+1) ;
     
     /*
      * Measuring how many ticks and their positions
      * and then plotting them
      */
     tic_quant = residues-Comienzo+1 ;
     tic_pos = unit_size / 2.0 ;
     
     for (j = 0; j <= tic_quant-1; j++) {
	  if (((j+Comienzo) % each_quant) == 0) {
	       upper_tic = 2470.0;
	       lower_tic = -40.0;
	  } else if ((each_quant != 5) && ((j+Comienzo) % 5) == 0) {
	       upper_tic = 2455.0;
	       lower_tic = -25.0;
	  } else {
	       upper_tic = 2440.0;
	       lower_tic = -10.0;
	  }
	  fprintf(psptr, "%.3f %.3f moveto\n", tic_pos, lower_tic);
	  fprintf(psptr, "%.3f %.3f lineto\n", tic_pos, 0.0);
	  fprintf(psptr, "%.3f %.3f moveto\n", lower_tic, tic_pos);
	  fprintf(psptr, "%.3f %.3f lineto\n", 0.0, tic_pos);
	  fprintf(psptr, "%.3f %.3f moveto\n", tic_pos, 2430.0);
	  fprintf(psptr, "%.3f %.3f lineto\n", tic_pos, upper_tic);
	  fprintf(psptr, "%.3f %.3f moveto\n", 2430.0, tic_pos);
	  fprintf(psptr, "%.3f %.3f lineto\n", upper_tic, tic_pos);
	  tic_pos = tic_pos + unit_size;
     }
     fprintf(psptr, "stroke\n");
     
     /*
      * Plotting the numbers in the axis
      */
     fprintf(psptr, "newpath\n");
     fprintf(psptr, "/Courier findfont\n");
     fprintf(psptr, " %.3f scalefont setfont\n", font2);
     
     Origen = div(Comienzo,each_quant);
     tic_pos = unit_size * (each_quant - 1) - 
	  ((Comienzo - Origen.quot * each_quant) -1) * unit_size ;
     tic_quant = (residues-Comienzo) / each_quant ;
     
     for (j = 1; j <= tic_quant; j++) {
	  if (j * each_quant + Comienzo > 999) {
	       tic_posit = tic_pos + unit_size / 2.0 - font2 * 1.00 ;       
	       number_posit = -95.0 - font2 * 1.60 ;
	  } else if (j * each_quant + Comienzo > 99) {
	       tic_posit = tic_pos + unit_size / 2.0 - font2 * 0.75 ;       
	       number_posit = -95.0 - font2 * 1.00 ;
	  } else if (j * each_quant + Comienzo > 9) {
	       tic_posit = tic_pos + unit_size / 2.0 - font2 * 0.50 ;
	       number_posit = -95.0 - font2 * 0.40 ;
	  } else if (j * each_quant + Comienzo >= 5) {
	       tic_posit = tic_pos + unit_size / 2.0 - font2 * 0.25 ;
	       number_posit = -95.0 ;
	  } else {
	       printf("> DRAW_maps: ** Something wrong !\n");
	       return;
	  }
	  
	  /*  X axe */
	  fprintf(psptr, " %.3f %.3f  moveto", tic_posit, -100.0);
	  fprintf(psptr, " (%d) show\n", 
		  (int)(0.5 + (tic_pos + unit_size) / unit_size) + Comienzo -1) ;
	  
	  /*  Y axe */
	  fprintf(psptr, " %.3f %.3f moveto", 
		  number_posit, tic_pos + unit_size / 2.0 - font2 * 0.25 );
	  fprintf(psptr, " (%d) show\n", 
		  (int)(0.5 + (tic_pos + unit_size) / unit_size) + Comienzo -1) ;
	  
	  /* Update tic_pos */
	  tic_pos = tic_pos + (unit_size * each_quant);
     }
     
     /* Scale the whole thing as the number of residues */
     fprintf(psptr, " %.9f %.9f scale\n", unit_size, unit_size);
     
     /* Building contact map. */
     fprintf(psptr, " newpath\n");
     
     for (cnstr = 1 ; cnstr <= kount_dist ; cnstr++) {
	  j = NOE[cnstr].residue_number1-Comienzo+1 ;
	  k = NOE[cnstr].residue_number2-Comienzo+1 ;
	  
	  if (j <= 0 || k <= 0) {
	       fprintf(stderr,"> DRAW_maps: ** Warning indices .le. 0 for CONSTR %d\n",cnstr);
	       j = 0 ;
	       k = 0 ;
	  }
	  
	  /* Intraresidue are drawn as solid figures */
	  if ( j == k ) {    /* Same residue, intraresidue constraint */
	       ss = 0.0 ;
	       bb = 0.0 ;
	       bs = 0.0 ;
	       vacio = 0 ;
	  } else {
	       ss = 0.80 ;
	       bb = 0.20 ;
	       bs = 0.50 ;
	       vacio = 1 ;
	  }
	  
	  if ((plotted[j][k] != 1) &&
	      (j <= Key->number_of_residues+Comienzo) &&
	      (k <= Key->number_of_residues+Comienzo)) {
	       
	       /* backbone - backbone - equis */
	       if (NOE[cnstr].code[0] == 'a') {
		    fprintf(psptr, " %.3f %.3f moveto\n", 
			    (float)j - (1.0 - tamano)/2.0, 
			    (float)k - (1.0 - tamano)/2.0);
		    if ( j == k )
			 fprintf(psptr, " diamond\n");
		    else
			 fprintf(psptr, " equis\n");
	       }
	       
	       /* side chain - side chain - square */
	       if (NOE[cnstr].code[0] == 'c')   {
		    fprintf(psptr, " %.3f %.3f moveto\n", 
			    (float)j - (1.0 - tamano)/2.0, 
			    (float)k - (1.0 - tamano)/2.0);
		    fprintf(psptr, " square\n");    
	       }
	       
	       /* backbone - side chain - circle */
	       if (NOE[cnstr].code[0] == 'b')  {
		    fprintf(psptr, " %.3f %.3f moveto\n",
			    (float)j - (1.0 - tamano)/2.0, 
			    (float)k - (1.0 - tamano)/2.0 - tamano * 0.5 );
		    fprintf(psptr, " %.3f %.3f translate\n", 
			    (float)j - (1.0 - tamano)/2.0 - tamano * 0.5 , 
			    (float)k - (1.0 - tamano)/2.0 - tamano * 0.5 );
		    fprintf(psptr, " circle\n");
		    fprintf(psptr, " %.3f %.3f translate\n",
			    -(float)j + (1.0 - tamano)/2.0 + tamano * 0.5 , 
			    -(float)k + (1.0 - tamano)/2.0 + tamano * 0.5 );
	       }
	       
	       /* 
		* Other part of the diagonal
		*/
	       
	       /* backbone - backbone - equis */
	       if (NOE[cnstr].code[0] == 'a')  {
		    fprintf(psptr, " %.3f %.3f moveto\n", 
			    (float)k - (1.0 - tamano)/2.0, 
			    (float)j - (1.0 - tamano)/2.0);
		    if ( j == k )
			 fprintf(psptr, " diamond\n");
		    else
			 fprintf(psptr, " equis\n");
	       }
	       
	       /* side chain - side chain - square */	 
	       if (NOE[cnstr].code[0] == 'c')  {
		    fprintf(psptr, " %.3f %.3f moveto\n", 
			    (float)k - (1.0 - tamano)/2.0, 
			    (float)j - (1.0 - tamano)/2.0);
		    fprintf(psptr, " square\n");    
	       }
	       
	       /* backbone - side chain - circle */
	       if (NOE[cnstr].code[0] == 'b')  {
		    fprintf(psptr, " %.3f %.3f moveto \n",
			    (float)k - (1.0 - tamano)/2.0, 
			    (float)j - (1.0 - tamano)/2.0 - tamano * 0.5 );
		    fprintf(psptr, " %.3f %.3f translate \n", 
			    (float)k - (1.0 - tamano)/2.0 - tamano * 0.5 , 
			    (float)j - (1.0 - tamano)/2.0 - tamano * 0.5 );
		    fprintf(psptr, " circle \n ");
		    fprintf(psptr, " %.3f %.3f translate \n",
			    -(float)k + (1.0 - tamano)/2.0 + tamano * 0.5 , 
			    -(float)j + (1.0 - tamano)/2.0 + tamano * 0.5 );
	       }
	       
	       /*
		* Draw the symbols
		*/
	       if (vacio == 1) {
		    fprintf(psptr, " %.9f %.9f scale", 1.0/unit_size, 1.0/unit_size);
		    fprintf(psptr, " stroke\n");
		    fprintf(psptr, " %.9f %.9f scale", unit_size, unit_size);
	       } else {
		    fprintf(psptr, " %.2f setgray\n", ss);
		    fprintf(psptr, "fill\n");
	       }
	       
	  }
     }
     
     fprintf(psptr, " showpage\n grestore\n");
     fprintf(psptr, "%%EOF\n");
     fclose(psptr);
     for (i = 0; i <= Key->number_of_residues+Comienzo+2; i++)
	  free(plotted[i]);
     free(plotted);
     if (DEBUG >= 1) fprintf(stdout," > Draw_maps(): leaving \n");
     return ;
}
