/* $Id: homology.c,v 3.32 2003/10/06 12:13:32 roberto Exp roberto $*/

#include "pdbstat.h"
#include <unistd.h>

#define MaxAtoms      20000
#define CONSTRAINTS  200000
#define LOWEST  1.90
#define ALLOW   0.10

/* External routines needed */
extern void  write_sequence(FILE *rtto, char formato[10], char type[10]);
extern void  write_constraints_diana(FILE *rrto);
extern void  write_constraints_diana_lol(FILE *rrto);
extern struct Library *Lib;
extern int   MaxLib ;
extern int   GetLibraryData(FILE *InputFile) ;      
extern FILE  *libfopen(char *, char *);
extern void  AssignSequenceNameToNOE(void) ;
extern void  AssignLibraryNames(struct noe *Puntero,int LibRecords,int option, char KeyAss[5]) ;
extern void  CheckLibraryNames(int LibRecords,int option) ; 
extern float ran1(long *idum) ;
extern char  *OneToThree(char Name[5]);
extern FILE  *safeopen(char *, char *, int longit);
extern void  InitNoeRecord(struct noe *Noe) ;
extern FILE  *efopen(char *, char *);

/* Local routines and definitions */
void homol_distance (int atom1, int atom2, int ik, int senal, FILE *salida, float LONGEST) ;

int tmpno = 99999;

/*
 * This routine creates (gets) a number of random distances from a structure (template, set of
 * coordinates).  It is used as the first step for homology modelling.  Its output goes to 
 * RandomDistances and also is stored in Noe structure (in memory) to do the translation needed
 * (ChangeAtoms) to convert the constraints in useful information for the target.
 *
 * It has been made almost fully automatic to run for HOMA. In doing this work residues 
 * are renumbered starting at one for the target no matter what is the real starting
 * number. 
 *
 * (c) Roberto Tejero, University of Valencia.
 *     Roberto Tejero, CABM, Rutgers University, 2003 -- improvements and HOMA.
 */
void GetDistancesFromTemplate(unsigned short TotalAtoms, int NDis, float LONGEST, long semilla)
{
     int   LibRecords = 0;
     int   senal, MAXCONS, it, jt ;
     int   count, atom1, atom2, small, large;
     int   **cons;
     char  name[MAXWORD], OutName[MAXWORD];
     static struct models   *model ;
     static struct atom_pdb *At1, *At2 ;
     static struct noe *Noe = NULL;
     FILE  *salida,  *LibFile ;

     TotalAtoms = TotalAtoms-1 ;
     
     /*
      * On Jun, 1997 it was changed to dynamic allocation of memory.
      * Now there is no restriction about atoms or constraints. Only
      * available memory should be the restriction
      */
     cons = (int **)malloc(sizeof(int) * (TotalAtoms+1)) ;
     for (atom1 = 0 ; atom1 <= TotalAtoms ; atom1++) {
	  cons[atom1] = (int *) malloc(sizeof(int) * (TotalAtoms+1)) ;
     }
     
     /*
      * Just to avoid some really high number of atoms
      */
     if (TotalAtoms > MaxAtoms) {
	  free(cons);
	  printf("> HOMOL: # of atoms: `%d' greater than maximum allowed `%d'\n",
		 (int)TotalAtoms, (int)MaxAtoms);
	  printf("> HOMOL: ... returning to main menu ... \n");
	  return ;
     }
     
     senal = 2 ;
     strcpy(OutName,"RandomDistances");
     salida = safeopen(OutName,"w",(int)strlen(OutName)) ; 
     
     if (senal == 1)
	  fprintf(salida,"!BIOSYM restraint 1 \n#NOE_distance \n");
     if ((senal == 2) || (senal == 3) || (senal == 4) )  {
	  fprintf(salida,"\n  NMR constraint file\n\n*\nTYPE NOE\nWEIGHT 1.0\n");
	  fprintf(salida,"!\n");
	  fprintf(salida,"!    ATOM SPEC 1        ATOM SPEC 2      LOW BOUND "\
		  "   UPP BOUND     cns #  atom #s\n");
	  fprintf(salida,"!\n");
     }
     
     /*
      * Number of desired random constraints
      */
     strcpy(name,"");
     
     if ( NDis <= 0 ) {
	  printf (" How many constraints dou you want [10000] ? : ");
	  GetFromKeyBoard(name, MAXWORD, stdin);
	  if (strcmp(name,"")==0) strcpy(name,"10000");
	  MAXCONS=atoi(name);
     } else {
	  MAXCONS = NDis ;
     }
     
     /*
      * Check to avoid any huge number of constraints ( currently 200000 )
      */
     if (MAXCONS > CONSTRAINTS) {
	  fprintf(stderr, " More constraints than currently allowed (%d). "
		  "*CHANGE CONSTRAINTS # and re-run command* \n", CONSTRAINTS);
	  return ;
     }
     
     /*
      * Longest constraint
      */
     strcpy(name,"");
     if ( LONGEST <= 0 ) {
	  printf(" What is the longest constraint do you want [99.99]? = ");
	  GetFromKeyBoard(name, MAXWORD, stdin);
	  if (strcmp(name,"")==0) strcpy(name,"99.99");
	  LONGEST=atof(name);
     }
     
     /*
      * Seed for random generator.
      * In Numerical recipies they say that we should use a negative
      * number, so the default is -32
      */
     strcpy(name,"");
     if ( semilla == 0 ) {
	  printf("\n What seed do you want for random generator [-32]? = ");
	  GetFromKeyBoard(name, MAXWORD, stdin);
	  if (strcmp(name,"")==0) strcpy(name,"-32") ;
	  semilla = atol(name) ;
     }
     printf("\n The value of seed is %d", (int) semilla);
     
     /* --------------  Selection of the distances -----------------   */
     
     for (atom1 = 1 ; atom1 <= TotalAtoms ; atom1++)
	  for (atom2 = 1 ; atom2 <= TotalAtoms ; atom2++)
	       cons[atom1][atom2]=0;
     
     count=0;
     
     printf (" Generating random constraints \n");
     printf (" Wait please .......\n");
     
     /*
      *  Generation of random constraints begins here, in this while.
      *  It runs through 1 to MAXCONS. After generating the random distance
      *  it passes three checks.
      */
     
     fprintf(stdout," Working in constraint ... ");
     
     while (count <= (MAXCONS -1) ) {
	  
     otradistancia:
	  
	  /* 
	   *  Random generation of the numbers for the TWO atoms of the distance.
	   *  This has been changed a little bit from the original code. Now the
	   *  program first locate the maximum random number RAND_MAX and then
	   *  generates the random number in the range 1 ... RAND_MAX
	   */      
	  atom1 = (int) (TotalAtoms * ran1(&semilla)) +1 ; 
	  atom2 = (int) (TotalAtoms * ran1(&semilla)) +1 ; 
	  
	  /*
	   *  Three checks are passed by the distance.
	   *  1. If atom1 or atom2 are greater than maximum number of atoms
	   *  2. If the distance is already done
	   *  3. If the distance is greater than the biggest we want
	   */
	  
	  if ((atom1 > TotalAtoms) || (atom2 > TotalAtoms)) goto otradistancia ;
	  
	  /*
	   * Avoid any hydrogen that can be left in coordinates
	   * We like to have *ONLY* constraints between non-hydrogen atoms
	   */
	  model = MODELS + key_model ;
	  At1 = ((MODELS + key_model)->PDB) + atom1 ;
	  At2 = ((MODELS + key_model)->PDB) + atom2 ;
	  
	  if ((At1->pdb_iupac[1]=='H') || (At2->pdb_iupac[1]=='H'))
	       goto otradistancia ;
	  
	  if ((cons[atom1][atom2]!=0) || (cons[atom2][atom1]!=0)) {
	       if (DEBUG > 0) 
		    printf("> HOMOL: Avoiding repeated distance: %5d-%d\n", atom1,atom2);
	       goto otradistancia;
	  }
	  
	  if ((distance(atom1,atom2) > LONGEST) || (distance(atom1,atom2) < LOWEST))
	       goto otradistancia ;
	  
	  /* Selecting distances */
	  
	  if (cons[atom1][atom2] == 0) {
	       if (abs(At1->pdb_num_res - At2->pdb_num_res) == 1) {
		    
		    if (At1->pdb_num_res < At2->pdb_num_res) {
			 small = atom1;
			 large = atom2;
		    } else { 
			 small = atom2;
			 large = atom1;
		    }
		    
		    if (strncmp(model->PDB[small].pdb_iupac, " CA",4) == 0 && 
			strncmp(model->PDB[large].pdb_iupac, " N ",4) != 0) { 
			 cons[atom1][atom2] = cons[atom2][atom1] = 1;
			 count = count+1;
		    } else if (strncmp(model->PDB[small].pdb_iupac," C ",4)==0 && 
			       strncmp(model->PDB[large].pdb_iupac," N ",4)!=0) { 
			 cons[atom1][atom2] = cons[atom2][atom1] = 1;
			 count = count+1;
		    } else if (strncmp(model->PDB[large].pdb_iupac," N ",4)==0 && 
			       strncmp(model->PDB[small].pdb_iupac," O ",4)!=0) { 
			 cons[atom1][atom2] = cons[atom2][atom1] = 1;
			 count = count+1;
		    } else  {
			 cons[atom1][atom2] = cons[atom2][atom1] = 1;
			 count = count + 1;
		    }
	       } else if (At1->pdb_num_res == At2->pdb_num_res) {
		    if (((strncmp(At1->pdb_iupac, " CA",4)== 0 &&
			  strncmp(At2->pdb_iupac, " CB",4)!= 0 && strncmp(At2->pdb_iupac, " CG",4)!= 0 && 
			  strncmp(At2->pdb_iupac, " CA",4)!= 0 && strncmp(At2->pdb_iupac, " N ",4)!= 0 &&
			  strncmp(At2->pdb_iupac, " C ",4)!= 0 && strncmp(At2->pdb_iupac, " O ",4)!= 0) && 
			 (strncmp(At1->pdb_iupac, " C ",4)== 0 &&
			  strncmp(At2->pdb_iupac, " CB",4)!= 0 && strncmp(At2->pdb_iupac, " C ",4)!= 0 &&
			  strncmp(At2->pdb_iupac, " CA",4)!= 0 && strncmp(At2->pdb_iupac, " N ",4)!= 0 &&
			  strncmp(At2->pdb_iupac, " O ",4)!= 0) &&
			 (strncmp(At1->pdb_iupac, " N ",4)== 0 &&
			  strncmp(At2->pdb_iupac, " N ",4)!= 0 && strncmp(At2->pdb_iupac, " CB",4)!= 0 &&
			  strncmp(At2->pdb_iupac, " CA",4)!= 0 && strncmp(At2->pdb_iupac, " C ",4)!= 0) &&
			 (strncmp(At1->pdb_iupac, " O ",4)== 0 &&
			  strncmp(At2->pdb_iupac, " CA",4)!= 0 && strncmp(At2->pdb_iupac, " O ",4)!= 0 &&
			  strncmp(At2->pdb_iupac, " C ",4)!= 0))){
			 cons[atom1][atom2] = cons[atom2][atom1] = 1;
			 count = count + 1;
		    }
	       } else  {
		    cons[atom1][atom2] = cons[atom2][atom1] = 1;
		    count = count + 1;
	       } 
	  }      
	  
	  if (DEBUG >=0 && fmod(count,10)==0.0)
	       fprintf(stdout,"%6d\b\b\b\b\b\b", count);
	  fflush(stdout);
     }           /* end of while (count <= (MAXCONS -1) ) */
     
     fprintf(stdout,"\n We have processed a total of %d distances\n", count);
     
     count=0; 
     fprintf(stdout," Loading GenCons library data ... \n");
     
     if (Lib == NULL) {  /* Load Library Data */
	  LibFile = libfopen("GenCons.Lib","r");
	  LibRecords = GetLibraryData(LibFile);
	  fclose(LibFile);
     } else
	  LibRecords = MaxLib ;
     
     if ( LibRecords < 0 ) {
	  fprintf(stdout, "\n Something wrong (2357) in homology routine\n\n");
	  exit (-1);
     }
     
     /* Ordering and printing of distances */
     fprintf(stdout," Ordering and printing distances in RandomDistances file ... \n");
     for (atom1 = 1; atom1 <= TotalAtoms; atom1++)
	  for (atom2 = atom1; atom2 <= TotalAtoms; atom2++) {      
	       if (cons[atom1][atom2] == 1 ) {
		    count = count + 1;
		    
		    /* Using my Noe structure to store NOE data */
		    NOE = (struct noe *)realloc(NOE,sizeof(struct noe)*(count+1)) ;
		    Noe = NOE + count ;
		    
		    InitNoeRecord(Noe);
		    
		    At1 = ((MODELS+key_model)->PDB)+atom1;
		    At2 = ((MODELS+key_model)->PDB)+atom2;
		    
		    /* storing atom1 */
		    Noe->residue_number1 = At1->pdb_num_res;
		    Noe->atom_number1    = atom1;
		    it = jt = 0;
		    while (At1->pdb_iupac[it++] != '\0') {
			 if (At1->pdb_iupac[it] != ' ')
			      Noe->atom_type1[jt++]=At1->pdb_iupac[it];
		    }
		    strcpy(Noe->residue_type1,At1->pdb_res);
		    strcpy(Noe->chain_id_1,At1->pdb_chain_id);
		    
		    /* storing atom2 */
		    Noe->residue_number2 = At2->pdb_num_res;
		    Noe->atom_number2    = atom2;
		    it = jt = 0;
		    while (At2->pdb_iupac[it++] != '\0') {
			 if (At2->pdb_iupac[it] != ' ')
			      Noe->atom_type2[jt++]=At2->pdb_iupac[it];
		    }
		    strcpy(Noe->residue_type2,At2->pdb_res);
		    strcpy(Noe->chain_id_2,At2->pdb_chain_id);
		    
		    /* General data */
		    Noe->Number          = count;
		    Noe->equil           = distance(atom1,atom2);
		    Noe->rgupp           = Noe->equil * ALLOW ; 
		    Noe->rglow           = Noe->equil * ALLOW ;
		    Noe->upper           = Noe->equil + Noe->rgupp ;
		    Noe->lower           = Noe->equil - Noe->rglow ;
		    Noe->volume          = 0.0 ;
		    Noe->next            = Noe ; 
		    
		    /* Assign different name conventions to atoms */
		    AssignLibraryNames(Noe,LibRecords,9,"NMR");
	       }
	  }
     
     constraintcount = count ;
     switch (senal) {
     case 4:
	  write_constraints_xplor(salida);
	  break;
     case 2:
	  write_constraints_congen(salida);
	  break;
     case 5:
	  write_constraints_diana(salida);
	  write_constraints_diana_lol(salida);
	  break;
     default:
	  fprintf(stderr,"\n\n **Error** (2359) in homology(), we are not supposed to get here\n\n");
	  exit (-1);
	  break;
     }
     
     (void) fflush(salida);
     (void) fclose(salida);
     free(cons);
     /* free(NOE);   */
     return ;
}



/*
 * ChangeAtoms,
 *
   This program takes a file with TWO alignements (two columns of numbers),
   a file with the distances derived using newdistances.c and generates
   a file with homologous constraints to run CONGEN in homology. 
   
   (c) Roberto Tejero, Universidad de Valencia, Dic. 1995

   On April 2003, CABM Rutgers University, the ability to deal with a six-column
   format for the alignment file has been added.  This is to have the code ready 
   to work on homology for dimers.  The 2nd and 5th columns hold the chain 
   identification.  A sample of the alignment file is

   	ALA   A  12   VAL   A  13

   Also -- when dealing with dimers the constraints for DYANA have been changed in
   such a way that when dealing with the second chain in the dimer a number (400) 
   is added to each residue number.  In other words, if one likes to deal with 
   homology for dimaer it should create a sequence file where PL LL and LP are added
   from last res + 1 till 399. 

   Checked on April 23, 2003 working for a dimer with fake alignment and with an old 
   case to check backwards compatibility. 
   
   El programa se ha desarrollado en Valencia para agilizar el borrado de
   restricciones en la homologia. Se le alimenta con un fichero en el cual
   se han derivado distancias para la proteina homologa utilizando newdis
   tances.c y un fichero (dos columnas de numeros) que presenta la alineacion
   para la proteina y su homologa. El programa despues de leer la alineacion
   lee linea a linea el fichero de distancias y cambia los numeros de los
   residuos de acuerdo con la numeracion de la proteina a modelar. 
   De momento parece tener en cuenta las inserciones y borrados en los 
   alineamientos de las secuencias, de todas formas se aconseja ardientemen
   te chequear en aquellos casos de inserciones y borrados 'especiales'.
   En ambos casos se desprecia TODO el intervalo del borrado o insercion
   asi como el residuo inmediato anterior e inmediato posterior. Esta
   caracteristica esta a falta de estudio y remodelacion.

   It has been continuously used in this way with no troubles so far. 2003
*/

int ChangeAtoms(char AlignmentFileName[MAXWORD], char Format[10], char BaseName[MAXWORD], int Acols)
{
     char  residuos[21][5], TmpOri[5], TmpDes[5], homologos[21][21][50], match[15][6] ;
     char  constr[MAXWORD], alinea[MAXWORD], palabra[MAXWORD], palabral[MAXWORD];
     char  palabraX[MAXWORD], palabraC[MAXWORD], linea[MAXLINE];
     
     char  resori[MAXRESIDUES][5], resdes[MAXRESIDUES][5];
     char  cadena1[MAXRESIDUES][2], cadena2[MAXRESIDUES][2];
     
     char   atomo1[6],  /* IUPAC name for atom1 */ 
	    atomo2[6],  /* IUPAC name for atom2 */
	    ResNam1[5], /* Residue name 1 */
	    ResNam2[5]; /* Residue name 2 */
     
     char   clave[10];   /* First word in distance file, it is used to stop */
     
     float  lower, upper;
     
     FILE   *cnstr,             /* Fichero de las distancias */
    	    *align,             /* Fichero del alineamiento */
	    *salidaD = '\0',    /* Fichero para escritura de los resultados */
	    *salidaDLOL = '\0', /* Fichero para escritura de LOL limites */
	    *salidaC = '\0',    /* Fichero para escritura de los resultados */
	    *salidaX = '\0',    /* Fichero para escritura de los resultados */
	    *debugging='\0';
     
     int    i, j, temp, ilin, k, l, lk,
	  origen[MAXRESIDUES], destino[MAXRESIDUES], 
	  MAXI, senal=0, residuo1, residuo2,
	  indice1=0, indice2=0;
     
     int    Prev = -9999, FirstResidue = -9999 ;
     char   WritePrevHeader = FALSE ;
     
     static struct noe *Noe ;
     
     div_t  Cont;
     static short DimerN = 0 ;
     
     Prev = -9999 ;

     /*
      * DimerN is used to create constraints for DYANA when a dimer is present.
      * Its value is usually 0 (monomer) but when a dimer is detected the second
      * chain is added 400 to residue number. So we have to create a linker sequence 
      * that goes from LastRes to 399 (400 being the starting res of second chain).
      */
     DimerN = 0 ;

     /* Make default of six columns for alignment in case of no specified */
     if ( Acols != 4 && Acols != 6 )
	     Acols = 4 ;
     
     fprintf(stdout," Changing atoms -- translating constraints into new file ... \n");
     
     /*
      *   The 20 naturally ocurring residues in their three-letter notation.
      *   It is used as a data-base for searching homologous atoms later.
      */
     strcpy(residuos[ 1],"GLY");
     strcpy(residuos[ 2],"ALA");
     strcpy(residuos[ 3],"VAL");
     strcpy(residuos[ 4],"ILE");
     strcpy(residuos[ 5],"LEU");
     strcpy(residuos[ 6],"PHE");
     strcpy(residuos[ 7],"PRO");
     strcpy(residuos[ 8],"MET");
     strcpy(residuos[ 9],"TRP");
     strcpy(residuos[10],"CYS");
     strcpy(residuos[11],"SER");
     strcpy(residuos[12],"THR");
     strcpy(residuos[13],"ASN");
     strcpy(residuos[14],"GLN"); 
     strcpy(residuos[15],"TYR");
     strcpy(residuos[16],"HIS");
     strcpy(residuos[17],"ASP");
     strcpy(residuos[18],"GLU");
     strcpy(residuos[19],"LYS");
     strcpy(residuos[20],"ARG");
     
     for (i = 1; i < 21; i++)
	  for (temp = 3; temp < 5; temp++)
	       residuos[i][temp] = '\0';
     
     /*
      * Database for searching homologous atoms.
      * The initial values of KK are meant to be taking a space that
      * can be used later. Also it indicates that there is NO atom only
      * a fake atom responding to the name of KK.
      * A length of *14* spaces is saved for storing the names of atoms.
      *
      * All residues have as homologous the main chain N, CA, C, O
      */
     for (i = 1; i <= 20; i++)
	  for (j = i+1; j <= 20; j++)
	       strcpy(homologos[i][j],"N CA C O KK  KK  KK  KK  KK  KK  KK  KK  KK  KK");
     
     strcpy(homologos[ 1][ 1],"N CA C O KK  KK  KK  KK  KK  KK  KK  KK  KK  KK");
     strcpy(homologos[ 2][ 2],"N CA C O CB  KK  KK  KK  KK  KK  KK  KK  KK  KK");
     strcpy(homologos[ 3][ 3],"N CA C O CB  CG1 CG2 KK  KK  KK  KK  KK  KK  KK");
     /*
      * This is rather confusing but in ILE sometimes you can find 
      * CD and others you can find CD1
      */
     strcpy(homologos[ 4][ 4],"N CA C O CB  CG1 CG2 CD1 KK  KK  KK  KK  KK  KK");
     strcpy(homologos[ 5][ 5],"N CA C O CB  CG  CD1 CD2 KK  KK  KK  KK  KK  KK");
     strcpy(homologos[ 6][ 6],"N CA C O CB  CG  CD1 CD2 CE1 CE2 CZ  KK  KK  KK");
     strcpy(homologos[ 7][ 7],"N CA C O CB  CG  CD  KK  KK  KK  KK  KK  KK  KK");
     strcpy(homologos[ 8][ 8],"N CA C O CB  CG  SD  CE  KK  KK  KK  KK  KK  KK");
     strcpy(homologos[ 9][ 9],"N CA C O CB  CG  CD1 CD2 NE1 CE2 CE3 CZ2 CZ3 CH2");
     strcpy(homologos[10][10],"N CA C O CB  SG  KK  KK  KK  KK  KK  KK  KK  KK");
     strcpy(homologos[11][11],"N CA C O CB  OG  KK  KK  KK  KK  KK  KK  KK  KK");
     strcpy(homologos[12][12],"N CA C O CB  OG1 CG2 KK  KK  KK  KK  KK  KK  KK");
     strcpy(homologos[13][13],"N CA C O CB  CG  OD1 ND2 KK  KK  KK  KK  KK  KK");
     strcpy(homologos[14][14],"N CA C O CB  CG  CD  OE1 NE2 KK  KK  KK  KK  KK");
     strcpy(homologos[15][15],"N CA C O CB  CG  CD1 CD2 CE1 CE2 CZ  KK  KK  KK");
     strcpy(homologos[16][16],"N CA C O CB  CG  ND1 CD2 CE1 NE2 KK  KK  KK  KK");
     strcpy(homologos[17][17],"N CA C O CB  CG  OD1 OD2 KK  KK  KK  KK  KK  KK");
     strcpy(homologos[18][18],"N CA C O CB  CG  CD  OE1 OE2 KK  KK  KK  KK  KK");
     strcpy(homologos[19][19],"N CA C O CB  CG  CD  CE  NZ  KK  KK  KK  KK  KK");
     strcpy(homologos[20][20],"N CA C O CB  CG  CD  NE  CZ  NH1 NH2 KK  KK  KK");
     
     /*
      *  CB for the rest of aminoacids but GLY
      */
     for (i=2 ; i<=20; i++)
	  for (j=i+1; j<=20; j++)
	       strcpy(homologos[i][j],"N CA C O CB  KK  KK  KK  KK  KK  KK  KK  KK  KK");
     /*
      * VAL con ILE
      */
     strcpy(homologos[3][4],"N CA C O CB  CG1 CG2 KK  KK  KK  KK  KK  KK  KK");
     
     /*
      * LEU with ALL others
      */
     strcpy(homologos[5][ 8],"N CA C O CB  CG  KK  KK  KK  KK  KK  KK  KK  KK");
     strcpy(homologos[5][13],"N CA C O CB  CG  KK  KK  KK  KK  KK  KK  KK  KK");
     strcpy(homologos[5][14],"N CA C O CB  CG  KK  KK  KK  KK  KK  KK  KK  KK");
     strcpy(homologos[5][17],"N CA C O CB  CG  KK  KK  KK  KK  KK  KK  KK  KK");
     strcpy(homologos[5][18],"N CA C O CB  CG  KK  KK  KK  KK  KK  KK  KK  KK");
     strcpy(homologos[5][19],"N CA C O CB  CG  KK  KK  KK  KK  KK  KK  KK  KK");
     strcpy(homologos[5][20],"N CA C O CB  CG  KK  KK  KK  KK  KK  KK  KK  KK");
     
     /*
      * PHE with ALL others but TYR
      */
     strcpy(homologos[6][15],"N CA C O CB  CG  CD1 CD2 CE1 CE2 CZ  KK  KK  KK");
     
     /*
      * TRP y HIS
      */
     strcpy(homologos[9][16],"N CA C O CB  CG  CD2 KK  KK  KK  KK  KK  KK  KK");
     
     /*
      *   ASN y ASP
      */
     strcpy(homologos[13][17],"N CA C O CB  CG  OD1 KK  KK  KK  KK  KK  KK  KK");
     
     /*
      *   GLN y GLU 
      */
     strcpy(homologos[14][18],"N CA C O CB  CG  OE1 KK  KK  KK  KK  KK  KK  KK");
     
     /*
      *  LYS y ARG 
      */
     strcpy(homologos[19][20],"N CA C O CB  CG  CD  KK  KK  KK  KK  KK  KK  KK");
     
     /*
      *  The other half of the matrix has the same elements
      */
     for (i = 1 ; i <= 20; i++)
	  for(j = i+1; j <= 20; j++)
	       strcpy(homologos[j][i],homologos[i][j]);
     
     cnstr = efopen("RandomDistances","r");
     senal = 3 ; 
     if ( strcmp(AlignmentFileName,"NoName")==0 ) {
	  printf(" Name of the file with alignments?_ ") ;
	  GetFromKeyBoard(alinea, MAXWORD, stdin);
     } else {
	  strcpy(alinea,AlignmentFileName);
     }
     align = efopen(alinea,"r") ;
     
     if (DEBUG > 0) {
	  debugging = safeopen("DebugHomolCambia","w",(int)strlen("DebugHomolCambia"));
     }
     
     /* 
      * Debugging
      */
     if (DEBUG > 0) {
	  fprintf(debugging,"\n   The twenty naturally ocurring aminos as stored in this routine\n");
	  for (i = 1; i <= 20 ; i++)
	       fprintf(debugging,"        %2d  -- %s \n", i, residuos[i]);
     }
     
     if (DEBUG > 0) {
	  fprintf(debugging,"\n   All posibilities of combining the twenty aminos for homologous atoms\n");
	  for (i = 1 ; i <= 20 ; i++)
	       for(j = i+1 ; j <= 20 ; j++)
		    fprintf(debugging,"   %4d  %4d  %-50s\n", i, j, homologos[i][j]);
     }
     
     /* Format for output file */
     if ( Format[0] == '\0' ) {
	  printf("\n");
	  printf(" Format for output file [congen | cns | xplor | dyana] :?_ ") ; 
	  GetFromKeyBoard(Format, MAXWORD, stdin);
     }
     if (strncasecmp(Format,"dyana",3)==0) senal = 1;
     else if (strncasecmp(Format,"xplor",3)==0) senal = 2;
     else if (strncasecmp(Format,"x-plo",3)==0) senal = 2;
     else if (strncasecmp(Format,"cns  ",3)==0) senal = 2;
     else if (strncasecmp(Format,"conge",3)==0) senal = 3;
     else if (strncasecmp(Format,"all",1)==0) senal = 4 ;
     else senal = 4 ;  /* Special case, _ALL_ formats, CONGEN, XPLOR/CNS, DYANA required */
     
     /*
      * Reading of the alignment file.
      * This file is composed by four columns of letters and numbers. 
      * In the first one are the Residue Names and number for the 
      * homologous protein with known structure (the template). In the
      * third and fourth columns there are the Residue Names and numbers 
      * of the protein to be modelled (the target).
      * In case of NO numbers either in origin (column 1) or final (column 4)
      * these are initialized to zero. This number (zero) is later used to detect
      * insertion and/or deletions.
      */
     i = 1 ;
     if (DEBUG > 0) {
	  fprintf(debugging,"\n  ----------  Original alignment begins --------\n");
     }
     while (fgets(linea,MAXLINE,align) != NULL) {
	  origen[i] = 0 ;
	  destino[i] = 0 ;
	  if ((int)strlen(linea) <= 5) continue ;
	  sscanf(linea,"%s",clave);
	  if (strncmp(clave,"#",1)!=0) {
	       if ( Acols == 4 )
	        	sscanf(linea,"%s %s %s %s", resori[i], TmpOri, resdes[i], TmpDes);  
	       else if ( Acols == 6 ) /* new six-column format */
	       		sscanf(linea,"%s %s %s %s %s %s", 
			       	resori[i], cadena1[i], TmpOri, resdes[i], cadena2[i], TmpDes);
	       else {      /* further, innecessary check */
		       fprintf(stdout,"  No recognized format for alignment file \n");
		       exit (-1) ;
	       }
				       
	       if ( strcmp(TmpOri,"*") !=0 ) 
		    origen[i] = atoi(TmpOri);
	       if ( strcmp(TmpDes,"*") !=0 ) 
		    destino[i] = atoi(TmpDes);
	       
	       /* * Check if it is one letter code and if so convert to three letter */
	       if ((strlen(resori[i])==1) && (strcmp(resori[i],"*")!=0))
		    strcpy(resori[i], OneToThree(resori[i]));
	       if ((strlen(resdes[i])==1) && (strcmp(resdes[i],"*")!=0))
		    strcpy(resdes[i], OneToThree(resdes[i]));
	       
	       if (DEBUG > 0) {
		    fprintf(debugging,"  %s %6d\t %s %6d\n", resori[i], origen[i], resdes[i], destino[i]);
	       }
	       i++ ;
	  }
     }
     MAXI = i-1 ;
     
     /* ---------------------------------------- Experimental code begins */
     /*
      * In case CONGEN, CNS or X-PLOR are selected we do a special check for first residue 
      * as they can start usually with numbers different from 1. Should this be the case
      * the whole sequence is renumbered sequentially from 1 to avoid trouble with the
      * Simulated Annealing protocols. All names are passed (copied) to seqres[] to be able
      * to write them and have ready all needed files (usually prot.seq and prot.cns are the only 
      * things needed to run CNS, X-PLOR).
      *
      * On Oct 2002, the renumbering starting from 1 is done for _ALL_ methods, DYANA included.
      * So, technically we do not need check on senal to do it. 
      */
     if ( senal >= 0 ) { 
	  j = 0 ;
	  for (i = 1; i < MAXRESIDUES; i++) 
	       strcpy(seqres[i],"");
	  for (i = 1; i <= MAXI; i++) {
	       if ( strncmp(resdes[i],"*",1) != 0 ) {
		    if ( FirstResidue == -9999 ) 
			 FirstResidue = destino[i] ; 
		    j++ ;
		    strcpy(seqres[j],resdes[i]) ;
	       }
	  }
     }
     /* ------------------------------------------ Experimental code finishes */
     
     if (DEBUG > 0) {
	  fprintf(debugging,"  ------------   Original alignment ends  -------\n");
	  fprintf(debugging,"\n  ------------   Constraint matchings  -------\n");
     }
     
     fprintf(stdout,"\n File %s has %d alignment records \n", alinea, MAXI) ;
     
     /*
      * Reading of file where distances are stored (line by line). 
      * Insertions and/or deletions are checked in loops (jumping to the label
      * 'salta').
      * Residue numbers are changed according to data in alignment file
      */
     j = 1 ;
     ilin = 0;
     
     /* Output file for changed constraints */
     if ( BaseName[0] == '\0' ) {
	  printf(" Base Name for the output file?_ ") ;
	  GetFromKeyBoard(palabra, MAXWORD, stdin);
     } else {
	  strcpy(palabra,BaseName);
     }
     
     /* 
      * according to output type create output/s file/s.
      * DYANA needs TWO, one for UPL and one for LOL
      */
     switch (senal) {
     case 1: /* Dyana */
	  strcpy(palabral,palabra);
	  strcat(palabra,".upl");
	  salidaD = safeopen(palabra,"w",(int)strlen(palabra)) ;
	  fprintf(salidaD,"#\n# UPL file\n#\n");
	  strcat(palabral,".lol");
	  salidaDLOL = safeopen(palabral,"w",(int)strlen(palabral)) ;
	  fprintf(salidaDLOL,"#\n# LOL file\n#\n");
	  break;
     case 2:  /* CNS X-PLOR */
	  strcat(palabra,".tbl");
	  salidaX = safeopen(palabra,"w",(int)strlen(palabra)) ;
	  break ;
     case 3:  /* Congen */
	  strcat(palabra,".cgn");
	  salidaC = safeopen(palabra,"w",(int)strlen(palabra)) ;
	  fprintf(salidaC,"*----- \n*  Title default produced by HOMA/HoMMeR command\n*\n");
	  fprintf(salidaC,"TYPE NOE\nWEIGHT 1.0\n");
	  break ;
     case 4:
     default:
	  /* Get basename for different formats */
	  strcpy(palabraX,palabra);
	  strcpy(palabraC,palabra);
	  strcpy(palabral,palabra);
	  /* Dyana */
	  strcat(palabra,".upl");
	  salidaD = safeopen(palabra,"w",(int)strlen(palabra)) ;
	  fprintf(salidaD,"#\n# UPL file\n#\n");
	  strcat(palabral,".lol");
	  salidaDLOL = safeopen(palabral,"w",(int)strlen(palabral)) ;
	  fprintf(salidaDLOL,"#\n# LOL file\n#\n");
	  /* Xplor/CNS  */
	  strcat(palabraX,".tbl");
	  salidaX = safeopen(palabraX,"w",(int)strlen(palabraX)) ;
	  /* CONGEN  */
	  strcat(palabraC,".cgn");
	  salidaC = safeopen(palabraC,"w",(int)strlen(palabraC)) ;
	  fprintf(salidaC,"*----- \n*  Title default produced by HOMA/HoMMeR command\n*\n");
	  fprintf(salidaC,"TYPE NOE\nWEIGHT 1.0\n");
	  break;
     }
     
     fprintf(stdout,"\n Processing file %s ..   ", constr); 
     
     /* Main loop of changing constraints*/
     for (ilin = 1; ilin <= constraintcount; ilin++) {
	  
	  /* Counter to make pretty the waiting */
	  if ( DEBUG >= 0 ) {
	       Cont = div(ilin,4);
	       if (Cont.rem == 1) fprintf(stdout,"\b\b/ ");
	       if (Cont.rem == 2) fprintf(stdout,"\b\b- ");
	       if (Cont.rem == 3) fprintf(stdout,"\b\b\\ ");
	       if (Cont.rem == 0) fprintf(stdout,"\b\b| ");
	       fflush(stdout);
	  }
	  fflush(stdout);
	  
	  /* The real thing starts */
	  Noe = NOE + ilin ;
	 
	  residuo1 = Noe->residue_number1 ;
	  residuo2 = Noe->residue_number2 ;
	  strcpy(atomo1,Noe->atom_type1);
	  strcpy(atomo2,Noe->atom_type2);
	  lower = Noe->lower;
	  upper = Noe->upper;
	  
	  /*
	   *     Checking of alignment file to see if there are some insertions or
	   *     deletions and in these cases change order of numbers.
	   *     The insertions or deletions are hoped to be taking into account
	   *     when checking the origin or end with value of 'zero'.
	   *     temp variable is used to double check that the residue really 
	   *     exists and it is NOT a deletion nor insertion. Should the loop
	   *     was unable to find the residue a jump is done (in this case
	   *     temp = 9999).
	   */
	  temp = 9999 ;
	  for (k = 1; k <= MAXI; k++) {
	  	if ( Acols == 4 ) {
		  	strcpy(cadena1[k],Noe->chain_id_1);
	  	}
	       if (residuo1 == origen[k] && strcmp(Noe->chain_id_1,cadena1[k])==0 ) {
		    if ((origen[k]==0)    || (origen[k+1]==0)  || (origen[k-1]==0)  || 
			(destino[k+1]==0) || (destino[k-1]==0) || (destino[k]==0)) 
			 goto salta ;
		    residuo1 = destino[k] ;
		    temp = residuo1 ;
		    break ;
	       }
	  }
	  
	  if (temp == 9999 ) goto salta ;
	  
	  temp = 9999 ;
	  for (l = 1; l <= MAXI; l++) {
	  	if ( Acols == 4 ) {
		  	strcpy(cadena2[l],Noe->chain_id_1);
	  	}
	       if (residuo2 == origen[l] && strcmp(Noe->chain_id_2,cadena2[l])==0 ) {
		    if ((origen[l]==0) || (origen[l+1]==0)  || (origen[l-1]==0)  || 
			(destino[l+1]==0) || (destino[l-1]==0) || (destino[l]==0)) 
			 goto salta ;
		    residuo2 = destino[l] ;
		    temp = residuo2 ;
		    break ;
	       }
	  }
	  
	  if (temp == 9999 ) goto salta ;
	  
	  /*
	   *  Here should come a deep check to assure the different residue
	   *  types. For that the alignment file should have four columns
	   *  (as is the case since JUL 96) as
	   *
	   *     COL1  COL2   COL3 COL4
	   *     ----  ----   ---- ----
	   *     res1  num1   res2 num2
	   *     ASN     1    TYR    3
	   *     ...   ...    ...  ...
	   *     
	   *  To do this a routine that residue by residue check if they are
	   *  the same and the degree of homology should be provided. 
	   *  In the extreme case where is no homology at all we only have
	   *  the mainchain homology ( N, CA, C, O).
	   */
	  for ( i = 1 ; i <= 20 ; i++)
	       if (strncmp(resdes[k],residuos[i],3)==0) {
		    strcpy(ResNam1,resdes[k]);
		    indice1=i ; 
		    break ;
	       }
	  
	  for ( i = 1 ; i <= 20 ; i++)
	       if (strncmp(resdes[l],residuos[i],3)==0) {
		    strcpy(ResNam2,resdes[l]);
		    indice2=i ; 
		    break ;
	       }
	  
	  sscanf(homologos[indice1][indice2], "%s %s %s %s %s %s %s %s %s %s %s %s %s %s",
		 match[ 1], match[ 2], match[ 3], match[ 4], match[ 5], match[ 6], match[ 7], 
		 match[ 8], match[ 9], match[10], match[11], match[12], match[13], match[14]);
	  
	  for (lk = 1; lk <= 20; lk++) {
	       if (lk > 14) goto salta ;
	       if (strcmp(atomo1,match[lk])==0)
		    break ;
	  }
	  
	  for (lk = 1; lk <= 20; lk++) {
	       if (lk > 14 ) goto salta ;
	       if (strcmp(atomo2,match[lk])==0)
		    break ;
	  }
	  
	  /*
	   *     Debugging -- print list of matched lines between template and unknown
	   */
	  if (DEBUG > 0) {
	       fprintf(debugging,"  Cnt: %6d/%-6d  Templ: %3d %3s  %3d %3s   Homol: %3d %3s  %3d %3s\n",
		       j,ilin, origen[k] , resori[k], origen[l],  resori[l], 
		       destino[k], resdes[k], destino[l], resdes[l]);
	  }
	  
	  /* For printing in DYANA format */
	  if (Prev != residuo1) {
	       Prev = residuo1 ;
	       WritePrevHeader = TRUE ;
	  } else {
	       WritePrevHeader = FALSE ;
	  }
	  
	  /*
	   *	Writing of distances with new numbers of residue.
	   *	The writing has been divided in several lines to speed up
	   *	the reading of 'future' programmers/checkers (I hope!).
	   */
	  switch (senal) {
	  case 4:               /* Special case, _ALL_ of them together */
	       /* Congen */
	       fprintf(salidaC,"  CONS ATOM %1s %3d %5s", Noe->chain_id_1, residuo1-FirstResidue+1, atomo1);
	       fprintf(salidaC," : ATOM %1s %3d %5s"  , Noe->chain_id_2, residuo2-FirstResidue+1, atomo2);
	       fprintf(salidaC,"  LOWER %6.2f  UPPER %6.2f", Noe->lower, Noe->upper);
	       /* X-PLOR, CNS */
	       fprintf(salidaC,"  ! %4d\n", j);
	       fprintf(salidaX,"assign (segid %2s and resid %3d and name %-4s)" 
		       " (segid %2s and resid %3d and name %-4s) %5.2f %5.2f %5.2f ",
		       Noe->chain_id_1, residuo1-FirstResidue+1, atomo1, 
		       Noe->chain_id_2, residuo2-FirstResidue+1, atomo2, 
		       Noe->equil, Noe->rglow, Noe->rgupp);
	       fprintf(salidaX,"  ! %4d\n", j);
	       /* Dyana */
	       if ( Noe->chain_id_2[0] != NOE[1].chain_id_1[0] ) {
		       DimerN = 400 ;
	       } else {
		       DimerN = 0 ;
	       }
	       if ( WritePrevHeader ) {
		    fprintf(salidaD,"%3d %3s\n", DimerN + residuo1-FirstResidue+1, ResNam1);
		    fprintf(salidaDLOL,"%3d %3s\n", DimerN + residuo1-FirstResidue+1, ResNam1);
	       }
	       fprintf(salidaD,"         %-5s%4d %3s  %-5s%6.1f     # %4d\n",
		       atomo1, DimerN + residuo2-FirstResidue+1, ResNam2, atomo2, Noe->upper, j);
	       fprintf(salidaDLOL,"         %-5s%4d %3s  %-5s%6.1f     # %4d\n",
		       atomo1, DimerN + residuo2-FirstResidue+1, ResNam2, atomo2, Noe->lower, j);
	       break;	
	  case 3:   /* Congen only */
	       fprintf(salidaC,"  CONS ATOM %1s %3d %5s", Noe->chain_id_1, residuo1-FirstResidue+1, atomo1);
	       fprintf(salidaC," : ATOM %1s %3d %5s"  , Noe->chain_id_2, residuo2-FirstResidue+1, atomo2);
	       fprintf(salidaC,"  LOWER %6.2f  UPPER %6.2f", Noe->lower, Noe->upper);
	       fprintf(salidaC,"  ! %4d\n", j);
	       break; 
	  case 2:   /* CNS, X-PLOR only */
	       fprintf(salidaX,"assign (segid %2s and  resid %3d and name %-4s)"
		       " (segid %2s and resid %3d and name %-4s) %5.2f %5.2f %5.2f ",
		       Noe->chain_id_1, residuo1-FirstResidue+1, atomo1, 
		       Noe->chain_id_2, residuo2-FirstResidue+1, atomo2, 
		       Noe->equil, Noe->rglow, Noe->rgupp);
	       fprintf(salidaX,"  ! %4d\n", j);
	       break;
	  case 1:   /* Dyana only */
	       if ( Noe->chain_id_2[0] != NOE[1].chain_id_1[0] ) {
		       DimerN = 400 ;
	       } else {
		       DimerN = 0 ;
	       }
	       if ( WritePrevHeader ) {
		    fprintf(salidaD,"%3d %3s\n", DimerN + residuo1-FirstResidue+1, ResNam1);
		    fprintf(salidaDLOL,"%3d %3s\n", DimerN + residuo1-FirstResidue+1, ResNam1);
	       }
	       fprintf(salidaD,"         %-5s%4d %3s  %-5s%6.1f     # %4d\n",
		       atomo1, DimerN + residuo2-FirstResidue+1, ResNam2, atomo2, Noe->upper, j);
	       fprintf(salidaDLOL,"         %-5s%4d %3s  %-5s%6.1f     # %4d\n",
		       atomo1, DimerN + residuo2-FirstResidue+1, ResNam2, atomo2, Noe->lower, j);
	       break;	
	  default:
	       fprintf(stdout,"\n\n Error (2361) in homology format not yet ready\n\n");
	       exit (-1);
	       break;
	  }
	  j++ ;
	  /*  Hasta aqui salta si no hay nada que hacer */
     salta:
	  ;
     }
     
     if ( senal == 3 ) {  /* means CONGEN output */
	  fprintf(salidaC,"END\n");
     }
     
     /*
      * Closing files opened in program -- just good behaviour 
      */
     fprintf(stdout,"\n A total of %d lines have been processed \n",ilin);
     fclose(cnstr) ;
     if ( senal == 1 ) {
  	  fclose(salidaD) ;
	  fclose(salidaDLOL);
     }
     if ( senal == 2 ) fclose(salidaX);
     if ( senal == 3 ) fclose(salidaC);
     if ( senal == 4 ) {
	  fclose(salidaD);
	  fclose(salidaDLOL);
	  fclose(salidaC);
	  fclose(salidaX);
     }
     fclose(align);
     if (DEBUG > 0) fclose(debugging);
     return (int) 0;
}
