/*$Id: hydrogen_bonding.c,v 3.32 2003/10/06 12:13:32 roberto Exp roberto $*/

#include "pdbstat.h"
extern FILE *safeopen(char *file, char *mode, int longit) ;
void CopyHbond(struct HydrogenBond *A, struct HydrogenBond *B) ;
void SortHbondRes1(struct HydrogenBond *HbPtr) ;
void SortHbondRes2(struct HydrogenBond *HbPtr) ;
void SwapAtoms(struct HydrogenBond *A) ;
void InitializeHbond(struct HydrogenBond *A) ;
int hbnds ;

void InitializeHbond(struct HydrogenBond *A) 
{
	static short it ;

	A->next = A ;
	A->rec  =99999;
	A->ResN1=99999;
	A->ResN2=99999;
	A->modeln=0;
	A->nStructures=0;
	A->ToPrint=TRUE;
	for (it=0; it<20; it++) A->AtomDesc1[it]='\0';
	for (it=0; it<20; it++) A->AtomDesc2[it]='\0';
	for (it=0; it<100; it++) A->Where[it]=' ';
	A->Where[99]='\0';
	return ;
}

void SwapAtoms(struct HydrogenBond *A)
{
	static char Temp[20];

	strcpy(Temp,A->AtomDesc1);
	strcpy(A->AtomDesc1,A->AtomDesc2);
	strcpy(A->AtomDesc2,Temp);
}

void CopyHbond(struct HydrogenBond *A, struct HydrogenBond *B)
{
     A->rec    = B->rec;
     A->modeln = B->modeln;
     A->ResN1  = B->ResN1 ;
     A->ResN2  = B->ResN2 ;
     A->nStructures = B->nStructures;
     A->ToPrint = B->ToPrint ;
     strcpy(A->AtomDesc1,B->AtomDesc1);
     strcpy(A->AtomDesc2,B->AtomDesc2);
     strcpy(A->Where,B->Where);
     A->next = A;
     B->next = B;
}

void SortHbondRes2(struct HydrogenBond *HbPtr)
{
     register int ICnt, JCnt ;
     static struct HydrogenBond *ht ;

     ht = (struct HydrogenBond *)malloc(sizeof(struct HydrogenBond)) ;
     for ( JCnt = 1 ; JCnt <= hbnds ; JCnt++) {
	  CopyHbond(ht,&HbPtr[JCnt]) ;
	  ICnt = JCnt - 1;
	  while ((ICnt >= 0) && (HbPtr[ICnt].ResN2 > ht->ResN2)){
	       CopyHbond(&HbPtr[ICnt+1],&HbPtr[ICnt]);
	       ICnt--;
	  }
	  CopyHbond(&HbPtr[ICnt+1],ht);
     }
     free (ht) ;
     return ;
}

void SortHbondRes1(struct HydrogenBond *HbPtr)
{
     register int ICnt, JCnt ;
     static struct HydrogenBond *ht ;

     ht = (struct HydrogenBond *)malloc(sizeof(struct HydrogenBond)) ;
     for ( JCnt = 1 ; JCnt <= hbnds ; JCnt++) {
	  CopyHbond(ht,&HbPtr[JCnt]) ;
	  ICnt = JCnt - 1;
	  while ((ICnt >= 0) && (HbPtr[ICnt].ResN1 > ht->ResN1)){
	       CopyHbond(&HbPtr[ICnt+1],&HbPtr[ICnt]);
	       ICnt--;
	  }
	  CopyHbond(&HbPtr[ICnt+1],ht);
     }
     free (ht) ;
     return ;
}
	

/* ========================================================================= */
void hydrogen_bonds(float distance_H_any,float angle_any, char Who[10])
/*
  ---
  --- This module to evaluate the possibility of hydrogen bonds just as distan
  --- ce criteria (no orientation nor angle). The cutoff in the distance can
  --- be entered through the keyboard. Angle capabilities were added Jun 1993.
  ---
  --- Feb 2003, the code has been revisited and changed/adapted/extended to handle 
  --- the case where we like to calculate hbonds for _ALL_ models in a family. Several
  --- routines for sorting data and a new data structure to hold the HBONDS has been
  --- created. The ouput mimetics the one in DYANA, MOLMOL. (RTT, CABM, Feb 2003)
  ---
  --- On june it was carefully ampliated to take care of angles between donor 
  --- acceptor and so on. (RTT)
  ---
  --- (Roberto Tejero, RTT, May 1993, CABM-Rutgers University).
  --- Code revisited/corrected in Nov 2002, CABM. Roberto Tejero
  ---
*/
{
     static int     ii, finalhbnds, hydr, oxyg ;
     static float   dha, dda, angle_dha, angle_daa ;
     static FILE    *hydrogen;
     static short   i, k, mdl, mdlfrom, mdlto;
     static struct  models *model ;
     static struct  atom_pdb  *AtomOne, *AtomTwo, *AtomOxy, *AtomHyd, *AtomAnt ;
     static struct  HydrogenBond *Hbond=NULL, *Hbond2=NULL ;
     static char    ForAll ;
     
      HBONDS = (struct HydrogenBond *)malloc(sizeof(struct HydrogenBond)) ;

      ForAll = FALSE ;

     /*
      * Check input and put standard cutoffs
      */
     if (distance_H_any <= 0.0) distance_H_any = 3.50 ;
     if (cristal == 0) 
	  if (angle_any <= 0.0) angle_any = 120.00 ;
     if (cristal == 1)
	  if (angle_any <= 0.0) angle_any = 60.00 ;
    
     if ( strncasecmp(Who,"all",3) == 0 ) {
	  ForAll = TRUE ;
	  mdlfrom = 1 ;
	  mdlto = total_models ;
     } else {
	  mdlfrom = mdlto = atoi(Who);
     }
     
     /*
      * Open file for results and write headers
      */
     strcpy(output_name,"HydrogenBondData_");
     strcat(output_name,name_of_the_file_coords);
     
     hydrogen = safeopen(output_name,"w",(int)strlen(output_name)) ;
     
     if ( strncasecmp(Who,"all",3)==0 ) {
	  fprintf(hydrogen,"# Possible hydrogen bonds for molecule `%s' all models (%d) \n# %7.2f angstrom and %7.2f degrees cutoff\n",
		  name_of_the_file_coords, total_models, distance_H_any, angle_any);
     } else {
	  printf("> HYDRGN_bnd: Calculating hydrogen bonds\n");
	  printf("> HYDRGN_bnd: with %5.2f angs %5.2f deg cutoff\n", distance_H_any,angle_any);
	  printf("> HYDRGN_bnd: Sample of Hydr. bonds for molecule `%s' model #%d\n", name_of_the_file_coords, (MODELS+mdlto)->modelno); 
	  
	  fprintf(hydrogen,"Possible hydrogen bonds for molecule `%s' model #%d \n %7.2f angstrom and %7.2f degrees cutoff\n",
		  name_of_the_file_coords, (MODELS+mdlto)->modelno, distance_H_any, angle_any);
     }

     if ( !ForAll ) {
	  fprintf(hydrogen, "                                             DIST    ANGLE    ANGLE   DIST   \n");	
	  fprintf(hydrogen, "      #     ATOM 1           ATOM2            H-A      DHA      HAA    D-A   \n");
	  fprintf(hydrogen, "     -------------------   --------------    -----   ------   ------ ----- \n");
	  
	  printf("> HYDRGN_bnd:                                            DIST    ANGLE    ANGLE   DIST \n");	
	  printf("> HYDRGN_bnd:      #     ATOM 1          ATOM2            H-A     DHA      HAA     D-A  \n");
	  printf("> HYDRGN_bnd:     ------------------  ---------------    -----   ------   ------  ---- \n");
     }
     
     hbnds = 0;
     for (mdl = mdlfrom; mdl <= mdlto; mdl++) {
	  key_model = mdl ;
	  model = MODELS + key_model ;
	  MAX = model->number_of_atoms + 1;
	  ii = 0;
	  hydr = 0;
	  oxyg = 0;
	  dda = 0.0;            /* distance donor - acceptor  */
	  dha = 0.0;            /* distance hydro - acceptor  */
	  angle_dha = 0.0;      /* angle donor-hydro-acceptor */
	  angle_daa = 0.0;      /* angle donor-accpetor-antecedant */
	  
	  if (DEBUG > 0 && cristal == 1)
	       printf("> HYDRGN_bnd: Working CRYSTAL structure, *NO* hydrogens \n");
	  
	  /*
	   * General loop of search. We explicitely *exclude* ALPHAS and
	   * BETAS from search.
	   *
	   * This is suposed to be a DEEP search for donors and acceptors
	   * we are looking for NH...O, OH...N and 
	   * in crystals N...O, or O...N
	   */
	  for (i = 1 ; i < MAX ; i++) {     
	       AtomOne = (model->PDB)+i ;
	       if ((AtomOne->pdb_iupac[2] == 'A') ||  (AtomOne->pdb_iupac[2] == 'B') )
		    continue ;
	       if ( AtomOne->pdb_iupac[1] != 'H' && AtomOne->pdb_iupac[1] != 'N' && AtomOne->pdb_iupac[1] != 'O' )
		    continue;

	       if ( strncmp(AtomOne->pdb_iupac," OE2",4)==0 && strncmp(AtomOne->pdb_res,"GLU",3)==0 )
		       continue ;
	       
	       for (k = i+1 ; k < MAX ; k++) { 
		    AtomTwo = (model->PDB)+k ;
		    
		    if ((AtomTwo->pdb_iupac[2] == 'A') || (AtomTwo->pdb_iupac[2] == 'B') )
			 continue ;
		    if ( AtomTwo->pdb_iupac[1] != 'H' && AtomTwo->pdb_iupac[1] != 'N' && AtomTwo->pdb_iupac[1] != 'O' )
			 continue;
		    /* Avoid N to N or O to O relations */
		    if ( AtomOne->pdb_iupac[1] == AtomTwo->pdb_iupac[1] )
			 continue ;
		    if ( strcmp(AtomOne->pdb_iupac,AtomTwo->pdb_iupac)==0 )
			    continue;
		    /*
		    if ( AtomOne->pdb_num_res == AtomTwo->pdb_num_res )
			 continue ;
		    */
	       	    if ( strncmp(AtomTwo->pdb_iupac," OE2",4)==0 && strncmp(AtomTwo->pdb_res,"GLU",3)==0 )
		        continue ;
		    
		    if (distance(i,k) < 4.5) { 
			 
			 if (DEBUG > 8 ) 
			      fprintf(stdout,"hydrogen_bonding(): Dealing with %3s %3d %4s  --  %3s %3d %4s\n",
				      AtomOne->pdb_res, AtomOne->pdb_num_res, AtomOne->pdb_iupac,
				      AtomTwo->pdb_res, AtomTwo->pdb_num_res, AtomTwo->pdb_iupac);
			 
			 if (cristal == 0)
			      dha = distance(i,k) ;
			 if (cristal == 1)
			      dda = distance(i,k) ;
			 
			 angle_dha = -9999.99 ;   
			 if (cristal == 1) angle_dha = 0.0 ;
			 angle_daa = -9999.99 ;   
			 
			 if (cristal == 0) {       /*  no crystal structure, we have hydrogens */
			      if (AtomOne->pdb_iupac[1]=='H') 
				   hydr = i ;
			      if (AtomTwo->pdb_iupac[1]=='H') 
				   hydr = k ;
			      AtomHyd = (model->PDB)+hydr ;
			      
			      /* Look for attached N or O */
			      for (ii = hydr - 15 && ii >= 1; ii <= hydr + 15 && ii != hydr && ii < MAX ; ii++) {
				   AtomAnt = (model->PDB) + ii ;
				   if (AtomHyd->pdb_num_res != AtomAnt->pdb_num_res)
					continue ;
				   if ((AtomAnt->pdb_iupac[1] != 'N') && (AtomAnt->pdb_iupac[1] != 'O')) 
					continue ;
				   if (distance(ii,hydr) < 1.35) {  /* If there is a bond */
					if (AtomOne->pdb_iupac[1]=='H') {
					     angle_dha = angulo(ii,i,k) / radperdeg ;
					     dda = distance(ii,k);
					}
					if (AtomTwo->pdb_iupac[1]=='H') {
					     angle_dha = angulo(i,k,ii) / radperdeg ;
					     dda = distance(ii,i) ;
					}
				   }
			      }
			      
			      /* Look for antecedent of aceptor, has to be a Carbon */
			      if ( (AtomOne->pdb_iupac[1]=='O') || (AtomOne->pdb_iupac[1]=='N') )
				   oxyg = i ;
			      if ( (AtomTwo->pdb_iupac[1]=='O') || (AtomTwo->pdb_iupac[1]=='N') )
				   oxyg = k ;
			      
			      AtomOxy = (model->PDB)+oxyg ;  
			      for (ii = oxyg - 15 && ii >= 1; ii <= oxyg + 15 && ii != oxyg && ii < MAX ; ii++) {
				   AtomAnt = (model->PDB) + ii ;  
				    if ( AtomOxy->pdb_num_res != AtomAnt->pdb_num_res )
				        continue; 
				   if ( AtomAnt->pdb_iupac[1] != 'C' )
					continue;
				   if ( distance(ii,oxyg) < 1.55 ) {
					if ( AtomOne->pdb_iupac[1]=='O' || AtomOne->pdb_iupac[1]=='N' ) {
					     angle_daa = angulo(ii,i,k) / radperdeg ;
					}
					if ( AtomTwo->pdb_iupac[1]=='O' || AtomTwo->pdb_iupac[1]=='N' ) {
					     angle_daa = angulo(i,k,ii) / radperdeg ;
					}
				   }
			      }
			 } /*  End of no crystal structure  */
			 
			 if (cristal == 1) { /* It's a crystal structure, NO HYDROGENS */
			      
			      if (AtomOne->pdb_iupac[1]=='O') 
				   oxyg = i ;
			      if (AtomTwo->pdb_iupac[1]=='O') 
				   oxyg = k ;
			      
			      AtomOxy = (model->PDB) + oxyg ;  		
			      for (ii = oxyg - 15 && ii >= 1 ; ii <= oxyg + 15 && ii != oxyg && ii < MAX ; ii++) {
				   AtomAnt = (model->PDB) + ii ;  
				   if (AtomOxy->pdb_num_res != AtomAnt->pdb_num_res) 
					continue ; 
				   if ( AtomAnt->pdb_iupac[1] != 'C') 
					continue; 
				   if (distance(ii,oxyg) < 1.55)  {
					if (AtomTwo->pdb_iupac[1] == 'O')
					     angle_daa = angulo(ii,k,i) / radperdeg ;
					if (AtomOne->pdb_iupac[1] == 'O')
					     angle_daa = angulo(i,k,ii) / radperdeg ;
				   }
			      }
			      if (DEBUG > 0) 
				   printf("> HYDRGN_bnd: %d %d %d %f %f\n", i, ii, k, dda, angle_daa);
			      
			 } /* End of It's a crystal structure, NO HYDROGENS */
			 
			 /*
			  * Refine Selecting final printout of the hydrogen bonds
			  * Using criteria:
			  *           d(D-A) (donor-acceptor) < cutoff def. 3.40
			  *           angle DHA (don-hyd-acc) < cutoff def. 120.0
			  */
			 
			 if (((cristal == 0) && (dha <= (distance_H_any - 1.08)) && 
			      (dda <= distance_H_any) && (fabs(angle_dha) >= angle_any) &&
			      (fabs(angle_dha) < 1111.0) && (dda > 1.10)) ||
			     ((cristal == 1) && (dda <= distance_H_any) && (fabs(angle_daa) >= angle_any) &&
			      (fabs(angle_daa) < 1111.0) && (dda > 1.10))) {
			      
			      hbnds++;
			      if ( !ForAll ) {
				   fprintf(hydrogen,"%3d[%3d]  %3s %2s %3d %5s  %3s %2s %3d %5s %5.2f %8.2f %8.2f %5.2f\n",
					   hbnds,model->modelno, 
					   AtomTwo->pdb_res, AtomTwo->pdb_chain_id, AtomTwo->pdb_num_res, AtomTwo->pdb_iupac,
					   AtomOne->pdb_res, AtomOne->pdb_chain_id, AtomOne->pdb_num_res, AtomOne->pdb_iupac,
					   dha, angle_dha, angle_daa, dda);
				   fprintf(stdout,"> HYDRGN_bnd:%3d[%3d] %3s %2s %3d %5s  %3s %2s %3d %5s %5.2f %8.2f %8.2f %5.2f\n",
					   hbnds, model->modelno,
					   AtomTwo->pdb_res, AtomTwo->pdb_chain_id, AtomTwo->pdb_num_res, AtomTwo->pdb_iupac,
					   AtomOne->pdb_res, AtomOne->pdb_chain_id, AtomOne->pdb_num_res, AtomOne->pdb_iupac,
					   dha, angle_dha, angle_daa, dda);
			      }
			      /* reallocate space for new hbond data */
			      HBONDS = (struct HydrogenBond *)realloc(HBONDS, sizeof(struct HydrogenBond)*(hbnds+2)) ; 
			    
			      Hbond = HBONDS+hbnds ;
			      InitializeHbond(Hbond) ;
			      Hbond->rec  = hbnds ;
			      Hbond->ResN1 = AtomTwo->pdb_num_res ;
			      sprintf(Hbond->AtomDesc1,"%-5s %3s %s %4d",
				      AtomTwo->pdb_iupac, AtomTwo->pdb_res, AtomTwo->pdb_chain_id,AtomTwo->pdb_num_res);
			      Hbond->ResN2 = AtomOne->pdb_num_res ;
			      sprintf(Hbond->AtomDesc2,"%-5s %3s %s %4d",
				      AtomOne->pdb_iupac, AtomOne->pdb_res, AtomOne->pdb_chain_id,AtomOne->pdb_num_res);
			      Hbond->nStructures=1 ;
			      if ( AtomTwo->pdb_iupac[1] != 'H' )
				      SwapAtoms(Hbond);
			      Hbond->ToPrint = TRUE ;
			      Hbond->modeln = mdl ; 
			      Hbond->Where[mdl]='+' ;
			      Hbond->next = Hbond ;
			 }
		    }
	       }
	  }
     }
    
     fprintf(stdout," > HYDRGN_bnd: %d Possible hydrogens bonds have been found \n", hbnds);
     if ( ForAll ) {
	  fprintf(hydrogen,"#       Atom 1             Atom 2            #\n");
	  fprintf(hydrogen,"#   ---------------   ----------------       --\n");
	  Hbond = &HBONDS[1];
	  while ( Hbond->next != NULL ){
	       Hbond2 = (Hbond + 1);
	       while ( Hbond2->next != NULL ) {
		    if ( strcmp(Hbond->AtomDesc1,Hbond2->AtomDesc1)==0 && strcmp(Hbond->AtomDesc2,Hbond2->AtomDesc2)==0) {
			 Hbond->nStructures += 1 ;
			 Hbond2->ToPrint = FALSE ;
			 Hbond->Where[Hbond2->modeln]='+';
		    }
		    Hbond2++ ;
	       }
	       Hbond++ ;
	  }
	  SortHbondRes2(&HBONDS[1]) ;
	  SortHbondRes1(&HBONDS[1]) ;
	  finalhbnds = 0 ;
	  Hbond = &HBONDS[1];
	  while ( Hbond->next != NULL ) { 
	       if ( Hbond->ToPrint ) { 
		    finalhbnds++ ;
		    fprintf(hydrogen,"   %s - %s     %3d", Hbond->AtomDesc1, Hbond->AtomDesc2, Hbond->nStructures);
		    fprintf(stdout,  "   %s - %s     %3d", Hbond->AtomDesc1, Hbond->AtomDesc2, Hbond->nStructures);
		    for ( ii = 0; ii <= total_models; ii++) {
			  fprintf(hydrogen,"%c", Hbond->Where[ii]);
			  fprintf(stdout,  "%c", Hbond->Where[ii]);
		    }
		    fprintf(hydrogen,"\n");
		    fprintf(stdout,"\n");
	       } 
	       Hbond++;
	  }
     	  fprintf(stdout," > HYDRGN_bnd: %d Possible hydrogen bonds written to `%s'\n", finalhbnds, output_name);
     	  fprintf(hydrogen,"    %d hydrogen bonds \n", finalhbnds) ;
     }
     
     if ( !ForAll )
     	  printf("> HYDRGN_bnd:    %d Possible hydrogens bonds written to `%s'\n", hbnds, output_name);
     fclose(hydrogen);
   
     free(HBONDS); 
     return ;
}
