/*$Id: kabsch_sander.c,v 3.32 2003/10/06 12:13:32 roberto Exp roberto $*/
#include "pdbstat.h"
/* ========================================================================= */
void kabsch_sander(char WhichModel[MAXWORD])
/*
  ---
  --- This module to evaluate the possibility of mainchain hydrogen bonding
  --- calculating the energy for that based on the paper by
  --- W. Kabsch and C. Sander, Biopolymers, 22,2577-2637 (1983).
  --- "Dictionary of protein secondary structure: pattern recognition of
  ---  hydrogen-bonded and geometrical features".
  ---
  --- (Roberto Tejero, RTT, May 1993, CABM-Rutgers University).
  ---
  --- File for results added in Dec, 1995 (Valencia).
  --- No *INTRA*residues added in Dec, 1995 (Valencia).
  --- Some changes in format added in Dec, 1995 (Valencia).

  --- added support for all models, format corrected, Munich Airport, May 1, 2003
  ---
*/
{
     
     static float    electron = 1.0 ;
     static float    q1, q2, f, energy, energyone ;
     static float    dON, dCH, dOH, dCN;
     static int      carbon, nitro, hydro, oxygen, hbnds ;
     static int      ii, critical_line;
     static FILE     *kabsch ;
     static short   i, k, mdl, startmdl, endmdl ;
     static struct models  *model ;
     
     q1 = 0.42 * electron ;
     q2 = 0.20 * electron ;
     f = 332.0 ;
     
     critical_line = 0;
     ii = 0;
     mdl = startmdl = endmdl = 0 ;

     if ( strncasecmp(WhichModel,"all",2)==0 ) {
     	   startmdl = 1 ;
	   endmdl = total_models;
     } else if ( strncasecmp(WhichModel,"average",2)==0 ) {
	     startmdl = endmdl = AverageModel ;
     } else {
	   startmdl = endmdl = atoi(WhichModel) ;
     }
     
     /* Open file for results */
     strcpy(output_name,name_of_the_file_coords);
     strcat(output_name,"_Kabsch_Sander");
     
     if ((kabsch=fopen(output_name,"w"))==NULL) {
	  printf(" Something went wrong while opening output file \n");
	  exit (-1);
     }
     
     /* --- Header for the output in file */
     fprintf(kabsch, "    #    ATOM 1           ATOM 2         E    dON   dCN \n");
     fprintf(kabsch, " -------------------  ---------------  ----- ----- ----- \n");
     
     /* --- Header for the output in  screen */
     printf("> KABSCH_sander:    #    ATOM 1            ATOM 2        E    dON   dCN \n");
     printf("> KABSCH_sander: -------------------  ---------------  ----- ----- ----- \n");
    
     for ( mdl = startmdl ; mdl <= endmdl ; mdl++ ) {
	  model = MODELS + mdl ;
	  hbnds = 0;
	  energy = 0.0;
	  for (i = 1 ; i < MAX ; i++)
	       for (k = i+1 ; k < MAX ; k++) { 
		    /*
		      --- Look for mainchain N-O distances lower than 5.2 A
		      --- throwing away the INTRAresidue O-N and N-O distances (Valencia, Dec. 95)
		    */
		    if ((((strncmp(model->PDB[i].pdb_iupac," N ",3)==0) && (strncmp(model->PDB[k].pdb_iupac," O ",3)==0)) ||
			 ((strncmp(model->PDB[i].pdb_iupac," O ",3)==0) && (strncmp(model->PDB[k].pdb_iupac," N ",3)==0))) &&
			(model->PDB[i].pdb_num_res != model->PDB[k].pdb_num_res))
			 if ((dON=(float)sqrt(pow((model->PDB[i].pdb_xcrd - model->PDB[k].pdb_xcrd),2)+
					      pow((model->PDB[i].pdb_ycrd - model->PDB[k].pdb_ycrd),2)+
					      pow((model->PDB[i].pdb_zcrd - model->PDB[k].pdb_zcrd),2))) < 5.2) { /* second level */
			      
			      dCN = 0.0 ;
			      dCH = 0.0 ;
			      dOH = 0.0;
			      hydro = 0;
			      nitro = 0;
			      carbon = 0;
			      oxygen =0;
			      
			      if (model->PDB[k].pdb_iupac[1]=='N') nitro = k ;
			      else oxygen = k;
			      if (model->PDB[i].pdb_iupac[1]=='N') nitro = i ;
			      else oxygen = i;
			      
			      for (ii = k - 15 && ii >= 1 ; ii <= k + 15 && ii < MAX ; ii++) {
				   if ((model->PDB[k].pdb_iupac[1]=='N') && 
				       ((strncmp(model->PDB[ii].pdb_iupac," H ",3)==0) ||
					(strncmp(model->PDB[ii].pdb_iupac,"1H ",3)==0) ||
					(strncmp(model->PDB[ii].pdb_iupac,"2H ",3)==0) ||
					(strncmp(model->PDB[ii].pdb_iupac,"3H ",3)==0)) &&
				       (model->PDB[k].pdb_num_res == 
					model->PDB[ii].pdb_num_res)) {
					hydro = ii;
					nitro = k ;
					break ;
				   }
				   if ((model->PDB[k].pdb_iupac[1]=='O') && 
				       (strncmp(model->PDB[ii].pdb_iupac," C ",3)==0) &&
				       (model->PDB[k].pdb_num_res == model->PDB[ii].pdb_num_res)) {
					carbon = ii;
					oxygen = k;
					break ;
				   }
			      }
			      
			      for (ii = i - 15 && i >= 1 ; ii <= i + 15 && ii < MAX ; ii++) {
				   if ((model->PDB[i].pdb_iupac[1]=='N') && 
				       ((strncmp(model->PDB[ii].pdb_iupac," H ",3)==0) ||
					(strncmp(model->PDB[ii].pdb_iupac,"1H ",3)==0) ||
					(strncmp(model->PDB[ii].pdb_iupac,"2H ",3)==0) ||
					(strncmp(model->PDB[ii].pdb_iupac,"3H ",3)==0)) &&
				       (model->PDB[i].pdb_num_res == model->PDB[ii].pdb_num_res)) {
					hydro = ii;
					nitro = i;
					break ;
				   }
				   if ((model->PDB[i].pdb_iupac[1]=='O') && 
				       (strncmp(model->PDB[ii].pdb_iupac," C ",3)==0) &&
				       (model->PDB[i].pdb_num_res == model->PDB[ii].pdb_num_res)) {
					carbon = ii;
					oxygen = i ;
					break ;
				   }
			      }
			      
			      if ((nitro > MAX-1) || (hydro > MAX-1) || (oxygen > MAX-1) ||
				  (nitro < 1 ) || (carbon < 1) || (oxygen < 1) || (carbon > MAX-1)) {
				   printf("> KABSCH_sander: ** something wrong with indices **\n");
				   printf("> KABSCH_sander: nitrogen: %4d, hydrogen: %4d, HBND: %4d\n",
					  nitro, hydro, hbnds + 1);
				   printf("> KABSCH_sander: carbon: %4d, oxygen: %4d, MAXATOM: %4d\n",
					  carbon, oxygen, MAX);
				   return;
			      }
			      
			      if (hydro != 0) {
				   dCH=(float)sqrt(pow((model->PDB[carbon].pdb_xcrd - model->PDB[hydro].pdb_xcrd),2)+
						   pow((model->PDB[carbon].pdb_ycrd - model->PDB[hydro].pdb_ycrd),2)+
						   pow((model->PDB[carbon].pdb_zcrd - model->PDB[hydro].pdb_zcrd),2)) ;
				   
				   dOH=(float)sqrt(pow((model->PDB[oxygen].pdb_xcrd - model->PDB[hydro].pdb_xcrd),2)+
						   pow((model->PDB[oxygen].pdb_ycrd - model->PDB[hydro].pdb_ycrd),2)+
						   pow((model->PDB[oxygen].pdb_zcrd - model->PDB[hydro].pdb_zcrd),2)) ;
			      } else {
				   dCH = 1.0 ;
				   dOH = 1.0 ;
			      }
			      
			      dCN=(float)sqrt(pow((model->PDB[carbon].pdb_xcrd - model->PDB[nitro].pdb_xcrd),2)+
					      pow((model->PDB[carbon].pdb_ycrd - model->PDB[nitro].pdb_ycrd),2)+
					      pow((model->PDB[carbon].pdb_zcrd - model->PDB[nitro].pdb_zcrd),2)) ;
			      
			      
			      if ((dON <= 0.0) || (dCH <= 0.0) || (dOH <= 0.0) || (dCN <= 0.0)) {
				   printf("> KABSCH_sander: ** something wrong with the distances **\n");
				   return;
			      }
			      
			      if (((q1 * q2 *(1.0/dON + 1.0/dCH - 1.0/dOH - 1.0/dCN)*f) < -0.5) && (dCN > 1.40) &&
				  (strcmp(model->PDB[i].pdb_res,"PRO")!=0) && (strcmp(model->PDB[k].pdb_res,"PRO")!=0)) {
				   energyone = (q1 * q2 *(1.0/dON + 1.0/dCH - 1.0/dOH - 1.0/dCN)*f);
				   energy = energy + energyone ;
				   hbnds++;
				   goto write;
			      }
			      
			      goto sigue;
			 write:
			      critical_line++ ;
			      if (critical_line < 25)   /* one screen of results */
				   printf("> KABSCH_sander: %4d %3s %s %3d %5s  %3s %s %3d %5s %5.2f %5.2f %5.2f \n",
					  hbnds,
					  model->PDB[i].pdb_res, model->PDB[i].pdb_chain_id, 
					  model->PDB[i].pdb_num_res, model->PDB[i].pdb_iupac,
					  model->PDB[k].pdb_res, model->PDB[k].pdb_chain_id, 
					  model->PDB[k].pdb_num_res, model->PDB[k].pdb_iupac,
					  energyone, dON, dCN);
			      fprintf(kabsch," %4d %3s %s %3d %5s  %3s %s %3d %5s %5.2f %5.2f %5.2f \n",
				      hbnds,
				      model->PDB[i].pdb_res, model->PDB[i].pdb_chain_id, 
				      model->PDB[i].pdb_num_res, model->PDB[i].pdb_iupac,
				      model->PDB[k].pdb_res, model->PDB[k].pdb_chain_id, 
				      model->PDB[k].pdb_num_res, model->PDB[k].pdb_iupac,
				      energyone, dON, dCN);
			 sigue:  ;
			 }
	       }

	  printf("> KABSCH_sander: Model: %3d, average EHbond: %.2f for %d hbonds; ET=%.2f Kcal\n", 
			  model->modelno, energy/hbnds, hbnds, energy);
	  fprintf(kabsch, " Model %3d, average EHbond: %.2f for %d hbonds; ET=%.2f Kcal\n", 
			  model->modelno, energy/hbnds, hbnds, energy);
     }     
     fclose(kabsch);
     return ;
}
