/*$Id: math.c,v 3.32 2003/10/06 12:13:32 roberto Exp roberto $*/

#include "pdbstat.h"  
#include "PdbStatMath.h"

int   is_allowed(char Id, int resnumber, int modelnumber) ;
float rmsd_coords(int reference, int kompar, int residue_range, int orientmode, char tipo[MAXWORD]);
int   FindSameAtom(int reference, int kompar, int iatom) ;
int   FindSameSelectedAtom(int reference, int kompar, int iatom, char who[3]) ;
char IsSameModel(int ModelNo1,int ModelNo2);
     
/* ========================================================================= */
float   dihang(int i, int j, int k, int l)
{
  /*
   * ---
   * --- This routine calculates the dihedral between 4 atoms (points in the
   * --- space) giving their indices i,j,k,l. Each point is characterized
   * --- by three coordinates (x,y,z). The code is totally similar to the
   * --- code you can find in DISMAN subroutine DIHANG.f
   * ---
   * --- On May 1993, several problems have been solved. Now all variables
   * --- are carefully defined as double. The math routines powf and sqrtf
   * --- have been changed to pow and sqrt to make consistent the variables
   * --- and the calculations. As it is now the routine has been checked 
   * --- deeply and seems to work properly.
   * ---
   * --- (Roberto Tejero, RTT, MAy 1993, CABM-Rutgers University)
   * ---
   */
     static double       ax,ay,az;
     static double       bx,by,bz;
     static double       cx,cy,cz;
     static double       axbx,axby,axbz;
     static double       bxcx,bxcy,bxcz;
     static double       sclprd,ang1;
     static double       signo ;
     static double       d;
     static struct models *Model ;
     
     ax = 0.0 ;
     ay = 0.0 ;
     az = 0.0 ;
     
     bx = 0.0 ;
     by = 0.0 ;
     bz = 0.0 ;
     
     cx = 0.0 ;
     cy = 0.0 ;
     cz = 0.0 ;
     
     axbx = 0.0 ;
     axby = 0.0 ;
     axbz = 0.0 ;
     
     bxcx = 0.0 ;
     bxcy = 0.0 ;
     bxcz = 0.0 ;
     
     sclprd = 1111.1 ;
     ang1 = 1111.1 ;
     d = 1111.1 ;
     
     signo = 1.0 ;
     
     Model = MODELS + key_model ;
     
     ax = (double)(Model->PDB[j].pdb_xcrd - Model->PDB[i].pdb_xcrd);
     ay = (double)(Model->PDB[j].pdb_ycrd - Model->PDB[i].pdb_ycrd);
     az = (double)(Model->PDB[j].pdb_zcrd - Model->PDB[i].pdb_zcrd);
     bx = (double)(Model->PDB[k].pdb_xcrd - Model->PDB[j].pdb_xcrd);
     by = (double)(Model->PDB[k].pdb_ycrd - Model->PDB[j].pdb_ycrd);
     bz = (double)(Model->PDB[k].pdb_zcrd - Model->PDB[j].pdb_zcrd);
     cx = (double)(Model->PDB[l].pdb_xcrd - Model->PDB[k].pdb_xcrd);
     cy = (double)(Model->PDB[l].pdb_ycrd - Model->PDB[k].pdb_ycrd);
     cz = (double)(Model->PDB[l].pdb_zcrd - Model->PDB[k].pdb_zcrd);
     
     /* */  
     axbx = ay * bz - az * by;
     axby = az * bx - ax * bz;
     axbz = ax * by - ay * bx;
     
     bxcx = by * cz - bz * cy;
     bxcy = bz * cx - bx * cz;
     bxcz = bx * cy - by * cx;
     
     /* */
     d = pow(bxcx,2) + pow(bxcy,2) + pow(bxcz,2);
     d = sqrt((pow(axbx,2) + pow(axby,2) + pow(axbz,2)) * d);
     if ((d == 1111.1) || (d == 0.0))
     {
	  printf("> DIHANG: ** some error in dihedral calculation \n");
	  return -2 * PI ;
     }
     
     /* */
     if (((d > 0.0) && ((axbx * bxcx + axby * bxcy + axbz * bxcz) < 0.0)) ||
	 ((d < 0.0) && ((axbx * bxcx + axby * bxcy + axbz * bxcz) > 0.0)))
	  signo = -1.0;
     
     if ((float)fabs(axbx * bxcx + axby * bxcy + axbz * bxcz) == 
	 (float)fabs(d))
	  ang1 = 0.9999999999 * signo ;
     else
	  ang1 = (axbx * bxcx + axby * bxcy + axbz * bxcz) / d ;
     
     if (fabs(ang1) > 1.0) {
	  printf("> DIHANG: ** Error in calculation %f %f %f  \n",
		 (axbx * bxcx + axby * bxcy + axbz * bxcz),
		 d,
		 ang1);
	  return -2.0 * PI ;
     }
     ang1 = acos (ang1);
     sclprd = ax * (by * cz - bz * cy) - 
	  ay * (bx * cz - bz * cx) +
	  az * (bx * cy - by * cx);
     if (sclprd == 1111.1) {
	  printf("> DIHANG: ** some error in dihedral calculation \n");
	  return -2.0 * PI ;
     }
     if (sclprd <= 0.0) ang1 = -ang1 ;
     if (fabs(ang1) > PI) {
	  printf("> DIHANG: ** some error in dihedral calculation \n");
	  return -2 * PI ;
     }
     return (float)ang1;
}



/* ========================================================================= */
 float distance(int i, int j)
 /*
  * ---
  * --- Calculates the distance between two atoms if we know the atom numbers
  * ---
  */
{
     static double      dst;
     static struct models *Model ;
     
     Model = MODELS + key_model ;
     
     MAX = Model->number_of_atoms ;
     
     if (MAX <= 0) {
	  printf("> DISTANCE: ** No molecule read ** \n");
	  return 0.0;
     }
     
     if ((i > MAX) || (j > MAX)) {
	  printf("> DISTANCE : Error in atom numbers, %d %d, Maximum allowed %d\n",
		 i,j, MAX-1);
	  return -111.1 ;
     }
     
     dst = sqrt(pow((Model->PDB[i].pdb_xcrd - Model->PDB[j].pdb_xcrd),2)+
		pow((Model->PDB[i].pdb_ycrd - Model->PDB[j].pdb_ycrd),2)+
		pow((Model->PDB[i].pdb_zcrd - Model->PDB[j].pdb_zcrd),2));
     
     return (float)dst;
}

/* ========================================================================= */
 void Vector(int Mdln, int i, int j, double *Target)
 /*
  * ---
  * --- Calculates the vector of two atoms if we know the atom numbers
  * ---
  */
{
     static struct models *Model ;
     
     Model = MODELS + Mdln ;
     MAX = Model->number_of_atoms ;
     Target[0] = Target[1] = Target[2] = 0.0;
     
     if (MAX <= 0) {
	  printf("> DISTANCE: ** No molecule read ** \n");
	  return ;
     }
     
     if ((i > MAX) || (j > MAX)) {
	  printf("> VECTOR : Error in atom numbers, %d %d, Maximum allowed %d\n", i,j, MAX-1);
	  return ;
     }
     
     Target[0] = Model->PDB[j].pdb_xcrd - Model->PDB[i].pdb_xcrd ;
     Target[1] = Model->PDB[j].pdb_ycrd - Model->PDB[i].pdb_ycrd ;
     Target[2] = Model->PDB[j].pdb_zcrd - Model->PDB[i].pdb_zcrd;
     
     return ;
}


/* ========================================================================= */
 float angulo(int i, int j, int k)
 /*
  * ---
  * --- This routine is used to calculate the angle between three atoms if we 
  * --- know the atom numbers. 
  * ---
  * --- (Roberto Tejero, RTT, June 1993, CABM-Rutgers University).
  * ---
  */
{
     static double    vector_1x, vector_1y, vector_1z ;
     static double    vector_2x, vector_2y, vector_2z ;
     static double    modulus_1, modulus_2 , angle , num ;
     static struct models *Model ;
     
     /*
      * ---
      * --- Checking
      * ---
      */
     
     Model = MODELS + key_model ;
     
     if (MAX <= 0) {
	  printf("> ANGLES: ** No molecule read ** \n");
	  return 0.0;
     }
     
     if ((i > MAX - 1) || (j > MAX - 1) || (k > MAX -1)) {
	  printf("> ANGLE : Error in atom numbers, %d %d %d, Maximum allowed %d",
		 i,j,k, MAX-1);
	  return -111.1 ;
     }
     
     vector_1x = Model->PDB[i].pdb_xcrd - Model->PDB[j].pdb_xcrd ;
     vector_1y = Model->PDB[i].pdb_ycrd - Model->PDB[j].pdb_ycrd ;
     vector_1z = Model->PDB[i].pdb_zcrd - Model->PDB[j].pdb_zcrd ;
     
     vector_2x = Model->PDB[k].pdb_xcrd - Model->PDB[j].pdb_xcrd ;
     vector_2y = Model->PDB[k].pdb_ycrd - Model->PDB[j].pdb_ycrd ;
     vector_2z = Model->PDB[k].pdb_zcrd - Model->PDB[j].pdb_zcrd ;
     
     num = (vector_1x * vector_2x) + (vector_1y * vector_2y) + (vector_1z * vector_2z) ;
     
     modulus_1 =  sqrt((vector_1x * vector_1x) + (vector_1y * vector_1y) + (vector_1z * vector_1z)) ;
     modulus_2 =  sqrt((vector_2x * vector_2x) + (vector_2y * vector_2y) + (vector_2z * vector_2z)) ;
     
     angle = acos(num/(modulus_1 * modulus_2));
     
     return (float) angle ;
}

/* ========================================================================= */
 float AngleVect(double *V1, double *V2)
 /*
  * Another addition, a routine that computes the angle between 
  * two vectors.
  * Roberto Tejero, CABM, 2002.
  */
{
     static double DotProd, Modulus[2], Angle ;
     
     DotProd = (V1[0] * V2[0]) + (V1[1] * V2[1]) + (V1[2] * V2[2]) ;
     
     Modulus[0] =  sqrt((V1[0] * V1[0]) + (V1[1] * V1[1]) + (V1[2] * V1[2])) ;
     Modulus[1] =  sqrt((V2[0] * V2[0]) + (V2[1] * V2[1]) + (V2[2] * V2[2])) ;
     
     Angle = acos(DotProd/(Modulus[0] * Modulus[1]));
     
     return (float) Angle ;
}


/* ========================================================================= */
float determinant(float U[3][3])
 /*
  * ---
  * --- Little piece of code just to obtain the determinant of
  * --- a 3X3 matrix.
  * ---
  * --- By now (Jul 1993) is NOT used !!
  * ---
  * --- (Roberto Tejero, RTT, June 1993, CABM-Rutgers University).
  * ---
  */
{
     static float      det;
     static int        i1, i2 ;
     static short i;
     
     det = 0.0 ;
     for (i = 1 ; i <= 3 ; i++) {
	  i1 = i + 1;
	  if (i1 > 3) i1 = i1 - 3;
	  i2 = i + 2;
	  if (i2 > 3) i2 = i2 - 3;
	  det = det + U[i][1] * (U[i1][2] * U[i2][3] - U[i2][2] * U[i1][3]);
     }
     return det;
}

/* ========================================================================= */
float rmsd_dihedrals(char clave[MAXWORD])
 /*
  * ---
  * --- This routine is devoted to calculate the rmsd between two
  * --- or more structures just based on the dihedrals omega, phi,
  * --- psi and chi1.
  * --- The planar_h and planar_o were added later.
  * --- 
  * --- (Roberto Tejero, RTT, Jul 1993, CABM-Rutgers University).
  * ---
  */
{
     static int            number_dihe, total ;
     static int            intervals, 
	  from[5],to[5];
     static char           dt[MAXWORD];
     static double         add_omega, 
	  add_phi, 
	  add_psi, 
	  add_chi1,
	  add_planar_h,
	  add_planar_o,
	  add_chiral;
     static float          angulon,
	  target;
     static short i,j,k ;
     static struct models *Key, *Ref, *Modi; 
     
     add_omega = 0.0;
     add_phi = 0.0;
     add_psi = 0.0;
     add_chi1 = 0.0;
     add_planar_h = 0.0;
     add_planar_o = 0.0;
     add_chiral = 0.0;
     number_dihe = 0; 
     total = 0;
     
     /*
      * To manage several intervals of superimposition
      */
     
     Key = MODELS + key_model ;
     Ref = MODELS + AverageModel ;
     
 newone:
     printf("> RMSD_dihdrl: How many intervals (MAXIMUM of five)?_: ");
     GetFromKeyBoard(dt, MAXWORD, stdin);
     intervals=atoi(dt);
     if ((intervals<1) || (intervals>5))  {
	  printf("> RMSD_dihdrl: Numb. of interv. too high,choose again 1-5\n");
	  goto newone;
     }
     
     for (i = 1 ; i <= intervals ; i++) {
	  printf("> RMSD_dihdrl: ranges for interval [%d] blank space separated ?_: ", i);
	  GetFromKeyBoard(dt, MAXWORD, stdin);
	  sscanf(dt,"%d %d ", &from[i],&to[i]);
     }
     
     if (intervals == 1) {
	  if (from[1] == 0) from[1] = 1;
	  if (to[1] == 0 || to[1] > Key->number_of_residues)
	       to[1] = Key->number_of_residues;
     }
     
     /*
      * some checking
      */
     for (i = 1 ; i <= intervals ; i++) {
	  if (from[i] <= 0 || from[i] > Key->number_of_residues ||
	      to[i] <=  0  || to[i] > Key->number_of_residues)
	       goto newone;
     }
     
     for ( j = 1 ; j <= intervals ; j++)
	  for ( i = 1 ; i <= total_models ; i++) {
	       Modi = MODELS + i;
	       /* for ( k = 1 ; k <= Key->number_of_residues ; k++) */
	       for ( k = from[j] ; k <= to[j] ; k++)  {
		    /* omega */
		    if (Modi->DIH[k].omega != 11111.0)  {
			 number_dihe++;
			 angulon = fabs(Modi->DIH[k].omega) ;
			 target = fabs(Ref->DIH[k].omega) ;
			 if (((Modi->DIH[k].omega >= 0.0) && (Ref->DIH[k].omega >= 0.0)) ||
			     ((Modi->DIH[k].omega <= 0.0) && (Ref->DIH[k].omega <= 0.0)))
			      add_omega = add_omega + 
				   pow((double)(angulon - target),2);
			 else
			      if ((angulon + target) < 180.0)
				   add_omega = add_omega + 
					pow((double)(angulon + target),2);
			      else
				   add_omega = add_omega + 
					pow((double)(360.0 - angulon - target),2);
		    }
		    /* phi */
		    if (Modi->DIH[k].phi != 11111.0)  {
			 number_dihe++;
			 angulon = fabs(Modi->DIH[k].phi) ;
			 target = fabs(Ref->DIH[k].phi) ;
			 if (((Modi->DIH[k].phi >= 0.0) && (Ref->DIH[k].phi >= 0.0)) ||
			     ((Modi->DIH[k].phi <= 0.0) && (Ref->DIH[k].phi <= 0.0)))
			      add_phi = add_phi + 
				   pow((double)(angulon - target),2);
			 else
			      if ((angulon + target) < 180.0)
				   add_phi = add_phi + 
					pow((double)(angulon + target),2);
			      else
				   add_phi = add_phi + 
					pow((double)(360.0 - angulon - target),2);
		    }
		    /* psi */
		    if (Modi->DIH[k].psi != 11111.0)  {
			 number_dihe++;
			 angulon = fabs(Modi->DIH[k].psi) ;
			 target = fabs(Ref->DIH[k].psi) ;
			 if (((Modi->DIH[k].psi >= 0.0) && (Ref->DIH[k].psi >= 0.0)) ||
			     ((Modi->DIH[k].psi <= 0.0) && (Ref->DIH[k].psi <= 0.0)))
			      add_psi = add_psi + 
				   pow((double)(angulon - target),2);
			 else
			      if ((angulon + target) < 180.0)
				   add_psi = add_psi + 
					pow((double)(angulon + target),2);
			      else
				   add_psi = add_psi + 
					pow((double)(360.0 - angulon - target),2);
		    }
		    /* chi1 */
		    if (Modi->DIH[k].chi1 != 11111.0)  {
			 number_dihe++;
			 angulon = fabs(Modi->DIH[k].chi1) ;
			 target = fabs(Ref->DIH[k].chi1) ;
			 if (((Modi->DIH[k].chi1 >= 0.0) && (Ref->DIH[k].chi1 >= 0.0)) ||
			     ((Modi->DIH[k].chi1 <= 0.0) && (Ref->DIH[k].chi1 <= 0.0)))
			      add_chi1 = add_chi1 + 
				   pow((double)(angulon - target),2);
			 else
			      if ((angulon + target) < 180.0)
				   add_chi1 = add_chi1 + 
					pow((double)(angulon + target),2);
			      else
				   add_chi1 = add_chi1 + 
					pow((double)(360.0 - angulon - target),2);
		    }
		    /* planar_o */
		    if (Modi->DIH[k].planar_o != 11111.0)  {
			 number_dihe++;
			 angulon = fabs(Modi->DIH[k].planar_o) ;
			 target = fabs(Ref->DIH[k].planar_o) ;
			 if (((Modi->DIH[k].planar_o >= 0.0) && (Ref->DIH[k].planar_o >= 0.0)) ||
			     ((Modi->DIH[k].planar_o <= 0.0) && (Ref->DIH[k].planar_o <= 0.0)))
			      add_planar_o = add_planar_o + 
				   pow((double)(angulon - target),2);
			 else
			      if ((angulon + target) < 180.0)
				   add_planar_o = add_planar_o + 
					pow((double)(angulon + target),2);
			      else
				   add_planar_o = add_planar_o + 
					pow((double)(360.0 - angulon - target),2);
		    }
		    /* planar_h */
		    if (Modi->DIH[k].planar_h != 11111.0)  {
			 number_dihe++;
			 angulon = fabs(Modi->DIH[k].planar_h) ;
			 target = fabs(Ref->DIH[k].planar_h) ;
			 if (((Modi->DIH[k].planar_h >= 0.0) && (Ref->DIH[k].planar_h >= 0.0)) ||
			     ((Modi->DIH[k].planar_h <= 0.0) && (Ref->DIH[k].planar_h <= 0.0)))
			      add_planar_h = add_planar_h + 
				   pow((double)(angulon - target),2);
			 else
			      if ((angulon + target) < 180.0)
				   add_planar_h = add_planar_h + 
					pow((double)(angulon + target),2);
			      else
				   add_planar_h = add_planar_h + 
					pow((double)(360.0 - angulon - target),2);
		    }
		    /* chiral */
		    if (Modi->DIH[k].chiral != 11111.0)  {
			 number_dihe++;
			 angulon = fabs(Modi->DIH[k].chiral) ;
			 target = fabs(Ref->DIH[k].chiral) ;
			 if (((Modi->DIH[k].chiral >= 0.0) && (Ref->DIH[k].chiral >= 0.0)) ||
			     ((Modi->DIH[k].chiral <= 0.0) && (Ref->DIH[k].chiral <= 0.0)))
			      add_chiral = add_chiral + 
				   pow((double)(angulon - target),2);
			 else
			      if ((angulon + target) < 180.0)
				   add_chiral = add_chiral + 
					pow((double)(angulon + target),2);
			      else
				   add_chiral = add_chiral + 
					pow((double)(360.0 - angulon - target),2);
		    }
	       }
	  }
     
     if (Ref->modelno != AverageModel)
	  total = total_models - 1;
     else
	  total = total_models ;
     
     if (total <= 0)  {
	  printf("> RMSD_dhdrl: **** Error with total of models, check \n");
	  return 9999.99;
     }
     
     if (strncmp(clave,"back",2)==0)
	  return (float) sqrt((add_omega + add_phi + add_psi ) / (total * number_dihe));
     else
	  return (float) sqrt((add_omega + add_phi + add_psi + add_chi1 +
			       add_planar_h + add_planar_o + add_chiral ) / (total * number_dihe));    
}





/* ========================================================================= */

/**********************************************************************
 *
 * This rotuine taken from ORIENT program by George F. Elkins and slightly
 * adapted to this case by Roberto Tejero.
 *
 * The following subroutine rotates the protein fragment "molecule" so as
 * to minimize the RMS deviation with "refmol".  The subroutine uses the
 * Lagrange multiplier method of Kabsch (ACTA Crystallographica Section A
 * 32, 922-923, and 34, 827-828). In Kabsch's method, a matrix U is
 * applied to "molecule" to give new coords "U*molecule".  The nine
 * elements of U are viewed as parameters.  The RMS between "U*molecule"
 * and refmol is minimized subject to the constraint that U be
 * orthogonal, and this leaves only three of the parameters independent,
 * as we expect.  The constraint is applied by introducing nine Lagrange
 * multipliers, one for each element of the orthogonality condition UTU =
 * 1.  Then we minimize by setting the first derivative of the object
 * function to zero.  The result of this is that U = R * (RTR)**(-1/2)
 * where R(I,J) is gotten by summing refmol(I) * molecule(J) over all
 * atoms, and RTR is R premultiplied by R transpose.  RTR is raised to
 * the minus one half power by the spectral theorem.  Note that this
 * doesn't work if any eigenvalues are negative, and so we check for
 * negative eigenvalues.  Theoretically RTR is positive definite, but in
 * practice errors occur.  Note also that U is not constrained to have
 * det = +1, and in practice det = -1 occurs frequently.
 *
 ***********************************************************************/

extern void do_copy_model(int from, int to);


/* ========================================================================= */
void optimal_rotate(int reference, int kompar, int residue_range,
                    int orientmode, char tipo[MAXWORD])
{
     /*
      * The following need to be double precision because that
      * is what the FORTRAN numerical routines expect.
      */
     static double RTR[3][3];
     static double U[3][3];
     static double R[3][3];
     static double detU;
     register int    i,j,k,l ;
     register int    iatom , katom;
     static struct models *model ;
     
/* To use LAPACK routines one have to define this */
#ifdef LAPACK
     LWORK = 4;
#endif
     Three1 = 3;
     Three2 = 3;
     
     model = MODELS ;
     
     /*
      * This does NOTHING, it is only to avoid a warning in mips4 compiler
      */
     orientmode++ ;
     orientmode-- ;
     residue_range++ ;
     residue_range-- ;
     
     if (DEBUG > 0) 
	  fprintf(stdout," > optimal_rotate(): Optimal rotation of %s atoms model %d\n",
		  tipo, kompar); 
     
     /* Check if we have a valid model just in case ...  */
     if (((model+kompar)->number_of_atoms <= 0) ||
	 ((model+reference)->number_of_atoms <= 0)) {
	  printf(" ** optimal_rotate(): No model, reference or comparation **\n");
	  return ;
     }
     
     /*
      * Calculate R[i][j], which is the sum over
      * i,j of molptr[i]*refptr[j] for all atoms of concern.
      */
     for (i = 0; i < 3; ++i)
	  for (j = 0; j < 3; ++j) {
	       R[i][j] = 0.0;
	       
	       for (iatom = 1; iatom <= (model+kompar)->number_of_atoms ; ++iatom) 
		    (model+kompar)->PDB[iatom].written = FALSE;
	       
	       for (katom = 1; katom <= (model+reference)->number_of_atoms ; ++katom) 
		    (model+reference)->PDB[katom].written = FALSE;
	       
	       for (iatom = 1; iatom <= (model+kompar)->number_of_atoms ; ++iatom) {
		    if (is_allowed('*',(model+kompar)->PDB[iatom].pdb_num_res,kompar)==0){
			 
			 /* Selected atom set */
			 if (strncmp(tipo,"selected",2)==0) {
			      if ((model+kompar)->PDB[iatom].Selected) {
				   if ((katom = FindSameSelectedAtom(reference,kompar,iatom,"op"))!=-1) {
					(model+reference)->PDB[katom].written = TRUE;
					R[i][j] = R[i][j] +
					     (double) (model+reference)->PDB[katom].coord[i] * 
					     (double) (model+kompar)->PDB[iatom].coord[j] ;
				   } else {
					;
				   }
			      }
			 }
			 
			 /* Backbone */
			 if (strncmp(tipo,"backbone",5)==0) {
			      if ((strncmp((model+kompar)->PDB[iatom].chain,"backbone",4)==0) &&
				  (strncmp((model+kompar)->PDB[iatom].pdb_iupac," O  ",4)!=0)) { 
				   if ((katom = FindSameAtom(reference,kompar,iatom))!=-1) {
					R[i][j] = R[i][j] +
					     (double) (model+reference)->PDB[katom].coord[i] * 
					     (double) (model+kompar)->PDB[iatom].coord[j] ;
				   } else {
					;
				   }
			      }
			 }
			 
			 if (strncmp(tipo,"backO",5)==0) {
			      if ((strncmp((model+kompar)->PDB[iatom].chain,"backbone",4)==0) ||
				  (strncmp((model+kompar)->PDB[iatom].pdb_iupac," O  ",4)==0)) {
				   if ((katom = FindSameAtom(reference,kompar,iatom))!=-1) {
					R[i][j] = R[i][j] +
					     (double) (model+reference)->PDB[katom].coord[i] * 
					     (double) (model+kompar)->PDB[iatom].coord[j] ;
				   } else {
					;
				   }
			      }
			 }
			 
			 if (strncmp(tipo,"backCB",5)==0) {
			      if ((strncmp((model+kompar)->PDB[iatom].chain,"backbone",4)==0) ||
				  (strncmp((model+kompar)->PDB[iatom].pdb_iupac," CB ",4)==0)) {
				   if ((katom = FindSameAtom(reference,kompar,iatom))!=-1) {
					R[i][j] = R[i][j] +
					     (double) (model+reference)->PDB[katom].coord[i] * 
					     (double) (model+kompar)->PDB[iatom].coord[j] ;
				   } else {
					;
				   }
			      }
			 }
			 
			 if (strncmp(tipo,"heavy",3)==0) {
			      if (strncasecmp((model+kompar)->PDB[iatom].type,"heavy",2)==0) {
				   if ((katom = FindSameAtom(reference,kompar,iatom))!=-1) {
					R[i][j] = R[i][j] +
					     (double) (model+reference)->PDB[katom].coord[i] *  
					     (double) (model+kompar)->PDB[iatom].coord[j] ;
				   } else {
					;
				   }
			      }
			 }
			 
			 if (strncmp(tipo,"all",3)==0) {
			      if ((katom = FindSameAtom(reference,kompar,iatom))!=-1) {
				   R[i][j] = R[i][j] +
					(double) (model+reference)->PDB[katom].coord[i] * 
					(double) (model+kompar)->PDB[iatom].coord[j] ;
			      }
			 } else {
			      ;
			 }
		    } /* if allowed */
	       }   /* loop for residue ranges */
	  }     /* for (int j = 0; j < 3; ++j) */
     
     if (DEBUG > 0) 
	  printf(" > optimal_rotate(): Calling EISPACK FORTRAN routines \n");
     
     for (i = 0; i < 3; ++i)
	  for (j = 0; j < 3; ++j) {
	       RTR[i][j] = 0.0;
	       for (k = 0; k < 3; ++k)
		    RTR[i][j] += R[k][i] * R[k][j];
	  }
     
#ifdef LAPACK
     /* Lapack routines */
     Lower='L';
     dsytrd_(&Lower,&Three1,RTR,&Three1,EigenValue,OffDiag,TAU,WORK,&LWORK,&ierr);
     if (ierr != 0) fprintf(stderr," Illegal %d\n", ierr);
     dorgtr_(&Lower,&Three1,RTR,&Three1,TAU,WORK,&LWORK,&ierr);
     if (ierr != 0) fprintf(stderr," Illegal %d\n", ierr);
     Lower='V';
     dsteqr_(&Lower,&Three1,EigenValue,OffDiag,RTR,&Three2,WORK,&ierr);
     if (ierr != 0) fprintf(stderr," Illegal %d\n", ierr);
     for (i = 0; i < 3; ++i) 
	  for (j = 0; j < 3; ++j)
	       EigenVector[i][j] = RTR[i][j];
#else  
     /* Two Eispack FORTRAN subroutines are called here. */
     tred2_(&Three1, &Three2, RTR, EigenValue, Offdiagonal, EigenVector); 
     
     /* EigenVector now contains an orthogonal matrix, hopefully */
     tql2_(&Three1, &Three2, EigenValue, Offdiagonal, EigenVector, &ierr);
     
     if (ierr != 0) 
	  printf(" > optimal_rotate: tql2_ returned *NO* convergence after 30 iterations.\n");
#endif
     
     if (DEBUG > 2) {
	  fprintf(stdout," > RTR[i][j] matrix is: \n");
	  for (i = 0; i < 3; ++i) {
	       fprintf(stdout,"\n");
	       for (j = 0; j < 3; ++j) {
		    fprintf(stdout,"  %13.10f ", RTR[i][j]);
	       }
	  }
	  fprintf(stdout,"\n");
     }
     
     /*  Test for negative eigenvalues */
     for (i = 0; i < 3; ++i) {
	  if (EigenValue[i] < 0.0) {
	       printf(" > ** optimal_rotate(): ** negative eigenvalue detected **"\
		      "for model %d \n",
		      kompar);
	       return;
	  }
	  EigenValue[i] = 1.0/sqrt(EigenValue[i]);
     }
     
     if (DEBUG > 0) 
	  printf(" > optimal_rotate(): Evaluating rotation matrix \n");
     
     /*  Now calculate rotation matrix U */
     for (i = 0; i < 3; ++i)
	  for (j = 0; j < 3; ++j) {
	       U[i][j] = 0.0;
	       for (k = 0; k < 3; k++)
		    for (l = 0; l < 3; l++)
			 U[i][j] = U[i][j] +
			      R[i][k] * EigenVector[l][k] * EigenValue[l] * EigenVector[l][j];
	  }
     
     if (DEBUG > 1) {
	  fprintf(stdout," > Rotation, U[i][j], matrix is: \n");
	  for (i = 0; i < 3; ++i) {
	       fprintf(stdout,"\n");
	       for (j = 0; j < 3; ++j) {
		    fprintf(stdout,"  %13.10f ", U[i][j]);
	       }
	  }
	  fprintf(stdout,"\n");
     }
     
     /*  Inversion test:  Calculate Det(U) */
     detU = U[0][0] * (U[1][1] * U[2][2] - U[2][1] * U[1][2])
	  - U[0][1] * (U[1][0] * U[2][2] - U[2][0] * U[1][2])
	  + U[0][2] * (U[1][0] * U[2][1] - U[2][0] * U[1][1]);
     if (detU < 0.0) {
	  printf(" >> optimal_rotate(): DET(U) is less than zero for model %d \n",
		 kompar);
	  (model+kompar)->determ = TRUE ;
	  return;
     } else
	  (model+kompar)->determ = FALSE ;
     
     if (DEBUG > 0) 
	  printf(" > optimal_rotate(): Copying model to temporary space\n"); 
     
     do_copy_model(kompar,AverageModel+1);
     
     for (iatom = 1 ; iatom <= (model+kompar)->number_of_atoms ; ++iatom)
	  for (i = 0; i < 3; ++i) {
	       MODELS[AverageModel+1].PDB[iatom].coord[i] = 0.0;
	       for (j = 0; j < 3; ++j)
		    MODELS[AverageModel+1].PDB[iatom].coord[i] = 
			 (float) MODELS[AverageModel+1].PDB[iatom].coord[i] + 
			 (float) U[i][j] * (float) (model+kompar)->PDB[iatom].coord[j];
	  }
     
     /*  Copy transformed molecule back into original structure */  
     do_copy_model(AverageModel+1,kompar);
     
     for (iatom = 1 ; iatom <= (model+kompar)->number_of_atoms ; ++iatom) {
	  (model+kompar)->PDB[iatom].pdb_xcrd = (model+kompar)->PDB[iatom].coord[0] ;
	  (model+kompar)->PDB[iatom].pdb_ycrd = (model+kompar)->PDB[iatom].coord[1] ;
	  (model+kompar)->PDB[iatom].pdb_zcrd = (model+kompar)->PDB[iatom].coord[2] ;
     }  
     return;
}



/* ========================================================================= */
float rmsd_coords(int reference, int kompar, int residue_range,
		  int orientmode, char tipo[MAXWORD])
 /*
  * ---
  * --- (Roberto Tejero, RTT, MAy 1993, CABM-Rutgers University)
  * ---
  */
     
{
     register int      iatom, katom, i;
     static int      atmcont ;
     static float    tmprmsd;
     static struct models *model ;
     
     model = MODELS ;
     
     katom = -1 ;
     tmprmsd = 0.0;
     atmcont = 0;
     
     /*
      * This does NOTHING is only to avoid a warning in mips4 compiler
      */
     orientmode++ ;
     orientmode-- ;
     residue_range++ ;
     residue_range-- ;
     
     /*
      * Check if we have a model, anyway
      */
     if (((model+kompar)->number_of_atoms <= 0) ||
	 ((model+reference)->number_of_atoms <= 0)) {
	  printf(" > rmsd_coords() : No model, reference or comparation \n ");
	  return (float) -9999.9 ;
     }
     
     for (iatom = 1; iatom <= (model+kompar)->number_of_atoms ; ++iatom) 
	  (model+kompar)->PDB[iatom].written = FALSE;
     
     for (katom = 1; katom <= (model+reference)->number_of_atoms ; ++katom) 
	  (model+reference)->PDB[katom].written = FALSE;
     
     /*
      * Start rmsd calculations
      */
     
     for (iatom = 1 ; iatom <= (model+kompar)->number_of_atoms ; ++iatom) {
	  if (is_allowed('*',(model+kompar)->PDB[iatom].pdb_num_res,kompar)==0) {
	       
	       /*
		* Only selected atom set
		* Added Dec. 1998, Valencia, RTT.
		*/
	       if (strncmp(tipo,"selected",2)==0) {
		    if ((model+kompar)->PDB[iatom].Selected) {
			 
			 if (DEBUG > 2)
			      fprintf(stdout," > rmsd_coords(): Working ... %3d %s%-3d %s \n",
				      (model+kompar)->modelno,
				      (model+kompar)->PDB[iatom].pdb_res,
				      (model+kompar)->PDB[iatom].pdb_num_res,
				      (model+kompar)->PDB[iatom].pdb_iupac);
			 
			 if ((katom = FindSameSelectedAtom(reference,kompar,iatom,"rm"))!=-1) {
			      atmcont++;
			      (model+reference)->PDB[katom].written = TRUE;
			      for (i = 0 ; i < 3 ; ++i) {
				   tmprmsd = tmprmsd +
					((model+reference)->PDB[katom].coord[i] -
					 (model+kompar)->PDB[iatom].coord[i]) *
					((model+reference)->PDB[katom].coord[i] -
					 (model+kompar)->PDB[iatom].coord[i]) ;
			      }
			 } else {
			      ;
			 }
		    }
	       }
	       
	       /*
		* Backbone N,CA,C plus CB
		*/
	       if (strncmp(tipo,"backCB",5)==0) {
		    if ((strncmp((model+kompar)->PDB[iatom].chain,"backbone",4)==0) ||
			(strncmp((model+kompar)->PDB[iatom].pdb_iupac," CB ",4)==0)) {
			 if ((katom = FindSameAtom(reference,kompar,iatom)) != -1) {
			      atmcont++;
			      for (i = 0 ; i < 3 ; ++i) {
				   tmprmsd = tmprmsd +
					((model+reference)->PDB[katom].coord[i] -
					 (model+kompar)->PDB[iatom].coord[i]) *
					((model+reference)->PDB[katom].coord[i] -
					 (model+kompar)->PDB[iatom].coord[i]) ;
			      }
			 } else {
			      ;
			 }
		    }
	       }
	       
	       /*
		* Backbone N,CA,C plus O
		*/
	       if (strncmp(tipo,"backO",5)==0) {
		    if ((strncmp((model+kompar)->PDB[iatom].chain,"backbone",4)==0) ||
			(strncmp((model+kompar)->PDB[iatom].pdb_iupac," O  ",4)==0)) {
			 if ((katom = FindSameAtom(reference,kompar,iatom)) != -1) {
			      atmcont++;
			      for (i = 0 ; i < 3 ; ++i) {
				   tmprmsd = tmprmsd +
					((model+reference)->PDB[katom].coord[i] -
					 (model+kompar)->PDB[iatom].coord[i]) *
					((model+reference)->PDB[katom].coord[i] -
					 (model+kompar)->PDB[iatom].coord[i]) ;
			      }
			 } else {
			      ;
			 }	    
		    }
	       }
	       
	       /*
		* Backbone N,CA,C 
		*/
	       if (strncmp(tipo,"backbone",5)==0) {
		    if ((strncmp((model+kompar)->PDB[iatom].chain,"backbone",4)==0) &&
			(strncmp((model+kompar)->PDB[iatom].pdb_iupac," O  ",4)!=0)) {
			 if ((katom = FindSameAtom(reference,kompar,iatom)) != -1) {
			      atmcont++;
			      for (i = 0 ; i < 3 ; ++i) {
				   tmprmsd = tmprmsd +
					((model+reference)->PDB[katom].coord[i] -
					 (model+kompar)->PDB[iatom].coord[i]) *
					((model+reference)->PDB[katom].coord[i] -
					 (model+kompar)->PDB[iatom].coord[i]) ;
			      }
			 } else {
			      ;
			 }
		    }
	       }
	       
	       /*
		* Heavy atoms
		*/
	       if (strncmp(tipo,"heavy",3)==0) {
		    if (strncmp((model+kompar)->PDB[iatom].type,"heavy",2)==0) {
			 katom = FindSameAtom(reference,kompar,iatom) ;
			 if (katom != -1) {
			      atmcont++;
			      for (i = 0 ; i < 3 ; ++i) {
				   tmprmsd = tmprmsd +
					((model+reference)->PDB[katom].coord[i] -
					 (model+kompar)->PDB[iatom].coord[i]) *
					((model+reference)->PDB[katom].coord[i] -
					 (model+kompar)->PDB[iatom].coord[i]) ;
			      }
			 } else {
			      ;
			 }
		    }
	       }
	       
	       /*
		* All atoms in molecule
		*/
	       if (strncmp(tipo,"all",3)==0) {
		    if ((katom = FindSameAtom(reference,kompar,iatom)) != -1) {
			 atmcont++;
			 for (i = 0 ; i < 3 ; ++i) {
			      tmprmsd = tmprmsd +
				   ((model+reference)->PDB[katom].coord[i] -
				    (model+kompar)->PDB[iatom].coord[i]) *
				   ((model+reference)->PDB[katom].coord[i] -
				    (model+kompar)->PDB[iatom].coord[i]) ;
			 }
		    } else {
			 ;
		    }
	       }
	  } else {
	       ;
	  }
     }
     
     return  (float) sqrt((double) tmprmsd / 
			  (double) atmcont );
}


/* ========================================================================= */
int is_allowed(char Id, int resnumber, int modelnumber)
 /*
  * --- Little routine to check if the atoms in a molecule are 
  * --- within the ranges selected for superposition.
  * --- It returns: 0 in case the atom is between allowed ranges
  * ---            -1 in case the atom is outside ranges allowed
  * ---
  * --- (Roberto Tejero, RTT, MAy 1993, CABM-Rutgers University)
  * ---
  */
     
{
     static int i, allowed = -1 ;
     static struct models *modelo ;
     
     modelo = MODELS + modelnumber ;
     
     /* Get ranges from struct model */
     if (SELECTED) {
	  RANGES.number = modelo->ranges.number ;
	  for (i=1; i<= RANGES.number; i++) {
	       RANGES.from[i] = modelo->ranges.from[i];
	       RANGES.to[i] = modelo->ranges.to[i];
	  }
     }
     
     /* Check ranges, just in case */
     if (RANGES.number <= 0)   RANGES.number = 1 ;
     if (RANGES.from[1] <= 0)  RANGES.from[1] = 1 ;
     if (RANGES.to[1] <= 0)    RANGES.to[1] = modelo->number_of_residues ;
     
     if (RANGES.to[1] > (modelo->number_of_residues + modelo->PDB[1].pdb_num_res) ) {
	  fprintf(stderr," ** Range bigger than residues in model, resetting to res max\n");
	  RANGES.to[1] = modelo->number_of_residues + modelo->PDB[1].pdb_num_res ;
     }
     
     /* look for allowed or not */
     for (i = 1; i <= RANGES.number ; ++i) {
	  if ((resnumber >= RANGES.from[i]) && (resnumber <= RANGES.to[i]) &&
			  (RANGES.chain_id[i][0] == Id || Id == '*')) {
	       allowed = 0 ;
	       break ;
	  } else {
	       allowed = -1 ; 
	  }
     }
     return allowed ;
}


/* ========================================================================= */
/*
 * Values needed for random-generating routine Ran1
*/
#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

float ran1(long *idum)
{
	int j;
	long k;
	static long iy=0;
	static long iv[NTAB];
	float temp;

	if (*idum <= 0 || !iy) {
		if (-(*idum) < 1) *idum=1;
		else *idum = -(*idum);
		for (j=NTAB+7;j>=0;j--) {
			k=(*idum)/IQ;
			*idum=IA*(*idum-k*IQ)-IR*k;
			if (*idum < 0) *idum += IM;
			if (j < NTAB) iv[j] = *idum;
		}
		iy=iv[0];
	}
	k=(*idum)/IQ;
	*idum=IA*(*idum-k*IQ)-IR*k;
	if (*idum < 0) *idum += IM;
	j=(int)(iy/NDIV);
	iy=iv[j];
	iv[j] = *idum;
	if ((temp=AM*(float)iy) > RNMX) return RNMX;
	else return temp;
}
#undef IA
#undef IM
#undef AM
#undef IQ
#undef IR
#undef NTAB
#undef NDIV
#undef EPS
#undef RNMX


/* ========================================================================= */
int FindSameAtom(int reference, int kompar, int iatom)
{
  /*
   * FindSameAtom is used to find the same atom name in a superposition
   */
  register int index ;
  static struct models *model;

  model = MODELS ;

  if ( reference == kompar ) return ((int) iatom);

  if (iatom < 31) 
    index = 1 ;
  else
    index = iatom - 1;

  for (index = 1 ; index <= (model+reference)->number_of_atoms+1; index++) 
    if (((model+reference)->PDB[index].pdb_num_res == (model+kompar)->PDB[iatom].pdb_num_res) &&
	(strcmp((model+reference)->PDB[index].pdb_chain_id, (model+kompar)->PDB[iatom].pdb_chain_id) == 0) &&
	(strcmp((model+reference)->PDB[index].pdb_res, (model+kompar)->PDB[iatom].pdb_res) == 0) &&
	(strcmp((model+reference)->PDB[index].pdb_iupac, (model+kompar)->PDB[iatom].pdb_iupac) == 0)) 
      break ;

  /*
   * Checking index value
   */
  if ( index > (model+reference)->number_of_atoms ) {
    index = -1 ;
    fprintf(stdout," FindSameAtom(): Atom _NOT_ Found %3d %s%-3d %s \n",
	    (model+kompar)->SerialNumber,
	    (model+kompar)->PDB[iatom].pdb_res,
	    (model+kompar)->PDB[iatom].pdb_num_res,
	    (model+kompar)->PDB[iatom].pdb_iupac);
  } else {
    if (DEBUG > 4) 
      fprintf(stdout," > FindSameAtom(): Found %3d %s%-3d %s -  %3d %s%-3d %s \n",
	      (model+kompar)->SerialNumber,
	      (model+kompar)->PDB[iatom].pdb_res,
	      (model+kompar)->PDB[iatom].pdb_num_res,
	      (model+kompar)->PDB[iatom].pdb_iupac,
	      (model+reference)->SerialNumber, 
	      (model+reference)->PDB[index].pdb_res,
	      (model+reference)->PDB[index].pdb_num_res,
	      (model+reference)->PDB[index].pdb_iupac);
  }

  return ((int) index) ;
}

/* ========================================================================= */
int FindSameSelectedAtom(int reference, int kompar, int iatom, char who[3])
{
  /*
   * FindSameSelectedAtom is used to find the same atom name in a superposition
   */
  register int index ;
  static struct models *model;
  
  model = MODELS ;  
  if ( reference == kompar ) return ((int) iatom);
  
  for (index = 1 ; index <= (model+reference)->number_of_atoms+1; index++) 
    if ( ((model+reference)->PDB[index].Selected) && (!(model+reference)->PDB[index].written)  ) 
      if ( (strcmp((model+reference)->PDB[index].pdb_iupac, (model+kompar)->PDB[iatom].pdb_iupac) == 0) &&
	   (strcmp((model+reference)->PDB[index].pdb_chain_id, (model+kompar)->PDB[iatom].pdb_chain_id) == 0) ) {
	if (DEBUG == -1)
	  fprintf(stdout,
		  " > Looking: %3d %s%-3d %s, Found: %3d %s%-3d %s \n",
		  (model+kompar)->SerialNumber,
		  (model+kompar)->PDB[iatom].pdb_res,
		  (model+kompar)->PDB[iatom].pdb_num_res,
		  (model+kompar)->PDB[iatom].pdb_iupac,
		  (model+reference)->SerialNumber, 
		  (model+reference)->PDB[index].pdb_res,
		  (model+reference)->PDB[index].pdb_num_res,
		  (model+reference)->PDB[index].pdb_iupac); 
	goto getout ;
      }

 getout:  
  /* Checking index value */
  if ( index > (model+reference)->number_of_atoms ) {
    if (strncmp(who,"rmsd",2)==0) fprintf(stdout," >Rmsd calcs ");
    if (strncmp(who,"opti",2)==0) fprintf(stdout," >Opt Rotate ");
    fprintf(stderr," FindSameSelectedAtom(): *Oops* atom"
	    " %d,  %s%-3d, %s _NOT_ found in Model %d\n",
	    (model+kompar)->SerialNumber,
	    (model+kompar)->PDB[iatom].pdb_res,
	    (model+kompar)->PDB[iatom].pdb_num_res,
	    (model+kompar)->PDB[iatom].pdb_iupac,
	    (model+reference)->modelno);
    index = -1 ;
  }
  return ((int) index) ;
}

/* ========================================================================= */
char IsSameModel(int ModelNo1,int ModelNo2)
{
  static struct models *model1, *model2 ;
  int Icounter ;

  model1 = MODELS+ModelNo1 ;
  model2 = MODELS+ModelNo2 ;

  for (Icounter =  model1->PDB[1].pdb_num_res ;
       Icounter <= model1->number_of_residues ;
       Icounter++) 
    if (strcmp(model1->DIH[Icounter].resname, model2->DIH[Icounter].resname)!=0)
      return FALSE ;
    
  return TRUE ;
}
