/*$Id: molecular_weight.c,v 3.32 2003/10/06 12:13:32 roberto Exp roberto $*/
                   
#include "pdbstat.h"
/* ========================================================================= */
void MolWeight(int choice)
/*
  ---
  --- Evaluates molecular weight of requested model and also determines if
  --- it is from crystal or not in a very drude way, NO hydrogrens.
  ---
  --- Choices: different values of choice
  ---     0         do not print results
  ---    any > 0    print results
  ---
  --- (Roberto Tejero, RTT, June 1993, CABM-Rutgers University).
  ---
*/
{
     static float  molecular_weight;
     static int    count, count_C, count_H, count_N, count_S, count_O, count_funny, count_Q ;
     static struct models *model ;
     
     model = MODELS+key_model ;
     MAX = model->number_of_atoms + 1;
     
     if (MAX <= 1) {
	  printf("> INFO_mol: ** No molecule read ** \n");
	  return;
     }
     
     cristal = 0 ;         /* equals 1 when crystal structure, no Hydrogens */
     molecular_weight = 0.0;
     count_C = 0;  count_H = 0;     count_O = 0;   count_N = 0;     
     count_S = 0;  count_funny = 0; count_Q = 0;
     
     for (count = 1 ; count < MAX ; count++) {
	  if      (model->PDB[count].pdb_iupac[1]=='N') count_N++;
	  else if (model->PDB[count].pdb_iupac[1]=='C') count_C++;
	  else if (model->PDB[count].pdb_iupac[1]=='O') count_O++;
	  else if (model->PDB[count].pdb_iupac[1]=='S') count_S++;
	  else if (model->PDB[count].pdb_iupac[1]=='H') count_H++;
	  else if (model->PDB[count].pdb_iupac[1]=='Q') count_Q++;
	  else 
	       count_funny++;
	  /* assigning masses to each atom */
	  model->PDB[count].pdb_masa=0.0;
	  
	  if (model->PDB[count].pdb_iupac[1]=='N') model->PDB[count].pdb_masa=14.0067;
	  if (model->PDB[count].pdb_iupac[1]=='H') model->PDB[count].pdb_masa=1.0079;
	  if (model->PDB[count].pdb_iupac[1]=='C') model->PDB[count].pdb_masa=12.011;
	  if (model->PDB[count].pdb_iupac[1]=='O') model->PDB[count].pdb_masa=15.9994;
	  if (model->PDB[count].pdb_iupac[1]=='S') model->PDB[count].pdb_masa=32.06;
	  if (model->PDB[count].pdb_iupac[1]=='P') model->PDB[count].pdb_masa=30.97376;
	  if (model->PDB[count].pdb_iupac[1]=='Q') model->PDB[count].pdb_masa=0.0;
	  
	  /* --- check --- */	  
	  if ((model->PDB[count].pdb_iupac[1] != 'Q') &&
	      (model->PDB[count].pdb_masa==0.0)) {
	       printf("> INFO_mol: ** Warning atom without mass, %4d %-3s %3d %-4s\n",
		      model->PDB[count].pdb_num,
		      model->PDB[count].pdb_res,
		      model->PDB[count].pdb_num_res,
		      model->PDB[count].pdb_iupac);
	       return ;
	  }
	  /* --- calculate molecular wegiht --- */
	  molecular_weight = molecular_weight + model->PDB[count].pdb_masa;
     }
     if (count_H <= 0) cristal = 1;
     
     /* Print only in case that choice != 0 */
     if (choice != 0) {
	  if (count_funny == 0)
	       printf("> INFO_mol: # atoms:    %7d (%d C, %d H, %d O, %d N, %d S, %d Q)\n",
		      model->number_of_atoms,
		      count_C,
		      count_H,
		      count_O,
		      count_N,
		      count_S,
		      count_Q);
	  else
	       printf("> INFO_mol: # atoms:    %7d (%d C, %d H, %d O, %d N, %d S, %d Q, %d funny)\n", 
		      model->number_of_atoms, 
		      count_C, 
		      count_H, 
		      count_O, 
		      count_N, 
		      count_S, 
		      count_Q, count_funny);
	  
	  printf("> INFO_mol: # residues:  %6d (Avg. mol. weight: %4.1f)\n",
		 model->PDB[MAX-1].pdb_num_res - model->PDB[1].pdb_num_res + 1,
		 molecular_weight/(model->PDB[MAX-1].pdb_num_res - model->PDB[1].pdb_num_res + 1));
	  printf("> INFO_mol: # -- M.W. :%8.1f g/mol. (%.2f kD)\n",
		 molecular_weight, molecular_weight/1000.0);
     }
     return ;
}    
