/*$Id: pdbstat.h,v 3.32 2003/10/06 12:13:32 roberto Exp roberto $*/

/*
  ---
*/

#include "general.h"

extern struct KnownDist {
  struct KnownDist *next ;
  int   Record;
  char  Atom1Name[5];
  char  Atom2Name[5];
  char  ResidueName[4];
  char  Intra ;
  char  Sequential ;
  double Dmin;
  double Dmax;
  char  Comment1[20];
  char  Comment2[20];
} DescriptionKnownDistances ;

extern struct KnownDist     *DistKnown ;

extern
struct HydrogenBond {
	struct HydrogenBond *next ;
	int  rec;
	int  modeln;
	int  ResN1 ;
	int  ResN2 ;
	int  nStructures ;
	char AtomDesc1[20];
	char AtomDesc2[20];
	char ToPrint ;
	char Where[100];
} Hbond ;

extern struct HydrogenBond *HBONDS ;

extern
struct ResAtom {
        char Label[6];
        int  Number ;
        int  NumberOne ;
        int  Len;
} AtomToLook ;

extern struct ResAtom *ResAt ;

extern 
struct AtMissing {
  struct AtMissing *next ;
  int  Atom ;
  int  Model ;
  char Chain[2];
  char Residue[5];
  int  ResNum ;
  char Name[6];
  char Found ;
} AtomsStored;

/*
 * Residual ACO Violation structure
 */
extern 
  struct ResidualACOViol {
         struct ResidualACOViol *next ;
	 int Model, Total ;
	 int NumLt5, NumLt10, NumGt10, GreaterThanThreshold ;
	 float MinViol, MaxViol, Rmsd, Average ;
 } RESACOVIOL;
extern struct ResidualACOViol *ResAcoViol ;

 /*
 * Residual Violation structure
 */

extern
 struct ResidualViol {
         struct ResidualViol *next ;
	 int Model, Total ;
	 int NumLt01, NumLt02, NumLt05, NumLt1, GreaterThanThreshold ;
	 float MinViol, MaxViol, Rmsd, Average ;
 } RESVIOL ;

extern struct ResidualViol *ResViol ;

extern
struct range {
     int   number;
     char  chain_id[MAXRANGES][2];
     int   from[MAXRANGES];
     int   to[MAXRANGES];
     char  quality[MAXRANGES][5];
} RANGES ;
extern struct range *ranges;     

extern
struct modelrange {
     int   number;
     char  chain_id[MAXMODELS][2];
     int   from[MAXMODELS];
     int   to[MAXMODELS];
} MODELRANGES ;
extern struct modelrange *modelranges;

extern
  struct jcoup {
    char  chain_id[2];
    char  name[8], nickname[8];
    int   residue_number1;
    int   residue_number1_orig;
    int   residue_number2;
    int   residue_number2_orig;
    char  residue_type1[5];
    char  residue_type2[5];
    int   atom_number1;
    int   atom_number2;
    char  atom_type1[6];
    char  atom_type2[6];
    float jc, error;
    float angle1;
    float angle2;
    char  flag[2];
  } jcoup_defs ;

extern struct jcoup *JCOUP ;

extern
struct noe {
     struct noe *next ;
     int   Number ;
     char  HyperName[8];
     char  LastRes1, LastRes2, FirstRes1, FirstRes2 ;
     char  segid_1[5], segid_2[5] ;
     char  chain_id_1[2],  chain_id_2[2];
     int   residue_number1;
     char  residue_type1[5];
     int   atom_number1;
     char  atom_type1[6];
     char  Atom1Ecepp[6] ;
     char  Atom1Dyana[6] ;
     char  Atom1Cyana[6] ;
     char  Atom1Diana[6] ;
     char  Atom1Disman[6] ;
     char  Atom1Xplor[6] ;
     char  Atom1Congen[6] ;
     char  Atom1Pdb[6] ;
     int   residue_number2;
     char  residue_type2[5];
     int   atom_number2;
     char  atom_type2[6];
     char  Atom2Ecepp[6] ;
     char  Atom2Dyana[6] ;
     char  Atom2Cyana[6] ;
     char  Atom2Diana[6] ;
     char  Atom2Disman[6] ;
     char  Atom2Xplor[6] ;
     char  Atom2Congen[6] ;
     char  Atom2Pdb[6] ;
     float equil, rgupp, rglow ;
     float lower, upper;
     float Dist[MAXMODELS];
     char  comment[MAXWORD];
     float volume;
     char  code[2];
     char ToPrint ;
     int iat1[20], iat2[20] ;
} noedefs ;

extern struct noe *NOE ;

/*
 * Structure for angle constraints
 */
extern struct aco {
   struct aco *next ;
   int   Number ;
   char  chain_id[2];
   int   residue_number ;
   char  residue_type[5];
   char  AngleName[6];
   int   Res1, Res2, Res3, Res4;
   char  Atom1[6], Atom2[6], Atom3[6], Atom4[6];
   float Weight, AngleMin, AngleMax, AngleEq, AngleRg ;
   float AngleModel[MAXMODELS];
 } acodefs ;

extern struct aco *ACO ;

/* Library of names for NOEs in different programs */
extern
 struct Library {         /* Library of names for atoms in different prog */
   struct Library *next;
   char Descriptor[5];    /* Dscriptor of record, either ATOM, NMR, DIHE */
   char ResidueName[5];   /* Name of the residue one or three letter */
   char EceppName[6];     /* Name of atom in ECEPP convention */
   char DianaName[6];     /* Name of atom in Diana convention */
   char DyanaName[6];     /* Name of atom in DYANA convention */
   char CyanaName[6];     /* Name of atom in DYANA convention */
   char CongenName[6];    /* Name of the atom in CONGEN convention */
   char XPlorName[6];     /* Name of the atom in X-PLOR convention */
   char DismanName[6];    /* Name of atom in Disman convention */
   char PdbName[6];       /* Name of atom in PDB convention */
   int  Number;           /* Number of the record. It is assigned in reading */
   char Done;
 } DescriptionLibrary ;

/* Library of dihedral names and specifications */
extern
 struct DiheLib {         
   struct DiheLib *next ;
   char ResidueName[5];   /* Name of the residue one or three leter */
   char Atom1[6], Atom2[6], Atom3[6], Atom4[6];
   char DihName[6];
   int Number;            /* Number of the record. It is assigned in reading */
 } DihedralLibrary ;

extern
  struct atom_pdb {
    struct atom_pdb *next ;
    char  Changed ;
    char  TruePdb ;
    char  Selected ;
    char  pdb_key[8];
    char  written ;
    int   pdb_num;
    char  pdb_iupac[6];
    char  pdb_res[5];
    char  pdb_chain_id[2];
    char  segid[5];
    char  chain[10] ;
    char  type[10] ;
    char  AtomType[2];
    int   pdb_num_res;
    int   pdb_num_res_orig;
    int   orden ;
    float pdb_xcrd;
    float pdb_ycrd;
    float pdb_zcrd;
    float pdb_oldx;
    float pdb_oldy;
    float pdb_oldz;
    float coord[3];
    float pdb_data1;
    float pdb_data2;
    float pdb_masa;
  } PDBatoms ;

 extern struct atom_pdb *PDB ;

extern
  struct dihedral {
    struct dihedral *next ;
    char  Selected ;
    char  BadResidue ;
    int   number_of_atoms ; /* Number of atoms in this Residue */
    int   orden ;           /* The order in sequential list: 1 .. N of Res */
    int   resnumber;        /* The Residue Number in PDB file */
    int   resnumber_orig;   /* The Residue Number in PDB file */
    char  resname[5];       /* The Name of the Residue in PDB file */
    char  thrletter[5];     /* three letter code */
    char  oneletter[2];     /* one letter code */
    char  fullname[25];     /* Full name of residue */
    char  chain_id[2] ;
    float omega;
    float phi;
    float psi;
    float chi1;
    float planar_o;
    float planar_h;
    float chiral;
    float chiral_sc;
  } DIHEDRALS ;

extern struct dihedral *DIH ;

/*
 * To manage multiple models it is not implemented yet
 * (look at Heather's stuff).
 * On July 1993 this structure has been implemented
 */
extern
 struct models {
   struct models *next ;
   char     Selected ;
   char     ResiduesSelected ;
   char     AtomsSelected ;
   char     family[30];
   float    energy;
   float    rmsd ;
   float    RadiusOfGyration ;
   char     written ;
   char     determ;     /* DET(U) in superpositions */
   char     BadOmega ;
   char     BadChiral ;
   char     BadSideChain ;
   char     BadRama ;
   char     BadModel ;
   char     Cis ;
   int      modelno;
   int      SerialNumber;
   int      number_of_residues;
   int      starting_res;
   int      number_of_atoms;
   float    x_centermass ;
   float    y_centermass ;
   float    z_centermass ;
   double   Ixx, Iyy, Izz, Ixy, Ixz, Iyz ;
   char     angulos[MAXRESIDUES+1][80];
   struct   dihedral DIH[MAXRESIDUES+1];
   struct   atom_pdb PDB[MAXATOMS+1];
   struct   range ranges;
   float    PrincipalAxisFrame[3];
   char     HeaderPdbModel[MAXREMARKMODRECS][MAXLINE];
 } MODELOS ;

extern  struct models *MODELS;

extern  char        HeaderPdb[MAXREMARKRECS][MAXLINE] ;
extern  float       allowed_d;
extern  float       allowed_j;
extern  char        history[1000][MAXWORD];
extern  int         count_history;
extern  int         DEBUG ;
extern  int         MAX;                       /* max atoms in pdb file */
extern  FILE        *fpi;
extern  FILE        *fpo;
extern  char        seqres[MAXRESIDUES+50][5];
extern  int         modelo;
extern  int         key_model ;
extern  int         constraintcount ;
extern  int         acocount ;
extern  int         sanicount ;    /* counted but not worked out */
extern  char        palabra[MAXWORD];
extern  char        name_of_the_file[MAXWORD]; /* generic for security */
extern  char        name_of_the_file_coords[MAXWORD];
extern  char        name_of_the_file_constr[MAXWORD];
extern  char        name_of_the_file_aco[MAXWORD];
extern  char        name_of_the_file_sequence[MAXWORD];
extern  char        entrada[MAXWORD];
extern  char        output_name[MAXWORD];
extern  char        line[MAXLINE];
extern  char        cadena[10];
extern  int         option, disulfide_bonds, infodisu[21][2];
extern  int         lee,escribe,kount_dist,kount_jcoup;
extern  int         numero_de_residuos;
extern  int         models, q_atoms, lone_pairs, normal_atoms, 
                    cristal, total_models;
extern  char        sequence_true;
 
extern  float dihang(int i, int j, int k, int l);
extern  float angulo(int i, int j, int k);
extern  float distance(int i, int j);

extern  void GetFromKeyBoard(char *s, int size, FILE *stream);
extern  void MolWeight(int choice);
extern  void CenterMass(int choice);
extern  int info_res(int moledo);
extern  void phi_psi_chi(FILE *rtti, int choice);
extern  void haz_ceros(int modelok);
extern  void hydrogen_bonds(float distance_H_any,float angle_any,char Who[10]);
extern  void kabsch_sander(char WhichModel[MAXWORD]);
extern  void close_cntcts(float threshold_close, char atom_type[5], FILE *closecontact);
extern  void menu(void);
extern  void reading(char what[20], char tipo[20], char nombre[MAXWORD]);
extern  void writing(char argumento[20]);
extern  void print_version(void);
extern  void check_label_pdb(int koption);
extern  int locate_file(char nombre[MAXWORD]);
extern  void write_analysis(void);
extern  void write_coords_pdb(FILE *rtto, int loption);
extern  void write_coords_pdb_xplor(FILE *rtto);
extern  void write_coords_disman(FILE *rtto);
extern  void write_sequence(FILE *rtto, char formato[10], char type[10]);
extern  void write_constraints_discover(FILE *rtto);  
extern  void write_constraints_disman(FILE *rtto);
extern  void write_constraints_impact(FILE *rtto);
extern  void write_constraints_congen(FILE *rtto);
extern  void write_constraints_xplor(FILE *rtto);
extern  void write_aco_dyana(FILE *rtto);
extern  void readwrite(FILE *entrasale, char WriteUPL, char WriteLOL);
extern  void copy(char to[],char from[],int inicio, int final);
extern  void equal_residue(int x,int y,int z);
extern  void Draw_maps(char atom_file[MAXWORD],int residues);
extern  int  GetLine(char s[],int limite,FILE *rtti);
extern  void read_coords_pdb(FILE *rtti);
extern  void read_coords_disman(FILE *rtti);
extern  void read_coords_congen(FILE *rtti);
extern  void read_coords_insight(FILE *rtti);
extern  int  read_constraints_congen(FILE *rtti);
extern  int  read_constraints_xplor(FILE *rtti);
extern  int  read_constraints_ecepp(FILE *rtti);
extern  int  read_upl_disman(FILE *rtti, char disdia[10], char LowUpp[10]);
extern  int  read_lol_disman(FILE *rtti, char disdia[10], char LowUpp[10]);
extern  int  read_constraints_impact(FILE *rtti);
extern  int  read_constraints_discover(FILE *rtti);
extern  int  read_constraints_toronto(FILE *rtti);
extern  int  read_aco_dyana(FILE *rtti);
extern  int  read_aco_congen(FILE *rtti);
extern  int  read_aco_xplor(FILE *rtti);
extern  int  reading_sequence(FILE *rtti);
extern  int  longitud(char s[]);

extern  void do_lower_case(char target[MAXWORD]);
extern  void prepare_congen(char filenam[MAXWORD],char catena[]);

extern  int temporalcount ;
extern  int  AverageModel ;
extern  int  ChainsInModel ;
extern  int  FirstResidue, LastResidue ;
extern  char HEAVY ;
extern  char BACKBONE ;
extern  char SELECTED ;
extern  char AcoTrue ;
extern  char ConsTrue ;
extern  char SaniTrue ;
extern  char AlreadyAssignedNoes ;
extern  char ErrorsInNoes ;
extern  char HomolConstraints ;
extern  char Xmer ;
extern  char XplorFmt ;
extern  char WriteHeader ;
extern  char WriteModelHeader ;
extern  int  MaxDistKnown;     /* Total number of known distances in cleaning */

/*
 * General counters
 */
extern float RMSD_MODEL_CUTOFF, ENERGY_CUTOFF;
extern int  TOPMODELS ;   /* Maximum number of models */
