/*$Id: pdbstat_main.h,v 3.32 2003/10/06 12:13:32 roberto Exp roberto $*/

/*
  ---
*/  

#include "general.h"

int   TOPMODELS = MAXMODELS-1 ;   /* Maximum number of models */

/*
 * Structure where we are going to store information about know distances
 * and fixed distances. This is used in cleaning the output as  a lot of
 * constraints can be meaningles because they are distances fixed by the
 * covalent geometry or they constrain nothing.
 * The data is read from a file Distances.Lib at run time, that can be changed 
 * without recompiling the program
 */
struct KnownDist {
  struct KnownDist *next ;
  int   Record;
  char  Atom1Name[5];
  char  Atom2Name[5];
  char  ResidueName[4];
  char  Intra ;
  char  Sequential ;
  double Dmin;
  double Dmax;
  char  Comment1[20];
  char  Comment2[20];
} DescriptionKnownDistances ;

struct KnownDist     *DistKnown ;

/*
 * Hydrogen Bonds
 */
struct HydrogenBond {
	struct HydrogenBond *next ;
	int  rec;
	int  modeln;
	int  ResN1 ;
	int  ResN2 ;
	int  nStructures ;
        char AtomDesc1[20];	
	char AtomDesc2[20];
	char ToPrint ; 
	char Where[100];
} PuentesDeHidrogeno ;

struct HydrogenBond *HBONDS = NULL ;

struct ResAtom {
     char Label[6];
     int  Number ;
     int  NumberOne ;
     int  Len;
} AtomToLook ;

struct ResAtom *ResAt ;

struct AtMissing {
     struct AtMissing *next ;
     int  Atom ;
     int  Model ;
     char Chain[2];
     char Residue[5];
     int  ResNum ;
     char Name[6];
     char Found ;
} AtomsStored;

/*
 * Residual ACO Violation structure
 */
struct ResidualACOViol {
     struct ResidualACOViol *next ;
     int Model, Total ;
     int NumLt5, NumLt10, NumGt10, GreaterThanThreshold ;
     float MinViol, MaxViol, Rmsd, Average ;
} RESACOVIOL;
struct ResidualACOViol *ResAcoViol = NULL ;

/*
 * Residual distance Violation structure
 */
struct ResidualViol {
     struct ResidualViol *next ;
     int Model, Total ;
     int NumLt01, NumLt02, NumLt05, NumLt1, GreaterThanThreshold ;
     float MinViol, MaxViol, Rmsd, Average ;
} RESVIOL;
struct ResidualViol *ResViol = NULL ;

/*
 * Number of model ranges for superpositions.
 */
struct modelrange {
     int   number;
     char  chain_id[MAXMODELS][2];
     int   from[MAXMODELS];
     int   to[MAXMODELS];
} MODELRANGES ;
struct modelrange *modelranges = NULL ;

/*
 * Number of residue ranges for superpositions.
 */
struct range {
     int   number;
     char  chain_id[MAXRANGES][2];
     int   from[MAXRANGES];
     int   to[MAXRANGES];
     char  quality[MAXRANGES][5];
} RANGES ;
struct range *ranges = NULL;


/*******************************************************************
 * NOTE: All this structures to be dynamically allocated and stored.
 * Right now the management of memory is worse than bad (je, je). It
 * MUST be corrected. It takes huge amounts of memory.
 ********************************************************************/

/*
 * Structure JCOUP for J couplings in the models
 */
struct jcoup {
     char  chain_id[2];
     char  name[8], nickname[8];
     int   residue_number1;
     int   residue_number1_orig;
     int   residue_number2;
     int   residue_number2_orig;
     char  residue_type1[5];
     char  residue_type2[5];
     int   atom_number1;
     int   atom_number2;
     char  atom_type1[6];
     char  atom_type2[6];
     float jc, error;
     float angle1;
     float angle2;
     char  flag[2];
} jcoup_defs ;
/*  } JCOUP[MAXJCOUP+1]; */

struct jcoup *JCOUP = NULL ;

/*
 * Structure NOE for defining NOES in the models
 */
struct noe {
     struct noe *next ;
     int   Number ;
     char  HyperName[8];
     char  LastRes1, LastRes2, FirstRes1, FirstRes2 ;
     char  segid_1[5], segid_2[5] ;
     char  chain_id_1[2],  chain_id_2[2];
     int   residue_number1;
     char  residue_type1[5];
     int   atom_number1;
     char  atom_type1[6];
     char  Atom1Ecepp[6] ;
     char  Atom1Dyana[6] ;
     char  Atom1Cyana[6] ;
     char  Atom1Diana[6] ;
     char  Atom1Disman[6] ;
     char  Atom1Xplor[6] ;
     char  Atom1Congen[6] ;
     char  Atom1Pdb[6] ;
     int   residue_number2;
     char  residue_type2[5];
     int   atom_number2;
     char  atom_type2[6];
     char  Atom2Ecepp[6] ;
     char  Atom2Dyana[6] ;
     char  Atom2Cyana[6] ;
     char  Atom2Diana[6] ;
     char  Atom2Disman[6] ;
     char  Atom2Xplor[6] ;
     char  Atom2Congen[6] ;
     char  Atom2Pdb[6] ;
     float equil, rgupp, rglow ;
     float lower, upper;
     float Dist[MAXMODELS];
     char  comment[MAXWORD];
     float volume;
     char  code[2];
     char ToPrint;
     int  iat1[20], iat2[20] ;
} noedefs ;

struct noe *NOE = NULL ;

/*
 * Structure for angle constraints
 */
struct aco {
     struct aco *next ;
     int   Number ;
     char  chain_id[2];
     int   residue_number ;
     char  residue_type[5];
     char  AngleName[6];
     int   Res1, Res2, Res3, Res4;
     char  Atom1[6], Atom2[6], Atom3[6], Atom4[6];
     float Weight, AngleMin, AngleMax, AngleEq, AngleRg ;
     float AngleModel[MAXMODELS];
} acodefs ;

struct aco *ACO = NULL ;

/* Library of names for NOEs in different programs */
struct Library {         /* Library of names for atoms in different prog */
     struct Library *next;
     char Descriptor[5];    /* Dscriptor of record, either ATOM, NMR, DIHE */
     char ResidueName[5];   /* Name of the residue one or three leter */
     char EceppName[6];     /* Name of atom in ECEPP convention */
     char DianaName[6];     /* Name of atom in Diana convention */
     char DyanaName[6];     /* Name of atom in DYANA convention */
     char CyanaName[6];     /* Name of atom in DYANA convention */
     char CongenName[6];    /* Name of the atom in CONGEN convention */
     char XPlorName[6];     /* Name of the atom in X-PLOR convention */
     char DismanName[6];    /* Name of atom in Disman convention */
     char PdbName[6];       /* Name of atom in PDB convention */
     int  Number;           /* Number of the record. It is assigned in reading */
     char Done;             
} DescriptionLibrary ;

/* Library of dihedral names and specifications */
struct DiheLib { 
     struct DiheLib *next ;
     char ResidueName[5];   /* Name of the residue one or three leter */
     char Atom1[6], Atom2[6], Atom3[6], Atom4[6];
     char DihName[6];
     int Number;            /* Number of the record. It is assigned in reading */
} DihedralLibrary ;

/*
 * Structure PDB for defining atom names, coordinates, etc.
 */
struct atom_pdb {
     struct atom_pdb *next ;
     char  Changed ;
     char  TruePdb ;
     char  Selected ;
     char  pdb_key[8];
     char  written ;
     int   pdb_num;
     char  pdb_iupac[6];
     char  pdb_res[5];
     char  pdb_chain_id[2];
     char  segid[5];
     char  chain[10] ;
     char  type[10] ;
     char  AtomType[2];
     int   pdb_num_res;
     int   pdb_num_res_orig;
     int   orden ;
     float pdb_xcrd;
     float pdb_ycrd;
     float pdb_zcrd;
     float pdb_oldx;
     float pdb_oldy;
     float pdb_oldz;
     float coord[3];
     float pdb_data1;
     float pdb_data2;
     float pdb_masa;
} PDBatoms ;

struct atom_pdb *PDB = NULL ;

/*
 * Structure DIHEDRAL for holding dihedral information of models.
 */
struct dihedral {
     struct dihedral *next ;
     char  Selected ;
     char  BadResidue ;
     int   number_of_atoms ; /* Number of atoms in this Residue */
     int   orden ;           /* The order in sequential list: 1 .. N of Res */
     int   resnumber;        /* The *CURRENT* Residue Number in PDB file */
     int   resnumber_orig;   /* The *ORIGINAL* Residue Number in PDB file */
     char  resname[5];       /* The Name of the Residue in PDB file */
     char  thrletter[5];     /* three letter code */
     char  oneletter[2];     /* one letter code */
     char  fullname[25];     /* Full name of residue */
     char  chain_id[2];
     float omega;
     float phi;
     float psi;
     float chi1;
     float planar_o;
     float planar_h;
     float chiral;
     float chiral_sc;
} DIHEDRALS ;

struct dihedral *DIH = NULL ;

/*
  ---
  --- To manage multiple models.
  --- (look at Heather's stuff).
  --- On July 1993 this structure has been implemented
  ---
*/
struct models {
     struct   models *next ;
     char     Selected ;
     char     ResiduesSelected ;
     char     AtomsSelected ;
     char     family[30];
     float    energy;
     float    rmsd ;
     float    RadiusOfGyration ;
     char     written ;
     char     BadOmega ;
     char     BadChiral ;
     char     BadSideChain ;
     char     BadRama ;
     char     BadModel ;
     char     Cis ;
     char     determ ;  /* to store BAD DET(U) in superpositions */
     int      modelno;
     int      SerialNumber ;
     int      number_of_residues;
     int      starting_res ;
     int      number_of_atoms;
     float    x_centermass ;
     float    y_centermass ;
     float    z_centermass ;
     double   Ixx, Iyy, Izz, Ixy, Ixz, Iyz ;
     char     angulos[MAXRESIDUES+1][80];
     struct   dihedral DIH[MAXRESIDUES+1];
     struct   atom_pdb PDB[MAXATOMS+1];
     struct   range ranges;
     float    PrincipalAxisFrame[3];
     char     HeaderPdbModel[MAXREMARKMODRECS][MAXLINE];
} MODELOS ;                 /* We dimension models+2 to store average
			       also in the last position of array */

  struct models *MODELS = NULL ;

  char        HeaderPdb[MAXREMARKRECS][MAXLINE] ;
  float       allowed_d;
  float       allowed_j;
  char        history[1000][MAXWORD];
  int         count_history;
  int         DEBUG ;
  int         MAX;                             /* max atoms in pdb file */
  FILE        *fpi;
  FILE        *fpo;
  int         modelo;
  int         key_model;
  int         constraintcount = 0;
  int         acocount = 0;
  int         sanicount = 0;   /* sani are counted but not worked out */
  char        palabra[MAXWORD];
  char        name_of_the_file[MAXWORD]; /* generic for security */
  char        name_of_the_file_coords[MAXWORD];
  char        name_of_the_file_constr[MAXWORD];
  char        name_of_the_file_aco[MAXWORD];
  char        name_of_the_file_sequence[MAXWORD];
  char        entrada[MAXWORD];
  char        output_name[MAXWORD];
  char        line[MAXLINE];
  char        cadena[10];
  char        seqres[MAXRESIDUES+50][5];
  int         option, disulfide_bonds, infodisu[21][2];
  int         lee,escribe,kount_dist,kount_jcoup;
  int         numero_de_residuos;
  int         models, q_atoms, lone_pairs, normal_atoms, 
              cristal, total_models;
  char        sequence_true;

  float dihang(int i, int j, int k, int l);
  float angulo(int i, int j, int k);
  float distance(int i, int j);

  void GetFromKeyBoard(char *s, int size, FILE *stream);
  void MolWeight(int choice);
  void CenterMass(int choice);
  int info_res(int moledo);
  void phi_psi_chi(FILE *rtti, int choice);
  void haz_ceros(int modelok);
  void evaluate_j(int numat,int numero_de_js,
		  char lettra_j[MAXJCOUP+1][5]);
  void hydrogen_bonds(float distance_H_any,float angle_any,char Who[10]);
  void kabsch_sander(char WhichModel[MAXWORD]);
  void close_cntcts(float threshold_close, char atom_type[5], FILE *closecontact);
  void menu(void);
  void reading(char what[20], char tipo[20], char nombre[MAXWORD]);
  void writing(char argumento[20]);
  void print_version(void);
  void check_label_pdb(int koption);
  int locate_file(char nombre[MAXWORD]);
  void write_analysis(void);
  void write_coords_pdb(FILE *rtto, int loption);
  void write_coords_pdb_xplor(FILE *rtto);
  void write_coords_disman(FILE *rtto);
  void write_sequence(FILE *rtto, char formato[10], char type[10]);
  void write_constraints_discover(FILE *rtto);  
  void write_constraints_disman(FILE *rtto);
  void write_constraints_impact(FILE *rtto);
  void write_constraints_congen(FILE *rtto);
  void write_constraints_xplor(FILE *rtto);
  void write_aco_dyana(FILE *rtto);
  void readwrite(FILE *entrasale, char WriteUPL, char WriteLOL);
  void copy(char to[],char from[],int inicio, int final);
  void equal_residue(int x,int y,int z);
  void Draw_maps(char atom_file[MAXWORD],int residues);
  int  GetLine(char s[],int limite,FILE *rtti);
  void read_coords_pdb(FILE *rtti);
  void read_coords_disman(FILE *rtti);
  void read_coords_congen(FILE *rtti);
  void read_coords_insight(FILE *rtti);
  int  read_constraints_congen(FILE *rtti);
  int  read_constraints_xplor(FILE *rtti);
  int  read_constraints_ecepp(FILE *rtti);
  int  read_upl_disman(FILE *rtti, char disdia[10], char LowUpp[10]);
  int  read_lol_disman(FILE *rtti, char disdia[10], char LowUpp[10]);
  int  read_constraints_impact(FILE *rtti);
  int  read_constraints_discover(FILE *rtti);
  int  read_constraints_toronto(FILE *rtti);
  int  reading_sequence(FILE *rtti);
  int  read_aco_dyana(FILE *rtti);
  int  read_aco_congen(FILE *rtti);
  int  read_aco_xplor(FILE *rtti);
  int  longitud(char s[]);
  void do_codes_for_constraints(void);
  void do_lower_case(char target[MAXWORD]);
  float rmsd_dihedrals(char clave[MAXWORD]);
  void prepare_congen(char filenam[MAXWORD],char catena[]);
  int temporalcount ;
  int  AverageModel ;
  int  ChainsInModel=1 ;  
  int  FirstResidue, LastResidue ;
  char HEAVY = FALSE ;
  char BACKBONE = FALSE ;
  char SELECTED = FALSE ;
  char AcoTrue = FALSE ;
  char ConsTrue = FALSE ;
  char SaniTrue = FALSE ;
  char ErrorsInNoes = FALSE;
  char AlreadyAssignedNoes = FALSE ;
  char HomolConstraints = FALSE ;
  char Xmer = FALSE ;
  char XplorFmt = FALSE ;
  char WriteHeader = FALSE ;
  char WriteModelHeader = FALSE ;
  int  MaxDistKnown;     /* Total number of known distances in cleaning */

/*
 * General counters
 */
 float RMSD_MODEL_CUTOFF = 9999.0, ENERGY_CUTOFF = 9999.0 ;
 int   DihLibRecords = 0;
