/*$Id: pdbstat_misc.c,v 3.32 2003/10/06 12:13:32 roberto Exp roberto $*/
#include "pdbstat.h"
void ShowModel(char options[MAXWORD]) ;
char *check_planar_sc(char buenos[10]);

/* ========================================================================= */
void print_version(void)
/*
  ---
  --- (Roberto Tejero, RTT, May 1993, CABM-Rutgers University).
  ---
*/
{
     printf( "\n       ** VERSION: %s ** \
           \n       ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ \
\n       ===============>  W E L C O M E  to PDBStat <================   \
\n PDBStat has become a tool able (between other things) of:\
\n    a) read and interconvert between DISMAN, DYANA, PDB, DISCOVER, IMPACT \
\n    b) elaborate some info about the molecule as MW, Radius of gyration, .. \
\n    c) investigate the setereochemical quality of a protein by evaluating \
\n       phi, psi, chi1, impropers and chirality", VERSION);
     printf(" \n The sequence of acts should be: \
\n       1.- read a file (coords, constr) \
\n       2.- read a sequence file (can be free format) \
\n       3.- Follow menu (calculating, writing ...) \
\n  \
\n   NOTE: Be aware that if you read in a constraint file you \
\n         *ALWAYS* need to read a sequence file also\n");
     printf("\n HAVE FUN AND ENJOY !! \
	 \n **** For more info type `help' or `menu' **** \n");
}


/* ========================================================================= */
void equal_residue(int i,int k,int l)
/*
  --- l equals zero when we are making equal after reading coordinates file
  --- l is different from zero when making equal after reading from constr
  --- file
  --- On June, 3 this routine was isolated and from that date is not used
  --- anymore (in the bulk program) !!.
  ---
  --- (Roberto Tejero, RTT, May 1993, CABM-Rutgers University).
  ---
*/

{
     static struct models *model;
     model = MODELS+key_model ;
     if (l == 0 ) {
	  copy(NOE[i].residue_type1,model->PDB[k].pdb_res,0,2);
	  copy(NOE[i].residue_type1,model->PDB[k].pdb_res,0,2);
	  NOE[i].residue_number1=model->PDB[k].pdb_num_res;
	  NOE[i].residue_number2=model->PDB[k].pdb_num_res;
	  return;
     }  else {
	  copy(model->PDB[k].pdb_res,NOE[i].residue_type1,0,2);
	  copy(model->PDB[k].pdb_res,NOE[i].residue_type1,0,2);
	  model->PDB[k].pdb_num_res=NOE[i].residue_number1;
	  model->PDB[k].pdb_num_res=NOE[i].residue_number2;
	  return;
     }
}


/* ========================================================================= */
void translate(int mode)
/*
 * mode equals 1 for moving to center of mass and -1 for going back
 *
 * (Roberto Tejero, RTT, May 1994, Universidad de Valencia).
 *
 */

{
     static short i, j;
     static struct models *model ;
     
     if ( mode == +1 ) 
	  for (i = 1 ; i <= total_models ; i++) {
	       model = MODELS+i;
	       for (j = 1 ; j <= model->number_of_atoms ; j++) {
		    model->PDB[j].pdb_xcrd = model->PDB[j].pdb_oldx ;
		    model->PDB[j].pdb_ycrd = model->PDB[j].pdb_oldy ;
		    model->PDB[j].pdb_zcrd = model->PDB[j].pdb_oldz ;
		    model->PDB[j].coord[0] = model->PDB[j].pdb_oldx ;
		    model->PDB[j].coord[1] = model->PDB[j].pdb_oldy ;
		    model->PDB[j].coord[2] = model->PDB[j].pdb_oldz ;
	       }
	  }
     
     if ( mode == -1 )
	  for (i = 1 ; i <= total_models ; i++) {
	       model = MODELS+i ;
	       for (j = 1 ; j <= model->number_of_atoms ; j++) {
		    model->PDB[j].pdb_xcrd = model->PDB[j].pdb_xcrd + mode * model->x_centermass ;
		    model->PDB[j].pdb_ycrd = model->PDB[j].pdb_ycrd + mode * model->y_centermass ;
		    model->PDB[j].pdb_zcrd = model->PDB[j].pdb_zcrd + mode * model->z_centermass ;
		    
		    model->PDB[j].coord[0] = model->PDB[j].pdb_xcrd ;
		    model->PDB[j].coord[1] = model->PDB[j].pdb_ycrd ;
		    model->PDB[j].coord[2] = model->PDB[j].pdb_zcrd ;
	       }
	  }
     return ;
}


/* ========================================================================= */
void XplorToIUPAC(void) {
     /*
      * Modify X-PLOR hydrogen atom names to fullfill IUPAC conventions. 
      *
      * (Roberto Tejero, RTT, Feb 2003, CABM Rutgers Univ.)
      */
     static short i, j;
     static struct models *model ;
     static struct atom_pdb *At ;

     /* Initialize all to be changed */
     for (i = 1 ; i <= total_models ; i++) {
	  model = MODELS+i;
	  for (j = 1 ; j <= model->number_of_atoms ; j++) {
	       At = (model->PDB) + j ;
	       At->Changed = FALSE ;
	  }
     }
 
     for (i = 1 ; i <= total_models ; i++) {
	  model = MODELS+i;
	  for (j = 1 ; j <= model->number_of_atoms ; j++) {
	       At = (model->PDB) + j ;
	       /* alpha methylenes */
	       if (! At->Changed && strcasecmp(At->pdb_iupac,"1HA  ")==0) {
		    strcpy(At->pdb_iupac,"2HA  ");
		    At->Changed = TRUE ;
	       }
	       if (! At->Changed && strcasecmp(At->pdb_iupac,"2HA  ")==0) {
		    strcpy(At->pdb_iupac,"1HA  ");
		    At->Changed = TRUE ;
	       }
	       
	       /* beta methylenes */
	       if  (strncasecmp(At->pdb_res,"ALA",3)!=0) { 
		 if ( !At->Changed && strcasecmp(At->pdb_iupac,"1HB  ")==0) {
		   strcpy(At->pdb_iupac,"3HB  ");
		   At->Changed = TRUE ;
		 }
	       }

	       if  (strncasecmp(At->pdb_res,"ALA",3)==0) {  /* ALA methyl */
		    if (!At->Changed && strcasecmp(At->pdb_iupac,"2HB  ")==0) {
			 strcpy(At->pdb_iupac,"1HB  ");
			 At->Changed = TRUE ;
		    }
		    if (!At->Changed && strcasecmp(At->pdb_iupac,"1HB  ")==0) {
			 strcpy(At->pdb_iupac,"2HB  ");
			 At->Changed = TRUE ;
		    }
	       }
	       
	       /* gamma methylenes */
	       if (! At->Changed && strcasecmp(At->pdb_iupac,"1HG  ")==0) {
		    strcpy(At->pdb_iupac,"3HG  ");
		    At->Changed = TRUE ;
	       }

	       if  (strncasecmp(At->pdb_res,"ILE",3)==0) {  /* ILE methylene */
		    if (! At->Changed && strcasecmp(At->pdb_iupac,"1HG1 ")==0) {
			 strcpy(At->pdb_iupac,"3HG1 ");
			 At->Changed = TRUE ;
		    }
	       }

	       /* gamma Methyls */
	       if  (strncasecmp(At->pdb_res,"VAL",3)==0) {
		    if (! At->Changed && strcasecmp(At->pdb_iupac,"1HG1 ")==0) {
			 strcpy(At->pdb_iupac,"2HG1 ");
			 At->Changed = TRUE ;
		    }
		    if (! At->Changed && strcasecmp(At->pdb_iupac,"2HG1 ")==0) {
			 strcpy(At->pdb_iupac,"1HG1 ");
			 At->Changed = TRUE ;
		    }
	       }

	       if (! At->Changed && strcasecmp(At->pdb_iupac,"1HG2 ")==0) {
		    strcpy(At->pdb_iupac,"2HG2 ");
		    At->Changed = TRUE ;
	       }
	       if (! At->Changed && strcasecmp(At->pdb_iupac,"2HG2 ")==0) {
		    strcpy(At->pdb_iupac,"1HG2 ");
		    At->Changed = TRUE ;
	       }
	       
	       /* delta methylenes */
	       if  (strncasecmp(At->pdb_res,"PHE",3)!=0 &&
		    strncasecmp(At->pdb_res,"TYR",3)!=0 &&
		    strncasecmp(At->pdb_res,"TRP",3)!=0 &&
		    strncasecmp(At->pdb_res,"HIS",3)!=0 ) {
		    if (!At->Changed && strcasecmp(At->pdb_iupac,"1HD  ")==0) {
			 strcpy(At->pdb_iupac,"3HD  ");
			 At->Changed = TRUE ;
		    }
	       }
	       if  (strncasecmp(At->pdb_res,"ASN",3)==0 ) {
		    if (! At->Changed && strcasecmp(At->pdb_iupac,"2HD2 ")==0) {
			 strcpy(At->pdb_iupac,"1HD2 ");
			 At->Changed = TRUE ;
		    }
		    if (! At->Changed && strcasecmp(At->pdb_iupac,"1HD2 ")==0) {
			 strcpy(At->pdb_iupac,"2HD2 ");
			 At->Changed = TRUE ;
		    }
	       }

	       /* Methyls */
	       if (! At->Changed && strcasecmp(At->pdb_iupac,"1HD1 ")==0) {
		    strcpy(At->pdb_iupac,"2HD1 ");
		    At->Changed = TRUE ;
	       }
	       if (! At->Changed && strcasecmp(At->pdb_iupac,"2HD1 ")==0) {
		    strcpy(At->pdb_iupac,"1HD1 ");
		    At->Changed = TRUE ;
	       }
	       if (! At->Changed && strcasecmp(At->pdb_iupac,"1HD2 ")==0) {
		    strcpy(At->pdb_iupac,"2HD2 ");
		    At->Changed = TRUE ;
	       }
	       if (! At->Changed && strcasecmp(At->pdb_iupac,"2HD2 ")==0) {
		    strcpy(At->pdb_iupac,"1HD2 ");
		    At->Changed = TRUE ;
	       }
	       
	       /* Epsilons */
	       if  (strncasecmp(At->pdb_res,"PHE",3)!=0 &&
		    strncasecmp(At->pdb_res,"TYR",3)!=0 &&
		    strncasecmp(At->pdb_res,"TRP",3)!=0 &&
		    strncasecmp(At->pdb_res,"HIS",3)!=0 && 
		    strncasecmp(At->pdb_res,"MET",3)!=0 ) {
		    if (! At->Changed && strcasecmp(At->pdb_iupac,"1HE  ")==0) {
			 strcpy(At->pdb_iupac,"3HE  ");
			 At->Changed = TRUE ;
		    }
	       }
	       /*
	       if ( strncasecmp(At->pdb_res,"LYS",3)!=0 ) {
		    if (! At->Changed && strcasecmp(At->pdb_iupac,"2HE  ")==0) {
			 strcpy(At->pdb_iupac,"1HE  ");
			 At->Changed = TRUE ;
		    }
	       }
	       */
	       if  (strncasecmp(At->pdb_res,"GLN",3)==0 ) {
		    if (!At->Changed && strcasecmp(At->pdb_iupac,"2HE2 ")==0) {
			 strcpy(At->pdb_iupac,"1HE2 ");
			 At->Changed = TRUE ;
		    }
		    if (!At->Changed && strcasecmp(At->pdb_iupac,"1HE2 ")==0) {
			 strcpy(At->pdb_iupac,"2HE2 ");
			 At->Changed = TRUE ;
		    }
	       }
	       
	       /* Zeta */
	       if  (strncasecmp(At->pdb_res,"LYS",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"1HZ  ")==0) {
			 strcpy(At->pdb_iupac,"2HZ  ");
			 At->Changed = TRUE ;
		    }
		    if (strcasecmp(At->pdb_iupac,"2HZ  ")==0) {
			 strcpy(At->pdb_iupac,"1HZ  ");
			 At->Changed = TRUE ;
		    }
	       }
	  }
     }
     return ;
}

/* ========================================================================= */
void ToIUPAC(void) {
     /*
      * Modify atom names to fullfill IUPAC conventions. It is also useful
      * for programs like DYANA, DIANA, etc. where methylenes are 2 and 3
      * in lieu of 1 and 2.
      *
      * (Roberto Tejero, RTT, Jan 2001, Universidad de Valencia).
      */
     static short i, j;
     static struct models *model ;
     static struct atom_pdb *At ;
     
     for (i = 1 ; i <= total_models ; i++) {
	  model = MODELS+i;
	  for (j = 1 ; j <= model->number_of_atoms ; j++) {
	       At = (model->PDB) + j ;
	       if  (strncasecmp(At->pdb_res,"ARG",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"1HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
		    else if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"3HB  ");
		    else if (strcasecmp(At->pdb_iupac,"1HG  ")==0)
			 strcpy(At->pdb_iupac,"2HG  ");
		    else if (strcasecmp(At->pdb_iupac,"2HG  ")==0)
			 strcpy(At->pdb_iupac,"3HG  ");
		    else if (strcasecmp(At->pdb_iupac,"1HD  ")==0)
			 strcpy(At->pdb_iupac,"2HD  ");
		    else if (strcasecmp(At->pdb_iupac,"2HD  ")==0)
			 strcpy(At->pdb_iupac,"3HD  ");
	       } else if  (strncasecmp(At->pdb_res,"ASN",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"1HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
		    else if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"3HB  ");
	       } else if  (strncasecmp(At->pdb_res,"ASP",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"1HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
		    else if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"3HB  ");
	       } else if  (strncasecmp(At->pdb_res,"CYS",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"1HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
		    else if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"3HB  ");
	       } else if  (strncasecmp(At->pdb_res,"GLN",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"1HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
		    else if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"3HB  ");
		    else if (strcasecmp(At->pdb_iupac,"1HG  ")==0)
			 strcpy(At->pdb_iupac,"2HG  ");
		    else if (strcasecmp(At->pdb_iupac,"2HG  ")==0)
			 strcpy(At->pdb_iupac,"3HG  ");
	       } else if  (strncasecmp(At->pdb_res,"GLU",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"1HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
		    else if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"3HB  ");
		    else if (strcasecmp(At->pdb_iupac,"1HG  ")==0)
			 strcpy(At->pdb_iupac,"2HG  ");
		    else if (strcasecmp(At->pdb_iupac,"2HG  ")==0)
			 strcpy(At->pdb_iupac,"3HG  ");
		    else if (strcasecmp(At->pdb_iupac," HE  ")==0)
			 strcpy(At->pdb_iupac,"2HE  ");
	       } else if  (strncasecmp(At->pdb_res,"HIS",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"1HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
		    else if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"3HB  ");
	       } else if  (strncasecmp(At->pdb_res,"ILE",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"1HG1 ")==0)
			 strcpy(At->pdb_iupac,"2HG1 ");
		    else if (strcasecmp(At->pdb_iupac,"2HG1 ")==0)
			 strcpy(At->pdb_iupac,"3HG1 ");
	       } else if  (strncasecmp(At->pdb_res,"LEU",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"1HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
		    else if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"3HB  ");
	       } else if  (strncasecmp(At->pdb_res,"LYS",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"1HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
		    else if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"3HB  ");
		    else if (strcasecmp(At->pdb_iupac,"1HG  ")==0)
			 strcpy(At->pdb_iupac,"2HG  ");
		    else if (strcasecmp(At->pdb_iupac,"2HG  ")==0)
			 strcpy(At->pdb_iupac,"3HG  ");
		    else if (strcasecmp(At->pdb_iupac,"1HD  ")==0)
			 strcpy(At->pdb_iupac,"2HD  ");
		    else if (strcasecmp(At->pdb_iupac,"2HD  ")==0)
			 strcpy(At->pdb_iupac,"3HD  ");
		    else if (strcasecmp(At->pdb_iupac,"1HE  ")==0)
			 strcpy(At->pdb_iupac,"2HE  ");
		    else if (strcasecmp(At->pdb_iupac,"2HE  ")==0)
			 strcpy(At->pdb_iupac,"3HE  ");
	       } else if  (strncasecmp(At->pdb_res,"MET",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"1HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
		    else if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"3HB  ");
		    else if (strcasecmp(At->pdb_iupac,"1HG  ")==0)
			 strcpy(At->pdb_iupac,"2HG  ");
		    else if (strcasecmp(At->pdb_iupac,"2HG  ")==0)
			 strcpy(At->pdb_iupac,"3HG  ");
	       } else if  (strncasecmp(At->pdb_res,"PHE",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"1HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
		    else if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"3HB  ");
	       } else if  (strncasecmp(At->pdb_res,"PRO",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"1HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
		    else if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"3HB  ");
		    else if (strcasecmp(At->pdb_iupac,"1HG  ")==0)
			 strcpy(At->pdb_iupac,"2HG  ");
		    else if (strcasecmp(At->pdb_iupac,"2HG  ")==0)
			 strcpy(At->pdb_iupac,"3HG  ");
		    else if (strcasecmp(At->pdb_iupac,"1HD  ")==0)
			 strcpy(At->pdb_iupac,"2HD  ");
		    else if (strcasecmp(At->pdb_iupac,"2HD  ")==0)
			 strcpy(At->pdb_iupac,"3HD  ");
	       } else if  (strncasecmp(At->pdb_res,"SER",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"1HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
		    else if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"3HB  ");
	       } else if  (strncasecmp(At->pdb_res,"TRP",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"1HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
		    else if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"3HB  ");
		    else if (strcasecmp(At->pdb_iupac," HD  ")==0)
			 strcpy(At->pdb_iupac," HD1 ");
		    else if (strcasecmp(At->pdb_iupac," HH  ")==0)
			 strcpy(At->pdb_iupac," HH2 ");
	       } else if  (strncasecmp(At->pdb_res,"TYR",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"1HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
		    else if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"3HB  ");
	       }
	  } 
     }
     return ;
}


/* ========================================================================= */
void ToPDB(void) {
     /*
      * Tansform --adapt-- atom names to regular PDB like XPLOR, CONGEN, etc
      * beta methylenes are 1 and 2 in lieu of 2 and 3 
      * (Roberto Tejero, RTT, Jan 2001, Universidad de Valencia).
      *
      */
     static short i, j;
     static struct models *model ;
     static struct atom_pdb *At ;
     
     for (i = 1 ; i <= total_models ; i++) {
	  model = MODELS+i;
	  for (j = 1 ; j <= model->number_of_atoms ; j++) {
	       At = (model->PDB) + j;
	       if  (strncasecmp(At->pdb_res,"ARG",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"1HB  ");
		    else if (strcasecmp(At->pdb_iupac,"3HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
		    else if (strcasecmp(At->pdb_iupac,"2HG  ")==0)
			 strcpy(At->pdb_iupac,"1HG  ");
		    else if (strcasecmp(At->pdb_iupac,"3HG  ")==0)
			 strcpy(At->pdb_iupac,"2HG  ");
		    else if (strcasecmp(At->pdb_iupac,"2HD  ")==0)
			 strcpy(At->pdb_iupac,"1HD  ");
		    else if (strcasecmp(At->pdb_iupac,"3HD  ")==0)
			 strcpy(At->pdb_iupac,"2HD  ");
	       } else if  (strncasecmp(At->pdb_res,"ASN",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"1HB  ");
		    else if (strcasecmp(At->pdb_iupac,"3HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
	       } else if  (strncasecmp(At->pdb_res,"ASP",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"1HB  ");
		    else if (strcasecmp(At->pdb_iupac,"3HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
	       } else if  (strncasecmp(At->pdb_res,"CYS",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"1HB  ");
		    else if (strcasecmp(At->pdb_iupac,"3HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
	       } else if  (strncasecmp(At->pdb_res,"GLN",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"1HB  ");
		    else if (strcasecmp(At->pdb_iupac,"3HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
		    else if (strcasecmp(At->pdb_iupac,"2HG  ")==0)
			 strcpy(At->pdb_iupac,"1HG  ");
		    else if (strcasecmp(At->pdb_iupac,"3HG  ")==0)
			 strcpy(At->pdb_iupac,"2HG  ");
	       } else if  (strncasecmp(At->pdb_res,"GLU",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"1HB  ");
		    else if (strcasecmp(At->pdb_iupac,"3HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
		    else if (strcasecmp(At->pdb_iupac,"2HG  ")==0)
			 strcpy(At->pdb_iupac,"1HG  ");
		    else if (strcasecmp(At->pdb_iupac,"3HG  ")==0)
			 strcpy(At->pdb_iupac,"2HG  ");
		    else if (strcasecmp(At->pdb_iupac,"HE2  ")==0)
			 strcpy(At->pdb_iupac," HE  ");
	       } else if  (strncasecmp(At->pdb_res,"HIS",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"1HB  ");
		    else if (strcasecmp(At->pdb_iupac,"3HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
	       } else if  (strncasecmp(At->pdb_res,"ILE",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"2HG1 ")==0)
			 strcpy(At->pdb_iupac,"1HG1 ");
		    else if (strcasecmp(At->pdb_iupac,"3HG1 ")==0)
			 strcpy(At->pdb_iupac,"2HG1 ");
	       } else if  (strncasecmp(At->pdb_res,"LEU",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"1HB  ");
		    else if (strcasecmp(At->pdb_iupac,"3HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
	       } else if  (strncasecmp(At->pdb_res,"LYS",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"1HB  ");
		    else if (strcasecmp(At->pdb_iupac,"3HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
		    else if (strcasecmp(At->pdb_iupac,"2HG  ")==0)
			 strcpy(At->pdb_iupac,"1HG  ");
		    else if (strcasecmp(At->pdb_iupac,"3HG  ")==0)
			 strcpy(At->pdb_iupac,"2HG  ");
		    else if (strcasecmp(At->pdb_iupac,"2HD  ")==0)
			 strcpy(At->pdb_iupac,"1HD  ");
		    else if (strcasecmp(At->pdb_iupac,"3HD  ")==0)
			 strcpy(At->pdb_iupac,"2HD  ");
		    else if (strcasecmp(At->pdb_iupac,"2HE  ")==0)
			 strcpy(At->pdb_iupac,"1HE  ");
		    else if (strcasecmp(At->pdb_iupac,"3HE  ")==0)
			 strcpy(At->pdb_iupac,"2HE  ");
	       } else if  (strncasecmp(At->pdb_res,"MET",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"1HB  ");
		    else if (strcasecmp(At->pdb_iupac,"3HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
		    else if (strcasecmp(At->pdb_iupac,"2HG  ")==0)
			 strcpy(At->pdb_iupac,"1HG  ");
		    else if (strcasecmp(At->pdb_iupac,"3HG  ")==0)
			 strcpy(At->pdb_iupac,"2HG  ");
	       } else if  (strncasecmp(At->pdb_res,"PHE",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"1HB  ");
		    else if (strcasecmp(At->pdb_iupac,"3HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
	       } else if  (strncasecmp(At->pdb_res,"PRO",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"1HB  ");
		    else if (strcasecmp(At->pdb_iupac,"3HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
		    else if (strcasecmp(At->pdb_iupac,"2HG  ")==0)
			 strcpy(At->pdb_iupac,"1HG  ");
		    else if (strcasecmp(At->pdb_iupac,"3HG  ")==0)
			 strcpy(At->pdb_iupac,"2HG  ");
		    else if (strcasecmp(At->pdb_iupac,"2HD  ")==0)
			 strcpy(At->pdb_iupac,"1HD  ");
		    else if (strcasecmp(At->pdb_iupac,"3HD  ")==0)
			 strcpy(At->pdb_iupac,"2HD  ");
	       } else if  (strncasecmp(At->pdb_res,"SER",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"1HB  ");
		    else if (strcasecmp(At->pdb_iupac,"3HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
	       } else if  (strncasecmp(At->pdb_res,"TRP",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"1HB  ");
		    else if (strcasecmp(At->pdb_iupac,"3HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
		    else if (strcasecmp(At->pdb_iupac,"HD1  ")==0)
			 strcpy(At->pdb_iupac," HD  ");
		    else if (strcasecmp(At->pdb_iupac,"HH2  ")==0)
			 strcpy(At->pdb_iupac," HH  ");
	       } else if  (strncasecmp(At->pdb_res,"TYR",3)==0) {
		    if (strcasecmp(At->pdb_iupac,"2HB  ")==0)
			 strcpy(At->pdb_iupac,"1HB  ");
		    else if (strcasecmp(At->pdb_iupac,"3HB  ")==0)
			 strcpy(At->pdb_iupac,"2HB  ");
	       }
	  }
     }
     return ;
}

/* ========================================================================= */
void ToDYANA(void) {
     /*
      * Transform --or adapt-- atom names to DYANA convention/format
      * Only H -> HN. Other tranformations can be done with ToIUPAC,
      * (Roberto Tejero, RTT, Jan 2001, Universidad de Valencia).
      *
      */
     static short i, j;
     static struct models *model ;
     
     for (i = 1 ; i <= total_models ; i++) {
	  model = MODELS+i;
	  for (j = 1 ; j <= model->number_of_atoms ; j++) {
	       if (strncmp(model->PDB[j].pdb_iupac," H ",3)==0)
		    strcpy(model->PDB[j].pdb_iupac," HN  ");
	       if (strncmp(model->PDB[j].pdb_iupac," OT1",4)==0)
		    strcpy(model->PDB[j].pdb_iupac," O   ");
	  }
     }
     return ;
}

/* ========================================================================= */
void ToXPLOR(void) {
     /*
      * Transform --or adapt-- atom names to X-PLOR convention/format
      * in its internal representation. 
      * (Roberto Tejero, RTT, Jan 2001, Universidad de Valencia).
      *
      */
     static short i, j;
     static struct models *model ;
     static char TmpChar ;
     
     for (i = 1 ; i <= total_models ; i++) {
	  model = MODELS+i;
	  for (j = 1 ; j <= model->number_of_atoms ; j++) {
	       if (model->PDB[j].pdb_num_res == LastResidue && 
		   strncmp(model->PDB[j].pdb_iupac," O ",3)==0) {
		    strcpy(model->PDB[j].pdb_iupac," OT1 ");
	       }
	       if (strncmp(model->PDB[j].pdb_iupac," H ",3)==0)
		    strcpy(model->PDB[j].pdb_iupac," HN  ");
	       if (model->PDB[j].pdb_iupac[0] == '1' || 
		   model->PDB[j].pdb_iupac[0] == '2' || 
		   model->PDB[j].pdb_iupac[0] == '3') {
		    if (model->PDB[j].pdb_iupac[2] == ' ') {
			 model->PDB[j].pdb_iupac[3] = model->PDB[j].pdb_iupac[0] ;
			 model->PDB[j].pdb_iupac[0] = ' ' ;
			 model->PDB[j].pdb_iupac[1] = 'H' ;
			 model->PDB[j].pdb_iupac[2] = 'T' ;
			 model->PDB[j].pdb_iupac[4] = ' ' ;
			 model->PDB[j].pdb_iupac[5] = '\0' ; 	  
		    } else if (model->PDB[j].pdb_iupac[3] == ' ') {
			 model->PDB[j].pdb_iupac[1] = model->PDB[j].pdb_iupac[1] ;
			 model->PDB[j].pdb_iupac[2] = model->PDB[j].pdb_iupac[2] ;
			 model->PDB[j].pdb_iupac[3] = model->PDB[j].pdb_iupac[0] ;
			 model->PDB[j].pdb_iupac[0] = ' ' ;
			 model->PDB[j].pdb_iupac[4] = ' ' ;
			 model->PDB[j].pdb_iupac[5] = '\0' ; 
		    } else if (model->PDB[j].pdb_iupac[4] == ' ') {
			 TmpChar = model->PDB[j].pdb_iupac[0] ;
			 model->PDB[j].pdb_iupac[0] = model->PDB[j].pdb_iupac[1] ;
			 model->PDB[j].pdb_iupac[1] = model->PDB[j].pdb_iupac[2] ;
			 model->PDB[j].pdb_iupac[2] = model->PDB[j].pdb_iupac[3] ;
			 model->PDB[j].pdb_iupac[3] = TmpChar ;
			 model->PDB[j].pdb_iupac[4] = ' ' ; 
			 model->PDB[j].pdb_iupac[5] = '\0' ; 
		    }
	       } 
	  }
     }
     return ;
}


void ShowModel(char options[MAXWORD])
{
     /*
      * Routine to print out several things stored in the structures in memory
      * Format slightly changed on May 3, 2003. RTT - CABM, Rutgers Univ.
      */
     static char Bad=FALSE, Coords=FALSE;
     static char OptionTaken[10];
     static short i,j;
     static int temp, lineas=0 ;
     static struct models *model;
     
     Bad = FALSE ;
     Coords = FALSE ;
     
     sscanf(options,"%*s %s", OptionTaken);
     if (strncasecmp(OptionTaken,"coords",2)==0) Coords = TRUE ;
     else if (strncasecmp(OptionTaken,"bad",2)==0) Bad = TRUE ;
     else Bad = FALSE ;
     
     if ( Coords ) {
	  lineas = 0;
	  strcpy(OptionTaken,"");
	  sscanf(options,"%*s %*s %s",OptionTaken);
	  if (strcmp(OptionTaken,"")==0)
	       temp = 1 ;
	  else if (strncmp(OptionTaken,"*",1)==0)
	       temp = total_models ;
	  else 
	       temp = atoi(OptionTaken) ;
	  if (temp <= 0) temp = 1;
	  if (temp > total_models+1) {
	       printf(" > ShowModel(): Model not in memory, using mod. 1 instead\n");
	       temp = 1;
	  }
	  for (i = 1; i <= total_models; i++) {
	       model = MODELS + i ;
	       for (j = 1; j <= model->number_of_atoms; j++) {
		    if ( model->PDB[j].Selected ) {
			 lineas++ ;
			 if ( !(lineas%30) ) {
			      printf("\n SHOW(): Press `q RET' to quit or `RET' to continue ");
			      GetFromKeyBoard(OptionTaken, MAXWORD, stdin);
			      if (strcmp(OptionTaken,"q")==0) goto getout ;
			 }
			 printf("\n> SHOW(): %6s %4d %4s %3s %1s %3d %8.3f %8.3f %8.3f - %-3d %s %s",
				model->PDB[j].pdb_key,
				model->PDB[j].pdb_num,
				model->PDB[j].pdb_iupac,
				model->PDB[j].pdb_res,
				model->PDB[j].pdb_chain_id,
				model->PDB[j].pdb_num_res,
				model->PDB[j].pdb_xcrd,
				model->PDB[j].pdb_ycrd,
				model->PDB[j].pdb_zcrd,
				model->modelno,
				model->PDB[j].type,
				model->PDB[j].chain);
		    }
	       }
	  }
     getout:
	  return ;
     }
     
     sscanf(options,"%*s %s",OptionTaken);
     if ((strncasecmp(OptionTaken,"bad",2)!=0) &&
	 (strncasecmp(OptionTaken,"good",2)!=0)) {
	  strcpy(OptionTaken,"");
	  sscanf(options,"%*s %*s %s",OptionTaken);
	  if (strcmp(OptionTaken,"")==0)
	       temp = 1 ;
	  else 
	       temp = atoi(OptionTaken) ;
	  if (temp <= 0) temp = 1;
	  if (temp > total_models) temp = 1;
	  
	  sscanf(options,"%*s %s",OptionTaken);
	  if (strncasecmp(OptionTaken,"omega",2) == 0) {
	       strcpy(OptionTaken, check_planar_sc("show"));
	       sscanf(options,"%*s %*s %s",OptionTaken);
	       printf("\n>");
	       for (i=1 ; i <= total_models; i++) {
		    model = MODELS+i ;
		    if (strncmp(OptionTaken,"*",1)==0) temp = i ;
		    if (temp == i)
			 for (j=1; j <= model->number_of_residues; j++)
			      if (model->DIH[j].omega != 11111.0)
				   printf("\n> SHOW(): Model %3d res [%s %s%4d]   OMEGA: %7.2f",
					  i, model->DIH[j].resname, model->DIH[j].chain_id, j, 
					  model->DIH[j].omega);
	       }
	       printf("\n>");
	       return ;
	  }
	  
	  if (strncasecmp(OptionTaken,"phi",2) == 0) {
	       strcpy(OptionTaken, check_planar_sc("show"));
	       sscanf(options,"%*s %*s %s",OptionTaken);
	       printf("\n>");
	       for (i=1 ; i <= total_models+1; i++) {
		    model = MODELS+i ;
		    if (strncmp(OptionTaken,"*",1)==0) temp = i ;
		    if (temp == i)
			 for (j=1; j <= model->number_of_residues; j++)
			      if (model->DIH[j].phi != 11111.0)
				   printf("\n> SHOW(): Model %3d res [%s %s%4d]   PHI: %7.2f",
					  i, model->DIH[j].resname, model->DIH[j].chain_id, j, 
					  model->DIH[j].phi);
	       }
	       printf("\n>");
	       return ;
	  }
	  
	  if (strncasecmp(OptionTaken,"psi",2) == 0) {
	       strcpy(OptionTaken, check_planar_sc("show"));
	       sscanf(options,"%*s %*s %s",OptionTaken);
	       printf("\n>");
	       for (i=1 ; i <= total_models+1; i++) {
		    model = MODELS+i ;
		    if (strncmp(OptionTaken,"*",1)==0) temp = i ;
		    if (temp == i)
			 for (j=1; j <= model->number_of_residues; j++)
			      if (model->DIH[j].psi != 11111.0)
				   printf("\n> SHOW(): Model %3d res [%s %s%4d]   PSI: %7.2f",
					  i, model->DIH[j].resname, model->DIH[j].chain_id, j, 
					  model->DIH[j].psi);
	       }
	       printf("\n>");
	       return ;
	  }
	  
	  if (strncasecmp(OptionTaken,"chi",2) == 0) {
	       strcpy(OptionTaken, check_planar_sc("show"));
	       sscanf(options,"%*s %*s %s",OptionTaken);
	       printf("\n>");
	       for (i=1 ; i <= total_models+1; i++) {
		    model = MODELS+i ;
		    if (strncmp(OptionTaken,"*",1)==0) temp = i ;
		    if (temp == i)
			 for (j=1; j <= model->number_of_residues; j++)
			      if (model->DIH[j].chi1 != 11111.0)
				   printf("\n> SHOW(): Model %3d res [%s %s%4d]   CHI: %7.2f",
					  i, model->DIH[j].resname, model->DIH[j].chain_id, j, 
					  model->DIH[j].chi1);
	       }
	       printf("\n>");
	       return ;
	  }
	  
	  if (strncasecmp(OptionTaken,"dih",2) == 0) {
	       strcpy(OptionTaken, check_planar_sc("show"));
	       sscanf(options,"%*s %*s %s",OptionTaken);
	       printf("\n>");
	       for (i=1 ; i <= total_models+1; i++) {
		    model = MODELS + i;
		    if (strncmp(OptionTaken,"*",1)==0) temp = i ;
		    if (temp == i)
			 for (j=1; j <= model->number_of_residues; j++) {
			      if (model->DIH[j].phi != 11111.0)
				   printf("\n> SHOW(): Mod %3d Res [%s %s%4d]   PHI: %7.2f",
					  i, model->DIH[j].resname, model->DIH[j].chain_id, j, model->DIH[j].phi);
			      else
				   printf("\n> SHOW(): Mod %3d Res [%s %s%4d]   PHI:        ",
					  i, model->DIH[j].resname, model->DIH[j].chain_id, j); 
			      if (model->DIH[j].psi != 11111.0)
				   printf("  PSI: %7.2f",model->DIH[j].psi);
			      else
				   printf("  PSI:        ");
			      if (model->DIH[j].chi1 != 11111.0)
				   printf("  CHI: %7.2f",model->DIH[j].chi1);
			      else
				   printf("  CHI:        ");
			      if (model->DIH[j].omega != 11111.0)
				   printf("  OMEGA: %7.2f",model->DIH[j].omega);
			      else
				   printf("  OMEGA:        ");
			 }
	       }
	       printf("\n>");
	       return ;
	  }
	  
	  if (strncasecmp(OptionTaken,"model",2) == 0) {
	       strcpy(OptionTaken, check_planar_sc("show"));
	       sscanf(options,"%*s %*s %s",OptionTaken);
	       printf("\n>");
	       for (i=1 ; i <= total_models+1; i++) {
		    model = MODELS+i;
		    if (strncmp(OptionTaken,"*",1)==0) temp = i ;
		    if (temp == i) {
			 printf("\n SHOW: Model %3d Number: %8d", i,model->modelno);
			 printf("\n SHOW: Model %3d Family: %s", i,model->family);
			 printf("\n SHOW: Model %3d ENERGY: %8.4f", i,model->energy);
			 printf("\n SHOW: Model %3d   Rmsd: %8.4f", i,model->rmsd);
			 if (model->determ)
			      printf("\n SHOW: Model %3d Has *bad* DET(U), less than zero",i);
			 if (model->BadOmega)
			      printf("\n SHOW: Model %3d Has *bad* omega",i);
			 else
			      printf("\n SHOW: Model %3d Has good omega",i);
			 if (model->BadChiral)
			      printf("\n SHOW: Model %3d Has *bad* chirality",i);
			 else
			      printf("\n SHOW: Model %3d Has good chirality",i);
			 if (model->BadSideChain)
			      printf("\n SHOW: Model %3d Has *bad* side chain chirality",i);
			 else
			      printf("\n SHOW: Model %3d Has good side chain chirality",i);
			 if (model->BadRama)
			      printf("\n SHOW: Model %3d Has *bad* Ramachandran values",i);
			 else
			      printf("\n SHOW: Model %3d Has good Ramachandran values",i);
			 if (model->Cis)
			      printf("\n SHOW: Model %3d Is  cis ",i);
			 else
			      printf("\n SHOW: Model %3d Is  trans ",i);
			 if (model->BadModel)
			      printf("\n SHOW: Model %3d Is  a *bad* model",i);
			 else
			      printf("\n SHOW: Model %3d Is  a good model",i);
		    }
	       }
	       printf("\n>");
	       return ;
	  }
     }
     
     sscanf(options,"%*s %*s %s",OptionTaken);
     if (strncasecmp(OptionTaken,"rama",2) == 0) {
	  strcpy(OptionTaken, check_planar_sc("show"));
	  printf("\n>");
	  for (i=1 ; i <= total_models+1; i++) {
	       model = MODELS+i ;
	       for (j=1; j <= model->number_of_residues; j++)
		    if  (model->DIH[j].phi > 0.0 && 
			 model->DIH[j].psi < 0.0 &&
			 model->DIH[j].phi != 11111.0 &&
			 model->DIH[j].psi != 11111.0 ) {
			 printf("\n> SHOW: Model %3d residue %4d (%s) has *bad* ramachandran"\
				" PHI %7.2f PSI %7.2f",
				i, j, model->DIH[j].resname,
				model->DIH[j].phi, model->DIH[j].psi);
		    }
	  }
	  printf("\n>");
	  return ;
     } else if (strncasecmp(OptionTaken,"models",2) == 0) {
	  strcpy(OptionTaken, check_planar_sc("show"));
	  printf("\n>");
	  for (i=1 ; i <= total_models; i++) {
	       model = MODELS+i ;
	       if ( Bad ) { 
		    if ( model->BadModel )
			 printf("\n> SHOW: Model %3d is a *BAD* model", i);
	       } else { 
		    if ( !model->BadModel )
			 printf("\n> SHOW: Model %3d is a *GOOD* model", i);
	       }
	  }
	  printf("\n>");
	  return ;
     } else if (strncasecmp(OptionTaken,"residues",2) == 0) {
	  strcpy(OptionTaken, check_planar_sc("show"));
	  printf("\n>");
	  for (i=1 ; i <= total_models; i++) {
	       model = MODELS+i ;
	       for (j=1; j <= model->number_of_residues; j++)
		    if ( Bad ) { 
			 if ( model->DIH[j].BadResidue )
			      printf("\n> SHOW: Model %3d, res %3d (%s) is a *BAD* residue", 
				     i,model->DIH[j].resnumber,model->DIH[j].resname);
		    } else { 
			 if ( !model->DIH[j].BadResidue )
			      printf("\n> SHOW: Model %3d, res %3d (%s) is a *BAD* residue", 
				     i,model->DIH[j].resnumber,model->DIH[j].resname);
		    }
	  }
	  printf("\n>");
	  return ;
     } else if (strncasecmp(OptionTaken,"ener",2)==0) {
	  printf("\n>");
	  for (i = 1; i <= total_models; i++) {
	       model = MODELS+i ;
	       if ( Bad ) {
		    if (model->energy > ENERGY_CUTOFF)
			 printf("\n> SHOW: Model %4d has *bad* energy: %10.2f", 
				i,model->energy);
	       } else {
		    if (model->energy < ENERGY_CUTOFF)
			 printf("\n> SHOW: Model %4d has good energy: %10.2f", 
				i,model->energy);
	       }
	  }
	  printf("\n>");
	  return ;
     } else if (strncasecmp(OptionTaken,"rmsd",2)==0) {
	  printf("\n>");
	  for (i = 1; i <= total_models; i++) {
	       model = MODELS+i ;
	       if ( Bad ) {
		    if (model->rmsd > RMSD_MODEL_CUTOFF)
			 printf("\n> SHOW: Model %4d has *bad* rmsd: %8.3f", 
				i,model->rmsd);
	       } else {
		    if (model->rmsd < RMSD_MODEL_CUTOFF)
			 printf("\n> SHOW: Model %4d has good rmsd: %8.3f", 
				i,model->rmsd);
	       }
	  }
	  printf("\n>");
	  return ;
     } else if (strncasecmp(OptionTaken,"dete",2)==0) {
	  printf("\n>");
	  for (i = 1; i <= total_models; i++) {
	       model = MODELS+i ;
	       if (model->determ == TRUE)
		    printf("\n> SHOW: Model %4d has *bad* DET(U)", i);
	  }
	  printf("\n>");
	  return ;
     } else if (strncasecmp(OptionTaken,"chi",2)==0) {
	  strcpy(OptionTaken, check_planar_sc("show"));
	  printf("\n>");
	  for (i=1 ; i <= total_models; i++) {
	       model = MODELS+i ;
	       for (j=1; j <= model->number_of_residues; j++)
		    if (model->DIH[j].chiral <= 0.0) 
			 printf("\n> SHOW: Model %3d residue %4d (%s) *bad* chirality %7.2f",
				i, j, model->DIH[j].resname, model->DIH[j].chiral);
	  }
	  printf("\n>");
	  return ;
     } else if (strncasecmp(OptionTaken,"omega",2)==0) {
	  strcpy(OptionTaken, check_planar_sc("show"));
	  printf("\n>");
	  for (i=1 ; i <= total_models; i++) {
	       model = MODELS + i;
	       for (j=1; j <= model->number_of_residues; j++)
		    if ((fabs(model->DIH[j].omega) < 170.0) &&
			(fabs(model->DIH[j].omega) > 10.0))
			 printf("\n> SHOW: Model %3d residue %4d (%s) *bad* omega %7.2f",
				i, j, model->DIH[j].resname, model->DIH[j].omega);
	  }
	  printf("\n>");
	  return ;
     } else { /* we shold never get here */
	  fprintf(stderr,"\n> SHOW: unknown option ");
	  return ;
     }  
     /*  return ; */
}
