/*$Id: read_constraints_congen.c,v 1.7 2003/10/06 12:13:32 roberto Exp roberto $*/

#include <ctype.h>
#include "pdbstat.h"
extern int NamesToDih(char Residue[5], char At1[6], char At2[6], char At3[6], char At4[6]) ;
extern void InitNoeRecord(struct noe *Noe) ;
struct DiheLib *DihLib ;


/* ========================================================================= */
int  read_constraints_congen(FILE *rtti)
/*
 * Although routine's name is self-explanatory this module was written
 * to read constraints in CONGEN format. It was primarily devised to ana
 * lyze the theoretical set of crambin constraints elaborated by Donna
 * Bassolino (we needed this code because we had about 1700 constraints
 * and we wanted to analyze the constraints by residue, etc. and can you
 * imagine doing that by hand?, HA!!!
 *
 * (Roberto Tejero, RTT, May 1993, CABM-Rutgers University).
 *
 */
{
  static int   len, lineno;
  static char  linea[MAXWORD], CgnWord[20] ;
  static short i ;
  struct noe *Noe = NULL ;

  constraintcount = 1;
  lineno = 0;
  
  while ((len=GetLine(line,MAXLINE,rtti)) > 0) 
    if (len > 2) {
      lineno++;
      /*
       * Just looking at the first five lines in the file
       */
      if(lineno < 5)  printf("> CNSTR_rdr: %s",line);
      
      (void) sscanf(line,"%s",linea);
      /*
       * Avoid comment lines to celanup a little
       * Be careful, *ANY* # from posion 0 -> 35 is considered as a
       * comment and consequently throwed away ...
       */
      for (i = 0; i <= 35; i++)	if (line[i]=='!') goto no_count_as_constraint ;
      sscanf(line,"%s",CgnWord);
      if ((strncasecmp(CgnWord,"CONS",4) == 0) && (constraintcount < MAXNOES)) {
	(void) sscanf(line,"%*s %s",linea);
	if (strncasecmp(linea,"ATOM",4)==0)  {
	  if (constraintcount == 10) 
	    fprintf(stdout,"> CNSTR_rdr: Oh, I'm reading! (he) cons file `%s' ..", name_of_the_file);
	  if (fmod(constraintcount,10) == 0.0)
	    fprintf(stdout,"%6d\b\b\b\b\b\b",constraintcount);
	  
	  /* allocate space for NOE */	
	  NOE = (struct noe *) realloc(NOE,sizeof(struct noe)*(constraintcount+1)) ;
	  Noe = NOE + constraintcount ;
          InitNoeRecord(Noe) ;
	  Noe->next = Noe ;
	  sscanf(line," %*s %*s %s %d %s %*s %*s %s %d %s %*s %f %*s %f \n", 
		 Noe->chain_id_1, &Noe->residue_number1, Noe->atom_type1,
		 Noe->chain_id_2, &Noe->residue_number2, Noe->atom_type2,
		 &Noe->lower, &Noe->upper);
	  Noe->Number = constraintcount ;
	  /* Storing data */
	  Noe->equil = (Noe->upper+Noe->lower)/2.0;
	  Noe->rgupp = Noe->upper - Noe->equil ;
	  Noe->rglow = Noe->equil - Noe->lower ;
	  strcpy(Noe->residue_type1,seqres[Noe->residue_number1]);
	  strcpy(Noe->residue_type2,seqres[Noe->residue_number2]);
	  constraintcount+=1 ;
	} else {
	  fprintf(stderr,"> CGEN_RDR: ** Warning **  line with NO noe: %s\n", line);
	}
      }
      no_count_as_constraint :
	;
    }
  ConsTrue = TRUE ;
  return constraintcount--;
}


/* ========================================================================= */
int  read_aco_congen(FILE *rtti)
/*
  ---
  ---
*/
{
  static int   len, lineno, Reg;
  static short i ;
  static char  Key[10] ;
  struct aco *Aco ;

  acocount=1;
  lineno = 0;
  
  while ((len=GetLine(line,MAXLINE,rtti)) > 0) {
    if ((len > 1) && (acocount < MAXNOES)) {
      lineno++;
      
      /* Just to see the first two lines in the file  */
      if (lineno < 3) printf("> CNSTR_rdr: %s",line);
      
      /* Skip no ACO lines  */
      sscanf(line,"%s",Key) ;
      if (strncasecmp(Key,"CONS",4)!=0) goto no_count_as_constraint ;
      
      /* Making a counter of constraints in file  */
      if (acocount == 10)	fprintf(stdout,"> CNSTR_rdr: I'm readinggg!! (he) .... ");
      if (fmod(acocount,10)==0.0)	fprintf(stdout,"%4d\b\b\b\b",acocount);
      
      /*
       * Avoid comment lines to celanup a little
       * Be careful, *ANY* ! from posion 0 -> 35 is considered as a
       * comment and consequently throwed away ...
       */
      for (i = 0; i <= 35; i++)
	if (line[i]=='!') goto no_count_as_constraint ;
      
      /*  Actual reading of file 	 */  
      ACO = (struct aco *)realloc(ACO,sizeof(struct aco)*(acocount+1)) ;
      Aco = ACO + acocount ;
      Aco->next = Aco ;
      /* CONS DIHE Z    1   C  : Z    2   N  : Z    2  CA  : Z    2  C   :  FORCE 201.0  MIN -100.0  MAX  -40.0 */
      /*
      sscanf(line," %*s %*s %*s %*d %*s %*s %*s %d %s %*s %*s %*d %s %*s %*s %*d %*s %*s %*s %*f %*s %f %*s %f", 
	     &ACO[acocount].residue_number,  
             Key1, Key2,
	     &ACO[acocount].AngleMin,
	     &ACO[acocount].AngleMax);
      */
      sscanf(line," %*s %*s %*s %d %s %*s %*s %d %s %*s %*s %d %s %*s %*s %d %s %*s %*s %*f %*s %f %*s %f", 
	     &Aco->Res1,  Aco->Atom1,  
	     &Aco->Res2,  Aco->Atom2,  
	     &Aco->Res3,  Aco->Atom3,  
	     &Aco->Res4,  Aco->Atom4,  
	     &Aco->AngleMin, &Aco->AngleMax);
      Aco->residue_number = Aco->Res2 ;
      strcpy(Aco->residue_type,seqres[Aco->residue_number]);

      /* Correct for PHI and PSI */
      if (Aco->Res1 == Aco->Res2-1) strcpy(Aco->Atom1,"C-");
      if (Aco->Res4 == Aco->Res3+1) strcpy(Aco->Atom4,"N+");
      
      Reg = NamesToDih(Aco->residue_type,Aco->Atom1,Aco->Atom2,Aco->Atom3,Aco->Atom4);
      strcpy(Aco->AngleName,DihLib[Reg].DihName);

      /* switch back to 0->180 and -180->0 */ 
      if (Aco->AngleMin > 180.0) Aco->AngleMin-=360.0 ;
      if (Aco->AngleMax > 180.0) Aco->AngleMax-=360.0 ;
      
      /* Evaluate Equilibrium value and range */
      if (Aco->AngleMin > Aco->AngleMax) {
	Aco->AngleEq = (360.0 + Aco->AngleMin + Aco->AngleMax)/2. ;
	if (Aco->AngleEq > 180.0)  Aco->AngleEq -= 360.0 ;
      } else {
	Aco->AngleEq = (Aco->AngleMin + Aco->AngleMax)/2. ;
	if (Aco->AngleEq > 180.0)  Aco->AngleEq -= 360.0 ;
      } 
      if (Aco->AngleMax < 0.0 && Aco->AngleMin >= 0.0)
	Aco->AngleRg = fabs(360.0 - Aco->AngleMax - Aco->AngleMin);
      else if ( Aco->AngleMin > Aco->AngleMax ) 
	Aco->AngleRg = fabs(360.0 - fabs(Aco->AngleMax - Aco->AngleMin));
      else
	Aco->AngleRg = fabs(Aco->AngleMax - Aco->AngleMin);
     
      Aco->AngleRg = Aco->AngleRg / 2;

      strcpy(Aco->chain_id,"A");
      Aco->Number = acocount ;
      acocount+=1 ;
    }
  no_count_as_constraint: /* do nothing */
    ;
  }
  
  AcoTrue = TRUE;
  return acocount--;
}


