/*$Id: read_coords_disman.c,v 1.7 2003/10/06 12:13:32 roberto Exp roberto $*/

#include "pdbstat.h"
extern short MoreModelsThanAllowed(int nmodels);
extern void atom_properties(int Model) ;
extern void amino_codes(void);


/* ========================================================================= */
void read_coords_disman(FILE *rtti)
     /*
       --- 
       --- As always please note that we need a lot of improvements.
       --- At this point we discard any information about conectivity
       --- between atoms that is present in DISMAN type files, may be
       --- it is worth to read this info making bigger the PDB struc
       --- to have room for this data. (RTT, May 1993, CABM, Rutgers).
       --- 
       --- May 1993, the code has been changed slightly just to use
       --- exactly the same format as the PDB reader. Note that now
       --- PDB[i].pdb_iupac begins with a blank space which is added
       --- after reading the temporary string tmp_str. We don't need
       --- anymore to take care in string comparisons for strings
       --- not beginning with blank space.
       ---
       --- (Roberto Tejero, RTT, May 1993, CABM-Rutgers University).
       ---
     */
{
  int       cont, commen_num, count, tempi,modelon;
  int       number_of_atoms;
  char      commen[100][MAXWORD];
  char      palabro[MAXWORD];
  static struct models *Model ;
  struct atom_pdb *At ;

  commen_num=0;
  q_atoms=0;
  cont = 0;
  models = 0;
  modelon = 0;
  number_of_atoms = 0;
  
  /*
   * Scanning and counting models. For that we use the word NUMBER
   * in the record NUMBER OF RESIDUES that appear in disman files
   */
  fprintf(stdout,"> DISMAN_rdr: ... Counting models in file `%s'; Model... %3d",
	  name_of_the_file_coords, models);
  
  while (fgets(line,MAXLINE,rtti) != NULL) {
    sscanf(line,"%s",commen[1]);
    if (strcmp(commen[1],"NUMBER")==0)  {
      models++;
      fprintf(stdout,"\b\b\b%3d",models);
    }
  }
  
  if (MoreModelsThanAllowed(models)) {
    printf("\n More models -%d- than allowed -%d-.\n"\
	   " You should set TOPMODELS to a greater value. \n\n",
           models, TOPMODELS);
    return ;
  }
  
  AverageModel++ ;
  
  if (models == 0) {
    models = 1;
    fprintf(stdout,"\b\b\b%3d",models);
  }
  
  fprintf(stdout,"\n");
  rewind(rtti);
  
  commen_num = 0;
  if (models > 1) {
    /* Choosing wanted model */
    modelon = 0;
    printf("> DISMAN_rdr: There are %d models\n",models);
    printf("> DISMAN_rdr: What model do you want [(1 to %d) or all] ?_ ", models);
    GetFromKeyBoard(palabro, MAXWORD, stdin);
    if (strncmp(palabro,"exit",1)==0) 
      return;    /* look for exit of this module */
    
    /* Reading all models */
  lee_all:
    
    AverageModel++ ;
    /*
     * Allocate space for each model being read in
     */
    MODELS = (struct models *)realloc(MODELS,sizeof(struct models)* (AverageModel+2)) ;
    if (strncmp(palabro,"all",2)==0)  {
      if (key_model == 0)
	fprintf(stdout,"> DISMAN_rdr: I'm reading! file `%s' model %3d", name_of_the_file_coords,
		key_model); 
      while (fgets(line,MAXLINE,rtti) != NULL) {
	sscanf(line,"%s",commen[1]);
	if (strcmp(commen[1],"NUMBER")==0) {
	  sscanf(line,"%*s %*s %*s %d %*s %*s %*s %d", &numero_de_residuos, &number_of_atoms);
	  goto sal;
	}
      }
    sal:
      if (key_model > 0) {
	if ((key_model % ChainsInModel == 0) && 
	    (key_model > ChainsInModel))
	  fprintf(stdout,"\b\b\b\b\b\b\b\b\b\b\b\b%3d, %d chain", key_model, ChainsInModel); 
	else if ((key_model % ChainsInModel == 0) && (key_model == ChainsInModel ))
	  fprintf(stdout,"%3d, %d chain", key_model, ChainsInModel); 
	else
	  fprintf(stdout,"\b\b\b%3d",key_model); 
      }
      if (key_model < models)  goto lee;
      fprintf(stdout,"\n"); 
      return;
    }
    /*
     * Reading a particular model
     */
    if ((atoi(palabro) < 1) || (atoi(palabro) > models)) {
      printf(">DISMAN_rdr: ** Model should be between 1 and %d\n", models);
      return ;
    }
    while (fgets(line,MAXLINE,rtti) != NULL) {
	sscanf(line,"%s",commen[1]);
	if (strcmp(commen[1],"NUMBER")==0) modelon++;
	if (atoi(palabro) == modelon)  {
	  AverageModel++ ;
	  /*
	   * Allocate space fopr model
	   */
	  MODELS = (struct models *)realloc(MODELS,sizeof(struct models)* (AverageModel+2)) ;
	  sscanf(line,"%*s %*s %*s %d %*s %*s %*s %d", &numero_de_residuos, &number_of_atoms);
	  printf("> DISMAN_rdr: Number of records: %6d\n", number_of_atoms);
	  goto lee;
	}
    }
  } else {
    /*
     * --- we have a single model in the file
     */
    
    while (fgets(line,MAXLINE,rtti) != NULL) {
      commen_num++;
      if (commen_num > 100) {
	printf("> DISMAN_rdr: *** Something wrong ...\n");
	return ;
      }
      sscanf(line,"%s",commen[1]);
      if (strcmp(commen[1],"NUMBER")==0)  {
	sscanf(line,"%*s %*s %*s %d %*s %*s %*s %d", &numero_de_residuos, &number_of_atoms);
	printf("> DISMAN_rdr: Number of records: %6d\n", number_of_atoms);
	goto lee;
      }
    }
  }
 lee:
  key_model++;
  Model = MODELS + key_model ;
  Model->modelno = key_model;
  Model->number_of_residues = numero_de_residuos;
  count=0;
  q_atoms = 0;
  /*
   * --- Actual reading of file
   */
  while(fgets(line,MAXLINE,rtti) != NULL) {
    count++;
    At = (Model->PDB) + count ;
    sscanf(line,"%d %*s %d %s %f %f %f",
	   &cont, &At->pdb_num_res, At->pdb_res,
	   &At->pdb_xcrd, &At->pdb_ycrd, &At->pdb_zcrd);
    tempi=At->pdb_num_res;
    
    At->coord[0] = At->pdb_xcrd;
    At->coord[1] = At->pdb_ycrd;
    At->coord[2] = At->pdb_zcrd;
      
    At->pdb_oldx = At->pdb_xcrd;
    At->pdb_oldy = At->pdb_ycrd;
    At->pdb_oldz = At->pdb_zcrd;
    
    /*===================================================*/
    /*
     * Storing residue number and name in array structure
     */
    if (At->pdb_num_res != Model->PDB[count-1].pdb_num_res) {
      copy(Model->DIH[tempi].resname, At->pdb_res,0,2);
      Model->DIH[tempi].resnumber = tempi;
    }
    /*===================================================*/
    
    At->pdb_num = cont;
    copy(At->pdb_iupac,line,5,9);
    if (At->pdb_iupac[1] == 'Q') q_atoms++;
    /*
     * Storing residue number and name if array structure
     */
    if (At->pdb_num_res != Model->PDB[count-1].pdb_num_res) {
      copy(Model->DIH[At->pdb_num_res].resname, At->pdb_res,0,2);
      Model->DIH[At->pdb_num_res].resnumber = At->pdb_num_res;
    }
    if (count == number_of_atoms) goto fin;
  }
 fin:
  /*
   * be careful in this case number_of_atoms is actually number_of_records
   * in file because atoms equal records minus q-atoms
   */
  
  MAX = count + 1;
  
  Model->number_of_atoms = count ;
  Model->starting_res = Model->PDB[1].pdb_num_res ;
  
  normal_atoms = count - q_atoms ;
  
  if (strncmp(palabro,"all",2)!=0)
    printf("> DISMAN_rdr: %4d records read, %4d atoms and %3d atoms Q DISMAN type \n\n",
	   count, normal_atoms, q_atoms);
  /*
   * ---
   * --- Check number of atoms and number of residues
   * ---
   */
  if (Model->number_of_atoms > MAXATOMS) {
    printf("> DISSMAN_rdr: *** More atoms (%d) than allowed (%d) \n",
	   Model->number_of_atoms, MAXATOMS);
    return;
  }
  if (Model->number_of_residues > MAXRESIDUES) {
    printf("> DISSMAN_rdr: *** More residues (%d) than allowed (%d) \n",
	   Model->number_of_residues, MAXRESIDUES);
    return;
  }
  /*
   * ---
   * --- Check atom names looking for errors in labels
   * ---
   */
  check_label_pdb(1);
  
  /*
   * ---
   * --- Asking for some info about the file just read
   * ---
   */
  
  (void) atom_properties(key_model) ;
  if (strncmp(palabro,"all",2)==0) {
    MolWeight(0);
    CenterMass(0);
  } else {
    printf("> =============================== INFO ================================ \n");
    MolWeight(1);
    CenterMass(2);
  }
  if (strncmp(palabro,"all",2)!=0) {
    info_res(key_model);
    printf("> ===================================================================== \n");
  }
  if ((strncmp(palabro,"all",2)==0) && (key_model < models)) {
    count = 0;
    q_atoms = 0 ;
    goto lee_all;
  }
  total_models = key_model;
  return;
}


