/*$Id: simulate_hyper.c,v 3.32 2003/10/06 12:13:32 roberto Exp roberto $*/

/*
//
// ----------------------------------------------------------------------------
//  Simulation of HYPER constraints from coordinates or from constraints.
//
//  This module creates a file with HYPER constraints for HYPER runs. 
//  the constraints are simulated from coordinates or tranlated from
//  a DYANA, X-PLOR or whatever format constraints file. 
//
//  It assumes the constraints are already loaded in memory in structures
//  NOE and JCOUP or it fills the arrays with simulated data.
//
//  Roberto Tejero. March 2001, Valencia
// -----------------------------------------------------------------------------
//
*/
#include "pdbstat.h"
void WriteNoeJcoupHyper(char From[13]) ;
void selec_dis_for_hyper(void);
void for_hyper(void);
extern FILE *safeopen(char *file, char *mode, int longit);
extern void InitNoeRecord(struct noe *Noe) ;

char RELATIVE_CNS = FALSE ;
char WantPriorKnowledge = FALSE ;
float RangeDih = 10.0 ;

/* ========================================================================= */
void evaluate_j(int numat, int numero_de_js)
{
  /*
    ---
    --- This routine calculates the simulated J values in function of
    --- atom coordinates. It was written down to check and use HYPER
    --- with 'simulated' data derived from a set of coordinates.
    ---
    --- (Roberto Tejero, RTT, May 1993, CABM-Rutgers University).
    ---
  */

  register int  iniciala,  finala;
  register int  numi, numj, numk, numl;
  static short  i;
  static float  ang;
  static struct models *model ;
  static struct jcoup  *J ;

  model = MODELS+key_model ;

  numi=0;
  numj=0;
  numk=0;
  numl=0;

  iniciala = numat - 40 ;
  finala = numat + 40;
  
  if (iniciala <= 0) {iniciala = 1;}
  if (finala > MAX ) {finala = MAX;}

  J = JCOUP + numero_de_js ;

  /* Selecting atoms for PHI calculation */
  if (strcmp(J->name,"j1")==0 || strcmp(J->name,"j1p")==0 ||
      strcmp(J->name,"j7")==0 || strcmp(J->name,"j8")==0 ||
      strcmp(J->name,"j11")==0 || strcmp(J->name,"j13")==0) {

    for (i=iniciala;i<finala;i++) {
      if ((model->PDB[i].pdb_num_res == (model->PDB[numat].pdb_num_res - 1 )) &&
	  (strncmp(model->PDB[i].pdb_iupac," C ",3)==0)) 
	numi=i;
      if (model->PDB[i].pdb_num_res == model->PDB[numat].pdb_num_res) {
	if (strncmp(model->PDB[i].pdb_iupac," N ",3)==0)   numj=i;
	if (strncmp(model->PDB[i].pdb_iupac," CA ",4)==0)  numk=i;
	if (strncmp(model->PDB[i].pdb_iupac," C ",3)==0)   numl=i;
      }
    }
  }
  
  /*  Selecting atoms for PSI calculation */
  if ((strcmp(J->name,"j6")==0) || (strcmp(J->name,"j6p")==0)) {
    for (i=iniciala;i<finala;i++) {
      if ((model->PDB[i].pdb_num_res == (model->PDB[numat].pdb_num_res + 1 )) &&
	  (strncmp(model->PDB[i].pdb_iupac," N ",3)==0)) 
	numl=i;
      if (model->PDB[i].pdb_num_res == model->PDB[numat].pdb_num_res) {
	if (strncmp(model->PDB[i].pdb_iupac," N ",3)==0)  numi=i;
	if (strncmp(model->PDB[i].pdb_iupac," CA ",4)==0) numj=i;
	if (strncmp(model->PDB[i].pdb_iupac," C ",3)==0) numk=i;
      }
    }
  }
  
  /* Selecting atoms for CHI1 calculation */
  if ((strcmp(J->name,"j2")==0) || (strcmp(J->name,"j3")==0) ||
      (strcmp(J->name,"j4")==0) || (strcmp(J->name,"j5")==0) ||
      (strcmp(J->name,"j9")==0) || (strcmp(J->name,"j10")==0) ||
      (strcmp(J->name,"j12")==0))    {
    for (i = iniciala; i < finala ; i++)
      if ((model->PDB[i].pdb_num_res == model->PDB[numat].pdb_num_res) &&
	  ((strncmp(model->PDB[i].pdb_res,"ALA",3)!=0) &&
	   (strncmp(model->PDB[i].pdb_res,"GLY",3)!=0))) {
	if (strncmp(model->PDB[i].pdb_iupac," N ",3)==0)  numi=i;
	if (strncmp(model->PDB[i].pdb_iupac," CA ",4)==0) numj=i;
	if (strncmp(model->PDB[i].pdb_iupac," CB ",4)==0) numk=i;

	if (strncmp(model->PDB[i].pdb_res,"CYS",3)==0) {
	  if (strncmp(model->PDB[i].pdb_iupac," SG ",4)==0)
	    numl=i;
	} else if (strncmp(model->PDB[i].pdb_res,"ILE",3)==0) {
	  if (strncmp(model->PDB[i].pdb_iupac," CG1 ",5)==0)
	    numl=i;
	} else if (strncmp(model->PDB[i].pdb_res,"SER",3)==0) {
	  if (strncmp(model->PDB[i].pdb_iupac," OG ",4)==0)
	    numl=i;
	} else if (strncmp(model->PDB[i].pdb_res,"THR",3)==0) {
	  if (strncmp(model->PDB[i].pdb_iupac," OG1 ",5)==0) 
	    numl=i;	
	} else if (strncmp(model->PDB[i].pdb_res,"VAL",3)==0) {
	  if (strncmp(model->PDB[i].pdb_iupac," CG1 ",5)==0)
	    numl=i;
	} else 
	  if (strncmp(model->PDB[i].pdb_iupac," CG ",4)==0)
	    numl=i;
      }
  }

  /* check that i, j, k, l values are OK */
  if ((numi==0) || (numj==0) || (numk==0) || (numl==0) ||
      (numi>MAX) || (numj>MAX) || (numk>MAX) || (numl>MAX)) {
    printf("\n>> ** Something wrong with atom: %4d (indices i: %4d j: %4d k: %4d l: %4d)",
	   numat,numi,numj,numk,numl); 
    return;
  }
  
  ang = dihang(numi,numj,numk,numl);

  /* Finally Karplus equations */
  if (strcmp(J->name,"j1")==0)
    J->jc = 1.55 - 1.42 * cos(ang-60.0*radperdeg) + 7.09 * cos(ang-60.0*radperdeg) * cos(ang-60.0*radperdeg);
  
  if (strcmp(J->name,"j1p")==0)
    J->jc = 1.55 - 1.42 * cos(ang+60.0*radperdeg) + 7.09 * cos(ang+60.0*radperdeg) * cos(ang+60.0*radperdeg);
  
  if (strcmp(J->name,"j2")==0)
    J->jc = 1.6 - 1.4 * cos(ang-120.0*radperdeg) + 9.4 * cos(ang-120.0*radperdeg) * cos(ang-120.0*radperdeg);
  
  if (strcmp(J->name,"j3")==0)
    J->jc = 1.6 - 1.4 * cos(ang) + 9.4 * cos(ang) * cos(ang);
  
  if (strcmp(J->name,"j4")==0)
    J->jc = 0.1 + 1.2 * cos(ang+120.0*radperdeg) - 4.4 * cos(ang+120.0*radperdeg) * cos(ang+120.0*radperdeg);
  
  if (strcmp(J->name,"j5")==0)
    J->jc = 0.1 + 1.2 * cos(ang-120.0*radperdeg) - 4.4 * cos(ang-120.0*radperdeg) * cos(ang-120.0*radperdeg);
  
  if (strcmp(J->name,"j6")==0)
    J->jc = -0.27 - 0.61 * cos(ang+60.0*radperdeg) - 0.88 * cos(ang+60.0*radperdeg) * cos(ang+60.0*radperdeg);
  
  /* Glycines have HA2 and this is the place for the new equation */
  if (strcmp(J->name,"j6p")==0)
    J->jc = -0.27 - 0.61 * cos(ang-60.0*radperdeg) - 0.88 * cos(ang-60.0*radperdeg) * cos(ang-60.0*radperdeg);
  
  /*
   * New J couplings added
   */

  /* JHN-C'   Phi-180 */
  if (strcmp(J->name,"j7")==0)
    J->jc = 0.0 - 1.01 * cos(ang+180.0*radperdeg) + 4.29 * cos(ang+180.0*radperdeg) * cos(ang+180.0*radperdeg);

  /* JHN-CB   Phi+60 */
  if (strcmp(J->name,"j8")==0) {
    J->jc = 0.13 - 0.74 * cos(ang+60.0*radperdeg) + 3.06 * cos(ang+60.0*radperdeg) * cos(ang+60.0*radperdeg) ;
    
    if (DEBUG > 3)
      printf("> PDBSTAT_hyper_j8: ResNum: %3d PHI: %6.1f j8:%6.2f \n",
	     model->PDB[numj].pdb_num_res, ang/radperdeg, J->jc  );
  }  
  
  /* JC'-HB2[HB1]   Chi1 */
  if (strcmp(J->name,"j9")==0)
    J->jc = 0.6 - 2.04 * cos(ang) + 7.20 * cos(ang) * cos(ang);

  /* JC'-HB3[HB2]   Chi1+120 */
  if (strcmp(J->name,"j10")==0)
    J->jc = 0.6 - 2.04 * cos(ang+120.0*radperdeg) + 7.20 * cos(ang+120.0*radperdeg) * cos(ang+120.0*radperdeg);
  
  /* JC-CB   Phi+60.0 */
  if (strcmp(J->name,"j11")==0)
    J->jc = 0.30 + 1.02 * cos(ang-120.0*radperdeg) + 1.28 * cos(ang-120.0*radperdeg) * cos(ang-120.0*radperdeg);

  /* JC'-C'   Phi  */  
  if (strcmp(J->name,"j13")==0)
    J->jc = 0.60 - 0.93 * cos(ang) + 1.36 * cos(ang) * cos(ang);

  /* JC'-CG   Chi1+60 */ 
  if (strcmp(J->name,"j12")==0)
    J->jc = 0.59 + 0.67* cos(ang+60.0*radperdeg) + 2.02 * cos(ang+60.0*radperdeg) * cos(ang+60.0*radperdeg);
  
  return;
}

/* ========================================================================= */
void for_hyper(void)
/*
  ---
  --- Special subroutine to obtain the input for HYPER from constraint/s
  --- file/s. The input can be any of the constraint/s file/s listed.
  ---
  --- (Roberto Tejero, RTT, May 1993, CABM-Rutgers University).
  ---
  --- As the atom types and everything is read from constraints file
  --- the format for comparison of strings is different.
  --- H and HN stand for different nomenclature in amide hydrogen
  --- HA1 is to take into account GLY
  --- 
  --- (Roberto Tejero, RTT, May 1993, CABM-Rutgers University).
  ---
*/
	
 /*
 // The strange value Noe->volume = -111111.o is usde as a flag to
 // tell that we have actually a HYPER constraint in the NOE
 */
{
  static int    i;
  static struct noe *Noe;

  printf("> HYPER_builder: Running for_hyper for %d constraints \n", kount_dist);

  for (i = 1 ; i <= kount_dist ; i++) {
    Noe = NOE+i;
    Noe->volume=0.0;
    if (Noe->residue_number2 <= Noe->residue_number1+1) {
      if (Noe->residue_number2 == Noe->residue_number1) {
	/*
	 *   Residue i to i
	 *   d1    dhn-ha 
	 *   d1p   dhn-ha2  in GLY
	 */
	if ( strcmp(Noe->Atom1Diana,"HN")==0 &&	strcmp(Noe->Atom2Diana,"HA")==0 ) {
	  strcpy(Noe->HyperName,"dhn-ha");
	  Noe->volume=-111111.0;
	}
	
	/* Clycines */
	if ( strcmp(Noe->Atom1Diana,"HN")==0 &&	strcmp(Noe->Atom2Diana,"HA1")==0 ) {
	  strcpy(Noe->HyperName,"d1    ");
	  Noe->volume=-111111.0;
	}
	if ( strcmp(Noe->Atom1Diana,"HN")==0 &&	strcmp(Noe->Atom2Diana,"HA2")==0 ) {
	  strcpy(Noe->HyperName,"d1p   ");
	  Noe->volume=-111111.0;
	}
	
	/*
	 * d2 and d3    dHN-HB* 
	 */
	if ( strcmp(Noe->Atom1Diana,"HN")==0 && strncmp(Noe->Atom2Diana,"HB",2)==0 ) {
	  if (strcmp(Noe->Atom2Diana,"HB2")==0)  {
	    strcpy(Noe->HyperName,"dhn-hb2");
	    Noe->volume=-111111.0;
	  } else if ( strcmp(Noe->Atom2Diana,"HB3")==0)  {
	    strcpy(Noe->HyperName,"dhn-hb3");
	    Noe->volume=-111111.0;
	  }
	  if ( (strncmp(Noe->residue_type2,"VAL",3)==0) ) {
	    strcpy(Noe->HyperName,"d3     ");
	    Noe->volume=-111111.0;
	  }
	  if ( (strncmp(Noe->residue_type2,"ILE",3)==0) || (strncmp(Noe->residue_type2,"THR",3)==0) ) {
	    strcpy(Noe->HyperName,"d2     ");
	    Noe->volume=-111111.0;
	  }
	}

	/*
	 * d5 and d6    dha-hb*
	 */
	if ( strcmp(Noe->Atom1Diana,"HA")==0 && strncmp(Noe->Atom2Diana,"HB",2)==0 ) {
	  if (strcmp(Noe->Atom2Diana,"HB2")==0)  {
	    strcpy(Noe->HyperName,"dha-hb2");
	    Noe->volume=-111111.0;
	  } else if ( strcmp(Noe->Atom2Diana,"HB3")==0)  {
	    strcpy(Noe->HyperName,"dha-hb3");
	    Noe->volume=-111111.0;
	  }
	  if ( (strncmp(Noe->residue_type2,"VAL",3)==0) ) {
	    strcpy(Noe->HyperName,"d6     ");
	    Noe->volume=-111111.0;
	  }
	  if ( (strncmp(Noe->residue_type2,"ILE",3)==0) || (strncmp(Noe->residue_type2,"THR",3)==0) ) {
	    strcpy(Noe->HyperName,"d5     ");
	    Noe->volume=-111111.0;
	  }
	}
      }
      
      /*
       *  Residue i to i+1, 
       *  distances d4, d8 and d9  dhn-hn,  dha-hn, dhb*-hn
       */
      if (Noe->residue_number2 == Noe->residue_number1+1) {
	/*
	 * d4    dhn-hn
	 */
	if ((strcmp(Noe->Atom1Diana,"HN")==0) && (strcmp(Noe->Atom2Diana,"HN")==0)) {
	  strcpy(Noe->HyperName,"dhn-hn ");
	  Noe->volume=-111111.0;
	}
	/*
	 * d7   dha-hn
	 * d7p  dha2-hn  in GLY
	 */
	if ( strcmp(Noe->Atom1Diana,"HA")==0 && strcmp(Noe->Atom2Diana,"HN")==0) {
	  strcpy(Noe->HyperName,"dha-hn ");
	  Noe->volume=-111111.0;
	}
	if ( strcmp(Noe->Atom1Diana,"HA1")==0 && strcmp(Noe->Atom2Diana,"HN")==0) {
	  strcpy(Noe->HyperName,"dha-hn ");
	  Noe->volume=-111111.0;
	}
	if ( strcmp(Noe->Atom1Diana,"HA2")==0 && strcmp(Noe->Atom2Diana,"HN")==0) {
	  strcpy(Noe->HyperName,"d7p    ");
	  Noe->volume=-111111.0;
	}
	/*
	 * d8 ,  dhb2-hn
	 * d9,   dhb3-hn
	 */
	if ( strncmp(Noe->Atom1Diana,"HB",2)==0 && strcmp(Noe->Atom2Diana,"HN")==0 ) {
	  if (strcmp(Noe->Atom1Diana,"HB2")==0) {
	    strcpy(Noe->HyperName,"dhb2-hn");			 
	    Noe->volume=-111111.0;
	  } else if (strcmp(Noe->Atom1Diana,"HB3")==0) {
	    strcpy(Noe->HyperName,"dhb3-hn");			 
	  }
	  if (strncmp(Noe->residue_type1,"VAL",3)==0) {
	    strcpy(Noe->HyperName,"d9   ");
	    Noe->volume=-111111.0;
	  }
	  if ( (strncmp(Noe->residue_type1,"ILE",3)==0) || (strncmp(Noe->residue_type1,"THR",3)==0) ) {
	    strcpy(Noe->HyperName,"d8   ");
	    Noe->volume=-111111.0;
	  }
	}
      }
    }
  }
  
  WriteNoeJcoupHyper("constraints");

  return;
}


/* ========================================================================= */
void WriteNoeJcoupHyper(char From[13])
	/*
	// kind of messy code but it works!!.
	*/
 /*
  * ---
  * --- (Roberto Tejero, RTT, May 1993, CABM-Rutgers University).
  * ---
  */
{
  FILE   *rtto;
  
  static int    i, k, DihIndex ;
  
  static char   first[5], last[6];
  
  static float  tmp, Lower, Upper;
  static float  tmpd23=0.0, tmpd56=0.0, tmpj23=0.0, tmpj45=0.0;
  static float  tmpd23n=0.0, tmpd56n=0.0, tmpj23n=0.0, tmpj45n=0.0;
  static char   to_write_rj23=FALSE, to_write_rd23=FALSE, 
                to_write_rj45=FALSE, to_write_rd56=FALSE;

  static float  allowed_rd = 0.5 , allowed_rj = 2.0 ;
  static struct models *model;
  static struct noe *Noe;

  model = MODELS+key_model;

  if ( RELATIVE_CNS ) {
     printf("> HYPER_builder: Uncertainty in relative distances [0.5A]?:_ ");
     GetFromKeyBoard(palabra, MAXWORD, stdin) ;
     if (strcmp(palabra,"") != 0)
         allowed_rd = atof(palabra) ;

     printf("> HYPER_builder: Uncertainty in relative J's [2.0unities]?:_ ");
     GetFromKeyBoard(palabra, MAXWORD, stdin) ;
     if (strcmp(palabra,"") != 0)
         allowed_rj = atof(palabra) ;
  }

  printf("\n> HYPER_builder: Output file name ?:_  ");
  GetFromKeyBoard(palabra, MAXWORD, stdin);
    
  rtto = safeopen(palabra,"w",(int)strlen(palabra));

  /* Header of out file with some extra info */
  fprintf(rtto,"\n //----------------------------------------------------------------------------");
  fprintf(rtto,"\n //   Simulate Hyper from Noe constraints or PDB coordinates, simulate_hyper.c ");
  fprintf(rtto,"\n //   Data coming from %s in file %s", From, name_of_the_file);
  fprintf(rtto,"\n //   Roberto Tejero, March 2001, Valencia ");
  fprintf(rtto,"\n //----------------------------------------------------------------------------");
  fprintf(rtto,
	  "\n // Input for hyper made by PDBSTAT for %s "\
	  "\n //       +/- %6.2f Angstroms allowed in distances "\
	  "\n //       +/- %6.2f Hertz allowed in J's "\
	  "\n //       +/- %6.2f Angstroms allowed in relative distances "\
	  "\n //       +/- %6.2f Hertz allowed in relative J's",
	  name_of_the_file, allowed_d, allowed_j, allowed_rd, allowed_rj);
  fprintf(rtto,"\n //----------------------------------------------------------------------------");
  fprintf(rtto,"\n\n djlist ' Test of Hyper '\n  {\n");
 
  if ((int)(model->DIH[NOE[1].residue_number1].phi) == 11111)
    model->DIH[NOE[1].residue_number1].phi=999.0;
  if ((int)(model->DIH[NOE[1].residue_number1].psi) == 11111)
    model->DIH[NOE[1].residue_number1].psi=999.0;
  if ((int)(model->DIH[NOE[1].residue_number1].chi1) == 11111)
    model->DIH[NOE[1].residue_number1].chi1=999.0;
  if (strlen(NOE[1].residue_type1)<3)
    strcpy(NOE[1].residue_type1,"RTT");
 
  DihIndex = NOE[1].residue_number1 - model->PDB[1].pdb_num_res + 1;

  fprintf(rtto,"      Residue_%s_%d[%.0f,%.0f,%.0f]\n        {\n",
	  NOE[1].residue_type1, NOE[1].residue_number1,
	  model->DIH[DihIndex].phi, model->DIH[DihIndex].psi, model->DIH[DihIndex].chi1);
 
  if ( WantPriorKnowledge ) {
	  fprintf(rtto,"\t      allow 'Dihedrals' { \n");
	  if ( model->DIH[DihIndex].phi != 999.0 )
              fprintf(rtto,"\t\t\t\t    phi %8.1f %8.1f\n", model->DIH[DihIndex].phi-RangeDih, model->DIH[DihIndex].phi+RangeDih) ;
	  if ( model->DIH[DihIndex].psi != 999.0 )
              fprintf(rtto,"\t\t\t\t    psi %8.1f %8.1f\n", model->DIH[DihIndex].psi-RangeDih, model->DIH[DihIndex].psi+RangeDih) ;
	  if ( model->DIH[DihIndex].chi1 != 999.0 )
              fprintf(rtto,"\t\t\t\t    chi %8.1f %8.1f\n", model->DIH[DihIndex].chi1-RangeDih, model->DIH[DihIndex].chi1+RangeDih) ;
	  fprintf(rtto,"\t\t\t\t }\n");
  }

  tmp=NOE[1].residue_number1;
  
  for (i = 1; i <= kount_dist; i++) {  
    Noe = NOE+i ;
    if (Noe->volume == -111111.0) {      /* If there is a constraint */ 
      strcpy(first,"true");
      strcpy(last,"false");
      if (Noe->residue_number1 == tmp)  {
	strcpy(first,"false");
      } else {
	tmp=Noe->residue_number1;
	strcpy(last,"true");
      }
      
      /* Initializing for relative distances */
      if (Noe->residue_number1 != NOE[i-1].residue_number1 )  {
	tmpd23 = 0.0 ;
	tmpd56 = 0.0 ;
	tmpd23n = 0.0 ;
	tmpd56n = 0.0 ;
	to_write_rd23 = FALSE ;
	to_write_rd56 = FALSE ;
      }
      
      if (strcmp(last,"true")==0) {
	tmpj23 = 0.0 ;
	tmpj45 = 0.0 ;
	tmpj23n = 0.0 ;
	tmpj45n = 0.0 ;
	to_write_rj23 = FALSE ;
	to_write_rj45 = FALSE ;

	/* Writing J coupling data */
	for (k = 1; k < kount_jcoup; k++) {
	  if ((strcmp(JCOUP[k].flag,"y")==0) &&
	      (JCOUP[k].residue_number1 == NOE[i-1].residue_number1)) {
	    fprintf(rtto,"              %-8s  %8.2f %8.2f %8.2f \n",
		    JCOUP[k].nickname,  /*should be k */
		    JCOUP[k].jc, JCOUP[k].jc + allowed_j, JCOUP[k].jc - allowed_j);
	    
	    if (strcmp(JCOUP[k].name,"j2")==0) tmpj23n=JCOUP[k].jc ;
	    if ((strcmp(JCOUP[k].name,"j3")==0) && (tmpj23==0.0)) {
	      tmpj23=tmpj23n/JCOUP[k].jc ;
	      if (RELATIVE_CNS == TRUE)
		to_write_rj23=TRUE ;
	    }

	    if (strcmp(JCOUP[k].name,"j4")==0) tmpj45n=JCOUP[k].jc ;
	    if ((strcmp(JCOUP[k].name,"j5")==0) && (tmpj45==0.0)) {
	      tmpj45=tmpj45n/JCOUP[k].jc ;
	      if (RELATIVE_CNS == TRUE)
		to_write_rj45=TRUE;
	    }
	    
	    if (RELATIVE_CNS == TRUE)
	      if ((tmpj23!=0) && (to_write_rj23 == TRUE)) {
		/*allowed_rj =  fabs(tmpj23) * 0.2 ; */
		fprintf(rtto,"            rj23r       %8.2f %8.2f %8.2f \n",
			tmpj23, tmpj23+allowed_rj, tmpj23-allowed_rj);
		to_write_rj23 = FALSE ;
	      }
	    
	    if (RELATIVE_CNS == TRUE)
	      if ((tmpj45!=0) && (to_write_rj45 == TRUE)) {
		/* allowed_rj = fabs(tmpj45) * 0.2 ; */
		fprintf(rtto,"            rj45r       %8.2f %8.2f %8.2f \n",
			tmpj45, tmpj45+allowed_rj, tmpj45-allowed_rj);
		to_write_rj45 = FALSE ;
	      }
	  }
	}
	fprintf(rtto,"        }\n");
      }
      
      if (strcmp(first,"true")==0) {
	if ((int)(model->DIH[Noe->residue_number1].phi) == 11111) model->DIH[Noe->residue_number1].phi=999.0;
	if ((int)(model->DIH[Noe->residue_number1].psi) == 11111) model->DIH[Noe->residue_number1].psi=999.0;
	if ((int)(model->DIH[Noe->residue_number1].chi1) == 11111) model->DIH[Noe->residue_number1].chi1=999.0;
	
	if (strlen(Noe->residue_type1) < 3)
	  strcpy(Noe->residue_type1,"RTT");
  
	DihIndex = Noe->residue_number1 - model->PDB[1].pdb_num_res + 1;
	
	fprintf(rtto,"      Residue_%s_%d[%.0f,%.0f,%.0f]\n        {\n",
		Noe->residue_type1, Noe->residue_number1,
		model->DIH[DihIndex].phi, model->DIH[DihIndex].psi, model->DIH[DihIndex].chi1);

        if ( WantPriorKnowledge ) {
	    fprintf(rtto,"\t      allow 'Dihedrals' { \n");
	    if ( model->DIH[Noe->residue_number1].phi != 999.0 )
                 fprintf(rtto,"\t\t\t\t    phi %8.1f %8.1f\n", model->DIH[DihIndex].phi-RangeDih, model->DIH[DihIndex].phi+RangeDih) ;
	    if ( model->DIH[Noe->residue_number1].psi != 999.0 )
                 fprintf(rtto,"\t\t\t\t    psi %8.1f %8.1f\n", model->DIH[DihIndex].psi-RangeDih, model->DIH[DihIndex].psi+RangeDih) ;
	    if ( model->DIH[Noe->residue_number1].chi1 != 999.0 )
                 fprintf(rtto,"\t\t\t\t    chi %8.1f %8.1f\n", model->DIH[DihIndex].chi1-RangeDih, model->DIH[DihIndex].chi1+RangeDih) ;
	    fprintf(rtto,"\t\t\t\t }\n");
        }
      }
      
      Lower = Noe->upper - allowed_d;
     
      if (strncmp(From,"coordinates",3)==0)
	      Upper = Noe->upper + allowed_d ;
      else
	      Upper = Noe->upper ;
      
      if (Lower < 1.6) Lower = 1.6;
      
      fprintf(rtto,"              %-8s  %8.2f %8.2f %8.2f \n", Noe->HyperName, Noe->upper, Upper, Lower);
      
      /*
       * Trying to get relative distance constraints
       */
      if (strcmp(Noe->HyperName,"d2")==0) tmpd23n=Noe->upper ;
      if ((strcmp(Noe->HyperName,"d3")==0) && (tmpd23==0.0)) {
	tmpd23=tmpd23n/Noe->upper ;
	if (RELATIVE_CNS == TRUE)
	  to_write_rd23 = TRUE ;
      }
      if (strcmp(Noe->HyperName,"d5")==0) tmpd56n=Noe->upper ;
      if ((strcmp(Noe->HyperName,"d6")==0) && (tmpd56==0.0)) {
	tmpd56=tmpd56n/Noe->upper ;
	if (RELATIVE_CNS == TRUE)
	  to_write_rd56 = TRUE ;
      }
      
      if (RELATIVE_CNS == TRUE)
	if ((tmpd23!=0) && (to_write_rd23 == TRUE)) {
	  fprintf(rtto,"            rd23r       %8.2f %8.2f %8.2f \n",
		  tmpd23, tmpd23+allowed_rd, tmpd23-allowed_rd);
	  to_write_rd23 = FALSE ;
	}      
      if (RELATIVE_CNS == TRUE)
	if ((tmpd56!=0) && (to_write_rd56==TRUE)) {
	  fprintf(rtto,"            rd56r       %8.2f %8.2f %8.2f \n",
		  tmpd56, tmpd56+allowed_rd, tmpd56-allowed_rd);
	  to_write_rd56 = FALSE ;
	}
    }
  }
  
  /* 
   * Print last residue's jcoup data 
   */
  for (k=1;k<=kount_jcoup;k++) 
    if ((strcmp(JCOUP[k].flag,"y")==0) &&
	(JCOUP[k].residue_number1==NOE[kount_dist].residue_number1)) {
      fprintf(rtto,"              %-8s  %8.2f %8.2f %8.2f \n",
	      JCOUP[k].nickname,  /* should be k */
	      JCOUP[k].jc, JCOUP[k].jc + allowed_j, JCOUP[k].jc - allowed_j);
  }
  
  fprintf(rtto,"        }\n  }\n");
  fflush(rtto);
  fclose(rtto);
  return;
}


/* ========================================================================= */
void selec_dis_for_hyper(void)

/*
  --- 
  --- Selection of the distances for running hyper. This modulus is capable
  --- to simulate input data for hyper from known coordinates.
  ---
  ---
  --- (Roberto Tejero, RTT, May 1993, CABM-Rutgers University).
  ---
*/

{
  static char   jotaschar[10];
  static short  jotas = 13 ;
  static short  i,k, contador_de_cons, contador_de_js;
  static struct models *model;
  static struct atom_pdb *Ati, *Atk ;
  static struct noe  *Noe ;
  static struct jcoup *J ;

  model = MODELS+key_model ;
  Noe = NOE ;

  contador_de_cons=0;
  contador_de_js=0;

  strcpy(jotaschar,"");
  printf("> HYPER_selecdist: How many jotas (none, 6 or 13)?_ : ");
  GetFromKeyBoard(jotaschar, MAXWORD, stdin);
  if (strcmp(jotaschar,"")==0) 
	  jotas=0 ;
  else if (strncasecmp(jotaschar,"none",1)==0) 
	  jotas=0;
  else 
	  jotas = atoi(jotaschar);

  strcpy(jotaschar,"");
  printf("> HYPER_selecdist: Do you like relative constraints [y/n]?_ : ");
  GetFromKeyBoard(jotaschar, MAXWORD, stdin);
  if (strncasecmp(jotaschar,"yes",1)==0) RELATIVE_CNS = TRUE ;
  printf("> HYPER_builder: Running hyper from coordinates for %d atoms \n", MAX-1);
  printf("> HYPER_selecdist: Do you like prior knowledge extracted from dihedral angles [y/n]?_ : ");
  GetFromKeyBoard(jotaschar, MAXWORD, stdin);
  if (strncasecmp(jotaschar,"yes",1)==0) {
	  WantPriorKnowledge = TRUE ;
          printf("> HYPER_selecdist: Amount to add/subtract to each angle ?_ : ");
          GetFromKeyBoard(jotaschar, MAXWORD, stdin);
	  RangeDih = atof(jotaschar);
  }
  
  if (DEBUG > 5)
    fprintf(stdout,"> HYPER_builder: Working residue ...");
  
  for (i = 1 ; i < MAX ; i++)
    for (k = i-30 ; k < MAX ; k++) {
	    Ati = (model->PDB) + i ;
	    Atk = (model->PDB) + k ;
      if (k < 0) k=1 ; 
      /* Same residue:  i to i */
      if (Atk->pdb_num_res == Ati->pdb_num_res) {
	if (DEBUG > 0) fprintf(stdout, "\b\b\b\b\b%5d",Ati->pdb_num_res);
	
	/* HN--HA$ */
	if ((strncmp(Ati->pdb_iupac," H ",3)==0) || (strncmp(Ati->pdb_iupac," HN ",4)==0))
	  if ((strncmp(Atk->pdb_iupac," HA ",4)==0) || (strncmp(Atk->pdb_iupac," HA1",4)==0) ||
	      (strncmp(Atk->pdb_iupac,"1HA ",4)==0)) {
	    contador_de_cons++;
	    NOE = (struct noe *)realloc(NOE,sizeof(struct noe)*(contador_de_cons+1)) ;
	    Noe = NOE + contador_de_cons ;
	    InitNoeRecord(Noe);
	    Noe->residue_number1 =     Ati->pdb_num_res;
	    Noe->residue_number2 =     Atk->pdb_num_res;
	    strcpy(Noe->residue_type1, Ati->pdb_res);
	    strcpy(Noe->residue_type2, Atk->pdb_res);
	    strcpy(Noe->atom_type1,    Ati->pdb_iupac);
	    strcpy(Noe->atom_type2,    Atk->pdb_iupac);
	    strcpy(Noe->HyperName,"dhn-ha");
	    Noe->volume = -111111.0;
	    Noe->upper = (float)sqrt(pow((Ati->pdb_xcrd - Atk->pdb_xcrd),2)+
			  pow((Ati->pdb_ycrd - Atk->pdb_ycrd),2)+
			  pow((Ati->pdb_zcrd - Atk->pdb_zcrd),2)) ;
	    if ( jotas > 0 ) {
	      contador_de_js++;
	      JCOUP = (struct jcoup *)realloc(JCOUP,sizeof(struct jcoup)*(contador_de_js+1)) ;
	      J = JCOUP + contador_de_js ;
	      strcpy(J->flag,"y");
	      strcpy(J->name,"j1");
	      strcpy(J->nickname,"jhn-ha");
	      J->residue_number1 = Noe->residue_number1;
	      evaluate_j(i,contador_de_js);
	    }
	  }
	
	/*
	 * Glycines have HA2 so we need d' (called d1p). and j1' (j1p)
	 */
	if ((strncmp(Ati->pdb_iupac," H ",3)==0) || (strncmp(Ati->pdb_iupac," HN ",4)==0))
	  if ((strncmp(Atk->pdb_iupac," HA2",4)==0) || (strncmp(Atk->pdb_iupac,"2HA ",4)==0)) {
	    contador_de_cons++;	    
	    NOE = (struct noe *)realloc(NOE,sizeof(struct noe)*(contador_de_cons+1)) ;
	    Noe = NOE+contador_de_cons ;
	    InitNoeRecord(Noe);
	    Noe->residue_number1     = Ati->pdb_num_res;
	    Noe->residue_number2     = Atk->pdb_num_res;
	    strcpy(Noe->residue_type1, Ati->pdb_res);
	    strcpy(Noe->residue_type2, Atk->pdb_res);
	    strcpy(Noe->atom_type1,    Ati->pdb_iupac);
	    strcpy(Noe->atom_type2,    Atk->pdb_iupac);
	    strcpy(Noe->HyperName,"d1p");
	    Noe->volume = -111111.0;
	    Noe->upper = (float)sqrt(pow((Ati->pdb_xcrd - Atk->pdb_xcrd),2)+
			  pow((Ati->pdb_ycrd - Atk->pdb_ycrd),2)+
			  pow((Ati->pdb_zcrd - Atk->pdb_zcrd),2));
	    if ( jotas > 0 ) {
	      contador_de_js++;
	      JCOUP = (struct jcoup *)realloc(JCOUP,sizeof(struct jcoup)*(contador_de_js+1)) ;
	      J = JCOUP + contador_de_js ;
	      strcpy(J->flag,"y");
	      strcpy(J->name,"j1p");
	      strcpy(J->nickname,"jhn-ha2");
	      J->residue_number1 = Noe->residue_number1;
	      evaluate_j(i,contador_de_js);
	    }
	  }

	/* HN--HB$ */
	if ((strncmp(Ati->pdb_iupac," H ",3)==0) || (strncmp(Ati->pdb_iupac," HN ",4)==0))
	  if ((strncmp(Atk->pdb_iupac," HB ",4)==0) || (strncmp(Atk->pdb_iupac,"1HB ",4)==0) ||
	      (strncmp(Atk->pdb_iupac,"2HB ",4)==0)) {
	    contador_de_cons++;	    
	    NOE = (struct noe *)realloc(NOE,sizeof(struct noe)*(contador_de_cons+1)) ;
	    Noe = NOE+contador_de_cons ;
	    InitNoeRecord(Noe);
	    if ((strncmp(Atk->pdb_iupac," HB ",4)==0) || (strncmp(Atk->pdb_iupac," HB1",4)==0) ||
		(strncmp(Atk->pdb_iupac,"1HB ",4)==0))
	      strcpy(Noe->HyperName,"dhn-hb2");
	    if ((strncmp(Atk->pdb_iupac," HB2",4)==0) || (strncmp(Atk->pdb_iupac,"2HB ",4)==0))
	      strcpy(Noe->HyperName,"dhn-hb3");
	    if (strncmp(Atk->pdb_res,"VAL",3)==0)
	      strcpy(Noe->HyperName,"d3");
	    Noe->residue_number1 =     Ati->pdb_num_res;
	    Noe->residue_number2 =     Atk->pdb_num_res;
	    strcpy(Noe->residue_type1, Ati->pdb_res);
	    strcpy(Noe->residue_type2, Atk->pdb_res);
	    strcpy(Noe->atom_type1,    Ati->pdb_iupac);
	    strcpy(Noe->atom_type2,    Atk->pdb_iupac);
	    Noe->upper = (float)sqrt(pow((Ati->pdb_xcrd - Atk->pdb_xcrd),2)+
			  pow((Ati->pdb_ycrd - Atk->pdb_ycrd),2)+
			  pow((Ati->pdb_zcrd - Atk->pdb_zcrd),2));
	    Noe->volume=-111111.0;
	  }

	/* HA--HB$ */
	if ((strncmp(Ati->pdb_iupac," HA ",4)==0) || (strncmp(Ati->pdb_iupac," HA1",4)==0) ||
	    (strncmp(Ati->pdb_iupac,"1HA ",4)==0))
	  if ((strncmp(Atk->pdb_iupac," HB ",4)==0) || (strncmp(Atk->pdb_iupac,"1HB ",4)==0) ||
	      (strncmp(Atk->pdb_iupac,"2HB ",4)==0)) { 
	    contador_de_cons++;	    
	    NOE = (struct noe *)realloc(NOE,sizeof(struct noe)*(contador_de_cons+1)) ;
	    Noe = NOE + contador_de_cons;
	    InitNoeRecord(Noe);
	    if ((strncmp(Atk->pdb_iupac," HB ",4)==0) || (strncmp(Atk->pdb_iupac," HB1",4)==0) ||
		(strncmp(Atk->pdb_iupac,"1HB ",4)==0))  {
	      strcpy(Noe->HyperName,"dha-hb2");
	      if (strncmp(Atk->pdb_res,"VAL",3)==0)
		strcpy(Noe->HyperName,"dha-hb3");
	      /* - j2 - */
	      if ( jotas > 0 )
		if (strncmp(Ati->pdb_res,"ALA",2)!=0) {
		  contador_de_js++;
		  JCOUP = (struct jcoup *)realloc(JCOUP,sizeof(struct jcoup)*(contador_de_js+1)) ;
		  J = JCOUP + contador_de_js ;
		  strcpy(J->flag,"y");
		  strcpy(J->name,"j2");
		  strcpy(J->nickname,"jha-hb2");
		  if (strncmp(Atk->pdb_res,"VAL",3)==0) {
		    strcpy(J->name,"j3");
		    strcpy(J->nickname,"jha-hb3");
		  }
		  J->residue_number1 = Ati->pdb_num_res;
		  evaluate_j(i,contador_de_js);
		}
	    }
	    
	    if ((strncmp(Atk->pdb_iupac," HB2",4)==0) || (strncmp(Atk->pdb_iupac,"2HB ",4)==0)) {
	      strcpy(Noe->HyperName,"dha-hb3");
	      /* - j3 - */
	      if ( jotas > 0 )
		if (strncmp(Ati->pdb_res,"ALA",2)!=0)  {
		  contador_de_js++;
		  JCOUP = (struct jcoup *)realloc(JCOUP,sizeof(struct jcoup)*(contador_de_js+1)) ;
		  J = JCOUP + contador_de_js ;
		  strcpy(J->flag,"y");
		  strcpy(J->name,"j3");
		  strcpy(J->nickname,"jha-hb3");
		  J->residue_number1 = Ati->pdb_num_res;
		  evaluate_j(i,contador_de_js);
		}
	    }
	    Noe->residue_number1 =     Ati->pdb_num_res;
	    Noe->residue_number2 =     Atk->pdb_num_res;
	    strcpy(Noe->residue_type1, Ati->pdb_res);
	    strcpy(Noe->residue_type2, Atk->pdb_res);
	    strcpy(Noe->atom_type1,    Ati->pdb_iupac);
	    strcpy(Noe->atom_type2,    Atk->pdb_iupac);
	    Noe->volume = -111111.0;
	    Noe->upper = (float)sqrt(pow((Ati->pdb_xcrd - Atk->pdb_xcrd),2)+
			  pow((Ati->pdb_ycrd - Atk->pdb_ycrd),2)+
			  pow((Ati->pdb_zcrd - Atk->pdb_zcrd),2));
	  }  /* end HA --- HB */
	
	/* N--HB1 - j4 - */
	if ( jotas > 0 )
	  if (strncmp(Ati->pdb_iupac," N ",3)==0) 
	    if ((strncmp(Atk->pdb_iupac," HB1",4)==0) || (strncmp(Atk->pdb_iupac,"1HB ",4)==0) ||
		(strncmp(Atk->pdb_iupac," HB ",4)==0)) {
	      if (strncmp(Ati->pdb_res,"ALA",2)!=0) {
		contador_de_js++;
		JCOUP = (struct jcoup *) realloc(JCOUP,sizeof(struct jcoup)*(contador_de_js+1)) ;
		J = JCOUP + contador_de_js ;
		strcpy(J->flag,"y");
		strcpy(J->name,"j4");
		strcpy(J->nickname,"jn-hb2");
		if (strncmp(Atk->pdb_res,"VAL",3)==0) {
		  strcpy(J->name,"j5");
		  strcpy(J->nickname,"jn-hb3");
		}
		J->residue_number1 = Ati->pdb_num_res;
		evaluate_j(i,contador_de_js);
	      }
	    }
	
	/* N--HB2 - j5 - */     
	if ( jotas > 0 )
	  if (strncmp(Ati->pdb_iupac," N ",3)==0)
	    if ((strncmp(Atk->pdb_iupac," HB2",4)==0) || (strncmp(Atk->pdb_iupac,"2HB ",4)==0)) {
	      if (strncmp(Ati->pdb_res,"ALA",2)!=0)  {
		contador_de_js++;
		JCOUP = (struct jcoup *) realloc(JCOUP,sizeof(struct jcoup)*(contador_de_js+1)) ;
		J = JCOUP + contador_de_js ;
		strcpy(J->flag,"y");
		strcpy(J->name,"j5");
		strcpy(J->nickname,"jn-hb3");
		J->residue_number1 = Ati->pdb_num_res;
		evaluate_j(i,contador_de_js);
	      }
	    }
	
	/* HN--C j7 */
	if (jotas >= 7) 	
	  if ((strncmp(Ati->pdb_iupac," H ",3)==0) || (strncmp(Ati->pdb_iupac," HN ",4)==0))
	    if ((strncmp(Atk->pdb_iupac," C ",3)==0)) {
	      if (Noe->residue_number1 > 1) {
		contador_de_js++;
		JCOUP = (struct jcoup *)realloc(JCOUP,sizeof(struct jcoup)*(contador_de_js+1)) ;
		J = JCOUP + contador_de_js ;
		strcpy(J->flag,"y");
		strcpy(J->name,"j7");
		strcpy(J->nickname,"jhn-c");
		J->residue_number1 = Ati->pdb_num_res;
		evaluate_j(i,contador_de_js);
	      }
	    }
	
	/* HN--CB j8 */
	if (jotas >= 8) 	
	  if ((strncmp(Ati->pdb_iupac," H ",3)==0) || (strncmp(Ati->pdb_iupac," HN ",4)==0))
	    if ((strncmp(Atk->pdb_iupac," CB ",4)==0)) {
	      if (Noe->residue_number1 > 1) {
		contador_de_js++;
		JCOUP = (struct jcoup *)realloc(JCOUP,sizeof(struct jcoup)*(contador_de_js+1)) ;
		J = JCOUP + contador_de_js ;
		strcpy(J->flag,"y");
		strcpy(J->name,"j8");
		strcpy(J->nickname,"jhn-cb");
		J->residue_number1 = Ati->pdb_num_res;
		evaluate_j(i,contador_de_js);
	      }
	    }

	/* C--HB1 - j9 - */
	if (jotas >= 9) 	
	  if (strncmp(Ati->pdb_iupac," C ",3)==0) 
	    if ((strncmp(Atk->pdb_iupac," HB1",4)==0) || (strncmp(Atk->pdb_iupac,"1HB ",4)==0) ||
		(strncmp(Atk->pdb_iupac," HB ",4)==0)) {
	      if (strncmp(Ati->pdb_res,"ALA",2)!=0) {
		contador_de_js++;
		JCOUP = (struct jcoup *)realloc(JCOUP,sizeof(struct jcoup)*(contador_de_js+1)) ;
		J = JCOUP + contador_de_js ;
		strcpy(J->flag,"y");
		strcpy(J->name,"j9");
		strcpy(J->nickname,"jc-hb2");
		if (strncmp(Atk->pdb_res,"VAL",3)==0) {
		  strcpy(J->name,"j10");
		  strcpy(J->nickname,"jc-hb3");
		}
		J->residue_number1 = Ati->pdb_num_res;
		evaluate_j(i,contador_de_js);
	      }
	    }
	
	/* C--HB2 - j10 - */     
	if (jotas >= 10) 	
	  if (strncmp(Ati->pdb_iupac," C ",3)==0)
	    if ((strncmp(Atk->pdb_iupac," HB2",4)==0) || (strncmp(Atk->pdb_iupac,"2HB ",4)==0)) {
	      if (strncmp(Ati->pdb_res,"ALA",2)!=0)  {
		contador_de_js++;
		JCOUP = (struct jcoup *)realloc(JCOUP,sizeof(struct jcoup)*(contador_de_js+1)) ;
		J = JCOUP + contador_de_js ;
		strcpy(J->flag,"y");
		strcpy(J->name,"j10");
		strcpy(J->nickname,"jc-hb3");
		J->residue_number1 = Ati->pdb_num_res;
		evaluate_j(i,contador_de_js);
	      }
	    }

	/* C--CG - jC'CG j12 - */     
	if (jotas >= 13) 	
	  if (strncmp(Ati->pdb_iupac," C ",3)==0)
	    if ((strncmp(Atk->pdb_iupac," CG ",4)==0) || (strncmp(Atk->pdb_iupac," CG1",4)==0)) {
	      contador_de_js++;
	      JCOUP = (struct jcoup *)realloc(JCOUP,sizeof(struct jcoup)*(contador_de_js+1)) ;
	      J = JCOUP + contador_de_js ;
	      strcpy(J->flag,"y");
	      strcpy(J->name,"j12");
	      strcpy(J->nickname,"jc-cg");
	      J->residue_number1 = Ati->pdb_num_res;
	      evaluate_j(i,contador_de_js);
	    }
      }
	
      /*
       * Sequential
       * residue i to i+1 
       */
      if (Atk->pdb_num_res == Ati->pdb_num_res + 1) {
	
	/* HA--Ni+1 JHN-N  J6*/
	if ( jotas > 0 )
	  if ((strncmp(Ati->pdb_iupac," HA ",4)==0) || (strncmp(Ati->pdb_iupac," HA1",4)==0) ||
	      (strncmp(Ati->pdb_iupac,"1HA ",4)==0))
	    if (strncmp(Atk->pdb_iupac," N ",3)==0) {
	      contador_de_js++;
	      JCOUP = (struct jcoup *)realloc(JCOUP,sizeof(struct jcoup)*(contador_de_js+1)) ;
	      J = JCOUP + contador_de_js ;
	      strcpy(J->flag,"y");
	      strcpy(J->name,"j6");
	      strcpy(J->nickname,"jha-n");
	      J->residue_number1 = Ati->pdb_num_res;
	      evaluate_j(i,contador_de_js);
	    }           
	
	/*
	 * Glycines have HA2 so we need J6' (called j6p)
	 */

	/* HA2--Ni+1 JHN-N  J6p*/
	if ( jotas > 0 )	
	  if ((strncmp(Ati->pdb_iupac," HA2",4)==0) || (strncmp(Ati->pdb_iupac,"2HA ",4)==0))
	    if (strncmp(Atk->pdb_iupac," N ",3)==0) {
	      contador_de_js++;
	      JCOUP = (struct jcoup *)realloc(JCOUP,sizeof(struct jcoup)*(contador_de_js+1)) ;
	      J = JCOUP + contador_de_js ;
	      strcpy(J->flag,"y");
	      strcpy(J->name,"j6p");
	      strcpy(J->nickname,"j6p");
	      J->residue_number1 = Ati->pdb_num_res;
	      evaluate_j(i,contador_de_js);
	    }           
	
	/*   Hi--Hi+1   */
	if ((strncmp(Ati->pdb_iupac," H ",3)==0)  && (strncmp(Atk->pdb_iupac," H ",3)==0)) {
	  contador_de_cons++;	  
	  NOE = (struct noe *)realloc(NOE,sizeof(struct noe)*(contador_de_cons+1)) ;
	  Noe = NOE+contador_de_cons;
          InitNoeRecord(Noe);
	  strcpy(Noe->HyperName,"dhn-hn");
	  Noe->residue_number1 = Ati->pdb_num_res;
	  Noe->residue_number2 = Atk->pdb_num_res;
	  strcpy(Noe->residue_type1, Ati->pdb_res);
	  strcpy(Noe->residue_type2, Atk->pdb_res);
	  strcpy(Noe->atom_type1, Ati->pdb_iupac);
	  strcpy(Noe->atom_type2, Atk->pdb_iupac);
	  Noe->volume=-111111.0;
	  Noe->upper = (float)sqrt(pow((Ati->pdb_xcrd - Atk->pdb_xcrd),2)+
			pow((Ati->pdb_ycrd - Atk->pdb_ycrd),2)+
			pow((Ati->pdb_zcrd - Atk->pdb_zcrd),2));
	}

	/*   HA--Hi+1   */
	if ((strncmp(Ati->pdb_iupac," HA ",4)==0) || (strncmp(Ati->pdb_iupac," HA1",4)==0) ||
	    (strncmp(Ati->pdb_iupac,"1HA ",4)==0))
	  if ((strncmp(Atk->pdb_iupac," H ",3)==0) || (strncmp(Atk->pdb_iupac," HN ",4)==0)) {
	    contador_de_cons++;	    
	    NOE = (struct noe *)realloc(NOE,sizeof(struct noe)*(contador_de_cons+1)) ;
	    Noe = NOE+contador_de_cons ;
            InitNoeRecord(Noe);
	    strcpy(Noe->HyperName,"dha-hn");

	    Noe->residue_number1 = Ati->pdb_num_res;
	    Noe->residue_number2 = Atk->pdb_num_res;
	    strcpy(Noe->residue_type1, Ati->pdb_res);
	    strcpy(Noe->residue_type2, Atk->pdb_res);
	    strcpy(Noe->atom_type1, Ati->pdb_iupac);
	    strcpy(Noe->atom_type2, Atk->pdb_iupac);
	    Noe->volume=-111111.0;
	    Noe->upper = (float)sqrt(pow((Ati->pdb_xcrd - Atk->pdb_xcrd),2)+
			  pow((Ati->pdb_ycrd - Atk->pdb_ycrd),2)+
			  pow((Ati->pdb_zcrd - Atk->pdb_zcrd),2));
	  }

	/*
	 * Glycines have HA2, so we need d7' (called d7p).
	 */
	if ((strncmp(Ati->pdb_iupac," HA2",4)==0) || (strncmp(Ati->pdb_iupac,"2HA ",4)==0))
	  if ((strncmp(Atk->pdb_iupac," H ",3)==0) || (strncmp(Atk->pdb_iupac," HN ",4)==0)) {
	    contador_de_cons++;	    
	    NOE = (struct noe *)realloc(NOE,sizeof(struct noe)*(contador_de_cons+1)) ;
	    Noe = NOE + contador_de_cons ;
            InitNoeRecord(Noe);
	    strcpy(Noe->HyperName,"d7p");
	    Noe->residue_number1 = Ati->pdb_num_res;
	    Noe->residue_number2 = Atk->pdb_num_res;
	    strcpy(Noe->residue_type1, Ati->pdb_res);
	    strcpy(Noe->residue_type2, Atk->pdb_res);
	    strcpy(Noe->atom_type1, Ati->pdb_iupac);
	    strcpy(Noe->atom_type2, Atk->pdb_iupac);
	    Noe->volume=-111111.0;
	    Noe->upper = (float)sqrt(pow((Ati->pdb_xcrd - Atk->pdb_xcrd),2)+
			  pow((Ati->pdb_ycrd - Atk->pdb_ycrd),2)+
			  pow((Ati->pdb_zcrd - Atk->pdb_zcrd),2));
	  }

	/*   HB$--Hi+1  */
	if ((strncmp(Ati->pdb_iupac," HB ",4)==0) || (strncmp(Ati->pdb_iupac,"1HB ",4)==0) ||
	    (strncmp(Ati->pdb_iupac,"2HB ",4)==0))
	  if (strncmp(Atk->pdb_iupac," H ",3)==0) {
	    contador_de_cons++;	    
	    NOE = (struct noe *)realloc(NOE,sizeof(struct noe)*(contador_de_cons+1)) ;
	    Noe = NOE + contador_de_cons ;
            InitNoeRecord(Noe);
	    if ((strncmp(Ati->pdb_iupac," HB ",4)==0) || (strncmp(Ati->pdb_iupac," HB1",4)==0) ||
		(strncmp(Ati->pdb_iupac,"1HB ",4)==0))
	      if (strncmp(Ati->pdb_res,"VAL",3)==0)
		strcpy(Noe->HyperName,"d9");
	      else
		strcpy(Noe->HyperName,"dhb2-hn");
	    if ((strncmp(Ati->pdb_iupac," HB2",4)==0) || (strncmp(Ati->pdb_iupac,"2HB ",4)==0))
	      strcpy(Noe->HyperName,"dhb3-hn");
	    Noe->residue_number1 = Ati->pdb_num_res;
	    Noe->residue_number2 = Atk->pdb_num_res;
	    strcpy(Noe->residue_type1, Ati->pdb_res);
	    strcpy(Noe->residue_type2, Atk->pdb_res);
	    strcpy(Noe->atom_type1, Ati->pdb_iupac);
	    strcpy(Noe->atom_type2, Atk->pdb_iupac);
	    Noe->volume=-111111.0;
	    Noe->upper = (float)sqrt(pow((Ati->pdb_xcrd - Atk->pdb_xcrd),2)+
			  pow((Ati->pdb_ycrd - Atk->pdb_ycrd),2)+
			  pow((Ati->pdb_zcrd - Atk->pdb_zcrd),2));
	  }	

	if (Ati->pdb_num_res > 1) {
	  /* C--CBi+1 - jC'CB j11 - */     
	  if (jotas >= 11)
	    if (strncmp(Ati->pdb_iupac," C ",3)==0)
	      if (strncmp(Atk->pdb_iupac," CB ",4)==0) {
		if (strncmp(Ati->pdb_res,"GLY",2)!=0) {
		  contador_de_js++;
		  JCOUP = (struct jcoup *)realloc(JCOUP,sizeof(struct jcoup)*(contador_de_js+1)) ;
		  J = JCOUP + contador_de_js ;
		  strcpy(J->flag,"y");
		  strcpy(J->name,"j11");
		  strcpy(J->nickname,"jc-cb");
		  J->residue_number1 = Ati->pdb_num_res;
		  evaluate_j(i,contador_de_js);
		}
	      }
	  
	  /* C--Ci+1 - jC'C' j13 - */     
	  if (jotas >= 12) 	
	    if (strncmp(Ati->pdb_iupac," C ",3)==0)
	      if (strncmp(Atk->pdb_iupac," C ",3)==0) {
		contador_de_js++;
		JCOUP = (struct jcoup *) realloc(JCOUP,sizeof(struct jcoup)*(contador_de_js+1)) ;
		J = JCOUP + contador_de_js ;
		strcpy(J->flag,"y");
		strcpy(J->name,"j13");
		strcpy(J->nickname,"jc-c");
		J->residue_number1 = Ati->pdb_num_res;
		evaluate_j(i,contador_de_js);
	      }
	} /* if Res number is .gt. 1 */
      }
    }

  if (DEBUG > 0) fprintf(stdout,"\n");

  printf("\n> HYPER_builder: %4d constr. and  %4d j's from coordinates \n",
	 contador_de_cons, contador_de_js);
  kount_dist = contador_de_cons;
  kount_jcoup = contador_de_js;

  WriteNoeJcoupHyper("coordinates");
  return ;
}
