/*$Id: util.c,v 3.32 2003/10/06 12:13:32 roberto Exp roberto $*/

/*
 * Little module where a few 'utilities' or routines for other programs
 * are stored. Some of the utilities have a better and nice counterpart
 * in 'C' standard library routines but the ones here were written long ago
 * while learning 'C' and were designed (if we can say that) to be more
 * adapted to my own code and idiosincracy.
 *
 * Other parts are headers and initialization for PosScript files taken
 * from ORIENT code written by George Elkins and slightly adapted to this
 * case.
 *
 * The final part are routines to handle a library of atom names very useful
 * for managing atom names in PDB files and also to manage and convert
 * between different NOE atom name conventions. The name for the library
 * is 'GenCons.Lib'
 */

#include "pdbstat.h"
#include <unistd.h>
#define min(A,B) ((A) < (B) ? (A) : (B))
#define max(A,B) ((A) > (B) ? (A) : (B))

short MoreModelsThanAllowed(int nmodels);
char ModelHasAtoms(int ModelN);
char IsThereAModel(void);
char IsThereConstraints(void);
char IsThereSequence(void);
void do_copy_model(int from, int to);

void OneToThree2(char Name[5]) ;
void AssignLibraryNames(struct noe *Noe, int LibRecords, int option, char KeyAss[5]) ;
void AssignSequenceNameToNOE(void) ;
void CheckLibraryNames(int LibRecords,int option, int from) ; 
void CheckPDBNames(int LibRecords,char option[3]) ; 

extern float DistNoe(int mod, int residue1, char Id1[2], char atom1[6], int residue2, char Id2[2], char atom2[6], 
		      FILE *DumpedNoes, char Average[10]);
extern float DihedralResModel(int modeln, int residue, char Angle[4], char ResName[5]);

static short int i,j,k ;
 
/* ========================================================================= */
void copy(char to[], char from[], int inicio, int final)
/*
  ---
  --- Little utility to work in string processing. It was taken from the
  --- original C book of Kernigan & Ritchie. 
  ---
  --- Please NOTE that as it is written we need to start the copying one
  --- character before. It needs to be fixed at certain time ....
  ---
  --- (Roberto Tejero, RTT, May 1993, CABM-Rutgers University).
  ---
*/
{
     k = 0;
     i = inicio;
     while (i <= final) {
	  to[k] = from[i];
	  i++;
	  k++;
	  if ((k>1) && ((from[i] == '\n') || (from[i] == '\0'))) break ;
     }
     to[k] = '\0';
     return ;
}


/* ========================================================================= */
int longitud(char s[])
/*
  ---
  --- Another little utility. It was used while I was learning C. There are
  --- better functions in the standard library.
  ---
  --- (Roberto Tejero, RTT, May 1993, CABM-Rutgers University).
  ---
*/
{
     i=0;
     while (s[i] != '\0')
	  ++i;
     return i;
}

/* ========================================================================= */
int GetLine(char s[],int limite,FILE *rtti)
/*
  ---
  --- Little utility taken from the Kernigan & Ritchie C book. It was 
  --- implemented in the code to allow working with PDB format.
  ---
  --- (Roberto Tejero, RTT, May 1993, CABM-Rutgers University).
  ---
*/
{
     static int c;
     c = 0;
     for (i = 0; i < limite && (c = fgetc(rtti)) != EOF && c != '\n'; ++i)
	  s[i]=c;
     if (c == '\n')  {
	  s[i] = c;
	  ++i;
     } 
     s[i] = '\0';
     /* if ( c == EOF ) i = 9999 ; */
     return i;
}

/* ========================================================================= */
int locate_file(char nombre[MAXWORD])
 /*
  ---
  ---
  --- (Roberto Tejero, RTT, May 1993, CABM-Rutgers University).
  ---
  */
{
     static int entero;
     
     if ( (strcmp(nombre,"")==0) || strcasecmp(nombre,"unknown")==0 ) {
	  printf("> LOCATE_file:  Input file  name ? : ");
	  GetFromKeyBoard(entrada,MAXWORD,stdin);
     } else
	  strcpy(entrada,nombre);
     
     if (strncmp(entrada,"exit",4)==0)  {
	  strcat(history[count_history]," *** exit *** ");
	  return 0;
     }
     strcat(history[count_history],entrada);
     
     /*
      * Opening the file and checking if it is compressed or not
      */
     if ((fpi=fopen(entrada,"r")) == NULL) {
	  perror("\n> LOCATE_file: *Error* opening input file");
	  entero = 0;
     } else {
	  printf("> LOCATE_file: file `%s' opened for reading \n", entrada);
	  
	  /*
	   * A little "make up" with the file name.
	   * It throws 'path' information up to the file, i.e. it deletes
	   * every / 
	   *
	   */
	  strcpy(name_of_the_file,entrada);
	  i = 0;
	  for (j = (int) strlen(name_of_the_file) ; j >= 0 ; j--)
	       if (name_of_the_file[j] == '/') break;
	  for (k = j + 1 ; k <= strlen(name_of_the_file) ; k++) {
	       name_of_the_file[i] = name_of_the_file[k];
	       i++;
	  }
	  name_of_the_file[i] = '\0';
	  entero = 1;
     }
     return entero;
}

/* ========================================================================= */
void do_lower_case(char target[MAXWORD])
/*
  ---
  ---
  --- (Roberto Tejero, RTT, June 1993, CABM-Rutgers University).
  ---
*/
{
     for (k = 0; k <= strlen(target) ; k++) {
	  target[k] = tolower(target[k]);
     }
     return ;
}


/* ========================================================================= */
void do_copy_model(int from, int to)
 /*
  * ---
  * --- Copying model from one place to another,
  * --- 
  * ---
  * --- (Roberto Tejero, RTT, July 1993, CABM-Rutgers University).
  * ---
  */
{
     static short k;
     static struct models *To, *From;
     
     /* if (to == total_models + 1) total_models++  ; */
     
     if (DEBUG >= 1) 
	  printf(" > util - do_copy_model(): Copying from model %4d to model %4d\n", from, to);
     
     To = MODELS+to ;
     From = MODELS+from ;
     
     To->Selected = From->Selected ;
     To->ResiduesSelected = From->ResiduesSelected ;
     To->AtomsSelected = From->AtomsSelected ;
     strcpy(To->family,From->family) ;
     To->energy = From->energy ;
     To->rmsd = From->rmsd ;
     To->RadiusOfGyration = From->RadiusOfGyration ;
     To->written = From->written ;
     To->determ = From->determ ;
     To->BadOmega = From->BadOmega ;
     To->BadChiral = From->BadChiral ;
     To->BadSideChain = From->BadSideChain ;
     To->BadRama = From->BadRama ;
     To->BadModel = From->BadModel ;
     To->Cis = From->Cis ;
     To->modelno = From->modelno ;
     To->SerialNumber = From->SerialNumber ;
     To->number_of_residues = From->number_of_residues ;
     To->starting_res = From->starting_res ;
     To->number_of_atoms = From->number_of_atoms ;
     To->x_centermass = From->x_centermass ;
     To->y_centermass = From->y_centermass ;
     To->z_centermass = From->z_centermass ;
     To->Ixx          = From->Ixx ;
     To->Iyy          = From->Iyy ;
     To->Izz          = From->Izz ;
     To->Ixy          = From->Ixy ;
     To->Ixz          = From->Ixz ;
     To->Iyz          = From->Iyz ;
     To->PrincipalAxisFrame[0] = From->PrincipalAxisFrame[0] ;
     To->PrincipalAxisFrame[1] = From->PrincipalAxisFrame[1] ;
     To->PrincipalAxisFrame[2] = From->PrincipalAxisFrame[2] ;
     
     for (k = 1; k <= From->number_of_residues ; k++) {
	  To->DIH[k].Selected = From->DIH[k].Selected ;
	  To->DIH[k].BadResidue = From->DIH[k].BadResidue ;
	  To->DIH[k].number_of_atoms = From->DIH[k].number_of_atoms ;
	  To->DIH[k].orden     = From->DIH[k].orden ;
	  To->DIH[k].resnumber = From->DIH[k].resnumber ;
	  To->DIH[k].resnumber_orig = From->DIH[k].resnumber_orig ;
	  strcpy(To->DIH[k].resname,From->DIH[k].resname) ;
	  strcpy(To->DIH[k].thrletter,From->DIH[k].thrletter) ;
	  strcpy(To->DIH[k].oneletter,From->DIH[k].oneletter) ;
	  strcpy(To->DIH[k].fullname,From->DIH[k].fullname) ;
	  strcpy(To->DIH[k].chain_id,From->DIH[k].chain_id) ;
	  To->DIH[k].omega     = From->DIH[k].omega ;
	  To->DIH[k].phi       = From->DIH[k].phi ;
	  To->DIH[k].psi       = From->DIH[k].psi ;
	  To->DIH[k].chi1      = From->DIH[k].chi1 ;
	  To->DIH[k].planar_o  = From->DIH[k].planar_o ;
	  To->DIH[k].planar_h  = From->DIH[k].planar_h ;
	  To->DIH[k].chiral    = From->DIH[k].chiral ;
	  To->DIH[k].chiral_sc = From->DIH[k].chiral_sc ;
     }
     
     for (k = 1; k <= From->number_of_atoms ; k++) {
	  To->PDB[k].TruePdb = From->PDB[k].TruePdb;
	  To->PDB[k].Selected = From->PDB[k].Selected;
	  strcpy(To->PDB[k].pdb_key,From->PDB[k].pdb_key);
	  To->PDB[k].written = From->PDB[k].written;
	  To->PDB[k].pdb_num = From->PDB[k].pdb_num;
	  strcpy(To->PDB[k].pdb_iupac,    From->PDB[k].pdb_iupac);
	  strcpy(To->PDB[k].pdb_res,      From->PDB[k].pdb_res);
	  strcpy(To->PDB[k].segid, From->PDB[k].segid);
	  strcpy(To->PDB[k].pdb_chain_id, From->PDB[k].pdb_chain_id);
	  strcpy(To->PDB[k].chain, From->PDB[k].chain);
	  strcpy(To->PDB[k].type, From->PDB[k].type);
	  strcpy(To->PDB[k].AtomType, From->PDB[k].AtomType);
	  To->PDB[k].pdb_num_res = From->PDB[k].pdb_num_res;
	  To->PDB[k].pdb_num_res_orig = From->PDB[k].pdb_num_res_orig;
	  To->PDB[k].orden    = From->PDB[k].orden ;
	  To->PDB[k].pdb_xcrd    = From->PDB[k].pdb_xcrd ;
	  To->PDB[k].pdb_ycrd    = From->PDB[k].pdb_ycrd ;
	  To->PDB[k].pdb_zcrd    = From->PDB[k].pdb_zcrd ;
	  To->PDB[k].pdb_oldx    = From->PDB[k].pdb_oldx ;
	  To->PDB[k].pdb_oldy    = From->PDB[k].pdb_oldy ;
	  To->PDB[k].pdb_oldz    = From->PDB[k].pdb_oldz ;
	  To->PDB[k].coord[0]    = From->PDB[k].coord[0] ;
	  To->PDB[k].coord[1]    = From->PDB[k].coord[1] ;
	  To->PDB[k].coord[2]    = From->PDB[k].coord[2] ;
	  To->PDB[k].pdb_data1   = From->PDB[k].pdb_data1;
	  To->PDB[k].pdb_data2   = From->PDB[k].pdb_data2;
	  To->PDB[k].pdb_masa    = From->PDB[k].pdb_masa;
     }

     k = 0;
     while ( strncmp(From->HeaderPdbModel[++k],"STOP",4) != 0 ) {
         if ( k >= MAXREMARKMODRECS-2 ) {
               fprintf(stdout, "\n .. ** do_copy_model(): Remark capacity exhausted for model, %d!!! \n",
                             From->modelno);
               fprintf(stdout,"    ** Although program will continue working fine (I hope) it is advisable\n");
               fprintf(stdout,"    ** to take Action:  Please, increase MAXREMARKMODRECS and recompile ** \n");
               fprintf(stdout,"    ** Anyway!, Taking only first %d REMARK records and continuing ** \n\n",
                                       MAXREMARKMODRECS-1);
       	       break ;
         }
         strcpy(To->HeaderPdbModel[k], From->HeaderPdbModel[k]) ;
     }
     strcpy(To->HeaderPdbModel[k+1],"STOP");
	                                                                                                                                                     
     return;
}


/* ========================================================================= */
/*
  ---
  --- Code for generating PS files for RAMACHANDRAN plots
  --- It has been extracted from code for phipsi utility in ORIENT
  --- done by George Elkins at CABM-Rutgers University.
  ---
  --- (Roberto Tejero, RTT, Oct 1993, Valencia University).
  ---
*/

FILE *efopen(char *, char *);
FILE *libfopen(char *, char *);
/* PostScript drawing functions */
void ps_drawgraph(FILE *);
void ps_initgraph(FILE *);
void ps_fileinit(FILE *);

/* ========================================================================= */
void ps_fileinit(FILE *outfile)
/* void ps_fileinit(FILE *outfile, char *filestr) */
{
  static FILE *psp; int c;
  
  /* write encapsulated PostScript header to output file */
  fprintf(outfile, "%%!PS-Adobe-1.0\n");
  fprintf(outfile, "%%%%Title: (Ramachandran Plot of file %s)\n", name_of_the_file); 
  fprintf(outfile, "%%%%Creator: %s\n",name_of_the_file); 
  /*  fprintf(outfile, "%%%%CreationDate: %s", gettime()); */
  fprintf(outfile, "%%%%DocumentNeededResources:font Helvetica Symbol\n");
  fprintf(outfile, "%%%%For: %s\n", getlogin());  
  fprintf(outfile, "%%%%BoundingBox: 10 10 610 610\n");
  fprintf(outfile, "%%%%Page: 1 1 \n");
  fprintf(outfile, "%%%%EndComments\n");
  
  /* copy prologue from psdefs.ps to output file */
  psp = libfopen("psdefs.ps", "r");
  while ((c = getc(psp)) != EOF) (void) putc(c, outfile);
  
  /* write data minima, maxima to PostScript file */
  fprintf(outfile, "/xmin {-180} def /xmax {180} def\n");
  fprintf(outfile, "/ymin {-180} def /ymax {180} def\n");
  
  fprintf (outfile,
     "/drawdot {(\\267) stringwidth pop -1 mul dup 2 div exch rmoveto (\\267) S } def\n");

  fprintf(outfile, "%%%%EndProlog\n");
  fclose(psp);
  return;
}

/* ========================================================================= */
void ps_drawgraph(FILE *outfile)
{
  static int      resindex, wanted_from, wanted_to;
  static char     select_resname[MAXWORD], wanted_range[MAXWORD],
                  title_ps[MAXWORD] , select_symbol[MAXWORD];
  static char     resname_temp[4];
  static struct   models *model ;

  ps_fileinit(outfile);
  ps_initgraph(outfile);
  
  printf("> RAMA: From model: space to model: _ [example: 1 1] : ?_ ");
  GetFromKeyBoard(wanted_range,MAXWORD,stdin);

  sscanf(wanted_range,"%d %d",&wanted_from, &wanted_to);
  if (wanted_to < wanted_from) {
    wanted_from = wanted_to ;
    wanted_to = wanted_from ;
  }
  if (wanted_from > total_models) wanted_from = 1;
  if (wanted_from <= 0)           wanted_from = 1;
  if (wanted_to >= total_models ) wanted_to = total_models ;
  if (wanted_to <= 0) wanted_to = total_models ;
  printf("> RAMA: Building Ramachandran plot from model %d to model %d \n",
	 wanted_from, wanted_to);
  
  printf("> RAMA: Title for PostScript drawing : ?_ ");
  GetFromKeyBoard(title_ps,MAXWORD,stdin);
  if (strcmp(title_ps,"")==0) strcpy(title_ps,name_of_the_file);

  printf("> RAMA: Select symbol for plots [resname, resnumber or model]:?_ ");
  GetFromKeyBoard(select_symbol, MAXWORD, stdin);
  do_lower_case(select_symbol);

  printf("> RAMA: Select residues (ALA, PRO, ... or all) :?_ ");
  GetFromKeyBoard(select_resname,MAXWORD,stdin);
  do_lower_case(select_resname);

  fprintf(outfile, "definemedfont\n");
  
  for (key_model = wanted_from; key_model <= wanted_to; key_model++) {
    model = MODELS+key_model;
    for (resindex = 2; 
	 resindex <= model->number_of_residues -1 ; 
	 resindex++) {
      copy(resname_temp,model->DIH[resindex].resname,0,2);
      do_lower_case(resname_temp);
      if ((strncmp(select_resname,resname_temp,3)==0) ||
	  ((strncmp(select_resname,"all",2)==0) && (strncmp(select_resname,"gly",2)!=0))) {
	if (strncmp(select_symbol,"resnum",5)==0)
	  fprintf(outfile, "%.2f %.2f M (%d) S\n",
		  model->DIH[resindex].phi, model->DIH[resindex].psi, resindex);
	if (strncmp(select_symbol,"resnam",5)==0)
	  fprintf(outfile, "%.2f %.2f M (%s) S\n",
		  model->DIH[resindex].phi, model->DIH[resindex].psi, model->DIH[resindex].resname);
      }
      if ((strncmp(select_symbol,"stru",2)==0) || (strncmp(select_symbol,"mod",2)==0))
	fprintf(outfile, "%.2f %.2f M (%d) S\n",
		model->DIH[resindex].phi, model->DIH[resindex].psi, model->modelno);
    }
  }
  fprintf(outfile, "definemedlargefont\n");
  fprintf(outfile, "-110 290 moveto\n");
  fprintf(outfile, "(File: %s) show\n", title_ps);
  fprintf(outfile, "-110 280 moveto\n");
  fprintf(outfile, "(Residue Range: %d to %d ) show\n",
          resindex = 2, MODELS[total_models].number_of_residues-1);
  /*  fprintf(outfile, "stroke\nshowpage\nrestore\n"); */
  fprintf(outfile, "stroke\nshowpage\n"); 
  fprintf(outfile, "%%%%Trailer\n");
  fprintf(outfile, "%%%%EOF\n");
  (void) fclose(outfile);
  return ;
}

/* ========================================================================= */
void ps_initgraph(FILE *f)
{

     /* prints the outline of the graph */
     fprintf(f, "95.0 DoScaleTranslate\n");
     fprintf(f, "newpath\n");
     fprintf(f, "definelargefont\n");
     fprintf(f, "plotframe\n");
     fprintf(f, "(Dihedral Angles) (degrees) (degrees) textlabels\n");
     
     /* X and Y axis scale and grid lines */
     for (i = -135; i < 180 ; i = i + 45)
     {
	  fprintf(f, "%d XGL\n", i);
	  fprintf(f, "%d 2.0 Xtick\n", i);
	  fprintf(f, "%d YGL\n", i);
	  fprintf(f, "%d 2.0 Ytick\n", i);
     }
     for (i = -180; i <= 180 ; i = i + 90)
     {
	  fprintf(f, "%d (%d) Xnumber\n", i,i);
	  fprintf(f, "%d 3.0 Xtick\n", i);
	  fprintf(f, "%d (%d) Ynumber\n", i,i);
	  fprintf(f, "%d 3.0 Ytick\n", i);
     }
     fprintf(f, "stroke\nnewpath\n");
     return;
}

/* ========================================================================= */
/* 
 * Try to safely open a file for writing. First it checks for any
 * existing file with the same name and if it exist add some numerical
 * extension to the name of the file
 */
FILE *safeopen(char *file, char *mode, int longit)    
{  
     static FILE *filepp;
     static char TestName[MAXWORD], Extension[5];
     static short int CountExt=0;
     
     /*
      * longit is _NOT_ really used. It is kept for compatibility reasons 
      * I felt really lazy to change the calls the routine avoiding longit
      */
     CountExt = 0 ;
     if ( longit > 100 ) 
	  fprintf(stdout," Long File Name \n");
     
     if ( DEBUG >= 1 ) 
	  fprintf(stdout," > util.c: safeopen(): Copying name \n");
     
     strcpy(TestName,file);
     
     if ( DEBUG >= 1 ) 
	  fprintf(stdout," > util.c: safeopen(): Checking file name `%s' \n", TestName);
     
     if (strcmp(mode,"w")==0) {
	  while ((filepp = fopen(TestName,"r")) != (FILE *) NULL) {
	       CountExt++ ;
	       if (CountExt > 100) break;
	       Extension[0]='\0';
	       if ( CountExt < 10) 
		    sprintf(Extension,"00%d",CountExt);
	       else  if (( CountExt > 10 ) && (CountExt < 100))
		    sprintf(Extension,"0%d",CountExt);
	       else
		    sprintf(Extension,"%d",CountExt);
	       strcpy(TestName,file);
	       strcat(TestName,Extension);
	       fclose(filepp);
	       if (DEBUG >= 1)  
		    fprintf(stdout," > util.c: safeopen(): .. Trying file name `%s'\n", TestName);
	       fflush(stdout) ;
	  }
     }  
     
     if ((filepp = fopen(TestName,mode)) == (FILE *) NULL) {
	  perror("> util.c: safeopen(): *** Can't open file");
	  exit(-1); 
     }
     fflush(stdout);
     return filepp;
}

/* ========================================================================= */
/*
 * Fopen a file, if it can't ask the user for a new name.
 * If the user types 'exit' -> die.
 */
FILE *efopen(char *file, char *mode)       /* fopen file, die if can't */
{
     FILE *fp;
     
     while ((fp = fopen(file,mode)) == (FILE *) NULL) {
	  perror(" ** -> efopen(): can't open file");
	  fprintf(stderr," efopen(): Name for input file :_ ");
	  GetFromKeyBoard(file, MAXWORD, stdin);
	  if (strncasecmp(file,"exit",2)==0) exit(-1);
     }
     return fp;
}

/* ========================================================================= */
/*
 * Fopen a file, if it can't ask the user for a new name.
 * If the user types 'exit' -> die.
 */

FILE *libfopen(char *file, char *mode)       /* fopen file, die if can't */
{
     static char *filename, *src, *dst;
     static char filebuf[100];
     FILE *fp=NULL;
     
     filename = dst = filebuf ;
     filename = file ;
     
     if (strcmp(mode,"r")!=0) fprintf(stderr," Mode not allowed for Library file/s \n");
     
     /* If it fails try to read AyudaPATH environment variable */
     if ( !fp && (src=(char*)getenv("AyudaPATH")) ) {
	  filename = dst = filebuf ;
	  while( *src ) 
	       *dst++ = *src++ ;
	  if ( (dst!=filebuf) && (*(dst-1) != '/') ) 
	       *dst++ = '/' ;
	  src = file ;
	  while ( (*dst++ = *src++) );
	  
	  if (DEBUG >= 1) fprintf(stdout," > util-libfopen():[2] Trying to open '%s'\n", filename);
	  
	  /* if everything fails tell the user to set environment variable and die gracefully */
	  if ( (fp = fopen(filename,mode)) == (FILE *) NULL ) { 
	       fprintf(stderr,"\n ** ERROR **: ") ; perror(filename);
	       fprintf(stderr," ** ERROR **: Either the file does not exist or you don't have read permission \n");
	       fprintf(stderr," ** ERROR **: Please, check permissions and existence of file  and re-run      \n");
	       fprintf(stderr," ** ERROR **: also check that AyudaPATH environment variable is properly set.\n\n");
	       fprintf(stderr," ** ERROR **: EXITING, right now .....................................       \n\n");
	       exit(-1);
	  } 
     }
     return fp;
}

/* ========================================================================= */
/*
 * Checks if model in memory has atoms 
 */
char ModelHasAtoms(int ModelN)
{
     static struct models *Model ;
     Model = MODELS + ModelN ;
     if (Model->number_of_atoms <= 0)
	  return FALSE ;
     else
	  return TRUE;
}

/* ========================================================================= */
/*
 * Checks if we have a model in memory
 */
char IsThereAModel(void)
{
     if (total_models <= 0) 
	  return FALSE;
     else
	  return TRUE;
}

/* ========================================================================= */
/*
 * Checks if we have read a constraint file
 */
char IsThereConstraints(void)
{
     if (constraintcount <= 0) 
	  return FALSE ;
     else
	  return TRUE ;
}

/* ========================================================================= */
/*
 * Checks if we have read a sequence file or if we have sequence data in
 * memory 
 */
char IsThereSequence(void)
{
     static struct models *Model ;
     
     Model = MODELS + key_model ;
     if (Model->number_of_residues <= 0) 
	  return FALSE ;
     else
	  return TRUE ;
}



/* ========================================================================= */
void Copyright_Notice(void) 
{
     printf("\n (C)");
     printf("\n (C) Pdbstat -- A program to evaluate some statistics of macromolecules,"); 
     printf("\n (C)            given the cartesian coordinates." );
     printf("\n (C)  "); 
     printf("\n (C) (c) Copyright 1992-2003 Roberto Tejero and Gaetano T. Montelione");
     printf("\n (C)     Center for Advanced Technology and Medicine (CABM)");
     printf("\n (C)     Rutgers University");
     printf("\n (C)");
}

/* ========================================================================= */
/*
 *
 * A incluir. Todavia en desarrollo.
 * Existe un shell en UTILITIES que hace lo mismo que aqui se pretende 
 *

void edit_secondary_structure(char[10] tipo, int inicial, int final, FILE *f) 
{
  int phi, psi ;

  switch (tipo)
    {
    case beta:
    case hoja:
    case sheet:
      phi=-119 ;
      psi=113 ;
      break ;
    case helix:
    case alpha:
    case helice:
      phi=-47;
      psi=-57;
      break;
    case anti:
      phi=-139;
      psi=135;
      break;
    default:
      fprintf(stderr, " EDIT_SCND: Secondary structure not recognized \n");
      return ;
      break;
    }
  fprintf(f," IC EDIT \n");
  for ( i = inicial ; i < final ; i++ ) 
    {
      fprintf(f," DIHE %3d N %3d CA %3d C  %3d N %5d \n", 
	      i, i, i, i+1, phi);
      fprintf(f," DIHE %3d C %3d C  %3d CA %3d C %5d \n", 
	      i, i+1, i+1, i+1, psi);
    }
  fprintf(f," END \n");
  return ;
}

*********************************************************************/

struct Library    *Lib ;
struct DiheLib    *DihLib ;
int  MaxLib = -9999 ;       /* Total number of records in Lib file */
extern int DihLibRecords ;

/* ========================================================================= */
/*
 * Once a library file is opened for reading this routine reads that file
 * and dinamically allocates space to store the Library data, it returns
 * the number of records found in the library file. Everything in that
 * file after symbol '#' is taken as a comment and has no meaning for this
 * routine.
 */
int GetLibraryData(FILE *InputFile)
{
     /* Reading Library atom names data */
     static int   DefinitionNumber=0, DihedralNumber=0 ;
     static char  InputLine[MAXLINE], MultiName[12] ;
     static char  Key[10], Multiple = FALSE;
     static short l=0, il=0;
     
     DefinitionNumber = 0 ;
     DihedralNumber = 0;
     
     for (il=0;il<12;il++)
	  MultiName[il]='\0';
     
     for (l=0;l<MAXLINE;l++) InputLine[l] = '\0';
     
     DihLib = (struct DiheLib *) malloc(sizeof(struct DiheLib));
     Lib = (struct Library *) malloc(sizeof(struct Library));
     
     if ( DihLibRecords != 0 ) {
	  fprintf(stdout,"\n ** Warning, DihLibRecords !=0, already read?? \n");
	  return MaxLib;
     }
     
     if (DEBUG >= 1) 
	  printf(" > util - GetLibraryData(): Allocating space for Library\n");
     
     while (fgets(InputLine,MAXLINE,InputFile) != NULL) {
	  if ((InputLine[0] != '#') && (strlen(InputLine) > 1)) {
	       sscanf(InputLine,"%s",Key);
	       /* Dihedral data */
	       if (strncasecmp(Key,"DIH",3)==0) {
		    DihedralNumber++ ;
		    DihLib = (struct DiheLib *) realloc(DihLib,sizeof(struct DiheLib)*(DihedralNumber+1));
		    
		    (DihLib+DihedralNumber)->Number = DihedralNumber ;
		    sscanf(InputLine,"%*s %*s %s %*s %s %s %s %s %s\n",
			   (DihLib+DihedralNumber)->ResidueName, (DihLib+DihedralNumber)->DihName,
			   (DihLib+DihedralNumber)->Atom1, (DihLib+DihedralNumber)->Atom2,
			   (DihLib+DihedralNumber)->Atom3, (DihLib+DihedralNumber)->Atom4);
		    
		    (DihLib+DihedralNumber)->next = (DihLib+DihedralNumber) ;
	       } else if (strncasecmp(Key,"ATOM",4)==0 || strncasecmp(Key,"NMR",3)==0 ) {
		    /* Either ATOM or NOE data */
		    DefinitionNumber++ ;
		    Lib = (struct Library *) realloc(Lib,sizeof(struct Library)*(DefinitionNumber+1));
		    (Lib+DefinitionNumber)->Number = DefinitionNumber ;
		    for (il = 0;il < 12; il++)
			 MultiName[il]='\0';
		    Multiple = FALSE ;
		    /* when record has multiple names */
		    sscanf(InputLine,"%*s %*s %*s %s\n", MultiName);
		    il = 0;
		    while ( MultiName[il] != '\0' ) {
			 if ( MultiName[il] == '=' ) {
			      Multiple = TRUE ;
			      break;
			 }
			 il++;
		    } 
		    if ( Multiple == TRUE ) {
			 copy((Lib+DefinitionNumber)->DyanaName,MultiName,0,il-1);
			 copy((Lib+DefinitionNumber)->DianaName,MultiName,il+1,(int)strlen(MultiName));
			 copy((Lib+DefinitionNumber)->CyanaName,MultiName,il+1,(int)strlen(MultiName));
		    } else {
			 copy((Lib+DefinitionNumber)->DianaName,MultiName,0,(int)strlen(MultiName));
			 copy((Lib+DefinitionNumber)->DyanaName,MultiName,0,(int)strlen(MultiName));
			 copy((Lib+DefinitionNumber)->CyanaName,MultiName,0,(int)strlen(MultiName));
		    }
		    
		    sscanf(InputLine,"%s %*s %s %*s %s %s %s %s %s \n",
			   (Lib+DefinitionNumber)->Descriptor,
			   (Lib+DefinitionNumber)->ResidueName,
			   (Lib+DefinitionNumber)->CongenName,
			   (Lib+DefinitionNumber)->XPlorName,
			   (Lib+DefinitionNumber)->DismanName,
			   (Lib+DefinitionNumber)->PdbName,
			   (Lib+DefinitionNumber)->EceppName);
		    (Lib+DefinitionNumber)->next = (Lib+DefinitionNumber) ;
	       } else {
		    fprintf(stdout,"\n  ..... GetLibraryData(): meaningless record in Lib `%s' \n", InputLine);
	       }
	  }
     }
     MaxLib = DefinitionNumber ;
     DihLibRecords = DihedralNumber ;
     if (DEBUG >= 1) 
	  printf(" > util - GetLibraryData(): %d Records read\n", DefinitionNumber);
     return (MaxLib) ;
}


/* ========================================================================= */
/*
 * Little utility to convert from one letter amino code to three letter
 * code. 
 */
void OneToThree2(char Name[5])
{
     switch (Name[0]) {
     case 'A' :
	  strcpy(Name,"ALA") ;
	  break ;
     case 'R' :
	  strcpy(Name,"ARG");
	  break ;
     case 'N' : 
	  strcpy(Name,"ASN");
	  break ;
     case 'D' :  
	  strcpy(Name,"ASP");
	  break ;
     case 'C' :  
	  strcpy(Name,"CYS");
	  break ;
     case 'E' :  
	  strcpy(Name,"GLU");
	  break ;
     case 'Q' :  
	  strcpy(Name,"GLN");
	  break ;
     case 'G' :  
	  strcpy(Name,"GLY") ;
	  break ;
     case 'H' :
	  strcpy(Name,"HIS") ;         
	  break ;
     case 'I' : 
	  strcpy(Name,"ILE");
	  break ;
     case 'L' :   
	  strcpy(Name,"LEU") ;         
	  break ;
     case 'K' : 
	  strcpy(Name,"LYS") ;
	  break ;
     case 'M' :   
	  strcpy(Name,"MET") ;         
	  break ;
     case 'F' :
	  strcpy(Name,"PHE") ;
	  break ;
     case 'P' :  
	  strcpy(Name,"PRO") ;         
	  break ; 
     case 'S' :
	  strcpy(Name,"SER") ;
	  break ;
     case 'T' :  
	  strcpy(Name,"THR") ;         
	  break ;
     case 'W' :   
	  strcpy(Name,"TRP") ;
	  break ;
     case 'Y' : 
	  strcpy(Name,"TYR") ;         
	  break ;
     case 'V' : 
	  strcpy(Name,"VAL") ;
	  break ;
     case 'U' :               /* special name for unknown residues */
	  strcpy(Name,"UUU") ;
	  break ;
     default :
	  fprintf(stderr, "\n util - OneToThree2(): ** Residue Name `%s' NOT recognized \n", 
		  Name);
	  exit (-1) ;
	  break ;
     } 
     
     return ;
}

/* ========================================================================= */
int DihToNames(char Residue[5], char Dih[5]) {
     static struct DiheLib *Reg ;
     Reg = &DihLib[1] ;
     while (Reg->next != NULL) {
	  if ( (strcasecmp(Reg->ResidueName,Residue)==0 || strcasecmp(Reg->ResidueName,"ANY")==0) &&
	       strcasecmp(Reg->DihName,Dih)==0 )
	       break ;
	  Reg++;
     }
     if (Reg->Number > DihLibRecords) {
	  fprintf(stderr," Ooooops [%-5s %-5s] no dihedral found, aplying UNKN \n", Residue, Dih);
	  return DihLibRecords;
     }
     return (Reg->Number);
}


/* ========================================================================= */
int NamesToDih(char Residue[5], char At1[6], char At2[6], char At3[6], char At4[6]) {
     static struct DiheLib *Reg ;
     Reg = &DihLib[1] ;
     while ( Reg->next != NULL ) {
	  if ( (strcasecmp(Reg->ResidueName,Residue)==0 || strcasecmp(Reg->ResidueName,"ANY")==0) &&
	       strcasecmp(Reg->Atom1,At1)==0 && strcasecmp(Reg->Atom2,At2)==0 && 
	       strcasecmp(Reg->Atom3,At3)==0 && strcasecmp(Reg->Atom4,At4)==0 ) 
	       break ;
	  Reg++;
     }
     if (Reg->Number > DihLibRecords) {
	  fprintf(stderr," Ooooops [%-5s %-6s %-6s %-6s %-6s] no dihedral found, aplying UNKN \n",
		  Residue, At1, At2, At3, At4);
	  return DihLibRecords;
     }
     return (Reg->Number);
}


/* ========================================================================= */
/*
 * Assigns Library names to each atom in an NOE file
 * It assigns different possible names DISMAN, DIANA, X-PLOR, CONGEN
 * found in the Library (GenCons.Lib). PDB names are not used to
 * assign to NOEs.
 */
void AssignLibraryNames(struct noe *Noe, int LibRecords, int option, char KeyAss[5])
{
     static struct Library *Record ;
     static char IsOneAssigned = FALSE, IsTwoAssigned = FALSE;
     static char TempRes1[5], TempRes2[5];
     
     Record = &Lib[1] ;
     IsOneAssigned = FALSE ;
     IsTwoAssigned = FALSE ;
     
     if (DEBUG >= 2) 
	  if ( !(Noe->Number % 100) )
	       printf(" > util(AssignLibraryNames): Assigning Noe %6d \n", Noe->Number);
     
     while (Record->next != NULL) {
	  if ( !HomolConstraints && strcmp(Record->Descriptor,KeyAss) != 0) {
	       Record++ ;
	       continue ;  
	  }
	  /* First atom in  NOE */
	  strcpy(TempRes1,Noe->residue_type1);
	  if ( Noe->LastRes1 ) {
	       if (strcasecmp(Noe->atom_type1,"O")==0) strcpy(TempRes1,"LST") ;
	  }
	  if ((strcasecmp(TempRes1,Record->ResidueName)==0 || strcmp(Record->ResidueName,"ANY")==0 )
	      && !IsOneAssigned) {
	       if ((option == 1 ) && (strcasecmp(Noe->atom_type1,Record->CongenName)==0)){
		    strcpy(Noe->Atom1Ecepp,Record->EceppName);
		    strcpy(Noe->Atom1Congen,Record->CongenName);
		    strcpy(Noe->Atom1Dyana,Record->DyanaName);
		    strcpy(Noe->Atom1Cyana,Record->CyanaName);
		    strcpy(Noe->Atom1Diana,Record->DianaName);
		    strcpy(Noe->Atom1Disman,Record->DismanName);
		    strcpy(Noe->Atom1Xplor,Record->XPlorName);
		    strcpy(Noe->Atom1Pdb,Record->PdbName);
		    IsOneAssigned = TRUE ;
	       } else if ((option == 2) && (strcasecmp(Noe->atom_type1,Record->DismanName)==0)) {
		    strcpy(Noe->Atom1Ecepp,Record->EceppName);
		    strcpy(Noe->Atom1Congen,Record->CongenName);
		    strcpy(Noe->Atom1Dyana,Record->DyanaName);
		    strcpy(Noe->Atom1Cyana,Record->CyanaName);
		    strcpy(Noe->Atom1Diana,Record->DianaName);
		    strcpy(Noe->Atom1Disman,Record->DismanName);
		    strcpy(Noe->Atom1Xplor,Record->XPlorName);
		    strcpy(Noe->Atom1Pdb,Record->PdbName);
		    IsOneAssigned = TRUE ;
	       } else if ((option == 3) && (strcasecmp(Noe->atom_type1,Record->DyanaName)==0)) {
		    strcpy(Noe->Atom1Ecepp,Record->EceppName);
		    strcpy(Noe->Atom1Congen,Record->CongenName);
		    strcpy(Noe->Atom1Dyana,Record->DyanaName);
		    strcpy(Noe->Atom1Cyana,Record->CyanaName);
		    strcpy(Noe->Atom1Diana,Record->DianaName);
		    strcpy(Noe->Atom1Disman,Record->DismanName);
		    strcpy(Noe->Atom1Xplor,Record->XPlorName);
		    strcpy(Noe->Atom1Pdb,Record->PdbName);
		    IsOneAssigned = TRUE ;
	       } else if ((option == 300) && (strcasecmp(Noe->atom_type1,Record->DianaName)==0)) {
		    strcpy(Noe->Atom1Ecepp,Record->EceppName);
		    strcpy(Noe->Atom1Congen,Record->CongenName);
		    strcpy(Noe->Atom1Dyana,Record->DyanaName);
		    strcpy(Noe->Atom1Cyana,Record->CyanaName);
		    strcpy(Noe->Atom1Diana,Record->DianaName);
		    strcpy(Noe->Atom1Disman,Record->DismanName);
		    strcpy(Noe->Atom1Xplor,Record->XPlorName);
		    strcpy(Noe->Atom1Pdb,Record->PdbName);
		    IsOneAssigned = TRUE ;
	       } else if ((option == 6) && (strcasecmp(Noe->atom_type1,Record->XPlorName)==0)) {
		    strcpy(Noe->Atom1Ecepp,Record->EceppName);
		    strcpy(Noe->Atom1Congen,Record->CongenName);
		    strcpy(Noe->Atom1Dyana,Record->DyanaName);
		    strcpy(Noe->Atom1Cyana,Record->CyanaName);
		    strcpy(Noe->Atom1Diana,Record->DianaName);
		    strcpy(Noe->Atom1Disman,Record->DismanName);
		    strcpy(Noe->Atom1Xplor,Record->XPlorName);
		    strcpy(Noe->Atom1Pdb,Record->PdbName);
		    IsOneAssigned = TRUE ;
	       } else if ((option == 8) && (strcasecmp(Noe->atom_type1,Record->EceppName)==0)) {
		    strcpy(Noe->Atom1Ecepp,Record->EceppName);
		    strcpy(Noe->Atom1Congen,Record->CongenName);
		    strcpy(Noe->Atom1Dyana,Record->DyanaName);
		    strcpy(Noe->Atom1Cyana,Record->CyanaName);
		    strcpy(Noe->Atom1Diana,Record->DianaName);
		    strcpy(Noe->Atom1Disman,Record->DismanName);
		    strcpy(Noe->Atom1Xplor,Record->XPlorName);
		    strcpy(Noe->Atom1Pdb,Record->PdbName);
		    IsOneAssigned = TRUE ;
	       } else if ((option == 9) && (strcasecmp(Noe->atom_type1,Record->PdbName)==0)) {
		    strcpy(Noe->Atom1Ecepp,Record->EceppName);
		    strcpy(Noe->Atom1Congen,Record->CongenName);
		    strcpy(Noe->Atom1Dyana,Record->DyanaName);
		    strcpy(Noe->Atom1Cyana,Record->CyanaName);
		    strcpy(Noe->Atom1Diana,Record->DianaName);
		    strcpy(Noe->Atom1Disman,Record->DismanName);
		    strcpy(Noe->Atom1Xplor,Record->XPlorName);
		    strcpy(Noe->Atom1Pdb,Record->PdbName);
		    IsOneAssigned = TRUE ;
	       }
	       /* for (i=(int)strlen(Noe->Atom1Pdb);i<=5;i++) Noe->Atom1Pdb[i]='\0'; */
	  }
	  
	  /* Second atom in  NOE */
	  strcpy(TempRes2,Noe->residue_type2);
	  if ( Noe->LastRes2 ) {
	       if (strcasecmp(Noe->atom_type2,"O")==0)
		    strcpy(TempRes2,"LST") ;
	  }
	  if ((strcasecmp(TempRes2,Record->ResidueName)==0 || strcmp(Record->ResidueName,"ANY")==0) 
	      && !IsTwoAssigned) {
	       if ((option == 1) && (strcasecmp(Noe->atom_type2,Record->CongenName)==0)) {
		    strcpy(Noe->Atom2Ecepp,Record->EceppName);
		    strcpy(Noe->Atom2Congen,Record->CongenName);
		    strcpy(Noe->Atom2Dyana,Record->DyanaName);
		    strcpy(Noe->Atom2Cyana,Record->CyanaName);
		    strcpy(Noe->Atom2Diana,Record->DianaName);
		    strcpy(Noe->Atom2Disman,Record->DismanName);
		    strcpy(Noe->Atom2Xplor,Record->XPlorName);
		    strcpy(Noe->Atom2Pdb,Record->PdbName);
		    IsTwoAssigned = TRUE;
	       } else if ((option == 2) && (strcasecmp(Noe->atom_type2,Record->DismanName)==0)) {
		    strcpy(Noe->Atom2Ecepp,Record->EceppName);
		    strcpy(Noe->Atom2Congen,Record->CongenName);
		    strcpy(Noe->Atom2Dyana,Record->DyanaName);
		    strcpy(Noe->Atom2Cyana,Record->CyanaName);
		    strcpy(Noe->Atom2Diana,Record->DianaName);
		    strcpy(Noe->Atom2Disman,Record->DismanName);
		    strcpy(Noe->Atom2Xplor,Record->XPlorName);
		    strcpy(Noe->Atom2Pdb,Record->PdbName);
		    IsTwoAssigned = TRUE;
	       } else if ((option == 3) && (strcasecmp(Noe->atom_type2,Record->DyanaName)==0)) {
		    strcpy(Noe->Atom2Ecepp,Record->EceppName);
		    strcpy(Noe->Atom2Congen,Record->CongenName);
		    strcpy(Noe->Atom2Dyana,Record->DyanaName);
		    strcpy(Noe->Atom2Cyana,Record->CyanaName);
		    strcpy(Noe->Atom2Diana,Record->DianaName);
		    strcpy(Noe->Atom2Disman,Record->DismanName);
		    strcpy(Noe->Atom2Xplor,Record->XPlorName);
		    strcpy(Noe->Atom2Pdb,Record->PdbName);
		    IsTwoAssigned = TRUE;
	       } else if ((option == 300) && (strcasecmp(Noe->atom_type2,Record->DianaName)==0)) {
		    strcpy(Noe->Atom2Ecepp,Record->EceppName);
		    strcpy(Noe->Atom2Congen,Record->CongenName);
		    strcpy(Noe->Atom2Dyana,Record->DyanaName);
		    strcpy(Noe->Atom2Cyana,Record->CyanaName);
		    strcpy(Noe->Atom2Diana,Record->DianaName);
		    strcpy(Noe->Atom2Disman,Record->DismanName);
		    strcpy(Noe->Atom2Xplor,Record->XPlorName);
		    strcpy(Noe->Atom2Pdb,Record->PdbName);
		    IsTwoAssigned = TRUE;
	       } else if ((option == 6) && (strcasecmp(Noe->atom_type2,Record->XPlorName)==0)) {
		    strcpy(Noe->Atom2Ecepp,Record->EceppName);
		    strcpy(Noe->Atom2Congen,Record->CongenName);
		    strcpy(Noe->Atom2Dyana,Record->DyanaName);
		    strcpy(Noe->Atom2Cyana,Record->CyanaName);
		    strcpy(Noe->Atom2Diana,Record->DianaName);
		    strcpy(Noe->Atom2Disman,Record->DismanName);
		    strcpy(Noe->Atom2Xplor,Record->XPlorName);
		    strcpy(Noe->Atom2Pdb,Record->PdbName);
		    IsTwoAssigned = TRUE;
	       } else if ((option == 8) && (strcasecmp(Noe->atom_type2,Record->EceppName)==0)) {
		    strcpy(Noe->Atom2Ecepp,Record->EceppName);
		    strcpy(Noe->Atom2Congen,Record->CongenName);
		    strcpy(Noe->Atom2Dyana,Record->DyanaName);
		    strcpy(Noe->Atom2Cyana,Record->CyanaName);
		    strcpy(Noe->Atom2Diana,Record->DianaName);
		    strcpy(Noe->Atom2Disman,Record->DismanName);
		    strcpy(Noe->Atom2Xplor,Record->XPlorName);
		    strcpy(Noe->Atom2Pdb,Record->PdbName);
		    IsTwoAssigned = TRUE;
	       } else if ((option == 9) && (strcasecmp(Noe->atom_type2,Record->PdbName)==0)) {
		    strcpy(Noe->Atom2Ecepp,Record->EceppName);
		    strcpy(Noe->Atom2Congen,Record->CongenName);
		    strcpy(Noe->Atom2Dyana,Record->DyanaName);
		    strcpy(Noe->Atom2Cyana,Record->CyanaName);
		    strcpy(Noe->Atom2Diana,Record->DianaName);
		    strcpy(Noe->Atom2Disman,Record->DismanName);
		    strcpy(Noe->Atom2Xplor,Record->XPlorName);
		    strcpy(Noe->Atom2Pdb,Record->PdbName);
		    IsTwoAssigned = TRUE;
	       }
	       /* for (i=(int)strlen(Noe->Atom2Pdb);i<=5;i++) Noe->Atom2Pdb[i]='\0'; */
	  }
	  
	  if (Record->Number == LibRecords) {
	       if (!IsOneAssigned || !IsTwoAssigned)  {
		    printf("\n AssignLibraryName(): NOE %5d -> %3s %3d '%-4s' %3s %3d '%-4s'",
			   Noe->Number,
			   Noe->residue_type1, Noe->residue_number1, Noe->atom_type1,
			   Noe->residue_type2, Noe->residue_number2, Noe->atom_type2);
		    if ( !IsOneAssigned )
			 printf(" * Atom 1 _NOT_ assigned");
		    if ( !IsTwoAssigned )
			 printf(" * Atom 2 _NOT_ assigned");
	       }
	       break;
	  }
	  Record++;	
     }
     AlreadyAssignedNoes = TRUE ;
     return ;
}

/* ========================================================================= */
/*
 * This routine assigns a residue name taken from seqres to each atom in an 
 * NOE file
 */
void AssignSequenceNameToNOE(void)
{
     static int h ; 
     if (DEBUG >= 1) 
	  printf(" > util: AssignSequenceNameToNOE(): ... assigning names\n");
     if (constraintcount > 1)
	  for (h = 1 ; h <= constraintcount; h++) {
	       strcpy(NOE[h].residue_type1,seqres[NOE[h].residue_number1]);
	       strcpy(NOE[h].residue_type2,seqres[NOE[h].residue_number2]);
	       
	       if ( NOE[h].residue_number1 == LastResidue ) 
		    NOE[h].LastRes1 = TRUE ;
	       else 
		    NOE[h].LastRes1 = FALSE ;
	       
	       if ( NOE[h].residue_number2 == LastResidue ) 
		    NOE[h].LastRes2 = TRUE ;
	       else
		    NOE[h].LastRes2 = FALSE ;
	  }
     
     return ;
}


/* ========================================================================= */
/*
 * This module checks that all atom names found in NOE file have a counter
 * part in Library file
 */
void CheckLibraryNames(int LibRecords,int option, int from) 
{
  static char   GoodAtom1=FALSE, GoodAtom2=FALSE;
  static struct noe *Noe ;
  static struct Library *Record ;
  static FILE   *Error ;

  Noe = &NOE[1] ;

  Error = safeopen("ErrorsInNoes","w",(int)strlen("ErrorsInNoes"));

  if (DEBUG >= 1) printf(" > util.c: CheckLibraryNames(): ... checking names\n");
  
   /* while ( Noe->Number > from && Noe->Number <= constraintcount ) { */
    while ( Noe->next != NULL ) {
    GoodAtom1 = FALSE;
    GoodAtom2 = FALSE;
    Record = &Lib[1] ;

    /* Search the library records and check if the atom exist */
    while ( Record->next != NULL ) {
      if ( !HomolConstraints && (strcmp(Record->Descriptor,"NMR") != 0) ) {
	      Record++ ;
	      continue ;  /* Discard ATOM records */
      }

      /* Check first atom */
      if ((strcasecmp(Noe->residue_type1,Record->ResidueName)==0) ||
		      (strcmp(Record->ResidueName,"ANY")==0)) {
	if (option == 8)
	  if (strcasecmp(Noe->atom_type1,Record->EceppName)==0) 
	    GoodAtom1=TRUE ;	
	if (option == 6)
	  if (strcasecmp(Noe->atom_type1,Record->XPlorName)==0) 
	    GoodAtom1=TRUE ;	
	if (option == 300)
	  if (strcasecmp(Noe->atom_type1,Record->DianaName)==0) 
	    GoodAtom1=TRUE ;	
	if (option == 3)
	  if (strcasecmp(Noe->atom_type1,Record->DyanaName)==0) 
	    GoodAtom1=TRUE ;	
	if (option == 2)
	  if (strcasecmp(Noe->atom_type1,Record->DismanName)==0) 
	    GoodAtom1=TRUE ;	
	if (option == 1 )
	  if (strcasecmp(Noe->atom_type1,Record->CongenName)==0) 
	    GoodAtom1=TRUE ;
      }

      /* Check second atom */
      if ((strcasecmp(Noe->residue_type2,Record->ResidueName)==0) ||
		      (strcmp(Record->ResidueName,"ANY")==0)) {
	if (option == 8)
	  if (strcasecmp(Noe->atom_type2,Record->EceppName)==0) 
	    GoodAtom2=TRUE ;	
	if (option == 6 )
	  if (strcasecmp(Noe->atom_type2,Record->XPlorName)==0) 
	    GoodAtom2=TRUE ;
	if (option == 300 )
	  if (strcasecmp(Noe->atom_type2,Record->DianaName)==0) 
	    GoodAtom2=TRUE ;
	if (option == 3)
	  if (strcasecmp(Noe->atom_type2,Record->DyanaName)==0) 
	    GoodAtom2=TRUE ;	
	if (option == 2 )
	  if (strcasecmp(Noe->atom_type2,Record->DismanName)==0) 
	    GoodAtom2=TRUE ;
	if (option == 1 )
	  if (strcasecmp(Noe->atom_type2,Record->CongenName)==0) 
	    GoodAtom2=TRUE ;
      }
      
      /* If both are OK we stop searching the library */
      if (GoodAtom1 && GoodAtom2) break;
      Record++;
    } 

    if ( !GoodAtom1 ) {
      Noe->ToPrint = FALSE ;
      fprintf(Error," ** Error in Atom1 name in NOE #%4d:   '%-4s' %3s %3d \n",
	      Noe->Number, Noe->atom_type1, Noe->residue_type1, Noe->residue_number1);

      fprintf(stderr," ** Error in Atom1 name in NOE #%4d:   '%-4s' %3s %3d \n",
	      Noe->Number, Noe->atom_type1, Noe->residue_type1, Noe->residue_number1);
      ErrorsInNoes = TRUE ;
    }    

    if ( !GoodAtom2 ) {
      Noe->ToPrint = FALSE ;
      fprintf(Error," ** Error in Atom2 name in NOE #%4d:   '%-4s' %3s %3d \n",
	      Noe->Number, Noe->atom_type2, Noe->residue_type2, Noe->residue_number2 );

      fprintf(stderr," ** Error in Atom2 name in NOE #%4d:   '%-4s' %3s %3d \n",
	      Noe->Number, Noe->atom_type2, Noe->residue_type2, Noe->residue_number2 );
      ErrorsInNoes = TRUE ;
    }
    (void) fflush(stderr);
    (void) fflush(stdout);
    Noe++;
  }
  return ;
}



/* ========================================================================= */

static struct AtMissing *Missing ;
void CopyMiss(struct AtMissing *From, struct AtMissing *To) ;

/*
 * This module looks into Library records file and locates all missing atoms.
 * Missing atom is the one that appears in Library file and doesn't appear in
 * PDB file, so special care must be taken as the PDB file can have missing
 * atoms depending on the protonation state for the different residues. 
 * It is usual to find atoms belonging to ASP, GLU, HIS as missing when they
 * are _NOT_ really missing.
 *
 * The purpose of this routine is to be really picky regarding possible missing
 * atoms.
 *
 * It is called from pdbstat_menu with an argument that can be any number
 * between 1 and total_models (the number is given as a char) or the
 * argument can be 'all' or '*' meaning that _ALL_ models are to be
 * searched for missing atoms.
 *
 * It calls to CopyMiss in order to sort the records by residue number.
 *
 * Written: March, 19th 1998. Roberto Tejero.
 */

void CheckPDBNames(int LibRecords,char option[3]) 
{
  static struct models *model ;
  static struct Library *Record ;
  static struct AtMissing *MissPtr ;
  static char   Name[5];
  static int    NumRes, From, To, Counter ;
  static int    Current, RecFound;
  static int    i, j, NumTemp;

  if ((strncasecmp(option,"all",1)==0) || (strncmp(option,"*",1)==0)) {
    From = 1 ;
    To = total_models;
  } else {
    From = To = atoi(option) ;
  }
  
  for ( Counter = From; Counter <= To; Counter++) {
    model = &MODELS[Counter];
  
    RecFound = 0 ;
    Missing = (struct AtMissing *) malloc(sizeof(struct AtMissing)*(RecFound+1));
    
    /* Initialize values for record zero */
    Missing[RecFound].Atom = 0 ; 
    Missing[RecFound].Model = 0 ;
    strcpy(Missing[RecFound].Chain," ");
    strcpy(Missing[RecFound].Residue," ");
    Missing[RecFound].ResNum = 0;
    strcpy(Missing[RecFound].Name," ");
    Missing[RecFound].Found = FALSE ;
    Missing[RecFound].next = &Missing[RecFound] ;
    
    Record = &Lib[1] ;
    
    /*
     * Loop over all records in libray that are defined.
     * In this sense 'xx' means no atom name defined for that record.
     * It usually signals a pseudoatom
     */
    while (Record->next != NULL) {
      if ( strcmp(Record->Descriptor,"ATOM") != 0) {
	      Record++ ;
	      continue ;  /* Discard no ATOM records */
      }
      if (strcasecmp(Record->PdbName,"xx")==0) {
	Record++; 
	continue ;
      }
      for (i = 0; i <= (int)strlen(Record->PdbName); i++)
	if (Record->PdbName[i]=='*') {
	  Record++;
	  goto Continue;
	}
      goto Sigue;
    Continue:
      continue;
    Sigue:
      Current = 1 ; 
      NumRes = 0 ;
      
      if (DEBUG > 0) fprintf(stdout,"\n");
      
      /*
       * Loop over all atoms in model under study
       * Locate the atoms belonging to the same residue as the
       * library record, increased RecFound, allocate memory
       * check and store results
       */
      while ((model->PDB[Current].pdb_num > 0) &&  
	     (model->PDB[Current].pdb_num <= model->number_of_atoms)) {
	if (strcasecmp(model->PDB[Current].pdb_res,Record->ResidueName)!=0) {
	  Current++;
	  continue;
	}
	
	if (model->PDB[Current].pdb_num_res != NumRes) {
	  NumRes = model->PDB[Current].pdb_num_res;
	  RecFound++;
	  Missing = (struct AtMissing *) realloc(Missing,sizeof(struct AtMissing)*(RecFound+1));
	  Missing[RecFound].Found = FALSE ;
	  Missing[RecFound].Atom = RecFound ; 
	  Missing[RecFound].Model = model->modelno ;
	  strcpy(Missing[RecFound].Chain,model->PDB[Current].pdb_chain_id);
	  strcpy(Missing[RecFound].Residue,Record->ResidueName);
	  Missing[RecFound].ResNum = NumRes;
	  strcpy(Missing[RecFound].Name,Record->PdbName);
    	  Missing[RecFound].next = &Missing[RecFound] ;
	  
	  if (DEBUG >= 2) 
	    fprintf(stdout," > Rec %5d Mod %2d Chain %s Res %3s %3d Atm %4s\n",
		    Missing[RecFound].Atom, Missing[RecFound].Model, 
		    Missing[RecFound].Chain, Missing[RecFound].Residue, 
		    Missing[RecFound].ResNum, Missing[RecFound].Name); 
	}
	
	/* Check if record is in PDB file */
	sscanf(model->PDB[Current].pdb_iupac,"%s",Name);	
	if ( (strcasecmp(Name,Record->PdbName)==0) || (strcasecmp(Name,Record->DyanaName)==0) ) {
	  model->PDB[Current].TruePdb = TRUE ;
	  Missing[RecFound].Found = TRUE ;
	  Current++ ;
	  continue ;
	} 
	Current++;
      }
      Record++;
    }

    /*
     * Sort the missing records by residue number to make output of
     * records pretty.
     * Bubble sort is used
     */
    for (j = 2 ; j <= RecFound ; j++) {
      NumTemp = (Missing+j)->ResNum;
      CopyMiss(Missing+j,Missing);
      i = j - 1 ;
      while ((i > 0) && ((Missing+i)->ResNum > NumTemp)) { 
	CopyMiss(Missing+i,Missing+i+1) ;
	i--;
      }
      CopyMiss(Missing,Missing+i+1);
    }

    /* Print Missing atoms */
    fprintf(stdout,"\n Searching Model %3d\n", Counter );
    MissPtr = &Missing[1];

    while (MissPtr->Atom > 0 && MissPtr->Atom <= RecFound ) {
      /*
       * Check for protonation state of different residues, i.e. sometimes
       * protons are missing but this is due to the protonation state of
       * the residue or one is missing when a disulfide bridge is found. 
       * These protons are usually
       *
       *     H     of  first residue (it is called H1, H2, H3) 
       *     1HD   of  ASP
       *     1HE   of  GLU
       *     1HG1  of  THR
       *     HE2   of  HIS
       *     HG    of  CYS  (when disulfide bridge).
       *
       */
      if (MissPtr->ResNum == 1)
	if (strcasecmp(MissPtr->Name,"H")==0)
	  MissPtr->Found = TRUE;
      
      if (strcasecmp(MissPtr->Residue,"ASP")==0)
	if (strcasecmp(MissPtr->Name,"1HD")==0)
	  MissPtr->Found = TRUE;
      
      if (strcasecmp(MissPtr->Residue,"ASP")==0)
	if (strcasecmp(MissPtr->Name,"HD1")==0)
	  MissPtr->Found = TRUE;
      
      if (strcasecmp(MissPtr->Residue,"GLU")==0)
	if (strcasecmp(MissPtr->Name,"1HE")==0)
	  MissPtr->Found = TRUE;
      
      if (strcasecmp(MissPtr->Residue,"GLU")==0)
	if (strcasecmp(MissPtr->Name,"HE1")==0)
	  MissPtr->Found = TRUE;
      
      if (strcasecmp(MissPtr->Residue,"THR")==0)
	if (strcasecmp(MissPtr->Name,"1HG1")==0)
	  MissPtr->Found = TRUE;
      
      if (strcasecmp(MissPtr->Residue,"HIS")==0)
	if (strcasecmp(MissPtr->Name,"HE2")==0)
	  MissPtr->Found = TRUE;
      
      if (strcasecmp(MissPtr->Residue,"CYS")==0)
	if (strcasecmp(MissPtr->Name,"HG")==0)
	  MissPtr->Found = TRUE;
 
      if (strcasecmp(MissPtr->Residue,"LYS")==0)
	if (strcasecmp(MissPtr->Name,"3HZ")==0)
	  MissPtr->Found = TRUE;
      
      if (strcasecmp(MissPtr->Residue,"LYS")==0)
	if (strcasecmp(MissPtr->Name,"HZ3")==0)
	  MissPtr->Found = TRUE;
      
      if (strcasecmp(MissPtr->Residue,"ARG")==0)
	if (strcasecmp(MissPtr->Name,"1HH1")==0)
	  MissPtr->Found = TRUE;
      
      if (strcasecmp(MissPtr->Residue,"ARG")==0)
	if (strcasecmp(MissPtr->Name,"2HH1")==0)
	  MissPtr->Found = TRUE;
      

      if ( !MissPtr->Found )
	fprintf(stdout,"  *Missing Atom*  ->  Model %3d Chain %s Res %3s %3d Atom %4s\n",
		MissPtr->Model, MissPtr->Chain, MissPtr->Residue, MissPtr->ResNum, MissPtr->Name); 
      MissPtr++;
    }

    /* Free memory to be reused */
    (void) free(Missing);  
  }
  return ;
}

/* ========================================================================= */
/*
 * Routine to simply copy records of missing atoms to help in
 * sorting them in ascending order of residue number to make output
 * pretty. It is called in the routine above
 *
 * Written: March, 19th, 1998. Roberto Tejero
 */

void CopyMiss(struct AtMissing *From, struct AtMissing *To)
{
  To->Atom = From->Atom ;
  To->ResNum = From->ResNum ;
  To->Model = From->Model;
  To->Found = From->Found;
  strcpy(To->Chain,From->Chain);
  strcpy(To->Residue,From->Residue);
  strcpy(To->Name,From->Name);
  return ;
}



/* ========================================================================= */

/************************************************************************
 *   Routine to print on screen the ACO data read from a
 *   file and converted to all the other formats.
 ************************************************************************/
 
void SeeAco(char Salida[MAXWORD])
{
  static short i;
  static struct aco *ACOCons;
  static char   Opt[5];
  static FILE   *DumpedACO='\0' ;

  for (i = (int)strlen(Salida); i <= MAXWORD-1; i++) Salida[i]='\0';
  ACOCons = &ACO[1] ;
  if (ACOCons == NULL) {
    fprintf(stdout," \n First read a ACO file \n");
    return ;
  }

  if (strcasecmp(Salida,"")!=0)
    DumpedACO=safeopen(Salida,"w",(int)strlen(Salida));
  else
    DumpedACO=stdout;
  
  fprintf(DumpedACO," Debug is: %d \n", DEBUG);
  
  while ( ACOCons->next != NULL ) {
    for (i=1;i<=total_models;i++)
      ACOCons->AngleModel[i] = DihedralResModel(i, ACOCons->residue_number, ACOCons->AngleName,
						ACOCons->residue_type);
    fprintf(DumpedACO,"%5d-> %3d %-4s %-5s {%-5s %-5s %-5s %-5s} %7.2f %7.2f",
	    ACOCons->Number,
	    ACOCons->residue_number, ACOCons->residue_type,
	    ACOCons->AngleName, ACOCons->Atom1,  ACOCons->Atom2,  ACOCons->Atom3,  ACOCons->Atom4,  
	    ACOCons->AngleMin, ACOCons->AngleMax);
    for (i=1;i<=total_models;i++) 
      fprintf(DumpedACO," %7.2f",ACOCons->AngleModel[i]);
    fprintf(DumpedACO,"\n");
    
    if (strcasecmp(Salida,"")==0) { /* Results are being sent to screen */
      if (fmod(ACOCons->Number,23) == 0.0) {
	(void) fprintf(stdout,"\n ** Type return to continue, 'q' or 'e' +"\
		       "return to quit \n");
	(void) GetFromKeyBoard(Opt,MAXWORD,stdin) ;
      }
      if ((strncmp(Opt,"exit",1)==0) || (strncmp(Opt,"quit",1)==0)) {
	        strcpy(Opt,"");
		return ;
      }
    }
    
    if (ACOCons->Number == acocount) {
      if (strcasecmp(Salida,"")!=0) {
	      fflush(DumpedACO);
	      fclose(DumpedACO);
      }
      return;
    }
    ACOCons++ ;
  }
  return;
}


/* ========================================================================= */

/************************************************************************
 *   Routine to print on screen the ACO data read from a
 *   file and converted to all the other formats.
 ************************************************************************/
 
void SeeViolAco(char Salida[MAXWORD], float Threshold)
{
     static short i, itmp, NumViol = 0, Printed = 1;
     static float Viol, ViolMax = 0.0, ViolMin = 9999.9999, AcoMaxViol = 0.0 ;
     static float Averages = 0.0;
     static struct aco *ACOCons;
     static float A1, A2 ;
     static char   Opt[5] , LineOut[2048], LineTmp[1024];
     static FILE   *DumpedACO='\0' ;
     static struct ResidualACOViol *ACOSummary ;
     
     for (i = (int)strlen(Salida); i <= MAXWORD-1; i++) Salida[i]='\0';
     
     Printed = 1 ;
     
     ACOCons = &ACO[1] ;
     if (ACOCons == NULL) {
	  fprintf(stdout," \n First read a ACO file \n");
	  return ;
     }
     
     if (strcasecmp(Salida,"")!=0)
	  if ( ConsTrue ) { 
	       DumpedACO=efopen(Salida,"a");
	  } else {
	       DumpedACO=safeopen(Salida,"w",(int)strlen(Salida));
	  }
     else
	  DumpedACO=stdout;
     
     if ( ResAcoViol == NULL )
	  ResAcoViol = (struct ResidualACOViol *)malloc(sizeof(struct ResidualACOViol)*(total_models+1)); 
     else
	  ResAcoViol = (struct ResidualACOViol *)realloc(ResAcoViol,sizeof(struct ResidualACOViol)*(total_models+1));
     
     for (i = 1; i <= total_models; i++) {
	  ACOSummary = ResAcoViol + i;
	  ACOSummary->Model = i;
	  ACOSummary->Total = 0;
	  ACOSummary->NumLt5 = ACOSummary->NumLt10 = ACOSummary->NumGt10 = 0;
	  ACOSummary->GreaterThanThreshold = 0;
	  ACOSummary->Average = 0.0 ;
	  ACOSummary->Rmsd = 0.0 ;
	  ACOSummary->MinViol = 99999.9999;
	  ACOSummary->MaxViol = 0.0;
     }
     
     /* Calculation of Violations */
     while (ACOCons->next != NULL) {
	  for (itmp = 0; itmp < 2048; itmp++)
	       LineOut[itmp]='\0';
	  
	  for (itmp = 0; itmp < 1024; itmp++)
	       LineTmp[itmp]='\0';
	  
	  for (i = 1; i <= total_models; i++) {
	       ACOCons->AngleModel[i] = DihedralResModel(i, ACOCons->residue_number, ACOCons->AngleName,
							 ACOCons->residue_type);
	       if (ACOCons->AngleModel[i] < 0) ACOCons->AngleModel[i] += 360.0 ;
	  }
	  
	  sprintf(LineOut,"%5d-> %3d %-4s %-5s %7.2f %7.2f",
		  ACOCons->Number, ACOCons->residue_number, ACOCons->residue_type,
		  ACOCons->AngleName, ACOCons->AngleMin, ACOCons->AngleMax);
	  
	  if (ACOCons->AngleMin < 0 ) ACOCons->AngleMin += 360.0 ;
	  if (ACOCons->AngleMax < 0 ) ACOCons->AngleMax += 360.0 ;
	  
	  NumViol = 0;
	  ViolMin = 9999.9999 ;
	  ViolMax = 0.0 ;
	  
	  for (i = 1; i <= total_models; i++) {  /* Violations */
	       ACOSummary = ResAcoViol + i;
	       ACOSummary->Model = i ;
	       
	       A1 = ACOCons->AngleMin;
	       A2 = ACOCons->AngleMax;
	       Viol = 0.0;
	       if ( ((A2<A1) && ((A1 > ACOCons->AngleModel[i] && A2 < ACOCons->AngleModel[i]) ||
				 (A1 < ACOCons->AngleModel[i] && A2 > ACOCons->AngleModel[i]))) ||
		    ((A2>A1) && ((A1 > ACOCons->AngleModel[i] && A2 > ACOCons->AngleModel[i]) ||
				 (A1 < ACOCons->AngleModel[i] && A2 < ACOCons->AngleModel[i]))) ) {
		    NumViol++;
		    A1 = fabs(A1-ACOCons->AngleModel[i]) ;
		    A2 = fabs(A2-ACOCons->AngleModel[i]) ;
		    
		    if ( A1 > 180.0 ) A1 = 360.0-A1;
		    if ( A2 > 180.0 ) A2 = 360.0-A2;
		    
		    if ( A1 <= A2 )
			 Viol = A1;
		    else
			 Viol = A2;
	       }

	       if ( Viol > ViolMax ) ViolMax = Viol ;
	       if ( Viol < ViolMin ) ViolMin = Viol ;
	       ACOSummary->MaxViol = max(ACOSummary->MaxViol,Viol) ;
	       ACOSummary->MinViol = min(ACOSummary->MinViol,Viol) ;

	       /* Give info on screen */
	       sprintf(LineTmp," %6.1f", Viol);
	       strcat(LineOut,LineTmp);
	       
	       /* For Summary of residual violations */
	       if ( Viol > 0.0 ) {
		    if ( Viol < 5.0 ) ACOSummary->NumLt5++ ;
		    if ( Viol >= 1.0 && Viol <= 10.0 ) ACOSummary->NumLt10++;
		    if ( Viol > 10.0 ) ACOSummary->NumGt10++;
		    if ( Viol >= Threshold ) ACOSummary->GreaterThanThreshold++ ; 
		    ACOSummary->Total++ ;
		    ACOSummary->Average = ACOSummary->Average + Viol ;
		    ACOSummary->Rmsd = ACOSummary->Rmsd + Viol * Viol ;
	       }
	  }
	  
	  if (NumViol == 0) {
		  ViolMin = ViolMax = 0.0;
	  }
	  
	  sprintf(LineTmp," -  %2d [%6.1f .. %6.1f] \n", NumViol, ViolMin, ViolMax);
	  strcat(LineOut,LineTmp);
	  
	  if ( ViolMax >  Threshold) {
	       fprintf(DumpedACO,"%s", LineOut);
	       Printed++ ;
	  }
	  
	  if (strcasecmp(Salida,"")==0) { /* Results are being sent to screen */
	       if ( Printed == 31 ) {
		    Printed = 1;
		    (void) fprintf(stdout,"\n ** Type return to continue, 'q' or 'e' +return to quit \n");
		    (void) GetFromKeyBoard(Opt,MAXWORD,stdin) ;
	       }
	       if ((strncmp(Opt,"exit",1)==0) || (strncmp(Opt,"quit",1)==0)) {
		    for (itmp = 0; itmp < 5; itmp++)
			 Opt[itmp]='\0';
		    return ;
	       }
	  }
	  if (ACOCons->Number == acocount)  break;
	  ACOCons++ ;
     }
     
     /* Print ACOSummary */
     fprintf(DumpedACO,"\n    ----  ACOSummary Of Residual ACO Constraint Violations ---- \n");
     
     fprintf(DumpedACO,"\n                     ");
     for (i=1;i<=total_models;i++) {
	  fprintf(DumpedACO," Mod %2d", i);
     }
     fprintf(DumpedACO,"        Averages  ");
     
     fprintf(DumpedACO,"\n                     ");
     for (i=1;i<=total_models;i++) {
	  fprintf(DumpedACO," ~~~~~~");
     }
     fprintf(DumpedACO,"      ~~~~~~~~~~~ ");
     
     fprintf(DumpedACO,"\n 1 - 10.  degrees   :");
     Averages = 0.0;
     for (i=1;i<=total_models;i++) {
	  ACOSummary = ResAcoViol + i;
	  fprintf(DumpedACO,"  %5d", ACOSummary->NumLt10);
	  Averages += ACOSummary->NumLt10 ;
     }
     fprintf(DumpedACO,"    %10.2f", Averages/total_models);
     
     fprintf(DumpedACO,"\n   > 10.  degrees   :");
     Averages = 0.0;
     for (i=1;i<=total_models;i++) {
	  ACOSummary = ResAcoViol + i;
	  fprintf(DumpedACO,"  %5d", ACOSummary->NumGt10);
	  Averages += ACOSummary->NumGt10 ;
     }
     fprintf(DumpedACO,"    %10.2f", Averages/total_models);
     
     fprintf(DumpedACO,"\n        Total       :");
     Averages = 0.0;
     for (i=1;i<=total_models;i++) {
	  ACOSummary = ResAcoViol + i;
	  fprintf(DumpedACO,"  %5d", ACOSummary->Total);
	  Averages += ACOSummary->Total ;
     }
     fprintf(DumpedACO,"    %10.2f", Averages/total_models);
     
     fprintf(DumpedACO,"\n Minimum Violation  :");
     Averages = 0.0;
     for (i=1;i<=total_models;i++) {
	  ACOSummary = ResAcoViol + i;
	  fprintf(DumpedACO," %6.1f", ACOSummary->MinViol);
	  Averages += ACOSummary->MinViol ;
     }
     fprintf(DumpedACO,"    %10.2f", Averages/total_models);
     
     fprintf(DumpedACO,"\n Maximum Violation  :");
     for (i=1;i<=total_models;i++) {
	  ACOSummary = ResAcoViol + i;
	  fprintf(DumpedACO," %6.1f", ACOSummary->MaxViol);
	  AcoMaxViol = max(ACOSummary->MaxViol,AcoMaxViol);
     }
     fprintf(DumpedACO,"    %10.2f", AcoMaxViol);
     
     fprintf(DumpedACO,"\n Average Violation  :");
     Averages = 0.0;
     for (i=1;i<=total_models;i++) {
	  ACOSummary = ResAcoViol + i;
	  fprintf(DumpedACO," %6.1f", ACOSummary->Average/acocount);
	  Averages += ACOSummary->Average ;
     }
     fprintf(DumpedACO,"    %10.2f", Averages/(total_models*acocount));
     
     fprintf(DumpedACO,"\n   RMS   Violation  :");
     Averages = 0.0 ;
     for (i=1;i<=total_models;i++) {
	  ACOSummary = ResAcoViol + i;
	  fprintf(DumpedACO," %6.3f", sqrt(ACOSummary->Rmsd/acocount));
	  Averages += ACOSummary->Rmsd;
     }
     fprintf(DumpedACO,"    %10.3f", sqrt(Averages/(total_models*acocount)));
     
     
     fprintf(DumpedACO,"\n\n");
     
     /* Close files if open */
     if (strcasecmp(Salida,"")!=0) {
	  fflush(DumpedACO);
	  fclose(DumpedACO);
     }
     /* free (ResAcoViol);  */
     return;
}

/* ========================================================================= */

/************************************************************************
 *   Routine to print on screen the NOE data read from a
 *   file and converted to all the other formats. It dumps
 *   on screen all the possible atom names:
 *   DISMAN, DIANA, CONGEN, X-PLOR
 ************************************************************************/
 
void SeeNoes(char Salida[MAXWORD], char Average[10])
{
     static short i, itmp, icnt=0;
     static struct noe *Cons;
     static char   Opt[5];
     static FILE   *DumpedNoes='\0' ;
     
     if (strcmp(Average,"")==0) 
	  strcpy(Average,"cent");
     
     for (i = (int)strlen(Salida); i <= MAXWORD-1; i++) 
	  Salida[i]='\0';

     Cons = &NOE[1] ;
     if (Cons == NULL) {
	  fprintf(stdout," \n First read a constraint file \n");
	  return ;
     }
     
     if (strcasecmp(Salida,"")!=0)
	  DumpedNoes=safeopen(Salida,"w",(int)strlen(Salida));
     else
	  DumpedNoes=stdout;
     
     fprintf(DumpedNoes," Debug is: %d \n", DEBUG);
     
     icnt = 0 ;
     while ( Cons->next != NULL ) {
	  if ( !Cons->ToPrint ) {   /* Avoid printing meaningless and duplicated */
	       Cons++ ;
	       continue ;
	  }
	  for (i = 1; i <= total_models; i++) 
	       if ( strncasecmp(Average,"sum",3)==0 || strncasecmp(Average,"ave",3)==0 || strncasecmp(Average,"cen",3)==0 )
		    Cons->Dist[i]=DistNoe(i, Cons->residue_number1, Cons->chain_id_1, Cons->atom_type1,
					  Cons->residue_number2, Cons->chain_id_2, Cons->atom_type2,
					  DumpedNoes,Average);
	  if ( strncasecmp(Average,"format",3)==0 ) {
	       fprintf(DumpedNoes,"%5d-> %3s%2s %3d %-4s{%-4s %-4s %-4s %-4s} - "\
		       "%3s %2s %3d %-4s{%-4s %-4s %-4s %-4s} %5.2f %5.2f",
		       Cons->Number, Cons->residue_type1, Cons->chain_id_1, Cons->residue_number1,
		       Cons->Atom1Disman, Cons->Atom1Diana, Cons->Atom1Congen, Cons->Atom1Xplor, Cons->Atom1Ecepp,
		       Cons->residue_type2, Cons->chain_id_2, Cons->residue_number2,
		       Cons->Atom2Disman, Cons->Atom2Diana, Cons->Atom2Congen, Cons->Atom2Xplor, Cons->Atom2Ecepp,
		       Cons->lower, Cons->upper);
	  } else {
	       fprintf(DumpedNoes,"%5d-> %3s%2s %3d %-4s - %3s %2s %3d %-4s %5.2f %5.2f",
		       Cons->Number,
		       Cons->residue_type1, Cons->chain_id_1, Cons->residue_number1, Cons->atom_type1,
		       Cons->residue_type2, Cons->chain_id_2, Cons->residue_number2, Cons->atom_type2,
		       Cons->lower, Cons->upper);
	  }
	  
	  for (i = 1; i <= total_models; i++) 
	       if ( strncasecmp(Average,"sum",3)==0 || strncasecmp(Average,"ave",3)==0 || strncasecmp(Average,"cen",3)==0 )
		    fprintf(DumpedNoes," %5.2f",Cons->Dist[i]);
	  
	  fprintf(DumpedNoes,"\n");
	  
	  icnt++ ;   /* counter to limit output on screen */
	  if (strcasecmp(Salida,"")==0) { /* Results are being sent to screen */
	       if (fmod(icnt,23) == 0.0) {
		    (void) fprintf(stdout,"\n ** Type return to continue, 'q' or 'e' +return to quit \n");
		    (void) GetFromKeyBoard(Opt,MAXWORD,stdin) ;
	       }
	       if ((strncmp(Opt,"exit",1)==0) || (strncmp(Opt,"quit",1)==0)) {
		    for (itmp = 0; itmp < 5; itmp++)
			 Opt[itmp]='\0';
		    return ;
	       }
	  }
	  
	  if (Cons->Number == constraintcount) {
	       if (strcasecmp(Salida,"")!=0) {
		    fflush(DumpedNoes);
		    fclose(DumpedNoes);
	       }
	       return;
	  }
	  Cons++ ;
     }
     if (strcasecmp(Salida,"")!=0) {
	  fflush(DumpedNoes);
	  fclose(DumpedNoes);
     }
     return;
}


/* ========================================================================= */

/************************************************************************
 *   Routine to print on screen the Residual Violations
 ************************************************************************/

void SeeResidualViol(char Salida[MAXWORD], char Average[10], float Threshold)
{
     static short i,  NumViol;
     static int Printed = 1;
     static struct noe *Cons;
     static char Opt[5] , LineOut[2048], LineTmp[1024];
     static float Viol = 0.0, MaxViol=0.0, MinViol=99999.99;
     static float Add02=0.0, Add05=0.0, Add1=0.0, AddAverage=0.0, AddRmsd=0.0, AddMaxViol = 0.0 ;
     static FILE *FileViol='\0';
     static struct ResidualViol *Summary = NULL ;
     
     Printed = 1;
     Add02 = Add05 = Add1 = AddAverage = AddRmsd = AddMaxViol = 0.0 ;
     
     if (strcmp(Average,"")==0) 
	  strcpy(Average,"cent");
     
     for (i = (int)strlen(Salida); i <= MAXWORD-1; i++) 
	  Salida[i]='\0';
     
     if (strcasecmp(Salida,"")!=0)
	  FileViol=safeopen(Salida,"w",(int)strlen(Salida));
     else
	  FileViol=stdout;
     
     fprintf(FileViol,"\n  Residual Violations greater than %.2f \n\n", Threshold);
     
     Cons = NOE + 1 ;
     if (Cons == NULL) {
	  fprintf(stdout," \n First read a constraint file \n");
	  return ;
     }
     
     if ( ResViol == NULL )
  	  ResViol = (struct ResidualViol *)malloc(sizeof(struct ResidualViol)*(total_models+1)); 
     else
	  ResViol = (struct ResidualViol *)realloc(ResViol,sizeof(struct ResidualViol)*(total_models+1));
    
     ResViol->next = ResViol ;

     for (i=0;i<=total_models;i++) {
	  Summary = ResViol + i;
	  Summary->Model=i;
	  Summary->Total = 0;
	  Summary->NumLt01 = Summary->NumLt02 = Summary->NumLt05 = Summary->NumLt1= 0;
          Summary->GreaterThanThreshold = 0;
	  Summary->Average = 0.0 ;
	  Summary->Rmsd = 0.0 ;
	  Summary->MinViol = 99999.9999;
	  Summary->MaxViol = 0.0;
     }
     
     while ( Cons->next != NULL ) {
	     if ( !Cons->ToPrint ) {  /* Avoid counting meaningless */
		     Cons++ ;
		     continue ;
	     }
	  for (i=1; i<= total_models; i++) 
	       Cons->Dist[i]=DistNoe(i,Cons->residue_number1,Cons->chain_id_1, Cons->atom_type1,
				     Cons->residue_number2,Cons->chain_id_2,Cons->atom_type2,FileViol,Average);
	  
	  LineOut[0]='\0';
	  LineTmp[0]='\0';
	  
	  sprintf(LineOut,"%5d-> %3s %2s %3d %-4s - %3s %2s %3d %-4s %5.2f %5.2f",
		  Cons->Number,
		  Cons->residue_type1,Cons->chain_id_1, Cons->residue_number1,Cons->atom_type1,
		  Cons->residue_type2,Cons->chain_id_2, Cons->residue_number2,Cons->atom_type2,
		  Cons->lower, Cons->upper);
	  
	  NumViol = 0;
	  MaxViol = 0.0;
	  MinViol = 9999.99;
	  
	  if ( strcasecmp(Salida,"")!=0 ) 
	       if ( Cons->Number == 10 ) fprintf(stdout,"  In progress  | ");
	  
	  if ( strcasecmp(Salida,"")!=0 ) {
	       if ( fmod(Cons->Number,50)==0.0 )  fprintf(stdout,"\b\b| ");
	       else if (fmod(Cons->Number,101)==0.0)  fprintf(stdout,"\b\b/ ");
	       else if (fmod(Cons->Number,151)==0.0)  fprintf(stdout,"\b\b- ");
	       else if (fmod(Cons->Number,201)==0.0)  fprintf(stdout,"\b\b\\ ");
	  }
	  
	  for (i=1;i<=total_models;i++) {
	       /* Point to the right Model */
	       Summary = ResViol + i ;
	       Summary->Model = i;
	       /* Analyze Min and Max Viol */
	       if (Cons->Dist[i] > Cons->upper) {
		    NumViol++;
		    Viol = Cons->Dist[i] - Cons->upper ;
		    if ( Viol > MaxViol ) MaxViol = Viol ;
		    if ( Viol < MinViol ) MinViol = Viol ;
		    if ( Viol > Summary->MaxViol ) Summary->MaxViol = Viol ;
		    if ( Viol < Summary->MinViol ) Summary->MinViol = Viol ;
		    /*
		      Summary->MaxViol = max(Summary->MaxViol,Viol) ;
		      Summary->MinViol = min(Summary->MinViol,Viol) ;
		    */
	       } else if (Cons->Dist[i] < Cons->lower) {
		    NumViol++;
		    Viol = Cons->lower - Cons->Dist[i] ;
		    if ( Viol > MaxViol ) MaxViol = Viol ;
		    if ( Viol < MinViol ) MinViol = Viol ;
		    /*
		     if ( Viol > Summary->MaxViol ) Summary->MaxViol = Viol ;
		     if ( Viol < Summary->MinViol ) Summary->MinViol = Viol ;
		     */
	       } else {
		    Viol = 0.0;
	       }

	       Summary->MaxViol = max(Summary->MaxViol,Viol) ;
	       Summary->MinViol = min(Summary->MinViol,Viol) ;

	       /* For Summary of residual violations */
	       if ( Viol > 0.0 ) {
		    if ( Viol < 0.1 ) Summary->NumLt01++ ;
		    if ( Viol >= 0.1 && Viol <= 0.2 ) Summary->NumLt02++;   /* This changed to new summary 0.1-0.2 */
		    if ( Viol > 0.2 && Viol <= 0.5 ) Summary->NumLt05++;    /* This changed to new summary 0.2-0.5 */
		    if ( Viol > 0.5 ) Summary->NumLt1++;                    /* This changed to new summary > 0.5 */
		    if ( Viol >= Threshold ) Summary->GreaterThanThreshold++ ; 
		    Summary->Total++ ;
		    Summary->Average = Summary->Average + Viol ;
		    Summary->Rmsd = Summary->Rmsd + Viol * Viol ;
	       }
	       /* */
	       /* if ( Summary->MinViol > 9999.0 ) Summary->MinViol = 0.0 ; */

	       /* Store in LineTmp to Print out line by line */
	       sprintf(LineTmp," %5.2f", Viol);
	       strcat(LineOut,LineTmp);
	       (MODELS+i)->energy = (MODELS+i)->energy + 1.0 * pow(Viol,2);
	  }
	  
	  if ( MinViol > 9999.0 ) MinViol = 0.0 ;
	  sprintf(LineTmp," - %3d [%5.2f .. %5.2f]\n", NumViol, MinViol, MaxViol);
	  strcat(LineOut,LineTmp);
	  
	  if ( MaxViol > Threshold ) {
	       fprintf(FileViol,"%s", LineOut);
	       Printed++ ;
	  }
	  
	  if (strcasecmp(Salida,"")==0) {     /* Results are being printed on screen */
	       if ( Printed == 31 ) {
		    Printed  = 1;
		    (void) fprintf(FileViol,"\n ** Type return to continue, 'q' or 'e' + return to quit \n");
		    (void) GetFromKeyBoard(Opt,MAXWORD,stdin) ;
	       }
	       if ((strncmp(Opt,"exit",1)==0) || (strncmp(Opt,"quit",1)==0)) {
		    strcpy(Opt,"");
		    return ;
	       }
	  }
	  /* if (Cons->Number == constraintcount)      break; */
	  Cons++ ;
     }
     
     /* Tell how many greater than threshold (cutu) */
     fprintf(FileViol,"   -------------------------------------------  \n");
     fprintf(FileViol,"       Number of Violations greater than %.2f  ", Threshold);
     for (i=1;i<=total_models;i++) {
	  Summary = ResViol + i;
	  fprintf(FileViol," %5d", Summary->GreaterThanThreshold);
     }
     fprintf(FileViol,"\n   -------------------------------------------  \n");
     
     /* Print Summary */
     fprintf(FileViol,"\n        ----  Summary Of Residual Distance Constraint Violations ---- ");
     /* fprintf(FileViol,"\n        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ "); */
     
     fprintf(FileViol,"\n                     ");
     for (i=1;i<=total_models;i++) {
	  fprintf(FileViol," Mod %2d", i);
     }
     fprintf(FileViol,"       Averages   ");
     
     fprintf(FileViol,"\n                     ");
     for (i=1;i<=total_models;i++) {
	  fprintf(FileViol," ~~~~~~");
     }
     fprintf(FileViol,"     ~~~~~~~~~~~  ");
     
     /* Now for the new way of reporting violations */
     fprintf(FileViol,"\n      0.1 - 0.2  ang:");
     for (i=1;i<=total_models;i++) {
	  Summary = ResViol + i;
	  fprintf(FileViol,"  %5d", Summary->NumLt02);
	  Add02 += Summary->NumLt02 ;
     }
     fprintf(FileViol,"   %10.2f", Add02/(total_models));
     
     fprintf(FileViol,"\n      0.2 - 0.5  ang:");
     for (i=1;i<=total_models;i++) {
	  Summary = ResViol + i;
	  fprintf(FileViol,"  %5d", Summary->NumLt05);
	  Add05 += Summary->NumLt05 ;
     }
     fprintf(FileViol,"   %10.2f", Add05/(total_models));
     
     fprintf(FileViol,"\n        > 0.5    ang:");
     for (i=1;i<=total_models;i++) {
	  Summary = ResViol + i;
	  fprintf(FileViol,"  %5d", Summary->NumLt1);
	  Add1 += Summary->NumLt1 ;
     }
     fprintf(FileViol,"   %10.2f", Add1/(total_models));
     
     Add1 = 0.0;
     fprintf(FileViol,"\n        Total       :");
     for (i=1;i<=total_models;i++) {
	  Summary = ResViol + i;
	  fprintf(FileViol,"  %5d", Summary->Total);
	  Add1 += Summary->Total ;
     }
     fprintf(FileViol,"   %10.2f", Add1/(total_models));
     
     fprintf(FileViol,"\n Minimum Violation  :");
     for (i=1;i<=total_models;i++) {
	  Summary = ResViol + i;
	  fprintf(FileViol," %6.3f", Summary->MinViol);
     }
     fprintf(FileViol,"   %10.3f", Summary->MinViol);
     
     fprintf(FileViol,"\n Maximum Violation  :");
     for (i=1;i<=total_models;i++) {
	  Summary = ResViol + i;
	  fprintf(FileViol," %6.3f", Summary->MaxViol);
	  AddMaxViol = max(AddMaxViol,Summary->MaxViol);
     }
     fprintf(FileViol,"   %10.3f", AddMaxViol);
     
     fprintf(FileViol,"\n Average Violation  :");
     for (i=1;i<=total_models;i++) {
	  Summary = ResViol + i;
	  fprintf(FileViol," %6.3f", Summary->Average/constraintcount);
	  AddAverage += Summary->Average ;
     }
     fprintf(FileViol,"   %10.3f", AddAverage/(constraintcount*total_models));
     
     fprintf(FileViol,"\n   RMS   Violation  :");
     for (i=1;i<=total_models;i++) {
	  Summary = ResViol + i;
	  fprintf(FileViol," %6.3f", sqrt(Summary->Rmsd/constraintcount));
	  AddRmsd += Summary->Rmsd ;
     }
     fprintf(FileViol,"   %10.3f", sqrt(AddRmsd/(constraintcount*total_models)));
     
     fprintf(FileViol,"\n\n");
     
     /* Close files if open */
     if (strcasecmp(Salida,"")!=0) {
	  fflush(FileViol);
	  fclose(FileViol);
     }
     if (strcasecmp(Salida,"")!=0 && Cons->Number % 50 ) fprintf(stdout,"\n");
     /* free(ResViol); */
     return;
}


/* ========================================================================= */

/************************************************************************
 *    Routine to print Library Data extracted from records 
 *    stored in GenCons.Lib (library data).
 ************************************************************************/
void SeeDihedralLibraryData(char tt[MAXWORD])
{
     static struct DiheLib *Reg ;
     static FILE *LibFile;
     static int LibRecords;
     static char Opt[5];
     
     if (Lib == NULL) {
	  /* Load Library Data */
	  if (DEBUG >= 1) 
	       printf(" > readin_all: .. Opening Library Data ..\n");
	  LibFile = libfopen("GenCons.Lib","r");
	  if (DEBUG >= 1) 
	       printf(" > reading_all: .. Getting Library Data ..\n");
	  LibRecords=GetLibraryData(LibFile);
	  if (DEBUG >= 1) 
	       printf(" > reading_all: .. Closing Library Data ..\n");
	  fclose(LibFile);
     } else
	  LibRecords = MaxLib ;
     if (strcasecmp(tt,"")==0) 
	     strcpy(tt,"all");
     Reg = &DihLib[1] ;
     fprintf(stdout,"\n Dihedral names stored in Library\n");
     fprintf(stdout,"\n ->                 ANGLE AT1   AT2   AT3   AT4       \n");
     fprintf(stdout," -> --------------- ----- ----  ----  ----  ---       \n");
     while (Reg->next != NULL ) {
	  if ((strcasecmp(Reg->ResidueName,tt)==0) || 
	      (strcasecmp(tt,"*")==0) || (strcasecmp(tt,"all")==0))
	       fprintf(stdout," -> %3d Res.: %3s   %-5s %-5s %-5s %-5s %-5s\n",
		       Reg->Number, Reg->ResidueName, Reg->DihName,
		       Reg->Atom1, Reg->Atom2, Reg->Atom3, Reg->Atom4);
	  if ((fmod(Reg->Number,23) == 0.0) && 
	      ((strcasecmp(tt,"*")==0) || (strcasecmp(tt,"all")==0))) {
	       (void) fprintf(stdout,"\n ** Type return to continue, 'q' or 'e' +"\
			      "return to quit \n");
	       (void) GetFromKeyBoard(Opt,MAXWORD,stdin) ;
	       if ((strncmp(Opt,"exit",1)==0) || (strncmp(Opt,"quit",1)==0)) {
		    strcpy(Opt,"");
		    return;
	       }
	       fprintf(stdout,"\n ->                 ANGLE AT1   AT2   AT3   AT4       \n");
	       fprintf(stdout," -> --------------- ----- ----  ----  ----  ---       \n");
	  }
	  if (Reg->Number == DihLibRecords) {
	       return ;
	  }
	  Reg++ ;
     }
     return ;
}

/* ========================================================================= */

/************************************************************************
 *    Routine to print Library Data extracted from records 
 *    stored in GenCons.Lib (library data).
 ************************************************************************/
void SeeLibraryData(char tt[MAXWORD], char Descriptor[6])
{
     static struct Library *registro;
     static FILE *LibFile;
     static int LibRecords;
     static char Opt[5];
     
     if (Lib == NULL) {
	  /* Load Library Data */
	  if (DEBUG >= 1) printf(" > readin_all: .. Opening Library Data ..\n");
	  LibFile = libfopen("GenCons.Lib","r");
	  if (DEBUG >= 1) printf(" > reading_all: .. Getting Library Data ..\n");
	  LibRecords=GetLibraryData(LibFile);
	  if (DEBUG >= 1) printf(" > reading_all: .. Closing Library Data ..\n");
	  fclose(LibFile);
     } else
	  LibRecords = MaxLib ;
     if (strcasecmp(tt,"")==0) 
	     strcpy(tt,"all");
     registro = &Lib[1] ;

     fprintf(stdout,"\n Names stored in Library\n");
     fprintf(stdout,"\n ->                      DISM  DIAN DYAN  CONG  XPLR  PDB  ECEPP\n");
     fprintf(stdout," -> -------------------- ----  ----  ----  ----  ----  ---  -----\n");
     while ( registro->next != NULL ) {
	  if ( ((strcasecmp(registro->ResidueName,tt)==0) || (strcasecmp(tt,"*")==0) || (strcasecmp(tt,"all")==0)) &&
	       ((strncmp(registro->Descriptor,Descriptor,3)==0 || strncmp(Descriptor,"all",3)==0) ))
	       fprintf(stdout," -> %3d Res.: %5s %3s   %-5s %-5s %-5s %-5s %-5s %-5s %-5s\n",
		       registro->Number, registro->Descriptor, registro->ResidueName,
		       registro->DismanName, registro->DianaName, registro->DyanaName, 
		       registro->CongenName, registro->XPlorName, registro->PdbName, registro->EceppName);
	  if ((fmod(registro->Number,23) == 0.0) && ((strcasecmp(tt,"*")==0) || (strcasecmp(tt,"all")==0))) {
	       (void) fprintf(stdout,"\n ** Type return to continue, 'q' or 'e' + return to quit \n");
	       (void) GetFromKeyBoard(Opt,MAXWORD,stdin) ;
	       if ((strncmp(Opt,"exit",1)==0) || (strncmp(Opt,"quit",1)==0)) {
		    strcpy(Opt,"");
		    return;
	       }
	       fprintf(stdout,"\n ->                      DISM  DIAN  CONG  XPLR  PDB  ECEPP\n");
	       fprintf(stdout," -> -------------------- ----  ----  ----  ----  ---   -----\n");
	  }
	  if (registro->Number == LibRecords) {
	       return ;
	  }
	  registro++ ;
     }
     return ;
}


/* ========================================================================= */
/*
 * Function created to avoid using gets().
 * As gets() is known to have some deep security breaches I tried to use
 * fgets(). fgets() reads one character at time up to LEN-1 and stops when 
 * it reaches EOF or newline, storing newline in the buffer and adding 
 * \0 at the end. This behaviour put some trobuble with the strings, mainly
 * when used for file names, so I replace \n by \0 after reading.
 *
 * RTT, Valencia, Jan, 1999
 */
void GetFromKeyBoard(char *s, int size, FILE *stream) 
{
     static unsigned short i=0;
     
     if ( fgets(s,size,stream) == NULL ) {
	  fprintf(stderr," Something wrong reading keyboard \n");
     } else { 
	  for (i = 0 ; i <= size-1 ; i++)
	       if ( s[i] == '\n' ) {
		    s[i] = '\0' ;
		    break ;
	       }
	  if ( DEBUG > 0)
	       fprintf(stderr,"\n > GetFromKeyBoard(): KeyBoard reads .. `%s' \n", s);
     }
     return ;
}

/* ======================================================================= */
short MoreModelsThanAllowed(int nmodels)
{
     if (nmodels > TOPMODELS)
	  return (short)1 ;
     else
	  return (short)0 ;
}

