package DataColumn;

use Utils;
use Scalar::Util qw(looks_like_number);
use POSIX qw(floor);

sub new {
	my $class = shift;
	my $self = {};
	$self->{number} = shift;
	$self->{name} = "ERROR";
	$self->{internal_name} = undef;
	$self->{data} = [];
	$self->{log_data} = [];
	$self->{sorted_data} = undef;
	$self->{max} = undef;
	$self->{min} = undef;
	$self->{abs_max} = undef;
	$self->{abs_min} = undef;
	$self->{float_count} = 0;
	$self->{int_count} = 0;
	$self->{size} = 0;
	$self->{total} = 0.0;
	$self->{log_total} = 0.0;
	$self->{std_dev} = undef;
	$self->{skewness} = undef;
	$self->{kurtosis} = undef;
	$self->{done_statistics} = undef;
	$self->{log_std_dev} = "";
	$self->{log_skewness} = "";
	$self->{log_kurtosis} = "";
	$self->{done_log_statistics} = undef;
	bless($self, $class);
	return $self;
}
sub get_data {
	my $self = shift;
	return $self->{data};
}
sub get_number {
	my $self = shift;
	return $self->{number};
}
sub get_internal_name {
	my $self = shift;
	return $self->{internal_name};
}
sub set_name {
	my $self = shift;
	$self->{internal_name} = shift;
}
sub get_name {
	my $self = shift;
	return $self->{name};
}
sub get {
	my $self = shift;
	my $index = shift;
#	print "data col -> get($index) returning: " . $self->{data}[$index] . "<br/>\n";
	return $self->{data}[$index];
}
sub push {
	my $self = shift;
	my $item = shift;
	if (looks_like_number($item)) {
		Utils->debug("pushing NUMBER $item onto column " . $self->{number});
		if ($item =~ /-?\d+\.\d+/ or $item =~ /e+/) {
			$self->{float_count}++;
		} else {
			#Utils->debug("************* $item");
		}
		unless ($item =~ /\D/) {
			$self->{int_count}++;
		}
		if (not $item == 0) {
			if ((not $self->{max} or $item > $self->{max})) {
				$self->{max} = $item;
			}
			if ((not $self->{min} or $item < $self->{min})) {
				$self->{min} = $item;
			}
			if ((not defined $self->{abs_max} or abs($item) > $self->{abs_max})) {
				$self->{abs_max} = abs($item);
			}
			if ((not defined $self->{abs_min} or abs($item) < $self->{abs_min})) {
				$self->{abs_min} = abs($item);
			}
		}
		my $array = $self->{data};
		my $log_array = $self->{log_data};
		my $returnval = push(@$array, $item);
		unless ($item == 0) {
			my $log_value = log(abs($item));
			$self->{log_total} += $log_value;
			push(@$log_array, $log_value);
		}
		$self->{size} = $self->{size} + 1;
		if (looks_like_number($item)) {
			if ($returnval) {
				$self->{total} += $item;
			}
		}
		return $returnval;
	} else {
		Utils->debug("$item is NAN");
	}
	Utils->debug("new size is " . $self->{size});
}
sub is_float_column {
	my $self = shift;
	Utils->debug("in is_float_col: " . $self->{float_count} . " : " . $self->{size});
	if ($self->{size} == 0) {
		return undef;
	}
	if ($self->{float_count} == $self->{size}) {
		return 1;
	} else {
		return 0;
	}
}
sub is_int_column {
	my $self = shift;
	if ($self->{size} == 0) {
		return undef;
	}
	if ($self->{int_count} == $self->{size}) {
		return 1;
	} else {
		return 0;
	}
}
sub unshift {
	my $self = shift;
	my $item = shift;
	my $array = $self->{data};
	my $returnval = unshift(@$array, $item);
	$self->{size} = $self->{size} + 1;
	if ($returnval) {
		$self->{total} += $item;
	}
	return $returnval;
}
sub get_size {
	my $self = shift;
	my $array = $self->{data};
	my $size = scalar(@$array);
	return $size;
}
sub do_statistics {
	my $self = shift;
	my $array = $self->{data};
	my @numbers = @$array;
	Utils->debug("size is: " . $self->{size});
	if ($self->{size} == 0) {
		return undef;
	}
	Utils->debug("scalar numbers is: " . scalar @numbers);
	# Step 1, find the mean of the numbers
	my $mean1 = $self->get_mean();
	Utils->debug("mean1 is $mean1");
	# Step 2, find the 2nd, 3rd and 4th central moments
	my ($moment2, $moment3, $moment4) = (0, 0, 0);
	foreach my $num (@numbers) {
		$moment2 += ($mean1 - $num)**2;
		$moment3 += ($mean1 - $num)**3;
		$moment4 += ($mean1 - $num)**4;
	}
	Utils->debug("moment2 is $moment2");
	my $central_moment_2 = $moment2 / (scalar @numbers);
	my $central_moment_3 = $moment3 / (scalar @numbers);
	my $central_moment_4 = $moment4 / (scalar @numbers);
	# Step 3, standard deviation is the square root of the
	# above mean
	Utils->debug("centralmoment2 is $central_moment_2");
	my $std_dev = sqrt($central_moment_2);
	Utils->debug("stdev is $std_dev");
	$self->{std_dev} = $std_dev;
	if ($std_dev == 0) {	#this means that all values in the column are the same.
		return undef;
	}
	$self->{skewness} = $central_moment_3 / ($std_dev**3);
	$self->{kurtosis} = $central_moment_4 / ($std_dev**4) - 3;
	$self->{done_statistics} = 1;
}
sub get_mean {
	my $self = shift;
	my $array = $self->{data};
	my $size = scalar(@$array);
	if ($size == 0) {
		return undef;
	}
	my $mean = $self->{total} / $size;
	return $mean;
}
sub get_std_dev {
	my $self = shift;
	unless (defined $self->{done_statistics}) {
		$self->do_statistics();
	}
	return $self->{std_dev};
}
sub get_skewness {
	my $self = shift;
	unless (defined $self->{done_statistics}) {
		$self->do_statistics();
	}
	return $self->{skewness};
}
sub get_kurtosis {
	my $self = shift;
	unless (defined $self->{done_statistics}) {
		$self->do_statistics();
	}
	return $self->{kurtosis};
}
###################
sub do_log_statistics {
	my $self = shift;
	my $log_array = $self->{log_data};
	my @numbers = @$log_array;
	Utils->debug("log statistics size is " . $self->{size});
	if (scalar(@numbers) == 0) {
		return undef;
	}
	Utils->debug("log numbers scalar is: " . scalar(@numbers));
	# Step 1, find the mean of the numbers
	my $mean1 = $self->get_log_mean();
	Utils->debug("log mean is $mean1");
	if ($mean1 == 0) {
		return undef;
	}
	# Step 2, find the 2nd, 3rd and 4th central moments
	my ($moment2, $moment3, $moment4) = (0, 0, 0);
	foreach my $num (@numbers) {
#		Utils->debug("log is $num, mean is $mean1");
		$moment2 += ($num - $mean1)**2;
		$moment3 += ($num - $mean1)**3;
		$moment4 += ($num - $mean1)**4;
	}
	Utils->debug("moment2 is $moment2");
	my $central_moment_2 = $moment2 / (scalar @numbers);
	my $central_moment_3 = $moment3 / (scalar @numbers);
	my $central_moment_4 = $moment4 / (scalar @numbers);
	# Step 3, standard deviation is the square root of the
	# above mean
	Utils->debug("centra_moment is $central_moment_2");
	my $std_dev = sqrt($central_moment_2);
	Utils->debug("log std dev is $std_dev");
	$self->{log_std_dev} = $std_dev;
	$self->{log_skewness} = $central_moment_3 / ($std_dev**3);
	$self->{log_kurtosis} = $central_moment_4 / ($std_dev**4) - 3;
	$self->{done_log_statistics} = 1;
}
sub get_log_mean {
	my $self = shift;
	my $array = $self->{log_data};
	my $size = scalar(@$array);
	if ($size == 0) {
		return undef;
	}
	my $mean = $self->{log_total} / $size;
	return $mean;
}
sub get_log_std_dev {
	my $self = shift;
	unless (defined $self->{done_log_statistics}) {
		$self->do_log_statistics();
	}
	return $self->{log_std_dev};
}
sub get_log_skewness {
	my $self = shift;
	unless (defined $self->{done_log_statistics}) {
		$self->do_log_statistics();
	}
	return $self->{log_skewness};
}
sub get_log_kurtosis {
	my $self = shift;
	unless (defined $self->{done_log_statistics}) {
		$self->do_log_statistics();
	}
	return $self->{log_kurtosis};
}
######################
sub get_max {
	my $self = shift;
	return $self->{max};
}
sub get_min {
	my $self = shift;
	return $self->{min};
}
sub get_abs_max {
	my $self = shift;
	return $self->{abs_max};
}
sub get_abs_min {
	my $self = shift;
	return $self->{abs_min};
}
sub get_abs_max_min_ratio {
	my $self = shift;
	my ($max, $min) = ($self->{abs_max}, $self->{abs_min});
	if ($min == 0) {
		$min = 0.00005;
	}
	return $max / $min;
}
sub get_sw {
	my $self = shift;
	return ($self->{max} - $self->{min});
}
sub get_median {
	my $self = shift;
	my $orig_ref = $self->{data};
	my @copy = sort(@$orig_ref);
	my $half_size = floor($self->{size} / 2);
	return $copy[$half_size];
}
return(1);
