package DataSet;

#use ControlFile;
use Utils;
use File::Path;
use POSIX qw( strftime );

sub new {
	my $class = shift;
	my $self = {};
	$self->{base_dir} = shift;
	$ENV{HOME} = $self->{base_dir};
	chdir;
	$self->{control_file} = undef;
# 	$self->{PROJECT_NAME} = undef;
	$self->{email} = undef;
	$self->{protein_name} = shift;
	$self->{bmrb} = undef;
	$self->{pdb} = undef;
	$self->{sequence} = undef;
	$self->{nCycles} = 5;
	$self->{center} = 0;
	$self->{peaklists} = {};
	my $yymmddHHMMSS = strftime("%Y%m%d\_%H\_%M\_%S", localtime);
	$self->{results_dir} = "rpf_" . $self->{protein_name} . "_$yymmddHHMMSS";
	bless($self, $class);
	return $self;
}
sub add_peaklist {
	my $self = shift;
	my $peaklist = shift;
	my $peaklist_name = $peaklist->get_name();
	if ($peaklist) {
# 		print "$peaklist_name<br/>";
		my $hash = $self->{peaklists};
# 		print "size of hash:  " . scalar keys( %$hash ) . ".\n<br/>";
# 		print $self->{peaklists} . "<br/>";
		$self->{peaklists}{$peaklist_name} = $peaklist;
# 		print "size of hash:  " . scalar keys( %$hash ) . ".\n<br/>";
	}
}
sub get_peaklists {
	my $self = shift;
	return $self->{peaklists};
}
sub get_peaklist_by_filename {
	my $self = shift;
	my $filename = shift;
	my $hash = $self->{peaklists};
	foreach my $key ( keys %$hash ) {
		my $peaklist = $self->{peaklists}{$key};
		if ($filename =~ $peaklist->get_filename) {
			return $peaklist;
		}
	}
	return undef;
}
sub get_project_path {
	my $self = shift;
# 	print "dataset->get_pr0j_path: " . $self->{base_dir} . "<br/>";
	return $self->{base_dir};
}
sub get_email {
	my $self = shift;
	return $self->{email};
}
sub set_email {
	my $self = shift;
	$self->{email} = shift;
}
# sub get_project_name {
# 	my $self = shift;
# 	return $self->{PROJECT_NAME};
# }
# sub set_project_name {
# 	my $self = shift;
# 	$self->{PROJECT_NAME} = shift;
# }
sub get_protein_name {
	my $self = shift;
	return $self->{protein_name};
}
sub set_protein_name {
	my $self = shift;
	$self->{protein_name} = shift;
}
sub get_control_file {
	my $self = shift;
	return $self->{control_file};
}
sub set_control_file {
	my $self = shift;
	$self->{control_file} = shift;
}
sub get_bmrb {
	my $self = shift;
	return $self->{bmrb};
}
sub set_bmrb {
	my $self = shift;
	$self->{bmrb} = shift;
}
sub get_pdb {
	my $self = shift;
	return $self->{pdb};
}
sub set_pdb {
	my $self = shift;
	$self->{pdb} = shift;
}
sub get_sequence {
	my $self = shift;
	return $self->{sequence};
}
sub set_sequence {
	my $self = shift;
	$self->{sequence} = shift;
}
sub get_results_dir {
	my $self = shift;
# 	print "dataset->get_results_path: " . $self->{results_dir} . "<br/>";
	return $self->{results_dir};
}
sub make_control_file {
# 	return;
	my $self = shift;
	my $upload_dir = shift;
	my $bmrb = $self->{bmrb}->get_filename();
	my $pdb = $self->{pdb};
# 	my $project_name = $self->{PROJECT_NAME};
	my $protein_name = $self->{protein_name};
#	$self->{control_file} = ControlFile->new($self->{protein_name});
	my $str = "[General]\n";
	$str .= "chemicalShiftFile=$bmrb\n";
	$str .= "seqFile=" . $self->{sequence}->get_filename() . "\n";
	$str .= "nCycles=" . $self->{nCycles} . "\n";
	$str .= "proteinName=$protein_name\n";
	$str .= "center=".$self->{center}."\n";
	#$str .= "[hyperCommand]\n";
	#$str .= "\$ASBIN/hyper -N\n";
	#$str .= "[xplorCommand]\n";
	my $hash = $self->{peaklists};
	foreach my $key ( keys %$hash ) {
		my $peaklist = $self->{peaklists}{$key};
#		$self->{control_file}->add_peaklist($peaklist);
		#$str .= "[" . $peaklist->get_name() . "]\n";
		$str .= $peaklist->get_section();
	}
	open ( FILE, ">$upload_dir/control_$protein_name" ) or die "$!";
	binmode FILE;
	print FILE $str;
	close FILE;
	return "$upload_dir/control_$protein_name";
}
sub run_rpf {
	my $self = shift;
	my $verbose = shift;
	my $control_file = $self->{base_dir} . "/control_" . $self->{protein_name};
	my $output_dir = $self->{results_dir};
	#mkdir($self->{base_dir} . "/" . $output_dir);
	my $hash = ($self->{peaklists});
	my $num_of_peaklists = scalar keys (%$hash);
	#print STDERR "verbose is $verbose\n";
	Utils->run_rpf($control_file, $self->get_pdb()->get_filepath(), $output_dir, $num_of_peaklists, $verbose);
}

return(1); 
