package PeakList;

use Utils;
use List::Util qw(max);
use List::Util qw(min);

sub new {
	my $class = shift;
	my $self = {};
	$self->{name} = shift;
	$self->{filename} = shift;
	$self->{filename} = $self->{name};
	$self->{filepath} = shift;
	$self->{waterFlag} = shift;
	$self->{intrachain} = shift;
	$self->{interchain} = shift;
	$self->{water} = shift;
	Utils->debug("########################################");
	Utils->debug("name: " . $self->{name}, "<br/>", $self->{filename}, "<br/>", "path: " . $self->{filepath} . "end", "<br/>", "waterFlag: " . $self->{waterFlag}, "<br/>", "intra " . $self->{intrachain}, "<br/>", $self->{interchain}, "<br/>", $_[6]);
	Utils->debug("########################################");
	Utils->debug("doing $self->{name}");
	$self->{label} = undef;
	$self->{hxa_buckets} = Peaks->new();
	$self->{hxb_buckets} = Peaks->new();
	$self->{columns} = {};
	$self->{col_count} = 0;
	$self->{height} = undef;
	$self->{dimensions} = undef;
	$self->{dimension_count} = undef;
	$self->{type_by_name} = {};
	$self->{first_five_lines} = [];
	bless($self, $class);
	$self->parse_columns();
	return $self;
}
sub assign_columns {
	my $self = shift;
	my $columns_ref = $self->{columns};
	my %columns = %$columns_ref;
	my ($h_count, $n_count, $i_count, $h_suffix) = (0, 0, 0, "a");
	for my $column (values(%columns)) {
		Utils->debug("trying to assing column " . $column->{number} . " and size is " . $column->{size});
		my $max = $column->get_abs_max();
		my $min = $column->get_abs_min();
		my $ratio = $column->get_abs_max_min_ratio();
		my $float = $column->is_float_column();
		Utils->debug("float column? $float");
		my $log_skewness = $column->get_log_skewness();
		my $log_kurtosis = $column->get_log_kurtosis();
		my $log_std_dev = $column->get_log_std_dev();
		Utils->debug("for col " . $column->get_number() . ": max: $max, min: $min, ratio: $ratio, float: $float");
		Utils->debug("________________________________ log_std_dev: " . $log_std_dev . ", log_kurtosis : " . $log_kurtosis . ", log_skewness: " . $log_skewness);
		if ($min and $max and $float and ($min > -1) and ($max < 15)) {
			my $atom = "H";
			Utils->debug("match H");
			$h_count++;
			$self->{dimensions}{"hx$h_suffix"} = Dimension->new($column->get_number(), $column);
			$self->{dimensions}{"hx$h_suffix"}->set_type("H");
			$self->{dimensions}{"hx$h_suffix"}->set_tol(0.05);
			$self->{dimensions}{"hx$h_suffix"}->set_shift(0.00);
			$self->{dimensions}{"hx$h_suffix"}->set_sign(0);
			$h_suffix = "b";
		} elsif ($min and $max and $float and ($min > 10) and ($max < 250)) {
			Utils->debug("match x");
			$n_count++;
			$self->{dimensions}{"x$n_count"} = Dimension->new($column->get_number(), $column);
			$self->{dimensions}{"x$n_count"}->set_name("x$n_count");
			$self->{dimensions}{"x$n_count"}->set_tol(0.5);
			$self->{dimensions}{"x$n_count"}->set_shift(0.00);
			$self->{dimensions}{"x$n_count"}->set_sign(0);
#			$self->{dimensions}{"x$n_count"}->set_sw(1000.00);
			Utils->debug("assigned column x$n_count");
		} elsif ($min and $max and $ratio and $ratio > 100 and ($log_skewness > 0 and $log_kurtosis < 10 and $log_kurtosis > -2)) {
			$self->{height} = $column;
			$i_count++;
			Utils->debug("match intensity: " . $self->{height}->get_number());
		}
	}
	my $name = $self->{name};
	Utils->debug("hcount $h_count, ncount: $n_count, dcount: " . $self->{dimension_count});
	(($h_count + $n_count == $self->{dimension_count}) and ($i_count == 1)) or (die "Unable to parse $name peaklist");
}
sub assign_hx_columns {
	my $self = shift;
	my $intensity_col = $self->{height};
	my ($x1, $hx1, $hx2, $x2_hxa_size, $x2_hxb_size);
	for (my $i = 0; $i < $intensity_col->get_size(); $i++) {
#		Utils->debug($self->{dimensions}{hxb}->get($i));
		$self->{hxa_buckets}->put($self->{dimensions}{x1}->get($i), $self->{dimensions}{hxa}->get($i));
		$self->{hxb_buckets}->put($self->{dimensions}{x1}->get($i), $self->{dimensions}{hxb}->get($i));
	}
	my $x1_hxa_size = $self->{hxa_buckets}->get_size();
	my $x1_hxb_size = $self->{hxb_buckets}->get_size();
	if (defined $self->{dimensions}{x2}) {
		$self->{hxa_buckets} = Peaks->new();
		$self->{hxb_buckets} = Peaks->new();
		for (my $i = 0; $i < $intensity_col->get_size(); $i++) {
	#		Utils->debug($self->{dimensions}{hxb}->get($i));
			$self->{hxa_buckets}->put($self->{dimensions}{x2}->get($i), $self->{dimensions}{hxa}->get($i));
			$self->{hxb_buckets}->put($self->{dimensions}{x2}->get($i), $self->{dimensions}{hxb}->get($i));
		}
		$x2_hxa_size = $self->{hxa_buckets}->get_size();
		$x2_hxb_size = $self->{hxb_buckets}->get_size();
	} else {
		$x2_hxa_size = 999999999999999999999999;
		$x2_hxb_size = 999999999999999999999999;
	}
	Utils->debug("MIN: " . min($x1_hxa_size, $x1_hxb_size, $x2_hxa_size, $x2_hxb_size));
	if ($x1_hxa_size == min($x1_hxa_size, $x1_hxb_size, $x2_hxa_size, $x2_hxb_size)) {
		$hx1 = $self->{dimensions}{hxa}->get_number();
		$hx2 = $self->{dimensions}{hxb}->get_number();
		$self->{dimensions}{hxa}->set_name("hx1");
		$self->{dimensions}{hxb}->set_name("hx2");
	} elsif ($x1_hxb_size == min($x1_hxb_size, $x2_hxa_size, $x2_hxb_size)) {
		$hx1 = $self->{dimensions}{hxb}->get_number();
		$hx2 = $self->{dimensions}{hxa}->get_number();
		$self->{dimensions}{hxa}->set_name("hx2");
		$self->{dimensions}{hxb}->set_name("hx1");
	} elsif ($x2_hxa_size and $x2_hxa_size == min($x2_hxa_size, $x2_hxb_size)) {
		$hx1 = $self->{dimensions}{hxb}->get_number();
		$hx2 = $self->{dimensions}{hxa}->get_number();
		$self->{dimensions}{hxa}->set_name("hx2");
		$self->{dimensions}{hxb}->set_name("hx1");
	} elsif ($x2_hxb_size) {
		$hx1 = $self->{dimensions}{hxa}->get_number();
		$hx2 = $self->{dimensions}{hxb}->get_number();
		$self->{dimensions}{hxa}->set_name("hx1");
		$self->{dimensions}{hxb}->set_name("hx2");
	} else {
		die "unable to assign";
	}
	Utils->debug($x1_hxa_size . " : " . $x1_hxb_size);
	$x1 = $self->{dimensions}{x1}->get_number();
	#Utils->debug("<br/><br/><hr/>x1: $x1, hx1: $hx1, hx2: $hx2<br/>" .$self->{dimensions}{x1}->get_name() . ", " . $self->{dimensions}{hxa}->get_name() . ", " . $self->{dimensions}{hxb}->get_name() . "<br/><hr/><br/>");
}
sub parse_columns {
	my $self = shift;
	my $filepath = $self->{filepath};
	open ( FILE, "<$filepath" ) or die "$!";
	binmode FILE;
    my $rec_num = 0;
    my $temp_line = "";
	my $line_no = 0;
	while( chomp($temp_line = <FILE>)) {
		$temp_line =~ s/^\s+//;
		$temp_line =~ s/\s+$//;
		if ($line_no++ < 10) {
			my $temp_line2 = $temp_line;
			$temp_line2 =~ s/^#\s+//;
			my @itemz = split(/(?<!\#)\s+(?!Height)/, $temp_line2);
			push(@{$self->{first_five_lines}}, \@itemz);
		}
		next if (($temp_line =~ /^\#/) or ($temp_line =~ /^\s*$/) or $temp_line =~ /assignment/i);
		$rec_num++;
		my @items = split(/\s+/, $temp_line);
		$self->{col_count} = max($self->{col_count}, scalar(@items));
		Utils->debug(scalar(@items));
		Utils->debug("number of columns found in peaklist: " . $self->{col_count});
		my $i = 1;
		for my $col (@items) {
			unless (defined $self->{columns}{$i}) {
				Utils->debug("adding new column to peaklist: $i, col_count is " . $self->{col_count});
				$self->{columns}{$i} = DataColumn->new($i);
			}
			$item = $items[$i - 1];
			$item =~ s/^\s+//;
			$item =~ s/\s+$//;
			#Utils->debug("adding $item to column $i");
			$self->{columns}{$i}->push($item);
			$i++;
		}
#		Utils->debug($items[5]);
    }
	my $i = 0;
	for ($i = 1; $i <= $self->{col_count}; $i++) {
		Utils->debug($self->{columns}{$i});
		Utils->debug($self->{columns}{$i}->get_size());
		Utils->debug("(max, min, sw, mean, std) for col $i: " . $self->{columns}{$i}->get_max() . ", " . $self->{columns}{$i}->get_min() . ", " . $self->{columns}{$i}->get_sw() . ", " . $self->{columns}{$i}->get_mean() . ", " . $self->{columns}{$i}->get_std_dev());
	}
	#Utils->debug("--------------------------------------------------");
    close(FILE);
}
sub get_dimension_by_col_num {
	my $self = shift;
	my $col_num = shift;
	my $dimension;
	if ($self->{height}->get_number() == $col_num) {
		return $self->{height};
	}
	foreach my $dim_name ( keys %{$self->{dimensions}} ) {
		$dimension = $self->{dimensions}{$dim_name};
#			print "dimnumber is: " . $dimension->get_number() . " colnumber is $col_num";
		if ($dimension->get_number() == $col_num) {
			return $dimension;
		}
	}
	return undef;
}
sub get_column_type_by_name {
	my $self = shift;
	my $name = shift;
	if (defined $self->{type_by_name}{$name}) {
		return $self->{type_by_name}{$name};
	}
	return undef;
}
sub get_column_by_name {
	my $self = shift;
	my $name = shift;
	my $hash = $self->{shift_columns};
	foreach my $key ( keys %$hash ) {
		my $column = $self->{shift_columns}{$key};
		if ($name =~ $peaklist->get_name()) {
			return $peaklist->get_type();
		}
	}
	return undef;
}
sub get_name {
	my $self = shift;
	return $self->{name};
}
sub get_filename {
	my $self = shift;
	return $self->{filename};
}
sub get_filepath {
	my $self = shift;
	return $self->{filepath};
}
sub set_filename {
	my $self = shift;
	$self->{filename} = shift;
}
sub set_filepath {
	my $self = shift;
	$self->{filepath} = shift;
}
sub is_in_water {
	my $self = shift;
	return $self->{waterFlag};
}
sub is_intrachain {
	my $self = shift;
	return $self->{intrachain};
}
sub is_interchain {
	my $self = shift;
	return $self->{interchain};
}
sub get_max_mean_column {
	my $self = shift;
	my $hash = $self->{shift_columns};
	my ($biggest_column, $mean);
	my $biggest_mean = 0.0;
	foreach my $key (keys %$hash) {
		$mean = $self->{shift_columns}{$key}->get_mean();
		if ($mean > $biggest_mean) {
			$biggest_mean = $mean;
			$biggest_column = $self->{shift_columns}{$key};
		}
	}
	return $biggest_column;
}
sub get_min_std_column {
	my $self = shift;
	my $hash = $self->{shift_columns};
	my ($smallest_column, $mean);
	my $smallest_std = 9999999;
	foreach my $key (keys %$hash) {
		$std = $self->{shift_columns}{$key}->get_std();
		if ($std > $smallest_std) {
			$smallest_std = $std;
			$smallest_column = $self->{shift_columns}{$key};
		}
	}
	return $smallest_column;
}
sub get_section {
	$self = shift;
	$str = "";
#	$str .= "[" . $self->{name} . "]\n";
	$str .= "[" . $self->get_filename() ."]\n";
	$str .= "dimension=" . $self->{dimension_count} . "\n";
	$str .= "haveIC=" . $self->{intrachain} . "\n";
# 	if ($self->{intrachain}) {
# 		Utils->debug("intrachain is set for " . $self->{name});
# 		$str .= "IC=1\n";
# 	} else {
# 		$str .= "IC=0\n";
# 	}
	$str .= "IC=" . $self->{interchain} . "\n";
# 	if ($self->{interchain}) {
# 		Utils->debug("interchain is set for " . $self->{name});
# 		$str .= "haveIC=1\n";
# 	} else {
# 		$str .= "haveIC=0\n";
# 	}
	$str .= "waterFlag=" . $self->{waterFlag} . "\n";
# 	if ($self->{waterFlag}) {
# 		$str .= "waterFlag=0\n";
# 	} else {
# 		$str .= "waterFlag=1\n";
# 	}
	if ($self->{water}) {
		$str .= "water=" . $self->{water} . "\n";
	}
#	$str .= "col.label=" . $self->{label}->get_number() . "\n";
	$str .= "col.intensity=" . $self->{height}->get_number() . "\n";
	$tmp_hash = $self->{dimensions};
	foreach my $key ( keys(%$tmp_hash) ) {
		my $dimension = $self->{dimensions}{$key};
		$str .= "col." . $dimension->get_name() . "=" . $dimension->get_number() . "\n";
		$str .= $dimension->get_name() . ".sw=" . $dimension->get_sw() . "\n";
		$str .= $dimension->get_name() . ".sign=" . $dimension->get_sign() . "\n";
		$str .= $dimension->get_name() . ".type=" . $dimension->get_type() . "\n";
		$str .= $dimension->get_name() . ".shift=" . $dimension->get_shift() . "\n";
		$str .= $dimension->get_name() . ".tol=" . $dimension->get_tol() . "\n";
	}
	return $str;
}
return(1); 
