package Results;

use ControlFile;
use Utils;
use Rasmol;
use File::Path;
use POSIX qw( strftime );
use Cwd;

sub new {
	my $class = shift;
	my $self = {};
	$self->{base_dir} = shift;
	$self->{results_dir} = shift;#$self->{session}->get('results_dir');
	$self->{protein_name} = shift;#$self->{session}->get('protein_name');
	$self->{pdb} = shift;
	$self->{peaklists} = {};
	$self->{ql_data} = ();
	$self->{ql_heading} = ();
	$self->{ql_residue} = ();
	$self->{name} = "boogy";
	bless($self, $class);
	$self->prepare_dataset();
	return $self;
}
sub prepare_dataset {
	my $self = shift;
	my $dir = $self->{results_dir};
	opendir(DIR, "$dir/source");
	my @contents = readdir(DIR);
	foreach my $file (@contents) {
		if ($file =~ /(.*)\.gz$/) {
			#print "gunzip -c $dir/source/$file > $1<br/>";
			system("gunzip -c $dir/source/$file > $1");
		}
	}
	closedir(DIR);
	opendir(DIR, $dir);
	my ($ovw_file) = grep {/ovw$/} readdir(DIR);
	$ovw_file or die "Overview file not found.";
	open(FILE, "$dir/$ovw_file");
	while (my $line = <FILE>) {
		if ($line =~ /\#\s*-c\s+(\S+)\/([^\/]+)\s*$/i) {
			$self->{control_file} = $2;
		}
		if ($line =~ /\#\s*-q\s+(\S+)\/([^\/]+)\s*$/i) {
			#print $2;
			$self->{pdb} = $2;
		}
	}
	close(FILE);
}
sub get_path {
	my $self = shift;
	my $type = shift;
	my $path = $self->{base_dir} . "/" . $self->{results_dir};
	unless (opendir(DIR, $path)){
		print STDERR "Cannot open $path\n";
		return;
	}
	my @contents = readdir(DIR);
	close(DIR);
	foreach $key ( @contents ) {
		if ($key =~ /$type/) {
			return $path . "/" . $key;
		}
	}
	#print "get $type path: $path <br/>";
	return $path;
}
sub get_ovw_path {
	my $self = shift;
	my $path = $self->{base_dir} . "/" . $self->{results_dir};
	unless (opendir(DIR, $path)){
		print STDERR "Cannot open $path\n";
		return;
	}
	my @contents = readdir(DIR);
	close(DIR);
	foreach $key ( @contents ) {
		if ($key =~ /ovw$/i) {
			return $path . "/" . $key;
		}
	}
# 	print "get_ovw_path: $path <br/>";
	return $path;
}
sub get_log_path {
	my $self = shift;
	my $path = $self->{base_dir} . "/" . $self->{results_dir};
# 	print "checking dir: $path";
	unless (opendir(DIR, $path)){
		print "Cannot open $path\n";
		return;
	}
	my @contents = readdir(DIR);
	close(DIR);
	foreach $key ( @contents ) {
		if ($key =~ /log$/i) {
			return $path . "/" . $key;
		}
	}
# 	print "get_log_path: $path <br/>";
	return $path;
}
sub parse_ovw {
	my $self = shift;
	my $file = $self->get_ovw_path();
	my @pklsts_fnames;
	my $pkl_name;
	my %data;
	my $summary = 0;
	my $query;
	my $precision;
	my $summary_line = 0;

	unless (-e $file) {
		print "$file doesnt exist!!\n";
		return;
	}
	unless (open(FILE, $file)) {
		print "Cannot open rpf ovw file \"$file\"<br/>";
		return;
	}
	my $line;
	#start reading input
	while (chomp($line = <FILE>)) {
#		print "$line<br/>";
		if ($line =~ /NOESY_Assign Version ([\d\.]+) /) {
			my $version = $1; 
			if ($version =~ /2\.1\./) {
				print "This is a $version version output. Please use an older version to open it<br/>";
			}
		}
		last if ($line =~ /^(\s*)final/i);
		if ($line =~ /^\#/) {
			$query = 'query';
			if ($line =~ /summary/i) {
				$line =~ /summary for (.+)/i;
				$pkl_name = $1;
				if ($pkl_name=~ /:$/) {
					$pkl_name =~s/:$//;
				}
				push @pklsts_fnames, $pkl_name;
				$summary = 1;
				$summary_line = 0;
			} elsif ($line =~ /for peaklist (.+)/i) {
				$pkl_name = $1;
				if ($pkl_name =~ /:$/) {
					$pkl_name=~s/:$//;
				}
				Utils->debug("pkl_name is: $pkl_name");
				$summary = 0;
			}
		} else {
			#line is data
			if ($summary) {
				if ($line =~ /m score/i) {
					my ($m, $score) = split(/=/, $line);
					$data{$pkl_name}{'M-score'} = $score;
				} elsif ($line =~ /average/i) {
					$data{$pkl_name}{'summary'}[$summary_line] = $line;
					$summary_line++;
				}
			} else {
				#data for each peaklist
				if ($line =~ /freely-rotating-chain/i) {
					$query = 'freely';
					$precision = 0;
					next;
				}
				if ($line =~ /^(\s*)recall/i) {
					my ($recall, $num) = split(/:/, $line);
					($num, my $other) = split(' ', $num);
					$data{$pkl_name}{$query}{'recall'} = $num;
					Utils->debug("data.$pkl_name.$query.recall = " . $num);
					$precision = 0;
					next;
				} elsif ($line =~ /^(\s*)precision/i) {
					$precision = 1;
					next;
				} elsif ($line =~ /^(\s*)f-measure/i) {
					$precision = 0;
					my ($fmeasure, $score) = split(' ', $line);
					$data{$pkl_name}{$query}{'F-measure'}{$fmeasure} = $score;
					Utils->debug("data.$pkl_name.$query.F-measure.$fmeasure = " . $score);
				} elsif ($line =~ /^\s*$/) {
					$precision = 0;
					next;
				}
				if ($precision == 1) {
					$line =~ /\s*(.+):\s*(.+)\s+(.+):\s*(.+)$/;
					$data{$pkl_name}{$query}{'precision'}{$1} = $2;
					Utils->debug("data.$pkl_name.$query." . $1 . " = " . $2);
					$data{$pkl_name}{$query}{'precision'}{$3} = $4;
					Utils->debug("data.$pkl_name.$query." . $3 . " = " . $4);
				}
			}
		}
	}
	#final reports 
	my (@vars) = split(' ', $line);
	$data{'final'}{'m-score'} = $vars[2];
	while (chomp ($line = <FILE>)) {
		next if ($line =~ /^\s*$/); 
		$line =~ /Model ([\d\w]+)/; 
		my $model = $1; 
		Utils->debug("model = $model<br/>");

		while (chomp($line = <FILE>)) {
			next if ($line =~ /^\s*$/);
			last if ($line =~ /summary|dp/i);
			$line =~ /final (.+) for (.+): (.+)$/i;
			$data{model}{$model}{$1}{$2} = $3;
			Utils->debug("data.model." . $1 . "." . $2 . " = " . $3);
		}

		#summary F-scores
		while (chomp($line = <FILE>)) {
			next if ($line =~ /^\s*$/);
			last if ($line =~ /dp/i);
			$line =~ /F-score of (.+):\s*(.+)$/i;
			$data{model}{$model}{'F-score'}{$1} = $2;
		}
		my ($dp, $score) = split(/:/, $line);
		#$data{'final'}{'dp-score'} = $score;
		$data{model}{$model}{'dp-score'}{'input query structures'} = $score;
	}
	return (\%data, \@pklsts_fnames);
}
sub print_compact_rpf_xml_scores {
	my ($self, $data, $pklsts_fnames) = @_;
	print "<rpf_scores protein_name='" . $self->{protein_name} . "'>\n";
	my $fin_msc = $data->{'final'}{'m-score'};
	my $fin_dp = $data->{'model'}{'ave'}{'dp-score'}{'input query structures'};
	print "<summary>\n";
	print "\t<m_scores>\n"; 
	print "\t\t<m_score type=\"final\" value=\"$fin_msc\"></m_score>\n";
	print "\t</m_scores>\n";
	print "\t<recall_scores>\n";
	print "\t\t<recall_score type=\"query\" value=\"" . $data->{'model'}{'ave'}{'Recall-score'}{'input query structures'} . "\"></recall_score>\n";
	print "\t\t<recall_score type=\"freely_rotating\" value=\"" . $data->{'model'}{'ave'}{'Recall-score'}{'freely-rotating-chain model'} . "\"></recall_score>\n";
	print "\t\t<recall_score type=\"ideal\" value=\"" . $data->{'model'}{'ave'}{'Recall-score'}{'ideal true structures'} . "\"></recall_score>\n";
	print "\t</recall_scores>\n";
	print "\t<precision_scores>\n";
	print "\t\t<precision_score type=\"query\" value=\"" . $data->{'model'}{'ave'}{'Precision-score'}{'input query structures'} . "\"></precision_score>\n";
	print "\t\t<precision_score type=\"freely_rotating\" value=\"" . $data->{'model'}{'ave'}{'Precision-score'}{'freely-rotating-chain model'} . "\"></precision_score>\n";
	print "\t\t<precision_score type=\"ideal\" value=\"" . $data->{'model'}{'ave'}{'Precision-score'}{'ideal true structures'} . "\"></precision_score>\n";
	print "\t</precision_scores>\n";
	print "\t<f_scores>\n";
	print "\t\t<f_score type=\"query\" value=\"" . $data->{'model'}{'ave'}{'F-score'}{'input query structures'} . "\"></f_score>\n";
	print "\t\t<f_score type=\"freely_rotating\" value=\"" . $data->{'model'}{'ave'}{'F-score'}{'freely-rotating-chain model'} . "\"></f_score>\n";
	print "\t\t<f_score type=\"ideal\" value=\"" . $data->{'model'}{'ave'}{'F-score'}{'ideal true structures'} . "\"></f_score>\n";
	print "\t</f_scores>\n";
	print "\t<dp_scores>\n";
	print "\t\t<dp_score type=\"final\" value=\"$fin_dp\"></dp_score>\n";
	print "\t</dp_scores>\n";
	print "</summary>\n";
	print "</rpf_scores>\n";	#end first tab
}
sub print_rpf_scores {
	my ($self, $data, $pklsts_fnames) = @_;
	print "<div id='summary' class='resulttabs'>\n";
	print "<div id='mol3d'>";
	$self->show_fp_distributions();
	print "</div><div id='summary_scores'>";
	my $fin_msc = $data->{'final'}{'m-score'};
	my $fin_dp = $data->{'model'}{'ave'}{'dp-score'}{'input query structures'};
	print "<div class=\"label\"><label class='key'>Final m-score:</label><label class='value'>$fin_msc</label></div>\n";
	#create a table
	#table headings
	print "<div id='summary_table_div' class='table'><table cellpadding='0' cellspacing='0' border='0' class='display' id='summary_table'><thead>\n";
	my $i = 0;
	print "<tr class='row'>\n";
	foreach ('Score type', 'Query', 'Freely Rotating', 'Ideal') {
		print "<th class=\"col\">$_</th>\n";
	}
	print "</tr></thead><tbody>\n";
	$i = 1;
	$j = 1;
	foreach $row ('Recall-score', 'Precision-score', 'F-score') {
		$j = 1;
		print "\t<tr class='row'>\n\t\t<td class=\"col\">$row</td>\n";
		foreach $col ('input query structures', 'freely-rotating-chain model', 'ideal true structures') {
			print "\t\t<td class=\"col\">" . $data->{'model'}{'ave'}{$row}{$col} . "</td>\n";
			$j++;
		}
		print "\t</tr>\n";
		$i++;
	}
	print "</tbody></table></div>\n";
	print "<div class=\"label\"><label class='key'>Final dp-score(midrange distances):</label><label class='value'>$fin_dp</label></div>\n";
	print "</div></div>\n";	#end first tab

	my $pl;
	my $pl_num = 1;
# 
# 	#add a 'single model' tab
	print "<div id='single-model' class='resulttabs hidetab'>\n";
# 
# 	#create a table
	my $rowNum = keys %{$data->{'model'}};

	#table headings
	print "<div id='singlemodel_table_div' class='table'><table cellpadding='0' cellspacing='0' border='0' class='display' id='singlemodel_table'>\n\t<thead><tr class='row'>\n";
	$i = 0;
	foreach ('Model #', 'Recall', 'Precision', 'F-measure', 'DP') {
		print "\t\t<th class='col'>" . $_ . "</th>\n";
		$i++;
	}
	print "\t</tr></thead><tbody>\n";	#end row
	$i = 1;
	foreach (sort {$a<=>$b} keys %{$data->{'model'}}) {
		next if ($_ eq 'ave'); 
# 		print "<div class='col'>" . $_ . "</div>\n";
		$i++;
	}

	$i = 1;
	$j = 1;
	my $dpsum = 0; 
	foreach $row (sort {$a<=>$b} keys %{$data->{'model'}}) {
		next if ($row eq 'ave'); 
		$j = 1;
		print "\t<tr class='row'>\n\t\t<td class='col'>$row</td>\n";
		foreach $col ('Recall-score', 'Precision-score', 'F-score', 'dp-score') { 
			print "\t\t<td class='col'>" . $data->{'model'}{$row}{$col}{'input query structures'} . "</td>\n";
			$j++;
			if ($col eq 'dp-score') {
				$dpsum += $data->{'model'}{$row}{$col}{'input query structures'};
			}
		}
		$i++;
		print "\t</tr>\n";
	}
	print "</tbody><tfoot></tfoot></table></div>\n";	#end single model table

	$rowNum++ if ($rowNum == 1);
	my $adp = $dpsum/($rowNum-1); 
	my $str = sprintf("%.3f", $adp);
	print "\t<div class='label'><label class='key'>Average dp-score(single):</label><label class='value'>$str</label></div>\n";
	print "</div>\n";		#end single model tab

	#Add one page for each peaklist
	foreach $pl (@$pklsts_fnames) {
		next unless $pl;
		print "<div id='peaklist$pl_num' class='resulttabs hidetab'>\n";
		print "\t<div class='label'><label class='key'>Peaklist FileName:</label><label class='value'>$pl</label></div>\n";
		my $mscr = ${$data->{$pl}}{'M-score'};
		print "\t<div class='label'><label class='key'>M-Score:</label><label class='value'>$mscr</label></div>\n";

		#summary lines
		my $sum_txt = "<div class='label'><label class='value'>".${$data->{$pl}}{'summary'}[0]."</label></div>";
		$sum_txt .= "<div class='label'><label class='value'>".${$data->{$pl}}{'summary'}[1]."</label></div>";
		$sum_txt .= "<div class='label'><label class='value'>".${$data->{$pl}}{'summary'}[2]."</label></div>";
		print "\t<div class='label'>$sum_txt</div>\n";

		print "<div id='peaklist_table_div' class='table'><table cellpadding='0' cellspacing='0' border='0' class='display peaklist_table' id='${pl}_peaklist_table'><thead>\n";
		my $k = 0;
		my $title;
		print "\t<tr class='row'>\n";
		foreach ('Score type', 'Query', 'Freely Rotating', 'Ideal') {
			print "\t\t<th class='col'>$_</th>\n";
			$k++;
		}
		print "\t</tr></thead><tbody>\n";

		print "\t<tr class='row'>\n";
		print "\t\t<td class='col'>Recall</td>\n";
		print "\t\t<td class='col'>" . ${$data->{$pl}}{'query'}{'recall'} . "</td>\n";
		print "\t\t<td class='col'>" . ${$data->{$pl}}{'freely'}{'recall'} . "</td>\n";
		print "\t\t<td class='col'>1</td>\n";
		print "\t</tr>\n";

		print "<tr class='row'>\n";
		print "<td class='col'>Precision</td>\n";
		#calculate precision
		my $f_score = ${$data->{$pl}}{'query'}{'F-measure'}{'F-measure:(0,PrecisionIdealW)'};
		my $precision = ($f_score)/(2 - $f_score);
		$precision = sprintf("%0.3f", $precision);
		print "<td class='col'>" . ${$data->{$pl}}{'query'}{'precision'}{'allw'} . "</td>\n";
		print "<td class='col'>" . ${$data->{$pl}}{'freely'}{'precision'}{'allw'} . "</td>\n";
		print "<td class='col'>" . $precision . "</td>\n";
		print "</tr>\n";

		print "<tr class='row'>\n";
		print "<td class='col'>F-measure</td>\n";
		print "<td class='col'>" . ${$data->{$pl}}{'query'}{'F-measure'}{'F-measure:(Recall,PrecisionW)'} . "</td>\n";
		print "<td class='col'>" . ${$data->{$pl}}{'freely'}{'F-measure'}{'F-measure:(ReallFree,PrecisionFreeW)'} . "</td>\n";
		print "<td class='col'>" . ${$data->{$pl}}{'query'}{'F-measure'}{'F-measure:(0,PrecisionIdealW)'} . "</td>\n";
		print "</tr>\n";

		print "</tbody></table></div>\n";		#end peaklist table
		$pl_num++;
		print "</div>\n";		#end peaklist tab
	}


# 	#add a 'Missing atoms' tab
# 	my $not_pg = $nb->add('Notes', -label=>'Missing Atoms');
# 	$not_pg->configure(-background=>$nb_bg);
# 
# 	my $txt_area = $not_pg->Scrolled('Text', -scrollbars=>'e', -background=>'white', -height=>10)->pack(-fill=>'x', -expand=>1, -padx=>3, -pady=>5);
# 	my $txt = $txt_area->Subwidget('text');
# 
# 	return (\%fpd_but, $ql_but, $qi_but, $nb, $txt);

}
sub parse_ql {
	my $self = shift;
    my $file = $self->get_path("QP");
    #two dimensional array with each element as a hash having three keys - distance, residue number, and atom name
    my @data;
    #my $data_count = 0;
    my $first_line_seen;
    my $pair_seen;
    my $section = -1;
    my @section_heading;
    my @atom;
    my $count = 0;

	my $sparky_line = "";
	my ($hx1_sparky, $hx2_sparky, $x1_sparky);

    my $other_first_line;

    my ($first, $first_name, $second, $second_name, $distance);
    my ($i, $j, $option);
    unless (-e $file) {
		print "No QL file specified\n";
		return;
    }
    my $opened = open(QL, $file);
    unless ($opened) {
		print "Cannot open QL file \"$file\"";
		return;
    }
	print "<table><thead><tr><th>Atom1</th><th>Atom2</th><th>Max Distance</th></tr></thead><tbody><tr>";
	print "<th><input type='text' name='search_residue1' value='Regex Atom1' class='search_init regex_search' /></th>";
	print "<th><input type='text' name='search_residue2' value='Regex Atom2' class='search_init regex_search' /></th>";
	print "<th><input type='text' name='search_distance' value='Set Max Distance' class='search_init' id='max_distance' /></th>";
	print "</tr></tbody></table>";
    #read until first line
#	print "<div class='table'><table cellpadding='0' cellspacing='0' border='0' class='display' id='fp_interactions_table'><thead><tr><th>Peaklist</th><th>Residue1</th><th>Residue2</th><th>distance</th><th>test</th></tr></thead><tbody>";
    while (chomp($line = <QL>)) {
		if ($line =~ /missing/i) {
			$line =~ /missing .+ for (.+)$/i;
			#print "section $section<br/>";
			unless ($section == -1) {
				print "</tbody>";
				#print "<tfoot><tr>";
				#print "<th><input type='text' name='search_peaklist' value='Search Peaklist' class='search_init regex_search' peaklist='". $section_heading[$section] ."' /></th>";
				#print "<th><input type='text' name='search_residue1' value='Search Residue1' class='search_init regex_search' peaklist='". $section_heading[$section] ."' /></th>";
				#print "<th><input type='text' name='search_residue2' value='Search Residue2' class='search_init regex_search' peaklist='". $section_heading[$section] ."' /></th>";
				#print "<th><input type='text' name='search_distance' value='Upper Limit' class='search_init max_distance' id='max_distance' peaklist='". $section_heading[$section] ."' /></th>";
				#print "</tr></tfoot>";
				print "</table>";
				print "<br/>";
				print "</div><br/>";
			}
			$section++;
			$section_heading[$section] = $1;

			#$data_count = 0;
			$first_line_seen = 0;
			$other_first_line = 1;
			$pair_seen = 0;
			
			print "<div class='fp_table_div'><div class='header'>" . $section_heading[$section] . "<input type='submit' name='save' value='save as sparky' onclick='return getNodes(\"" . $section_heading[$section] . "\");' /></div><table cellpadding='0' cellspacing='0' border='0' class='display fp_interactions_table' id='" . $section_heading[$section] . "'><thead><tr><th>Atom1</th><th>Atom2</th><th>distance</th><th>test</th></tr></thead><tbody>";
		} elsif ($line =~ /^atom/i) {
			#only look at first pair
			if (!$pair_seen) {
				if (!$first_line_seen) {
					#first line
					$line =~ /^atom:\s*\d+\s*((\d+)(\w)-(\w+))\s+(\S)+\s+(\S+).*\{\s*(.+)\s*\}/;
					$atom1_sparky = $1;
					$hx1_sparky = $6;
					$x1_sparky = $7;
					$first = $2;
					$first_name = $4;
					$first_line_seen = 1;
				} else {
					#second line
					$line =~ /^atom:\s*\d+\s*((\d+)(\w)-(\w+))\s+(\S)+\s+(\S+).*\{\s*(.+)\s*\}/;
					$atom2_sparky = $1;
					$hx2_sparky = $6;
					$second = $2;
					$second_name = $4;
					$pair_seen = 1;
					$sparky_line .= "$first;$second;$hx1_sparky;$x1_sparky;$hx2_sparky";
				}
			} else {
				#other lines
				if ($other_first_line) {
					#		   atom:   4        3    E - QG       G
					$line =~ /^atom:\s*\d+\s*((\d+)(\w)-(\w+))\s+(\S)+\s+(\S+).*\{\s*(.+)\s*\}/;
					$first = $2;
					my $amb_atom = $4;
					$atom1_sparky = $1;
					$hx1_sparky = $6;
					$x1_sparky = $7;
					unless ($amb_atom eq $first_name) {
						$amb_atom =~ /(\w+)(\d)$/;
						my $amb_num = $2;
						my $amb_atm = $1;
						$first_name =~ /(\w+)(\d)$/;
						my $fir_num = $2;
						my $fir_atm = $1;

						if ($amb_num and $fir_num) {
							if (($amb_atm eq $fir_atm) and ($amb_num != $fir_num)) {
								$first_name = $amb_atm."*";
							}
						}
					}
				} else {
					#second line of second atom
					$line =~ /^atom:\s*\d+\s*((\d+)\w-(\w+))\s+(\S)+\s+(\S+).*\{\s*(.+)\s*\}/;
					$second = $2;
					my $amb_atom = $3;
					$atom2_sparky = $1;
					$hx2_sparky = $5;
					unless ($amb_atom eq $second_name) {
						$amb_atom =~ /(\w+)(\d)$/;
						my $amb_num = $2;
						my $amb_atm = $1;
						$second_name =~ /(\w+)(\d)$/;
						my $sec_num = $2;
						my $sec_atm = $1;

						if ($amb_num and $sec_num) {
							if (($amb_atm eq $sec_atm) and ($amb_num != $sec_num)) {
								$second_name = $amb_atm."*";
							}
						}
					}
					$sparky_line .= "EOL$first;$second;$hx1_sparky;$x1_sparky;$hx2_sparky";
				}
				$other_first_line = !$other_first_line;
			}
			
			#store atom and number in number_to_atom array
			$line =~ /^atom:\s*\d+\s+(\d+)([a-zA-Z])-.+/;
			$atom[$1] = $2;
		} elsif ($line =~ /^::sumavg:\s*(.+)$/i) {
			$distance = $1;
			unless (defined ($data[$section][$first]{$first_name}[$second]{$second_name})) {
				$data[$section][$first]{$first_name}[$second]{$second_name} = $distance;
			#print "parse_ql: data[$section][$first]{$first_name}[$second]{$second_name} = $distance\n";
				print "<tr><td>".$first.$atom[$first]."-$first_name</td><td>$second".$atom[$second] ."-$second_name</td><td>$distance</td><td>$sparky_line</td></tr>\n";
			#print "data[$section][$ql_data{'1-residue number'}]{$ql_data{'1-atom name'}}[$ql_data{'2-residue number'}]{$ql_data{'2-atom name'}} = $ql_data{'distance'}\n";
			#print "data[$section][$ql_data{'2-residue number'}]{$ql_data{'2-atom name'}}[$ql_data{'1-residue number'}]{$ql_data{'1-atom name'}} = $ql_data{'distance'}\n";
			} else {
# 				print "<tr><td>dup ** ".$section_heading[$section]."</td><td>".$first.$atom[$first]."-$first_name</td><td>$second".$atom[$second] ."-$second_name</td><td>$distance</td></tr>\n";
			}
			#$data_count++;
			$first_line_seen = 0;
			$other_first_line = 1;
			$pair_seen = 0;
			$sparky_line = "";
		}
    }
	print "</tbody>";
	#print "<tfoot><tr>";
	#print "<th><input type='text' name='search_peaklist' value='Search Peaklist' class='search_init regex_search' /></th>";
	#print "<th><input type='text' name='search_residue1' value='Search Residue1' class='search_init regex_search' /></th>";
	#print "<th><input type='text' name='search_residue2' value='Search Residue2' class='search_init regex_search' /></th>";
	#print "<th><input type='text' name='search_distance' value='Upper Limit' class='search_init' id='max_distance' /></th>";
	#print "</tr></tfoot>";
	print "</table>";
	print "<br/>";
	print "</div>";
	$self->{ql_data} = \@data;
	$self->{ql_section} = \@section_heading;
	$self->{ql_residue} = \@atom;
#	my ($data, $section, $residue) = ($self->{ql_data}, $self->{ql_section}, $self->{ql_residue});
    return ($self->{ql_data}, $self->{ql_section}, $self->{ql_residue});
}

sub get_ql_parse_data {
	my $self = shift;
    my $file = $self->get_path("QP");
    #two dimensional array with each element as a hash having three keys - distance, residue number, and atom name
    my @data;
    #my $data_count = 0;
    my $first_line_seen;
    my $pair_seen;
    my $section = -1;
    my @section_heading;
    my @atom;
    my $count = 0;

    my $other_first_line;

    my ($first, $first_name, $second, $second_name, $distance);
    my ($i, $j, $option);
    unless (-e $file) {
		print "No QL file specified\n";
		return;
    }
    my $opened = open(QL, $file);
    unless ($opened) {
		print "Cannot open QL file \"$file\"";
		return;
    }
    #read until first line
    while (chomp($line = <QL>)) {
		if ($line =~ /missing/i) {
			$line =~ /missing .+ for (.+)$/i;
			
			$section++;
			$section_heading[$section] = $1;

			#$data_count = 0;
			$first_line_seen = 0;
			$other_first_line = 1;
			$pair_seen = 0;
		} elsif ($line =~ /^atom/i) {
			#only look at first pair
			if (!$pair_seen) {
				if (!$first_line_seen) {
					#first line
					$line =~ /^atom:\s*\d+\s*(\d+)\w-(\w+)\s*.*/;
					$first = $1;
					$first_name = $2;
					$first_line_seen = 1;
				} else {
					#second line
					$line =~ /^atom:\s*\d+\s*(\d+)\w-(\w+)\s*.*/;
					$second = $1;
					$second_name = $2;
					$pair_seen = 1;
				}
			} else {
				#other lines
				if ($other_first_line) {
					$line =~ /^atom:\s*\d+\s*(\d+)\w-(\w+)\s*.*/;
					my $amb_atom = $2;
					
					unless ($amb_atom eq $first_name) {
						$amb_atom =~ /(\w+)(\d)$/;
						my $amb_num = $2;
						my $amb_atm = $1;
						$first_name =~ /(\w+)(\d)$/;
						my $fir_num = $2;
						my $fir_atm = $1;

						if ($amb_num and $fir_num) {
							if (($amb_atm eq $fir_atm) and ($amb_num != $fir_num)) {
								$first_name = $amb_atm."*";
							}
						}
					}
				} else {
					$line =~ /^atom:\s*\d+\s*(\d+)\w-(\w+)\s*.*/;
					my $amb_atom = $2;
					
					unless ($amb_atom eq $second_name) {
						$amb_atom =~ /(\w+)(\d)$/;
						my $amb_num = $2;
						my $amb_atm = $1;
						$second_name =~ /(\w+)(\d)$/;
						my $sec_num = $2;
						my $sec_atm = $1;

						if ($amb_num and $sec_num) {
							if (($amb_atm eq $sec_atm) and ($amb_num != $sec_num)) {
								$second_name = $amb_atm."*";
							}
						}
					}
				}
				$other_first_line = !$other_first_line;
			}
			
			#store atom and number in number_to_atom array
			$line =~ /^atom:\s*\d+\s+(\d+)([a-zA-Z])-.+/;
			$atom[$1] = $2;
		} elsif ($line =~ /^::sumavg:\s*(.+)$/i) {
			$distance = $1;
			unless (defined ($data[$section][$first]{$first_name}[$second]{$second_name})) {
				$data[$section][$first]{$first_name}[$second]{$second_name} = $distance;
			} else {
			}
			#$data_count++;
			$first_line_seen = 0;
			$other_first_line = 1;
			$pair_seen = 0;

		}
    }
	return (\@data, \@section_heading, \@atom);
	#$self->{ql_data} = \@data;
	#$self->{ql_section} = \@section_heading;
	#$self->{ql_residue} = \@atom;
	#my ($data, $section, $residue) = ($self->{ql_data}, $self->{ql_section}, $self->{ql_residue});
	#return (\@$data, \@$section, \@$residue);
	#return ($self->{ql_data}, $self->{ql_section}, $self->{ql_residue});
}

sub display_qi {
	my $self = shift;
	my $control_file = shift;
	my $output_dir = shift;
	my $session = shift;
	#get the qi file
	my $qi_file = $self->get_path("QR");
	unless ($qi_file) {
		return;
	}
    my $opened = open(QI, $qi_file);
    unless ($opened) {
		print "Cannot open QI file \"$qi_file\"";
		return;
    }
	#parse the qi file
	my %data;
	my ($line, $count, $pklst);
	while (chomp($line = <QI>)) {
		if ($line =~ /inconsistent\speaks\sfor\s(.+)$/i) {
			$pklst = $1;
			$count = 0;
		} elsif($line =~ /peak\:/) {
			#print $line . "\n";\
			$data{$pklst}[$count] = $line;
			$count++;
		}
    }
	my $str = "<ul id='pl_fn_list' type='none' class='tab_menu'>";
	foreach $pklst ( keys %data ) {
		$str .= "<li onclick=\"swap_sections('pl_fn_$pklst', 'pl_fn_divs');\">" . $pklst . "</li>";
	}
	$str .= "</ul>";
	print $str;
	$pklst = undef;
	my $column;
	my $peaklist;
	my $pk_count = 1;
	my %pk_match;
	my $display = "block";
	print "<div>Double click a row to view all possible assignments.<br/></div>";
	foreach $pklst (keys %data) {
    	my $data_match_ref = $self->parse_match($pklst);
#		$peaklist = $self->{dataset}->get_peaklist_by_filename($pklst);
		$peaklist_name = $pklst;
		$display = "none" unless ($pk_count == 1);
		print "<div style='display:$display' id='pl_fn_${peaklist_name}' class='pl_fn_divs'><div>$peaklist_name</div><table cellpadding='0' cellspacing='0' border='0' class='display pl_fn_table' name='fn_$peaklist_name' id='${pklst}_fn_table'><thead><tr>";
		print "";
		open(HANDLE, ">$output_dir/${pklst}_fn.sparky");
		print HANDLE "### Peaklist: $pklst\n";
		#push(@sparky_lines, "### Peaklist: $pklist");
#		print "<thead><tr><th>peak</th>";
#		print $peaklist->get_column_type_by_name("hx1");
#		my $columns_hash_ref = $peaklist->{shift_columns};
#		foreach my $key (keys %$columns_hash_ref) {
#			$column = $peaklist->{shift_columns}{$key};
#			print "<th>" . $column->get_name() . "</th>";
#		}
#		print "<th>intensity</td><th>label</th></tr></thead>";
#		$new_match = $self->get_path($pklst . "_match.gz");
		#$match_file = &unzip_file($new_match);

		my $sparkylineHTML = "<span class='tab1 column'>#label</span>\t";
		my $sparkyline = "#label\t";
		my $i;
		my $count = scalar(@{$data{$pklst}});
		my $first_line = $data{$pklst}[0];
		#                          1        2           3             4              5             6        7     8              9          10     11           12            13       14         15
		if ($first_line =~ /^-> (peak):\s+(\d+)\s+\(([hx12]+)\)\s*([0-9\.]+)\s+\(([hx12]+)\)\s*([0-9\.]+)\s+(\(([hx12]+)\)\s*([0-9\.]+)\s+)?(\(([hx12]+)\)\s*([0-9\.]+)\s+)?(\S+)\s+\((label)\)\s*(\S+)/i) {
			if (defined $1) {
				print "<th>$1</th>";
			}
			if (defined $3) {
				print "<th>$3</th>";
				$sparkylineHTML .= "\t<span class='tab1 column'>$3</span>\t";
				$sparkyline .= "\t$3\t";
			}
			if (defined $5) {
				print "<th>$5</th>";
				$sparkylineHTML .= "\t<span class='tab1 column'>$5</span>\t";
				$sparkyline .= "\t$5\t";
			}
			if (defined $8) {
				print "<th>$8</th>";
				$sparkylineHTML .= "\t<span class='tab1 column'>$8</span>\t";
				$sparkyline .= "\t$8\t";
			}
			if (defined $11) {
				print "<th>$11</th>";
				$sparkylineHTML .= "\t<span class='tab1 column'>$11</span>\t";
				$sparkyline .= "\t$11\t";
			}
			if (defined $12) {
				print "<th>$12</th>";
			}
			if (defined $13) {
				print "<th>Intensity</th>";
				#$sparkylineHTML .= "\t<span class='tab1 column'>Intensity</span>\t";
			}
			if (defined $14) {
				print "<th>$14</th>";
			}
			$sparkylineHTML .= "\t<span class='tab1 column'>note</span>";
			$sparkyline .= "\tnote";
		}
		#print HANDLE "$sparkylineHTML\n";
		print HANDLE "$sparkyline\n";
		#push(@sparky_lines, $sparkylineHTML);
		print "</tr></thead><tbody>";
		for ($i = 0; $i<$count; $i++) {
			my $dim_count = 0;
			my $line = $data{$pklst}[$i];
			$sparkylineHTML = "\t";
			$sparkyline = "\t";
		#                             1         2             3              4             5            6  7             8              9  10            11             12                   13       14         15
			if ($line =~ /^-> peak:\s+(\d+)\s+\(([hx12]+)\)\s*([0-9\.]+)\s+\(([hx12]+)\)\s*([0-9\.]+)\s+(\(([hx12]+)\)\s*([0-9\.]+)\s+)?(\(([hx12]+)\)\s*([0-9\.]+)\s+)?(\S+)\s+\(label\)\s*(\S+)/i) {
				#print "<tr onclick='getUnassignedPeaks(\"$pklst\", \"$1\", \"" . $data_match_ref->{$1}[0] . "\"); '>";
				print "<tr ondblclick=\"showInfo(event, '";
				my $info_ref = $data_match_ref->{$1};
				for my $line (@$info_ref) {
					if ($line =~ /h?x[12]/i) {
						print "<br/>";
					}
					print "$line<br/>";
				}
				print "');\">";
				#print @$info[1];
				#if (defined $13) {
				#	unless ($13 =~ /^\d+$/) {
				#		$sparkylineHTML .= "\t$13";
				#	}
				#}
				if (defined $1) {
					print "<td>$1</td>";
				}
				if (defined $3) {
					print "<td>$3</td>";
					$sparkylineHTML .= "\t<span class='tab1 column'>$3</span>\t";
					$sparkyline .= "\t$3\t";
					$dim_count++;
				}
				if (defined $5) {
					print "<td>$5</td>";
					$sparkylineHTML .= "\t<span class='tab1 column'>$5</span>\t";
					$sparkyline .= "\t$5\t";
					$dim_count++;
				}
				if (defined $8) {
					print "<td>$8</td>";
					$sparkylineHTML .= "\t<span class='tab1 column'>$8</span>\t";
					$sparkyline .= "\t$8\t";
					$dim_count++;
				}
				if (defined $11) {
					print "<td>$11</td>";
					$sparkylineHTML .= "\t<span class='tab1 column'>$11</span>\t";
					$sparkyline .= "\t$11\t";
					$dim_count++;
				}
				if (defined $12) {
					#$sparkylineHTML .= "\t<span class='tab1 column'>$12</span>\t";
					#$sparkyline .= "\t$12\t";
					print "<td>$12</td>";
				}
				if (defined $13) {
					print "<td>$13</td>";
				}
				if (defined $15) {
					print "<td>$15</td>";
				}
#				foreach my $key (keys %$columns_hash_ref) {
#					$column = $peaklist->{shift_columns}{$key};
					#print "<td>" . $column->get_name() . "</td>";
#				}
				print "</tr>";
				$sparkylineHTML .= "\tfalse negative";
				$sparkyline .= "\tfalse negative";
				my $tempHTML = "<span class='tab1 column'>?";
				my $temp = "?";
				for(my $i = 1; $i < $dim_count; $i++) {
					$tempHTML .= "-?";
					$temp .= "-?";
				}
				$tempHTML .= "</span>";
				$sparkylineHTML = $temp . $sparkylineHTML;
				$sparkyline = $temp . $sparkyline;
			}
			print HANDLE "$sparkyline\n";
			#print HANDLE "$sparkylineHTML\n";
			#push(@sparky_lines, $sparkylineHTML);
		    #$pk_match{$data{$pklst}[$i]} = $match_file; 
			#print "<div>$data{$pklst}[$i]</div>";
		}
		close(HANDLE);
		$pk_count++;
		print "</tbody></table>";
		#print "<a href=\"logfile\" target=\"file_dialog_iframe\" onClick=\"return display_file('${pklst}_match', '" . $self->{sessionid} . "'); \">sparky file</a><br/>";
		#print "<a href=\"sparky\" target=\"file_dialog_iframe\" onClick=\"return display_file('${pklst}_fn.sparky', '" . $session->get_session_id() . "'); \">View</a> in sparky format<br/>";
		print "<a href=\"sparky\" target=\"file_dialog_iframe\" onClick=\"return download_file('${pklst}_fn.sparky', '" . $session->get_session_id() . "'); \">Download</a> in sparky format<br/>";
		#print "<div>View in Sparky Format</div>";
		print "</div>";
    }
}
sub parse_match {
	#print "calling parse match<br/>\n";
    #given a filename, and a peak line, returns an array with all the atom lines contained all possible assignments of the peak
    my ($self, $peaklist) = @_;
	my $file = $self->get_path($peaklist);
	#print "path of gz file is $file<br/>\n";
	$file = $self->unzip_file($file, "${peaklist}_match");
    unless (-e $file) {
		return;
    }
    unless (open(FILE, $file)) {
		print STDERR "Cannot open match file \"$file\"";
		return;
    }
    my $data = {};
    my $line;
	my $peak;
    while (chomp($line = <FILE>)) {
		#print STDERR $line . "\n";
		if ($line =~ /^peak/) {
			#print STDERR "$line\n";
			$line =~ /(peak:\s+(\S+)\s+.+\(label\))/;
			$peak = $2;
			my @atoms;
			$data->{$peak} = \@atoms;
		} elsif (($line =~ /atom:/) or ($line =~ /assigned\sby/i)) {
			my $atoms = $data->{$peak};
			push(@$atoms, $line);
		}
    }
	#my @keys = keys(%$data);
	#print "size of hash: " . scalar(@keys) . "<br/>\n";
	#print "data: $data\n";
	#for my $key (keys %$data) {
	#	my $val = $data->{$key};
	#	print STDERR "key: $key, val $val\n";
	#}
    return ($data);
}
sub create_sparky_list_fn {
    my ($control_file, $data_ref, $dir, $parent, $pklist, $note) = @_;
    #get and parse control file
	my $file = $self->get_ovw_path();
    my ($comments_ref, $general_ref, $commands_ref, $peaklists_ref, $col_names_ref, $pklst_count) = &openContFile($control_file);
    my %data = %$data_ref;
    print "dir is $dir, and pklist is $pklist, control_file is $control_file, pklist is $pklist\n";
    my %col_names = %$col_names_ref;
    my ($plist, $value, $name);
    my $full_plist;
    #pklist is the key to hash $data, full_plist is the key to hash col_names
    foreach $plist (keys %col_names) {
		if ($plist =~ /$pklist/) {
			$full_plist = $plist;
		}
    }
    #open sparky output file
    my $template = "create_sparky_fn";
    $temp_sparky = getTempFileName($template); 
    unless (open(OUT, "> $temp_sparky")) {
		print "Cannot open sparky file for writing\n";
		return;
    }
    print OUT "### Peaklist $full_plist\n";
    print OUT "# label     ";
    my $lbl_exists = 0;
    #my @col_headings;
    #$col_heading[0] = 'label';
    #my $col_headings_count = 0;
    foreach $value (sort {$a <=> $b} keys %{$col_names{$full_plist}}) {
		if ($col_names{$full_plist}{$value} =~ /^(x1|x2|hx1|hx2)$/) {
		    print OUT "$col_names{$full_plist}{$value}     ";
		}
    }
    print OUT "note\n\n";
    my $i;
    if (!$note) {
		$note = 'false_negative';
	}
    my $label = "?-?-?";
    my $space = "     ";
    for ($i = 0; $i < scalar(@{$data{$pklist}}); $i++) {
		my %info;
		my $line = $data{$pklist}[$i];
		#print "$line\n";
		my $new_line;
		#my $id = $i+1;
		#$line =~ /\(label\)(.+)$/;
		#my ($label) = ($1, $2);
		#unless ($label)
		#{
		#   $label = "?-?-?";
		#}
		#$new_line = $new_line."$label\t";
		$new_line = "$label$space";
		foreach $value (sort {$a <=> $b} keys %{$col_names{$full_plist}}) {
		    my $heading = $col_names{$full_plist}{$value};
		    next unless ($heading =~ /^(x1|x2|hx1|hx2)$/);	    
		    $line =~ /\($heading\)([.\d]+)\s/;
		    my $temp = $1;
		    $new_line = "$new_line$temp$space";
		}
		print OUT "$new_line$note\n";
    }
    close(OUT);
    #open the sparky file
    &open_sparky("Peaklist $pklist", $parent, $temp_sparky);
}
sub unzip_file {
	#print "calling unzip<br/>\n";
    my ($self, $file, $fname) = @_;
    system("gunzip -c $file > $fname");
    return $fname;
}

sub show_fp_distributions {
	my $self = shift;
#	my $dataset = $self->{dataset};
#	my $pdb = $dataset->{pdb};
#	my $pdb_file = $pdb->get_filepath();
	my $pdb_file = $self->{pdb};
	#print "pdbfilepath: $pdb_file </br>";
	my $rasmol = Rasmol->new($self->{protein_name}, $self->get_path("QP"), $pdb_file, $self->{base_dir} . "/" . $self->{results_dir}, $self);
# 	my $rasmol_script_file = $self->get_path("rasmol");
	my $rasmol_script_file = $rasmol->get_script_path();
	my $doc_root = $ENV{'DOCUMENT_ROOT'};
	#print "<br/>1:<br/>$pdb_file<br/>";
	#print $self->{base_dir};
	$pdb_file = $self->{base_dir} . "/" . $pdb_file;
	$pdb_file =~ /$doc_root(.*)/;
	#print "<br/>1:<br/>$pdb_file<br/>";
	$pdb_file = "http://" . $ENV{'HTTP_HOST'} . $1;
	#print $pdb_file;
	my $jmol_script = "load $pdb_file; select *; cpk off; color [0,0,255]; wireframe off; ";
	unless (open(RASMOL, $rasmol_script_file)) {
		print "Unable to load false positive distrubtion information.<br/>";
		return;
	}
	my $temp_line;
	my $atoms_seen = undef;
	while (chomp($temp_line = <RASMOL>)) {
		if ($temp_line =~ /atoms/i) {
			$atoms_seen = 1;
		}
		next if ((not defined $atoms_seen) or ($temp_line =~ /^\#/) or ($temp_line =~ /^\s*$/) or ($temp_line =~ /^load/i) or ($temp_line =~ /zoom/i) or ($temp_line =~ /restrict/i));
		$jmol_script .= "$temp_line; ";
	}
	print <<EOF;
	<script type="text/javascript">
		showJmol('$jmol_script');
	</script>
EOF
}

return(1); 
