#!/usr/bin/perl -w

# parse_bmrb.pl
#
# This file extracts the sequence and assigned chemical shifts table from a BMRB file and displays them in a format compatible with LACS software.
# Usage: bmrb_parse.pl [-d] <bmrb_file>
#	-d: enable verbose debugging print statements from the source code
#
use strict;
BEGIN {
sub source {
    my $name = shift;

    open my $fh, "<", $name
        or die "could not open $name: $!";

    while (<$fh>) {
        chomp;
				next if ($_ =~ /^#/);
				next if ($_ !~ /\w/);
				my $line = $_;
				$line =~ s/export//;
				$line =~ s/#.*$//;
        my ($k, $v) = split /=/, $line, 2;
        $v =~ s/^(['"])(.*)\1/$2/; #' fix highlighter
        $v =~ s/\$([a-zA-Z]\w*)/$ENV{$1}/g;
        $v =~ s/`(.*?)`/`$1`/ge; #dangerous

        $k =~ s/\s+//g;
      	$v =~ s/\s+//g;

        $ENV{$k} = $v;
    }
}

source("$ENV{'ASPATH'}/env.sh");

my $script_name = "";

	#add the parent directory of the script to lib include path
	use Cwd;
	$ENV{'ASLIB'} = $ENV{'ASPATH'} . '/lib';
	push(@INC, $ENV{'ASLIB'});
}
use strict;
use ASDP::Bmrb;
my $usage = " USAGE: bmrb2lacs.pl [-d] <bmrb_file>
\t-d: enable verbose debugging print statements from the source code\n";
if ((scalar(@ARGV) < 1) || ($ARGV[0] =~ /^-h/)) {
        die($usage);
}
my ($debug, $chem_shifts, $sequence, $lacs, $talos) = (0, 0, 0, 0, 0);
if ($ARGV[0] =~ /^-/) {
        if ($ARGV[0] =~ /d/) {
                $debug = 1;
        }
        shift(@ARGV);
}
my $bmrb_file = $ARGV[0];
my $bmrb = Bmrb->new($bmrb_file, $bmrb_file, $debug);
my $chem_shifts_table_ref = $bmrb->extract_bmrb();
####################################################################################
#remove comments to enable printing of one letter sequence codes to the output file.
#the format will still be LACS compatible
####################################################################################
#my $seq = $bmrb->extract_one_letter_sequence();
#print "_Mol_residue_sequence\n;\n";
#for (my $i=1; $i<=length($seq); $i++){
#	print substr($seq, $i-1, 1);
#	print "\n" if ($i % 20 == 0);
#}
#print "\n;\n";
####################################################################################
print "loop_\n";
print "\t_Atom_shift_assign_ID\n\t_Residue_seq_code\n\t_Residue_label\n\t_Atom_name\n\t_Atom_type\n\t_Chem_shift_value\n\t_Chem_shift_value_error\n\t_Chem_shift_ambiguity_code\n\n";
foreach my $line (@$chem_shifts_table_ref) {
	foreach my $item (@$line) {
		print $item . " ";
	}
	print "\n";
}
print "stop_\n";
