#!/usr/bin/perl -w

# parse_bmrb.pl
#
# This file extracts the sequence and assigned chemical shifts table from a BMRB file and displays them in a format compatible with TALOS software.
# Usage: bmrb_parse.pl [-d] <bmrb_file>
#	-d: enable verbose debugging print statements from the source code
#
use strict;
BEGIN {
sub source {
    my $name = shift;

    open my $fh, "<", $name
        or die "could not open $name: $!";

    while (<$fh>) {
        chomp;
				next if ($_ =~ /^#/);
				next if ($_ !~ /\w/);
				my $line = $_;
				$line =~ s/export//;
				$line =~ s/#.*$//;
        my ($k, $v) = split /=/, $line, 2;
        $v =~ s/^(['"])(.*)\1/$2/; #' fix highlighter
        $v =~ s/\$([a-zA-Z]\w*)/$ENV{$1}/g;
        $v =~ s/`(.*?)`/`$1`/ge; #dangerous

        $k =~ s/\s+//g;
      	$v =~ s/\s+//g;

        $ENV{$k} = $v;
    }
}

source("$ENV{'ASPATH'}/env.sh");

my $script_name = "";

	#add the parent directory of the script to lib include path
	use Cwd;
	$ENV{'ASLIB'} = $ENV{'ASPATH'} . '/lib';
	push(@INC, $ENV{'ASLIB'});
}
use strict;
use ASDP::Bmrb;
my $usage = " USAGE: bmrb2talos.pl [-d] <bmrb_file>
\t-d: enable verbose debugging print statements from the source code\n";
if ((scalar(@ARGV) < 1) || ($ARGV[0] =~ /^-h/)) {
        die($usage);
}
my ($debug, $chem_shifts, $sequence, $lacs, $talos) = (0, 0, 0, 0, 0);
if ($ARGV[0] =~ /^-/) {
        if ($ARGV[0] =~ /d/) {
                $debug = 1;
        }
        shift(@ARGV);
}
my $bmrb_file = $ARGV[0];
my $bmrb = Bmrb->new($bmrb_file, $bmrb_file, $debug);
my $seq = $bmrb->extract_one_letter_sequence();
my $chem_shifts_table_ref = $bmrb->extract_bmrb();
print "DATA FIRST_RESID 1\n\n";
print "DATA SEQUENCE  ". $seq . "\n\n";
print "VARS   RESID RESNAME ATOMNAME SHIFT\nFORMAT %4d %1s %4s %8.3f\n";
foreach my $array_ref (@$chem_shifts_table_ref) {
	my @array = @$array_ref;
	my $atom = $array[3];
	$atom = "HN" if ($array[3] eq "H");
	print (sprintf("%4d %1s %4s %8.3f\n", $array[1], $bmrb->{three2one}->{uc($array[2])}, $atom, $array[5]));
}
