#!/usr/bin/perl

#********************************************************
#*Welcome to the ASDP configuration tool.
#*This script will setup your ASDP Environment.
#*You can run this script multiple times.
#********************************************************


use strict;
my $installer = Installer->new();
$installer->printIntro();

my $aspath = $installer->getInstallDirectory();

my $regex = "\"s+\\\"\\\$ENV{'ASPATH'}+\\\"\\$aspath\+\"";

print "sed -i '' $regex $aspath/bin/\*";
if ($^O =~ /darwin/) #macOS
{
	system("sed -i '' $regex $aspath/bin/\*");
} else #linux
{
	system("sed -i $regex $aspath/bin/\*");
}

print "
***** ASDP successfully configured *****
";

BEGIN {
	package Installer;
	use Cwd;
	sub new {
		my $class = shift;
		my $self = {};
		bless($self, $class);
		return $self;
	}
#################################################################################################
#################################################################################################
	sub printIntro {
		my $self = shift;
		print "********************************************************\n";
		print "*Welcome to the ASDP configuration tool.               *\n";
		print "*This script will setup your ASDP Environment.         *\n";
		print "********************************************************\n\n";
	}
#################################################################################################
#################################################################################################
	sub getInstallDirectory {
		my $self = shift;
		my $as_dir = Cwd::abs_path($0);
		$as_dir =~ /^(.*)\/([^\/]+)\s*$/;
		$as_dir = $1;
		my $script_name = $2;
		print "Select your ASDP directory:\n";
		print "Best guess is $as_dir  Is this correct? (y/n)";
		my $response = <STDIN>;
		unless ($response =~ /^y$/) {
			print "Please enter the full path of your ASDP directory:\n";
			$as_dir = <STDIN>;
			chomp($as_dir);
		}
		unless ($self->confirmDirectory($as_dir, "NOESYASSIGN/Release/noesyassign")) {
			print STDERR "ASDP not found at $as_dir\n";
			exit(1);
		}
		print "You have chosen: $as_dir\n";
		$self->{ASPATH} = $as_dir;
		return $as_dir;
	}
##################################################################################################
	sub confirmDirectory {
		my ($self, $dir, $filepath) = @_;
		if (-e "$dir/$filepath") {
			return 1;
		}
		return 0;
	}
}
