my $scale = 1.2;

my $width = $scale * 45;
my $height = $scale * 65;

my $x_div = $width + $width/2;
my $y_div = $height + $height/2;;

sub place_anti_sheets
{
    my ($canvas, $x, $y, $drawn_sheets_ref, $drawn_registers_ref, $data_ref, $help_ref, $key) = @_;
    #print "\n1. place_anti_sheets (x = $x, y = $y)\n";

    unless ($key)
    {
	$key = 'Temp_ids';
    }

    my ($i, $j);
    my ($sheet_num, $up, $left_to_right, $reg_num);

    my @drawn_sheets = @$drawn_sheets_ref;
    my @drawn_registers = @$drawn_registers_ref;
    my %stats;

    #first sheet
    $sheet_num = 0;
    $left_to_right = 1; #first =1, then alternating to 0, 1, etc. 
    $reg_num = 0;

    # draw the first strand 
    # up is decided based on the hn-hn, ha-ha evidence for the first residue 
    $up = &get_first_up($data_ref, $drawn_registers[0], $drawn_sheets[0], $left_to_right, 'anti', $key);
    my ($start_prev, $end_prev) = &get_res($drawn_sheets[0]);
    &create_peptide($canvas, $start_prev, $end_prev, $x, $y, $left_to_right, $up, $data_ref);
    $stats{$drawn_sheets[0]}{'coords'}{'x'} = $x;
    $stats{$drawn_sheets[0]}{'coords'}{'y'} = $y;
    $stats{$drawn_sheets[0]}{'up'} = $up;
    $stats{$drawn_sheets[0]}{'left_to_right'} = $left_to_right;
    
    #print "2. Done drawing peptide $drawn_sheets[0] with register $drawn_registers[0] at ($x, $y), up is $up\n";

    # draw other strands 
    for ($sheet_num = 1; $sheet_num <= $#drawn_sheets; $sheet_num++)
    {
	$left_to_right = !$left_to_right;
	
	my ($start_cur, $end_cur) = &get_res($drawn_sheets[$sheet_num]);
	my ($reg1,$reg2) = &get_res($drawn_registers[$reg_num]);

	my $temp_sheet1 = &get_sheet($data_ref, $reg1, $key);
	my $temp_sheet2 = &get_sheet($data_ref, $reg2, $key);
	
	#print "3a. Sheet is $drawn_sheets[$sheet_num], $reg1 belongs to sheet $temp_sheet1, $reg2 is from sheet $temp_sheet2\n";

	my $mod_sheet = ($start_cur+1).'-'.($end_cur-1);
	
	unless (($drawn_sheets[$sheet_num] eq $temp_sheet2) or ($mod_sheet eq $temp_sheet2))
	{
	    my $temp = $reg1;
	    $reg1 = $reg2;
	    $reg2 = $temp;
	}
	
	if ($left_to_right)
	{
	    #first residue's up status for current strand about to draw 
	    $up = &change_anti_up($up, $end_prev, $reg1, $start_cur, $reg2);
	}
	else
	{
	    $up = &change_anti_up($up, $start_prev, $reg1, $end_cur, $reg2);
	}

	#print "3b. Sheet is $drawn_sheets[$sheet_num], start_cur is $start_cur, end_cur is $end_cur, reg1 is $reg1, and reg2 is $reg2\n";
	
	# get coords of regiter 1 - $x changed 
	($x, $y) = &get_coords_of_res($x, $y, $start_prev, $end_prev, $reg1, !$left_to_right);

	# get corrds of start of strands given register 2, $x changed  
	($x, $y) = &get_coords_of_start($x, $y, $start_cur, $end_cur, $reg2, $left_to_right);

	# change $y 
	($x, $y) = &get_coords_next_sheet($x, $y);
	
	#print "3c. Creating pepetide $drawn_sheets[$sheet_num] with register $drawn_registers[$reg_num] at ($x,$y), left_to_right is $left_to_right, and up is $up\n";
	# retured for comparing with bg. 
	$stats{$drawn_sheets[$sheet_num]}{'coords'}{'x'} = $x;
	$stats{$drawn_sheets[$sheet_num]}{'coords'}{'y'} = $y;
	$stats{$drawn_sheets[$sheet_num]}{'up'} = $up;
	$stats{$drawn_sheets[$sheet_num]}{'left_to_right'} = $left_to_right;
	
	# start_cur and end_cur are the start and end residues of the strand being drawn, left_to_right shows the direction of the strand, up determines if the first residue's Ha is pointing up or not
	if ($left_to_right)
	{
	    &create_peptide($canvas, $start_cur, $end_cur, $x, $y, $left_to_right, $up, $data_ref);
	}
	else
	{
	    &create_peptide($canvas, $end_cur, $start_cur, $x, $y, $left_to_right, $up, $data_ref);
	}
	
	($start_prev, $end_prev) = ($start_cur, $end_cur);
	$reg_num++;
    }

    $canvas->update;
    my ($a, $b, $c, $d) = $canvas->bbox('all');	
    my $c_wid = abs($c - $a) * 1.01;
    my $c_ht = abs($d - $b) * 1.01;
    $canvas->configure(-width =>$c_wid, height=>$c_ht);
    $canvas->configure(-scrollregion=>[$canvas->bbox('all')], -confine=>1);
    $canvas->update;

    #print "returning height of canvas as $c_ht\n";
    
    return ($c_ht, \%stats);

}


sub place_para_sheets
{
    my ($canvas, $x, $y, $drawn_sheets_ref, $drawn_registers_ref, $data_ref, $help_ref, $key) = @_;
    

    unless ($key)
    {
	$key = 'Temp_ids';
    }
    
    my ($i, $j);
    my ($sheet_num, $up, $left_to_right, $reg_num);

    my @drawn_sheets = @$drawn_sheets_ref;
    my @drawn_registers = @$drawn_registers_ref;
    my %stats;

    #print "\n1. place_para_sheets (x = $x, y = $y), sheets are @drawn_sheets, registers are @drawn_registers\n";

    #first sheet
    $sheet_num = 0;
    $left_to_right = 1;
    $reg_num = 0;
    $up = &get_first_up($data_ref, $drawn_registers[0], $drawn_sheets[0], $left_to_right, 'para', $key);
    my ($start_prev, $end_prev) = &get_res($drawn_sheets[0]);
    &create_peptide($canvas, $start_prev, $end_prev, $x, $y, $left_to_right, $up, $data_ref);
    $stats{$drawn_sheets[0]}{'coords'}{'x'} = $x;
    $stats{$drawn_sheets[0]}{'coords'}{'y'} = $y;
    $stats{$drawn_sheets[0]}{'up'} = $up;
    $stats{$drawn_sheets[0]}{'left_to_right'} = $left_to_right;
    
    #print "2. Done drawing peptide $drawn_sheets[0] with register $drawn_registers[0] at ($x, $y), up is $up\n";

    for ($sheet_num = 1; $sheet_num <= $#drawn_sheets; $sheet_num++)
    {
	my ($start_cur, $end_cur) = &get_res($drawn_sheets[$sheet_num]);
	my ($reg1,$reg2) = &get_res($drawn_registers[$reg_num]);
	
	my $temp_sheet1 = &get_sheet($data_ref, $reg1, $key);
	my $temp_sheet2 = &get_sheet($data_ref, $reg2, $key);
	
	#print "3a. Sheet is $drawn_sheets[$sheet_num], $reg1 belongs to sheet $temp_sheet1, $reg2 is from sheet $temp_sheet2\n";

	my $mod_sheet = ($start_cur+1).'-'.($end_cur-1);
	unless (($drawn_sheets[$sheet_num] eq $temp_sheet2) or ($mod_sheet eq $temp_sheet2))
	{
	    my $temp = $reg1;
	    $reg1 = $reg2;
	    $reg2 = $temp;
	}
	
	$up = &change_para_up($up, $start_prev, $reg1, $start_cur, $reg2);
	
	#print "3b. Sheet $drawn_sheets[$sheet_num] - start_cur is $start_cur, end_cur is $end_cur, reg1 is $reg1, and reg2 is $reg2\n";
	
	($x, $y) = &get_coords_of_res($x, $y, $start_prev, $end_prev, $reg1, $left_to_right);
	($x, $y) = &get_coords_of_start($x, $y, $start_cur, $end_cur, $reg2, $left_to_right);
	($x, $y) = &get_coords_next_sheet($x, $y);
	
	#print "3c. Creating pepetide $drawn_sheets[$sheet_num] with register $drawn_registers[$reg_num] at ($x,$y), left_to_right is $left_to_right, and up is $up\n";
	$stats{$drawn_sheets[$sheet_num]}{'coords'}{'x'} = $x;
	$stats{$drawn_sheets[$sheet_num]}{'coords'}{'y'} = $y;
	$stats{$drawn_sheets[$sheet_num]}{'up'} = $up;
	$stats{$drawn_sheets[$sheet_num]}{'left_to_right'} = $left_to_right;
	
	&create_peptide($canvas, $start_cur, $end_cur, $x, $y, $left_to_right, $up, $data_ref);
	
	($start_prev, $end_prev) = ($start_cur, $end_cur);
	$reg_num++;
    }

    $canvas->update;
    my ($a, $b, $c, $d) = $canvas->bbox('all');
    my $c_wid = abs($c - $a) * 1.01;
    my $c_ht = abs($d - $b) * 1.01;
    $canvas->configure(-width =>$c_wid, height=>$c_ht);
    $canvas->configure(-scrollregion=>[$canvas->bbox('all')], -confine=>1);
    $canvas->update;

    #print "returning height of canvas as $c_ht\n";
    
    return ($c_ht, \%stats);

}


sub place_anti_bg
{
    my ($canvas, $drawn_sheets_ref, $drawn_registers_ref, $stats_ref, $data_ref, $as_dir, $parent, $help_ref, $key, $missing_ref) = @_;

    my @drawn_sheets = @$drawn_sheets_ref;
    my @drawn_registers = @$drawn_registers_ref;
    my %stats = %$stats_ref;
    my %data = %$data_ref;

    unless ($key)
    {
	$key = 'Temp_ids';
    }

    #get constraints
    unless (opendir(DIR, $as_dir))
    {
	&err_msg("Cannot open directory \"$as_dir\"", $parent);
	return;
    }
    

    my @contents = sort {$a cmp $b || $a <=> $b} readdir(DIR);
    close(DIR);
    
   
    #$my (@cycles) = grep {/^CYCLE.+final$/} @contents;
    #@cycles = sort {$a <=> $b || $a cmp $b} @cycles;
    #my $final_cycle = $cycles[$#cycles];

    #print "place_anti_bg: as_dir is $as_dir, contents are @contents\n";
    
    my ($dir) = grep {/^CYCLE1.+final$/} @contents;
    #print "dir is $dir\n";
    unless ($dir)
    {
	($dir) = grep {/Working/i} @contents;
    }
    
    unless ($dir)
    {
	&err_msg("No Cycle 1 or Working cycle in directory $as_dir", $parent);
	return;
    }
    $dir = File::Spec->rel2abs($dir, $as_dir);

    #get assOrder file and parse it
    my (@assOrder_files) = &get_assOrders($dir, $parent);
    #print "assOrders are @assOrder_files\n";
    my $file;
    my %data_from_files;
    foreach $file (@assOrder_files)
    {
	$data_from_files{$file} = &parse_assOrder_whole($file, $parent);
    }
    
    my ($hbond_upl, $prot_upl, $format) = &get_upls($dir, $parent);
    unless ($hbond_upl and $prot_upl)
    {
	&err_msg("No upl files in directory \"$dir\"", $parent);
	return;
    }

    my ($prot_data_ref, $prot_res_ref);
    ($prot_data_ref, $prot_res_ref, $prot_lines_ref) = &parse_upl($prot_upl, $prot_data_ref, $prot_res_ref, $parent, 1, 0, $format);
    my @prot_data = @$prot_data_ref;
    my @prot_lines = @$prot_lines_ref;
        
    my ($hbond_data_ref, $hbond_res_ref);
    ($hbond_data_ref, $hbond_res_ref, $hbond_lines_ref) = &parse_upl($hbond_upl, $hbond_data_ref, $hbond_res_ref, $parent, 1, 0, $format);
    my @hbond_data = @$hbond_data_ref;
    my @hbond_lines = @$hbond_lines_ref;
   
    #print "place_anti_bg: hbond_upl is $hbond_upl, and prot_upl is $prot_upl, format is $format\n";
    
    my ($res_above, $res_below, $next_below);
    my ($sheet_num, $reg_num);
    $reg_num = 0;
    
    for ($sheet_num = 0; $sheet_num < $#drawn_sheets; $sheet_num++)
    {
	my ($first_res, $last_res) = &get_res($drawn_sheets[$sheet_num]);
	my $sheet_above = $drawn_sheets[$sheet_num];
	my $sheet_below = $drawn_sheets[$sheet_num+1];
	

	my %info;
	#print "a. sheet_above is $sheet_above, sheet_below is $sheet_below\n";

	if ($stats{$drawn_sheets[$sheet_num]}{'left_to_right'})
	{
	    for ($res_above = $first_res; $res_above <= $last_res; $res_above++)
	    {
		#get res_below
		my ($reg1, $reg2) = &get_res($drawn_registers[$reg_num]);
		unless ($sheet_above eq &get_sheet($data_ref, $reg1, $key))
		{
		    my $temp = $reg1;
		    $reg1 = $reg2;
		    $reg2 = $temp;
		}
		$res_below = $reg2 + ($reg1 - $res_above);
		$next_below = $res_below + 1;

		#print "b. res_above is $res_above, res_below is $res_below, next_below is $next_below\n";
		
		unless (&belongs_to_sheet($sheet_below, $res_below, $stats{$sheet_below}{'left_to_right'}))
		{
		    $res_below = 0;
		}
		unless (&belongs_to_sheet($sheet_below, $next_below, $stats{$sheet_below}{'left_to_right'}))
		{
		    $next_below = 0;
		}

		#print "b. res_above is $res_above, res_below is $res_below, next_below is $next_below\n";
		
		#get_coords
		my ($first_below, $last_below) = &get_res($sheet_below);
		my ($ax, $ay) = &get_coords_of_res($stats{$sheet_above}{'coords'}{'x'}, $stats{$sheet_above}{'coords'}{'y'}, $first_res, 
						   $last_res, $res_above, $stats{$sheet_above}{'left_to_right'});
		my $res_up = &get_res_up($stats{$sheet_above}{'up'}, $first_res, $res_above);
		$ax += $x_div;

		# give coordiantes for each atom in one residue 
		my $elem_above_ref = &get_coords_elements($ax, $ay, $res_up, $stats{$sheet_above}{'left_to_right'});
		my %elem_above = %$elem_above_ref;

		my ($bx, $by) = &get_coords_of_res($stats{$sheet_below}{'coords'}{'x'}, $stats{$sheet_below}{'coords'}{'y'}, 
							   $first_below, $last_below, $res_below, $stats{$sheet_below}{'left_to_right'});
		$bx += $x_div;
	
		my $elem_below_ref = &get_coords_elements($bx, $by, !$res_up, $stats{$sheet_below}{'left_to_right'});
		my %elem_below = %$elem_below_ref;

		my ($nx, $ny) = &get_coords_of_res($stats{$sheet_below}{'coords'}{'x'}, $stats{$sheet_below}{'coords'}{'y'}, $first_below, $last_below, $next_below, 
						   $stats{$sheet_below}{'left_to_right'});
		$nx += $x_div;		
		my $elem_next_ref = &get_coords_elements($nx, $ny, $res_up, $stats{$sheet_below}{'left_to_right'});
		my %elem_next = %$elem_next_ref;
		
		#print "c. Residue $res_above is at ($ax, $ay) and up is $up\n";
		#$canvas->createText($ax, $ay, -text=>$res_above);
		
		
		
		#draw interactions
		if ($res_up)
		{
		    if ($res_below)
		    {

			my $hbond_dat = $hbond_data[$res_above]{'O'}[$res_below]{'HN'};
			my $hbond_line = $hbond_lines[$res_above]{'O'}[$res_below]{'HN'};

			&create_H_bond($elem_above{'OC'}{'x'}, $elem_above{'OC'}{'y'}, $elem_below{'Hn'}{'x'}, $elem_below{'Hn'}{'y'}, $canvas, $help_ref, $hbond_dat, $hbond_line);

			$hbond_dat = $hbond_data[$res_below]{'O'}[$res_above]{'HN'};
			$hbond_line = $hbond_lines[$res_below]{'O'}[$res_above]{'HN'};
			&create_H_bond($elem_above{'Hn'}{'x'}, $elem_above{'Hn'}{'y'}, $elem_below{'OC'}{'x'}, $elem_below{'OC'}{'y'}, $canvas, $help_ref, $hbond_dat, $hbond_line);

			if ($res_above <= $res_below)
			{
			    $prot_dat = $prot_data[$res_above]{'HN'}[$res_below]{'HN'};
			    $prot_line = $prot_lines[$res_above]{'HN'}[$res_below]{'HN'};
			}
			else
			{
			    $prot_dat = $prot_data[$res_below]{'HN'}[$res_above]{'HN'};
			    $prot_line = $prot_lines[$res_below]{'HN'}[$res_above]{'HN'};
			}

			my ($peaks_ref) = &get_peak(\%data_from_files, $res_above, 'H', $res_below, 'H');
			#my @peaks = @$peaks_ref;
    #print "placing insig_interaction from ($x1,$y1) to (
			#print "Residues ($res_above,Hn) and ($res_below,Hn) - peaks are (@peaks)\n";
			($info{'res1'},$info{'atm1'}, $info{'res2'}, $info{'atm2'}) = ($res_above, 'H', $res_below, 'H');
			&place_insig_interaction($elem_above{'Hn'}{'x'}, $elem_above{'Hn'}{'y'}, $elem_below{'Hn'}{'x'}, $elem_below{'Hn'}{'y'}, $canvas, $prot_dat, $prot_line, $peaks_ref, $help_ref, \%info, $missing_ref);
			
			if ($data{'NHS'}{$res_above})
			{
			    &box_H($elem_above{'Hn'}{'x'}, $elem_above{'Hn'}{'y'}, $canvas, 1);
			}
			else
			{
			    &box_H($elem_above{'Hn'}{'x'}, $elem_above{'Hn'}{'y'}, $canvas);
			}
			
			if ($data{'NHS'}{$res_below})
			{
			    &box_H($elem_below{'Hn'}{'x'}, $elem_below{'Hn'}{'y'}, $canvas, 1);
			}
			else
			{
			    &box_H($elem_below{'Hn'}{'x'}, $elem_below{'Hn'}{'y'}, $canvas);
			}
		    }

		    if ($next_below)
		    {
			if ($res_above <= $next_below)
			{
			    $prot_dat = $prot_data[$res_above]{'HN'}[$next_below]{'HA'};
			    $prot_line = $prot_lines[$res_above]{'HN'}[$next_below]{'HA'};
			}
			else
			{
			    $prot_dat = $prot_data[$next_below]{'HA'}[$res_above]{'HN'};
			    $prot_line = $prot_lines[$next_below]{'HA'}[$res_above]{'HN'};
			}
			
			my ($peaks_ref) = &get_peak(\%data_from_files, $res_above, 'H', $next_below, 'HA');
			#my @peaks = @$peaks_ref;
			#print "Residues ($res_above,Hn) and ($next_below,HA) - peaks are (@peaks)\n";

			($info{'res1'},$info{'atm1'}, $info{'res2'}, $info{'atm2'}) = ($res_above, 'H', $next_below, 'HA');
			&place_insig_interaction($elem_above{'Hn'}{'x'}, $elem_above{'Hn'}{'y'}, $elem_next{'Ha'}{'x'}, $elem_next{'Ha'}{'y'}, $canvas, $prot_dat, $prot_line, $peaks_ref, $help_ref, \%info, $missing_ref);
		    }
		}
		else
		{
		    if ($res_below)
		    {			
			if ($res_above <= $res_below)
			{
			    $prot_dat = $prot_data[$res_above]{'HA'}[$res_below]{'HA'};
			    $prot_line = $prot_lines[$res_above]{'HA'}[$res_below]{'HA'};
			}
			else
			{
			    $prot_dat = $prot_data[$res_below]{'HA'}[$res_above]{'HA'};
			    $prot_line = $prot_lines[$res_below]{'HA'}[$res_above]{'HA'};
			}
			#print "***residue $res_above is down (Ha), res_below is $res_below (Ha) (first_below is $first_below, last_below is $last_below) is at ($bx,$by), placing insig_interaction\n (line = $prot_line), prot_dat = $prot_dat\n";
			
			my ($peaks_ref) = &get_peak(\%data_from_files, $res_above, 'HA', $res_below, 'HA');
			my @peaks = @$peaks_ref;
			#print "Residues ($res_above,HA) and ($res_below,HA) - peaks are (@peaks)\n";

			($info{'res1'},$info{'atm1'}, $info{'res2'}, $info{'atm2'}) = ($res_above, 'HA', $res_below, 'HA');
			&place_insig_interaction($elem_above{'Ha'}{'x'}, $elem_above{'Ha'}{'y'}, $elem_below{'Ha'}{'x'}, $elem_below{'Ha'}{'y'}, $canvas, $prot_dat, $prot_line, $peaks_ref, $help_ref, \%info, $missing_ref);
		    }
		    
		    if ($next_below)
		    {
			if ($res_above <= $next_below)
			{
			    $prot_dat = $prot_data[$res_above]{'HA'}[$next_below]{'HN'};
			    $prot_line = $prot_lines[$res_above]{'HA'}[$next_below]{'HN'};
			}
			else
			{
			    $prot_dat = $prot_data[$next_below]{'HN'}[$res_above]{'HA'};
			    $prot_line = $prot_lines[$next_below]{'HN'}[$res_above]{'HA'};
			}

			my ($peaks_ref) = &get_peak(\%data_from_files, $res_above, 'HA', $next_below, 'H');
			#my @peaks = @$peaks_ref;
			#print "Residues ($res_above,HA) and ($next_below,Hn) - peaks are (@peaks)\n";

			($info{'res1'},$info{'atm1'}, $info{'res2'}, $info{'atm2'}) = ($res_above, 'HA', $next_below, 'H');
			&place_insig_interaction($elem_above{'Ha'}{'x'}, $elem_above{'Ha'}{'y'}, $elem_next{'Hn'}{'x'}, $elem_next{'Hn'}{'y'}, $canvas, $prot_dat, $prot_line, $peaks_ref, $help_ref, \%info, $missing_ref);

			if ($data{'NHS'}{$next_below})
			{
			    &box_H($elem_next{'Hn'}{'x'}, $elem_next{'Hn'}{'y'}, $canvas, 1);
			}
			else
			{
			    &box_H($elem_next{'Hn'}{'x'}, $elem_next{'Hn'}{'y'}, $canvas);
			}
		    }
		}
	    }
	}
	else
	{
	    #print "Sheet is $sheet_above, right to left, going through resides $first_res to $last_res\n";
	    for ($res_above = $last_res; $res_above >= $first_res; $res_above--)
	    {
		#get res_below
		my ($reg1, $reg2) = &get_res($drawn_registers[$reg_num]);
		unless ($sheet_above eq &get_sheet($data_ref, $reg1, $key))
		{
		    my $temp = $reg1;
		    $reg1 = $reg2;
		    $reg2 = $temp;
		}
		$res_below = $reg2 + ($reg1 - $res_above);
		$next_below = $res_below +1;

		#print "b. res_above is $res_above, res_below is $res_below, next_below is $next_below\n";
		
		unless (&belongs_to_sheet($sheet_below, $res_below, $stats{$sheet_below}{'left_to_right'}))
		{
		    $res_below = 0;
		}
		unless (&belongs_to_sheet($sheet_below, $next_below, $stats{$sheet_below}{'left_to_right'}))
		{
		    $next_below = 0;
		}
		
		#print "c. res_above is $res_above, res_below is $res_below, next_below is $next_below\n";

		#get_coords
		my ($first_below, $last_below) = &get_res($sheet_below);
		my ($ax, $ay) = &get_coords_of_res($stats{$sheet_above}{'coords'}{'x'}, $stats{$sheet_above}{'coords'}{'y'}, $first_res, 
						   $last_res, $res_above, $stats{$sheet_above}{'left_to_right'});
		$ax += $x_div;
		my $res_up = &get_res_up($stats{$sheet_above}{'up'}, $last_res, $res_above);
		
		my $elem_above_ref = &get_coords_elements($ax, $ay, $res_up, $stats{$sheet_above}{'left_to_right'});
		my %elem_above = %$elem_above_ref;

		my ($bx, $by) = &get_coords_of_res($stats{$sheet_below}{'coords'}{'x'}, $stats{$sheet_below}{'coords'}{'y'}, 
							   $first_below, $last_below, $res_below, $stats{$sheet_below}{'left_to_right'});
		$bx += $x_div;
		my $elem_below_ref = &get_coords_elements($bx, $by, !$res_up, $stats{$sheet_below}{'left_to_right'});
		my %elem_below = %$elem_below_ref;

		my ($nx, $ny) = &get_coords_of_res($stats{$sheet_below}{'coords'}{'x'}, $stats{$sheet_below}{'coords'}{'y'}, $first_below, $last_below, $next_below, 
							   $stats{$sheet_below}{'left_to_right'});
		$nx += $x_div;
		my $elem_next_ref = &get_coords_elements($nx, $ny, $res_up, $stats{$sheet_below}{'left_to_right'});
		my %elem_next = %$elem_next_ref;

		my ($prot_dat, $prot_line);
		#draw interactions
		if ($res_up)
		{
		    if ($res_below)
		    {
			my $hbond_dat = $hbond_data[$res_above]{'O'}[$res_below]{'HN'};
			my $hbond_line = $hbond_lines[$res_above]{'O'}[$res_below]{'HN'};
			
			&create_H_bond($elem_above{'OC'}{'x'}, $elem_above{'OC'}{'y'}, $elem_below{'Hn'}{'x'}, $elem_below{'Hn'}{'y'}, $canvas, $help_ref, $hbond_dat, $hbond_line);
			
			$hbond_dat = $hbond_data[$res_below]{'O'}[$res_above]{'HN'};
			$hbond_line = $hbond_lines[$res_below]{'O'}[$res_above]{'HN'};

			&create_H_bond($elem_above{'Hn'}{'x'}, $elem_above{'Hn'}{'y'}, $elem_below{'OC'}{'x'}, $elem_below{'OC'}{'y'}, $canvas, $help_ref, $hbond_dat, $hbond_line);
			
			if ($res_above <= $res_below)
			{
			    $prot_dat = $prot_data[$res_above]{'HN'}[$res_below]{'HN'};
			    $prot_line = $prot_lines[$res_above]{'HN'}[$res_below]{'HN'};
			}
			else
			{
			    $prot_dat = $prot_data[$res_below]{'HN'}[$res_above]{'HN'};
			    $prot_line = $prot_lines[$res_below]{'HN'}[$res_above]{'HN'};
			}
			
			my ($peaks_ref) = &get_peak(\%data_from_files, $res_above, 'H', $res_below, 'H');
			($info{'res1'},$info{'atm1'}, $info{'res2'}, $info{'atm2'}) = ($res_above, 'H', $res_below, 'H');
			&place_insig_interaction($elem_above{'Hn'}{'x'}, $elem_above{'Hn'}{'y'}, $elem_below{'Hn'}{'x'}, $elem_below{'Hn'}{'y'}, $canvas, $prot_dat, $prot_line, $peaks_ref, $help_ref, \%info, $missing_ref);
			if ($res_above <= $next_below)
			{
			    $prot_dat = $prot_data[$res_above]{'HN'}[$next_below]{'HA'};
			    $prot_line = $prot_lines[$res_above]{'HN'}[$next_below]{'HA'};
			}
			else
			{
			    $prot_dat = $prot_data[$next_below]{'HA'}[$res_above]{'HN'};
			    $prot_line = $prot_lines[$next_below]{'HA'}[$res_above]{'HN'};
			}
			if ($data{'NHS'}{$res_above})
			{
			    &box_H($elem_above{'Hn'}{'x'}, $elem_above{'Hn'}{'y'}, $canvas, 1);
			}
			else
			{
			    &box_H($elem_above{'Hn'}{'x'}, $elem_above{'Hn'}{'y'}, $canvas);
			}

			if ($data{'NHS'}{$res_below})
			{
			    &box_H($elem_below{'Hn'}{'x'}, $elem_below{'Hn'}{'y'}, $canvas, 1);
			}
			else
			{
			    &box_H($elem_below{'Hn'}{'x'}, $elem_below{'Hn'}{'y'}, $canvas);
			}
		    }
		    
		    if ($next_below)
		    {
			if ($res_above <= $next_below)
			{
			    $prot_dat = $prot_data[$res_above]{'HN'}[$next_below]{'HA'};
			    $prot_line = $prot_lines[$res_above]{'HN'}[$next_below]{'HA'};
			}
			else
			{
			    $prot_dat = $prot_data[$next_below]{'HA'}[$res_above]{'HN'};
			    $prot_line = $prot_lines[$next_below]{'HA'}[$res_above]{'HN'};
			}
			
			my ($peaks_ref) = &get_peak(\%data_from_files, $res_above, 'H', $next_below, 'HA');
			($info{'res1'},$info{'atm1'}, $info{'res2'}, $info{'atm2'}) = ($res_above, 'H', $next_below, 'HA');
			&place_insig_interaction($elem_above{'Hn'}{'x'}, $elem_above{'Hn'}{'y'}, $elem_next{'Ha'}{'x'}, $elem_next{'Ha'}{'y'}, $canvas, $prot_dat, $prot_line, $peaks_ref, $help_ref, \%info, $missing_ref);
		    }
		}
		else
		{
		    if ($res_below)
		    {
			if ($res_above <= $res_below)
			{
			    $prot_dat = $prot_data[$res_above]{'HA'}[$res_below]{'HA'};
			    $prot_line = $prot_lines[$res_above]{'HA'}[$res_below]{'HA'};
			}
			else
			{
			    $prot_dat = $prot_data[$res_below]{'HA'}[$res_above]{'HA'};
			    $prot_line = $prot_lines[$res_below]{'HA'}[$res_above]{'HA'};
			}

			my ($peaks_ref) = &get_peak(\%data_from_files, $res_above, 'HA', $res_below, 'HA');
			($info{'res1'},$info{'atm1'}, $info{'res2'}, $info{'atm2'}) = ($res_above, 'HA', $res_below, 'HA');
			&place_insig_interaction($elem_above{'Ha'}{'x'}, $elem_above{'Ha'}{'y'}, $elem_below{'Ha'}{'x'}, $elem_below{'Ha'}{'y'}, $canvas, $prot_dat, $prot_line, $peaks_ref, $help_ref, \%info, $missing_ref);
		    }
		    
		    if ($next_below)
		    {
			if ($res_above <= $next_below)
			{
			    $prot_dat = $prot_data[$res_above]{'HA'}[$next_below]{'HN'};
			    $prot_line = $prot_lines[$res_above]{'HA'}[$next_below]{'HN'};
			}
			else
			{
			    $prot_dat = $prot_data[$next_below]{'HN'}[$res_above]{'HA'};
			    $prot_line = $prot_lines[$next_below]{'HN'}[$res_above]{'HA'};
			}

			my ($peaks_ref) = &get_peak(\%data_from_files, $res_above, 'HA', $next_below, 'H');
			($info{'res1'},$info{'atm1'}, $info{'res2'}, $info{'atm2'}) = ($res_above, 'HA', $next_below, 'H');
			&place_insig_interaction($elem_above{'Ha'}{'x'}, $elem_above{'Ha'}{'y'}, $elem_next{'Hn'}{'x'}, $elem_next{'Hn'}{'y'}, $canvas, $prot_dat, $prot_line, $peaks_ref, $help_ref, \%info, $missing_ref);
			
			if ($data{'NHS'}{$next_below})
			{
			    &box_H($elem_next{'Hn'}{'x'}, $elem_next{'Hn'}{'y'}, $canvas, 1);
			}
			else
			{
			    &box_H($elem_next{'Hn'}{'x'}, $elem_next{'Hn'}{'y'}, $canvas);
			}
		    }
		}
	    }
	}
	
	$reg_num++;
    }

    return ($prot_res_ref);
    
}
				    


sub get_peak
{
    my ($data_from_files_ref, $res1, $atm1, $res2, $atm2) = @_;
    
    my @peaks;
    my $count = 0;
    my %data_from_file = %$data_from_files_ref;
    my $file;

    foreach $file (keys %data_from_file)
    {
	my $data_ref = $data_from_file{$file};
	my @data = @$data_ref;

	if ($data[$res1]{$atm1}[$res2]{$atm2}{'peak'})
	{
	    $peaks[$count] = "$data[$res1]{$atm1}[$res2]{$atm2}{'peak'}; ($data[$res1]{$atm1}[$res2]{$atm2}{'atom 1'}); ($data[$res1]{$atm1}[$res2]{$atm2}{'atom 2'})";
	    $count++;
	}
	
	if ($data[$res2]{$atm2}[$res1]{$atm1}{'peak'})
	{
	    $peaks[$count] = "$data[$res2]{$atm2}[$res1]{$atm1}{'peak'}; ($data[$res2]{$atm2}[$res1]{$atm1}{'atom 1'}); ($data[$res2]{$atm2}[$res1]{$atm1}{'atom 2'})";
	    $count++;
	}
	
    }
    return (\@peaks);

}



sub belongs_to_sheet
{
    my ($sheet, $res, $left_to_right) = @_;
    #print "belongs_to_sheet: sheet is $sheet, res is $res\n";
    
    my ($first, $last) = &get_res($sheet);

    if (($res >= $first) and ($res <= $last))
    {
	return 1;
    }
    
    #print "returning 0\n";
    return 0;
}


sub get_coords_of_res
{
    my ($x, $y, $start_res, $end_res, $res_num, $left_to_right) = @_;
    
    my ($res_x, $ret_x);

    if ($left_to_right)
    {
	$res_x = abs($res_num - $start_res) * $x_div;
	if ($res_num < $start_res)
	{
	    $ret_x = $x - $res_x;
	}
	else
	{
	    $ret_x = $x + $res_x;
	}
    }
    else
    {
	$res_x = abs($res_num - $end_res) * $x_div;
	if ($res_num > $end_res)
	{
	    $ret_x = $x - $res_x;
	}
	else
	{
	    $ret_x = $x + $res_x;
	}
    }

    #print "get_coords_of_res: start - ($x, $y) at $start_res, residue $res_num at ($ret_x, $y), left_to_right = $left_to_right\n";
    return ($ret_x, $y);
}


sub place_para_bg
{
    my ($canvas, $drawn_sheets_ref, $drawn_registers_ref, $stats_ref, $data_ref, $as_dir, $parent, $help_ref, $key, $missing_ref) = @_;

    my @drawn_sheets = @$drawn_sheets_ref;
    my @drawn_registers = @$drawn_registers_ref;
    my %stats = %$stats_ref;
    my %data = %$data_ref;
    my %info;

    unless ($key)
    {
	$key = 'Temp_ids';
    }

    #get constraints
    unless (opendir(DIR, $as_dir))
    {
	&err_msg("Cannot open directory \"$as_dir\"", $parent);
	return;
    }

    my @contents = sort {$a cmp $b || $a <=> $b} readdir(DIR);
    close(DIR);
    
    #print "place_para_bg: as_dir is $as_dir, contents are @contents\n";
    
    my ($dir) = grep {/^CYCLE1.+final$/} @contents;
    #print "dir is $dir\n";
    unless ($dir)
    {
	($dir) = grep {/Working/i} @contents;
    }
    
    unless ($dir)
    {
	&err_msg("No Cycle 1 or Working cycle in directory $as_dir", $parent);
	return;
    }
    $dir = File::Spec->rel2abs($dir, $as_dir);

    #get assOrder file and parse it
    my (@assOrder_files) = &get_assOrders($dir, $parent);
    #print "assOrders are @assOrder_files\n";
    my $file;
    my %data_from_files;
    foreach $file (@assOrder_files)
    {
	$data_from_files{$file} = &parse_assOrder_whole($file, $parent);
    }

    my ($hbond_upl, $prot_upl, $format) = &get_upls($dir, $parent);
    unless ($hbond_upl and $prot_upl)
    {
	&err_msg("No upl files in directory \"$dir\"", $parent);
	return;
    }
    my ($prot_data_ref, $prot_res_ref);
    ($prot_data_ref, $prot_res_ref, $prot_lines_ref) = &parse_upl($prot_upl, $prot_data_ref, $prot_res_ref, $parent, 1, 0, $format);
    my @prot_data = @$prot_data_ref;
    my @prot_lines = @$prot_lines_ref;
        
    my ($hbond_data_ref, $hbond_res_ref);
    ($hbond_data_ref, $hbond_res_ref, $hbond_lines_ref) = &parse_upl($hbond_upl, $hbond_data_ref, $hbond_res_ref, $parent, 1, 0, $format);
    my @hbond_data = @$hbond_data_ref;
    my @hbond_lines = @$hbond_lines_ref;

    #print "place_para_bg...\n";
    
    my ($res_above, $res_below, $next_below);
    my ($sheet_num, $reg_num);
    $reg_num = 0;
    
    for ($sheet_num = 0; $sheet_num < $#drawn_sheets; $sheet_num++)
    {
	my ($first_res, $last_res) = &get_res($drawn_sheets[$sheet_num]);
	my $sheet_above = $drawn_sheets[$sheet_num];
	my $sheet_below = $drawn_sheets[$sheet_num+1];

	#print "1. sheet_above is $sheet_above, sheet_below is $sheet_below\n";

	for ($res_above = $first_res; $res_above <= $last_res; $res_above++)
	{
	    #get res_below
	    my ($reg1, $reg2) = &get_res($drawn_registers[$reg_num]);
	    unless ($sheet_above eq &get_sheet($data_ref, $reg1, $key))
	    {
		my $temp = $reg1;
		$reg1 = $reg2;
		$reg2 = $temp;
	    }

	    $res_below = $reg2 - ($reg1 - $res_above);
	    $prev_below = $res_below - 1;
	    $next_below = $res_below + 1;

	    unless (&belongs_to_sheet($sheet_below, $res_below, $stats{$sheet_below}{'left_to_right'}))
	    {
		#unless ($sheet_key)
		$res_below = 0;
	    }
	    unless (&belongs_to_sheet($sheet_below, $next_below, $stats{$sheet_below}{'left_to_right'}))
	    {
		$next_below = 0;
	    }
	    unless (&belongs_to_sheet($sheet_below, $prev_below, $stats{$sheet_below}{'left_to_right'}))
	    {
		$prev_below = 0;
	    }
	    
	    #print "a. res_above is $res_above, res_below is $res_below, next_below is $next_below, prev_below is $prev_below\n";
	    
	    #get_coords
	    my ($first_below, $last_below) = &get_res($sheet_below);
	    my ($ax, $ay) = &get_coords_of_res($stats{$sheet_above}{'coords'}{'x'}, $stats{$sheet_above}{'coords'}{'y'}, $first_res, 
					       $last_res, $res_above, $stats{$sheet_above}{'left_to_right'});
	    my $res_up = &get_res_up($stats{$sheet_above}{'up'}, $first_res, $res_above);
	    $ax += $x_div;
	    
	    my $elem_above_ref = &get_coords_elements($ax, $ay, $res_up, 1);
	    my %elem_above = %$elem_above_ref;
	    
	    my ($bx, $by) = &get_coords_of_res($stats{$sheet_below}{'coords'}{'x'}, $stats{$sheet_below}{'coords'}{'y'}, 
					       $first_below, $last_below, $res_below, $stats{$sheet_below}{'left_to_right'});
	    $bx += $x_div;
	    
	    my $elem_below_ref = &get_coords_elements($bx, $by, $res_up, $stats{$sheet_below}{'left_to_right'});
	    my %elem_below = %$elem_below_ref;
	    
	    my ($nx, $ny) = &get_coords_of_res($stats{$sheet_below}{'coords'}{'x'}, $stats{$sheet_below}{'coords'}{'y'}, $first_below, $last_below, $next_below, 
							   $stats{$sheet_below}{'left_to_right'});
	    $nx += $x_div;
	    my $elem_next_ref = &get_coords_elements($nx, $ny, !$res_up, $stats{$sheet_below}{'left_to_right'});
	    my %elem_next = %$elem_next_ref;

	    my ($px, $py) = &get_coords_of_res($stats{$sheet_below}{'coords'}{'x'}, $stats{$sheet_below}{'coords'}{'y'}, $first_below, $last_below, $prev_below, 
					       $stats{$sheet_below}{'left_to_right'});
	    $px += $x_div;
	    my $elem_prev_ref = &get_coords_elements($px, $py, !$res_up, $stats{$sheet_below}{'left_to_right'});
	    my %elem_prev = %$elem_prev_ref;
	    
	    #print "b. Residue $res_above is at ($ax, $ay) and up is $up\n";
	    #$canvas->createText($ax, $ay, -text=>$res_above);
	    my ($prot_dat, $prot_line);

	    if ($res_up)
	    {
		if ($res_below)
		{
		    if ($res_above <= $res_below)
		    {
			$prot_dat = $prot_data[$res_above]{'HN'}[$res_below]{'HA'};
			$prot_line = $prot_lines[$res_above]{'HN'}[$res_below]{'HA'};
		    }
		    else
		    {
			$prot_dat = $prot_data[$res_below]{'HA'}[$res_above]{'HN'};
			$prot_line = $prot_lines[$res_below]{'HA'}[$res_above]{'HN'};
		    }
		    my ($peaks_ref) = &get_peak(\%data_from_files, $res_above, 'H', $res_below, 'HA');
		    ($info{'res1'},$info{'atm1'}, $info{'res2'}, $info{'atm2'}) = ($res_above, 'H', $res_below, 'HA');
		    &place_insig_interaction($elem_above{'Hn'}{'x'}, $elem_above{'Hn'}{'y'}, $elem_below{'Ha'}{'x'}, $elem_below{'Ha'}{'y'}, $canvas, $prot_dat, $prot_line, $peaks_ref, $help_ref, \%info, $missing_ref);
		}
		if ($prev_below)
		{
		    my $hbond_dat = $hbond_data[$prev_below]{'O'}[$res_above]{'HN'};
		    my $hbond_line = $hbond_lines[$prev_below]{'O'}[$res_above]{'HN'};
		    
		    &create_H_bond($elem_above{'Hn'}{'x'}, $elem_above{'Hn'}{'y'}, $elem_prev{'OC'}{'x'}, $elem_prev{'OC'}{'y'}, $canvas, $help_ref, $hbond_dat, $hbond_line);
		    if ($data{'NHS'}{$prev_below})
		    {
			&box_H($elem_above{'Hn'}{'x'}, $elem_above{'Hn'}{'y'}, $canvas, 1);
		    }
		    else
		    {
			&box_H($elem_above{'Hn'}{'x'}, $elem_above{'Hn'}{'y'}, $canvas);
		    }
			###############
		    if ($res_above <= $prev_below)
		    {
			$prot_dat = $prot_data[$res_above]{'HN'}[$prev_below]{'HN'};
			$prot_line = $prot_lines[$res_above]{'HN'}[$prev_below]{'HN'};
		    }
		    else
		    {
			$prot_dat = $prot_data[$prev_below]{'HN'}[$res_above]{'HN'};
			$prot_line = $prot_lines[$prev_below]{'HN'}[$res_above]{'HN'};
		    }
		    my ($peaks_ref) = &get_peak(\%data_from_files, $res_above, 'H', $prev_below, 'H');
		    ($info{'res1'},$info{'atm1'}, $info{'res2'}, $info{'atm2'}) = ($res_above, 'H', $prev_below, 'H');
		    &place_insig_interaction($elem_above{'Hn'}{'x'}, $elem_above{'Hn'}{'y'}, $elem_prev{'Hn'}{'x'}, $elem_prev{'Hn'}{'y'}, $canvas, $prot_dat, $prot_line, $peaks_ref, $help_ref, \%info, $missing_ref);
			###############
		}
		if ($next_below)
		{
		    my $hbond_dat = $hbond_data[$res_above]{'O'}[$next_below]{'HN'};
		    my $hbond_line = $hbond_lines[$res_above]{'O'}[$next_below]{'HN'};
		    
		    &create_H_bond($elem_above{'OC'}{'x'}, $elem_above{'OC'}{'y'}, $elem_next{'Hn'}{'x'}, $elem_next{'Hn'}{'y'}, $canvas, $help_ref, $hbond_dat, $hbond_line);
		    
		    if ($res_above <= $next_below)
		    {
			$prot_dat = $prot_data[$res_above]{'HN'}[$next_below]{'HN'};
			$prot_line = $prot_lines[$res_above]{'HN'}[$next_below]{'HN'};
		    }
		    else
		    {
			$prot_dat = $prot_data[$next_below]{'HN'}[$res_above]{'HN'};
			$prot_line = $prot_lines[$next_below]{'HN'}[$res_above]{'HN'};
		    }
		    my ($peaks_ref) = &get_peak(\%data_from_files, $res_above, 'H', $next_below, 'H');
		    ($info{'res1'},$info{'atm1'}, $info{'res2'}, $info{'atm2'}) = ($res_above, 'H', $next_below, 'H');
		    &place_insig_interaction($elem_above{'Hn'}{'x'}, $elem_above{'Hn'}{'y'}, $elem_next{'Hn'}{'x'}, $elem_next{'Hn'}{'y'}, $canvas, $prot_dat, $prot_line, $peaks_ref, $help_ref, \%info, $missing_ref);
		}
	    }
	    else
	    {
		if ($res_below)
		{
		    if ($res_above <= $res_below)
		    {
			$prot_dat = $prot_data[$res_above]{'HA'}[$res_below]{'HN'};
			$prot_line = $prot_lines[$res_above]{'HA'}[$res_below]{'HN'};
		    }
		    else
		    {
			$prot_dat = $prot_data[$res_below]{'HN'}[$res_above]{'HA'};
			$prot_line = $prot_lines[$res_below]{'HN'}[$res_above]{'HA'};
			
		    }
		    
		    my ($peaks_ref) = &get_peak(\%data_from_files, $res_above, 'HA', $res_below, 'H');
		    ($info{'res1'},$info{'atm1'}, $info{'res2'}, $info{'atm2'}) = ($res_above, 'HA', $res_below, 'H');
		    &place_insig_interaction($elem_above{'Ha'}{'x'}, $elem_above{'Ha'}{'y'}, $elem_below{'Hn'}{'x'}, $elem_below{'Hn'}{'y'}, $canvas, $prot_dat, $prot_line, $peaks_ref, $help_ref, \%info, $missing_ref);

		    if ($data{'NHS'}{$res_below})
		    {
			&box_H($elem_below{'Hn'}{'x'}, $elem_below{'Hn'}{'y'}, $canvas, 1);
		    }
		    else
		    {
			&box_H($elem_below{'Hn'}{'x'}, $elem_below{'Hn'}{'y'}, $canvas);
		    }
		}
	    }
	    
	}
	$reg_num++;
    }
    
   
    return ($prot_res_ref);
}


#given the first sheet, and the register, return the whether the start of the first sheet should be up or down
sub get_first_up
{
    my ($data_ref, $my_reg, $sheet, $left_to_right, $key, $sheet_key) = @_;
    #print "get_first_up: my_reg: $my_reg, sheet: $sheet, key is $key\n";

    unless ($sheet_key)
    {
	$sheet_key = 'Temp_ids';
    }

    my %data = %$data_ref;
    my ($strands, $register, $interaction, $up, $temp_res, $temp_atm);
    
    my ($start, $end);
    if ($left_to_right)
    {
	($start, $end) = &get_res($sheet);
    }
    else
    {
	($end, $start) = &get_res($sheet);
    }
    my ($reg1, $reg2) = &get_res($my_reg);
    

    foreach $strands (keys %{$data{'Evidences'}})
    {
	foreach $register (keys %{$data{'Evidences'}{$strands}})
	{
	    if ($register =~ /$my_reg/ and ($register =~ /^\s*$key/))
	    {
		foreach $interaction (keys %{$data{'Evidences'}{$strands}{$register}})
		{
		    $interaction =~ /(\d+)(\w+)\-(\d+)(\w+)/;
		    my ($res1, $atm1, $res2, $atm2) = ($1, $2, $3, $4);
		    
		    my $temp_sheet1 = &get_sheet($data_ref, $res1, $sheet_key);
		    my $temp_sheet2 = &get_sheet($data_ref, $res2, $sheet_key);

		    #print "get_first_up: interaction $interaction, $res1 belongs to sheet $temp_sheet1, $res2 belongs to sheet $temp_sheet2\n";

		    my ($temp1, $temp2) = &get_res($sheet);
		    my $mod_sheet = ($temp1+1).'-'.($temp2-1);
		    
		    if (($sheet eq $temp_sheet1) or ($mod_sheet eq $temp_sheet1))
		    {
			$temp_atm = $atm1;
			$temp_res = $res1;
		    }
		    elsif (($sheet eq $temp_sheet2) or ($mod_sheet eq $temp_sheet2))
		    {
			$temp_atm = $atm2;
			$temp_res = $res2;
		    }
		    else
		    {
			next;
		    }
		    
		    #print "get_first_up: temps in sheet - ($temp_res, $temp_atm)\n";
		    if ($temp_atm eq 'Ha')
		    {
			$up = 0;

			#$up = $inter;
			#print "get_first_up: (interaction - $interaction), atm is Ha, so up is $up for residue $temp_res\n";
			
			($up) = &get_res_up($up, $start, $temp_res);
			return ($up);
		    }
		    elsif ($temp_atm eq 'Hn')
		    {
			$up = 1;
			#print "get_first_up: (interaction - $interaction), atm is Hn, so up is $up for residue $temp_res\n";
			
			$up = &get_res_up($up, $start, $temp_res);
			return ($up);
		    }
		    else
		    {
			next;
		    }
		}

	    }
	}
    }
}

sub place_arrow
{
    my ($canvas, $x1, $y1, $x2, $y2, $key, $line, $help_ref) = @_;

    ($x1, $y1, $x2, $y2) = ($x1, $y1 + 5, $x2, $y2 - 5);

    my $arrow;
    

    if ($key eq 'double')
    {
	$arrow = $canvas->createLine($x1, $y1, $x2, $y2, -arrow=>'both', -capstyle=>'round', -fill=>'blue', -width=>"2");
    }
    elsif ($key eq 'single')
    {
	$arrow = $canvas->createLine($x1, $y1, $x2, $y2, -dash=>'-', -fill=>'blue', -width=>"2");
    }

    $canvas->bind($arrow, '<Button-1>'=>sub
		  {
		      $$help_ref = $line;
		  });
    $canvas->bind($arrow, '<Enter>'=> sub 
		  {
		      $canvas->configure(-cursor=>"crosshair");
		  });
    $canvas->bind($arrow, '<Leave>'=>sub
		  {
		      $canvas->configure(-cursor=>"top_left_arrow");
		  });
    
    return ($arrow);

}

sub get_sheets_id
{
    my ($data_ref, $res1, $res2) = @_;

    my ($id1, $id2, $i);
    my %data = %$data_ref;
    
    my $count = scalar(@{$data{'Final'}});
    #print "get_sheets_id: count = $count\n";

    for ($i = 1; $i<$count; $i++)
    {
	#print "data{'Final'}[$i] = $data{'Final'}[$i]\n";
	my $line = $data{'Final'}[$i];
	$line =~ /(\d+)\-(\d+)$/;
	
	my ($start, $end) = ($1, $2);

	if (($start <= $res1) and ($end >= $res1))
	{
	    $id1 = $i;
	}

	if (($start <= $res2) and ($end >= $res2))
	{
	    $id2 = $i;
	}
    }

    #search for strands
    my $strands;
    my $sheet_id;
    foreach $strands (keys %{$data{'Evidences'}})
    {
	if (($strands eq "$id1-$id2") or ($strands eq "$id2-$id1"))
	{
	    $sheet_id = $strands;
	}
    }

    return ($sheet_id);
}



sub get_coords_of_start
{
    my ($x, $y, $start_res, $end_res, $res_num, $left_to_right) = @_;

    my $start_x;
    if ($left_to_right)
    {
	$start_x = abs($res_num - $start_res) * $x_div;
    }
    else
    {
	$start_x = abs($res_num - $end_res) * $x_div;
    }
    
    my $temp = $x - $start_x;
    #print "get_coords_start: $res_num at ($x,$y), start of ($start_res,$end_res) at ($temp,$y)\n";
    return ($x - $start_x, $y);

}


sub get_coords_next_sheet
{
    my ($x, $y) = @_;

    my $new_y = $y + $y_div;
    return ($x, $new_y);

}


sub create_peptide
{
    my ($canvas, $start_res, $end_res, $x1, $y1, $left_to_right, $up, $data_ref)= @_;
	#print "creating a peptide. startres is $start_res.  endres is $end_res\n";
    my %data = %$data_ref;
    my $cur_res;
    my ($x2, $y2) = ($x1+$x_div, $y1);
	my $font;
	my $peptide_bond_width = 1;

    if ($left_to_right)
    {
	for ($cur_res = $start_res; $cur_res<=$end_res; $cur_res++)
	{
		if ($cur_res == $start_res or $cur_res == $end_res) {
			#print "\tcur res is $cur_res. setting to normal\n";
			$font = "Helvetica 8 normal";
			$peptide_bond_width = 1;
		} else {
			#print "\tcur res is $cur_res. setting to bold\n";
			$font = "Helvetica 8 bold";
			$peptide_bond_width = 2;
			if ($cur_res == $start_res + 1) { 
				$peptide_bond_width = 1;
			}
		}
	    &place_residue($x2, $y2, $canvas, $cur_res, $up, $data{'AA'}[$cur_res], $font);
	    
	    unless ($cur_res == $start_res)
	    {
		&make_peptide_bond($x1, $y1, $x2, $y2, $canvas, $up, $peptide_bond_width);
	    }
	    
	    $x1 += $x_div;
	    $x2 += $x_div;
	    
	    $up = !$up;
	}
    }
    else
    {
	for ($cur_res = $start_res; $cur_res>=$end_res; $cur_res--)
	{
		if ($cur_res == $start_res or $cur_res == $end_res) {
			#print "\tcur res is $cur_res. setting to normal\n";
			$font = "Helvetica 8 normal";
			$peptide_bond_width = 1;
		} else {
			#print "cur res is $cur_res. setting to bold\n";
			$font = "Helvetica 8 bold";
			$peptide_bond_width = 2;
			if ($cur_res == $start_res + 1) { 
				$peptide_bond_width = 1;
			}
		}
	    &place_inverse_residue($x2, $y2, $canvas, $cur_res, $up, $data{'AA'}[$cur_res], $font);
	    unless ($cur_res == $start_res)
	    {
		&make_peptide_bond($x1, $y1, $x2, $y2, $canvas, $up, $peptide_bond_width);
	    }

	    $x1 += $x_div;
	    $x2 += $x_div;
	    $up = !$up;

	}
    }
}

sub create_H_bond
{
    my ($x1, $y1, $x2, $y2, $canvas, $help_ref, $dat, $line) = @_;

    if ($dat)
    {
	my $bond = $canvas->createLine($x1, $y1+5, $x2, $y2-5, -fill=>'magenta', -width=>10);
	$canvas->bind($bond, '<Button-1>'=>sub
				      {
					  $$help_ref = "H Bond: $line";
				      });
	$canvas->bind($bond, '<Enter>'=> sub 
		  {
		      $canvas->configure(-cursor=>"crosshair");
		  });
	$canvas->bind($bond, '<Leave>'=>sub
		  {
		      $canvas->configure(-cursor=>"top_left_arrow");
		  });
    }
    else
    {
	$canvas->createLine($x1, $y1+5, $x2, $y2-5, -fill=>'#444444', -width=>2, -dash=>'.');
    }
}

sub box_H
{
    my ($x, $y, $canvas, $nhs) = @_;
    
    #print "box_H: at ($x,$y)\n";
    if ($nhs)
    {
	$canvas->createRectangle($x-7, $y-7, $x+7, $y+7, -outline=>'orange');
    }
    else
    {
	$canvas->createRectangle($x-7, $y-7, $x+7, $y+7, -outline=>'black');
    }

}


sub place_inverse_residue
{
    my ($x1, $y1, $canvas, $residue_num, $up, $res_name, $font) = @_;
	
    my $coords_ref = &get_coords_elements($x1, $y1, $up, 0);
    my %coords = %$coords_ref;
    
    my $element;
    foreach $element (keys %coords)
    {
	#print "placing element: $element at ($coords{$element}{'x'},$coords{$element}{'y'})\n";
	&place_element($element, $coords{$element}{'x'}, $coords{$element}{'y'}, $canvas, $font);
    }

    my $bond_width = 1;
	if ($font =~ /bold/i) {
		$bond_width = 2;
	}
    if ($up)
    {
	#place bonds
	&make_single_bond($x1+($width/2), $y1+($height/10), $x1+($width/2), $y1+(3*$height/10), $canvas, $bond_width);
	&make_single_bond($x1+($width/8), $y1+(3*$height/5), $x1+(3*$width/8), $y1+(2*$height/5), $canvas, $bond_width);
	&make_double_bond($x1-2, $y1+(7*$height/10), $x1-2, $y1+(9*$height/10), $canvas, $bond_width);
	&make_single_bond($x1+(5*$width/8), $y1+(2*$height/5), $x1+(7*$width/8), $y1+(3*$height/5), $canvas, $bond_width);
	&make_single_bond($x1+$width, $y1+(7*$height/10), $x1+$width, $y1+(9*$height/10), $canvas, $bond_width);
	
	my $txt = "$residue_num$res_name";
	#print "\tcreating text $txt with $font\n";
	$canvas->createText($x1+($width/2), $y1+(3*$height/5), -text=>$txt, -font=>$font);
    }
    else
    {
	#place bonds
	&make_single_bond($x1+($width/2), $y1+(7*$height/10), $x1+($width/2), $y1+(9*$height/10), $canvas, $bond_width);
	&make_single_bond($x1+($width/8), $y1+(2*$height/5), $x1+(3*$width/8), $y1+(3*$height/5), $canvas, $bond_width);
	&make_double_bond($x1-2, $y1+($height/10), $x1-2, $y1+(3*$height/10), $canvas, $bond_width);
	&make_single_bond($x1+(5*$width/8), $y1+(3*$height/5), $x1+(7*$width/8), $y1+(2*$height/5), $canvas, $bond_width);
	&make_single_bond($x1+$width, $y1+($height/10), $x1+$width, $y1+(3*$height/10), $canvas, $bond_width);

	my $txt = "$residue_num$res_name";
	$canvas->createText($x1+($width/2), $y1+(2*$height/5), -text=>$txt, -font=>$font);
    }

}

sub get_coords_elements
{
    my ($x, $y, $up, $left_to_right) = @_;
    #print "get_coords_element: ($x, $y), residue up = $up, left_to_right = $left_to_right\n";
    my %coords;

    if ($left_to_right)
    {
	if ($up)
	{
	    $coords{'Ha'}{'x'} = $x + $width/2;
	    $coords{'Ha'}{'y'} = $y;
	    $coords{'CA'}{'x'} = $x + ($width/2);
	    $coords{'CA'}{'y'} = $y + (2*$height/5);
	    $coords{'N'}{'x'} = $x;
	    $coords{'N'}{'y'} = $y + (3*$height/5);
	    $coords{'Hn'}{'x'} = $x;
	    $coords{'Hn'}{'y'} = $y + $height;
	    $coords{'C'}{'x'} = $x + $width;
	    $coords{'C'}{'y'} = $y + (3*$height/5);
	    $coords{'OC'}{'x'} = $x + $width;
	    $coords{'OC'}{'y'} = $y + $height;
	}
	else
	{
	    $coords{'Ha'}{'x'} = $x + $width/2;
	    $coords{'Ha'}{'y'} = $y + $height;
	    $coords{'CA'}{'x'} = $x + ($width/2);
	    $coords{'CA'}{'y'} = $y + (3*$height/5);
	    $coords{'N'}{'x'} = $x;
	    $coords{'N'}{'y'} = $y + (2*$height/5);
	    $coords{'Hn'}{'x'} = $x;
	    $coords{'Hn'}{'y'} = $y;
	    $coords{'C'}{'x'} = $x + $width;
	    $coords{'C'}{'y'} = $y + (2*$height/5);
	    $coords{'OC'}{'x'} = $x + $width;
	    $coords{'OC'}{'y'} = $y;
	}
    }
    else
    {
	if ($up)
	{
	    $coords{'Ha'}{'x'} = $x + $width/2;
	    $coords{'Ha'}{'y'} = $y;
	    $coords{'CA'}{'x'} = $x + ($width/2);
	    $coords{'CA'}{'y'} = $y + (2*$height/5);
	    $coords{'N'}{'x'} = $x + $width;
	    $coords{'N'}{'y'} = $y + (3*$height/5);
	    $coords{'Hn'}{'x'} = $x + $width;
	    $coords{'Hn'}{'y'} = $y + $height;
	    $coords{'C'}{'x'} = $x;
	    $coords{'C'}{'y'} = $y + (3*$height/5);
	    $coords{'OC'}{'x'} = $x;
	    $coords{'OC'}{'y'} = $y + $height;
	}
	else
	{
	    $coords{'Ha'}{'x'} = $x + $width/2;
	    $coords{'Ha'}{'y'} = $y + $height;
	    $coords{'CA'}{'x'} = $x + ($width/2);
	    $coords{'CA'}{'y'} = $y + (3*$height/5);
	    $coords{'N'}{'x'} = $x + $width;
	    $coords{'N'}{'y'} = $y + (2*$height/5);
	    $coords{'Hn'}{'x'} = $x + $width;
	    $coords{'Hn'}{'y'} = $y;
	    $coords{'C'}{'x'} = $x ;
	    $coords{'C'}{'y'} = $y + (2*$height/5);
	    $coords{'OC'}{'x'} = $x;
	    $coords{'OC'}{'y'} = $y;
	}
    }

    return (\%coords);

}

# add interactions using evidences from assOrder file  
sub place_insig_interaction
{
    my ($x1, $y1, $x2, $y2, $canvas, $dat, $line, $peaks_ref, $help_ref, $info_ref, $missing_ref) = @_;

    my @peaks = @$peaks_ref;
    
    #print "placing insig_interaction from ($x1,$y1) to ($x2,$y2) with dat = $dat, and line = $line, peaks are @peaks\n";
    my $msg = $line;
    if ($peaks[1])
    {
	$msg = $msg."\n$peaks[0]\n$peaks[1]";
	&place_arrow($canvas, $x1, $y1, $x2, $y2, 'double', $msg, $help_ref);
	#print "placed_double_arrow\n";
    }
    elsif ($peaks[0])
    {
	$msg = $msg."\n$peaks[0]";
	&place_arrow($canvas, $x1, $y1, $x2, $y2, 'single', $msg, $help_ref);
	
    }
    else
    {
	$canvas->createLine($x1, $y1+5, $x2, $y2-5, -fill=>'#444488');
	#no peaks found missing interaction
	
	my $key = "$info_ref->{'res1'}-$info_ref->{'atm1'} $info_ref->{'res2'}-$info_ref->{'atm2'}";
	$missing_ref->{$key} = 1;
    }
}


sub place_residue
{
    my ($x1, $y1, $canvas, $residue_num, $up, $res_name, $font) = @_;
    
    my $coords_ref = &get_coords_elements($x1, $y1, $up, 1);
    my %coords = %$coords_ref;
    
    my $element;
    foreach $element (keys %coords)
    {
	&place_element($element, $coords{$element}{'x'}, $coords{$element}{'y'}, $canvas, $font);
    }
    my $bond_width = 1;
	if ($font =~ /bold/i) {
		$bond_width = 2;
	}
    my $txt = "$residue_num$res_name";
    #place atoms
    if ($up)
    {
	#place bonds
	&make_single_bond($x1+($width/2), $y1+($height/10), $x1+($width/2), $y1+(3*$height/10), $canvas, $bond_width);
	&make_single_bond($x1+($width/8), $y1+(3*$height/5), $x1+(3*$width/8), $y1+(2*$height/5), $canvas, $bond_width);
	&make_single_bond($x1+(5*$width/8), $y1+(2*$height/5), $x1+(7*$width/8), $y1+(3*$height/5), $canvas, $bond_width);
	&make_single_bond($x1, $y1+(7*$height/10), $x1, $y1+(9*$height/10), $canvas, $bond_width);
	&make_double_bond($x1+$width-2, $y1+(7*$height/10), $x1+$width-2, $y1+(9*$height/10), $canvas, $bond_width);

	#place residue number
	$canvas->createText($x1+($width/2), $y1+(3*$height/5), -text=>$txt, -font=>$font);
	
    }
    else
    {
	#place bonds
	&make_single_bond($x1, $y1+($height/10), $x1, $y1+(3*$height/10), $canvas, $bond_width);
	&make_single_bond($x1+($width/8), $y1+(2*$height/5), $x1+(3*$width/8), $y1+(3*$height/5), $canvas, $bond_width);
	&make_single_bond($x1+(5*$width/8), $y1+(3*$height/5), $x1+(7*$width/8), $y1+(2*$height/5), $canvas, $bond_width);
	&make_single_bond($x1+($width/2), $y1+(7*$height/10), $x1+($width/2), $y1+(9*$height/10), $canvas, $bond_width);
	&make_double_bond($x1+$width-2, $y1+($height/10), $x1+$width-2, $y1+(3*$height/10), $canvas, $bond_width);

	$canvas->createText($x1+($width/2), $y1+(2*$height/5), -text=>$txt, -font=>$font);
    }


}

sub make_peptide_bond
{
    my ($x1, $y1, $x2, $y2, $canvas, $up, $bond_width) = @_;

    if ($up)
    {
	$canvas->createLine($x1+$width+5, $y1+(2*$height/5), $x2-5, $y2+(3*$height/5), -fill=>'red', -width=>$bond_width);
    }
    else
    {
	$canvas->createLine($x1+$width+5, $y1+(3*$height/5), $x2-5, $y2+(2*$height/5), -fill=>'red', -width=>$bond_width);
    }
    
}


sub make_double_bond
{
    my ($x1, $y1, $x2, $y2, $canvas, $width) = @_;
    
    &make_single_bond($x1, $y1, $x2, $y2, $canvas, $width);
    &make_single_bond($x1+3, $y1, $x2+3, $y2, $canvas, $width);

}

sub make_single_bond
{
    my ($x1, $y1, $x2, $y2, $canvas, $width) = @_;

    $canvas->createLine($x1, $y1, $x2, $y2, -width=>$width);

}


sub place_element
{
    my ($letter, $x, $y, $canvas, $font) = @_;
    
    if ($letter =~ /^(\w)\w$/)
    {
	$letter = $1;
    }
    
    #print "place_element: $letter, $x, $y\n";
    my $element = $canvas->createText($x, $y, -text=>$letter, -font=>$font);
    return ($element);

}

return 1;
