
my %default_distances = (
			 "d(HAi - HNi+1),Anti_Beta" => 2.2,
			 "d(HAi - HNi+1),Para_Beta" => 2.2,
			 "d(HAi - HNi+1),Alpha_Helix" => 3.4,
			 "d(HNi - HNi+1),Alpha_Helix" => 2.6,
			 "d(HNi - HNi+2),Alpha_Helix" => 4.1,
			 "d(HAi - HNi+2),Alpha_Helix" => 3.8,
			 "d(HAi - HNi+3),Alpha_Helix" => 3.3);
			 

sub calib_k
{
    my ($parent) = @_;

    my $main = $parent->Toplevel();

    my %data;
    my %count;

    #directory
    my $dir;
    my $frm1 = $main->Frame()->pack(-fill=>'both', -expand=>1);
    my $lbl1 = $frm1->Label(-text=>"AS Output Directory : ")->pack(-side=>'left');
    my $ent1 = $frm1->Entry(-background=>'white', -textvariable=>\$dir)->pack(-side=>'left', -fill=>'x', -expand=>1);
    my $but1 = $frm1->Button(-text=>'Browse', -command=>sub
			     {
				 $dir = &OpenDir('AS Dir', $main, Cwd::cwd);
			     })->pack(-side=>'left');
    my $but2 = $main->Button(-text=>"Show Plots")->pack(-fill=>'both', -expand=>1);
    $but2->configure(-command=>sub
		     {
			 if (-d $dir)
			 {
			     # parse the .sec file to get the secondary structure elements
			     
			     my ($sec_str_ref, $sec_data_ref) = &get_sec_str_k($dir, $main);
			     return unless ($sec_str_ref);
			     
			     #get and parse assOrder file
			     unless (opendir(DIR, $dir))
			     {
				 &err_msg("Cannot open directory \"$dir\"", $main);
				 return;
			     }
			     
			     my @contents = sort {$a cmp $b || $a <=> $b} readdir(DIR);
			     close(DIR);
			     
			     my ($cyc_dir) = grep {/^CYCLE1.+final$/} @contents;
			     unless ($cyc_dir)
			     {
				 ($cyc_dir) = grep {/Working/i} @contents;
			     }
			     
			     unless ($cyc_dir)
			     {
				 &err_msg("No Cycle 1 or Working cycle in directory $as_dir", $main);
				 return;
			     }
			     $cyc_dir = File::Spec->rel2abs($cyc_dir, $dir);

			     #get and parse the assOrder files
			     
			     my (@assOrder_files) = &get_assOrders($cyc_dir, $main);
			     foreach $file (@assOrder_files)
			     {
				 my ($data_ref, $count_ref) = &parse_assOrder_k($file, $main, $sec_data_ref, $sec_str_ref, \%data, \%count_ref);
				 %data = %$data_ref;
				 %count = %$count_ref;
			     }
			 
			     #print "dir is $dir, assOrders are @assOrder_files\n";
			     

			     #show window with option of three plots, %data contains data from those files
			     &show_plots($main, $dir, \%data, \%count);
			 }
		     });
}
   
    


sub show_plots
{
    my ($parent, $dir, $data_ref, $count_ref) = @_;
    
    my $win = $parent->Toplevel();
    $win->configure(-title=>"Plots for AS Dir $dir");
    
    #buttons
    my $lbl = $win->Label(-text=>"Show Plots")->pack(-fill=>'x');
    my $but1 = $win->Button(-text=>"1 - (1/r^6) vs Intensity")->pack(-fill=>'x');
    my $but2 = $win->Button(-text=>"2 - (ln r) vs (ln Intensity)")->pack(-fill=>'x');
    my $but3 = $win->Button(-text=>"3 - (1/r^exp) vs Intensity")->pack(-fill=>'x');
    
    my $file;
    my %data = %$data_ref;   #data from assOrder files, intensities
    #my %count = %$count_ref;
    
    $but1->configure(-command=>sub
		     {

			 # asks which of the eight distances to display
			 my ($dist_ref) = &ask_distances($win, "1 - (1/r^6) vs Intensity");   #contains r values
			 my %dist = %$dist_ref;
			 my %mod_dist;
			 my $lbl;
			 my $i;
			 my $max_intensity = 0;
			 my $max_dist = 0;
			 foreach $lbl (keys %dist)
			 {
			     #print "$lbl, dist is ${$dist{$lbl}}\n";
			     if (${$dist{$lbl}} and ($data{$lbl}))
			     {
				 # mod_dist contains 1/r^6 values
				 $mod_dist{$lbl} = sprintf("%0.4f", 1 / (${$dist{$lbl}} ** 6));
				 
				 #print "$lbl - distance is ${$dist{$lbl}}; mod_dist is $mod_dist{$lbl}\n";
				 
				 #print "$lbl\nDistance is $mod_dist{$lbl}\nIntensities are (@{$data{$lbl}})\n\n";
				 for ($i = 0; $i<scalar(@{$data{$lbl}}); $i++)
				 {
				     # max_intensity and $max_dist are calculated for scaling the x and y axis in the plots
				     if ($data{$lbl}[$i] > $max_intensity)
				     {
					 $max_intensity = $data{$lbl}[$i];
				     }
				     
				 }
				 
				 if ($mod_dist{$lbl} > $max_dist)
				 {
				     $max_dist = $mod_dist{$lbl};
				 }
			     }
			 }
			 
		     return unless ($dist_ref);
		     #put points on the canvas
		     &show_plot_data($win, \%dist, \%mod_dist, \%data, "1/(r ^ 6)", 'Intensity', $max_intensity, $max_dist);  #this function is in file create_k_plot.pl
		     
		    
		 });


    $but2->configure(-command=>sub
		     {
			 my ($dist_ref) = &ask_distances($win, "2 - (ln r) vs (ln Intensity)");
			 my %dist = %$dist_ref;
			 my %mod_dist;
			 my %mod_int;
			 my $lbl;
			 my $i;
			 my $max_intensity = 0;
			 my $max_dist = 0;
			 foreach $lbl (keys %dist)
			 {
			     if (${$dist{$lbl}} and ($data{$lbl}))
			     {
				 $mod_dist{$lbl} = sprintf("%0.4f", log(${$dist{$lbl}}));
				 
				 #print "$lbl - distance is ${$dist{$lbl}}; mod_dist is $mod_dist{$lbl}\n";
				 
				 #print "$lbl\nlog of Distance is $mod_dist{$lbl}\nIntensities are (@{$data{$lbl}})\n\n";
				 for ($i = 0; $i<scalar(@{$data{$lbl}}); $i++)
				 {
				     next unless ($data{$lbl}[$i]);
				     # calculate log(intensity) and put in hash mod_int
				     $mod_int{$lbl}[$i] = sprintf("%0.5f", log($data{$lbl}[$i]));
				     if ($mod_int{$lbl}[$i] > $max_intensity)
				     {
					 $max_intensity = $mod_int{$lbl}[$i];
				     }
				     
				 }
				 
				 if ($mod_dist{$lbl} > $max_dist)
				 {
				     $max_dist = $mod_dist{$lbl};
				 }
			     }
			 }
			 
			 return unless ($dist_ref);
			 #put points on the canvas
			 &show_plot_data($win, \%dist, \%mod_dist, \%mod_int, "log(Distance)", "log(Intensity)", $max_intensity, $max_dist, \%data);
		     });

    $but3->configure(-command=>sub
		     {
			 my ($dist_ref) = &ask_distances($win, "3 - (1/r^exp) vs Intensity");
			 my $exp;
			 #ask exponent
			 my $new_win = $win->Toplevel();
			 $new_win->configure(-title=>"Enter Exponent for Distance calculation");
			 my $frm1 = $new_win->Frame()->pack(-fill=>'both', -expand=>1);
			 my $lbl1 = $frm1->Label(-text=>"Exponent in (1 / r ^ exp) = ")->pack(-side=>'left');
			 my $ent1 = $frm1->Entry(-background=>'white', -textvariable=>\$exp)->pack(-side=>'left');
			 my $but1 = $new_win->Button(-text=>"OK", -command=>sub
						     {
							 if ($exp)
							 {
							     $new_win->destroy();
							 }
						     })->pack(-fill=>'x', -expand=>1);

			 
			 $new_win->waitWindow();

			 my %dist = %$dist_ref;
			 my %mod_dist;
			 my %mod_int;
			 my $lbl;
			 my $i;
			 my $max_intensity = 0;
			 my $max_dist = 0;
			 foreach $lbl (keys %dist)
			 {
			     if (${$dist{$lbl}} and ($data{$lbl}))
			     {
				 $mod_dist{$lbl} = sprintf("%0.4f", (${$dist{$lbl}} ** $exp));
				 
				 #print "$lbl - distance is ${$dist{$lbl}}; mod_dist is $mod_dist{$lbl}\n";
				 
				 #print "$lbl\nlog of Distance is $mod_dist{$lbl}\nIntensities are (@{$data{$lbl}})\n\n";
				 for ($i = 0; $i<scalar(@{$data{$lbl}}); $i++)
				 {
				     if ($data{$lbl}[$i] > $max_intensity)
				     {
					 $max_intensity = $data{$lbl}[$i];
				     }
				     
				 }
				 
				 if ($mod_dist{$lbl} > $max_dist)
				 {
				     $max_dist = $mod_dist{$lbl};
				 }
			     }
			 }
			 
			 return unless ($dist_ref);
			 #put points on the canvas
			 &show_plot_data($win, \%dist, \%mod_dist, \%data, "1/(r ^ $exp)", "Intensity", $max_intensity, $max_dist);
		     });
}





sub ask_distances
{
    my ($parent, $title) = @_;

    my %dist;
    my $win = $parent->Toplevel();
    $win->configure(-title=>$title);

    my $frm1 = $win->Frame(-relief=>'ridge', -borderwidth=>2)->pack(-fill=>'x', -expand=>1, -padx=>5, -pady=>5);
    $frm1->Label(-relief=>'ridge', -borderwidth=>2, -text=>'Atoms')->pack(-side=>'left', -fill=>'x', -expand=>1);
    $frm1->Label(-relief=>'ridge', -borderwidth=>2, -text=>'Sec. Str.')->pack(-side=>'left', -fill=>'x', -expand=>1);
    $frm1->Label(-relief=>'ridge', -borderwidth=>2, -text=>'Distance')->pack(-side=>'left', -fill=>'x', -expand=>1);

    my @lbls = ("d(HAi - HNi+1),Anti_Beta", "d(HAi - HNi+1),Para_Beta", "d(HAi - HNi+1),Alpha_Helix", "d(HNi - HNi+1),Alpha_Helix", "d(HNi - HNi+2),Alpha_Helix", "d(HAi - HNi+2),Alpha_Helix", "d(HAi - HNi+3),Alpha_Helix");

    my $lbl;
    foreach $lbl (@lbls)
    {
	my ($lbl1, $lbl2) = split(',', $lbl);
	($dist{$lbl}) = &create_lbl_frm($win, $lbl1, $lbl2);
	${$dist{$lbl}} = $default_distances{$lbl};
    }

    my $but = $win->Button(-text=>'Show')->pack(-fill=>'x');

    $but->configure(-command=>sub
		    {
			$win->destroy();
		    });
    $win->waitWindow;
    
    
    return (\%dist);
}

sub create_lbl_frm
{
    my ($parent, $lbl1, $lbl2) = @_;

    my $frame = $parent->Frame()->pack(-fill=>'x', -expand=>1, -padx=>5, -pady=>5);

    $frame->Label(-text=>$lbl1)->pack(-side=>'left', -fill=>'x', -padx=>2);
    $frame->Label(-text=>$lbl2)->pack(-side=>'left', -fill=>'x', -padx=>2);
    
    my $ent_var;
    $frame->Entry(-background=>'white', -textvariable=>\$ent_var)->pack(-side=>'left', -fill=>'x');

    return (\$ent_var);
}


return 1;
