sub conv_all # call conversion pdb function 
{
    #given a pdb file, converts it to the IUPAC (bmrb) format and vice versa
    my ($pdb_file, $out_file, $parent, $key) = @_;

    unless (open(PDB, $pdb_file))
    {
	&err_msg("Cannot open pdb file \"$pdb_fil\"", $parent);
	return;
    }
    
    unless (open(OUT, ">$out_file"))
    {
	&err_msg("Cannot open output file \"$out_file\"", $parent);
	return;
    }

    my $line;
    my ($num, $temp, $spc, $i);

    my ($atoms_ref) = &parse_atom_nom("atom_nom.tbl");
    my %atoms = %$atoms_ref;

    print "pdb_file is $pdb_file, out_file is $out_file\n";

    while (chomp($line = <PDB>))
    {
	if ($line =~ /^atom/i)
	{
	    my $res_name = &get_info($line, 17, 3);
	    my $res_atm = &get_info($line, 12, 4);
	    my $new_atm;
	    
	    if ($key eq 'PDB,IUPAC')
	    {
		$new_atm = &conv_pdb_to_iupac_atom($res_atm, $res_name);
	    }
	    elsif ($key eq 'IUPAC,PDB')
	    {
		$new_atm = &conv_iupac_to_pdb_atom($res_atm, $res_name);
	    }
	    elsif ($key eq 'IUPAC,XPLOR')
	    {
		$new_atm = &conv_iupac_to_xplor_atom($res_atm, $res_name);
	    }
	    elsif ($key eq 'XPLOR,IUPAC')
	    {
		$new_atm = &conv_xplor_to_iupac_atom($res_atm, $res_name);
	    }
	    elsif ($key eq 'PDB,XPLOR')
	    {
		$new_atm = &conv_pdb_to_iupac_atom($res_atm, $res_name);
		$new_atm = &conv_iupac_to_xplor_atom($new_atm, $res_name);
	    }
	    elsif ($key eq 'XPLOR,PDB')
	    {
		$new_atm = &conv_xplor_to_iupac_atom($res_atm, $res_name);
		$new_atm = &conv_iupac_to_pdb_atom($new_atm, $res_name);
	    }
	    #$line =~ s/$res_atm/$new_atm/;
	    $temp = length($new_atm);
	    
	    $num = 4 - $temp;
	    for ($i = 0; $i<$num; $i++)
	    {
		$new_atm = $new_atm.' ';
	    }
	    
	    substr($line, 12, 4, $new_atm);
	    $temp = &get_info($line, 12, 4);
	    #print "($res_name) $res_atm is converted to $new_atm (temp is $temp)\n";
	    print OUT "$line\n";
	}
	else
	{
	    print OUT "$line\n";
	}

    }

    #print "length of $t is ".length($t)."\n";
    return 1;
}

sub conv_iupac_to_xplor_atom
{
    my ($atom, $res_name) = @_;
    my $new_atom = $atom;

    if ($res_name =~ /(ARG|ASP|ASN|CYS|GLU|GLN|GLY|LEU|PRO|SER)/)
    {
	if ($atom =~ /(H.*\D)(\d)$/)
	{
	    my $num = $2;
	    $num--;
	    $new_atom = $1.$num;
	}
    }
    elsif ($res_name =~ /(HIS|PHE|TRP|TYR)/)
    {
	if ($atom =~ /(HB.*\D)(\d)$/)
	{
	    my $num = $2;
	    $num--;
	    $new_atom = $1.$num;
	}
    }
    elsif ($res_name =~ /MET/)
    {
	if ($atom =~ /(^H[BG].*)(\d)$/)
	{
	    my $num = $2;
	    $num--;
	    $new_atom = $1.$num;
	}
    }
    elsif ($res_name =~ /ILE/)
    {
	if ($atom =~ /^(HG1.*)(\d)$/)
	{
	    my $num = $2;
	    $num--;
	    $new_atom = $1.$num;
	}
    }
    elsif ($res_name =~ /LYS/)
    {
	if ($atom =~ /(H[BGDE])(\d)$/)
	{
	    my $num = $2;
	    $num--;
	    $new_atom = $1.$num;
	}
    }
    
    if ($atom eq 'H')
    {
	$new_atom = 'HN';
    }

    return ($new_atom);
    
}

sub conv_xplor_to_iupac_atom
{
    my ($atom, $res_name) = @_;
    my $new_atom = $atom;

    if ($res_name =~ /(ARG|ASP|ASN|CYS|GLU|GLN|GLY|LEU|PRO|SER)/)
    {
	if ($atom =~ /(H.*\D)(\d)$/)
	{
	    my $num = $2;
	    $num++;
	    $new_atom = $1.$num;
	}
    }
    elsif ($res_name =~ /(HIS|PHE|TRP|TYR)/)
    {
	if ($atom =~ /(HB.*\D)(\d)$/)
	{
	    my $num = $2;
	    $num++;
	    $new_atom = $1.$num;
	}
    }
    elsif ($res_name =~ /(ILE|MET)/)
    {
	my $find;
	$find = 'HG1' if ($res_name =~ /ILE/);
	$find = 'HE' if ($res_name =~ /MET/);

	if ($atom =~ /($find.*)(\d)$/)
	{
	    my $num = $2;
	    $num++;
	    $new_atom = $1.$num;
	}
    }
    elsif ($res_name =~ /LYS/)
    {
	if ($atom =~ /(H[BGDE])(\d)$/)
	{
	    my $num = $2;
	    $num++;
	    $new_atom = $1.$num;
	}
    }
    
    if ($atom eq 'HN')
    {
	$new_atom = 'H';
    }

    return ($new_atom);
    
}

sub conv_pdb_to_iupac_atom
{
    my ($atom, $res_name) = @_;
    my $new_atom = $atom;
    
    if ($res_name =~ /(GLY|ASP|CYS|GLU|HIS|PHE|PRO|SER|TRP|TYR)/)
    {
	if ($atom =~ /^(\d)(H.+)$/)
	{
	    my $num = $1;
	    $num++;
	    $new_atom = $2.$num;
	}
    }
    elsif ($res_name =~ /(ARG|ASN|GLN|LEU)/)
    {
	if ($atom =~ /^(\d)(H\w\d)$/)
	{
	    $new_atom = $2.$1;
	}
	elsif ($atom =~ /^(\d)(H.*\D)$/)
	{
	    my $num = $1;
	    $num++;
	    $new_atom = $2.$num;
	}
    }
    elsif ($res_name =~ /(LYS|MET)/)
    {
	my $temp;
	$temp = 'HZ' if ($res_name =~ /LYS/);
	$temp = 'HE' if ($res_name =~ /MET/);
	
	if ($atom =~ /^(\d)($temp.*)$/)
	{
	    $new_atom = $2.$1;
	}
	elsif ($atom =~ /^(\d)(H.+)$/)
	{
	    my $num = $1;
	    $num++;
	    $new_atom = $2.$num;
	}
    }
    elsif ($res_name =~ /ILE/)
    {
	if ($atom =~ /^(\d)(HG1.*)$/)
	{
	    my $num = $1;
	    $num++;
	    $new_atom = $2.$num;
	}
	elsif ($atom =~ /^(\d)(H.*)$/)
	{
	    $new_atom = $2.$1;
	}
    }
    else
    {
	if ($atom =~ /^(\d)(H.+)$/)
	{
	    $new_atom = $2.$1;
	}
    }

    if ($atom eq 'HN')
    {
	$new_atom = 'H';
    }

    return ($new_atom);
}


sub conv_iupac_to_pdb_atom
{
    my ($atom, $res_name) = @_;
    my $new_atom = $atom;

    if ($res_name =~ /(ALA|THR|VAL)/)
    {
	if ($atom =~ /^(H.*)(\d)$/)
	{
	    $new_atom = $2.$1;
	}
    }
    elsif (($res_name =~ /THR/) and ()
    {
	
    }
    elsif ($res_name =~ /(GLY)/)
    {
	if ($atom =~ /(HA2|2HA)/)
	{
	    $new_atom = '1HA';
	}
	elsif ($atom =~ /(HA3|3HA)/)
	{
	    $new_atom = '2HA';
	}
    }
    elsif ($res_name =~ /(HIS|PHE|TRP|TYR|ASP|CYS|SER)/)
    {
	if ($atom =~ /^(HB.*)(\d)$/)
	{
	    my $num = $2;
	    $num--;
	    $new_atom = $num.$2;
	}
	elsif ($atom =~ /^(\d)(HB.*)$/)
	{
	    my $num = $1;
	    $num--;
	    $new_atom = $num.$2;
	}
    }
    elsif ($res_name =~ /(ARG|GLN|LEU|GLU|PRO)/)
    {
	if ($atom =~ /^(H[BGD])(\d)$/)
	{
	    my $num = $2;
	    $num--;
	    $new_atom = $num.$1;
	}
	elsif ($atom =~ /^(\d)(H[BGD])$/)
	{
	    my $num = $1;
	    $num--;
	    $new_atom = $num.$2;
	}
	elsif ($atom =~ /^(H.*)(\d)$/)
	{
	    if (($res_name =~ /GLN/) and ($atom =~ /HE/))
	    {
		if ($atom =~ /(HE21|1HE2)/)
		{
		    $new_atom = '2HE2';
		}
		elsif ($atom =~ /(HE22|2HE2)/)
		{
		    $new_atom = '1HE2';
		}
	    }
	    elsif (($res_name =~ /PRO/) and ($atom =~ /H\d/))
	    {
		if ($atom =~ /(H3|3H)/)
		{
		    $new_atom = 'H1';
		}
	    }
	    else
	    {
		$new_atom = $2.$1 unless ($res_name =~ /GLU/);
	    }
	}
    }
    elsif ($res_name =~ /ASN/)
    {
	if ($atom =~ /^(HB)(\d)$/)
	{
	    my $num = $2;
	    $num--;
	    $new_atom = $num.$1;
	}
	elsif ($atom =~ /^(\d)(HB)$/)
	{
	    my $num = $1;
	    $num--;
	    $new_atom = $num.$2;
	}
	elsif (($atom =~ /^1HD2$/) or ($atom =~ /^HD21$/))
	{
	    $new_atom = '2HD2';
	}
	elsif ($atom =~ /^(2HD2|HD22)$/)
	{
	    $new_atom = '1HD2';
	}
	elsif ($atom =~ /^(H.*)(\d)$/)
	{
	    $new_atom = $2.$1;
	}
    }
    elsif ($res_name =~ /(ILE)/)
    {
	if ($atom =~ /(HG1)(\d)$/)
	{
	    my $num = $2;
	    $num--;
	    $new_atom = $num.$1;
	}
	elsif ($atom =~ /(\d)(HG1)$/)
	{
	    my $num = $1;
	    $num--;
	    $new_atom = $num.$2;
	}
	elsif ($atom =~ /(H.*)(\d)$/)
	{
	    $new_atom = $2.$1;
	}
    }
    elsif ($res_name =~ /(LYS)/)
    {
	if ($atom =~ /^(H[BGDE])(\d)$/)
	{
	    my $num = $2;
	    $num--;
	    $new_atom = $num.$1;
	}
	elsif ($atom =~ /^(\d)(H[BGDE])$/)
	{
	    my $num = $1;
	    $num--;
	    $new_atom = $num.$2;
	}
	elsif ($atom =~ /^(H.*)(\d)$/)
	{
	    $new_atom = $2.$1;
	}
    }
    elsif ($res_name =~ /(MET)/)
    {
	if ($atom =~ /^(H[BG])(\d)$/)
	{
	    my $num = $2;
	    $num--;
	    $new_atom = $num.$1;
	}
	elsif ($atom =~ /^(\d)(H[BG])$/)
	{
	    my $num = $1;
	    $num--;
	    $new_atom = $num.$2;
	}
	elsif ($atom =~ /(H.*)(\d)$/)
	{
	    $new_atom = $2.$1;
	}
    }
    
    if ($atom eq 'HN')
    {
	$new_atom = 'H';
    }


    return ($new_atom);


}


# old stuff take this out 
sub conv_pdb_gui
{

    #convert pdb format to IUPAC format
    my ($main, $basepath) = @_;

    my ($pdb_file, $out_file);
    
    #create window with two entries for pdb file and output file

    my $win = $main->Toplevel(-title=>"File conversion from pdb format to iupac format");
    
    my $lbl = $win->Label(-text=>"PDB to IUPAC format", -relief=>'ridge')->pack(-fill=>'x', -padx=>3, -pady=>5);
    
    my $frm1 = $win->Frame()->pack(-fill=>'x', -padx=>3, -pady=>3);
    my $pdb_lbl = $frm1->Label(-text=>'Input PDB File: ')->pack(-side=>'left', -padx=>3);
    my $pdb_ent = $frm1->Entry(-background=>'white', -textvariable=>\$pdb_file)->pack(-fill=>'x', -padx=>3, -side=>'left');
    my $pdb_brw = $frm1->Button(-text=>'Browse', -command=>sub 
				{
				    $pdb_file = &OpenFile('PDB File', $win, $basepath);
				})->pack(-side=>'left', -padx=>3);

    
    my $frm2 = $win->Frame()->pack(-fill=>'x', -padx=>3, -pady=>3);
    my $out_lbl = $frm2->Label(-text=>'Output File: ')->pack(-side=>'left', -padx=>3);
    my $out_ent = $frm2->Entry(-background=>'white', -textvariable=>\$out_file)->pack(-fill=>'x', -padx=>3, -side=>'left');
    my $out_brw = $frm2->Button(-text=>'Browse', -command=>sub 
				{
				    $out_file = &OpenFile('Output File', $win, $basepath);
				})->pack(-side=>'left', -padx=>3);

    my $con_but = $win->Button(-text=>'Convert', -command=>sub
			       {
				   unless (-f $pdb_file)
				   {
				       &err_msg("Enter a pdb file to convert", $win);
				       return;
				   }

				   unless ($out_file)
				   {
				       &err_msg("Enter an output file", $win);
				       return;
				   }
				   my $warn = "Please choose a different name for the output file";
				   if ($out_file eq $pdb_file)
				   {
				       my $dlg = $win->Dialog(-text=>$warn, -buttons=>['Ok'], -bitmap=>'warning');
				       my $ans = $dlg->Show;

				       return;
				   }
				   
				   if (&conv_pdb($pdb_file, $out_file, $win)) #call conversion function 
				   {
				       my $dlg = $win->Dialog(-text=>"Conversion Done!", -buttons=>['OK'], -bitmap=>'info');
				       my $ans = $dlg->Show;

				       if ($ans eq 'OK')
				       {
					   $win->destroy();
				       }
				       
				   }
				   else
				   {
				       &err_msg("Could not convert pdb file", $win);
				   }
			       })->pack(-padx=>3, -pady=>5, -ipadx=>2, -ipady=>2);
    
}

return 1;
