my $scale = 6;
my $col1 = "#FFEFEF";
my $col2 = "#E3F2E3";

my $bg1 = "#D7E6E6";

my %aa = (
    "ALA"=>"A",
    "ARG"=>"R",
    "ASN"=>"N",
    "ASP"=>"D",
    "CYS"=>"C",
    "GLN"=>"Q",
    "GLU"=>"E",
    "GLY"=>"G",
    "HIS"=>"H",
    "ILE"=>"I",
    "LEU"=>"L",
    "LYS"=>"K",
    "MET"=>"M",
    "PHE"=>"F",
    "PRO"=>"P",
    "SER"=>"S",
    "THR"=>"T",
    "TRP"=>"W",
    "TYR"=>"Y",
    "VAL"=>"V"
	  );


sub create_contact_map
{

    my ($data_ref, $residue_ref, $parent, $dir, $start_res, $end_res);
    ($data_ref, $residue_ref, $parent, $dir, $scale, $start_res, $end_res) = @_;
    
    my @residue = @$residue_ref;
    my @data = @$data_ref;
    my $rel_dir = File::Spec->abs2rel($dir);
    my $total_residues;
    

    unless ($start_res)
    {
	$start_res = 1;
    }
    unless ($end_res)
    {
	$end_res = $#residue;
    }
    
    my $bg1 = 'white';
    my $bg2 = '#EEDDEE';
    my $bg3 = '#AADDEE';

    $total_residues = ($end_res - $start_res) + 1;

    print "create_contact_map: total_residues is $total_residues\n";

    #create toplevel
    my $window = $parent->Toplevel(-title=>"Contact Map for $rel_dir");
    $window->configure(-background=>$bg1);


    my $m_frm = $window->Frame(-relief=>'ridge', -borderwidth=>2, -background=>$bg1)->pack(-side=>'top', -fill=>'x', -expand=>1, -pady=>5);
    my %zoom_vars;
    my $m_lbl = $m_frm->Label(-text=>"Zoom in (Enter Residue Numbers) {Total Residues - $total_residues}", -background=>$bg2)->pack(-fill=>'x', -expand=>1);

    my $m_frm_sub = $m_frm->Frame(-background=>$bg2)->pack(-fill=>'x', -expand=>1);

    my $m_frm1 = $m_frm_sub->Frame(-background=>$bg3, -relief=>'groove', -borderwidth=>2)->pack(-fill=>'x', -expand=>1, -side=>'left', -padx=>5);
    my $m1_lbl1 = $m_frm1->Label(-text=>'X from : ', -background=>$bg3)->pack(-side=>'left');
    my $m1_ent1 = $m_frm1->Entry(-background=>'white', -textvariable=>\$zoom_vars{'x1'})->pack(-side=>'left');
    my $m1_lbl2 = $m_frm1->Label(-text=>' to : ', -background=>$bg3)->pack(-side=>'left');
    my $m1_ent2 = $m_frm1->Entry(-background=>'white', -textvariable=>\$zoom_vars{'x2'})->pack(-side=>'left');

    my $m_frm2 = $m_frm_sub->Frame(-background=>$bg3, -relief=>'groove', -borderwidth=>2)->pack(-fill=>'x', -expand=>1, -side=>'left', -padx=>5);
    my $m2_lbl1 = $m_frm2->Label(-text=>'Y from : ', -background=>$bg3)->pack(-side=>'left');
    my $m2_ent1 = $m_frm2->Entry(-background=>'white', -textvariable=>\$zoom_vars{'y1'})->pack(-side=>'left');
    my $m2_lbl2 = $m_frm2->Label(-text=>' to : ', -background=>$bg3)->pack(-side=>'left');
    my $m2_ent2 = $m_frm2->Entry(-background=>'white', -textvariable=>\$zoom_vars{'y2'})->pack(-side=>'left');

    my $m3_but = $m_frm_sub->Button(-text=>'Zoom')->pack(-side=>'left');

    my $q_frm = $window->Frame(-relief=>'ridge', -borderwidth=>3, -background=>$bg2)->pack(-side=>'top', -fill=>'x', -expand=>1);
    my ($res1_var, $res2_var);
    
    my $q_lbl = $q_frm->Label(-text=>'QUERY - ', -background=>$bg2)->pack(-side=>'top', -expand=>1);
    my $query_frm = $q_frm->Frame(-background=>$bg2)->pack(-side=>'top', -fill=>'x', -expand=>1);
    
    my $res1_frm = $query_frm->Frame()->pack(-side=>'left', -padx=>5, -pady=>5, -expand=>1);
    my $res1_lbl = $res1_frm->Label(-text=>'Residue 1 : ', -background=>$bg2)->pack(-side=>'left', -expand=>1);
    my $res1_ent = $res1_frm->Entry(-textvariable=>\$res1_var, -background=>'white')->pack(-side=>'left', -expand=>1);

    my $res2_frm = $query_frm->Frame(-background=>$bg2)->pack(-side=>'left', -padx=>5, -pady=>5, -expand=>1);
    my $res2_lbl = $res2_frm->Label(-text=>'Residue 2 : ', -background=>$bg2)->pack(-side=>'left', -expand=>1);
    my $res2_ent = $res2_frm->Entry(-textvariable=>\$res2_var, -background=>'white')->pack(-side=>'left', -expand=>1);
    
    my $show_but = $query_frm->Button(-text=>'Show')->pack(-side=>'left', -padx=>5, -pady=>5, -expand=>1);
    
    #create canvas
    $window->update;
    my $width = 100 * $scale;
    
    my $canvas = $window->Canvas(-background=>'white', -width=>$width, -height=>$width)->pack(-ipadx=>10, -ipady=>10);
    my $coords_ref;


    # creates empty canvas
    ($canvas, $coords_ref) = &create_canvas($canvas, $window, $residue_ref, $width, $start_res, $start_res, $end_res, $end_res);
    $canvas->configure(-scrollregion=>[$canvas->bbox("all")], -confine=>1);
    my $display = '';

    my $display_lbl = $window->Label(-textvariable=>\$display, -background=>'#DDEEDC', -relief=>'groove', -borderwidth=>2)
	->pack(-fill=>'x', -expand=>1, -padx=>5, -pady=>5, -ipadx=>5, -ipady=>10);

    my $help_txt = "Single-click on a point to display it's residues\n";
    $help_txt = $help_txt."Double-click on a point to show all it's interactions\n\n";
    $help_txt = $help_txt."Red indicates a backbone interaction\n";
    $help_txt = $help_txt."Green indicates an intra-residue interaction";
    

    my $help_lbl = $window->Label(-text=>$help_txt, -background=>'white', -relief=>'groove', -borderwidth=>2)
	->pack(-fill=>'x', -expand=>1, -padx=>5, -pady=>5, -ipadx=>5, -ipady=>10);

    #put data into canvas
    &put_contact_data($data_ref, $residue_ref, $canvas, $width, \$display, $window, $dir, $start_res, $start_res, $end_res, $end_res, $coords_ref);

    $show_but->configure(-command=>sub
			 {
			     if ($res1_var)
			     {
				 if ($res2_var)
				 {
				     &create_single_query($data_ref, $residue_ref, $res1_var, $res2_var, $window, $dir);
				 }
				 else
				 {
				     &create_mult_query($data_ref, $residue_ref, $res1_var, $window, $dir);
				 }
			     }
			 });
    $res1_ent->bind('<Return>'=>sub
		    {
			#print "Residue 1 is $res1_var, and Residue 2 is $res2_var\n";
			if ($res1_var)
			{
			    if ($res2_var)
			    {
				&create_single_query($data_ref, $residue_ref, $res1_var, $res2_var, $window, $dir);
			    }
			    else
			    {
				&create_mult_query($data_ref, $residue_ref, $res1_var, $window, $dir);
			    }
			}
		    });

    $res2_ent->bind('<Return>'=>sub
		    {
			#print "Residue 1 is $res1_var, and Residue 2 is $res2_var\n";
			if ($res1_var)
			{
			    if ($res2_var)
			    {
				&create_single_query($data_ref, $residue_ref, $res1_var, $res2_var, $window, $dir);
			    }
			    else
			    {
				&create_mult_query($data_ref, $residue_ref, $res1_var, $window, $dir);
			    }
			}
		    });

    $m3_but->configure(-command=>sub
				 {
				     #check the values in zoom_vars
				     foreach (qw/x1 y1 x2 y2/)
				     {
					 unless ($zoom_vars{$_} =~ /^\d+$/)
					 {
					     &err_msg("Please Enter a Residue Number for $_", $window);
					     return;
					 }
				     }
				     
				     foreach (qw/x1 y1/)
				     {
					 if ($zoom_vars{$_} < 1)
					 {
					     $zoom_vars{$_} = 1;
					 }
				     }
				     
				     foreach (qw/x2 y2/)
				     {
					 if ($zoom_vars{$_} > ($#residue + 1))
					 {
					     $zoom_vars{$_} = $#residue + 1;
					 }
				     }
				     
				     if ($zoom_vars{'x1'} > $zoom_vars{'x2'})
				     {
					 $zoom_vars{'x1'} = 1;
					 $zoom_vars{'x2'} = $#residue+1;
				     }
				     
				     if ($zoom_vars{'y1'} > $zoom_vars{'y2'})
				     {
					 $zoom_vars{'y1'} = 1;
					 $zoom_vars{'y2'} = $#residue+1;
				     }

				     #$start_res = $zoom_vars{'res1'};
				     #$end_res = $zoom_vars{'res2'};
				     
				     $canvas->delete("all");
				     my $coords_ref;
				     $canvas->configure(-cursor=>'watch');
				     
				     ($canvas, $coords_ref) = &create_canvas($canvas, $window, $residue_ref, $width, $zoom_vars{'x1'}, $zoom_vars{'y1'}, $zoom_vars{'x2'}, $zoom_vars{'y2'});
				     &put_contact_data($data_ref, $residue_ref, $canvas, $width, \$display, $window, $dir, $zoom_vars{'x1'}, $zoom_vars{'y1'}, $zoom_vars{'x2'}, $zoom_vars{'y2'}, $coords_ref);
				     
				     $canvas->configure(-cursor=>'top_left_arrow');
				 });
    
    $window->update;

    $window->minsize($window->width, $window->height);
    
}


sub put_contact_data
{
    my ($data_ref, $residue_ref, $canvas, $tot_width, $display_ref, $window, $dir, $x1, $y1, $x2, $y2, $coords_ref) = @_;

    my @data = @$data_ref;
    
    my ($i, $i_atom, $j, $j_atom);
    
    for ($i = $x1; $i<=$x2; $i++)
    {
	foreach $i_atom (keys %{$data[$i]})
	{
	    for ($j = $y1; $j<=$y2; $j++)
	    {
		foreach $j_atom (keys %{$data[$i]{$i_atom}[$j]})
		{
		    #print "interaction $i-$i_atom $j-$j_atom\n";
		    #print "data[$i]{$i_atom}[$j]{$j_atom} = $data[$i]{$i_atom}[$j]{$j_atom}\n";
		    my $info = "Interaction: Residues $i and $j";
		    my ($point, $pointB);
		    
		    #print "$info\n";
		    if ($i == $j)
		    {
			$point = &place_interaction($i, $j, $canvas, $tot_width, $info, $display_ref, 'green', $x1, $y1, $x2, $y2, $coords_ref);
			$canvas->itemconfigure($point, -tags=>['diag', $i]);
			
		    }
		    elsif (((($i_atom eq 'HA') or ($i_atom eq 'HN')) and (($j_atom eq 'HA') or ($j_atom eq 'HN'))) or (($i_atom =~ /^o$/i) or ($j_atom =~ /^o$/i)))
		    {
			#backbone interactions
			my ($res_1, $res_2);
			if ($i<$j)
			{
			    ($res_1, $res_2) = ($i, $j);
			}
			else
			{
			    ($res_1, $res_2) = ($j, $i);
			}
			
			$point = &place_interaction($res_1, $res_2, $canvas, $tot_width, $info, $display_ref, 'black', $x1, $y1, $x2, $y2, $coords_ref);
			$pointB = &place_interaction($res_2, $res_1, $canvas, $tot_width, "Backbone interaction: Residues $j and $i", $display_ref, 'red', $x1, $y1, $x2, $y2, $coords_ref);
			
			$canvas->bind($pointB, '<Double-Button-1>'=>sub
				      {
					  &create_single_query($data_ref, $residue_ref, $res_1, $res_2, $window, $dir);
				      });
			#print "placing bb interaction between residues ($i-$i_atom) and ($j-$j_atom)\n";
		    }
		    else
		    {
			my ($temp_i, $temp_j) = ($i, $j);
			if ($i > $j)
			{
			    $temp_i = $j;
			    $temp_j = $i;
			}
			#side chain
			$point = &place_interaction($temp_i, $temp_j, $canvas, $tot_width, $info, $display_ref, 'black', $x1, $y1, $x2, $y2, $coords_ref);
			#print "placing ss interaction between ($i-$i_atom) and ($j-$j_atom)\n";
		    }

		    my ($res1, $res2) = ($i, $j);
		    $canvas->bind($point, '<Double-Button-1>'=>sub
				  {
				      &create_single_query($data_ref, $residue_ref, $res1, $res2, $window, $dir);
				  });
		}
	    }
	}
    }
    
    my ($hor_line, $ver_line);
    $canvas->CanvasBind('<Button1-ButtonRelease>'=>sub
			{
			    #&remove_lines($canvas, $hor_line, $ver_line);
			    my $item = $canvas->find('withtag', 'current');
			    
			    if ($item)
			    {
				my (@tags) = $canvas->itemcget($item, -tags);
				if (@tags)
				{
				    if ($tags[0] eq 'diag')
				    {
					#diagonal point
					&remove_lines($canvas, $hor_line, $ver_line);
					($hor_line, $ver_line) = &create_lines($canvas, $tags[1], $tot_width, $x1, $y1, $x2, $y2, $coords_ref);
				    }
				}
			    }
			});
    
}

sub create_single_query
{
    #create query for one res1 and one res2
    
    #given a residue 1 and a residue 2 - get dist_hash from create_query, and then show those results with show_query_results
    my ($data_ref, $residue_ref, $res1, $res2, $parent, $dir) = @_;
    #print "create_single_query between $res1 and $res2\n";

    my %dist_hash;
    
    my $i = $res1; my $j = $res2; 
    if ($res1 > $res2) { $i = $res2; $j = $res1;} 

    &create_query($data_ref, $residue_ref, $i, $j, \%dist_hash);
    &show_query_results(\%dist_hash, $residue_ref, $parent, $dir, $res1, $res2);
    
}

sub create_mult_query
{
    #given residue 1, loop through all residues, and call create_query for each residue, storing the interactions in dist_hash, and then call show_query_results

    my ($data_ref, $residue_ref, $res1, $parent, $dir) = @_;
    #print "create_mult_query for $res1\n";
    my %dist_hash;
    my @residue = @$residue_ref;
    my $res2;
    for ($res2 = 0; $res2<=$#residue; $res2++)
    {
	my $i = $res1; my $j = $res2; 
	if ($res1 > $res2) { $i = $res2; $j = $res1;} 

	&create_query($data_ref, $residue_ref, $i, $j, \%dist_hash);
    }
    &show_query_results(\%dist_hash, , $residue_ref, $parent, $dir, $res1, 0);
    
}

#contrait - res1 <= res2, 
sub create_query
{
    #given a residue 1 and a residue 2, return dist_hash updated with all those interactions

    my ($data_ref, $residue_ref, $res1, $res2, $dist_hash_ref) = @_;
    #print "create_query: dir is $dir\n";
    
    my @data = @$data_ref;
    my @residue = @$residue_ref;

    #print "create_query: getting interactions for res1 = $res1, res2 = $res2\n";
    
    my ($i, $i_atom, $j, $j_atom);
    #search for interactions res1 to res2
    #let i be res1 and j be res2
    ($i, $j) = ($res1, $res2);
    
    
    foreach $i_atom (keys %{$data[$i]})
    {
	foreach $j_atom (keys %{$data[$i]{$i_atom}[$j]})
	{
	    $dist_hash_ref->{$i.$aa{$residue[$i]}."-$i_atom $j".$aa{$residue[$j]}."-$j_atom"} = $data[$i]{$i_atom}[$j]{$j_atom};
	}
    }

#since res1 <= res2, not need for this 
    #search for interactoins res2 to res1
    #let i be res2 and j be res1

#    ($i, $j) = ($res2, $res1);
    
#    foreach $i_atom (keys %{$data[$i]})
#    {
#	foreach $j_atom (keys %{$data[$i]{$i_atom}[$j]})
#	{
#	    $dist_hash_ref->{"$j".$aa{$residue[$j]}."-$j_atom $i".$aa{$residue[$i]}."-$i_atom"} = $data[$i]{$i_atom}[$j]{$j_atom};
#	}
#    }
    
}

sub show_query_results
{
    #given a hash - displays the results in a new window
    my ($dist_hash_ref, $residue_ref, $parent, $dir, $res1, $res2) = @_;

    unless ($res2)
    {
	$res2 = 'all';
    }

    my %dist_hash = %$dist_hash_ref;
    
    my $query_win = $parent->Toplevel(-title=>"Interactions between $res1 and $res2");
    my $lbl = $query_win->Label(-text=>"Interactions between $res1 and $res2", -relief=>'ridge', -borderwidth=>2, -background=>$bg1)->pack(-ipadx=>5, -ipady=>10);
    my $res_frm = $query_win->Frame(-background=>$bg1)->pack(-fill=>'both', -expand=>1);
    
    my $table = $res_frm->Table(
				-rows=>15,
				-columns=>3,
				-fixedrows=>1,
				-relief=>'sunken',
				-background=>$bg1,
				-scrollbars=>'e')->pack(-side=>'top', -fill=>'both', -expand=>1);

    my $title;
    my @col_but;
    my $column = 0;
    my $row = 1;
    my $k;
    my $key;
    
    foreach $title ('Residue 1 - Residue 1 Atom', 'Residue 2 - Residue 2 Atom', "Distance constraints\nbetween the two residues")
    {
	my $p = $column;
	$col_but[$column] = $table->Button(-text=>$title, -background=>'#746091', -command=>sub {&sort_col($table, $p)});
	$table->put(0, $column, $col_but[$column]);
	$column++;
    }
    
    foreach $key (keys %dist_hash)
    {
	my $line = "$key = $dist_hash{$key}";
	if ($row % 2)
	{
	    &add_res_row($table, $row, $line, $col1);
	}
	else
	{
	    &add_res_row($table, $row, $line, $col2);
	}
	$row++;
    }


    my $pkl_frm = $query_win->Frame(-background=>$bg1)->pack(-ipadx=>5, -ipady=>7);
    my $show_pkl_but = $pkl_frm->Button(-text=>'Show peaklist information')->pack(-expand=>1);

    $show_pkl_but->configure(-command=>sub
			     {
				 print "Showing peaklist information: directory is $dir, residues are $res1 and $res2\n";
				 
				 my (@assOrder_files) = &get_assOrders($dir, $query_win);
				 my %data_from_file;

				 #print "create_query: From $dir:\nassOrder_files are @assOrder_files\n";

				 foreach (@assOrder_files)
				 {
				     my $data_ref;
				     if ($res2 eq 'all')
				     {
					 ($data_ref) = &parse_assOrder_mult($_, $res1, $query_win);
				     }
				     else
				     {
					 ($data_ref) = &parse_assOrder_single($_, $res1, $res2, $query_win);
				     }
				     
				     $data_from_file{$_} = $data_ref;
				     
				 }

				 #display results - function is in file display_peak_info.pl
				 &display_peak_info(\%data_from_file, $query_win, $res1, $res2);
				 
			     });

    $query_win->raise;
}


sub add_res_row
{
    my ($table, $row, $data, $color) = @_;

    #$color = 'white';
    
    $data =~ /^(.+) (.+) = ([\d\.]+)/;
    $table->put($row, 0, &create_label($table, $1, $color));
    $table->put($row, 1, &create_label($table, $2, $color));
    $table->put($row, 2, &create_label($table, $3, $color));
    
}

sub sort_col
{
    my ($table, $column) = @_;

    
    my @data_array;
    $data_array[0][0] = '';
    
    my ($row, $col);
    my $total_rows = $table->totalRows;
    my $total_cols = $table->totalColumns;

    for ($row = 1; $row<$total_rows; $row++)
    {
	for ($col = 0; $col<$total_cols; $col++)
	{
	    my $lbl = $table->get($row, $col);
	    my $txt = $lbl->cget(-text);

	    $data_array[$row][$col] = $txt;
	}
    }

    #print "data_array contents:\n";
    

    my @sorted = sort {$a->[$column] <=> $b->[$column] || $a->[$column] cmp $b->[$column]} @data_array;

    if (@sorted)
    {
	#put data in table
	for ($row = 1; $row<$total_rows; $row++)
	{
	    for ($col = 0; $col<$total_cols; $col++)
	    {
		if ($row % 2)
		{
		    $table->put($row, $col, &create_label($table, $sorted[$row][$col], $col1));
		}
		else
		{
		    $table->put($row, $col, &create_label($table, $sorted[$row][$col], $col2));
		}
	    }
	}
    }

}

sub create_label
{
    my ($table, $data, $color) = @_;

    my $label = $table->Label(-text=>$data, -background=>$color);
    return $label;
}



sub place_interaction
{
    my ($i, $j, $canvas, $tot_width, $info, $display_ref, $color, $x1, $y1, $x2, $y2, $coords) = @_;

    #my $color;
    my $x = $coords->{'big'} + ((($i - $x1 + 1) / $coords->{'div_x'}) * $coords->{'x_div'});
    my $y = $coords->{'by'} - ((($j - $y1 + 1)/ $coords->{'div_y'}) * $coords->{'y_div'});
    
    #$my $div = ($tot_width - 30)/($end_res - $start_res + 2);
    #$div = &round_num($div, 1);

    #my $x = 25 + (($i - $start_res + 1) * $div);
    #$x = &round_num($x, 1);
    
    #my $y = $tot_width - (25 + (($j - $start_res + 1) * $div));
    #$y = &round_num($y, 1);

    #print "placing interaction ($i, $j) at coords ($x, $y), tot_width = $tot_width, div = $coords->{'div'}, start = $start_res, end = $end_res\n";
    
    my $point = $canvas->createText($x, $y, -text=>'x', -fill=>$color);
    $canvas->bind($point, '<Button-1>'=>sub
		  {
		      $$display_ref = $info;
		  });
    $canvas->bind($point, '<Enter>'=>sub
		  {
		      $canvas->itemconfigure($point, -text=>'@', -fill=>'blue');
		      $canvas->configure(-cursor=>'crosshair');
		  });
    $canvas->bind($point, '<Leave>'=>sub
		  {
		      $canvas->itemconfigure($point, -text=>'x', -fill=>$color);
		      $canvas->configure(-cursor=>'top_left_arrow');
		  });

    return ($point);
    
}


sub round_num
{
    my ($num, $unit) = @_;

    if ($unit == 1)
    {
	$num = sprintf("%0.0f", $num);
    }
    else
    {
	$num = ($num / $unit) + 0.5;
	$num = sprintf("%0.0f", $num);
	$num = $num * $unit;
    }
    
    return ($num);

}

sub create_lines
{
    my ($canvas, $i, $tot_width, $x1, $y1, $x2, $y2, $coords) = @_;
    
    my $x = $coords->{'big'} + ((($i - $x1 + 1) / $coords->{'div_x'}) * $coords->{'x_div'});
    my $y = $coords->{'by'} - ((($i - $y1 + 1)/ $coords->{'div_y'}) * $coords->{'y_div'});

    #my $div = ($tot_width - 30)/($end_res - $start_res + 2);
    #$div = &round_num($div, 1);

    #my $x = 25 + (($i - $start_res + 1) * $div);
    #$x = &round_num($x, 1);
    my $hor_line = $canvas->createLine($x, $coords->{'small'}, $x, $coords->{'by'}, -fill=>'magenta');

    #my $y = $tot_width - (25 + (($i - $start_res + 1) * $div));
    #$y = &round_num($y, 1);
    my $ver_line = $canvas->createLine($coords->{'big'}, $y, $coords->{'bx'}, $y, -fill=>'magenta');

    #print "lines at $i - ($x, $y)\n";
    return ($hor_line, $ver_line);

}

sub remove_lines
{
    my ($canvas, $hor_line, $ver_line) = @_;


    if ($hor_line)
    {
	$canvas->delete($hor_line);
    }
    
    if ($ver_line)
    {
	$canvas->delete($ver_line);
    }
}


sub create_canvas
{
    my ($canvas, $window, $residue_ref, $tot_width, $x1, $y1, $x2, $y2) = @_;
    my @residue = @$residue_ref;
    my %coords;

    $coords{'big'} = 25;
    $coords{'small'} = 5;

    my ($ax, $ay, $bx, $by) = ($coords{'big'}, $coords{'small'}, $tot_width - $coords{'small'}, $tot_width - $coords{'big'});

    my $rect1 = $canvas->createRectangle($ax, $ay, $bx, $by);

    my $rem_x = 10 - (($x2 - $x1 + 1) % 10);
    my $tot_x = (($x2 - $x1 + 1) + $rem_x);
    my $div_x = $tot_x / 10;

    my $rem_y = 10 - (($y2 - $y1 + 1) % 10);
    my $tot_y = (($y2 - $y1 + 1) + $rem_y);
    my $div_y = $tot_y / 10;
    
    my $x_div = sprintf("%0.0f", (($bx - $ax) / 10));
    my $y_div = sprintf("%0.0f", (($by - $ay) / 10));

    $coords{'div_x'} = $div_x;
    $coords{'div_y'} = $div_y;
    $coords{'x_div'} = $x_div;
    $coords{'y_div'} = $y_div;
    $coords{'bx'} = $bx;
    $coords{'by'} = $by;

    #my $div = ($tot_width - 30)/(($end_res - $start_res) + 2);
    #$div = &round_num($div, 1);
    #print "create_canvas: tot_width = $tot_width, ($x1, $y1) to ($x2, $y2), x_div = $x_div, y_div = $y_div, rect is ($ax, $ay, $bx, $by), divs are ($div_x, $div_y), tots are ($tot_x, $tot_y)\n";

    my $x = $ax;
    my $y = $by;
    
    my $i;
    for ($i = $x1 - 1; $i <= ($x1 + $tot_x); $i += $div_x)
    {
	&place_mark('x', $x, $by, $i, $canvas);
	#print "i = $i, x = $x\n";
	$x += $x_div;
    }


    for ($i = $y1 - 1; $i <= ($y1 + $tot_y); $i += $div_y)
    {
	&place_mark('y', $ax, $y, $i, $canvas, $tot_width);
	$y -= $y_div;
    }
    
    return ($canvas, \%coords);
}


sub place_mark
{
    my ($axis, $x, $y, $label, $canvas, $tot_width) = @_;

    if ($axis eq 'x')
    {
	my $line = $canvas->createLine($x, 5, $x, $y, -dash=>'.');
	my $text = $canvas->createText($x, $y+10, -text=>$label);
    }
    else
    {
	my $line = $canvas->createLine($x, $y, $tot_width - 5, $y, -dash=>'.');
	my $text = $canvas->createText(10, $y, -text=>$label);
    }
}

sub display_multiple 
{
    my ($data_ref, $c_dir, $q_dir, $parent) = @_;

    my %data = %$data_ref;

    #display the data
    my $win = $parent->Toplevel(-title=>"NOE Assignments");
    
    my $heading = $win->Label(-text=>"NOE Assignments", -relief=>'ridge')->pack(-fill=>'x',-ipady=>3);
    my $help_txt = "Double-click on a peak to display its all assignments";
    my $help_lbl = $win->Label(-text=>$help_txt)->pack(-padx=>10, -pady=>5);
   
    my $nb = $win->NoteBook()->pack(-fill=>'both', -expand=>1);
    my $main = $nb->add('Main', -label=>'Main');

    #getmatch file

    my $pklst;
    my $pk_count = 1;
    my %pk_match; 
    foreach $pklst (keys %data)
    { 
	#$pklst = basename($pkl_fname); 
	#$pklst =~ s/_assOrder//; 

	my $page = $nb->add($pklst, -label=>"Peaklist $pk_count");
	
	my $pk_lbl = $page->Label(-text=>"Peaklist $pklst" . " total: " . scalar(@{$data{$pklst}}), -relief=>'ridge')->pack(-fill=>'x', -pady=>2);
	
	my $scr_lbx = $page->Scrolled('Listbox', -scrollbars=>"osoe")->pack(-fill=>'both', -expand=>1, -padx=>5, -pady=>5);
	my $lbx = $scr_lbx->Subwidget("listbox");
	$lbx->configure(-selectmode=>'single', -background=>'white');

	$pk_count++;

	foreach $k (@{$data{$pklst}}) # for each peak 
	{
	    $lbx->insert('end', "$k");
	    $pk_match{$k} = $pklst;
	}
	
	my $sp_but = $page->Button(-text=>'Create Sparky File', -command=>sub
				  {
				      &create_sparky_list_fn($data_ref, $q_dir, $win, $pklst , "NOE assignments");
				  })->pack(-padx=>5, -pady=>5, -fill=>'x');
	
	$lbx->bind('<Double-Button-1>'=>sub
		   {
		       my $line = $lbx->get($lbx->curselection());
		       
		       $nline = $line; 
		       #print "line is $line\n";
		       $nline =~ s/^-> //; 
		       
		       #all assignments & matches

                   if ($nline =~ /^peak/)
                   {		       
                      $lbx->configure(-cursor=>"watch");
		      $lbx->update; 

		      my $pkl_fname = $pk_match{$line}; 
		      print "pkl_fname $pkl_fname\n";

                       my ($data_adnl_ref) = &parse_assOrder_adnl($win, "$c_dir/$pkl_fname" . "_assOrder", $nline);
                       
		      #$pkl = basename($pkl_fname); 
                      #$pkl_fname =~ s/_assOrder/_match\.gz/;

                       $match_file = $q_dir . "/" . $pkl_fname . "_match.gz"; 
 
                       print "match file is $match_file\n";

		      my $top = $parent->Toplevel(-title=>"Peak Assignment Information"); 
		      my $adn_frm = $top->Frame()->pack(-fill=>'both', -expand=>1, -padx=>5, -pady=>5);
		      my $adn_lbl = $adn_frm->Label(-text=>"All assignments", -relief=>'ridge', -borderwidth=>2)->pack(-padx=>
															      5, -pady=>5);
		      my $scr_txt = $adn_frm->Scrolled("ROText", -scrollbars=>'se')->pack(-fill=>'both', -expand=>1);
		      my $adn_txt = $scr_txt->Subwidget("rotext");
		      $adn_txt->configure(-background=>'white');

                       &display_adnl($data_adnl_ref, $adn_txt, $match_file, $nline, $top);
                       $lbx->configure(-cursor=>"top_left_arrow");
                   }
		   });
    }

    $nb->delete('Main');
    
    my $cls_but = $win->Button(-text=>'Close', -command=>sub
			       {
				   $win->destroy();
			       })->pack(-fill=>'x', -padx=>5, -pady=>5);
      
}



return 1;

