sub create_graph
{
    
    my ($graph_frm, $graph_data, $cycle_num, $window, $basepath) = @_;
    
    my $canvas = $graph_frm->Canvas(-background=>'white', -relief=>'sunken', -cursor=>"crosshair", -width=>680, -height=>530)->pack(-fill=>'both', -expand=>1, -padx=>5, -pady=>5);
    

    my ($x, $y) = (20,0);
    my $help_rect = $canvas->createRectangle($x, $y, $x+500, $y+25, -outline=>'brown', -width=>3);
    
    my $help_txt = "Move mouse over one of the points to disply its value";
    
    my $help_lbl = $canvas->Label(-textvariable=>\$help_txt, -background=>'white', -width=>60, -height=>1, -borderwidth=>2, -foreground=>'#660099');
    my $can_help = $canvas->createWindow($x+270, $y+14, -window=>$help_lbl);

    my ($gx1, $gy1, $gx2, $gy2) = (20, 35, 520, 500);

    my $grid = $canvas->createRectangle($gx1, $gy1, $gx2, $gy2, -outline=>'red');
    
    my ($marks, $lines, $rect) = &place_marks($gx1, $gy1, $gx2, $gy2, $canvas, $cycle_num);

    &put_data($canvas, $graph_data, $marks, $cycle_num, $help_lbl, \$help_txt, $lines, $rect);


    #Legend
    &create_legend(530, 20, $canvas);
    &create_save_button($canvas, $graph_data, $window, $basepath, $cycle_num);
    
    $canvas->configure(-scrollregion=>[$canvas->bbox('all')], -confine=>1);
}


sub create_legend
{
    my ($x, $y, $canvas) = @_;

    my $leg_rect = $canvas->createRectangle($x, $y, $x+130, $y+95);

    my $txt = $canvas->createText($x+30, $y+10, -text=>"Legend");
    my $line = $canvas->createLine($x, $y+17, $x+130, $y+17);

    &put_precision($x+20, $y+32, $canvas);
    $txt = $canvas->createText($x+67, $y+30, -text=>" - Precision");

    &put_recall($x+20, $y+45, $canvas);
    $txt = $canvas->createText($x+60, $y+45, -text=>" - Recall");
    
    &put_fscore($x+20, $y+62, $canvas);
    $txt = $canvas->createText($x+63, $y+62, -text=>" - F-score");
    
    &put_dp($x+20, $y+82, $canvas);
    $txt = $canvas->createText($x+67, $y+82, -text=>" - DP-Score");
    
}


sub put_precision
{
    my ($x, $y, $canvas, $color) = @_;
    unless ($color)
    {
	$color = 'red';
    }

   
    my $triangle = $canvas->createPolygon($x-7, $y, $x, $y-7, $x+7,  $y, -fill=>$color);
    
    return $triangle;
}

sub put_recall
{
    my ($x, $y, $canvas, $color) = @_;
    
    unless ($color)
    {
	$color = 'green';
    }
    
    my $circle = $canvas->createOval($x-4, $y-4, $x+4, $y+4,  -fill=>$color);
    return $circle;
}

sub put_fscore
{
    my ($x, $y, $canvas, $color) = @_;
    unless ($color)
    {
	$color = 'blue';
    }
    my $square = $canvas->createRectangle($x-3, $y-3, $x+3, $y+3, -fill=>$color);
    return $square;
}

sub put_dp
{
    my ($x, $y, $canvas, $color) = @_;
    unless ($color)
    {
	$color = 'purple';
    }
    my $pentagon = $canvas->createPolygon($x-3, $y+5, $x-5, $y, $x, $y-3, $x+5, $y, $x+3, $y+5, -fill=>$color);
    #my $pentagon = $canvas->createPolygon($x-6, $y, $x, $y-6, $x+6, $y, $x+5, $y+6, $x-5, $y+6,  -fill=>$color);
    return $pentagon;
}


sub place_marks
{
    my ($x1, $y1, $x2, $y2, $canvas, $num_cycles) = @_;
    my $num = 0;

    my $x_dist = $x2 - $x1;
    my $x_div = ($x_dist)/($num_cycles+1);
    
    my %marks;
    my %lines;
    my %rect;

    #marks on x-axis
    my ($i, $j);
    $i = $x1+$x_div;

    for ($num = 1; $num <= $num_cycles; $num++)
    {
	$rect{$num} = $canvas->createPolygon($i-15, $y1+5, $i+15, $y1+5, $i+15, $y2, $i-15, $y2, -fill=>'white');
	$lines{$num} = $canvas->createLine($i, $y1, $i, $y2+5, -dash=>'.');
	my $cyc = $num;

	my $text = $canvas->createText($i, $y2+20, -text=>"Cyc $num");
	$marks{$num} = $i;
	
	$i+=$x_div;
    }
    

    $num = 1.0;

    #marks on y-axis
    my $y_dist = $y2-$y1;
    #print "place_marks: y_dist is $y_dist\n";
    
    my $y_div = $y_dist/10;

    for ($i = $y1; $i<=$y2; $i+=$y_div)
    {
	my $line = $canvas->createLine($x1-5, $i, $x1+5, $i);
	my $text = $canvas->createText($x1-15, $i, -text=>$num);
	$num = sprintf("%0.1f", $num-0.1);
    }

    return (\%marks, \%lines, \%rect);

}


sub put_data
{
    my ($canvas, $data_ref, $marks_ref, $cycle_num, $help_lbl, $help_txt, $lines_ref, $rect_ref) = @_;

    my %data = %$data_ref;
    my %lines = %$lines_ref;
    my %marks = %$marks_ref;
    my %rect = %$rect_ref;
    my $k;
    my $option;

    for ($k = 1; $k<=$cycle_num; $k++)
    {
	my $cyc_num = $k;
	
	foreach $option (qw/Precision-score Recall-score F-score dp-score/)
	{
	    my $opt = $option;
	    my $number = $data{$option}{$k};
	    next if ($number <=0);

	   
	    my $x = $marks{$k};
	    my $y = &format_number($number);
	    my $point;

	    if ($option =~ /precision/i)
	    {$point = &put_precision($x, $y, $canvas);}
	    elsif ($option =~ /recall/i)
	    {$point = &put_recall($x, $y, $canvas);}
	    elsif ($option =~ /f-score/i)
	    {$point = &put_fscore($x, $y, $canvas);}
	    elsif ($option =~ /dp/i)
	    {$point = &put_dp($x, $y, $canvas);}



	    my $txt_box;
	    #bind the point
	    $canvas->bind($point, '<Enter>'=> sub 
			  {
			      $$help_txt = "Cycle $cyc_num - $opt = $data{$opt}{$cyc_num}";
			      #print "help_txt - Cycle $cyc_num - data{$opt}{$cyc_num} = $data{$opt}{$cyc_num}\n";
			      $help_lbl->configure(-foreground=>'#FF3300');
			  });
	    $canvas->bind($point, '<Leave>'=>sub
			  {
			      $$help_txt = "Move mouse over one of the points to display its value";
			      $help_lbl->configure(-foreground=>'#660099');
			  });
	}
	
	
	
	$canvas->bind($rect{$cyc_num}, '<Enter>'=>sub
		      {
			  $$help_txt = "Cycle $cyc_num - Precision=".$data{'Precision-score'}{$cyc_num}.", Recall=".$data{'Recall-score'}{$cyc_num}.
			      ", F-score=".$data{'F-score'}{$cyc_num}.", DP=".$data{'dp-score'}{$cyc_num};
			  $help_lbl->configure(-foreground=>'#FF3300');
		      });
	$canvas->bind($rect{$cyc_num}, '<Leave>'=>sub
		      {
			  $$help_txt = "Move mouse over one of the points to display its value";
			  $help_lbl->configure(-foreground=>'#660099');
		      });
	$canvas->bind($lines{$cyc_num}, '<Enter>'=>sub
		      {
			  $$help_txt = "Cycle $cyc_num - Precision=".$data{'Precision-score'}{$cyc_num}.", Recall=".$data{'Recall-score'}{$cyc_num}.
			      ", F-score=".$data{'F-score'}{$cyc_num}.", DP=".$data{'dp-score'}{$cyc_num};
			  $help_lbl->configure(-foreground=>'#FF3300');
		      });
    }
    
    
}

sub format_number
{
    my ($number) = @_;

    $number = 1-$number;
    
    $number = $number * 465;
    
    $number = sprintf("%.0f", $number);
    
    $number  = $number + 35;


    return ($number);
}




sub write_rpf_data
{
    my ($data, $file, $num_cycles, $parent) = @_;

    unless (open(FILE, ">$file"))
    {
	&err_msg("Cannot write to file $file", $parent);
	return;
    }
    
    my ($opt, $k);
    
    #first row
    print FILE " \t \t Recall \t Precision \t F-Score \t DP-Score \n\n";

    for ($k = 1; $k<=$num_cycles; $k++)
    {
	print FILE "Cycle $k: \t ";
	
	foreach $opt (qw/Recall-score Precision-score F-score dp-score/)
	{
	    print FILE ${$data->{$opt}}{$k};
	    
	    if ($opt =~ /dp/)
	    {
		print FILE " \n";
	    }
	    else
	    {
		print FILE " \t\t ";
	    }
	}
    }
   
    close(FILE);

}

sub create_save_button
{
    my ($canvas, $data, $window, $basepath, $num_cycles) = @_;
    
    my ($x, $y) = (600, 180);

    my $button = $canvas->Button(-text=>"Save data as Table...", -height=>2, -command=>sub 
				 {
				     my $file = &OpenFile('RPF Table', $window, $basepath); 
				     if ($file)
				     {
					 $file = File::Spec->rel2abs($file, $basepath);
					 &write_rpf_data($data, $file, $num_cycles, $window);
				     }
				 });
    my $can_but = $canvas->createWindow($x, $y, -window=>$button);
}



return 1;
