sub create_sparky_list_fp
{
    #my ($parent, $dist_hash_ref, $residue_ref, $pklist, $dir, $ql_file, $note, %partial_peaks) = @_;
    my ($parent, $dist_hash_ref, $residue_ref, $pklist, $dir, $ql_file, $note) = @_;
    
    #dist_hash is a hash of format dist_hash{"res1-atm1 res2-atm2"} = <distance>
    my %dist_hash = %$dist_hash_ref;
    my @residue = @$residue_ref;
    #print "create_sparky_list_fp: Printing dist_hash_array, query is $query\n";
    
    my $space = "     ";

    #get and parse control file
    my ($control_file) = &get_control_file($dir);
    my ($comments_ref, $general_ref, $commands_ref, $peaklists_ref, $col_names_ref, $pklst_count) = &openContFile($control_file);
    
    my $csi_file = $general_ref->{'chemicalShiftFile'};

    #parse csi file
    my ($csi_data_ref) = &parse_csi($csi_file, $parent);
    my %peaklists = %$peaklists_ref;
    my %col_names = %$col_names_ref;
    my %csi_data = %$csi_data_ref;
    

    #open output sparky file
    my $template = "create_sparky_fp";
    #my ($FOUT, $temp_sparky) = tempfile($template . "XXXX", DIR => "$HOME/.autostructure/", UNLINK=>1); 
    $temp_sparky = getTempFileName($template); 

    unless (open(OUT, "> $temp_sparky"))
    {
	&err_msg("Cannot open sparky file for writing", $parent);
	return;
    }    

    print OUT "\#\# False Positive Interactions from $ql_file \#\# \n\n";
    #print "\#\# False Positive Interactions from file $ql_file\#\# \n\n";
    my ($k);
    
    print OUT "\# Peaklist $pklist\n";
    #print "\# Peaklist $heading[$k]\n\#";
    my $value;
    my $lbl_exists = 0;
    my $id = 1;
    
    print OUT "\#label$space";

    #find the pklist in the control file 
    my $npklist; 
    foreach my $k (%col_names)
    {
	$bk = basename($k);

	if ($bk eq basename($pklist))
	{
	    $npklist = $k; 
	    last; 
	} 
    }

    unless ($npklist)
    {
	&err_msg("Cannot find $pklist in control-file", $parent); 
	return; 
    }

    $pklist = $npklist; 

    foreach $value (sort {$a <=> $b} keys %{$col_names{$pklist}})
    {
	if ($col_names{$pklist}{$value} =~ /^(x1|x2|hx1|hx2)$/)
	{
	    print OUT "$col_names{$pklist}{$value}$space";
	    #print "$col_names{$heading[$k]}{$value}\t"
	}
    }
    print OUT "note\n\n";
    #print "\n";
    
    foreach $key (sort keys %dist_hash)
    {
	#print "dist_hash_array[$heading[$k]]{$key} = $dist_hash_array[$k]{$key}\n";
	$key =~ /^(\d+)\-(.+)\s+(\d+)\-(.+)$/;
	my ($res1, $atm1, $res2, $atm2) = ($1, $2, $3, $4);
	
	my @atoms1 = &get_mult_atoms($csi_data_ref, $atm1, $res1);
	my @atoms2 = &get_mult_atoms($csi_data_ref, $atm2, $res2);
	
	my ($name);
	
	if ($peaklists{$pklist}{'x2.type'})
	{
	    #4D
	    &err_msg("4D not implemented", $parent);
	    return; 
	} 
	elsif ($peaklists{$pklist}{'x1.type'} !~ /\w/) #2D
	{
	    foreach $hx1 (@atoms1)
	    {
		foreach $hx2 (@atoms2)
		{
		    next if (($res1 == $res2) && ($hx1 eq $hx2));
		    
		    $label = '';
		    foreach $value (sort {$a <=> $b} keys %{$col_names{$pklist}})
		    {
			if ($col_names{$pklist}{$value} =~ /^(hx1|hx2)$/)
			{
			    $label = $label."$residue[$res1]$res1$hx1-" if ($col_names{$pklist}{$value} eq 'hx1');
			    $label = $label."$residue[$res2]$res2$hx2-" if ($col_names{$pklist}{$value} eq 'hx2');
			}
		    }
		    $label =~ s/\-$//;
		    print OUT "$label$space";
		    foreach $value (sort {$a <=> $b} keys %{$col_names{$pklist}})
		    {
			my $key = $col_names{$pklist}{$value};
			if ($key =~ /^(hx1|hx2)$/)
			{
			    print OUT "$csi_data{$res1}{$hx1}$space" if ($key eq 'hx1');
			    print OUT "$csi_data{$res2}{$hx2}$space" if ($key eq 'hx2');			    
			}
		    }
		    print OUT "$note\n";
		}
	    }
	}
	else
	{
	    #3D
	    #get type
	    my $type = $peaklists{$pklist}{'x1.type'};
	    my %peaks;
	    my $pk_count = 0;
	    my $value;
	    my ($x1, $hx1, $hx2);

	    if ($type =~ /[C13|CN]/i)
	    {
		my (@carbs1) = &get_carbon($csi_data_ref, $atm1, $res1); 
		#my (@carbs2) = &get_carbon($csi_data_ref, $atm2, $res2);
		my (@carbs2); #atom1 is hx1. no need for @carbs2
		
		#print "($res1 $atm1) to ($res2 $atm2); carbs1 - @carbs1; carbs2 - @carbs2, atoms1 - @atoms1, atoms2 - @atoms2\n";
		
		if (@carbs1)
		{
		    #x1 is carbon 1, hx1 is res1, hx2 is res2
		    #print "x1 - $res1 @carbs1, hx1 - $res1 @atoms1, hx2 - $res2 @atoms2\n";
		    foreach $x1 (@carbs1)
		    {
			foreach $hx1 (@atoms1)
			{
			    my $sx1 = substr($x1,1); 
			    next if ($hx1 !~ /$sx1/);

			    if ($type eq 'C13-ali')
			    {
				next if (($residue[$res1] eq 'F') && ($x1 ne 'CA') && ($x1 ne 'CB'));  
				next if (($residue[$res1] eq 'W') && ($x1 ne 'CA') && ($x1 ne 'CB'));  
				next if (($residue[$res1] eq 'H') && ($x1 ne 'CA') && ($x1 ne 'CB'));  
				next if (($residue[$res1] eq 'Y') && ($x1 ne 'CA') && ($x1 ne 'CB'));  
			    }
			    if ($type eq 'C13-aro')
			    {
				next if (!(($residue[$res1] eq 'F') && ($x1 ne 'CA') && ($x1 ne 'CB')) && 
					 !(($residue[$res1] eq 'W') && ($x1 ne 'CA') && ($x1 ne 'CB')) && 
					 !(($residue[$res1] eq 'H') && ($x1 ne 'CA') && ($x1 ne 'CB')) && 
					 !(($residue[$res1] eq 'Y') && ($x1 ne 'CA') && ($x1 ne 'CB')));  
			    }

			    foreach $hx2 (@atoms2)
			    {
				next if (($res1 == $res2) && ($hx1 eq $hx2));

				$label = '';
				foreach $value (sort {$a <=> $b} keys %{$col_names{$pklist}})
				{
				    if ($col_names{$pklist}{$value} =~ /^(x1|hx1|hx2)$/)
				    {
					$label = $label."$residue[$res1]$res1$hx1-" if ($col_names{$pklist}{$value} eq 'hx1');
					$label = $label."$residue[$res2]$res2$hx2-" if ($col_names{$pklist}{$value} eq 'hx2');
					$label = $label."$residue[$res1]$res1$x1-" if ($col_names{$pklist}{$value} eq 'x1');
				    }
				}
				$label =~ s/\-$//;
				print OUT "$label$space";
				foreach $value (sort {$a <=> $b} keys %{$col_names{$pklist}})
				{
				    my $key = $col_names{$pklist}{$value};
				    if ($key =~ /^(x1|hx1|hx2)$/)
				    {
					print OUT "$csi_data{$res1}{$x1}$space" if ($key eq 'x1');
					print OUT "$csi_data{$res1}{$hx1}$space" if ($key eq 'hx1');
					print OUT "$csi_data{$res2}{$hx2}$space" if ($key eq 'hx2');
					
					#print "$csi_data{$res1}{$x1}\t" if ($key eq 'x1');
					#print "$csi_data{$res1}{$hx1}\t" if ($key eq 'hx1');
					#print "$csi_data{$res2}{$hx2}\t" if ($key eq 'hx2');
				    }
				}
				print OUT "$note\n";
				#print "\n";
				
			    }
			}
		    }
		    
		}
		if (@carbs2)
		{
		    #x1 is carbon 2, hx1 is res2, hx1 is res1
		    ##print "x1 - $res2 @carbs2, hx1 - $res2 @atoms2, hx2 - $res1 @atoms1\n";
		    foreach $x1 (@carbs2)
		    {
			foreach $hx1 (@atoms2)
			{
			    my $sx1 = substr($x1,1); 
			    next if ($hx1 !~ /$sx1/);

			    if ($type eq 'C13-ali')
			    {
				next if (($residue[$res2] eq 'F') && ($x1 ne 'CA') && ($x1 ne 'CB'));  
				next if (($residue[$res2] eq 'W') && ($x1 ne 'CA') && ($x1 ne 'CB'));  
				next if (($residue[$res2] eq 'H') && ($x1 ne 'CA') && ($x1 ne 'CB'));  
				next if (($residue[$res2] eq 'Y') && ($x1 ne 'CA') && ($x1 ne 'CB'));  
			    }
			    if ($type eq 'C13-aro')
			    {
				next if (!(($residue[$res2] eq 'F') && ($x1 ne 'CA') && ($x1 ne 'CB')) && 
					 !(($residue[$res2] eq 'W') && ($x1 ne 'CA') && ($x1 ne 'CB')) && 
					 !(($residue[$res2] eq 'H') && ($x1 ne 'CA') && ($x1 ne 'CB')) && 
					 !(($residue[$res2] eq 'Y') && ($x1 ne 'CA') && ($x1 ne 'CB')));  
			    }

			    foreach $hx2 (@atoms1)
			    {

				next if (($res1 == $res2) && ($hx1 eq $hx2));

				$label = '';
				foreach $value (sort {$a <=> $b} keys %{$col_names{$pklist}})
				{
				    if ($col_names{$pklist}{$value} =~ /^(x1|hx1|hx2)$/)
				    {
					$label = $label."$residue[$res2]$res2$hx1-" if ($col_names{$pklist}{$value} eq 'hx1');
					$label = $label."$residue[$res1]$res1$hx2-" if ($col_names{$pklist}{$value} eq 'hx2');
					$label = $label."$residue[$res2]$res2$x1-" if ($col_names{$pklist}{$value} eq 'x1');
				    }
				}
				$label =~ s/\-$//;
				print OUT "$label$space";
				foreach $value (sort {$a <=> $b} keys %{$col_names{$pklist}})
				{
				    my $key = $col_names{$pklist}{$value};
				    if ($key =~ /^(x1|hx1|hx2)$/)
				    {
					print OUT "$csi_data{$res2}{$x1}$space" if ($key eq 'x1');
					print OUT "$csi_data{$res2}{$hx1}$space" if ($key eq 'hx1');
					print OUT "$csi_data{$res1}{$hx2}$space" if ($key eq 'hx2');
					
					#print "$csi_data{$res2}{$x1}\t" if ($key eq 'x1');
					#print "$csi_data{$res2}{$hx1}\t" if ($key eq 'hx1');
					#print "$csi_data{$res1}{$hx2}\t" if ($key eq 'hx2');
				    }
					
				}
				print OUT "$note\n";
				#print "\n";
			
			    }
			}
		    }
		}
		
	    }

	    if ($type =~ /[CN|N15]/i)
	    {
		my (@ntr1) = &get_nitrogen($csi_data_ref, $atm1, $res1);
		#my (@ntr2) = &get_nitrogen($csi_data_ref, $atm2, $res2);
		my (@ntr2);

		#print "($res1 $atm1) to ($res2 $atm2); ntr1 - @ntr1; ntr2 - @ntr2; atoms1 - @atoms1; atoms2 - @atoms2\n";
		
		if (@ntr1)
		{
		    #x1 is carbon 1, hx1 is res1, hx2 is res2
		    #print "x1 - $res1 @ntr1, hx1 - $res1 @atoms1, hx2 - $res2 @atoms2\n";
		    foreach $x1 (@ntr1)
		    {
			foreach $hx1 (@atoms1)
			{
			    my $sx1 = substr($x1,1); 
			    next if (($hx1 !~ /$sx1/) && ($x1 ne 'N'));

			    foreach $hx2 (@atoms2)
			    {
				next if (($res1 == $res2) && ($hx1 eq $hx2));
				$label = '';
				foreach $value (sort {$a <=> $b} keys %{$col_names{$pklist}})
				{
				    if ($col_names{$pklist}{$value} =~ /^(x1|hx1|hx2)$/)
				    {
					$label = $label."$residue[$res1]$res1$hx1-" if ($col_names{$pklist}{$value} eq 'hx1');
					$label = $label."$residue[$res2]$res2$hx2-" if ($col_names{$pklist}{$value} eq 'hx2');
					$label = $label."$residue[$res1]$res1$x1-" if ($col_names{$pklist}{$value} eq 'x1');
				    }
				}
				$label =~ s/\-$//;
				print OUT "$label$space";
				foreach $value (sort {$a <=> $b} keys %{$col_names{$pklist}})
				{
				    my $key = $col_names{$pklist}{$value};
				    if ($key =~ /^(x1|hx1|hx2)$/)
				    {
					print OUT "$csi_data{$res1}{$x1}$space" if ($key eq 'x1');
					print OUT "$csi_data{$res1}{$hx1}$space" if ($key eq 'hx1');
					print OUT "$csi_data{$res2}{$hx2}$space" if ($key eq 'hx2');
					
					#print "$csi_data{$res1}{$x1}\t" if ($key eq 'x1');
					#print "$csi_data{$res1}{$hx1}\t" if ($key eq 'hx1');
					#print "$csi_data{$res2}{$hx2}\t" if ($key eq 'hx2');
				    }
				}
				print OUT "$note\n";
				
			    }
			}
		    }
		}
		if (@ntr2)
		{
		    #x1 is carbon 2, hx1 is res2, hx1 is res1
		    #print "x1 - $res2 @ntr2, hx1 - $res2 @atoms2, hx2 - $res1 @atoms1\n";
		    foreach $x1 (@ntr2)
		    {
			foreach $hx1 (@atoms2)
			{
			    my $sx1 = substr($x1,1); 
			    next if (($hx1 !~ /$sx1/) && ($x1 ne 'N'));

			    foreach $hx2 (@atoms1)
			    {
				next if (($res1 == $res2) && ($hx1 eq $hx2));
				$label = '';
				foreach $value (sort {$a <=> $b} keys %{$col_names{$pklist}})
				{
				    if ($col_names{$pklist}{$value} =~ /^(x1|hx1|hx2)$/)
				    {
					$label = $label."$residue[$res2]$res2$hx1-" if ($col_names{$pklist}{$value} eq 'hx1');
					$label = $label."$residue[$res1]$res1$hx2-" if ($col_names{$pklist}{$value} eq 'hx2');
					$label = $label."$residue[$res2]$res2$x1-" if ($col_names{$pklist}{$value} eq 'x1');
				    }
				}
				$label =~ s/\-$//;
				print OUT "$label$space";
				foreach $value (sort {$a <=> $b} keys %{$col_names{$pklist}})
				{
				    my $key = $col_names{$pklist}{$value};
				    if ($key =~ /^(x1|hx1|hx2)$/)
				    {
					print OUT "$csi_data{$res2}{$x1}$space" if ($key eq 'x1');
					print OUT "$csi_data{$res2}{$hx1}$space" if ($key eq 'hx1');
					print OUT "$csi_data{$res1}{$hx2}$space" if ($key eq 'hx2');
					
					#print "$csi_data{$res2}{$x1}\t" if ($key eq 'x1');
					#print "$csi_data{$res2}{$hx1}\t" if ($key eq 'hx1');
					#print "$csi_data{$res1}{$hx2}\t" if ($key eq 'hx2');
				    }
				}
				print OUT "$note\n";
				#print "\n";
				
			    }
			}
		    }
		}
	    }
		
	}
    }

    close(OUT);
    
    &open_sparky("$ql_file", $parent, $temp_sparky);
}

sub create_sparky_list_fn
{
    my ($data_ref, $dir, $parent, $pklist, $note) = @_;
   
    #get and parse control file
    my ($control_file) = &get_control_file($dir);
    my ($comments_ref, $general_ref, $commands_ref, $peaklists_ref, $col_names_ref, $pklst_count) = &openContFile($control_file);
    my %data = %$data_ref;

    print "dir is $dir, and pklist is $pklist, control_file is $control_file, pklist is $pklist\n";
    
    my %col_names = %$col_names_ref;
    my ($plist, $value, $name);
    my $full_plist;

    #pklist is the key to hash $data, full_plist is the key to hash col_names
    
    foreach $plist (keys %col_names)
    {
	if ($plist =~ /$pklist/)
	{
	    $full_plist = $plist;
	}
    }
    
    #open sparky output file
    my $template = "create_sparky_fn";
    $temp_sparky = getTempFileName($template); 

    unless (open(OUT, "> $temp_sparky"))
    {
	print "Cannot open sparky file for writing\n";
	return;
    }

    print OUT "### Peaklist $full_plist\n";

    print OUT "# label     ";
    
    my $lbl_exists = 0;
    
    #my @col_headings;
    #$col_heading[0] = 'label';
    #my $col_headings_count = 0;

    foreach $value (sort {$a <=> $b} keys %{$col_names{$full_plist}})
    {
	if ($col_names{$full_plist}{$value} =~ /^(x1|x2|hx1|hx2)$/)
	{
	    print OUT "$col_names{$full_plist}{$value}     ";
	}
    }
    
    print OUT "note\n\n";
    
    my $i;
    if (!$note)
	{
	 $note = 'false_negative';
	}
    my $label = "?-?-?";
    my $space = "     ";
    
    for ($i = 0; $i < scalar(@{$data{$pklist}}); $i++)
    { 
	my %info;
	my $line = $data{$pklist}[$i];
	#print "$line\n";
	my $new_line;

	#my $id = $i+1;

	#$line =~ /\(label\)(.+)$/;
	#my ($label) = ($1, $2);
	
	#unless ($label)
	#{
	 #   $label = "?-?-?";
	#}
	#$new_line = $new_line."$label\t";

	$new_line = "$label$space";
	
	foreach $value (sort {$a <=> $b} keys %{$col_names{$full_plist}})
	{
	    my $heading = $col_names{$full_plist}{$value};
	    next unless ($heading =~ /^(x1|x2|hx1|hx2)$/);
	    
	    $line =~ /\($heading\)([.\d]+)\s/;
	    my $temp = $1;
	    $new_line = "$new_line$temp$space";
	}
	
	print OUT "$new_line$note\n";
    }

    close(OUT);
    
    #open the sparky file
    &open_sparky("Peaklist $pklist", $parent, $temp_sparky);
}

sub open_sparky
{
    my ($title, $parent, $temp_sparky) = @_;

    my $win = $parent->Toplevel();
    $win->configure(-title=>"Sparky List - $title");
    
    my $lbl = $win->Label(-text=>"$title", -background=>'white', -relief=>'ridge', -borderwidth=>2)->pack(-padx=>3, -pady=>3, -fill=>'x');
    my $s_txt = $win->Scrolled('Text', -scrollbars=>'e')->pack(-fill=>'both', -expand=>1, -padx=>3, -pady=>3);
    my $txt = $s_txt->Subwidget('text');
    $txt->configure(-background=>'white');
    my $frm = $win->Frame()->pack(-fill=>'x');
    my $but1 = $frm->Button(-text=>'Save')->pack(-fill=>'x', -padx=>3, -pady=>3, -side=>'left', -expand=>1);
    my $but2 = $frm->Button(-text=>"Close", -command=>sub
			    {
				$win->destroy();
			    })->pack(-fill=>'x', -side=>'left', -expand=>1);
    unless (open(IN, $temp_sparky))
    {
	print "Cannot open sparky file\n";
	return;
    }

    my (@lines) = <IN>;
    my %lines_hash;
    #print "@lines\n";

    close(IN);

    my $line;
    foreach $line (@lines)
    {
	#$s_txt->insert('end', $line);
	unless ($lines_hash{$line})
	{
	    $lines_hash{$line} = 1;
	    $s_txt->insert('end', $line);
	}
    }
    
    #save sparky file
    my $perm_sparky = '';
    
    $but1->configure(-command=>sub
		     {
			 unless ($perm_sparky)
			 {
			     #create new file
			     ($perm_sparky) = &SaveFile('Save Sparky', $win, Cwd::cwd);
			 }
			 
			 #open and write to the sparky file
			 unless (open(IN, "> $perm_sparky"))
			 {
			     &err_msg("Cannot open sparky file $perm_sparky", $win);
			     return;
			}
			
			my @lines = $txt->get('1.0', 'end');
			
			print IN @lines;
			close(IN);
			&err_msg("Done Saving!", $win);
			$win->configure(-title=>"Sparky file $perm_sparky");
		    });

    
    
    $win->waitWindow();

}


sub get_carbon
{
    my ($csi_data_ref, $atom, $res_num) = @_;

    my %csi_data = %$csi_data_ref;
    
    my @carbons = grep {/^C/i} sort {$a cmp $b || $a <=> $b} keys %{$csi_data{$res_num}};
    #print "1. carbons of residue $res_num are @carbons (atom is $atom)\n";
    my @car;
    if ($atom =~ /^[HQ](\w)(.*)$/)
    {

	#print $atom . " " . $res_num . " " . $1 . " " . $2 . "\n"; 

	my ($t1, $t2) = ($1, $2);
	
	(@car) = grep {/$t1/} @carbons;

	#print "2. carbons of residue $res_num, $atom are @car (t1 = $t1, t2 = $t2)\n";
	if ($car[1] and ($t2 ne '*'))
	{
	    (@car) = grep {/$t2/} @car if ($t2);
	}

	#foreach (@car)
	#{
	#    print $_ . " "; 
	#}
	#print "\n";
    }

    return (@car);
}

sub get_nitrogen
{
    my ($csi_data_ref, $atom, $res_num) = @_;

    my %csi_data = %$csi_data_ref;
    
    my @nitros = grep {/^N/i} sort {$a cmp $b || $a <=> $b} keys %{$csi_data{$res_num}};
    #print "1. nitrogens of residue $res_num are @nitros (atom is $atom)\n";
    my @ntr;
    my $t1;
    if ($atom eq 'H')
    {
	@ntr = 'N';
    }

    if ($atom =~ /^H(\w\w)/)
    {
	($t1) = ($1);
	(@ntr) = grep {/$t1/} @nitros;
	
    }
    #print "2. ntr of residue $res_num, $atom are @ntr (t1 = $t1)\n";
    return (@ntr);

}

sub get_mult_atoms
{
    my ($csi_data_ref, $atom, $res_num) = @_;

    my %csi_data = %$csi_data_ref;
    my @hydros = grep {/^H/i} keys %{$csi_data{$res_num}};
    my @hyd;
    $hyd[0] = $atom;
    #print "1. hydrogs of residue $res_num are @hydros (atom is $atom)\n";
    if ($atom =~ /^H(\w+)\*$/)
    {
	my ($t1) = $1;
	(@hyd) = grep {/$t1/} @hydros;
	#print "2a. hyd of residue $res_num ($atom) is @hyd\n";
    }
    elsif ($atom =~ /^Q(\w+)$/)
    {
	my ($t1) = $1;
	(@hyd) = grep {/$t1/} @hydros;
	#print "2b. hyd of residue $res_num ($atom) is @hyd\n";
    }
    
    #print "3. hyd of residue $res_num ($atom) is @hyd\n";
    return (@hyd);

}



return 1;
