sub create_tree
{
    my $root;
    
    my ($view_pg, $win, $tree);
    
    ($view_pg, $win, $root, $tree) = @_;

    unless (-d $root)
    {
	return;
    }
    #print "1. create_tree\n";

    my $view_frm = $view_pg->Frame(-background=>'white')->pack(-fill=>'both', -expand=>1, -padx=>2, -pady=>2);
    $tree = $view_frm->Scrolled('Tree', -scrollbars=>'osoe', -background=>'white', -indent=>15, -command=>sub 
				{
				    my ($fname) = @_;
				    
				    $fname =~ s/\./\//g;
				    
				    $fname =~ s/:/\./g;
				    
				    $fname =~ s/Root/$root/;
				    
				    
				    if (-f $fname)
				    {
					my $pid;
					if ($pid = fork) 
					{
					    #parent
					    $SIG{CHILD} = 'IGNORE';
					    return;
					}
					elsif (defined $pid) 
					{ 
					    #child
					    exec 'xterm', '-title', "$fname", '-e', 'less', "$fname";
					}
				    }
				}
				)->pack(-fill=>'both', -expand=>1);
    #print "2. adding root $root\n";
    &add_root($tree, $root);
    

    #buttons frame - update and close
    my $but_frm = $view_frm->Frame(-relief=>'ridge', -borderwidth=>2)->pack(-fill=>'x', -side=>'bottom');

    my $upd_but = $but_frm->Button(-text=>'Update', -command=>sub {&update_tree($tree, $root);})
	->pack(-fill=>'x', -expand=>1, -padx=>3, -pady=>3, -side=>'left');

}



    


sub update_tree
{
    my ($tree, $root) = @_;
    
    $tree->delete('all');
    &add_root($tree, $root);
}



sub add_root
{
    my ($tree, $root) = @_;

    $tree->add('Root', -text=>"Output Dir = $root");
    #print "2. add_root - $root\n";
    &add_dir($root, 'Root', $tree);
    

    my $opencmd = [
		   sub {
		       my ($d, $path) = @_;

		       my $full_path = $path;
		       $full_path =~ s/\./\//g;
		       $full_path =~ s/\:/\./g;
		       $full_path=~ s/Root/$root/;

		       &add_dir($full_path, $path, $tree);
		       $tree->setmode($path, 'close');
		       
		   }, $tree];

    my $closecmd = [    
			sub {
			    
			    my ($d, $path) = @_;
			    
			    $tree->delete('offsprings', $path);
			    $tree->setmode($path, 'open');
			    
			}, $tree];
    
    $tree->configure(-opencmd=>$opencmd, -closecmd=>$closecmd);

    $tree->setmode('Root', 'close');
}

sub add_dir
{
    my ($dir, $alias, $tree) = @_;
    #print "add_dir - $dir, alias - $alias\n";

    opendir (DIR, $dir) or die "Cannot open $dir: $!\n";
    
    my @names = sort {lc($a) cmp lc($b)} readdir(DIR);
    my $name;
    
    foreach $name (@names)
    {
	next if $name=~/^\./;
	my $mod_name = $name;

	if ($name =~ /\./)
	{
	    $mod_name =~ s/\./:/g;
	}
	
	$mod_name = $alias.'.'.$mod_name;

	if (-d (File::Spec->rel2abs($name, $dir)))
	{
	    $tree->add($mod_name, -text=>$name);
	    $tree->setmode($mod_name, 'open');
	    
	}
	elsif (-f (File::Spec->rel2abs($name, $dir)))
	{
	    $tree->add($mod_name, -text=>$name);
	}
    }

    $tree->setmode($alias, 'open');
    $tree->close($alias);
    close(DIR);
}


return 1;
