sub display_peak_info
{
    my ($data_ref, $parent, $res1, $res2) = @_;
    
    my %data_from_file = %$data_ref;
    
    my ($fname, $key, $i);

    my $win = $parent->Toplevel(-title=>"Peaklist information for Residues $res1 and $res2");
    $win->minsize(qw/1200 400/);

    my $lbl = $win->Label(-text=>"Peaks showing interactions between residues $res1 and $res2", -relief=>'ridge', -borderwidth=>2)->pack(-ipadx=>5, -ipady=>8, -fill=>'x');

    my $nb = $win->NoteBook()->pack(-fill=>'both', -expand=>1);
    my $help_txt = "Double-click on a peak to display its additional and all possible assignments";
    my $help_lbl = $win->Label(-text=>$help_txt)->pack(-padx=>10, -pady=>5);
    
    my $adn_frm = $win->Frame()->pack(-fill=>'both', -expand=>1, -padx=>5, -pady=>5);
    my $adn_lbl = $adn_frm->Label(-text=>"All assignments", -relief=>'ridge', -borderwidth=>2)->pack(-padx=>5, -pady=>5);
    my $scr_txt = $adn_frm->Scrolled("ROText", -scrollbars=>'se')->pack(-fill=>'both', -expand=>1);
    my $adn_txt = $scr_txt->Subwidget("rotext");
    $adn_txt->configure(-background=>'white');
   
    my $main = $nb->add('Main', -label=>'Main');

    my $page_num = 1;
    
    foreach $fname (keys %data_from_file)
    {
	my $cur_pg_num = $page_num;
	my $page = $nb->add($cur_pg_num, -label=>"Peaklist $cur_pg_num");
	if ($cur_pg_num == 1)
	{
	    $nb->delete('Main');
	}

	my $pkl = basename($fname);
	my $dir = dirname($fname);

	#print "$pkl\n";

	my @data = @{$data_from_file{$fname}};

	my $record_num = scalar(@data);
	my $pkl_lbl = $page->Label(-text=>"Peaklist = $pkl\nNumber of Records = $record_num")->pack(-padx=>10, -pady=>5);
	
	my $scr_lbx = $page->Scrolled("Listbox", -scrollbars=>'se')->pack(-fill=>'both', -expand=>1);
	my $lbx = $scr_lbx->Subwidget("listbox");
	$lbx->configure(-background=>'white', -selectmode=>'single');

	unless (($res1) and ($res2))
	{
	    $lbx->insert('end', "No data from this peaklist\n");
	    $page_num++;
	    next;
	}

	if ($record_num < 1)
	{
	    $lbx->insert('end', "No peaks found");
	    $page_num++;
	    next;
	}


	for ($i = 0; $i<$record_num; $i++)
	{
	    $lbx->insert('end', "- $data[$i]{'peak'}");
	    $lbx->insert('end', "  $data[$i]{'atom 1'}");
	    $lbx->insert('end', "  $data[$i]{'atom 2'}");
	    my $peak = "- $data[$i]{'peak'}";
	    my $pkl_fname = $fname;
	    my ($data_ref) = &parse_assOrder_adnl($win, $pkl_fname, $peak);
	    my @data = @$data_ref;
	    my $data_count = $#data;
	    $lbx->insert('end', "  $data_count additional assignments");
	    $lbx->insert('end', "");
	}
	
	$lbx->bind('<Double-Button-1>'=>sub
	       {
		   my $line = $lbx->get($lbx->curselection());
		   #chomp($line);

		   if ($line =~ /^- peak/)
		   {
		       #get additional assignments of peak
		       my $pkl_fname = $fname;
		       my ($data_adnl_ref) = &parse_assOrder_adnl($win, $pkl_fname, $line);

		       
		      
		       $pkl =~ s/_assOrder/_match\.gz/;
		       $match_file = $dir . "/../" . $pkl; 

		       #my (@match_files) = &get_match_files($dir, $win);
		       

		       #my $old_cur = $win->cget('cursor');
		       #print "old cursor is $old_cur\n";
		       $lbx->configure(-cursor=>"watch");
		       $adn_txt->delete('1.0', 'end');
		       $lbx->update();
		       &display_adnl($data_adnl_ref, $adn_txt, $match_file, $line, $win);
		       $lbx->configure(-cursor=>"top_left_arrow");
		   }
	       });

	$page_num++;
    }
}

sub display_adnl
{
    my ($data_ref, $ro_txt, $match_file, $peak, $parent) = @_;

    my @data = @$data_ref;
    
    my $count = $#data+1;
    my $i;

    if ($count < 1)
    {
	$ro_txt->insert('end', "No assignments\n");
    }
    else
    {
	$ro_txt->insert('end', "***NOE assgnments ***\n\n");
	for ($i = 0; $i<=$count; $i++)
	{
	    $ro_txt->insert('end', "$data[$i]{'peak'}\n");
	    $ro_txt->insert('end', "$data[$i]{'atom 1'}\n");
	    $ro_txt->insert('end', "$data[$i]{'atom 2'}\n");
	    $ro_txt->insert('end', "\n");
	}
    }

    #uncompress the match file
    unless (-e $match_file)
    {
	print "match file $match_file does not exist\n";
	return;
    }
    
    $match_file = &unzip_file($match_file);
    
    #parses the match file for peak $peak and displays that information
    my ($data_mat_ref) = &parse_match($match_file, $peak, $parent);
    
    $ro_txt->insert('end', "\n\n***All possible assignments (last cycle)***\n\n");

    my (@data_mat) = @$data_mat_ref;
    
    for ($i = 0; $i<=$#data_mat; $i++)
    {
	$ro_txt->insert('end', "$data_mat[$i]\n");
    }
}


return 1;
