sub display_qi
{
    my ($q_dir, $parent) = @_;

    #get the qi file
    my ($qi_file) = &get_qi_file($q_dir, $parent);
    unless ($qi_file)
    {
	return;
    }
    #parse the qi file
    my ($data_ref) = &parse_qi($qi_file, $parent);
    my %data = %$data_ref;

    #display the data
    my $win = $parent->Toplevel(-title=>"False negative interactions");
    
    my $heading = $win->Label(-text=>"False negetive interactions from file\n$qi_file", -relief=>'ridge')->pack(-fill=>'x',-ipady=>3);
    my $help_txt = "Double-click on a peak to display its all possible assignments";
    my $help_lbl = $win->Label(-text=>$help_txt)->pack(-padx=>10, -pady=>5);
   
    my $nb = $win->NoteBook()->pack(-fill=>'both', -expand=>1);
    my $main = $nb->add('Main', -label=>'Main');

    #getmatch file

    my $pklst;
    my $pk_count = 1;

    my %pk_match; 
    foreach $pklst (keys %data)
    {
	my $page = $nb->add($pklst, -label=>"Peaklist $pk_count");
	
	my $pk_lbl = $page->Label(-text=>"Peaklist $pklst", -relief=>'ridge')->pack(-fill=>'x', -pady=>2);
	
	my $scr_lbx = $page->Scrolled('Listbox', -scrollbars=>"osoe")->pack(-fill=>'both', -expand=>1, -padx=>5, -pady=>5);
	my $lbx = $scr_lbx->Subwidget("listbox");
	$lbx->configure(-selectmode=>'single', -background=>'white');

	$new_match = "$q_dir" . "/" . $pklst . "_match.gz";
	$match_file =&unzip_file($new_match); 

	#my $template = $pklst . "_fn"; 
	#$template =~ s/\//_/g; 
	#my (undef, $fn_name) = tempfile($template . "XXXX", DIR => "$HOME/.autostructure/", OPEN=>0); 

	my $i;
	my $count = scalar(@{$data{$pklst}});

	for ($i = 0; $i<$count; $i++)
	{
	    #$lbx->insert('end', "$data{$pklst}[$i]\n");
	    $lbx->insert('end', "$data{$pklst}[$i]");
	    $pk_match{$data{$pklst}[$i]} = $match_file; 
	}
	$pk_count++;

	my $sp_but = $page->Button(-text=>'Create Sparky File', -command=>sub
				  {
				      &create_sparky_list_fn($data_ref, $q_dir, $win, $pklst);
				  })->pack(-padx=>5, -pady=>5, -fill=>'x');
	
	$lbx->bind('<Double-Button-1>'=>sub
		   {
		       my $line = $lbx->get($lbx->curselection());
		       
		       $nline = $line; 
		       #print "line is $line\n";
		       $nline =~ s/^-> //; 
		       
                   if ($nline =~ /^peak/)
                   {		       
                       #print "match file is $match_file\n";
                       $lbx->configure(-cursor=>"watch");
		       $match_file = $pk_match{$line}; 
                       $lbx->update();
                       &disp_match($win, $match_file, $nline);
                       $lbx->configure(-cursor=>"top_left_arrow");
                   }
		   });
    }

    $nb->delete('Main');
    
    my $cls_but = $win->Button(-text=>'Close', -command=>sub
			       {
				   $win->destroy();
			       })->pack(-fill=>'x', -padx=>5, -pady=>5);
    

    
}

sub parse_qi
{
    my ($qi_file, $parent) = @_;

    unless (open(FILE, $qi_file))
    {
	&err_msg("Cannot open QI file \"$qi_file\"", $parent);
	return;
    }

    my %data;
    my ($line, $count, $pklst);
    
    while (chomp($line = <FILE>))
    {
	if ($line =~ /inconsistent\speaks\sfor\s(.+)$/i)
	{
	    $pklst = $1;
	    $count = 0;
	}
	elsif($line =~ /peak\:/)
	{
	    #print $line . "\n";

	    $data{$pklst}[$count] = $line;
	    $count++;
	}
    }
    return (\%data);


}


sub get_qi_file
{
    my ($dir, $parent) = @_;

    unless (opendir(DIR, $dir))
    {
	&err_msg("Cannot open directory \"$dir\"", $parent);
	return;
    }

    my @contents = readdir(DIR);
    close(DIR);
    
    my ($file) = grep {/\.qi$|qr$/i} @contents;
    if ($file)
    {
	$file = File::Spec->rel2abs($file, $dir);
    }

    return ($file);
}


return 1;
