my %aa = (
    "ALA"=>"A",
    "ARG"=>"R",
    "ASN"=>"N",
    "ASP"=>"D",
    "CYS"=>"C",
    "GLN"=>"Q",
    "GLU"=>"E",
    "GLY"=>"G",
    "HIS"=>"H",
    "ILE"=>"I",
    "LEU"=>"L",
    "LYS"=>"K",
    "MET"=>"M",
    "PHE"=>"F",
    "PRO"=>"P",
    "SER"=>"S",
    "THR"=>"T",
    "TRP"=>"W",
    "TYR"=>"Y",
    "VAL"=>"V"
	  );

sub distance_query
{
    #given pdb file, create interface for querying distance between two atoms
    my ($pdb_file, $parent) = @_;
    #print "distance_query: pdb_file is $pdb_file\n";

    unless (-f $pdb_file)
    {
	return;
    }


    #data of format - $data{'ATOM'}[$model]{'14L'}{'HA'}{'x'} = #
    #                 $data{'RESIDUE'}[14] = 'L';
    my ($data_ref, $count_ref) = &parse_pdb($pdb_file, $parent);


    my (%vars);
    #format of vars - $vars{'res1_num'}
    #                 $vars{'res1_atm'}
    #                 $vars{'res2_num'}
    #                 $vars{'res2_atm'}

    my ($query_win, $sub_but) = &create_dist_query_win($parent, \%vars, $count_ref);

    $sub_but->configure(-command=>sub
			{
			    if (&check_dist_vars(\%vars, $query_win))
			    {
				#perform query
				my ($d_eff) = &find_distance_eff(\%vars, $query_win, $data_ref, $count_ref);
			    }
			});

}

sub find_distance_eff
{
    my ($vars_ref, $parent, $data_ref, $count_ref) = @_;

    my @res1_arr = ($vars_ref->{'res1_atm'});
    my @res2_arr = ($vars_ref->{'res2_atm'});
    my ($res1_count, $res2_count) = (0,0);

    my %data = %$data_ref;
    my %count = %$count_ref;

    #find all atoms of form res1_num-res1_atm and store in res1_arr
    #all atoms of form res2_num-res2_atm, store in res2_arr
    #assume model number 0

    $vars_ref->{'res1_name'} = $data{'RESIDUE'}[$vars_ref->{'res1_num'}];
    $vars_ref->{'res2_name'} = $data{'RESIDUE'}[$vars_ref->{'res2_num'}];

    #print "querying res1 as $vars_ref->{'res1_num'}$vars_ref->{'res1_name'}-$vars_ref->{'res1_atm'} and res2 as $vars_ref->{'res2_num'}$vars_ref->{'res2_name'}-$vars_ref->{'res2_atm'}\n";
    
    
    my $atom;
    my $ct = 0;
    foreach $atom (keys %{$data{'ATOM'}[0]{$vars_ref->{'res1_num'}.$vars_ref->{'res1_name'}}})
    {
	if ($atom =~ /^$vars_ref->{'res1_atm'}\d+/)
	{
	    $res1_arr[$ct] = $atom;
	    $ct++;
	}
    }
    $ct = 0;

    foreach $atom (keys %{$data{'ATOM'}[0]{$vars_ref->{'res2_num'}.$vars_ref->{'res2_name'}}})
    {
	if ($atom =~ /^$vars_ref->{'res2_atm'}\d+/)
	{
	    $res2_arr[$ct] = $atom;
	    $ct++;
	}
    }
    #print "res1_arr = @res1_arr, res2_arr = @res2_arr\n";

    my $d_eff;

    my $tot_dist = 0;
    
    #assume model 0
    $vars_ref->{'model'} = '0';
    $vars_ref->{'res1_name'} = $aa{$vars_ref->{'res1_name'}};
    $vars_ref->{'res2_name'} = $aa{$vars_ref->{'res2_name'}};
    
    for ($i = 0; $i<=$#res1_arr; $i++)
    {
	$vars_ref->{'res1_atm'} = $res1_arr[$i];
	for ($j = 0; $j<=$#res2_arr; $j++)
	{
	    $vars_ref->{'res2_atm'} = $res2_arr[$j];

	    my ($distance) = &get_unique_dist($vars_ref, $data_ref);
	    $distance = sprintf("%0.2f", $distance);
	    my $mod_dist = ($distance) ** (-6);
	    
	    $tot_dist += $mod_dist;
	}
    }
    #print "total distance = $tot_dist\n";
	
    if ($tot_dist)
    {
	$d_eff = ($tot_dist) ** (-1/6);
	$d_eff = sprintf("%0.2f", $d_eff);

	my $msg = "The effective distance from $vars_ref->{'res1_num'}-$vars_ref->{'res1_atm'} and $vars_ref->{'res2_num'}-$vars_ref->{'res2_atm'} in chain $vars_ref->{'chain'} is";
	print "$msg $d_eff\n";
	&err_msg("$msg \n\n$d_eff\n", $parent);
    }
    
    return ($d_eff);
    
}

sub pt_distance
{
    #given two 3D points, returns the distance between them
    my ($x1, $y1, $z1, $x2, $y2, $z2) = @_;

    my $term1 = ($x2 - $x1) ** (2);
    my $term2 = ($y2 - $y1) ** (2);
    my $term3 = ($z2 - $z1) ** (2);
    

    my $dist = sqrt($term1 + $term2 + $term3);
    #$dist = sprintf("%0.2f", $dist);
    
   
    return ($dist);
}



sub check_dist_vars
{
    my ($vars_ref, $parent) = @_;

    unless ($vars_ref->{'res1_num'})
    {
	&err_msg("Enter a number for Residue 1", $parent);
	return 0;
    }

    unless ($vars_ref->{'res1_atm'})
    {
	&err_msg("Enter an atom for Residue 1", $parent);
	return 0;
    }

    unless ($vars_ref->{'res2_num'})
    {
	&err_msg("Enter a number for Residue 2", $parent);
	return 0;
    }

    unless ($vars_ref->{'res2_atm'})
    {
	&err_msg("Enter an atom for Residue 2", $parent);
	return 0;
    }

    return 1;
}


sub create_dist_query_win
{
    my ($parent, $vars_ref, $count_ref) = @_;

    my %count = %$count_ref;
    #my @chain_list;
    #my $chain_count = 0;
    #my $chain;

    #foreach $chain (sort keys %{$count{'SEQRES'}})
    #{
	#$chain_list[$chain_count] = $chain;
	#$chain_count++;
    #}
    
    my $win = $parent->Toplevel;
    $win->configure(-title=>"Distance query");

    my $res1_frm = $win->Frame()->pack(-side=>'top', -fill=>'x', -expand=>1);
    my $res1_lbl = $res1_frm->Label(-text=>"Residue 1:", -relief=>'ridge', -borderwidth=>1)->pack(-side=>'left', -padx=>3, -pady=>5, -ipadx=>3, -ipady=>3);
    my $res1_num_lbl = $res1_frm->Label(-text=>"Number:")->pack(-side=>'left', -padx=>2, -pady=>5);
    my $res1_num_ent = $res1_frm->Entry(-background=>'white', -textvariable=>\$vars_ref->{'res1_num'})->pack(-side=>'left', -padx=>3, -pady=>5);
    my $res1_atm_lbl = $res1_frm->Label(-text=>"Atom:")->pack(-side=>'left', -padx=>3, -pady=>5);
    my $res1_atm_ent = $res1_frm->Entry(-background=>'white', -textvariable=>\$vars_ref->{'res1_atm'})->pack(-side=>'left', -padx=>3, -pady=>5);

    my $res2_frm = $win->Frame()->pack(-side=>'top', -fill=>'x', -expand=>1);
    my $res2_lbl = $res2_frm->Label(-text=>"Residue 2:", -relief=>'ridge', -borderwidth=>1)->pack(-side=>'left', -padx=>3, -pady=>5, -ipadx=>3, -ipady=>3);
    my $res2_num_lbl = $res2_frm->Label(-text=>"Number:")->pack(-side=>'left', -padx=>2, -pady=>5);
    my $res2_num_ent = $res2_frm->Entry(-background=>'white', -textvariable=>\$vars_ref->{'res2_num'})->pack(-side=>'left', -padx=>3, -pady=>5);
    my $res2_atm_lbl = $res2_frm->Label(-text=>"Atom:")->pack(-side=>'left', -padx=>3, -pady=>5);
    my $res2_atm_ent = $res2_frm->Entry(-background=>'white', -textvariable=>\$vars_ref->{'res2_atm'})->pack(-side=>'left', -padx=>3, -pady=>5);

    #my $chn_frm = $win->Frame()->pack(-side=>'top', -fill=>'x', -expand=>1);
    #my $chn_lbl = $chn_frm->Label(-text=>'Chain: ', -relief=>'ridge', -borderwidth=>1)->pack(-side=>'left', -padx=>3, -pady=>5, -ipadx=>3, -ipady=>5);
    #my $chn_opt = $chn_frm->Optionmenu(-options=>\@chain_list, -variable=>\$vars_ref->{'chain'})->pack(-side=>'left', -padx=>3, -pady=>3);

    my $but_frm = $win->Frame()->pack(-side=>'top', -fill=>'x', -expand=>1, -padx=>5, -pady=>5);
    my $sub_but = $but_frm->Button(-text=>"Query")->pack(-expand=>1, -padx=>3, -pady=>3, -ipadx=>2, -ipady=>2);
    
    return ($win, $sub_but);
}


return 1;
