#!/usr/bin/perl


use Env;
use Tk;
use strict;
use Tk::Dialog;
use Tk::NoteBook;
use Tk::Optionmenu;
use Tk::Balloon;
use Tk::Table;
use Tk::BrowseEntry;
use File::Spec;
use Time::localtime;
use File::Basename;
use File::Path;
use Tk::Tree;
use Tk::Dirlist;
use Tk::ProgressBar;
use Tk::ROText;
use LWP;
use Shell 'rm';
use Shell 'cp';
use LWP::UserAgent;
use HTTP::Request::Common qw(POST);
use File::Temp qw / tempfile /;

require "displayControl.pl";
require "FileSelection.pl";
require "display_pklst.pl";
require "add_pklst.pl";
require "openContFile.pl";
require "start_gui.pl";
require "start_child.pl";
require "OpenOutDir.pl";
require "RPF_gui.pl";
require "cns_gui.pl";
require "dp_ranks_frame.pl";
require "parse_ovw.pl";
require "rpf_scores_frame.pl";
require "RPF_mult_gui.pl";
require "create_tree.pl";
require "create_graph.pl";
require "parse_ql.pl";
require "calc_rasmol.pl";
require "show_interactions.pl";
require "parse_upl.pl";
require "create_contact_map.pl";
require "parse_assOrder.pl";
require "display_peak_info.pl";
require "parse_match.pl";
require "parse_pdb.pl";
require "distance_query.pl";
require "parse_unassign.pl";
require "unassign_distance.pl";
#require "new.pl";
require "parse_as_ovw.pl";
require "create_as_sum.pl";
require "beta.pl";
require "display_qi.pl";
require "conv_all.pl";
require "parse_sec.pl";
require "sec_str_disp.pl";
require "parse_nhs.pl";
require "parse_seq_pred.pl";
require "parse_jfile.pl";
require "show_betas.pl";
require "prof_pred.pl";
require "convert_gui.pl";
require "convert_bmrb.pl";
require "all_betas.pl";
require "create_sparky_list.pl";
require "parse_csi.pl";
require "parse_atom_nom.pl";
require "calib_k.pl";
require "create_k_plot.pl";
require "get_sec_str_k.pl";
require "parse_assOrder_k.pl";
require "tools.pl";


#global variables
my $basepath = Cwd::cwd;
my $start_vals;
my $rec_dir;
my $rpf_vals;
my $cns_vals;
my $prog_cont;
my $prog_var = 0;
my $ovw_file;

my %tempFile; 

my $main = MainWindow->new();
$main->optionAdd('*font', 'Helvetica 8 normal');
$main->configure(-title=>"Automated analysis of protein structures from NMR data");;
$main->minsize(qw(500 500));


#Menubar
my $menubar = $main->Menu;
$main->configure(-menu=>$menubar);

my $file = $menubar->cascade(-label=>'File', -tearoff=>0);
my $auto = $menubar->cascade(-label=>'ASDP', -tearoff=>0);
#my $results = $menubar->cascade(-label=>'Results', -tearoff=>0);
my $rpf = $menubar->cascade(-label=>'RPF', -tearoff=>0);
my $refine = $menubar->cascade(-label=>'Refine', -tearoff=>0);
my $pdb_tools = $menubar->cascade(-label=>'PdbTools', -tearoff=>0);
my $bmrb_tools = $menubar->cascade(-label=>'BmrbTools', -tearoff=>0);
my $tools = $menubar->cascade(-label=>'MiscTools', -tearoff=>0);
#my $options = $menubar->cascade(-label=>'Options', -tearoff=>0);
my $help = $menubar->cascade(-label=>'Help', -tearoff=>0);

$file->command(-label=>'New Control File', -command=>\&new);
$file->command(-label=>'Open Control File', -command=>\&open);
$file->separator();
$file->command(-label=>'Preferences', -state=>'disabled', -command=>\&pref);
$file->command(-label=>'Quit', -command=>sub { 
    $main->destroy();
});

$rpf->command(-label=>'Calc', -command=>sub {$rpf_vals = &rpf_gui($main, $basepath);});

#$rpf_cas->command(-label=>'One', -command=>sub {$rpf_vals = &rpf_gui($main, $basepath);});
#$rpf_cas->command(-label=>'For AutoStructuer Output Dir', -command=>sub {$rpf_vals = &rpf_mult_gui($main, $basepath);});

$rpf->command(-label=>'Open RPF directory', -command=>\&open_rpf);
#$rpf->command(-label=>'Open an RPF File', -command=>sub {my $file = &OpenFile('ovw', $main, $basepath); &open_rpf_file($file);});
#$rpf->command(-label=>'Open RPF Output Directory', -command=>\&open_rpf_dir);

my $calc = $auto->cascade(-label=>'Calc', -tearoff=>0);
$calc->command(-label=>'Rerun', -command=>sub {&start(1)}, -state=>'disabled');
$calc->command(-label=>'Start', -command=>sub {&start(0)}, -state=>'normal');
$calc->command(-label=>'Stop', -command=>sub {$prog_cont = 0; &stop_load(); $prog_var = 0;}, -state=>'disabled');

$auto->command(-label=>'Open Recent-Calc AS OutputDir', -command=>\&rec_calc, -state=>'disabled');
$auto->command(-label=>'Open AS OutputDir', -command=>\&open_as_dir);

$pdb_tools->command(-label=>'Distance Query', -command=>\&pdb_dist_query);
#$pdb_tools->command(-label=>'PDB to IUPAC', -command=>sub {&conv_pdb_gui($main, $basepath);});
$pdb_tools->command(-label=>'Format conversions', -command=>sub {&convert_gui($main);});

$bmrb_tools->command(-label=>'Bmrb conversion', -command=>sub {&convert_bmrb($main);});

$refine->command(-label=>'Calc', -command=>sub {$cns_vals = &cns_gui($main, $basepath, "new");});
$refine->command(-label=>'Open CNS dir', -command=>sub {$cns_vals = &cns_gui($main, $basepath, "open");});

$tools->command(-label=>'Open Directory', -command=>\&open_dir);
$tools->command(-label=>'Secondary Structure Prediction', -command=>sub {&prof_pred($main);});
#$tools->command(-label=>'Format conversions', -command=>sub {&convert_gui($main);});
$tools->command(-label=>"Calibrate K", -command=>sub {&calib_k($main);}); # function is in file calib_k.pl

$help->command(-label=>'About', -command=>sub {&err_msg("ASDP Version 1.0", $main);});

#Status bar 1
my $stat1;
my $frame1 = $main->Frame()->pack(-side=>'top', -fill=>'x', -expand=>1);
my $status1 = $frame1->Label(-textvariable=>\$stat1, -relief=>'sunken', -background=>'grey', -foreground=>'black')->pack(-side=>'top', -fill=>'x', -expand=>1);


#basepath frame
my $bg = '#BBBBBB';
my $bp_frm = $main->Frame()->pack(-fill=>'x', -expand=>1, -padx=>3, -pady=>3);
my $bp_lbl = $bp_frm->Label(-text=>'Current Working Directory: ', -background=>$bg)->pack(-side=>'left', -padx=>3);
    
my $bpth_ent = $bp_frm->Entry(-textvariable=>\$basepath, -background=>$bg)->pack(-side=>'left', -padx=>3, -fill=>'x', -expand=>1);
my $chg_bpth = $bp_frm->Button(-text=>'Change Dir', -background=>$bg, -state=>'disabled')->pack(-side=>'left', -padx=>3);

my $brw_bpth = $bp_frm->Button(-text=>'Browse', -background=>$bg, -command=>sub
				   {
				       my $new_basepath = &OpenDir('Basepath', $main, $basepath);
				       my $old_basepath = $basepath;
				       
				       if (-d $new_basepath)
				       {
					   $basepath = $new_basepath;
					   chdir $basepath;
					   $basepath = Cwd::cwd;
				       }
				       
				       &err_msg("The Current Working Directory has been changed to \"$basepath\"\nClose any previously opened windows to update their working directory!", 
						$main);
				       $chg_bpth->configure(-state=>'disabled');
				       
				   })->pack(-side=>'left', -padx=>3);

$chg_bpth->configure(-command=>sub {&change_basepath(\$basepath, $main, $chg_bpth);});
    
$bpth_ent->bind('<KeyPress>'=>sub {$chg_bpth->configure(-state=>'normal');});

$bpth_ent->bind('<Key-Return>'=>sub {&change_basepath(\$basepath, $main, $chg_bpth);});
    

#TextArea
my $frame2 = $main->Frame()->pack(-side=>'top', -fill=>'both', -expand=>1);

my $Stxt = $frame2->Scrolled("Text", -scrollbars=>'e')->pack(-side=>'top', -fill=>'both', -expand=>1);
my $txtArea = $Stxt->Subwidget("text");
$txtArea->configure(-background=>'white', -foreground=>'black', width=>100, height=>30);

my $log_but = $frame2->Button(-text=>"Open Log File", -command=>sub
				  {
				      if ($start_vals->{'o_ent_val'})
				      {
					  my ($log_file) = &get_log_file($start_vals->{'o_ent_val'}, $main);
					  print "log_file is $log_file\n";
					  return unless ($log_file);

					  my $pid;
					  if ($pid = fork) 
					  {
					      #parent
					      $SIG{CHILD} = 'IGNORE';
					      return;
					  }
					  elsif (defined $pid) 
					  { 
					      #child
					      exec 'xterm', '-title', "$log_file", '-e', 'less', "$log_file";
					  }
				      }
				  })->pack(-fill=>'x', -padx=>5, -pady=>5);

#my $log_lbl = $frame2->Label(-text=>'Log File')->pack(-side=>'top', -fill=>'x', -expand=>1);

#my $Slog_txt = $frame2->Scrolled("Text", -scrollbars=>'e')->pack(-side=>'top', -fill=>'both', -expand=>1);
#my $log_txt = $Slog_txt->Subwidget("text");
#$log_txt->configure(-background=>'white', -foreground=>'black', -height=>10, -width=>100);

#Status bars 2 and 3
my $stat2;
my $stat3;
my $frame3 = $main->Frame()->pack(-side=>'top', -fill=>'x', -expand=>1);
my $status2 = $frame3->Label(-relief=>'sunken', -textvariable=>\$stat2, -background=>'grey', -foreground=>'black')->pack(-side=>'left', -fill=>'x', -expand=>1);
my $empty = $frame3->Label(-background=>'grey', -width=>10, -relief=>'sunken')->pack(-side=>'left', -fill=>'x', -expand=>1);
my $status3 = $frame3->Label(-relief=>'sunken', -textvariable=>\$stat3, -background=>'grey', -foreground=>'black')->pack(-side=>'left', -fill=>'x', -expand=>1);


#Progress bar

my $prog_bar = $empty->ProgressBar(-variable=>\$prog_var)->pack(-fill=>'both', -expand=>1);

#bindings for menu shortcuts
$main->bind("<Control-n>"=>\&new);
$main->bind("<Control-o>"=>\&open);
$main->bind("<Control-p>"=>\&pref);


#make .autostructure
my $HOME = $ENV{'HOME'};
unless (-d "$HOME/.asdp")
{
    mkdir "$HOME/.asdp";
}

$main->OnDestroy(sub
		 {
		     if (-d "$HOME/.asdp")
		     {
			 #print "removing .asdp\n";
			 foreach my $key (keys %tempFile)
			 {
			     print "removed temp file $tempFile{$key}" . "\n";
			     unlink $tempFile{$key}; 
			 }
		     }
		 });


MainLoop;




sub about
{
    my $box = $main->Dialog(-title=>'About', -text=>'Test Copy', bitmap=>'info', -buttons=>['OK'])->Show();
}

sub new
{
    &displayControl($main, '','new');
}

sub open
{
    my $file_sel = &OpenFile('Control', $main, $basepath);
   
    
    if (-e $file_sel)
    {
	print "Opening control file $file_sel\n";
	&displayControl($main, $file_sel,'open');
    }
}

sub pref
{
    my $cont_file = &OpenFile('Control', $main, $basepath);
    if (-f $cont_file)
    {
	&create_sparky_list($cont_file);
    }
}

sub pdb_dist_query
{
    my ($pdb_file) = &OpenFile('pdb', $main, $basepath);

    if (-f $pdb_file)
    {
	&distance_query($pdb_file, $main);
    }
}

sub start
{
    my ($rerun) = @_;
    
    if ($rerun)
    {
	my $del = &dir_exist($start_vals->{'o_ent_val'});
	print "rerun: del is $del\n";
	
	if ($del =~ /yes/i)
	{
	    print "removing dir ".$start_vals->{'o_ent_val'}."\n";
	    rmtree ($start_vals->{'o_ent_val'});
	}
	else
	{
	    return;
	}
    }
    else
    {
	$start_vals = &start_gui($main);
    }    
    
    unless ($start_vals->{'o_ent_val'})
    {
	return;
    }
    

    $prog_var = 0;

    #update status bar
    my $stat_text = "Control_File: ".$start_vals->{'c_ent_val'}.", Output directory: ".$start_vals->{'o_ent_val'};
    my $opt_txt;

    my $cont_file = $start_vals->{'c_ent_val'};
    my $cycles = &openContFile($cont_file, 0, 1);
    
    $cycles = $cycles+1;
    $prog_bar->configure(-to=>$cycles);
    

    while ((my $key, my $value) = each (%$start_vals))
    {

	unless ($key =~ /^[co]/ or $key =~ /ent/)
	{
	    if ($value==1)
	    {
		$key=~ /^(\w)_cbt_val/;
		my $opt = $1;
		$opt_txt = $opt_txt." -$opt,";
	    }
	}
    }

    if ($opt_txt)
    {
	$opt_txt =~ s/(\w+),$/$1/;
	$stat_text = $stat_text.", Options: ".$opt_txt;
    }
    
    $stat1 = $stat_text;
    #$log_txt->insert('end', 'In start');
    
    $auto->cget(-menu)->entryconfigure('Open Recent-Calc AS OutputDir', -state=>'normal');

    my $log_txt;
    #call sub that forks and runs script
    unless(&start_child($start_vals, $txtArea, $calc, $main, \$stat2, $log_txt, \$prog_cont, \$prog_var))
    {
	return;
    }
    
    $stat2 = "Calculation is running...";
    my $out_dir = $start_vals->{'o_ent_val'};

    $prog_var++;
    
    $prog_cont = 1;
    my $id = $main->after(3000, sub {&load_prog_bar(1);});
}

sub rec_calc
{
    print "outdir is ".$start_vals->{'o_ent_val'}."\n";
    if ($start_vals->{'o_ent_val'})
    {
	&OpenASOutDir($main, $start_vals->{'o_ent_val'});
    }
}

sub open_as_dir
{
    my $dir = &OpenDir('AS Output Directory', $main, $basepath);
    if (-d $dir)
    {
	$dir = File::Spec->rel2abs($dir);
	&OpenASOutDir($main, $dir);
    }
}

sub open_dir
{
    my $dir = &OpenDir('Directory', $main, $basepath);
    if (-d $dir)
    {
	$dir = File::Spec->rel2abs($dir, $basepath);
	
	my $win = $main->Toplevel(-title=>'Open Directory');
	$win->minsize(qw/500 700/);
	
	my $lbl_frm = $win->Frame(-relief=>'ridge', -borderwidth=>2)->pack(-fill=>'x', -pady=>2);
	
	my $dir_lbl = $lbl_frm->Label(-text=>"Directory: $dir")->pack(-padx=>2, -pady=>2, -side=>'top');
	
	my $nb_frm = $win->Frame()->pack(-fill=>'both', -expand=>1, -side=>'top');
	
	my $nb = $nb_frm->NoteBook()->pack(-fill=>'both', -expand=>1, -padx=>2, -pady=>2);
	my $view_pg = $nb->add('view', -label=>'Tree View');
	my $tree;
	
	&create_tree($view_pg, $win, $dir, $tree);

	my $cls_but = $win->Button(-text=>'Close', -command=>sub {$win->destroy();})->pack(-fill=>'x', -expand=>1, -padx=>3, -pady=>3, -side=>'left');
	$win->waitWindow();
	
    }
}



sub err_open
{
    my ($msg) = @_;

    my $err_dlg = $main->Dialog(-text=>$msg, -title=>'Error', -bitmap=>'error', -buttons=>['OK']);
    $err_dlg->Show();
    
    
}
#end sub err_open


sub dir_exist
{
    my $dir = $start_vals->{'o_ent_val'};
    print "dir is $dir\n";

    my $warn = "The directory $dir already exists!\nClicking YES will erase the contents of $dir!!\nAre you sure you want to overwrite?";

    my $dlg = $main->Dialog(-text=>$warn, -font=>[-size=>11, -weight=>'bold'], -buttons=>['Yes', 'No'], -bitmap=>'warning', -background=>'cyan');
    my $ans = $dlg->Show();


    print "You clicked $ans\n";
    return $ans;
}

sub load_prog_bar
{
    my ($cyc_dirs) = @_;
    my $cycles;
    
    
    if (opendir(ODIR, $start_vals->{'o_ent_val'}))
    {
	$cycles = grep /^CYCLE.+_final/i ,readdir(ODIR);

	$cycles++;
	
	my $diff = ($cycles) - ($cyc_dirs);
	$prog_var = $prog_var + $diff;
    }

    if ($prog_cont)
    {
	my $id = $main->after(8000, sub {&load_prog_bar($cycles)});
    }
    else
    {
	$prog_var = 0;
	return;
    }
    
}


sub open_rpf
{
    my ($rpf_dir) = &OpenDir('rpf', $main, $basepath);
    $rpf_dir = File::Spec->rel2abs($rpf_dir, $basepath);
    
    unless (opendir(DIR, $rpf_dir))
    {
	print "Cannot open $rpf_dir\n";
	return;
    }

    my @contents = readdir(DIR);
    close(DIR);

    my (@qcyc) = grep {/^Q.+final$/i} @contents;
    
    if ($#qcyc > 0)
    {
	&open_rpf_dir($rpf_dir);
    }
    else
    {
	my ($ovw_file) = grep {/_DP\.ovw$/} @contents;
	print "ovw_file is $ovw_file\n";
	if ($ovw_file)
	{
	    $ovw_file = File::Spec->rel2abs($ovw_file, $rpf_dir);
	    
	    &open_rpf_file($ovw_file, $rpf_dir);
	}
    }
    
}

sub open_rpf_file
{

    my ($rpf_file, $rpf_dir) = @_;
  
    unless (-e $rpf_file)
    {
	return;
    }

    my $rpf_win = $main->Toplevel(-title=>'Open RPF File');
    
    my $full = $rpf_win->Frame(-background=>'#78A1BF')->pack(-fill=>'both', -expand=>1, -padx=>1, -pady=>1);

    my $lbl = $full->Label(-text=>"File - $rpf_file", -background=>'white', -relief=>'ridge', -borderwidth=>2)->pack(-padx=>1, -pady=>1, -ipadx=>3, -ipady=>3);
    my $disp = $full->Frame(-background=>'#78A1BF')->pack(-fill=>'both', -expand=>1, -padx=>2, -pady=>2);

    my ($data, $pklsts_fnames) = &parse_ovw($rpf_file, $rpf_win);

    my ($fpd_but, $ql_but, $qi_but, $nb) = &rpf_scores_frame($disp, $data, $pklsts_fnames, $rpf_win);
    my ($pdb_fname, $ql_fname) = &get_fpd_fnames($rpf_dir, $rpf_file, $rpf_win);
    print "pdb_file is $pdb_fname, ql_fname is $ql_fname\n";

    my $key;
    
    foreach $key (keys %$fpd_but)
    {
	my $temp_key = $key;
	$fpd_but->{$temp_key}->configure(-command=>sub
					 {
					     
					     if ($pdb_fname && $ql_fname)
					     {
						 $pdb_fname = File::Spec->rel2abs($pdb_fname, $rpf_dir);
						 $ql_fname = File::Spec->rel2abs($ql_fname, $rpf_dir);
						 print "open_rpf_file: ql_fname is $ql_fname, pdb_fname is $pdb_fname\n";
						 
						 &calc_rasmol($rpf_win, $pdb_fname, $ql_fname, $temp_key);
					     }
					     else
					     {
						 unless ($pdb_fname)
						 {
						     &err_msg("No pdb file in $rpf_dir", $rpf_win);
						     return 0;
						 }
						 
						 unless ($ql_fname)
						 {
						     &err_msg("No ql file in $rpf_dir", $rpf_win);
						     return 0;
						 }
					     }
					 });
    }

    $ql_but->configure(-command=>sub
		       {
			   #my ($pdb_fname, $ql_fname) = &get_fpd_fnames($rpf_dir, $rpf_file, $rpf_win);
			   if ($ql_fname && $pdb_fname)
			   {
			       $ql_fname = File::Spec->rel2abs($ql_fname, $rpf_dir);
			       &show_interactions($rpf_win, $ql_fname);
			   }
			   else
			   {
			       unless ($pdb_fname)
			       {
				   &err_msg("No pdb file in $rpf_dir", $rpf_win);
				   return 0;
			       }

			       unless ($ql_fname)
			       {
				   &err_msg("No ql file in $rpf_dir", $rpf_win);
				   return 0;
			       }
			   }
			   
		       });
    $qi_but->configure(-command=>sub
		       {
			   &display_qi($rpf_dir, $rpf_win);
		       });

    my $tree;
    my $tre_pg = $nb->add('tree', -label=>'Tree View');

    $nb->pageconfigure('tree', -createcmd=>sub
		       {
			   &create_tree($tre_pg, $rpf_win, $rpf_dir, $tree);
		       });
    
    
    my $but_frm = $full->Frame(-background=>'#78A1Bf')->pack(-side=>'bottom', -fill=>'x', -expand=>1);

    my $sav_but = $but_frm->Button(-text=>"Save", -command=>sub 
				   {
				       my $rpf_file = &SaveFile('rpf', $rpf_win, $basepath);
				       if ($rpf_file)
				       {
					   $rpf_file = File::Spec->rel2abs($rpf_file, $basepath);
					   print "open_rpf_file: save as $rpf_file\n";
					   
					   &write_rpf_data($rpf_file, $data);
				       }
				   }, -width=>20)->pack(-pady=>3, -side=>'left', -fill=>'x', -expand=>1);


    my $cls_but = $but_frm->Button(-text=>"Close", -command=>sub {$rpf_win->destroy;}, -width=>20)->pack(-pady=>3, -side=>'left', -fill=>'x', -expand=>1);
}


sub open_rpf_dir
{
    #get parent directory
    my ($dir) = @_;

    my $bg = '#DFF1FF';
    my $bg1 = '#B1D3EC';

    unless (opendir(PD, $dir))
    {
	return;
    }


    #get all cycle directories from the parent directory
    my @unsorted = grep {/^Q.+final$/i} readdir(PD);
    my @qdirs;
    my $m;

    foreach $m (@unsorted)
    {
	$m =~ /QCYCLE(\d+)-/;
	$qdirs[$1] = $m;
    }

    
    close(PD);


    unless ($#unsorted > -1)
    {
	return;
    }

    my $win = $main->Toplevel(-title=>'RPF Scores');
    $win->configure(-background=>$bg);
    
    #label with name of parent directory
    my $lbl_frm = $win->Frame(-background=>$bg)->pack();
    my $dir_lbl = $lbl_frm->Label(-text=>"RPF Parent Directory: $dir", -background=>$bg)->pack(-pady=>3);

    #notebook with two pages - Cycle view and Tree view
    my $big_nb_frm = $win->Frame(-background=>$bg)->pack(-fill=>'both', -expand=>1);
    my $nb = $big_nb_frm->NoteBook(-background=>$bg1, -backpagecolor=>$bg, -inactivebackground=>'#6096BF')->pack(-fill=>'both', -expand=>1);
    my $cyc_pg = $nb->add('cycle', -label=>'Cycle View');
    my $tre_pg = $nb->add('tree', -label=>'Tree View');

    $cyc_pg->configure(-background=>$bg1);
    $tre_pg->configure(-background=>$bg1);

    #Tree View of dir
    my $tree;
    &create_tree($tre_pg, $win, $dir, $tree);



    #Cycle view notebook
    my $nb_frm = $cyc_pg->Frame(-background=>$bg)->pack(-fill=>'both', -expand=>1);
    my $rpf_nb = $nb_frm->NoteBook(-background=>'#78A1BF', -backpagecolor=>$bg1, -inactivebackground=>'#ACB7BF')->pack(-fill=>'both', -expand=>1);
    my $main = $rpf_nb->add('Main', -label=>'Main');

    my $qdir;
    my $cycle_num = 1;
    my %graph_data;


    #loop through array of cycle directories, and get and display the ovw files
    for ($m = 0; $m<=$#qdirs; $m++)
    {
	next unless ($qdirs[$m]);

	$qdir = $qdirs[$m];
    	$qdir = File::Spec->rel2abs($qdir, $dir);
	
	#get the ovw file from the directory
	unless (opendir(QD, $qdir))
	{
	    next;
	}
	my @contents = readdir(QD);

	my ($ovw_file) = grep {/\.ovw$/} @contents;
	$ovw_file = File::Spec->rel2abs($ovw_file, $qdir);

	my ($notes_file) = grep {/\.note$/} @contents;
	$notes_file = File::Spec->rel2abs($notes_file, $qdir);
	my @notes;
	
	if (open(NOTE, $notes_file))
	{
	    @notes = <NOTE>;
	}
	
	close(NOTE);
	close(QD);

	unless (-e $ovw_file)
	{
	    next;
	}

	my ($data, $pklsts_fnames) = &parse_ovw($ovw_file, $win);

	#add page to notebook for this ovw file
	my $rpf_pg = $rpf_nb->add("Cycle $cycle_num", -label=>"Cycle $cycle_num");
	$rpf_pg->configure(-background=>'#78A1BF');

	#label with name of cycle dir
	my $lbl = $rpf_pg->Label(-text=>$qdir, -background=>'#78A1BF')->pack();
	my $rpf_pg_frm = $rpf_pg->Frame(-background=>'#78A1BF')->pack(-fill=>'both', -expand=>1);

	my ($fpd_but, $ql_but, $qi_but, $nb, $txt) = &rpf_scores_frame($rpf_pg_frm, $data, $pklsts_fnames, $win);
	my $temp_num = $cycle_num;
	my $temp_dir = $qdir;
	my $key;

	foreach $key (keys %$fpd_but)
	{
	    my $temp_key = $key;
	    $fpd_but->{$temp_key}->configure(-command=>sub
					     {
						 my ($pdb_fname, $ql_fname) = &get_fpd_fnames($temp_dir, $ovw_file, $win);
						 if ($pdb_fname)
						 {
						     $pdb_fname = File::Spec->rel2abs($pdb_fname, $temp_dir);
						     $ql_fname = File::Spec->rel2abs($ql_fname, $temp_dir);
						     
						     &calc_rasmol($win, $pdb_fname, $ql_fname, $temp_key);
						 }
					     });
	}
	
	$ql_but->configure(-command=>sub 
			   {
			       my ($pdb_fname, $ql_fname) = &get_fpd_fnames($temp_dir, $ovw_file, $win);
			       if ($ql_fname)
			       {
				   $ql_fname = File::Spec->rel2abs($ql_fname, $temp_dir);
				   &show_interactions($win, $ql_fname);
			       }
			   });
	$qi_but->configure(-command=>sub
			   {
			       &display_qi($qdir, $win);
			   });

	$txt->insert('end', @notes);

	#$graph_data{'Precision-score'}{$cycle_num} = ${$data->{'final'}}{'Precision-score'}{'input query structures'};
	#$graph_data{'Recall-score'}{$cycle_num} = ${$data->{'final'}}{'Recall-score'}{'input query structures'};
	#$graph_data{'F-score'}{$cycle_num} = ${$data->{'final'}}{'F-score'}{'input query structures'};
	#$graph_data{'dp-score'}{$cycle_num} = ${$data->{'final'}}{'dp-score'};
	$graph_data{'Precision-score'}{$cycle_num} = $data->{'model'}{'ave'}{'Precision-score'}{'input query structures'};
	$graph_data{'Recall-score'}{$cycle_num} = $data->{'model'}{'ave'}{'Recall-score'}{'input query structures'};
	$graph_data{'F-score'}{$cycle_num} = $data->{'model'}{'ave'}{'F-score'}{'input query structures'};
	$graph_data{'dp-score'}{$cycle_num} = $data->{'model'}{'ave'}{'dp-score'}{'input query structures'};

	$cycle_num++;
    }
    $cycle_num--;
    
    my $graph_frm = $win->Frame(-background=>$bg)->pack(-fill=>'both', -expand=>1);
    &create_graph($graph_frm, \%graph_data, $cycle_num, $win, $basepath);

    $win->update();
    
    $rpf_nb->delete('Main');

}

