sub openContFile
{
    #parses the Control File and returns the data
    my ($file, $prot, $cycles, $nhs, $jfile, $ec) = @_;
    
    unless (-e $file)
    {
	&err_open("File does not exist");
	return;
    }

    my $opened = open (CFILE, $file);
    unless ($opened)
    {
	&err_open("File cannot be opened");
    }

    my $dir = dirname($file);

    #print "Control File is $file, and directory is $dir\n";


    my $line;
    my @comments;
    my $heading;
    my %general;
    my %commands;
    my %peaklists;
    my $pklst_count = 0;
    my %col_names;
    my %extra_ops;

    while ($line = <CFILE>)
    {
	chomp($line);
	#if the line is a comment
	if ($line =~ /^\#/)
	{
	    $line =~ s/^\#//;
	    push (@comments, $line);
	    next;
	}
	next if ($line =~ /^(\s)*$/);#blank lines
	

	#check if line is a heading - General, Command or Peaklist
	if ($line =~ /^\[([\~\w\-\_\.\/]+)\]$/)
	{
	    $heading = $1;
	    unless ($heading =~ /general/i or $heading =~ /command/i)
	    {
		$pklst_count++;
		
	    }
	}
	else
	{
	    if ($heading =~ /general/i)
	    {
		#general section
		my ($command, $value) = split(/=/,$line);
		$general{$command} = $value;
	
		#return protein name from control file
		if ($prot and ($command eq 'proteinName'))
		{
		    close(CFILE);
		    return $value;
		}
		
		#return cycles from control file
		elsif ($cycles and ($command eq 'nCycles'))
		{
		    close(CFILE);
		    return $value;
		} 
		
		#return nhslowlist from control file
		elsif ($nhs and ($command eq 'NHSlowList'))
		{
		    close(CFILE);
		    return $value;
		
		} 

		#return JListFile from control file
		elsif ($jfile and ($command eq 'JListFile'))
		{
		    close(CFILE);
		    return $value;
		} 
		#return EC file control file 
		elsif ($ec and ($command eq 'EC'))
		{
		    close (CFILE); 
		    return $value; 
		} 
	    }
	    elsif ($heading =~ /command/i)
	    {
		$commands{$heading} = $line;
	    }
	    else  #peaklist
	    {
		
		#peaklist options
		my ($option, $value) = split(/=/,$line);
		
		if ($option =~ /col/i)
		{
		    my ($col, $name) = split(/\./, $option);
		    $col_names{$heading}{$value} = $name;
		}
		else
		{
		    $peaklists{$heading}{$option} = $value;
		}
	    }
	}
	#end if reading in commands
    }
    #end while reading in file
    
    $pklst_count--;
    
    close(CFILE);

    return '' if ($ec || $jfile || $nhs || $cycles || $prot); 
    return (\@comments, \%general, \%commands, \%peaklists, \%col_names, $pklst_count);
    
}
return 1;
