# this parses the file for k calibration purposes

sub parse_assOrder_k
{
    my ($file, $parent, $sec_data_ref, $sec_str_ref, $data_ref, $count_ref) = @_;

    unless (open(FILE, $file))
    {
	print "Cannot open file \"$file\"\n";
	return;
    }

    #print "assOrder file is $file\n";

    my %data = %$data_ref;  # this hash contains the intensities in the format $data{<key>}[#] = intensity, key = d(atom1 - atom2,sec_str)
    my %count = %$count_ref;
    my $line;
    my ($key1, $key2);

    my %sec_str = %$sec_str_ref;
    
    while (chomp($line = <FILE>))
    {
	if ($line =~ /^peak/i)
	{
	    my $atom1_line = <FILE>;
	    my $atom2_line = <FILE>;

	    $atom1_line =~ /\s(\d+)\w\-(\S+)\s/;
	    my ($res1, $atm1) = ($1, $2);

	    $atom2_line =~ /\s(\d+)\w\-(\S+)\s/;
	    my ($res2, $atm2) = ($1, $2);
	    
	    next unless ($atm1 =~ /^(H|HA\d*)$/);
	    next unless ($atm2 =~ /^(H|HA\d*)$/);

	    $line =~ /\s(\S+)\s\(label\)/;
	    my $intensity = $1;

	    #both the residues should belong to either of helix, para or anti
	    my ($temp_sheet1, $temp_sheet2) = (&get_sheet_k($sec_data_ref, $res1, 'Final'), &get_sheet_k($sec_data_ref, $res2, 'Final'));

	    if ($sec_str{'Helix'}{$temp_sheet1} and $sec_str{'Helix'}{$temp_sheet2})
	    {
		#both belong to helix
		$key2 = 'Alpha_Helix';
	    }
	    elsif ($sec_str{'Anti'}{$temp_sheet1} and $sec_str{'Anti'}{$temp_sheet2})
	    {
		#both belong to Antiparallel betas
		$key2 = 'Anti_Beta';
	    }
	    elsif ($sec_str{'Para'}{$temp_sheet1} and $sec_str{'Para'}{$temp_sheet2})
	    {
		#both belongs to Parallel betas
		$key2 = 'Para_Beta';
	    }
	    else
	    {
		next;
	    }
	    
	    #print "$res1-$atm1 $res2-$atm2 $intensity, $key2\n";

	    #skip if at the end of the secondary structure
	    my ($start, $end) = &get_res($temp_sheet1);
	    next if (($res1 == $start) or ($res1 == $end));
	    ($start, $end) = &get_res($temp_sheet2);
	    next if (($res2 == $start) or ($res2 == $end));

	    
	    ## check if far from diagonal ####### incomplete ###########
	    my %coords;
	    if ($line =~ /\(hx1\)([.\d]+)\s/)
	    {
		$coords{'hx1'} = $1;
	    }
	    if ($line =~ /\(x1\)([.\d]+)\s/)
	    {
		$coords{'x1'} = $1;
	    }
	    if ($line =~ /\(hx2\)([.\d]+)\s/)
	    {
		$coords{'hx2'} = $1;
	    }
	    if ($line =~ /\(x2\)([.\d]+)\s/)
	    {
		$coords{'x2'} = $1;
	    }
	    
	    # cs coords are in the hash %coords - need to check if far from diagonal ###
	    


	    #check diff between residues
	    if ((abs($res1 - $res2)) == 1)
	    {
		if ($res1 < $res2)
		{
		    if (($atm1 =~ /^HA/) and ($atm2 =~ /^H$/))
		    {
			$key1 = "d(HAi - HNi+1),$key2";
			$data{$key1}[$count{$key1}] = $intensity;
			$count{$key1}++;
		    }
		    elsif (($atm1 =~ /^H$/) and ($atm2 =~ /^H$/) and ($key2 eq 'Alpha_Helix'))
		    {
			$key1 = "d(HNi - HNi+1),Alpha_Helix";
			$data{$key1}[$count{$key1}] = $intensity;
			$count{$key1}++;
		    }
		}
		else
		{
		    if (($atm2 =~ /^HA/) and ($atm1 =~ /^H$/))
		    {
			$key1 = "d(HAi - HNi+1),$key2";
			$data{$key1}[$count{$key1}] = $intensity;
			$count{$key1}++;
		    }
		    elsif (($atm1 =~ /^H$/) and ($atm2 =~ /^H$/) and ($key2 eq 'Alpha_Helix'))
		    {
			$key1 = "d(HNi - HNi+1),Alpha_Helix";
			$data{$key1}[$count{$key1}] = $intensity;
			$count{$key1}++;
		    }
		}
	    }
	    elsif (abs($res1 - $res2) == 2)
	    {
		if ($res1 < $res2)
		{
		    if (($atm1 =~ /^HA/) and ($atm2 =~ /^H$/) and ($key2 eq 'Alpha_Helix'))
		    {
			$key1 = "d(HAi - HNi+2),$key2";
			$data{$key1}[$count{$key1}] = $intensity;
			$count{$key1}++;
		    }
		    elsif (($atm1 =~ /^H$/) and ($atm2 =~ /^H$/) and ($key2 eq 'Alpha_Helix'))
		    {
			$key1 = "d(HNi - HNi+2),Alpha_Helix";
			$data{$key1}[$count{$key1}] = $intensity;
			$count{$key1}++;
		    }
		}
		else
		{
		    if (($atm2 =~ /^HA/) and ($atm1 =~ /^H$/) and ($key2 eq 'Alpha_Helix'))
		    {
			$key1 = "d(HAi - HNi+2),$key2";
			$data{$key1}[$count{$key1}] = $intensity;
			$count{$key1}++;
		    }
		    elsif (($atm1 =~ /^H$/) and ($atm2 =~ /^H$/) and ($key2 eq 'Alpha_Helix'))
		    {
			$key1 = "d(HNi - HNi+2),Alpha_Helix";
			$data{$key1}[$count{$key1}] = $intensity;
			$count{$key1}++;
		    }
		}
	    }
	    elsif (abs($res1 - $res2) == 3)
	    {
		
		if ($res1 < $res2)
		{
		    if (($atm1 =~ /^HA/) and ($atm2 =~ /^H$/) and ($key2 eq 'Alpha_Helix'))
		    {
			$key1 = "d(HAi - HNi+3),$key2";
			$data{$key1}[$count{$key1}] = $intensity;
			$count{$key1}++;
		    }
		}
		else
		{
		    if (($atm2 =~ /^HA/) and ($atm1 =~ /^H$/) and ($key2 eq 'Alpha_Helix'))
		    {
			$key1 = "d(HAi - HNi+3),$key2";
			$data{$key1}[$count{$key1}] = $intensity;
			$count{$key1}++;
		    }
		}
	    }
	    else
	    {
		next;
	    }
	}
    }

    
    if(0){
    print "Contents of data from file $file are...\n";
    my ($key, $i);
    foreach $key (keys %data)
    {
	for ($i = 0; $i<scalar(@{$data{$key}}); $i++)
	{
	    print "data{$key}[$i] = $data{$key}[$i]\n";
	}
	print "\n";
    }
}

    return (\%data, \%count);

}


sub get_sheet_k
{
    my ($data_ref, $res, $key) = @_;
    my ($sheet);

    unless ($key)
    {
	$key = 'Temp_ids';
    }

    my $id;
    
    my %data = %$data_ref;
    my $count = scalar(@{$data{$key}});
    
    #get sheet of res1
    for ($id = 0; $id < $count; $id++)
    {
	my $line = $data{$key}[$id];
	$line =~ /(\w+)\s+(\d+)\-(\d+)$/;
	
	my $type = $1;
	my ($start, $end) = ($2, $3);
	if ((($start-1) <= $res) and (($end+1) >= $res))
	{
	    $sheet = "$start-$end";
	}
    }
    
    #print "$res is from $sheet\n";
    return ($sheet);
}

return 1;
