sub parse_ovw
{
    my ($file, $parent) = @_;
    my @pklsts_fnames;
    my $pkl_name;
    my %data;
    my $summary = 0;
    my $query;
    my $precision;
    my $summary_line = 0;
    
    unless (-e $file)
    {
	#print "$file doesnt exist!!\n";
	return;
    }

    unless (open(FILE, $file))
    {
	&err_msg("Cannot open rpf ovw file \"$file\"", $parent);
	return;
    }

    my $line;
    
    #start reading input
    while (chomp($line = <FILE>))
    {
	if ($line =~ /NOESY_Assign Version ([\d\.]+) /)
	{
	    my $version = $1; 
	    if ($version =~ /2\.1\./)
	    {
		&err_msg("This is a $version version output. Please use an older version to open it", $parent); 
	    }
	}
	
	last if ($line =~ /^(\s*)final/i);
	if ($line =~ /^\#/)
	{
	    $query = 'query';
	    if ($line =~ /summary/i)
	    {
		$line =~ /summary for (.+)/i;
		$pkl_name = $1;

		if ($pkl_name=~ /:$/)
		{$pkl_name =~s/:$//;}
		push @pklsts_fnames, $pkl_name;
		$summary = 1;
		$summary_line = 0;
	    }
	    elsif ($line =~ /for peaklist (.+)/i)
	    {
		$pkl_name = $1;
		if ($pkl_name =~ /:$/)
		{$pkl_name=~s/:$//;}

		$summary = 0;
	    }
	}
	else
	{
	    #line is data
	    if ($summary)
	    {
		if ($line =~ /m score/i)
		{
		    my ($m, $score) = split(/=/, $line);
		    $data{$pkl_name}{'M-score'} = $score;
		}
		elsif ($line =~ /average/i)
		{
		    $data{$pkl_name}{'summary'}[$summary_line] = $line;
		    $summary_line++;
		}
	    }
	    else
	    {
		#data for each peaklist
		
		if ($line =~ /freely-rotating-chain/i)
		{
		    $query = 'freely';
		    $precision = 0;
		    next;
		}
		
		if ($line =~ /^(\s*)recall/i)
		{
		    my ($recall, $num) = split(/:/, $line);
		    ($num, my $other) = split(' ', $num);
		    $data{$pkl_name}{$query}{'recall'} = $num;
		    $precision = 0;
		    next;
		}
		elsif ($line =~ /^(\s*)precision/i)
		{
		    $precision = 1;
		    next;
		}
		elsif ($line =~ /^(\s*)f-measure/i)
		{
		    $precision = 0;
		    
		    my ($fmeasure, $score) = split(' ', $line);
		    $data{$pkl_name}{$query}{'F-measure'}{$fmeasure} = $score;
			
		}
		elsif ($line =~ /^\s*$/)
		{
		    $precision = 0;
		    next;
		}
		
		
		if ($precision == 1)
		{
		    $line =~ /\s*(.+):\s*(.+)\s+(.+):\s*(.+)$/;
		    
		    $data{$pkl_name}{$query}{'precision'}{$1} = $2;
		    $data{$pkl_name}{$query}{'precision'}{$3} = $4;
		}
	
		
	    }
	    
	}
    }

    #final reports 
    my (@vars) = split(' ', $line);
    
    $data{'final'}{'m-score'} = $vars[2];
    
while (chomp ($line = <FILE>))
{
    next if ($line =~ /^\s*$/); 
    $line =~ /Model ([\d\w]+)/; 

    my $model = $1; 
 
    while (chomp($line = <FILE>))
    {
	next if ($line =~ /^\s*$/);
	last if ($line =~ /summary/i);

	$line =~ /final (.+) for (.+): (.+)$/i;
	$data{model}{$model}{$1}{$2} = $3; 
    }

    #summary F-scores
    while (chomp($line = <FILE>))
    {
	next if ($line =~ /^\s*$/);
	last if ($line =~ /dp/i);
	$line =~ /F-score of (.+):\s*(.+)$/i;
	$data{model}{$model}{'F-score'}{$1} = $2;
    }

    my ($dp, $score) = split(/:/, $line);
    #$data{'final'}{'dp-score'} = $score;
    $data{model}{$model}{'dp-score'}{'input query structures'} = $score;
}    
    return (\%data, \@pklsts_fnames);
    
}
return 1;
