sub parse_pdb
{
  #given a pdb file, return %data with the pdb data in it, where the keys are the record types.

  my ($pdb_file, $parent) = @_;
  print "parse_pdb: parsing file $pdb_file\n";

  unless (-f $pdb_file)
  {
      print "No pdb file specified\n";
      return;
  }

  unless (open(FILE, $pdb_file))
  {
      &err_msg("Cannot open pdb file \"$pdb_file\"", $parent);
      return;
  }

  my ($line);
  my (%data, %count);
  my $key;

  foreach $key (qw/TITLE MODEL/)
  {
      $count{$key} = 0;
  }

  my $model_seen = 0;

  while (($line = <FILE>))
  {
      chomp $line;
      if ($line =~ /^HEADER/i)
      {
	  $line =~ /^HEADER\s+(.+)$/;
	  $data{'HEADER'} = $1;
      }
      elsif ($line =~ /^TITLE/i)
      {
	  $line =~ /^TITLE\s+(.+)$/;
	  $data{'TITLE'}[$count{'TITLE'}] = $1;
	  $count{'TITLE'} = $count{'TITLE'} + 1;
      }
      elsif ($line =~ /^SEQRES/i)
      {
	  #character 12 in $line is the chain identifier
	  my $chain = substr($line, 11, 1);
	  unless ($chain)
	  {
	      $chain = '0';
	  }
	  
	  unless ($count{'SEQRES'}{$chain})
	  {
	      $count{'SEQRES'}{$chain} = 1;
	  }
	  
	  
	  #data{SEQRES}{<chain>}[<residue number>] = name of residue, where residue number = count{SEQRES}{<chain>}
	  #split the line to get the array of residues
	  my ($res_line) = substr($line, 19, (length($line) - 19));

	  my (@residues) = split(/\s+/, $res_line);
	  my $res_num;
	  
	  for ($res_num = 0; $res_num<=$#residues; $res_num++)
	  {
	      $data{'SEQRES'}{$chain}[$count{'SEQRES'}{$chain}] = $residues[$res_num];
	      $count{'SEQRES'}{$chain} = $count{'SEQRES'}{$chain} + 1;
	  }
	  

      }
      elsif ($line =~ /^HELIX/)
      {
	  unless ($count{'HELIX'})
	  {
	      $count{'HELIX'} = 0;
	  }
	  
	  $data{'HELIX'}[$count{'HELIX'}] = $line;
	  $count{'HELIX'} = $count{'HELIX'} + 1;
	  
	  
      }
      elsif ($line =~ /^SHEET/)
      {
	  unless ($count{'SHEET'})
	  {
	      $count{'SHEET'} = 0;
	  }
	  
	  $data{'SHEET'}[$count{'SHEET'}] = $line;
	  $count{'SHEET'} = $count{'SHEET'} + 1;
	  

      }
      elsif ($line =~ /^TURN/)
      {
	  unless ($count{'TURN'})
	  {
	      $count{'TURN'} = 0;
	  }
	  
	  $data{'TURN'}[$count{'TURN'}] = $line;
	  $count{'TURN'} = $count{'TURN'} + 1;
	  
      }
      elsif ($line =~ /^MODEL/)
      {
	  if ($model_seen)
	  {
	      $count{'MODEL'} = $count{'MODEL'} + 1;
	  }
	  else
	  {
	      $count{'MODEL'} = 0;
	  }

	  $model_seen = 1;
	  
      }
      elsif ($line =~ /^ATOM/)
      {
	  #atom data stored - $data{$atom}{<model>}[<serial number>]{<name>} - 13-16
	  #                                                          {<residue name>} - 18-20
	  #                                                          {<chain>} - 22
	  #                                                          {<residue sequence number>} - 23-26
	  #                                                          {x coordinate} - 31-38
	  #                                                          {y coordinate} - 39-46
	  #                                                          {z coordinate} - 47-54

	  #my $number = &get_info($line, 6, 5);

	  my $res_num = &get_info($line, 22, 4);
	  my $res_name = &get_info($line, 17, 3);
	  my $res_atm = &conv_atom(&get_info($line, 12, 4), $res_name);
	  
	  my $chain = &get_info($line, 21, 1);

	  #print "$res_num$res_name-$res_atm\n";
	  
	  $data{'ATOM'}[$count{'MODEL'}]{"$res_num$res_name"}{$res_atm}{'x'} = &get_info($line, 30, 8);
	  $data{'ATOM'}[$count{'MODEL'}]{"$res_num$res_name"}{$res_atm}{'y'} = &get_info($line, 38, 8);
	  $data{'ATOM'}[$count{'MODEL'}]{"$res_num$res_name"}{$res_atm}{'z'} = &get_info($line, 46, 8);
	  $data{'RESIDUE'}[$res_num] = $res_name;

	  #print "$res_num $res_name-$res_atm - ".$data{'ATOM'}[$count{'MODEL'}]{"$res_num$res_name"}{$res_atm}{'x'}.",".$data{'ATOM'}[$count{'MODEL'}]{"$res_num$res_name"}{$res_atm}{'y'}.",".$data{'ATOM'}[$count{'MODEL'}]{"$res_num$res_name"}{$res_atm}{'z'}."\n";
	  #print "data{'ATOM'}[$count{'MODEL'}]{\"$res_num$res_name\"}{$res_atm}{'x'} = $data{'ATOM'}[$count{'MODEL'}]{\"$res_num$res_name\"}{$res_atm}{'x'}\n";
	  
	  if (0)
	  {
	      $data{'ATOM'}{$count{'MODEL'}}[$number]{'name'} = &get_info($line, 12, 4);
	      $data{'ATOM'}{$count{'MODEL'}}[$number]{'resName'} = &get_info($line, 17, 3);
	      $data{'ATOM'}{$count{'MODEL'}}[$number]{'chainID'} = &get_info($line, 21, 1);
	      $data{'ATOM'}{$count{'MODEL'}}[$number]{'resSeq'} = &get_info($line, 22, 4);
	      $data{'ATOM'}{$count{'MODEL'}}[$number]{'x'} = &get_info($line, 30, 8);
	      $data{'ATOM'}{$count{'MODEL'}}[$number]{'y'} = &get_info($line, 38, 8);
	      $data{'ATOM'}{$count{'MODEL'}}[$number]{'z'} = &get_info($line, 46, 8);
	  }
	  
      }
  }
  
  
  
  return (\%data, \%count);
  
}

sub conv_atom
{
    my ($atom, $res_name) = @_;
    my $new_atom = $atom;

    
    if ($atom =~ /^(\d)(.+)$/)
    {
	#if ($res_name eq 'GLY')
	#{
	#    my $num = $1;
	#    $num++;
	#    $new_atom = $2.$num;
	#}
	#else
	{
	    $new_atom = $2.$1;
	}
    }
    elsif ($atom eq 'HN')
    {
	$new_atom = 'H';
    }
    
    return ($new_atom);
    
}


sub get_info
{
    my ($line, $start, $length) = @_;

    my $var = substr($line, $start, $length);

#print $line . "-" . $var . "\n";
    $var =~ s/^\s+//;
    $var =~ s/\s+$//;
#print $var . "\n";
    return ($var);
}





return 1;
