 sub parse_ql
{
    my ($file, $parent) = @_;
 
    #two dimensional array with each element as a hash having three keys - distance, residue number, and atom name
    
    my @data;
    
    #my $data_count = 0;
    my $first_line_seen;
    my $pair_seen;
    my $section = -1;
    my @section_heading;
    my @atom;
    my $count = 0;

    my $other_first_line;

    my ($first, $first_name, $second, $second_name, $distance);
    my ($i, $j, $option);

    unless (-e $file)
    {
	print "No QL file specified\n";
	return;
    }

    my $opened = open(QL, $file);

    unless ($opened)
    {
	&err_msg("Cannot open QL file \"$file\"", $parent);
	return;
    }
    
    
    #read until first line
    while (chomp($line = <QL>))
    {
	if ($line =~ /missing/i)
	{
	    $line =~ /missing .+ for (.+)$/i;
	    
	    $section++;
	    $section_heading[$section] = $1;

	    #$data_count = 0;
	    $first_line_seen = 0;
	    $other_first_line = 1;
	    $pair_seen = 0;
	}
	elsif ($line =~ /^atom/i)
	{
	    #only look at first pair
	    if (!$pair_seen)
	    {
		if (!$first_line_seen)
		{
		    #first line
		    
		    $line =~ /^atom:\s*\d+\s*(\d+)\w-(\w+)\s*.*/;
		    $first = $1;
		    $first_name = $2;
		    $first_line_seen = 1;
		    
		}
		else
		{
		    #second line
		    
		    $line =~ /^atom:\s*\d+\s*(\d+)\w-(\w+)\s*.*/;
		    $second = $1;
		    $second_name = $2;
		    $pair_seen = 1;
		}
		
	    }
	    else
	    {
		#other lines
		if ($other_first_line)
		{
		    $line =~ /^atom:\s*\d+\s*(\d+)\w-(\w+)\s*.*/;
		    my $amb_atom = $2;
		    
		    unless ($amb_atom eq $first_name)
		    {
			$amb_atom =~ /(\w+)(\d)$/;
			my $amb_num = $2;
			my $amb_atm = $1;
			$first_name =~ /(\w+)(\d)$/;
			my $fir_num = $2;
			my $fir_atm = $1;

			if ($amb_num and $fir_num)
			{
			    if (($amb_atm eq $fir_atm) and ($amb_num != $fir_num))
			    {
				$first_name = $amb_atm."*";
			    }
			}
		    }
		}
		else
		{
		    $line =~ /^atom:\s*\d+\s*(\d+)\w-(\w+)\s*.*/;
		    my $amb_atom = $2;
		    
		    unless ($amb_atom eq $second_name)
		    {
			$amb_atom =~ /(\w+)(\d)$/;
			my $amb_num = $2;
			my $amb_atm = $1;
			$second_name =~ /(\w+)(\d)$/;
			my $sec_num = $2;
			my $sec_atm = $1;

			if ($amb_num and $sec_num)
			{
			    if (($amb_atm eq $sec_atm) and ($amb_num != $sec_num))
			    {
				$second_name = $amb_atm."*";
			    }
			}
		    }
		}
		$other_first_line = !$other_first_line;
		
	    }
	    
	    #store atom and number in number_to_atom array
	    $line =~ /^atom:\s*\d+\s+(\d+)([a-zA-Z])-.+/;
	    $atom[$1] = $2;
	}
	elsif ($line =~ /^::sumavg:\s*(.+)$/i)
	{
	    $distance = $1;
	    
	    $data[$section][$first]{$first_name}[$second]{$second_name} = $distance;
	    #print "parse_ql: data[$section][$first]{$first_name}[$second]{$second_name} = $distance\n";
	    

	    #print "data[$section][$ql_data{'1-residue number'}]{$ql_data{'1-atom name'}}[$ql_data{'2-residue number'}]{$ql_data{'2-atom name'}} = $ql_data{'distance'}\n";
	    #print "data[$section][$ql_data{'2-residue number'}]{$ql_data{'2-atom name'}}[$ql_data{'1-residue number'}]{$ql_data{'1-atom name'}} = $ql_data{'distance'}\n";

	    
	    #$data_count++;
	    $first_line_seen = 0;
	    $other_first_line = 1;
	    $pair_seen = 0;

	    
	}	
    }
    return (\@data, \@section_heading, \@atom);

}


return 1;
