sub parse_sec
{
    my ($file, $parent) = @_;

    unless (open(FILE, $file))
    {
	&err_msg("Cannot open sec file \"$file\"", $parent);
	return;
    }

    my $line;
    my %data;
    my %count;
    my $key;
    my $type;
    my ($strands, $dir_line, $interaction, $inter_count);

    foreach $type (qw/removed added candidates/)
    {
	$count{$type} = 0;
    }

    while (chomp($line = <FILE>))
    {
	if ($line =~ /^summary/i)
	{
	    $key = 'CSI';
	}
	elsif ($line =~ /^combined\sanalysis\sof/i)
	{
	    $key = 'RCSI';
	}
	elsif ($line =~ /^validate\sthe\sbeta/i)
	{
	    $key = 'Beta';
	}
	elsif ($line =~ /^final\ssecondary/i)
	{
	    $key = 'Final';
	}
	elsif ($line =~ /^antiparallel\sstrands/i)
	{
	    $key = 'Antiparallel';
	}
	elsif ($line =~ /^parallel\sstrands/i)
	{
	    $key = 'Parallel';
	}
	elsif ($line =~ /^selected.+registers/i)
	{
	    $key = 'Evidences';
	}
	elsif ($line =~ /^secondary\sstruture\sis\stemp\w*\sdefined\sas/i)
	{
	    $key = 'Temp_ids';
	    #print "key is now Temp_ids\n";
	}
	elsif ($line =~ /^\*\*/)
	{
	    $data{'Candidates'}[$count{'Candidates'}] = $line;
	    $count{'Candidates'}++;
	}
	
	else
	{
	    if ($key eq 'CSI')
	    {
		if ($line =~ /^(\d+)\s+(\w)\s+.+\s+(.*\d)$/)
		{
		    $data{'CSI'}[$1] = $3;
		    $data{'AA'}[$1] = $2;
		    $data{'Raw CSI'}[$count{'Raw CSI'}] = $line;
		    $count{'Raw CSI'}++;
		    #print "data{'CSI'}{$1} = $2\n";
		    #print "data{'AA'}[$1] = $2\n";
		}
		elsif ($line =~ /^\#/)
		{
		    $data{'Raw CSI'}[0] = $line;
		    $count{'Raw CSI'} = 1;
		}
		elsif ($line =~ /^(\-|\s)+$/)
		{
		    $data{'Raw CSI'}[1] = $line;
		    $count{'Raw CSI'} = 2;
		}
	    }
	    elsif ($key eq 'RCSI')
	    {
		if ($line =~ /\s+(\d+)\sis\sremoved\sfrom/i)
		{
		    $data{'RCSI'}{'removed'}[$count{'removed'}] = $line;
		    my $res = $1;
		    if ($data{'RCSI'}{'res_added'}{$res})
		    {
			$data{'RCSI'}{'res_removed'}{$res} = 2;
		    }
		    else
		    {
			$data{'RCSI'}{'res_removed'}{$res} = 1;
		    }
		    #print "data{'RCSI'}{'removed'}[$count{'removed'}] = $line, data{'RCSI'}{'res_removed'}{$res} = $data{'RCSI'}{'res_removed'}{$res}\n";
		    $count{'removed'}++;
		}
		elsif ($line =~ /\s+(\d+)\sis\sadded/i)
		{
		    my $res = $1;
		    $data{'RCSI'}{'added'}[$count{'added'}] = $line;
		    
		    if ($data{'RCSI'}{'res_removed'}{$res})
		    {
			$data{'RCSI'}{'res_added'}{$res} = 2;
		    }
		    else
		    {
			$data{'RCSI'}{'res_added'}{$res} = 1;
		    }
		    #print "data{'RCSI'}{'added'}[$count{'added'}] = $line; data{'RCSI'}{'res_added'}{$res} = $data{'RCSI'}{'res_added'}{$res}\n";
		    $count{'added'}++;
		}
	    }
	    elsif ($key eq 'Beta')
	    {
		if ($line =~ /^remove\s(\d+)\s/i)
		{
		    my $res = $1;
		    $data{'RCSI'}{'removed'}[$count{'removed'}] = $line;
		    if ($data{'RCSI'}{'res_added'}{$res})
		    {
			$data{'RCSI'}{'res_removed'}{$res} = 2;
		    }
		    else
		    {
			$data{'RCSI'}{'res_removed'}{$res} = 1;
		    }
		    #print "data{'RCSI'}{'removed'}[$count{'removed'}] = $line, data{'RCSI'}{'res_removed'}{$res} = $data{'RCSI'}{'res_removed'}{$res}\n";
		    #print "data{'RCSI'}{'removed'}[$count{'removed'}] = $line\n";
		    $count{'removed'}++;
		}
		elsif ($line =~ /^insert\s(\d+)\s/i)
		{
		    my $res = $1;
		    $data{'RCSI'}{'added'}[$count{'added'}] = $line;
		    if ($data{'RCSI'}{'res_removed'}{$res})
		    {
			$data{'RCSI'}{'res_added'}{$res} = 2;
		    }
		    else
		    {
			$data{'RCSI'}{'res_added'}{$res} = 1;
		    }
		    #print "data{'RCSI'}{'added'}[$count{'added'}] = $line; data{'RCSI'}{'res_added'}{$res} = $data{'RCSI'}{'res_added'}{$res}\n";
		    #print "B: data{'RCSI'}{'added'}[$count{'added'}] = $line\n";
		    $count{'added'}++;
		}
		elsif ($line =~ /^case/i)
		{
		    $line =~ /\:\s*([\-\d]+)\s*$/;
		    my $register = $1;
		    $data{'removed_candidates'}{$register} = 1;
		    #print "Register $register is removed\n";
		}
	    }
	    elsif ($key eq 'Final')
	    {
		if ($line =~ /^ID.(\d+)\s+(.+)$/)
		{
		    $data{'Final'}[$1] = $2;
		    #print "data{'Final'}[$1] = $2\n";
		}
	    }
	    elsif ($key eq 'Temp_ids')
	    {
		if ($line =~ /^ID.(\d+)\s+(.+)$/i)
		{
		    $data{'Temp_ids'}[$1] = $2;
		    #print "data{'Temp_ids'}[$1] = $2\n";
		}
	    }
	    elsif ($key eq 'Antiparallel')
	    {
		next unless ($line =~ /^\d/);

		my (@strands) = split(/\s/, $line);
		#print "anti: @strands\n";
		
		my $i;
		for ($i = 0; $i<=$#strands; $i++)
		{
		    $data{'Antiparallel'}[$i] = $strands[$i];
		    #print "data{'Antiparallel'}[$i] = $data{'Antiparallel'}[$i]\n";
		}
	    }
	    elsif ($key eq 'Parallel')
	    {
		next unless ($line =~ /^\d/);

		my (@strands) = split(/\s+/, $line);
		#print "par: @strands\n";
		
		my $i;
		for ($i = 0; $i<=$#strands; $i++)
		{
		    $data{'Parallel'}[$i] = $strands[$i];
		    #print "data{'Parallel'}[$i] = $data{'Parallel'}[$i]\n";
		}
	    }
	    elsif ($key eq 'Evidences')
	    {
		#$data{'Evidences'}{$strands}{$dir_line}{$interaction}[inter_count] = peak
		
		if ($line =~ /^Strands\s(\d+\-\d+)$/i)
		{
		    $strands = $1;
		    #print "strands = $strands\n";
		}
		elsif ($line =~ /(anti_para|para)/)
		{
		    $line =~ s/^\s*//;
		    $dir_line = $line;
		    #print "dir_line = $dir_line\n";
		}
		elsif ($line =~ /(\d+\w+\-\d+\w+)/)
		{
		    $interaction = $1;
		    $inter_count = 0;
		}
		elsif ($line =~ /peak\:\s+(.+)$/)
		{
		    
		    $data{'Evidences'}{$strands}{$dir_line}{$interaction}[$inter_count] = $1;
		    #print "data{'Evidences'}{$strands}{$dir_line}{$interaction}[$inter_count] = $1\n";



#Strands 12-14 --> $strands = 12-14
 # anti_para->76-91 Total score: 1 --> $dir_line = <line>
  #      76Hn-91Hn Score: 1  -->  $interaction = "76Hn-91Hn"
   #             peak:  2121 (hx1)8.04  (hx2)8.59  (x1)105.45  1.12e+06 (label)223    -->  $data{'Evidences'}{$strands}{$dir_line}{$interaction}[0] = <line>;

		    if ($dir_line =~ /9\-16/)
		    {
			#print "data{'Evidences'}{$strands}{$dir_line}{$interaction}[$inter_count] = $data{'Evidences'}{$strands}{$dir_line}{$interaction}[$inter_count]\n";
		    }		    
		    $inter_count++;
		}
		
	    }
	}
    }

    #print "parse: data{'RCSI'}{'added'}[0] = $data{'RCSI'}{'added'}[0]\n";
    return (\%data);
}

return 1;
