my %aa = (
    "ALA"=>"A",
    "ARG"=>"R",
    "ASN"=>"N",
    "ASP"=>"D",
    "CYS"=>"C",
    "GLN"=>"Q",
    "GLU"=>"E",
    "GLY"=>"G",
    "HIS"=>"H",
    "ILE"=>"I",
    "LEU"=>"L",
    "LYS"=>"K",
    "MET"=>"M",
    "PHE"=>"F",
    "PRO"=>"P",
    "SER"=>"S",
    "THR"=>"T",
    "TRP"=>"W",
    "TYR"=>"Y",
    "VAL"=>"V"
	  );


sub parse_upl
{

    my ($file, $data_ref, $residue_ref, $parent, $start_res, $end_res, $format) = @_;
    #print "\nparse_upl: file is $file\n";
    
    unless (-f $file)
    {
	print "upl file does not exist\n";
	return;
    }

    unless ($start_res)
    {
	$start_res = 1;
    }

    #print "parse_upl: from residue $start_res to $end_res  $format\n";
    my @data = @$data_ref;
    my @residue = @$residue_ref;
    my @lines;

    unless (open(FILE, $file))
    {
	&err_msg("Cannot open upl file \"$file\"", $parent);
	return;
    }

    #$data[<Atom 1>]{<Atom 1 name>}[<Atom 2>]{<Atom 2 name>} = distance

    my $line;
    my ($res1, $atm1, $res2, $atm2, $d1, $d2); 

    while (chomp($line = <FILE>))
    {
	#next if ($line =~ /^\#/);
	next if ($line =~ /^\s*$/);

	#print "line is $line\n";
	if ($format eq 'dyana')
	{
	    if ($line =~ /^\s*(\d+)\s+(\w\w\w)\s*$/)
	    {
		$res1 = $1;
		$residue[$res1] = $aa{$2};
	    }
	    elsif ($line =~ /^\s*([\w\d]+)\s+(\d+)\s(\w\w\w)\s+([\w\d]+)\s+([\d\.]+)/)
	    {
		($atm1, $res2, $atm2, $d1) = ($1, $2, $4, $5);
		
		next if ($res1 < $start_res);
		next if ($res2 < $start_res);
		if ($end_res)
		{
		    next if ($res1 > $end_res);
		    next if ($res2 > $end_res);
		}

		#$atm1 = 'HN' if ($atm1 eq 'H');
		#$atm2 = 'HN' if ($atm2 eq 'H');
 
		$data[$res1]{$atm1}[$res2]{$atm2} = $d1;
		$lines[$res1]{$atm1}[$res2]{$atm2} = $line;
		#print "$res1$atm1 to $res2$atm2\n";
	    }
	    elsif ($line =~ /[\s\#]*(\d+)\s(\w\w\w)\s+([\w\d]+)\s+(\d+)\s(\w\w\w)\s+([\w\d]+)\s+([\d\.]+)/)
	    {
		($res1, $atm1, $res2, $atm2, $d1) = ($1, $3, $4, $6, $7);
		next if ($res1 < $start_res);
		next if ($res2 < $start_res);
		if ($end_res)
		{
		    next if ($res1 > $end_res);
		    next if ($res2 > $end_res);
		}
		
		#$atm1 = 'HN' if ($atm1 eq 'H');
		#$atm2 = 'HN' if ($atm2 eq 'H'); 
      
		$residue[$res1] = $aa{$2};
		
		$data[$res1]{$atm1}[$res2]{$atm2} = $d1;
		$lines[$res1]{$atm1}[$res2]{$atm2} = $line;

		#print "$res1$atm1 to $res2$atm2\n";
	    }
	    
	}
	elsif ($format eq 'xplor')
	{
	    $line =~ /assign\s+\(resid\s(\d+)\s+and\sname\s([\w\d\#]+)\s+\)\(resid\s(\d+)\s+and\sname\s([\w\d\#]+)\s*\)\s+([\d\.]+)\s+[\d\.]+\s+([\d\.]+)/;
	    
	    if (($1) and ($2) and ($3) and ($4) and ($5))
	    {
		next if ($1 < $start_res);
		next if ($3 < $start_res);

		if ($end_res)
		{
		    next if ($1 > $end_res);
		    next if ($3 > $end_res);
		}
		
		
		#$residue[$1] = $2;
		($res1, $atm1, $res2, $atm2, $d1, $d2) = ($1, $2, $3, $4, $5, $6);

		$atm1 = 'H' if ($atm1 eq 'HN');
		$atm2 = 'H' if ($atm2 eq 'HN'); 

		#if ($atm1 =~ /HA[1\#]/)
		#{
		#    $atm1 = 'HA';
		#}
		#if ($atm2 =~ /HA[1\#]/)
		#{
		#    $atm2 = 'HA';
		#}
		
		$data[$res1]{$atm1}[$res2]{$atm2} = $d1 + $d2;
		$lines[$res1]{$atm1}[$res2]{$atm2} = $line;

	    }
	}
	

    }
    #print "parse_upl: returning data\n";
    #print "parse_upl: total residues is $#residue\n";

    $data_ref = \@data;
 
    if ($format eq 'xplor')
    {
	$residue_ref = &get_xplor_residues($residue_ref, $file, $parent);
    }
    else
    {
	$residue_ref = \@residue;
    }

    return($data_ref, $residue_ref, \@lines);
}


sub get_xplor_residues
{
    my ($residue_ref, $upl_file, $parent) = @_;
    my $dir = dirname($upl_file);
    
    #get sequence file
    unless (opendir(DIR, $dir))
    {
	&err_msg("Cannot open directory \"$dir\"", $parent);
	return;
    }
    my ($seq_file) = grep {/\.Sequence$/} readdir(DIR);
    close(DIR);

    unless ($seq_file)
    {
	&err_msg("No Sequence file in \"$dir\"", $parent);
	return;
    }
    $seq_file = File::Spec->rel2abs($seq_file, $dir);

    my @residue = @$residue_ref;

    #read sequence file
    unless(open(SEQ, $seq_file))
    {
	&err_msg("Cannot open sequence file in \"$dir\"", $parent);
	return;
    }
    
    my $count = 1;
    my $line;
    my $i;

    while (chomp($line = <SEQ>))
    {
	my @res = split(/\s+/, $line);
	for ($i = 0; $i<=$#res; $i++)
	{
	    $residue[$count] = $aa{$res[$i]};
	    $count++;
	}
	
    }
    
    return (\@residue);

}


return 1;
