my $light_red = '#ff8800';
my $dark_red = '#cc0000';

sub sec_str_disp
{
    my ($frame, $parent, $as_dir) = @_;
    #print "sec_str_disp: as_dir is $as_dir\n";

    my $frm1 = $frame->Frame()->pack(-fill=>'x', -pady=>3, -padx=>3);
    my $imp_but = $frm1->Button(-text=>'Import Secondary Structure Prediction in CASP Format')->pack(-padx=>3, -pady=>3);

    #get .sec file
    my $sec_file = &get_sec_file($as_dir, $parent);
    
    unless (-f $sec_file)
    {
	&err_msg("No sec file found in \"$dir\"", $parent);
	return;
    }
    #print "sec_str_disp: sec_file is $sec_file\n";

    my ($data_ref) = &parse_sec($sec_file, $parent);
   
    my $frm2 = $frame->Frame()->pack(-fill=>'both', -expand=>1, -padx=>3, -pady=>3);

    my %data = %$data_ref;
    my $res_count = scalar(@{$data{'CSI'}});
    #print "res_count is $res_count\n";
    $frm2->update();
    my $frm_width = $frm2->width();
    $frm_width -= 10;

    #create empty canvas
    my ($canvas, $help_ref, $coords_ref) = &create_sec_canvas($frm2, $parent, $res_count, $frm_width, $data_ref);

    #load data onto canvas
    &show_sec_data($canvas, $data_ref, $parent, $frm_width, $help_ref, $coords_ref, $as_dir);

    $imp_but->configure(-command=>sub
			{
			    
			    #my ($pred_file) = &OpenFile('Sec Str Pred', $parent, Cwd::cwd);
			    #print "pred_file is $pred_file\n";

			    #parse pred file
			    my $pred_data_ref;
			    &import_pred($parent, \$pred_data_ref);
			    &show_pred_data($canvas, $pred_data_ref, $help_ref, $coords_ref, $as_dir);
			    
			});

}


#import prediction of sec structure from PROF
sub import_pred
{
    my ($parent, $pred_data_ref_ref) = @_;
    my ($file, $seq_pred, $pred_data_ref);

    my $window = $parent->Toplevel(-title=>'Import Prediction');
    
    my $lbl = $window->Label(-text=>'Import PROF Secondary Structure Prediction', -relief=>'raised')->pack(-fill=>'x', -ipady=>3);
    
    my $lbl1 = $window->Label(-text=>'Upload File with PROF information in text format')->pack(-padx=>3, -pady=>3, -ipady=>3);
    my $frm1 = $window->Frame()->pack(-padx=>5, -pady=>5);
    my $fil_lbl = $frm1->Label(-text=>'File : ')->pack(-side=>'left');
    my $fil_ent = $frm1->Entry(-textvariable=>\$file, -background=>'white')->pack(-side=>'left');
    my $fil_brw = $frm1->Button(-text=>'Browse', -command=>sub
				{
				    $file = &OpenFile('PROF', $window, Cwd::cwd);
				    $file = File::Spec->rel2abs($file, Cwd::cwd);
				})->pack(-side=>'left');
    
    my $lbl2 = $window->Label(-text=>'Or paste the PROF Prediction in the box below')->pack(-padx=>3, -pady=>3, -ipady=>3);
    my $stxt = $window->Scrolled('Text', -scrollbars=>'e')->pack(-fill=>'both', -expand=>1);
    my $txt = $stxt->Subwidget('text');
    $txt->configure(-background=>'white');

    my $frm2 = $window->Frame()->pack(-padx=>5, -pady=>5);
    my $sub_but = $frm2->Button(-text=>'Submit')->pack(-side=>'left');
    my $cls_but = $frm2->Button(-text=>'Close', -command=>sub {$window->destroy();})->pack(-side=>'left');
    
    
    $sub_but->configure(-command=>sub
			{
			    if ($file)
			    {
				($$pred_data_ref_ref) = &parse_seq_pred($file, $window);
				$window->destroy();
			    }
			    else
			    {
				my ($data) = $txt->get('1.0', 'end');
				my (@lines) = split(/\n/, $data);
				
				my @data;
				my $res_num = 0;
				my $i;
				for ($i = 0; $i<$#lines; $i++)
				{
				    if ($lines[$i] =~ /(\w)\s+(\w)\s+([\d\.]+)$/)
				    {
					$data[$res_num]{'pred'} = $2;
					$data[$res_num]{'score'} = $3;
					$res_num++;
				    }
				}
				
				($$pred_data_ref_ref) = (\@data);
				$window->destroy();
			    }
			});
    $window->waitWindow();
}


#show prediction of sec str from PROF
sub show_pred_data
{
    my ($canvas, $data_ref, $help_ref, $coords_ref, $as_dir) = @_;
    
    my @data = @$data_ref;
    my %coords = %$coords_ref;
    my ($x, $y, $end_x) = ($coords{'Prediction'}{'x'},$coords{'Prediction'}{'y'}, $coords{'Prediction'}{'end_x'});
    my ($x_div, $y_div) = ($coords{'x_div'}, $coords{'y_div'});
    #print "show_pred_data: x = $x, y = $y, end_x = $end_x\n";
    
    my $res_num;
    my $x_div = $coords{'x_div'};
    my $base = $y + ($y_div/3);
    my $top = $base - $y_div;
    my $div = ($base - $top)/2;
    
    $canvas->createText($x - 10, $base, -text=>'0');
    $canvas->createText($x - 10, $base - $div, -text=>'0.5');
    $canvas->createText($x - 10, $top, -text=>'1.0');
    

    for ($res_num = 1; $res_num <= $#data; $res_num++)
    {
	my $cx = $x + ($res_num * $x_div);
	next if ($data[$res_num]{'pred'} eq 'C');

	my $bar = &place_pred($canvas, $cx, $x_div/2, $base, $top, $data[$res_num]{'score'}, $data[$res_num]{'pred'});
	my $temp_num = $res_num;
	$canvas->bind($bar, '<Button-1>'=>sub
		      {
			  $$help_ref = "Residue $temp_num: Prediction - $data[$temp_num]{'pred'}, Score - $data[$temp_num]{'score'}";
		      });
    }
    
    $canvas->createLine($x, $base, $end_x, $base);
    $canvas->update;
    #print "show_pred_data: done\n";
}


sub place_pred
{
    my ($canvas, $x, $width, $base, $top, $value, $type) = @_;

    my $color;
    my $bar;

    #value between 0 to 1
    my $div = ($base - $top)/10;
    $value = 10 * &round_num($value, 0.1);
    my $height = $div * $value;
    
    if ($type eq 'H')
    {
	#Helix - Red
	$color = 'red';
    }
    elsif ($type eq 'E')
    {
	#Extended element - blue
	$color = 'blue';
    }
    
    $bar = &place_bar($canvas, $base, $base - $height, $x, $width, $color);
    return ($bar);
    
}



sub show_sec_data
{
    my ($canvas, $data_ref, $parent, $width, $help_ref, $coords_ref, $as_dir) = @_;
    my %data = %$data_ref;
    my %coords = %$coords_ref;

    my ($i, $j);
    my $res_count = scalar(@{$data{'CSI'}});
    my $x_div = $coords{'x_div'};
    my $y_div = $coords{'y_div'};
    my $div = $x_div/2;

    #CSI - read from data_ref - data{'CSI'}[#], if 0, nothing, if 1, put rect above line, if -1, put rect below line
    my ($x, $y, $end_x) = ($coords{'CSI'}{'x'},$coords{'CSI'}{'y'}, $coords{'CSI'}{'end_x'});
    
    $canvas->createLine($x, $y, $end_x, $y, -fill=>'brown');
    $canvas->createText($x-15, $y-10, -text=>'H', -fill=>'red');
    $canvas->createText($x-15, $y+10, -text=>'E', -fill=>'blue');

    my $cy = $y;
    for ($i = 1; $i<$res_count; $i++)
    {
	my $cx = $x + ($i * $x_div);
	my $csi = $data{'CSI'}[$i];
		
	if ($csi == 1)
	{
	    &place_csi($cx-$div, $y-($y_div/3), $x_div, $y_div/3, $canvas, $help_ref, 'black');
	    #$canvas->createText($cx, $cy, -text=>'&');
	}
	elsif ($csi == -1)
	{
	    &place_csi($cx-$div, $y, $x_div, $y_div/3, $canvas, $help_ref, 'black');
	    #$canvas->createText($cx, $cy, -text=>'#');
	}
    }

    #Refined CSI - go through data{CSI}, place residues, then go thruogh data{RCSI}, for each removed, place red X, for each added, show in green
    ($x, $y, $end_x) = ($coords{"Combined\nCSI,JNOE"}{'x'},$coords{"Combined\nCSI,JNOE"}{'y'}, $coords{"Combined\nCSI,JNOE"}{'end_x'});
    
    $cy = $y;
    for ($i = 1; $i<$res_count; $i++)
    {
	my $cx = $x + ($i * $x_div);
	my $csi = $data{'CSI'}[$i];
		
	if ($csi == 1)
	{
	    &place_csi($cx-$div, $y-($y_div/3), $x_div, $y_div/3, $canvas, $help_ref, 'black');
	    #$canvas->createText($cx, $cy, -text=>'&');
	}
	elsif ($csi == -1)
	{
	    &place_csi($cx-$div, $y, $x_div, $y_div/3, $canvas, $help_ref, 'black');
	    #$canvas->createText($cx, $cy, -text=>'#');
	}
    }
    
    my %csi_res;
    #print "\n";
    #removed residues
    for ($j = 0; $j<scalar(@{$data{'RCSI'}{'removed'}}); $j++)
    {
	my $line = $data{'RCSI'}{'removed'}[$j];
	$line =~ /(\d+)\sis\sremoved.+\:(.+)$/i;
	my $res = $1;
	my $msg = "Residue $res is removed because $2";
	$csi_res{$res} = $2;
	unless ($res)
	{
	    $line =~ /remove\s(\d+)\sas\s(.+)$/i;
	    $res = $1;
	    $msg = "Residue $res is removed because $2";
	    $csi_res{$res} = $2;
	}
	next unless ($res);
	#print "res removed = $res, msg = $msg\n";

	my $cx = $x + ($res * $x_div);
	my $cy = $y;
	
	#print "Residue $res was removed because $csi_res{$res}\n";
	#$csi_res{$res} = $msg;
	
	if ($data{'CSI'}[$res] == 1)
	{
	    my $box;
	    $box = &place_csi($cx-$div, $y-($y_div/3), $x_div, $y_div/3, $canvas, $help_ref, 'white');
	    
	    my $rem;
	    my $color;
	    if ($msg =~ /\sJ3/)
	    {
		$color = $dark_red;
	    }
	    else
	    {
		$color = $light_red;
	    }
	    $rem = $canvas->createOval($cx-($div/2), $cy-($y_div/6), $cx+($div/2), $cy-($y_div/3), -fill=>$color, -outline=>$color);
	    $canvas->bind($rem, '<Button-1>'=>sub
			  {
			      $$help_ref = $msg;
			  });
	    $canvas->bind($box, '<Button-1>'=>sub
			  {
			      $$help_ref = $msg;
			  });
	    
	}
	elsif ($data{'CSI'}[$res] == -1)
	{
	    my $box = &place_csi($cx-$div, $y, $x_div, $y_div/3, $canvas, $help_ref, 'white');
	    my $rem;

	    if ($msg =~ /\sJ3/)
	    {
		$rem = $canvas->createOval($cx-($div/2), $cy+($y_div/6), $cx+($div/2), $cy+($y_div/3), -fill=>$dark_red, -outline=>$dark_red);
	    }
	    else
	    {
		$rem = $canvas->createOval($cx-($div/2), $cy+($y_div/6), $cx+($div/2), $cy+($y_div/3), -fill=>$light_red, -outline=>$light_red);
	    }
	    $canvas->bind($rem, '<Button-1>'=>sub
			  {
			      $$help_ref = $msg;
			  });
	    $canvas->bind($box, '<Button-1>'=>sub
			  {
			      $$help_ref = $msg;
			  });
	}
    }

    #print "\n";
    #added residues
    my $m;
    for ($m = 0; $m<scalar(@{$data{'RCSI'}{'added'}}); $m++)
    {
	my $line = $data{'RCSI'}{'added'}[$m];
	$line =~ /(\d+)\sis\sadded(.+)\:(.+)$/i;
	my $res = $1;
	my $tmp_msg = $3;
	my $str = $2;
	unless ($res)
	{
	    $line =~ /^insert\s(\d+)\sinto\s([-\w]+)\,\s(.+)$/i;
	    $res = $1;
	    $tmp_msg = $3;
	    $str = $2;
	}
	
	
	#print "Residue $res is added because $tmp_msg\n";

	my $msg;
	
	if ($data{'RCSI'}{'res_removed'}{$res} == 2)
	{
	    $msg = "Residue $res was first added because $tmp_msg and was then removed because $csi_res{$res}";
	    #$msg = $csi_res{$res}." and is added because $tmp_msg";
	}
	elsif ($data{'RCSI'}{'res_added'}{$res} == 2)
	{
	    $msg = "Residue $res was first removed because $csi_res{$res} and then added because $tmp_msg";
	}
	else
	{
	    $msg = "Residue $res is added because $tmp_msg";
	}
	
	#print "$msg\n";

	#print "m = $m, ($res, $msg, $str)\n";
	next unless ($res);
	#print "added: $res\n";
	
	my $cx = $x + ($res * $x_div);
	my $cy = $y;

	my $color;
	if ($msg =~ /\sJ3/)
	{
	    $color = $dark_red;
	}
	else
	{
	    $color = $light_red;
	}
	
	if (($str =~/element/) or ($str =~ /sheet/))
	{
	    my $box = &place_csi($cx-$div, $cy, $x_div, $y_div/3, $canvas, $help_ref, 'green');
	    
	    $canvas->bind($box, '<Button-1>'=>sub
			  {
			      $$help_ref = $msg;
			  });
	    if ($csi_res{$res})
	    {
		my $oval = $canvas->createOval($cx-($div/2), $cy+($y_div/6), $cx+($div/2), $cy+($y_div/3), -fill=>$color, -outline=>$color);
		$canvas->bind($oval, '<Button-1>'=>sub
			      {
				  $$help_ref = $msg;
			      });
	    }
	}
	else
	{
	    my $box = &place_csi($cx-$div, $cy-($y_div/3), $x_div, $y_div/3, $canvas, $help_ref, 'green');
	    $canvas->bind($box, '<Button-1>'=>sub
			  {
			      $$help_ref = $msg;
			  });

	    if ($csi_res{$res})
	    {
		my $oval = $canvas->createOval($cx-($div/2), $cy-($y_div/6), $cx+($div/2), $cy-($y_div/3), -fill=>$color, -outline=>$color);
		$canvas->bind($oval, '<Button-1>'=>sub
			      {
				  $$help_ref = $msg;
			      });
	    }
	}
    }

    $canvas->createLine($x, $y, $end_x, $y, -fill=>'brown');
    
   
    #final secondary structure, go through data{'Final'}[#], get start and end residues, place letter at that resiude, and place '-' for residues in between
    ($x, $y, $end_x) = ($coords{'Final Sec Str'}{'x'},$coords{'Final Sec Str'}{'y'}, $coords{'Final Sec Str'}{'end_x'});
    my ($start_res, $end_res, $prev_end);
    for ($j = 0; $j<scalar(@{$data{'Final'}}); $j++)
    {
	my $line = $data{'Final'}[$j];
	$line =~ /^(\w+)\s+(\d+)-(\d+)$/;
	my $sec_str = $1;
	$start_res = $2;
	$end_res = $3;
	
	if ($sec_str eq 'Helix')
	{
	    #up
	    my $cy = $y;
	    for ($k = $start_res; $k<=$end_res; $k++)
	    {
		my $cx = $x + ($k * $x_div);
		my $box = &place_csi($cx-$div, $cy - ($y_div/3), $x_div, $y_div/3, $canvas, $help_ref, 'black');
		my $p = $k;
		$canvas->bind($box, '<Button-1>'=>sub
			      {
				  $$help_ref = "Residue $p part of final secondary structure";
			      });
	    }
	}
	elsif ($sec_str eq 'Sheet')
	{
	    my $cy = $y;
	    for ($k = $start_res; $k<=$end_res; $k++)
	    {
		my $cx = $x + ($k * $x_div);
		my $box = &place_csi($cx-$div, $cy, $x_div, $y_div/3, $canvas, $help_ref, 'black');
		my $p = $k;
		$canvas->bind($box, '<Button-1>'=>sub
			      {
				  $$help_ref = "Residue $p part of final secondary structure";
			      });
	    }
	}
    }
    $canvas->createLine($x, $y, $end_x, $y, -fill=>'black');
    
    
    #NHSlowList
    ($x, $y, $end_x) = ($coords{'NHSlowList'}{'x'},$coords{'NHSlowList'}{'y'}, $coords{'NHSlowList'}{'end_x'});
    #-get control file
    my ($cont_file) = &get_control_file($as_dir);
    my ($nhs_filename) = &openContFile($cont_file, 0, 0, 1);
    

    if ($nhs_filename)
    {
	my $nhs_file;
	my $dirname = dirname($nhs_filename);
	#print "2. dirname is $dirname\n";
	if ($dirname)
	{
	    my $parent_dir = dirname($as_dir);
	    $nhs_file = $parent_dir."/$nhs_filename";
	    
	}
	else
	{
	    $nhs_file = $as_dir."/source/".$nhs_filename;
	   
	}
	#print "3. nhs_file is $nhs_file\n";
	$nhs_file =~ s/\s+$//;
	$nhs_file =~ s/^\s+//;

	unless (-f $nhs_file)
	{
	    #print "3a.($nhs_file) doesnt exist, adding .gz\n";
	    $nhs_file = $nhs_file.".gz";
	}
	if (-f $nhs_file)
	{
	    if ($nhs_file =~ /\.gz$/)
	    {
		$nhs_file = &unzip_file($nhs_file);
	    }
	    
	    #print "4. nhs_file is $nhs_file\n";
	    

	    #parse the nhs file
	    my ($nhs_data_ref) = &parse_nhs($nhs_file, $parent);
	    my @nhs_data = @$nhs_data_ref;
	    #print "5. size of nhs_data is $#nhs_data\n";
	    
	    for ($k = 0; $k<=$#nhs_data; $k++)
	    {
		my $res = $nhs_data[$k];
		$data{'NHS'}{$res} = 1;
		#print "res\n";
		
		my $cx = $x + ($res * $x_div);
		my $cy = $y;
		
		&place_nhs($cx, $cy, $canvas, $help_ref, $res);
	    }
	}
    }


    #JValues
    ($x, $y, $end_x) = ($coords{'JFile'}{'x'},$coords{'JFile'}{'y'}, $coords{'JFile'}{'end_x'});


    my ($jfilename) = &openContFile($cont_file, 0,0,0,1);
    
    #j scale
    my $base = $y + ($y_div/2);
    my $top = $base - $y_div;
    
    $canvas->createText($x-10, $base, -text=>'4', -fill=>'orange');
    $canvas->createText($x-10, $base - (($base-$top)/2), -text=>'6', -fill=>'dark red');
    $canvas->createText($x-10, $top, -text=>'8', -fill=>'blue');
    
    if ($jfilename)
    {
	my $dirname = dirname($jfilename);
	if ($dirname)
	{
	    my $parent_dir = dirname($as_dir);
	    $jfilename = $parent_dir."/$jfilename";
	}
	else
	{
	    $jfilename = $as_dir."/source/$jfilename";
	}

	unless (-f $jfilename)
	{
	    $jfilename = $jfilename.".gz";
	}
	if (-f $jfilename)
	{
	    if ($jfilename =~ /\.gz$/)
	    {
		($jfilename) = &unzip_file($jfilename);
	    }

	    my ($j_data_ref) = &parse_jfile($jfilename, $parent);

	    my @jdata = @$j_data_ref;
	    
	    for ($i = 0; $i<=$#jdata; $i++)
	    {
		my ($res, $jvalue) = ($jdata[$i]{'res'}, $jdata[$i]{'jvalue'});
		
		#print "i = $i, ($res, $jvalue)\n";
		my $cx = $x + ($res * $x_div) - ($div/2);
		
		my $bar = &place_jvalue($canvas, $cx, $div, $base, $top, $jvalue);
		$canvas->bind($bar, '<Button-1>'=>sub
			      {
				  $$help_ref = "Residue $res: J-value = $jvalue";
			      });
	    }
	    $canvas->createLine($x, $base, $end_x, $base);

	}
    }

    $base = $base + $y_div/2;
    
    #Raw CSI and .sec file buttons
    my $raw_but = $canvas->Button(-text=>'Raw CSI', -command=>sub
				  {
				      &show_raw_csi($parent, $data_ref);
				  });
    $canvas->createWindow($x, $base, -window=>$raw_but, -anchor=>'nw');
    $x += 5 * $x_div;
    
    #$canvas->createText($x, $base, -text=>'$', -fill=>'blue');
    my $sec_but = $canvas->Button(-text=>"Open .sec file", -command=>sub
				  {
				      my ($sec_file) = &get_sec_file($as_dir, $parent);
				      my $pid;
				      if ($pid = fork) 
				      {
					  #parent
					  $SIG{CHILD} = 'IGNORE';
					  return;
				      }
				      elsif (defined $pid) 
				      { 
					  #child
					  exec 'xterm', '-title', "$sec_file", '-e', 'less', "$sec_file";
				      }
				  });
    $canvas->createWindow($x, $base, -window=>$sec_but, -anchor=>'nw');

    $x += 7 * $x_div;
    $data_ref = \%data;
    my $reg_but = $canvas->Button(-text=>"All possible registers", -command=>sub
				  {
				      &all_betas($parent, $data_ref);
				  });
    $canvas->createWindow($x, $base, -window=>$reg_but, -anchor=>'nw');
    

    ($x, $y, $end_x) = ($coords{'Beta'}{'x'},$coords{'Beta'}{'y'}, $coords{'Beta'}{'end_x'});

    #Warning from candidate registers
  
    my $c;
    print "Printing Candidates...\n";
    for ($c = 0; $c < scalar(@{$data{'Candidates'}}); $c++)
    {
	print "$data{'Candidates'}[$c]\n";
	if ($data{'Candidates'}[$c] =~ /warning/i)
	{
	    my $line = $data{'Candidates'}[$c];
	    my $lbl = $canvas->Label(-text=>$line, -background=>'yellow', -relief=>'groove', -borderwidth=>2);
	    $canvas->createWindow($x, $y, -window=>$lbl, -anchor=>'nw');
	    $y += $y_div;
	}
    }
    print "End\n";



    #Antiparallel registers
    my $anti_lbl = $canvas->Label(-text=>'Antiparallel beta sheets with constraints and assigned NOESY crosspeaks', -background=>'cyan', -relief=>'ridge');
    $canvas->createWindow($x, $y, -window=>$anti_lbl, -anchor=>'nw');

    my ($tx, $ty) = ($x, $y);
    $tx += 20 * $x_div;
    my $ant_but = $canvas->Button(-text=>"Create Sparky List");
    $canvas->createWindow($tx, $ty, -window=>$ant_but, -anchor=>'nw');
    #$canvas->createText($x, $y, -text=>'Antiparallel beta sheets with constraints and assigned NOESY crosspeaks', -anchor=>'nw');
     
    #print "1. placing anti sheets...\n";
    
    $y += 4 * $y_div/5;

    # $data_ref - reference to the hash %data that is from parsing the .sec file
    ($x, $y, $anti_missing_ref, $anti_res_ref) = &place_sheets($canvas, $x, $y, $y_div, $data_ref, $help_ref, $parent, $as_dir, 'Antiparallel');
    $y += $y_div;
    
    #print "done placing anti sheets...\n";

    
    #Parallel registers
    my $para_lbl = $canvas->Label(-text=>'Parallel beta sheets with constraints and assigned NOESY crosspeaks', -background=>'cyan', -relief=>'ridge');
    $canvas->createWindow($x, $y, -window=>$para_lbl, -anchor=>'nw');
    ($tx, $ty) = ($x, $y);
    $tx += 20 * $x_div;
    my $par_but = $canvas->Button(-text=>"Create Sparky List");
    $canvas->createWindow($tx, $ty, -window=>$par_but, -anchor=>'nw');

    #$canvas->createText($x, $y, -text=>'Parallel beta sheets with constraints and assigned NOESY crosspeaks', -anchor=>'nw');
    
    $y += 4 * $y_div/5;

    # $data_ref - reference to the hash %data that is from parsing the .sec file
    ($x, $y, $para_missing_ref, $para_res_ref) = &place_sheets($canvas, $x, $y, $y_div, $data_ref, $help_ref, $parent, $as_dir, 'Parallel');
    $y += $y_div/2;
    $canvas->createText($x, $y, -text=>'#', -fill=>'white');


    #configure sparky buttons
    $ant_but->configure(-command=>sub
			{
			    my ($pklist) = &peaklist_choice($as_dir, $parent);
			    #print "pklist is $pklist\n";
			    my @heading = ($pklist);
			    
			    my %missing = %$anti_missing_ref;
			    my %dist_hash;
			    my $key;
			    
			    foreach $key (keys %missing)
			    {
				$dist_hash{$key} = 1;
				#print "anti 2 : missing{$key} = $missing{$key}\n";
			    }

			    &create_sparky_list_fp($parent, \%dist_hash, $anti_res_ref, $pklist, $as_dir, "Missing anti beta interactions", 'missing_beta_anti');
			    
			});

    $par_but->configure(-command=>sub
			{
			    my ($pklist) = &peaklist_choice($as_dir, $parent);
			    #print "pklist is $pklist\n";
			    my @heading = ($pklist);
			    
			    my %missing = %$para_missing_ref;
			    my %dist_hash;
			    my $key;
			    
			    foreach $key (keys %missing)
			    {
				$dist_hash{$key} = 1;
			    }
			    
			    &create_sparky_list_fp($parent, \%dist_hash, $para_res_ref, $pklist, $as_dir, "Missing para beta interactions", 'missing_beta_para');
			    
			});

    
    $canvas->configure(-scrollregion=>[$canvas->bbox('all')], -confine=>1);
    $canvas->update();
}


sub peaklist_choice
{
    my ($dir, $parent) = @_;
    print "1. dir is $dir\n";
    #get and parse control file
    my ($control_file) = &get_control_file($dir);
    return unless ($control_file);
    print "2. peaklist_choice: control file is $control_file\n";
    my ($comments_ref, $general_ref, $commands_ref, $peaklists_ref, $col_names_ref, $pklst_count) = &openContFile($control_file);

    my @peaklists = keys (%$peaklists_ref);
    
    my $win = $parent->Toplevel();
    $win->configure(-title=>"Choose Peaklist");
    my $lbl = $win->Label(-background=>'white', -text=>"Choose a peaklist", -relief=>'ridge', -borderwidth=>2)->pack(-padx=>3, -pady=>3, -ipadx=>3, -ipadx=>3, -fill=>'x');

    my $frm = $win->Frame(-relief=>'groove', -borderwidth=>2)->pack(-fill=>'both', -expand=>1, -padx=>3, -pady=>3);

    my $temp_pklist;
    my $sel_pklist;
    foreach $pklist (@peaklists)
    {
	$frm->Radiobutton(-text=>$pklist, -variable=>\$sel_pklist, -indicatoron=>1, -value=>$pklist)->pack(-fill=>'x', -padx=>3, -pady=>3);
    }
    
    my $but = $win->Button(-text=>"Select")->pack(-fill=>'x', -padx=>3, -pady=>3);
    $but->configure(-command=>sub
		    {
			$win->destroy();
			
		    });
    $win->waitWindow;
    return ($sel_pklist);
}



sub show_raw_csi
{
    my ($parent, $data_ref) = @_;

    my $window = $parent->Toplevel(-title=>'Raw CSI');
    
    my $stxt = $window->Scrolled('Text', -scrollbars=>'e')->pack(-fill=>'both', -expand=>1);
    my $txt = $stxt->Subwidget('text');
    $txt->configure(-background=>'white');
    
    my %data = %$data_ref;
    my $count = scalar(@{$data{'Raw CSI'}});
    my $i;

    for ($i = 0; $i<$count; $i++)
    {
	$txt->insert('end', "$data{'Raw CSI'}[$i]\n");
    }

    my $cls_but = $window->Button(-text=>'Close', -command=>sub
				  {
				      $window->destroy();
				  })->pack(-fill=>'x', -ipady=>3);
}



sub place_jvalue
{
    my ($canvas, $x, $width, $base, $top, $jvalue) = @_;
    
    my $bar;
    my $div = ($base - $top)/5;

    if ($jvalue <= 4)
    {
	$bar = &place_bar($canvas, $base, ($base - $div), $x, $width, 'orange');
    }
    elsif ($jvalue >= 8)
    {
	$bar = &place_bar($canvas, $base, $top, $x, $width, 'blue');
    }
    else
    {
	my $num = $jvalue - 3.00;
	$bar = &place_bar($canvas, $base, $base - ($div * $num), $x, $width, 'dark red');
    }

    return ($bar);

}

sub place_bar
{
    my ($canvas, $base, $top, $x, $width, $color) = @_;

    my $rect = $canvas->createRectangle($x, $top, $x+$width, $base, -fill=>$color, -outline=>$color);
    $canvas->bind($rect, '<Enter>'=> sub 
		  {
		      $canvas->configure(-cursor=>"crosshair");
		  });
    $canvas->bind($rect, '<Leave>'=>sub
		  {
		      $canvas->configure(-cursor=>"top_left_arrow");
		  });
    return ($rect);
}


sub place_nhs
{
    my ($x, $y, $canvas, $help_ref, $residue) = @_;
    
    my $num = 5;
    
    my $nhs = $canvas->createPolygon($x-$num, $y, $x, $y-$num, $x+$num, $y, $x, $y+$num, -fill=>'purple');
    if ($help_ref and $residue)
    {
	$canvas->bind($nhs, '<Button-1>'=>sub
		      {
			  $$help_ref = "Residue $residue is in NHSlowList";
		      });
	$canvas->bind($nhs, '<Enter>'=> sub 
		      {
			  $canvas->configure(-cursor=>"crosshair");
		      });
	$canvas->bind($nhs, '<Leave>'=>sub
		      {
			  $canvas->configure(-cursor=>"top_left_arrow");
		      });
    }
}

sub place_csi
{
    my ($x, $y, $width, $height, $canvas, $help_ref, $color, $up) = @_;
    
    my $box = $canvas->createRectangle($x, $y, $x+$width, $y+$height, -fill=>$color, -outline=>$color);
    $canvas->bind($box, '<Enter>'=> sub 
		  {
		      $canvas->configure(-cursor=>"crosshair");
		  });
    $canvas->bind($box, '<Leave>'=>sub
		  {
		      $canvas->configure(-cursor=>"top_left_arrow");
		  });
    return ($box);
    
}



sub create_sec_canvas
{
    my ($frame, $parent, $res_count, $frm_width, $data_ref) = @_;
    my %data = %$data_ref;

    my %coords;
    
    my $scr_canvas = $frame->Scrolled('Canvas', -scrollbars=>'se')->pack(-padx=>3, -pady=>3, -fill=>'both', -expand=>1, -ipadx=>5, -ipady=>5);
    my $canvas = $scr_canvas->Subwidget('canvas');

    $canvas->configure(-background=>'white');
    my ($x, $y) = (80, 30);

    #place residues on x axis
    my $x_div = 25;
    $coords{'x_div'} = $x_div;
    my $i;
    
    for ($i = 1; $i<$res_count; $i++)
    {
	my $cx = $x + ($i * $x_div);
	#my $cy = $y - 10;

	$canvas->createText($cx, $y - 25, -text=>$i);
	$canvas->createText($cx, $y - 12, -text=>$data{'AA'}[$i]);
	$canvas->createLine($cx, $y - 5, $cx, $y + 5);
    }
    $canvas->createLine($x, $y, $x + ($res_count * $x_div), $y);

    #place titles on y axis
    my $y_div = 50;
    $coords{'y_div'} = $y_div;
    my $dy = $y;

    my $key;
    foreach $key ('Prediction', 'CSI', "Combined\nCSI,JNOE", "Final Sec Str", 'NHSlowList', 'JFile')
    {
	$dy += $y_div;
	$canvas->createText($x-40, $dy, -text=>$key);
	
	#$canvas->createLine($x, $dy + ($y_div / 2), $x + ($res_count * $x_div), $dy + ($y_div / 2));
	
	#$canvas->createText($x, $dy, -text=>'*');
	
	$coords{$key}{'x'} = $x;
	$coords{$key}{'y'} = $dy;
	$coords{$key}{'end_x'} = $x+($res_count * $x_div);
	
    }
    $canvas->createLine($x, 30, $x, $dy + ($y_div/2));

    #coords for beta sheet representation
    $dy += 2 * $y_div;
    $coords{'Beta'}{'x'} = $x;
    $coords{'Beta'}{'y'} = $dy;
    $coords{'Beta'}{'end_x'} = $x + ($res_count * $x_div);

    $canvas->configure(-scrollregion=>[$canvas->bbox('all')], -confine=>1);
    
    #$canvas->createRectangle($x-40, $dy+50, $x+350,$dy+150, -outline=>'red');
    my $help_msg = "Click on any item to view it's information";

    my $sub_frame = $frame->Frame()->pack(-fill=>'x');
    my $help_lbl = $sub_frame->Label(-textvariable=>\$help_msg, -background=>'white', -foreground=>'black')
	->pack(-side=>'left', -fill=>'x', -expand=>1, -padx=>5, -pady=>5, -ipady=>5, -ipadx=>5);
    my $leg_but = $sub_frame->Button(-text=>'Legend', -command=>sub
				     {
					 &create_sec_legend($parent);
				     })->pack(-side=>'left', -padx=>5, -pady=>5, -ipadx=>3, -ipady=>5);

    
    #$canvas->createWindow($x+135, $dy+100, -window=>$help_lbl);


    return ($canvas, \$help_msg, \%coords);
}


sub create_sec_legend
{
    my ($parent) = @_;
    my ($x, $y) = (0, 0);
    my ($x_div, $y_div) = (100, 120);
    
    my ($end_x, $end_y) = (500, $y + 7 * $y_div);
    
    my $window = $parent->Toplevel(-title=>'Legend');
        
    my $lbl = $window->Label(-text=>'Secondary Structure - Legend', -background=>'white')->pack(-fill=>'x', -padx=>3, -pady=>3, -ipadx=>3, -ipady=>3);
    
    my $scr_canvas = $window->Scrolled('Canvas', -scrollbars=>'osoe')->pack(-fill=>'both', -expand=>1);
    my $canvas = $scr_canvas->Subwidget('canvas');

    #my $canvas = $window->Canvas()->pack(-fill=>'both', -expand=>1);

    $canvas->configure(-background=>'white');
    
    $canvas->createLine($x, $y, $end_x, $y);
    $canvas->createLine($x, $y, $x, $end_y);
    $canvas->createLine($end_x, $y, $end_x, $end_y);
    $canvas->createLine($x + $x_div, $y, $x + $x_div, $end_y);

    #Prediction
    my ($cx, $cy) = ($x + $x_div/2, $y + $y_div/2);
    $canvas->createText($cx, $cy, -text=>'Prediction');
    
    
    $cy = $cy - $y_div/2 + 10;
    my $key;
    foreach $key ('Helix', 'Extended Element')
    {
	$cx = $cx + $x_div;
   	$canvas->createText($cx, $cy, -text=>$key);
    }
    
    $cx = $x + $x_div/2;
    $cy = $y + $y_div;
    foreach $key ('Red', 'Blue')
    {
	$cx += $x_div;
	&place_bar($canvas, $cy, $cy - (3*$y_div/4), $cx, $x_div/4, $key);
    }
    
    $canvas->createLine($x, $cy, $end_x, $cy);
    
    #CSI
    ($cx, $cy) = ($x + $x_div/2, $y + 3*$y_div/2);
    $canvas->createText($cx, $cy, -text=>'CSI');
    
    ($cx, $cy) = ($cx + $x_div, $cy - $y_div/2 + 10);
    $canvas->createText($cx, $cy, -text=>'Helix');
    ($cx, $cy) = ($cx + 2*$x_div, $cy);
    $canvas->createText($cx, $cy, -text=>'Extended Element');
    
    ($cx, $cy) = ($x + 3*$x_div/2, $y + 3*$y_div/2);
    &place_csi($cx-10, $cy-$y_div/4, 20, $y_div/4, $canvas, 0, 'black');
    
    $cx += 2*$x_div;
    &place_csi($cx-10, $cy, 20, $y_div/4, $canvas, 0, 'black');

    $canvas->createLine($x + 3*$x_div/2, $y + 3*$y_div/2, $x + 7*$x_div/2, $y + 3*$y_div/2);
    $canvas->createLine($x, $y + 2*$y_div, $end_x, $y + 2*$y_div);
    
    #Refined CSI
    ($cx, $cy) = ($x + $x_div/2, $y + 5*$y_div/2);
    $canvas->createText($cx, $cy, -text=>'Refined CSI');
    
    ($cx, $cy) = ($cx + $x_div, $cy - $y_div/2 + 10);
    $canvas->createText($cx, $cy, -text=>'Removed residue');
    ($cx, $cy) = ($cx + 2*$x_div, $cy);
    $canvas->createText($cx, $cy, -text=>'Added residue');
    
    ($cx, $cy) = ($cx - 2*$x_div, $y + 5*$y_div/2);
    $canvas->createOval($cx-($x_div/5), $cy-($y_div/6), $cx+($x_div/5), $cy-($y_div/3), -fill=>'red', -outline=>'red');

    ($cx, $cy) = ($cx + 2*$x_div, $cy);
    $canvas->createLine($x + 3*$x_div/2 - 15, $cy, $cx+10, $cy);
    &place_csi($cx - 10, $cy, 20, $y_div/4, $canvas, 0, 'green');
    $canvas->createLine($x, $y + 3*$y_div, $end_x, $y + 3*$y_div);
    
    #NHSlowList
    ($cx, $cy) = ($x + $x_div/2, $y + 7*$y_div/2);
    $canvas->createText($cx, $cy, -text=>'NHSlowList');
    
    ($cx, $cy) = ($cx + $x_div, $cy);
    &place_nhs($cx, $cy, $canvas);
    ($cx, $cy) = ($cx + 10, $cy - 5);
    $canvas->createText($cx, $cy, -text=>" - Residue in NHSlowList File, specified in Control File", -anchor=>'nw');
    
    $canvas->createLine($x, $y + 4*$y_div, $end_x, $y + 4*$y_div);
    
    #JFile
    ($cx, $cy) = ($x + $x_div/2, $y + 9*$y_div/2);
    $canvas->createText($cx, $cy, -text=>'JFile');
    
    ($cx, $cy) = ($x, $y + 4*$y_div + 10);
    foreach $key ("JValue <= 4", "4 < JValue < 8", "8 <= JValue")
    {
	$cx += 3*$x_div/2;
	$canvas->createText($cx, $cy, -text=>$key);
    }
    
    my $help;

    my $div_x = 3*$x_div/4;
    ($cx, $cy) = ($x + 3*$x_div/2, $y + 5*$y_div);
    
    &place_jvalue($canvas, $cx - 10, 20, $cy, $cy - 3*$y_div/3, 4);
    $cx += 2*$div_x;
    &place_jvalue($canvas, $cx-10, 20, $cy, $cy - 3*$y_div/4, 6);
    $cx += 2*$div_x;
    &place_jvalue($canvas, $cx-10, 20, $cy, $cy - 3*$y_div/4, 8);
    

    #Beta Sheet
    $canvas->createLine($x, $y + 5*$y_div, $end_x, $y + 5*$y_div);
    
    ($cx, $cy) = ($x + $x_div/2, $y + 11*$y_div/2);
    $canvas->createText($cx, $cy, -text=>'Beta Strands');
    ($cx, $cy) = ($x + $x_div, $cy - $y_div/4);
    &place_arrow($canvas, $cx+20, $cy, $cx+70, $cy, 'single');
    $canvas->createText($cx+80, $cy, -text=>" - One peak for this interaction", -anchor=>'w');
    $cy += $y_div/3;
    &place_arrow($canvas, $cx+20, $cy, $cx+70, $cy, 'double');
    $canvas->createText($cx+80, $cy, -text=>" - Two peaks for this interaction", -anchor=>'w');
    $cy += $y_div/3;
    $canvas->createLine($cx+20, $cy, $cx+70, $cy, -fill=>'#444488');
    $canvas->createText($cx+80, $cy, -text=>" - No peaks for this interaction", -anchor=>'w');

    $canvas->createLine($x, $y + 6*$y_div, $end_x, $y + 6*$y_div);
    
    #H Bond
    ($cx, $cy) = ($x + $x_div/2, $y + 13*$y_div/2);
    $canvas->createText($cx, $cy, -text=>'H Bonds');
    ($cx, $cy) = ($x + $x_div, $cy - $y_div/4);
    &create_H_bond($cx+10, $cy, $cx+50, $cy, $canvas, \$help, 1, 1);
    $canvas->createText($cx+60, $cy, -text=>"- H Bond found in file", -anchor=>'w');
    $cy += $y_div/2;
    &create_H_bond($cx+10, $cy, $cx+40, $cy, $canvas, \$help);
    $canvas->createText($cx+50, $cy, -text=>" - H Bond not found in file", -anchor=>'w');

    $canvas->createLine($cx + 2* $x_div, $y + 6 * $y_div, $cx + 2 * $x_div, $y + 7 * $y_div);
    
    ($cx, $cy) = ($cx+ 9 * $x_div/4, $cy - $y_div/2);
			
    $canvas->createText($cx,$cy, -text=>"H");
    &box_H($cx,$cy, $canvas, 1);
    $canvas->createText($cx+15, $cy, -text=>"- H found in NHSlowList", -anchor=>'w');
    $cy += $y_div/2;
    $canvas->createText($cx,$cy, -text=>"H");
    &box_H($cx,$cy, $canvas, 0);
    $canvas->createText($cx+15, $cy, -text=>"- H not found in NHSlowList", -anchor=>'w');

    $canvas->createLine($x, $end_y, $end_x, $end_y);
    $canvas->createText($x, $y, -text=>"@", -fill=>'red');
    $canvas->createText($end_x, $end_y, -text=>"@", -fill=>'green');
    
    $canvas->configure(-scrollregion=>[$x, $y, $end_x, $end_y], -confine=>1);
    $scr_canvas->configure(-width=>$end_x, -height=>$end_y);
    
    $window->minsize($end_x, $end_y);
    $canvas->update;
    $window->update;
}


sub get_sec_file
{
    my ($dir, $parent) = @_;

    unless (opendir(DIR, $dir))
    {
	&err_msg("Cannot open AS directory \"$dir\"", $parent);
	return;
    }

    my @contents = readdir(DIR);
    close(DIR);

    my ($sec) = grep {/\.sec$/} @contents;
    if ($sec)
    {
	$sec = File::Spec->rel2abs($sec, $dir);
    }
    return ($sec);
    
}

return 1;
