my $bg1 = '#AAAAAA';
my $colorA = '#ECDFFF';
my $colorB = '#FFFCDF';


sub show_interactions
{
    my ($parent, $ql_file) = @_;
    my (%vars);
    
    #print "show_interactions: start, ql_file is $ql_file\n";
    #data_ref and heading_ref are from parsing the ql_file
    my ($data_ref, $heading_ref, $residue_ref) = &parse_ql($ql_file, $parent);
    
    #create interface
    my $window = $parent->Toplevel;
    $window->configure(-title=>"Query QL File");
    my $src_but;
    &show_int_gui($window, \%vars, \$src_but, $ql_file);

    my $dir = dirname($ql_file);
    #print "ql_file is $ql_file, dir is $dir\n";
    
    #configure search button
    $src_but->configure(-command=>sub
			{
			    if (&check_vars(\%vars, $residue_ref, $window))
			    {
				$window->raise if Tk::Exists($window);
				#print "show_interactions: check successful\n";
				foreach $key (%vars)
				{
				    if ($vars{$key})
				    {
					#print "show_interactions: vars{$key} = $vars{$key}\n";
				    }
				}

				&search_hash_for_i($data_ref, $heading_ref, $residue_ref, \%vars, $window, $dir, $ql_file);
				#print "show_interactions: done creating results!\n";
				
			    }
			    $window->raise if Tk::Exists($window);
			});
    
}


#allow cases
# by residue 1(hx1) and/or atoms 
# by residue 2(hx2) and/or atoms 
# by residue 1 and 2, and/or atoms  
# by residue 1,2 and/or atoms and distances

# by distances
# by distances and atoms (1 and 2)   
sub check_vars
{
    my ($vars_ref, $residue_ref, $parent) = @_;
    my $key;
    #print "In check vars...\n";

    unless (($vars_ref->{'Residue 1'}) || ($vars_ref->{'distance'}) || ($vars_ref->{'Residue 2'}))
    {
	#unless ($vars_ref->{'distance'})
	{
	    #print "Neither number 1 nor distance entered\n";
	    &err_msg("Please enter either a number for Residue 1,2 or the distance", $parent);
	    return 0;
	}
    }

    foreach $key ('Residue 1', 'Residue 2')
    {
	if ($vars_ref->{$key})
	{
	    unless ($vars_ref->{$key} =~ /^\d+$/)
	    {
		print "check_vars: $vars_ref->{$key} contains a non-digit\n";
		$key =~ /Residue (\d)$/;
		&err_msg("Invalid entry for Residue $1: Please enter a number", $window);
		return 0;
	    }
	}
    }
     
   if ($vars_ref->{'distance'})
    {
	unless ($vars_ref->{'distance'} =~ /^[\d\.]+$/)
	{
	    #print "distance is not a number\n";
	    &err_msg("Distance should be a number", $parent);
	    return 0;
	}
    }

    if ($vars_ref->{'Residue 1'})
    {
	unless ($residue_ref->[$vars_ref->{'Residue 1'}])
	{
	    &err_msg("Residue $vars_ref->{'Residue 1'} does not exist", $parent);
	    return 0;
	}
    }
    if ($vars_ref->{'Residue 2'})
    {
	unless ($residue_ref->[$vars_ref->{'Residue 2'}])
	{
	    &err_msg("Residue $vars_ref->{'Residue 2'} does not exist", $parent);
	    return 0;
	}
    }
    return 1;
    
}

sub match_atom
{
    my ($atom, $filter) = @_; 

    return 1 if ($filter eq '\w');

    $filter = uc($filter); 

    # Q<=>H 
    if ($filter =~ /^Q/i) #exact compare 
    {
	return ($atom eq $filter); 
    }

    if ($atom =~ /^Q/i)
    {
	$atom =~ s/^Q/H/i; 
	$atom .= '*';
    }

    if ($filter =~ /\*/i) 
    {
	$filter =~ s/\*//; 
	return ($atom =~ /$filter/i); 
    } 
    return ($atom eq $filter);     
}

sub search_hash_for_i
{
    my ($data_ref, $heading_ref, $residue_ref, $vars_ref, $window, $dir, $ql_file) = @_;
    
    #data_ref and heading_ref are from the ql_file

    my @residue = @$residue_ref;
    my @heading = @$heading_ref;
    my @data = @$data_ref;

    my $i = $vars_ref->{'Residue 1'};
    my $j = $vars_ref->{'Residue 2'};
    my $i_atom_filter = $vars_ref->{'Residue 1 Atom'};
    my $j_filter = $vars_ref->{'Residue 2'};
    my $j_atom_filter = $vars_ref->{'Residue 2 Atom'};
    my $distance_comp_filter = $vars_ref->{'dist comp'};
    my $distance = $vars_ref->{'distance'};

    my $query = &construct_query($vars_ref);
    #print "search hash for i: query is $query\n";
    
    #print "search_hash_for_i: distance_comp_filter is $distance_comp_filter, and distance is $distance\n";

    my @dist_hash_array;
    my $k;
    my $key;

    if ($i || $j) #cases by residue 1 and/or atoms 
    {
 	#print "search_hash_for_i: Residue 1 given as $i\n";
	
	#search all sections - peaklists

	for ($k = 0; $k<=$#heading; $k++)
	{
	    my %dist_hash;

	    if ($i)
	    {
		&getLeftCountofi($data_ref, $i, $k, 0, \%dist_hash, $residue_ref, "", $j_filter, "", $distance_comp_filter, $distance, 1);
	    } elsif ($j)
	    {
		&getRightCountofi($data_ref, $j, $k, 0, \%dist_hash, $residue_ref, "", "", "", $distance_comp_filter, $distance, 1);
	    }

	    if (($i_atom_filter =~ /\w/) || ($j_atom_filter =~ /\w/)) #by atoms too 
	    {
		if ($j_atom_filter eq '')
		{
		    $j_atom_filter = '\w'; 
		}
		if ($i_atom_filter eq '')
		{
		    $i_atom_filter = '\w'; 
		}

		%dist_copy = %dist_hash; 
		foreach $kk (keys %dist_copy)
		{
		    my ($ii, $ii_atom, $jj, $jj_atom) = $kk =~ /(\d+)-([\w\W]+) (\d+)-([\w\W]+)/; 

		    #print "$ii $ii_atom $jj $jj_atom $i_atom_filter $j_atom_filter\n";

		    my $d = 0; 
		    if ($ii == $jj)
		    {
			if (!(((&match_atom($ii_atom, $i_atom_filter)) && (&match_atom($jj_atom, $j_atom_filter))) ||
			      ((&match_atom($jj_atom, $i_atom_filter)) && (&match_atom($ii_atom, $j_atom_filter)))))
			{
			    $d = 1; 
			}
		    }
		    elsif (!(((&match_atom($ii_atom, $i_atom_filter)) && (&match_atom($jj_atom, $j_atom_filter)))))	
		    {
			$d = 1;  
		    }
		    delete $dist_hash{$kk} if ($d ==1); 
		}
	    }
	    #print "Residue 1 given, printing dist_hash\n";
	    #&print_hash(\%dist_hash, $k);
    
	    $dist_hash_array[$k] = \%dist_hash;
	    #&show_results($window, \%dist_hash, $heading_ref, $residue_ref, $query, $data_ref, $dir, $ql_file);
	    #print "search_hash_for_i: Residue 1 given - created dist_hash_array[$k]\n";
	}
	
	#display info from dist_hash_array
	&show_results($window, \@dist_hash_array, $heading_ref, $residue_ref, $query, $data_ref, $dir, $ql_file);
	return;
    }
    elsif ($distance) #by distances and/or atoms 
    {
	#print "search_hash_for_i: Number 1 not given, distance given as $distance\n";
	#no atom specified, search by distance
	for ($k = 0; $k<=$#heading; $k++)
	{
	    my %dist_hash;
	    for ($i = 0; $i<=$#residue; $i++)
	    {
		#print "search_hash_for_i - section $k: residue[$i] = $residue[$i]\n";
		next unless ($residue[$i]);

		#print "residue $i, filtering distances $distance_comp_filter than $distance\n";
		&getLeftCountofi($data_ref, $i, $k, 0, \%dist_hash, $residue_ref, "", $j_filter, "", $distance_comp_filter, $distance, 1);
		&getRightCountofi($data_ref, $i, $k, 0, \%dist_hash, $residue_ref, "", $j_filter, "", $distance_comp_filter, $distance, 1);
	    }
	    if ($i_atom_filter =~ /\w/) #by atoms too 
	    {
		if ($j_atom_filter eq '')
		{
		    $j_atom_filter = '\w'; 
		}

		%dist_copy = %dist_hash; 
		foreach $kk (keys %dist_copy)
		{
		    my ($ii, $ii_atom, $jj, $jj_atom) = $kk =~ /(\d+)-([\w\W]+) (\d+)-([\w\W]+)/; 
		    
		    #print "$i $i_atom $j $j_atom $i_atom_filter $j_atom_filter\n";
		    if (!(((&match_atom($ii_atom, $i_atom_filter)) && (&match_atom($jj_atom, $j_atom_filter))))) # ||
			 # ((&match_atom($jj_atom, $i_atom_filter)) && (&match_atom($ii_atom, $j_atom_filter)))))
		    {
			#print "$kk deleted\n";
			#print "$ii_atom $jj_atom $i_atom_filter -$j_atom_filter-\n";
			#print "--" . ($ii_atom =~ /$i_atom_filter/i) . "--" . ($jj_atom =~ /$j_atom_filter/i) . "\n\n"; 
			delete $dist_hash{$kk}; 
		    }
		    
		}
	    }
	    
	    $dist_hash_array[$k] = \%dist_hash;
	    #&show_results($window, \%dist_hash, $heading_ref, $residue_ref, $query, $data_ref, $dir, $ql_file);
	    #print "search_hash_for_i: distance given - created dist_hash_array[$k]\n";
	}

	#display info from dist_hash_array
	&show_results($window, \@dist_hash_array, $heading_ref, $residue_ref, $query, $data_ref, $dir, $ql_file);
	return;
    }
    else
    {
	#print "search_hash_for_i: Neither number 1 nor distance entered\n";
	&err_msg("Please enter data", $window);
	return;
    }
}


sub show_results
{
    my ($parent, $dist_hash_array_ref, $heading_ref, $residue_ref, $query, $ql_data_ref, $dir, $ql_file) = @_;
    #print "show_results:start\n";

    my $nb_bg = '#D1E2Ef';
    my $nb_back = '#78A1BF';

    my @dist_hash_array = @$dist_hash_array_ref;
    my @heading = @$heading_ref;
    my @residue = @$residue_ref; 

    my $bg2 = '#FFECBF';

    my $top = $parent->Toplevel;
    $top->configure(-title=>"Interactions results");
    #$top->minsize(qw/500 500/);
    
    my $pklst_head = "\n";;
    my $i;

    $pklst_head = "\nSearching for $query\n";

    for ($i = 0; $i<=$#heading; $i++)
    {
	$pklst_head = $pklst_head."Peaklist $i - $heading[$i]\n";
    }
    
    my $frame = $top->Frame(-background=>$bg2)->pack(-fill=>'both', -expand=>1);
    my $label = $frame->Label(-relief=>'ridge', -borderwidth=>1, -text=>$pklst_head, -background=>$bg2)->pack(-side=>'top', -fill=>'x');
    
    #my $nb = $frame->NoteBook(-backpagecolor=>$bg2, -background=>$bg2, -inactivebackground=>)->pack(-fill=>'both', -expand=>1, -padx=>2, -pady=>2);
    my $nb = $frame->NoteBook(-backpagecolor=>$bg2, -background=>$bg2)->pack(-fill=>'both', -expand=>1, -padx=>2, -pady=>2);

#for each peaklist - tab
    my %fp_but; 

for ($i=0; $i<=$#heading; $i++)
{
    my $pb = $nb->add("pk-$i", -label=>"PeakList $i"); 
    $pb->configure(-background=>$bg2); 
    
    my $res_frame = $pb->Frame(-background=>'white')->pack(-fill=>'both', -expand=>1);
    
    my $table = &createTable($res_frame, $dist_hash_array_ref, $residue_ref);
   
    my $row = 1;
    my $A = 1;
    my @data_array;

    #print "show_results: putting in data\n";
    #for ($k = 0; $k<=$#dist_hash_array; $k++)
    $k = $i;
    {
	#print "show_results: section $k\n";
	my %dist_hash = %{$dist_hash_array[$k]};
	
	foreach $key (sort keys %dist_hash)
	{
	    #print "show_results: dist_hash{$key} = $dist_hash{$key}\n";
	    if ($A)
	    {
		&add_row($table, $key, $dist_hash{$key}, $k, $row, $colorA, $residue_ref);
	    }
	    else
	    {
		&add_row($table, $key, $dist_hash{$key}, $k, $row, $colorB, $residue_ref);
	    }
	    $A = !$A;
	    $row++;
	}
    }
    sort_column(0, $table, $dist_hash_array_ref, $residue_ref); #sort by distance

    $fp_but{$heading[$i]} = $pb->Button(-text=>"Create Sparky List as $heading[$i]")->pack(-fill=>'x', -padx=>5, -pady=>5);
}

$top->raise;

foreach $key (keys %fp_but)
{
    my $temp_key = $key;
    $fp_but{$temp_key}->configure(-command=>sub
		       {
			   #my ($pklist) = &peaklist_choice($dir, $parent);
			   my ($pklist) = $temp_key; 

			   print "Peaklist selected is $pklist\n";
			   my @dist_hash_array = @$dist_hash_array_ref;
			   my ($k, $j);
			   for ($k = 0; $k<=$#heading; $k++)
			   {
			       if ($heading[$k] eq $pklist)
			       {
				   $j = $k;
			       }
			   }
			   my %dist_hash = %{$dist_hash_array[$j]};

			   #my $fp_name = $pklist . "_fp" . get_timestamp(); 
			   #$fp_name =~ s/\//_/g; 

			   #this function is in the file "create_sparky_list.pl"
			   &create_sparky_list_fp($top, \%dist_hash, $residue_ref, $pklist, $dir, "Peaklist : $pklist", 'false_positive');
		       });
}

$top->waitWindow; 
    #print "show_results: end\n";
    
}


sub show_int_gui
{
    my ($window, $vars, $src_but, $ql_file) = @_;
        
    my $full = $window->Frame(-background=>$bg1)->pack(-fill=>'both', -expand=>1);

    my $lbl_frm = $full->Frame(-background=>$bg1)->pack(-fill=>'x', -expand=>1);
    my $title_lbl = $lbl_frm->Label(-text=>"Interactions from file \"$ql_file\"")->pack(-fill=>'x', -expand=>1);
    
    my $res_frm = $full->Frame(-background=>$bg1)->pack(-fill=>'both', -expand=>1);
    my $res1_frm = $res_frm->Frame(-background=>$bg1, -relief=>'ridge', -borderwidth=>1)->pack(-side=>'left', -fill=>'both', -expand=>1, -padx=>3, -pady=>5);
    my $res2_frm = $res_frm->Frame(-background=>$bg1, -relief=>'ridge', -borderwidth=>1)->pack(-side=>'left', -fill=>'both', -expand=>1, -padx=>3, -pady=>5);

    my $res1_lbl = $res1_frm->Label(-text=>'Residue 1(hx1):')->pack(-fill=>'x', -expand=>1);
    my $res1_lbl_frm = $res1_frm->Frame(-background=>$bg1)->pack(-fill=>'both', -expand=>1);
    my $res1_num_lbl = $res1_lbl_frm->Label(-background=>$bg1, -text=>'Number')->pack(-fill=>'x', -expand=>1, -side=>'left');
    my $res1_nam_lbl = $res1_lbl_frm->Label(-background=>$bg1, -text=>'Atom name')->pack(-fill=>'x', -expand=>1, -side=>'left');

    my $res1_ent_frm = $res1_frm->Frame(-background=>$bg1)->pack(-fill=>'both', -expand=>1);
    my $res1_ent1 = $res1_ent_frm->Entry(-background=>'white', -textvariable=>\$vars->{'Residue 1'})->pack(-side=>'left');
    my $res1_ent2 = $res1_ent_frm->Entry(-background=>'white', -textvariable=>\$vars->{'Residue 1 Atom'})->pack(-side=>'left');

    my $res2_lbl = $res2_frm->Label(-text=>'Residue 2 (hx2):')->pack(-fill=>'x', -expand=>1);
    my $res2_lbl_frm = $res2_frm->Frame(-background=>$bg1)->pack(-fill=>'both', -expand=>1);
    my $res2_num_lbl = $res2_lbl_frm->Label(-background=>$bg1, -text=>'Number')->pack(-fill=>'x', -expand=>1, -side=>'left');
    my $res2_nam_lbl = $res2_lbl_frm->Label(-background=>$bg1, -text=>'Atom name')->pack(-fill=>'x', -expand=>1, -side=>'left');

    my $res2_ent_frm = $res2_frm->Frame(-background=>$bg1)->pack(-fill=>'both', -expand=>1);
    my $res2_ent1 = $res2_ent_frm->Entry(-background=>'white', -textvariable=>\$vars->{'Residue 2'})->pack(-side=>'left');
    my $res2_ent2 = $res2_ent_frm->Entry(-background=>'white', -textvariable=>\$vars->{'Residue 2 Atom'})->pack(-side=>'left');

    my $dist_frm = $full->Frame(-background=>$bg1)->pack(-fill=>'both', -expand=>1);
    my $dist_lbl = $dist_frm->Label(-text=>'Summation Distance : ', -background=>$bg1)->pack(-side=>'left', -padx=>3, -pady=>3);
    my $dist_opt = $dist_frm->Optionmenu(-options=>['<=', '>=', '='], -variable=>\$vars->{'dist comp'})->pack(-side=>'left', -padx=>3, -pady=>3);
    my $dist_ent = $dist_frm->Entry(-background=>'white', -textvariable=>\$vars->{'distance'})->pack(-side=>'left', -padx=>3, -pady=>3);

    my $src_frm = $full->Frame(-background=>$bg1)->pack(-fill=>'both', -expand=>1, -pady=>3);
    $$src_but = $src_frm->Button(-text=>'Search')->pack();
}


sub print_hash
{
    my ($hash_ref, $section) = @_;

    print "PRINTING HASH...\n";
    print "Section number $section\n";
    
    my $key;

    foreach $key (keys %$hash_ref)
    {
	print "dist_hash{$key} = $hash_ref->{$key}\n";
    }
    print "Done printing hash\n";
}


sub add_row
{
    my ($table, $line, $distance, $section, $row, $color, $residue_ref) = @_;
    #print "adding row $row, line is $line\n";

    $line =~ /(\d+)-(.+) (\d+)-(.+)/;

    $table->put($row, 0, &createLabel($table, $1.$residue_ref->[$1]." - $2", $color));
    $table->put($row, 1, &createLabel($table, $3.$residue_ref->[$3]." - $4", $color));
    $table->put($row, 2, &createLabel($table, $distance, $color));
    #$table->put($row, 3, &createLabel($table, $section, $color));
}


sub createLabel
{
    my ($table, $data, $color) = @_;

    my $label = $table->Label(-text=>$data, -background=>$color);
    return $label;
}



sub sort_column
{
    my ($column, $table, $dist_hash_array_ref, $residue_ref) = @_;

    #print "sort_column: clicked column $column\n";

    my @residue = @$residue_ref;
    my @dist_hash_array = @$dist_hash_array_ref;
    
    my ($row, $col);
    my $A = 1;
    
    
    my $total_rows = $table->totalRows;
    my $total_cols = $table->totalColumns;

    my @data_array;
    $data_array[0][0] = '';
    
    for ($row = 1; $row<$total_rows; $row++)
    {
	my $colA = 0;
	for ($col = 0; $col<$total_cols; $col++)
	{
	    my $label = $table->get($row, $col);
	    my $txt = $label->cget(-text);
	    if ($col == 0 or $col == 1)
	    {
		$txt =~ /(\d+)\w - ([\w*]+)/;
		$data_array[$row][$colA] = $1;
		$colA++;
		$data_array[$row][$colA] = $2;
	    }
	    else
	    {
		$data_array[$row][$colA] = $txt;
	    }
	    $colA++;
	}
	
    }
    
    #print "sort_column: sorting column $column\n";
    my @sorted_table;
   
    if ($column == 0)
    {
	@sorted_table = sort {$a->[0] <=> $b->[0] || $a->[1] cmp $b->[1]} @data_array;
	
    }
    elsif ($column == 1)
    {
	@sorted_table = sort {$a->[2] <=> $b->[2] || $a->[3] cmp $b->[3]} @data_array;
    }
    else
    {
	(@sorted_table) = sort {$a->[$column+2] <=> $b->[$column+2]} @data_array;
    }

    if (@sorted_table)
    {
	#put data in table
	#print "sort_column: putting data in table\n";
	for ($row = 1; $row<$total_rows; $row++)
	{
	    my $colA = 0;
	    for ($col = 0; $col<$total_cols; $col++)
	    {
		if ($A)
		{
		    if ($col == 0 or $col == 1)
		    {
			my $res = $sorted_table[$row][$colA];
			$colA++;
			my $data = $res.$residue_ref->[$res]." - ".$sorted_table[$row][$colA];
			$table->put($row, $col, &createLabel($table, $data, $colorA));
			
		    }
		    else
		    {
			$table->put($row, $col, &createLabel($table, $sorted_table[$row][$colA], $colorA));
		    }
		    $colA++;
		}
		else
		{
		    if ($col == 0 or $col == 1)
		    {
			my $res = $sorted_table[$row][$colA];
			$colA++;
			my $data = $res.$residue_ref->[$res]." - ".$sorted_table[$row][$colA];
			$table->put($row, $col, &createLabel($table, $data, $colorB));
			
		    }
		    else
		    {
			$table->put($row, $col, &createLabel($table, $sorted_table[$row][$colA], $colorB));
		    }
		    $colA++;
		}
	    }
	    $A = !$A;
	}
    }


}


sub createTable
{
    my ($frame, $dist_hash_array_ref, $residue_ref) = @_;
    my $bg2 = '#FFECBF';

    $frame->configure(-background=>$bg2);
    my $table = $frame->Table(
			      -rows=>15,
			      -columns=>4,
			      -fixedrows=>1,
			      -relief=>'sunken',
			      -background=>$bg2,
			      -scrollbars=>'e')->pack(-side=>'top', -fill=>'both', -expand=>1);

    #column headings
    my @col_but;
    my $column = 0;
    my $row = 1;
    my $k;
    my $key;
    
    #print "show_results: creating column headings\n";
    foreach $title ('Residue 1 - Residue 1 Atom', 'Residue 2 - Residue 2 Atom', "Summation Distance\nbetween the two atoms") #, 'Peaklist number')
    {
	my $p = $column;
	$col_but[$column] = $table->Button(-text=>$title, -background=>'#746091', -command=>sub {&sort_column($p, $table, $dist_hash_array_ref, $residue_ref);});
	$table->put(0, $column, $col_but[$column]);
	$column++;
    }

    return ($table);
}

sub construct_query
{
    my ($vars_ref) = @_;

    my $query;
    my $key;
    
    foreach $key (keys %$vars_ref)
    {
	if ($vars_ref->{$key})
	{
	    if ($key eq 'distance')
	    {
		$query = $query."$key $vars_ref->{'dist comp'} $vars_ref->{'distance'}, ";
	    }
	    else
	    {
		unless ($key eq 'dist comp')
		{
		    $query = $query."$key = $vars_ref->{$key}, ";
		}
	    }
	}
	
	
    }
    if ($query)
    {
	$query =~ /(.+), $/;
	$query = ($1);
    }

    return ($query);
}


return 1;
