
# open AS output directory
sub OpenASOutDir
{
    my ($main, $root) = @_;
    
    my $win = $main->Toplevel(-title=>'Open Output Directory');
    $win->minsize(qw/800 600/);

    my $lbl_frm = $win->Frame(-relief=>'ridge', -borderwidth=>2)->pack(-fill=>'x', -pady=>2);
    
    my $dir_lbl = $lbl_frm->Label(-text=>"Directory: $root")->pack(-padx=>2, -pady=>2, -side=>'top');
    
    my $nb_frm = $win->Frame()->pack(-fill=>'both', -expand=>1, -side=>'top');

    my $nb = $nb_frm->NoteBook()->pack(-fill=>'both', -expand=>1, -padx=>2, -pady=>2);

    
    #Summary tab
    my $sum_pg = $nb->add('summary', -label=>'Summary');
    
    #button to open .ovw file
    
    #Summary page with the plot of peaks per cycle
    &load_sum_pg($win, $root, $sum_pg);
    
    #RPF tab
    unless (opendir(DIR, $root)) {
	&err_msg("Cannot open ASDP directory \"$root\"", $parent);
	return;
    }
    my @contents = readdir(DIR);
    close(DIR);
    if ((grep {/_DP.ovw/} @contents) and (grep {/\.QP/} @contents)) {
        my $rpf_pg = $nb->add('_DP', -label=>'RPF scores');
        my $rpf_frame = $rpf_pg->Frame(-background=>'#78A1BF')->pack(-fill=>'both', -expand=>1, -padx=>2, -pady=>2);
        &open_rpf_file_in_frame($win, $root, $rpf_frame);
    }

    #Initial Fold Analysis tab
    my $init_pg = $nb->add('initial', -label=>'Initial Fold Analysis');
    my $init_nb = $init_pg->NoteBook()->pack(-fill=>'both', -expand=>1, -padx=>2, -pady=>2);
    my $sec_pg = $init_nb->add('sec_str', -label=>'Secondary Structure');
    
    #Contact Map tab
    my $cnt_pg = $nb->add('contact', -label=>"Contact Maps");

    #Unassigned peaks tab
    my $unas_pg = $nb->add('unassign', -label=>'Checklist');
    $nb->pageconfigure('unassign', -createcmd=>sub
		       {
			   $win->configure(-cursor=>['watch','red','green']);
			   my $each;
			   my @children = &get_children($win);
			   foreach $each (@children)
			   {
			       $each->configure(-cursor=>['watch','red','green']);
			   }

			   #$win->update;

			   #print "loading checklist, children were @children\n";
			   
			   #loads unassigned peaks (checklist)
			   &load_unassign($unas_pg, $win, $root);	   
		       });

    #assignments tab
    my $asgn_pg = $nb->add('assignment', -label=>'NOE Assignments');
    my $asgn_nb = $asgn_pg->NoteBook()->pack(-fill=>'both', -expand=>1, -padx=>2, -pady=>2);

    #all peaks
    my $all_pg = $asgn_nb->add('all', -label=>'All peaks');
    my $all_lbl = $all_pg->Label(-text=>"Double-Click on a cycle", -relief=>'ridge')->pack(-fill=>'x', -padx=>5, -pady=>5);
    my $all_lbx = $all_pg->Listbox(-selectmode=>'single', -background=>'white')->pack(-fill=>'both', -expand=>1, -padx=>5, -pady=>5);
    &add_dirs($all_lbx, $root);   #shows the list of cycle directories

    #peak with one assignment
    my $one_pg = $asgn_nb->add('one', -label=>'Peaks with one assignment');
    my $one_lbl = $one_pg->Label(-text=>"Double-Click on a cycle", -relief=>'ridge')->pack(-fill=>'x', -padx=>5, -pady=>5);
    my $one_lbx = $one_pg->Listbox(-selectmode=>'single', -background=>'white')->pack(-fill=>'both', -expand=>1, -padx=>5, -pady=>5);
    &add_dirs($one_lbx, $root);   #shows the list of cycle directories

    #peaks with multiple assignments
    my $multi_pg = $asgn_nb->add('multi', -label=>'Peaks with multiple assignments');
    my $multi_lbl = $multi_pg->Label(-text=>"Double-Click on a cycle", -relief=>'ridge')->pack(-fill=>'x', -padx=>5, -pady=>5);
    my $multi_lbx = $multi_pg->Listbox(-selectmode=>'single', -background=>'white')->pack(-fill=>'both', -expand=>1, -padx=>5, -pady=>5);
    &add_dirs($multi_lbx, $root);   #shows the list of cycle directories

    #peaks with zero assignments
    my $zero_pg = $asgn_nb->add('zero', -label=>'Peaks not assigned');
    my $zero_lbl = $zero_pg->Label(-text=>"Double-Click on a cycle", -relief=>'ridge')->pack(-fill=>'x', -padx=>5, -pady=>5);
    my $zero_lbx = $zero_pg->Listbox(-selectmode=>'single', -background=>'white')->pack(-fill=>'both', -expand=>1, -padx=>5, -pady=>5);
    &add_dirs($zero_lbx, $root);   #shows the list of cycle directories

    #Structures tab
    my $str_pg = $nb->add('str', -label=>"Structures");
    my $str_lbl = $str_pg->Label(-text=>"Double-Click on a cycle to display it's structure", -relief=>'ridge')->pack(-fill=>'x', -padx=>5, -pady=>5);
    my $str_lbx = $str_pg->Listbox(-selectmode=>'single', -background=>'white')->pack(-fill=>'both', -expand=>1, -padx=>5, -pady=>5);
    &add_dirs($str_lbx, $root);   #shows the list of cycle directories

    #Log File Tab
    my $log_pg = $nb->add('log', -label=>'Log File');
    my $s_log_txt = $log_pg->Scrolled('Text', -scrollbars=>'e', -background=>'white', -height=>10)->pack(-fill=>'both', -expand=>1, -padx=>3, -pady=>5);
    my $log_txt = $s_log_txt->Subwidget('text');
    my $log_upd = $log_pg->Button(-text=>'Update')->pack(-fill=>'x', -padx=>3, -pady=>3);

    #Tree view tab
    my $view_pg = $nb->add('view', -label=>'Tree View');
    my $tree;

    &create_tree($view_pg, $win, $root, $tree);

    #Contact Maps
    my $lbl = $cnt_pg->Label(-text=>"Double-Click on a Cycle to display it's Contact Map", -relief=>'groove', -borderwidth=>2)->pack(-fill=>'x', -padx=>5, -pady=>5, -ipadx=>5, -ipady=>5);

    my $mag_frm = $cnt_pg->Frame()->pack(-fill=>'x', -pady=>5, -padx=>5);
    my $mag_lbl = $mag_frm->Label(-text=>'Magnification: ')->pack(-side=>'left', -padx=>5);
    
    my @choices = ('x 1', 'x 2', 'x 3', 'x 4', 'x 5', 'x 6', 'x 7');
    my $mag = 'x 4';
    my $opt_menu = $mag_frm->Optionmenu(-options=>\@choices, -variable=>\$mag)->pack(-side=>'left', -padx=>5, -pady=>5);

    my $lbx = $cnt_pg->Listbox(-selectmode=>'single', -background=>'white')->pack(-fill=>'both', -expand=>1, -padx=>5, -pady=>5);
    &add_dirs($lbx, $root);

    $lbx->bind('<Double-Button-1>'=>sub
	       {
		   #get upl file from directory
		   my $dir = $lbx->get($lbx->curselection());
		   
		   return unless($dir);
		   $dir = File::Spec->rel2abs($dir, $root);
		   
		   my ($hbond_upl, $prot_upl, $format) = &get_upls($dir, $win);
		   unless ($hbond_upl and $prot_upl)
		   {
		       &err_msg("No upl files in directory \"$dir\"", $win);
		       return;
		   }

		   my (@data, @residue);
		   my $data_ref = \@data;
		   my $residue_ref = \@residue;
		   my ($start_res, $end_res);

		   #dyana format upl file
		   ($data_ref, $residue_ref) = &parse_upl($hbond_upl, $data_ref, $residue_ref, $win, $start_res, $end_res, $format);
		   ($data_ref, $residue_ref) = &parse_upl($prot_upl, $data_ref, $residue_ref, $win, $start_res, $end_res, $format);
		   

		   if ($residue_ref)
		   {
		       $mag =~ /(\d)$/;
		       $mag_num = $1;
		       #print "mag is $mag, and mag_num is $mag_num\n";
		       my $scale = $mag_num + 2;
		       
		       #print "scale is $scale\n";

		       #creates contact map for this cycle
		       &create_contact_map($data_ref, $residue_ref, $win, $dir, $scale, $start_res, $end_res);
		   }

	       });

    #all peaks
   $all_lbx->bind('<Double-Button-1>'=>sub
		     {
			 my $dir = $lbx->get($all_lbx->curselection());
			 
			 return unless($dir);
			 $dir = File::Spec->rel2abs($dir, $root);
			 
			 my (@assOrder_files) = &get_assOrders($dir, $win);

			 my %data_from_file; #with assignment information 
			 
			 #print "create_query: From $dir:\nassOrder_files are @assOrder_files\n";
			 
			 foreach (@assOrder_files)
			 {
			     my $data_ref;
			     ($data_ref) = &parse_assOrder_multiple($_, $win, 0); 

			     $pklst = basename($_); 
			     $pklst =~ s/_assOrder//; 

			     $data_from_file{$pklst} = $data_ref;
			 }
			 &display_multiple(\%data_from_file, $dir, $root, $win);			 
		     }); 

    #peaks with one assignments
   $one_lbx->bind('<Double-Button-1>'=>sub
		     {
			 my $dir = $lbx->get($one_lbx->curselection());
			 
			 return unless($dir);
			 $dir = File::Spec->rel2abs($dir, $root);
			 
			 my (@assOrder_files) = &get_assOrders($dir, $win);

			 my %data_from_file;
			 
			 #print "create_query: From $dir:\nassOrder_files are @assOrder_files\n";
			 
			 foreach (@assOrder_files)
			 {
			     my $data_ref;
			     ($data_ref) = &parse_assOrder_multiple($_, $win, 1); 

			     $pklst = basename($_); 
			     $pklst =~ s/_assOrder//; 

			     $data_from_file{$pklst} = $data_ref;
			 }
			 &display_multiple(\%data_from_file, $dir, $root, $win);			 
		     });
 
   $zero_lbx->bind('<Double-Button-1>'=>sub
		     {
			 my $dir = $lbx->get($zero_lbx->curselection());
			 
			 return unless($dir);
			 $dir = File::Spec->rel2abs($dir, $root);
			 
			 my (@assOrder_files) = &get_assOrders($dir, $win);

			 my %data_from_file;
			 
			 #print "create_query: From $dir:\nassOrder_files are @assOrder_files\n";
			 
			 foreach (@assOrder_files)
			 {
			     my $data_ref;
			     ($data_ref) = &parse_assOrder_multiple($_, $win, 3); 

			     $pklst = basename($_); 
			     $pklst =~ s/_assOrder//; 

			     $data_from_file{$pklst} = $data_ref;
			 }
			 &display_multiple(\%data_from_file, $dir, $root, $win);			 
		     });
    #peaks with multiple assignments
    $multi_lbx->bind('<Double-Button-1>'=>sub
		     {
			 my $dir = $lbx->get($multi_lbx->curselection());
			 
			 return unless($dir);
			 $dir = File::Spec->rel2abs($dir, $root);
			 
			 my (@assOrder_files) = &get_assOrders($dir, $win);

			 my %data_from_file;
			 
			 #print "create_query: From $dir:\nassOrder_files are @assOrder_files\n";
			 
			 foreach (@assOrder_files)
			 {
			     my $data_ref;
			     ($data_ref) = &parse_assOrder_multiple($_, $win, 2); 

			     $pklst = basename($_); 
			     $pklst =~ s/_assOrder//; 

			     $data_from_file{$pklst} = $data_ref;
			 }
			 &display_multiple(\%data_from_file, $dir, $root, $win);			 
		     }); 

    #generates rasmol display for each structure (using the pdb file frm that cycle)
    $str_lbx->bind('<Double-Button-1>'=>sub
	       {
		   #get upl file from directory
		   my $dir = $str_lbx->get($str_lbx->curselection());
		   
		   return unless($dir);
		   $dir = File::Spec->rel2abs($dir, $root);

		   #get_pdb_file
		   my ($pdb_file) = &get_key_file($dir, $win, 'pdb');
		   
		   unless ($pdb_file)
		   {
		       &err_msg("No pdb file in directory \"$dir\"", $win);
		       return;
		   }

		   my $script;
		   my $image_mode = "cartoons";    #trace or cartoons

		   $script = "#!rasmol -script\n# File: 1\n# Creator: RasMol Version 2.7.1\n\nzap\n";
		   $script = $script."load pdb \"$pdb_file\"\n";
		   $script = $script."background white\nset ambient 40\nset specular off\n\nreset\nslab off\n\nset bonds off\nset axes off\nset boundingbox off\nset unitcell off\nset bondmode and\ndots off\n\n\n";
		   $script = $script."# Avoid Colour Problems!\nselect all\ncolour bonds none\ncolour backbone none\ncolour hbonds none\ncolour ssbonds none\ncolour ribbons none\ncolour cpk\n\n# Atoms\nselect *\ncolor [0,0,255]\n";
		   $script = $script."wireframe off\nselect *\n$image_mode\n";

		   my $template = "pdbrasmol_script";
		   my $script_fname = getTempFileName($template); 

		   my $write_success = &write_script_file($script_fname, $script, $win);
		   
		   if ($write_success)
		   {
		       my $command = "xterm -e $RASMOL -script $script_fname";
		       my $pid = fork();
		       
		       unless (defined($pid))
		       {
			   &err_msg("Cannot fork - Unable to open Rasmol window!", $win);
			   return;
		       }
		       
		       unless ($pid)
		       {
			   #child
			   exec $command;
		       }
		   }
		   
	       });
    
    
    
    #update button for contact maps tab
    my $upd_but = $cnt_pg->Button(-text=>'Update')->pack(-fill=>'x');
    $upd_but->configure(-command=>sub
			{
			    $lbx->delete(0, 'end');
			    &add_dirs($lbx, $root);
			});

    #update button for structures tab
    my $str_upd_but = $str_pg->Button(-text=>'Update')->pack(-fill=>'x');
    $str_upd_but->configure(-command=>sub
			{
			    $str_lbx->delete(0, 'end');
			    &add_dirs($str_lbx, $root);
			});

    

    my $cls_but = $win->Button(-text=>'Close', -command=>sub {$win->destroy();})->pack(-fill=>'x', -expand=>1, -padx=>3, -pady=>3, -side=>'left');
    
    #displays secondary structure info from the .sec file
    &sec_str_disp($sec_pg, $win, $root);
    
    my ($log_file) = &get_log_file($root, $win);

    $nb->pageconfigure('log', -createcmd=>sub
		       {
			   if (open(LOG, $log_file))
			   {
			       my (@lines) = <LOG>;
			       my $line;
			       foreach $line (@lines)
			       {
				   $log_txt->insert('end', $line);
			       }
			       close(LOG);
			   }
		       });
    #updates log file tab
    $log_upd->configure(-command=>sub
			{
			    $log_txt->delete(0, 'end');
			    if (open(LOG, $log_file))
			    {
				my (@lines) = <LOG>;
				foreach $line (@lines)
				{
				    $log_txt->insert('end', $line);
				}
				close(LOG);
			    }
			});
    

    $win->update;
    
}

sub load_unassign
{
    my ($unas_pg, $win, $root) = @_;
    
    my $unas_frm = $unas_pg->Frame()->pack(-fill=>'both', -expand=>1);
    
    my $scr_unas_lbx = $unas_frm->Scrolled('Listbox', -scrollbars=>"osoe")->pack(-fill=>'both', -expand=>1, -padx=>8, -pady=>8);
    my $unas_lbx = $scr_unas_lbx->Subwidget("listbox");
    $unas_lbx->configure(-selectmode=>'single', -background=>'white');
    
    #get unassign file
    my ($unassign, $pdb, $assOrder_ref, $match_ref) = &get_unassign($root, $win);

    my $unas_lbl = $unas_frm->Label(-text=>"List of peaks that are inconsistent with\n $pdb\n used for intermediate analysis", -foreground=>'red', -relief=>'ridge')
	->pack(-fill=>'x');
    my $upd_unas_but = $unas_frm->Button(-text=>'Update')->pack(-fill=>'x', -padx=>3, -pady=>3);
    $upd_unas_but->configure(-command=>sub
			     {
				 $unas_lbx->delete('1.0', 'end');
				 if ($unassign)
				 {
				     $win->configure(-cursor=>"watch");
				     my ($un_data_ref) = &parse_unassign($unassign, $win);
				     &put_unassign_data($unas_lbx, $win, $un_data_ref, $unassign, $pdb, $assOrder_ref, $match_ref);
				     $win->configure(-cursor=>"top_left_arrow");
				 }
			     });


    
    my $each;
    $win->configure(-cursor=>"top_left_arrow");
    my @children = &get_children($win);
    foreach $each (@children)
    {
	$each->configure(-cursor=>"top_left_arrow");
    }
    #$win->update; 

   if ($unassign)
    {
	#$sum_pg->configure(-cursor=>'watch');
	#$win->configure(-cursor=>"watch");
	my ($un_data_ref) = &parse_unassign($unassign, $win);
	&put_unassign_data($unas_lbx, $win, $un_data_ref, $unassign, $pdb, $assOrder_ref, $match_ref);
	#$win->configure(-cursor=>"top_left_arrow");
	#$sum_pg->configure(-cursor=>'top_left_arrow');
	#$nb->raise('unassign');
	
    }
    
}

sub disp_match
{
    my ($parent, $match_file, $peak) = @_;
    unless (-f $match_file)
    {
	&err_msg("match file $match_file does not exist", $parent);
	return;
    }

    my ($data_mat_ref) = &parse_match($match_file, $peak, $parent);
    
    my $top = $parent->Toplevel;
    my $lbl = $top->Label(-text=>"All possbile assignments (last cycle) for peak\n$peak", -borderwidth=>2, -relief=>'ridge')->pack(-side=>'top', -fill=>'x', -padx=>3, -pady=>3);

    my $sro_txt = $top->Scrolled("ROText", -scrollbars=>'osoe')->pack(-fill=>'both', -expand=>1, -padx=>5, -pady=>5);
    my $ro_txt = $sro_txt->Subwidget("rotext");

    $ro_txt->configure(-background=>'white');
    
    my @data_mat = @$data_mat_ref;
    my $i;

    for ($i = 0; $i<=$#data_mat; $i++)
    {
	$ro_txt->insert('end', "$data_mat[$i]\n");
    }
    
}


sub put_unassign_data
{
    my ($lbx, $parent, $data_ref, $unas_file, $pdb_file, $assOrder_ref, $match_ref) = @_;
    #print "pdb file is $pdb_file\n";

    my (@assOrder) = @$assOrder_ref;
    my (@match) = @$match_ref;

    my %pk_match;

    #get pdb data
    #print "starting to parse pdb\n";
    my ($pdb_data_ref, $pdb_count_ref) = &parse_pdb($pdb_file, $parent);
    
    #print "starting to convert pdb\n";
    #my ($pdb_new_data_ref) = &convert_pdb_data($pdb_data_ref, $pdb_count_ref);
    #print "done parsing pdb\n";

    my %data = %$data_ref;
    #my $data_count = scalar(@data);

    #print "put_unassign_data: data_count is $data_count\n";
    
    my $i;
    my ($atom1, $atom2);
    my $cycle;
    
    my @lbx_data;
    my $lbx_count = 0;
    my @min_dist;

    #display entries sorted by distance

    #print "filling lbx_data\n";
    foreach $cycle (sort {$a <=> $b || $a cmp $b} keys (%data))
    {
	#print "cycle is $cycle\n";
	next unless ($data{$cycle}[0]{'reason'});
	my ($cyc, $plname) = split(' ', $cycle);
	
	my $data_count = scalar(@{$data{$cycle}});
	for ($i = 0; $i<$data_count; $i++)
	{
	    my $option;
	    next unless ($data{$cycle}[$i]{'reason'});
	    
	    #check if peak exists in assOrder file, if it does, dont display it

	    my $j;
	    for ($j = 0; $j<=$#assOrder; $j++)
	    {
		if ($assOrder[$j] =~ /.*$plname.*/)
		{
		    $assOrder_file = $assOrder[$j];
		}
	    }
	    
	    my ($peak_assigned) = &check_peak_assigned($data{$cycle}[$i]{'peak'}, $assOrder_file);
	    next if ($peak_assigned);

	    $lbx_data[$lbx_count][0] = $plname;
	    $lbx_data[$lbx_count][1] = $cyc;
	    
	    $pk_match{$data{$cycle}[$i]{'peak'}} = $plname;
	    #print "pk_match{$data{$cycle}[$i]{'peak'}} = $plname\n";
	    
	    $count = 2;
	    foreach $option (qw/peak atom1 atom2 reason/)
	    {
		$lbx_data[$lbx_count][$count] = $data{$cycle}[$i]{$option};
		$count++;
	    }

	    #function unassign_distance is in the file "unassign_distance.pl"
	    my (%distance) = &unassign_distance($data{$cycle}[$i]{'atom1'}, $data{$cycle}[$i]{'atom2'}, $parent, $unas_file, $pdb_data_ref); 

	    #print "distance{'num'} = $distance{'num'}\n";
	    $lbx_data[$lbx_count][6] = \%distance;

	    if ($distance{'num'} eq 'single')
	    {
		$lbx_data[$lbx_count][7] = $distance{'mid'};
	    }
	    else
	    {
		$lbx_data[$lbx_count][7] = $distance{'min'}{'mid'};
		
	    }

	    next if ($lbx_data[$lbx_count][7] < 5.0); 

	    $lbx_count++;
	}
    }
    
    #sort data according to min distance
    my (@sorted_data) = sort {$b->[7] <=> $a->[7]} @lbx_data;
    
    #put data into lbx
    for ($i = 0; $i<$lbx_count; $i++)
    {
	$lbx->insert('end', "Peaklist = $sorted_data[$i][0], Cycle = $sorted_data[$i][1]");
	$lbx->insert('end', $sorted_data[$i][2]);
	$lbx->insert('end', $sorted_data[$i][3]);
	$lbx->insert('end', $sorted_data[$i][4]);
	$lbx->insert('end', $sorted_data[$i][5]);
	
	my %distance = %{$sorted_data[$i][6]};
	
	if ($distance{'num'} eq 'single')
	{
	    #print "single\n";
	    $lbx->insert('end', "Distance : $distance{'mid'} +/- $distance{'range'}");
	}
	else
	{
	    #print "multiple\n";
	    #print "sorted_data[$i][6]{'min'}{'mean'} = $sorted_data[$i][6]{'min'}{'mean'}\n";
	    $lbx->insert('end', "Distance : Min($distance{'min'}{'mid'} +/- $distance{'min'}{'range'}), Mean($distance{'mean'}{'mid'} +/- $distance{'mean'}{'range'}), Max($distance{'max'}{'mid'} +/- $distance{'max'}{'range'})");
	}
	$lbx->insert('end', "");
    }
    
    
    #print "done inserting data, number of records = $lbx_count\n";
    $lbx->bind('<Double-Button-1>'=>sub
	       {
		   my $line = $lbx->get($lbx->curselection());
		   print "line is $line\n";

		   if ($line =~ /^peak/)
		   {
		       #get match file name
		       my $plname = $pk_match{$line};
		       print "plname is $plname\n";
		       my ($match_file, $i);
		       
		       for ($i = 0; $i<=$#match; $i++)
		       {
			   if ($match[$i] =~ /.*$plname.*/)
			   {
			       $match_file = $match[$i];
			   }
		       }
		       #print "match file is $match_file\n";
		       $lbx->configure(-cursor=>"watch");
		       $lbx->update();
		       &disp_match($parent, $match_file, $line);
		       $lbx->configure(-cursor=>"top_left_arrow");
		   }
	       });
}


sub check_peak_assigned
{
    my ($peak, $assOrder_file) = @_;
    unless (-f $assOrder_file)
    {
	print "assOrder file $assOrder_file does not exist\n";
	return 0;
    }

    unless (open(FILE, $assOrder_file))
    {
	print "Cannot open assOrder file $assOrder_file\n";
	return 0;
    }

    my ($line);
    while (chomp($line = <FILE>))
    {
	if ($line eq $peak)
	{
	    return 1;
	}
    }
    
    return 0;
    
}


sub get_unassign
{
    my ($dir, $parent) = @_;
    my ($pdb, @assOrder, @match);

    print "get_unassign: dir is $dir\n";

    unless (-d $dir)
    {
	return;
    }

    unless (opendir(DIR, $dir))
    {
	&err_msg("Cannot open directory \"$dir\"", $parent);
	return;
    }
    
    my @contents = readdir(DIR);
    close(DIR);

    my ($unassign) = grep {/\.unassign$/} @contents;
    $unassign = File::Spec->rel2abs($unassign, $dir);
    
    #get match file
    (@match) = grep {/_match\.gz$/} @contents;
    for ($p = 0; $p<=$#match; $p++)
    {
	my $new_match = $match[$p];
	$new_match =~ s/\.gz$//;
	#print "new_match is $new_match";
	$match[$p] = File::Spec->rel2abs($match[$p], $dir);
	$match[$p] = &unzip_file($match[$p]);
    }

   my (@cycles) = grep {/^CYCLE.+final$/} @contents;
    
    
    unless (@cycles)
    {
	return;
    }

    my @nums;
    my $num_count = 0;
    my $cyc;

    foreach $cyc (@cycles)
    {
	$cyc =~ /^CYCLE(\d+)-(.*)/i;
	$nums[$num_count][0] = $1;
	$nums[$num_count][1] = $2;
	$num_count++;
    }
    
    unless (@nums)
    {
	return;
    }
    
    my @sorted_num = sort {$a->[0] <=> $b->[0]} @nums;

    my $cycle_dir = "CYCLE".$sorted_num[$#sorted_num][0]."-".$sorted_num[$#sorted_num][1];
    
    

    #(@cycles) = sort {$a <=> $b} @cycles;
    #print "@cycles\n";
    #my $cycle_dir = $cycles[$#cycles];
    #print "cycle dir is $cycle_dir\n";

    $cycle_dir = File::Spec->rel2abs($cycle_dir, $dir);
    
    if ($cycle_dir)
    {
	if (opendir(CYCLE, $cycle_dir))
	{
	    my (@con) = readdir(CYCLE);
	    close(CYCLE);
	    
	    #get protein name from control file in source directory
	    my $cont_file = &get_control_file($dir);
	    my ($prot_name) = &openContFile($cont_file, 1);
	    #print "cont_file is $cont_file, prot_name is $prot_name\n";

	    #get pdb file
	    ($pdb) = grep {/$prot_name\.pdb$/} @con;
	    $pdb = File::Spec->rel2abs($pdb, $cycle_dir);
	    #print "pdb is $pdb\n";

	    #get assOrder file
	    (@assOrder) = grep {/\_assOrder$/} @con;
	    #print "get_unassign: assOrder = $assOrder\n";
	    my $p;
	    
	    for ($p = 0; $p<=$#assOrder; $p++)
	    {
		$assOrder[$p] = File::Spec->rel2abs($assOrder[$p], $cycle_dir);
	    }
	}
	else
	{
	    print "Cannot open Cycle dir \"$cycle_dir\"\n";
	}
    }
    #print "match files are @match\n";
    return ($unassign, $pdb, \@assOrder, \@match);
}

sub add_dirs
{
    my ($lbx, $root) = @_;
    
    unless (opendir(ROOT, $root))
    {
	return;
    }

    my (@contents) = sort {$a <=> $b || $a cmp $b} readdir(ROOT);
    close(ROOT);
    my $item;

    foreach $item (@contents)
    {
	if ($item =~ /^cycle/i or $item=~/^WorkingCycle/i)
	{
	    $lbx->insert('end', $item);
	}
    }
    
}


sub get_upls
{
    my ($dir, $parent) = @_;
    my ($hbond_upl, $prot_upl, $format);

    unless (-d $dir)
    {
	print "\"$dir\" does not exist";
	return;
    }

    unless (opendir(DIR, $dir))
    {
	&err_msg("Cannot open dir \"$dir\"", $parent);
	return;
    }

    my @dir_contents = readdir(DIR);
    close(DIR);

    my ($upl1, $upl2) = grep {/\.upl$/} @dir_contents;
    $format = 'dyana';

    unless($upl1 and $upl2)
    {
	#no dyana format, find xplor format
	($upl1) = grep {/hbond\.tbl$/} @dir_contents;
	($upl2) = grep {/noe\.tbl$/} @dir_contents;
	$format = 'xplor';
	
	unless ($upl1 and $upl2)
	{
	    print "get_upls: no upls in directory\n";
	    return;
	}
    }
     
    if ($upl1 =~ /hbond/)
    {
	$hbond_upl = File::Spec->rel2abs($upl1, $dir);
	$prot_upl = File::Spec->rel2abs($upl2, $dir);
    }
    else
    {
	$hbond_upl = File::Spec->rel2abs($upl2, $dir);
	$prot_upl = File::Spec->rel2abs($upl1, $dir);
    }

    return($hbond_upl, $prot_upl, $format);
}


sub load_sum_pg
{
    my ($parent, $dir, $page) = @_;
    my $debug = 0;
    
    #get ovw file
    unless (opendir(DIR, $dir))
    {
	&err_msg("Cannot open directory \"$dir\"", $parent);
	return;
    }
	my @contents = readdir(DIR);
    my ($ovw_file) = grep {/_AS\.ovw/} @contents;
	unless ($ovw_file =~ /_AS/) {
		($ovw_file) = grep {/_NA\.ovw/} @contents;
	}
    $ovw_file = File::Spec->rel2abs($ovw_file, $dir);
    close(DIR);
    
    my $ovw_but = $page->Button(-text=>"Open .ovw file", -command=>sub
				{
				    my $pid;
				    if ($pid = fork) 
				    {
					#parent
					$SIG{CHILD} = 'IGNORE';
					return;
				    }
				    elsif (defined $pid) 
				    { 
					#child
					exec 'xterm', '-title', "$ovw_file", '-e', 'less', "$ovw_file";
				    }
				})->pack(-expand=>1, -fill=>'x', -ipadx=>5, -pady=>5);
    
    
    #parse ovw file
    my ($ovw_data_ref, $ovw_heading_ref) = &parse_as_ovw($ovw_file, $parent);

    if ($ovw_data_ref)
    {
	#create plot and display area
	&create_as_sum($parent, $page, $ovw_data_ref, $ovw_heading_ref, $dir);
    }

    if ($debug)
    {
	#print statements
	print "sub load_sum_pg:\n";
	print "AS outdir is $dir\n";
	print "ovw file is $ovw_file\n";

	print "done sub load_sum_pg\n";
    }
    
}


sub get_control_file
{
    my ($dir) = @_;

    #get source directory
    unless (opendir(DIR, $dir))
    {
	return;
    }

    my ($source_dir, $ovw); 
    while (defined($filename = readdir(DIR)))
    {
	$source_dir = $filename if ($filename =~ /^source$/); 
	$ovw = $filename if (($filename =~ /_AS\.ovw$/) or ($filename =~ /_NA\.ovw$/));
    }
    close(DIR);
    $ovw = File::Spec->rel2abs($ovw, $dir); 
    open(OVW, "<$ovw"); 

    my ($cont_file);
    while (<OVW>) 
    {
	chomp; 
	if ($_ =~ /^\#\s+-c ([\w\W]+)\s+/)
	{
	    ($cont_file) = $1; 
	    last; 
	}
    }

    #print "cont_file: " . $cont_file . "\n"; 
    $cont_file = basename($cont_file); #get the control-file name; 
    $cont_file =~ s/\s//g; 

    unless ($source_dir)
    {
	return;
    }

    $source_dir = File::Spec->rel2abs($source_dir, $dir);

    unless (opendir(SOURCE, $source_dir))
    {
	return;
    }

#    print "-" . $cont_file . "-\n"; 

#    while (defined($filename = readdir(SOURCE)))
#    {
#	print $filename; 
#	if ($filename =~ /$cont_file/)
#	{
#	    $cont2_file = $filename;
#	}
#    }
#    close(SOURCE);

    ($cont2_file) = grep {/$cont_file/} readdir(SOURCE); # get the file from the source directory
    close(SOURCE);

    unless ($cont2_file)
    {
	return;
    }

    $cont2_file = File::Spec->rel2abs($cont2_file, $source_dir);

    if ($cont2_file =~ /\.gz/)
    {
	$cont2_file = &unzip_file($cont2_file);
    }
    return($cont2_file);
}

sub open_rpf_file_in_frame
{
    my ($rpf_win, $rpf_dir, $full) = @_;
      
    unless (opendir(DIR, $rpf_dir))
    {
	print "Cannot open $rpf_dir\n";
	return;
    }

    my @contents = readdir(DIR);
    close(DIR);
	my ($rpf_file) = grep {/_DP\.ovw$/} @contents;
	$rpf_file =~ /^(.+)_DP\.ovw$/;
	my $protein_name = $1;
	print "protein_name is $protein_name\n";
	print "ovw_file is $rpf_file\n";
	print "rpf_dir is $rpf_dir\n";
	if ($rpf_file)
	{
	    $rpf_file = File::Spec->rel2abs($rpf_file, $rpf_dir);
	}

    my $lbl = $full->Label(-text=>"File - $rpf_file", -background=>'white', -relief=>'ridge', -borderwidth=>2)->pack(-padx=>1, -pady=>1, -ipadx=>3, -ipady=>3);
    my $disp = $full->Frame(-background=>'#78A1BF')->pack(-fill=>'both', -expand=>1, -padx=>2, -pady=>2);

    my ($data, $pklsts_fnames) = &parse_ovw($rpf_file, $rpf_win);

    my ($fpd_but, $ql_but, $qi_but, $nb) = &rpf_scores_frame($disp, $data, $pklsts_fnames, $rpf_win);
    my ($pdb_fname, $ql_fname) = &get_pdb_ql_fnames($rpf_dir, $protein_name, $rpf_win);
    #print "pdb_file is $pdb_fname, ql_fname is $ql_fname\n";

    my $key;
    
    foreach $key (keys %$fpd_but)
    {
	my $temp_key = $key;
	$fpd_but->{$temp_key}->configure(-command=>sub
					 {
					     
					     if ($pdb_fname && $ql_fname)
					     {
						 $pdb_fname = File::Spec->rel2abs($pdb_fname, $rpf_dir);
						 $ql_fname = File::Spec->rel2abs($ql_fname, $rpf_dir);
						 #print "open_rpf_file: ql_fname is $ql_fname, pdb_fname is $pdb_fname\n";
						 
						 &calc_rasmol($rpf_win, $pdb_fname, $ql_fname, $temp_key);
					     }
					     else
					     {
						 unless ($pdb_fname)
						 {
						     &err_msg("No pdb file in $rpf_dir", $rpf_win);
						     return 0;
						 }
						 
						 unless ($ql_fname)
						 {
						     &err_msg("No ql file in $rpf_dir", $rpf_win);
						     return 0;
						 }
					     }
					 });
    }

    $ql_but->configure(-command=>sub
		       {
			   #my ($pdb_fname, $ql_fname) = &get_fpd_fnames($rpf_dir, $rpf_file, $rpf_win);
			   if ($ql_fname && $pdb_fname)
			   {
			       $ql_fname = File::Spec->rel2abs($ql_fname, $rpf_dir);
			       &show_interactions($rpf_win, $ql_fname);
			   }
			   else
			   {
			       unless ($pdb_fname)
			       {
				   &err_msg("No pdb file in $rpf_dir", $rpf_win);
				   return 0;
			       }

			       unless ($ql_fname)
			       {
				   &err_msg("No ql file in $rpf_dir", $rpf_win);
				   return 0;
			       }
			   }
			   
		       });
    $qi_but->configure(-command=>sub
		       {
			   &display_qi($rpf_dir, $rpf_win);
		       });

    my $tree;
    my $tre_pg = $nb->add('tree', -label=>'Tree View');

    $nb->pageconfigure('tree', -createcmd=>sub
		       {
			   &create_tree($tre_pg, $rpf_win, $rpf_dir, $tree);
		       });
    
    
    my $but_frm = $full->Frame(-background=>'#78A1Bf')->pack(-side=>'bottom', -fill=>'x', -expand=>1);

    my $sav_but = $but_frm->Button(-text=>"Save", -command=>sub 
				   {
				       my $rpf_file = &SaveFile('rpf', $rpf_win, $basepath);
				       if ($rpf_file)
				       {
					   $rpf_file = File::Spec->rel2abs($rpf_file, $basepath);
					   #print "open_rpf_file: save as $rpf_file\n";
					   
					   &write_rpf_data($rpf_file, $data);
				       }
				   }, -width=>20)->pack(-pady=>3, -side=>'left', -fill=>'x', -expand=>1);


    my $cls_but = $but_frm->Button(-text=>"Close", -command=>sub {$rpf_win->destroy;}, -width=>20)->pack(-pady=>3, -side=>'left', -fill=>'x', -expand=>1);
}

sub get_pdb_ql_fnames {
    my ($dir, $protein_name, $parent) = @_;
    my $basepath = Cwd::cwd;
    #print "get_fpd_names: dir is $dir\n";
    my ($pdb_file_name) = &get_pdb_from_last_cycle($dir, $protein_name, $parent);
    my $pdb_file_basename = basename($pdb_file_name);
    #chop($pdb_file_name);
    #print "pdb file name is: $pdb_file_name\n";
	unless ($pdb_file_name =~ /^\//) {
		($pdb_file_name) = File::Spec->rel2abs($pdb_file_name, $dir);
	}
	#system("cp $pdb_file_name $dir/$pdb_file_basename");
    #only keep the first model
    open(PDB, "<$pdb_file_name"); 
    @lines = <PDB>;
    close PDB; 
    unlink($pdb_file); 
    my $single_model_pdb = "$dir/$pdb_file_basename.single";
    open(PDB, ">$single_model_pdb"); 
    my $model = 0; 
    foreach my $l (@lines)
    {
	#print "$model $l"; 
	$model++ if ($l =~ /^MODEL|model/);
	last if ($model > 1); 
	print PDB "$l"; 
    }

    unless (opendir(DIR, $dir))
    {
	&err_msg("Cannot open directory \"$dir\"", $parent);
	return 0;
    }
    
    my @dir_contents = readdir(DIR);
    close(DIR);

    my ($ql_file) = grep {/_DP\.qp$/i} @dir_contents;
    
    chdir $basepath;
    #$pdb_file = File::Spec->rel2abs($pdb_file, $dir);
    #$ql_file = File::Spec->rel2abs($ql_file, $dir);
    #print "get_fpd_fnames: pdb_file is $single_model_pdb, and ql_file is $ql_file $dir\n"; 
    return ($single_model_pdb, $ql_file);
}

sub get_pdb_from_last_cycle {
    my ($dir, $protein_name, $parent) = @_;
	opendir(DIR, $dir);
	my @contents = readdir(DIR);
	close(DIR);
	my $last_cycle = 0;
	my $cycle_dir;
	foreach my $item (@contents) {
		if ($item =~ /CYCLE(\d+)-.+final/i) {
			if ($1 > $last_cycle) {
				$last_cycle = $1;
				$cycle_dir = $item;
			}
		}
	}
	#print "last cycle is $cycle_dir\n";
	opendir(DIR, "$dir/$cycle_dir");
	@contents = readdir(DIR);
	close(DIR);
	#print "searching for $protein_name.pdb inside $cycle_dir\n";
	foreach $item (@contents) {
		if ($item =~ /^$protein_name\.pdb$/) {
			my $pdb = "$dir/$cycle_dir/$item";
			return $pdb;
		}
	}
	return "um";
}


return 1;


