my %aa = (
    "ALA"=>"A",
    "ARG"=>"R",
    "ASN"=>"N",
    "ASP"=>"D",
    "CYS"=>"C",
    "GLN"=>"Q",
    "GLU"=>"E",
    "GLY"=>"G",
    "HIS"=>"H",
    "ILE"=>"I",
    "LEU"=>"L",
    "LYS"=>"K",
    "MET"=>"M",
    "PHE"=>"F",
    "PRO"=>"P",
    "SER"=>"S",
    "THR"=>"T",
    "TRP"=>"W",
    "TYR"=>"Y",
    "VAL"=>"V"
	  );

my %rev_aa = reverse %aa;
my $debug = 0;


sub unassign_distance
{
    #given two atoms, return distance between them
    
    my ($atom1, $atom2, $parent, $unas_file, $pdb_data_ref) = @_;
    #print "\nunassign_distance:\n";
    #print "atom1: \"$atom1\"\natom2: \"$atom2\"\n" if ($debug);

    #my ($res1_num, $res1_name, $res1_atm, $res1_ambi, $res2_num, $res2_name, $res2_atm, $res2_ambi);
    my %info;
    #my %distance;
    $distance{'num'} = 'multiple';
    
    #get the two atoms
    $atom1 =~ /atom:\s+\d+\s+(\d+)(\w)-([\w\d]+)\s+.+ambi=(\d)\s+/;
    $info{'res1_num'} = $1;
    $info{'res1_name'} = $2;
    $info{'res1_atm'} = $3;
    $info{'res1_ambi'} = $4;
    $info{'res1_deg'} = 0;
    if ($info{'res1_ambi'} == 1)
    {
	$info{'res1_deg'} = 1;
    }

    $atom2 =~ /atom:\s+\d+\s+(\d+)(\w)-([\w\d]+)\s+.+ambi=(\d)\s+/;
    $info{'res2_num'} = $1;
    $info{'res2_name'} = $2;
    $info{'res2_atm'} = $3;
    $info{'res2_ambi'} = $4;
    $info{'res2_deg'} = 0;
    if ($info{'res2_ambi'} == 1)
    {
	$info{'res2_deg'} = 1;

	if ($info{'res1_ambi'} == 1)
	{
	    #$distance{'num'} = 'single';
	}
    }
    
    if (($info{'res1_atm'} =~ /^Q/))
    {
	$info{'res1_deg'} = 1;
	$info{'res1_ambi'} = 1;
    }
    if (($info{'res2_atm'} =~ /^Q/))
    {
	$info{'res2_deg'} = 1;
	$info{'res2_ambi'} = 1;
    }
	
    

    my $atom_1 = "$info{'res1_num'}$info{'res1_name'}-$info{'res1_atm'}";
    my $atom_2 = "$info{'res2_num'}$info{'res2_name'}-$info{'res2_atm'}";

    print "\nDistance between $atom_1 and $atom_2\n" if ($debug);
    
    my (%distance) = &get_model_distance(\%info, $pdb_data_ref);
    #my (%distance) = &get_distance(\%info, $pdb_data_ref);
    #print "distance = @distance\n\n" if ($debug);
    
    return (%distance);

    #given atom_1 and atom_2, find distance between them
}

sub get_minmax
{
    #given an array of distances, returns the mean and midrange of those numbers
    my ($code, @dist) = @_;
    #print "get_minimax: dist = @dist\n";
    
    my ($sum, $total, $max, $mean) = (0,0,0,0);
    my $i;
    my $min = $dist[0];
    
    for ($i = 0; $i<=$#dist; $i++)
    {
	if ($min > $dist[$i])
	{
	    $min = $dist[$i];
	}
	if ($max < $dist[$i])
	{
	    $max = $dist[$i];
	}
	$sum += $dist[$i];
    }

    $total = $#dist + 1;
    $mean = $sum / $total;
    $mean = sprintf("%0.2f", $mean);
    
    if ($code)
    {
	return ($min, $mean, $max);
    }

    #else return mean and midrange
    my $mid = sprintf("%0.2f", ($max + $min)/2);
    my $range = sprintf("%0.2f", ($max - $min)/2);
    
    #print "get_minimax: returning mean = $mean, midrange = $midrange\n";
    return ($mid, $range);
}


sub get_model_distance
{
    my ($info_ref, $pdb_data_ref) = @_;

    #my $debug = 0;

    #my %pdb_data_hash = %$pdb_data_ref;
    my $models = scalar(@{$pdb_data_ref->{'ATOM'}});
    
    my %distance;
    print "get_model_distance: start\n" if ($debug);

    if ($info_ref->{'res1_ambi'} == 1 and $info_ref->{'res2_ambi'} == 1)
    {
	$distance{'num'} = 'single';
    }
    else
    {
	$distance{'num'} = 'multiple';
    }

    my ($model);
    my @dist_single;
    
    if ($distance{'num'} eq 'single')
    {
	my @dist_single;
	#print "single interaction\n";
	for ($model = 0; $model<$models; $model++)
	{
	    $info_ref->{'model'} = $model;
	    my (@inter) = &get_distance($info_ref, $pdb_data_ref);
	    $dist_single[$model] = $inter[0];
	}
	($distance{'mid'}, $distance{'range'}) = &get_minmax(0, @dist_single);
	print "get_model_dist: distance{mid} = $distance{'mid'}, distance{range} = $distance{'range'}\n" if ($debug);
	
    }
    else
    {
	my (@dist_min, @dist_mean, @dist_max);
	#print "get_model_dist: multiple interactions\n";
	
	for ($model = 0; $model<$models; $model++)
	{
	    $info_ref->{'model'} = $model;
	    my (@inter) = &get_distance($info_ref, $pdb_data_ref);
	    ($dist_min[$model], $dist_mean[$model], $dist_max[$model]) = &get_minmax(1, @inter);
	}
	($distance{'min'}{'mid'}, $distance{'min'}{'range'}) = &get_minmax(0, @dist_min);
	($distance{'mean'}{'mid'}, $distance{'mean'}{'range'}) = &get_minmax(0, @dist_mean);
	($distance{'max'}{'mid'}, $distance{'max'}{'range'}) = &get_minmax(0, @dist_max);

	my ($key1, $key2);
	#print "get_model_dist: distance: ";
	foreach $key1 (qw/min mean max/)
	{
	    foreach $key2 (qw/mid range/)
	    {
		print "$key1-$key2 = $distance{$key1}{$key2}, " if ($debug);
	    }
	}
	print "\n" if ($debug);

    }

    return (%distance);
    #print "num_models = $num_models\n";
    
}


sub get_distance
{
    my ($info_ref, $pdb_data_ref) = @_;

    if (($info_ref->{'res1_ambi'} == 1) and ($info_ref->{'res2_ambi'} == 1))
    {
	my @d_effs = &get_summation_dist($info_ref, $pdb_data_ref);
	print "get_distance: returning @d_effs\n" if ($debug);
	return (@d_effs);
    }
    else
    {
 	#print "get_distance: ambi not 1, calculating min\n";
	my (@atoms_1, @atoms_2);
	if (($info_ref->{'res1_ambi'} >1) and ($info_ref->{'res1_atm'} !~ /^Q/))
	{
	    #res 1 is ambigous
	    (@atoms_1) = &get_ambi_atoms($info_ref->{'res1_name'}, $info_ref->{'res1_atm'});
	}
	else
	{
	    $atoms_1[0] = $info_ref->{'res1_atm'};
	}

	if (($info_ref->{'res2_ambi'} > 1) and ($info_ref->{'res2_atm'} !~ /^Q/))
	{
	    (@atoms_2) = &get_ambi_atoms($info_ref->{'res2_name'}, $info_ref->{'res2_atm'});
	}
	else
	{
	    $atoms_2[0] = $info_ref->{'res2_atm'};
	}

	print "get_distance: atoms_1 is @atoms_1, atoms_2 is @atoms_2\n" if ($debug);
	
	#foreach atom in atoms1 and atoms2, calc distance for each pair and store in array
	my (@d_effs, $d_count, $i, $j);
	my %new_info = %$info_ref;
	
	$d_count = 0; 
	for ($i = 0; $i<=$#atoms_1; $i++)
	{
	    $new_info{'res1_atm'} = $atoms_1[$i];
	    $new_info{'res1_ambi'} = 1;
	    for ($j = 0; $j<=$#atoms_2; $j++)
	    {
		$new_info{'res2_atm'} = $atoms_2[$j];
		$new_info{'res2_ambi'} = 1;
		print "get_distance: call get_distance between $new_info{'res1_num'}-$new_info{'res1_atm'} and $new_info{'res2_num'}-$new_info{'res2_atm'}\n" if ($debug);
		
		($d_effs[$d_count]) = &get_distance(\%new_info, $pdb_data_ref);
		$d_count++;
	    }
	}
	
	print "get_distance: returning @d_effs\n" if ($debug);
	return (@d_effs);
    }
    
    
}


sub get_summation_dist
{
    my ($info_ref, $pdb_data_ref) = @_;

    #print "get_summation_dist:\n";
    #print "atom1: $info_ref->{'res1_num'}$info_ref->{'res1_name'}-$info_ref->{'res1_atm'}\n";
    #print "atom2: $info_ref->{'res2_num'}$info_ref->{'res2_name'}-$info_ref->{'res2_atm'}\n";
    #print "deg1 = $info_ref->{'res1_deg'}, deg2 = $info_ref->{'res2_deg'}\n";

    my (@atoms_1, @atoms_2);

    (@atoms_1) = &get_deg_atoms($info_ref->{'res1_name'}, $info_ref->{'res1_atm'});
    (@atoms_2) = &get_deg_atoms($info_ref->{'res2_name'}, $info_ref->{'res2_atm'});
    
    my ($i, $j, $tot_dist);
    
    print "get_summation_dist: atoms_1 are @atoms_1, atoms_2 are @atoms_2\n" if ($debug);
    
    $tot_dist = 0;
    my %new_info = %$info_ref;

    for ($i = 0; $i<=$#atoms_1; $i++)
    {
	$new_info{'res1_atm'} = $atoms_1[$i];
	for ($j = 0; $j<=$#atoms_2; $j++)
	{
	    $new_info{'res2_atm'} = $atoms_2[$j];
	    my $distance = &get_unique_dist(\%new_info, $pdb_data_ref);
	    $distance = sprintf("%0.2f", $distance);
	    if ($distance)
	    {
		$distance = $distance ** (-6);
		$tot_dist += $distance;
	    }
	}
    }
    
    
    my $d_eff = $tot_dist ** (-1/6);
    #$d_eff = sprintf("%0.2f", $d_eff);
    
    print "get_summation_dist: summed distance between $info_ref->{'res1_num'}$info_ref->{'res1_atm'} and $info_ref->{'res2_num'}$info_ref->{'res2_atm'} = $d_eff\n" if ($debug);
    return ($d_eff);
    
}


sub get_unique_dist
{
    my ($info_ref, $pdb_data_ref) = @_;
    
    my $atom1 = ($info_ref->{'res1_atm'});
    my $atom2 = ($info_ref->{'res2_atm'});
    my $model = $info_ref->{'model'};

    my %pdb_data = %$pdb_data_ref;

    my $pdb_atm_1 = "$info_ref->{'res1_num'}$rev_aa{$info_ref->{'res1_name'}}";
    my $pdb_atm_2 = "$info_ref->{'res2_num'}$rev_aa{$info_ref->{'res2_name'}}";

    
    if (($info_ref->{'res1_name'} eq 'G') && ($atom1 eq 'HA3'))
    {
	unless ($pdb_data{'ATOM'}[$model]{$pdb_atm_1}{$atom1}{'x'})
	{
	    $atom1 = "HA1"; #fix it for dyana and xplor/iupac naming HA1/HA2 
	}
    }
    if (($info_ref->{'res2_name'} eq 'G') && ($atom2 eq 'HA3'))
    {
	unless ($pdb_data{'ATOM'}[$model]{$pdb_atm_2}{$atom2}{'x'})
	{
	    $atom2 = "HA1"; #fix it for dyana and xplor/iupac naming HA1/HA2 
	}
    }
    
    print "get_unique_dist: pdb_atm_1 = $pdb_atm_1 (atom $atom1), and pdb_atm_2 = $pdb_atm_2 (atom $atom2)\n" if ($debug);

    my ($x1, $y1, $z1) = ($pdb_data{'ATOM'}[$model]{$pdb_atm_1}{$atom1}{'x'}, $pdb_data{'ATOM'}[$model]{$pdb_atm_1}{$atom1}{'y'}, $pdb_data{'ATOM'}[$model]{$pdb_atm_1}{$atom1}{'z'});
    my ($x2, $y2, $z2) = ($pdb_data{'ATOM'}[$model]{$pdb_atm_2}{$atom2}{'x'}, $pdb_data{'ATOM'}[$model]{$pdb_atm_2}{$atom2}{'y'}, $pdb_data{'ATOM'}[$model]{$pdb_atm_2}{$atom2}{'z'});

    my $distance = &pt_distance($x1, $y1, $z1, $x2, $y2, $z2);
    $distance = sprintf("%0.2f", $distance);

    print "get_unique_dist: returning dist, model $model between $pdb_atm_1-$atom1($x1, $y1, $z1) and $pdb_atm_2-$atom2($x2, $y2, $z2) as $distance\n" if ($debug);
    

    unless ($x1 and $y1 and $z1 and $x2 and $y2 and $z2)
    {
	print "unique distance not found\n";
	print "get_unique_dist: $pdb_atm_1-$atom1($x1, $y1, $z1) and $pdb_atm_2-$atom2($x2, $y2, $z2) as $distance\n";
	return (10000);
    }
    

    return ($distance);
    
}


sub get_ambi_atoms
{
    my ($res_name, $res_atm) = @_;
    
    if ($res_atm =~ /^HA/)
    {
	if ($res_name eq 'G')
	{
	    return ('HA2', 'HA3');
	}
    }
    elsif ($res_atm =~ /^HB/)
    {
	if (($res_name eq 'L') or ($res_name eq 'R') or ($res_name eq 'D') or ($res_name eq 'C') or ($res_name eq 'E') or ($res_name eq 'Q') or 
	    ($res_name eq 'H') or ($res_name eq 'K') or ($res_name eq 'M') or ($res_name eq 'F') or ($res_name eq 'S') or ($res_name eq 'W') or ($res_name eq 'Y') or 
	    ($res_name eq 'N') or ($res_name eq 'P'))
	{
	    return ('HB2', 'HB3');
	}
    }
    elsif ($res_atm =~ /^HG/)
    {
	if (($res_name eq 'R') or ($res_name eq 'E') or ($res_name eq 'Q') or ($res_name eq 'K') or ($res_name eq 'P') or ($res_name eq 'M'))
	{
	    return ('HG2', 'HG3');
	}
	elsif ($res_name eq 'I')
	{
	    if ($res_atm =~ /^HG1/)
	    {
		return ('HG12', 'HG13');
	    }
	}
	elsif ($res_name eq 'V')
	{
	    return ('HG1', 'HG2');
	}
    }
    elsif ($res_atm =~ /^HD/)
    {
	if (($res_name eq 'R') or ($res_name eq 'K') or ($res_name eq 'P'))
	{
	    return ('HD2', 'HD3');
	}
	elsif ($res_name =~ /^[LFY]$/)
	{
	    return ('HD1', 'HD2');
	}
	elsif ($res_name eq 'N')
	{
	    if ($res_atm =~ /^HD2/) 
	    {
		return ('HD21', 'HD22');
	    }
	}
    }
    elsif ($res_atm =~ /^HE/)
    {
	if ($res_name eq 'K') { return ('HE2', 'HE3');}

	elsif (($res_name eq 'F') or ($res_name eq 'Y')) {return ('HE1', 'HE2');}
	
	elsif ($res_name eq 'Q') {return ('HE21', 'HE22');}
    }
    elsif ($res_atm =~ /^HZ/)
    {
	if ($res_name eq 'W')
	{
	    return ('HZ2', 'HZ3');
	}
    }
    
    return ($res_atm);
    
}


sub get_deg_atoms
{
    my ($res_name, $res_atm) = @_;
    
    if ($res_atm =~ /^QA/)
    {
	if ($res_name eq 'G') {return ('HA2', 'HA3')};
    }
    elsif ($res_atm =~ /^QB/)
    {
	if ($res_name =~ /^[LPRNDCQEHKMFSWY]$/) {return ('HB2','HB3')};
    }
    elsif ($res_atm =~ /^HB/)
    {
	if ($res_name =~ /^[A]$/) 
	{return ('HB1','HB2','HB3');}
    }
    elsif ($res_atm =~ /^QG/)
    {
	if ($res_name =~ /^[PRQEKM]$/) {return ('HG2', 'HG3');}
	if (($res_name =~ /^I$/) and ($res_atm =~ /^QG1$/)) {return ('HG12','HG13');}
	if ($res_name =~ /^V$/) {return ('HG11', 'HG12', 'HG13', 'HG21', 'HG22', 'HG23');}
    }
    elsif ($res_atm =~ /^HG/)
    {
	if ($res_name =~ /^[IT]$/) {return ('HG21','HG22','HG23');}
	if ($res_name =~ /V/)
	{
	    if ($res_atm =~ /^HG1/) {return ('HG11', 'HG12', 'HG13');}
	    if ($res_atm =~ /^HG2/) {return ('HG21', 'HG22', 'HG23');}
	}
    }
    elsif ($res_atm =~ /^QD/)
    {
	if ($res_name =~ /^[PRK]$/) {return ('HD2', 'HD3');}
	if ($res_name =~ /^[N]$/) {return ('HD21','HD22');}
	if ($res_name =~ /^L$/) {return ('HD11', 'HD12', 'HD13', 'HD21', 'HD22', 'HD23');}
	if ($res_name =~ /^[FY]$/) {return ('HD1','HD2');}
    }
    elsif ($res_atm =~ /^HD/)
    {
	if ($res_name =~ /^[IL]$/)
	{
	    if ($res_atm =~ /^HD1/) {return ('HD11','HD12','HD13');}
	    if ($res_atm =~ /^HD2/) {return ('HD21','HD22','HD23');}
	}
    }
    elsif ($res_atm =~ /^QE/)
    {
	if ($res_name =~ /^[Q]$/) {return ('HE21','HE22');}
	if ($res_name =~ /^[K]$/) {return ('HE2','HE3');}
	if ($res_name =~ /^[FY]$/) {return ('HE1','HE2');}
    }
    elsif ($res_atm =~ /^HE/)
    {
	if ($res_name =~ /M/) {return ('HE1','HE2','HE3');}
    }
    elsif ($res_atm =~ /^QZ/)
    {
	if ($res_name =~ /W/) {return ('HZ2','HZ3');}
    }
    
    return ($res_atm);
    
}




sub check_methyl
{
    #given an atom, checks if its part of a methyl group or not
    my ($res_name, $res_atm) = @_;

    if ($res_name eq 'L')
    {
	#two methyl groups
	return 1 if ($res_atm =~ /^HD/);
	return 0;
    }
    elsif ($res_name eq 'A')
    {
	#one methyl group
	return 1 if ($res_atm =~ /^HB/);
	return 0;
    }
    elsif ($res_name eq 'V')
    {
	#two methyl groups
	return 1 if ($res_atm =~ /^HG/);
	return 0;
    }
    elsif ($res_name eq 'M')
    {
	#one methyl group
	return 1 if ($res_atm =~ /^HE/);
	return 0;
    }
    elsif ($res_name eq 'I')
    {
	#two methyl groups
	return 1 if ($res_atm =~ /^HG2/ or $res_atm =~ /^HD1/);
	return 0;
    }
    elsif ($res_name eq 'T')
    {
	return 1 if ($res_atm =~ /HG2/);
	return 0;
    }
    return 0;

}




return 1;
