package AS;

use ASDP::ControlFile;
use ASDP::Bmrb;
use ASDP::Utils;
use Cwd;
use File::Basename;

sub new {
	my $class = shift;
	my $self = {};
	$self->{output_dir} = shift;
	$self->{control_filepath} = shift;
	$self->{bmrb_filepath} = undef;
	bless($self, $class);
	Utils->debug("AS object created. control filepath is: " . $self->{control_filepath});
	return $self;
}
sub correct_input {
	my $self = shift;

	my $control_file = ControlFile->new($self->{control_filepath}, $self->{debug});

	my $dirname = dirname($control_file); 

	my $bmrb_file = $control_file->{chemicalShiftFile};
	my $sequence_file = $control_file->{seqFile};
	my $new_bmrb_file = $self->{output_dir} . "/input1/" . basename($bmrb_file);
#	my $new_bmrb_file = $dirname . "/" . basename($bmrb_file); 

	system("cp $bmrb_file $new_bmrb_file");
	$self->{bmrb_filepath} = $new_bmrb_file;
	Utils->debug("\nbmrb: $bmrb_file");
	($sequence_file) ? (Utils->debug("sequence: $sequence_file")) : (Utils->debug("sequence: not found in control file"));
	my $bmrb = Bmrb->new($new_bmrb_file, $new_bmrb_file, $ENV{'DEBUG'});
	if ($sequence_file) {
		my $new_sequence_file = $self->{output_dir} . "/input1/" . basename($sequence_file);
#	    my $new_sequence_file = $dirname . "/" . basename($sequence_file); 
	    system("cp $sequence_file $new_sequence_file");
	    $control_file->update_value("[General]", "lines", "seqFile", $new_sequence_file);
	} else {
		my $new_sequence_file = "${new_bmrb_file}_sequence";
		Utils->debug("No sequence file found in control file - creating one from the bmrb file.");
		my $sequence_text = $bmrb->extract_sequence();
		open(SEQ_FILE, ">$new_sequence_file");
		print SEQ_FILE $sequence_text;
		close(SEQ_FILE);
		$control_file->update_value("[General]", "lines", "seqFile", "$new_sequence_file");
	}
	my $chem_shifts_table_ref = $bmrb->extract_bmrb();
	open(NEW_BMRB, ">$new_bmrb_file");
	foreach my $array (@$chem_shifts_table_ref) {
		my $line = "";
		foreach my $item (@$array) {
			$line .= $item . " ";
		}
		Utils->debug("$line");
		print NEW_BMRB "$line\n";
	}
	close(NEW_BMRB);
	$control_file->update_value("[General]", "lines", "chemicalShiftFile", "$new_bmrb_file");
	$control_file->write_to_disk($self->{control_filepath});
}
sub execute {
	my $self = shift;
	my $program = shift;
	my $args = shift;
	my $output_handle = shift;
	my $pid;
	Utils->debug("cwd: " . cwd());
	Utils->debug("command to execute is:");
	Utils->debug("$program $args 2>&1|");
	unless (defined($$pid = open(ASDP_PROG, "$program $args 2>&1|"))) {
		die("Could not launch autostructure sub process\n");
		return 0;
	}
	my $c;
	while (defined ($c = (getc ASDP_PROG))) {
		$| = 1;				#tell perl not to buffer output
		print $output_handle $c;
	}
	print $output_handle "\n";
	close(ASDP_PROG);		#this causes parent process to block on piped process
}
sub cleanup_temp_files {
	my $self = shift;
	#system("chmod -R 0777 " . $self->{output_dir});
	my $debug = (defined $ENV{'DEBUG'}) ? ($ENV{'DEBUG'}) : 0;
	unless ($debug) {
		unlink($self->{output_dir} . "/input1");
	}
}
return(1);
