package ControlFile;

# Perl library to parse and modify the values in an Autostructure control file
#
# created 11/02/09 - Gautam Singh

use Cwd;

sub new {
	my $class = shift;
	my $self = {};
	$self->{filepath} = shift;
	$self->{debug} = shift;
	$self->{section} = undef;
	$self->{chemicalShiftFile} = undef;
	$self->{seqFile} = undef;
	$self->{peaklists} = [];
	$self->{general} = {};
	$self->{num_of_peaklists} = 0;
	$self->{section_order} = [];
	bless($self, $class);
	$self->debug("Control File object created. control filepath is: " . $self->{filepath});
	$self->debug("loading_control_values");
	$self->load_control_values();
	return $self;
}
sub load_control_values {
	my $self = shift;
	my ($temp_line);
	$self->debug("about to open " . $self->{filepath});
	open ( FILE, $self->{filepath} ) or die "$!";
	binmode FILE;
	while($temp_line = <FILE>) {
		chomp $temp_line;
		next if (($temp_line =~ /^\#/) or ($temp_line =~ /^\s*$/));
		if ($temp_line =~ /(\[.+\])/) {
			$self->debug("section is: " . $1);
			my $section_name = $1;
			$self->load_section($1, FILE);
		}
	}
	close FILE;
	unlink FILE;
}
sub load_section {
	my $self = shift;
	my $section_name = shift;
	my $file_handle = shift;
	$self->debug("parsing out section from file: $section_name");
	#print $section_name . "********\n";
	push(@{$self->{section_order}}, $section_name);
	$self->{sections}{$section_name} = {};
	unless ($section_name =~ /general/i) {
		$self->{num_of_peaklists}++;
		push(@{$self->{peaklists}}, $section_name);
	}
	my ($temp_line);
	while( $temp_line = <$file_handle>) {
		chomp $temp_line;
		next if (($temp_line =~ /^\s*$/));
		#print $temp_line;
		if ($temp_line =~ /^\#/) {
			#save the comment
			#print "found comment: $temp_line\n";
			unless (defined $self->{sections}{$section_name}{comments}) {
				my @array;
				#print "creating a new comments data structure for section: $section_name\n";
				$self->{sections}{$section_name}{comments} = \@array;
			}
			push(@{$self->{sections}{$section_name}{comments}}, $temp_line);
			next;
		}
		if ($temp_line =~ /^chemicalShiftFile=(\S*)/) {
			$self->{chemicalShiftFile} = $1;
		}
		if ($temp_line =~ /^seqFile=(\S*)/) {
			$self->{seqFile} = $1;
		}
		if ($temp_line =~ /^(\[.+\])/) {
			return $self->load_section($1, $file_handle);
		}
		if ($section_name =~ /(\[.+Command\])/i) {
			#chomp($temp_line = <$file_handle>);
			#unless (defined $self->{sections}{$section_name}{lines}) {
				$self->{sections}{$section_name}{command} = $temp_line;
				$self->debug("command: " . $temp_line);
			#}
			#push(@{$self->{sections}{$section_name}{lines}}, $temp_line);
		}
		if ($temp_line =~ /([^=\s\.]+)\.([^=\s\.]+)=([^=\s]+)/) {
			unless (defined $self->{sections}{$section_name}{$1}) {
				$self->{sections}{$section_name}{$1} = {};
			}
			$self->{sections}{$section_name}{$1}{$2} = $3;
		} else {
			if ($temp_line =~ /([^=\s\.]+)=([^=\s]+)/) {
				unless (defined $self->{sections}{$section_name}{lines}) {
					$self->{sections}{$section_name}{lines} = {};
				}
				$self->{sections}{$section_name}{lines}{$1} = $2;
			} else {
				#$self->debug("WARNING - skipping line: " . $temp_line);
			}
		}
	}
}
sub update_value {
	my $self = shift;
	my $section = shift;
	my $element = shift;
	my $attr = shift;
	my $value = shift;
	$self->debug("updating: $section, $element, $attr -> $value");
	$self->{sections}{$section}{$element}{$attr} = $value;
}
sub write_to_disk {
	my $self = shift;
	my $new_filepath = shift;
	open ( FILE, ">" . $new_filepath ) or die "$!";
	binmode FILE;
	$self->debug("new control file:");
	foreach my $section_name (@{$self->{section_order}}) {
		my $section = $self->{sections}{$section_name};
		$self->debug("section_name $section_name");
		#TODO check here to see if section name is a file and change it to full file path if it is.
		my $section_name_new = $section_name;
		if ($section_name =~ /\[(.+)\]/) {
			my $section_without_braces = $1;
			if (-e $section_without_braces) {
				my $prefix = "";
				$self->debug("peaklist file found in control file: $section_without_braces");
				unless ($section_without_braces =~ /^\//) {
					$self->debug("$section_without_braces doesn't start with a / so changing to absolute path.");
					$prefix = cwd() . "/";
				}
				$section_name_new = "[" . $prefix . "$section_without_braces]";
			}
		}
		print FILE "$section_name_new\n";
		#print out all the stuff in this section
		foreach my $element_name (keys (%$section)) {
			my $element = $self->{sections}{$section_name}{$element_name};
			$self->debug("element $element_name");
			if (not ($element_name eq "lines")) {
				if ($element_name eq "command") {
					print FILE $element . "\n";
				} elsif ($element_name eq "comments") {
					foreach my $comment (@$element) {
						print FILE "$comment\n";
					}
				} else {
					foreach my $attr_name (keys (%$element)) {
						my $value = $self->{sections}{$section_name}{$element_name}{$attr_name};
						$self->debug("* $element_name.$attr_name=$value");
						print FILE "$element_name.$attr_name=$value\n";
					}
				}
			} else {
				foreach my $attr_name (keys (%$element)) {
					my $value = $self->{sections}{$section_name}{"lines"}{$attr_name};
					$self->debug("* $attr_name=$value");
					my $value_new = $value;
					if ($attr_name =~ /file|aco|par|upl|xpl|dihe/i) {
						my $prefix = "";
						$self->debug("File found in control file: $value");
						unless ($value =~ /^\//) {
							$self->debug("$value doesn't start with a / so changing it to an absolute path.");
							$prefix = cwd() . "/";
						}
						if (-e $value) {
							$value_new = $prefix . $value;
						}
					}
					print FILE "$attr_name=$value_new\n";
				}
			}
		}
	}
	close FILE;
}
sub debug {
	my $self = shift;
	my $msg = shift;
	if ($self->{debug}) {
		print "$msg\n";
	}
}
return(1);
