package CreateProc;

use ASDP::Cyana;
#use ASDP::CyanaMPI;
use ASDP::CyanaEC;
use ASDP::Xplor;
use ASDP::DP;
use ASDP::Utils;
#use ASDP::CNS;
use Cwd;

sub new {
	my $class = shift;
	my $self = {};
	$self->{options} = shift;
	$self->{protocol} = undef;
	$self->{starttime} = `date`;
	bless($self, $class);
	unless (defined $self->{options}->{na}) {
		$self->display_help();
		exit(1);
	}
	my $options_ref = $self->{options};
	$self->set_options($options_ref);
	#$self->create_seeds(10000);
	return $self;
}

sub calculate_structures {
	my $self = shift;
	if ($self->{options}->{pr} =~ /cyana/i) {
		#if (defined $self->{options}->{mpi}) { #mpi
		#	Utils->debug("in calc structures.  creating a new cyana-mpi protocol.");
		#	$self->{protocol} = CyanaEC->new($self->{options});
		#	#$self->{protocol}->rama();
		#} els
		if (defined $self->{options}->{ec}) { #ec
			Utils->debug("in calc structures.  creating a new cyana-ec protocol.");
			$self->{protocol} = CyanaEC->new($self->{options});
			$self->{protocol}->runAMBI();
		} else {
			Utils->debug("in calc structures.  creating a new cyana protocol.");
			$self->{protocol} = Cyana->new($self->{options});
			#$self->{protocol}->rama();
			if ($self->{options}->{ambi})
			{
			    $self->{protocol}->runAMBI();
			}
		}
	} elsif ($self->{options}->{pr} =~ /xplor/i) {
		Utils->debug("in calc structures.  creating a new xplor protocol.");
		$self->{protocol} = Xplor->new($self->{options});
		$self->{protocol}->prepare_sequence();
	} else {
		print STDERR $self->{options}->{pr} . " protocol no supported.\n";
	}

	Utils->debug("in calc structures. run.");

	$self->{protocol}->run($self->create_seeds(10000));

	Utils->debug("in calc structures. afterrun.");

	$self->{protocol}->wait_for_structure_calculations();
	$self->{structtime} = `date`;
	my @results = $self->{protocol}->do_overview(1);
	Utils->debug("current results is: @results");
	my $model_ref = $self->{protocol}->split_models(@results, $self->{options}->{np}*$self->{options}->{nstr});
	$self->pick_best_structures(@results, $model_ref);
	$self->{protocol}->do_overview(2);
	if (defined $self->{options}->{'cns'}) {
		my $cns = CNS->new($self->{options});
		$cns->run($self->create_seeds(1000000000));
		$cns->wait_for_structure_calculations();
	}
	return;
}
sub pick_best_structures {
	my ($self, $pdb_file, $energies, $model_ref) = @_;
	#my $dp = DP->new(($pdb_file, $self->{options}->{cf}, $self->{options}->{nb}, $self->{options}->{np}, $self->{options}->{nstr}, $self->{options}->{que}));

	print "queue $self->{options}->{que}\n";

	my $dp = DP->new($pdb_file, $self->{options});
	#$dp->split_models();
	my $rank_ref;
	if (defined $self->{options}->{'nodp'}) {
		my @rank;
		foreach (my $i = 1; $i <= $self->{options}->{nb}; $i++) {
			push(@rank, "model$i");
		}
		$rank_ref = \@rank;
	} else {
		Utils->debug("calling DP with options:");
		Utils->debug("\t pdb_file: $pdb_file");
		Utils->debug("\t ctr_file: " . $self->{options}->{cf});
		Utils->debug("\t num_best: " . $self->{options}->{nb});
		my $dp_hash_ref = $dp->run_rpf($model_ref);
		$rank_ref = $self->{protocol}->get_top_ranked_models($dp_hash_ref, $energies);
	}

	Utils->debug("about to call merge_models with: $pdb_file, $rank_ref, $self->{options}->{nb}");
	$self->{protocol}->merge_models($pdb_file, $rank_ref, $self->{options}->{nb});
	$dp->do_cleanup() if (!Utils->isDebug());
}
sub print_statistics {
	my $self = shift;
	my $name = $self->{options}->{na};
	my $starttime = (defined $self->{starttime}) ? $self->{starttime} : "n/a";
	my $structtime = (defined $self->{structtime}) ? $self->{structtime} : "n/a";
	my $finaltime = `date`;
	open(STATS, ">Statistics");
	# Collect some statistics
	print STATS "\n";
	print STATS "       ******** GLOBAL STATISTICS ********** \n";
	print STATS " Beginning  at:    $starttime\n";
	print STATS " Structures at:    $structtime\n";
	print STATS " Ending     at:    $finaltime\n";
	print STATS "                           \n";
	if ((defined $self->{options}->{qu}) and $self->{options}->{qu} =~ /dqs/i) {
		print STATS "       ********   DQS STATISTICS  ********** \n";
		close(STATS);
		system("cat CalcStr${name}*.o* | sort | head -1           >> Statistics");
		system("cat CalcStr${name}*.o* | sort | tail -1           >> Statistics");
	} else {
		close(STATS);
	}
}
sub create_seeds {
	my $self = shift;
	my $multiplier = shift;
	system("date > TrashFile");
	my $trigger = `date +%s`;
	my $processes = $self->{options}->{np};
	my @semillas = `awk '{
		for (i=1; i<=$processes;i++) {
			print int(rand()*$multiplier)
		}
	}' TrashFile`;
	system("rm TrashFile");
	$self->{seeds} = \@semillas;
	return \@semillas;
}
sub set_options {
	my $self = shift;
	my $options = $self->{options};
	foreach (keys %{$options}) {
		print "$_ \t::: " . $options->{$_} . "\n";
	}
	if (defined $options->{cis}) {
		my $cis = $options->{cis};
		open(FILE_CIS, ">>FileCis.$$");
		my @items = split(/,/, $cis);
		foreach my $ires (@items) {
			my $temp = $ires - 1;
			#print FILE_CIS "     patch CISP                   \n";
			#RT for new Xplor
			print FILE_CIS "     patch CIPP                   \n";
			print FILE_CIS " 	      reference=-=(resid $temp) \n";
			print FILE_CIS "	      reference=+=(resid $ires) \n";
			print FILE_CIS "     end                          \n";
		}
		close(FILE_CIS);
	}
}
sub display_help {
	print "\n";
	print "         CreateProc -- a tool to launch structre calc/refinement over a cluster\n";
	print "         ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n";
	print "\n";
	print "Usage: CreateProc -na NamProt -np NumProc -ns NumStr -nb NumBest -que [PBS|DQS] -pro [xpl|cya] -mod [rand|extended]\n";
	print "\n";
	print " ** Full set of options [not all are needed for a perfect run, use defaults]\n";
	print "\n";
	print "   -na  Name          Name of your protein\n";
	print "   -np  NumProc       Number of processor (queues) to be taken,                  def 50\n";
	print "   -ns  NumStr        Number of structures to be calc. on each processor,        def  2\n";
	print "   -nb  NumBest       Number of final str. selected,                             def 20\n";
	print "   -qu  [PBS|DQS|NO]  Que system to use, one of PBS or DQS, or NOQUE             def PBS\n";
	print "   -pr  [xpl|cya]     Protocol to use, X-POR, CYANA                              def cyana\n";
	print "   -mpi Num           Total number of structures to be calc with CYANA-MPI 	\n";
	print "   -mo  [rand|exte]   starting model [random|ranphi|extended|model] (for XPLOR)  def random\n";
	print "   -av  [center|sum]  average for the distance restraints (for XPLOR)            def sum\n";
	print "   -st  Num           Starting residue number (for XPLOR)                        def 1\n";
	print "   -hotad Num         Number of adjusting steps (for XPLOR)                      def 3000\n";
	print "   -hotcy Num         Number of cycles on Hot Stage of SA (for XPLOR)            def 4\n";
	print "   -hotst Num         Step length for cycles in Hot Stage of SA                  def 1000\n";
	print "   -da  Float         Axial component if using RDC/XPLOR, mandatory no default\n";
	print "   -rh  Float         Rhombic component if using RDC/XPLOR, mandatory no default\n";
  print "   -da2 Float         Axial component if using RDC/XPLOR, second alignment, mandatory no default\n";
  print "   -rh2 Float         Rhombic component if using RDC/XPLOR, second alignment, mandatory no default\n";
	print "   -cis n1,n2         CIS pep info in comma separated list, i.e 56,89,21\n";
	print "   -ss  n1-n2,n3-n4   S-S bridge info in comma separated, dash-separated pairs, i.e. 2-24,30-40\n";
	print "   -rdc               hasRDC\n";
	print "   -ec                use EC approach\n";
	print "   -steps             steps used in CyanaEC protocol                             def 10000\n";
  print "   -upl               upl used in CyanaEC protocol                               def 2.4,6.0\n";
	print "   -lib String        Special library for structure calculation\n";
	print "   -cf  String        name of your control file to calculate RPF\n";
	print "   -he                Display this help message\n";
	print "\n";
	print " Examples:\n";
	print "      $0 -na brct -np 10 -nstr 10 -pr xpl -mo random -cf ctrl_file\n";
	print "      $0 -na ER14 -np 10 -nstr 6 -nb 10 -pr cyana -cf ctrl_file\n";
	print "\n";
}
sub do_cleanup {
	my $self = shift;
	my $name = $self->{options}->{na};
	$self->{protocol}->do_cleanup();
}
return(1);
