# EC-NMR

package CyanaEC;

use Cwd;
use ASDP::Protocol;
use ASDP::DP;
use ASDP::Utils;
use File::Copy;

@ISA = qw(Protocol);

sub new {
	my $class = shift;
	my $self = $class->SUPER::new( @_ );
	$self->{calc_duration} = 2;
	$self->{max_calc_wait} = 60;
	$self->{command} = $ENV{'CyanaCmd'};
	my $options_ref = shift;
	bless($self, $class);
	return $self;
}

sub wait_for_structure_calculations {
	my $self = shift;
	my $name = $self->{options}->{na};
	my $max_wait = (defined $self->{max_calc_wait} ? $self->{max_calc_wait} : 60);
	my $wait_time = (defined $self->{calc_duration} ? $self->{calc_duration} : 2);
	my $accumulated_wait = 0;
	Utils->debug("in CreateProc->wait_for_structure_calculations()");
	until (-e "Done${name}") {
		$accumulated_wait += $wait_time;
		sleep($wait_time);
	}
	Utils->debug("total wait was: $accumulated_wait seconds.");
	#$self->{structtime} = `date`;
}

sub split_models {
        my $self = shift;
	my $pdb  = shift;
	my $energy = shift;
	my $total = shift;
        my $name = $self->{options}->{na};
        Utils->debug("splitting pdb into models");
        my %models= ();

                open(F, "<" . $pdb) or die "failed to open file pdb file: $!\n";
                my $temp_line;
                foreach (my $i = 1; $i <= $total; $i++) {
                        if ($temp_line =~ /^END\s*$/) {
                                last;
                        }
                        open(MODEL, ">model$i");
                        Utils->debug("creating model$i");
			my $last = 0;
			do {
                                $temp_line = <F>;
                                unless ($temp_line) {
                                        $last=1;
                                }
                                if ($temp_line =~ /^ENDMDL\s*$/) {
                                        $self->{num_models}++;
					$last = 1;
                                }
                                if ($temp_line =~ /^END\s*$/) {
                                        $last=1;
                                }
                                print MODEL $temp_line;
			} while (!$last);
                        print MODEL "END\n";
                        #push(@models, "model$i");
                        $models{$i} = "model$i";
                        close(MODEL);
                }

	$self->{models} = \%models;
	return \%models;
}

sub merge_models {
        my $self = shift;
	my $pdb = shift;
        my $rank_models_ref = shift;
	my $n_model = shift;

        my @rank_models = @$rank_models_ref;

        my %loaded_models;
        @loaded_models{@rank_models} = ();
        open(PDB_READ, "<" . $pdb) or die "failed to open " . $pdb . ":$!\n";
        my $model_number;
        my @model_lines;
        while (<PDB_READ>) {
                if (/MODEL\s+(\S+)/) {
                    $model_number = $1;
                 }
                if (exists $loaded_models{"model$model_number"}) {
                     push(@model_lines, $_);
                }
                 if (/^ENDMDL\s*$/) {
                        my @new_array = @model_lines;
                       @model_lines = ();
                       $loaded_models{"model$model_number"} = \@new_array;
                  }
          }
          close(PDB_READ);
          open(F, ">" . $pdb) or die "failed to open " . $pdb . ":$!\n";
          for (my $i = 0; $i < $n_model; $i++) {
                  my $item = $rank_models[$i];
                  my @lines = @{$loaded_models{$item}};
                  foreach my $line (@lines) {
                           print F $line;
                  }
		}
                  print F "END\n";
                  close(F);

	return;
}

sub do_overview {
	my $self = shift;
	my $run_number = shift;
	my $name = $self->{options}->{na};
	#my $total_str = $self->{options}->{nstr} * $self->{options}->{np};
	#$total_str = 100 if ($total_str > 100);
	my ($pdb_file, $ovw_file);
	if ($run_number == 1) {			#run_number 1 is for all models
		#($pdb_file, $ovw_file) = $self->run_overview("read_all $name???.ang", "overview ${name} $total_str pdb hbond=1");
		copy(final . ".ovw", "all_models.ovw");
		copy(final . ".pdb", "all_models.pdb"); #mv to all_models.pdb and select top 20
		return ("all_models.pdb", "all_models.ovw");
	} elsif ($run_number == 2) {		#run_number 2 is for the best models (after rpf is run on all models)
		copy("all_models.pdb", ${name} . ".pdb"); #only the top 20 are cp to the name.pdb
		$total_str = $self->{options}->{nb}; $total_str = 100 if ($total_str > 100);
		($pdb_file, $ovw_file) = $self->run_overview("read pdb ${name}.pdb", "overview ${name} $total_str hbond=1");
	} else {
		die("unidentified run_number for cyana overview");
	}
	return ($pdb_file, $ovw_file);
}

sub run {
	my $self = shift;
	my $seeds_ref = shift;
	Utils->debug("in CyanaEC->run()");
	my $name = $self->{options}->{na};
	$self->prepare_run();
	Utils->debug("after CyanaEC->run()");
	$self->do_batch(2377, "Calc" . $name, "");
}

sub run_overview {
	my $self = shift;
	my $read_command = shift;
	my $overview_command = shift;
	my $name = $self->{options}->{na};
	open(OVW_CYA, ">myOverview.cya");
	print OVW_CYA "cyanalib\n";
	if (defined $self->{options}->{lib}) {
		print OVW_CYA "read lib " . $self->{options}->{lib} . " append\n";
	}
#	print OVW_CYA "readdata $name\n";
	print OVW_CYA "$read_command\n";
#	print OVW_CYA "read upl final.upl append\n";
	print OVW_CYA "read upl $name\_EC.upl append\n";
#	print OVW_CYA "read $name.pdb\n";
#	print OVW_CYA "read lol hbond.lol append\n";
#	print OVW_CYA "read upl hbond.upl append\n";
	print OVW_CYA "read aco $name\n";
	if (defined $self->{options}->{rdc}) {
#	    print OVW_CYA "#$self->{options}->{rdc}--\n";
		
		print OVW_CYA "read rdc $name\n";
		print OVW_CYA "rdcscale\n";
	}
	print OVW_CYA "$overview_command\n";
	print OVW_CYA "cut_upl=0.5\n";
	print OVW_CYA "quit\n";
	close(OVW_CYA);
	unless (-f "myOverview.cya") {
		print "myOverview.cya doesn't exist\n";
		sleep(2);
	}

	#???
	#Utils->qsub($ENV{'SUBMITPBS'}, $ENV{'CyanaCmd'} . " < " . getcwd() . "/myOverview.cya > " . getcwd() . "/LogCyanaOvw", "overview");
	system($ENV{'CyanaCmd'} . " < myOverview.cya > LogOverview");

	my $cwd = getcwd();
	my $pdb_file = "$cwd/${name}.pdb";
	my $ovw_file = "$cwd/${name}.ovw";
	return ($pdb_file, $ovw_file);
}

sub runAMBI {
    my $self = shift;

    my $name = $self->{options}->{na};
    #prepre EC constraints
    open(SA, ">init.cya");
    print SA "name := $self->{options}->{na}\n";
    print SA "cyanalib\n";
    print SA "read seq $self->{options}->{na}\n";
#    print SA "rdcscale\n";
    print SA "cut_upl=0.5\n";
    print SA "weight_rdc = 0.003\n";
#    print SA "weight_rdc = 0.005\n";
#    print SA "weight_aco = 10\n";
    open(AMBI, ">ambiEC.cya");
#    print AMBI "name := $self->{options}->{na}\n";
#    print AMBI "cyanalib\n";
#    print AMBI "read seq $self->{options}->{na}\n";
    open(EC, "<$name.ec");
    while (<EC>)
    {
	chomp;
	$_ =~ s/^\s+//g;
	my (@list) = split(/\s+/, $_);
	$list[2] = 1 if ($list[2] !~ /\d/);
	print AMBI "distances make first=\"C* $list[0]\" second=\"C* $list[1]\" ambiguous upl=$list[2]\tweight=$list[3]\n";
    }
    print AMBI "write upl $name\_EC.upl\n";
    close AMBI;
    	my $commands = $ENV{'CyanaCmd'} . " ambiEC.cya > logAmbi\n";
        $commands .= "cat < $name\_EC.upl >> $name.upl\n";
    print "exec $commands\n";
	`$commands`;
}

##important###
sub prepare_batch {
	my $self = shift;
	my $seed = shift;
	my $cmdname = shift;
	my $process = shift;
	my $name = $self->{options}->{na};
	my $filename = "Smt${cmdname}.sh";

	Utils->debug("in CyanaEC->prepare_batch() creating Smt${cmdname}.sh");

	my $commands;

	#RT if run on local machine, noq
	if ($self->{options}->{que} =~ /NO/i) {
		$commands .= $ENV{'CyanaCmd'} . " autorunEC.cya > logEC";
		$self->{queue}->create_batch($filename, $name, $commands);
	} else {
		$commands .= $ENV{'CyanaCmd'} . " -t openmpi autorunEC.cya > logEC";
    $self->{queue}->create_batch_MPI($filename, $name, $commands);
	}
}

sub prepare_run {
    my $self = shift;

    Utils->debug("in CyanaEC->prepare_run");

    #prepare autorunEC
    open(SA, "<$ENV{'ASLIB'}/runEC.cya");
    $old = $/;
    undef $/;
    $sa = <SA>;
    $/ = $old;
    #if ($self->{options}->{peaks} =~ /peaks/i) {
		$sa =~ s/\$peaks/$self->{options}->{peaks}/;
    #}
    if (!defined $self->{options}->{rdc}) {

	$sa =~ s/\$name\.rdc,*//;
	#print OVW_CYA "rdcdistances\n";
	#print OVW_CYA "read rdc $name\n";
    }
    if (defined $self->{options}->{steps}) {
	#Utils->debug("in steps");
	if ($self->{options}->{steps} > 10000)
	{
	    #Utils->debug(">");
	    $sa =~ s/steps\s+\:= 10000/steps       := $self->{options}->{steps}/;
	}
    }
    if (defined $self->{options}->{upl}) {
	#Utils->debug(">");
	$sa =~ s/upl_values\s+\:= 2.4,5.0/upl_values  := $self->{options}->{upl}/;
    }

    $sa =~ s/\$name/$self->{options}->{na}/g;

    #my $total = $self->{options}->{np}*$self->{options}->{nstr};
		my $total = $self->{options}->{mpi};

    my $thalf = int $total/5;
    $sa =~ s/\$t1/$total/;
    $sa =~ s/\$t2/$thalf/;

    my $tols = $self->{options}->{tols};

		$sa =~ s/\$tols/$tols/;

    open(SA, ">autorunEC.cya");
    #print SA "read lol hbond.lol append\n";
    #print SA "read upl hbond.upl append\n";
    print SA $sa;
    close SA;
}

sub get_top_ranked_models {
	my $self = shift;
	my $dp_hash_ref = shift;
	my $ovw_file = shift;
	Utils->debug("loading target fn values from overview file: " . $self->{ovw_file});
	open(OVW,"<" . $ovw_file) or die("failed to open overview file " . $ovw_file . ":$!\n");
	my $in_flag = 0; #flag the model energy value region
	my %energy = ();
	#my $weight = 100;
	my $min = 1000000;
	while (<OVW>) {
		last if (/^\s+Ave/);
		if ($in_flag){
			if (/\s+(\d+)\s+(\S+)/) {
				Utils->debug("target fn. for $1\tis $2");
				$energy{$1} = $2; # / $weight;
				$min = $2 if ($2 < $min);
			}
			next;
		}
		if (/^\s+str/) {
			$in_flag = 1;
		}
	}
	close OVW;
	my $weight = $min*25;

	my %rank = ();
	#print "keys in dp_hash_ref " . scalar(keys %{$dp_hash_ref}) . "\n";
	foreach (reverse sort {$dp_hash_ref->{$a}<=>$dp_hash_ref->{$b}} keys %{$dp_hash_ref}) {
		$rank{$_} = ((1.0 * $energy{$1}/$weight) - (1.0 * $dp_hash_ref->{$_})) if (/(\d+)/);
		#print "RANK $_ : " . $dp_hash_ref->{$_} . "\n";
		#print "dp_score for $_ = " . $dp_hash_ref->{$_} . "\n";
		#print "Energy for $1 = " . $energy{$1} . "\n";
		#print "*energy - dp =* " . $rank{$_} . "\n";
	}
	my @rank_models = sort {$rank{$a}<=>$rank{$b}} keys %rank;
	#print "keys in rank " . scalar(keys %rank) . "\n";
	Utils->debug("BEST RANKED model is " . $rank_models[0] . " with a score of: " . $rank{$rank_models[0]});
	open(RANKS, ">RANKS");
	print RANKS "#total = (target / $weight) - dp\n";
	print RANKS "#model  \ttarget\tdp\ttotal\n";
	for (my $i = 0; $i < scalar(@rank_models); $i++) {
		if ($rank_models[$i] =~ /(\d+)/) {
			print RANKS $rank_models[$i] . "  \t" . ($energy{$1}) . "\t" . $dp_hash_ref->{$rank_models[$i]} . "\t" . $rank{$rank_models[$i]} . "\n";
		}
	}
	close(RANKS);
	return \@rank_models;
}

sub do_cleanup {
	my $self = shift;
	my $name = $self->{options}->{na};
	#TODO condense the rm's into one command to minimize process forking!

#	system("more Smt*sh > SubCmd_ASDP");
	#system("cat run_*sh > SubCmd_DP");
#	system("more calc_para* > SubCmd_AS" . ref($self));
#	system("more Calc_e* > LogError");
#	system("rm -f ${name}?*.pdb");
#	system("rm -f log* *ang Calc_*");


	system("rm -f Done*");
	system("more log* > Log" . ref($self));
	system("more Smt* > SubCmd_ECNMR" . ref($self));

	unless ($ENV{'DEBUG'} == 1) {
	system("rm -f all_models.pdb log* Smt* Calc*"); #" SedFile* FileCis* SSBridges* SSNoes*");
	system("rm -f ${name}???.ang");
	#system("rm -rf rpf_out");
	}

	#system("cat calc_para* > Calc_para_" . ref($self));
	#system("rm -f calc_para*");
}
return(1);
