#run DP scores with single cpu

#todo - DP with mpi

package DP;

use Cwd;
use IO::Handle;
use ASDP::Utils;
use ASDP::Protocol;

@ISA = qw(Protocol);

sub new {
	my $class = shift;
	my $self = $class->SUPER::new( @_ );

	$self->{coor_file} = shift;
	$self->{num_models} = 0;
	$self->{models} = undef;
	$self->{rpf_outdir} = $ENV{'RPF_DIR'};
	$self->{dp_hash} = undef;
	$self->{rpf_scores} = {};
	$self->{options} = shift;
	#my $options_ref = shift;
	bless($self, $class);

#print $self->{options}->{cf} . "\n";
	$self->{ctrl_file} = $self->{options}->{cf};
	$self->{n_model} = $self->{options}->{nb};

	die("number of model should be an integer\n") unless($self->{n_model} =~ /\d+/);
	return $self;
}


# sub new {
# 	my $class = shift;
# 	my $self = {};
# 	$self->{coor_file} = shift;
# 	#$self->{ovw_file} = shift;
# 	$self->{ctrl_file} = shift;
# 	$self->{n_model} = shift;
# 	$self->{num_processes} = shift;
# 	$self->{structures_per_process} = shift;
# 	#$self->{que} = shift;
# 	unless (($self->{que}) and $self->{que} =~ /\S+/) {
# 		$self->{que} = "PBS";
# 	}
# 	$self->{num_models} = 0;
# 	$self->{models} = undef;
# 	$self->{rpf_outdir} = '/rpf_out';
# 	$self->{dp_hash} = undef;
# 	$self->{rpf_scores} = {};
# 	bless($self, $class);
# 	if (defined $ENV{'RPF_DIR'}) {
# 		$self->{rpf_outdir} = $ENV{'RPF_DIR'};
# 	}
# 	#Utils->debug("rpf dir is " . $self->{rpf_outdir});
# 	#die("coordinate file '" . $self->{coor_file} . "' does not exist\n") unless(-e $self->{coor_file});
# 	die("number of model should be an integer\n") unless($self->{n_model} =~ /\d+/);
# 	return $self;
# }

#RT -- removed commented, trying to get Refinement working, 2019
sub split_models_DP {
	my $self = shift;
	my $name = $self->{options}->{na};
	Utils->debug("splitting pdb into models");
	my %models= ();

	if (!defined $self->{options}->{ec})
	{
		#print "not ec\n";
		opendir(DIR, ".") || die "Error in opening current dir\n";
		while ($filename = readdir(DIR))
		{
			#print "$filename\n";
			if ($filename =~ /$name(\d\d\d).pdb/m)
			{
			my $id = $1 * 1;
			$models{$id} = $filename;
			#print "$id $models{$id}\n"
			#	push(@models, $filename);
			}
		}
	} else {
		#print "is ec\n";
		open(F, "<" . $self->{coor_file}) or die "failed to open file pdb file: $!\n";
		my $temp_line;
		foreach (my $i = 0; $i < $self->{num_processes}*$self->{structure_per_process}; $i++) {
			if ($temp_line =~ /^END\s*$/) {
				last;
			}
			open(MODEL, ">model$i");
			Utils->debug("creating model$i");
			foreach (my $j = 0; $j < 1; ) {
				$temp_line = <F>;
				unless ($temp_line) {
					last;
				}
				if ($temp_line =~ /^ENDMDL\s*$/) {
					$self->{num_models}++;
					$j++;
				}
				if ($temp_line =~ /^END\s*$/) {
					last;
				}
				print MODEL $temp_line;
			}
			print MODEL "END\n";
			push(@models, "model$i");
			close(MODEL);
		}
	}
	$self->{models} = \%models;
}


sub run_rpf {
	my ($self, $models_ref) = @_;
	my @nArry;
	#my %models = %{$self->{models}};
	my %models = %{$models_ref};
	my $num_models = scalar(keys %models);
	#print "$num_models\n";

	my %dp_hash;
	if (-d $self->{rpf_outdir}) {
		system("rm -rf " . $self->{rpf_outdir}) && die"failed to delete rpf_outdir:$!\n";
	}

	die("ctrl file '" . $self->{ctrl_file} . "' does not exist\n") unless(-e $self->{ctrl_file});
	mkdir($self->{rpf_outdir}) or die "failed to create directory " . $self->{rpf_outdir} . ":$!\n";
	my $info = ""; #cyana cycle information
	$info = $1 if ($self->{coor_file} =~ /(Cycle.*)\//i);
	#Utils->debug("about to create " . $ENV{'DP_WD'} . "/${info}rpf_summary");
	#my $summary_filename = "/${info}rpf_summary";
	#if ($cns) {
	#	$summary_filename .= "_$cns";
	#}
	#open(SUM, ">" . $ENV{'DP_WD'} . "/${info}rpf_summary") or die "failed to open rpf_summary:$!\n";
	#print SUM "NAME\tDP_Score\n";
	#for (my $process_num = 0; $process_num < $self->{num_processes} and $i + $process_num < $num_models; $process_num++) {
	for (my $i = 1; $i <= scalar(keys %models); $i++) {

		#print "$i $models{$i}\n";

		#$self->prepare_batch($models{$i}, $i);

		$self->do_batch(0, $models{$i}, $i);

		#$self->do_batch($models{$i}, $i);
		#print "$i $models{$i} batch\n";
		#$self->batch_PBS_RPF($models[$i], $i);
		#system($ENV{'SUBMITPBS'} . " run_$i.sh -q \@master3");
		#push @nArry, $models[$i];
	}
	sleep(15); #wait for the queue flush
	for (my $i = 1; $i <= scalar(keys %models); $i++) {
		$self->load_scores($i, $models{$i});
	}
	for (my $i = 1; $i <= scalar(keys %models); $i++) {
		#my ($dp,$f,$recall,$precision) = $self->get_scores_for_model($i);
		my $scores = $self->get_scores_for_model($i);
		my ($dp, $f, $recall, $precision) = @$scores;
		if (defined $dp) {
			#Utils->debug("model$i\t: $dp $f $recall $precision");
			#print SUM "model$i\t$dp\t$f\t$recall\t$precision\n";
			#print SUM "model$i\t$dp\n";
			$dp_hash{"model$i"} = $dp; # use -s to report dp only
		}
	}
	#close SUM;
	$self->{dp_hash} = \%dp_hash;
	return \%dp_hash;
}
sub load_scores {
	my $self  = shift;
	my $process = shift;
	my $model_dir = shift;
	while (not (-f "Done_$model_dir")) {
		#Utils->debug("rpf not done. *Done_$model_dir* does not exist");
		sleep(2);
	}
	unless (-e $self->{rpf_outdir} . "/$model_dir") {
		Utils->debug("ERROR: no rpf output directory found for $model_dir");
	}
	opendir(RPF_RESULT, $self->{rpf_outdir} . "/$model_dir");
	my @files = readdir(RPF_RESULT);
	foreach (@files) {
		if (/\.ovw$/) {
			my $scores = $self->parse_ovw($self->{rpf_outdir} . "/$model_dir/$_");
			foreach $score_index (keys %{$scores}) {
				#my $model = $process * $self->{structures_per_process} + $score_index;
				#$self->{rpf_scores}->{$model} = $scores->{$score_index};
				$self->{rpf_scores}->{$process} = $scores->{$score_index};
				#print "$process $model_dir" . ${$scores->{$score_index}}[0] .  "in load scores\n"
			}
			return;
		}
	}
	Utils->debug("ERROR: no ovw file found inside rpf output folder for $model_dir");
}
sub get_scores_for_model {
	my $self  = shift;
	my $model = shift;
	if (defined $self->{rpf_scores}->{$model}) {
		return $self->{rpf_scores}->{$model};
	}
	return undef;
}

sub get_top_ranked_models {
	my $self = shift;
	my $ovw_file = shift;
	open(OVW,"<" . $ovw_file) or die("failed to open overview file " . $ovw_file . ":$!\n");
	my $in_flag = 0; #flag the model energy value region
	my %energy = ();
	my $weight = 100;
	while (<OVW>) {
		last if (/^\s+Ave/);
		if ($in_flag) {
			if (/\s+(\d+)\s+([\d\.\-]+)/) {
				$energy{$1} = $2 / $weight;
			}
			next;
		}
		if (/^\s+str/) {
			$in_flag = 1;
		}
	}
	close OVW;
	my %rank = ();
	#print "\n";
	foreach (reverse sort {$self->{dp_hash}->{$a}<=>$self->{dp_hash}->{$b}} keys %{$self->{dp_hash}}) {
		#print "$_ : " . $self->{dp_hash}->{$_} . "\n";
		$rank{$_} = $energy{$1} - $self->{dp_hash}->{$_} if (/(\d+)/);
	}
	my @rank_models = sort {$rank{$a}<=>$rank{$b}} keys %rank;
	$self->merge_models_DP(\@rank_models);
}

#RT -- removed commented, trying to get Refinement workin, 2019
sub merge_models_DP {
	my $self = shift;
	my $rank_models_ref = shift;
	my @rank_models = @$rank_models_ref;

	if (defined $self->{options}->{ec})
	{
		my %loaded_models;
		@loaded_models{@rank_models} = ();
		open(PDB_READ, "<" . $self->{coor_file}) or die "failed to open " . $self->{coor_file} . ":$!\n";
		my $model_number;
		my @model_lines;
		while (<PDB_READ>) {
			if (/MODEL\s+(\S+)/) {
				$model_number = $1;
			}
			if (exists $loaded_models{"model$model_number"}) {
				push(@model_lines, $_);
			}
			if (/^ENDMDL\s*$/) {
				my @new_array = @model_lines;
				@model_lines = ();
				$loaded_models{"model$model_number"} = \@new_array;
			}
		}
		close(PDB_READ);
       		open(F, ">" . $self->{coor_file}) or die "failed to open " . $self->{coor_file} . ":$!\n";
       		for (my $i = 0; $i < $self->{n_model}; $i++) {
                	my $item = $rank_models[$i];
                	my @lines = @{$loaded_models{$item}};
                	foreach my $line (@lines) {
                        	print F $line;
        	        }
                	print F "END\n";
			close(F);
		}
	} else {
		open(F, ">" . $self->{coor_file}) or die "failed to open " . $self->{coor_file} . ":$!\n";
		my $name = $self->{coor_file};
		$name  =~ s/.pdb//g;

		for (my $i = 0; $i < $self->{n_model}; $i++) {
			my $item = $rank_models[$i];
			$item =~ s/model//g;
			my $istr = sprintf("%03d", $item);
			open(MODEL, "<" . $name . $istr . ".pdb") or die "failed to open " . $name . $istr . ".pdb : $!\n";
			print F "MODEL    $istr\n";
			while ($line = <MODEL>)
			{
				next if ($line !~ /^ATOM|TER|HETATM|REMARK/);
				print F $line;
			}
			print F "ENDMDL\n";
		}

		print F "END\n";
		close(F);
	}
	return;
}

sub prepare_batch {
	my $self = shift;
	my ($seed, $pdb_file, $id) = @_;
	my $filename = $ENV{'DP_WD'} . "/Smt" . $pdb_file . "$id" . ".sh";
	my $autostr_path = $ENV{'NOESYASSIGN'} ; #. "/noesyassign";
	my $ctrl_file = $self->{ctrl_file};
	my $rpf_dir = $self->{rpf_outdir};
	my $option = "s";
	my $commands = "";
	$commands .= "export ASLIB=" . $ENV{'ASLIB'} . "\n";
	$commands .= "\n";
	$commands .= "$autostr_path -c $ctrl_file -q " . $ENV{'DP_WD'} . "/$pdb_file -o $rpf_dir/$pdb_file -$option\n";
	$commands .= "touch Done_$pdb_file\n";
	#print "$filename $id\n";
	$self->{queue}->create_batch($filename, $id, $commands); #send to queue
	#create_batch($filename, "$id", $commands);
}

sub do_cleanup {
	my $self = shift;
	system("rm Done*");
	unless ($ENV{'DEBUG'} == 1) {
		#system("more Smt*sh > SubCmd_DP");
		system("rm -f model*");
		#system("zip -mqT all_models.zip model*");
		system("rm -f run_*");
	}
}
sub parse_ovw {
	my $self = shift;
	my $ovw_file = shift;
	my $current_model = 0;
	my $scores = ();
	open(F, $ovw_file) or die "failed to open $ovw_file: $!\n";
	my ($dp, $f, $recall, $precision);
	$dp=0; $f=0; $recall=0; $precision=0;
	while (<F>) {
		if (/Model\s+(\S+)\s*$/) {
			$current_model = $1;
		}
		if (/DP-Score: (\S+)/) {
			$dp = $1;
			unless ($current_model =~ /ave/i) {
				$scores->{$current_model} = [$dp, $f, $recall, $precision];
				my $score_array = $scores->{$current_model};
				Utils->debug("model$current_model new score: $dp, $f, $recall, $precision. size of scores is " . scalar(keys (%$scores)));
			}
		}
		if (/Final Recall-score for input query structures:\s+(\S+)/) {
			$recall = $1;
		}
		if (/Final Precision-score for input query structures:\s+(\S+)/) {
			$precision = $1;
		}
		if (/F-score of input query structures:\s+(\S+)/) {
			$f = $1;
		}
	}
	close(F);
	#if (!defined ($scores->{$current_model}))
	#{
	#	$scores->{$current_model} = [$dp, $f, $recall, $precision];
	#}
	return $scores;
}
return 1;
