#cluster protocol
#should be generlized, that no code need to be changed here for differeint queues

package Protocol;

use Cwd;
use ASDP::DP;
use ASDP::Utils;
use queue::PBS;
use queue::Custom;
use queue::SLURM;
use queue::NoQ;

sub new {
	my $class = shift;
	my $self = {};
	$self->{options} = shift;
	$self->{calc_duration} = 2;
	$self->{max_calc_wait} = 60;
	$self->{command} = $ENV{'CyanaCmd'};
	my $options_ref = shift;
	bless($self, $class);
	return $self;
}

sub run {
	my $self = shift;
	my $seeds_ref = shift;
	Utils->debug("in Protocol->run()");
	#my $processes = $self->{options}->{np}; #same
	my $process = $self->{options}->{np}; #same
	my $name = $self->{options}->{na};
	Utils->debug("processes: " . $process . ", NS is: " . $self->{options}->{nstr});
	my @seeds = @{$seeds_ref};
	foreach my $seed (@{$seeds_ref}) {
		$seed += 0;
		#$seed = 2377;
		#my $structures = $process * $self->{options}->{nstr};
		my $structures = (defined $self->{options}->{structures}) ? $self->{options}->{structures} : $process * $self->{options}->{nstr};
		my $begin = $structures - $self->{options}->{nstr} + 1;
		my $end = $begin + $self->{options}->{nstr} - 1;
		Utils->debug("current SEED is: " . $seed . " structures is: " . $structures . " BEGIN is: " . $begin . " END is: " . $end);
		$self->prepare_run($process, $structures, $begin, $end, $seed);
		$self->do_batch($seed, "Calc" . $name, $process);
		$process--;
	}
}

sub wait_for_structure_calculations {
	my $self = shift;
	my $name = $self->{options}->{na};
	my $max_wait = (defined $self->{max_calc_wait} ? $self->{max_calc_wait} : 60);
	my $wait_time = (defined $self->{calc_duration} ? $self->{calc_duration} : 2);
	my $accumulated_wait = 0;
	Utils->debug("in CreateProc->wait_for_structure_calculations()");
	for (my $i = 1; $i <= $self->{options}->{np}; $i++) {
#	    next;
		if (-e "Done${name}${i}") {
			next;
		} else {
			$accumulated_wait += $wait_time;
			sleep($wait_time);
			$i--;
		}
	}
	Utils->debug("total wait was: $accumulated_wait seconds.");
}

sub split_models {
       my $self = shift;
       my $name = $self->{options}->{na};
       Utils->debug("splitting pdb into models");
       my %models= ();

       opendir(DIR, ".") || die "Error in opening current dir\n";
       while ($filename = readdir(DIR))
       {
             #print "$filename\n";
	     #RT this line serves only for CYANA, not for Xplor $name_sa
	     #if ($filename =~ /$name(\d\d\d).pdb/m or $filename =~ /$name_sa_(\d\d\d).pdb/m)
	     if ($filename =~ /$name(\d\d\d).pdb/m)
             {
                   my $id = $1 * 1;
                   $models{$id} = $filename;
                   #print "$id $models{$id}\n"
                   #push(@models, $filename);
              }
	}
	$self->{models}=\%models;
	return \%models;
}

sub merge_models {
       my $self = shift;
       my $pdb = shift;
       my $rank_models_ref = shift;
       my $n_model = shift;

       Utils->debug("in merge_models() with: $pdb, $rank_models_ref, $n_model");

       my @rank_models = @$rank_models_ref;

       open(F, ">" . $pdb) or die "Protocol.pm: failed to open " . $pdb . ":$!\n";
       my $name = $pdb;
       $name  =~ s/.pdb//g;

       for (my $i = 0; $i < $n_model; $i++) {
          my $item = $rank_models[$i];
	  			Utils->debug("merging $item");
          $item =~ s/model//g;
          my $istr = sprintf("%03d", $item);
          #print "$i++$item++$istr++$name\n";
          #my @lines = @{$loaded_models{$item}};

          open(MODEL, "<" . $name . $istr . ".pdb") or die "Protocol.pm: merge_models(): failed to open " . $name . $istr . ".pdb : $!\n";
          print F "MODEL    $istr\n";
          while ($line = <MODEL>)
          {
             next if ($line !~ /^ATOM|TER|HETATM|REMARK/);
             print F $line;
          }
          print F "ENDMDL\n";
      }

      print F "END\n";
      close(F);
      return;
}

sub do_batch {
	my $self = shift;
	my ($seed, $cmdname, $process) = @_;
	my $name = $self->{options}->{na};
	my $command;

	#print "do_batch " . $self->{options}->{que} . " " . $self->{options}->{na} . "\n";

	Utils->debug("in Protocol->do_batch() que is: " . $self->{options}->{que} . " looking for file: Smt${cmdname}${process}.sh");

	if ($self->{options}->{que} =~ /PBS/i) {
		$self->{queue} = PBS->new($self->{options});
	}	elsif ($self->{options}->{que} =~ /SLURM/i) {
			$self->{queue} = SLURM->new($self->{options});
	} elsif ($self->{options}->{que} =~ /Custom/i) {
		$self->{queue} = Custom->new($self->{options});
	} elsif ($self->{options}->{que} =~ /NO/i) {
		$self->{queue} = NoQ->new($self->{options});
	} else {
		die "this queue is not supported yet: " . $self->{options}->{que} . "\n";
	}

	$self->prepare_batch(@_);
	Utils->debug("submitting to " . $self->{options}->{que} . " Smt${cmdname}${process}.sh");
	$command = $ENV{'SUBMITCmd'} . " Smt${cmdname}${process}.sh";
	Utils->debug(" " . $command);
	sleep(0.2);
	system($ENV{'SUBMITCmd'} . " Smt${cmdname}${process}.sh");

	if ($? == -1) {
		die "failed to execute command: $command\nERROR: $!\n";
	}
}

return(1);
