package ControlFile;

use Utils;

sub new {
	my $class = shift;
	my $self = {};
	$self->{filepath} = shift;
	$self->{sections} = undef;
	$self->{peaklists} = [];
	$self->{general} = [];
	$self->{num_of_peaklists} = 0;
	bless($self, $class);
	$self->load_control_values();
	return $self;
}
sub load_control_values {
	my $self = shift;
	my ($temp_line);
	Utils->debug("about to open " . $self->{filepath});
	open ( FILE, $self->{filepath} ) or die "$!";
	binmode FILE;
	while( chomp($temp_line = <FILE>)) {
		next if (($temp_line =~ /^\#/) or ($temp_line =~ /^\s*$/));
		if ($temp_line =~ /(\[.+\])/) {
			my $section_name = $1;
			$self->load_section($1, FILE);
		}
	}
	close FILE;
	unlink FILE;
}
sub load_section {
	my $self = shift;
	my $section_name = shift;
	Utils->debug("section is: " . $section_name);
	my $file_handle = shift;
	$self->{sections}{$section_name} = {};
	unless ($section_name =~ /general/i) {
		$self->{num_of_peaklists}++;
		push(@{$self->{peaklists}}, $section_name);
	}
	my ($temp_line);
	while( chomp($temp_line = <$file_handle>)) {
		next if (($temp_line =~ /^\#/) or ($temp_line =~ /^\s*$/));
		if ($temp_line =~ /(\[.+\])/) {
			return $self->load_section($1, $file_handle);
		}
		if ($temp_line =~ /([^=\s\.]+)\.([^=\s\.]+)=([^=\s]+)/) {
			unless (defined $self->{sections}{$section_name}{$1}) {
				$self->{sections}{$section_name}{$1} = {};
			}
			$self->{sections}{$section_name}{$1}{$2} = $3;
		} else {
			unless (defined $self->{sections}{$section_name}{lines}) {
				$self->{sections}{$section_name}{lines} = [];
			}
			push(@{$self->{sections}{$section_name}{lines}}, $temp_line);
		}
	}
}
sub update_value {
	my $self = shift;
	my $section = shift;
	my $element = shift;
	my $attr = shift;
	my $value = shift;
	Utils->debug("updating: $section, $element, $attr -> $value");
	$self->{sections}{$section}{$element}{$attr} = $value;
}
sub write_to_disk {
	my $self = shift;
	my $upload_dir = shift;
	open ( FILE, ">" . $self->{filepath} ) or die "$!";
	binmode FILE;
	Utils->debug("new control file:");
	foreach my $section_name (keys (%{$self->{sections}})) {
		my $section = $self->{sections}{$section_name};
		Utils->debug("* $section_name");
		print FILE "$section_name\n";
		foreach my $element_name (keys (%$section)) {
			my $element = $self->{sections}{$section_name}{$element_name};
			Utils->debug("* $element");
			if (ref($element) eq "HASH") {
				foreach my $attr_name (keys (%$element)) {
					my $value = $self->{sections}{$section_name}{$element_name}{$attr_name};
					Utils->debug("* $element_name.$attr_name=$value");
					print FILE "$element_name.$attr_name=$value\n";
				}
			} else {
				foreach my $line (@$element) {
					Utils->debug("* $line");
					print FILE "$line\n";
				}
			}
		}
	}
	close FILE;
}
return(1);
