package Rasmol;

use Utils;

my $bg = '#EFF8FF';
my $but_bg = '#265580';
my $but_fg = $bg;
my $cbt_act = '#D1E4F2';
my $but_act = '#2D4455';

sub new {
	my $class = shift;
	my $self = {};
	$self->{protein_name} = shift;
	$self->{ql_file} = shift;
	$self->{pdb_file} = shift;
	$self->{results_path} = shift;
	$self->{results} = shift;
	#print $self->{protein_name} . " _ " . $self->{ql_file} . " _ " . $self->{pdb_file} . " _ " . $self->{results_path} . " _ " . $self->{results} . "\n";
	$self->{filter} = "H";
	$self->{script_file} = $self->{results_path} . "/" . $self->{protein_name} . ".rasmol";
	bless($self, $class);
	my @count_array = $self->get_count_array($ql_file, $filter);
	my $script = $self->create_rasmol_script(\@count_array, $pdb_file);
	my $write_success = $self->write_script_file($self->{script_file}, $script);
	return $self;
}

sub get_script_path {
	my $self = shift;
	return $self->{script_file};
}

sub create_rasmol_script {
    my ($self, $count_array_ref, $pdb_file) = @_;
    my @count_array = @$count_array_ref;
    my $script;
    my $color;
    my ($i, $j);
    #options for rasmol
    my $image_mode = "cartoons";    #trace or cartoons
    my $RED = 370;
    my $ORANGE = 268;
    my $YELLOW = 165;
    my $GREEN = 121;
    my $BLUE_GREEN = 78;
    my $BLUE = 35;
    $script = "#!rasmol -script\n# File: 1\n# Creator: RasMol Version 2.7.1\n\nzap\n";
    $script = $script."load pdb \"$pdb_file\" ";
    $script = $script."background white\nset ambient 40\nset specular off\n\nreset\nslab off\n\nset bonds off\nset axes off\nset boundingbox off\nset unitcell off\nset bondmode and\ndots off\n\n\n";
    $script = $script."# Avoid Colour Problems!\nselect all\ncolour bonds none\ncolour backbone none\ncolour hbonds none\ncolour ssbonds none\ncolour ribbons none\ncolour cpk\n\n# Atoms\nselect *\ncolor [0,0,255]\n";
    $script = $script."wireframe off\nselect *\n$image_mode\n";
    #loop through count_array
    for ($i = 0; $i<=$#count_array; $i++) {
		#print "count_array[$i] = $count_array[$i]\n";
		next unless ($count_array[$i]);
		my $count = $count_array[$i];
		#default is blue
		next unless ($count > $BLUE);
		if ($count >= $RED) {
		    $color = "[255, 0, 0]";
		    $script = $script.$self->create_color_script($color, $i, 1);
		    next;
		} elsif ($count >= $ORANGE) {
			my $green_component = (($RED - $count) * 128)/($RED - $ORANGE);
			$green_component = sprintf("%.0f", $green_component);
			$color = "[255, $green_component, 0]";
			$script = $script.$self->create_color_script($color, $i, 1);
		} elsif ($count >= $YELLOW) {
			my $green_component = 127 + (($ORANGE - $count) * 128)/($ORANGE - $YELLOW);
			$green_component = sprintf("%.0f", $green_component);
			$color = "[255, $green_component, 0]";
			$script = $script.$self->create_color_script($color, $i, 1);
		} elsif ($count >= $GREEN) {
			my $red_component = (($count - $GREEN) * 255)/($YELLOW - $GREEN);
			$red_component = sprintf("%.0f", $red_component);
			$color = "[$red_component, 255, 0]";
			$script = $script.$self->create_color_script($color, $i, 0);
		} elsif ($count >= $BLUE_GREEN) {
			my $blue_component = (($GREEN - $count) * 255)/($GREEN - $BLUE_GREEN);
			$blue_component = sprintf("%.0f", $blue_component);
			$color = "[0, 255, $blue_component]";
			$script = $script.$self->create_color_script($color, $i, 0);
		} elsif ($count > $BLUE) {
			my $green_component = (($count - $BLUE) * 255)/($BLUE_GREEN - $BLUE);
			$green_component = sprintf("%.0f", $green_component);
			$color = "[0, $green_component, 255]";
			$script = $script.$self->create_color_script($color, $i, 0);
		}
    }    
    $script = $script."restrict */1\ncolour label black\nbackground white\nzoom 150\n";
    return($script);
}

sub create_color_script {
    my ($self, $color, $residue, $extra) = @_;
    my $script = "select $residue\n";
    $script = $script."color $color\n";
    if ($extra) {
		$script = $script."wireframe 80\n";
		$script = $script."select $residue and *.ca\n";
		$script = $script."label $residue\n";
		$script = $script."select $residue and *.ca\n";
		$script = $script."set fontsize 10\n";
		$script = $script."center 1\n";
    }
    return ($script);
}

sub get_count_array {
    my ($self, $file, $filter) = @_;
    my $basepath = Cwd::cwd;
    my ($data_ref, $heading_ref, $residue_ref) = $self->{results}->get_ql_parse_data();
    my @heading = @$heading_ref;
    my @residue = @$residue_ref;
    my @data = @{$data_ref};
    #array of counts for each residue
    my @count;
    my $dummy = 0;
    my $section = 1;
    my ($i, $j, $first_atom, $second_atom);
    #print "\n\ncalc_count: number of residues = $#residue\n";
    for ($i=0; $i<=$#residue; $i++) {
		next unless $residue[$i];
		#print "Residue[$i] = $residue[$i]\n";
		$count[$i] = $self->getCount(\@data, $i, $residue_ref, $heading_ref, $filter);
		#print "get_count_array: Residue $i - $residue[$i], count is $count[$i]\n";
    }
    return (@count);
}

#given i, returns the count of residue i, exluding intra residue contacts
sub getCount {
	my ($self, $data_ref, $i, $residue_ref, $heading_ref, $filter) = @_;
	my @data = @$data_ref;
	my @residue = @$residue_ref;
	my @heading = @$heading_ref;
	my %dist_hash;
	my $dist = 0;
	my $count = 0;
	#print "\n******getCount: getting count of residue $i*******\n";
	my ($j, $i_atom, $j_atom, $k, $dist_hash_ref);
	$dist_hash_ref = \%dist_hash;
	#print "**Residue $i on left\n";
	#Left residue = $i, Right residue = $j - loop
#	if ($key =~ /summary/i) { #for the overal summary
		for ($k = 0; $k<=$#heading; $k++) {
			#next if ($k == 1);
			#next if ($k == 3);
			$self->getLeftCountofi($data_ref, $i, $k, $filter, $dist_hash_ref, $residue_ref, 0); # 0 - no intra 
		}
		for ($k = 0; $k<=$#heading; $k++) {
			#next if ($k == 1);
			#next if ($k == 3);
			$self->getRightCountofi($data_ref, $i, $k, $filter, $dist_hash_ref, $residue_ref, 0);
		}
#	}
	#sum up all distances from dist_hash
	my $key;
	$count = 0;
	#clean-up by removing the non-symmetric ones 
	my %nonsym; 
	foreach $key (keys %$dist_hash_ref) {
		my ($a1, $a2) = split '\s', $key;
		if ($dist_hash_ref->{$a2 . ' ' . $a1} != $dist_hash_ref->{$key}) {
			$nonsym{$key} = 1; 
		}
	}
	foreach $key (keys %$dist_hash_ref) {
		next if ($nonsym{$key} == 1);
		my $dist = $dist_hash_ref->{$key};
		#$dist = 30000 * ($dist ** (-6))/2;
		$dist = 15000 * ($dist ** (-6))/2; #devided by 2 - remove the symmetry one 
		$count = $count + $dist;
	}
	#print "total distance for residue $i is $dist\n";
	#print "getCount: returning count of $i as $count\n";
	return $count;
}

#filter is used in calc_rasmol analysis
#getRightCountofi (report x-i (x<i)
#getLeftCountofi (report i-x (x>=i)) 
#i-x (i<=x)
sub getLeftCountofi {
    my ($self, $data_ref, $i, $k, $filter, $dist_hash_ref, $residue_ref, $i_atom_filter, $j_filter, $j_atom_filter, $distance_comp_filter, $distance_filter, $intra) = @_;
    #print "getLeftCountofi: i = $i, k = $k, i_atom_filter = $i_atom_filter, j_filter = $j_filter, j_atom_filter = $j_atom_fitler, distance_comp_filter = $distance_comp_filter, distance = $distance_filter, filter = $filter\n";

    my @residue = @$residue_ref;
    my @data = @$data_ref;
    my ($i_atom, $j, $j_atom);
    foreach $i_atom (keys %{$data[$k][$i]}) {
		if ($i_atom =~ /^$filter$/i) {
			#print "filtered i: $i-$i_atom\n";
			next;
		}	
		for ($j = 0; $j<scalar(@{$data[$k][$i]{$i_atom}}); $j++) {
			next unless $residue[$j];
			next if (($i == $j) && !$intra);  
			#next if (($i == $j)); 		
		    if ($j_filter) {
				next unless ($j == $j_filter);
	    	}
		    foreach $j_atom (keys %{$data[$k][$i]{$i_atom}[$j]}) {
				if ($j_atom =~ /^$filter$/i) {
				    #print "filtered j: $j-$j_atom\n";
				    next;
				}
				#atom filter		    
				if ($j == $i) { #intra -
					if ($j_atom_filter eq '') {
						$j_atom_filter = '\w'; 
					}
					if ($i_atom_filter eq '') {
						$i_atom_filter = '\w'; 
					}	    
					next unless ((($i_atom =~ /$i_atom_filter/i) && ($j_atom=~ /$j_atom_filter/i)) ||(($i_atom =~ /$j_atom_filter/i) && ($j_atom=~ /$i_atom_filter/i)));  		    
				} else {
					if (($j_atom_filter)) { # j atom only 
						next unless (($j_atom =~ /$j_atom_filter/i));
					} 
					if (($i_atom_filter)) { # i atom only 
						next unless (($i_atom =~ /$i_atom_filter/i));
					}
				}
				if ($distance_filter) {
					my $distance = $data[$k][$i]{$i_atom}[$j]{$j_atom};
					#print "getLeftCountofi: distance = $distance, only distances $distance_comp_filter than $distance_filter.  ";
					if ($distance_comp_filter eq '=') {
						$distance_comp_filter = '==';
						#print "distance_comp_filter is now $distance_comp_filter\n";
					}
					unless (eval("$distance $distance_comp_filter $distance_filter")) {
						#print "distance $distance filtered\n";
						next;
					}
				}
				#print "residue $i on left - distance from $i-$i_atom to $j-$j_atom is $data[$i]{$i_atom}[$j]{$j_atom}\n";
				$dist_hash_ref->{"$i-$i_atom $j-$j_atom"} = $data[$k][$i]{$i_atom}[$j]{$j_atom};
				#print "getLeftCountofi: dist_hash_ref->{$i-$i_atom $j-$j_atom} = ".$dist_hash_ref->{"$i-$i_atom $j-$j_atom"}."\n";
			}
		}
    }
}


#x-i, x < i 
sub getRightCountofi {
    my ($self, $data_ref, $i, $k, $filter, $dist_hash_ref, $residue_ref, $i_atom_filter, $j_filter, $j_atom_filter, $distance_comp_filter, $distance_filter, $intra) = @_;
    #print "getRightCountofi: i = $i, k = $k, i_atom_filter = $i_atom_filter, j_filter = $j_filter, j_atom_filter = $j_atom_fitler, distance_comp_filter = $distance_comp_filter, distance = $distance_filter, filter = $filter\n";
    my @residue = @$residue_ref;
    my @data = @$data_ref;
    my ($i_atom, $j_atom) = ("", "");
    #print "**Residue $i on right\n";
    #right residue = i, left residue = j
    for ($j = 0; $j<scalar(@{$data[$k]}); $j++) {
		next unless $residue[$j];
		#next if ($i == $j);
		next if (($i == $j) && !$intra);#remove repeat entry from getRightCountofI
		if ($j_filter) {
		    next unless ($j == $j_filter);
		}	
		foreach $j_atom (keys %{$data[$k][$j]}) {
		    if ($j_atom_filter) {
				next unless ($j_atom =~ /$j_atom_filter/i);
	    	}
		    if ($j_atom =~ /^$filter$/i) {
				#print "filtered j: $j-$j_atom\n";
				next;
	    	}
		    foreach $i_atom (keys %{$data[$k][$j]{$j_atom}[$i]}) {
				if ($i_atom_filter) {
				    next unless ($i_atom =~ /$i_atom_filter/i);
				}
				if ($i_atom =~ /^$filter$/i) {
				    #print "filtered i: $i-$i_atom\n";
				    next;
				}
				if ($distance_filter) {
				    my $distance = $data[$k][$j]{$j_atom}[$i]{$i_atom};
				    #print "getRightCountofi: distance $distance, only distances $distance_comp_filter than $distance_filter.  ";
				    if ($distance_comp_filter eq '=') {
						$distance_comp_filter = '==';
						#print "distance_comp_filter is now $distance_comp_filter\n";
		    		}
				    unless (eval("$distance $distance_comp_filter $distance_filter")) {
						#print "distance $distance is filtered\n";
						next;
				    }
				    #next unless (eval("$distance $distance_comp_filter $distance_filter"));
				}
				#print "residue $i on right - distance from $j-$j_atom to $i-$i_atom is $data[$j]{$j_atom}[$i]{$i_atom}\n";
				unless ($dist_hash_ref->{"$j-$j_atom $i-$i_atom"}) {
				    #$dist_hash_ref->{"$i-$i_atom $j-$j_atom"} = $data[$k][$j]{$j_atom}[$i]{$i_atom};
				    $dist_hash_ref->{"$j-$j_atom $i-$i_atom"} = $data[$k][$j]{$j_atom}[$i]{$i_atom};
				    #print "getRightCountofi: dist_hash_ref->{$i-$i_atom $j-$j_atom} = ".$dist_hash_ref->{"$i-$i_atom $j-$j_atom"}."\n";
				}
	    	}
		}
    }
}

sub write_script_file {
    my ($self, $scr_fname, $script) = @_;
	#print "writing script file $scr_fname\n";
    unless (open(FILE, ">$scr_fname")) {
		return 0;
    }
    print FILE $script;
    close(FILE);
    return 1;
}

return 1;

