package Sequence;

use Utils;

sub new {
	my $class = shift;
	my $self = {};
	$self->{protein_name} = shift;
	$self->{filename} = $self->{protein_name} . "_sequence";
	$self->{filepath} = $ENV{'BASE_DIR'} . "/" . $self->{filename};
 	Utils->debug($self->{filepath});
	$self->{bmrb} = shift;
	bless($self, $class);
	$self->create_sequence_file();
	return $self;
}
sub create_sequence_file {
	my $self = shift;
	open(SEQ_FILE, ">" . $self->{filepath});
	Utils->debug("opening: " . $self->{filepath});
	my $sequence_text = $self->{bmrb}->extract_sequence();
	Utils->debug($sequence_text);
	print SEQ_FILE $sequence_text;
	close(SEQ_FILE);
}
sub create_sequence_file_deprecated {		#this function was used to extract sequence from pdb
	my $self = shift;
	my %AAcode_h = ('ALA' => 'A', 'CYS' => 'C', 'ASP' => 'D', 'GLU' => 'E', 'PHE' => 'F', 'GLY' => 'G', 'HIS' => 'H', 'ILE' => 'I', 'LYS' => 'K', 'LEU' => 'L', 'MET' => 'M', 'ASN' => 'N', 'PRO' => 'P', 'GLN' => 'Q', 'ARG' => 'R', 'SER' => 'S',  'THR'=> 'T', 'VAL' => 'V', 'TRP' => 'W', 'TYR' => 'Y', 'ASX' => 'B', 'GLX' => 'Z', 'CPR' => 'P');
	my $nbrffile_s = "cluster.nbrf"; #some initial line has to be written
	my $pdbfile_s = $self->{pdb}->get_filepath();
	my $bmrb_form_seq_file_s = $self->{filepath};

	#    print("$pdbfile_s\n"); REMOVE comment
		
	my $no_residue_n = 0;
	my $line_s;

	my @sequence_a; # stores 1 letter code
	my @three_letter_seq_a; # stores 3 letter code
	my @list_resnums_a; # stores the list of residue numbers, in ascending order
	my @res_name_num_list_a = (); # stores array containing residue name and number in arrays
	# Get the primary sequence
	open(IF, "<$pdbfile_s") || die("Can't open $pdbfile_s : $!");
	while ($line_s = <IF>) {
		if ($line_s =~ /^ENDMDL/) {
			last;
		}

		if ($line_s =~ /^ATOM/) { 
			# to check for alternate atom location
			my $alt_loc_s = substr($line_s, 16, 1);
			if (($alt_loc_s =~ /\S+/) && ($alt_loc_s ne "A")) {
				next;
			}
	
			# check to see if insertion-based residue numbering used
			if (substr($line_s, 26, 1) =~ /\w+/) {
				#EXCEPTION?
				#open(LF, ">>log_user") || die("Can't open error log in get_seq.pl : $!");
				#print(LF "$pdbfile_s has insertion codes in residue numbers. Please remove these with the apporpriate script.\n");
				#close(LF);
				die("$pdbfile_s has insertion codes in residue numbers.");
			}
		
			my $residue_s = substr($line_s, 17, 3);
			my $num_of_res = int(substr($line_s, 22, 4));
			my $atoms_type_s = substr($line_s, 12, 4);

			if ($atoms_type_s =~ / CA /) {
				push(@sequence_a, $AAcode_h{$residue_s});
				push(@three_letter_seq_a, $residue_s);
				push(@list_resnums_a, $num_of_res);
				push(@res_name_num_list_a, [ $num_of_res, $residue_s ]);
				$no_residue_n++;
			}
		}
	}
	close(IF);
	# for bmrb sequence
	open(BSF, ">$bmrb_form_seq_file_s") || die("Can't open $bmrb_form_seq_file_s : $!");
	my $count_n = 0;
	foreach my $res_rec_s (@res_name_num_list_a) {
		#print(BSF "$$res_rec_s[0]\t$$res_rec_s[0]\t$$res_rec_s[1]\t");
		print(BSF "$$res_rec_s[0]\t@\t$$res_rec_s[1]\t");
		$count_n++;
		if ($count_n == 5) {
			print(BSF "\n");
			$count_n = 0;
		}
	}
	close(BSF);
}
sub get_bmrb {
	my $self = shift;
	return $self->{bmrb};
}
sub set_bmrb {
	my $self = shift;
	$self->{bmrb} = shift;
}
sub get_filename {
	my $self = shift;
	return $self->{filename};
}
sub set_filename {
	my $self = shift;
	$self->{filename} = shift;
}
sub get_filepath {
	my $self = shift;
	return $self->{filepath};
}
sub set_filepath {
	my $self = shift;
	$self->{filepath} = shift;
}
return(1); 
 
