package Xplor;

use Cwd;
use ASDP::Protocol;
use ASDP::Utils;
use File::Copy;

@ISA = qw(Protocol);

sub new {
	my $class = shift;
	my $self = $class->SUPER::new( @_ );
	$self->{calc_duration} = 2;
	$self->{max_calc_wait} = 60;
	$self->{command} = $ENV{'XplorCmd'};
	$self->{first_residue_n} = $self->{options}->{st} - 1;
	$self->{template_file} = $ENV{'ASLIB'} . "/sa_for_AS.inp";
	$self->{template} = "sa_for_AS";
  # RT
  $self->{axis_file_both} = $ENV{'ASLIB'} . "/axis2.pdb";
  $self->{axis_file} = $ENV{'ASLIB'} . "/axis.pdb";
  #
	bless($self, $class);
	$self->prepare_ssbridge();
	$self->prepare_pdb();
	return $self;
}

sub do_overview {
	my $self = shift;
	my $run_number = shift;
	my $name = $self->{options}->{na};
	my ($pdb_file, $ovw_file);
	if ($run_number == 1) {			#run_number 1 is for all models
		$energy_hash_ref = $self->group_models();
		$pdb_file = $self->run_overview1();
		#system("mv ${name}.pdb all_models.pdb");
		return ($pdb_file, $energy_hash_ref);
	} elsif ($run_number == 2) {		#run_number 2 is for the best models (after rpf is run on all models)
		#copy("all_models.pdb", ${name} . ".pdb");
		$self->run_overview2();
		return;
	} else {
		die("unidentified run_number for xplor overview");
	}
}

sub run_overview1 {
	my $self = shift;
	my $input_coords = shift;
	my $name = $self->{options}->{na};
	open(OVW_XPL, ">script.pdbstat1.$$");
	my $from_str = $self->{options}->{nb} + 1;
	my $to_str = $self->{options}->{nstr} * $self->{options}->{np};
	#print OVW_XPL "debug -1\n";
	print OVW_XPL "read coords pdb grouped_models.pdb\n";
	print OVW_XPL "all\n";
	#print OVW_XPL "class\n";
	print OVW_XPL "to IUPAC\n";
	print OVW_XPL "write coords pdb all all_models.pdb\n";
	print OVW_XPL "quit\n";
	close(OVW_XPL);
	sleep(2);
	Utils->debug("about to execute: " . $ENV{"PDBSTAT"} . " -s < script.pdbstat1.$$");
	Utils->execute($ENV{"PDBStat"} . " -s < script.pdbstat1.$$");
	#system($ENV{"PDBSTAT"} . " < script.pdbstat.$$");
	my $cwd = getcwd();
	#my $pdb_file = "$cwd/all_models.pdb";
	copy("all_models.pdb", $self->{options}->{na} . ".pdb") ;
	my $pdb_file = "$cwd/$self->{options}->{na}" . ".pdb";

	#RT, split all_models.pdb in IUPAC for further analysis.
        open(F, "<" . "all_models.pdb") ;
	my $i = 0;
	while (<F>) {
		if (/^MODEL\s+(\d+)\s*$/) {
			$i++ ;
			if (MODEL->opened()) {
				print MODEL "END\n";
				close MODEL;
			}
			open(MODEL, ">" . $name . sprintf("%03d", $i) . ".pdb");
			#open(MODEL, ">" . "all_models" . sprintf("%03d", $i) . ".pdb");
		} else {
                	if (/^END\s*$/) {
                        	if (MODEL->opened()) {
                                  	print MODEL "END\n";
                                  	close MODEL;
                        	}
                        	last;
                	}
		}
                unless (/^MODEL/ or /ENDMDL/) {
                    if (MODEL->opened()) {
                        next if (substr($_, 17, 3) =~ /PL|LL5|ORI/);
                        if (/^ATOM/) {
                            my $new = $_;
                            substr($new, 20, 1, " "); #remove residue modifiers at position 20 such as CYS(S), HIS(T)
                            print MODEL $new;
                        } else {
                            print MODEL $_;
                        }
                    }
                }
	}
	close F ;
	#RT ---ends ---
	return ($pdb_file);
}

sub run_overview2 {
	my $self = shift;
	my $name = $self->{options}->{na};
	open(OVW_XPL, ">script.pdbstat2.$$");
	my $from_str = $self->{options}->{nb} + 1;
	my $to_str = $self->{options}->{nstr} * $self->{options}->{np};
	#print OVW_XPL "debug -1\n";
	print OVW_XPL "read coords pdb ${name}.pdb\n";
	print OVW_XPL "all\n";
	#print OVW_XPL "dele $from_str $to_str\n";
	print OVW_XPL "order 0.90\n";
	print OVW_XPL "rmsd best backbone\n";
	print OVW_XPL "Rmsd_Best\n";
	print OVW_XPL "read cons xplor ${name}_noe.tbl\n";
	print OVW_XPL "read cons xplor ${name}_hbond.tbl\n";
	print OVW_XPL "set cutup  0.3\n";
	#J.H avoid crushing in IL13 data set.
	#echo "see viol noe $AveNoe Violations.NOE"  >> script.pdbstat.$$
	#echo "see noe $AveNoe Distances"            >> script.pdbstat.$$
        #RT -- added two lines in new format
        print OVW_XPL "to IUPAC\n";
        print OVW_XPL "noe to IUPAC\n";
        print OVW_XPL "see viol sum Violations.NOE\n";
        print OVW_XPL "see noe sum Distances\n";
        #RT -- end
        print OVW_XPL "hbond all\n";
	print OVW_XPL "kabs all\n";
        print OVW_XPL "write coords pdb all $name.pdb\n";
        #RT
	print OVW_XPL "read cons xplor ${name}_dihe.tbl\n";
	print OVW_XPL "set cutaco 5.0\n";
	print OVW_XPL "see viol aco Violations.ACO\n";
	print OVW_XPL "quit\n";
	close(OVW_XPL);
	sleep(2);

	system($ENV{"PDBStat"} . " -s < script.pdbstat2.$$");
	#RT mv HydrogenBondData that caused trouble for second run
	system("mv HydrogenBondData_${name}.pdb HydrogenBondData.txt");
}

sub get_top_ranked_models {
	my $self = shift;
	my $dp_hash_ref = shift;
	my $energy_hash_ref = shift;
	my %energy = %$energy_hash_ref;

	my $minEnergy = 1000000000;
	foreach my $key (keys %energy) {
		Utils->debug("energy for $key is: " . $energy{$key});
		$minEnergy = $energy{$key} if ($minEnergy > $energy{$key});
	}
	my %rank = ();

	my $factor = int(abs($minEnergy)/100+0.5)*200;

	#print "keys in dp_hash_ref " . scalar(keys %{$dp_hash_ref}) . "\n";
	foreach (reverse sort {$dp_hash_ref->{$a}<=>$dp_hash_ref->{$b}} keys %{$dp_hash_ref}) {
		Utils->debug("dp score for $_ is " . $dp_hash_ref->{$_});
		#$rank{$_} = ((1.0 * $energy{$1}) + (2000.0 * (1 - $dp_hash_ref->{$_}))) if (/(\d+)/);
		$rank{$_} = ((1.0 * $energy{$1}) + ($factor * (1 - $dp_hash_ref->{$_}))) if (/(\d+)/);
	}
	my @rank_models = sort {$rank{$a}<=>$rank{$b}} keys %rank;
	#print "keys in rank " . scalar(keys %rank) . "\n";
	Utils->debug("BEST RANKED model is " . $rank_models[0] . " with a score of: " . $rank{$rank_models[0]});
	open(RANKS, ">RANKS");
	#print RANKS "#total = energy + 2000(1 - dp)\n";
	print RANKS "#total = energy + $factor(1 - dp)\n";
	print RANKS "#model  \tenergy   \tdp\ttotal\n";
	for (my $i = 0; $i < scalar(@rank_models); $i++) {
		if ($rank_models[$i] =~ /(\d+)/) {
			print RANKS $rank_models[$i] . "  \t" . ($energy{$1}) . "   \t" . $dp_hash_ref->{$rank_models[$i]} . "\t" . $rank{$rank_models[$i]} . "\n";
		}
	}
	close(RANKS);
	return \@rank_models;
}

sub group_models {
	my $self = shift;
	my $name = $self->{options}->{na};
	my $model = 0;
	system("rm -f grouped_models.pdb");
	my @models = <${name}_sa_*>;
	my %energy = ();
	open(ALL_SA, ">grouped_models.pdb");
	foreach my $model_file (@models) {
		$model++;
		open(MODEL, "<$model_file");
		while (<MODEL>) {
			chomp;
			if (/.*FILENAME*/) {
				print ALL_SA "MODEL          $model\n";
			} elsif ( /.*overall\s*:\s*(\S+)/ ) {
				$energy{$model} = $1;
				#print "energy for model$model found in $model_file is $1\n";
				print ALL_SA "$_\n";
			} elsif ( /^REMARK/ ) {
				print ALL_SA "$_\n";
			} elsif ( /^END/) {
				print ALL_SA "ENDMDL\n";
			} elsif ( / ANI / ) {

			} else {
				print ALL_SA "$_\n";
			}
		}
		close(MODEL);
	}
	print ALL_SA "END\n";
	close(ALL_SA);
	return \%energy;
}

sub prepare_batch {
	my $self = shift;
	my $seed = shift;
	my $cmdname = shift;
	my $process = shift;
	my $name = $self->{options}->{na};
	my $filename = "Smt${cmdname}${process}.sh";
	my $commands = "";
	$commands .= "date\n";
	$commands .= "hostname\n";
#	$commands .= "export ASLIB=/soft/software/asdp-2.3/lib\n" ;
	$commands .= "export ASLIB=" . $ENV{'ASLIB'} . "\n";
	$commands .= $ENV{'XplorCmd'} . " < " . $self->{TARGET} . " > " . $self->{TARGET} . ".out\n";
	$commands .= "touch Done${name}${process}\n";
	$self->{queue}->create_batch($filename, "${name}_${process}", $commands);
}


sub prepare_sequence {
	# Simplistic way to deal with sequences that do not start at residue
	# number 1. It keeps adding ALA at the beginning of the file and then
	# the script to run X-PLOR is edited to delete those fake residues
	# from the PSF.
	#
	# Creates fake sequence file padding ALA's at the beginning of file,
	# as many as start residue number minus 1. In further cycles this if
	# is not executed as the sequence file has been already faked.

        my $self = shift;
        my $name = $self->{options}->{na};
        Utils->debug("in Xplor->prepare_sequence()");

	if ($self->{options}->{st} > 1) {
		system("rm -f Sequence.$$");
		open(SEQ, ">>Sequence.$$");
		my $work_res = $self->{options}->{st} - 1;
		while ($work_res >= 1) {
	    		print SEQ "ALA \n";
	    		$work_res--;
		}
		system("cat $name.seqXplor >> Sequence.$$");
		#cat tmpSeq.$$  $Name.seqXplor > ${Name}.Sequence
	} else {
		system("cp $name.seqXplor  Sequence.$$");
	}
	system("cp Sequence.$$ " . ${name} . ".Sequence");

	#clean-up
	if (!(defined $self->{options}->{debug}))
	{
		system("rm Sequence.$$");
		system("rm " . ${name} . ".seqXplor");
	}
}
sub prepare_run {
	my $self = shift;
	my ($process, $structures, $begin, $end, $seed) = @_;
	my $name = $self->{options}->{na};
	Utils->debug("in Xplor->prepare_run()");

	#
	# Edit the template for SA accordingly. Here we proceed to delete the
	# fake residues padded from the PSF file, if sequence starts at 1 do
	# nothing.
	open(SEDFILE, ">>SedFile.$$");
	if ($self->{first_residue_n} == 0) {
		print SEDFILE "/dele sele (resid/d\n";
	} else {
		print SEDFILE "/dele sele (resid/s/FINAL/" . $self->{first_residue_n} . "/\n";
	}
 	# cis residue info
	if (defined $self->{options}->{cis}) {
	       print SEDFILE "/evaluate (\$HaveCis/s/no/yes/\n";
	       print SEDFILE "/CISpep/r FileCis.$$\n";
	}
	if (defined $self->{options}->{ss}) {
	        print SEDFILE "/evaluate (\$HaveDisu/s/no/yes/\n";
		print SEDFILE "/SSBridge/r SSBridges.$$\n";
	}
	print SEDFILE "s/PROTEIN/$name/;s/BEGIN/$begin/;s/END/$end/;s/SEMILLA/$seed/\n";
	my $dbg = "no";
	if (defined $self->{options}->{debug}) {
		$dbg = "yes";
	}
	print SEDFILE "/evaluate (\$MyDebug/s/DBG/" . $dbg . "/\n";
	print SEDFILE "/name=\"CHAI\"/s/CHAI/" . $self->{options}->{ch} . "   /\n";
	print SEDFILE "/evaluate (\$From/s/Mdlunk/" . $self->{options}->{mo} . "/\n";
	print SEDFILE "/evaluate (\$Noeavg/s/unk/" . $self->{options}->{av} . "/\n";

	print SEDFILE "/HOTADJUST/s/HOTADJUST/" . $self->{options}->{hotad} . "/\n";
	print SEDFILE "/HOTCYCLES/s/HOTCYCLES/" . $self->{options}->{hotcy} . "/\n";
	print SEDFILE "/evaluate (\$hot_nst/s/HOTNSTEPS/" . $self->{options}->{hotst} . "/\n";		#same as hotle???

	###################################################################
	#if (-z $DA_RDC -a -z $RHOMB_RDC) {
	#	print SEDFILE "/DA_RDC/d\n";
	#	print SEDFILE "/DA_RHOM/d\n";
	#} else {
	#	print SEDFILE "/DA_RDC/s/DA_RDC/$DA_RDC/\n";
	#	print SEDFILE "/RHOMB_RDC/s/RHOMB_RDC/$RHOMB_RDC/\n";
	#}
        # RTT, Jan 2019, trying to get two media working
        if ((defined $self->{options}->{da}) and (defined $self->{options}->{rh})) {
                print SEDFILE "/DA1_RDC/s/DA1_RDC/" . $self->{options}->{da} . "/\n";
                print SEDFILE "/RHOMB1_RDC/s/RHOMB1_RDC/" . $self->{options}->{rh} . "/\n";
        } else {
                print SEDFILE "/DA1_RDC/d\n";
                print SEDFILE "/RHOMB1_RDC/d\n";
        }
        # a possible second media in alignments, RTT 2019
        if ((defined $self->{options}->{da2}) and (defined $self->{options}->{rh2})) {
                print SEDFILE "/DA2_RDC/s/DA2_RDC/" . $self->{options}->{da2} . "/\n";
                print SEDFILE "/RHOMB2_RDC/s/RHOMB2_RDC/" . $self->{options}->{rh2} . "/\n";
        } else {
                print SEDFILE "/DA2_RDC/d\n";
                print SEDFILE "/RHOMB2_RDC/d\n";
        }
	###################################################################


	#
	# check for some files and edit accordingly
	if (-f "${name}_noe.tbl") {
		print SEDFILE "/evaluate (\$HaveNoe/s/no/yes/\n";
	}
	if (-f "${name}_dihe.tbl") {
		print SEDFILE "/evaluate (\$HaveDih/s/no/yes/\n";
	}
	if (-f "${name}_hbond.tbl") {
		print SEDFILE "/evaluate (\$HaveHbond/s/no/yes/\n";
	}
	if (-f "${name}_sani.tbl") {
		print SEDFILE "/evaluate (\$HaveRDC/s/no/yes/\n";
	}
	#RT -- second media present
	if (-f "${name}_sani2.tbl") {
		print SEDFILE "/evaluate (\$Have2RDC/s/no/yes/\n";
	}
	if (-f "${name}_Jhnha.tbl") {
		print SEDFILE "/evaluate (\$HaveJcoup/s/no/yes/\n";
	}
	if (-f "${name}_shiftsCACB.tbl") {
		print SEDFILE "/evaluate (\$HaveCaCbShifts/s/no/yes/\n";
	}
	close(SEDFILE);
	$self->{TARGET} = $self->{template} . "_${process}.inp";
	#J.H
	my $command = "sed -f SedFile.$$ < " . $self->{template_file} . " > " . $self->{template} . "_${process}.inp";
	`$command`;
	unlink("SedFile.$$");
}

sub prepare_pdb {
	#move to xplor
	my $self = shift;
	if ($self->{starting_model} eq "model") {
		my $chain = $self->{options}->{ch};
		my $name = $self->{options}->{na};
		system("cp ${name}_0.pdb ${name}_0.pdb.ORIG");
		open(ORIG_PDB, "<${name}_0.pdb.ORIG");
		open(NEW_PDB, ">${name}_0.pdb");
		while ((defined ORIG_PDB) and my $temp_line = <ORIG_PDB>) {
			my @items = split(/\s+/, $temp_line);
			if ($items[2] =~ /[COSN]/i) {
				$items[4] = $chain;
				for (my $i = 0; $i < scalar(@items); $i++)  {
					print NEW_PDB ($items[$i] . " ");
				}
				print NEW_PDB ("$chain\n");
			}
		}
		close(ORIG_PDB);
		close(NEW_PDB);
	}
}

sub prepare_ssbridge {
	my $self = shift;
	if (defined $self->{options}->{ss}) {
		open(SSBRIDGES, ">>SSBridges.$$");
		open(SSNOES, ">>SSNOES.$$");
		print SSNOES "    noe\n";
		my @bridges = split(/,/, $self->{options}->{ss});
		foreach my $bridge (@bridges) {
			print SSBRIDGES "    patch disu \n";
			my $ref = 0;
			my @resids = split(/-/, $bridge);
			foreach my $resid (@resids) {
				$ref++;
				print SSNOES "        assign (resid $resid and name sg) ";		#dont put newline at the end
				print SSBRIDGES "        reference=$ref=(resid $resid)\n";
			}
			print SSBRIDGES "    end \n";
			print SSNOES "  2.02 0.1 0.1 \n";
		}
		print SSNOES "    end \n";
		close(SSBRIDGES);
		close(SSNOES);
	}
}

sub do_cleanup {
	my $self = shift;
	my $name = $self->{options}->{na};
	#TODO condense the rm's into one command to minimize process forking!
	system("rm -f Done*");
	system("cat *.inp > CalcXplor.inp");
	system("cat *.inp.out > Log" . ref($self));
        # RT, to save space on disk
        system("gzip Log" . ref($self));

	unless ($ENV{'DEBUG'} == 1) {
		#RT -- copied from Cyana.pm
		system("cat Smt*sh > SubCmd_AS");
		system("cat run_*sh > SubCmd_DP");

		system("rm -f all_models.pdb best_models.pdb grouped_models.pdb");
	  system("rm -f log* Calc* Smt* SedFile* FileCis* SSBridges* SSNoes* sa_for_AS*");
		system("rm -f ${name}?*.pdb");
		system("rm -rf run*.sh rpf_out");
	}
}
return(1);
