#!perl

use strict;

my ($pdb) = @ARGV;

open(PDB, "<$pdb");
open(PDBCT, ">PDBCT.txt");

my %atom;
my $model = 0;
my %pdbCT;
while(<PDB>)
{
    chomp;
    if ($_ =~ /^MODEL\s+\d+\s*$/)
    {
	$model++;
	#print "$_\t$model\n";
    }
    next if ($_ !~ /^ATOM/);
    my ($atomName, $res, $x, $y, $z) = $_ =~ /ATOM\s+\d+\s*([\w\d]+)\s*\w+\s*\s*(\d+)\s+(-*\d+\.\d+)\s*(-*\d+\.\d+)\s*(-*\d+\.\d+)\s*\w*/;

    print "$_\n$atomName $res $x $y $z\n";

    $model = 1 if ($model < 1);
    $atom{$res . "-" . $atomName}{$model} = $x . "/" . $y . "/" . $z;
    $pdbCT{"$res\t$res"}++;
}

foreach my $a1 (sort keys %atom)
{
    foreach my $a2 (sort keys %atom)
    {
	next if ($a1 eq $a2);
	next if ($a2 < $a1);

	my ($num1, $name1) = split(/-/, $a1);
	my ($num2, $name2) = split(/-/, $a2);

	print "$a1-$a2\n";
	my $pair = ""; my $pairCT="";
	if ($num1 < $num2)
	{
	    $pair = "$num1-$num2";
	    $pairCT = "$num1\t$num2";
	} else
	{
	    $pair = "$num2-$num1";
	    $pairCT = "$num2\t$num1";
	}

#	next if (abs($num1 - $num2) < 5);
	next if (defined $pdbCT{$pair});

#	next if ($name1 !~ /^C/);
#	next if ($name2 !~ /^C/);

	#next if (($num1 != 2) && ($num1 !=104));
	#next if (($num2 != 2) && ($num2 != 104));

#	print "$num1-$name1, $num2-$name2\n";
	#my $dismax = -1;
	my $disCount = 0;
	for(my $i = 1; $i <= $model; $i++)
	{
	    my ($x1, $y1, $z1) = split(/\//, $atom{$a1}{$i});
	    my ($x2, $y2, $z2) = split(/\//, $atom{$a2}{$i});
	    my $dist = sqrt(($x1-$x2)**2+($y1-$y2)**2+($z1-$z2)**2);
	    $disCount++ if ($dist < 5.00001);
	    #$dismax = $dist if ($dist > $dismax);
	    print "$i $atom{$a1}{$i} $atom{$a2}{$i} $dist\n"; #  $dismax\n"; #if ($dist < 5.00001);
	}

	next if ($disCount/$model < 0.60001);

	$pdbCT{$pairCT}++;
#	print "AmbigousNMRDistance\t$name1\t$num1\t$name2\t$num2\tBOUNDED\t1.80\t7.0\t0.3\tNOE\n" if (($dismax < 5.00001) && ($ec =~ /\/$pair\//));
    }
}

print "total model $model found\n";

foreach my $pair (sort keys %pdbCT)
{
    print PDBCT "$pair\n";
}
