#!/usr/bin/perl

#********************************************************
#*Welcome to the ASDP configuration tool.
#*This script will setup your ASDP Environment.
#*You can run this script multiple times.
#********************************************************


use strict;
my $installer = Installer->new();
$installer->printIntro();

my $aspath = $installer->getInstallDirectory();
my $cyanaCmd = $installer->getCommand("cyana", "Cyana");
my $cyanaMPICmd = $installer->getCommand("cyana", "CyanaMPI");
my $mpiNodes = "";
my $mpiProcsPerNode = "";
my $lambootFile = "";
if ($cyanaMPICmd =~ /\S+/) {
	$mpiNodes = $installer->getValue("Number of nodes to be used by CyanaMPI");
	$mpiProcsPerNode = $installer->getValue("Number of processes to be launched on each node");
	$lambootFile = $installer->getValue("Full path of the lamboot file");
}
my $xplorCmd = $installer->getCommand("xplor", "Xplor");
#my $cnsCmd = $installer->getCommand("cns", "CNS");
#my $cnsENV = "";
#if ($cnsCmd =~ /\S+/) {
#	my $cnsENV = $installer->getValue("location of cns_solve_env file. (typically your cns install directory)");
#}

if (($xplorCmd =~ /\S+/) or ($cnsCmd =~ /\S+/)) {
	print "Sed is required to run Xplor and CNS\n";
	my $sed = $installer->getCommand("sed", "sed (Stream Editor)");
}
my $qsubCmd = $installer->getCommand("qsub", "PBS Queue (qsub)");
my $pbsdefault;
if ($qsubCmd =~ /\S+/) {
	$pbsdefault = $installer->getValue("Default master server for PBS queue (PBS_DEFAULT)");
}

my $customQCmd = $installer->getCommand("customqueue", "Non-PBS Cluster Queue");

$installer->writeScript_pdbstat($aspath . "/bin", "pdbstat");
#$installer->writeScript_dp($aspath . "/bin", "dp.pl", $qsubCmd);
$installer->writeScript_createproc($aspath . "/bin", "CreateProc", $cyanaCmd, $xplorCmd, $qsubCmd, $pbsdefault, $cyanaMPICmd, $mpiNodes, $mpiProcsPerNode, $lambootFile, $customQCmd);
#$installer->writeScript_refinement($aspath . "/bin", "Refinement", $qsubCmd, $pbsdefault, $cnsCmd, $cnsENV, $customQCmd);

#my $rasmolCmd = $installer->getCommand("rasmol", "Rasmol");
#$installer->writeScript_view($aspath . "/bin", "View", $rasmolCmd);

$installer->writeScript_asdpgui($aspath . "/bin", "asdp-gui", $rasmolCmd);
$installer->checkForTk();

print "
***** ASDP successfully configured *****
";

BEGIN {
	package Installer;
	use Cwd;
	sub new {
		my $class = shift;
		my $self = {};
		bless($self, $class);
		return $self;
	}
#################################################################################################
#################################################################################################
	sub printIntro {
		my $self = shift;
		print "********************************************************\n";
		print "*Welcome to the ASDP configuration tool.               *\n";
		print "*This script will setup your ASDP Environment.         *\n";
		print "********************************************************\n\n";
	}
#################################################################################################
#################################################################################################
	sub getInstallDirectory {
		my $self = shift;
		my $as_dir = Cwd::abs_path($0);
		$as_dir =~ /^(.*)\/([^\/]+)\s*$/;
		$as_dir = $1;
		my $script_name = $2;
		print "Select your ASDP directory:\n";
		print "Best guess is $as_dir  Is this correct? (y/n)";
		my $response = <STDIN>;
		unless ($response =~ /^y$/) {
			print "Please enter the full path of your ASDP directory:\n";
			$as_dir = <STDIN>;
			chomp($as_dir);
		}
		unless ($self->confirmDirectory($as_dir, "bin/noesyassign")) {
			print STDERR "ASDP not found at $as_dir\n";
			exit(1);
		}
		print "You have chosen: $as_dir\n";
		$self->{ASPATH} = $as_dir;
		return $as_dir;
	}
#################################################################################################
#################################################################################################
	sub writeScript_dp {
		my ($self, $dir, $filename, $qsubCmd) = @_;
		my $ASPATH = $self->{ASPATH};
		open(FILE, ">$dir/$filename");
		print FILE <<EOF;
#!/usr/bin/perl -w
use strict;
use warnings;
use Cwd;

my \$script_name = "";
BEGIN {
	#add the parent directory of the script to lib include path
	my \$script_dir = \$0;
	\$script_dir =~ /^(.*)\\/([^\\/]+)\\s*\$/;
	\$script_name = \$2;
	push(\@INC, \$1);
}
use ASDP::DP;

my \$debug = 0;
\$ENV{'SUBMITPBS'} = "$qsubCmd";
\$ENV{'ASPATH'} = "/farm/users/gautams/projects/ASDP/ASDP-dev";
\$ENV{'ASLIB'} = \$ENV{'ASPATH'} . '/lib';
\$ENV{'ASBIN'} = \$ENV{'ASPATH'} . '/bin';
\$ENV{'ASDPSCRIPT'} = \$ENV{'ASBIN'} . '/asdp';
\$ENV{'PBS_DEFAULT'} = 'master3.cabm.rutgers.edu';		#cluster
\$ENV{'DP_WD'} = getcwd;									#dp working directory
\$ENV{'RPF_DIR'} = getcwd . '/rpf_out';					#temp dir for rpf output
\$ENV{'MAX_PROCESSES'} = 50;
die "Usage: perl pickModelByDP.pl <ensemble_pdb_file> <ovw_file> <ctrl_file> <n_model>\\n" if (scalar \@ARGV < 4);
my \$dp = DP->new(\@ARGV, \$debug);
\$dp->split_models();
\$dp->run_rpf();
\$dp->get_top_ranked_models();


EOF
		close FILE;
		chmod(0775, "$dir/$filename");
	}
#################################################################################################
#################################################################################################
	sub writeScript_view {
		my ($self, $dir, $filename, $rasmolCmd) = @_;
		my $ASPATH = $self->{ASPATH};
		open(FILE, ">$dir/$filename");
		print FILE <<EOF;

#!/bin/sh
name=\$1
egrep -v 'PL|LL5|LP' \${name}.pdb > View_tmp
echo read coords pdb View_tmp      > View_Script
echo all                          >> View_Script
echo order 0.90                   >> View_Script
echo rmsd best backbone           >> View_Script
echo View_momo                    >> View_Script
echo write                        >> View_Script
echo View_best                    >> View_Script
echo coords                  >> View_Script
echo backbone                >> View_Script
echo all                     >> View_Script
echo pdb                     >> View_Script
echo quit                    >> View_Script
#
$dir/pdbstat < View_Script > /dev/null

xterm -exec $rasmolCmd View_best

# Cleaning
rm View_momo* View_tmp* View_best* View_Script* Order*
exit

EOF
		close FILE;
		chmod(0775, "$dir/$filename");
	}
#################################################################################################
#################################################################################################
	sub writeScript_asdpgui {
		my ($self, $dir, $filename, $rasmolCmd) = @_;
		my $ASPATH = $self->{ASPATH};
		open(FILE, ">$dir/$filename");
		print FILE <<EOF;
#!/usr/bin/perl -w

# asdp perl script
#
# This script sets up the environment and dataset to run asdp
# Usage: asdp <options>
#

my \$script_name;
BEGIN {
	use Cwd;
	\$ENV{'DEBUG'} = 0;			#set to 1 to turn on debugging print outs
	#add the parent directory of the script to lib include path
	my \$script_dir = \$0;
	\$script_dir =~ /^(.*)\\/([^\\/]+)\\s*\$/;
	\$script_dir = \$1;
	\$script_name = \$2;
	unless (\$script_dir =~ /^\\//) {
		\$script_dir = getcwd() . "/\$script_dir";
	}
	\$script_dir .= "/..";
	\$ENV{'ASPATH'} = \$script_dir;
	\$ENV{'ASBIN'} = \$ENV{'ASPATH'} . '/bin';
	\$ENV{'AyudaPATH'} = \$ENV{'ASPATH'} . '/PDBSTAT';
	\$ENV{'NOESYASSIGN'} = \$ENV{'ASBIN'} . "/noesyassign";
	\$ENV{'RASMOL'} = "$rasmolCmd";
	\$ENV{'EXECODE'} = \$ENV{'ASBIN'} . "/asdp";
	\$ENV{'PERL5LIB'} = \$ENV{'ASPATH'} . "/gui";
	\$ENV{'ASLIB'} = \$ENV{'ASPATH'} . '/lib';
	push(\@INC, \$ENV{'ASLIB'});
	push(\@INC, \$ENV{'ASPATH'});
}

#start with gui
exec(\$ENV{'PERL5LIB'} . "/main_window.pl &");

EOF
		close FILE;
		chmod(0775, "$dir/$filename");
	}
#################################################################################################
#################################################################################################
	sub writeScript_pdbstat {
		my ($self, $dir, $filename) = @_;
		my $ASPATH = $self->{ASPATH};
		open(FILE, ">$dir/$filename");
		print FILE <<EOF;
#!/bin/sh

#AutoStructure Path
ASPATH=$ASPATH

AyudaPATH=\$ASPATH/PDBSTAT

#export
export AyudaPATH
#start with noesy_assign
\$AyudaPATH/pdbstat \$*

EOF
		close FILE;
		chmod(0775, "$dir/$filename");
	}
#################################################################################################
#################################################################################################
	sub writeScript_createproc {
		my ($self, $dir, $filename, $cyanaCmd, $xplorCmd, $qsubCmd, $pbsdefault, $cyanaMPIcmd, $mpiNodes, $mpiProcsPerNode, $lambootFile, $customQCmd) = @_;
		my $ASPATH = $self->{ASPATH};
		open(FILE, ">$dir/$filename");
		print FILE <<EOF;
#!/usr/bin/perl
use strict;
use warnings;

\$ENV{'PBS_DEFAULT'} = "$pbsdefault";
\$ENV{'LAMBOOT_FILE'} = "$lambootFile";
\$ENV{'SUBMITPBS'} = "$qsubCmd";
\$ENV{'SUBMITCUSTOM'} = "$customQCmd";
\$ENV{'CyanaCmd'} = "$cyanaCmd";
\$ENV{'XplorCmd'} = "$xplorCmd";
\$ENV{'CyanaCmdMPI'} = "$cyanaMPIcmd";
\$ENV{'MPI_NODES'} = "$mpiNodes";
\$ENV{'MPI_PROCESSES_PER_NODE'} = "$mpiProcsPerNode";

my \$script_name = "";
my \$starttime = `date`;
BEGIN {
	use Cwd;
	my \$debug = 0;
	#add the parent directory of the script to lib include path
	my \$script_dir = \$0;
	\$script_dir =~ /^(.*)\\/([^\\/]+)\\s*\$/;
	\$script_dir = \$1;
	\$script_name = \$2;
	unless (\$script_dir =~ /^\\//) {
		\$script_dir = getcwd() . "/\$script_dir";
	}
	\$script_dir .= "/..";
	\$ENV{'ASPATH'} = \$script_dir;
	\$ENV{'ASLIB'} = \$ENV{'ASPATH'} . '/lib';
	\$ENV{'ASBIN'} = \$ENV{'ASPATH'} . '/bin';
	\$ENV{'ASDPSCRIPT'} = \$ENV{'ASBIN'} . '/asdp';
	\$ENV{'DP_WD'} = getcwd;							#dp working directory
	\$ENV{'RPF_DIR'} = getcwd . '/rpf_out';					#temp dir for rpf output
	###########################################################
	\$ENV{'machine'} = `uname -n`;							#
	\$ENV{'PDBStat'} = \$ENV{'ASBIN'} . '/pdbstat';
	\$ENV{'AyudaPATH'} = \$ENV{'ASPATH'} . '/PDBSTAT';
	push(\@INC, \$ENV{'ASLIB'});
	push(\@INC, \$ENV{'ASPATH'});
}
use ASDP::CreateProc;

#DEFAULT OPTIONS
my \$options = {
	np => 50,
	nstr => 2,
	nb => 20,
	que => "PBS",
	pr => "cyana",
	mo => "random",
	av => "sum",
	ch => "A",
	st => 1,
	hotad => 3000,
	hotcy => 4,
	hotst => 1000
};
for (my \$i = 0; \$i <= \$\#ARGV; \$i++) {
	if (\$ARGV[\$i] =~ /^-(\\S+)\$/) {
		my \$key = \$1;
		if (defined \$ARGV[\$i+1] and \$ARGV[\$i+1] =~ /^[^\\-\\s]/) {
			my \$value = \$ARGV[++\$i];
			\$options->{\$key} = \$value;
		} else {
			\$options->{\$key} = \$key;
		}
	} else {
		print STDERR "ERROR: invalid option " . \$ARGV[\$i] . "\\n";
	}
}
if (defined \$options->{'de'}) {
	\$ENV{'DEBUG'} = 1;
} else {
	\$ENV{'DEBUG'} = 0;
}
my \$create_proc = CreateProc->new(\$options);
\$create_proc->calculate_structures();
\$create_proc->print_statistics();
\$create_proc->do_cleanup();
#\$create_proc->set_queue(\$qu);

EOF
		close FILE;
		chmod(0775, "$dir/$filename");
	}
#################################################################################################
#################################################################################################
	sub writeScript_refinement {
		my ($self, $dir, $filename, $qsubCmd, $pbsdefault, $cnsCmd, $cnsENV, $customQCmd) = @_;
		my $ASPATH = $self->{ASPATH};
		open(FILE, ">$dir/$filename");
		print FILE <<EOF;
#!/usr/bin/perl
use strict;
use warnings;

\$ENV{'PBS_DEFAULT'} = "$pbsdefault";
\$ENV{'SUBMITPBS'} = "$qsubCmd";
\$ENV{'SUBMITCUSTOM'} = "$customQCmd";
\$ENV{'CnsCommand'} = "$cnsCmd";
\$ENV{'CNSENV'} = "$cnsENV";
\$ENV{'WATREFLIB'} = "$ASPATH/cns";

my \$script_name = "";
my \$starttime = `date`;
BEGIN {
	use Cwd;
	my \$debug = 0;
	#add the parent directory of the script to lib include path
	my \$script_dir = \$0;
	\$script_dir =~ /^(.*)\\/([^\\/]+)\\s*\$/;
	\$script_dir = \$1;
	\$script_name = \$2;
	unless (\$script_dir =~ /^\\//) {
		\$script_dir = getcwd() . "/\$script_dir";
	}
	\$script_dir .= "/..";
	\$ENV{'ASPATH'} = \$script_dir;
	\$ENV{'ASLIB'} = \$ENV{'ASPATH'} . '/lib';
	\$ENV{'ASBIN'} = \$ENV{'ASPATH'} . '/bin';
	\$ENV{'ASDPSCRIPT'} = \$ENV{'ASBIN'} . '/asdp';
	\$ENV{'DP_WD'} = getcwd;							#dp working directory
	\$ENV{'RPF_DIR'} = getcwd . '/rpf_out';					#temp dir for rpf output
	\$ENV{'WATREFLIB'} = \$ENV{'ASPATH'} . '/cns';
	###########################################################
	\$ENV{'machine'} = `uname -n`;							#
	\$ENV{'PDBStat'} = \$ENV{'ASBIN'} . '/pdbstat';
	\$ENV{'AyudaPATH'} = \$ENV{'ASPATH'} . '/PDBSTAT';
	push(\@INC, \$ENV{'ASPATH'});
	push(\@INC, \$ENV{'ASLIB'});
}
use ASDP::Refinement;

#DEFAULT OPTIONS
my \$options = {
	np => 20,
	nstr => 1,
	nb => 20,
	que => "PBS",
	pr => "cns",
	mo => "random",
	av => "sum",
	ch => "A",
	st => 1,
	sc => 1,		#appscale
	params => "OPLSX",
	hotcy => 1000,
	coolcy => 100,
	heatcy => 200,
	seed =>	31415
};
for (my \$i = 0; \$i <= \$#ARGV; \$i++) {
	if (\$ARGV[\$i] =~ /^-(\\S+)\$/) {
		my \$key = \$1;
		if (defined \$ARGV[\$i+1] and \$ARGV[\$i+1] =~ /^[^\\-\\s]/) {
			my \$value = \$ARGV[++\$i];
			\$options->{\$key} = \$value;
		} else {
			\$options->{\$key} = \$key;
		}
	} else {
		print STDERR "ERROR: invalid option " . \$ARGV[\$i] . "\n";
	}
}
if (defined \$options->{'de'}) {
	\$ENV{'DEBUG'} = 1;
} else {
	\$ENV{'DEBUG'} = 0;
}
my \$refinement = Refinement->new(\$options);
\$refinement->do_refinement();
\$refinement->print_statistics();
\$refinement->do_cleanup();

EOF
		close FILE;
		chmod(0775, "$dir/$filename");
	}
#################################################################################################
#################################################################################################
	sub getCommand {
		my ($self, $cmd, $program) = @_;
		print "\n***** Do you have $program installed? *****\n";
		my $guess = "";
		my $output = system("which $cmd > /dev/null 2>&1");
		if ($output == 0) {
			my $path = `which $cmd`;
			chomp($path);
			print "Best guess is $path.  Is this correct? (y/n)";
			my $answer = <STDIN>;
			chomp($answer);
			if ($answer =~ /^y$/i) {
				return $path;
			}
		}
		print "Enter the full path of the program or leave blank if unavailable: ";
		my $answer = <STDIN>;
		chomp($answer);
		return $answer;
	}
#################################################################################################
	sub getValue {
		my ($self, $option) = @_;
		print "\n***** Select the value of $option. *****\n";
		print "Enter the value or leave blank if unavailable: ";
		my $answer = <STDIN>;
		chomp($answer);
		return $answer;
	}
#################################################################################################
	sub confirmDirectory {
		my ($self, $dir, $filepath) = @_;
		if (-e "$dir/$filepath") {
			return 1;
		}
		return 0;
	}
#################################################################################################
	sub checkForTk {
		my $self = shift;
		unless ($self->checkForPerlDependencies("Tk")) {
			print "Unable to locate perl module Tk.  Please note that while the gui interface to AutoAssign will not work without Tk, you can still use the command line.\n";
			print "Tk can be installed from http://search.cpan.org.\n";
		}
	}
	sub checkProgram {
		my $self = shift;
		my $program = shift;
		my $pid = system("$program >/dev/null 2>/dev/null");
		if ($pid > -1 and $pid < 32000) {
			return 1;
		}
		return 0;
	}
#################################################################################################
	sub checkForPerlDependencies {
		my ($self, $module) = @_;
		my $got_module = eval "require $module; 1" ? 1 : 0;
		return $got_module;
	}
}
